//javascript/closure/base.js
/**
 * @license
 * Copyright The Closure Library Authors.
 * SPDX-License-Identifier: Apache-2.0
 */

/**
 * @fileoverview Bootstrap for the Google JS Library (Closure).
 *
 * In uncompiled mode base.js will attempt to load Closure's deps file, unless
 * the global <code>CLOSURE_NO_DEPS</code> is set to true.  This allows projects
 * to include their own deps file(s) from different locations.
 *
 * Avoid including base.js more than once. This is strictly discouraged and not
 * supported. goog.require(...) won't work properly in that case.
 *
 * @provideGoog
 */


/**
 * @define {boolean} Overridden to true by the compiler.
 */
var COMPILED = false;


/**
 * Base namespace for the Closure library.  Checks to see goog is already
 * defined in the current scope before assigning to prevent clobbering if
 * base.js is loaded more than once.
 *
 * @const
 */
var goog = goog || {};

/**
 * Reference to the global object.
 * https://www.ecma-international.org/ecma-262/9.0/index.html#sec-global-object
 *
 * More info on this implementation here:
 * https://docs.google.com/document/d/1NAeW4Wk7I7FV0Y2tcUFvQdGMc89k2vdgSXInw8_nvCI/edit
 *
 * @const
 * @suppress {undefinedVars} self won't be referenced unless `this` is falsy.
 * @type {!Global}
 */
goog.global =
    // Check `this` first for backwards compatibility.
    // Valid unless running as an ES module or in a function wrapper called
    //   without setting `this` properly.
    // Note that base.js can't usefully be imported as an ES module, but it may
    // be compiled into bundles that are loadable as ES modules.
    this ||
    // https://developer.mozilla.org/en-US/docs/Web/API/Window/self
    // For in-page browser environments and workers.
    self;


/**
 * A hook for overriding the define values in uncompiled mode.
 *
 * In uncompiled mode, `CLOSURE_UNCOMPILED_DEFINES` may be defined before
 * loading base.js.  If a key is defined in `CLOSURE_UNCOMPILED_DEFINES`,
 * `goog.define` will use the value instead of the default value.  This
 * allows flags to be overwritten without compilation (this is normally
 * accomplished with the compiler's "define" flag).
 *
 * Example:
 * <pre>
 *   var CLOSURE_UNCOMPILED_DEFINES = {'goog.DEBUG': false};
 * </pre>
 *
 * @type {Object<string, (string|number|boolean)>|undefined}
 */
goog.global.CLOSURE_UNCOMPILED_DEFINES;


/**
 * A hook for overriding the define values in uncompiled or compiled mode,
 * like CLOSURE_UNCOMPILED_DEFINES but effective in compiled code.  In
 * uncompiled code CLOSURE_UNCOMPILED_DEFINES takes precedence.
 *
 * Also unlike CLOSURE_UNCOMPILED_DEFINES the values must be number, boolean or
 * string literals or the compiler will emit an error.
 *
 * While any @define value may be set, only those set with goog.define will be
 * effective for uncompiled code.
 *
 * Example:
 * <pre>
 *   var CLOSURE_DEFINES = {'goog.DEBUG': false} ;
 * </pre>
 *
 * @type {Object<string, (string|number|boolean)>|undefined}
 */
goog.global.CLOSURE_DEFINES;


/**
 * Builds an object structure for the provided namespace path, ensuring that
 * names that already exist are not overwritten. For example:
 * "a.b.c" -> a = {};a.b={};a.b.c={};
 * Used by goog.provide and goog.exportSymbol.
 * @param {string} name name of the object that this file defines.
 * @param {*=} opt_object the object to expose at the end of the path.
 * @param {Object=} opt_objectToExportTo The object to add the path to; default
 *     is `goog.global`.
 * @private
 */
goog.exportPath_ = function(name, opt_object, opt_objectToExportTo) {
  var parts = name.split('.');
  var cur = opt_objectToExportTo || goog.global;

  // Internet Explorer exhibits strange behavior when throwing errors from
  // methods externed in this manner.  See the testExportSymbolExceptions in
  // base_test.html for an example.
  if (!(parts[0] in cur) && typeof cur.execScript != 'undefined') {
    cur.execScript('var ' + parts[0]);
  }

  for (var part; parts.length && (part = parts.shift());) {
    if (!parts.length && opt_object !== undefined) {
      // last part and we have an object; use it
      cur[part] = opt_object;
    } else if (cur[part] && cur[part] !== Object.prototype[part]) {
      cur = cur[part];
    } else {
      cur = cur[part] = {};
    }
  }
};


/**
 * Defines a named value. In uncompiled mode, the value is retrieved from
 * CLOSURE_DEFINES or CLOSURE_UNCOMPILED_DEFINES if the object is defined and
 * has the property specified, and otherwise used the defined defaultValue.
 * When compiled the default can be overridden using the compiler options or the
 * value set in the CLOSURE_DEFINES object. Returns the defined value so that it
 * can be used safely in modules. Note that the value type MUST be either
 * boolean, number, or string.
 *
 * @param {string} name The distinguished name to provide.
 * @param {T} defaultValue
 * @return {T} The defined value.
 * @template T
 */
goog.define = function(name, defaultValue) {
  var value = defaultValue;
  if (!COMPILED) {
    var uncompiledDefines = goog.global.CLOSURE_UNCOMPILED_DEFINES;
    var defines = goog.global.CLOSURE_DEFINES;
    if (uncompiledDefines &&
        // Anti DOM-clobbering runtime check (b/37736576).
        /** @type {?} */ (uncompiledDefines).nodeType === undefined &&
        Object.prototype.hasOwnProperty.call(uncompiledDefines, name)) {
      value = uncompiledDefines[name];
    } else if (
        defines &&
        // Anti DOM-clobbering runtime check (b/37736576).
        /** @type {?} */ (defines).nodeType === undefined &&
        Object.prototype.hasOwnProperty.call(defines, name)) {
      value = defines[name];
    }
  }
  return value;
};


/**
 * @define {number} Integer year indicating the set of browser features that are
 * guaranteed to be present.  This is defined to include exactly features that
 * work correctly on all "modern" browsers that are stable on January 1 of the
 * specified year.  For example,
 * ```js
 * if (goog.FEATURESET_YEAR >= 2019) {
 *   // use APIs known to be available on all major stable browsers Jan 1, 2019
 * } else {
 *   // polyfill for older browsers
 * }
 * ```
 * This is intended to be the primary define for removing
 * unnecessary browser compatibility code (such as ponyfills and workarounds),
 * and should inform the default value for most other defines:
 * ```js
 * const ASSUME_NATIVE_PROMISE =
 *     goog.define('ASSUME_NATIVE_PROMISE', goog.FEATURESET_YEAR >= 2016);
 * ```
 *
 * The default assumption is that IE9 is the lowest supported browser, which was
 * first available Jan 1, 2012.
 *
 * TODO(mathiasb): Reference more thorough documentation when it's available.
 */
goog.FEATURESET_YEAR = goog.define('goog.FEATURESET_YEAR', 2012);


/**
 * @define {boolean} DEBUG is provided as a convenience so that debugging code
 * that should not be included in a production. It can be easily stripped
 * by specifying --define goog.DEBUG=false to the Closure Compiler aka
 * JSCompiler. For example, most toString() methods should be declared inside an
 * "if (goog.DEBUG)" conditional because they are generally used for debugging
 * purposes and it is difficult for the JSCompiler to statically determine
 * whether they are used.
 */
goog.DEBUG = goog.define('goog.DEBUG', true);


/**
 * @define {string} LOCALE defines the locale being used for compilation. It is
 * used to select locale specific data to be compiled in js binary. BUILD rule
 * can specify this value by "--define goog.LOCALE=<locale_name>" as a compiler
 * option.
 *
 * Take into account that the locale code format is important. You should use
 * the canonical Unicode format with hyphen as a delimiter. Language must be
 * lowercase, Language Script - Capitalized, Region - UPPERCASE.
 * There are few examples: pt-BR, en, en-US, sr-Latin-BO, zh-Hans-CN.
 *
 * See more info about locale codes here:
 * http://www.unicode.org/reports/tr35/#Unicode_Language_and_Locale_Identifiers
 *
 * For language codes you should use values defined by ISO 693-1. See it here
 * http://www.w3.org/WAI/ER/IG/ert/iso639.htm. There is only one exception from
 * this rule: the Hebrew language. For legacy reasons the old code (iw) should
 * be used instead of the new code (he).
 *
 * MOE:begin_intracomment_strip
 * See http://g3doc/i18n/identifiers/g3doc/synonyms.
 * MOE:end_intracomment_strip
 */
goog.LOCALE = goog.define('goog.LOCALE', 'en');  // default to en


/**
 * @define {boolean} Whether this code is running on trusted sites.
 *
 * On untrusted sites, several native functions can be defined or overridden by
 * external libraries like Prototype, Datejs, and JQuery and setting this flag
 * to false forces closure to use its own implementations when possible.
 *
 * If your JavaScript can be loaded by a third party site and you are wary about
 * relying on non-standard implementations, specify
 * "--define goog.TRUSTED_SITE=false" to the compiler.
 */
goog.TRUSTED_SITE = goog.define('goog.TRUSTED_SITE', true);


/**
 * @define {boolean} Whether a project is expected to be running in strict mode.
 *
 * This define can be used to trigger alternate implementations compatible with
 * running in EcmaScript Strict mode or warn about unavailable functionality.
 * @see https://goo.gl/PudQ4y
 *
 */
goog.STRICT_MODE_COMPATIBLE = goog.define('goog.STRICT_MODE_COMPATIBLE', false);


/**
 * @define {boolean} Whether code that calls {@link goog.setTestOnly} should
 *     be disallowed in the compilation unit.
 */
goog.DISALLOW_TEST_ONLY_CODE =
    goog.define('goog.DISALLOW_TEST_ONLY_CODE', COMPILED && !goog.DEBUG);


/**
 * @define {boolean} Whether to use a Chrome app CSP-compliant method for
 *     loading scripts via goog.require. @see appendScriptSrcNode_.
 */
goog.ENABLE_CHROME_APP_SAFE_SCRIPT_LOADING =
    goog.define('goog.ENABLE_CHROME_APP_SAFE_SCRIPT_LOADING', false);


/**
 * Defines a namespace in Closure.
 *
 * A namespace may only be defined once in a codebase. It may be defined using
 * goog.provide() or goog.module().
 *
 * The presence of one or more goog.provide() calls in a file indicates
 * that the file defines the given objects/namespaces.
 * Provided symbols must not be null or undefined.
 *
 * In addition, goog.provide() creates the object stubs for a namespace
 * (for example, goog.provide("goog.foo.bar") will create the object
 * goog.foo.bar if it does not already exist).
 *
 * Build tools also scan for provide/require/module statements
 * to discern dependencies, build dependency files (see deps.js), etc.
 *
 * @see goog.require
 * @see goog.module
 * @param {string} name Namespace provided by this file in the form
 *     "goog.package.part".
 */
goog.provide = function(name) {
  if (goog.isInModuleLoader_()) {
    throw new Error('goog.provide cannot be used within a module.');
  }
  if (!COMPILED) {
    // Ensure that the same namespace isn't provided twice.
    // A goog.module/goog.provide maps a goog.require to a specific file
    if (goog.isProvided_(name)) {
      throw new Error('Namespace "' + name + '" already declared.');
    }
  }

  goog.constructNamespace_(name);
};


/**
 * @param {string} name Namespace provided by this file in the form
 *     "goog.package.part".
 * @param {Object=} opt_obj The object to embed in the namespace.
 * @private
 */
goog.constructNamespace_ = function(name, opt_obj) {
  if (!COMPILED) {
    delete goog.implicitNamespaces_[name];

    var namespace = name;
    while ((namespace = namespace.substring(0, namespace.lastIndexOf('.')))) {
      if (goog.getObjectByName(namespace)) {
        break;
      }
      goog.implicitNamespaces_[namespace] = true;
    }
  }

  goog.exportPath_(name, opt_obj);
};


/**
 * Returns CSP nonce, if set for any script tag.
 * @param {?Window=} opt_window The window context used to retrieve the nonce.
 *     Defaults to global context.
 * @return {string} CSP nonce or empty string if no nonce is present.
 */
goog.getScriptNonce = function(opt_window) {
  if (opt_window && opt_window != goog.global) {
    return goog.getScriptNonce_(opt_window.document);
  }
  if (goog.cspNonce_ === null) {
    goog.cspNonce_ = goog.getScriptNonce_(goog.global.document);
  }
  return goog.cspNonce_;
};


/**
 * According to the CSP3 spec a nonce must be a valid base64 string.
 * @see https://www.w3.org/TR/CSP3/#grammardef-base64-value
 * @private @const
 */
goog.NONCE_PATTERN_ = /^[\w+/_-]+[=]{0,2}$/;


/**
 * @private {?string}
 */
goog.cspNonce_ = null;


/**
 * Returns CSP nonce, if set for any script tag.
 * @param {!Document} doc
 * @return {string} CSP nonce or empty string if no nonce is present.
 * @private
 */
goog.getScriptNonce_ = function(doc) {
  var script = doc.querySelector && doc.querySelector('script[nonce]');
  if (script) {
    // Try to get the nonce from the IDL property first, because browsers that
    // implement additional nonce protection features (currently only Chrome) to
    // prevent nonce stealing via CSS do not expose the nonce via attributes.
    // See https://github.com/whatwg/html/issues/2369
    var nonce = script['nonce'] || script.getAttribute('nonce');
    if (nonce && goog.NONCE_PATTERN_.test(nonce)) {
      return nonce;
    }
  }
  return '';
};


/**
 * Module identifier validation regexp.
 * Note: This is a conservative check, it is very possible to be more lenient,
 *   the primary exclusion here is "/" and "\" and a leading ".", these
 *   restrictions are intended to leave the door open for using goog.require
 *   with relative file paths rather than module identifiers.
 * @private
 */
goog.VALID_MODULE_RE_ = /^[a-zA-Z_$][a-zA-Z0-9._$]*$/;


/**
 * Defines a module in Closure.
 *
 * Marks that this file must be loaded as a module and claims the namespace.
 *
 * A namespace may only be defined once in a codebase. It may be defined using
 * goog.provide() or goog.module().
 *
 * goog.module() has three requirements:
 * - goog.module may not be used in the same file as goog.provide.
 * - goog.module must be the first statement in the file.
 * - only one goog.module is allowed per file.
 *
 * When a goog.module annotated file is loaded, it is enclosed in
 * a strict function closure. This means that:
 * - any variables declared in a goog.module file are private to the file
 * (not global), though the compiler is expected to inline the module.
 * - The code must obey all the rules of "strict" JavaScript.
 * - the file will be marked as "use strict"
 *
 * NOTE: unlike goog.provide, goog.module does not declare any symbols by
 * itself. If declared symbols are desired, use
 * goog.module.declareLegacyNamespace().
 *
 * MOE:begin_intracomment_strip
 * See the goog.module announcement at http://go/goog.module-announce
 * MOE:end_intracomment_strip
 *
 * See the public goog.module proposal: http://goo.gl/Va1hin
 *
 * @param {string} name Namespace provided by this file in the form
 *     "goog.package.part", is expected but not required.
 * @return {void}
 */
goog.module = function(name) {
  if (typeof name !== 'string' || !name ||
      name.search(goog.VALID_MODULE_RE_) == -1) {
    throw new Error('Invalid module identifier');
  }
  if (!goog.isInGoogModuleLoader_()) {
    throw new Error(
        'Module ' + name + ' has been loaded incorrectly. Note, ' +
        'modules cannot be loaded as normal scripts. They require some kind of ' +
        'pre-processing step. You\'re likely trying to load a module via a ' +
        'script tag or as a part of a concatenated bundle without rewriting the ' +
        'module. For more info see: ' +
        'https://github.com/google/closure-library/wiki/goog.module:-an-ES6-module-like-alternative-to-goog.provide.');
  }
  if (goog.moduleLoaderState_.moduleName) {
    throw new Error('goog.module may only be called once per module.');
  }

  // Store the module name for the loader.
  goog.moduleLoaderState_.moduleName = name;
  if (!COMPILED) {
    // Ensure that the same namespace isn't provided twice.
    // A goog.module/goog.provide maps a goog.require to a specific file
    if (goog.isProvided_(name)) {
      throw new Error('Namespace "' + name + '" already declared.');
    }
    delete goog.implicitNamespaces_[name];
  }
};


/**
 * @param {string} name The module identifier.
 * @return {?} The module exports for an already loaded module or null.
 *
 * Note: This is not an alternative to goog.require, it does not
 * indicate a hard dependency, instead it is used to indicate
 * an optional dependency or to access the exports of a module
 * that has already been loaded.
 * @suppress {missingProvide}
 */
goog.module.get = function(name) {
  return goog.module.getInternal_(name);
};


/**
 * @param {string} name The module identifier.
 * @return {?} The module exports for an already loaded module or null.
 * @private
 */
goog.module.getInternal_ = function(name) {
  if (!COMPILED) {
    if (name in goog.loadedModules_) {
      return goog.loadedModules_[name].exports;
    } else if (!goog.implicitNamespaces_[name]) {
      var ns = goog.getObjectByName(name);
      return ns != null ? ns : null;
    }
  }
  return null;
};


/**
 * Types of modules the debug loader can load.
 * @enum {string}
 */
goog.ModuleType = {
  ES6: 'es6',
  GOOG: 'goog'
};


/**
 * @private {?{
 *   moduleName: (string|undefined),
 *   declareLegacyNamespace:boolean,
 *   type: ?goog.ModuleType
 * }}
 */
goog.moduleLoaderState_ = null;


/**
 * @private
 * @return {boolean} Whether a goog.module or an es6 module is currently being
 *     initialized.
 */
goog.isInModuleLoader_ = function() {
  return goog.isInGoogModuleLoader_() || goog.isInEs6ModuleLoader_();
};


/**
 * @private
 * @return {boolean} Whether a goog.module is currently being initialized.
 */
goog.isInGoogModuleLoader_ = function() {
  return !!goog.moduleLoaderState_ &&
      goog.moduleLoaderState_.type == goog.ModuleType.GOOG;
};


/**
 * @private
 * @return {boolean} Whether an es6 module is currently being initialized.
 */
goog.isInEs6ModuleLoader_ = function() {
  var inLoader = !!goog.moduleLoaderState_ &&
      goog.moduleLoaderState_.type == goog.ModuleType.ES6;

  if (inLoader) {
    return true;
  }

  var jscomp = goog.global['$jscomp'];

  if (jscomp) {
    // jscomp may not have getCurrentModulePath if this is a compiled bundle
    // that has some of the runtime, but not all of it. This can happen if
    // optimizations are turned on so the unused runtime is removed but renaming
    // and Closure pass are off (so $jscomp is still named $jscomp and the
    // goog.provide/require calls still exist).
    if (typeof jscomp.getCurrentModulePath != 'function') {
      return false;
    }

    // Bundled ES6 module.
    return !!jscomp.getCurrentModulePath();
  }

  return false;
};


/**
 * Provide the module's exports as a globally accessible object under the
 * module's declared name.  This is intended to ease migration to goog.module
 * for files that have existing usages.
 * @suppress {missingProvide}
 */
goog.module.declareLegacyNamespace = function() {
  if (!COMPILED && !goog.isInGoogModuleLoader_()) {
    throw new Error(
        'goog.module.declareLegacyNamespace must be called from ' +
        'within a goog.module');
  }
  if (!COMPILED && !goog.moduleLoaderState_.moduleName) {
    throw new Error(
        'goog.module must be called prior to ' +
        'goog.module.declareLegacyNamespace.');
  }
  goog.moduleLoaderState_.declareLegacyNamespace = true;
};


/**
 * Associates an ES6 module with a Closure module ID so that is available via
 * goog.require. The associated ID  acts like a goog.module ID - it does not
 * create any global names, it is merely available via goog.require /
 * goog.module.get / goog.forwardDeclare / goog.requireType. goog.require and
 * goog.module.get will return the entire module as if it was import *'d. This
 * allows Closure files to reference ES6 modules for the sake of migration.
 *
 * @param {string} namespace
 * @suppress {missingProvide}
 */
goog.declareModuleId = function(namespace) {
  if (!COMPILED) {
    if (!goog.isInEs6ModuleLoader_()) {
      throw new Error(
          'goog.declareModuleId may only be called from ' +
          'within an ES6 module');
    }
    if (goog.moduleLoaderState_ && goog.moduleLoaderState_.moduleName) {
      throw new Error(
          'goog.declareModuleId may only be called once per module.');
    }
    if (namespace in goog.loadedModules_) {
      throw new Error(
          'Module with namespace "' + namespace + '" already exists.');
    }
  }
  if (goog.moduleLoaderState_) {
    // Not bundled - debug loading.
    goog.moduleLoaderState_.moduleName = namespace;
  } else {
    // Bundled - not debug loading, no module loader state.
    var jscomp = goog.global['$jscomp'];
    if (!jscomp || typeof jscomp.getCurrentModulePath != 'function') {
      throw new Error(
          'Module with namespace "' + namespace +
          '" has been loaded incorrectly.');
    }
    var exports = jscomp.require(jscomp.getCurrentModulePath());
    goog.loadedModules_[namespace] = {
      exports: exports,
      type: goog.ModuleType.ES6,
      moduleId: namespace
    };
  }
};


/**
 * Marks that the current file should only be used for testing, and never for
 * live code in production.
 *
 * In the case of unit tests, the message may optionally be an exact namespace
 * for the test (e.g. 'goog.stringTest'). The linter will then ignore the extra
 * provide (if not explicitly defined in the code).
 *
 * @param {string=} opt_message Optional message to add to the error that's
 *     raised when used in production code.
 */
goog.setTestOnly = function(opt_message) {
  if (goog.DISALLOW_TEST_ONLY_CODE) {
    opt_message = opt_message || '';
    throw new Error(
        'Importing test-only code into non-debug environment' +
        (opt_message ? ': ' + opt_message : '.'));
  }
};


/**
 * Forward declares a symbol. This is an indication to the compiler that the
 * symbol may be used in the source yet is not required and may not be provided
 * in compilation.
 *
 * The most common usage of forward declaration is code that takes a type as a
 * function parameter but does not need to require it. By forward declaring
 * instead of requiring, no hard dependency is made, and (if not required
 * elsewhere) the namespace may never be required and thus, not be pulled
 * into the JavaScript binary. If it is required elsewhere, it will be type
 * checked as normal.
 *
 * Before using goog.forwardDeclare, please read the documentation at
 * https://github.com/google/closure-compiler/wiki/Bad-Type-Annotation to
 * understand the options and tradeoffs when working with forward declarations.
 *
 * @param {string} name The namespace to forward declare in the form of
 *     "goog.package.part".
 * @deprecated See go/noforwarddeclaration, Use `goog.requireType` instead.
 */
goog.forwardDeclare = function(name) {};


/**
 * Forward declare type information. Used to assign types to goog.global
 * referenced object that would otherwise result in unknown type references
 * and thus block property disambiguation.
 */
goog.forwardDeclare('Document');
goog.forwardDeclare('HTMLScriptElement');
goog.forwardDeclare('XMLHttpRequest');


if (!COMPILED) {
  /**
   * Check if the given name has been goog.provided. This will return false for
   * names that are available only as implicit namespaces.
   * @param {string} name name of the object to look for.
   * @return {boolean} Whether the name has been provided.
   * @private
   */
  goog.isProvided_ = function(name) {
    return (name in goog.loadedModules_) ||
        (!goog.implicitNamespaces_[name] && goog.getObjectByName(name) != null);
  };

  /**
   * Namespaces implicitly defined by goog.provide. For example,
   * goog.provide('goog.events.Event') implicitly declares that 'goog' and
   * 'goog.events' must be namespaces.
   *
   * @type {!Object<string, (boolean|undefined)>}
   * @private
   */
  goog.implicitNamespaces_ = {'goog.module': true};

  // NOTE: We add goog.module as an implicit namespace as goog.module is defined
  // here and because the existing module package has not been moved yet out of
  // the goog.module namespace. This satisifies both the debug loader and
  // ahead-of-time dependency management.
}


/**
 * Returns an object based on its fully qualified external name.  The object
 * is not found if null or undefined.  If you are using a compilation pass that
 * renames property names beware that using this function will not find renamed
 * properties.
 *
 * @param {string} name The fully qualified name.
 * @param {Object=} opt_obj The object within which to look; default is
 *     |goog.global|.
 * @return {?} The value (object or primitive) or, if not found, null.
 */
goog.getObjectByName = function(name, opt_obj) {
  var parts = name.split('.');
  var cur = opt_obj || goog.global;
  for (var i = 0; i < parts.length; i++) {
    cur = cur[parts[i]];
    if (cur == null) {
      return null;
    }
  }
  return cur;
};


/**
 * Globalizes a whole namespace, such as goog or goog.lang.
 *
 * @param {!Object} obj The namespace to globalize.
 * @param {Object=} opt_global The object to add the properties to.
 * @deprecated Properties may be explicitly exported to the global scope, but
 *     this should no longer be done in bulk.
 */
goog.globalize = function(obj, opt_global) {
  var global = opt_global || goog.global;
  for (var x in obj) {
    global[x] = obj[x];
  }
};


/**
 * Adds a dependency from a file to the files it requires.
 * @param {string} relPath The path to the js file.
 * @param {!Array<string>} provides An array of strings with
 *     the names of the objects this file provides.
 * @param {!Array<string>} requires An array of strings with
 *     the names of the objects this file requires.
 * @param {boolean|!Object<string>=} opt_loadFlags Parameters indicating
 *     how the file must be loaded.  The boolean 'true' is equivalent
 *     to {'module': 'goog'} for backwards-compatibility.  Valid properties
 *     and values include {'module': 'goog'} and {'lang': 'es6'}.
 */
goog.addDependency = function(relPath, provides, requires, opt_loadFlags) {
  if (!COMPILED && goog.DEPENDENCIES_ENABLED) {
    goog.debugLoader_.addDependency(relPath, provides, requires, opt_loadFlags);
  }
};


// MOE:begin_strip
/**
 * Whether goog.require should throw an exception if it fails.
 * @type {boolean}
 */
goog.useStrictRequires = false;


// MOE:end_strip


// NOTE(nnaze): The debug DOM loader was included in base.js as an original way
// to do "debug-mode" development.  The dependency system can sometimes be
// confusing, as can the debug DOM loader's asynchronous nature.
//
// With the DOM loader, a call to goog.require() is not blocking -- the script
// will not load until some point after the current script.  If a namespace is
// needed at runtime, it needs to be defined in a previous script, or loaded via
// require() with its registered dependencies.
//
// User-defined namespaces may need their own deps file. For a reference on
// creating a deps file, see:
// MOE:begin_strip
// Internally: http://go/deps-files and http://go/be#js_deps
// MOE:end_strip
// Externally: https://developers.google.com/closure/library/docs/depswriter
//
// Because of legacy clients, the DOM loader can't be easily removed from
// base.js.  Work was done to make it disableable or replaceable for
// different environments (DOM-less JavaScript interpreters like Rhino or V8,
// for example). See bootstrap/ for more information.


/**
 * @define {boolean} Whether to enable the debug loader.
 *
 * If enabled, a call to goog.require() will attempt to load the namespace by
 * appending a script tag to the DOM (if the namespace has been registered).
 *
 * If disabled, goog.require() will simply assert that the namespace has been
 * provided (and depend on the fact that some outside tool correctly ordered
 * the script).
 */
goog.ENABLE_DEBUG_LOADER = goog.define('goog.ENABLE_DEBUG_LOADER', true);


/**
 * @param {string} msg
 * @private
 */
goog.logToConsole_ = function(msg) {
  if (goog.global.console) {
    goog.global.console['error'](msg);
  }
};


/**
 * Implements a system for the dynamic resolution of dependencies that works in
 * parallel with the BUILD system.
 *
 * Note that all calls to goog.require will be stripped by the compiler.
 *
 * @see goog.provide
 * @param {string} namespace Namespace (as was given in goog.provide,
 *     goog.module, or goog.declareModuleId) in the form
 *     "goog.package.part".
 * @return {?} If called within a goog.module or ES6 module file, the associated
 *     namespace or module otherwise null.
 */
goog.require = function(namespace) {
  if (!COMPILED) {
    // Might need to lazy load on old IE.
    if (goog.ENABLE_DEBUG_LOADER) {
      goog.debugLoader_.requested(namespace);
    }

    // If the object already exists we do not need to do anything.
    if (goog.isProvided_(namespace)) {
      if (goog.isInModuleLoader_()) {
        return goog.module.getInternal_(namespace);
      }
    } else if (goog.ENABLE_DEBUG_LOADER) {
      var moduleLoaderState = goog.moduleLoaderState_;
      goog.moduleLoaderState_ = null;
      try {
        goog.debugLoader_.load_(namespace);
      } finally {
        goog.moduleLoaderState_ = moduleLoaderState;
      }
    }

    return null;
  }
};


/**
 * Requires a symbol for its type information. This is an indication to the
 * compiler that the symbol may appear in type annotations, yet it is not
 * referenced at runtime.
 *
 * When called within a goog.module or ES6 module file, the return value may be
 * assigned to or destructured into a variable, but it may not be otherwise used
 * in code outside of a type annotation.
 *
 * Note that all calls to goog.requireType will be stripped by the compiler.
 *
 * @param {string} namespace Namespace (as was given in goog.provide,
 *     goog.module, or goog.declareModuleId) in the form
 *     "goog.package.part".
 * @return {?}
 */
goog.requireType = function(namespace) {
  // Return an empty object so that single-level destructuring of the return
  // value doesn't crash at runtime when using the debug loader. Multi-level
  // destructuring isn't supported.
  return {};
};


/**
 * Path for included scripts.
 * @type {string}
 */
goog.basePath = '';


/**
 * A hook for overriding the base path.
 * @type {string|undefined}
 */
goog.global.CLOSURE_BASE_PATH;


/**
 * Whether to attempt to load Closure's deps file. By default, when uncompiled,
 * deps files will attempt to be loaded.
 * @type {boolean|undefined}
 */
goog.global.CLOSURE_NO_DEPS;


/**
 * A function to import a single script. This is meant to be overridden when
 * Closure is being run in non-HTML contexts, such as web workers. It's defined
 * in the global scope so that it can be set before base.js is loaded, which
 * allows deps.js to be imported properly.
 *
 * The first parameter the script source, which is a relative URI. The second,
 * optional parameter is the script contents, in the event the script needed
 * transformation. It should return true if the script was imported, false
 * otherwise.
 * @type {(function(string, string=): boolean)|undefined}
 */
goog.global.CLOSURE_IMPORT_SCRIPT;


/**
 * Null function used for default values of callbacks, etc.
 * @return {void} Nothing.
 */
goog.nullFunction = function() {};


/**
 * When defining a class Foo with an abstract method bar(), you can do:
 * Foo.prototype.bar = goog.abstractMethod
 *
 * Now if a subclass of Foo fails to override bar(), an error will be thrown
 * when bar() is invoked.
 *
 * @type {!Function}
 * @throws {Error} when invoked to indicate the method should be overridden.
 * @deprecated Use "@abstract" annotation instead of goog.abstractMethod in new
 *     code. See
 *     https://github.com/google/closure-compiler/wiki/@abstract-classes-and-methods
 */
goog.abstractMethod = function() {
  throw new Error('unimplemented abstract method');
};


/**
 * Adds a `getInstance` static method that always returns the same
 * instance object.
 * @param {!Function} ctor The constructor for the class to add the static
 *     method to.
 * @suppress {missingProperties} 'instance_' isn't a property on 'Function'
 *     but we don't have a better type to use here.
 */
goog.addSingletonGetter = function(ctor) {
  // instance_ is immediately set to prevent issues with sealed constructors
  // such as are encountered when a constructor is returned as the export object
  // of a goog.module in unoptimized code.
  // Delcare type to avoid conformance violations that ctor.instance_ is unknown
  /** @type {undefined|!Object} @suppress {underscore} */
  ctor.instance_ = undefined;
  ctor.getInstance = function() {
    if (ctor.instance_) {
      return ctor.instance_;
    }
    if (goog.DEBUG) {
      // NOTE: JSCompiler can't optimize away Array#push.
      goog.instantiatedSingletons_[goog.instantiatedSingletons_.length] = ctor;
    }
    // Cast to avoid conformance violations that ctor.instance_ is unknown
    return /** @type {!Object|undefined} */ (ctor.instance_) = new ctor;
  };
};


/**
 * All singleton classes that have been instantiated, for testing. Don't read
 * it directly, use the `goog.testing.singleton` module. The compiler
 * removes this variable if unused.
 * @type {!Array<!Function>}
 * @private
 */
goog.instantiatedSingletons_ = [];


/**
 * @define {boolean} Whether to load goog.modules using `eval` when using
 * the debug loader.  This provides a better debugging experience as the
 * source is unmodified and can be edited using Chrome Workspaces or similar.
 * However in some environments the use of `eval` is banned
 * so we provide an alternative.
 */
goog.LOAD_MODULE_USING_EVAL = goog.define('goog.LOAD_MODULE_USING_EVAL', true);


/**
 * @define {boolean} Whether the exports of goog.modules should be sealed when
 * possible.
 */
goog.SEAL_MODULE_EXPORTS = goog.define('goog.SEAL_MODULE_EXPORTS', goog.DEBUG);


/**
 * The registry of initialized modules:
 * The module identifier or path to module exports map.
 * @private @const {!Object<string, {exports:?,type:string,moduleId:string}>}
 */
goog.loadedModules_ = {};


/**
 * True if the debug loader enabled and used.
 * @const {boolean}
 */
goog.DEPENDENCIES_ENABLED = !COMPILED && goog.ENABLE_DEBUG_LOADER;


/**
 * @define {string} How to decide whether to transpile.  Valid values
 * are 'always', 'never', and 'detect'.  The default ('detect') is to
 * use feature detection to determine which language levels need
 * transpilation.
 */
// NOTE(sdh): we could expand this to accept a language level to bypass
// detection: e.g. goog.TRANSPILE == 'es5' would transpile ES6 files but
// would leave ES3 and ES5 files alone.
goog.TRANSPILE = goog.define('goog.TRANSPILE', 'detect');

/**
 * @define {boolean} If true assume that ES modules have already been
 * transpiled by the jscompiler (in the same way that transpile.js would
 * transpile them - to jscomp modules). Useful only for servers that wish to use
 * the debug loader and transpile server side. Thus this is only respected if
 * goog.TRANSPILE is "never".
 */
goog.ASSUME_ES_MODULES_TRANSPILED =
    goog.define('goog.ASSUME_ES_MODULES_TRANSPILED', false);


/**
 * @define {string} If a file needs to be transpiled what the output language
 * should be. By default this is the highest language level this file detects
 * the current environment supports. Generally this flag should not be set, but
 * it could be useful to override. Example: If the current environment supports
 * ES6 then by default ES7+ files will be transpiled to ES6, unless this is
 * overridden.
 *
 * Valid values include: es3, es5, es6, es7, and es8. Anything not recognized
 * is treated as es3.
 *
 * Note that setting this value does not force transpilation. Just if
 * transpilation occurs this will be the output. So this is most useful when
 * goog.TRANSPILE is set to 'always' and then forcing the language level to be
 * something lower than what the environment detects.
 */
goog.TRANSPILE_TO_LANGUAGE = goog.define('goog.TRANSPILE_TO_LANGUAGE', '');


/**
 * @define {string} Path to the transpiler.  Executing the script at this
 * path (relative to base.js) should define a function $jscomp.transpile.
 */
goog.TRANSPILER = goog.define('goog.TRANSPILER', 'transpile.js');


/**
 * @package {?boolean}
 * Visible for testing.
 */
goog.hasBadLetScoping = null;


/**
 * @return {boolean}
 * @package Visible for testing.
 */
goog.useSafari10Workaround = function() {
  if (goog.hasBadLetScoping == null) {
    var hasBadLetScoping;
    try {
      hasBadLetScoping = !eval(
          '"use strict";' +
          'let x = 1; function f() { return typeof x; };' +
          'f() == "number";');
    } catch (e) {
      // Assume that ES6 syntax isn't supported.
      hasBadLetScoping = false;
    }
    goog.hasBadLetScoping = hasBadLetScoping;
  }
  return goog.hasBadLetScoping;
};


/**
 * @param {string} moduleDef
 * @return {string}
 * @package Visible for testing.
 */
goog.workaroundSafari10EvalBug = function(moduleDef) {
  return '(function(){' + moduleDef +
      '\n' +  // Terminate any trailing single line comment.
      ';' +   // Terminate any trailing expression.
      '})();\n';
};


/**
 * @param {function(?):?|string} moduleDef The module definition.
 */
goog.loadModule = function(moduleDef) {
  // NOTE: we allow function definitions to be either in the from
  // of a string to eval (which keeps the original source intact) or
  // in a eval forbidden environment (CSP) we allow a function definition
  // which in its body must call `goog.module`, and return the exports
  // of the module.
  var previousState = goog.moduleLoaderState_;
  try {
    goog.moduleLoaderState_ = {
      moduleName: '',
      declareLegacyNamespace: false,
      type: goog.ModuleType.GOOG
    };
    var exports;
    if (goog.isFunction(moduleDef)) {
      exports = moduleDef.call(undefined, {});
    } else if (typeof moduleDef === 'string') {
      if (goog.useSafari10Workaround()) {
        moduleDef = goog.workaroundSafari10EvalBug(moduleDef);
      }

      exports = goog.loadModuleFromSource_.call(undefined, moduleDef);
    } else {
      throw new Error('Invalid module definition');
    }

    var moduleName = goog.moduleLoaderState_.moduleName;
    if (typeof moduleName === 'string' && moduleName) {
      // Don't seal legacy namespaces as they may be used as a parent of
      // another namespace
      if (goog.moduleLoaderState_.declareLegacyNamespace) {
        goog.constructNamespace_(moduleName, exports);
      } else if (
          goog.SEAL_MODULE_EXPORTS && Object.seal &&
          typeof exports == 'object' && exports != null) {
        Object.seal(exports);
      }

      var data = {
        exports: exports,
        type: goog.ModuleType.GOOG,
        moduleId: goog.moduleLoaderState_.moduleName
      };
      goog.loadedModules_[moduleName] = data;
    } else {
      throw new Error('Invalid module name \"' + moduleName + '\"');
    }
  } finally {
    goog.moduleLoaderState_ = previousState;
  }
};


/**
 * @private @const
 */
goog.loadModuleFromSource_ = /** @type {function(string):?} */ (function() {
  // NOTE: we avoid declaring parameters or local variables here to avoid
  // masking globals or leaking values into the module definition.
  'use strict';
  var exports = {};
  eval(arguments[0]);
  return exports;
});


/**
 * Normalize a file path by removing redundant ".." and extraneous "." file
 * path components.
 * @param {string} path
 * @return {string}
 * @private
 */
goog.normalizePath_ = function(path) {
  var components = path.split('/');
  var i = 0;
  while (i < components.length) {
    if (components[i] == '.') {
      components.splice(i, 1);
    } else if (
        i && components[i] == '..' && components[i - 1] &&
        components[i - 1] != '..') {
      components.splice(--i, 2);
    } else {
      i++;
    }
  }
  return components.join('/');
};


/**
 * Provides a hook for loading a file when using Closure's goog.require() API
 * with goog.modules.  In particular this hook is provided to support Node.js.
 *
 * @type {(function(string):string)|undefined}
 */
goog.global.CLOSURE_LOAD_FILE_SYNC;


/**
 * Loads file by synchronous XHR. Should not be used in production environments.
 * @param {string} src Source URL.
 * @return {?string} File contents, or null if load failed.
 * @private
 */
goog.loadFileSync_ = function(src) {
  if (goog.global.CLOSURE_LOAD_FILE_SYNC) {
    return goog.global.CLOSURE_LOAD_FILE_SYNC(src);
  } else {
    try {
      /** @type {XMLHttpRequest} */
      var xhr = new goog.global['XMLHttpRequest']();
      xhr.open('get', src, false);
      xhr.send();
      // NOTE: Successful http: requests have a status of 200, but successful
      // file: requests may have a status of zero.  Any other status, or a
      // thrown exception (particularly in case of file: requests) indicates
      // some sort of error, which we treat as a missing or unavailable file.
      return xhr.status == 0 || xhr.status == 200 ? xhr.responseText : null;
    } catch (err) {
      // No need to rethrow or log, since errors should show up on their own.
      return null;
    }
  }
};


/**
 * Lazily retrieves the transpiler and applies it to the source.
 * @param {string} code JS code.
 * @param {string} path Path to the code.
 * @param {string} target Language level output.
 * @return {string} The transpiled code.
 * @private
 */
goog.transpile_ = function(code, path, target) {
  var jscomp = goog.global['$jscomp'];
  if (!jscomp) {
    goog.global['$jscomp'] = jscomp = {};
  }
  var transpile = jscomp.transpile;
  if (!transpile) {
    var transpilerPath = goog.basePath + goog.TRANSPILER;
    var transpilerCode = goog.loadFileSync_(transpilerPath);
    if (transpilerCode) {
      // This must be executed synchronously, since by the time we know we
      // need it, we're about to load and write the ES6 code synchronously,
      // so a normal script-tag load will be too slow. Wrapped in a function
      // so that code is eval'd in the global scope.
      (function() {
        (0, eval)(transpilerCode + '\n//# sourceURL=' + transpilerPath);
      }).call(goog.global);
      // Even though the transpiler is optional, if $gwtExport is found, it's
      // a sign the transpiler was loaded and the $jscomp.transpile *should*
      // be there.
      if (goog.global['$gwtExport'] && goog.global['$gwtExport']['$jscomp'] &&
          !goog.global['$gwtExport']['$jscomp']['transpile']) {
        throw new Error(
            'The transpiler did not properly export the "transpile" ' +
            'method. $gwtExport: ' + JSON.stringify(goog.global['$gwtExport']));
      }
      // transpile.js only exports a single $jscomp function, transpile. We
      // grab just that and add it to the existing definition of $jscomp which
      // contains the polyfills.
      goog.global['$jscomp'].transpile =
          goog.global['$gwtExport']['$jscomp']['transpile'];
      jscomp = goog.global['$jscomp'];
      transpile = jscomp.transpile;
    }
  }
  if (!transpile) {
    // The transpiler is an optional component.  If it's not available then
    // replace it with a pass-through function that simply logs.
    var suffix = ' requires transpilation but no transpiler was found.';
    // MOE:begin_strip
    suffix +=  // Provide a more appropriate message internally.
        ' Please add "//javascript/closure:transpiler" as a data ' +
        'dependency to ensure it is included.';
    // MOE:end_strip
    transpile = jscomp.transpile = function(code, path) {
      // TODO(sdh): figure out some way to get this error to show up
      // in test results, noting that the failure may occur in many
      // different ways, including in loadModule() before the test
      // runner even comes up.
      goog.logToConsole_(path + suffix);
      return code;
    };
  }
  // Note: any transpilation errors/warnings will be logged to the console.
  return transpile(code, path, target);
};

//==============================================================================
// Language Enhancements
//==============================================================================


/**
 * This is a "fixed" version of the typeof operator.  It differs from the typeof
 * operator in such a way that null returns 'null' and arrays return 'array'.
 * @param {?} value The value to get the type of.
 * @return {string} The name of the type.
 */
goog.typeOf = function(value) {
  var s = typeof value;
  if (s == 'object') {
    if (value) {
      // Check these first, so we can avoid calling Object.prototype.toString if
      // possible.
      //
      // IE9 and below improperly marshals typeof across execution contexts, but
      // a cross-context object will still return false for "instanceof Object".
      if (value instanceof Array) {
        return 'array';
      } else if (value instanceof Object) {
        return s;
      }

      // HACK: In order to use an Object prototype method on the arbitrary
      //   value, the compiler requires the value be cast to type Object,
      //   even though the ECMA spec explicitly allows it.
      var className = Object.prototype.toString.call(
          /** @type {!Object} */ (value));
      // In Firefox 3.6, attempting to access iframe window objects' length
      // property throws an NS_ERROR_FAILURE, so we need to special-case it
      // here.
      if (className == '[object Window]') {
        return 'object';
      }

      // We cannot always use constructor == Array or instanceof Array because
      // different frames have different Array objects. In IE6, if the iframe
      // where the array was created is destroyed, the array loses its
      // prototype. Then dereferencing val.splice here throws an exception, so
      // we can't use goog.isFunction. Calling typeof directly returns 'unknown'
      // so that will work. In this case, this function will return false and
      // most array functions will still work because the array is still
      // array-like (supports length and []) even though it has lost its
      // prototype.
      // Mark Miller noticed that Object.prototype.toString
      // allows access to the unforgeable [[Class]] property.
      //  15.2.4.2 Object.prototype.toString ( )
      //  When the toString method is called, the following steps are taken:
      //      1. Get the [[Class]] property of this object.
      //      2. Compute a string value by concatenating the three strings
      //         "[object ", Result(1), and "]".
      //      3. Return Result(2).
      // and this behavior survives the destruction of the execution context.
      if ((className == '[object Array]' ||
           // In IE all non value types are wrapped as objects across window
           // boundaries (not iframe though) so we have to do object detection
           // for this edge case.
           typeof value.length == 'number' &&
               typeof value.splice != 'undefined' &&
               typeof value.propertyIsEnumerable != 'undefined' &&
               !value.propertyIsEnumerable('splice')

               )) {
        return 'array';
      }
      // HACK: There is still an array case that fails.
      //     function ArrayImpostor() {}
      //     ArrayImpostor.prototype = [];
      //     var impostor = new ArrayImpostor;
      // this can be fixed by getting rid of the fast path
      // (value instanceof Array) and solely relying on
      // (value && Object.prototype.toString.vall(value) === '[object Array]')
      // but that would require many more function calls and is not warranted
      // unless closure code is receiving objects from untrusted sources.

      // IE in cross-window calls does not correctly marshal the function type
      // (it appears just as an object) so we cannot use just typeof val ==
      // 'function'. However, if the object has a call property, it is a
      // function.
      if ((className == '[object Function]' ||
           typeof value.call != 'undefined' &&
               typeof value.propertyIsEnumerable != 'undefined' &&
               !value.propertyIsEnumerable('call'))) {
        return 'function';
      }

    } else {
      return 'null';
    }

  } else if (s == 'function' && typeof value.call == 'undefined') {
    // In Safari typeof nodeList returns 'function', and on Firefox typeof
    // behaves similarly for HTML{Applet,Embed,Object}, Elements and RegExps. We
    // would like to return object for those and we can detect an invalid
    // function by making sure that the function object has a call method.
    return 'object';
  }
  return s;
};


/**
 * Returns true if the specified value is an array.
 * @param {?} val Variable to test.
 * @return {boolean} Whether variable is an array.
 * @deprecated Use Array.isArray instead.
 */
goog.isArray = function(val) {
  return goog.typeOf(val) == 'array';
};


/**
 * Returns true if the object looks like an array. To qualify as array like
 * the value needs to be either a NodeList or an object with a Number length
 * property. Note that for this function neither strings nor functions are
 * considered "array-like".
 *
 * @param {?} val Variable to test.
 * @return {boolean} Whether variable is an array.
 */
goog.isArrayLike = function(val) {
  var type = goog.typeOf(val);
  // We do not use goog.isObject here in order to exclude function values.
  return type == 'array' || type == 'object' && typeof val.length == 'number';
};


/**
 * Returns true if the object looks like a Date. To qualify as Date-like the
 * value needs to be an object and have a getFullYear() function.
 * @param {?} val Variable to test.
 * @return {boolean} Whether variable is a like a Date.
 */
goog.isDateLike = function(val) {
  return goog.isObject(val) && typeof val.getFullYear == 'function';
};


/**
 * Returns true if the specified value is a function.
 * @param {?} val Variable to test.
 * @return {boolean} Whether variable is a function.
 */
goog.isFunction = function(val) {
  return goog.typeOf(val) == 'function';
};


/**
 * Returns true if the specified value is an object.  This includes arrays and
 * functions.
 * @param {?} val Variable to test.
 * @return {boolean} Whether variable is an object.
 */
goog.isObject = function(val) {
  var type = typeof val;
  return type == 'object' && val != null || type == 'function';
  // return Object(val) === val also works, but is slower, especially if val is
  // not an object.
};


/**
 * Gets a unique ID for an object. This mutates the object so that further calls
 * with the same object as a parameter returns the same value. The unique ID is
 * guaranteed to be unique across the current session amongst objects that are
 * passed into `getUid`. There is no guarantee that the ID is unique or
 * consistent across sessions. It is unsafe to generate unique ID for function
 * prototypes.
 *
 * @param {Object} obj The object to get the unique ID for.
 * @return {number} The unique ID for the object.
 */
goog.getUid = function(obj) {
  // TODO(arv): Make the type stricter, do not accept null.
  return Object.prototype.hasOwnProperty.call(obj, goog.UID_PROPERTY_) &&
      obj[goog.UID_PROPERTY_] ||
      (obj[goog.UID_PROPERTY_] = ++goog.uidCounter_);
};


/**
 * Whether the given object is already assigned a unique ID.
 *
 * This does not modify the object.
 *
 * @param {!Object} obj The object to check.
 * @return {boolean} Whether there is an assigned unique id for the object.
 */
goog.hasUid = function(obj) {
  return !!obj[goog.UID_PROPERTY_];
};


/**
 * Removes the unique ID from an object. This is useful if the object was
 * previously mutated using `goog.getUid` in which case the mutation is
 * undone.
 * @param {Object} obj The object to remove the unique ID field from.
 */
goog.removeUid = function(obj) {
  // TODO(arv): Make the type stricter, do not accept null.

  // In IE, DOM nodes are not instances of Object and throw an exception if we
  // try to delete.  Instead we try to use removeAttribute.
  if (obj !== null && 'removeAttribute' in obj) {
    obj.removeAttribute(goog.UID_PROPERTY_);
  }

  try {
    delete obj[goog.UID_PROPERTY_];
  } catch (ex) {
  }
};


/**
 * Name for unique ID property. Initialized in a way to help avoid collisions
 * with other closure JavaScript on the same page.
 * @type {string}
 * @private
 */
goog.UID_PROPERTY_ = 'closure_uid_' + ((Math.random() * 1e9) >>> 0);


/**
 * Counter for UID.
 * @type {number}
 * @private
 */
goog.uidCounter_ = 0;


/**
 * Adds a hash code field to an object. The hash code is unique for the
 * given object.
 * @param {Object} obj The object to get the hash code for.
 * @return {number} The hash code for the object.
 * @deprecated Use goog.getUid instead.
 */
goog.getHashCode = goog.getUid;


/**
 * Removes the hash code field from an object.
 * @param {Object} obj The object to remove the field from.
 * @deprecated Use goog.removeUid instead.
 */
goog.removeHashCode = goog.removeUid;


/**
 * Clones a value. The input may be an Object, Array, or basic type. Objects and
 * arrays will be cloned recursively.
 *
 * WARNINGS:
 * <code>goog.cloneObject</code> does not detect reference loops. Objects that
 * refer to themselves will cause infinite recursion.
 *
 * <code>goog.cloneObject</code> is unaware of unique identifiers, and copies
 * UIDs created by <code>getUid</code> into cloned results.
 *
 * @param {*} obj The value to clone.
 * @return {*} A clone of the input value.
 * @deprecated goog.cloneObject is unsafe. Prefer the goog.object methods.
 */
goog.cloneObject = function(obj) {
  var type = goog.typeOf(obj);
  if (type == 'object' || type == 'array') {
    if (typeof obj.clone === 'function') {
      return obj.clone();
    }
    var clone = type == 'array' ? [] : {};
    for (var key in obj) {
      clone[key] = goog.cloneObject(obj[key]);
    }
    return clone;
  }

  return obj;
};


/**
 * A native implementation of goog.bind.
 * @param {?function(this:T, ...)} fn A function to partially apply.
 * @param {T} selfObj Specifies the object which this should point to when the
 *     function is run.
 * @param {...*} var_args Additional arguments that are partially applied to the
 *     function.
 * @return {!Function} A partially-applied form of the function goog.bind() was
 *     invoked as a method of.
 * @template T
 * @private
 */
goog.bindNative_ = function(fn, selfObj, var_args) {
  return /** @type {!Function} */ (fn.call.apply(fn.bind, arguments));
};


/**
 * A pure-JS implementation of goog.bind.
 * @param {?function(this:T, ...)} fn A function to partially apply.
 * @param {T} selfObj Specifies the object which this should point to when the
 *     function is run.
 * @param {...*} var_args Additional arguments that are partially applied to the
 *     function.
 * @return {!Function} A partially-applied form of the function goog.bind() was
 *     invoked as a method of.
 * @template T
 * @private
 */
goog.bindJs_ = function(fn, selfObj, var_args) {
  if (!fn) {
    throw new Error();
  }

  if (arguments.length > 2) {
    var boundArgs = Array.prototype.slice.call(arguments, 2);
    return function() {
      // Prepend the bound arguments to the current arguments.
      var newArgs = Array.prototype.slice.call(arguments);
      Array.prototype.unshift.apply(newArgs, boundArgs);
      return fn.apply(selfObj, newArgs);
    };

  } else {
    return function() {
      return fn.apply(selfObj, arguments);
    };
  }
};


/**
 * Partially applies this function to a particular 'this object' and zero or
 * more arguments. The result is a new function with some arguments of the first
 * function pre-filled and the value of this 'pre-specified'.
 *
 * Remaining arguments specified at call-time are appended to the pre-specified
 * ones.
 *
 * Also see: {@link #partial}.
 *
 * Usage:
 * <pre>var barMethBound = goog.bind(myFunction, myObj, 'arg1', 'arg2');
 * barMethBound('arg3', 'arg4');</pre>
 *
 * @param {?function(this:T, ...)} fn A function to partially apply.
 * @param {T} selfObj Specifies the object which this should point to when the
 *     function is run.
 * @param {...*} var_args Additional arguments that are partially applied to the
 *     function.
 * @return {!Function} A partially-applied form of the function goog.bind() was
 *     invoked as a method of.
 * @template T
 * @suppress {deprecated} See above.
 */
goog.bind = function(fn, selfObj, var_args) {
  // TODO(nicksantos): narrow the type signature.
  if (Function.prototype.bind &&
      // NOTE(nicksantos): Somebody pulled base.js into the default Chrome
      // extension environment. This means that for Chrome extensions, they get
      // the implementation of Function.prototype.bind that calls goog.bind
      // instead of the native one. Even worse, we don't want to introduce a
      // circular dependency between goog.bind and Function.prototype.bind, so
      // we have to hack this to make sure it works correctly.
      Function.prototype.bind.toString().indexOf('native code') != -1) {
    goog.bind = goog.bindNative_;
  } else {
    goog.bind = goog.bindJs_;
  }
  return goog.bind.apply(null, arguments);
};


/**
 * Like goog.bind(), except that a 'this object' is not required. Useful when
 * the target function is already bound.
 *
 * Usage:
 * var g = goog.partial(f, arg1, arg2);
 * g(arg3, arg4);
 *
 * @param {Function} fn A function to partially apply.
 * @param {...*} var_args Additional arguments that are partially applied to fn.
 * @return {!Function} A partially-applied form of the function goog.partial()
 *     was invoked as a method of.
 */
goog.partial = function(fn, var_args) {
  var args = Array.prototype.slice.call(arguments, 1);
  return function() {
    // Clone the array (with slice()) and append additional arguments
    // to the existing arguments.
    var newArgs = args.slice();
    newArgs.push.apply(newArgs, arguments);
    return fn.apply(/** @type {?} */ (this), newArgs);
  };
};


/**
 * Copies all the members of a source object to a target object. This method
 * does not work on all browsers for all objects that contain keys such as
 * toString or hasOwnProperty. Use goog.object.extend for this purpose.
 *
 * NOTE: Some have advocated for the use of goog.mixin to setup classes
 * with multiple inheritence (traits, mixins, etc).  However, as it simply
 * uses "for in", this is not compatible with ES6 classes whose methods are
 * non-enumerable.  Changing this, would break cases where non-enumerable
 * properties are not expected.
 *
 * @param {Object} target Target.
 * @param {Object} source Source.
 * @deprecated Prefer Object.assign
 */
goog.mixin = function(target, source) {
  for (var x in source) {
    target[x] = source[x];
  }

  // For IE7 or lower, the for-in-loop does not contain any properties that are
  // not enumerable on the prototype object (for example, isPrototypeOf from
  // Object.prototype) but also it will not include 'replace' on objects that
  // extend String and change 'replace' (not that it is common for anyone to
  // extend anything except Object).
};


/**
 * @return {number} An integer value representing the number of milliseconds
 *     between midnight, January 1, 1970 and the current time.
 * @deprecated Use Date.now
 */
goog.now = (goog.TRUSTED_SITE && Date.now) || (function() {
             // Unary plus operator converts its operand to a number which in
             // the case of
             // a date is done by calling getTime().
             return +new Date();
           });


/**
 * Evals JavaScript in the global scope.  In IE this uses execScript, other
 * browsers use goog.global.eval. If goog.global.eval does not evaluate in the
 * global scope (for example, in Safari), appends a script tag instead.
 * Throws an exception if neither execScript or eval is defined.
 * @param {string} script JavaScript string.
 */
goog.globalEval = function(script) {
  if (goog.global.execScript) {
    goog.global.execScript(script, 'JavaScript');
  } else if (goog.global.eval) {
    // Test to see if eval works
    if (goog.evalWorksForGlobals_ == null) {
      try {
        goog.global.eval('var _evalTest_ = 1;');
      } catch (ignore) {
      }
      if (typeof goog.global['_evalTest_'] != 'undefined') {
        try {
          delete goog.global['_evalTest_'];
        } catch (ignore) {
          // Microsoft edge fails the deletion above in strict mode.
        }
        goog.evalWorksForGlobals_ = true;
      } else {
        goog.evalWorksForGlobals_ = false;
      }
    }

    if (goog.evalWorksForGlobals_) {
      goog.global.eval(script);
    } else {
      /** @type {!Document} */
      var doc = goog.global.document;
      var scriptElt =
          /** @type {!HTMLScriptElement} */ (doc.createElement('script'));
      scriptElt.type = 'text/javascript';
      scriptElt.defer = false;
      // Note(pupius): can't use .innerHTML since "t('<test>')" will fail and
      // .text doesn't work in Safari 2.  Therefore we append a text node.
      scriptElt.appendChild(doc.createTextNode(script));
      doc.head.appendChild(scriptElt);
      doc.head.removeChild(scriptElt);
    }
  } else {
    throw new Error('goog.globalEval not available');
  }
};


/**
 * Indicates whether or not we can call 'eval' directly to eval code in the
 * global scope. Set to a Boolean by the first call to goog.globalEval (which
 * empirically tests whether eval works for globals). @see goog.globalEval
 * @type {?boolean}
 * @private
 */
goog.evalWorksForGlobals_ = null;


/**
 * Optional map of CSS class names to obfuscated names used with
 * goog.getCssName().
 * @private {!Object<string, string>|undefined}
 * @see goog.setCssNameMapping
 */
goog.cssNameMapping_;


/**
 * Optional obfuscation style for CSS class names. Should be set to either
 * 'BY_WHOLE' or 'BY_PART' if defined.
 * @type {string|undefined}
 * @private
 * @see goog.setCssNameMapping
 */
goog.cssNameMappingStyle_;



/**
 * A hook for modifying the default behavior goog.getCssName. The function
 * if present, will receive the standard output of the goog.getCssName as
 * its input.
 *
 * @type {(function(string):string)|undefined}
 */
goog.global.CLOSURE_CSS_NAME_MAP_FN;


/**
 * Handles strings that are intended to be used as CSS class names.
 *
 * This function works in tandem with @see goog.setCssNameMapping.
 *
 * Without any mapping set, the arguments are simple joined with a hyphen and
 * passed through unaltered.
 *
 * When there is a mapping, there are two possible styles in which these
 * mappings are used. In the BY_PART style, each part (i.e. in between hyphens)
 * of the passed in css name is rewritten according to the map. In the BY_WHOLE
 * style, the full css name is looked up in the map directly. If a rewrite is
 * not specified by the map, the compiler will output a warning.
 *
 * When the mapping is passed to the compiler, it will replace calls to
 * goog.getCssName with the strings from the mapping, e.g.
 *     var x = goog.getCssName('foo');
 *     var y = goog.getCssName(this.baseClass, 'active');
 *  becomes:
 *     var x = 'foo';
 *     var y = this.baseClass + '-active';
 *
 * If one argument is passed it will be processed, if two are passed only the
 * modifier will be processed, as it is assumed the first argument was generated
 * as a result of calling goog.getCssName.
 *
 * @param {string} className The class name.
 * @param {string=} opt_modifier A modifier to be appended to the class name.
 * @return {string} The class name or the concatenation of the class name and
 *     the modifier.
 */
goog.getCssName = function(className, opt_modifier) {
  // String() is used for compatibility with compiled soy where the passed
  // className can be non-string objects.
  if (String(className).charAt(0) == '.') {
    throw new Error(
        'className passed in goog.getCssName must not start with ".".' +
        ' You passed: ' + className);
  }

  var getMapping = function(cssName) {
    return goog.cssNameMapping_[cssName] || cssName;
  };

  var renameByParts = function(cssName) {
    // Remap all the parts individually.
    var parts = cssName.split('-');
    var mapped = [];
    for (var i = 0; i < parts.length; i++) {
      mapped.push(getMapping(parts[i]));
    }
    return mapped.join('-');
  };

  var rename;
  if (goog.cssNameMapping_) {
    rename =
        goog.cssNameMappingStyle_ == 'BY_WHOLE' ? getMapping : renameByParts;
  } else {
    rename = function(a) {
      return a;
    };
  }

  var result =
      opt_modifier ? className + '-' + rename(opt_modifier) : rename(className);

  // The special CLOSURE_CSS_NAME_MAP_FN allows users to specify further
  // processing of the class name.
  if (goog.global.CLOSURE_CSS_NAME_MAP_FN) {
    return goog.global.CLOSURE_CSS_NAME_MAP_FN(result);
  }

  return result;
};


/**
 * Sets the map to check when returning a value from goog.getCssName(). Example:
 * <pre>
 * goog.setCssNameMapping({
 *   "goog": "a",
 *   "disabled": "b",
 * });
 *
 * var x = goog.getCssName('goog');
 * // The following evaluates to: "a a-b".
 * goog.getCssName('goog') + ' ' + goog.getCssName(x, 'disabled')
 * </pre>
 * When declared as a map of string literals to string literals, the JSCompiler
 * will replace all calls to goog.getCssName() using the supplied map if the
 * --process_closure_primitives flag is set.
 *
 * @param {!Object} mapping A map of strings to strings where keys are possible
 *     arguments to goog.getCssName() and values are the corresponding values
 *     that should be returned.
 * @param {string=} opt_style The style of css name mapping. There are two valid
 *     options: 'BY_PART', and 'BY_WHOLE'.
 * @see goog.getCssName for a description.
 */
goog.setCssNameMapping = function(mapping, opt_style) {
  goog.cssNameMapping_ = mapping;
  goog.cssNameMappingStyle_ = opt_style;
};


/**
 * To use CSS renaming in compiled mode, one of the input files should have a
 * call to goog.setCssNameMapping() with an object literal that the JSCompiler
 * can extract and use to replace all calls to goog.getCssName(). In uncompiled
 * mode, JavaScript code should be loaded before this base.js file that declares
 * a global variable, CLOSURE_CSS_NAME_MAPPING, which is used below. This is
 * to ensure that the mapping is loaded before any calls to goog.getCssName()
 * are made in uncompiled mode.
 *
 * A hook for overriding the CSS name mapping.
 * @type {!Object<string, string>|undefined}
 */
goog.global.CLOSURE_CSS_NAME_MAPPING;


if (!COMPILED && goog.global.CLOSURE_CSS_NAME_MAPPING) {
  // This does not call goog.setCssNameMapping() because the JSCompiler
  // requires that goog.setCssNameMapping() be called with an object literal.
  goog.cssNameMapping_ = goog.global.CLOSURE_CSS_NAME_MAPPING;
}


/**
 * Gets a localized message.
 *
 * This function is a compiler primitive. If you give the compiler a localized
 * message bundle, it will replace the string at compile-time with a localized
 * version, and expand goog.getMsg call to a concatenated string.
 *
 * Messages must be initialized in the form:
 * <code>
 * var MSG_NAME = goog.getMsg('Hello {$placeholder}', {'placeholder': 'world'});
 * </code>
 *
 * This function produces a string which should be treated as plain text. Use
 * {@link goog.html.SafeHtmlFormatter} in conjunction with goog.getMsg to
 * produce SafeHtml.
 *
 * @param {string} str Translatable string, places holders in the form {$foo}.
 * @param {Object<string, string>=} opt_values Maps place holder name to value.
 * @param {{html: boolean}=} opt_options Options:
 *     html: Escape '<' in str to '&lt;'. Used by Closure Templates where the
 *     generated code size and performance is critical which is why {@link
 *     goog.html.SafeHtmlFormatter} is not used. The value must be literal true
 *     or false.
 * @return {string} message with placeholders filled.
 */
goog.getMsg = function(str, opt_values, opt_options) {
  if (opt_options && opt_options.html) {
    // Note that '&' is not replaced because the translation can contain HTML
    // entities.
    str = str.replace(/</g, '&lt;');
  }
  if (opt_values) {
    str = str.replace(/\{\$([^}]+)}/g, function(match, key) {
      return (opt_values != null && key in opt_values) ? opt_values[key] :
                                                         match;
    });
  }
  return str;
};


/**
 * Gets a localized message. If the message does not have a translation, gives a
 * fallback message.
 *
 * This is useful when introducing a new message that has not yet been
 * translated into all languages.
 *
 * This function is a compiler primitive. Must be used in the form:
 * <code>var x = goog.getMsgWithFallback(MSG_A, MSG_B);</code>
 * where MSG_A and MSG_B were initialized with goog.getMsg.
 *
 * @param {string} a The preferred message.
 * @param {string} b The fallback message.
 * @return {string} The best translated message.
 */
goog.getMsgWithFallback = function(a, b) {
  return a;
};


/**
 * Exposes an unobfuscated global namespace path for the given object.
 * Note that fields of the exported object *will* be obfuscated, unless they are
 * exported in turn via this function or goog.exportProperty.
 *
 * Also handy for making public items that are defined in anonymous closures.
 *
 * ex. goog.exportSymbol('public.path.Foo', Foo);
 *
 * ex. goog.exportSymbol('public.path.Foo.staticFunction', Foo.staticFunction);
 *     public.path.Foo.staticFunction();
 *
 * ex. goog.exportSymbol('public.path.Foo.prototype.myMethod',
 *                       Foo.prototype.myMethod);
 *     new public.path.Foo().myMethod();
 *
 * @param {string} publicPath Unobfuscated name to export.
 * @param {*} object Object the name should point to.
 * @param {Object=} opt_objectToExportTo The object to add the path to; default
 *     is goog.global.
 */
goog.exportSymbol = function(publicPath, object, opt_objectToExportTo) {
  goog.exportPath_(publicPath, object, opt_objectToExportTo);
};


/**
 * Exports a property unobfuscated into the object's namespace.
 * ex. goog.exportProperty(Foo, 'staticFunction', Foo.staticFunction);
 * ex. goog.exportProperty(Foo.prototype, 'myMethod', Foo.prototype.myMethod);
 * @param {Object} object Object whose static property is being exported.
 * @param {string} publicName Unobfuscated name to export.
 * @param {*} symbol Object the name should point to.
 */
goog.exportProperty = function(object, publicName, symbol) {
  object[publicName] = symbol;
};


/**
 * Inherit the prototype methods from one constructor into another.
 *
 * Usage:
 * <pre>
 * function ParentClass(a, b) { }
 * ParentClass.prototype.foo = function(a) { };
 *
 * function ChildClass(a, b, c) {
 *   ChildClass.base(this, 'constructor', a, b);
 * }
 * goog.inherits(ChildClass, ParentClass);
 *
 * var child = new ChildClass('a', 'b', 'see');
 * child.foo(); // This works.
 * </pre>
 *
 * @param {!Function} childCtor Child class.
 * @param {!Function} parentCtor Parent class.
 * @suppress {strictMissingProperties} superClass_ and base is not defined on
 *    Function.
 */
goog.inherits = function(childCtor, parentCtor) {
  /** @constructor */
  function tempCtor() {}
  tempCtor.prototype = parentCtor.prototype;
  childCtor.superClass_ = parentCtor.prototype;
  childCtor.prototype = new tempCtor();
  /** @override */
  childCtor.prototype.constructor = childCtor;

  /**
   * Calls superclass constructor/method.
   *
   * This function is only available if you use goog.inherits to
   * express inheritance relationships between classes.
   *
   * NOTE: This is a replacement for goog.base and for superClass_
   * property defined in childCtor.
   *
   * @param {!Object} me Should always be "this".
   * @param {string} methodName The method name to call. Calling
   *     superclass constructor can be done with the special string
   *     'constructor'.
   * @param {...*} var_args The arguments to pass to superclass
   *     method/constructor.
   * @return {*} The return value of the superclass method/constructor.
   */
  childCtor.base = function(me, methodName, var_args) {
    // Copying using loop to avoid deop due to passing arguments object to
    // function. This is faster in many JS engines as of late 2014.
    var args = new Array(arguments.length - 2);
    for (var i = 2; i < arguments.length; i++) {
      args[i - 2] = arguments[i];
    }
    return parentCtor.prototype[methodName].apply(me, args);
  };
};


/**
 * Allow for aliasing within scope functions.  This function exists for
 * uncompiled code - in compiled code the calls will be inlined and the aliases
 * applied.  In uncompiled code the function is simply run since the aliases as
 * written are valid JavaScript.
 *
 * MOE:begin_intracomment_strip
 * See the goog.scope document at http://go/goog.scope
 *
 * For more on goog.scope deprecation, see the style guide entry:
 * http://go/jsstyle#appendices-legacy-exceptions-goog-scope
 * MOE:end_intracomment_strip
 *
 * @param {function()} fn Function to call.  This function can contain aliases
 *     to namespaces (e.g. "var dom = goog.dom") or classes
 *     (e.g. "var Timer = goog.Timer").
 * @deprecated Use goog.module instead.
 */
goog.scope = function(fn) {
  if (goog.isInModuleLoader_()) {
    throw new Error('goog.scope is not supported within a module.');
  }
  fn.call(goog.global);
};


/*
 * To support uncompiled, strict mode bundles that use eval to divide source
 * like so:
 *    eval('someSource;//# sourceUrl sourcefile.js');
 * We need to export the globally defined symbols "goog" and "COMPILED".
 * Exporting "goog" breaks the compiler optimizations, so we required that
 * be defined externally.
 * NOTE: We don't use goog.exportSymbol here because we don't want to trigger
 * extern generation when that compiler option is enabled.
 */
if (!COMPILED) {
  goog.global['COMPILED'] = COMPILED;
}


//==============================================================================
// goog.defineClass implementation
//==============================================================================


/**
 * Creates a restricted form of a Closure "class":
 *   - from the compiler's perspective, the instance returned from the
 *     constructor is sealed (no new properties may be added).  This enables
 *     better checks.
 *   - the compiler will rewrite this definition to a form that is optimal
 *     for type checking and optimization (initially this will be a more
 *     traditional form).
 *
 * @param {Function} superClass The superclass, Object or null.
 * @param {goog.defineClass.ClassDescriptor} def
 *     An object literal describing
 *     the class.  It may have the following properties:
 *     "constructor": the constructor function
 *     "statics": an object literal containing methods to add to the constructor
 *        as "static" methods or a function that will receive the constructor
 *        function as its only parameter to which static properties can
 *        be added.
 *     all other properties are added to the prototype.
 * @return {!Function} The class constructor.
 * @deprecated Use ES6 class syntax instead.
 */
goog.defineClass = function(superClass, def) {
  // TODO(johnlenz): consider making the superClass an optional parameter.
  var constructor = def.constructor;
  var statics = def.statics;
  // Wrap the constructor prior to setting up the prototype and static methods.
  if (!constructor || constructor == Object.prototype.constructor) {
    constructor = function() {
      throw new Error(
          'cannot instantiate an interface (no constructor defined).');
    };
  }

  var cls = goog.defineClass.createSealingConstructor_(constructor, superClass);
  if (superClass) {
    goog.inherits(cls, superClass);
  }

  // Remove all the properties that should not be copied to the prototype.
  delete def.constructor;
  delete def.statics;

  goog.defineClass.applyProperties_(cls.prototype, def);
  if (statics != null) {
    if (statics instanceof Function) {
      statics(cls);
    } else {
      goog.defineClass.applyProperties_(cls, statics);
    }
  }

  return cls;
};


/**
 * @typedef {{
 *   constructor: (!Function|undefined),
 *   statics: (Object|undefined|function(Function):void)
 * }}
 */
goog.defineClass.ClassDescriptor;


/**
 * @define {boolean} Whether the instances returned by goog.defineClass should
 *     be sealed when possible.
 *
 * When sealing is disabled the constructor function will not be wrapped by
 * goog.defineClass, making it incompatible with ES6 class methods.
 */
goog.defineClass.SEAL_CLASS_INSTANCES =
    goog.define('goog.defineClass.SEAL_CLASS_INSTANCES', goog.DEBUG);


/**
 * If goog.defineClass.SEAL_CLASS_INSTANCES is enabled and Object.seal is
 * defined, this function will wrap the constructor in a function that seals the
 * results of the provided constructor function.
 *
 * @param {!Function} ctr The constructor whose results maybe be sealed.
 * @param {Function} superClass The superclass constructor.
 * @return {!Function} The replacement constructor.
 * @private
 */
goog.defineClass.createSealingConstructor_ = function(ctr, superClass) {
  if (!goog.defineClass.SEAL_CLASS_INSTANCES) {
    // Do now wrap the constructor when sealing is disabled. Angular code
    // depends on this for injection to work properly.
    return ctr;
  }

  // Compute whether the constructor is sealable at definition time, rather
  // than when the instance is being constructed.
  var superclassSealable = !goog.defineClass.isUnsealable_(superClass);

  /**
   * @this {Object}
   * @return {?}
   */
  var wrappedCtr = function() {
    // Don't seal an instance of a subclass when it calls the constructor of
    // its super class as there is most likely still setup to do.
    var instance = ctr.apply(this, arguments) || this;
    instance[goog.UID_PROPERTY_] = instance[goog.UID_PROPERTY_];

    if (this.constructor === wrappedCtr && superclassSealable &&
        Object.seal instanceof Function) {
      Object.seal(instance);
    }
    return instance;
  };

  return wrappedCtr;
};


/**
 * @param {Function} ctr The constructor to test.
 * @return {boolean} Whether the constructor has been tagged as unsealable
 *     using goog.tagUnsealableClass.
 * @private
 */
goog.defineClass.isUnsealable_ = function(ctr) {
  return ctr && ctr.prototype &&
      ctr.prototype[goog.UNSEALABLE_CONSTRUCTOR_PROPERTY_];
};


// TODO(johnlenz): share these values with the goog.object
/**
 * The names of the fields that are defined on Object.prototype.
 * @type {!Array<string>}
 * @private
 * @const
 */
goog.defineClass.OBJECT_PROTOTYPE_FIELDS_ = [
  'constructor', 'hasOwnProperty', 'isPrototypeOf', 'propertyIsEnumerable',
  'toLocaleString', 'toString', 'valueOf'
];


// TODO(johnlenz): share this function with the goog.object
/**
 * @param {!Object} target The object to add properties to.
 * @param {!Object} source The object to copy properties from.
 * @private
 */
goog.defineClass.applyProperties_ = function(target, source) {
  // TODO(johnlenz): update this to support ES5 getters/setters

  var key;
  for (key in source) {
    if (Object.prototype.hasOwnProperty.call(source, key)) {
      target[key] = source[key];
    }
  }

  // For IE the for-in-loop does not contain any properties that are not
  // enumerable on the prototype object (for example isPrototypeOf from
  // Object.prototype) and it will also not include 'replace' on objects that
  // extend String and change 'replace' (not that it is common for anyone to
  // extend anything except Object).
  for (var i = 0; i < goog.defineClass.OBJECT_PROTOTYPE_FIELDS_.length; i++) {
    key = goog.defineClass.OBJECT_PROTOTYPE_FIELDS_[i];
    if (Object.prototype.hasOwnProperty.call(source, key)) {
      target[key] = source[key];
    }
  }
};


/**
 * Sealing classes breaks the older idiom of assigning properties on the
 * prototype rather than in the constructor. As such, goog.defineClass
 * must not seal subclasses of these old-style classes until they are fixed.
 * Until then, this marks a class as "broken", instructing defineClass
 * not to seal subclasses.
 * @param {!Function} ctr The legacy constructor to tag as unsealable.
 */
goog.tagUnsealableClass = function(ctr) {
  if (!COMPILED && goog.defineClass.SEAL_CLASS_INSTANCES) {
    ctr.prototype[goog.UNSEALABLE_CONSTRUCTOR_PROPERTY_] = true;
  }
};


/**
 * Name for unsealable tag property.
 * @const @private {string}
 */
goog.UNSEALABLE_CONSTRUCTOR_PROPERTY_ = 'goog_defineClass_legacy_unsealable';


// There's a bug in the compiler where without collapse properties the
// Closure namespace defines do not guard code correctly. To help reduce code
// size also check for !COMPILED even though it redundant until this is fixed.
if (!COMPILED && goog.DEPENDENCIES_ENABLED) {
  // MOE:begin_strip
  // TODO(b/67050526) This object is obsolete but some people are relying on
  // it internally. Keep it around until we migrate them.
  /**
   * @private
   * @type {{
   *   loadFlags: !Object<string, !Object<string, string>>,
   *   nameToPath: !Object<string, string>,
   *   requires: !Object<string, !Object<string, boolean>>,
   *   visited: !Object<string, boolean>,
   *   written: !Object<string, boolean>,
   *   deferred: !Object<string, string>
   * }}
   */
  goog.dependencies_ = {
    loadFlags: {},  // 1 to 1

    nameToPath: {},  // 1 to 1

    requires: {},  // 1 to many

    // Used when resolving dependencies to prevent us from visiting file
    // twice.
    visited: {},

    written: {},  // Used to keep track of script files we have written.

    deferred: {}  // Used to track deferred module evaluations in old IEs
  };

  /**
   * @return {!Object}
   * @private
   */
  goog.getLoader_ = function() {
    return {
      dependencies_: goog.dependencies_,
      writeScriptTag_: goog.writeScriptTag_
    };
  };


  /**
   * @param {string} src The script url.
   * @param {string=} opt_sourceText The optionally source text to evaluate
   * @return {boolean} True if the script was imported, false otherwise.
   * @private
   */
  goog.writeScriptTag_ = function(src, opt_sourceText) {
    if (goog.inHtmlDocument_()) {
      /** @type {!HTMLDocument} */
      var doc = goog.global.document;

      // If the user tries to require a new symbol after document load,
      // something has gone terribly wrong. Doing a document.write would
      // wipe out the page. This does not apply to the CSP-compliant method
      // of writing script tags.
      if (!goog.ENABLE_CHROME_APP_SAFE_SCRIPT_LOADING &&
          doc.readyState == 'complete') {
        // Certain test frameworks load base.js multiple times, which tries
        // to write deps.js each time. If that happens, just fail silently.
        // These frameworks wipe the page between each load of base.js, so this
        // is OK.
        var isDeps = /\bdeps.js$/.test(src);
        if (isDeps) {
          return false;
        } else {
          throw Error('Cannot write "' + src + '" after document load');
        }
      }

      if (opt_sourceText === undefined) {
        var script = '<script type="text/javascript" src="' + src + '"></' +
            'script>';
        doc.write(
            goog.TRUSTED_TYPES_POLICY_ ?
                goog.TRUSTED_TYPES_POLICY_.createHTML(script) :
                script);
      } else {
        var script = '<script type="text/javascript">' +
            goog.protectScriptTag_(opt_sourceText) + '</' +
            'script>';
        doc.write(
            goog.TRUSTED_TYPES_POLICY_ ?
                goog.TRUSTED_TYPES_POLICY_.createHTML(script) :
                script);
      }
      return true;
    } else {
      return false;
    }
  };
  // MOE:end_strip

  /**
   * Tries to detect whether is in the context of an HTML document.
   * @return {boolean} True if it looks like HTML document.
   * @private
   */
  goog.inHtmlDocument_ = function() {
    /** @type {!Document} */
    var doc = goog.global.document;
    return doc != null && 'write' in doc;  // XULDocument misses write.
  };


  /**
   * We'd like to check for if the document readyState is 'loading'; however
   * there are bugs on IE 10 and below where the readyState being anything other
   * than 'complete' is not reliable.
   * @return {boolean}
   * @private
   */
  goog.isDocumentLoading_ = function() {
    // attachEvent is available on IE 6 thru 10 only, and thus can be used to
    // detect those browsers.
    /** @type {!HTMLDocument} */
    var doc = goog.global.document;
    return doc.attachEvent ? doc.readyState != 'complete' :
                             doc.readyState == 'loading';
  };


  /**
   * Tries to detect the base path of base.js script that bootstraps Closure.
   * @private
   */
  goog.findBasePath_ = function() {
    if (goog.global.CLOSURE_BASE_PATH != undefined &&
        // Anti DOM-clobbering runtime check (b/37736576).
        typeof goog.global.CLOSURE_BASE_PATH === 'string') {
      goog.basePath = goog.global.CLOSURE_BASE_PATH;
      return;
    } else if (!goog.inHtmlDocument_()) {
      return;
    }
    /** @type {!Document} */
    var doc = goog.global.document;
    // If we have a currentScript available, use it exclusively.
    var currentScript = doc.currentScript;
    if (currentScript) {
      var scripts = [currentScript];
    } else {
      var scripts = doc.getElementsByTagName('SCRIPT');
    }
    // Search backwards since the current script is in almost all cases the one
    // that has base.js.
    for (var i = scripts.length - 1; i >= 0; --i) {
      var script = /** @type {!HTMLScriptElement} */ (scripts[i]);
      var src = script.src;
      var qmark = src.lastIndexOf('?');
      var l = qmark == -1 ? src.length : qmark;
      if (src.substr(l - 7, 7) == 'base.js') {
        goog.basePath = src.substr(0, l - 7);
        return;
      }
    }
  };

  goog.findBasePath_();

  /** @struct @constructor @final */
  goog.Transpiler = function() {
    /** @private {?Object<string, boolean>} */
    this.requiresTranspilation_ = null;
    /** @private {string} */
    this.transpilationTarget_ = goog.TRANSPILE_TO_LANGUAGE;
  };


  // MOE:begin_strip
  // LINT.IfChange
  // MOE:end_strip
  /**
   * Returns a newly created map from language mode string to a boolean
   * indicating whether transpilation should be done for that mode as well as
   * the highest level language that this environment supports.
   *
   * Guaranteed invariant:
   * For any two modes, l1 and l2 where l2 is a newer mode than l1,
   * `map[l1] == true` implies that `map[l2] == true`.
   *
   * Note this method is extracted and used elsewhere, so it cannot rely on
   * anything external (it should easily be able to be transformed into a
   * standalone, top level function).
   *
   * @private
   * @return {{
   *   target: string,
   *   map: !Object<string, boolean>
   * }}
   */
  goog.Transpiler.prototype.createRequiresTranspilation_ = function() {
    var transpilationTarget = 'es3';
    var /** !Object<string, boolean> */ requiresTranspilation = {'es3': false};
    var transpilationRequiredForAllLaterModes = false;

    /**
     * Adds an entry to requiresTranspliation for the given language mode.
     *
     * IMPORTANT: Calls must be made in order from oldest to newest language
     * mode.
     * @param {string} modeName
     * @param {function(): boolean} isSupported Returns true if the JS engine
     *     supports the given mode.
     */
    function addNewerLanguageTranspilationCheck(modeName, isSupported) {
      if (transpilationRequiredForAllLaterModes) {
        requiresTranspilation[modeName] = true;
      } else if (isSupported()) {
        transpilationTarget = modeName;
        requiresTranspilation[modeName] = false;
      } else {
        requiresTranspilation[modeName] = true;
        transpilationRequiredForAllLaterModes = true;
      }
    }

    /**
     * Does the given code evaluate without syntax errors and return a truthy
     * result?
     */
    function /** boolean */ evalCheck(/** string */ code) {
      try {
        return !!eval(code);
      } catch (ignored) {
        return false;
      }
    }

    var userAgent = goog.global.navigator && goog.global.navigator.userAgent ?
        goog.global.navigator.userAgent :
        '';

    // Identify ES3-only browsers by their incorrect treatment of commas.
    addNewerLanguageTranspilationCheck('es5', function() {
      return evalCheck('[1,].length==1');
    });
    addNewerLanguageTranspilationCheck('es6', function() {
      // Edge has a non-deterministic (i.e., not reproducible) bug with ES6:
      // https://github.com/Microsoft/ChakraCore/issues/1496.
      // MOE:begin_strip
      // TODO(joeltine): Our internal web-testing version of Edge will need to
      // be updated before we can remove this check. See http://b/34945376.
      // MOE:end_strip
      var re = /Edge\/(\d+)(\.\d)*/i;
      var edgeUserAgent = userAgent.match(re);
      if (edgeUserAgent) {
        // The Reflect.construct test below is flaky on Edge. It can sometimes
        // pass or fail on 40 15.15063, so just exit early for Edge and treat
        // it as ES5. Until we're on a more up to date version just always use
        // ES5. See https://github.com/Microsoft/ChakraCore/issues/3217.
        return false;
      }
      // Test es6: [FF50 (?), Edge 14 (?), Chrome 50]
      //   (a) default params (specifically shadowing locals),
      //   (b) destructuring, (c) block-scoped functions,
      //   (d) for-of (const), (e) new.target/Reflect.construct
      var es6fullTest =
          'class X{constructor(){if(new.target!=String)throw 1;this.x=42}}' +
          'let q=Reflect.construct(X,[],String);if(q.x!=42||!(q instanceof ' +
          'String))throw 1;for(const a of[2,3]){if(a==2)continue;function ' +
          'f(z={a}){let a=0;return z.a}{function f(){return 0;}}return f()' +
          '==3}';

      return evalCheck('(()=>{"use strict";' + es6fullTest + '})()');
    });
    // ** and **= are the only new features in 'es7'
    addNewerLanguageTranspilationCheck('es7', function() {
      return evalCheck('2 ** 2 == 4');
    });
    // async functions are the only new features in 'es8'
    addNewerLanguageTranspilationCheck('es8', function() {
      return evalCheck('async () => 1, true');
    });
    addNewerLanguageTranspilationCheck('es9', function() {
      return evalCheck('({...rest} = {}), true');
    });
    addNewerLanguageTranspilationCheck('es_next', function() {
      return false;  // assume it always need to transpile
    });
    return {target: transpilationTarget, map: requiresTranspilation};
  };
  // MOE:begin_strip
  // LINT.ThenChange(//depot/google3/java/com/google/testing/web/devtools/updatebrowserinfo/requires_transpilation.js)
  // MOE:end_strip


  /**
   * Determines whether the given language needs to be transpiled.
   * @param {string} lang
   * @param {string|undefined} module
   * @return {boolean}
   */
  goog.Transpiler.prototype.needsTranspile = function(lang, module) {
    if (goog.TRANSPILE == 'always') {
      return true;
    } else if (goog.TRANSPILE == 'never') {
      return false;
    } else if (!this.requiresTranspilation_) {
      var obj = this.createRequiresTranspilation_();
      this.requiresTranspilation_ = obj.map;
      this.transpilationTarget_ = this.transpilationTarget_ || obj.target;
    }
    if (lang in this.requiresTranspilation_) {
      if (this.requiresTranspilation_[lang]) {
        return true;
      } else if (
          goog.inHtmlDocument_() && module == 'es6' &&
          !('noModule' in goog.global.document.createElement('script'))) {
        return true;
      } else {
        return false;
      }
    } else {
      throw new Error('Unknown language mode: ' + lang);
    }
  };


  /**
   * Lazily retrieves the transpiler and applies it to the source.
   * @param {string} code JS code.
   * @param {string} path Path to the code.
   * @return {string} The transpiled code.
   */
  goog.Transpiler.prototype.transpile = function(code, path) {
    // TODO(johnplaisted): We should delete goog.transpile_ and just have this
    // function. But there's some compile error atm where goog.global is being
    // stripped incorrectly without this.
    return goog.transpile_(code, path, this.transpilationTarget_);
  };


  /** @private @final {!goog.Transpiler} */
  goog.transpiler_ = new goog.Transpiler();

  /**
   * Rewrites closing script tags in input to avoid ending an enclosing script
   * tag.
   *
   * @param {string} str
   * @return {string}
   * @private
   */
  goog.protectScriptTag_ = function(str) {
    return str.replace(/<\/(SCRIPT)/ig, '\\x3c/$1');
  };


  /**
   * A debug loader is responsible for downloading and executing javascript
   * files in an unbundled, uncompiled environment.
   *
   * This can be custimized via the setDependencyFactory method, or by
   * CLOSURE_IMPORT_SCRIPT/CLOSURE_LOAD_FILE_SYNC.
   *
   * @struct @constructor @final @private
   */
  goog.DebugLoader_ = function() {
    /** @private @const {!Object<string, !goog.Dependency>} */
    this.dependencies_ = {};
    /** @private @const {!Object<string, string>} */
    this.idToPath_ = {};
    /** @private @const {!Object<string, boolean>} */
    this.written_ = {};
    /** @private @const {!Array<!goog.Dependency>} */
    this.loadingDeps_ = [];
    /** @private {!Array<!goog.Dependency>} */
    this.depsToLoad_ = [];
    /** @private {boolean} */
    this.paused_ = false;
    /** @private {!goog.DependencyFactory} */
    this.factory_ = new goog.DependencyFactory(goog.transpiler_);
    /** @private @const {!Object<string, !Function>} */
    this.deferredCallbacks_ = {};
    /** @private @const {!Array<string>} */
    this.deferredQueue_ = [];
  };

  /**
   * @param {!Array<string>} namespaces
   * @param {function(): undefined} callback Function to call once all the
   *     namespaces have loaded.
   */
  goog.DebugLoader_.prototype.bootstrap = function(namespaces, callback) {
    var cb = callback;
    function resolve() {
      if (cb) {
        goog.global.setTimeout(cb, 0);
        cb = null;
      }
    }

    if (!namespaces.length) {
      resolve();
      return;
    }

    var deps = [];
    for (var i = 0; i < namespaces.length; i++) {
      var path = this.getPathFromDeps_(namespaces[i]);
      if (!path) {
        throw new Error('Unregonized namespace: ' + namespaces[i]);
      }
      deps.push(this.dependencies_[path]);
    }

    var require = goog.require;
    var loaded = 0;
    for (var i = 0; i < namespaces.length; i++) {
      require(namespaces[i]);
      deps[i].onLoad(function() {
        if (++loaded == namespaces.length) {
          resolve();
        }
      });
    }
  };


  /**
   * Loads the Closure Dependency file.
   *
   * Exposed a public function so CLOSURE_NO_DEPS can be set to false, base
   * loaded, setDependencyFactory called, and then this called. i.e. allows
   * custom loading of the deps file.
   */
  goog.DebugLoader_.prototype.loadClosureDeps = function() {
    // Circumvent addDependency, which would try to transpile deps.js if
    // transpile is set to always.
    var relPath = 'deps.js';
    this.depsToLoad_.push(this.factory_.createDependency(
        goog.normalizePath_(goog.basePath + relPath), relPath, [], [], {},
        false));
    this.loadDeps_();
  };


  /**
   * Notifies the debug loader when a dependency has been requested.
   *
   * @param {string} absPathOrId Path of the dependency or goog id.
   * @param {boolean=} opt_force
   */
  goog.DebugLoader_.prototype.requested = function(absPathOrId, opt_force) {
    var path = this.getPathFromDeps_(absPathOrId);
    if (path &&
        (opt_force || this.areDepsLoaded_(this.dependencies_[path].requires))) {
      var callback = this.deferredCallbacks_[path];
      if (callback) {
        delete this.deferredCallbacks_[path];
        callback();
      }
    }
  };


  /**
   * Sets the dependency factory, which can be used to create custom
   * goog.Dependency implementations to control how dependencies are loaded.
   *
   * @param {!goog.DependencyFactory} factory
   */
  goog.DebugLoader_.prototype.setDependencyFactory = function(factory) {
    this.factory_ = factory;
  };


  /**
   * Travserses the dependency graph and queues the given dependency, and all of
   * its transitive dependencies, for loading and then starts loading if not
   * paused.
   *
   * @param {string} namespace
   * @private
   */
  goog.DebugLoader_.prototype.load_ = function(namespace) {
    if (!this.getPathFromDeps_(namespace)) {
      var errorMessage = 'goog.require could not find: ' + namespace;

      goog.logToConsole_(errorMessage);
      // MOE:begin_strip

      // NOTE(nicksantos): We could always throw an error, but this would
      // break legacy users that depended on this failing silently. Instead,
      // the compiler should warn us when there are invalid goog.require
      // calls. For now, we simply give clients a way to turn strict mode on.
      if (goog.useStrictRequires) {
        throw Error(errorMessage);
      }

      // In external Closure, always error.
      // MOE:end_strip
      // MOE:insert throw Error(errorMessage);
    } else {
      var loader = this;

      var deps = [];

      /** @param {string} namespace */
      var visit = function(namespace) {
        var path = loader.getPathFromDeps_(namespace);

        if (!path) {
          throw new Error('Bad dependency path or symbol: ' + namespace);
        }

        if (loader.written_[path]) {
          return;
        }

        loader.written_[path] = true;

        var dep = loader.dependencies_[path];
        // MOE:begin_strip
        if (goog.dependencies_.written[dep.relativePath]) {
          return;
        }
        // MOE:end_strip
        for (var i = 0; i < dep.requires.length; i++) {
          if (!goog.isProvided_(dep.requires[i])) {
            visit(dep.requires[i]);
          }
        }

        deps.push(dep);
      };

      visit(namespace);

      var wasLoading = !!this.depsToLoad_.length;
      this.depsToLoad_ = this.depsToLoad_.concat(deps);

      if (!this.paused_ && !wasLoading) {
        this.loadDeps_();
      }
    }
  };


  /**
   * Loads any queued dependencies until they are all loaded or paused.
   *
   * @private
   */
  goog.DebugLoader_.prototype.loadDeps_ = function() {
    var loader = this;
    var paused = this.paused_;

    while (this.depsToLoad_.length && !paused) {
      (function() {
        var loadCallDone = false;
        var dep = loader.depsToLoad_.shift();

        var loaded = false;
        loader.loading_(dep);

        var controller = {
          pause: function() {
            if (loadCallDone) {
              throw new Error('Cannot call pause after the call to load.');
            } else {
              paused = true;
            }
          },
          resume: function() {
            if (loadCallDone) {
              loader.resume_();
            } else {
              // Some dep called pause and then resume in the same load call.
              // Just keep running this same loop.
              paused = false;
            }
          },
          loaded: function() {
            if (loaded) {
              throw new Error('Double call to loaded.');
            }

            loaded = true;
            loader.loaded_(dep);
          },
          pending: function() {
            // Defensive copy.
            var pending = [];
            for (var i = 0; i < loader.loadingDeps_.length; i++) {
              pending.push(loader.loadingDeps_[i]);
            }
            return pending;
          },
          /**
           * @param {goog.ModuleType} type
           */
          setModuleState: function(type) {
            goog.moduleLoaderState_ = {
              type: type,
              moduleName: '',
              declareLegacyNamespace: false
            };
          },
          /** @type {function(string, string, string=)} */
          registerEs6ModuleExports: function(
              path, exports, opt_closureNamespace) {
            if (opt_closureNamespace) {
              goog.loadedModules_[opt_closureNamespace] = {
                exports: exports,
                type: goog.ModuleType.ES6,
                moduleId: opt_closureNamespace || ''
              };
            }
          },
          /** @type {function(string, ?)} */
          registerGoogModuleExports: function(moduleId, exports) {
            goog.loadedModules_[moduleId] = {
              exports: exports,
              type: goog.ModuleType.GOOG,
              moduleId: moduleId
            };
          },
          clearModuleState: function() {
            goog.moduleLoaderState_ = null;
          },
          defer: function(callback) {
            if (loadCallDone) {
              throw new Error(
                  'Cannot register with defer after the call to load.');
            }
            loader.defer_(dep, callback);
          },
          areDepsLoaded: function() {
            return loader.areDepsLoaded_(dep.requires);
          }
        };

        try {
          dep.load(controller);
        } finally {
          loadCallDone = true;
        }
      })();
    }

    if (paused) {
      this.pause_();
    }
  };


  /** @private */
  goog.DebugLoader_.prototype.pause_ = function() {
    this.paused_ = true;
  };


  /** @private */
  goog.DebugLoader_.prototype.resume_ = function() {
    if (this.paused_) {
      this.paused_ = false;
      this.loadDeps_();
    }
  };


  /**
   * Marks the given dependency as loading (load has been called but it has not
   * yet marked itself as finished). Useful for dependencies that want to know
   * what else is loading. Example: goog.modules cannot eval if there are
   * loading dependencies.
   *
   * @param {!goog.Dependency} dep
   * @private
   */
  goog.DebugLoader_.prototype.loading_ = function(dep) {
    this.loadingDeps_.push(dep);
  };


  /**
   * Marks the given dependency as having finished loading and being available
   * for require.
   *
   * @param {!goog.Dependency} dep
   * @private
   */
  goog.DebugLoader_.prototype.loaded_ = function(dep) {
    for (var i = 0; i < this.loadingDeps_.length; i++) {
      if (this.loadingDeps_[i] == dep) {
        this.loadingDeps_.splice(i, 1);
        break;
      }
    }

    for (var i = 0; i < this.deferredQueue_.length; i++) {
      if (this.deferredQueue_[i] == dep.path) {
        this.deferredQueue_.splice(i, 1);
        break;
      }
    }

    if (this.loadingDeps_.length == this.deferredQueue_.length &&
        !this.depsToLoad_.length) {
      // Something has asked to load these, but they may not be directly
      // required again later, so load them now that we know we're done loading
      // everything else. e.g. a goog module entry point.
      while (this.deferredQueue_.length) {
        this.requested(this.deferredQueue_.shift(), true);
      }
    }

    dep.loaded();
  };


  /**
   * @param {!Array<string>} pathsOrIds
   * @return {boolean}
   * @private
   */
  goog.DebugLoader_.prototype.areDepsLoaded_ = function(pathsOrIds) {
    for (var i = 0; i < pathsOrIds.length; i++) {
      var path = this.getPathFromDeps_(pathsOrIds[i]);
      if (!path ||
          (!(path in this.deferredCallbacks_) &&
           !goog.isProvided_(pathsOrIds[i]))) {
        return false;
      }
    }

    return true;
  };


  /**
   * @param {string} absPathOrId
   * @return {?string}
   * @private
   */
  goog.DebugLoader_.prototype.getPathFromDeps_ = function(absPathOrId) {
    if (absPathOrId in this.idToPath_) {
      return this.idToPath_[absPathOrId];
    } else if (absPathOrId in this.dependencies_) {
      return absPathOrId;
    } else {
      return null;
    }
  };


  /**
   * @param {!goog.Dependency} dependency
   * @param {!Function} callback
   * @private
   */
  goog.DebugLoader_.prototype.defer_ = function(dependency, callback) {
    this.deferredCallbacks_[dependency.path] = callback;
    this.deferredQueue_.push(dependency.path);
  };


  /**
   * Interface for goog.Dependency implementations to have some control over
   * loading of dependencies.
   *
   * @record
   */
  goog.LoadController = function() {};


  /**
   * Tells the controller to halt loading of more dependencies.
   */
  goog.LoadController.prototype.pause = function() {};


  /**
   * Tells the controller to resume loading of more dependencies if paused.
   */
  goog.LoadController.prototype.resume = function() {};


  /**
   * Tells the controller that this dependency has finished loading.
   *
   * This causes this to be removed from pending() and any load callbacks to
   * fire.
   */
  goog.LoadController.prototype.loaded = function() {};


  /**
   * List of dependencies on which load has been called but which have not
   * called loaded on their controller. This includes the current dependency.
   *
   * @return {!Array<!goog.Dependency>}
   */
  goog.LoadController.prototype.pending = function() {};


  /**
   * Registers an object as an ES6 module's exports so that goog.modules may
   * require it by path.
   *
   * @param {string} path Full path of the module.
   * @param {?} exports
   * @param {string=} opt_closureNamespace Closure namespace to associate with
   *     this module.
   */
  goog.LoadController.prototype.registerEs6ModuleExports = function(
      path, exports, opt_closureNamespace) {};


  /**
   * Sets the current module state.
   *
   * @param {goog.ModuleType} type Type of module.
   */
  goog.LoadController.prototype.setModuleState = function(type) {};


  /**
   * Clears the current module state.
   */
  goog.LoadController.prototype.clearModuleState = function() {};


  /**
   * Registers a callback to call once the dependency is actually requested
   * via goog.require + all of the immediate dependencies have been loaded or
   * all other files have been loaded. Allows for lazy loading until
   * require'd without pausing dependency loading, which is needed on old IE.
   *
   * @param {!Function} callback
   */
  goog.LoadController.prototype.defer = function(callback) {};


  /**
   * @return {boolean}
   */
  goog.LoadController.prototype.areDepsLoaded = function() {};


  /**
   * Basic super class for all dependencies Closure Library can load.
   *
   * This default implementation is designed to load untranspiled, non-module
   * scripts in a web broswer.
   *
   * For transpiled non-goog.module files {@see goog.TranspiledDependency}.
   * For goog.modules see {@see goog.GoogModuleDependency}.
   * For untranspiled ES6 modules {@see goog.Es6ModuleDependency}.
   *
   * @param {string} path Absolute path of this script.
   * @param {string} relativePath Path of this script relative to goog.basePath.
   * @param {!Array<string>} provides goog.provided or goog.module symbols
   *     in this file.
   * @param {!Array<string>} requires goog symbols or relative paths to Closure
   *     this depends on.
   * @param {!Object<string, string>} loadFlags
   * @struct @constructor
   */
  goog.Dependency = function(
      path, relativePath, provides, requires, loadFlags) {
    /** @const */
    this.path = path;
    /** @const */
    this.relativePath = relativePath;
    /** @const */
    this.provides = provides;
    /** @const */
    this.requires = requires;
    /** @const */
    this.loadFlags = loadFlags;
    /** @private {boolean} */
    this.loaded_ = false;
    /** @private {!Array<function()>} */
    this.loadCallbacks_ = [];
  };


  /**
   * @return {string} The pathname part of this dependency's path if it is a
   *     URI.
   */
  goog.Dependency.prototype.getPathName = function() {
    var pathName = this.path;
    var protocolIndex = pathName.indexOf('://');
    if (protocolIndex >= 0) {
      pathName = pathName.substring(protocolIndex + 3);
      var slashIndex = pathName.indexOf('/');
      if (slashIndex >= 0) {
        pathName = pathName.substring(slashIndex + 1);
      }
    }
    return pathName;
  };


  /**
   * @param {function()} callback Callback to fire as soon as this has loaded.
   * @final
   */
  goog.Dependency.prototype.onLoad = function(callback) {
    if (this.loaded_) {
      callback();
    } else {
      this.loadCallbacks_.push(callback);
    }
  };


  /**
   * Marks this dependency as loaded and fires any callbacks registered with
   * onLoad.
   * @final
   */
  goog.Dependency.prototype.loaded = function() {
    this.loaded_ = true;
    var callbacks = this.loadCallbacks_;
    this.loadCallbacks_ = [];
    for (var i = 0; i < callbacks.length; i++) {
      callbacks[i]();
    }
  };


  /**
   * Whether or not document.written / appended script tags should be deferred.
   *
   * @private {boolean}
   */
  goog.Dependency.defer_ = false;


  /**
   * Map of script ready / state change callbacks. Old IE cannot handle putting
   * these properties on goog.global.
   *
   * @private @const {!Object<string, function(?):undefined>}
   */
  goog.Dependency.callbackMap_ = {};


  /**
   * @param {function(...?):?} callback
   * @return {string}
   * @private
   */
  goog.Dependency.registerCallback_ = function(callback) {
    var key = Math.random().toString(32);
    goog.Dependency.callbackMap_[key] = callback;
    return key;
  };


  /**
   * @param {string} key
   * @private
   */
  goog.Dependency.unregisterCallback_ = function(key) {
    delete goog.Dependency.callbackMap_[key];
  };


  /**
   * @param {string} key
   * @param {...?} var_args
   * @private
   * @suppress {unusedPrivateMembers}
   */
  goog.Dependency.callback_ = function(key, var_args) {
    if (key in goog.Dependency.callbackMap_) {
      var callback = goog.Dependency.callbackMap_[key];
      var args = [];
      for (var i = 1; i < arguments.length; i++) {
        args.push(arguments[i]);
      }
      callback.apply(undefined, args);
    } else {
      var errorMessage = 'Callback key ' + key +
          ' does not exist (was base.js loaded more than once?).';
      // MOE:begin_strip
      // TODO(johnplaisted): Some people internally are mistakenly loading
      // base.js twice, and this can happen while a dependency is loading,
      // wiping out state.
      goog.logToConsole_(errorMessage);
      // MOE:end_strip
      // MOE:insert throw Error(errorMessage);
    }
  };


  /**
   * Starts loading this dependency. This dependency can pause loading if it
   * needs to and resume it later via the controller interface.
   *
   * When this is loaded it should call controller.loaded(). Note that this will
   * end up calling the loaded method of this dependency; there is no need to
   * call it explicitly.
   *
   * @param {!goog.LoadController} controller
   */
  goog.Dependency.prototype.load = function(controller) {
    if (goog.global.CLOSURE_IMPORT_SCRIPT) {
      if (goog.global.CLOSURE_IMPORT_SCRIPT(this.path)) {
        controller.loaded();
      } else {
        controller.pause();
      }
      return;
    }

    if (!goog.inHtmlDocument_()) {
      goog.logToConsole_(
          'Cannot use default debug loader outside of HTML documents.');
      if (this.relativePath == 'deps.js') {
        // Some old code is relying on base.js auto loading deps.js failing with
        // no error before later setting CLOSURE_IMPORT_SCRIPT.
        // CLOSURE_IMPORT_SCRIPT should be set *before* base.js is loaded, or
        // CLOSURE_NO_DEPS set to true.
        goog.logToConsole_(
            'Consider setting CLOSURE_IMPORT_SCRIPT before loading base.js, ' +
            'or setting CLOSURE_NO_DEPS to true.');
        controller.loaded();
      } else {
        controller.pause();
      }
      return;
    }

    /** @type {!HTMLDocument} */
    var doc = goog.global.document;

    // If the user tries to require a new symbol after document load,
    // something has gone terribly wrong. Doing a document.write would
    // wipe out the page. This does not apply to the CSP-compliant method
    // of writing script tags.
    if (doc.readyState == 'complete' &&
        !goog.ENABLE_CHROME_APP_SAFE_SCRIPT_LOADING) {
      // Certain test frameworks load base.js multiple times, which tries
      // to write deps.js each time. If that happens, just fail silently.
      // These frameworks wipe the page between each load of base.js, so this
      // is OK.
      var isDeps = /\bdeps.js$/.test(this.path);
      if (isDeps) {
        controller.loaded();
        return;
      } else {
        throw Error('Cannot write "' + this.path + '" after document load');
      }
    }

    if (!goog.ENABLE_CHROME_APP_SAFE_SCRIPT_LOADING &&
        goog.isDocumentLoading_()) {
      var key = goog.Dependency.registerCallback_(function(script) {
        if (!goog.DebugLoader_.IS_OLD_IE_ || script.readyState == 'complete') {
          goog.Dependency.unregisterCallback_(key);
          controller.loaded();
        }
      });
      var nonceAttr = !goog.DebugLoader_.IS_OLD_IE_ && goog.getScriptNonce() ?
          ' nonce="' + goog.getScriptNonce() + '"' :
          '';
      var event =
          goog.DebugLoader_.IS_OLD_IE_ ? 'onreadystatechange' : 'onload';
      var defer = goog.Dependency.defer_ ? 'defer' : '';
      var script = '<script src="' + this.path + '" ' + event +
          '="goog.Dependency.callback_(\'' + key +
          '\', this)" type="text/javascript" ' + defer + nonceAttr + '><' +
          '/script>';
      doc.write(
          goog.TRUSTED_TYPES_POLICY_ ?
              goog.TRUSTED_TYPES_POLICY_.createHTML(script) :
              script);
    } else {
      var scriptEl =
          /** @type {!HTMLScriptElement} */ (doc.createElement('script'));
      scriptEl.defer = goog.Dependency.defer_;
      scriptEl.async = false;
      scriptEl.type = 'text/javascript';

      // If CSP nonces are used, propagate them to dynamically created scripts.
      // This is necessary to allow nonce-based CSPs without 'strict-dynamic'.
      var nonce = goog.getScriptNonce();
      if (nonce) {
        scriptEl.setAttribute('nonce', nonce);
      }

      if (goog.DebugLoader_.IS_OLD_IE_) {
        // Execution order is not guaranteed on old IE, halt loading and write
        // these scripts one at a time, after each loads.
        controller.pause();
        scriptEl.onreadystatechange = function() {
          if (scriptEl.readyState == 'loaded' ||
              scriptEl.readyState == 'complete') {
            controller.loaded();
            controller.resume();
          }
        };
      } else {
        scriptEl.onload = function() {
          scriptEl.onload = null;
          controller.loaded();
        };
      }

      scriptEl.src = goog.TRUSTED_TYPES_POLICY_ ?
          goog.TRUSTED_TYPES_POLICY_.createScriptURL(this.path) :
          this.path;
      doc.head.appendChild(scriptEl);
    }
  };


  /**
   * @param {string} path Absolute path of this script.
   * @param {string} relativePath Path of this script relative to goog.basePath.
   * @param {!Array<string>} provides Should be an empty array.
   *     TODO(johnplaisted) add support for adding closure namespaces to ES6
   *     modules for interop purposes.
   * @param {!Array<string>} requires goog symbols or relative paths to Closure
   *     this depends on.
   * @param {!Object<string, string>} loadFlags
   * @struct @constructor
   * @extends {goog.Dependency}
   */
  goog.Es6ModuleDependency = function(
      path, relativePath, provides, requires, loadFlags) {
    goog.Es6ModuleDependency.base(
        this, 'constructor', path, relativePath, provides, requires, loadFlags);
  };
  goog.inherits(goog.Es6ModuleDependency, goog.Dependency);


  /** @override */
  goog.Es6ModuleDependency.prototype.load = function(controller) {
    if (goog.global.CLOSURE_IMPORT_SCRIPT) {
      if (goog.global.CLOSURE_IMPORT_SCRIPT(this.path)) {
        controller.loaded();
      } else {
        controller.pause();
      }
      return;
    }

    if (!goog.inHtmlDocument_()) {
      goog.logToConsole_(
          'Cannot use default debug loader outside of HTML documents.');
      controller.pause();
      return;
    }

    /** @type {!HTMLDocument} */
    var doc = goog.global.document;

    var dep = this;

    // TODO(johnplaisted): Does document.writing really speed up anything? Any
    // difference between this and just waiting for interactive mode and then
    // appending?
    function write(src, contents) {
      if (contents) {
        var script = '<script type="module" crossorigin>' + contents + '</' +
            'script>';
        doc.write(
            goog.TRUSTED_TYPES_POLICY_ ?
                goog.TRUSTED_TYPES_POLICY_.createHTML(script) :
                script);
      } else {
        var script = '<script type="module" crossorigin src="' + src + '"></' +
            'script>';
        doc.write(
            goog.TRUSTED_TYPES_POLICY_ ?
                goog.TRUSTED_TYPES_POLICY_.createHTML(script) :
                script);
      }
    }

    function append(src, contents) {
      var scriptEl =
          /** @type {!HTMLScriptElement} */ (doc.createElement('script'));
      scriptEl.defer = true;
      scriptEl.async = false;
      scriptEl.type = 'module';
      scriptEl.setAttribute('crossorigin', true);

      // If CSP nonces are used, propagate them to dynamically created scripts.
      // This is necessary to allow nonce-based CSPs without 'strict-dynamic'.
      var nonce = goog.getScriptNonce();
      if (nonce) {
        scriptEl.setAttribute('nonce', nonce);
      }

      if (contents) {
        scriptEl.textContent = goog.TRUSTED_TYPES_POLICY_ ?
            goog.TRUSTED_TYPES_POLICY_.createScript(contents) :
            contents;
      } else {
        scriptEl.src = goog.TRUSTED_TYPES_POLICY_ ?
            goog.TRUSTED_TYPES_POLICY_.createScriptURL(src) :
            src;
      }

      doc.head.appendChild(scriptEl);
    }

    var create;

    if (goog.isDocumentLoading_()) {
      create = write;
      // We can ONLY call document.write if we are guaranteed that any
      // non-module script tags document.written after this are deferred.
      // Small optimization, in theory document.writing is faster.
      goog.Dependency.defer_ = true;
    } else {
      create = append;
    }

    // Write 4 separate tags here:
    // 1) Sets the module state at the correct time (just before execution).
    // 2) A src node for this, which just hopefully lets the browser load it a
    //    little early (no need to parse #3).
    // 3) Import the module and register it.
    // 4) Clear the module state at the correct time. Guaranteed to run even
    //    if there is an error in the module (#3 will not run if there is an
    //    error in the module).
    var beforeKey = goog.Dependency.registerCallback_(function() {
      goog.Dependency.unregisterCallback_(beforeKey);
      controller.setModuleState(goog.ModuleType.ES6);
    });
    create(undefined, 'goog.Dependency.callback_("' + beforeKey + '")');

    // TODO(johnplaisted): Does this really speed up anything?
    create(this.path, undefined);

    var registerKey = goog.Dependency.registerCallback_(function(exports) {
      goog.Dependency.unregisterCallback_(registerKey);
      controller.registerEs6ModuleExports(
          dep.path, exports, goog.moduleLoaderState_.moduleName);
    });
    create(
        undefined,
        'import * as m from "' + this.path + '"; goog.Dependency.callback_("' +
            registerKey + '", m)');

    var afterKey = goog.Dependency.registerCallback_(function() {
      goog.Dependency.unregisterCallback_(afterKey);
      controller.clearModuleState();
      controller.loaded();
    });
    create(undefined, 'goog.Dependency.callback_("' + afterKey + '")');
  };


  /**
   * Superclass of any dependency that needs to be loaded into memory,
   * transformed, and then eval'd (goog.modules and transpiled files).
   *
   * @param {string} path Absolute path of this script.
   * @param {string} relativePath Path of this script relative to goog.basePath.
   * @param {!Array<string>} provides goog.provided or goog.module symbols
   *     in this file.
   * @param {!Array<string>} requires goog symbols or relative paths to Closure
   *     this depends on.
   * @param {!Object<string, string>} loadFlags
   * @struct @constructor @abstract
   * @extends {goog.Dependency}
   */
  goog.TransformedDependency = function(
      path, relativePath, provides, requires, loadFlags) {
    goog.TransformedDependency.base(
        this, 'constructor', path, relativePath, provides, requires, loadFlags);
    /** @private {?string} */
    this.contents_ = null;

    /**
     * Whether to lazily make the synchronous XHR (when goog.require'd) or make
     * the synchronous XHR when initially loading. On FireFox 61 there is a bug
     * where an ES6 module cannot make a synchronous XHR (rather, it can, but if
     * it does then no other ES6 modules will load after).
     *
     * tl;dr we lazy load due to bugs on older browsers and eager load due to
     * bugs on newer ones.
     *
     * https://bugzilla.mozilla.org/show_bug.cgi?id=1477090
     *
     * @private @const {boolean}
     */
    this.lazyFetch_ = !goog.inHtmlDocument_() ||
        !('noModule' in goog.global.document.createElement('script'));
  };
  goog.inherits(goog.TransformedDependency, goog.Dependency);


  /** @override */
  goog.TransformedDependency.prototype.load = function(controller) {
    var dep = this;

    function fetch() {
      dep.contents_ = goog.loadFileSync_(dep.path);

      if (dep.contents_) {
        dep.contents_ = dep.transform(dep.contents_);
        if (dep.contents_) {
          dep.contents_ += '\n//# sourceURL=' + dep.path;
        }
      }
    }

    if (goog.global.CLOSURE_IMPORT_SCRIPT) {
      fetch();
      if (this.contents_ &&
          goog.global.CLOSURE_IMPORT_SCRIPT('', this.contents_)) {
        this.contents_ = null;
        controller.loaded();
      } else {
        controller.pause();
      }
      return;
    }


    var isEs6 = this.loadFlags['module'] == goog.ModuleType.ES6;

    if (!this.lazyFetch_) {
      fetch();
    }

    function load() {
      if (dep.lazyFetch_) {
        fetch();
      }

      if (!dep.contents_) {
        // loadFileSync_ or transform are responsible. Assume they logged an
        // error.
        return;
      }

      if (isEs6) {
        controller.setModuleState(goog.ModuleType.ES6);
      }

      var namespace;

      try {
        var contents = dep.contents_;
        dep.contents_ = null;
        goog.globalEval(contents);
        if (isEs6) {
          namespace = goog.moduleLoaderState_.moduleName;
        }
      } finally {
        if (isEs6) {
          controller.clearModuleState();
        }
      }

      if (isEs6) {
        // Due to circular dependencies this may not be available for require
        // right now.
        goog.global['$jscomp']['require']['ensure'](
            [dep.getPathName()], function() {
              controller.registerEs6ModuleExports(
                  dep.path,
                  goog.global['$jscomp']['require'](dep.getPathName()),
                  namespace);
            });
      }

      controller.loaded();
    }

    // Do not fetch now; in FireFox 47 the synchronous XHR doesn't block all
    // events. If we fetched now and then document.write'd the contents the
    // document.write would be an eval and would execute too soon! Instead write
    // a script tag to fetch and eval synchronously at the correct time.
    function fetchInOwnScriptThenLoad() {
      /** @type {!HTMLDocument} */
      var doc = goog.global.document;

      var key = goog.Dependency.registerCallback_(function() {
        goog.Dependency.unregisterCallback_(key);
        load();
      });

      var script = '<script type="text/javascript">' +
          goog.protectScriptTag_('goog.Dependency.callback_("' + key + '");') +
          '</' +
          'script>';
      doc.write(
          goog.TRUSTED_TYPES_POLICY_ ?
              goog.TRUSTED_TYPES_POLICY_.createHTML(script) :
              script);
    }

    // If one thing is pending it is this.
    var anythingElsePending = controller.pending().length > 1;

    // If anything else is loading we need to lazy load due to bugs in old IE.
    // Specifically script tags with src and script tags with contents could
    // execute out of order if document.write is used, so we cannot use
    // document.write. Do not pause here; it breaks old IE as well.
    var useOldIeWorkAround =
        anythingElsePending && goog.DebugLoader_.IS_OLD_IE_;

    // Additionally if we are meant to defer scripts but the page is still
    // loading (e.g. an ES6 module is loading) then also defer. Or if we are
    // meant to defer and anything else is pending then defer (those may be
    // scripts that did not need transformation and are just script tags with
    // defer set to true, and we need to evaluate after that deferred script).
    var needsAsyncLoading = goog.Dependency.defer_ &&
        (anythingElsePending || goog.isDocumentLoading_());

    if (useOldIeWorkAround || needsAsyncLoading) {
      // Note that we only defer when we have to rather than 100% of the time.
      // Always defering would work, but then in theory the order of
      // goog.require calls would then matter. We want to enforce that most of
      // the time the order of the require calls does not matter.
      controller.defer(function() {
        load();
      });
      return;
    }
    // TODO(johnplaisted): Externs are missing onreadystatechange for
    // HTMLDocument.
    /** @type {?} */
    var doc = goog.global.document;

    var isInternetExplorer =
        goog.inHtmlDocument_() && 'ActiveXObject' in goog.global;

    // Don't delay in any version of IE. There's bug around this that will
    // cause out of order script execution. This means that on older IE ES6
    // modules will load too early (while the document is still loading + the
    // dom is not available). The other option is to load too late (when the
    // document is complete and the onload even will never fire). This seems
    // to be the lesser of two evils as scripts already act like the former.
    if (isEs6 && goog.inHtmlDocument_() && goog.isDocumentLoading_() &&
        !isInternetExplorer) {
      goog.Dependency.defer_ = true;
      // Transpiled ES6 modules still need to load like regular ES6 modules,
      // aka only after the document is interactive.
      controller.pause();
      var oldCallback = doc.onreadystatechange;
      doc.onreadystatechange = function() {
        if (doc.readyState == 'interactive') {
          doc.onreadystatechange = oldCallback;
          load();
          controller.resume();
        }
        if (goog.isFunction(oldCallback)) {
          oldCallback.apply(undefined, arguments);
        }
      };
    } else {
      // Always eval on old IE.
      if (goog.DebugLoader_.IS_OLD_IE_ || !goog.inHtmlDocument_() ||
          !goog.isDocumentLoading_()) {
        load();
      } else {
        fetchInOwnScriptThenLoad();
      }
    }
  };


  /**
   * @param {string} contents
   * @return {string}
   * @abstract
   */
  goog.TransformedDependency.prototype.transform = function(contents) {};


  /**
   * Any non-goog.module dependency which needs to be transpiled before eval.
   *
   * @param {string} path Absolute path of this script.
   * @param {string} relativePath Path of this script relative to goog.basePath.
   * @param {!Array<string>} provides goog.provided or goog.module symbols
   *     in this file.
   * @param {!Array<string>} requires goog symbols or relative paths to Closure
   *     this depends on.
   * @param {!Object<string, string>} loadFlags
   * @param {!goog.Transpiler} transpiler
   * @struct @constructor
   * @extends {goog.TransformedDependency}
   */
  goog.TranspiledDependency = function(
      path, relativePath, provides, requires, loadFlags, transpiler) {
    goog.TranspiledDependency.base(
        this, 'constructor', path, relativePath, provides, requires, loadFlags);
    /** @protected @const*/
    this.transpiler = transpiler;
  };
  goog.inherits(goog.TranspiledDependency, goog.TransformedDependency);


  /** @override */
  goog.TranspiledDependency.prototype.transform = function(contents) {
    // Transpile with the pathname so that ES6 modules are domain agnostic.
    return this.transpiler.transpile(contents, this.getPathName());
  };


  /**
   * An ES6 module dependency that was transpiled to a jscomp module outside
   * of the debug loader, e.g. server side.
   *
   * @param {string} path Absolute path of this script.
   * @param {string} relativePath Path of this script relative to goog.basePath.
   * @param {!Array<string>} provides goog.provided or goog.module symbols
   *     in this file.
   * @param {!Array<string>} requires goog symbols or relative paths to Closure
   *     this depends on.
   * @param {!Object<string, string>} loadFlags
   * @struct @constructor
   * @extends {goog.TransformedDependency}
   */
  goog.PreTranspiledEs6ModuleDependency = function(
      path, relativePath, provides, requires, loadFlags) {
    goog.PreTranspiledEs6ModuleDependency.base(
        this, 'constructor', path, relativePath, provides, requires, loadFlags);
  };
  goog.inherits(
      goog.PreTranspiledEs6ModuleDependency, goog.TransformedDependency);


  /** @override */
  goog.PreTranspiledEs6ModuleDependency.prototype.transform = function(
      contents) {
    return contents;
  };


  /**
   * A goog.module, transpiled or not. Will always perform some minimal
   * transformation even when not transpiled to wrap in a goog.loadModule
   * statement.
   *
   * @param {string} path Absolute path of this script.
   * @param {string} relativePath Path of this script relative to goog.basePath.
   * @param {!Array<string>} provides goog.provided or goog.module symbols
   *     in this file.
   * @param {!Array<string>} requires goog symbols or relative paths to Closure
   *     this depends on.
   * @param {!Object<string, string>} loadFlags
   * @param {boolean} needsTranspile
   * @param {!goog.Transpiler} transpiler
   * @struct @constructor
   * @extends {goog.TransformedDependency}
   */
  goog.GoogModuleDependency = function(
      path, relativePath, provides, requires, loadFlags, needsTranspile,
      transpiler) {
    goog.GoogModuleDependency.base(
        this, 'constructor', path, relativePath, provides, requires, loadFlags);
    /** @private @const */
    this.needsTranspile_ = needsTranspile;
    /** @private @const */
    this.transpiler_ = transpiler;
  };
  goog.inherits(goog.GoogModuleDependency, goog.TransformedDependency);


  /** @override */
  goog.GoogModuleDependency.prototype.transform = function(contents) {
    if (this.needsTranspile_) {
      contents = this.transpiler_.transpile(contents, this.getPathName());
    }

    if (!goog.LOAD_MODULE_USING_EVAL || goog.global.JSON === undefined) {
      return '' +
          'goog.loadModule(function(exports) {' +
          '"use strict";' + contents +
          '\n' +  // terminate any trailing single line comment.
          ';return exports' +
          '});' +
          '\n//# sourceURL=' + this.path + '\n';
    } else {
      return '' +
          'goog.loadModule(' +
          goog.global.JSON.stringify(
              contents + '\n//# sourceURL=' + this.path + '\n') +
          ');';
    }
  };


  /**
   * Whether the browser is IE9 or earlier, which needs special handling
   * for deferred modules.
   * @const @private {boolean}
   */
  goog.DebugLoader_.IS_OLD_IE_ = !!(
      !goog.global.atob && goog.global.document && goog.global.document['all']);


  /**
   * @param {string} relPath
   * @param {!Array<string>|undefined} provides
   * @param {!Array<string>} requires
   * @param {boolean|!Object<string>=} opt_loadFlags
   * @see goog.addDependency
   */
  goog.DebugLoader_.prototype.addDependency = function(
      relPath, provides, requires, opt_loadFlags) {
    provides = provides || [];
    relPath = relPath.replace(/\\/g, '/');
    var path = goog.normalizePath_(goog.basePath + relPath);
    if (!opt_loadFlags || typeof opt_loadFlags === 'boolean') {
      opt_loadFlags = opt_loadFlags ? {'module': goog.ModuleType.GOOG} : {};
    }
    var dep = this.factory_.createDependency(
        path, relPath, provides, requires, opt_loadFlags,
        goog.transpiler_.needsTranspile(
            opt_loadFlags['lang'] || 'es3', opt_loadFlags['module']));
    this.dependencies_[path] = dep;
    for (var i = 0; i < provides.length; i++) {
      this.idToPath_[provides[i]] = path;
    }
    this.idToPath_[relPath] = path;
  };


  /**
   * Creates goog.Dependency instances for the debug loader to load.
   *
   * Should be overridden to have the debug loader use custom subclasses of
   * goog.Dependency.
   *
   * @param {!goog.Transpiler} transpiler
   * @struct @constructor
   */
  goog.DependencyFactory = function(transpiler) {
    /** @protected @const */
    this.transpiler = transpiler;
  };


  /**
   * @param {string} path Absolute path of the file.
   * @param {string} relativePath Path relative to closure’s base.js.
   * @param {!Array<string>} provides Array of provided goog.provide/module ids.
   * @param {!Array<string>} requires Array of required goog.provide/module /
   *     relative ES6 module paths.
   * @param {!Object<string, string>} loadFlags
   * @param {boolean} needsTranspile True if the file needs to be transpiled
   *     per the goog.Transpiler.
   * @return {!goog.Dependency}
   */
  goog.DependencyFactory.prototype.createDependency = function(
      path, relativePath, provides, requires, loadFlags, needsTranspile) {
    // MOE:begin_strip
    var provide, require;
    for (var i = 0; provide = provides[i]; i++) {
      goog.dependencies_.nameToPath[provide] = relativePath;
      goog.dependencies_.loadFlags[relativePath] = loadFlags;
    }
    for (var j = 0; require = requires[j]; j++) {
      if (!(relativePath in goog.dependencies_.requires)) {
        goog.dependencies_.requires[relativePath] = {};
      }
      goog.dependencies_.requires[relativePath][require] = true;
    }
    // MOE:end_strip

    if (loadFlags['module'] == goog.ModuleType.GOOG) {
      return new goog.GoogModuleDependency(
          path, relativePath, provides, requires, loadFlags, needsTranspile,
          this.transpiler);
    } else if (needsTranspile) {
      return new goog.TranspiledDependency(
          path, relativePath, provides, requires, loadFlags, this.transpiler);
    } else {
      if (loadFlags['module'] == goog.ModuleType.ES6) {
        if (goog.TRANSPILE == 'never' && goog.ASSUME_ES_MODULES_TRANSPILED) {
          return new goog.PreTranspiledEs6ModuleDependency(
              path, relativePath, provides, requires, loadFlags);
        } else {
          return new goog.Es6ModuleDependency(
              path, relativePath, provides, requires, loadFlags);
        }
      } else {
        return new goog.Dependency(
            path, relativePath, provides, requires, loadFlags);
      }
    }
  };


  /** @private @const */
  goog.debugLoader_ = new goog.DebugLoader_();


  /**
   * Loads the Closure Dependency file.
   *
   * Exposed a public function so CLOSURE_NO_DEPS can be set to false, base
   * loaded, setDependencyFactory called, and then this called. i.e. allows
   * custom loading of the deps file.
   */
  goog.loadClosureDeps = function() {
    goog.debugLoader_.loadClosureDeps();
  };


  /**
   * Sets the dependency factory, which can be used to create custom
   * goog.Dependency implementations to control how dependencies are loaded.
   *
   * Note: if you wish to call this function and provide your own implemnetation
   * it is a wise idea to set CLOSURE_NO_DEPS to true, otherwise the dependency
   * file and all of its goog.addDependency calls will use the default factory.
   * You can call goog.loadClosureDeps to load the Closure dependency file
   * later, after your factory is injected.
   *
   * @param {!goog.DependencyFactory} factory
   */
  goog.setDependencyFactory = function(factory) {
    goog.debugLoader_.setDependencyFactory(factory);
  };


  if (!goog.global.CLOSURE_NO_DEPS) {
    goog.debugLoader_.loadClosureDeps();
  }


  /**
   * Bootstraps the given namespaces and calls the callback once they are
   * available either via goog.require. This is a replacement for using
   * `goog.require` to bootstrap Closure JavaScript. Previously a `goog.require`
   * in an HTML file would guarantee that the require'd namespace was available
   * in the next immediate script tag. With ES6 modules this no longer a
   * guarantee.
   *
   * @param {!Array<string>} namespaces
   * @param {function(): ?} callback Function to call once all the namespaces
   *     have loaded. Always called asynchronously.
   */
  goog.bootstrap = function(namespaces, callback) {
    goog.debugLoader_.bootstrap(namespaces, callback);
  };
}


/**
 * @define {string} Trusted Types policy name. If non-empty then Closure will
 * use Trusted Types.
 */
goog.TRUSTED_TYPES_POLICY_NAME =
    goog.define('goog.TRUSTED_TYPES_POLICY_NAME', '');


/**
 * Returns the parameter.
 * @param {string} s
 * @return {string}
 * @private
 */
goog.identity_ = function(s) {
  return s;
};


/**
 * Creates Trusted Types policy if Trusted Types are supported by the browser.
 * The policy just blesses any string as a Trusted Type. It is not visibility
 * restricted because anyone can also call TrustedTypes.createPolicy directly.
 * However, the allowed names should be restricted by a HTTP header and the
 * reference to the created policy should be visibility restricted.
 * @param {string} name
 * @return {?TrustedTypePolicy}
 */
goog.createTrustedTypesPolicy = function(name) {
  var policy = null;
  // TODO(koto): Remove window.TrustedTypes variant when the newer API ships.
  var policyFactory = goog.global.trustedTypes || goog.global.TrustedTypes;
  if (!policyFactory || !policyFactory.createPolicy) {
    return policy;
  }
  // TrustedTypes.createPolicy throws if called with a name that is already
  // registered, even in report-only mode. Until the API changes, catch the
  // error not to break the applications functionally. In such case, the code
  // will fall back to using regular Safe Types.
  // TODO(koto): Remove catching once createPolicy API stops throwing.
  try {
    policy = policyFactory.createPolicy(name, {
      createHTML: goog.identity_,
      createScript: goog.identity_,
      createScriptURL: goog.identity_,
      createURL: goog.identity_
    });
  } catch (e) {
    goog.logToConsole_(e.message);
  }
  return policy;
};


/** @private @const {?TrustedTypePolicy} */
goog.TRUSTED_TYPES_POLICY_ = goog.TRUSTED_TYPES_POLICY_NAME ?
    goog.createTrustedTypesPolicy(goog.TRUSTED_TYPES_POLICY_NAME + '#base') :
    null;

//javascript/apps/jspb/binary/arith.js
/**
 * @fileoverview This file contains helper code used by jspb.utils to
 * handle 64-bit integer conversion to/from strings.
 *
 * @author cfallin@google.com (Chris Fallin)
 *
 * TODO(haberman): move this to javascript/closure/math?
 */

goog.provide('jspb.arith.Int64');
goog.provide('jspb.arith.UInt64');

/**
 * UInt64 implements some 64-bit arithmetic routines necessary for properly
 * handling 64-bit integer fields. It implements lossless integer arithmetic on
 * top of JavaScript's number type, which has only 53 bits of precision, by
 * representing 64-bit integers as two 32-bit halves.
 *
 * @param {number} lo The low 32 bits.
 * @param {number} hi The high 32 bits.
 * @constructor
 */
jspb.arith.UInt64 = function(lo, hi) {
  /**
   * The low 32 bits.
   * @public {number}
   */
  this.lo = lo;
  /**
   * The high 32 bits.
   * @public {number}
   */
  this.hi = hi;
};


/**
 * Compare two 64-bit numbers. Returns -1 if the first is
 * less, +1 if the first is greater, or 0 if both are equal.
 * @param {!jspb.arith.UInt64} other
 * @return {number}
 */
jspb.arith.UInt64.prototype.cmp = function(other) {
  if (this.hi < other.hi || (this.hi == other.hi && this.lo < other.lo)) {
    return -1;
  } else if (this.hi == other.hi && this.lo == other.lo) {
    return 0;
  } else {
    return 1;
  }
};


/**
 * Right-shift this number by one bit.
 * @return {!jspb.arith.UInt64}
 */
jspb.arith.UInt64.prototype.rightShift = function() {
  var hi = this.hi >>> 1;
  var lo = (this.lo >>> 1) | ((this.hi & 1) << 31);
  return new jspb.arith.UInt64(lo >>> 0, hi >>> 0);
};


/**
 * Left-shift this number by one bit.
 * @return {!jspb.arith.UInt64}
 */
jspb.arith.UInt64.prototype.leftShift = function() {
  var lo = this.lo << 1;
  var hi = (this.hi << 1) | (this.lo >>> 31);
  return new jspb.arith.UInt64(lo >>> 0, hi >>> 0);
};


/**
 * Test the MSB.
 * @return {boolean}
 */
jspb.arith.UInt64.prototype.msb = function() {
  return !!(this.hi & 0x80000000);
};


/**
 * Test the LSB.
 * @return {boolean}
 */
jspb.arith.UInt64.prototype.lsb = function() {
  return !!(this.lo & 1);
};


/**
 * Test whether this number is zero.
 * @return {boolean}
 */
jspb.arith.UInt64.prototype.zero = function() {
  return this.lo == 0 && this.hi == 0;
};


/**
 * Add two 64-bit numbers to produce a 64-bit number.
 * @param {!jspb.arith.UInt64} other
 * @return {!jspb.arith.UInt64}
 */
jspb.arith.UInt64.prototype.add = function(other) {
  var lo = ((this.lo + other.lo) & 0xffffffff) >>> 0;
  var hi =
      (((this.hi + other.hi) & 0xffffffff) >>> 0) +
      (((this.lo + other.lo) >= 0x100000000) ? 1 : 0);
  return new jspb.arith.UInt64(lo >>> 0, hi >>> 0);
};


/**
 * Subtract two 64-bit numbers to produce a 64-bit number.
 * @param {!jspb.arith.UInt64} other
 * @return {!jspb.arith.UInt64}
 */
jspb.arith.UInt64.prototype.sub = function(other) {
  var lo = ((this.lo - other.lo) & 0xffffffff) >>> 0;
  var hi =
      (((this.hi - other.hi) & 0xffffffff) >>> 0) -
      (((this.lo - other.lo) < 0) ? 1 : 0);
  return new jspb.arith.UInt64(lo >>> 0, hi >>> 0);
};


/**
 * Multiply two 32-bit numbers to produce a 64-bit number.
 * @param {number} a The first integer:  must be in [0, 2^32-1).
 * @param {number} b The second integer: must be in [0, 2^32-1).
 * @return {!jspb.arith.UInt64}
 */
jspb.arith.UInt64.mul32x32 = function(a, b) {
  // Directly multiplying two 32-bit numbers may produce up to 64 bits of
  // precision, thus losing precision because of the 53-bit mantissa of
  // JavaScript numbers. So we multiply with 16-bit digits (radix 65536)
  // instead.
  var aLow = (a & 0xffff);
  var aHigh = (a >>> 16);
  var bLow = (b & 0xffff);
  var bHigh = (b >>> 16);
  var productLow =
      // 32-bit result, result bits 0-31, take all 32 bits
      (aLow * bLow) +
      // 32-bit result, result bits 16-47, take bottom 16 as our top 16
      ((aLow * bHigh) & 0xffff) * 0x10000 +
      // 32-bit result, result bits 16-47, take bottom 16 as our top 16
      ((aHigh * bLow) & 0xffff) * 0x10000;
  var productHigh =
      // 32-bit result, result bits 32-63, take all 32 bits
      (aHigh * bHigh) +
      // 32-bit result, result bits 16-47, take top 16 as our bottom 16
      ((aLow * bHigh) >>> 16) +
      // 32-bit result, result bits 16-47, take top 16 as our bottom 16
      ((aHigh * bLow) >>> 16);

  // Carry. Note that we actually have up to *two* carries due to addition of
  // three terms.
  while (productLow >= 0x100000000) {
    productLow -= 0x100000000;
    productHigh += 1;
  }

  return new jspb.arith.UInt64(productLow >>> 0, productHigh >>> 0);
};


/**
 * Multiply this number by a 32-bit number, producing a 96-bit number, then
 * truncate the top 32 bits.
 * @param {number} a The multiplier.
 * @return {!jspb.arith.UInt64}
 */
jspb.arith.UInt64.prototype.mul = function(a) {
  // Produce two parts: at bits 0-63, and 32-95.
  var lo = jspb.arith.UInt64.mul32x32(this.lo, a);
  var hi = jspb.arith.UInt64.mul32x32(this.hi, a);
  // Left-shift hi by 32 bits, truncating its top bits. The parts will then be
  // aligned for addition.
  hi.hi = hi.lo;
  hi.lo = 0;
  return lo.add(hi);
};


/**
 * Divide a 64-bit number by a 32-bit number to produce a
 * 64-bit quotient and a 32-bit remainder.
 * @param {number} _divisor
 * @return {!Array<jspb.arith.UInt64>} array of [quotient, remainder],
 * unless divisor is 0, in which case an empty array is returned.
 */
jspb.arith.UInt64.prototype.div = function(_divisor) {
  if (_divisor == 0) {
    return [];
  }

  // We perform long division using a radix-2 algorithm, for simplicity (i.e.,
  // one bit at a time). TODO: optimize to a radix-2^32 algorithm, taking care
  // to get the variable shifts right.
  var quotient = new jspb.arith.UInt64(0, 0);
  var remainder = new jspb.arith.UInt64(this.lo, this.hi);
  var divisor = new jspb.arith.UInt64(_divisor, 0);
  var unit = new jspb.arith.UInt64(1, 0);

  // Left-shift the divisor and unit until the high bit of divisor is set.
  while (!divisor.msb()) {
    divisor = divisor.leftShift();
    unit = unit.leftShift();
  }

  // Perform long division one bit at a time.
  while (!unit.zero()) {
    // If divisor < remainder, add unit to quotient and subtract divisor from
    // remainder.
    if (divisor.cmp(remainder) <= 0) {
      quotient = quotient.add(unit);
      remainder = remainder.sub(divisor);
    }
    // Right-shift the divisor and unit.
    divisor = divisor.rightShift();
    unit = unit.rightShift();
  }

  return [quotient, remainder];
};


/**
 * Convert a 64-bit number to a string.
 * @return {string}
 * @override
 */
jspb.arith.UInt64.prototype.toString = function() {
  var result = '';
  var num = this;
  while (!num.zero()) {
    var divResult = num.div(10);
    var quotient = divResult[0], remainder = divResult[1];
    result = remainder.lo + result;
    num = quotient;
  }
  if (result == '') {
    result = '0';
  }
  return result;
};


/**
 * Parse a string into a 64-bit number. Returns `null` on a parse error.
 * @param {string} s
 * @return {?jspb.arith.UInt64}
 */
jspb.arith.UInt64.fromString = function(s) {
  var result = new jspb.arith.UInt64(0, 0);
  // optimization: reuse this instance for each digit.
  var digit64 = new jspb.arith.UInt64(0, 0);
  for (var i = 0; i < s.length; i++) {
    if (s[i] < '0' || s[i] > '9') {
      return null;
    }
    var digit = parseInt(s[i], 10);
    digit64.lo = digit;
    result = result.mul(10).add(digit64);
  }
  return result;
};


/**
 * Make a copy of the uint64.
 * @return {!jspb.arith.UInt64}
 */
jspb.arith.UInt64.prototype.clone = function() {
  return new jspb.arith.UInt64(this.lo, this.hi);
};


/**
 * Int64 is like UInt64, but modifies string conversions to interpret the stored
 * 64-bit value as a twos-complement-signed integer. It does *not* support the
 * full range of operations that UInt64 does: only add, subtract, and string
 * conversions.
 *
 * N.B. that multiply and divide routines are *NOT* supported. They will throw
 * exceptions. (They are not necessary to implement string conversions, which
 * are the only operations we really need in jspb.)
 *
 * @param {number} lo The low 32 bits.
 * @param {number} hi The high 32 bits.
 * @constructor
 */
jspb.arith.Int64 = function(lo, hi) {
  /**
   * The low 32 bits.
   * @public {number}
   */
  this.lo = lo;
  /**
   * The high 32 bits.
   * @public {number}
   */
  this.hi = hi;
};


/**
 * Add two 64-bit numbers to produce a 64-bit number.
 * @param {!jspb.arith.Int64} other
 * @return {!jspb.arith.Int64}
 */
jspb.arith.Int64.prototype.add = function(other) {
  var lo = ((this.lo + other.lo) & 0xffffffff) >>> 0;
  var hi =
      (((this.hi + other.hi) & 0xffffffff) >>> 0) +
      (((this.lo + other.lo) >= 0x100000000) ? 1 : 0);
  return new jspb.arith.Int64(lo >>> 0, hi >>> 0);
};


/**
 * Subtract two 64-bit numbers to produce a 64-bit number.
 * @param {!jspb.arith.Int64} other
 * @return {!jspb.arith.Int64}
 */
jspb.arith.Int64.prototype.sub = function(other) {
  var lo = ((this.lo - other.lo) & 0xffffffff) >>> 0;
  var hi =
      (((this.hi - other.hi) & 0xffffffff) >>> 0) -
      (((this.lo - other.lo) < 0) ? 1 : 0);
  return new jspb.arith.Int64(lo >>> 0, hi >>> 0);
};


/**
 * Make a copy of the int64.
 * @return {!jspb.arith.Int64}
 */
jspb.arith.Int64.prototype.clone = function() {
  return new jspb.arith.Int64(this.lo, this.hi);
};


/**
 * Convert a 64-bit number to a string.
 * @return {string}
 * @override
 */
jspb.arith.Int64.prototype.toString = function() {
  // If the number is negative, find its twos-complement inverse.
  var sign = (this.hi & 0x80000000) != 0;
  var num = new jspb.arith.UInt64(this.lo, this.hi);
  if (sign) {
    num = new jspb.arith.UInt64(0, 0).sub(num);
  }
  return (sign ? '-' : '') + num.toString();
};


/**
 * Parse a string into a 64-bit number. Returns `null` on a parse error.
 * @param {string} s
 * @return {?jspb.arith.Int64}
 */
jspb.arith.Int64.fromString = function(s) {
  var hasNegative = (s.length > 0 && s[0] == '-');
  if (hasNegative) {
    s = s.substring(1);
  }
  var num = jspb.arith.UInt64.fromString(s);
  if (num === null) {
    return null;
  }
  if (hasNegative) {
    num = new jspb.arith.UInt64(0, 0).sub(num);
  }
  return new jspb.arith.Int64(num.lo, num.hi);
};

//javascript/apps/jspb/binary/constants.js
/**
 * @fileoverview This file contains constants and typedefs used by
 * jspb.BinaryReader and BinaryWriter.
 *
 * @author aappleby@google.com (Austin Appleby)
 */

goog.provide('jspb.AnyFieldType');
goog.provide('jspb.BinaryConstants');
goog.provide('jspb.BinaryMessage');
goog.provide('jspb.BuilderFunction');
goog.provide('jspb.ByteSource');
goog.provide('jspb.ClonerFunction');
goog.provide('jspb.ComparerFunction');
goog.provide('jspb.ConstBinaryMessage');
goog.provide('jspb.PrunerFunction');
goog.provide('jspb.ReaderFunction');
goog.provide('jspb.RecyclerFunction');
goog.provide('jspb.RepeatedFieldType');
goog.provide('jspb.ScalarFieldType');
goog.provide('jspb.WriterFunction');


goog.forwardDeclare('jspb.BinaryReader');
goog.forwardDeclare('jspb.BinaryWriter');
goog.forwardDeclare('jspb.Message');
goog.forwardDeclare('jsprotolib.BinaryExtension');



/**
 * Base interface class for all const messages.
 * @interface
 */
jspb.ConstBinaryMessage = function() {};

/**
 * Generate a debug string for this proto that is in proto2 text format.
 * @return {string} The debug string.
 */
jspb.ConstBinaryMessage.prototype.toDebugString;

/**
 * Helper to generate a debug string for this proto at some indent level. The
 * first line is not indented.
 * @param {number} indentLevel The number of spaces by which to indent lines.
 * @return {string} The debug string.
 * @protected
 */
jspb.ConstBinaryMessage.prototype.toDebugStringInternal;

/**
 * Base interface class for all messages. Does __not__ define any methods, as
 * doing so on a widely-used interface defeats dead-code elimination.
 * @interface
 * @extends {jspb.ConstBinaryMessage}
 */
jspb.BinaryMessage = function() {};


/**
 * The types convertible to Uint8Arrays. Strings are assumed to be
 * base64-encoded.
 * @typedef {ArrayBuffer|Uint8Array|Array<number>|string}
 */
jspb.ByteSource;


/**
 * A scalar field in jspb can be a boolean, number, or string.
 * @typedef {boolean|number|string}
 */
jspb.ScalarFieldType;


/**
 * A repeated field in jspb is an array of scalars, blobs, or messages.
 * @typedef {!Array<jspb.ScalarFieldType>|
             !Array<!Uint8Array>|
             !Array<!jspb.ConstBinaryMessage>|
             !Array<!jspb.BinaryMessage>}
 */
jspb.RepeatedFieldType;


/**
 * A field in jspb can be a scalar, a block of bytes, another proto, or an
 * array of any of the above.
 * @typedef {jspb.ScalarFieldType|
             jspb.RepeatedFieldType|
             !Uint8Array|
             !jspb.ConstBinaryMessage|
             !jspb.BinaryMessage|
             !jsprotolib.BinaryExtension}
 */
jspb.AnyFieldType;


/**
 * A builder function creates an instance of a message object.
 * @typedef {function():!jspb.BinaryMessage}
 */
jspb.BuilderFunction;


/**
 * A cloner function creates a deep copy of a message object.
 * @typedef {function(jspb.ConstBinaryMessage):jspb.BinaryMessage}
 */
jspb.ClonerFunction;


/**
 * A recycler function destroys an instance of a message object.
 * @typedef {function(!jspb.BinaryMessage):void}
 */
jspb.RecyclerFunction;


/**
 * A reader function initializes a message using data from a BinaryReader.
 * @typedef {function(!jspb.BinaryMessage, !jspb.BinaryReader):void}
 */
jspb.ReaderFunction;


/**
 * A writer function serializes a message to a BinaryWriter.
 * @typedef {function((!jspb.Message|!jspb.ConstBinaryMessage),
 *                    !jspb.BinaryWriter):void}
 */
jspb.WriterFunction;


/**
 * A pruner function removes default-valued fields and empty submessages from a
 * message and returns either the pruned message or null if the entire message
 * was pruned away.
 * @typedef {function(?jspb.BinaryMessage):?jspb.BinaryMessage}
 */
jspb.PrunerFunction;


/**
 * A comparer function returns true if two protos are equal.
 * @typedef {function(?jspb.ConstBinaryMessage,
 *                     ?jspb.ConstBinaryMessage):boolean}
 */
jspb.ComparerFunction;


/**
 * Field type codes, taken from proto2/public/wire_format_lite.h.
 * @enum {number}
 */
jspb.BinaryConstants.FieldType = {
  INVALID: -1,
  DOUBLE: 1,
  FLOAT: 2,
  INT64: 3,
  UINT64: 4,
  INT32: 5,
  FIXED64: 6,
  FIXED32: 7,
  BOOL: 8,
  STRING: 9,
  GROUP: 10,
  MESSAGE: 11,
  BYTES: 12,
  UINT32: 13,
  ENUM: 14,
  SFIXED32: 15,
  SFIXED64: 16,
  SINT32: 17,
  SINT64: 18,

  // Extended types for Javascript

  FHASH64: 30, // 64-bit hash string, fixed-length encoding.
  VHASH64: 31  // 64-bit hash string, varint encoding.
};


/**
 * Wire-format type codes, taken from proto2/public/wire_format_lite.h.
 * @enum {number}
 */
jspb.BinaryConstants.WireType = {
  INVALID: -1,
  VARINT: 0,
  FIXED64: 1,
  DELIMITED: 2,
  START_GROUP: 3,
  END_GROUP: 4,
  FIXED32: 5
};


/**
 * Translates field type to wire type.
 * @param {jspb.BinaryConstants.FieldType} fieldType
 * @return {jspb.BinaryConstants.WireType}
 */
jspb.BinaryConstants.FieldTypeToWireType = function(fieldType) {
  var fieldTypes = jspb.BinaryConstants.FieldType;
  var wireTypes = jspb.BinaryConstants.WireType;
  switch (fieldType) {
    case fieldTypes.INT32:
    case fieldTypes.INT64:
    case fieldTypes.UINT32:
    case fieldTypes.UINT64:
    case fieldTypes.SINT32:
    case fieldTypes.SINT64:
    case fieldTypes.BOOL:
    case fieldTypes.ENUM:
    case fieldTypes.VHASH64:
      return wireTypes.VARINT;

    case fieldTypes.DOUBLE:
    case fieldTypes.FIXED64:
    case fieldTypes.SFIXED64:
    case fieldTypes.FHASH64:
      return wireTypes.FIXED64;

    case fieldTypes.STRING:
    case fieldTypes.MESSAGE:
    case fieldTypes.BYTES:
      return wireTypes.DELIMITED;

    case fieldTypes.FLOAT:
    case fieldTypes.FIXED32:
    case fieldTypes.SFIXED32:
      return wireTypes.FIXED32;

    case fieldTypes.INVALID:
    case fieldTypes.GROUP:
    default:
      return wireTypes.INVALID;
  }
};


/**
 * Flag to indicate a missing field.
 * @const {number}
 */
jspb.BinaryConstants.INVALID_FIELD_NUMBER = -1;


/**
 * The smallest denormal float32 value.
 * @const {number}
 */
jspb.BinaryConstants.FLOAT32_EPS = 1.401298464324817e-45;


/**
 * The smallest normal float64 value.
 * @const {number}
 */
jspb.BinaryConstants.FLOAT32_MIN = 1.1754943508222875e-38;


/**
 * The largest finite float32 value.
 * @const {number}
 */
jspb.BinaryConstants.FLOAT32_MAX = 3.4028234663852886e+38;


/**
 * The smallest denormal float64 value.
 * @const {number}
 */
jspb.BinaryConstants.FLOAT64_EPS = 5e-324;


/**
 * The smallest normal float64 value.
 * @const {number}
 */
jspb.BinaryConstants.FLOAT64_MIN = 2.2250738585072014e-308;


/**
 * The largest finite float64 value.
 * @const {number}
 */
jspb.BinaryConstants.FLOAT64_MAX = 1.7976931348623157e+308;


/**
 * Convenience constant equal to 2^20.
 * @const {number}
 */
jspb.BinaryConstants.TWO_TO_20 = 1048576;


/**
 * Convenience constant equal to 2^23.
 * @const {number}
 */
jspb.BinaryConstants.TWO_TO_23 = 8388608;


/**
 * Convenience constant equal to 2^31.
 * @const {number}
 */
jspb.BinaryConstants.TWO_TO_31 = 2147483648;


/**
 * Convenience constant equal to 2^32.
 * @const {number}
 */
jspb.BinaryConstants.TWO_TO_32 = 4294967296;


/**
 * Convenience constant equal to 2^52.
 * @const {number}
 */
jspb.BinaryConstants.TWO_TO_52 = 4503599627370496;


/**
 * Convenience constant equal to 2^63.
 * @const {number}
 */
jspb.BinaryConstants.TWO_TO_63 = 9223372036854775808;


/**
 * Convenience constant equal to 2^64.
 * @const {number}
 */
jspb.BinaryConstants.TWO_TO_64 = 18446744073709551616;


/**
 * Eight-character string of zeros, used as the default 64-bit hash value.
 * @const {string}
 */
jspb.BinaryConstants.ZERO_HASH = '\0\0\0\0\0\0\0\0';

//third_party/javascript/closure/debug/error.js
/**
 * @license
 * Copyright The Closure Library Authors.
 * SPDX-License-Identifier: Apache-2.0
 */

/**
 * @fileoverview Provides a base class for custom Error objects such that the
 * stack is correctly maintained.
 *
 * You should never need to throw goog.debug.Error(msg) directly, Error(msg) is
 * sufficient.
 */

goog.provide('goog.debug.Error');



/**
 * Base class for custom error objects.
 * @param {*=} opt_msg The message associated with the error.
 * @constructor
 * @extends {Error}
 */
goog.debug.Error = function(opt_msg) {

  // Attempt to ensure there is a stack trace.
  if (Error.captureStackTrace) {
    Error.captureStackTrace(this, goog.debug.Error);
  } else {
    const stack = new Error().stack;
    if (stack) {
      /** @override */
      this.stack = stack;
    }
  }

  if (opt_msg) {
    /** @override */
    this.message = String(opt_msg);
  }

  /**
   * Whether to report this error to the server. Setting this to false will
   * cause the error reporter to not report the error back to the server,
   * which can be useful if the client knows that the error has already been
   * logged on the server.
   * @type {boolean}
   */
  this.reportErrorToServer = true;
};
goog.inherits(goog.debug.Error, Error);


/** @override */
goog.debug.Error.prototype.name = 'CustomError';

//third_party/javascript/closure/dom/nodetype.js
/**
 * @license
 * Copyright The Closure Library Authors.
 * SPDX-License-Identifier: Apache-2.0
 */

/**
 * @fileoverview Definition of goog.dom.NodeType.
 */

goog.provide('goog.dom.NodeType');


/**
 * Constants for the nodeType attribute in the Node interface.
 *
 * These constants match those specified in the Node interface. These are
 * usually present on the Node object in recent browsers, but not in older
 * browsers (specifically, early IEs) and thus are given here.
 *
 * In some browsers (early IEs), these are not defined on the Node object,
 * so they are provided here.
 *
 * See http://www.w3.org/TR/DOM-Level-2-Core/core.html#ID-1950641247
 * @enum {number}
 */
goog.dom.NodeType = {
  ELEMENT: 1,
  ATTRIBUTE: 2,
  TEXT: 3,
  CDATA_SECTION: 4,
  ENTITY_REFERENCE: 5,
  ENTITY: 6,
  PROCESSING_INSTRUCTION: 7,
  COMMENT: 8,
  DOCUMENT: 9,
  DOCUMENT_TYPE: 10,
  DOCUMENT_FRAGMENT: 11,
  NOTATION: 12
};

//third_party/javascript/closure/asserts/asserts.js
/**
 * @license
 * Copyright The Closure Library Authors.
 * SPDX-License-Identifier: Apache-2.0
 */

/**
 * @fileoverview Utilities to check the preconditions, postconditions and
 * invariants runtime.
 *
 * Methods in this package are given special treatment by the compiler
 * for type-inference. For example, <code>goog.asserts.assert(foo)</code>
 * will make the compiler treat <code>foo</code> as non-nullable. Similarly,
 * <code>goog.asserts.assertNumber(foo)</code> informs the compiler about the
 * type of <code>foo</code>. Where applicable, such assertions are preferable to
 * casts by jsdoc with <code>@type</code>.
 *
 * The compiler has an option to disable asserts. So code like:
 * <code>
 * var x = goog.asserts.assert(foo());
 * goog.asserts.assert(bar());
 * </code>
 * will be transformed into:
 * <code>
 * var x = foo();
 * </code>
 * The compiler will leave in foo() (because its return value is used),
 * but it will remove bar() because it assumes it does not have side-effects.
 *
 * Additionally, note the compiler will consider the type to be "tightened" for
 * all statements <em>after</em> the assertion. For example:
 * <code>
 * const /** ?Object &#ast;/ value = foo();
 * goog.asserts.assert(value);
 * // "value" is of type {!Object} at this point.
 * </code>
 */

goog.provide('goog.asserts');
goog.provide('goog.asserts.AssertionError');

goog.require('goog.debug.Error');
goog.require('goog.dom.NodeType');


/**
 * @define {boolean} Whether to strip out asserts or to leave them in.
 */
goog.asserts.ENABLE_ASSERTS =
    goog.define('goog.asserts.ENABLE_ASSERTS', goog.DEBUG);



/**
 * Error object for failed assertions.
 * @param {string} messagePattern The pattern that was used to form message.
 * @param {!Array<*>} messageArgs The items to substitute into the pattern.
 * @constructor
 * @extends {goog.debug.Error}
 * @final
 */
goog.asserts.AssertionError = function(messagePattern, messageArgs) {
  goog.debug.Error.call(this, goog.asserts.subs_(messagePattern, messageArgs));

  /**
   * The message pattern used to format the error message. Error handlers can
   * use this to uniquely identify the assertion.
   * @type {string}
   */
  this.messagePattern = messagePattern;
};
goog.inherits(goog.asserts.AssertionError, goog.debug.Error);


/** @override */
goog.asserts.AssertionError.prototype.name = 'AssertionError';


/**
 * The default error handler.
 * @param {!goog.asserts.AssertionError} e The exception to be handled.
 */
goog.asserts.DEFAULT_ERROR_HANDLER = function(e) {
  throw e;
};


/**
 * The handler responsible for throwing or logging assertion errors.
 * @private {function(!goog.asserts.AssertionError)}
 */
goog.asserts.errorHandler_ = goog.asserts.DEFAULT_ERROR_HANDLER;


/**
 * Does simple python-style string substitution.
 * subs("foo%s hot%s", "bar", "dog") becomes "foobar hotdog".
 * @param {string} pattern The string containing the pattern.
 * @param {!Array<*>} subs The items to substitute into the pattern.
 * @return {string} A copy of `str` in which each occurrence of
 *     {@code %s} has been replaced an argument from `var_args`.
 * @private
 */
goog.asserts.subs_ = function(pattern, subs) {
  var splitParts = pattern.split('%s');
  var returnString = '';

  // Replace up to the last split part. We are inserting in the
  // positions between split parts.
  var subLast = splitParts.length - 1;
  for (var i = 0; i < subLast; i++) {
    // keep unsupplied as '%s'
    var sub = (i < subs.length) ? subs[i] : '%s';
    returnString += splitParts[i] + sub;
  }
  return returnString + splitParts[subLast];
};


/**
 * Throws an exception with the given message and "Assertion failed" prefixed
 * onto it.
 * @param {string} defaultMessage The message to use if givenMessage is empty.
 * @param {Array<*>} defaultArgs The substitution arguments for defaultMessage.
 * @param {string|undefined} givenMessage Message supplied by the caller.
 * @param {Array<*>} givenArgs The substitution arguments for givenMessage.
 * @throws {goog.asserts.AssertionError} When the value is not a number.
 * @private
 */
goog.asserts.doAssertFailure_ = function(
    defaultMessage, defaultArgs, givenMessage, givenArgs) {
  var message = 'Assertion failed';
  if (givenMessage) {
    message += ': ' + givenMessage;
    var args = givenArgs;
  } else if (defaultMessage) {
    message += ': ' + defaultMessage;
    args = defaultArgs;
  }
  // The '' + works around an Opera 10 bug in the unit tests. Without it,
  // a stack trace is added to var message above. With this, a stack trace is
  // not added until this line (it causes the extra garbage to be added after
  // the assertion message instead of in the middle of it).
  var e = new goog.asserts.AssertionError('' + message, args || []);
  goog.asserts.errorHandler_(e);
};


/**
 * Sets a custom error handler that can be used to customize the behavior of
 * assertion failures, for example by turning all assertion failures into log
 * messages.
 * @param {function(!goog.asserts.AssertionError)} errorHandler
 */
goog.asserts.setErrorHandler = function(errorHandler) {
  if (goog.asserts.ENABLE_ASSERTS) {
    goog.asserts.errorHandler_ = errorHandler;
  }
};


/**
 * Checks if the condition evaluates to true if goog.asserts.ENABLE_ASSERTS is
 * true.
 * @template T
 * @param {T} condition The condition to check.
 * @param {string=} opt_message Error message in case of failure.
 * @param {...*} var_args The items to substitute into the failure message.
 * @return {T} The value of the condition.
 * @throws {goog.asserts.AssertionError} When the condition evaluates to false.
 * @closurePrimitive {asserts.truthy}
 */
goog.asserts.assert = function(condition, opt_message, var_args) {
  if (goog.asserts.ENABLE_ASSERTS && !condition) {
    goog.asserts.doAssertFailure_(
        '', null, opt_message, Array.prototype.slice.call(arguments, 2));
  }
  return condition;
};


/**
 * Checks if `value` is `null` or `undefined` if goog.asserts.ENABLE_ASSERTS is
 * true.
 *
 * @param {T} value The value to check.
 * @param {string=} opt_message Error message in case of failure.
 * @param {...*} var_args The items to substitute into the failure message.
 * @return {R} `value` with its type narrowed to exclude `null` and `undefined`.
 *
 * @template T
 * @template R :=
 *     mapunion(T, (V) =>
 *         cond(eq(V, 'null'),
 *             none(),
 *             cond(eq(V, 'undefined'),
 *                 none(),
 *                 V)))
 *  =:
 *
 * @throws {!goog.asserts.AssertionError} When `value` is `null` or `undefined`.
 * @closurePrimitive {asserts.matchesReturn}
 */
goog.asserts.assertExists = function(value, opt_message, var_args) {
  if (goog.asserts.ENABLE_ASSERTS && value == null) {
    goog.asserts.doAssertFailure_(
        'Expected to exist: %s.', [value], opt_message,
        Array.prototype.slice.call(arguments, 2));
  }
  return value;
};


/**
 * Fails if goog.asserts.ENABLE_ASSERTS is true. This function is useful in case
 * when we want to add a check in the unreachable area like switch-case
 * statement:
 *
 * <pre>
 *  switch(type) {
 *    case FOO: doSomething(); break;
 *    case BAR: doSomethingElse(); break;
 *    default: goog.asserts.fail('Unrecognized type: ' + type);
 *      // We have only 2 types - "default:" section is unreachable code.
 *  }
 * </pre>
 *
 * @param {string=} opt_message Error message in case of failure.
 * @param {...*} var_args The items to substitute into the failure message.
 * @throws {goog.asserts.AssertionError} Failure.
 * @closurePrimitive {asserts.fail}
 */
goog.asserts.fail = function(opt_message, var_args) {
  if (goog.asserts.ENABLE_ASSERTS) {
    goog.asserts.errorHandler_(new goog.asserts.AssertionError(
        'Failure' + (opt_message ? ': ' + opt_message : ''),
        Array.prototype.slice.call(arguments, 1)));
  }
};


/**
 * Checks if the value is a number if goog.asserts.ENABLE_ASSERTS is true.
 * @param {*} value The value to check.
 * @param {string=} opt_message Error message in case of failure.
 * @param {...*} var_args The items to substitute into the failure message.
 * @return {number} The value, guaranteed to be a number when asserts enabled.
 * @throws {goog.asserts.AssertionError} When the value is not a number.
 * @closurePrimitive {asserts.matchesReturn}
 */
goog.asserts.assertNumber = function(value, opt_message, var_args) {
  if (goog.asserts.ENABLE_ASSERTS && typeof value !== 'number') {
    goog.asserts.doAssertFailure_(
        'Expected number but got %s: %s.', [goog.typeOf(value), value],
        opt_message, Array.prototype.slice.call(arguments, 2));
  }
  return /** @type {number} */ (value);
};


/**
 * Checks if the value is a string if goog.asserts.ENABLE_ASSERTS is true.
 * @param {*} value The value to check.
 * @param {string=} opt_message Error message in case of failure.
 * @param {...*} var_args The items to substitute into the failure message.
 * @return {string} The value, guaranteed to be a string when asserts enabled.
 * @throws {goog.asserts.AssertionError} When the value is not a string.
 * @closurePrimitive {asserts.matchesReturn}
 */
goog.asserts.assertString = function(value, opt_message, var_args) {
  if (goog.asserts.ENABLE_ASSERTS && typeof value !== 'string') {
    goog.asserts.doAssertFailure_(
        'Expected string but got %s: %s.', [goog.typeOf(value), value],
        opt_message, Array.prototype.slice.call(arguments, 2));
  }
  return /** @type {string} */ (value);
};


/**
 * Checks if the value is a function if goog.asserts.ENABLE_ASSERTS is true.
 * @param {*} value The value to check.
 * @param {string=} opt_message Error message in case of failure.
 * @param {...*} var_args The items to substitute into the failure message.
 * @return {!Function} The value, guaranteed to be a function when asserts
 *     enabled.
 * @throws {goog.asserts.AssertionError} When the value is not a function.
 * @closurePrimitive {asserts.matchesReturn}
 */
goog.asserts.assertFunction = function(value, opt_message, var_args) {
  if (goog.asserts.ENABLE_ASSERTS && !goog.isFunction(value)) {
    goog.asserts.doAssertFailure_(
        'Expected function but got %s: %s.', [goog.typeOf(value), value],
        opt_message, Array.prototype.slice.call(arguments, 2));
  }
  return /** @type {!Function} */ (value);
};


/**
 * Checks if the value is an Object if goog.asserts.ENABLE_ASSERTS is true.
 * @param {*} value The value to check.
 * @param {string=} opt_message Error message in case of failure.
 * @param {...*} var_args The items to substitute into the failure message.
 * @return {!Object} The value, guaranteed to be a non-null object.
 * @throws {goog.asserts.AssertionError} When the value is not an object.
 * @closurePrimitive {asserts.matchesReturn}
 */
goog.asserts.assertObject = function(value, opt_message, var_args) {
  if (goog.asserts.ENABLE_ASSERTS && !goog.isObject(value)) {
    goog.asserts.doAssertFailure_(
        'Expected object but got %s: %s.', [goog.typeOf(value), value],
        opt_message, Array.prototype.slice.call(arguments, 2));
  }
  return /** @type {!Object} */ (value);
};


/**
 * Checks if the value is an Array if goog.asserts.ENABLE_ASSERTS is true.
 * @param {*} value The value to check.
 * @param {string=} opt_message Error message in case of failure.
 * @param {...*} var_args The items to substitute into the failure message.
 * @return {!Array<?>} The value, guaranteed to be a non-null array.
 * @throws {goog.asserts.AssertionError} When the value is not an array.
 * @closurePrimitive {asserts.matchesReturn}
 */
goog.asserts.assertArray = function(value, opt_message, var_args) {
  if (goog.asserts.ENABLE_ASSERTS && !Array.isArray(value)) {
    goog.asserts.doAssertFailure_(
        'Expected array but got %s: %s.', [goog.typeOf(value), value],
        opt_message, Array.prototype.slice.call(arguments, 2));
  }
  return /** @type {!Array<?>} */ (value);
};


/**
 * Checks if the value is a boolean if goog.asserts.ENABLE_ASSERTS is true.
 * @param {*} value The value to check.
 * @param {string=} opt_message Error message in case of failure.
 * @param {...*} var_args The items to substitute into the failure message.
 * @return {boolean} The value, guaranteed to be a boolean when asserts are
 *     enabled.
 * @throws {goog.asserts.AssertionError} When the value is not a boolean.
 * @closurePrimitive {asserts.matchesReturn}
 */
goog.asserts.assertBoolean = function(value, opt_message, var_args) {
  if (goog.asserts.ENABLE_ASSERTS && typeof value !== 'boolean') {
    goog.asserts.doAssertFailure_(
        'Expected boolean but got %s: %s.', [goog.typeOf(value), value],
        opt_message, Array.prototype.slice.call(arguments, 2));
  }
  return /** @type {boolean} */ (value);
};


/**
 * Checks if the value is a DOM Element if goog.asserts.ENABLE_ASSERTS is true.
 * @param {*} value The value to check.
 * @param {string=} opt_message Error message in case of failure.
 * @param {...*} var_args The items to substitute into the failure message.
 * @return {!Element} The value, likely to be a DOM Element when asserts are
 *     enabled.
 * @throws {goog.asserts.AssertionError} When the value is not an Element.
 * @closurePrimitive {asserts.matchesReturn}
 * @deprecated Use goog.asserts.dom.assertIsElement instead.
 */
goog.asserts.assertElement = function(value, opt_message, var_args) {
  if (goog.asserts.ENABLE_ASSERTS &&
      (!goog.isObject(value) ||
       /** @type {!Node} */ (value).nodeType != goog.dom.NodeType.ELEMENT)) {
    goog.asserts.doAssertFailure_(
        'Expected Element but got %s: %s.', [goog.typeOf(value), value],
        opt_message, Array.prototype.slice.call(arguments, 2));
  }
  return /** @type {!Element} */ (value);
};


/**
 * Checks if the value is an instance of the user-defined type if
 * goog.asserts.ENABLE_ASSERTS is true.
 *
 * The compiler may tighten the type returned by this function.
 *
 * Do not use this to ensure a value is an HTMLElement or a subclass! Cross-
 * document DOM inherits from separate - though identical - browser classes, and
 * such a check will unexpectedly fail. Please use the methods in
 * goog.asserts.dom for these purposes.
 *
 * @param {?} value The value to check.
 * @param {function(new: T, ...)} type A user-defined constructor.
 * @param {string=} opt_message Error message in case of failure.
 * @param {...*} var_args The items to substitute into the failure message.
 * @throws {goog.asserts.AssertionError} When the value is not an instance of
 *     type.
 * @return {T}
 * @template T
 * @closurePrimitive {asserts.matchesReturn}
 */
goog.asserts.assertInstanceof = function(value, type, opt_message, var_args) {
  if (goog.asserts.ENABLE_ASSERTS && !(value instanceof type)) {
    goog.asserts.doAssertFailure_(
        'Expected instanceof %s but got %s.',
        [goog.asserts.getType_(type), goog.asserts.getType_(value)],
        opt_message, Array.prototype.slice.call(arguments, 3));
  }
  return value;
};


/**
 * Checks whether the value is a finite number, if goog.asserts.ENABLE_ASSERTS
 * is true.
 *
 * @param {*} value The value to check.
 * @param {string=} opt_message Error message in case of failure.
 * @param {...*} var_args The items to substitute into the failure message.
 * @throws {goog.asserts.AssertionError} When the value is not a number, or is
 *     a non-finite number such as NaN, Infinity or -Infinity.
 * @return {number} The value initially passed in.
 */
goog.asserts.assertFinite = function(value, opt_message, var_args) {
  if (goog.asserts.ENABLE_ASSERTS &&
      (typeof value != 'number' || !isFinite(value))) {
    goog.asserts.doAssertFailure_(
        'Expected %s to be a finite number but it is not.', [value],
        opt_message, Array.prototype.slice.call(arguments, 2));
  }
  return /** @type {number} */ (value);
};

/**
 * Checks that no enumerable keys are present in Object.prototype. Such keys
 * would break most code that use {@code for (var ... in ...)} loops.
 */
goog.asserts.assertObjectPrototypeIsIntact = function() {
  for (var key in Object.prototype) {
    goog.asserts.fail(key + ' should not be enumerable in Object.prototype.');
  }
};


/**
 * Returns the type of a value. If a constructor is passed, and a suitable
 * string cannot be found, 'unknown type name' will be returned.
 * @param {*} value A constructor, object, or primitive.
 * @return {string} The best display name for the value, or 'unknown type name'.
 * @private
 */
goog.asserts.getType_ = function(value) {
  if (value instanceof Function) {
    return value.displayName || value.name || 'unknown type name';
  } else if (value instanceof Object) {
    return /** @type {string} */ (value.constructor.displayName) ||
        value.constructor.name || Object.prototype.toString.call(value);
  } else {
    return value === null ? 'null' : typeof value;
  }
};

//third_party/javascript/closure/array/array.js
/**
 * @license
 * Copyright The Closure Library Authors.
 * SPDX-License-Identifier: Apache-2.0
 */

/**
 * @fileoverview Utilities for manipulating arrays.
 */


goog.provide('goog.array');

goog.require('goog.asserts');


/**
 * @define {boolean} NATIVE_ARRAY_PROTOTYPES indicates whether the code should
 * rely on Array.prototype functions, if available.
 *
 * The Array.prototype functions can be defined by external libraries like
 * Prototype and setting this flag to false forces closure to use its own
 * goog.array implementation.
 *
 * If your javascript can be loaded by a third party site and you are wary about
 * relying on the prototype functions, specify
 * "--define goog.NATIVE_ARRAY_PROTOTYPES=false" to the JSCompiler.
 *
 * Setting goog.TRUSTED_SITE to false will automatically set
 * NATIVE_ARRAY_PROTOTYPES to false.
 */
goog.NATIVE_ARRAY_PROTOTYPES =
    goog.define('goog.NATIVE_ARRAY_PROTOTYPES', goog.TRUSTED_SITE);


/**
 * @define {boolean} If true, JSCompiler will use the native implementation of
 * array functions where appropriate (e.g., `Array#filter`) and remove the
 * unused pure JS implementation.
 */
goog.array.ASSUME_NATIVE_FUNCTIONS = goog.define(
    'goog.array.ASSUME_NATIVE_FUNCTIONS', goog.FEATURESET_YEAR > 2012);


/**
 * Returns the last element in an array without removing it.
 * Same as goog.array.last.
 * @param {IArrayLike<T>|string} array The array.
 * @return {T} Last item in array.
 * @template T
 */
goog.array.peek = function(array) {
  return array[array.length - 1];
};


/**
 * Returns the last element in an array without removing it.
 * Same as goog.array.peek.
 * @param {IArrayLike<T>|string} array The array.
 * @return {T} Last item in array.
 * @template T
 */
goog.array.last = goog.array.peek;

// NOTE(arv): Since most of the array functions are generic it allows you to
// pass an array-like object. Strings have a length and are considered array-
// like. However, the 'in' operator does not work on strings so we cannot just
// use the array path even if the browser supports indexing into strings. We
// therefore end up splitting the string.


/**
 * Returns the index of the first element of an array with a specified value, or
 * -1 if the element is not present in the array.
 *
 * See {@link http://tinyurl.com/developer-mozilla-org-array-indexof}
 *
 * @param {IArrayLike<T>|string} arr The array to be searched.
 * @param {T} obj The object for which we are searching.
 * @param {number=} opt_fromIndex The index at which to start the search. If
 *     omitted the search starts at index 0.
 * @return {number} The index of the first matching array element.
 * @template T
 */
goog.array.indexOf = goog.NATIVE_ARRAY_PROTOTYPES &&
        (goog.array.ASSUME_NATIVE_FUNCTIONS || Array.prototype.indexOf) ?
    function(arr, obj, opt_fromIndex) {
      goog.asserts.assert(arr.length != null);

      return Array.prototype.indexOf.call(arr, obj, opt_fromIndex);
    } :
    function(arr, obj, opt_fromIndex) {
      var fromIndex = opt_fromIndex == null ?
          0 :
          (opt_fromIndex < 0 ? Math.max(0, arr.length + opt_fromIndex) :
                               opt_fromIndex);

      if (typeof arr === 'string') {
        // Array.prototype.indexOf uses === so only strings should be found.
        if (typeof obj !== 'string' || obj.length != 1) {
          return -1;
        }
        return arr.indexOf(obj, fromIndex);
      }

      for (var i = fromIndex; i < arr.length; i++) {
        if (i in arr && arr[i] === obj) return i;
      }
      return -1;
    };


/**
 * Returns the index of the last element of an array with a specified value, or
 * -1 if the element is not present in the array.
 *
 * See {@link http://tinyurl.com/developer-mozilla-org-array-lastindexof}
 *
 * @param {!IArrayLike<T>|string} arr The array to be searched.
 * @param {T} obj The object for which we are searching.
 * @param {?number=} opt_fromIndex The index at which to start the search. If
 *     omitted the search starts at the end of the array.
 * @return {number} The index of the last matching array element.
 * @template T
 */
goog.array.lastIndexOf = goog.NATIVE_ARRAY_PROTOTYPES &&
        (goog.array.ASSUME_NATIVE_FUNCTIONS || Array.prototype.lastIndexOf) ?
    function(arr, obj, opt_fromIndex) {
      goog.asserts.assert(arr.length != null);

      // Firefox treats undefined and null as 0 in the fromIndex argument which
      // leads it to always return -1
      var fromIndex = opt_fromIndex == null ? arr.length - 1 : opt_fromIndex;
      return Array.prototype.lastIndexOf.call(arr, obj, fromIndex);
    } :
    function(arr, obj, opt_fromIndex) {
      var fromIndex = opt_fromIndex == null ? arr.length - 1 : opt_fromIndex;

      if (fromIndex < 0) {
        fromIndex = Math.max(0, arr.length + fromIndex);
      }

      if (typeof arr === 'string') {
        // Array.prototype.lastIndexOf uses === so only strings should be found.
        if (typeof obj !== 'string' || obj.length != 1) {
          return -1;
        }
        return arr.lastIndexOf(obj, fromIndex);
      }

      for (var i = fromIndex; i >= 0; i--) {
        if (i in arr && arr[i] === obj) return i;
      }
      return -1;
    };


/**
 * Calls a function for each element in an array. Skips holes in the array.
 * See {@link http://tinyurl.com/developer-mozilla-org-array-foreach}
 *
 * @param {IArrayLike<T>|string} arr Array or array like object over
 *     which to iterate.
 * @param {?function(this: S, T, number, ?): ?} f The function to call for every
 *     element. This function takes 3 arguments (the element, the index and the
 *     array). The return value is ignored.
 * @param {S=} opt_obj The object to be used as the value of 'this' within f.
 * @template T,S
 */
goog.array.forEach = goog.NATIVE_ARRAY_PROTOTYPES &&
        (goog.array.ASSUME_NATIVE_FUNCTIONS || Array.prototype.forEach) ?
    function(arr, f, opt_obj) {
      goog.asserts.assert(arr.length != null);

      Array.prototype.forEach.call(arr, f, opt_obj);
    } :
    function(arr, f, opt_obj) {
      var l = arr.length;  // must be fixed during loop... see docs
      var arr2 = (typeof arr === 'string') ? arr.split('') : arr;
      for (var i = 0; i < l; i++) {
        if (i in arr2) {
          f.call(/** @type {?} */ (opt_obj), arr2[i], i, arr);
        }
      }
    };


/**
 * Calls a function for each element in an array, starting from the last
 * element rather than the first.
 *
 * @param {IArrayLike<T>|string} arr Array or array
 *     like object over which to iterate.
 * @param {?function(this: S, T, number, ?): ?} f The function to call for every
 *     element. This function
 *     takes 3 arguments (the element, the index and the array). The return
 *     value is ignored.
 * @param {S=} opt_obj The object to be used as the value of 'this'
 *     within f.
 * @template T,S
 */
goog.array.forEachRight = function(arr, f, opt_obj) {
  var l = arr.length;  // must be fixed during loop... see docs
  var arr2 = (typeof arr === 'string') ? arr.split('') : arr;
  for (var i = l - 1; i >= 0; --i) {
    if (i in arr2) {
      f.call(/** @type {?} */ (opt_obj), arr2[i], i, arr);
    }
  }
};


/**
 * Calls a function for each element in an array, and if the function returns
 * true adds the element to a new array.
 *
 * See {@link http://tinyurl.com/developer-mozilla-org-array-filter}
 *
 * @param {IArrayLike<T>|string} arr Array or array
 *     like object over which to iterate.
 * @param {?function(this:S, T, number, ?):boolean} f The function to call for
 *     every element. This function
 *     takes 3 arguments (the element, the index and the array) and must
 *     return a Boolean. If the return value is true the element is added to the
 *     result array. If it is false the element is not included.
 * @param {S=} opt_obj The object to be used as the value of 'this'
 *     within f.
 * @return {!Array<T>} a new array in which only elements that passed the test
 *     are present.
 * @template T,S
 */
goog.array.filter = goog.NATIVE_ARRAY_PROTOTYPES &&
        (goog.array.ASSUME_NATIVE_FUNCTIONS || Array.prototype.filter) ?
    function(arr, f, opt_obj) {
      goog.asserts.assert(arr.length != null);

      return Array.prototype.filter.call(arr, f, opt_obj);
    } :
    function(arr, f, opt_obj) {
      var l = arr.length;  // must be fixed during loop... see docs
      var res = [];
      var resLength = 0;
      var arr2 = (typeof arr === 'string') ? arr.split('') : arr;
      for (var i = 0; i < l; i++) {
        if (i in arr2) {
          var val = arr2[i];  // in case f mutates arr2
          if (f.call(/** @type {?} */ (opt_obj), val, i, arr)) {
            res[resLength++] = val;
          }
        }
      }
      return res;
    };


/**
 * Calls a function for each element in an array and inserts the result into a
 * new array.
 *
 * See {@link http://tinyurl.com/developer-mozilla-org-array-map}
 *
 * @param {IArrayLike<VALUE>|string} arr Array or array like object
 *     over which to iterate.
 * @param {function(this:THIS, VALUE, number, ?): RESULT} f The function to call
 *     for every element. This function takes 3 arguments (the element,
 *     the index and the array) and should return something. The result will be
 *     inserted into a new array.
 * @param {THIS=} opt_obj The object to be used as the value of 'this' within f.
 * @return {!Array<RESULT>} a new array with the results from f.
 * @template THIS, VALUE, RESULT
 */
goog.array.map = goog.NATIVE_ARRAY_PROTOTYPES &&
        (goog.array.ASSUME_NATIVE_FUNCTIONS || Array.prototype.map) ?
    function(arr, f, opt_obj) {
      goog.asserts.assert(arr.length != null);

      return Array.prototype.map.call(arr, f, opt_obj);
    } :
    function(arr, f, opt_obj) {
      var l = arr.length;  // must be fixed during loop... see docs
      var res = new Array(l);
      var arr2 = (typeof arr === 'string') ? arr.split('') : arr;
      for (var i = 0; i < l; i++) {
        if (i in arr2) {
          res[i] = f.call(/** @type {?} */ (opt_obj), arr2[i], i, arr);
        }
      }
      return res;
    };


/**
 * Passes every element of an array into a function and accumulates the result.
 *
 * See {@link http://tinyurl.com/developer-mozilla-org-array-reduce}
 *
 * For example:
 * var a = [1, 2, 3, 4];
 * goog.array.reduce(a, function(r, v, i, arr) {return r + v;}, 0);
 * returns 10
 *
 * @param {IArrayLike<T>|string} arr Array or array
 *     like object over which to iterate.
 * @param {function(this:S, R, T, number, ?) : R} f The function to call for
 *     every element. This function
 *     takes 4 arguments (the function's previous result or the initial value,
 *     the value of the current array element, the current array index, and the
 *     array itself)
 *     function(previousValue, currentValue, index, array).
 * @param {?} val The initial value to pass into the function on the first call.
 * @param {S=} opt_obj  The object to be used as the value of 'this'
 *     within f.
 * @return {R} Result of evaluating f repeatedly across the values of the array.
 * @template T,S,R
 */
goog.array.reduce = goog.NATIVE_ARRAY_PROTOTYPES &&
        (goog.array.ASSUME_NATIVE_FUNCTIONS || Array.prototype.reduce) ?
    function(arr, f, val, opt_obj) {
      goog.asserts.assert(arr.length != null);
      if (opt_obj) {
        f = goog.bind(f, opt_obj);
      }
      return Array.prototype.reduce.call(arr, f, val);
    } :
    function(arr, f, val, opt_obj) {
      var rval = val;
      goog.array.forEach(arr, function(val, index) {
        rval = f.call(/** @type {?} */ (opt_obj), rval, val, index, arr);
      });
      return rval;
    };


/**
 * Passes every element of an array into a function and accumulates the result,
 * starting from the last element and working towards the first.
 *
 * See {@link http://tinyurl.com/developer-mozilla-org-array-reduceright}
 *
 * For example:
 * var a = ['a', 'b', 'c'];
 * goog.array.reduceRight(a, function(r, v, i, arr) {return r + v;}, '');
 * returns 'cba'
 *
 * @param {IArrayLike<T>|string} arr Array or array
 *     like object over which to iterate.
 * @param {?function(this:S, R, T, number, ?) : R} f The function to call for
 *     every element. This function
 *     takes 4 arguments (the function's previous result or the initial value,
 *     the value of the current array element, the current array index, and the
 *     array itself)
 *     function(previousValue, currentValue, index, array).
 * @param {?} val The initial value to pass into the function on the first call.
 * @param {S=} opt_obj The object to be used as the value of 'this'
 *     within f.
 * @return {R} Object returned as a result of evaluating f repeatedly across the
 *     values of the array.
 * @template T,S,R
 */
goog.array.reduceRight = goog.NATIVE_ARRAY_PROTOTYPES &&
        (goog.array.ASSUME_NATIVE_FUNCTIONS || Array.prototype.reduceRight) ?
    function(arr, f, val, opt_obj) {
      goog.asserts.assert(arr.length != null);
      goog.asserts.assert(f != null);
      if (opt_obj) {
        f = goog.bind(f, opt_obj);
      }
      return Array.prototype.reduceRight.call(arr, f, val);
    } :
    function(arr, f, val, opt_obj) {
      var rval = val;
      goog.array.forEachRight(arr, function(val, index) {
        rval = f.call(/** @type {?} */ (opt_obj), rval, val, index, arr);
      });
      return rval;
    };


/**
 * Calls f for each element of an array. If any call returns true, some()
 * returns true (without checking the remaining elements). If all calls
 * return false, some() returns false.
 *
 * See {@link http://tinyurl.com/developer-mozilla-org-array-some}
 *
 * @param {IArrayLike<T>|string} arr Array or array
 *     like object over which to iterate.
 * @param {?function(this:S, T, number, ?) : boolean} f The function to call for
 *     for every element. This function takes 3 arguments (the element, the
 *     index and the array) and should return a boolean.
 * @param {S=} opt_obj  The object to be used as the value of 'this'
 *     within f.
 * @return {boolean} true if any element passes the test.
 * @template T,S
 */
goog.array.some = goog.NATIVE_ARRAY_PROTOTYPES &&
        (goog.array.ASSUME_NATIVE_FUNCTIONS || Array.prototype.some) ?
    function(arr, f, opt_obj) {
      goog.asserts.assert(arr.length != null);

      return Array.prototype.some.call(arr, f, opt_obj);
    } :
    function(arr, f, opt_obj) {
      var l = arr.length;  // must be fixed during loop... see docs
      var arr2 = (typeof arr === 'string') ? arr.split('') : arr;
      for (var i = 0; i < l; i++) {
        if (i in arr2 && f.call(/** @type {?} */ (opt_obj), arr2[i], i, arr)) {
          return true;
        }
      }
      return false;
    };


/**
 * Call f for each element of an array. If all calls return true, every()
 * returns true. If any call returns false, every() returns false and
 * does not continue to check the remaining elements.
 *
 * See {@link http://tinyurl.com/developer-mozilla-org-array-every}
 *
 * @param {IArrayLike<T>|string} arr Array or array
 *     like object over which to iterate.
 * @param {?function(this:S, T, number, ?) : boolean} f The function to call for
 *     for every element. This function takes 3 arguments (the element, the
 *     index and the array) and should return a boolean.
 * @param {S=} opt_obj The object to be used as the value of 'this'
 *     within f.
 * @return {boolean} false if any element fails the test.
 * @template T,S
 */
goog.array.every = goog.NATIVE_ARRAY_PROTOTYPES &&
        (goog.array.ASSUME_NATIVE_FUNCTIONS || Array.prototype.every) ?
    function(arr, f, opt_obj) {
      goog.asserts.assert(arr.length != null);

      return Array.prototype.every.call(arr, f, opt_obj);
    } :
    function(arr, f, opt_obj) {
      var l = arr.length;  // must be fixed during loop... see docs
      var arr2 = (typeof arr === 'string') ? arr.split('') : arr;
      for (var i = 0; i < l; i++) {
        if (i in arr2 && !f.call(/** @type {?} */ (opt_obj), arr2[i], i, arr)) {
          return false;
        }
      }
      return true;
    };


/**
 * Counts the array elements that fulfill the predicate, i.e. for which the
 * callback function returns true. Skips holes in the array.
 *
 * @param {!IArrayLike<T>|string} arr Array or array like object
 *     over which to iterate.
 * @param {function(this: S, T, number, ?): boolean} f The function to call for
 *     every element. Takes 3 arguments (the element, the index and the array).
 * @param {S=} opt_obj The object to be used as the value of 'this' within f.
 * @return {number} The number of the matching elements.
 * @template T,S
 */
goog.array.count = function(arr, f, opt_obj) {
  var count = 0;
  goog.array.forEach(arr, function(element, index, arr) {
    if (f.call(/** @type {?} */ (opt_obj), element, index, arr)) {
      ++count;
    }
  }, opt_obj);
  return count;
};


/**
 * Search an array for the first element that satisfies a given condition and
 * return that element.
 * @param {IArrayLike<T>|string} arr Array or array
 *     like object over which to iterate.
 * @param {?function(this:S, T, number, ?) : boolean} f The function to call
 *     for every element. This function takes 3 arguments (the element, the
 *     index and the array) and should return a boolean.
 * @param {S=} opt_obj An optional "this" context for the function.
 * @return {T|null} The first array element that passes the test, or null if no
 *     element is found.
 * @template T,S
 */
goog.array.find = function(arr, f, opt_obj) {
  var i = goog.array.findIndex(arr, f, opt_obj);
  return i < 0 ? null : typeof arr === 'string' ? arr.charAt(i) : arr[i];
};


/**
 * Search an array for the first element that satisfies a given condition and
 * return its index.
 * @param {IArrayLike<T>|string} arr Array or array
 *     like object over which to iterate.
 * @param {?function(this:S, T, number, ?) : boolean} f The function to call for
 *     every element. This function
 *     takes 3 arguments (the element, the index and the array) and should
 *     return a boolean.
 * @param {S=} opt_obj An optional "this" context for the function.
 * @return {number} The index of the first array element that passes the test,
 *     or -1 if no element is found.
 * @template T,S
 */
goog.array.findIndex = function(arr, f, opt_obj) {
  var l = arr.length;  // must be fixed during loop... see docs
  var arr2 = (typeof arr === 'string') ? arr.split('') : arr;
  for (var i = 0; i < l; i++) {
    if (i in arr2 && f.call(/** @type {?} */ (opt_obj), arr2[i], i, arr)) {
      return i;
    }
  }
  return -1;
};


/**
 * Search an array (in reverse order) for the last element that satisfies a
 * given condition and return that element.
 * @param {IArrayLike<T>|string} arr Array or array
 *     like object over which to iterate.
 * @param {?function(this:S, T, number, ?) : boolean} f The function to call
 *     for every element. This function
 *     takes 3 arguments (the element, the index and the array) and should
 *     return a boolean.
 * @param {S=} opt_obj An optional "this" context for the function.
 * @return {T|null} The last array element that passes the test, or null if no
 *     element is found.
 * @template T,S
 */
goog.array.findRight = function(arr, f, opt_obj) {
  var i = goog.array.findIndexRight(arr, f, opt_obj);
  return i < 0 ? null : typeof arr === 'string' ? arr.charAt(i) : arr[i];
};


/**
 * Search an array (in reverse order) for the last element that satisfies a
 * given condition and return its index.
 * @param {IArrayLike<T>|string} arr Array or array
 *     like object over which to iterate.
 * @param {?function(this:S, T, number, ?) : boolean} f The function to call
 *     for every element. This function
 *     takes 3 arguments (the element, the index and the array) and should
 *     return a boolean.
 * @param {S=} opt_obj An optional "this" context for the function.
 * @return {number} The index of the last array element that passes the test,
 *     or -1 if no element is found.
 * @template T,S
 */
goog.array.findIndexRight = function(arr, f, opt_obj) {
  var l = arr.length;  // must be fixed during loop... see docs
  var arr2 = (typeof arr === 'string') ? arr.split('') : arr;
  for (var i = l - 1; i >= 0; i--) {
    if (i in arr2 && f.call(/** @type {?} */ (opt_obj), arr2[i], i, arr)) {
      return i;
    }
  }
  return -1;
};


/**
 * Whether the array contains the given object.
 * @param {IArrayLike<?>|string} arr The array to test for the presence of the
 *     element.
 * @param {*} obj The object for which to test.
 * @return {boolean} true if obj is present.
 */
goog.array.contains = function(arr, obj) {
  return goog.array.indexOf(arr, obj) >= 0;
};


/**
 * Whether the array is empty.
 * @param {IArrayLike<?>|string} arr The array to test.
 * @return {boolean} true if empty.
 */
goog.array.isEmpty = function(arr) {
  return arr.length == 0;
};


/**
 * Clears the array.
 * @param {IArrayLike<?>} arr Array or array like object to clear.
 */
goog.array.clear = function(arr) {
  // For non real arrays we don't have the magic length so we delete the
  // indices.
  if (!Array.isArray(arr)) {
    for (var i = arr.length - 1; i >= 0; i--) {
      delete arr[i];
    }
  }
  arr.length = 0;
};


/**
 * Pushes an item into an array, if it's not already in the array.
 * @param {Array<T>} arr Array into which to insert the item.
 * @param {T} obj Value to add.
 * @template T
 */
goog.array.insert = function(arr, obj) {
  if (!goog.array.contains(arr, obj)) {
    arr.push(obj);
  }
};


/**
 * Inserts an object at the given index of the array.
 * @param {IArrayLike<?>} arr The array to modify.
 * @param {*} obj The object to insert.
 * @param {number=} opt_i The index at which to insert the object. If omitted,
 *      treated as 0. A negative index is counted from the end of the array.
 */
goog.array.insertAt = function(arr, obj, opt_i) {
  goog.array.splice(arr, opt_i, 0, obj);
};


/**
 * Inserts at the given index of the array, all elements of another array.
 * @param {IArrayLike<?>} arr The array to modify.
 * @param {IArrayLike<?>} elementsToAdd The array of elements to add.
 * @param {number=} opt_i The index at which to insert the object. If omitted,
 *      treated as 0. A negative index is counted from the end of the array.
 */
goog.array.insertArrayAt = function(arr, elementsToAdd, opt_i) {
  goog.partial(goog.array.splice, arr, opt_i, 0).apply(null, elementsToAdd);
};


/**
 * Inserts an object into an array before a specified object.
 * @param {Array<T>} arr The array to modify.
 * @param {T} obj The object to insert.
 * @param {T=} opt_obj2 The object before which obj should be inserted. If obj2
 *     is omitted or not found, obj is inserted at the end of the array.
 * @template T
 */
goog.array.insertBefore = function(arr, obj, opt_obj2) {
  var i;
  if (arguments.length == 2 || (i = goog.array.indexOf(arr, opt_obj2)) < 0) {
    arr.push(obj);
  } else {
    goog.array.insertAt(arr, obj, i);
  }
};


/**
 * Removes the first occurrence of a particular value from an array.
 * @param {IArrayLike<T>} arr Array from which to remove
 *     value.
 * @param {T} obj Object to remove.
 * @return {boolean} True if an element was removed.
 * @template T
 */
goog.array.remove = function(arr, obj) {
  var i = goog.array.indexOf(arr, obj);
  var rv;
  if ((rv = i >= 0)) {
    goog.array.removeAt(arr, i);
  }
  return rv;
};


/**
 * Removes the last occurrence of a particular value from an array.
 * @param {!IArrayLike<T>} arr Array from which to remove value.
 * @param {T} obj Object to remove.
 * @return {boolean} True if an element was removed.
 * @template T
 */
goog.array.removeLast = function(arr, obj) {
  var i = goog.array.lastIndexOf(arr, obj);
  if (i >= 0) {
    goog.array.removeAt(arr, i);
    return true;
  }
  return false;
};


/**
 * Removes from an array the element at index i
 * @param {IArrayLike<?>} arr Array or array like object from which to
 *     remove value.
 * @param {number} i The index to remove.
 * @return {boolean} True if an element was removed.
 */
goog.array.removeAt = function(arr, i) {
  goog.asserts.assert(arr.length != null);

  // use generic form of splice
  // splice returns the removed items and if successful the length of that
  // will be 1
  return Array.prototype.splice.call(arr, i, 1).length == 1;
};


/**
 * Removes the first value that satisfies the given condition.
 * @param {IArrayLike<T>} arr Array or array
 *     like object over which to iterate.
 * @param {?function(this:S, T, number, ?) : boolean} f The function to call
 *     for every element. This function
 *     takes 3 arguments (the element, the index and the array) and should
 *     return a boolean.
 * @param {S=} opt_obj An optional "this" context for the function.
 * @return {boolean} True if an element was removed.
 * @template T,S
 */
goog.array.removeIf = function(arr, f, opt_obj) {
  var i = goog.array.findIndex(arr, f, opt_obj);
  if (i >= 0) {
    goog.array.removeAt(arr, i);
    return true;
  }
  return false;
};


/**
 * Removes all values that satisfy the given condition.
 * @param {IArrayLike<T>} arr Array or array
 *     like object over which to iterate.
 * @param {?function(this:S, T, number, ?) : boolean} f The function to call
 *     for every element. This function
 *     takes 3 arguments (the element, the index and the array) and should
 *     return a boolean.
 * @param {S=} opt_obj An optional "this" context for the function.
 * @return {number} The number of items removed
 * @template T,S
 */
goog.array.removeAllIf = function(arr, f, opt_obj) {
  var removedCount = 0;
  goog.array.forEachRight(arr, function(val, index) {
    if (f.call(/** @type {?} */ (opt_obj), val, index, arr)) {
      if (goog.array.removeAt(arr, index)) {
        removedCount++;
      }
    }
  });
  return removedCount;
};


/**
 * Returns a new array that is the result of joining the arguments.  If arrays
 * are passed then their items are added, however, if non-arrays are passed they
 * will be added to the return array as is.
 *
 * Note that ArrayLike objects will be added as is, rather than having their
 * items added.
 *
 * goog.array.concat([1, 2], [3, 4]) -> [1, 2, 3, 4]
 * goog.array.concat(0, [1, 2]) -> [0, 1, 2]
 * goog.array.concat([1, 2], null) -> [1, 2, null]
 *
 * There is bug in all current versions of IE (6, 7 and 8) where arrays created
 * in an iframe become corrupted soon (not immediately) after the iframe is
 * destroyed. This is common if loading data via goog.net.IframeIo, for example.
 * This corruption only affects the concat method which will start throwing
 * Catastrophic Errors (#-2147418113).
 *
 * See http://endoflow.com/scratch/corrupted-arrays.html for a test case.
 *
 * Internally goog.array should use this, so that all methods will continue to
 * work on these broken array objects.
 *
 * @param {...*} var_args Items to concatenate.  Arrays will have each item
 *     added, while primitives and objects will be added as is.
 * @return {!Array<?>} The new resultant array.
 */
goog.array.concat = function(var_args) {
  return Array.prototype.concat.apply([], arguments);
};


/**
 * Returns a new array that contains the contents of all the arrays passed.
 * @param {...!Array<T>} var_args
 * @return {!Array<T>}
 * @template T
 */
goog.array.join = function(var_args) {
  return Array.prototype.concat.apply([], arguments);
};


/**
 * Converts an object to an array.
 * @param {IArrayLike<T>|string} object  The object to convert to an
 *     array.
 * @return {!Array<T>} The object converted into an array. If object has a
 *     length property, every property indexed with a non-negative number
 *     less than length will be included in the result. If object does not
 *     have a length property, an empty array will be returned.
 * @template T
 */
goog.array.toArray = function(object) {
  var length = object.length;

  // If length is not a number the following is false. This case is kept for
  // backwards compatibility since there are callers that pass objects that are
  // not array like.
  if (length > 0) {
    var rv = new Array(length);
    for (var i = 0; i < length; i++) {
      rv[i] = object[i];
    }
    return rv;
  }
  return [];
};


/**
 * Does a shallow copy of an array.
 * @param {IArrayLike<T>|string} arr  Array or array-like object to
 *     clone.
 * @return {!Array<T>} Clone of the input array.
 * @template T
 */
goog.array.clone = goog.array.toArray;


/**
 * Extends an array with another array, element, or "array like" object.
 * This function operates 'in-place', it does not create a new Array.
 *
 * Example:
 * var a = [];
 * goog.array.extend(a, [0, 1]);
 * a; // [0, 1]
 * goog.array.extend(a, 2);
 * a; // [0, 1, 2]
 *
 * @param {Array<VALUE>} arr1  The array to modify.
 * @param {...(IArrayLike<VALUE>|VALUE)} var_args The elements or arrays of
 *     elements to add to arr1.
 * @template VALUE
 */
goog.array.extend = function(arr1, var_args) {
  for (var i = 1; i < arguments.length; i++) {
    var arr2 = arguments[i];
    if (goog.isArrayLike(arr2)) {
      var len1 = arr1.length || 0;
      var len2 = arr2.length || 0;
      arr1.length = len1 + len2;
      for (var j = 0; j < len2; j++) {
        arr1[len1 + j] = arr2[j];
      }
    } else {
      arr1.push(arr2);
    }
  }
};


/**
 * Adds or removes elements from an array. This is a generic version of Array
 * splice. This means that it might work on other objects similar to arrays,
 * such as the arguments object.
 *
 * @param {IArrayLike<T>} arr The array to modify.
 * @param {number|undefined} index The index at which to start changing the
 *     array. If not defined, treated as 0.
 * @param {number} howMany How many elements to remove (0 means no removal. A
 *     value below 0 is treated as zero and so is any other non number. Numbers
 *     are floored).
 * @param {...T} var_args Optional, additional elements to insert into the
 *     array.
 * @return {!Array<T>} the removed elements.
 * @template T
 */
goog.array.splice = function(arr, index, howMany, var_args) {
  goog.asserts.assert(arr.length != null);

  return Array.prototype.splice.apply(arr, goog.array.slice(arguments, 1));
};


/**
 * Returns a new array from a segment of an array. This is a generic version of
 * Array slice. This means that it might work on other objects similar to
 * arrays, such as the arguments object.
 *
 * @param {IArrayLike<T>|string} arr The array from
 * which to copy a segment.
 * @param {number} start The index of the first element to copy.
 * @param {number=} opt_end The index after the last element to copy.
 * @return {!Array<T>} A new array containing the specified segment of the
 *     original array.
 * @template T
 */
goog.array.slice = function(arr, start, opt_end) {
  goog.asserts.assert(arr.length != null);

  // passing 1 arg to slice is not the same as passing 2 where the second is
  // null or undefined (in that case the second argument is treated as 0).
  // we could use slice on the arguments object and then use apply instead of
  // testing the length
  if (arguments.length <= 2) {
    return Array.prototype.slice.call(arr, start);
  } else {
    return Array.prototype.slice.call(arr, start, opt_end);
  }
};


/**
 * Removes all duplicates from an array (retaining only the first
 * occurrence of each array element).  This function modifies the
 * array in place and doesn't change the order of the non-duplicate items.
 *
 * For objects, duplicates are identified as having the same unique ID as
 * defined by {@link goog.getUid}.
 *
 * Alternatively you can specify a custom hash function that returns a unique
 * value for each item in the array it should consider unique.
 *
 * Runtime: N,
 * Worstcase space: 2N (no dupes)
 *
 * @param {IArrayLike<T>} arr The array from which to remove
 *     duplicates.
 * @param {Array=} opt_rv An optional array in which to return the results,
 *     instead of performing the removal inplace.  If specified, the original
 *     array will remain unchanged.
 * @param {function(T):string=} opt_hashFn An optional function to use to
 *     apply to every item in the array. This function should return a unique
 *     value for each item in the array it should consider unique.
 * @template T
 */
goog.array.removeDuplicates = function(arr, opt_rv, opt_hashFn) {
  var returnArray = opt_rv || arr;
  var defaultHashFn = function(item) {
    // Prefix each type with a single character representing the type to
    // prevent conflicting keys (e.g. true and 'true').
    return goog.isObject(item) ? 'o' + goog.getUid(item) :
                                 (typeof item).charAt(0) + item;
  };
  var hashFn = opt_hashFn || defaultHashFn;

  var seen = {}, cursorInsert = 0, cursorRead = 0;
  while (cursorRead < arr.length) {
    var current = arr[cursorRead++];
    var key = hashFn(current);
    if (!Object.prototype.hasOwnProperty.call(seen, key)) {
      seen[key] = true;
      returnArray[cursorInsert++] = current;
    }
  }
  returnArray.length = cursorInsert;
};


/**
 * Searches the specified array for the specified target using the binary
 * search algorithm.  If no opt_compareFn is specified, elements are compared
 * using <code>goog.array.defaultCompare</code>, which compares the elements
 * using the built in < and > operators.  This will produce the expected
 * behavior for homogeneous arrays of String(s) and Number(s). The array
 * specified <b>must</b> be sorted in ascending order (as defined by the
 * comparison function).  If the array is not sorted, results are undefined.
 * If the array contains multiple instances of the specified target value, the
 * left-most instance will be found.
 *
 * Runtime: O(log n)
 *
 * @param {IArrayLike<VALUE>} arr The array to be searched.
 * @param {TARGET} target The sought value.
 * @param {function(TARGET, VALUE): number=} opt_compareFn Optional comparison
 *     function by which the array is ordered. Should take 2 arguments to
 *     compare, the target value and an element from your array, and return a
 *     negative number, zero, or a positive number depending on whether the
 *     first argument is less than, equal to, or greater than the second.
 * @return {number} Lowest index of the target value if found, otherwise
 *     (-(insertion point) - 1). The insertion point is where the value should
 *     be inserted into arr to preserve the sorted property.  Return value >= 0
 *     iff target is found.
 * @template TARGET, VALUE
 */
goog.array.binarySearch = function(arr, target, opt_compareFn) {
  return goog.array.binarySearch_(
      arr, opt_compareFn || goog.array.defaultCompare, false /* isEvaluator */,
      target);
};


/**
 * Selects an index in the specified array using the binary search algorithm.
 * The evaluator receives an element and determines whether the desired index
 * is before, at, or after it.  The evaluator must be consistent (formally,
 * goog.array.map(goog.array.map(arr, evaluator, opt_obj), goog.math.sign)
 * must be monotonically non-increasing).
 *
 * Runtime: O(log n)
 *
 * @param {IArrayLike<VALUE>} arr The array to be searched.
 * @param {function(this:THIS, VALUE, number, ?): number} evaluator
 *     Evaluator function that receives 3 arguments (the element, the index and
 *     the array). Should return a negative number, zero, or a positive number
 *     depending on whether the desired index is before, at, or after the
 *     element passed to it.
 * @param {THIS=} opt_obj The object to be used as the value of 'this'
 *     within evaluator.
 * @return {number} Index of the leftmost element matched by the evaluator, if
 *     such exists; otherwise (-(insertion point) - 1). The insertion point is
 *     the index of the first element for which the evaluator returns negative,
 *     or arr.length if no such element exists. The return value is non-negative
 *     iff a match is found.
 * @template THIS, VALUE
 */
goog.array.binarySelect = function(arr, evaluator, opt_obj) {
  return goog.array.binarySearch_(
      arr, evaluator, true /* isEvaluator */, undefined /* opt_target */,
      opt_obj);
};


/**
 * Implementation of a binary search algorithm which knows how to use both
 * comparison functions and evaluators. If an evaluator is provided, will call
 * the evaluator with the given optional data object, conforming to the
 * interface defined in binarySelect. Otherwise, if a comparison function is
 * provided, will call the comparison function against the given data object.
 *
 * This implementation purposefully does not use goog.bind or goog.partial for
 * performance reasons.
 *
 * Runtime: O(log n)
 *
 * @param {IArrayLike<?>} arr The array to be searched.
 * @param {function(?, ?, ?): number | function(?, ?): number} compareFn
 *     Either an evaluator or a comparison function, as defined by binarySearch
 *     and binarySelect above.
 * @param {boolean} isEvaluator Whether the function is an evaluator or a
 *     comparison function.
 * @param {?=} opt_target If the function is a comparison function, then
 *     this is the target to binary search for.
 * @param {Object=} opt_selfObj If the function is an evaluator, this is an
 *     optional this object for the evaluator.
 * @return {number} Lowest index of the target value if found, otherwise
 *     (-(insertion point) - 1). The insertion point is where the value should
 *     be inserted into arr to preserve the sorted property.  Return value >= 0
 *     iff target is found.
 * @private
 */
goog.array.binarySearch_ = function(
    arr, compareFn, isEvaluator, opt_target, opt_selfObj) {
  var left = 0;            // inclusive
  var right = arr.length;  // exclusive
  var found;
  while (left < right) {
    var middle = left + ((right - left) >>> 1);
    var compareResult;
    if (isEvaluator) {
      compareResult = compareFn.call(opt_selfObj, arr[middle], middle, arr);
    } else {
      // NOTE(dimvar): To avoid this cast, we'd have to use function overloading
      // for the type of binarySearch_, which the type system can't express yet.
      compareResult = /** @type {function(?, ?): number} */ (compareFn)(
          opt_target, arr[middle]);
    }
    if (compareResult > 0) {
      left = middle + 1;
    } else {
      right = middle;
      // We are looking for the lowest index so we can't return immediately.
      found = !compareResult;
    }
  }
  // left is the index if found, or the insertion point otherwise.
  // Avoiding bitwise not operator, as that causes a loss in precision for array
  // indexes outside the bounds of a 32-bit signed integer.  Array indexes have
  // a maximum value of 2^32-2 https://tc39.es/ecma262/#array-index
  return found ? left : -left - 1;
};


/**
 * Sorts the specified array into ascending order.  If no opt_compareFn is
 * specified, elements are compared using
 * <code>goog.array.defaultCompare</code>, which compares the elements using
 * the built in < and > operators.  This will produce the expected behavior
 * for homogeneous arrays of String(s) and Number(s), unlike the native sort,
 * but will give unpredictable results for heterogeneous lists of strings and
 * numbers with different numbers of digits.
 *
 * This sort is not guaranteed to be stable.
 *
 * Runtime: Same as <code>Array.prototype.sort</code>
 *
 * @param {Array<T>} arr The array to be sorted.
 * @param {?function(T,T):number=} opt_compareFn Optional comparison
 *     function by which the
 *     array is to be ordered. Should take 2 arguments to compare, and return a
 *     negative number, zero, or a positive number depending on whether the
 *     first argument is less than, equal to, or greater than the second.
 * @template T
 */
goog.array.sort = function(arr, opt_compareFn) {
  // TODO(arv): Update type annotation since null is not accepted.
  arr.sort(opt_compareFn || goog.array.defaultCompare);
};


/**
 * Sorts the specified array into ascending order in a stable way.  If no
 * opt_compareFn is specified, elements are compared using
 * <code>goog.array.defaultCompare</code>, which compares the elements using
 * the built in < and > operators.  This will produce the expected behavior
 * for homogeneous arrays of String(s) and Number(s).
 *
 * Runtime: Same as <code>Array.prototype.sort</code>, plus an additional
 * O(n) overhead of copying the array twice.
 *
 * @param {Array<T>} arr The array to be sorted.
 * @param {?function(T, T): number=} opt_compareFn Optional comparison function
 *     by which the array is to be ordered. Should take 2 arguments to compare,
 *     and return a negative number, zero, or a positive number depending on
 *     whether the first argument is less than, equal to, or greater than the
 *     second.
 * @template T
 */
goog.array.stableSort = function(arr, opt_compareFn) {
  var compArr = new Array(arr.length);
  for (var i = 0; i < arr.length; i++) {
    compArr[i] = {index: i, value: arr[i]};
  }
  var valueCompareFn = opt_compareFn || goog.array.defaultCompare;
  function stableCompareFn(obj1, obj2) {
    return valueCompareFn(obj1.value, obj2.value) || obj1.index - obj2.index;
  }
  goog.array.sort(compArr, stableCompareFn);
  for (var i = 0; i < arr.length; i++) {
    arr[i] = compArr[i].value;
  }
};


/**
 * Sort the specified array into ascending order based on item keys
 * returned by the specified key function.
 * If no opt_compareFn is specified, the keys are compared in ascending order
 * using <code>goog.array.defaultCompare</code>.
 *
 * Runtime: O(S(f(n)), where S is runtime of <code>goog.array.sort</code>
 * and f(n) is runtime of the key function.
 *
 * @param {Array<T>} arr The array to be sorted.
 * @param {function(T): K} keyFn Function taking array element and returning
 *     a key used for sorting this element.
 * @param {?function(K, K): number=} opt_compareFn Optional comparison function
 *     by which the keys are to be ordered. Should take 2 arguments to compare,
 *     and return a negative number, zero, or a positive number depending on
 *     whether the first argument is less than, equal to, or greater than the
 *     second.
 * @template T,K
 */
goog.array.sortByKey = function(arr, keyFn, opt_compareFn) {
  var keyCompareFn = opt_compareFn || goog.array.defaultCompare;
  goog.array.sort(
      arr, function(a, b) { return keyCompareFn(keyFn(a), keyFn(b)); });
};


/**
 * Sorts an array of objects by the specified object key and compare
 * function. If no compare function is provided, the key values are
 * compared in ascending order using <code>goog.array.defaultCompare</code>.
 * This won't work for keys that get renamed by the compiler. So use
 * {'foo': 1, 'bar': 2} rather than {foo: 1, bar: 2}.
 * @param {Array<Object>} arr An array of objects to sort.
 * @param {string} key The object key to sort by.
 * @param {Function=} opt_compareFn The function to use to compare key
 *     values.
 */
goog.array.sortObjectsByKey = function(arr, key, opt_compareFn) {
  goog.array.sortByKey(arr, function(obj) { return obj[key]; }, opt_compareFn);
};


/**
 * Tells if the array is sorted.
 * @param {!IArrayLike<T>} arr The array.
 * @param {?function(T,T):number=} opt_compareFn Function to compare the
 *     array elements.
 *     Should take 2 arguments to compare, and return a negative number, zero,
 *     or a positive number depending on whether the first argument is less
 *     than, equal to, or greater than the second.
 * @param {boolean=} opt_strict If true no equal elements are allowed.
 * @return {boolean} Whether the array is sorted.
 * @template T
 */
goog.array.isSorted = function(arr, opt_compareFn, opt_strict) {
  var compare = opt_compareFn || goog.array.defaultCompare;
  for (var i = 1; i < arr.length; i++) {
    var compareResult = compare(arr[i - 1], arr[i]);
    if (compareResult > 0 || compareResult == 0 && opt_strict) {
      return false;
    }
  }
  return true;
};


/**
 * Compares two arrays for equality. Two arrays are considered equal if they
 * have the same length and their corresponding elements are equal according to
 * the comparison function.
 *
 * @param {IArrayLike<?>} arr1 The first array to compare.
 * @param {IArrayLike<?>} arr2 The second array to compare.
 * @param {Function=} opt_equalsFn Optional comparison function.
 *     Should take 2 arguments to compare, and return true if the arguments
 *     are equal. Defaults to {@link goog.array.defaultCompareEquality} which
 *     compares the elements using the built-in '===' operator.
 * @return {boolean} Whether the two arrays are equal.
 */
goog.array.equals = function(arr1, arr2, opt_equalsFn) {
  if (!goog.isArrayLike(arr1) || !goog.isArrayLike(arr2) ||
      arr1.length != arr2.length) {
    return false;
  }
  var l = arr1.length;
  var equalsFn = opt_equalsFn || goog.array.defaultCompareEquality;
  for (var i = 0; i < l; i++) {
    if (!equalsFn(arr1[i], arr2[i])) {
      return false;
    }
  }
  return true;
};


/**
 * 3-way array compare function.
 * @param {!IArrayLike<VALUE>} arr1 The first array to
 *     compare.
 * @param {!IArrayLike<VALUE>} arr2 The second array to
 *     compare.
 * @param {function(VALUE, VALUE): number=} opt_compareFn Optional comparison
 *     function by which the array is to be ordered. Should take 2 arguments to
 *     compare, and return a negative number, zero, or a positive number
 *     depending on whether the first argument is less than, equal to, or
 *     greater than the second.
 * @return {number} Negative number, zero, or a positive number depending on
 *     whether the first argument is less than, equal to, or greater than the
 *     second.
 * @template VALUE
 */
goog.array.compare3 = function(arr1, arr2, opt_compareFn) {
  var compare = opt_compareFn || goog.array.defaultCompare;
  var l = Math.min(arr1.length, arr2.length);
  for (var i = 0; i < l; i++) {
    var result = compare(arr1[i], arr2[i]);
    if (result != 0) {
      return result;
    }
  }
  return goog.array.defaultCompare(arr1.length, arr2.length);
};


/**
 * Compares its two arguments for order, using the built in < and >
 * operators.
 * @param {VALUE} a The first object to be compared.
 * @param {VALUE} b The second object to be compared.
 * @return {number} A negative number, zero, or a positive number as the first
 *     argument is less than, equal to, or greater than the second,
 *     respectively.
 * @template VALUE
 */
goog.array.defaultCompare = function(a, b) {
  return a > b ? 1 : a < b ? -1 : 0;
};


/**
 * Compares its two arguments for inverse order, using the built in < and >
 * operators.
 * @param {VALUE} a The first object to be compared.
 * @param {VALUE} b The second object to be compared.
 * @return {number} A negative number, zero, or a positive number as the first
 *     argument is greater than, equal to, or less than the second,
 *     respectively.
 * @template VALUE
 */
goog.array.inverseDefaultCompare = function(a, b) {
  return -goog.array.defaultCompare(a, b);
};


/**
 * Compares its two arguments for equality, using the built in === operator.
 * @param {*} a The first object to compare.
 * @param {*} b The second object to compare.
 * @return {boolean} True if the two arguments are equal, false otherwise.
 */
goog.array.defaultCompareEquality = function(a, b) {
  return a === b;
};


/**
 * Inserts a value into a sorted array. The array is not modified if the
 * value is already present.
 * @param {IArrayLike<VALUE>} array The array to modify.
 * @param {VALUE} value The object to insert.
 * @param {function(VALUE, VALUE): number=} opt_compareFn Optional comparison
 *     function by which the array is ordered. Should take 2 arguments to
 *     compare, and return a negative number, zero, or a positive number
 *     depending on whether the first argument is less than, equal to, or
 *     greater than the second.
 * @return {boolean} True if an element was inserted.
 * @template VALUE
 */
goog.array.binaryInsert = function(array, value, opt_compareFn) {
  var index = goog.array.binarySearch(array, value, opt_compareFn);
  if (index < 0) {
    goog.array.insertAt(array, value, -(index + 1));
    return true;
  }
  return false;
};


/**
 * Removes a value from a sorted array.
 * @param {!IArrayLike<VALUE>} array The array to modify.
 * @param {VALUE} value The object to remove.
 * @param {function(VALUE, VALUE): number=} opt_compareFn Optional comparison
 *     function by which the array is ordered. Should take 2 arguments to
 *     compare, and return a negative number, zero, or a positive number
 *     depending on whether the first argument is less than, equal to, or
 *     greater than the second.
 * @return {boolean} True if an element was removed.
 * @template VALUE
 */
goog.array.binaryRemove = function(array, value, opt_compareFn) {
  var index = goog.array.binarySearch(array, value, opt_compareFn);
  return (index >= 0) ? goog.array.removeAt(array, index) : false;
};


/**
 * Splits an array into disjoint buckets according to a splitting function.
 * @param {IArrayLike<T>} array The array.
 * @param {function(this:S, T, number, !IArrayLike<T>):?} sorter Function to
 *     call for every element.  This takes 3 arguments (the element, the index
 *     and the array) and must return a valid object key (a string, number,
 *     etc), or undefined, if that object should not be placed in a bucket.
 * @param {S=} opt_obj The object to be used as the value of 'this' within
 *     sorter.
 * @return {!Object<!Array<T>>} An object, with keys being all of the unique
 *     return values of sorter, and values being arrays containing the items for
 *     which the splitter returned that key.
 * @template T,S
 */
goog.array.bucket = function(array, sorter, opt_obj) {
  var buckets = {};

  for (var i = 0; i < array.length; i++) {
    var value = array[i];
    var key = sorter.call(/** @type {?} */ (opt_obj), value, i, array);
    if (key !== undefined) {
      // Push the value to the right bucket, creating it if necessary.
      var bucket = buckets[key] || (buckets[key] = []);
      bucket.push(value);
    }
  }

  return buckets;
};


/**
 * Creates a new object built from the provided array and the key-generation
 * function.
 * @param {IArrayLike<T>} arr Array or array like object over
 *     which to iterate whose elements will be the values in the new object.
 * @param {?function(this:S, T, number, ?) : string} keyFunc The function to
 *     call for every element. This function takes 3 arguments (the element, the
 *     index and the array) and should return a string that will be used as the
 *     key for the element in the new object. If the function returns the same
 *     key for more than one element, the value for that key is
 *     implementation-defined.
 * @param {S=} opt_obj The object to be used as the value of 'this'
 *     within keyFunc.
 * @return {!Object<T>} The new object.
 * @template T,S
 */
goog.array.toObject = function(arr, keyFunc, opt_obj) {
  var ret = {};
  goog.array.forEach(arr, function(element, index) {
    ret[keyFunc.call(/** @type {?} */ (opt_obj), element, index, arr)] =
        element;
  });
  return ret;
};


/**
 * Creates a range of numbers in an arithmetic progression.
 *
 * Range takes 1, 2, or 3 arguments:
 * <pre>
 * range(5) is the same as range(0, 5, 1) and produces [0, 1, 2, 3, 4]
 * range(2, 5) is the same as range(2, 5, 1) and produces [2, 3, 4]
 * range(-2, -5, -1) produces [-2, -3, -4]
 * range(-2, -5, 1) produces [], since stepping by 1 wouldn't ever reach -5.
 * </pre>
 *
 * @param {number} startOrEnd The starting value of the range if an end argument
 *     is provided. Otherwise, the start value is 0, and this is the end value.
 * @param {number=} opt_end The optional end value of the range.
 * @param {number=} opt_step The step size between range values. Defaults to 1
 *     if opt_step is undefined or 0.
 * @return {!Array<number>} An array of numbers for the requested range. May be
 *     an empty array if adding the step would not converge toward the end
 *     value.
 */
goog.array.range = function(startOrEnd, opt_end, opt_step) {
  var array = [];
  var start = 0;
  var end = startOrEnd;
  var step = opt_step || 1;
  if (opt_end !== undefined) {
    start = startOrEnd;
    end = opt_end;
  }

  if (step * (end - start) < 0) {
    // Sign mismatch: start + step will never reach the end value.
    return [];
  }

  if (step > 0) {
    for (var i = start; i < end; i += step) {
      array.push(i);
    }
  } else {
    for (var i = start; i > end; i += step) {
      array.push(i);
    }
  }
  return array;
};


/**
 * Returns an array consisting of the given value repeated N times.
 *
 * @param {VALUE} value The value to repeat.
 * @param {number} n The repeat count.
 * @return {!Array<VALUE>} An array with the repeated value.
 * @template VALUE
 */
goog.array.repeat = function(value, n) {
  var array = [];
  for (var i = 0; i < n; i++) {
    array[i] = value;
  }
  return array;
};


/**
 * Returns an array consisting of every argument with all arrays
 * expanded in-place recursively.
 *
 * @param {...*} var_args The values to flatten.
 * @return {!Array<?>} An array containing the flattened values.
 */
goog.array.flatten = function(var_args) {
  var CHUNK_SIZE = 8192;

  var result = [];
  for (var i = 0; i < arguments.length; i++) {
    var element = arguments[i];
    if (Array.isArray(element)) {
      for (var c = 0; c < element.length; c += CHUNK_SIZE) {
        var chunk = goog.array.slice(element, c, c + CHUNK_SIZE);
        var recurseResult = goog.array.flatten.apply(null, chunk);
        for (var r = 0; r < recurseResult.length; r++) {
          result.push(recurseResult[r]);
        }
      }
    } else {
      result.push(element);
    }
  }
  return result;
};


/**
 * Rotates an array in-place. After calling this method, the element at
 * index i will be the element previously at index (i - n) %
 * array.length, for all values of i between 0 and array.length - 1,
 * inclusive.
 *
 * For example, suppose list comprises [t, a, n, k, s]. After invoking
 * rotate(array, 1) (or rotate(array, -4)), array will comprise [s, t, a, n, k].
 *
 * @param {!Array<T>} array The array to rotate.
 * @param {number} n The amount to rotate.
 * @return {!Array<T>} The array.
 * @template T
 */
goog.array.rotate = function(array, n) {
  goog.asserts.assert(array.length != null);

  if (array.length) {
    n %= array.length;
    if (n > 0) {
      Array.prototype.unshift.apply(array, array.splice(-n, n));
    } else if (n < 0) {
      Array.prototype.push.apply(array, array.splice(0, -n));
    }
  }
  return array;
};


/**
 * Moves one item of an array to a new position keeping the order of the rest
 * of the items. Example use case: keeping a list of JavaScript objects
 * synchronized with the corresponding list of DOM elements after one of the
 * elements has been dragged to a new position.
 * @param {!IArrayLike<?>} arr The array to modify.
 * @param {number} fromIndex Index of the item to move between 0 and
 *     {@code arr.length - 1}.
 * @param {number} toIndex Target index between 0 and {@code arr.length - 1}.
 */
goog.array.moveItem = function(arr, fromIndex, toIndex) {
  goog.asserts.assert(fromIndex >= 0 && fromIndex < arr.length);
  goog.asserts.assert(toIndex >= 0 && toIndex < arr.length);
  // Remove 1 item at fromIndex.
  var removedItems = Array.prototype.splice.call(arr, fromIndex, 1);
  // Insert the removed item at toIndex.
  Array.prototype.splice.call(arr, toIndex, 0, removedItems[0]);
  // We don't use goog.array.insertAt and goog.array.removeAt, because they're
  // significantly slower than splice.
};


/**
 * Creates a new array for which the element at position i is an array of the
 * ith element of the provided arrays.  The returned array will only be as long
 * as the shortest array provided; additional values are ignored.  For example,
 * the result of zipping [1, 2] and [3, 4, 5] is [[1,3], [2, 4]].
 *
 * This is similar to the zip() function in Python.  See {@link
 * http://docs.python.org/library/functions.html#zip}
 *
 * @param {...!IArrayLike<?>} var_args Arrays to be combined.
 * @return {!Array<!Array<?>>} A new array of arrays created from
 *     provided arrays.
 */
goog.array.zip = function(var_args) {
  if (!arguments.length) {
    return [];
  }
  var result = [];
  var minLen = arguments[0].length;
  for (var i = 1; i < arguments.length; i++) {
    if (arguments[i].length < minLen) {
      minLen = arguments[i].length;
    }
  }
  for (var i = 0; i < minLen; i++) {
    var value = [];
    for (var j = 0; j < arguments.length; j++) {
      value.push(arguments[j][i]);
    }
    result.push(value);
  }
  return result;
};


/**
 * Shuffles the values in the specified array using the Fisher-Yates in-place
 * shuffle (also known as the Knuth Shuffle). By default, calls Math.random()
 * and so resets the state of that random number generator. Similarly, may reset
 * the state of any other specified random number generator.
 *
 * Runtime: O(n)
 *
 * @param {!Array<?>} arr The array to be shuffled.
 * @param {function():number=} opt_randFn Optional random function to use for
 *     shuffling.
 *     Takes no arguments, and returns a random number on the interval [0, 1).
 *     Defaults to Math.random() using JavaScript's built-in Math library.
 */
goog.array.shuffle = function(arr, opt_randFn) {
  var randFn = opt_randFn || Math.random;

  for (var i = arr.length - 1; i > 0; i--) {
    // Choose a random array index in [0, i] (inclusive with i).
    var j = Math.floor(randFn() * (i + 1));

    var tmp = arr[i];
    arr[i] = arr[j];
    arr[j] = tmp;
  }
};


/**
 * Returns a new array of elements from arr, based on the indexes of elements
 * provided by index_arr. For example, the result of index copying
 * ['a', 'b', 'c'] with index_arr [1,0,0,2] is ['b', 'a', 'a', 'c'].
 *
 * @param {!IArrayLike<T>} arr The array to get a indexed copy from.
 * @param {!IArrayLike<number>} index_arr An array of indexes to get from arr.
 * @return {!Array<T>} A new array of elements from arr in index_arr order.
 * @template T
 */
goog.array.copyByIndex = function(arr, index_arr) {
  var result = [];
  goog.array.forEach(index_arr, function(index) { result.push(arr[index]); });
  return result;
};


/**
 * Maps each element of the input array into zero or more elements of the output
 * array.
 *
 * @param {!IArrayLike<VALUE>|string} arr Array or array like object
 *     over which to iterate.
 * @param {function(this:THIS, VALUE, number, ?): !Array<RESULT>} f The function
 *     to call for every element. This function takes 3 arguments (the element,
 *     the index and the array) and should return an array. The result will be
 *     used to extend a new array.
 * @param {THIS=} opt_obj The object to be used as the value of 'this' within f.
 * @return {!Array<RESULT>} a new array with the concatenation of all arrays
 *     returned from f.
 * @template THIS, VALUE, RESULT
 */
goog.array.concatMap = function(arr, f, opt_obj) {
  return goog.array.concat.apply([], goog.array.map(arr, f, opt_obj));
};

//third_party/javascript/closure/crypt/crypt.js
/**
 * @license
 * Copyright The Closure Library Authors.
 * SPDX-License-Identifier: Apache-2.0
 */

/**
 * @fileoverview Namespace with crypto related helper functions.
 */

goog.provide('goog.crypt');

goog.require('goog.array');
goog.require('goog.asserts');


/**
 * Turns a string into an array of bytes; a "byte" being a JS number in the
 * range 0-255. Multi-byte characters are written as little-endian.
 * @param {string} str String value to arrify.
 * @return {!Array<number>} Array of numbers corresponding to the
 *     UCS character codes of each character in str.
 */
goog.crypt.stringToByteArray = function(str) {
  var output = [], p = 0;
  for (var i = 0; i < str.length; i++) {
    var c = str.charCodeAt(i);
    // NOTE: c <= 0xffff since JavaScript strings are UTF-16.
    if (c > 0xff) {
      output[p++] = c & 0xff;
      c >>= 8;
    }
    output[p++] = c;
  }
  return output;
};


/**
 * Turns an array of numbers into the string given by the concatenation of the
 * characters to which the numbers correspond.
 * @param {!Uint8Array|!Array<number>} bytes Array of numbers representing
 *     characters.
 * @return {string} Stringification of the array.
 */
goog.crypt.byteArrayToString = function(bytes) {
  var CHUNK_SIZE = 8192;

  // Special-case the simple case for speed's sake.
  if (bytes.length <= CHUNK_SIZE) {
    return String.fromCharCode.apply(null, bytes);
  }

  // The remaining logic splits conversion by chunks since
  // Function#apply() has a maximum parameter count.
  // See discussion: http://goo.gl/LrWmZ9

  var str = '';
  for (var i = 0; i < bytes.length; i += CHUNK_SIZE) {
    var chunk = goog.array.slice(bytes, i, i + CHUNK_SIZE);
    str += String.fromCharCode.apply(null, chunk);
  }
  return str;
};


/**
 * Turns an array of numbers into the hex string given by the concatenation of
 * the hex values to which the numbers correspond.
 * @param {Uint8Array|Array<number>} array Array of numbers representing
 *     characters.
 * @param {string=} opt_separator Optional separator between values
 * @return {string} Hex string.
 */
goog.crypt.byteArrayToHex = function(array, opt_separator) {
  return goog.array
      .map(
          array,
          function(numByte) {
            var hexByte = numByte.toString(16);
            return hexByte.length > 1 ? hexByte : '0' + hexByte;
          })
      .join(opt_separator || '');
};


/**
 * Converts a hex string into an integer array.
 * @param {string} hexString Hex string of 16-bit integers (two characters
 *     per integer).
 * @return {!Array<number>} Array of {0,255} integers for the given string.
 */
goog.crypt.hexToByteArray = function(hexString) {
  goog.asserts.assert(
      hexString.length % 2 == 0, 'Key string length must be multiple of 2');
  var arr = [];
  for (var i = 0; i < hexString.length; i += 2) {
    arr.push(parseInt(hexString.substring(i, i + 2), 16));
  }
  return arr;
};


/**
 * Converts a JS string to a UTF-8 "byte" array.
 * @param {string} str 16-bit unicode string.
 * @return {!Array<number>} UTF-8 byte array.
 */
goog.crypt.stringToUtf8ByteArray = function(str) {
  // TODO(pupius): Use native implementations if/when available
  var out = [], p = 0;
  for (var i = 0; i < str.length; i++) {
    var c = str.charCodeAt(i);
    if (c < 128) {
      out[p++] = c;
    } else if (c < 2048) {
      out[p++] = (c >> 6) | 192;
      out[p++] = (c & 63) | 128;
    } else if (
        ((c & 0xFC00) == 0xD800) && (i + 1) < str.length &&
        ((str.charCodeAt(i + 1) & 0xFC00) == 0xDC00)) {
      // Surrogate Pair
      c = 0x10000 + ((c & 0x03FF) << 10) + (str.charCodeAt(++i) & 0x03FF);
      out[p++] = (c >> 18) | 240;
      out[p++] = ((c >> 12) & 63) | 128;
      out[p++] = ((c >> 6) & 63) | 128;
      out[p++] = (c & 63) | 128;
    } else {
      out[p++] = (c >> 12) | 224;
      out[p++] = ((c >> 6) & 63) | 128;
      out[p++] = (c & 63) | 128;
    }
  }
  return out;
};


/**
 * Converts a UTF-8 byte array to JavaScript's 16-bit Unicode.
 * @param {Uint8Array|Array<number>} bytes UTF-8 byte array.
 * @return {string} 16-bit Unicode string.
 */
goog.crypt.utf8ByteArrayToString = function(bytes) {
  // TODO(pupius): Use native implementations if/when available
  var out = [], pos = 0, c = 0;
  while (pos < bytes.length) {
    var c1 = bytes[pos++];
    if (c1 < 128) {
      out[c++] = String.fromCharCode(c1);
    } else if (c1 > 191 && c1 < 224) {
      var c2 = bytes[pos++];
      out[c++] = String.fromCharCode((c1 & 31) << 6 | c2 & 63);
    } else if (c1 > 239 && c1 < 365) {
      // Surrogate Pair
      var c2 = bytes[pos++];
      var c3 = bytes[pos++];
      var c4 = bytes[pos++];
      var u = ((c1 & 7) << 18 | (c2 & 63) << 12 | (c3 & 63) << 6 | c4 & 63) -
          0x10000;
      out[c++] = String.fromCharCode(0xD800 + (u >> 10));
      out[c++] = String.fromCharCode(0xDC00 + (u & 1023));
    } else {
      var c2 = bytes[pos++];
      var c3 = bytes[pos++];
      out[c++] =
          String.fromCharCode((c1 & 15) << 12 | (c2 & 63) << 6 | c3 & 63);
    }
  }
  return out.join('');
};


/**
 * XOR two byte arrays.
 * @param {!Uint8Array|!Int8Array|!Array<number>} bytes1 Byte array 1.
 * @param {!Uint8Array|!Int8Array|!Array<number>} bytes2 Byte array 2.
 * @return {!Array<number>} Resulting XOR of the two byte arrays.
 */
goog.crypt.xorByteArray = function(bytes1, bytes2) {
  goog.asserts.assert(
      bytes1.length == bytes2.length, 'XOR array lengths must match');

  var result = [];
  for (var i = 0; i < bytes1.length; i++) {
    result.push(bytes1[i] ^ bytes2[i]);
  }
  return result;
};

//third_party/javascript/closure/dom/asserts.js
/**
 * @license
 * Copyright The Closure Library Authors.
 * SPDX-License-Identifier: Apache-2.0
 */

goog.provide('goog.dom.asserts');

goog.require('goog.asserts');

/**
 * @fileoverview Custom assertions to ensure that an element has the appropriate
 * type.
 *
 * Using a goog.dom.safe wrapper on an object on the incorrect type (via an
 * incorrect static type cast) can result in security bugs: For instance,
 * g.d.s.setAnchorHref ensures that the URL assigned to the .href attribute
 * satisfies the SafeUrl contract, i.e., is safe to dereference as a hyperlink.
 * However, the value assigned to a HTMLLinkElement's .href property requires
 * the stronger TrustedResourceUrl contract, since it can refer to a stylesheet.
 * Thus, using g.d.s.setAnchorHref on an (incorrectly statically typed) object
 * of type HTMLLinkElement can result in a security vulnerability.
 * Assertions of the correct run-time type help prevent such incorrect use.
 *
 * In some cases, code using the DOM API is tested using mock objects (e.g., a
 * plain object such as {'href': url} instead of an actual Location object).
 * To allow such mocking, the assertions permit objects of types that are not
 * relevant DOM API objects at all (for instance, not Element or Location).
 *
 * Note that instanceof checks don't work straightforwardly in older versions of
 * IE, or across frames (see,
 * http://stackoverflow.com/questions/384286/javascript-isdom-how-do-you-check-if-a-javascript-object-is-a-dom-object,
 * http://stackoverflow.com/questions/26248599/instanceof-htmlelement-in-iframe-is-not-element-or-object).
 *
 * Hence, these assertions may pass vacuously in such scenarios. The resulting
 * risk of security bugs is limited by the following factors:
 *  - A bug can only arise in scenarios involving incorrect static typing (the
 *    wrapper methods are statically typed to demand objects of the appropriate,
 *    precise type).
 *  - Typically, code is tested and exercised in multiple browsers.
 */

/**
 * Asserts that a given object is a Location.
 *
 * To permit this assertion to pass in the context of tests where DOM APIs might
 * be mocked, also accepts any other type except for subtypes of {!Element}.
 * This is to ensure that, for instance, HTMLLinkElement is not being used in
 * place of a Location, since this could result in security bugs due to stronger
 * contracts required for assignments to the href property of the latter.
 *
 * @param {?Object} o The object whose type to assert.
 * @return {!Location}
 */
goog.dom.asserts.assertIsLocation = function(o) {
  if (goog.asserts.ENABLE_ASSERTS) {
    var win = goog.dom.asserts.getWindow_(o);
    if (win) {
      if (!o || (!(o instanceof win.Location) && o instanceof win.Element)) {
        goog.asserts.fail(
            'Argument is not a Location (or a non-Element mock); got: %s',
            goog.dom.asserts.debugStringForType_(o));
      }
    }
  }
  return /** @type {!Location} */ (o);
};


/**
 * Asserts that a given object is either the given subtype of Element
 * or a non-Element, non-Location Mock.
 *
 * To permit this assertion to pass in the context of tests where DOM
 * APIs might be mocked, also accepts any other type except for
 * subtypes of {!Element}.  This is to ensure that, for instance,
 * HTMLScriptElement is not being used in place of a HTMLImageElement,
 * since this could result in security bugs due to stronger contracts
 * required for assignments to the src property of the latter.
 *
 * The DOM type is looked up in the window the object belongs to.  In
 * some contexts, this might not be possible (e.g. when running tests
 * outside a browser, cross-domain lookup). In this case, the
 * assertions are skipped.
 *
 * @param {?Object} o The object whose type to assert.
 * @param {string} typename The name of the DOM type.
 * @return {!Element} The object.
 * @private
 */
// TODO(bangert): Make an analog of goog.dom.TagName to correctly handle casts?
goog.dom.asserts.assertIsElementType_ = function(o, typename) {
  if (goog.asserts.ENABLE_ASSERTS) {
    var win = goog.dom.asserts.getWindow_(o);
    if (win && typeof win[typename] != 'undefined') {
      if (!o ||
          (!(o instanceof win[typename]) &&
           (o instanceof win.Location || o instanceof win.Element))) {
        goog.asserts.fail(
            'Argument is not a %s (or a non-Element, non-Location mock); ' +
                'got: %s',
            typename, goog.dom.asserts.debugStringForType_(o));
      }
    }
  }
  return /** @type {!Element} */ (o);
};

/**
 * Asserts that a given object is a HTMLAnchorElement.
 *
 * To permit this assertion to pass in the context of tests where elements might
 * be mocked, also accepts objects that are not of type Location nor a subtype
 * of Element.
 *
 * @param {?Object} o The object whose type to assert.
 * @return {!HTMLAnchorElement}
 * @deprecated Use goog.asserts.dom.assertIsHtmlAnchorElement instead.
 */
goog.dom.asserts.assertIsHTMLAnchorElement = function(o) {
  return /** @type {!HTMLAnchorElement} */ (
      goog.dom.asserts.assertIsElementType_(o, 'HTMLAnchorElement'));
};

/**
 * Asserts that a given object is a HTMLButtonElement.
 *
 * To permit this assertion to pass in the context of tests where elements might
 * be mocked, also accepts objects that are not a subtype of Element.
 *
 * @param {?Object} o The object whose type to assert.
 * @return {!HTMLButtonElement}
 * @deprecated Use goog.asserts.dom.assertIsHtmlButtonElement instead.
 */
goog.dom.asserts.assertIsHTMLButtonElement = function(o) {
  return /** @type {!HTMLButtonElement} */ (
      goog.dom.asserts.assertIsElementType_(o, 'HTMLButtonElement'));
};

/**
 * Asserts that a given object is a HTMLLinkElement.
 *
 * To permit this assertion to pass in the context of tests where elements might
 * be mocked, also accepts objects that are not a subtype of Element.
 *
 * @param {?Object} o The object whose type to assert.
 * @return {!HTMLLinkElement}
 * @deprecated Use goog.asserts.dom.assertIsHtmlLinkElement instead.
 */
goog.dom.asserts.assertIsHTMLLinkElement = function(o) {
  return /** @type {!HTMLLinkElement} */ (
      goog.dom.asserts.assertIsElementType_(o, 'HTMLLinkElement'));
};

/**
 * Asserts that a given object is a HTMLImageElement.
 *
 * To permit this assertion to pass in the context of tests where elements might
 * be mocked, also accepts objects that are not a subtype of Element.
 *
 * @param {?Object} o The object whose type to assert.
 * @return {!HTMLImageElement}
 * @deprecated Use goog.asserts.dom.assertIsHtmlImageElement instead.
 */
goog.dom.asserts.assertIsHTMLImageElement = function(o) {
  return /** @type {!HTMLImageElement} */ (
      goog.dom.asserts.assertIsElementType_(o, 'HTMLImageElement'));
};

/**
 * Asserts that a given object is a HTMLAudioElement.
 *
 * To permit this assertion to pass in the context of tests where elements might
 * be mocked, also accepts objects that are not a subtype of Element.
 *
 * @param {?Object} o The object whose type to assert.
 * @return {!HTMLAudioElement}
 * @deprecated Use goog.asserts.dom.assertIsHtmlAudioElement instead.
 */
goog.dom.asserts.assertIsHTMLAudioElement = function(o) {
  return /** @type {!HTMLAudioElement} */ (
      goog.dom.asserts.assertIsElementType_(o, 'HTMLAudioElement'));
};

/**
 * Asserts that a given object is a HTMLVideoElement.
 *
 * To permit this assertion to pass in the context of tests where elements might
 * be mocked, also accepts objects that are not a subtype of Element.
 *
 * @param {?Object} o The object whose type to assert.
 * @return {!HTMLVideoElement}
 * @deprecated Use goog.asserts.dom.assertIsHtmlVideoElement instead.
 */
goog.dom.asserts.assertIsHTMLVideoElement = function(o) {
  return /** @type {!HTMLVideoElement} */ (
      goog.dom.asserts.assertIsElementType_(o, 'HTMLVideoElement'));
};

/**
 * Asserts that a given object is a HTMLInputElement.
 *
 * To permit this assertion to pass in the context of tests where elements might
 * be mocked, also accepts objects that are not a subtype of Element.
 *
 * @param {?Object} o The object whose type to assert.
 * @return {!HTMLInputElement}
 * @deprecated Use goog.asserts.dom.assertIsHtmlInputElement instead.
 */
goog.dom.asserts.assertIsHTMLInputElement = function(o) {
  return /** @type {!HTMLInputElement} */ (
      goog.dom.asserts.assertIsElementType_(o, 'HTMLInputElement'));
};

/**
 * Asserts that a given object is a HTMLTextAreaElement.
 *
 * To permit this assertion to pass in the context of tests where elements might
 * be mocked, also accepts objects that are not a subtype of Element.
 *
 * @param {?Object} o The object whose type to assert.
 * @return {!HTMLTextAreaElement}
 * @deprecated Use goog.asserts.dom.assertIsHtmlTextAreaElement instead.
 */
goog.dom.asserts.assertIsHTMLTextAreaElement = function(o) {
  return /** @type {!HTMLTextAreaElement} */ (
      goog.dom.asserts.assertIsElementType_(o, 'HTMLTextAreaElement'));
};

/**
 * Asserts that a given object is a HTMLCanvasElement.
 *
 * To permit this assertion to pass in the context of tests where elements might
 * be mocked, also accepts objects that are not a subtype of Element.
 *
 * @param {?Object} o The object whose type to assert.
 * @return {!HTMLCanvasElement}
 * @deprecated Use goog.asserts.dom.assertIsHtmlCanvasElement instead.
 */
goog.dom.asserts.assertIsHTMLCanvasElement = function(o) {
  return /** @type {!HTMLCanvasElement} */ (
      goog.dom.asserts.assertIsElementType_(o, 'HTMLCanvasElement'));
};

/**
 * Asserts that a given object is a HTMLEmbedElement.
 *
 * To permit this assertion to pass in the context of tests where elements might
 * be mocked, also accepts objects that are not a subtype of Element.
 *
 * @param {?Object} o The object whose type to assert.
 * @return {!HTMLEmbedElement}
 * @deprecated Use goog.asserts.dom.assertIsHtmlEmbedElement instead.
 */
goog.dom.asserts.assertIsHTMLEmbedElement = function(o) {
  return /** @type {!HTMLEmbedElement} */ (
      goog.dom.asserts.assertIsElementType_(o, 'HTMLEmbedElement'));
};

/**
 * Asserts that a given object is a HTMLFormElement.
 *
 * To permit this assertion to pass in the context of tests where elements might
 * be mocked, also accepts objects that are not a subtype of Element.
 *
 * @param {?Object} o The object whose type to assert.
 * @return {!HTMLFormElement}
 * @deprecated Use goog.asserts.dom.assertIsHtmlFormElement instead.
 */
goog.dom.asserts.assertIsHTMLFormElement = function(o) {
  return /** @type {!HTMLFormElement} */ (
      goog.dom.asserts.assertIsElementType_(o, 'HTMLFormElement'));
};

/**
 * Asserts that a given object is a HTMLFrameElement.
 *
 * To permit this assertion to pass in the context of tests where elements might
 * be mocked, also accepts objects that are not a subtype of Element.
 *
 * @param {?Object} o The object whose type to assert.
 * @return {!HTMLFrameElement}
 * @deprecated Use goog.asserts.dom.assertIsHtmlFrameElement instead.
 */
goog.dom.asserts.assertIsHTMLFrameElement = function(o) {
  return /** @type {!HTMLFrameElement} */ (
      goog.dom.asserts.assertIsElementType_(o, 'HTMLFrameElement'));
};

/**
 * Asserts that a given object is a HTMLIFrameElement.
 *
 * To permit this assertion to pass in the context of tests where elements might
 * be mocked, also accepts objects that are not a subtype of Element.
 *
 * @param {?Object} o The object whose type to assert.
 * @return {!HTMLIFrameElement}
 * @deprecated Use goog.asserts.dom.assertIsHtmlIFrameElement instead.
 */
goog.dom.asserts.assertIsHTMLIFrameElement = function(o) {
  return /** @type {!HTMLIFrameElement} */ (
      goog.dom.asserts.assertIsElementType_(o, 'HTMLIFrameElement'));
};

/**
 * Asserts that a given object is a HTMLObjectElement.
 *
 * To permit this assertion to pass in the context of tests where elements might
 * be mocked, also accepts objects that are not a subtype of Element.
 *
 * @param {?Object} o The object whose type to assert.
 * @return {!HTMLObjectElement}
 * @deprecated Use goog.asserts.dom.assertIsHtmlObjectElement instead.
 */
goog.dom.asserts.assertIsHTMLObjectElement = function(o) {
  return /** @type {!HTMLObjectElement} */ (
      goog.dom.asserts.assertIsElementType_(o, 'HTMLObjectElement'));
};

/**
 * Asserts that a given object is a HTMLScriptElement.
 *
 * To permit this assertion to pass in the context of tests where elements might
 * be mocked, also accepts objects that are not a subtype of Element.
 *
 * @param {?Object} o The object whose type to assert.
 * @return {!HTMLScriptElement}
 * @deprecated Use goog.asserts.dom.assertIsHtmlScriptElement instead.
 */
goog.dom.asserts.assertIsHTMLScriptElement = function(o) {
  return /** @type {!HTMLScriptElement} */ (
      goog.dom.asserts.assertIsElementType_(o, 'HTMLScriptElement'));
};

/**
 * Returns a string representation of a value's type.
 *
 * @param {*} value An object, or primitive.
 * @return {string} The best display name for the value.
 * @private
 */
goog.dom.asserts.debugStringForType_ = function(value) {
  if (goog.isObject(value)) {
    try {
      return /** @type {string|undefined} */ (value.constructor.displayName) ||
          value.constructor.name || Object.prototype.toString.call(value);
    } catch (e) {
      return '<object could not be stringified>';
    }
  } else {
    return value === undefined ? 'undefined' :
                                 value === null ? 'null' : typeof value;
  }
};

/**
 * Gets window of element.
 * @param {?Object} o
 * @return {?Window}
 * @private
 * @suppress {strictMissingProperties} ownerDocument not defined on Object
 */
goog.dom.asserts.getWindow_ = function(o) {
  try {
    var doc = o && o.ownerDocument;
    // This can throw “Blocked a frame with origin "chrome-extension://..." from
    // accessing a cross-origin frame” in Chrome extension.
    // MOE:begin_strip See
    // http://yaqs/6574360854528000#a6424563401097216.
    // MOE:end_strip
    var win =
        doc && /** @type {?Window} */ (doc.defaultView || doc.parentWindow);
    win = win || /** @type {!Window} */ (goog.global);
    // This can throw “Permission denied to access property "Element" on
    // cross-origin object”.
    if (win.Element && win.Location) {
      return win;
    }
  } catch (ex) {
  }
  return null;
};

//third_party/javascript/closure/functions/functions.js
/**
 * @license
 * Copyright The Closure Library Authors.
 * SPDX-License-Identifier: Apache-2.0
 */

/**
 * @fileoverview Utilities for creating functions. Loosely inspired by these
 * java classes from the Guava library:
 * com.google.common.base.Functions
 * https://google.github.io/guava/releases/snapshot-jre/api/docs/index.html?com/google/common/base/Functions.html
 *
 * com.google.common.base.Predicates
 * https://google.github.io/guava/releases/snapshot-jre/api/docs/index.html?com/google/common/base/Predicates.html
 *
 * More about these can be found at
 * https://github.com/google/guava/wiki/FunctionalExplained
 */


goog.provide('goog.functions');


/**
 * Creates a function that always returns the same value.
 * @param {T} retValue The value to return.
 * @return {function():T} The new function.
 * @template T
 */
goog.functions.constant = function(retValue) {
  return function() { return retValue; };
};


/**
 * Always returns false.
 * @type {function(...): boolean}
 */
goog.functions.FALSE = function() {
  return false;
};


/**
 * Always returns true.
 * @type {function(...): boolean}
 */
goog.functions.TRUE = function() {
  return true;
};


/**
 * Always returns NULL.
 * @type {function(...): null}
 */
goog.functions.NULL = function() {
  return null;
};


/**
 * A simple function that returns the first argument of whatever is passed
 * into it.
 * @param {T=} opt_returnValue The single value that will be returned.
 * @param {...*} var_args Optional trailing arguments. These are ignored.
 * @return {T} The first argument passed in, or undefined if nothing was passed.
 * @template T
 */
goog.functions.identity = function(opt_returnValue, var_args) {
  return opt_returnValue;
};


/**
 * Creates a function that always throws an error with the given message.
 * @param {string} message The error message.
 * @return {!Function} The error-throwing function.
 */
goog.functions.error = function(message) {
  return function() {
    throw new Error(message);
  };
};


/**
 * Creates a function that throws the given object.
 * @param {*} err An object to be thrown.
 * @return {!Function} The error-throwing function.
 */
goog.functions.fail = function(err) {
  return function() { throw err; };
};


/**
 * Given a function, create a function that keeps opt_numArgs arguments and
 * silently discards all additional arguments.
 * @param {Function} f The original function.
 * @param {number=} opt_numArgs The number of arguments to keep. Defaults to 0.
 * @return {!Function} A version of f that only keeps the first opt_numArgs
 *     arguments.
 */
goog.functions.lock = function(f, opt_numArgs) {
  opt_numArgs = opt_numArgs || 0;
  return function() {
    const self = /** @type {*} */ (this);
    return f.apply(self, Array.prototype.slice.call(arguments, 0, opt_numArgs));
  };
};


/**
 * Creates a function that returns its nth argument.
 * @param {number} n The position of the return argument.
 * @return {!Function} A new function.
 */
goog.functions.nth = function(n) {
  return function() { return arguments[n]; };
};


/**
 * Like goog.partial(), except that arguments are added after arguments to the
 * returned function.
 *
 * Usage:
 * function f(arg1, arg2, arg3, arg4) { ... }
 * var g = goog.functions.partialRight(f, arg3, arg4);
 * g(arg1, arg2);
 *
 * @param {!Function} fn A function to partially apply.
 * @param {...*} var_args Additional arguments that are partially applied to fn
 *     at the end.
 * @return {!Function} A partially-applied form of the function goog.partial()
 *     was invoked as a method of.
 */
goog.functions.partialRight = function(fn, var_args) {
  const rightArgs = Array.prototype.slice.call(arguments, 1);
  return function() {
    const self = /** @type {*} */ (this);
    const newArgs = Array.prototype.slice.call(arguments);
    newArgs.push.apply(newArgs, rightArgs);
    return fn.apply(self, newArgs);
  };
};


/**
 * Given a function, create a new function that swallows its return value
 * and replaces it with a new one.
 * @param {Function} f A function.
 * @param {T} retValue A new return value.
 * @return {function(...?):T} A new function.
 * @template T
 */
goog.functions.withReturnValue = function(f, retValue) {
  return goog.functions.sequence(f, goog.functions.constant(retValue));
};


/**
 * Creates a function that returns whether its argument equals the given value.
 *
 * Example:
 * var key = goog.object.findKey(obj, goog.functions.equalTo('needle'));
 *
 * @param {*} value The value to compare to.
 * @param {boolean=} opt_useLooseComparison Whether to use a loose (==)
 *     comparison rather than a strict (===) one. Defaults to false.
 * @return {function(*):boolean} The new function.
 */
goog.functions.equalTo = function(value, opt_useLooseComparison) {
  return function(other) {
    return opt_useLooseComparison ? (value == other) : (value === other);
  };
};


/**
 * Creates the composition of the functions passed in.
 * For example, (goog.functions.compose(f, g))(a) is equivalent to f(g(a)).
 * @param {function(...?):T} fn The final function.
 * @param {...Function} var_args A list of functions.
 * @return {function(...?):T} The composition of all inputs.
 * @template T
 */
goog.functions.compose = function(fn, var_args) {
  const functions = arguments;
  const length = functions.length;
  return function() {
    const self = /** @type {*} */ (this);
    let result;
    if (length) {
      result = functions[length - 1].apply(self, arguments);
    }

    for (let i = length - 2; i >= 0; i--) {
      result = functions[i].call(self, result);
    }
    return result;
  };
};


/**
 * Creates a function that calls the functions passed in in sequence, and
 * returns the value of the last function. For example,
 * (goog.functions.sequence(f, g))(x) is equivalent to f(x),g(x).
 * @param {...Function} var_args A list of functions.
 * @return {!Function} A function that calls all inputs in sequence.
 */
goog.functions.sequence = function(var_args) {
  const functions = arguments;
  const length = functions.length;
  return function() {
    const self = /** @type {*} */ (this);
    let result;
    for (let i = 0; i < length; i++) {
      result = functions[i].apply(self, arguments);
    }
    return result;
  };
};


/**
 * Creates a function that returns true if each of its components evaluates
 * to true. The components are evaluated in order, and the evaluation will be
 * short-circuited as soon as a function returns false.
 * For example, (goog.functions.and(f, g))(x) is equivalent to f(x) && g(x).
 * @param {...Function} var_args A list of functions.
 * @return {function(...?):boolean} A function that ANDs its component
 *      functions.
 */
goog.functions.and = function(var_args) {
  const functions = arguments;
  const length = functions.length;
  return function() {
    const self = /** @type {*} */ (this);
    for (let i = 0; i < length; i++) {
      if (!functions[i].apply(self, arguments)) {
        return false;
      }
    }
    return true;
  };
};


/**
 * Creates a function that returns true if any of its components evaluates
 * to true. The components are evaluated in order, and the evaluation will be
 * short-circuited as soon as a function returns true.
 * For example, (goog.functions.or(f, g))(x) is equivalent to f(x) || g(x).
 * @param {...Function} var_args A list of functions.
 * @return {function(...?):boolean} A function that ORs its component
 *    functions.
 */
goog.functions.or = function(var_args) {
  const functions = arguments;
  const length = functions.length;
  return function() {
    const self = /** @type {*} */ (this);
    for (let i = 0; i < length; i++) {
      if (functions[i].apply(self, arguments)) {
        return true;
      }
    }
    return false;
  };
};


/**
 * Creates a function that returns the Boolean opposite of a provided function.
 * For example, (goog.functions.not(f))(x) is equivalent to !f(x).
 * @param {!Function} f The original function.
 * @return {function(...?):boolean} A function that delegates to f and returns
 * opposite.
 */
goog.functions.not = function(f) {
  return function() {
    const self = /** @type {*} */ (this);
    return !f.apply(self, arguments);
  };
};


/**
 * Generic factory function to construct an object given the constructor
 * and the arguments. Intended to be bound to create object factories.
 *
 * Example:
 *
 * var factory = goog.partial(goog.functions.create, Class);
 *
 * @param {function(new:T, ...)} constructor The constructor for the Object.
 * @param {...*} var_args The arguments to be passed to the constructor.
 * @return {T} A new instance of the class given in `constructor`.
 * @template T
 * @deprecated This function does not work with ES6 class constructors. Use
 *     arrow functions + spread args instead.
 */
goog.functions.create = function(constructor, var_args) {
  /**
   * @constructor
   * @final
   */
  const temp = function() {};
  temp.prototype = constructor.prototype;

  // obj will have constructor's prototype in its chain and
  // 'obj instanceof constructor' will be true.
  const obj = new temp();

  // obj is initialized by constructor.
  // arguments is only array-like so lacks shift(), but can be used with
  // the Array prototype function.
  constructor.apply(obj, Array.prototype.slice.call(arguments, 1));
  return obj;
};


/**
 * @define {boolean} Whether the return value cache should be used.
 *    This should only be used to disable caches when testing.
 */
goog.functions.CACHE_RETURN_VALUE =
    goog.define('goog.functions.CACHE_RETURN_VALUE', true);


/**
 * Gives a wrapper function that caches the return value of a parameterless
 * function when first called.
 *
 * When called for the first time, the given function is called and its
 * return value is cached (thus this is only appropriate for idempotent
 * functions).  Subsequent calls will return the cached return value. This
 * allows the evaluation of expensive functions to be delayed until first used.
 *
 * To cache the return values of functions with parameters, see goog.memoize.
 *
 * @param {function():T} fn A function to lazily evaluate.
 * @return {function():T} A wrapped version the function.
 * @template T
 */
goog.functions.cacheReturnValue = function(fn) {
  let called = false;
  let value;

  return function() {
    if (!goog.functions.CACHE_RETURN_VALUE) {
      return fn();
    }

    if (!called) {
      value = fn();
      called = true;
    }

    return value;
  };
};


/**
 * Wraps a function to allow it to be called, at most, once. All
 * additional calls are no-ops.
 *
 * This is particularly useful for initialization functions
 * that should be called, at most, once.
 *
 * @param {function():*} f Function to call.
 * @return {function():undefined} Wrapped function.
 */
goog.functions.once = function(f) {
  // Keep a reference to the function that we null out when we're done with
  // it -- that way, the function can be GC'd when we're done with it.
  let inner = f;
  return function() {
    if (inner) {
      const tmp = inner;
      inner = null;
      tmp();
    }
  };
};


/**
 * Wraps a function to allow it to be called, at most, once per interval
 * (specified in milliseconds). If the wrapper function is called N times within
 * that interval, only the Nth call will go through.
 *
 * This is particularly useful for batching up repeated actions where the
 * last action should win. This can be used, for example, for refreshing an
 * autocomplete pop-up every so often rather than updating with every keystroke,
 * since the final text typed by the user is the one that should produce the
 * final autocomplete results. For more stateful debouncing with support for
 * pausing, resuming, and canceling debounced actions, use
 * `goog.async.Debouncer`.
 *
 * @param {function(this:SCOPE, ...?)} f Function to call.
 * @param {number} interval Interval over which to debounce. The function will
 *     only be called after the full interval has elapsed since the last call.
 * @param {SCOPE=} opt_scope Object in whose scope to call the function.
 * @return {function(...?): undefined} Wrapped function.
 * @template SCOPE
 */
goog.functions.debounce = function(f, interval, opt_scope) {
  let timeout = 0;
  return /** @type {function(...?)} */ (function(var_args) {
    goog.global.clearTimeout(timeout);
    const args = arguments;
    timeout = goog.global.setTimeout(function() {
      f.apply(opt_scope, args);
    }, interval);
  });
};


/**
 * Wraps a function to allow it to be called, at most, once per interval
 * (specified in milliseconds). If the wrapper function is called N times in
 * that interval, both the 1st and the Nth calls will go through.
 *
 * This is particularly useful for limiting repeated user requests where the
 * the last action should win, but you also don't want to wait until the end of
 * the interval before sending a request out, as it leads to a perception of
 * slowness for the user.
 *
 * @param {function(this:SCOPE, ...?)} f Function to call.
 * @param {number} interval Interval over which to throttle. The function can
 *     only be called once per interval.
 * @param {SCOPE=} opt_scope Object in whose scope to call the function.
 * @return {function(...?): undefined} Wrapped function.
 * @template SCOPE
 */
goog.functions.throttle = function(f, interval, opt_scope) {
  let timeout = 0;
  let shouldFire = false;
  let args = [];

  const handleTimeout = function() {
    timeout = 0;
    if (shouldFire) {
      shouldFire = false;
      fire();
    }
  };

  const fire = function() {
    timeout = goog.global.setTimeout(handleTimeout, interval);
    f.apply(opt_scope, args);
  };

  return /** @type {function(...?)} */ (function(var_args) {
    args = arguments;
    if (!timeout) {
      fire();
    } else {
      shouldFire = true;
    }
  });
};


/**
 * Wraps a function to allow it to be called, at most, once per interval
 * (specified in milliseconds). If the wrapper function is called N times within
 * that interval, only the 1st call will go through.
 *
 * This is particularly useful for limiting repeated user requests where the
 * first request is guaranteed to have all the data required to perform the
 * final action, so there's no need to wait until the end of the interval before
 * sending the request out.
 *
 * @param {function(this:SCOPE, ...?)} f Function to call.
 * @param {number} interval Interval over which to rate-limit. The function will
 *     only be called once per interval, and ignored for the remainer of the
 *     interval.
 * @param {SCOPE=} opt_scope Object in whose scope to call the function.
 * @return {function(...?): undefined} Wrapped function.
 * @template SCOPE
 */
goog.functions.rateLimit = function(f, interval, opt_scope) {
  let timeout = 0;

  const handleTimeout = function() {
    timeout = 0;
  };

  return /** @type {function(...?)} */ (function(var_args) {
    if (!timeout) {
      timeout = goog.global.setTimeout(handleTimeout, interval);
      f.apply(opt_scope, arguments);
    }
  });
};

//javascript/closure/i18n/bidi.js
// Copyright 2007 The Closure Library Authors. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS-IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

/**
 * @fileoverview Utility functions for supporting Bidi issues.
 */


/**
 * Namespace for bidi supporting functions.
 */
goog.provide('goog.i18n.bidi');
goog.provide('goog.i18n.bidi.Dir');
goog.provide('goog.i18n.bidi.DirectionalString');
goog.provide('goog.i18n.bidi.Format');


/**
 * @define {boolean} FORCE_RTL forces the {@link goog.i18n.bidi.IS_RTL} constant
 * to say that the current locale is a RTL locale.  This should only be used
 * if you want to override the default behavior for deciding whether the
 * current locale is RTL or not.
 *
 * {@see goog.i18n.bidi.IS_RTL}
 */
goog.i18n.bidi.FORCE_RTL = goog.define('goog.i18n.bidi.FORCE_RTL', false);


/**
 * Constant that defines whether or not the current locale is a RTL locale.
 * If {@link goog.i18n.bidi.FORCE_RTL} is not true, this constant will default
 * to check that {@link goog.LOCALE} is one of a few major RTL locales.
 *
 * <p>This is designed to be a maximally efficient compile-time constant. For
 * example, for the default goog.LOCALE, compiling
 * "if (goog.i18n.bidi.IS_RTL) alert('rtl') else {}" should produce no code. It
 * is this design consideration that limits the implementation to only
 * supporting a few major RTL locales, as opposed to the broader repertoire of
 * something like goog.i18n.bidi.isRtlLanguage.
 *
 * <p>Since this constant refers to the directionality of the locale, it is up
 * to the caller to determine if this constant should also be used for the
 * direction of the UI.
 *
 * {@see goog.LOCALE}
 *
 * @type {boolean}
 *
 * TODO(aharon): write a test that checks that this is a compile-time constant.
 */
// LINT.IfChange
goog.i18n.bidi.IS_RTL =
    goog.i18n.bidi.FORCE_RTL ||
    ((goog.LOCALE.substring(0, 2).toLowerCase() == 'ar' ||
      goog.LOCALE.substring(0, 2).toLowerCase() == 'fa' ||
      goog.LOCALE.substring(0, 2).toLowerCase() == 'he' ||
      goog.LOCALE.substring(0, 2).toLowerCase() == 'iw' ||
      goog.LOCALE.substring(0, 2).toLowerCase() == 'ps' ||
      goog.LOCALE.substring(0, 2).toLowerCase() == 'sd' ||
      goog.LOCALE.substring(0, 2).toLowerCase() == 'ug' ||
      goog.LOCALE.substring(0, 2).toLowerCase() == 'ur' ||
      goog.LOCALE.substring(0, 2).toLowerCase() == 'yi') &&
     (goog.LOCALE.length == 2 || goog.LOCALE.substring(2, 3) == '-' ||
      goog.LOCALE.substring(2, 3) == '_')) ||
    (  // Specific to CKB (Central Kurdish)
        goog.LOCALE.length >= 3 &&
        goog.LOCALE.substring(0, 3).toLowerCase() == 'ckb' &&
        (goog.LOCALE.length == 3 || goog.LOCALE.substring(3, 4) == '-' ||
         goog.LOCALE.substring(3, 4) == '_')) ||
    (  // 2 letter language codes with RTL scripts
        goog.LOCALE.length >= 7 &&
        ((goog.LOCALE.substring(2, 3) == '-' ||
          goog.LOCALE.substring(2, 3) == '_') &&
         (goog.LOCALE.substring(3, 7).toLowerCase() == 'adlm' ||
          goog.LOCALE.substring(3, 7).toLowerCase() == 'arab' ||
          goog.LOCALE.substring(3, 7).toLowerCase() == 'hebr' ||
          goog.LOCALE.substring(3, 7).toLowerCase() == 'nkoo' ||
          goog.LOCALE.substring(3, 7).toLowerCase() == 'rohg' ||
          goog.LOCALE.substring(3, 7).toLowerCase() == 'thaa'))) ||
    (  // 3 letter languages codes with RTL scripts
        goog.LOCALE.length >= 8 &&
        ((goog.LOCALE.substring(3, 4) == '-' ||
          goog.LOCALE.substring(3, 4) == '_') &&
         (goog.LOCALE.substring(4, 8).toLowerCase() == 'adlm' ||
          goog.LOCALE.substring(4, 8).toLowerCase() == 'arab' ||
          goog.LOCALE.substring(4, 8).toLowerCase() == 'hebr' ||
          goog.LOCALE.substring(4, 8).toLowerCase() == 'nkoo' ||
          goog.LOCALE.substring(4, 8).toLowerCase() == 'rohg' ||
          goog.LOCALE.substring(4, 8).toLowerCase() == 'thaa')));
//    closure/RtlLocalesTest.java)

// TODO(b/77919903): Add additional scripts and languages that are RTL,
// e.g., mende, samaritan, etc.


/**
 * Unicode formatting characters and directionality string constants.
 * @enum {string}
 */
goog.i18n.bidi.Format = {
  /** Unicode "Left-To-Right Embedding" (LRE) character. */
  LRE: '\u202A',
  /** Unicode "Right-To-Left Embedding" (RLE) character. */
  RLE: '\u202B',
  /** Unicode "Pop Directional Formatting" (PDF) character. */
  PDF: '\u202C',
  /** Unicode "Left-To-Right Mark" (LRM) character. */
  LRM: '\u200E',
  /** Unicode "Right-To-Left Mark" (RLM) character. */
  RLM: '\u200F'
};


/**
 * Directionality enum.
 * @enum {number}
 */
goog.i18n.bidi.Dir = {
  /**
   * Left-to-right.
   */
  LTR: 1,

  /**
   * Right-to-left.
   */
  RTL: -1,

  /**
   * Neither left-to-right nor right-to-left.
   */
  NEUTRAL: 0
};


/**
 * 'right' string constant.
 * @type {string}
 */
goog.i18n.bidi.RIGHT = 'right';


/**
 * 'left' string constant.
 * @type {string}
 */
goog.i18n.bidi.LEFT = 'left';


/**
 * 'left' if locale is RTL, 'right' if not.
 * @type {string}
 */
goog.i18n.bidi.I18N_RIGHT =
    goog.i18n.bidi.IS_RTL ? goog.i18n.bidi.LEFT : goog.i18n.bidi.RIGHT;


/**
 * 'right' if locale is RTL, 'left' if not.
 * @type {string}
 */
goog.i18n.bidi.I18N_LEFT =
    goog.i18n.bidi.IS_RTL ? goog.i18n.bidi.RIGHT : goog.i18n.bidi.LEFT;


/**
 * Convert a directionality given in various formats to a goog.i18n.bidi.Dir
 * constant. Useful for interaction with different standards of directionality
 * representation.
 *
 * @param {goog.i18n.bidi.Dir|number|boolean|null} givenDir Directionality given
 *     in one of the following formats:
 *     1. A goog.i18n.bidi.Dir constant.
 *     2. A number (positive = LTR, negative = RTL, 0 = neutral).
 *     3. A boolean (true = RTL, false = LTR).
 *     4. A null for unknown directionality.
 * @param {boolean=} opt_noNeutral Whether a givenDir of zero or
 *     goog.i18n.bidi.Dir.NEUTRAL should be treated as null, i.e. unknown, in
 *     order to preserve legacy behavior.
 * @return {?goog.i18n.bidi.Dir} A goog.i18n.bidi.Dir constant matching the
 *     given directionality. If given null, returns null (i.e. unknown).
 */
goog.i18n.bidi.toDir = function(givenDir, opt_noNeutral) {
  if (typeof givenDir == 'number') {
    // This includes the non-null goog.i18n.bidi.Dir case.
    return givenDir > 0 ?
        goog.i18n.bidi.Dir.LTR :
        givenDir < 0 ? goog.i18n.bidi.Dir.RTL :
                       opt_noNeutral ? null : goog.i18n.bidi.Dir.NEUTRAL;
  } else if (givenDir == null) {
    return null;
  } else {
    // Must be typeof givenDir == 'boolean'.
    return givenDir ? goog.i18n.bidi.Dir.RTL : goog.i18n.bidi.Dir.LTR;
  }
};


/**
 * A practical pattern to identify strong LTR character in the BMP.
 * This pattern is not theoretically correct according to the Unicode
 * standard. It is simplified for performance and small code size.
 * It also partially supports LTR scripts beyond U+FFFF by including
 * UTF-16 high surrogate values corresponding to mostly L-class code
 * point ranges.
 * However, low surrogate values and private-use regions are not included
 * in this RegEx.
 * @type {string}
 * @private
 */
goog.i18n.bidi.ltrChars_ =
    'A-Za-z\u00C0-\u00D6\u00D8-\u00F6\u00F8-\u02B8\u0300-\u0590\u0900-\u1FFF' +
    '\u200E\u2C00-\uD801\uD804-\uD839\uD83C-\uDBFF' +
    '\uF900-\uFB1C\uFE00-\uFE6F\uFEFD-\uFFFF';

/**
 * A practical pattern to identify strong RTL character. This pattern is not
 * theoretically correct according to the Unicode standard. It is simplified
 * for performance and small code size.
 * It also partially supports RTL scripts beyond U+FFFF by including
 * UTF-16 high surrogate values corresponding to mostly R- or AL-class
 * code point ranges.
 * However, low surrogate values and private-use regions are not included
 * in this RegEx.
 * @type {string}
 * @private
 */
goog.i18n.bidi.rtlChars_ =
    '\u0591-\u06EF\u06FA-\u08FF\u200F\uD802-\uD803\uD83A-\uD83B' +
    '\uFB1D-\uFDFF\uFE70-\uFEFC';

/**
 * Simplified regular expression for an HTML tag (opening or closing) or an HTML
 * escape. We might want to skip over such expressions when estimating the text
 * directionality.
 * @type {RegExp}
 * @private
 */
goog.i18n.bidi.htmlSkipReg_ = /<[^>]*>|&[^;]+;/g;


/**
 * Returns the input text with spaces instead of HTML tags or HTML escapes, if
 * opt_isStripNeeded is true. Else returns the input as is.
 * Useful for text directionality estimation.
 * Note: the function should not be used in other contexts; it is not 100%
 * correct, but rather a good-enough implementation for directionality
 * estimation purposes.
 * @param {string} str The given string.
 * @param {boolean=} opt_isStripNeeded Whether to perform the stripping.
 *     Default: false (to retain consistency with calling functions).
 * @return {string} The given string cleaned of HTML tags / escapes.
 * @private
 */
goog.i18n.bidi.stripHtmlIfNeeded_ = function(str, opt_isStripNeeded) {
  return opt_isStripNeeded ? str.replace(goog.i18n.bidi.htmlSkipReg_, '') : str;
};


/**
 * Regular expression to check for RTL characters, BMP and high surrogate.
 * @type {RegExp}
 * @private
 */
goog.i18n.bidi.rtlCharReg_ = new RegExp('[' + goog.i18n.bidi.rtlChars_ + ']');


/**
 * Regular expression to check for LTR characters.
 * @type {RegExp}
 * @private
 */
goog.i18n.bidi.ltrCharReg_ = new RegExp('[' + goog.i18n.bidi.ltrChars_ + ']');


/**
 * Test whether the given string has any RTL characters in it.
 * @param {string} str The given string that need to be tested.
 * @param {boolean=} opt_isHtml Whether str is HTML / HTML-escaped.
 *     Default: false.
 * @return {boolean} Whether the string contains RTL characters.
 */
goog.i18n.bidi.hasAnyRtl = function(str, opt_isHtml) {
  return goog.i18n.bidi.rtlCharReg_.test(
      goog.i18n.bidi.stripHtmlIfNeeded_(str, opt_isHtml));
};


/**
 * Test whether the given string has any RTL characters in it.
 * @param {string} str The given string that need to be tested.
 * @return {boolean} Whether the string contains RTL characters.
 * @deprecated Use hasAnyRtl.
 */
goog.i18n.bidi.hasRtlChar = goog.i18n.bidi.hasAnyRtl;


/**
 * Test whether the given string has any LTR characters in it.
 * @param {string} str The given string that need to be tested.
 * @param {boolean=} opt_isHtml Whether str is HTML / HTML-escaped.
 *     Default: false.
 * @return {boolean} Whether the string contains LTR characters.
 */
goog.i18n.bidi.hasAnyLtr = function(str, opt_isHtml) {
  return goog.i18n.bidi.ltrCharReg_.test(
      goog.i18n.bidi.stripHtmlIfNeeded_(str, opt_isHtml));
};


/**
 * Regular expression pattern to check if the first character in the string
 * is LTR.
 * @type {RegExp}
 * @private
 */
goog.i18n.bidi.ltrRe_ = new RegExp('^[' + goog.i18n.bidi.ltrChars_ + ']');


/**
 * Regular expression pattern to check if the first character in the string
 * is RTL.
 * @type {RegExp}
 * @private
 */
goog.i18n.bidi.rtlRe_ = new RegExp('^[' + goog.i18n.bidi.rtlChars_ + ']');


/**
 * Check if the first character in the string is RTL or not.
 * @param {string} str The given string that need to be tested.
 * @return {boolean} Whether the first character in str is an RTL char.
 */
goog.i18n.bidi.isRtlChar = function(str) {
  return goog.i18n.bidi.rtlRe_.test(str);
};


/**
 * Check if the first character in the string is LTR or not.
 * @param {string} str The given string that need to be tested.
 * @return {boolean} Whether the first character in str is an LTR char.
 */
goog.i18n.bidi.isLtrChar = function(str) {
  return goog.i18n.bidi.ltrRe_.test(str);
};


/**
 * Check if the first character in the string is neutral or not.
 * @param {string} str The given string that need to be tested.
 * @return {boolean} Whether the first character in str is a neutral char.
 */
goog.i18n.bidi.isNeutralChar = function(str) {
  return !goog.i18n.bidi.isLtrChar(str) && !goog.i18n.bidi.isRtlChar(str);
};


/**
 * Regular expressions to check if a piece of text is of LTR directionality
 * on first character with strong directionality.
 * @type {RegExp}
 * @private
 */
goog.i18n.bidi.ltrDirCheckRe_ = new RegExp(
    '^[^' + goog.i18n.bidi.rtlChars_ + ']*[' + goog.i18n.bidi.ltrChars_ + ']');


/**
 * Regular expressions to check if a piece of text is of RTL directionality
 * on first character with strong directionality.
 * @type {RegExp}
 * @private
 */
goog.i18n.bidi.rtlDirCheckRe_ = new RegExp(
    '^[^' + goog.i18n.bidi.ltrChars_ + ']*[' + goog.i18n.bidi.rtlChars_ + ']');


/**
 * Check whether the first strongly directional character (if any) is RTL.
 * @param {string} str String being checked.
 * @param {boolean=} opt_isHtml Whether str is HTML / HTML-escaped.
 *     Default: false.
 * @return {boolean} Whether RTL directionality is detected using the first
 *     strongly-directional character method.
 */
goog.i18n.bidi.startsWithRtl = function(str, opt_isHtml) {
  return goog.i18n.bidi.rtlDirCheckRe_.test(
      goog.i18n.bidi.stripHtmlIfNeeded_(str, opt_isHtml));
};


/**
 * Check whether the first strongly directional character (if any) is RTL.
 * @param {string} str String being checked.
 * @param {boolean=} opt_isHtml Whether str is HTML / HTML-escaped.
 *     Default: false.
 * @return {boolean} Whether RTL directionality is detected using the first
 *     strongly-directional character method.
 * @deprecated Use startsWithRtl.
 */
goog.i18n.bidi.isRtlText = goog.i18n.bidi.startsWithRtl;


/**
 * Check whether the first strongly directional character (if any) is LTR.
 * @param {string} str String being checked.
 * @param {boolean=} opt_isHtml Whether str is HTML / HTML-escaped.
 *     Default: false.
 * @return {boolean} Whether LTR directionality is detected using the first
 *     strongly-directional character method.
 */
goog.i18n.bidi.startsWithLtr = function(str, opt_isHtml) {
  return goog.i18n.bidi.ltrDirCheckRe_.test(
      goog.i18n.bidi.stripHtmlIfNeeded_(str, opt_isHtml));
};


/**
 * Check whether the first strongly directional character (if any) is LTR.
 * @param {string} str String being checked.
 * @param {boolean=} opt_isHtml Whether str is HTML / HTML-escaped.
 *     Default: false.
 * @return {boolean} Whether LTR directionality is detected using the first
 *     strongly-directional character method.
 * @deprecated Use startsWithLtr.
 */
goog.i18n.bidi.isLtrText = goog.i18n.bidi.startsWithLtr;


/**
 * Regular expression to check if a string looks like something that must
 * always be LTR even in RTL text, e.g. a URL. When estimating the
 * directionality of text containing these, we treat these as weakly LTR,
 * like numbers.
 * @type {RegExp}
 * @private
 */
goog.i18n.bidi.isRequiredLtrRe_ = /^http:\/\/.*/;


/**
 * Check whether the input string either contains no strongly directional
 * characters or looks like a url.
 * @param {string} str String being checked.
 * @param {boolean=} opt_isHtml Whether str is HTML / HTML-escaped.
 *     Default: false.
 * @return {boolean} Whether neutral directionality is detected.
 */
goog.i18n.bidi.isNeutralText = function(str, opt_isHtml) {
  str = goog.i18n.bidi.stripHtmlIfNeeded_(str, opt_isHtml);
  return goog.i18n.bidi.isRequiredLtrRe_.test(str) ||
      !goog.i18n.bidi.hasAnyLtr(str) && !goog.i18n.bidi.hasAnyRtl(str);
};


/**
 * Regular expressions to check if the last strongly-directional character in a
 * piece of text is LTR.
 * @type {RegExp}
 * @private
 */
goog.i18n.bidi.ltrExitDirCheckRe_ = new RegExp(
    '[' + goog.i18n.bidi.ltrChars_ + ']' +
    '[^' + goog.i18n.bidi.rtlChars_ + ']*$');


/**
 * Regular expressions to check if the last strongly-directional character in a
 * piece of text is RTL.
 * @type {RegExp}
 * @private
 */
goog.i18n.bidi.rtlExitDirCheckRe_ = new RegExp(
    '[' + goog.i18n.bidi.rtlChars_ + ']' +
    '[^' + goog.i18n.bidi.ltrChars_ + ']*$');


/**
 * Check if the exit directionality a piece of text is LTR, i.e. if the last
 * strongly-directional character in the string is LTR.
 * @param {string} str String being checked.
 * @param {boolean=} opt_isHtml Whether str is HTML / HTML-escaped.
 *     Default: false.
 * @return {boolean} Whether LTR exit directionality was detected.
 */
goog.i18n.bidi.endsWithLtr = function(str, opt_isHtml) {
  return goog.i18n.bidi.ltrExitDirCheckRe_.test(
      goog.i18n.bidi.stripHtmlIfNeeded_(str, opt_isHtml));
};


/**
 * Check if the exit directionality a piece of text is LTR, i.e. if the last
 * strongly-directional character in the string is LTR.
 * @param {string} str String being checked.
 * @param {boolean=} opt_isHtml Whether str is HTML / HTML-escaped.
 *     Default: false.
 * @return {boolean} Whether LTR exit directionality was detected.
 * @deprecated Use endsWithLtr.
 */
goog.i18n.bidi.isLtrExitText = goog.i18n.bidi.endsWithLtr;


/**
 * Check if the exit directionality a piece of text is RTL, i.e. if the last
 * strongly-directional character in the string is RTL.
 * @param {string} str String being checked.
 * @param {boolean=} opt_isHtml Whether str is HTML / HTML-escaped.
 *     Default: false.
 * @return {boolean} Whether RTL exit directionality was detected.
 */
goog.i18n.bidi.endsWithRtl = function(str, opt_isHtml) {
  return goog.i18n.bidi.rtlExitDirCheckRe_.test(
      goog.i18n.bidi.stripHtmlIfNeeded_(str, opt_isHtml));
};


/**
 * Check if the exit directionality a piece of text is RTL, i.e. if the last
 * strongly-directional character in the string is RTL.
 * @param {string} str String being checked.
 * @param {boolean=} opt_isHtml Whether str is HTML / HTML-escaped.
 *     Default: false.
 * @return {boolean} Whether RTL exit directionality was detected.
 * @deprecated Use endsWithRtl.
 */
goog.i18n.bidi.isRtlExitText = goog.i18n.bidi.endsWithRtl;


/**
 * A regular expression for matching right-to-left language codes.
 * See {@link #isRtlLanguage} for the design.
 * Note that not all RTL scripts are included.
 * @type {!RegExp}
 * @private
 */
goog.i18n.bidi.rtlLocalesRe_ = new RegExp(
    '^(ar|ckb|dv|he|iw|fa|nqo|ps|sd|ug|ur|yi|' +
        '.*[-_](Adlm|Arab|Hebr|Nkoo|Rohg|Thaa))' +
        '(?!.*[-_](Latn|Cyrl)($|-|_))($|-|_)',
    'i');


/**
 * Check if a BCP 47 / III language code indicates an RTL language, i.e. either:
 * - a language code explicitly specifying one of the right-to-left scripts,
 *   e.g. "az-Arab", or<p>
 * - a language code specifying one of the languages normally written in a
 *   right-to-left script, e.g. "fa" (Farsi), except ones explicitly specifying
 *   Latin or Cyrillic script (which are the usual LTR alternatives).<p>
 * The list of right-to-left scripts appears in the 100-199 range in
 * http://www.unicode.org/iso15924/iso15924-num.html, of which Arabic and
 * Hebrew are by far the most widely used. We also recognize Thaana, and N'Ko,
 * which also have significant modern usage. Adlam and Rohingya
 * scripts are now included since they can be expected to be used in the
 * future. The rest (Syriac, Samaritan, Mandaic, etc.) seem to have extremely
 * limited or no modern usage and are not recognized to save on code size. The
 * languages usually written in a right-to-left script are taken as those with
 * Suppress-Script: Hebr|Arab|Thaa|Nkoo|Adlm|Rohg in
 * http://www.iana.org/assignments/language-subtag-registry,
 * as well as Central (or Sorani) Kurdish (ckb), Sindhi (sd) and Uyghur (ug).
 * Other subtags of the language code, e.g. regions like EG (Egypt), are
 * ignored.
 * @param {string} lang BCP 47 (a.k.a III) language code.
 * @return {boolean} Whether the language code is an RTL language.
 */
goog.i18n.bidi.isRtlLanguage = function(lang) {
  return goog.i18n.bidi.rtlLocalesRe_.test(lang);
};


/**
 * Regular expression for bracket guard replacement in text.
 * @type {RegExp}
 * @private
 */
goog.i18n.bidi.bracketGuardTextRe_ =
    /(\(.*?\)+)|(\[.*?\]+)|(\{.*?\}+)|(<.*?>+)/g;


/**
 * Apply bracket guard using LRM and RLM. This is to address the problem of
 * messy bracket display frequently happens in RTL layout.
 * This function works for plain text, not for HTML. In HTML, the opening
 * bracket might be in a different context than the closing bracket (such as
 * an attribute value).
 * @param {string} s The string that need to be processed.
 * @param {boolean=} opt_isRtlContext specifies default direction (usually
 *     direction of the UI).
 * @return {string} The processed string, with all bracket guarded.
 */
goog.i18n.bidi.guardBracketInText = function(s, opt_isRtlContext) {
  const useRtl = opt_isRtlContext === undefined ? goog.i18n.bidi.hasAnyRtl(s) :
                                                  opt_isRtlContext;
  const mark = useRtl ? goog.i18n.bidi.Format.RLM : goog.i18n.bidi.Format.LRM;
  return s.replace(goog.i18n.bidi.bracketGuardTextRe_, mark + '$&' + mark);
};


/**
 * Enforce the html snippet in RTL directionality regardless of overall context.
 * If the html piece was enclosed by tag, dir will be applied to existing
 * tag, otherwise a span tag will be added as wrapper. For this reason, if
 * html snippet starts with a tag, this tag must enclose the whole piece. If
 * the tag already has a dir specified, this new one will override existing
 * one in behavior (tested on FF and IE).
 * @param {string} html The string that need to be processed.
 * @return {string} The processed string, with directionality enforced to RTL.
 */
goog.i18n.bidi.enforceRtlInHtml = function(html) {
  if (html.charAt(0) == '<') {
    return html.replace(/<\w+/, '$& dir=rtl');
  }
  // '\n' is important for FF so that it won't incorrectly merge span groups
  return '\n<span dir=rtl>' + html + '</span>';
};


/**
 * Enforce RTL on both end of the given text piece using unicode BiDi formatting
 * characters RLE and PDF.
 * @param {string} text The piece of text that need to be wrapped.
 * @return {string} The wrapped string after process.
 */
goog.i18n.bidi.enforceRtlInText = function(text) {
  return goog.i18n.bidi.Format.RLE + text + goog.i18n.bidi.Format.PDF;
};


/**
 * Enforce the html snippet in RTL directionality regardless or overall context.
 * If the html piece was enclosed by tag, dir will be applied to existing
 * tag, otherwise a span tag will be added as wrapper. For this reason, if
 * html snippet starts with a tag, this tag must enclose the whole piece. If
 * the tag already has a dir specified, this new one will override existing
 * one in behavior (tested on FF and IE).
 * @param {string} html The string that need to be processed.
 * @return {string} The processed string, with directionality enforced to RTL.
 */
goog.i18n.bidi.enforceLtrInHtml = function(html) {
  if (html.charAt(0) == '<') {
    return html.replace(/<\w+/, '$& dir=ltr');
  }
  // '\n' is important for FF so that it won't incorrectly merge span groups
  return '\n<span dir=ltr>' + html + '</span>';
};


/**
 * Enforce LTR on both end of the given text piece using unicode BiDi formatting
 * characters LRE and PDF.
 * @param {string} text The piece of text that need to be wrapped.
 * @return {string} The wrapped string after process.
 */
goog.i18n.bidi.enforceLtrInText = function(text) {
  return goog.i18n.bidi.Format.LRE + text + goog.i18n.bidi.Format.PDF;
};


/**
 * Regular expression to find dimensions such as "padding: .3 0.4ex 5px 6;"
 * @type {RegExp}
 * @private
 */
goog.i18n.bidi.dimensionsRe_ =
    /:\s*([.\d][.\w]*)\s+([.\d][.\w]*)\s+([.\d][.\w]*)\s+([.\d][.\w]*)/g;


/**
 * Regular expression for left.
 * @type {RegExp}
 * @private
 */
goog.i18n.bidi.leftRe_ = /left/gi;


/**
 * Regular expression for right.
 * @type {RegExp}
 * @private
 */
goog.i18n.bidi.rightRe_ = /right/gi;


/**
 * Placeholder regular expression for swapping.
 * @type {RegExp}
 * @private
 */
goog.i18n.bidi.tempRe_ = /%%%%/g;


/**
 * Swap location parameters and 'left'/'right' in CSS specification. The
 * processed string will be suited for RTL layout. Though this function can
 * cover most cases, there are always exceptions. It is suggested to put
 * those exceptions in separate group of CSS string.
 * @param {string} cssStr CSS spefication string.
 * @return {string} Processed CSS specification string.
 */
goog.i18n.bidi.mirrorCSS = function(cssStr) {
  return cssStr
      .
      // reverse dimensions
      replace(goog.i18n.bidi.dimensionsRe_, ':$1 $4 $3 $2')
      .replace(goog.i18n.bidi.leftRe_, '%%%%')
      .  // swap left and right
      replace(goog.i18n.bidi.rightRe_, goog.i18n.bidi.LEFT)
      .replace(goog.i18n.bidi.tempRe_, goog.i18n.bidi.RIGHT);
};


/**
 * Regular expression for hebrew double quote substitution, finding quote
 * directly after hebrew characters.
 * @type {RegExp}
 * @private
 */
goog.i18n.bidi.doubleQuoteSubstituteRe_ = /([\u0591-\u05f2])"/g;


/**
 * Regular expression for hebrew single quote substitution, finding quote
 * directly after hebrew characters.
 * @type {RegExp}
 * @private
 */
goog.i18n.bidi.singleQuoteSubstituteRe_ = /([\u0591-\u05f2])'/g;


/**
 * Replace the double and single quote directly after a Hebrew character with
 * GERESH and GERSHAYIM. In such case, most likely that's user intention.
 * @param {string} str String that need to be processed.
 * @return {string} Processed string with double/single quote replaced.
 */
goog.i18n.bidi.normalizeHebrewQuote = function(str) {
  return str.replace(goog.i18n.bidi.doubleQuoteSubstituteRe_, '$1\u05f4')
      .replace(goog.i18n.bidi.singleQuoteSubstituteRe_, '$1\u05f3');
};


/**
 * Regular expression to split a string into "words" for directionality
 * estimation based on relative word counts.
 * @type {RegExp}
 * @private
 */
goog.i18n.bidi.wordSeparatorRe_ = /\s+/;


/**
 * Regular expression to check if a string contains any numerals. Used to
 * differentiate between completely neutral strings and those containing
 * numbers, which are weakly LTR.
 *
 * Native Arabic digits (\u0660 - \u0669) are not included because although they
 * do flow left-to-right inside a number, this is the case even if the  overall
 * directionality is RTL, and a mathematical expression using these digits is
 * supposed to flow right-to-left overall, including unary plus and minus
 * appearing to the right of a number, and this does depend on the overall
 * directionality being RTL. The digits used in Farsi (\u06F0 - \u06F9), on the
 * other hand, are included, since Farsi math (including unary plus and minus)
 * does flow left-to-right.
 * TODO: Consider other systems of digits, e.g., Adlam.
 *
 * @type {RegExp}
 * @private
 */
goog.i18n.bidi.hasNumeralsRe_ = /[\d\u06f0-\u06f9]/;


/**
 * This constant controls threshold of RTL directionality.
 * @type {number}
 * @private
 */
goog.i18n.bidi.rtlDetectionThreshold_ = 0.40;


/**
 * Estimates the directionality of a string based on relative word counts.
 * If the number of RTL words is above a certain percentage of the total number
 * of strongly directional words, returns RTL.
 * Otherwise, if any words are strongly or weakly LTR, returns LTR.
 * Otherwise, returns UNKNOWN, which is used to mean "neutral".
 * Numbers are counted as weakly LTR.
 * @param {string} str The string to be checked.
 * @param {boolean=} opt_isHtml Whether str is HTML / HTML-escaped.
 *     Default: false.
 * @return {goog.i18n.bidi.Dir} Estimated overall directionality of `str`.
 */
goog.i18n.bidi.estimateDirection = function(str, opt_isHtml) {
  let rtlCount = 0;
  let totalCount = 0;
  let hasWeaklyLtr = false;
  const tokens = goog.i18n.bidi.stripHtmlIfNeeded_(str, opt_isHtml)
                     .split(goog.i18n.bidi.wordSeparatorRe_);
  for (let i = 0; i < tokens.length; i++) {
    const token = tokens[i];
    if (goog.i18n.bidi.startsWithRtl(token)) {
      rtlCount++;
      totalCount++;
    } else if (goog.i18n.bidi.isRequiredLtrRe_.test(token)) {
      hasWeaklyLtr = true;
    } else if (goog.i18n.bidi.hasAnyLtr(token)) {
      totalCount++;
    } else if (goog.i18n.bidi.hasNumeralsRe_.test(token)) {
      hasWeaklyLtr = true;
    }
  }

  return totalCount == 0 ?
      (hasWeaklyLtr ? goog.i18n.bidi.Dir.LTR : goog.i18n.bidi.Dir.NEUTRAL) :
      (rtlCount / totalCount > goog.i18n.bidi.rtlDetectionThreshold_ ?
           goog.i18n.bidi.Dir.RTL :
           goog.i18n.bidi.Dir.LTR);
};


/**
 * Check the directionality of a piece of text, return true if the piece of
 * text should be laid out in RTL direction.
 * @param {string} str The piece of text that need to be detected.
 * @param {boolean=} opt_isHtml Whether str is HTML / HTML-escaped.
 *     Default: false.
 * @return {boolean} Whether this piece of text should be laid out in RTL.
 */
goog.i18n.bidi.detectRtlDirectionality = function(str, opt_isHtml) {
  return goog.i18n.bidi.estimateDirection(str, opt_isHtml) ==
      goog.i18n.bidi.Dir.RTL;
};


/**
 * Sets text input element's directionality and text alignment based on a
 * given directionality. Does nothing if the given directionality is unknown or
 * neutral.
 * @param {Element} element Input field element to set directionality to.
 * @param {goog.i18n.bidi.Dir|number|boolean|null} dir Desired directionality,
 *     given in one of the following formats:
 *     1. A goog.i18n.bidi.Dir constant.
 *     2. A number (positive = LRT, negative = RTL, 0 = neutral).
 *     3. A boolean (true = RTL, false = LTR).
 *     4. A null for unknown directionality.
 */
goog.i18n.bidi.setElementDirAndAlign = function(element, dir) {
  if (element) {
    const htmlElement = /** @type {!HTMLElement} */ (element);
    dir = goog.i18n.bidi.toDir(dir);
    if (dir) {
      htmlElement.style.textAlign = dir == goog.i18n.bidi.Dir.RTL ?
          goog.i18n.bidi.RIGHT :
          goog.i18n.bidi.LEFT;
      htmlElement.dir = dir == goog.i18n.bidi.Dir.RTL ? 'rtl' : 'ltr';
    }
  }
};


/**
 * Sets element dir based on estimated directionality of the given text.
 * @param {!Element} element
 * @param {string} text
 */
goog.i18n.bidi.setElementDirByTextDirectionality = function(element, text) {
  const htmlElement = /** @type {!HTMLElement} */ (element);
  switch (goog.i18n.bidi.estimateDirection(text)) {
    case (goog.i18n.bidi.Dir.LTR):
      htmlElement.dir = 'ltr';
      break;
    case (goog.i18n.bidi.Dir.RTL):
      htmlElement.dir = 'rtl';
      break;
    default:
      // Default for no direction, inherit from document.
      htmlElement.removeAttribute('dir');
  }
};



/**
 * Strings that have an (optional) known direction.
 *
 * Implementations of this interface are string-like objects that carry an
 * attached direction, if known.
 * @interface
 */
goog.i18n.bidi.DirectionalString = function() {};


/**
 * Interface marker of the DirectionalString interface.
 *
 * This property can be used to determine at runtime whether or not an object
 * implements this interface.  All implementations of this interface set this
 * property to `true`.
 * @type {boolean}
 */
goog.i18n.bidi.DirectionalString.prototype
    .implementsGoogI18nBidiDirectionalString;


/**
 * Retrieves this object's known direction (if any).
 * @return {?goog.i18n.bidi.Dir} The known direction. Null if unknown.
 */
goog.i18n.bidi.DirectionalString.prototype.getDirection;

//third_party/javascript/closure/dom/htmlelement.js
/**
 * @license
 * Copyright The Closure Library Authors.
 * SPDX-License-Identifier: Apache-2.0
 */

goog.provide('goog.dom.HtmlElement');



/**
 * This subclass of HTMLElement is used when only a HTMLElement is possible and
 * not any of its subclasses. Normally, a type can refer to an instance of
 * itself or an instance of any subtype. More concretely, if HTMLElement is used
 * then the compiler must assume that it might still be e.g. HTMLScriptElement.
 * With this, the type check knows that it couldn't be any special element.
 *
 * @constructor
 * @extends {HTMLElement}
 */
goog.dom.HtmlElement = function() {};

//third_party/javascript/closure/dom/tagname.js
/**
 * @license
 * Copyright The Closure Library Authors.
 * SPDX-License-Identifier: Apache-2.0
 */

/**
 * @fileoverview Defines the goog.dom.TagName class. Its constants enumerate
 * all HTML tag names specified in either the W3C HTML 4.01 index of elements
 * or the HTML5.1 specification.
 *
 * References:
 * https://www.w3.org/TR/html401/index/elements.html
 * https://www.w3.org/TR/html51/dom.html#elements
 */
goog.provide('goog.dom.TagName');

goog.require('goog.dom.HtmlElement');


/**
 * A tag name with the type of the element stored in the generic.
 * @param {string} tagName
 * @constructor
 * @template T
 */
goog.dom.TagName = function(tagName) {
  /** @private {string} */
  this.tagName_ = tagName;
};


/**
 * Returns the tag name.
 * @return {string}
 * @override
 */
goog.dom.TagName.prototype.toString = function() {
  return this.tagName_;
};


// Closure Compiler unconditionally converts the following constants to their
// string value (goog.dom.TagName.A -> 'A'). These are the consequences:
// 1. Don't add any members or static members to goog.dom.TagName as they
//    couldn't be accessed after this optimization.
// 2. Keep the constant name and its string value the same:
//    goog.dom.TagName.X = new goog.dom.TagName('Y');
//    is converted to 'X', not 'Y'.


/** @type {!goog.dom.TagName<!HTMLAnchorElement>} */
goog.dom.TagName.A = new goog.dom.TagName('A');


/** @type {!goog.dom.TagName<!goog.dom.HtmlElement>} */
goog.dom.TagName.ABBR = new goog.dom.TagName('ABBR');


/** @type {!goog.dom.TagName<!goog.dom.HtmlElement>} */
goog.dom.TagName.ACRONYM = new goog.dom.TagName('ACRONYM');


/** @type {!goog.dom.TagName<!goog.dom.HtmlElement>} */
goog.dom.TagName.ADDRESS = new goog.dom.TagName('ADDRESS');


/** @type {!goog.dom.TagName<!HTMLAppletElement>} */
goog.dom.TagName.APPLET = new goog.dom.TagName('APPLET');


/** @type {!goog.dom.TagName<!HTMLAreaElement>} */
goog.dom.TagName.AREA = new goog.dom.TagName('AREA');


/** @type {!goog.dom.TagName<!goog.dom.HtmlElement>} */
goog.dom.TagName.ARTICLE = new goog.dom.TagName('ARTICLE');


/** @type {!goog.dom.TagName<!goog.dom.HtmlElement>} */
goog.dom.TagName.ASIDE = new goog.dom.TagName('ASIDE');


/** @type {!goog.dom.TagName<!HTMLAudioElement>} */
goog.dom.TagName.AUDIO = new goog.dom.TagName('AUDIO');


/** @type {!goog.dom.TagName<!goog.dom.HtmlElement>} */
goog.dom.TagName.B = new goog.dom.TagName('B');


/** @type {!goog.dom.TagName<!HTMLBaseElement>} */
goog.dom.TagName.BASE = new goog.dom.TagName('BASE');


/** @type {!goog.dom.TagName<!HTMLBaseFontElement>} */
goog.dom.TagName.BASEFONT = new goog.dom.TagName('BASEFONT');


/** @type {!goog.dom.TagName<!goog.dom.HtmlElement>} */
goog.dom.TagName.BDI = new goog.dom.TagName('BDI');


/** @type {!goog.dom.TagName<!goog.dom.HtmlElement>} */
goog.dom.TagName.BDO = new goog.dom.TagName('BDO');


/** @type {!goog.dom.TagName<!goog.dom.HtmlElement>} */
goog.dom.TagName.BIG = new goog.dom.TagName('BIG');


/** @type {!goog.dom.TagName<!HTMLQuoteElement>} */
goog.dom.TagName.BLOCKQUOTE = new goog.dom.TagName('BLOCKQUOTE');


/** @type {!goog.dom.TagName<!HTMLBodyElement>} */
goog.dom.TagName.BODY = new goog.dom.TagName('BODY');


/** @type {!goog.dom.TagName<!HTMLBRElement>} */
goog.dom.TagName.BR = new goog.dom.TagName('BR');


/** @type {!goog.dom.TagName<!HTMLButtonElement>} */
goog.dom.TagName.BUTTON = new goog.dom.TagName('BUTTON');


/** @type {!goog.dom.TagName<!HTMLCanvasElement>} */
goog.dom.TagName.CANVAS = new goog.dom.TagName('CANVAS');


/** @type {!goog.dom.TagName<!HTMLTableCaptionElement>} */
goog.dom.TagName.CAPTION = new goog.dom.TagName('CAPTION');


/** @type {!goog.dom.TagName<!goog.dom.HtmlElement>} */
goog.dom.TagName.CENTER = new goog.dom.TagName('CENTER');


/** @type {!goog.dom.TagName<!goog.dom.HtmlElement>} */
goog.dom.TagName.CITE = new goog.dom.TagName('CITE');


/** @type {!goog.dom.TagName<!goog.dom.HtmlElement>} */
goog.dom.TagName.CODE = new goog.dom.TagName('CODE');


/** @type {!goog.dom.TagName<!HTMLTableColElement>} */
goog.dom.TagName.COL = new goog.dom.TagName('COL');


/** @type {!goog.dom.TagName<!HTMLTableColElement>} */
goog.dom.TagName.COLGROUP = new goog.dom.TagName('COLGROUP');


/** @type {!goog.dom.TagName<!goog.dom.HtmlElement>} */
goog.dom.TagName.COMMAND = new goog.dom.TagName('COMMAND');


/** @type {!goog.dom.TagName<!goog.dom.HtmlElement>} */
goog.dom.TagName.DATA = new goog.dom.TagName('DATA');


/** @type {!goog.dom.TagName<!HTMLDataListElement>} */
goog.dom.TagName.DATALIST = new goog.dom.TagName('DATALIST');


/** @type {!goog.dom.TagName<!goog.dom.HtmlElement>} */
goog.dom.TagName.DD = new goog.dom.TagName('DD');


/** @type {!goog.dom.TagName<!HTMLModElement>} */
goog.dom.TagName.DEL = new goog.dom.TagName('DEL');


/** @type {!goog.dom.TagName<!HTMLDetailsElement>} */
goog.dom.TagName.DETAILS = new goog.dom.TagName('DETAILS');


/** @type {!goog.dom.TagName<!goog.dom.HtmlElement>} */
goog.dom.TagName.DFN = new goog.dom.TagName('DFN');


/** @type {!goog.dom.TagName<!HTMLDialogElement>} */
goog.dom.TagName.DIALOG = new goog.dom.TagName('DIALOG');


/** @type {!goog.dom.TagName<!HTMLDirectoryElement>} */
goog.dom.TagName.DIR = new goog.dom.TagName('DIR');


/** @type {!goog.dom.TagName<!HTMLDivElement>} */
goog.dom.TagName.DIV = new goog.dom.TagName('DIV');


/** @type {!goog.dom.TagName<!HTMLDListElement>} */
goog.dom.TagName.DL = new goog.dom.TagName('DL');


/** @type {!goog.dom.TagName<!goog.dom.HtmlElement>} */
goog.dom.TagName.DT = new goog.dom.TagName('DT');


/** @type {!goog.dom.TagName<!goog.dom.HtmlElement>} */
goog.dom.TagName.EM = new goog.dom.TagName('EM');


/** @type {!goog.dom.TagName<!HTMLEmbedElement>} */
goog.dom.TagName.EMBED = new goog.dom.TagName('EMBED');


/** @type {!goog.dom.TagName<!HTMLFieldSetElement>} */
goog.dom.TagName.FIELDSET = new goog.dom.TagName('FIELDSET');


/** @type {!goog.dom.TagName<!goog.dom.HtmlElement>} */
goog.dom.TagName.FIGCAPTION = new goog.dom.TagName('FIGCAPTION');


/** @type {!goog.dom.TagName<!goog.dom.HtmlElement>} */
goog.dom.TagName.FIGURE = new goog.dom.TagName('FIGURE');


/** @type {!goog.dom.TagName<!HTMLFontElement>} */
goog.dom.TagName.FONT = new goog.dom.TagName('FONT');


/** @type {!goog.dom.TagName<!goog.dom.HtmlElement>} */
goog.dom.TagName.FOOTER = new goog.dom.TagName('FOOTER');


/** @type {!goog.dom.TagName<!HTMLFormElement>} */
goog.dom.TagName.FORM = new goog.dom.TagName('FORM');


/** @type {!goog.dom.TagName<!HTMLFrameElement>} */
goog.dom.TagName.FRAME = new goog.dom.TagName('FRAME');


/** @type {!goog.dom.TagName<!HTMLFrameSetElement>} */
goog.dom.TagName.FRAMESET = new goog.dom.TagName('FRAMESET');


/** @type {!goog.dom.TagName<!HTMLHeadingElement>} */
goog.dom.TagName.H1 = new goog.dom.TagName('H1');


/** @type {!goog.dom.TagName<!HTMLHeadingElement>} */
goog.dom.TagName.H2 = new goog.dom.TagName('H2');


/** @type {!goog.dom.TagName<!HTMLHeadingElement>} */
goog.dom.TagName.H3 = new goog.dom.TagName('H3');


/** @type {!goog.dom.TagName<!HTMLHeadingElement>} */
goog.dom.TagName.H4 = new goog.dom.TagName('H4');


/** @type {!goog.dom.TagName<!HTMLHeadingElement>} */
goog.dom.TagName.H5 = new goog.dom.TagName('H5');


/** @type {!goog.dom.TagName<!HTMLHeadingElement>} */
goog.dom.TagName.H6 = new goog.dom.TagName('H6');


/** @type {!goog.dom.TagName<!HTMLHeadElement>} */
goog.dom.TagName.HEAD = new goog.dom.TagName('HEAD');


/** @type {!goog.dom.TagName<!goog.dom.HtmlElement>} */
goog.dom.TagName.HEADER = new goog.dom.TagName('HEADER');


/** @type {!goog.dom.TagName<!goog.dom.HtmlElement>} */
goog.dom.TagName.HGROUP = new goog.dom.TagName('HGROUP');


/** @type {!goog.dom.TagName<!HTMLHRElement>} */
goog.dom.TagName.HR = new goog.dom.TagName('HR');


/** @type {!goog.dom.TagName<!HTMLHtmlElement>} */
goog.dom.TagName.HTML = new goog.dom.TagName('HTML');


/** @type {!goog.dom.TagName<!goog.dom.HtmlElement>} */
goog.dom.TagName.I = new goog.dom.TagName('I');


/** @type {!goog.dom.TagName<!HTMLIFrameElement>} */
goog.dom.TagName.IFRAME = new goog.dom.TagName('IFRAME');


/** @type {!goog.dom.TagName<!HTMLImageElement>} */
goog.dom.TagName.IMG = new goog.dom.TagName('IMG');


/** @type {!goog.dom.TagName<!HTMLInputElement>} */
goog.dom.TagName.INPUT = new goog.dom.TagName('INPUT');


/** @type {!goog.dom.TagName<!HTMLModElement>} */
goog.dom.TagName.INS = new goog.dom.TagName('INS');


/** @type {!goog.dom.TagName<!HTMLIsIndexElement>} */
goog.dom.TagName.ISINDEX = new goog.dom.TagName('ISINDEX');


/** @type {!goog.dom.TagName<!goog.dom.HtmlElement>} */
goog.dom.TagName.KBD = new goog.dom.TagName('KBD');


// HTMLKeygenElement is deprecated.
/** @type {!goog.dom.TagName<!goog.dom.HtmlElement>} */
goog.dom.TagName.KEYGEN = new goog.dom.TagName('KEYGEN');


/** @type {!goog.dom.TagName<!HTMLLabelElement>} */
goog.dom.TagName.LABEL = new goog.dom.TagName('LABEL');


/** @type {!goog.dom.TagName<!HTMLLegendElement>} */
goog.dom.TagName.LEGEND = new goog.dom.TagName('LEGEND');


/** @type {!goog.dom.TagName<!HTMLLIElement>} */
goog.dom.TagName.LI = new goog.dom.TagName('LI');


/** @type {!goog.dom.TagName<!HTMLLinkElement>} */
goog.dom.TagName.LINK = new goog.dom.TagName('LINK');


/** @type {!goog.dom.TagName<!goog.dom.HtmlElement>} */
goog.dom.TagName.MAIN = new goog.dom.TagName('MAIN');


/** @type {!goog.dom.TagName<!HTMLMapElement>} */
goog.dom.TagName.MAP = new goog.dom.TagName('MAP');


/** @type {!goog.dom.TagName<!goog.dom.HtmlElement>} */
goog.dom.TagName.MARK = new goog.dom.TagName('MARK');


/** @type {!goog.dom.TagName<!goog.dom.HtmlElement>} */
goog.dom.TagName.MATH = new goog.dom.TagName('MATH');


/** @type {!goog.dom.TagName<!HTMLMenuElement>} */
goog.dom.TagName.MENU = new goog.dom.TagName('MENU');


/** @type {!goog.dom.TagName<!HTMLMenuItemElement>} */
goog.dom.TagName.MENUITEM = new goog.dom.TagName('MENUITEM');


/** @type {!goog.dom.TagName<!HTMLMetaElement>} */
goog.dom.TagName.META = new goog.dom.TagName('META');


/** @type {!goog.dom.TagName<!HTMLMeterElement>} */
goog.dom.TagName.METER = new goog.dom.TagName('METER');


/** @type {!goog.dom.TagName<!goog.dom.HtmlElement>} */
goog.dom.TagName.NAV = new goog.dom.TagName('NAV');


/** @type {!goog.dom.TagName<!goog.dom.HtmlElement>} */
goog.dom.TagName.NOFRAMES = new goog.dom.TagName('NOFRAMES');


/** @type {!goog.dom.TagName<!goog.dom.HtmlElement>} */
goog.dom.TagName.NOSCRIPT = new goog.dom.TagName('NOSCRIPT');


/** @type {!goog.dom.TagName<!HTMLObjectElement>} */
goog.dom.TagName.OBJECT = new goog.dom.TagName('OBJECT');


/** @type {!goog.dom.TagName<!HTMLOListElement>} */
goog.dom.TagName.OL = new goog.dom.TagName('OL');


/** @type {!goog.dom.TagName<!HTMLOptGroupElement>} */
goog.dom.TagName.OPTGROUP = new goog.dom.TagName('OPTGROUP');


/** @type {!goog.dom.TagName<!HTMLOptionElement>} */
goog.dom.TagName.OPTION = new goog.dom.TagName('OPTION');


/** @type {!goog.dom.TagName<!HTMLOutputElement>} */
goog.dom.TagName.OUTPUT = new goog.dom.TagName('OUTPUT');


/** @type {!goog.dom.TagName<!HTMLParagraphElement>} */
goog.dom.TagName.P = new goog.dom.TagName('P');


/** @type {!goog.dom.TagName<!HTMLParamElement>} */
goog.dom.TagName.PARAM = new goog.dom.TagName('PARAM');


/** @type {!goog.dom.TagName<!HTMLPictureElement>} */
goog.dom.TagName.PICTURE = new goog.dom.TagName('PICTURE');


/** @type {!goog.dom.TagName<!HTMLPreElement>} */
goog.dom.TagName.PRE = new goog.dom.TagName('PRE');


/** @type {!goog.dom.TagName<!HTMLProgressElement>} */
goog.dom.TagName.PROGRESS = new goog.dom.TagName('PROGRESS');


/** @type {!goog.dom.TagName<!HTMLQuoteElement>} */
goog.dom.TagName.Q = new goog.dom.TagName('Q');


/** @type {!goog.dom.TagName<!goog.dom.HtmlElement>} */
goog.dom.TagName.RP = new goog.dom.TagName('RP');


/** @type {!goog.dom.TagName<!goog.dom.HtmlElement>} */
goog.dom.TagName.RT = new goog.dom.TagName('RT');


/** @type {!goog.dom.TagName<!goog.dom.HtmlElement>} */
goog.dom.TagName.RTC = new goog.dom.TagName('RTC');


/** @type {!goog.dom.TagName<!goog.dom.HtmlElement>} */
goog.dom.TagName.RUBY = new goog.dom.TagName('RUBY');


/** @type {!goog.dom.TagName<!goog.dom.HtmlElement>} */
goog.dom.TagName.S = new goog.dom.TagName('S');


/** @type {!goog.dom.TagName<!goog.dom.HtmlElement>} */
goog.dom.TagName.SAMP = new goog.dom.TagName('SAMP');


/** @type {!goog.dom.TagName<!HTMLScriptElement>} */
goog.dom.TagName.SCRIPT = new goog.dom.TagName('SCRIPT');


/** @type {!goog.dom.TagName<!goog.dom.HtmlElement>} */
goog.dom.TagName.SECTION = new goog.dom.TagName('SECTION');


/** @type {!goog.dom.TagName<!HTMLSelectElement>} */
goog.dom.TagName.SELECT = new goog.dom.TagName('SELECT');


/** @type {!goog.dom.TagName<!goog.dom.HtmlElement>} */
goog.dom.TagName.SMALL = new goog.dom.TagName('SMALL');


/** @type {!goog.dom.TagName<!HTMLSourceElement>} */
goog.dom.TagName.SOURCE = new goog.dom.TagName('SOURCE');


/** @type {!goog.dom.TagName<!HTMLSpanElement>} */
goog.dom.TagName.SPAN = new goog.dom.TagName('SPAN');


/** @type {!goog.dom.TagName<!goog.dom.HtmlElement>} */
goog.dom.TagName.STRIKE = new goog.dom.TagName('STRIKE');


/** @type {!goog.dom.TagName<!goog.dom.HtmlElement>} */
goog.dom.TagName.STRONG = new goog.dom.TagName('STRONG');


/** @type {!goog.dom.TagName<!HTMLStyleElement>} */
goog.dom.TagName.STYLE = new goog.dom.TagName('STYLE');


/** @type {!goog.dom.TagName<!goog.dom.HtmlElement>} */
goog.dom.TagName.SUB = new goog.dom.TagName('SUB');


/** @type {!goog.dom.TagName<!goog.dom.HtmlElement>} */
goog.dom.TagName.SUMMARY = new goog.dom.TagName('SUMMARY');


/** @type {!goog.dom.TagName<!goog.dom.HtmlElement>} */
goog.dom.TagName.SUP = new goog.dom.TagName('SUP');


/** @type {!goog.dom.TagName<!goog.dom.HtmlElement>} */
goog.dom.TagName.SVG = new goog.dom.TagName('SVG');


/** @type {!goog.dom.TagName<!HTMLTableElement>} */
goog.dom.TagName.TABLE = new goog.dom.TagName('TABLE');


/** @type {!goog.dom.TagName<!HTMLTableSectionElement>} */
goog.dom.TagName.TBODY = new goog.dom.TagName('TBODY');


/** @type {!goog.dom.TagName<!HTMLTableCellElement>} */
goog.dom.TagName.TD = new goog.dom.TagName('TD');


/** @type {!goog.dom.TagName<!HTMLTemplateElement>} */
goog.dom.TagName.TEMPLATE = new goog.dom.TagName('TEMPLATE');


/** @type {!goog.dom.TagName<!HTMLTextAreaElement>} */
goog.dom.TagName.TEXTAREA = new goog.dom.TagName('TEXTAREA');


/** @type {!goog.dom.TagName<!HTMLTableSectionElement>} */
goog.dom.TagName.TFOOT = new goog.dom.TagName('TFOOT');


/** @type {!goog.dom.TagName<!HTMLTableCellElement>} */
goog.dom.TagName.TH = new goog.dom.TagName('TH');


/** @type {!goog.dom.TagName<!HTMLTableSectionElement>} */
goog.dom.TagName.THEAD = new goog.dom.TagName('THEAD');


/** @type {!goog.dom.TagName<!goog.dom.HtmlElement>} */
goog.dom.TagName.TIME = new goog.dom.TagName('TIME');


/** @type {!goog.dom.TagName<!HTMLTitleElement>} */
goog.dom.TagName.TITLE = new goog.dom.TagName('TITLE');


/** @type {!goog.dom.TagName<!HTMLTableRowElement>} */
goog.dom.TagName.TR = new goog.dom.TagName('TR');


/** @type {!goog.dom.TagName<!HTMLTrackElement>} */
goog.dom.TagName.TRACK = new goog.dom.TagName('TRACK');


/** @type {!goog.dom.TagName<!goog.dom.HtmlElement>} */
goog.dom.TagName.TT = new goog.dom.TagName('TT');


/** @type {!goog.dom.TagName<!goog.dom.HtmlElement>} */
goog.dom.TagName.U = new goog.dom.TagName('U');


/** @type {!goog.dom.TagName<!HTMLUListElement>} */
goog.dom.TagName.UL = new goog.dom.TagName('UL');


/** @type {!goog.dom.TagName<!goog.dom.HtmlElement>} */
goog.dom.TagName.VAR = new goog.dom.TagName('VAR');


/** @type {!goog.dom.TagName<!HTMLVideoElement>} */
goog.dom.TagName.VIDEO = new goog.dom.TagName('VIDEO');


/** @type {!goog.dom.TagName<!goog.dom.HtmlElement>} */
goog.dom.TagName.WBR = new goog.dom.TagName('WBR');

//third_party/javascript/closure/object/object.js
/**
 * @license
 * Copyright The Closure Library Authors.
 * SPDX-License-Identifier: Apache-2.0
 */

/**
 * @fileoverview Utilities for manipulating objects/maps/hashes.
 */

goog.provide('goog.object');


/**
 * Whether two values are not observably distinguishable. This
 * correctly detects that 0 is not the same as -0 and two NaNs are
 * practically equivalent.
 *
 * The implementation is as suggested by harmony:egal proposal.
 *
 * @param {*} v The first value to compare.
 * @param {*} v2 The second value to compare.
 * @return {boolean} Whether two values are not observably distinguishable.
 * @see http://wiki.ecmascript.org/doku.php?id=harmony:egal
 */
goog.object.is = function(v, v2) {
  if (v === v2) {
    // 0 === -0, but they are not identical.
    // We need the cast because the compiler requires that v2 is a
    // number (although 1/v2 works with non-number). We cast to ? to
    // stop the compiler from type-checking this statement.
    return v !== 0 || 1 / v === 1 / /** @type {?} */ (v2);
  }

  // NaN is non-reflexive: NaN !== NaN, although they are identical.
  return v !== v && v2 !== v2;
};


/**
 * Calls a function for each element in an object/map/hash.
 *
 * @param {Object<K,V>} obj The object over which to iterate.
 * @param {function(this:T,V,?,Object<K,V>):?} f The function to call
 *     for every element. This function takes 3 arguments (the value, the
 *     key and the object) and the return value is ignored.
 * @param {T=} opt_obj This is used as the 'this' object within f.
 * @template T,K,V
 */
goog.object.forEach = function(obj, f, opt_obj) {
  for (const key in obj) {
    f.call(/** @type {?} */ (opt_obj), obj[key], key, obj);
  }
};


/**
 * Calls a function for each element in an object/map/hash. If that call returns
 * true, adds the element to a new object.
 *
 * @param {Object<K,V>} obj The object over which to iterate.
 * @param {function(this:T,V,?,Object<K,V>):boolean} f The function to call
 *     for every element. This
 *     function takes 3 arguments (the value, the key and the object)
 *     and should return a boolean. If the return value is true the
 *     element is added to the result object. If it is false the
 *     element is not included.
 * @param {T=} opt_obj This is used as the 'this' object within f.
 * @return {!Object<K,V>} a new object in which only elements that passed the
 *     test are present.
 * @template T,K,V
 */
goog.object.filter = function(obj, f, opt_obj) {
  const res = {};
  for (const key in obj) {
    if (f.call(/** @type {?} */ (opt_obj), obj[key], key, obj)) {
      res[key] = obj[key];
    }
  }
  return res;
};


/**
 * For every element in an object/map/hash calls a function and inserts the
 * result into a new object.
 *
 * @param {Object<K,V>} obj The object over which to iterate.
 * @param {function(this:T,V,?,Object<K,V>):R} f The function to call
 *     for every element. This function
 *     takes 3 arguments (the value, the key and the object)
 *     and should return something. The result will be inserted
 *     into a new object.
 * @param {T=} opt_obj This is used as the 'this' object within f.
 * @return {!Object<K,R>} a new object with the results from f.
 * @template T,K,V,R
 */
goog.object.map = function(obj, f, opt_obj) {
  const res = {};
  for (const key in obj) {
    res[key] = f.call(/** @type {?} */ (opt_obj), obj[key], key, obj);
  }
  return res;
};


/**
 * Calls a function for each element in an object/map/hash. If any
 * call returns true, returns true (without checking the rest). If
 * all calls return false, returns false.
 *
 * @param {Object<K,V>} obj The object to check.
 * @param {function(this:T,V,?,Object<K,V>):boolean} f The function to
 *     call for every element. This function
 *     takes 3 arguments (the value, the key and the object) and should
 *     return a boolean.
 * @param {T=} opt_obj This is used as the 'this' object within f.
 * @return {boolean} true if any element passes the test.
 * @template T,K,V
 */
goog.object.some = function(obj, f, opt_obj) {
  for (const key in obj) {
    if (f.call(/** @type {?} */ (opt_obj), obj[key], key, obj)) {
      return true;
    }
  }
  return false;
};


/**
 * Calls a function for each element in an object/map/hash. If
 * all calls return true, returns true. If any call returns false, returns
 * false at this point and does not continue to check the remaining elements.
 *
 * @param {Object<K,V>} obj The object to check.
 * @param {?function(this:T,V,?,Object<K,V>):boolean} f The function to
 *     call for every element. This function
 *     takes 3 arguments (the value, the key and the object) and should
 *     return a boolean.
 * @param {T=} opt_obj This is used as the 'this' object within f.
 * @return {boolean} false if any element fails the test.
 * @template T,K,V
 */
goog.object.every = function(obj, f, opt_obj) {
  for (const key in obj) {
    if (!f.call(/** @type {?} */ (opt_obj), obj[key], key, obj)) {
      return false;
    }
  }
  return true;
};


/**
 * Returns the number of key-value pairs in the object map.
 *
 * @param {Object} obj The object for which to get the number of key-value
 *     pairs.
 * @return {number} The number of key-value pairs in the object map.
 */
goog.object.getCount = function(obj) {
  let rv = 0;
  for (const key in obj) {
    rv++;
  }
  return rv;
};


/**
 * Returns one key from the object map, if any exists.
 * For map literals the returned key will be the first one in most of the
 * browsers (a know exception is Konqueror).
 *
 * @param {Object} obj The object to pick a key from.
 * @return {string|undefined} The key or undefined if the object is empty.
 */
goog.object.getAnyKey = function(obj) {
  for (const key in obj) {
    return key;
  }
};


/**
 * Returns one value from the object map, if any exists.
 * For map literals the returned value will be the first one in most of the
 * browsers (a know exception is Konqueror).
 *
 * @param {Object<K,V>} obj The object to pick a value from.
 * @return {V|undefined} The value or undefined if the object is empty.
 * @template K,V
 */
goog.object.getAnyValue = function(obj) {
  for (const key in obj) {
    return obj[key];
  }
};


/**
 * Whether the object/hash/map contains the given object as a value.
 * An alias for goog.object.containsValue(obj, val).
 *
 * @param {Object<K,V>} obj The object in which to look for val.
 * @param {V} val The object for which to check.
 * @return {boolean} true if val is present.
 * @template K,V
 */
goog.object.contains = function(obj, val) {
  return goog.object.containsValue(obj, val);
};


/**
 * Returns the values of the object/map/hash.
 *
 * @param {Object<K,V>} obj The object from which to get the values.
 * @return {!Array<V>} The values in the object/map/hash.
 * @template K,V
 */
goog.object.getValues = function(obj) {
  const res = [];
  let i = 0;
  for (const key in obj) {
    res[i++] = obj[key];
  }
  return res;
};


/**
 * Returns the keys of the object/map/hash.
 *
 * @param {Object} obj The object from which to get the keys.
 * @return {!Array<string>} Array of property keys.
 */
goog.object.getKeys = function(obj) {
  const res = [];
  let i = 0;
  for (const key in obj) {
    res[i++] = key;
  }
  return res;
};


/**
 * Get a value from an object multiple levels deep.  This is useful for
 * pulling values from deeply nested objects, such as JSON responses.
 * Example usage: getValueByKeys(jsonObj, 'foo', 'entries', 3)
 *
 * @param {!Object} obj An object to get the value from.  Can be array-like.
 * @param {...(string|number|!IArrayLike<number|string>)}
 *     var_args A number of keys
 *     (as strings, or numbers, for array-like objects).  Can also be
 *     specified as a single array of keys.
 * @return {*} The resulting value.  If, at any point, the value for a key
 *     in the current object is null or undefined, returns undefined.
 */
goog.object.getValueByKeys = function(obj, var_args) {
  const isArrayLike = goog.isArrayLike(var_args);
  const keys = isArrayLike ?
      /** @type {!IArrayLike<number|string>} */ (var_args) :
      arguments;

  // Start with the 2nd parameter for the variable parameters syntax.
  for (let i = isArrayLike ? 0 : 1; i < keys.length; i++) {
    if (obj == null) return undefined;
    obj = obj[keys[i]];
  }

  return obj;
};


/**
 * Whether the object/map/hash contains the given key.
 *
 * @param {Object} obj The object in which to look for key.
 * @param {?} key The key for which to check.
 * @return {boolean} true If the map contains the key.
 */
goog.object.containsKey = function(obj, key) {
  return obj !== null && key in obj;
};


/**
 * Whether the object/map/hash contains the given value. This is O(n).
 *
 * @param {Object<K,V>} obj The object in which to look for val.
 * @param {V} val The value for which to check.
 * @return {boolean} true If the map contains the value.
 * @template K,V
 */
goog.object.containsValue = function(obj, val) {
  for (const key in obj) {
    if (obj[key] == val) {
      return true;
    }
  }
  return false;
};


/**
 * Searches an object for an element that satisfies the given condition and
 * returns its key.
 * @param {Object<K,V>} obj The object to search in.
 * @param {function(this:T,V,string,Object<K,V>):boolean} f The
 *      function to call for every element. Takes 3 arguments (the value,
 *     the key and the object) and should return a boolean.
 * @param {T=} opt_this An optional "this" context for the function.
 * @return {string|undefined} The key of an element for which the function
 *     returns true or undefined if no such element is found.
 * @template T,K,V
 */
goog.object.findKey = function(obj, f, opt_this) {
  for (const key in obj) {
    if (f.call(/** @type {?} */ (opt_this), obj[key], key, obj)) {
      return key;
    }
  }
  return undefined;
};


/**
 * Searches an object for an element that satisfies the given condition and
 * returns its value.
 * @param {Object<K,V>} obj The object to search in.
 * @param {function(this:T,V,string,Object<K,V>):boolean} f The function
 *     to call for every element. Takes 3 arguments (the value, the key
 *     and the object) and should return a boolean.
 * @param {T=} opt_this An optional "this" context for the function.
 * @return {V} The value of an element for which the function returns true or
 *     undefined if no such element is found.
 * @template T,K,V
 */
goog.object.findValue = function(obj, f, opt_this) {
  const key = goog.object.findKey(obj, f, opt_this);
  return key && obj[key];
};


/**
 * Whether the object/map/hash is empty.
 *
 * @param {Object} obj The object to test.
 * @return {boolean} true if obj is empty.
 */
goog.object.isEmpty = function(obj) {
  for (const key in obj) {
    return false;
  }
  return true;
};


/**
 * Removes all key value pairs from the object/map/hash.
 *
 * @param {Object} obj The object to clear.
 */
goog.object.clear = function(obj) {
  for (const i in obj) {
    delete obj[i];
  }
};


/**
 * Removes a key-value pair based on the key.
 *
 * @param {Object} obj The object from which to remove the key.
 * @param {?} key The key to remove.
 * @return {boolean} Whether an element was removed.
 */
goog.object.remove = function(obj, key) {
  let rv;
  if (rv = key in /** @type {!Object} */ (obj)) {
    delete obj[key];
  }
  return rv;
};


/**
 * Adds a key-value pair to the object. Throws an exception if the key is
 * already in use. Use set if you want to change an existing pair.
 *
 * @param {Object<K,V>} obj The object to which to add the key-value pair.
 * @param {string} key The key to add.
 * @param {V} val The value to add.
 * @template K,V
 */
goog.object.add = function(obj, key, val) {
  if (obj !== null && key in obj) {
    throw new Error('The object already contains the key "' + key + '"');
  }
  goog.object.set(obj, key, val);
};


/**
 * Returns the value for the given key.
 *
 * @param {Object<K,V>} obj The object from which to get the value.
 * @param {string} key The key for which to get the value.
 * @param {R=} opt_val The value to return if no item is found for the given
 *     key (default is undefined).
 * @return {V|R|undefined} The value for the given key.
 * @template K,V,R
 */
goog.object.get = function(obj, key, opt_val) {
  if (obj !== null && key in obj) {
    return obj[key];
  }
  return opt_val;
};


/**
 * Adds a key-value pair to the object/map/hash.
 *
 * @param {Object<K,V>} obj The object to which to add the key-value pair.
 * @param {string} key The key to add.
 * @param {V} value The value to add.
 * @template K,V
 */
goog.object.set = function(obj, key, value) {
  obj[key] = value;
};


/**
 * Adds a key-value pair to the object/map/hash if it doesn't exist yet.
 *
 * @param {Object<K,V>} obj The object to which to add the key-value pair.
 * @param {string} key The key to add.
 * @param {V} value The value to add if the key wasn't present.
 * @return {V} The value of the entry at the end of the function.
 * @template K,V
 */
goog.object.setIfUndefined = function(obj, key, value) {
  return key in /** @type {!Object} */ (obj) ? obj[key] : (obj[key] = value);
};


/**
 * Sets a key and value to an object if the key is not set. The value will be
 * the return value of the given function. If the key already exists, the
 * object will not be changed and the function will not be called (the function
 * will be lazily evaluated -- only called if necessary).
 *
 * This function is particularly useful when used with an `Object` which is
 * acting as a cache.
 *
 * @param {!Object<K,V>} obj The object to which to add the key-value pair.
 * @param {string} key The key to add.
 * @param {function():V} f The value to add if the key wasn't present.
 * @return {V} The value of the entry at the end of the function.
 * @template K,V
 */
goog.object.setWithReturnValueIfNotSet = function(obj, key, f) {
  if (key in obj) {
    return obj[key];
  }

  const val = f();
  obj[key] = val;
  return val;
};


/**
 * Compares two objects for equality using === on the values.
 *
 * @param {!Object<K,V>} a
 * @param {!Object<K,V>} b
 * @return {boolean}
 * @template K,V
 */
goog.object.equals = function(a, b) {
  for (const k in a) {
    if (!(k in b) || a[k] !== b[k]) {
      return false;
    }
  }
  for (const k in b) {
    if (!(k in a)) {
      return false;
    }
  }
  return true;
};


/**
 * Returns a shallow clone of the object.
 *
 * @param {Object<K,V>} obj Object to clone.
 * @return {!Object<K,V>} Clone of the input object.
 * @template K,V
 */
goog.object.clone = function(obj) {
  // We cannot use the prototype trick because a lot of methods depend on where
  // the actual key is set.

  const res = {};
  for (const key in obj) {
    res[key] = obj[key];
  }
  return res;
  // We could also use goog.mixin but I wanted this to be independent from that.
};


/**
 * Clones a value. The input may be an Object, Array, or basic type. Objects and
 * arrays will be cloned recursively.
 *
 * WARNINGS:
 * <code>goog.object.unsafeClone</code> does not detect reference loops. Objects
 * that refer to themselves will cause infinite recursion.
 *
 * <code>goog.object.unsafeClone</code> is unaware of unique identifiers, and
 * copies UIDs created by <code>getUid</code> into cloned results.
 *
 * @param {T} obj The value to clone.
 * @return {T} A clone of the input value.
 * @template T
 */
goog.object.unsafeClone = function(obj) {
  const type = goog.typeOf(obj);
  if (type == 'object' || type == 'array') {
    if (goog.isFunction(obj.clone)) {
      return obj.clone();
    }
    const clone = type == 'array' ? [] : {};
    for (const key in obj) {
      clone[key] = goog.object.unsafeClone(obj[key]);
    }
    return clone;
  }

  return obj;
};


/**
 * Returns a new object in which all the keys and values are interchanged
 * (keys become values and values become keys). If multiple keys map to the
 * same value, the chosen transposed value is implementation-dependent.
 *
 * @param {Object} obj The object to transpose.
 * @return {!Object} The transposed object.
 */
goog.object.transpose = function(obj) {
  const transposed = {};
  for (const key in obj) {
    transposed[obj[key]] = key;
  }
  return transposed;
};


/**
 * The names of the fields that are defined on Object.prototype.
 * @type {Array<string>}
 * @private
 */
goog.object.PROTOTYPE_FIELDS_ = [
  'constructor', 'hasOwnProperty', 'isPrototypeOf', 'propertyIsEnumerable',
  'toLocaleString', 'toString', 'valueOf'
];


/**
 * Extends an object with another object.
 * This operates 'in-place'; it does not create a new Object.
 *
 * Example:
 * var o = {};
 * goog.object.extend(o, {a: 0, b: 1});
 * o; // {a: 0, b: 1}
 * goog.object.extend(o, {b: 2, c: 3});
 * o; // {a: 0, b: 2, c: 3}
 *
 * @param {Object} target The object to modify. Existing properties will be
 *     overwritten if they are also present in one of the objects in
 *     `var_args`.
 * @param {...(Object|null|undefined)} var_args The objects from which values
 *     will be copied.
 * @deprecated Prefer Object.assign
 */
goog.object.extend = function(target, var_args) {
  let key;
  let source;
  for (let i = 1; i < arguments.length; i++) {
    source = arguments[i];
    for (key in source) {
      target[key] = source[key];
    }

    // For IE the for-in-loop does not contain any properties that are not
    // enumerable on the prototype object (for example isPrototypeOf from
    // Object.prototype) and it will also not include 'replace' on objects that
    // extend String and change 'replace' (not that it is common for anyone to
    // extend anything except Object).

    for (let j = 0; j < goog.object.PROTOTYPE_FIELDS_.length; j++) {
      key = goog.object.PROTOTYPE_FIELDS_[j];
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }
};


/**
 * Creates a new object built from the key-value pairs provided as arguments.
 * @param {...*} var_args If only one argument is provided and it is an array
 *     then this is used as the arguments, otherwise even arguments are used as
 *     the property names and odd arguments are used as the property values.
 * @return {!Object} The new object.
 * @throws {Error} If there are uneven number of arguments or there is only one
 *     non array argument.
 */
goog.object.create = function(var_args) {
  const argLength = arguments.length;
  if (argLength == 1 && Array.isArray(arguments[0])) {
    return goog.object.create.apply(null, arguments[0]);
  }

  if (argLength % 2) {
    throw new Error('Uneven number of arguments');
  }

  const rv = {};
  for (let i = 0; i < argLength; i += 2) {
    rv[arguments[i]] = arguments[i + 1];
  }
  return rv;
};


/**
 * Creates a new object where the property names come from the arguments but
 * the value is always set to true
 * @param {...*} var_args If only one argument is provided and it is an array
 *     then this is used as the arguments, otherwise the arguments are used
 *     as the property names.
 * @return {!Object} The new object.
 */
goog.object.createSet = function(var_args) {
  const argLength = arguments.length;
  if (argLength == 1 && Array.isArray(arguments[0])) {
    return goog.object.createSet.apply(null, arguments[0]);
  }

  const rv = {};
  for (let i = 0; i < argLength; i++) {
    rv[arguments[i]] = true;
  }
  return rv;
};


/**
 * Creates an immutable view of the underlying object, if the browser
 * supports immutable objects.
 *
 * In default mode, writes to this view will fail silently. In strict mode,
 * they will throw an error.
 *
 * @param {!Object<K,V>} obj An object.
 * @return {!Object<K,V>} An immutable view of that object, or the
 *     original object if this browser does not support immutables.
 * @template K,V
 */
goog.object.createImmutableView = function(obj) {
  let result = obj;
  if (Object.isFrozen && !Object.isFrozen(obj)) {
    result = Object.create(obj);
    Object.freeze(result);
  }
  return result;
};


/**
 * @param {!Object} obj An object.
 * @return {boolean} Whether this is an immutable view of the object.
 */
goog.object.isImmutableView = function(obj) {
  return !!Object.isFrozen && Object.isFrozen(obj);
};


/**
 * Get all properties names on a given Object regardless of enumerability.
 *
 * <p> If the browser does not support `Object.getOwnPropertyNames` nor
 * `Object.getPrototypeOf` then this is equivalent to using
 * `goog.object.getKeys`
 *
 * @param {?Object} obj The object to get the properties of.
 * @param {boolean=} opt_includeObjectPrototype Whether properties defined on
 *     `Object.prototype` should be included in the result.
 * @param {boolean=} opt_includeFunctionPrototype Whether properties defined on
 *     `Function.prototype` should be included in the result.
 * @return {!Array<string>}
 * @public
 */
goog.object.getAllPropertyNames = function(
    obj, opt_includeObjectPrototype, opt_includeFunctionPrototype) {
  if (!obj) {
    return [];
  }

  // Naively use a for..in loop to get the property names if the browser doesn't
  // support any other APIs for getting it.
  if (!Object.getOwnPropertyNames || !Object.getPrototypeOf) {
    return goog.object.getKeys(obj);
  }

  const visitedSet = {};

  // Traverse the prototype chain and add all properties to the visited set.
  let proto = obj;
  while (proto &&
         (proto !== Object.prototype || !!opt_includeObjectPrototype) &&
         (proto !== Function.prototype || !!opt_includeFunctionPrototype)) {
    const names = Object.getOwnPropertyNames(proto);
    for (let i = 0; i < names.length; i++) {
      visitedSet[names[i]] = true;
    }
    proto = Object.getPrototypeOf(proto);
  }

  return goog.object.getKeys(visitedSet);
};


/**
 * Given a ES5 or ES6 class reference, return its super class / super
 * constructor.
 *
 * This should be used in rare cases where you need to walk up the inheritance
 * tree (this is generally a bad idea). But this work with ES5 and ES6 classes,
 * unlike relying on the superClass_ property.
 *
 * Note: To start walking up the hierarchy from an instance call this with its
 * `constructor` property; e.g. `getSuperClass(instance.constructor)`.
 *
 * @param {function(new: ?)} constructor
 * @return {?Object}
 */
goog.object.getSuperClass = function(constructor) {
  var proto = Object.getPrototypeOf(constructor.prototype);
  return proto && proto.constructor;
};

//third_party/javascript/closure/dom/tags.js
/**
 * @license
 * Copyright The Closure Library Authors.
 * SPDX-License-Identifier: Apache-2.0
 */

/**
 * @fileoverview Utilities for HTML element tag names.
 */
goog.provide('goog.dom.tags');

goog.require('goog.object');


/**
 * The void elements specified by
 * http://www.w3.org/TR/html-markup/syntax.html#void-elements.
 * @const @private {!Object<string, boolean>}
 */
goog.dom.tags.VOID_TAGS_ = goog.object.createSet(
    'area', 'base', 'br', 'col', 'command', 'embed', 'hr', 'img', 'input',
    'keygen', 'link', 'meta', 'param', 'source', 'track', 'wbr');


/**
 * Checks whether the tag is void (with no contents allowed and no legal end
 * tag), for example 'br'.
 * @param {string} tagName The tag name in lower case.
 * @return {boolean}
 */
goog.dom.tags.isVoidTag = function(tagName) {
  return goog.dom.tags.VOID_TAGS_[tagName] === true;
};

//third_party/javascript/closure/html/trustedtypes.js
/**
 * @license
 * Copyright The Closure Library Authors.
 * SPDX-License-Identifier: Apache-2.0
 */

/**
 * @fileoverview Policy to convert strings to Trusted Types. See
 * https://github.com/WICG/trusted-types for details.
 *
 * @visibility {//javascript/closure:__pkg__}
 * @visibility {//javascript/closure/bin/sizetests:__pkg__}
 * @visibility {//javascript/closure/dom:__pkg__}
 */

goog.provide('goog.html.trustedtypes');

/** @package @const {?TrustedTypePolicy} */
goog.html.trustedtypes.PRIVATE_DO_NOT_ACCESS_OR_ELSE_POLICY =
    goog.TRUSTED_TYPES_POLICY_NAME ?
    goog.createTrustedTypesPolicy(goog.TRUSTED_TYPES_POLICY_NAME + '#html') :
    null;

//third_party/javascript/closure/string/typedstring.js
/**
 * @license
 * Copyright The Closure Library Authors.
 * SPDX-License-Identifier: Apache-2.0
 */

goog.provide('goog.string.TypedString');



/**
 * Wrapper for strings that conform to a data type or language.
 *
 * Implementations of this interface are wrappers for strings, and typically
 * associate a type contract with the wrapped string.  Concrete implementations
 * of this interface may choose to implement additional run-time type checking,
 * see for example `goog.html.SafeHtml`. If available, client code that
 * needs to ensure type membership of an object should use the type's function
 * to assert type membership, such as `goog.html.SafeHtml.unwrap`.
 * @interface
 */
goog.string.TypedString = function() {};


/**
 * Interface marker of the TypedString interface.
 *
 * This property can be used to determine at runtime whether or not an object
 * implements this interface.  All implementations of this interface set this
 * property to `true`.
 * @type {boolean}
 */
goog.string.TypedString.prototype.implementsGoogStringTypedString;


/**
 * Retrieves this wrapped string's value.
 * @return {string} The wrapped string's value.
 */
goog.string.TypedString.prototype.getTypedStringValue;

//third_party/javascript/closure/string/const.js
/**
 * @license
 * Copyright The Closure Library Authors.
 * SPDX-License-Identifier: Apache-2.0
 */

goog.provide('goog.string.Const');

goog.require('goog.asserts');
goog.require('goog.string.TypedString');



/**
 * Wrapper for compile-time-constant strings.
 *
 * Const is a wrapper for strings that can only be created from program
 * constants (i.e., string literals).  This property relies on a custom Closure
 * compiler check that `goog.string.Const.from` is only invoked on
 * compile-time-constant expressions.
 *
 * Const is useful in APIs whose correct and secure use requires that certain
 * arguments are not attacker controlled: Compile-time constants are inherently
 * under the control of the application and not under control of external
 * attackers, and hence are safe to use in such contexts.
 *
 * Instances of this type must be created via its factory method
 * `goog.string.Const.from` and not by invoking its constructor.  The
 * constructor intentionally takes no parameters and the type is immutable;
 * hence only a default instance corresponding to the empty string can be
 * obtained via constructor invocation.  Use goog.string.Const.EMPTY
 * instead of using this constructor to get an empty Const string.
 *
 * @see goog.string.Const#from
 * @constructor
 * @final
 * @struct
 * @implements {goog.string.TypedString}
 * @param {Object=} opt_token package-internal implementation detail.
 * @param {string=} opt_content package-internal implementation detail.
 */
goog.string.Const = function(opt_token, opt_content) {
  /**
   * The wrapped value of this Const object.  The field has a purposely ugly
   * name to make (non-compiled) code that attempts to directly access this
   * field stand out.
   * @private {string}
   */
  this.stringConstValueWithSecurityContract__googStringSecurityPrivate_ =
      ((opt_token ===
        goog.string.Const.GOOG_STRING_CONSTRUCTOR_TOKEN_PRIVATE_) &&
       opt_content) ||
      '';

  /**
   * A type marker used to implement additional run-time type checking.
   * @see goog.string.Const#unwrap
   * @const {!Object}
   * @private
   */
  this.STRING_CONST_TYPE_MARKER__GOOG_STRING_SECURITY_PRIVATE_ =
      goog.string.Const.TYPE_MARKER_;
};


/**
 * @override
 * @const
 */
goog.string.Const.prototype.implementsGoogStringTypedString = true;


/**
 * Returns this Const's value as a string.
 *
 * IMPORTANT: In code where it is security-relevant that an object's type is
 * indeed `goog.string.Const`, use `goog.string.Const.unwrap`
 * instead of this method.
 *
 * @see goog.string.Const#unwrap
 * @override
 */
goog.string.Const.prototype.getTypedStringValue = function() {
  return this.stringConstValueWithSecurityContract__googStringSecurityPrivate_;
};


if (goog.DEBUG) {
  /**
   * Returns a debug-string representation of this value.
   *
   * To obtain the actual string value wrapped inside an object of this type,
   * use `goog.string.Const.unwrap`.
   *
   * @see goog.string.Const#unwrap
   * @override
   */
  goog.string.Const.prototype.toString = function() {
    return 'Const{' +
        this.stringConstValueWithSecurityContract__googStringSecurityPrivate_ +
        '}';
  };
}


/**
 * Performs a runtime check that the provided object is indeed an instance
 * of `goog.string.Const`, and returns its value.
 * @param {!goog.string.Const} stringConst The object to extract from.
 * @return {string} The Const object's contained string, unless the run-time
 *     type check fails. In that case, `unwrap` returns an innocuous
 *     string, or, if assertions are enabled, throws
 *     `goog.asserts.AssertionError`.
 */
goog.string.Const.unwrap = function(stringConst) {
  // Perform additional run-time type-checking to ensure that stringConst is
  // indeed an instance of the expected type.  This provides some additional
  // protection against security bugs due to application code that disables type
  // checks.
  if (stringConst instanceof goog.string.Const &&
      stringConst.constructor === goog.string.Const &&
      stringConst.STRING_CONST_TYPE_MARKER__GOOG_STRING_SECURITY_PRIVATE_ ===
          goog.string.Const.TYPE_MARKER_) {
    return stringConst
        .stringConstValueWithSecurityContract__googStringSecurityPrivate_;
  } else {
    goog.asserts.fail(
        'expected object of type Const, got \'' + stringConst + '\'');
    return 'type_error:Const';
  }
};


/**
 * Creates a Const object from a compile-time constant string.
 *
 * It is illegal to invoke this function on an expression whose
 * compile-time-constant value cannot be determined by the Closure compiler.
 *
 * Correct invocations include,
 * <pre>
 *   var s = goog.string.Const.from('hello');
 *   var t = goog.string.Const.from('hello' + 'world');
 * </pre>
 *
 * In contrast, the following are illegal:
 * <pre>
 *   var s = goog.string.Const.from(getHello());
 *   var t = goog.string.Const.from('hello' + world);
 * </pre>
 *
 * @param {string} s A constant string from which to create a Const.
 * @return {!goog.string.Const} A Const object initialized to stringConst.
 */
goog.string.Const.from = function(s) {
  return new goog.string.Const(
      goog.string.Const.GOOG_STRING_CONSTRUCTOR_TOKEN_PRIVATE_, s);
};

/**
 * Type marker for the Const type, used to implement additional run-time
 * type checking.
 * @const {!Object}
 * @private
 */
goog.string.Const.TYPE_MARKER_ = {};

/**
 * @type {!Object}
 * @private
 * @const
 */
goog.string.Const.GOOG_STRING_CONSTRUCTOR_TOKEN_PRIVATE_ = {};

/**
 * A Const instance wrapping the empty string.
 * @const {!goog.string.Const}
 */
goog.string.Const.EMPTY = goog.string.Const.from('');

//third_party/javascript/closure/html/safescript.js
/**
 * @license
 * Copyright The Closure Library Authors.
 * SPDX-License-Identifier: Apache-2.0
 */

/**
 * @fileoverview The SafeScript type and its builders.
 *
 * TODO(xtof): Link to document stating type contract.
 */

goog.provide('goog.html.SafeScript');

goog.require('goog.asserts');
goog.require('goog.html.trustedtypes');
goog.require('goog.string.Const');
goog.require('goog.string.TypedString');



/**
 * A string-like object which represents JavaScript code and that carries the
 * security type contract that its value, as a string, will not cause execution
 * of unconstrained attacker controlled code (XSS) when evaluated as JavaScript
 * in a browser.
 *
 * Instances of this type must be created via the factory method
 * `goog.html.SafeScript.fromConstant` and not by invoking its
 * constructor. The constructor intentionally takes no parameters and the type
 * is immutable; hence only a default instance corresponding to the empty string
 * can be obtained via constructor invocation.
 *
 * A SafeScript's string representation can safely be interpolated as the
 * content of a script element within HTML. The SafeScript string should not be
 * escaped before interpolation.
 *
 * Note that the SafeScript might contain text that is attacker-controlled but
 * that text should have been interpolated with appropriate escaping,
 * sanitization and/or validation into the right location in the script, such
 * that it is highly constrained in its effect (for example, it had to match a
 * set of whitelisted words).
 *
 * A SafeScript can be constructed via security-reviewed unchecked
 * conversions. In this case producers of SafeScript must ensure themselves that
 * the SafeScript does not contain unsafe script. Note in particular that
 * {@code &lt;} is dangerous, even when inside JavaScript strings, and so should
 * always be forbidden or JavaScript escaped in user controlled input. For
 * example, if {@code &lt;/script&gt;&lt;script&gt;evil&lt;/script&gt;"} were
 * interpolated inside a JavaScript string, it would break out of the context
 * of the original script element and `evil` would execute. Also note
 * that within an HTML script (raw text) element, HTML character references,
 * such as "&lt;" are not allowed. See
 * http://www.w3.org/TR/html5/scripting-1.html#restrictions-for-contents-of-script-elements.
 *
 * @see goog.html.SafeScript#fromConstant
 * @constructor
 * @final
 * @struct
 * @implements {goog.string.TypedString}
 */
goog.html.SafeScript = function() {
  /**
   * The contained value of this SafeScript.  The field has a purposely
   * ugly name to make (non-compiled) code that attempts to directly access this
   * field stand out.
   * @private {!TrustedScript|string}
   */
  this.privateDoNotAccessOrElseSafeScriptWrappedValue_ = '';

  /**
   * A type marker used to implement additional run-time type checking.
   * @see goog.html.SafeScript#unwrap
   * @const {!Object}
   * @private
   */
  this.SAFE_SCRIPT_TYPE_MARKER_GOOG_HTML_SECURITY_PRIVATE_ =
      goog.html.SafeScript.TYPE_MARKER_GOOG_HTML_SECURITY_PRIVATE_;
};


/**
 * @override
 * @const
 */
goog.html.SafeScript.prototype.implementsGoogStringTypedString = true;


/**
 * Type marker for the SafeScript type, used to implement additional
 * run-time type checking.
 * @const {!Object}
 * @private
 */
goog.html.SafeScript.TYPE_MARKER_GOOG_HTML_SECURITY_PRIVATE_ = {};


/**
 * Creates a SafeScript object from a compile-time constant string.
 *
 * @param {!goog.string.Const} script A compile-time-constant string from which
 *     to create a SafeScript.
 * @return {!goog.html.SafeScript} A SafeScript object initialized to
 *     `script`.
 */
goog.html.SafeScript.fromConstant = function(script) {
  var scriptString = goog.string.Const.unwrap(script);
  if (scriptString.length === 0) {
    return goog.html.SafeScript.EMPTY;
  }
  return goog.html.SafeScript.createSafeScriptSecurityPrivateDoNotAccessOrElse(
      scriptString);
};


/**
 * Creates a SafeScript from a compile-time constant string but with arguments
 * that can vary at run-time. The code argument should be formatted as an
 * inline function (see example below). The arguments will be JSON-encoded and
 * provided as input to the function specified in code.
 *
 * Example Usage:
 *
 *     let safeScript = SafeScript.fromConstantAndArgs(
 *         Const.from('function(arg1, arg2) { doSomething(arg1, arg2); }'),
 *         arg1,
 *         arg2);
 *
 * This produces a SafeScript equivalent to the following:
 *
 *     (function(arg1, arg2) { doSomething(arg1, arg2); })("value1", "value2");
 *
 * @param {!goog.string.Const} code
 * @param {...*} var_args
 * @return {!goog.html.SafeScript}
 */
goog.html.SafeScript.fromConstantAndArgs = function(code, var_args) {
  var args = [];
  for (var i = 1; i < arguments.length; i++) {
    args.push(goog.html.SafeScript.stringify_(arguments[i]));
  }
  return goog.html.SafeScript.createSafeScriptSecurityPrivateDoNotAccessOrElse(
      '(' + goog.string.Const.unwrap(code) + ')(' + args.join(', ') + ');');
};


/**
 * Creates a SafeScript JSON representation from anything that could be passed
 * to JSON.stringify.
 * @param {*} val
 * @return {!goog.html.SafeScript}
 */
goog.html.SafeScript.fromJson = function(val) {
  return goog.html.SafeScript.createSafeScriptSecurityPrivateDoNotAccessOrElse(
      goog.html.SafeScript.stringify_(val));
};


/**
 * Returns this SafeScript's value as a string.
 *
 * IMPORTANT: In code where it is security relevant that an object's type is
 * indeed `SafeScript`, use `goog.html.SafeScript.unwrap` instead of
 * this method. If in doubt, assume that it's security relevant. In particular,
 * note that goog.html functions which return a goog.html type do not guarantee
 * the returned instance is of the right type. For example:
 *
 * <pre>
 * var fakeSafeHtml = new String('fake');
 * fakeSafeHtml.__proto__ = goog.html.SafeHtml.prototype;
 * var newSafeHtml = goog.html.SafeHtml.htmlEscape(fakeSafeHtml);
 * // newSafeHtml is just an alias for fakeSafeHtml, it's passed through by
 * // goog.html.SafeHtml.htmlEscape() as fakeSafeHtml
 * // instanceof goog.html.SafeHtml.
 * </pre>
 *
 * @see goog.html.SafeScript#unwrap
 * @override
 */
goog.html.SafeScript.prototype.getTypedStringValue = function() {
  return this.privateDoNotAccessOrElseSafeScriptWrappedValue_.toString();
};


if (goog.DEBUG) {
  /**
   * Returns a debug string-representation of this value.
   *
   * To obtain the actual string value wrapped in a SafeScript, use
   * `goog.html.SafeScript.unwrap`.
   *
   * @see goog.html.SafeScript#unwrap
   * @override
   */
  goog.html.SafeScript.prototype.toString = function() {
    return 'SafeScript{' +
        this.privateDoNotAccessOrElseSafeScriptWrappedValue_ + '}';
  };
}


/**
 * Performs a runtime check that the provided object is indeed a
 * SafeScript object, and returns its value.
 *
 * @param {!goog.html.SafeScript} safeScript The object to extract from.
 * @return {string} The safeScript object's contained string, unless
 *     the run-time type check fails. In that case, `unwrap` returns an
 *     innocuous string, or, if assertions are enabled, throws
 *     `goog.asserts.AssertionError`.
 */
goog.html.SafeScript.unwrap = function(safeScript) {
  return goog.html.SafeScript.unwrapTrustedScript(safeScript).toString();
};


/**
 * Unwraps value as TrustedScript if supported or as a string if not.
 * @param {!goog.html.SafeScript} safeScript
 * @return {!TrustedScript|string}
 * @see goog.html.SafeScript.unwrap
 */
goog.html.SafeScript.unwrapTrustedScript = function(safeScript) {
  // Perform additional Run-time type-checking to ensure that
  // safeScript is indeed an instance of the expected type.  This
  // provides some additional protection against security bugs due to
  // application code that disables type checks.
  // Specifically, the following checks are performed:
  // 1. The object is an instance of the expected type.
  // 2. The object is not an instance of a subclass.
  // 3. The object carries a type marker for the expected type. "Faking" an
  // object requires a reference to the type marker, which has names intended
  // to stand out in code reviews.
  if (safeScript instanceof goog.html.SafeScript &&
      safeScript.constructor === goog.html.SafeScript &&
      safeScript.SAFE_SCRIPT_TYPE_MARKER_GOOG_HTML_SECURITY_PRIVATE_ ===
          goog.html.SafeScript.TYPE_MARKER_GOOG_HTML_SECURITY_PRIVATE_) {
    return safeScript.privateDoNotAccessOrElseSafeScriptWrappedValue_;
  } else {
    goog.asserts.fail('expected object of type SafeScript, got \'' +
        safeScript + '\' of type ' + goog.typeOf(safeScript));
    return 'type_error:SafeScript';
  }
};


/**
 * Converts the given value to a embeddabel JSON string and returns it. The
 * resulting string can be embedded in HTML because the '<' character is
 * encoded.
 *
 * @param {*} val
 * @return {string}
 * @private
 */
goog.html.SafeScript.stringify_ = function(val) {
  var json = JSON.stringify(val);
  return json.replace(/</g, '\\x3c');
};

/**
 * Package-internal utility method to create SafeScript instances.
 *
 * @param {string} script The string to initialize the SafeScript object with.
 * @return {!goog.html.SafeScript} The initialized SafeScript object.
 * @package
 */
goog.html.SafeScript.createSafeScriptSecurityPrivateDoNotAccessOrElse =
    function(script) {
  return new goog.html.SafeScript().initSecurityPrivateDoNotAccessOrElse_(
      script);
};


/**
 * Called from createSafeScriptSecurityPrivateDoNotAccessOrElse(). This
 * method exists only so that the compiler can dead code eliminate static
 * fields (like EMPTY) when they're not accessed.
 * @param {string} script
 * @return {!goog.html.SafeScript}
 * @private
 */
goog.html.SafeScript.prototype.initSecurityPrivateDoNotAccessOrElse_ = function(
    script) {
  this.privateDoNotAccessOrElseSafeScriptWrappedValue_ =
      goog.html.trustedtypes.PRIVATE_DO_NOT_ACCESS_OR_ELSE_POLICY ?
      goog.html.trustedtypes.PRIVATE_DO_NOT_ACCESS_OR_ELSE_POLICY.createScript(
          script) :
      script;
  return this;
};


/**
 * A SafeScript instance corresponding to the empty string.
 * @const {!goog.html.SafeScript}
 */
goog.html.SafeScript.EMPTY =
    goog.html.SafeScript.createSafeScriptSecurityPrivateDoNotAccessOrElse('');

//third_party/javascript/closure/fs/url.js
/**
 * @license
 * Copyright The Closure Library Authors.
 * SPDX-License-Identifier: Apache-2.0
 */

/**
 * @fileoverview Wrapper for URL and its createObjectUrl and revokeObjectUrl
 * methods that are part of the HTML5 File API.
 */

goog.provide('goog.fs.url');


/**
 * Creates a blob URL for a blob object.
 * Throws an error if the browser does not support Object Urls.
 *
 * @param {!File|!Blob|!MediaSource|!MediaStream} obj The object for which
 *   to create the URL.
 * @return {string} The URL for the object.
 */
goog.fs.url.createObjectUrl = function(obj) {
  return goog.fs.url.getUrlObject_().createObjectURL(obj);
};


/**
 * Revokes a URL created by {@link goog.fs.url.createObjectUrl}.
 * Throws an error if the browser does not support Object Urls.
 *
 * @param {string} url The URL to revoke.
 */
goog.fs.url.revokeObjectUrl = function(url) {
  goog.fs.url.getUrlObject_().revokeObjectURL(url);
};


/**
 * @record
 * @private
 */
goog.fs.url.UrlObject_ = function() {};

/**
 * @param {!File|!Blob|!MediaSource|!MediaStream} arg
 * @return {string}
 */
goog.fs.url.UrlObject_.prototype.createObjectURL = function(arg) {};

/**
 * @param {string} s
 */
goog.fs.url.UrlObject_.prototype.revokeObjectURL = function(s) {};


/**
 * Get the object that has the createObjectURL and revokeObjectURL functions for
 * this browser.
 *
 * @return {!goog.fs.url.UrlObject_} The object for this browser.
 * @private
 */
goog.fs.url.getUrlObject_ = function() {
  const urlObject = goog.fs.url.findUrlObject_();
  if (urlObject != null) {
    return urlObject;
  } else {
    throw new Error('This browser doesn\'t seem to support blob URLs');
  }
};


/**
 * Finds the object that has the createObjectURL and revokeObjectURL functions
 * for this browser.
 *
 * @return {?goog.fs.url.UrlObject_} The object for this browser or null if the
 *     browser does not support Object Urls.
 * @private
 */
goog.fs.url.findUrlObject_ = function() {
  // This is what the spec says to do
  // http://dev.w3.org/2006/webapi/FileAPI/#dfn-createObjectURL
  if (goog.global.URL !== undefined &&
      goog.global.URL.createObjectURL !== undefined) {
    return /** @type {!goog.fs.url.UrlObject_} */ (goog.global.URL);
    // This is what Chrome does (as of 10.0.648.6 dev)
  } else if (
      goog.global.webkitURL !== undefined &&
      goog.global.webkitURL.createObjectURL !== undefined) {
    return /** @type {!goog.fs.url.UrlObject_} */ (goog.global.webkitURL);
    // This is what the spec used to say to do
  } else if (goog.global.createObjectURL !== undefined) {
    return /** @type {!goog.fs.url.UrlObject_} */ (goog.global);
  } else {
    return null;
  }
};


/**
 * Checks whether this browser supports Object Urls. If not, calls to
 * createObjectUrl and revokeObjectUrl will result in an error.
 *
 * @return {boolean} True if this browser supports Object Urls.
 */
goog.fs.url.browserSupportsObjectUrls = function() {
  return goog.fs.url.findUrlObject_() != null;
};

//third_party/javascript/closure/fs/blob.js
/**
 * @license
 * Copyright The Closure Library Authors.
 * SPDX-License-Identifier: Apache-2.0
 */

/**
 * @fileoverview Wrappers for the HTML5 File API. These wrappers closely mirror
 * the underlying APIs, but use Closure-style events and Deferred return values.
 * Their existence also makes it possible to mock the FileSystem API for testing
 * in browsers that don't support it natively.
 *
 * When adding public functions to anything under this namespace, be sure to add
 * its mock counterpart to goog.testing.fs.
 */

goog.provide('goog.fs.blob');

goog.require('goog.array');

/**
 * Concatenates one or more values together and converts them to a Blob.
 *
 * @param {...(string|!Blob|!ArrayBuffer)} var_args The values that will make up
 *     the resulting blob.
 * @return {!Blob} The blob.
 */
goog.fs.blob.getBlob = function(var_args) {
  var BlobBuilder = goog.global.BlobBuilder || goog.global.WebKitBlobBuilder;

  if (BlobBuilder !== undefined) {
    var bb = new BlobBuilder();
    for (var i = 0; i < arguments.length; i++) {
      bb.append(arguments[i]);
    }
    return bb.getBlob();
  } else {
    return goog.fs.blob.getBlobWithProperties(goog.array.toArray(arguments));
  }
};


/**
 * Creates a blob with the given properties.
 * See https://developer.mozilla.org/en-US/docs/Web/API/Blob for more details.
 *
 * @param {!Array<string|!Blob|!ArrayBuffer>} parts The values that will make up
 *     the resulting blob (subset supported by both BlobBuilder.append() and
 *     Blob constructor).
 * @param {string=} opt_type The MIME type of the Blob.
 * @param {string=} opt_endings Specifies how strings containing newlines are to
 *     be written out.
 * @return {!Blob} The blob.
 */
goog.fs.blob.getBlobWithProperties = function(parts, opt_type, opt_endings) {
  var BlobBuilder = goog.global.BlobBuilder || goog.global.WebKitBlobBuilder;

  if (BlobBuilder !== undefined) {
    var bb = new BlobBuilder();
    for (var i = 0; i < parts.length; i++) {
      bb.append(parts[i], opt_endings);
    }
    return bb.getBlob(opt_type);
  } else if (goog.global.Blob !== undefined) {
    var properties = {};
    if (opt_type) {
      properties['type'] = opt_type;
    }
    if (opt_endings) {
      properties['endings'] = opt_endings;
    }
    return new Blob(parts, properties);
  } else {
    throw new Error('This browser doesn\'t seem to support creating Blobs');
  }
};

//third_party/javascript/closure/html/trustedresourceurl.js
/**
 * @license
 * Copyright The Closure Library Authors.
 * SPDX-License-Identifier: Apache-2.0
 */

/**
 * @fileoverview The TrustedResourceUrl type and its builders.
 *
 * TODO(xtof): Link to document stating type contract.
 */

goog.provide('goog.html.TrustedResourceUrl');

goog.require('goog.asserts');
goog.require('goog.fs.blob');
goog.require('goog.fs.url');
goog.require('goog.html.SafeScript');
goog.require('goog.html.trustedtypes');
goog.require('goog.i18n.bidi.Dir');
goog.require('goog.i18n.bidi.DirectionalString');
goog.require('goog.string.Const');
goog.require('goog.string.TypedString');



/**
 * A URL which is under application control and from which script, CSS, and
 * other resources that represent executable code, can be fetched.
 *
 * Given that the URL can only be constructed from strings under application
 * control and is used to load resources, bugs resulting in a malformed URL
 * should not have a security impact and are likely to be easily detectable
 * during testing. Given the wide number of non-RFC compliant URLs in use,
 * stricter validation could prevent some applications from being able to use
 * this type.
 *
 * Instances of this type must be created via the factory method,
 * (`fromConstant`, `fromConstants`, `format` or
 * `formatWithParams`), and not by invoking its constructor. The constructor
 * is organized in a way that only methods from that file can call it and
 * initialize with non-empty values. Anyone else calling constructor will
 * get default instance with empty value.
 *
 * @see goog.html.TrustedResourceUrl#fromConstant
 * @constructor
 * @final
 * @struct
 * @implements {goog.i18n.bidi.DirectionalString}
 * @implements {goog.string.TypedString}
 * @param {!Object=} opt_token package-internal implementation detail.
 * @param {!TrustedScriptURL|string=} opt_content package-internal
 *     implementation detail.
 */
goog.html.TrustedResourceUrl = function(opt_token, opt_content) {
  /**
   * The contained value of this TrustedResourceUrl.  The field has a purposely
   * ugly name to make (non-compiled) code that attempts to directly access this
   * field stand out.
   * @const
   * @private {!TrustedScriptURL|string}
   */
  this.privateDoNotAccessOrElseTrustedResourceUrlWrappedValue_ =
      ((opt_token ===
        goog.html.TrustedResourceUrl.CONSTRUCTOR_TOKEN_PRIVATE_) &&
       opt_content) ||
      '';

  /**
   * A type marker used to implement additional run-time type checking.
   * @see goog.html.TrustedResourceUrl#unwrap
   * @const {!Object}
   * @private
   */
  this.TRUSTED_RESOURCE_URL_TYPE_MARKER_GOOG_HTML_SECURITY_PRIVATE_ =
      goog.html.TrustedResourceUrl.TYPE_MARKER_GOOG_HTML_SECURITY_PRIVATE_;
};


/**
 * @override
 * @const
 */
goog.html.TrustedResourceUrl.prototype.implementsGoogStringTypedString = true;


/**
 * Returns this TrustedResourceUrl's value as a string.
 *
 * IMPORTANT: In code where it is security relevant that an object's type is
 * indeed `TrustedResourceUrl`, use
 * `goog.html.TrustedResourceUrl.unwrap` instead of this method. If in
 * doubt, assume that it's security relevant. In particular, note that
 * goog.html functions which return a goog.html type do not guarantee that
 * the returned instance is of the right type. For example:
 *
 * <pre>
 * var fakeSafeHtml = new String('fake');
 * fakeSafeHtml.__proto__ = goog.html.SafeHtml.prototype;
 * var newSafeHtml = goog.html.SafeHtml.htmlEscape(fakeSafeHtml);
 * // newSafeHtml is just an alias for fakeSafeHtml, it's passed through by
 * // goog.html.SafeHtml.htmlEscape() as fakeSafeHtml instanceof
 * // goog.html.SafeHtml.
 * </pre>
 *
 * @see goog.html.TrustedResourceUrl#unwrap
 * @override
 */
goog.html.TrustedResourceUrl.prototype.getTypedStringValue = function() {
  return this.privateDoNotAccessOrElseTrustedResourceUrlWrappedValue_
      .toString();
};


/**
 * @override
 * @const
 */
goog.html.TrustedResourceUrl.prototype.implementsGoogI18nBidiDirectionalString =
    true;


/**
 * Returns this URLs directionality, which is always `LTR`.
 * @override
 */
goog.html.TrustedResourceUrl.prototype.getDirection = function() {
  return goog.i18n.bidi.Dir.LTR;
};


/**
 * Creates a new TrustedResourceUrl with params added to URL. Both search and
 * hash params can be specified.
 *
 * @param {string|?Object<string, *>|undefined} searchParams Search parameters
 *     to add to URL. See goog.html.TrustedResourceUrl.stringifyParams_ for
 *     exact format definition.
 * @param {(string|?Object<string, *>)=} opt_hashParams Hash parameters to add
 *     to URL. See goog.html.TrustedResourceUrl.stringifyParams_ for exact
 *     format definition.
 * @return {!goog.html.TrustedResourceUrl} New TrustedResourceUrl with params.
 */
goog.html.TrustedResourceUrl.prototype.cloneWithParams = function(
    searchParams, opt_hashParams) {
  var url = goog.html.TrustedResourceUrl.unwrap(this);
  var parts = goog.html.TrustedResourceUrl.URL_PARAM_PARSER_.exec(url);
  var urlBase = parts[1];
  var urlSearch = parts[2] || '';
  var urlHash = parts[3] || '';

  return goog.html.TrustedResourceUrl
      .createTrustedResourceUrlSecurityPrivateDoNotAccessOrElse(
          urlBase +
          goog.html.TrustedResourceUrl.stringifyParams_(
              '?', urlSearch, searchParams) +
          goog.html.TrustedResourceUrl.stringifyParams_(
              '#', urlHash, opt_hashParams));
};


if (goog.DEBUG) {
  /**
   * Returns a debug string-representation of this value.
   *
   * To obtain the actual string value wrapped in a TrustedResourceUrl, use
   * `goog.html.TrustedResourceUrl.unwrap`.
   *
   * @see goog.html.TrustedResourceUrl#unwrap
   * @override
   */
  goog.html.TrustedResourceUrl.prototype.toString = function() {
    return 'TrustedResourceUrl{' +
        this.privateDoNotAccessOrElseTrustedResourceUrlWrappedValue_ + '}';
  };
}


/**
 * Performs a runtime check that the provided object is indeed a
 * TrustedResourceUrl object, and returns its value.
 *
 * @param {!goog.html.TrustedResourceUrl} trustedResourceUrl The object to
 *     extract from.
 * @return {string} The trustedResourceUrl object's contained string, unless
 *     the run-time type check fails. In that case, `unwrap` returns an
 *     innocuous string, or, if assertions are enabled, throws
 *     `goog.asserts.AssertionError`.
 */
goog.html.TrustedResourceUrl.unwrap = function(trustedResourceUrl) {
  return goog.html.TrustedResourceUrl.unwrapTrustedScriptURL(trustedResourceUrl)
      .toString();
};


/**
 * Unwraps value as TrustedScriptURL if supported or as a string if not.
 * @param {!goog.html.TrustedResourceUrl} trustedResourceUrl
 * @return {!TrustedScriptURL|string}
 * @see goog.html.TrustedResourceUrl.unwrap
 */
goog.html.TrustedResourceUrl.unwrapTrustedScriptURL = function(
    trustedResourceUrl) {
  // Perform additional Run-time type-checking to ensure that
  // trustedResourceUrl is indeed an instance of the expected type.  This
  // provides some additional protection against security bugs due to
  // application code that disables type checks.
  // Specifically, the following checks are performed:
  // 1. The object is an instance of the expected type.
  // 2. The object is not an instance of a subclass.
  // 3. The object carries a type marker for the expected type. "Faking" an
  // object requires a reference to the type marker, which has names intended
  // to stand out in code reviews.
  if (trustedResourceUrl instanceof goog.html.TrustedResourceUrl &&
      trustedResourceUrl.constructor === goog.html.TrustedResourceUrl &&
      trustedResourceUrl
              .TRUSTED_RESOURCE_URL_TYPE_MARKER_GOOG_HTML_SECURITY_PRIVATE_ ===
          goog.html.TrustedResourceUrl
              .TYPE_MARKER_GOOG_HTML_SECURITY_PRIVATE_) {
    return trustedResourceUrl
        .privateDoNotAccessOrElseTrustedResourceUrlWrappedValue_;
  } else {
    goog.asserts.fail('expected object of type TrustedResourceUrl, got \'' +
        trustedResourceUrl + '\' of type ' + goog.typeOf(trustedResourceUrl));
    return 'type_error:TrustedResourceUrl';
  }
};


/**
 * Creates a TrustedResourceUrl from a format string and arguments.
 *
 * The arguments for interpolation into the format string map labels to values.
 * Values of type `goog.string.Const` are interpolated without modifcation.
 * Values of other types are cast to string and encoded with
 * encodeURIComponent.
 *
 * `%{<label>}` markers are used in the format string to indicate locations
 * to be interpolated with the valued mapped to the given label. `<label>`
 * must contain only alphanumeric and `_` characters.
 *
 * The format string must match goog.html.TrustedResourceUrl.BASE_URL_.
 *
 * Example usage:
 *
 *    var url = goog.html.TrustedResourceUrl.format(goog.string.Const.from(
 *        'https://www.google.com/search?q=%{query}'), {'query': searchTerm});
 *
 *    var url = goog.html.TrustedResourceUrl.format(goog.string.Const.from(
 *        '//www.youtube.com/v/%{videoId}?hl=en&fs=1%{autoplay}'), {
 *        'videoId': videoId,
 *        'autoplay': opt_autoplay ?
 *            goog.string.Const.from('&autoplay=1') : goog.string.Const.EMPTY
 *    });
 *
 * While this function can be used to create a TrustedResourceUrl from only
 * constants, fromConstant() and fromConstants() are generally preferable for
 * that purpose.
 *
 * @param {!goog.string.Const} format The format string.
 * @param {!Object<string, (string|number|!goog.string.Const)>} args Mapping
 *     of labels to values to be interpolated into the format string.
 *     goog.string.Const values are interpolated without encoding.
 * @return {!goog.html.TrustedResourceUrl}
 * @throws {!Error} On an invalid format string or if a label used in the
 *     the format string is not present in args.
 */
goog.html.TrustedResourceUrl.format = function(format, args) {
  var formatStr = goog.string.Const.unwrap(format);
  if (!goog.html.TrustedResourceUrl.BASE_URL_.test(formatStr)) {
    throw new Error('Invalid TrustedResourceUrl format: ' + formatStr);
  }
  var result = formatStr.replace(
      goog.html.TrustedResourceUrl.FORMAT_MARKER_, function(match, id) {
        if (!Object.prototype.hasOwnProperty.call(args, id)) {
          throw new Error(
              'Found marker, "' + id + '", in format string, "' + formatStr +
              '", but no valid label mapping found ' +
              'in args: ' + JSON.stringify(args));
        }
        var arg = args[id];
        if (arg instanceof goog.string.Const) {
          return goog.string.Const.unwrap(arg);
        } else {
          return encodeURIComponent(String(arg));
        }
      });
  return goog.html.TrustedResourceUrl
      .createTrustedResourceUrlSecurityPrivateDoNotAccessOrElse(result);
};


/**
 * @private @const {!RegExp}
 */
goog.html.TrustedResourceUrl.FORMAT_MARKER_ = /%{(\w+)}/g;


/**
 * The URL must be absolute, scheme-relative or path-absolute. So it must
 * start with:
 * - https:// followed by allowed origin characters.
 * - // followed by allowed origin characters.
 * - Any absolute or relative path.
 *
 * Based on
 * https://url.spec.whatwg.org/commit-snapshots/56b74ce7cca8883eab62e9a12666e2fac665d03d/#url-parsing
 * an initial / which is not followed by another / or \ will end up in the "path
 * state" and from there it can only go to "fragment state" and "query state".
 *
 * We don't enforce a well-formed domain name. So '.' or '1.2' are valid.
 * That's ok because the origin comes from a compile-time constant.
 *
 * A regular expression is used instead of goog.uri for several reasons:
 * - Strictness. E.g. we don't want any userinfo component and we don't
 *   want '/./, nor \' in the first path component.
 * - Small trusted base. goog.uri is generic and might need to change,
 *   reasoning about all the ways it can parse a URL now and in the future
 *   is error-prone.
 * - Code size. We expect many calls to .format(), many of which might
 *   not be using goog.uri.
 * - Simplicity. Using goog.uri would likely not result in simpler nor shorter
 *   code.
 * @private @const {!RegExp}
 */
goog.html.TrustedResourceUrl.BASE_URL_ = new RegExp(
    '^((https:)?//[0-9a-z.:[\\]-]+/'  // Origin.
        + '|/[^/\\\\]'                // Absolute path.
        + '|[^:/\\\\%]+/'             // Relative path.
        + '|[^:/\\\\%]*[?#]'          // Query string or fragment.
        + '|about:blank#'             // about:blank with fragment.
        + ')',
    'i');

/**
 * RegExp for splitting a URL into the base, search field, and hash field.
 *
 * @private @const {!RegExp}
 */
goog.html.TrustedResourceUrl.URL_PARAM_PARSER_ =
    /^([^?#]*)(\?[^#]*)?(#[\s\S]*)?/;


/**
 * Formats the URL same as TrustedResourceUrl.format and then adds extra URL
 * parameters.
 *
 * Example usage:
 *
 *     // Creates '//www.youtube.com/v/abc?autoplay=1' for videoId='abc' and
 *     // opt_autoplay=1. Creates '//www.youtube.com/v/abc' for videoId='abc'
 *     // and opt_autoplay=undefined.
 *     var url = goog.html.TrustedResourceUrl.formatWithParams(
 *         goog.string.Const.from('//www.youtube.com/v/%{videoId}'),
 *         {'videoId': videoId},
 *         {'autoplay': opt_autoplay});
 *
 * @param {!goog.string.Const} format The format string.
 * @param {!Object<string, (string|number|!goog.string.Const)>} args Mapping
 *     of labels to values to be interpolated into the format string.
 *     goog.string.Const values are interpolated without encoding.
 * @param {string|?Object<string, *>|undefined} searchParams Parameters to add
 *     to URL. See goog.html.TrustedResourceUrl.stringifyParams_ for exact
 *     format definition.
 * @param {(string|?Object<string, *>)=} opt_hashParams Hash parameters to add
 *     to URL. See goog.html.TrustedResourceUrl.stringifyParams_ for exact
 *     format definition.
 * @return {!goog.html.TrustedResourceUrl}
 * @throws {!Error} On an invalid format string or if a label used in the
 *     the format string is not present in args.
 */
goog.html.TrustedResourceUrl.formatWithParams = function(
    format, args, searchParams, opt_hashParams) {
  var url = goog.html.TrustedResourceUrl.format(format, args);
  return url.cloneWithParams(searchParams, opt_hashParams);
};


/**
 * Creates a TrustedResourceUrl object from a compile-time constant string.
 *
 * Compile-time constant strings are inherently program-controlled and hence
 * trusted.
 *
 * @param {!goog.string.Const} url A compile-time-constant string from which to
 *     create a TrustedResourceUrl.
 * @return {!goog.html.TrustedResourceUrl} A TrustedResourceUrl object
 *     initialized to `url`.
 */
goog.html.TrustedResourceUrl.fromConstant = function(url) {
  return goog.html.TrustedResourceUrl
      .createTrustedResourceUrlSecurityPrivateDoNotAccessOrElse(
          goog.string.Const.unwrap(url));
};


/**
 * Creates a TrustedResourceUrl object from a compile-time constant strings.
 *
 * Compile-time constant strings are inherently program-controlled and hence
 * trusted.
 *
 * @param {!Array<!goog.string.Const>} parts Compile-time-constant strings from
 *     which to create a TrustedResourceUrl.
 * @return {!goog.html.TrustedResourceUrl} A TrustedResourceUrl object
 *     initialized to concatenation of `parts`.
 */
goog.html.TrustedResourceUrl.fromConstants = function(parts) {
  var unwrapped = '';
  for (var i = 0; i < parts.length; i++) {
    unwrapped += goog.string.Const.unwrap(parts[i]);
  }
  return goog.html.TrustedResourceUrl
      .createTrustedResourceUrlSecurityPrivateDoNotAccessOrElse(unwrapped);
};

/**
 * Creates a TrustedResourceUrl object by generating a Blob from a SafeScript
 * object and then calling createObjectURL with that blob.
 *
 * SafeScript objects are trusted to contain executable JavaScript code.
 *
 * Caller must call goog.fs.url.revokeObjectUrl() on the unwrapped url to
 * release the underlying blob.
 *
 * Throws if browser doesn't support blob construction.
 *
 * @param {!goog.html.SafeScript} safeScript A script from which to create a
 *     TrustedResourceUrl.
 * @return {!goog.html.TrustedResourceUrl} A TrustedResourceUrl object
 *     initialized to a new blob URL.
 */
goog.html.TrustedResourceUrl.fromSafeScript = function(safeScript) {
  var blob = goog.fs.blob.getBlobWithProperties(
      [goog.html.SafeScript.unwrap(safeScript)], 'text/javascript');
  var url = goog.fs.url.createObjectUrl(blob);
  return goog.html.TrustedResourceUrl
      .createTrustedResourceUrlSecurityPrivateDoNotAccessOrElse(url);
};


/**
 * Type marker for the TrustedResourceUrl type, used to implement additional
 * run-time type checking.
 * @const {!Object}
 * @private
 */
goog.html.TrustedResourceUrl.TYPE_MARKER_GOOG_HTML_SECURITY_PRIVATE_ = {};


/**
 * Package-internal utility method to create TrustedResourceUrl instances.
 *
 * @param {string} url The string to initialize the TrustedResourceUrl object
 *     with.
 * @return {!goog.html.TrustedResourceUrl} The initialized TrustedResourceUrl
 *     object.
 * @package
 */
goog.html.TrustedResourceUrl
    .createTrustedResourceUrlSecurityPrivateDoNotAccessOrElse = function(url) {
  var value = goog.html.trustedtypes.PRIVATE_DO_NOT_ACCESS_OR_ELSE_POLICY ?
      goog.html.trustedtypes.PRIVATE_DO_NOT_ACCESS_OR_ELSE_POLICY
          .createScriptURL(url) :
      url;
  return new goog.html.TrustedResourceUrl(
      goog.html.TrustedResourceUrl.CONSTRUCTOR_TOKEN_PRIVATE_, value);
};


/**
 * Stringifies the passed params to be used as either a search or hash field of
 * a URL.
 *
 * @param {string} prefix The prefix character for the given field ('?' or '#').
 * @param {string} currentString The existing field value (including the prefix
 *     character, if the field is present).
 * @param {string|?Object<string, *>|undefined} params The params to set or
 *     append to the field.
 * - If `undefined` or `null`, the field remains unchanged.
 * - If a string, then the string will be escaped and the field will be
 *   overwritten with that value.
 * - If an Object, that object is treated as a set of key-value pairs to be
 *   appended to the current field. Note that JavaScript doesn't guarantee the
 *   order of values in an object which might result in non-deterministic order
 *   of the parameters. However, browsers currently preserve the order. The
 *   rules for each entry:
 *   - If an array, it will be processed as if each entry were an additional
 *     parameter with exactly the same key, following the same logic below.
 *   - If `undefined` or `null`, it will be skipped.
 *   - Otherwise, it will be turned into a string, escaped, and appended.
 * @return {string}
 * @private
 */
goog.html.TrustedResourceUrl.stringifyParams_ = function(
    prefix, currentString, params) {
  if (params == null) {
    // Do not modify the field.
    return currentString;
  }
  if (typeof params === 'string') {
    // Set field to the passed string.
    return params ? prefix + encodeURIComponent(params) : '';
  }
  // Add on parameters to field from key-value object.
  for (var key in params) {
    var value = params[key];
    var outputValues = Array.isArray(value) ? value : [value];
    for (var i = 0; i < outputValues.length; i++) {
      var outputValue = outputValues[i];
      if (outputValue != null) {
        if (!currentString) {
          currentString = prefix;
        }
        currentString += (currentString.length > prefix.length ? '&' : '') +
            encodeURIComponent(key) + '=' +
            encodeURIComponent(String(outputValue));
      }
    }
  }
  return currentString;
};

/**
 * Token used to ensure that object is created only from this file. No code
 * outside of this file can access this token.
 * @private {!Object}
 * @const
 */
goog.html.TrustedResourceUrl.CONSTRUCTOR_TOKEN_PRIVATE_ = {};

//third_party/javascript/closure/string/internal.js
/**
 * @license
 * Copyright The Closure Library Authors.
 * SPDX-License-Identifier: Apache-2.0
 */

/**
 * @fileoverview String functions called from Closure packages that couldn't
 * depend on each other. Outside Closure, use goog.string function which
 * delegate to these.
 * @visibility {//javascript/closure:__pkg__}
 * @visibility {//javascript/closure/bin/sizetests:__pkg__}
 * @visibility {//javascript/closure/dom:__pkg__}
 * @visibility {//javascript/closure/html:__pkg__}
 * @visibility {//javascript/closure/labs/useragent:__pkg__}
 */


goog.provide('goog.string.internal');


/**
 * Fast prefix-checker.
 * @param {string} str The string to check.
 * @param {string} prefix A string to look for at the start of `str`.
 * @return {boolean} True if `str` begins with `prefix`.
 * @see goog.string.startsWith
 */
goog.string.internal.startsWith = function(str, prefix) {
  return str.lastIndexOf(prefix, 0) == 0;
};


/**
 * Fast suffix-checker.
 * @param {string} str The string to check.
 * @param {string} suffix A string to look for at the end of `str`.
 * @return {boolean} True if `str` ends with `suffix`.
 * @see goog.string.endsWith
 */
goog.string.internal.endsWith = function(str, suffix) {
  const l = str.length - suffix.length;
  return l >= 0 && str.indexOf(suffix, l) == l;
};


/**
 * Case-insensitive prefix-checker.
 * @param {string} str The string to check.
 * @param {string} prefix  A string to look for at the end of `str`.
 * @return {boolean} True if `str` begins with `prefix` (ignoring
 *     case).
 * @see goog.string.caseInsensitiveStartsWith
 */
goog.string.internal.caseInsensitiveStartsWith = function(str, prefix) {
  return goog.string.internal.caseInsensitiveCompare(
             prefix, str.substr(0, prefix.length)) == 0;
};


/**
 * Case-insensitive suffix-checker.
 * @param {string} str The string to check.
 * @param {string} suffix A string to look for at the end of `str`.
 * @return {boolean} True if `str` ends with `suffix` (ignoring
 *     case).
 * @see goog.string.caseInsensitiveEndsWith
 */
goog.string.internal.caseInsensitiveEndsWith = function(str, suffix) {
  return (
      goog.string.internal.caseInsensitiveCompare(
          suffix, str.substr(str.length - suffix.length, suffix.length)) == 0);
};


/**
 * Case-insensitive equality checker.
 * @param {string} str1 First string to check.
 * @param {string} str2 Second string to check.
 * @return {boolean} True if `str1` and `str2` are the same string,
 *     ignoring case.
 * @see goog.string.caseInsensitiveEquals
 */
goog.string.internal.caseInsensitiveEquals = function(str1, str2) {
  return str1.toLowerCase() == str2.toLowerCase();
};


/**
 * Checks if a string is empty or contains only whitespaces.
 * @param {string} str The string to check.
 * @return {boolean} Whether `str` is empty or whitespace only.
 * @see goog.string.isEmptyOrWhitespace
 */
goog.string.internal.isEmptyOrWhitespace = function(str) {
  // testing length == 0 first is actually slower in all browsers (about the
  // same in Opera).
  // Since IE doesn't include non-breaking-space (0xa0) in their \s character
  // class (as required by section 7.2 of the ECMAScript spec), we explicitly
  // include it in the regexp to enforce consistent cross-browser behavior.
  return /^[\s\xa0]*$/.test(str);
};


/**
 * Trims white spaces to the left and right of a string.
 * @param {string} str The string to trim.
 * @return {string} A trimmed copy of `str`.
 */
goog.string.internal.trim =
    (goog.TRUSTED_SITE && String.prototype.trim) ? function(str) {
      return str.trim();
    } : function(str) {
      // Since IE doesn't include non-breaking-space (0xa0) in their \s
      // character class (as required by section 7.2 of the ECMAScript spec),
      // we explicitly include it in the regexp to enforce consistent
      // cross-browser behavior.
      // NOTE: We don't use String#replace because it might have side effects
      // causing this function to not compile to 0 bytes.
      return /^[\s\xa0]*([\s\S]*?)[\s\xa0]*$/.exec(str)[1];
    };


/**
 * A string comparator that ignores case.
 * -1 = str1 less than str2
 *  0 = str1 equals str2
 *  1 = str1 greater than str2
 *
 * @param {string} str1 The string to compare.
 * @param {string} str2 The string to compare `str1` to.
 * @return {number} The comparator result, as described above.
 * @see goog.string.caseInsensitiveCompare
 */
goog.string.internal.caseInsensitiveCompare = function(str1, str2) {
  const test1 = String(str1).toLowerCase();
  const test2 = String(str2).toLowerCase();

  if (test1 < test2) {
    return -1;
  } else if (test1 == test2) {
    return 0;
  } else {
    return 1;
  }
};


/**
 * Converts \n to <br>s or <br />s.
 * @param {string} str The string in which to convert newlines.
 * @param {boolean=} opt_xml Whether to use XML compatible tags.
 * @return {string} A copy of `str` with converted newlines.
 * @see goog.string.newLineToBr
 */
goog.string.internal.newLineToBr = function(str, opt_xml) {
  return str.replace(/(\r\n|\r|\n)/g, opt_xml ? '<br />' : '<br>');
};


/**
 * Escapes double quote '"' and single quote '\'' characters in addition to
 * '&', '<', and '>' so that a string can be included in an HTML tag attribute
 * value within double or single quotes.
 * @param {string} str string to be escaped.
 * @param {boolean=} opt_isLikelyToContainHtmlChars
 * @return {string} An escaped copy of `str`.
 * @see goog.string.htmlEscape
 */
goog.string.internal.htmlEscape = function(
    str, opt_isLikelyToContainHtmlChars) {
  if (opt_isLikelyToContainHtmlChars) {
    str = str.replace(goog.string.internal.AMP_RE_, '&amp;')
              .replace(goog.string.internal.LT_RE_, '&lt;')
              .replace(goog.string.internal.GT_RE_, '&gt;')
              .replace(goog.string.internal.QUOT_RE_, '&quot;')
              .replace(goog.string.internal.SINGLE_QUOTE_RE_, '&#39;')
              .replace(goog.string.internal.NULL_RE_, '&#0;');
    return str;

  } else {
    // quick test helps in the case when there are no chars to replace, in
    // worst case this makes barely a difference to the time taken
    if (!goog.string.internal.ALL_RE_.test(str)) return str;

    // str.indexOf is faster than regex.test in this case
    if (str.indexOf('&') != -1) {
      str = str.replace(goog.string.internal.AMP_RE_, '&amp;');
    }
    if (str.indexOf('<') != -1) {
      str = str.replace(goog.string.internal.LT_RE_, '&lt;');
    }
    if (str.indexOf('>') != -1) {
      str = str.replace(goog.string.internal.GT_RE_, '&gt;');
    }
    if (str.indexOf('"') != -1) {
      str = str.replace(goog.string.internal.QUOT_RE_, '&quot;');
    }
    if (str.indexOf('\'') != -1) {
      str = str.replace(goog.string.internal.SINGLE_QUOTE_RE_, '&#39;');
    }
    if (str.indexOf('\x00') != -1) {
      str = str.replace(goog.string.internal.NULL_RE_, '&#0;');
    }
    return str;
  }
};


/**
 * Regular expression that matches an ampersand, for use in escaping.
 * @const {!RegExp}
 * @private
 */
goog.string.internal.AMP_RE_ = /&/g;


/**
 * Regular expression that matches a less than sign, for use in escaping.
 * @const {!RegExp}
 * @private
 */
goog.string.internal.LT_RE_ = /</g;


/**
 * Regular expression that matches a greater than sign, for use in escaping.
 * @const {!RegExp}
 * @private
 */
goog.string.internal.GT_RE_ = />/g;


/**
 * Regular expression that matches a double quote, for use in escaping.
 * @const {!RegExp}
 * @private
 */
goog.string.internal.QUOT_RE_ = /"/g;


/**
 * Regular expression that matches a single quote, for use in escaping.
 * @const {!RegExp}
 * @private
 */
goog.string.internal.SINGLE_QUOTE_RE_ = /'/g;


/**
 * Regular expression that matches null character, for use in escaping.
 * @const {!RegExp}
 * @private
 */
goog.string.internal.NULL_RE_ = /\x00/g;


/**
 * Regular expression that matches any character that needs to be escaped.
 * @const {!RegExp}
 * @private
 */
goog.string.internal.ALL_RE_ = /[\x00&<>"']/;


/**
 * Do escaping of whitespace to preserve spatial formatting. We use character
 * entity #160 to make it safer for xml.
 * @param {string} str The string in which to escape whitespace.
 * @param {boolean=} opt_xml Whether to use XML compatible tags.
 * @return {string} An escaped copy of `str`.
 * @see goog.string.whitespaceEscape
 */
goog.string.internal.whitespaceEscape = function(str, opt_xml) {
  // This doesn't use goog.string.preserveSpaces for backwards compatibility.
  return goog.string.internal.newLineToBr(
      str.replace(/  /g, ' &#160;'), opt_xml);
};


/**
 * Determines whether a string contains a substring.
 * @param {string} str The string to search.
 * @param {string} subString The substring to search for.
 * @return {boolean} Whether `str` contains `subString`.
 * @see goog.string.contains
 */
goog.string.internal.contains = function(str, subString) {
  return str.indexOf(subString) != -1;
};


/**
 * Determines whether a string contains a substring, ignoring case.
 * @param {string} str The string to search.
 * @param {string} subString The substring to search for.
 * @return {boolean} Whether `str` contains `subString`.
 * @see goog.string.caseInsensitiveContains
 */
goog.string.internal.caseInsensitiveContains = function(str, subString) {
  return goog.string.internal.contains(
      str.toLowerCase(), subString.toLowerCase());
};


/**
 * Compares two version numbers.
 *
 * @param {string|number} version1 Version of first item.
 * @param {string|number} version2 Version of second item.
 *
 * @return {number}  1 if `version1` is higher.
 *                   0 if arguments are equal.
 *                  -1 if `version2` is higher.
 * @see goog.string.compareVersions
 */
goog.string.internal.compareVersions = function(version1, version2) {
  let order = 0;
  // Trim leading and trailing whitespace and split the versions into
  // subversions.
  const v1Subs = goog.string.internal.trim(String(version1)).split('.');
  const v2Subs = goog.string.internal.trim(String(version2)).split('.');
  const subCount = Math.max(v1Subs.length, v2Subs.length);

  // Iterate over the subversions, as long as they appear to be equivalent.
  for (let subIdx = 0; order == 0 && subIdx < subCount; subIdx++) {
    let v1Sub = v1Subs[subIdx] || '';
    let v2Sub = v2Subs[subIdx] || '';

    do {
      // Split the subversions into pairs of numbers and qualifiers (like 'b').
      // Two different RegExp objects are use to make it clear the code
      // is side-effect free
      const v1Comp = /(\d*)(\D*)(.*)/.exec(v1Sub) || ['', '', '', ''];
      const v2Comp = /(\d*)(\D*)(.*)/.exec(v2Sub) || ['', '', '', ''];
      // Break if there are no more matches.
      if (v1Comp[0].length == 0 && v2Comp[0].length == 0) {
        break;
      }

      // Parse the numeric part of the subversion. A missing number is
      // equivalent to 0.
      const v1CompNum = v1Comp[1].length == 0 ? 0 : parseInt(v1Comp[1], 10);
      const v2CompNum = v2Comp[1].length == 0 ? 0 : parseInt(v2Comp[1], 10);

      // Compare the subversion components. The number has the highest
      // precedence. Next, if the numbers are equal, a subversion without any
      // qualifier is always higher than a subversion with any qualifier. Next,
      // the qualifiers are compared as strings.
      order = goog.string.internal.compareElements_(v1CompNum, v2CompNum) ||
          goog.string.internal.compareElements_(
              v1Comp[2].length == 0, v2Comp[2].length == 0) ||
          goog.string.internal.compareElements_(v1Comp[2], v2Comp[2]);
      // Stop as soon as an inequality is discovered.

      v1Sub = v1Comp[3];
      v2Sub = v2Comp[3];
    } while (order == 0);
  }

  return order;
};


/**
 * Compares elements of a version number.
 *
 * @param {string|number|boolean} left An element from a version number.
 * @param {string|number|boolean} right An element from a version number.
 *
 * @return {number}  1 if `left` is higher.
 *                   0 if arguments are equal.
 *                  -1 if `right` is higher.
 * @private
 */
goog.string.internal.compareElements_ = function(left, right) {
  if (left < right) {
    return -1;
  } else if (left > right) {
    return 1;
  }
  return 0;
};

//third_party/javascript/closure/html/safeurl.js
/**
 * @license
 * Copyright The Closure Library Authors.
 * SPDX-License-Identifier: Apache-2.0
 */

/**
 * @fileoverview The SafeUrl type and its builders.
 *
 * TODO(xtof): Link to document stating type contract.
 */

goog.provide('goog.html.SafeUrl');

goog.require('goog.asserts');
goog.require('goog.fs.url');
goog.require('goog.html.TrustedResourceUrl');
goog.require('goog.i18n.bidi.Dir');
goog.require('goog.i18n.bidi.DirectionalString');
goog.require('goog.string.Const');
goog.require('goog.string.TypedString');
goog.require('goog.string.internal');



/**
 * A string that is safe to use in URL context in DOM APIs and HTML documents.
 *
 * A SafeUrl is a string-like object that carries the security type contract
 * that its value as a string will not cause untrusted script execution
 * when evaluated as a hyperlink URL in a browser.
 *
 * Values of this type are guaranteed to be safe to use in URL/hyperlink
 * contexts, such as assignment to URL-valued DOM properties, in the sense that
 * the use will not result in a Cross-Site-Scripting vulnerability. Similarly,
 * SafeUrls can be interpolated into the URL context of an HTML template (e.g.,
 * inside a href attribute). However, appropriate HTML-escaping must still be
 * applied.
 *
 * Note that, as documented in `goog.html.SafeUrl.unwrap`, this type's
 * contract does not guarantee that instances are safe to interpolate into HTML
 * without appropriate escaping.
 *
 * Note also that this type's contract does not imply any guarantees regarding
 * the resource the URL refers to.  In particular, SafeUrls are <b>not</b>
 * safe to use in a context where the referred-to resource is interpreted as
 * trusted code, e.g., as the src of a script tag.
 *
 * Instances of this type must be created via the factory methods
 * (`goog.html.SafeUrl.fromConstant`, `goog.html.SafeUrl.sanitize`),
 * etc and not by invoking its constructor. The constructor is organized in a
 * way that only methods from that file can call it and initialize with
 * non-empty values. Anyone else calling constructor will get default instance
 * with empty value.
 *
 * @see goog.html.SafeUrl#fromConstant
 * @see goog.html.SafeUrl#from
 * @see goog.html.SafeUrl#sanitize
 * @constructor
 * @final
 * @struct
 * @implements {goog.i18n.bidi.DirectionalString}
 * @implements {goog.string.TypedString}
 * @param {!Object=} opt_token package-internal implementation detail.
 * @param {string=} opt_content package-internal implementation detail.
 */
goog.html.SafeUrl = function(opt_token, opt_content) {
  /**
   * The contained value of this SafeUrl.  The field has a purposely ugly
   * name to make (non-compiled) code that attempts to directly access this
   * field stand out.
   * @private {string}
   */
  this.privateDoNotAccessOrElseSafeUrlWrappedValue_ =
      ((opt_token === goog.html.SafeUrl.CONSTRUCTOR_TOKEN_PRIVATE_) &&
       opt_content) ||
      '';

  /**
   * A type marker used to implement additional run-time type checking.
   * @see goog.html.SafeUrl#unwrap
   * @const {!Object}
   * @private
   */
  this.SAFE_URL_TYPE_MARKER_GOOG_HTML_SECURITY_PRIVATE_ =
      goog.html.SafeUrl.TYPE_MARKER_GOOG_HTML_SECURITY_PRIVATE_;
};


/**
 * The innocuous string generated by goog.html.SafeUrl.sanitize when passed
 * an unsafe URL.
 *
 * about:invalid is registered in
 * http://www.w3.org/TR/css3-values/#about-invalid.
 * http://tools.ietf.org/html/rfc6694#section-2.2.1 permits about URLs to
 * contain a fragment, which is not to be considered when determining if an
 * about URL is well-known.
 *
 * Using about:invalid seems preferable to using a fixed data URL, since
 * browsers might choose to not report CSP violations on it, as legitimate
 * CSS function calls to attr() can result in this URL being produced. It is
 * also a standard URL which matches exactly the semantics we need:
 * "The about:invalid URI references a non-existent document with a generic
 * error condition. It can be used when a URI is necessary, but the default
 * value shouldn't be resolveable as any type of document".
 *
 * @const {string}
 */
goog.html.SafeUrl.INNOCUOUS_STRING = 'about:invalid#zClosurez';


/**
 * @override
 * @const
 */
goog.html.SafeUrl.prototype.implementsGoogStringTypedString = true;


/**
 * Returns this SafeUrl's value as a string.
 *
 * IMPORTANT: In code where it is security relevant that an object's type is
 * indeed `SafeUrl`, use `goog.html.SafeUrl.unwrap` instead of this
 * method. If in doubt, assume that it's security relevant. In particular, note
 * that goog.html functions which return a goog.html type do not guarantee that
 * the returned instance is of the right type.
 *
 * IMPORTANT: The guarantees of the SafeUrl type contract only extend to the
 * behavior of browsers when interpreting URLs. Values of SafeUrl objects MUST
 * be appropriately escaped before embedding in a HTML document. Note that the
 * required escaping is context-sensitive (e.g. a different escaping is
 * required for embedding a URL in a style property within a style
 * attribute, as opposed to embedding in a href attribute).
 *
 * @see goog.html.SafeUrl#unwrap
 * @override
 */
goog.html.SafeUrl.prototype.getTypedStringValue = function() {
  return this.privateDoNotAccessOrElseSafeUrlWrappedValue_.toString();
};


/**
 * @override
 * @const
 */
goog.html.SafeUrl.prototype.implementsGoogI18nBidiDirectionalString = true;


/**
 * Returns this URLs directionality, which is always `LTR`.
 * @override
 */
goog.html.SafeUrl.prototype.getDirection = function() {
  return goog.i18n.bidi.Dir.LTR;
};


if (goog.DEBUG) {
  /**
   * Returns a debug string-representation of this value.
   *
   * To obtain the actual string value wrapped in a SafeUrl, use
   * `goog.html.SafeUrl.unwrap`.
   *
   * @see goog.html.SafeUrl#unwrap
   * @override
   */
  goog.html.SafeUrl.prototype.toString = function() {
    return 'SafeUrl{' + this.privateDoNotAccessOrElseSafeUrlWrappedValue_ + '}';
  };
}


/**
 * Performs a runtime check that the provided object is indeed a SafeUrl
 * object, and returns its value.
 *
 * IMPORTANT: The guarantees of the SafeUrl type contract only extend to the
 * behavior of  browsers when interpreting URLs. Values of SafeUrl objects MUST
 * be appropriately escaped before embedding in a HTML document. Note that the
 * required escaping is context-sensitive (e.g. a different escaping is
 * required for embedding a URL in a style property within a style
 * attribute, as opposed to embedding in a href attribute).
 *
 * @param {!goog.html.SafeUrl} safeUrl The object to extract from.
 * @return {string} The SafeUrl object's contained string, unless the run-time
 *     type check fails. In that case, `unwrap` returns an innocuous
 *     string, or, if assertions are enabled, throws
 *     `goog.asserts.AssertionError`.
 */
goog.html.SafeUrl.unwrap = function(safeUrl) {
  // Perform additional Run-time type-checking to ensure that safeUrl is indeed
  // an instance of the expected type.  This provides some additional protection
  // against security bugs due to application code that disables type checks.
  // Specifically, the following checks are performed:
  // 1. The object is an instance of the expected type.
  // 2. The object is not an instance of a subclass.
  // 3. The object carries a type marker for the expected type. "Faking" an
  // object requires a reference to the type marker, which has names intended
  // to stand out in code reviews.
  if (safeUrl instanceof goog.html.SafeUrl &&
      safeUrl.constructor === goog.html.SafeUrl &&
      safeUrl.SAFE_URL_TYPE_MARKER_GOOG_HTML_SECURITY_PRIVATE_ ===
          goog.html.SafeUrl.TYPE_MARKER_GOOG_HTML_SECURITY_PRIVATE_) {
    return safeUrl.privateDoNotAccessOrElseSafeUrlWrappedValue_;
  } else {
    goog.asserts.fail('expected object of type SafeUrl, got \'' +
        safeUrl + '\' of type ' + goog.typeOf(safeUrl));
    return 'type_error:SafeUrl';
  }
};


/**
 * Creates a SafeUrl object from a compile-time constant string.
 *
 * Compile-time constant strings are inherently program-controlled and hence
 * trusted.
 *
 * @param {!goog.string.Const} url A compile-time-constant string from which to
 *         create a SafeUrl.
 * @return {!goog.html.SafeUrl} A SafeUrl object initialized to `url`.
 */
goog.html.SafeUrl.fromConstant = function(url) {
  return goog.html.SafeUrl.createSafeUrlSecurityPrivateDoNotAccessOrElse(
      goog.string.Const.unwrap(url));
};


/**
 * A pattern that matches Blob or data types that can have SafeUrls created
 * from URL.createObjectURL(blob) or via a data: URI.
 *
 * This has some parameter support (most notably, we haven't implemented the
 * more complex parts like %-encoded characters or non-alphanumerical ones for
 * simplicity's sake). The specs are fairly complex, and they don't
 * always match Chrome's behavior: we settled on a subset where we're confident
 * all parties involved agree.
 *
 * The spec is available at https://mimesniff.spec.whatwg.org/ (and see
 * https://tools.ietf.org/html/rfc2397 for data: urls, which override some of
 * it).
 * @const
 * @private
 */
goog.html.SAFE_MIME_TYPE_PATTERN_ = new RegExp(
    // Note: Due to content-sniffing concerns, only add MIME types for
    // media formats.
    '^(?:audio/(?:3gpp2|3gpp|aac|L16|midi|mp3|mp4|mpeg|oga|ogg|opus|x-m4a|x-wav|wav|webm)|' +
        'image/(?:bmp|gif|jpeg|jpg|png|tiff|webp|x-icon)|' +
        // TODO(b/68188949): Due to content-sniffing concerns, text/csv should
        // be removed from the whitelist.
        'text/csv|' +
        'video/(?:mpeg|mp4|ogg|webm|quicktime))' +
        '(?:;\\w+=(?:\\w+|"[\\w;=]+"))*$',  // MIME type parameters
    'i');


/**
 * @param {string} mimeType The MIME type to check if safe.
 * @return {boolean} True if the MIME type is safe and creating a Blob via
 *   `SafeUrl.fromBlob()` with that type will not fail due to the type. False
 *   otherwise.
 */
goog.html.SafeUrl.isSafeMimeType = function(mimeType) {
  return goog.html.SAFE_MIME_TYPE_PATTERN_.test(mimeType);
};


/**
 * Creates a SafeUrl wrapping a blob URL for the given `blob`.
 *
 * The blob URL is created with `URL.createObjectURL`. If the MIME type
 * for `blob` is not of a known safe audio, image or video MIME type,
 * then the SafeUrl will wrap {@link #INNOCUOUS_STRING}.
 *
 * @see http://www.w3.org/TR/FileAPI/#url
 * @param {!Blob} blob
 * @return {!goog.html.SafeUrl} The blob URL, or an innocuous string wrapped
 *   as a SafeUrl.
 */
goog.html.SafeUrl.fromBlob = function(blob) {
  var url = goog.html.SafeUrl.isSafeMimeType(blob.type) ?
      goog.fs.url.createObjectUrl(blob) :
      goog.html.SafeUrl.INNOCUOUS_STRING;
  return goog.html.SafeUrl.createSafeUrlSecurityPrivateDoNotAccessOrElse(url);
};


/**
 * Creates a SafeUrl wrapping a blob URL created for a MediaSource.
 * @param {!MediaSource} mediaSource
 * @return {!goog.html.SafeUrl} The blob URL.
 */
goog.html.SafeUrl.fromMediaSource = function(mediaSource) {
  goog.asserts.assert(
      'MediaSource' in goog.global, 'No support for MediaSource');
  const url = mediaSource instanceof MediaSource ?
      goog.fs.url.createObjectUrl(mediaSource) :
      goog.html.SafeUrl.INNOCUOUS_STRING;
  return goog.html.SafeUrl.createSafeUrlSecurityPrivateDoNotAccessOrElse(url);
};


/**
 * Matches a base-64 data URL, with the first match group being the MIME type.
 * @const
 * @private
 */
goog.html.DATA_URL_PATTERN_ = /^data:([^,]*);base64,[a-z0-9+\/]+=*$/i;


/**
 * Creates a SafeUrl wrapping a data: URL, after validating it matches a
 * known-safe audio, image or video MIME type.
 *
 * @param {string} dataUrl A valid base64 data URL with one of the whitelisted
 *     audio, image or video MIME types.
 * @return {!goog.html.SafeUrl} A matching safe URL, or {@link INNOCUOUS_STRING}
 *     wrapped as a SafeUrl if it does not pass.
 */
goog.html.SafeUrl.fromDataUrl = function(dataUrl) {
  // RFC4648 suggest to ignore CRLF in base64 encoding.
  // See https://tools.ietf.org/html/rfc4648.
  // Remove the CR (%0D) and LF (%0A) from the dataUrl.
  var filteredDataUrl = dataUrl.replace(/(%0A|%0D)/g, '');
  // There's a slight risk here that a browser sniffs the content type if it
  // doesn't know the MIME type and executes HTML within the data: URL. For this
  // to cause XSS it would also have to execute the HTML in the same origin
  // of the page with the link. It seems unlikely that both of these will
  // happen, particularly in not really old IEs.
  var match = filteredDataUrl.match(goog.html.DATA_URL_PATTERN_);
  var valid = match && goog.html.SafeUrl.isSafeMimeType(match[1]);
  return goog.html.SafeUrl.createSafeUrlSecurityPrivateDoNotAccessOrElse(
      valid ? filteredDataUrl : goog.html.SafeUrl.INNOCUOUS_STRING);
};


/**
 * Creates a SafeUrl wrapping a tel: URL.
 *
 * @param {string} telUrl A tel URL.
 * @return {!goog.html.SafeUrl} A matching safe URL, or {@link INNOCUOUS_STRING}
 *     wrapped as a SafeUrl if it does not pass.
 */
goog.html.SafeUrl.fromTelUrl = function(telUrl) {
  // There's a risk that a tel: URL could immediately place a call once
  // clicked, without requiring user confirmation. For that reason it is
  // handled in this separate function.
  if (!goog.string.internal.caseInsensitiveStartsWith(telUrl, 'tel:')) {
    telUrl = goog.html.SafeUrl.INNOCUOUS_STRING;
  }
  return goog.html.SafeUrl.createSafeUrlSecurityPrivateDoNotAccessOrElse(
      telUrl);
};


/**
 * Matches a sip/sips URL. We only allow urls that consist of an email address.
 * The characters '?' and '#' are not allowed in the local part of the email
 * address.
 * @const
 * @private
 */
goog.html.SIP_URL_PATTERN_ = new RegExp(
    '^sip[s]?:[+a-z0-9_.!$%&\'*\\/=^`{|}~-]+@([a-z0-9-]+\\.)+[a-z0-9]{2,63}$',
    'i');


/**
 * Creates a SafeUrl wrapping a sip: URL. We only allow urls that consist of an
 * email address. The characters '?' and '#' are not allowed in the local part
 * of the email address.
 *
 * @param {string} sipUrl A sip URL.
 * @return {!goog.html.SafeUrl} A matching safe URL, or {@link INNOCUOUS_STRING}
 *     wrapped as a SafeUrl if it does not pass.
 */
goog.html.SafeUrl.fromSipUrl = function(sipUrl) {
  if (!goog.html.SIP_URL_PATTERN_.test(decodeURIComponent(sipUrl))) {
    sipUrl = goog.html.SafeUrl.INNOCUOUS_STRING;
  }
  return goog.html.SafeUrl.createSafeUrlSecurityPrivateDoNotAccessOrElse(
      sipUrl);
};


/**
 * Creates a SafeUrl wrapping a fb-messenger://share URL.
 *
 * @param {string} facebookMessengerUrl A facebook messenger URL.
 * @return {!goog.html.SafeUrl} A matching safe URL, or {@link INNOCUOUS_STRING}
 *     wrapped as a SafeUrl if it does not pass.
 */
goog.html.SafeUrl.fromFacebookMessengerUrl = function(facebookMessengerUrl) {
  if (!goog.string.internal.caseInsensitiveStartsWith(
          facebookMessengerUrl, 'fb-messenger://share')) {
    facebookMessengerUrl = goog.html.SafeUrl.INNOCUOUS_STRING;
  }
  return goog.html.SafeUrl.createSafeUrlSecurityPrivateDoNotAccessOrElse(
      facebookMessengerUrl);
};

/**
 * Creates a SafeUrl wrapping a whatsapp://send URL.
 *
 * @param {string} whatsAppUrl A WhatsApp URL.
 * @return {!goog.html.SafeUrl} A matching safe URL, or {@link INNOCUOUS_STRING}
 *     wrapped as a SafeUrl if it does not pass.
 */
goog.html.SafeUrl.fromWhatsAppUrl = function(whatsAppUrl) {
  if (!goog.string.internal.caseInsensitiveStartsWith(
          whatsAppUrl, 'whatsapp://send')) {
    whatsAppUrl = goog.html.SafeUrl.INNOCUOUS_STRING;
  }
  return goog.html.SafeUrl.createSafeUrlSecurityPrivateDoNotAccessOrElse(
      whatsAppUrl);
};

/**
 * Creates a SafeUrl wrapping a sms: URL.
 *
 * @param {string} smsUrl A sms URL.
 * @return {!goog.html.SafeUrl} A matching safe URL, or {@link INNOCUOUS_STRING}
 *     wrapped as a SafeUrl if it does not pass.
 */
goog.html.SafeUrl.fromSmsUrl = function(smsUrl) {
  if (!goog.string.internal.caseInsensitiveStartsWith(smsUrl, 'sms:') ||
      !goog.html.SafeUrl.isSmsUrlBodyValid_(smsUrl)) {
    smsUrl = goog.html.SafeUrl.INNOCUOUS_STRING;
  }
  return goog.html.SafeUrl.createSafeUrlSecurityPrivateDoNotAccessOrElse(
      smsUrl);
};


/**
 * Validates SMS URL `body` parameter, which is optional and should appear at
 * most once and should be percent-encoded if present. Rejects many malformed
 * bodies, but may spuriously reject some URLs and does not reject all malformed
 * sms: URLs.
 *
 * @param {string} smsUrl A sms URL.
 * @return {boolean} Whether SMS URL has a valid `body` parameter if it exists.
 * @private
 */
goog.html.SafeUrl.isSmsUrlBodyValid_ = function(smsUrl) {
  var hash = smsUrl.indexOf('#');
  if (hash > 0) {
    smsUrl = smsUrl.substring(0, hash);
  }
  var bodyParams = smsUrl.match(/[?&]body=/gi);
  // "body" param is optional
  if (!bodyParams) {
    return true;
  }
  // "body" MUST only appear once
  if (bodyParams.length > 1) {
    return false;
  }
  // Get the encoded `body` parameter value.
  var bodyValue = smsUrl.match(/[?&]body=([^&]*)/)[1];
  if (!bodyValue) {
    return true;
  }
  try {
    decodeURIComponent(bodyValue);
  } catch (error) {
    return false;
  }
  return /^(?:[a-z0-9\-_.~]|%[0-9a-f]{2})+$/i.test(bodyValue);
};


/**
 * Creates a SafeUrl wrapping a ssh: URL.
 *
 * @param {string} sshUrl A ssh URL.
 * @return {!goog.html.SafeUrl} A matching safe URL, or {@link INNOCUOUS_STRING}
 *     wrapped as a SafeUrl if it does not pass.
 */
goog.html.SafeUrl.fromSshUrl = function(sshUrl) {
  if (!goog.string.internal.caseInsensitiveStartsWith(sshUrl, 'ssh://')) {
    sshUrl = goog.html.SafeUrl.INNOCUOUS_STRING;
  }
  return goog.html.SafeUrl.createSafeUrlSecurityPrivateDoNotAccessOrElse(
      sshUrl);
};

/**
 * Sanitizes a Chrome extension URL to SafeUrl, given a compile-time-constant
 * extension identifier. Can also be restricted to chrome extensions.
 *
 * @param {string} url The url to sanitize. Should start with the extension
 *     scheme and the extension identifier.
 * @param {!goog.string.Const|!Array<!goog.string.Const>} extensionId The
 *     extension id to accept, as a compile-time constant or an array of those.
 *
 * @return {!goog.html.SafeUrl} Either `url` if it's deemed safe, or
 *     `INNOCUOUS_STRING` if it's not.
 */
goog.html.SafeUrl.sanitizeChromeExtensionUrl = function(url, extensionId) {
  return goog.html.SafeUrl.sanitizeExtensionUrl_(
      /^chrome-extension:\/\/([^\/]+)\//, url, extensionId);
};

/**
 * Sanitizes a Firefox extension URL to SafeUrl, given a compile-time-constant
 * extension identifier. Can also be restricted to chrome extensions.
 *
 * @param {string} url The url to sanitize. Should start with the extension
 *     scheme and the extension identifier.
 * @param {!goog.string.Const|!Array<!goog.string.Const>} extensionId The
 *     extension id to accept, as a compile-time constant or an array of those.
 *
 * @return {!goog.html.SafeUrl} Either `url` if it's deemed safe, or
 *     `INNOCUOUS_STRING` if it's not.
 */
goog.html.SafeUrl.sanitizeFirefoxExtensionUrl = function(url, extensionId) {
  return goog.html.SafeUrl.sanitizeExtensionUrl_(
      /^moz-extension:\/\/([^\/]+)\//, url, extensionId);
};

/**
 * Sanitizes a Edge extension URL to SafeUrl, given a compile-time-constant
 * extension identifier. Can also be restricted to chrome extensions.
 *
 * @param {string} url The url to sanitize. Should start with the extension
 *     scheme and the extension identifier.
 * @param {!goog.string.Const|!Array<!goog.string.Const>} extensionId The
 *     extension id to accept, as a compile-time constant or an array of those.
 *
 * @return {!goog.html.SafeUrl} Either `url` if it's deemed safe, or
 *     `INNOCUOUS_STRING` if it's not.
 */
goog.html.SafeUrl.sanitizeEdgeExtensionUrl = function(url, extensionId) {
  return goog.html.SafeUrl.sanitizeExtensionUrl_(
      /^ms-browser-extension:\/\/([^\/]+)\//, url, extensionId);
};

/**
 * Private helper for converting extension URLs to SafeUrl, given the scheme for
 * that particular extension type. Use the sanitizeFirefoxExtensionUrl,
 * sanitizeChromeExtensionUrl or sanitizeEdgeExtensionUrl unless you're building
 * new helpers.
 *
 * @private
 * @param {!RegExp} scheme The scheme to accept as a RegExp extracting the
 *     extension identifier.
 * @param {string} url The url to sanitize. Should start with the extension
 *     scheme and the extension identifier.
 * @param {!goog.string.Const|!Array<!goog.string.Const>} extensionId The
 *     extension id to accept, as a compile-time constant or an array of those.
 *
 * @return {!goog.html.SafeUrl} Either `url` if it's deemed safe, or
 *     `INNOCUOUS_STRING` if it's not.
 */
goog.html.SafeUrl.sanitizeExtensionUrl_ = function(scheme, url, extensionId) {
  var matches = scheme.exec(url);
  if (!matches) {
    url = goog.html.SafeUrl.INNOCUOUS_STRING;
  } else {
    var extractedExtensionId = matches[1];
    var acceptedExtensionIds;
    if (extensionId instanceof goog.string.Const) {
      acceptedExtensionIds = [goog.string.Const.unwrap(extensionId)];
    } else {
      acceptedExtensionIds = extensionId.map(function unwrap(x) {
        return goog.string.Const.unwrap(x);
      });
    }
    if (acceptedExtensionIds.indexOf(extractedExtensionId) == -1) {
      url = goog.html.SafeUrl.INNOCUOUS_STRING;
    }
  }
  return goog.html.SafeUrl.createSafeUrlSecurityPrivateDoNotAccessOrElse(url);
};


/**
 * Creates a SafeUrl from TrustedResourceUrl. This is safe because
 * TrustedResourceUrl is more tightly restricted than SafeUrl.
 *
 * @param {!goog.html.TrustedResourceUrl} trustedResourceUrl
 * @return {!goog.html.SafeUrl}
 */
goog.html.SafeUrl.fromTrustedResourceUrl = function(trustedResourceUrl) {
  return goog.html.SafeUrl.createSafeUrlSecurityPrivateDoNotAccessOrElse(
      goog.html.TrustedResourceUrl.unwrap(trustedResourceUrl));
};


/**
 * A pattern that recognizes a commonly useful subset of URLs that satisfy
 * the SafeUrl contract.
 *
 * This regular expression matches a subset of URLs that will not cause script
 * execution if used in URL context within a HTML document. Specifically, this
 * regular expression matches if (comment from here on and regex copied from
 * Soy's EscapingConventions):
 * (1) Either a protocol in a whitelist (http, https, mailto or ftp).
 * (2) or no protocol.  A protocol must be followed by a colon. The below
 *     allows that by allowing colons only after one of the characters [/?#].
 *     A colon after a hash (#) must be in the fragment.
 *     Otherwise, a colon after a (?) must be in a query.
 *     Otherwise, a colon after a single solidus (/) must be in a path.
 *     Otherwise, a colon after a double solidus (//) must be in the authority
 *     (before port).
 *
 * @private
 * @const {!RegExp}
 */
goog.html.SAFE_URL_PATTERN_ =
    /^(?:(?:https?|mailto|ftp):|[^:/?#]*(?:[/?#]|$))/i;

/**
 * Public version of goog.html.SAFE_URL_PATTERN_. Updating
 * goog.html.SAFE_URL_PATTERN_ doesn't seem to be backward compatible.
 * Namespace is also changed to goog.html.SafeUrl so it can be imported using
 * goog.require('goog.dom.SafeUrl').
 *
 * TODO(bangert): Remove SAFE_URL_PATTERN_
 * @const {!RegExp}
 */
goog.html.SafeUrl.SAFE_URL_PATTERN = goog.html.SAFE_URL_PATTERN_;


/**
 * Creates a SafeUrl object from `url`. If `url` is a
 * goog.html.SafeUrl then it is simply returned. Otherwise the input string is
 * validated to match a pattern of commonly used safe URLs.
 *
 * `url` may be a URL with the http, https, mailto or ftp scheme,
 * or a relative URL (i.e., a URL without a scheme; specifically, a
 * scheme-relative, absolute-path-relative, or path-relative URL).
 *
 * @see http://url.spec.whatwg.org/#concept-relative-url
 * @param {string|!goog.string.TypedString} url The URL to validate.
 * @return {!goog.html.SafeUrl} The validated URL, wrapped as a SafeUrl.
 */
goog.html.SafeUrl.sanitize = function(url) {
  if (url instanceof goog.html.SafeUrl) {
    return url;
  } else if (typeof url == 'object' && url.implementsGoogStringTypedString) {
    url = /** @type {!goog.string.TypedString} */ (url).getTypedStringValue();
  } else {
    url = String(url);
  }
  if (!goog.html.SAFE_URL_PATTERN_.test(url)) {
    url = goog.html.SafeUrl.INNOCUOUS_STRING;
  }
  return goog.html.SafeUrl.createSafeUrlSecurityPrivateDoNotAccessOrElse(url);
};

/**
 * Creates a SafeUrl object from `url`. If `url` is a
 * goog.html.SafeUrl then it is simply returned. Otherwise the input string is
 * validated to match a pattern of commonly used safe URLs.
 *
 * `url` may be a URL with the http, https, mailto or ftp scheme,
 * or a relative URL (i.e., a URL without a scheme; specifically, a
 * scheme-relative, absolute-path-relative, or path-relative URL).
 *
 * This function asserts (using goog.asserts) that the URL matches this pattern.
 * If it does not, in addition to failing the assert, an innocous URL will be
 * returned.
 *
 * @see http://url.spec.whatwg.org/#concept-relative-url
 * @param {string|!goog.string.TypedString} url The URL to validate.
 * @param {boolean=} opt_allowDataUrl Whether to allow valid data: URLs.
 * @return {!goog.html.SafeUrl} The validated URL, wrapped as a SafeUrl.
 */
goog.html.SafeUrl.sanitizeAssertUnchanged = function(url, opt_allowDataUrl) {
  if (url instanceof goog.html.SafeUrl) {
    return url;
  } else if (typeof url == 'object' && url.implementsGoogStringTypedString) {
    url = /** @type {!goog.string.TypedString} */ (url).getTypedStringValue();
  } else {
    url = String(url);
  }
  if (opt_allowDataUrl && /^data:/i.test(url)) {
    var safeUrl = goog.html.SafeUrl.fromDataUrl(url);
    if (safeUrl.getTypedStringValue() == url) {
      return safeUrl;
    }
  }
  if (!goog.asserts.assert(
          goog.html.SAFE_URL_PATTERN_.test(url),
          '%s does not match the safe URL pattern', url)) {
    url = goog.html.SafeUrl.INNOCUOUS_STRING;
  }
  return goog.html.SafeUrl.createSafeUrlSecurityPrivateDoNotAccessOrElse(url);
};



/**
 * Type marker for the SafeUrl type, used to implement additional run-time
 * type checking.
 * @const {!Object}
 * @private
 */
goog.html.SafeUrl.TYPE_MARKER_GOOG_HTML_SECURITY_PRIVATE_ = {};


/**
 * Package-internal utility method to create SafeUrl instances.
 *
 * @param {string} url The string to initialize the SafeUrl object with.
 * @return {!goog.html.SafeUrl} The initialized SafeUrl object.
 * @package
 */
goog.html.SafeUrl.createSafeUrlSecurityPrivateDoNotAccessOrElse = function(
    url) {
  return new goog.html.SafeUrl(
      goog.html.SafeUrl.CONSTRUCTOR_TOKEN_PRIVATE_, url);
};


/**
 * A SafeUrl corresponding to the special about:blank url.
 * @const {!goog.html.SafeUrl}
 */
goog.html.SafeUrl.ABOUT_BLANK =
    goog.html.SafeUrl.createSafeUrlSecurityPrivateDoNotAccessOrElse(
        'about:blank');

/**
 * Token used to ensure that object is created only from this file. No code
 * outside of this file can access this token.
 * @private {!Object}
 * @const
 */
goog.html.SafeUrl.CONSTRUCTOR_TOKEN_PRIVATE_ = {};

//third_party/javascript/closure/html/safestyle.js
/**
 * @license
 * Copyright The Closure Library Authors.
 * SPDX-License-Identifier: Apache-2.0
 */

/**
 * @fileoverview The SafeStyle type and its builders.
 *
 * TODO(xtof): Link to document stating type contract.
 */

goog.provide('goog.html.SafeStyle');

goog.require('goog.array');
goog.require('goog.asserts');
goog.require('goog.html.SafeUrl');
goog.require('goog.string.Const');
goog.require('goog.string.TypedString');
goog.require('goog.string.internal');



/**
 * A string-like object which represents a sequence of CSS declarations
 * ({@code propertyName1: propertyvalue1; propertyName2: propertyValue2; ...})
 * and that carries the security type contract that its value, as a string,
 * will not cause untrusted script execution (XSS) when evaluated as CSS in a
 * browser.
 *
 * Instances of this type must be created via the factory methods
 * (`goog.html.SafeStyle.create` or
 * `goog.html.SafeStyle.fromConstant`) and not by invoking its
 * constructor. The constructor intentionally takes no parameters and the type
 * is immutable; hence only a default instance corresponding to the empty string
 * can be obtained via constructor invocation.
 *
 * SafeStyle's string representation can safely be:
 * <ul>
 *   <li>Interpolated as the content of a *quoted* HTML style attribute.
 *       However, the SafeStyle string *must be HTML-attribute-escaped* before
 *       interpolation.
 *   <li>Interpolated as the content of a {}-wrapped block within a stylesheet.
 *       '<' characters in the SafeStyle string *must be CSS-escaped* before
 *       interpolation. The SafeStyle string is also guaranteed not to be able
 *       to introduce new properties or elide existing ones.
 *   <li>Interpolated as the content of a {}-wrapped block within an HTML
 *       &lt;style&gt; element. '<' characters in the SafeStyle string
 *       *must be CSS-escaped* before interpolation.
 *   <li>Assigned to the style property of a DOM node. The SafeStyle string
 *       should not be escaped before being assigned to the property.
 * </ul>
 *
 * A SafeStyle may never contain literal angle brackets. Otherwise, it could
 * be unsafe to place a SafeStyle into a &lt;style&gt; tag (where it can't
 * be HTML escaped). For example, if the SafeStyle containing
 * "{@code font: 'foo &lt;style/&gt;&lt;script&gt;evil&lt;/script&gt;'}" were
 * interpolated within a &lt;style&gt; tag, this would then break out of the
 * style context into HTML.
 *
 * A SafeStyle may contain literal single or double quotes, and as such the
 * entire style string must be escaped when used in a style attribute (if
 * this were not the case, the string could contain a matching quote that
 * would escape from the style attribute).
 *
 * Values of this type must be composable, i.e. for any two values
 * `style1` and `style2` of this type,
 * {@code goog.html.SafeStyle.unwrap(style1) +
 * goog.html.SafeStyle.unwrap(style2)} must itself be a value that satisfies
 * the SafeStyle type constraint. This requirement implies that for any value
 * `style` of this type, `goog.html.SafeStyle.unwrap(style)` must
 * not end in a "property value" or "property name" context. For example,
 * a value of {@code background:url("} or {@code font-} would not satisfy the
 * SafeStyle contract. This is because concatenating such strings with a
 * second value that itself does not contain unsafe CSS can result in an
 * overall string that does. For example, if {@code javascript:evil())"} is
 * appended to {@code background:url("}, the resulting string may result in
 * the execution of a malicious script.
 *
 * TODO(mlourenco): Consider whether we should implement UTF-8 interchange
 * validity checks and blacklisting of newlines (including Unicode ones) and
 * other whitespace characters (\t, \f). Document here if so and also update
 * SafeStyle.fromConstant().
 *
 * The following example values comply with this type's contract:
 * <ul>
 *   <li><pre>width: 1em;</pre>
 *   <li><pre>height:1em;</pre>
 *   <li><pre>width: 1em;height: 1em;</pre>
 *   <li><pre>background:url('http://url');</pre>
 * </ul>
 * In addition, the empty string is safe for use in a CSS attribute.
 *
 * The following example values do NOT comply with this type's contract:
 * <ul>
 *   <li><pre>background: red</pre> (missing a trailing semi-colon)
 *   <li><pre>background:</pre> (missing a value and a trailing semi-colon)
 *   <li><pre>1em</pre> (missing an attribute name, which provides context for
 *       the value)
 * </ul>
 *
 * @see goog.html.SafeStyle#create
 * @see goog.html.SafeStyle#fromConstant
 * @see http://www.w3.org/TR/css3-syntax/
 * @constructor
 * @final
 * @struct
 * @implements {goog.string.TypedString}
 */
goog.html.SafeStyle = function() {
  /**
   * The contained value of this SafeStyle.  The field has a purposely
   * ugly name to make (non-compiled) code that attempts to directly access this
   * field stand out.
   * @private {string}
   */
  this.privateDoNotAccessOrElseSafeStyleWrappedValue_ = '';

  /**
   * A type marker used to implement additional run-time type checking.
   * @see goog.html.SafeStyle#unwrap
   * @const {!Object}
   * @private
   */
  this.SAFE_STYLE_TYPE_MARKER_GOOG_HTML_SECURITY_PRIVATE_ =
      goog.html.SafeStyle.TYPE_MARKER_GOOG_HTML_SECURITY_PRIVATE_;
};


/**
 * @override
 * @const
 */
goog.html.SafeStyle.prototype.implementsGoogStringTypedString = true;


/**
 * Type marker for the SafeStyle type, used to implement additional
 * run-time type checking.
 * @const {!Object}
 * @private
 */
goog.html.SafeStyle.TYPE_MARKER_GOOG_HTML_SECURITY_PRIVATE_ = {};


/**
 * Creates a SafeStyle object from a compile-time constant string.
 *
 * `style` should be in the format
 * {@code name: value; [name: value; ...]} and must not have any < or >
 * characters in it. This is so that SafeStyle's contract is preserved,
 * allowing the SafeStyle to correctly be interpreted as a sequence of CSS
 * declarations and without affecting the syntactic structure of any
 * surrounding CSS and HTML.
 *
 * This method performs basic sanity checks on the format of `style`
 * but does not constrain the format of `name` and `value`, except
 * for disallowing tag characters.
 *
 * @param {!goog.string.Const} style A compile-time-constant string from which
 *     to create a SafeStyle.
 * @return {!goog.html.SafeStyle} A SafeStyle object initialized to
 *     `style`.
 */
goog.html.SafeStyle.fromConstant = function(style) {
  var styleString = goog.string.Const.unwrap(style);
  if (styleString.length === 0) {
    return goog.html.SafeStyle.EMPTY;
  }
  goog.asserts.assert(
      goog.string.internal.endsWith(styleString, ';'),
      'Last character of style string is not \';\': ' + styleString);
  goog.asserts.assert(
      goog.string.internal.contains(styleString, ':'),
      'Style string must contain at least one \':\', to ' +
          'specify a "name: value" pair: ' + styleString);
  return goog.html.SafeStyle.createSafeStyleSecurityPrivateDoNotAccessOrElse(
      styleString);
};


/**
 * Returns this SafeStyle's value as a string.
 *
 * IMPORTANT: In code where it is security relevant that an object's type is
 * indeed `SafeStyle`, use `goog.html.SafeStyle.unwrap` instead of
 * this method. If in doubt, assume that it's security relevant. In particular,
 * note that goog.html functions which return a goog.html type do not guarantee
 * the returned instance is of the right type. For example:
 *
 * <pre>
 * var fakeSafeHtml = new String('fake');
 * fakeSafeHtml.__proto__ = goog.html.SafeHtml.prototype;
 * var newSafeHtml = goog.html.SafeHtml.htmlEscape(fakeSafeHtml);
 * // newSafeHtml is just an alias for fakeSafeHtml, it's passed through by
 * // goog.html.SafeHtml.htmlEscape() as fakeSafeHtml
 * // instanceof goog.html.SafeHtml.
 * </pre>
 *
 * @see goog.html.SafeStyle#unwrap
 * @override
 */
goog.html.SafeStyle.prototype.getTypedStringValue = function() {
  return this.privateDoNotAccessOrElseSafeStyleWrappedValue_;
};


if (goog.DEBUG) {
  /**
   * Returns a debug string-representation of this value.
   *
   * To obtain the actual string value wrapped in a SafeStyle, use
   * `goog.html.SafeStyle.unwrap`.
   *
   * @see goog.html.SafeStyle#unwrap
   * @override
   */
  goog.html.SafeStyle.prototype.toString = function() {
    return 'SafeStyle{' + this.privateDoNotAccessOrElseSafeStyleWrappedValue_ +
        '}';
  };
}


/**
 * Performs a runtime check that the provided object is indeed a
 * SafeStyle object, and returns its value.
 *
 * @param {!goog.html.SafeStyle} safeStyle The object to extract from.
 * @return {string} The safeStyle object's contained string, unless
 *     the run-time type check fails. In that case, `unwrap` returns an
 *     innocuous string, or, if assertions are enabled, throws
 *     `goog.asserts.AssertionError`.
 */
goog.html.SafeStyle.unwrap = function(safeStyle) {
  // Perform additional Run-time type-checking to ensure that
  // safeStyle is indeed an instance of the expected type.  This
  // provides some additional protection against security bugs due to
  // application code that disables type checks.
  // Specifically, the following checks are performed:
  // 1. The object is an instance of the expected type.
  // 2. The object is not an instance of a subclass.
  // 3. The object carries a type marker for the expected type. "Faking" an
  // object requires a reference to the type marker, which has names intended
  // to stand out in code reviews.
  if (safeStyle instanceof goog.html.SafeStyle &&
      safeStyle.constructor === goog.html.SafeStyle &&
      safeStyle.SAFE_STYLE_TYPE_MARKER_GOOG_HTML_SECURITY_PRIVATE_ ===
          goog.html.SafeStyle.TYPE_MARKER_GOOG_HTML_SECURITY_PRIVATE_) {
    return safeStyle.privateDoNotAccessOrElseSafeStyleWrappedValue_;
  } else {
    goog.asserts.fail('expected object of type SafeStyle, got \'' +
        safeStyle + '\' of type ' + goog.typeOf(safeStyle));
    return 'type_error:SafeStyle';
  }
};


/**
 * Package-internal utility method to create SafeStyle instances.
 *
 * @param {string} style The string to initialize the SafeStyle object with.
 * @return {!goog.html.SafeStyle} The initialized SafeStyle object.
 * @package
 */
goog.html.SafeStyle.createSafeStyleSecurityPrivateDoNotAccessOrElse = function(
    style) {
  return new goog.html.SafeStyle().initSecurityPrivateDoNotAccessOrElse_(style);
};


/**
 * Called from createSafeStyleSecurityPrivateDoNotAccessOrElse(). This
 * method exists only so that the compiler can dead code eliminate static
 * fields (like EMPTY) when they're not accessed.
 * @param {string} style
 * @return {!goog.html.SafeStyle}
 * @private
 */
goog.html.SafeStyle.prototype.initSecurityPrivateDoNotAccessOrElse_ = function(
    style) {
  this.privateDoNotAccessOrElseSafeStyleWrappedValue_ = style;
  return this;
};


/**
 * A SafeStyle instance corresponding to the empty string.
 * @const {!goog.html.SafeStyle}
 */
goog.html.SafeStyle.EMPTY =
    goog.html.SafeStyle.createSafeStyleSecurityPrivateDoNotAccessOrElse('');


/**
 * The innocuous string generated by goog.html.SafeStyle.create when passed
 * an unsafe value.
 * @const {string}
 */
goog.html.SafeStyle.INNOCUOUS_STRING = 'zClosurez';


/**
 * A single property value.
 * @typedef {string|!goog.string.Const|!goog.html.SafeUrl}
 */
goog.html.SafeStyle.PropertyValue;


/**
 * Mapping of property names to their values.
 * We don't support numbers even though some values might be numbers (e.g.
 * line-height or 0 for any length). The reason is that most numeric values need
 * units (e.g. '1px') and allowing numbers could cause users forgetting about
 * them.
 * @typedef {!Object<string, ?goog.html.SafeStyle.PropertyValue|
 *     ?Array<!goog.html.SafeStyle.PropertyValue>>}
 */
goog.html.SafeStyle.PropertyMap;


/**
 * Creates a new SafeStyle object from the properties specified in the map.
 * @param {goog.html.SafeStyle.PropertyMap} map Mapping of property names to
 *     their values, for example {'margin': '1px'}. Names must consist of
 *     [-_a-zA-Z0-9]. Values might be strings consisting of
 *     [-,.'"%_!# a-zA-Z0-9[\]], where ", ', and [] must be properly balanced.
 *     We also allow simple functions like rgb() and url() which sanitizes its
 *     contents. Other values must be wrapped in goog.string.Const. URLs might
 *     be passed as goog.html.SafeUrl which will be wrapped into url(""). We
 *     also support array whose elements are joined with ' '. Null value causes
 *     skipping the property.
 * @return {!goog.html.SafeStyle}
 * @throws {Error} If invalid name is provided.
 * @throws {goog.asserts.AssertionError} If invalid value is provided. With
 *     disabled assertions, invalid value is replaced by
 *     goog.html.SafeStyle.INNOCUOUS_STRING.
 */
goog.html.SafeStyle.create = function(map) {
  var style = '';
  for (var name in map) {
    if (!/^[-_a-zA-Z0-9]+$/.test(name)) {
      throw new Error('Name allows only [-_a-zA-Z0-9], got: ' + name);
    }
    var value = map[name];
    if (value == null) {
      continue;
    }
    if (Array.isArray(value)) {
      value = goog.array.map(value, goog.html.SafeStyle.sanitizePropertyValue_)
                  .join(' ');
    } else {
      value = goog.html.SafeStyle.sanitizePropertyValue_(value);
    }
    style += name + ':' + value + ';';
  }
  if (!style) {
    return goog.html.SafeStyle.EMPTY;
  }
  return goog.html.SafeStyle.createSafeStyleSecurityPrivateDoNotAccessOrElse(
      style);
};


/**
 * Checks and converts value to string.
 * @param {!goog.html.SafeStyle.PropertyValue} value
 * @return {string}
 * @private
 */
goog.html.SafeStyle.sanitizePropertyValue_ = function(value) {
  if (value instanceof goog.html.SafeUrl) {
    var url = goog.html.SafeUrl.unwrap(value);
    return 'url("' + url.replace(/</g, '%3c').replace(/[\\"]/g, '\\$&') + '")';
  }
  var result = value instanceof goog.string.Const ?
      goog.string.Const.unwrap(value) :
      goog.html.SafeStyle.sanitizePropertyValueString_(String(value));
  // These characters can be used to change context and we don't want that even
  // with const values.
  if (/[{;}]/.test(result)) {
    throw new goog.asserts.AssertionError(
        'Value does not allow [{;}], got: %s.', [result]);
  }
  return result;
};


/**
 * Checks string value.
 * @param {string} value
 * @return {string}
 * @private
 */
goog.html.SafeStyle.sanitizePropertyValueString_ = function(value) {
  // Some CSS property values permit nested functions. We allow one level of
  // nesting, and all nested functions must also be in the FUNCTIONS_RE_ list.
  var valueWithoutFunctions =
      value.replace(goog.html.SafeStyle.FUNCTIONS_RE_, '$1')
          .replace(goog.html.SafeStyle.FUNCTIONS_RE_, '$1')
          .replace(goog.html.SafeStyle.URL_RE_, 'url');
  if (!goog.html.SafeStyle.VALUE_RE_.test(valueWithoutFunctions)) {
    goog.asserts.fail(
        'String value allows only ' + goog.html.SafeStyle.VALUE_ALLOWED_CHARS_ +
        ' and simple functions, got: ' + value);
    return goog.html.SafeStyle.INNOCUOUS_STRING;
  } else if (goog.html.SafeStyle.COMMENT_RE_.test(value)) {
    goog.asserts.fail('String value disallows comments, got: ' + value);
    return goog.html.SafeStyle.INNOCUOUS_STRING;
  } else if (!goog.html.SafeStyle.hasBalancedQuotes_(value)) {
    goog.asserts.fail('String value requires balanced quotes, got: ' + value);
    return goog.html.SafeStyle.INNOCUOUS_STRING;
  } else if (!goog.html.SafeStyle.hasBalancedSquareBrackets_(value)) {
    goog.asserts.fail(
        'String value requires balanced square brackets and one' +
        ' identifier per pair of brackets, got: ' + value);
    return goog.html.SafeStyle.INNOCUOUS_STRING;
  }
  return goog.html.SafeStyle.sanitizeUrl_(value);
};


/**
 * Checks that quotes (" and ') are properly balanced inside a string. Assumes
 * that neither escape (\) nor any other character that could result in
 * breaking out of a string parsing context are allowed;
 * see http://www.w3.org/TR/css3-syntax/#string-token-diagram.
 * @param {string} value Untrusted CSS property value.
 * @return {boolean} True if property value is safe with respect to quote
 *     balancedness.
 * @private
 */
goog.html.SafeStyle.hasBalancedQuotes_ = function(value) {
  var outsideSingle = true;
  var outsideDouble = true;
  for (var i = 0; i < value.length; i++) {
    var c = value.charAt(i);
    if (c == "'" && outsideDouble) {
      outsideSingle = !outsideSingle;
    } else if (c == '"' && outsideSingle) {
      outsideDouble = !outsideDouble;
    }
  }
  return outsideSingle && outsideDouble;
};


/**
 * Checks that square brackets ([ and ]) are properly balanced inside a string,
 * and that the content in the square brackets is one ident-token;
 * see https://www.w3.org/TR/css-syntax-3/#ident-token-diagram.
 * For practicality, and in line with other restrictions posed on SafeStyle
 * strings, we restrict the character set allowable in the ident-token to
 * [-_a-zA-Z0-9].
 * @param {string} value Untrusted CSS property value.
 * @return {boolean} True if property value is safe with respect to square
 *     bracket balancedness.
 * @private
 */
goog.html.SafeStyle.hasBalancedSquareBrackets_ = function(value) {
  var outside = true;
  var tokenRe = /^[-_a-zA-Z0-9]$/;
  for (var i = 0; i < value.length; i++) {
    var c = value.charAt(i);
    if (c == ']') {
      if (outside) return false;  // Unbalanced ].
      outside = true;
    } else if (c == '[') {
      if (!outside) return false;  // No nesting.
      outside = false;
    } else if (!outside && !tokenRe.test(c)) {
      return false;
    }
  }
  return outside;
};


/**
 * Characters allowed in goog.html.SafeStyle.VALUE_RE_.
 * @private {string}
 */
goog.html.SafeStyle.VALUE_ALLOWED_CHARS_ = '[-,."\'%_!# a-zA-Z0-9\\[\\]]';


/**
 * Regular expression for safe values.
 *
 * Quotes (" and ') are allowed, but a check must be done elsewhere to ensure
 * they're balanced.
 *
 * Square brackets ([ and ]) are allowed, but a check must be done elsewhere
 * to ensure they're balanced. The content inside a pair of square brackets must
 * be one alphanumeric identifier.
 *
 * ',' allows multiple values to be assigned to the same property
 * (e.g. background-attachment or font-family) and hence could allow
 * multiple values to get injected, but that should pose no risk of XSS.
 *
 * The expression checks only for XSS safety, not for CSS validity.
 * @const {!RegExp}
 * @private
 */
goog.html.SafeStyle.VALUE_RE_ =
    new RegExp('^' + goog.html.SafeStyle.VALUE_ALLOWED_CHARS_ + '+$');


/**
 * Regular expression for url(). We support URLs allowed by
 * https://www.w3.org/TR/css-syntax-3/#url-token-diagram without using escape
 * sequences. Use percent-encoding if you need to use special characters like
 * backslash.
 * @private @const {!RegExp}
 */
goog.html.SafeStyle.URL_RE_ = new RegExp(
    '\\b(url\\([ \t\n]*)(' +
        '\'[ -&(-\\[\\]-~]*\'' +  // Printable characters except ' and \.
        '|"[ !#-\\[\\]-~]*"' +    // Printable characters except " and \.
        '|[!#-&*-\\[\\]-~]*' +    // Printable characters except [ "'()\\].
        ')([ \t\n]*\\))',
    'g');

/**
 * Names of functions allowed in FUNCTIONS_RE_.
 * @private @const {!Array<string>}
 */
goog.html.SafeStyle.ALLOWED_FUNCTIONS_ = [
  'calc',
  'cubic-bezier',
  'fit-content',
  'hsl',
  'hsla',
  'linear-gradient',
  'matrix',
  'minmax',
  'repeat',
  'rgb',
  'rgba',
  '(rotate|scale|translate)(X|Y|Z|3d)?',
];


/**
 * Regular expression for simple functions.
 * @private @const {!RegExp}
 */
goog.html.SafeStyle.FUNCTIONS_RE_ = new RegExp(
    '\\b(' + goog.html.SafeStyle.ALLOWED_FUNCTIONS_.join('|') + ')' +
        '\\([-+*/0-9a-z.%\\[\\], ]+\\)',
    'g');


/**
 * Regular expression for comments. These are disallowed in CSS property values.
 * @private @const {!RegExp}
 */
goog.html.SafeStyle.COMMENT_RE_ = /\/\*/;


/**
 * Sanitize URLs inside url().
 *
 * NOTE: We could also consider using CSS.escape once that's available in the
 * browsers. However, loosely matching URL e.g. with url\(.*\) and then escaping
 * the contents would result in a slightly different language than CSS leading
 * to confusion of users. E.g. url(")") is valid in CSS but it would be invalid
 * as seen by our parser. On the other hand, url(\) is invalid in CSS but our
 * parser would be fine with it.
 *
 * @param {string} value Untrusted CSS property value.
 * @return {string}
 * @private
 */
goog.html.SafeStyle.sanitizeUrl_ = function(value) {
  return value.replace(
      goog.html.SafeStyle.URL_RE_, function(match, before, url, after) {
        var quote = '';
        url = url.replace(/^(['"])(.*)\1$/, function(match, start, inside) {
          quote = start;
          return inside;
        });
        var sanitized = goog.html.SafeUrl.sanitize(url).getTypedStringValue();
        return before + quote + sanitized + quote + after;
      });
};


/**
 * Creates a new SafeStyle object by concatenating the values.
 * @param {...(!goog.html.SafeStyle|!Array<!goog.html.SafeStyle>)} var_args
 *     SafeStyles to concatenate.
 * @return {!goog.html.SafeStyle}
 */
goog.html.SafeStyle.concat = function(var_args) {
  var style = '';

  /**
   * @param {!goog.html.SafeStyle|!Array<!goog.html.SafeStyle>} argument
   */
  var addArgument = function(argument) {
    if (Array.isArray(argument)) {
      goog.array.forEach(argument, addArgument);
    } else {
      style += goog.html.SafeStyle.unwrap(argument);
    }
  };

  goog.array.forEach(arguments, addArgument);
  if (!style) {
    return goog.html.SafeStyle.EMPTY;
  }
  return goog.html.SafeStyle.createSafeStyleSecurityPrivateDoNotAccessOrElse(
      style);
};

//third_party/javascript/closure/html/safestylesheet.js
/**
 * @license
 * Copyright The Closure Library Authors.
 * SPDX-License-Identifier: Apache-2.0
 */

/**
 * @fileoverview The SafeStyleSheet type and its builders.
 *
 * TODO(xtof): Link to document stating type contract.
 */

goog.provide('goog.html.SafeStyleSheet');

goog.require('goog.array');
goog.require('goog.asserts');
goog.require('goog.html.SafeStyle');
goog.require('goog.object');
goog.require('goog.string.Const');
goog.require('goog.string.TypedString');
goog.require('goog.string.internal');



/**
 * A string-like object which represents a CSS style sheet and that carries the
 * security type contract that its value, as a string, will not cause untrusted
 * script execution (XSS) when evaluated as CSS in a browser.
 *
 * Instances of this type must be created via the factory method
 * `goog.html.SafeStyleSheet.fromConstant` and not by invoking its
 * constructor. The constructor intentionally takes no parameters and the type
 * is immutable; hence only a default instance corresponding to the empty string
 * can be obtained via constructor invocation.
 *
 * A SafeStyleSheet's string representation can safely be interpolated as the
 * content of a style element within HTML. The SafeStyleSheet string should
 * not be escaped before interpolation.
 *
 * Values of this type must be composable, i.e. for any two values
 * `styleSheet1` and `styleSheet2` of this type,
 * {@code goog.html.SafeStyleSheet.unwrap(styleSheet1) +
 * goog.html.SafeStyleSheet.unwrap(styleSheet2)} must itself be a value that
 * satisfies the SafeStyleSheet type constraint. This requirement implies that
 * for any value `styleSheet` of this type,
 * `goog.html.SafeStyleSheet.unwrap(styleSheet1)` must end in
 * "beginning of rule" context.

 * A SafeStyleSheet can be constructed via security-reviewed unchecked
 * conversions. In this case producers of SafeStyleSheet must ensure themselves
 * that the SafeStyleSheet does not contain unsafe script. Note in particular
 * that {@code &lt;} is dangerous, even when inside CSS strings, and so should
 * always be forbidden or CSS-escaped in user controlled input. For example, if
 * {@code &lt;/style&gt;&lt;script&gt;evil&lt;/script&gt;"} were interpolated
 * inside a CSS string, it would break out of the context of the original
 * style element and `evil` would execute. Also note that within an HTML
 * style (raw text) element, HTML character references, such as
 * {@code &amp;lt;}, are not allowed. See
 *
 http://www.w3.org/TR/html5/scripting-1.html#restrictions-for-contents-of-script-elements
 * (similar considerations apply to the style element).
 *
 * @see goog.html.SafeStyleSheet#fromConstant
 * @constructor
 * @final
 * @struct
 * @implements {goog.string.TypedString}
 */
goog.html.SafeStyleSheet = function() {
  /**
   * The contained value of this SafeStyleSheet.  The field has a purposely
   * ugly name to make (non-compiled) code that attempts to directly access this
   * field stand out.
   * @private {string}
   */
  this.privateDoNotAccessOrElseSafeStyleSheetWrappedValue_ = '';

  /**
   * A type marker used to implement additional run-time type checking.
   * @see goog.html.SafeStyleSheet#unwrap
   * @const {!Object}
   * @private
   */
  this.SAFE_STYLE_SHEET_TYPE_MARKER_GOOG_HTML_SECURITY_PRIVATE_ =
      goog.html.SafeStyleSheet.TYPE_MARKER_GOOG_HTML_SECURITY_PRIVATE_;
};


/**
 * @override
 * @const
 */
goog.html.SafeStyleSheet.prototype.implementsGoogStringTypedString = true;


/**
 * Type marker for the SafeStyleSheet type, used to implement additional
 * run-time type checking.
 * @const {!Object}
 * @private
 */
goog.html.SafeStyleSheet.TYPE_MARKER_GOOG_HTML_SECURITY_PRIVATE_ = {};


/**
 * Creates a style sheet consisting of one selector and one style definition.
 * Use {@link goog.html.SafeStyleSheet.concat} to create longer style sheets.
 * This function doesn't support @import, @media and similar constructs.
 * @param {string} selector CSS selector, e.g. '#id' or 'tag .class, #id'. We
 *     support CSS3 selectors: https://w3.org/TR/css3-selectors/#selectors.
 * @param {!goog.html.SafeStyle.PropertyMap|!goog.html.SafeStyle} style Style
 *     definition associated with the selector.
 * @return {!goog.html.SafeStyleSheet}
 * @throws {Error} If invalid selector is provided.
 */
goog.html.SafeStyleSheet.createRule = function(selector, style) {
  if (goog.string.internal.contains(selector, '<')) {
    throw new Error('Selector does not allow \'<\', got: ' + selector);
  }

  // Remove strings.
  var selectorToCheck =
      selector.replace(/('|")((?!\1)[^\r\n\f\\]|\\[\s\S])*\1/g, '');

  // Check characters allowed in CSS3 selectors.
  if (!/^[-_a-zA-Z0-9#.:* ,>+~[\]()=^$|]+$/.test(selectorToCheck)) {
    throw new Error(
        'Selector allows only [-_a-zA-Z0-9#.:* ,>+~[\\]()=^$|] and ' +
        'strings, got: ' + selector);
  }

  // Check balanced () and [].
  if (!goog.html.SafeStyleSheet.hasBalancedBrackets_(selectorToCheck)) {
    throw new Error('() and [] in selector must be balanced, got: ' + selector);
  }

  if (!(style instanceof goog.html.SafeStyle)) {
    style = goog.html.SafeStyle.create(style);
  }
  var styleSheet = selector + '{' +
      goog.html.SafeStyle.unwrap(style).replace(/</g, '\\3C ') + '}';
  return goog.html.SafeStyleSheet
      .createSafeStyleSheetSecurityPrivateDoNotAccessOrElse(styleSheet);
};


/**
 * Checks if a string has balanced () and [] brackets.
 * @param {string} s String to check.
 * @return {boolean}
 * @private
 */
goog.html.SafeStyleSheet.hasBalancedBrackets_ = function(s) {
  var brackets = {'(': ')', '[': ']'};
  var expectedBrackets = [];
  for (var i = 0; i < s.length; i++) {
    var ch = s[i];
    if (brackets[ch]) {
      expectedBrackets.push(brackets[ch]);
    } else if (goog.object.contains(brackets, ch)) {
      if (expectedBrackets.pop() != ch) {
        return false;
      }
    }
  }
  return expectedBrackets.length == 0;
};


/**
 * Creates a new SafeStyleSheet object by concatenating values.
 * @param {...(!goog.html.SafeStyleSheet|!Array<!goog.html.SafeStyleSheet>)}
 *     var_args Values to concatenate.
 * @return {!goog.html.SafeStyleSheet}
 */
goog.html.SafeStyleSheet.concat = function(var_args) {
  var result = '';

  /**
   * @param {!goog.html.SafeStyleSheet|!Array<!goog.html.SafeStyleSheet>}
   *     argument
   */
  var addArgument = function(argument) {
    if (Array.isArray(argument)) {
      goog.array.forEach(argument, addArgument);
    } else {
      result += goog.html.SafeStyleSheet.unwrap(argument);
    }
  };

  goog.array.forEach(arguments, addArgument);
  return goog.html.SafeStyleSheet
      .createSafeStyleSheetSecurityPrivateDoNotAccessOrElse(result);
};


/**
 * Creates a SafeStyleSheet object from a compile-time constant string.
 *
 * `styleSheet` must not have any &lt; characters in it, so that
 * the syntactic structure of the surrounding HTML is not affected.
 *
 * @param {!goog.string.Const} styleSheet A compile-time-constant string from
 *     which to create a SafeStyleSheet.
 * @return {!goog.html.SafeStyleSheet} A SafeStyleSheet object initialized to
 *     `styleSheet`.
 */
goog.html.SafeStyleSheet.fromConstant = function(styleSheet) {
  var styleSheetString = goog.string.Const.unwrap(styleSheet);
  if (styleSheetString.length === 0) {
    return goog.html.SafeStyleSheet.EMPTY;
  }
  // > is a valid character in CSS selectors and there's no strict need to
  // block it if we already block <.
  goog.asserts.assert(
      !goog.string.internal.contains(styleSheetString, '<'),
      'Forbidden \'<\' character in style sheet string: ' + styleSheetString);
  return goog.html.SafeStyleSheet
      .createSafeStyleSheetSecurityPrivateDoNotAccessOrElse(styleSheetString);
};


/**
 * Returns this SafeStyleSheet's value as a string.
 *
 * IMPORTANT: In code where it is security relevant that an object's type is
 * indeed `SafeStyleSheet`, use `goog.html.SafeStyleSheet.unwrap`
 * instead of this method. If in doubt, assume that it's security relevant. In
 * particular, note that goog.html functions which return a goog.html type do
 * not guarantee the returned instance is of the right type. For example:
 *
 * <pre>
 * var fakeSafeHtml = new String('fake');
 * fakeSafeHtml.__proto__ = goog.html.SafeHtml.prototype;
 * var newSafeHtml = goog.html.SafeHtml.htmlEscape(fakeSafeHtml);
 * // newSafeHtml is just an alias for fakeSafeHtml, it's passed through by
 * // goog.html.SafeHtml.htmlEscape() as fakeSafeHtml
 * // instanceof goog.html.SafeHtml.
 * </pre>
 *
 * @see goog.html.SafeStyleSheet#unwrap
 * @override
 */
goog.html.SafeStyleSheet.prototype.getTypedStringValue = function() {
  return this.privateDoNotAccessOrElseSafeStyleSheetWrappedValue_;
};


if (goog.DEBUG) {
  /**
   * Returns a debug string-representation of this value.
   *
   * To obtain the actual string value wrapped in a SafeStyleSheet, use
   * `goog.html.SafeStyleSheet.unwrap`.
   *
   * @see goog.html.SafeStyleSheet#unwrap
   * @override
   */
  goog.html.SafeStyleSheet.prototype.toString = function() {
    return 'SafeStyleSheet{' +
        this.privateDoNotAccessOrElseSafeStyleSheetWrappedValue_ + '}';
  };
}


/**
 * Performs a runtime check that the provided object is indeed a
 * SafeStyleSheet object, and returns its value.
 *
 * @param {!goog.html.SafeStyleSheet} safeStyleSheet The object to extract from.
 * @return {string} The safeStyleSheet object's contained string, unless
 *     the run-time type check fails. In that case, `unwrap` returns an
 *     innocuous string, or, if assertions are enabled, throws
 *     `goog.asserts.AssertionError`.
 */
goog.html.SafeStyleSheet.unwrap = function(safeStyleSheet) {
  // Perform additional Run-time type-checking to ensure that
  // safeStyleSheet is indeed an instance of the expected type.  This
  // provides some additional protection against security bugs due to
  // application code that disables type checks.
  // Specifically, the following checks are performed:
  // 1. The object is an instance of the expected type.
  // 2. The object is not an instance of a subclass.
  // 3. The object carries a type marker for the expected type. "Faking" an
  // object requires a reference to the type marker, which has names intended
  // to stand out in code reviews.
  if (safeStyleSheet instanceof goog.html.SafeStyleSheet &&
      safeStyleSheet.constructor === goog.html.SafeStyleSheet &&
      safeStyleSheet
              .SAFE_STYLE_SHEET_TYPE_MARKER_GOOG_HTML_SECURITY_PRIVATE_ ===
          goog.html.SafeStyleSheet.TYPE_MARKER_GOOG_HTML_SECURITY_PRIVATE_) {
    return safeStyleSheet.privateDoNotAccessOrElseSafeStyleSheetWrappedValue_;
  } else {
    goog.asserts.fail('expected object of type SafeStyleSheet, got \'' +
        safeStyleSheet + '\' of type ' + goog.typeOf(safeStyleSheet));
    return 'type_error:SafeStyleSheet';
  }
};


/**
 * Package-internal utility method to create SafeStyleSheet instances.
 *
 * @param {string} styleSheet The string to initialize the SafeStyleSheet
 *     object with.
 * @return {!goog.html.SafeStyleSheet} The initialized SafeStyleSheet object.
 * @package
 */
goog.html.SafeStyleSheet.createSafeStyleSheetSecurityPrivateDoNotAccessOrElse =
    function(styleSheet) {
  return new goog.html.SafeStyleSheet().initSecurityPrivateDoNotAccessOrElse_(
      styleSheet);
};


/**
 * Called from createSafeStyleSheetSecurityPrivateDoNotAccessOrElse(). This
 * method exists only so that the compiler can dead code eliminate static
 * fields (like EMPTY) when they're not accessed.
 * @param {string} styleSheet
 * @return {!goog.html.SafeStyleSheet}
 * @private
 */
goog.html.SafeStyleSheet.prototype.initSecurityPrivateDoNotAccessOrElse_ =
    function(styleSheet) {
  this.privateDoNotAccessOrElseSafeStyleSheetWrappedValue_ = styleSheet;
  return this;
};


/**
 * A SafeStyleSheet instance corresponding to the empty string.
 * @const {!goog.html.SafeStyleSheet}
 */
goog.html.SafeStyleSheet.EMPTY =
    goog.html.SafeStyleSheet
        .createSafeStyleSheetSecurityPrivateDoNotAccessOrElse('');

//third_party/javascript/closure/labs/useragent/util.js
/**
 * @license
 * Copyright The Closure Library Authors.
 * SPDX-License-Identifier: Apache-2.0
 */

/**
 * @fileoverview Utilities used by goog.labs.userAgent tools. These functions
 * should not be used outside of goog.labs.userAgent.*.
 *
 * @visibility {//javascript/abc/libs/objects3d:__subpackages__}
 * @visibility {//javascript/closure/dom:__subpackages__}
 * @visibility {//javascript/closure/style:__pkg__}
 * @visibility {//javascript/closure/testing:__pkg__}
 * @visibility {//javascript/closure/useragent:__subpackages__}
 * @visibility {//testing/puppet/modules:__pkg__}
 * @visibility {:util_legacy_users}
 */

goog.provide('goog.labs.userAgent.util');

goog.require('goog.string.internal');


/**
 * Gets the native userAgent string from navigator if it exists.
 * If navigator or navigator.userAgent string is missing, returns an empty
 * string.
 * @return {string}
 * @private
 */
goog.labs.userAgent.util.getNativeUserAgentString_ = function() {
  var navigator = goog.labs.userAgent.util.getNavigator_();
  if (navigator) {
    var userAgent = navigator.userAgent;
    if (userAgent) {
      return userAgent;
    }
  }
  return '';
};


/**
 * Getter for the native navigator.
 * This is a separate function so it can be stubbed out in testing.
 * @return {!Navigator}
 * @private
 */
goog.labs.userAgent.util.getNavigator_ = function() {
  return goog.global.navigator;
};


/**
 * A possible override for applications which wish to not check
 * navigator.userAgent but use a specified value for detection instead.
 * @private {string}
 */
goog.labs.userAgent.util.userAgent_ =
    goog.labs.userAgent.util.getNativeUserAgentString_();


/**
 * Applications may override browser detection on the built in
 * navigator.userAgent object by setting this string. Set to null to use the
 * browser object instead.
 * @param {?string=} opt_userAgent The User-Agent override.
 */
goog.labs.userAgent.util.setUserAgent = function(opt_userAgent) {
  goog.labs.userAgent.util.userAgent_ =
      opt_userAgent || goog.labs.userAgent.util.getNativeUserAgentString_();
};


/**
 * @return {string} The user agent string.
 */
goog.labs.userAgent.util.getUserAgent = function() {
  return goog.labs.userAgent.util.userAgent_;
};


/**
 * @param {string} str
 * @return {boolean} Whether the user agent contains the given string.
 */
goog.labs.userAgent.util.matchUserAgent = function(str) {
  var userAgent = goog.labs.userAgent.util.getUserAgent();
  return goog.string.internal.contains(userAgent, str);
};


/**
 * @param {string} str
 * @return {boolean} Whether the user agent contains the given string, ignoring
 *     case.
 */
goog.labs.userAgent.util.matchUserAgentIgnoreCase = function(str) {
  var userAgent = goog.labs.userAgent.util.getUserAgent();
  return goog.string.internal.caseInsensitiveContains(userAgent, str);
};


/**
 * Parses the user agent into tuples for each section.
 * @param {string} userAgent
 * @return {!Array<!Array<string>>} Tuples of key, version, and the contents
 *     of the parenthetical.
 */
goog.labs.userAgent.util.extractVersionTuples = function(userAgent) {
  // Matches each section of a user agent string.
  // Example UA:
  // Mozilla/5.0 (iPad; U; CPU OS 3_2_1 like Mac OS X; en-us)
  // AppleWebKit/531.21.10 (KHTML, like Gecko) Mobile/7B405
  // This has three version tuples: Mozilla, AppleWebKit, and Mobile.

  var versionRegExp = new RegExp(
      // Key. Note that a key may have a space.
      // (i.e. 'Mobile Safari' in 'Mobile Safari/5.0')
      '(\\w[\\w ]+)' +

          '/' +                // slash
          '([^\\s]+)' +        // version (i.e. '5.0b')
          '\\s*' +             // whitespace
          '(?:\\((.*?)\\))?',  // parenthetical info. parentheses not matched.
      'g');

  var data = [];
  var match;

  // Iterate and collect the version tuples.  Each iteration will be the
  // next regex match.
  while (match = versionRegExp.exec(userAgent)) {
    data.push([
      match[1],  // key
      match[2],  // value
      // || undefined as this is not undefined in IE7 and IE8
      match[3] || undefined  // info
    ]);
  }

  return data;
};

//third_party/javascript/closure/labs/useragent/browser.js
/**
 * @license
 * Copyright The Closure Library Authors.
 * SPDX-License-Identifier: Apache-2.0
 */

/**
 * @fileoverview Closure user agent detection (Browser).
 * @see <a href="http://www.useragentstring.com/">User agent strings</a>
 * For more information on rendering engine, platform, or device see the other
 * sub-namespaces in goog.labs.userAgent, goog.labs.userAgent.platform,
 * goog.labs.userAgent.device respectively.)
 */

goog.provide('goog.labs.userAgent.browser');

goog.require('goog.array');
goog.require('goog.labs.userAgent.util');
goog.require('goog.object');
goog.require('goog.string.internal');


// TODO(nnaze): Refactor to remove excessive exclusion logic in matching
// functions.


/**
 * @return {boolean} Whether the user's browser is Opera.  Note: Chromium
 *     based Opera (Opera 15+) is detected as Chrome to avoid unnecessary
 *     special casing.
 * @private
 */
goog.labs.userAgent.browser.matchOpera_ = function() {
  return goog.labs.userAgent.util.matchUserAgent('Opera');
};


/**
 * @return {boolean} Whether the user's browser is IE.
 * @private
 */
goog.labs.userAgent.browser.matchIE_ = function() {
  return goog.labs.userAgent.util.matchUserAgent('Trident') ||
      goog.labs.userAgent.util.matchUserAgent('MSIE');
};


/**
 * @return {boolean} Whether the user's browser is Edge. This refers to EdgeHTML
 * based Edge.
 * @private
 */
goog.labs.userAgent.browser.matchEdgeHtml_ = function() {
  return goog.labs.userAgent.util.matchUserAgent('Edge');
};


/**
 * @return {boolean} Whether the user's browser is Chromium based Edge.
 * @private
 */
goog.labs.userAgent.browser.matchEdgeChromium_ = function() {
  return goog.labs.userAgent.util.matchUserAgent('Edg/');
};


/**
 * @return {boolean} Whether the user's browser is Chromium based Opera.
 * @private
 */
goog.labs.userAgent.browser.matchOperaChromium_ = function() {
  return goog.labs.userAgent.util.matchUserAgent('OPR');
};


/**
 * @return {boolean} Whether the user's browser is Firefox.
 * @private
 */
goog.labs.userAgent.browser.matchFirefox_ = function() {
  return goog.labs.userAgent.util.matchUserAgent('Firefox') ||
      goog.labs.userAgent.util.matchUserAgent('FxiOS');
};


/**
 * @return {boolean} Whether the user's browser is Safari.
 * @private
 */
goog.labs.userAgent.browser.matchSafari_ = function() {
  return goog.labs.userAgent.util.matchUserAgent('Safari') &&
      !(goog.labs.userAgent.browser.matchChrome_() ||
        goog.labs.userAgent.browser.matchCoast_() ||
        goog.labs.userAgent.browser.matchOpera_() ||
        goog.labs.userAgent.browser.matchEdgeHtml_() ||
        goog.labs.userAgent.browser.matchEdgeChromium_() ||
        goog.labs.userAgent.browser.matchOperaChromium_() ||
        goog.labs.userAgent.browser.matchFirefox_() ||
        goog.labs.userAgent.browser.isSilk() ||
        goog.labs.userAgent.util.matchUserAgent('Android'));
};


/**
 * @return {boolean} Whether the user's browser is Coast (Opera's Webkit-based
 *     iOS browser).
 * @private
 */
goog.labs.userAgent.browser.matchCoast_ = function() {
  return goog.labs.userAgent.util.matchUserAgent('Coast');
};


/**
 * @return {boolean} Whether the user's browser is iOS Webview.
 * @private
 */
goog.labs.userAgent.browser.matchIosWebview_ = function() {
  // iOS Webview does not show up as Chrome or Safari. Also check for Opera's
  // WebKit-based iOS browser, Coast.
  return (goog.labs.userAgent.util.matchUserAgent('iPad') ||
          goog.labs.userAgent.util.matchUserAgent('iPhone')) &&
      !goog.labs.userAgent.browser.matchSafari_() &&
      !goog.labs.userAgent.browser.matchChrome_() &&
      !goog.labs.userAgent.browser.matchCoast_() &&
      !goog.labs.userAgent.browser.matchFirefox_() &&
      goog.labs.userAgent.util.matchUserAgent('AppleWebKit');
};


/**
 * @return {boolean} Whether the user's browser is any Chromium browser. This
 * returns true for Chrome, Opera 15+, and Edge Chromium.
 * @private
 */
goog.labs.userAgent.browser.matchChrome_ = function() {
  return (goog.labs.userAgent.util.matchUserAgent('Chrome') ||
          goog.labs.userAgent.util.matchUserAgent('CriOS')) &&
      !goog.labs.userAgent.browser.matchEdgeHtml_();
};


/**
 * @return {boolean} Whether the user's browser is the Android browser.
 * @private
 */
goog.labs.userAgent.browser.matchAndroidBrowser_ = function() {
  // Android can appear in the user agent string for Chrome on Android.
  // This is not the Android standalone browser if it does.
  return goog.labs.userAgent.util.matchUserAgent('Android') &&
      !(goog.labs.userAgent.browser.isChrome() ||
        goog.labs.userAgent.browser.isFirefox() ||
        goog.labs.userAgent.browser.isOpera() ||
        goog.labs.userAgent.browser.isSilk());
};


/**
 * @return {boolean} Whether the user's browser is Opera.
 */
goog.labs.userAgent.browser.isOpera = goog.labs.userAgent.browser.matchOpera_;


/**
 * @return {boolean} Whether the user's browser is IE.
 */
goog.labs.userAgent.browser.isIE = goog.labs.userAgent.browser.matchIE_;


/**
 * @return {boolean} Whether the user's browser is EdgeHTML based Edge.
 */
goog.labs.userAgent.browser.isEdge = goog.labs.userAgent.browser.matchEdgeHtml_;


/**
 * @return {boolean} Whether the user's browser is Chromium based Edge.
 */
goog.labs.userAgent.browser.isEdgeChromium =
    goog.labs.userAgent.browser.matchEdgeChromium_;

/**
 * @return {boolean} Whether the user's browser is Chromium based Opera.
 */
goog.labs.userAgent.browser.isOperaChromium =
    goog.labs.userAgent.browser.matchOperaChromium_;

/**
 * @return {boolean} Whether the user's browser is Firefox.
 */
goog.labs.userAgent.browser.isFirefox =
    goog.labs.userAgent.browser.matchFirefox_;


/**
 * @return {boolean} Whether the user's browser is Safari.
 */
goog.labs.userAgent.browser.isSafari = goog.labs.userAgent.browser.matchSafari_;


/**
 * @return {boolean} Whether the user's browser is Coast (Opera's Webkit-based
 *     iOS browser).
 */
goog.labs.userAgent.browser.isCoast = goog.labs.userAgent.browser.matchCoast_;


/**
 * @return {boolean} Whether the user's browser is iOS Webview.
 */
goog.labs.userAgent.browser.isIosWebview =
    goog.labs.userAgent.browser.matchIosWebview_;


/**
 * @return {boolean} Whether the user's browser is any Chromium based browser (
 * Chrome, Blink-based Opera (15+) and Edge Chromium).
 */
goog.labs.userAgent.browser.isChrome = goog.labs.userAgent.browser.matchChrome_;


/**
 * @return {boolean} Whether the user's browser is the Android browser.
 */
goog.labs.userAgent.browser.isAndroidBrowser =
    goog.labs.userAgent.browser.matchAndroidBrowser_;


/**
 * For more information, see:
 * http://docs.aws.amazon.com/silk/latest/developerguide/user-agent.html
 * @return {boolean} Whether the user's browser is Silk.
 */
goog.labs.userAgent.browser.isSilk = function() {
  return goog.labs.userAgent.util.matchUserAgent('Silk');
};


/**
 * @return {string} The browser version or empty string if version cannot be
 *     determined. Note that for Internet Explorer, this returns the version of
 *     the browser, not the version of the rendering engine. (IE 8 in
 *     compatibility mode will return 8.0 rather than 7.0. To determine the
 *     rendering engine version, look at document.documentMode instead. See
 *     http://msdn.microsoft.com/en-us/library/cc196988(v=vs.85).aspx for more
 *     details.)
 */
goog.labs.userAgent.browser.getVersion = function() {
  var userAgentString = goog.labs.userAgent.util.getUserAgent();
  // Special case IE since IE's version is inside the parenthesis and
  // without the '/'.
  if (goog.labs.userAgent.browser.isIE()) {
    return goog.labs.userAgent.browser.getIEVersion_(userAgentString);
  }

  var versionTuples =
      goog.labs.userAgent.util.extractVersionTuples(userAgentString);

  // Construct a map for easy lookup.
  var versionMap = {};
  goog.array.forEach(versionTuples, function(tuple) {
    // Note that the tuple is of length three, but we only care about the
    // first two.
    var key = tuple[0];
    var value = tuple[1];
    versionMap[key] = value;
  });

  var versionMapHasKey = goog.partial(goog.object.containsKey, versionMap);

  // Gives the value with the first key it finds, otherwise empty string.
  function lookUpValueWithKeys(keys) {
    var key = goog.array.find(keys, versionMapHasKey);
    return versionMap[key] || '';
  }

  // Check Opera before Chrome since Opera 15+ has "Chrome" in the string.
  // See
  // http://my.opera.com/ODIN/blog/2013/07/15/opera-user-agent-strings-opera-15-and-beyond
  if (goog.labs.userAgent.browser.isOpera()) {
    // Opera 10 has Version/10.0 but Opera/9.8, so look for "Version" first.
    // Opera uses 'OPR' for more recent UAs.
    return lookUpValueWithKeys(['Version', 'Opera']);
  }

  // Check Edge before Chrome since it has Chrome in the string.
  if (goog.labs.userAgent.browser.isEdge()) {
    return lookUpValueWithKeys(['Edge']);
  }

  // Check Chromium Edge before Chrome since it has Chrome in the string.
  if (goog.labs.userAgent.browser.isEdgeChromium()) {
    return lookUpValueWithKeys(['Edg']);
  }

  if (goog.labs.userAgent.browser.isChrome()) {
    return lookUpValueWithKeys(['Chrome', 'CriOS', 'HeadlessChrome']);
  }

  // Usually products browser versions are in the third tuple after "Mozilla"
  // and the engine.
  var tuple = versionTuples[2];
  return tuple && tuple[1] || '';
};


/**
 * @param {string|number} version The version to check.
 * @return {boolean} Whether the browser version is higher or the same as the
 *     given version.
 */
goog.labs.userAgent.browser.isVersionOrHigher = function(version) {
  return goog.string.internal.compareVersions(
             goog.labs.userAgent.browser.getVersion(), version) >= 0;
};


/**
 * Determines IE version. More information:
 * http://msdn.microsoft.com/en-us/library/ie/bg182625(v=vs.85).aspx#uaString
 * http://msdn.microsoft.com/en-us/library/hh869301(v=vs.85).aspx
 * http://blogs.msdn.com/b/ie/archive/2010/03/23/introducing-ie9-s-user-agent-string.aspx
 * http://blogs.msdn.com/b/ie/archive/2009/01/09/the-internet-explorer-8-user-agent-string-updated-edition.aspx
 *
 * @param {string} userAgent the User-Agent.
 * @return {string}
 * @private
 */
goog.labs.userAgent.browser.getIEVersion_ = function(userAgent) {
  // IE11 may identify itself as MSIE 9.0 or MSIE 10.0 due to an IE 11 upgrade
  // bug. Example UA:
  // Mozilla/5.0 (MSIE 9.0; Windows NT 6.1; WOW64; Trident/7.0; rv:11.0)
  // like Gecko.
  // See http://www.whatismybrowser.com/developers/unknown-user-agent-fragments.
  var rv = /rv: *([\d\.]*)/.exec(userAgent);
  if (rv && rv[1]) {
    return rv[1];
  }

  var version = '';
  var msie = /MSIE +([\d\.]+)/.exec(userAgent);
  if (msie && msie[1]) {
    // IE in compatibility mode usually identifies itself as MSIE 7.0; in this
    // case, use the Trident version to determine the version of IE. For more
    // details, see the links above.
    var tridentVersion = /Trident\/(\d.\d)/.exec(userAgent);
    if (msie[1] == '7.0') {
      if (tridentVersion && tridentVersion[1]) {
        switch (tridentVersion[1]) {
          case '4.0':
            version = '8.0';
            break;
          case '5.0':
            version = '9.0';
            break;
          case '6.0':
            version = '10.0';
            break;
          case '7.0':
            version = '11.0';
            break;
        }
      } else {
        version = '7.0';
      }
    } else {
      version = msie[1];
    }
  }
  return version;
};

//third_party/javascript/closure/html/safehtml.js
/**
 * @license
 * Copyright The Closure Library Authors.
 * SPDX-License-Identifier: Apache-2.0
 */


/**
 * @fileoverview The SafeHtml type and its builders.
 *
 * TODO(xtof): Link to document stating type contract.
 */

goog.provide('goog.html.SafeHtml');

goog.require('goog.array');
goog.require('goog.asserts');
goog.require('goog.dom.TagName');
goog.require('goog.dom.tags');
goog.require('goog.html.SafeScript');
goog.require('goog.html.SafeStyle');
goog.require('goog.html.SafeStyleSheet');
goog.require('goog.html.SafeUrl');
goog.require('goog.html.TrustedResourceUrl');
goog.require('goog.html.trustedtypes');
goog.require('goog.i18n.bidi.Dir');
goog.require('goog.i18n.bidi.DirectionalString');
goog.require('goog.labs.userAgent.browser');
goog.require('goog.object');
goog.require('goog.string.Const');
goog.require('goog.string.TypedString');
goog.require('goog.string.internal');



/**
 * A string that is safe to use in HTML context in DOM APIs and HTML documents.
 *
 * A SafeHtml is a string-like object that carries the security type contract
 * that its value as a string will not cause untrusted script execution when
 * evaluated as HTML in a browser.
 *
 * Values of this type are guaranteed to be safe to use in HTML contexts,
 * such as, assignment to the innerHTML DOM property, or interpolation into
 * a HTML template in HTML PC_DATA context, in the sense that the use will not
 * result in a Cross-Site-Scripting vulnerability.
 *
 * Instances of this type must be created via the factory methods
 * (`goog.html.SafeHtml.create`, `goog.html.SafeHtml.htmlEscape`),
 * etc and not by invoking its constructor.  The constructor intentionally
 * takes no parameters and the type is immutable; hence only a default instance
 * corresponding to the empty string can be obtained via constructor invocation.
 *
 * Note that there is no `goog.html.SafeHtml.fromConstant`. The reason is that
 * the following code would create an unsafe HTML:
 *
 * ```
 * goog.html.SafeHtml.concat(
 *     goog.html.SafeHtml.fromConstant(goog.string.Const.from('<script>')),
 *     goog.html.SafeHtml.htmlEscape(userInput),
 *     goog.html.SafeHtml.fromConstant(goog.string.Const.from('<\/script>')));
 * ```
 *
 * There's `goog.dom.constHtmlToNode` to create a node from constant strings
 * only.
 *
 * @see goog.html.SafeHtml.create
 * @see goog.html.SafeHtml.htmlEscape
 * @constructor
 * @final
 * @struct
 * @implements {goog.i18n.bidi.DirectionalString}
 * @implements {goog.string.TypedString}
 */
goog.html.SafeHtml = function() {
  /**
   * The contained value of this SafeHtml.  The field has a purposely ugly
   * name to make (non-compiled) code that attempts to directly access this
   * field stand out.
   * @private {!TrustedHTML|string}
   */
  this.privateDoNotAccessOrElseSafeHtmlWrappedValue_ = '';

  /**
   * A type marker used to implement additional run-time type checking.
   * @see goog.html.SafeHtml.unwrap
   * @const {!Object}
   * @private
   */
  this.SAFE_HTML_TYPE_MARKER_GOOG_HTML_SECURITY_PRIVATE_ =
      goog.html.SafeHtml.TYPE_MARKER_GOOG_HTML_SECURITY_PRIVATE_;

  /**
   * This SafeHtml's directionality, or null if unknown.
   * @private {?goog.i18n.bidi.Dir}
   */
  this.dir_ = null;
};


/**
 * @define {boolean} Whether to strip out error messages or to leave them in.
 */
goog.html.SafeHtml.ENABLE_ERROR_MESSAGES =
    goog.define('goog.html.SafeHtml.ENABLE_ERROR_MESSAGES', goog.DEBUG);


/**
 * Whether the `style` attribute is supported. Set to false to avoid the byte
 * weight of `goog.html.SafeStyle` where unneeded. An error will be thrown if
 * the `style` attribute is used.
 * @define {boolean}
 */
goog.html.SafeHtml.SUPPORT_STYLE_ATTRIBUTE =
    goog.define('goog.html.SafeHtml.SUPPORT_STYLE_ATTRIBUTE', true);


/**
 * @override
 * @const
 */
goog.html.SafeHtml.prototype.implementsGoogI18nBidiDirectionalString = true;


/** @override */
goog.html.SafeHtml.prototype.getDirection = function() {
  return this.dir_;
};


/**
 * @override
 * @const
 */
goog.html.SafeHtml.prototype.implementsGoogStringTypedString = true;


/**
 * Returns this SafeHtml's value as string.
 *
 * IMPORTANT: In code where it is security relevant that an object's type is
 * indeed `SafeHtml`, use `goog.html.SafeHtml.unwrap` instead of
 * this method. If in doubt, assume that it's security relevant. In particular,
 * note that goog.html functions which return a goog.html type do not guarantee
 * that the returned instance is of the right type. For example:
 *
 * <pre>
 * var fakeSafeHtml = new String('fake');
 * fakeSafeHtml.__proto__ = goog.html.SafeHtml.prototype;
 * var newSafeHtml = goog.html.SafeHtml.htmlEscape(fakeSafeHtml);
 * // newSafeHtml is just an alias for fakeSafeHtml, it's passed through by
 * // goog.html.SafeHtml.htmlEscape() as fakeSafeHtml
 * // instanceof goog.html.SafeHtml.
 * </pre>
 *
 * @see goog.html.SafeHtml.unwrap
 * @override
 */
goog.html.SafeHtml.prototype.getTypedStringValue = function() {
  return this.privateDoNotAccessOrElseSafeHtmlWrappedValue_.toString();
};


if (goog.DEBUG) {
  /**
   * Returns a debug string-representation of this value.
   *
   * To obtain the actual string value wrapped in a SafeHtml, use
   * `goog.html.SafeHtml.unwrap`.
   *
   * @see goog.html.SafeHtml.unwrap
   * @override
   */
  goog.html.SafeHtml.prototype.toString = function() {
    return 'SafeHtml{' + this.privateDoNotAccessOrElseSafeHtmlWrappedValue_ +
        '}';
  };
}


/**
 * Performs a runtime check that the provided object is indeed a SafeHtml
 * object, and returns its value.
 * @param {!goog.html.SafeHtml} safeHtml The object to extract from.
 * @return {string} The SafeHtml object's contained string, unless the run-time
 *     type check fails. In that case, `unwrap` returns an innocuous
 *     string, or, if assertions are enabled, throws
 *     `goog.asserts.AssertionError`.
 */
goog.html.SafeHtml.unwrap = function(safeHtml) {
  return goog.html.SafeHtml.unwrapTrustedHTML(safeHtml).toString();
};


/**
 * Unwraps value as TrustedHTML if supported or as a string if not.
 * @param {!goog.html.SafeHtml} safeHtml
 * @return {!TrustedHTML|string}
 * @see goog.html.SafeHtml.unwrap
 */
goog.html.SafeHtml.unwrapTrustedHTML = function(safeHtml) {
  // Perform additional run-time type-checking to ensure that safeHtml is indeed
  // an instance of the expected type.  This provides some additional protection
  // against security bugs due to application code that disables type checks.
  // Specifically, the following checks are performed:
  // 1. The object is an instance of the expected type.
  // 2. The object is not an instance of a subclass.
  // 3. The object carries a type marker for the expected type. "Faking" an
  // object requires a reference to the type marker, which has names intended
  // to stand out in code reviews.
  if (safeHtml instanceof goog.html.SafeHtml &&
      safeHtml.constructor === goog.html.SafeHtml &&
      safeHtml.SAFE_HTML_TYPE_MARKER_GOOG_HTML_SECURITY_PRIVATE_ ===
          goog.html.SafeHtml.TYPE_MARKER_GOOG_HTML_SECURITY_PRIVATE_) {
    return safeHtml.privateDoNotAccessOrElseSafeHtmlWrappedValue_;
  } else {
    goog.asserts.fail('expected object of type SafeHtml, got \'' +
        safeHtml + '\' of type ' + goog.typeOf(safeHtml));
    return 'type_error:SafeHtml';
  }
};


/**
 * Shorthand for union of types that can sensibly be converted to strings
 * or might already be SafeHtml (as SafeHtml is a goog.string.TypedString).
 * @private
 * @typedef {string|number|boolean|!goog.string.TypedString|
 *           !goog.i18n.bidi.DirectionalString}
 */
goog.html.SafeHtml.TextOrHtml_;


/**
 * Returns HTML-escaped text as a SafeHtml object.
 *
 * If text is of a type that implements
 * `goog.i18n.bidi.DirectionalString`, the directionality of the new
 * `SafeHtml` object is set to `text`'s directionality, if known.
 * Otherwise, the directionality of the resulting SafeHtml is unknown (i.e.,
 * `null`).
 *
 * @param {!goog.html.SafeHtml.TextOrHtml_} textOrHtml The text to escape. If
 *     the parameter is of type SafeHtml it is returned directly (no escaping
 *     is done).
 * @return {!goog.html.SafeHtml} The escaped text, wrapped as a SafeHtml.
 */
goog.html.SafeHtml.htmlEscape = function(textOrHtml) {
  if (textOrHtml instanceof goog.html.SafeHtml) {
    return textOrHtml;
  }
  var textIsObject = typeof textOrHtml == 'object';
  var dir = null;
  if (textIsObject && textOrHtml.implementsGoogI18nBidiDirectionalString) {
    dir = /** @type {!goog.i18n.bidi.DirectionalString} */ (textOrHtml)
              .getDirection();
  }
  var textAsString;
  if (textIsObject && textOrHtml.implementsGoogStringTypedString) {
    textAsString = /** @type {!goog.string.TypedString} */ (textOrHtml)
                       .getTypedStringValue();
  } else {
    textAsString = String(textOrHtml);
  }
  return goog.html.SafeHtml.createSafeHtmlSecurityPrivateDoNotAccessOrElse(
      goog.string.internal.htmlEscape(textAsString), dir);
};


/**
 * Returns HTML-escaped text as a SafeHtml object, with newlines changed to
 * &lt;br&gt;.
 * @param {!goog.html.SafeHtml.TextOrHtml_} textOrHtml The text to escape. If
 *     the parameter is of type SafeHtml it is returned directly (no escaping
 *     is done).
 * @return {!goog.html.SafeHtml} The escaped text, wrapped as a SafeHtml.
 */
goog.html.SafeHtml.htmlEscapePreservingNewlines = function(textOrHtml) {
  if (textOrHtml instanceof goog.html.SafeHtml) {
    return textOrHtml;
  }
  var html = goog.html.SafeHtml.htmlEscape(textOrHtml);
  return goog.html.SafeHtml.createSafeHtmlSecurityPrivateDoNotAccessOrElse(
      goog.string.internal.newLineToBr(goog.html.SafeHtml.unwrap(html)),
      html.getDirection());
};


/**
 * Returns HTML-escaped text as a SafeHtml object, with newlines changed to
 * &lt;br&gt; and escaping whitespace to preserve spatial formatting. Character
 * entity #160 is used to make it safer for XML.
 * @param {!goog.html.SafeHtml.TextOrHtml_} textOrHtml The text to escape. If
 *     the parameter is of type SafeHtml it is returned directly (no escaping
 *     is done).
 * @return {!goog.html.SafeHtml} The escaped text, wrapped as a SafeHtml.
 */
goog.html.SafeHtml.htmlEscapePreservingNewlinesAndSpaces = function(
    textOrHtml) {
  if (textOrHtml instanceof goog.html.SafeHtml) {
    return textOrHtml;
  }
  var html = goog.html.SafeHtml.htmlEscape(textOrHtml);
  return goog.html.SafeHtml.createSafeHtmlSecurityPrivateDoNotAccessOrElse(
      goog.string.internal.whitespaceEscape(goog.html.SafeHtml.unwrap(html)),
      html.getDirection());
};


/**
 * Coerces an arbitrary object into a SafeHtml object.
 *
 * If `textOrHtml` is already of type `goog.html.SafeHtml`, the same
 * object is returned. Otherwise, `textOrHtml` is coerced to string, and
 * HTML-escaped. If `textOrHtml` is of a type that implements
 * `goog.i18n.bidi.DirectionalString`, its directionality, if known, is
 * preserved.
 *
 * @param {!goog.html.SafeHtml.TextOrHtml_} textOrHtml The text or SafeHtml to
 *     coerce.
 * @return {!goog.html.SafeHtml} The resulting SafeHtml object.
 * @deprecated Use goog.html.SafeHtml.htmlEscape.
 */
goog.html.SafeHtml.from = goog.html.SafeHtml.htmlEscape;


/**
 * @const
 * @private
 */
goog.html.SafeHtml.VALID_NAMES_IN_TAG_ = /^[a-zA-Z0-9-]+$/;


/**
 * Set of attributes containing URL as defined at
 * http://www.w3.org/TR/html5/index.html#attributes-1.
 * @private @const {!Object<string,boolean>}
 */
goog.html.SafeHtml.URL_ATTRIBUTES_ = goog.object.createSet(
    'action', 'cite', 'data', 'formaction', 'href', 'manifest', 'poster',
    'src');


/**
 * Tags which are unsupported via create(). They might be supported via a
 * tag-specific create method. These are tags which might require a
 * TrustedResourceUrl in one of their attributes or a restricted type for
 * their content.
 * @private @const {!Object<string,boolean>}
 */
goog.html.SafeHtml.NOT_ALLOWED_TAG_NAMES_ = goog.object.createSet(
    goog.dom.TagName.APPLET, goog.dom.TagName.BASE, goog.dom.TagName.EMBED,
    goog.dom.TagName.IFRAME, goog.dom.TagName.LINK, goog.dom.TagName.MATH,
    goog.dom.TagName.META, goog.dom.TagName.OBJECT, goog.dom.TagName.SCRIPT,
    goog.dom.TagName.STYLE, goog.dom.TagName.SVG, goog.dom.TagName.TEMPLATE);


/**
 * @typedef {string|number|goog.string.TypedString|
 *     goog.html.SafeStyle.PropertyMap|undefined}
 */
goog.html.SafeHtml.AttributeValue;


/**
 * Creates a SafeHtml content consisting of a tag with optional attributes and
 * optional content.
 *
 * For convenience tag names and attribute names are accepted as regular
 * strings, instead of goog.string.Const. Nevertheless, you should not pass
 * user-controlled values to these parameters. Note that these parameters are
 * syntactically validated at runtime, and invalid values will result in
 * an exception.
 *
 * Example usage:
 *
 * goog.html.SafeHtml.create('br');
 * goog.html.SafeHtml.create('div', {'class': 'a'});
 * goog.html.SafeHtml.create('p', {}, 'a');
 * goog.html.SafeHtml.create('p', {}, goog.html.SafeHtml.create('br'));
 *
 * goog.html.SafeHtml.create('span', {
 *   'style': {'margin': '0'}
 * });
 *
 * To guarantee SafeHtml's type contract is upheld there are restrictions on
 * attribute values and tag names.
 *
 * - For attributes which contain script code (on*), a goog.string.Const is
 *   required.
 * - For attributes which contain style (style), a goog.html.SafeStyle or a
 *   goog.html.SafeStyle.PropertyMap is required.
 * - For attributes which are interpreted as URLs (e.g. src, href) a
 *   goog.html.SafeUrl, goog.string.Const or string is required. If a string
 *   is passed, it will be sanitized with SafeUrl.sanitize().
 * - For tags which can load code or set security relevant page metadata,
 *   more specific goog.html.SafeHtml.create*() functions must be used. Tags
 *   which are not supported by this function are applet, base, embed, iframe,
 *   link, math, object, script, style, svg, and template.
 *
 * @param {!goog.dom.TagName|string} tagName The name of the tag. Only tag names
 *     consisting of [a-zA-Z0-9-] are allowed. Tag names documented above are
 *     disallowed.
 * @param {?Object<string, ?goog.html.SafeHtml.AttributeValue>=} opt_attributes
 *     Mapping from attribute names to their values. Only attribute names
 *     consisting of [a-zA-Z0-9-] are allowed. Value of null or undefined causes
 *     the attribute to be omitted.
 * @param {!goog.html.SafeHtml.TextOrHtml_|
 *     !Array<!goog.html.SafeHtml.TextOrHtml_>=} opt_content Content to
 *     HTML-escape and put inside the tag. This must be empty for void tags
 *     like <br>. Array elements are concatenated.
 * @return {!goog.html.SafeHtml} The SafeHtml content with the tag.
 * @throws {Error} If invalid tag name, attribute name, or attribute value is
 *     provided.
 * @throws {goog.asserts.AssertionError} If content for void tag is provided.
 */
goog.html.SafeHtml.create = function(tagName, opt_attributes, opt_content) {
  goog.html.SafeHtml.verifyTagName(String(tagName));
  return goog.html.SafeHtml.createSafeHtmlTagSecurityPrivateDoNotAccessOrElse(
      String(tagName), opt_attributes, opt_content);
};


/**
 * Verifies if the tag name is valid and if it doesn't change the context.
 * E.g. STRONG is fine but SCRIPT throws because it changes context. See
 * goog.html.SafeHtml.create for an explanation of allowed tags.
 * @param {string} tagName
 * @throws {Error} If invalid tag name is provided.
 * @package
 */
goog.html.SafeHtml.verifyTagName = function(tagName) {
  if (!goog.html.SafeHtml.VALID_NAMES_IN_TAG_.test(tagName)) {
    throw new Error(
        goog.html.SafeHtml.ENABLE_ERROR_MESSAGES ?
            'Invalid tag name <' + tagName + '>.' :
            '');
  }
  if (tagName.toUpperCase() in goog.html.SafeHtml.NOT_ALLOWED_TAG_NAMES_) {
    throw new Error(
        goog.html.SafeHtml.ENABLE_ERROR_MESSAGES ?

            'Tag name <' + tagName + '> is not allowed for SafeHtml.' :
            '');
  }
};


/**
 * Creates a SafeHtml representing an iframe tag.
 *
 * This by default restricts the iframe as much as possible by setting the
 * sandbox attribute to the empty string. If the iframe requires less
 * restrictions, set the sandbox attribute as tight as possible, but do not rely
 * on the sandbox as a security feature because it is not supported by older
 * browsers. If a sandbox is essential to security (e.g. for third-party
 * frames), use createSandboxIframe which checks for browser support.
 *
 * @see https://developer.mozilla.org/en/docs/Web/HTML/Element/iframe#attr-sandbox
 *
 * @param {?goog.html.TrustedResourceUrl=} opt_src The value of the src
 *     attribute. If null or undefined src will not be set.
 * @param {?goog.html.SafeHtml=} opt_srcdoc The value of the srcdoc attribute.
 *     If null or undefined srcdoc will not be set.
 * @param {?Object<string, ?goog.html.SafeHtml.AttributeValue>=} opt_attributes
 *     Mapping from attribute names to their values. Only attribute names
 *     consisting of [a-zA-Z0-9-] are allowed. Value of null or undefined causes
 *     the attribute to be omitted.
 * @param {!goog.html.SafeHtml.TextOrHtml_|
 *     !Array<!goog.html.SafeHtml.TextOrHtml_>=} opt_content Content to
 *     HTML-escape and put inside the tag. Array elements are concatenated.
 * @return {!goog.html.SafeHtml} The SafeHtml content with the tag.
 * @throws {Error} If invalid tag name, attribute name, or attribute value is
 *     provided. If opt_attributes contains the src or srcdoc attributes.
 */
goog.html.SafeHtml.createIframe = function(
    opt_src, opt_srcdoc, opt_attributes, opt_content) {
  if (opt_src) {
    // Check whether this is really TrustedResourceUrl.
    goog.html.TrustedResourceUrl.unwrap(opt_src);
  }

  var fixedAttributes = {};
  fixedAttributes['src'] = opt_src || null;
  fixedAttributes['srcdoc'] =
      opt_srcdoc && goog.html.SafeHtml.unwrap(opt_srcdoc);
  var defaultAttributes = {'sandbox': ''};
  var attributes = goog.html.SafeHtml.combineAttributes(
      fixedAttributes, defaultAttributes, opt_attributes);
  return goog.html.SafeHtml.createSafeHtmlTagSecurityPrivateDoNotAccessOrElse(
      'iframe', attributes, opt_content);
};


/**
 * Creates a SafeHtml representing a sandboxed iframe tag.
 *
 * The sandbox attribute is enforced in its most restrictive mode, an empty
 * string. Consequently, the security requirements for the src and srcdoc
 * attributes are relaxed compared to SafeHtml.createIframe. This function
 * will throw on browsers that do not support the sandbox attribute, as
 * determined by SafeHtml.canUseSandboxIframe.
 *
 * The SafeHtml returned by this function can trigger downloads with no
 * user interaction on Chrome (though only a few, further attempts are blocked).
 * Firefox and IE will block all downloads from the sandbox.
 *
 * @see https://developer.mozilla.org/en/docs/Web/HTML/Element/iframe#attr-sandbox
 * @see https://lists.w3.org/Archives/Public/public-whatwg-archive/2013Feb/0112.html
 *
 * @param {string|!goog.html.SafeUrl=} opt_src The value of the src
 *     attribute. If null or undefined src will not be set.
 * @param {string=} opt_srcdoc The value of the srcdoc attribute.
 *     If null or undefined srcdoc will not be set. Will not be sanitized.
 * @param {!Object<string, ?goog.html.SafeHtml.AttributeValue>=} opt_attributes
 *     Mapping from attribute names to their values. Only attribute names
 *     consisting of [a-zA-Z0-9-] are allowed. Value of null or undefined causes
 *     the attribute to be omitted.
 * @param {!goog.html.SafeHtml.TextOrHtml_|
 *     !Array<!goog.html.SafeHtml.TextOrHtml_>=} opt_content Content to
 *     HTML-escape and put inside the tag. Array elements are concatenated.
 * @return {!goog.html.SafeHtml} The SafeHtml content with the tag.
 * @throws {Error} If invalid tag name, attribute name, or attribute value is
 *     provided. If opt_attributes contains the src, srcdoc or sandbox
 *     attributes. If browser does not support the sandbox attribute on iframe.
 */
goog.html.SafeHtml.createSandboxIframe = function(
    opt_src, opt_srcdoc, opt_attributes, opt_content) {
  if (!goog.html.SafeHtml.canUseSandboxIframe()) {
    throw new Error(
        goog.html.SafeHtml.ENABLE_ERROR_MESSAGES ?
            'The browser does not support sandboxed iframes.' :
            '');
  }

  var fixedAttributes = {};
  if (opt_src) {
    // Note that sanitize is a no-op on SafeUrl.
    fixedAttributes['src'] =
        goog.html.SafeUrl.unwrap(goog.html.SafeUrl.sanitize(opt_src));
  } else {
    fixedAttributes['src'] = null;
  }
  fixedAttributes['srcdoc'] = opt_srcdoc || null;
  fixedAttributes['sandbox'] = '';
  var attributes =
      goog.html.SafeHtml.combineAttributes(fixedAttributes, {}, opt_attributes);
  return goog.html.SafeHtml.createSafeHtmlTagSecurityPrivateDoNotAccessOrElse(
      'iframe', attributes, opt_content);
};


/**
 * Checks if the user agent supports sandboxed iframes.
 * @return {boolean}
 */
goog.html.SafeHtml.canUseSandboxIframe = function() {
  return goog.global['HTMLIFrameElement'] &&
      ('sandbox' in goog.global['HTMLIFrameElement'].prototype);
};


/**
 * Creates a SafeHtml representing a script tag with the src attribute.
 * @param {!goog.html.TrustedResourceUrl} src The value of the src
 * attribute.
 * @param {?Object<string, ?goog.html.SafeHtml.AttributeValue>=}
 * opt_attributes
 *     Mapping from attribute names to their values. Only attribute names
 *     consisting of [a-zA-Z0-9-] are allowed. Value of null or undefined
 *     causes the attribute to be omitted.
 * @return {!goog.html.SafeHtml} The SafeHtml content with the tag.
 * @throws {Error} If invalid attribute name or value is provided. If
 *     opt_attributes contains the src attribute.
 */
goog.html.SafeHtml.createScriptSrc = function(src, opt_attributes) {
  // TODO(mlourenco): The charset attribute should probably be blocked. If
  // its value is attacker controlled, the script contains attacker controlled
  // sub-strings (even if properly escaped) and the server does not set charset
  // then XSS is likely possible.
  // https://html.spec.whatwg.org/multipage/scripting.html#dom-script-charset

  // Check whether this is really TrustedResourceUrl.
  goog.html.TrustedResourceUrl.unwrap(src);

  var fixedAttributes = {'src': src};
  var defaultAttributes = {};
  var attributes = goog.html.SafeHtml.combineAttributes(
      fixedAttributes, defaultAttributes, opt_attributes);
  return goog.html.SafeHtml.createSafeHtmlTagSecurityPrivateDoNotAccessOrElse(
      'script', attributes);
};


/**
 * Creates a SafeHtml representing a script tag. Does not allow the language,
 * src, text or type attributes to be set.
 * @param {!goog.html.SafeScript|!Array<!goog.html.SafeScript>}
 *     script Content to put inside the tag. Array elements are
 *     concatenated.
 * @param {?Object<string, ?goog.html.SafeHtml.AttributeValue>=} opt_attributes
 *     Mapping from attribute names to their values. Only attribute names
 *     consisting of [a-zA-Z0-9-] are allowed. Value of null or undefined causes
 *     the attribute to be omitted.
 * @return {!goog.html.SafeHtml} The SafeHtml content with the tag.
 * @throws {Error} If invalid attribute name or attribute value is provided. If
 *     opt_attributes contains the language, src, text or type attribute.
 */
goog.html.SafeHtml.createScript = function(script, opt_attributes) {
  for (var attr in opt_attributes) {
    var attrLower = attr.toLowerCase();
    if (attrLower == 'language' || attrLower == 'src' || attrLower == 'text' ||
        attrLower == 'type') {
      throw new Error(
          goog.html.SafeHtml.ENABLE_ERROR_MESSAGES ?
              'Cannot set "' + attrLower + '" attribute' :
              '');
    }
  }

  var content = '';
  script = goog.array.concat(script);
  for (var i = 0; i < script.length; i++) {
    content += goog.html.SafeScript.unwrap(script[i]);
  }
  // Convert to SafeHtml so that it's not HTML-escaped. This is safe because
  // as part of its contract, SafeScript should have no dangerous '<'.
  var htmlContent =
      goog.html.SafeHtml.createSafeHtmlSecurityPrivateDoNotAccessOrElse(
          content, goog.i18n.bidi.Dir.NEUTRAL);
  return goog.html.SafeHtml.createSafeHtmlTagSecurityPrivateDoNotAccessOrElse(
      'script', opt_attributes, htmlContent);
};


/**
 * Creates a SafeHtml representing a style tag. The type attribute is set
 * to "text/css".
 * @param {!goog.html.SafeStyleSheet|!Array<!goog.html.SafeStyleSheet>}
 *     styleSheet Content to put inside the tag. Array elements are
 *     concatenated.
 * @param {?Object<string, ?goog.html.SafeHtml.AttributeValue>=} opt_attributes
 *     Mapping from attribute names to their values. Only attribute names
 *     consisting of [a-zA-Z0-9-] are allowed. Value of null or undefined causes
 *     the attribute to be omitted.
 * @return {!goog.html.SafeHtml} The SafeHtml content with the tag.
 * @throws {Error} If invalid attribute name or attribute value is provided. If
 *     opt_attributes contains the type attribute.
 */
goog.html.SafeHtml.createStyle = function(styleSheet, opt_attributes) {
  var fixedAttributes = {'type': 'text/css'};
  var defaultAttributes = {};
  var attributes = goog.html.SafeHtml.combineAttributes(
      fixedAttributes, defaultAttributes, opt_attributes);

  var content = '';
  styleSheet = goog.array.concat(styleSheet);
  for (var i = 0; i < styleSheet.length; i++) {
    content += goog.html.SafeStyleSheet.unwrap(styleSheet[i]);
  }
  // Convert to SafeHtml so that it's not HTML-escaped. This is safe because
  // as part of its contract, SafeStyleSheet should have no dangerous '<'.
  var htmlContent =
      goog.html.SafeHtml.createSafeHtmlSecurityPrivateDoNotAccessOrElse(
          content, goog.i18n.bidi.Dir.NEUTRAL);
  return goog.html.SafeHtml.createSafeHtmlTagSecurityPrivateDoNotAccessOrElse(
      'style', attributes, htmlContent);
};


/**
 * Creates a SafeHtml representing a meta refresh tag.
 * @param {!goog.html.SafeUrl|string} url Where to redirect. If a string is
 *     passed, it will be sanitized with SafeUrl.sanitize().
 * @param {number=} opt_secs Number of seconds until the page should be
 *     reloaded. Will be set to 0 if unspecified.
 * @return {!goog.html.SafeHtml} The SafeHtml content with the tag.
 */
goog.html.SafeHtml.createMetaRefresh = function(url, opt_secs) {

  // Note that sanitize is a no-op on SafeUrl.
  var unwrappedUrl = goog.html.SafeUrl.unwrap(goog.html.SafeUrl.sanitize(url));

  if (goog.labs.userAgent.browser.isIE() ||
      goog.labs.userAgent.browser.isEdge()) {
    // IE/EDGE can't parse the content attribute if the url contains a
    // semicolon. We can fix this by adding quotes around the url, but then we
    // can't parse quotes in the URL correctly. Also, it seems that IE/EDGE
    // did not unescape semicolons in these URLs at some point in the past. We
    // take a best-effort approach.
    //
    // If the URL has semicolons (which may happen in some cases, see
    // http://www.w3.org/TR/1999/REC-html401-19991224/appendix/notes.html#h-B.2
    // for instance), wrap it in single quotes to protect the semicolons.
    // If the URL has semicolons and single quotes, url-encode the single quotes
    // as well.
    //
    // This is imperfect. Notice that both ' and ; are reserved characters in
    // URIs, so this could do the wrong thing, but at least it will do the wrong
    // thing in only rare cases.
    if (goog.string.internal.contains(unwrappedUrl, ';')) {
      unwrappedUrl = "'" + unwrappedUrl.replace(/'/g, '%27') + "'";
    }
  }
  var attributes = {
    'http-equiv': 'refresh',
    'content': (opt_secs || 0) + '; url=' + unwrappedUrl
  };

  // This function will handle the HTML escaping for attributes.
  return goog.html.SafeHtml.createSafeHtmlTagSecurityPrivateDoNotAccessOrElse(
      'meta', attributes);
};


/**
 * @param {string} tagName The tag name.
 * @param {string} name The attribute name.
 * @param {!goog.html.SafeHtml.AttributeValue} value The attribute value.
 * @return {string} A "name=value" string.
 * @throws {Error} If attribute value is unsafe for the given tag and attribute.
 * @private
 */
goog.html.SafeHtml.getAttrNameAndValue_ = function(tagName, name, value) {
  // If it's goog.string.Const, allow any valid attribute name.
  if (value instanceof goog.string.Const) {
    value = goog.string.Const.unwrap(value);
  } else if (name.toLowerCase() == 'style') {
    if (goog.html.SafeHtml.SUPPORT_STYLE_ATTRIBUTE) {
      value = goog.html.SafeHtml.getStyleValue_(value);
    } else {
      throw new Error(
          goog.html.SafeHtml.ENABLE_ERROR_MESSAGES ?
              'Attribute "style" not supported.' :
              '');
    }
  } else if (/^on/i.test(name)) {
    // TODO(jakubvrana): Disallow more attributes with a special meaning.
    throw new Error(
        goog.html.SafeHtml.ENABLE_ERROR_MESSAGES ? 'Attribute "' + name +
                '" requires goog.string.Const value, "' + value + '" given.' :
                                                   '');
    // URL attributes handled differently according to tag.
  } else if (name.toLowerCase() in goog.html.SafeHtml.URL_ATTRIBUTES_) {
    if (value instanceof goog.html.TrustedResourceUrl) {
      value = goog.html.TrustedResourceUrl.unwrap(value);
    } else if (value instanceof goog.html.SafeUrl) {
      value = goog.html.SafeUrl.unwrap(value);
    } else if (typeof value === 'string') {
      value = goog.html.SafeUrl.sanitize(value).getTypedStringValue();
    } else {
      throw new Error(
          goog.html.SafeHtml.ENABLE_ERROR_MESSAGES ?
              'Attribute "' + name + '" on tag "' + tagName +
                  '" requires goog.html.SafeUrl, goog.string.Const, or' +
                  ' string, value "' + value + '" given.' :
              '');
    }
  }

  // Accept SafeUrl, TrustedResourceUrl, etc. for attributes which only require
  // HTML-escaping.
  if (value.implementsGoogStringTypedString) {
    // Ok to call getTypedStringValue() since there's no reliance on the type
    // contract for security here.
    value =
        /** @type {!goog.string.TypedString} */ (value).getTypedStringValue();
  }

  goog.asserts.assert(
      typeof value === 'string' || typeof value === 'number',
      'String or number value expected, got ' + (typeof value) +
          ' with value: ' + value);
  return name + '="' + goog.string.internal.htmlEscape(String(value)) + '"';
};


/**
 * Gets value allowed in "style" attribute.
 * @param {!goog.html.SafeHtml.AttributeValue} value It could be SafeStyle or a
 *     map which will be passed to goog.html.SafeStyle.create.
 * @return {string} Unwrapped value.
 * @throws {Error} If string value is given.
 * @private
 */
goog.html.SafeHtml.getStyleValue_ = function(value) {
  if (!goog.isObject(value)) {
    throw new Error(
        goog.html.SafeHtml.ENABLE_ERROR_MESSAGES ?
            'The "style" attribute requires goog.html.SafeStyle or map ' +
                'of style properties, ' + (typeof value) + ' given: ' + value :
            '');
  }
  if (!(value instanceof goog.html.SafeStyle)) {
    // Process the property bag into a style object.
    value = goog.html.SafeStyle.create(value);
  }
  return goog.html.SafeStyle.unwrap(value);
};


/**
 * Creates a SafeHtml content with known directionality consisting of a tag with
 * optional attributes and optional content.
 * @param {!goog.i18n.bidi.Dir} dir Directionality.
 * @param {string} tagName
 * @param {?Object<string, ?goog.html.SafeHtml.AttributeValue>=} opt_attributes
 * @param {!goog.html.SafeHtml.TextOrHtml_|
 *     !Array<!goog.html.SafeHtml.TextOrHtml_>=} opt_content
 * @return {!goog.html.SafeHtml} The SafeHtml content with the tag.
 */
goog.html.SafeHtml.createWithDir = function(
    dir, tagName, opt_attributes, opt_content) {
  var html = goog.html.SafeHtml.create(tagName, opt_attributes, opt_content);
  html.dir_ = dir;
  return html;
};


/**
 * Creates a new SafeHtml object by joining the parts with separator.
 * @param {!goog.html.SafeHtml.TextOrHtml_} separator
 * @param {!Array<!goog.html.SafeHtml.TextOrHtml_|
 *     !Array<!goog.html.SafeHtml.TextOrHtml_>>} parts Parts to join. If a part
 *     contains an array then each member of this array is also joined with the
 *     separator.
 * @return {!goog.html.SafeHtml}
 */
goog.html.SafeHtml.join = function(separator, parts) {
  var separatorHtml = goog.html.SafeHtml.htmlEscape(separator);
  var dir = separatorHtml.getDirection();
  var content = [];

  /**
   * @param {!goog.html.SafeHtml.TextOrHtml_|
   *     !Array<!goog.html.SafeHtml.TextOrHtml_>} argument
   */
  var addArgument = function(argument) {
    if (Array.isArray(argument)) {
      goog.array.forEach(argument, addArgument);
    } else {
      var html = goog.html.SafeHtml.htmlEscape(argument);
      content.push(goog.html.SafeHtml.unwrap(html));
      var htmlDir = html.getDirection();
      if (dir == goog.i18n.bidi.Dir.NEUTRAL) {
        dir = htmlDir;
      } else if (htmlDir != goog.i18n.bidi.Dir.NEUTRAL && dir != htmlDir) {
        dir = null;
      }
    }
  };

  goog.array.forEach(parts, addArgument);
  return goog.html.SafeHtml.createSafeHtmlSecurityPrivateDoNotAccessOrElse(
      content.join(goog.html.SafeHtml.unwrap(separatorHtml)), dir);
};


/**
 * Creates a new SafeHtml object by concatenating values.
 * @param {...(!goog.html.SafeHtml.TextOrHtml_|
 *     !Array<!goog.html.SafeHtml.TextOrHtml_>)} var_args Values to concatenate.
 * @return {!goog.html.SafeHtml}
 */
goog.html.SafeHtml.concat = function(var_args) {
  return goog.html.SafeHtml.join(
      goog.html.SafeHtml.EMPTY, Array.prototype.slice.call(arguments));
};


/**
 * Creates a new SafeHtml object with known directionality by concatenating the
 * values.
 * @param {!goog.i18n.bidi.Dir} dir Directionality.
 * @param {...(!goog.html.SafeHtml.TextOrHtml_|
 *     !Array<!goog.html.SafeHtml.TextOrHtml_>)} var_args Elements of array
 *     arguments would be processed recursively.
 * @return {!goog.html.SafeHtml}
 */
goog.html.SafeHtml.concatWithDir = function(dir, var_args) {
  var html = goog.html.SafeHtml.concat(goog.array.slice(arguments, 1));
  html.dir_ = dir;
  return html;
};


/**
 * Type marker for the SafeHtml type, used to implement additional run-time
 * type checking.
 * @const {!Object}
 * @private
 */
goog.html.SafeHtml.TYPE_MARKER_GOOG_HTML_SECURITY_PRIVATE_ = {};


/**
 * Package-internal utility method to create SafeHtml instances.
 *
 * @param {string} html The string to initialize the SafeHtml object with.
 * @param {?goog.i18n.bidi.Dir} dir The directionality of the SafeHtml to be
 *     constructed, or null if unknown.
 * @return {!goog.html.SafeHtml} The initialized SafeHtml object.
 * @package
 */
goog.html.SafeHtml.createSafeHtmlSecurityPrivateDoNotAccessOrElse = function(
    html, dir) {
  return new goog.html.SafeHtml().initSecurityPrivateDoNotAccessOrElse_(
      html, dir);
};


/**
 * Called from createSafeHtmlSecurityPrivateDoNotAccessOrElse(). This
 * method exists only so that the compiler can dead code eliminate static
 * fields (like EMPTY) when they're not accessed.
 * @param {string} html
 * @param {?goog.i18n.bidi.Dir} dir
 * @return {!goog.html.SafeHtml}
 * @private
 */
goog.html.SafeHtml.prototype.initSecurityPrivateDoNotAccessOrElse_ = function(
    html, dir) {
  this.privateDoNotAccessOrElseSafeHtmlWrappedValue_ =
      goog.html.trustedtypes.PRIVATE_DO_NOT_ACCESS_OR_ELSE_POLICY ?
      goog.html.trustedtypes.PRIVATE_DO_NOT_ACCESS_OR_ELSE_POLICY.createHTML(
          html) :
      html;
  this.dir_ = dir;
  return this;
};


/**
 * Like create() but does not restrict which tags can be constructed.
 *
 * @param {string} tagName Tag name. Set or validated by caller.
 * @param {?Object<string, ?goog.html.SafeHtml.AttributeValue>=} opt_attributes
 * @param {(!goog.html.SafeHtml.TextOrHtml_|
 *     !Array<!goog.html.SafeHtml.TextOrHtml_>)=} opt_content
 * @return {!goog.html.SafeHtml}
 * @throws {Error} If invalid or unsafe attribute name or value is provided.
 * @throws {goog.asserts.AssertionError} If content for void tag is provided.
 * @package
 */
goog.html.SafeHtml.createSafeHtmlTagSecurityPrivateDoNotAccessOrElse = function(
    tagName, opt_attributes, opt_content) {
  var dir = null;
  var result = '<' + tagName;
  result += goog.html.SafeHtml.stringifyAttributes(tagName, opt_attributes);

  var content = opt_content;
  if (content == null) {
    content = [];
  } else if (!Array.isArray(content)) {
    content = [content];
  }

  if (goog.dom.tags.isVoidTag(tagName.toLowerCase())) {
    goog.asserts.assert(
        !content.length, 'Void tag <' + tagName + '> does not allow content.');
    result += '>';
  } else {
    var html = goog.html.SafeHtml.concat(content);
    result += '>' + goog.html.SafeHtml.unwrap(html) + '</' + tagName + '>';
    dir = html.getDirection();
  }

  var dirAttribute = opt_attributes && opt_attributes['dir'];
  if (dirAttribute) {
    if (/^(ltr|rtl|auto)$/i.test(dirAttribute)) {
      // If the tag has the "dir" attribute specified then its direction is
      // neutral because it can be safely used in any context.
      dir = goog.i18n.bidi.Dir.NEUTRAL;
    } else {
      dir = null;
    }
  }

  return goog.html.SafeHtml.createSafeHtmlSecurityPrivateDoNotAccessOrElse(
      result, dir);
};


/**
 * Creates a string with attributes to insert after tagName.
 * @param {string} tagName
 * @param {?Object<string, ?goog.html.SafeHtml.AttributeValue>=} opt_attributes
 * @return {string} Returns an empty string if there are no attributes, returns
 *     a string starting with a space otherwise.
 * @throws {Error} If attribute value is unsafe for the given tag and attribute.
 * @package
 */
goog.html.SafeHtml.stringifyAttributes = function(tagName, opt_attributes) {
  var result = '';
  if (opt_attributes) {
    for (var name in opt_attributes) {
      if (!goog.html.SafeHtml.VALID_NAMES_IN_TAG_.test(name)) {
        throw new Error(
            goog.html.SafeHtml.ENABLE_ERROR_MESSAGES ?
                'Invalid attribute name "' + name + '".' :
                '');
      }
      var value = opt_attributes[name];
      if (value == null) {
        continue;
      }
      result +=
          ' ' + goog.html.SafeHtml.getAttrNameAndValue_(tagName, name, value);
    }
  }
  return result;
};


/**
 * @param {!Object<string, ?goog.html.SafeHtml.AttributeValue>} fixedAttributes
 * @param {!Object<string, string>} defaultAttributes
 * @param {?Object<string, ?goog.html.SafeHtml.AttributeValue>=} opt_attributes
 *     Optional attributes passed to create*().
 * @return {!Object<string, ?goog.html.SafeHtml.AttributeValue>}
 * @throws {Error} If opt_attributes contains an attribute with the same name
 *     as an attribute in fixedAttributes.
 * @package
 */
goog.html.SafeHtml.combineAttributes = function(
    fixedAttributes, defaultAttributes, opt_attributes) {
  var combinedAttributes = {};
  var name;

  for (name in fixedAttributes) {
    goog.asserts.assert(name.toLowerCase() == name, 'Must be lower case');
    combinedAttributes[name] = fixedAttributes[name];
  }
  for (name in defaultAttributes) {
    goog.asserts.assert(name.toLowerCase() == name, 'Must be lower case');
    combinedAttributes[name] = defaultAttributes[name];
  }

  if (opt_attributes) {
    for (name in opt_attributes) {
      var nameLower = name.toLowerCase();
      if (nameLower in fixedAttributes) {
        throw new Error(
            goog.html.SafeHtml.ENABLE_ERROR_MESSAGES ?
                'Cannot override "' + nameLower + '" attribute, got "' + name +
                    '" with value "' + opt_attributes[name] + '"' :
                '');
      }
      if (nameLower in defaultAttributes) {
        delete combinedAttributes[nameLower];
      }
      combinedAttributes[name] = opt_attributes[name];
    }
  }

  return combinedAttributes;
};


/**
 * A SafeHtml instance corresponding to the HTML doctype: "<!DOCTYPE html>".
 * @const {!goog.html.SafeHtml}
 */
goog.html.SafeHtml.DOCTYPE_HTML =
    goog.html.SafeHtml.createSafeHtmlSecurityPrivateDoNotAccessOrElse(
        '<!DOCTYPE html>', goog.i18n.bidi.Dir.NEUTRAL);


/**
 * A SafeHtml instance corresponding to the empty string.
 * @const {!goog.html.SafeHtml}
 */
goog.html.SafeHtml.EMPTY =
    goog.html.SafeHtml.createSafeHtmlSecurityPrivateDoNotAccessOrElse(
        '', goog.i18n.bidi.Dir.NEUTRAL);


/**
 * A SafeHtml instance corresponding to the <br> tag.
 * @const {!goog.html.SafeHtml}
 */
goog.html.SafeHtml.BR =
    goog.html.SafeHtml.createSafeHtmlSecurityPrivateDoNotAccessOrElse(
        '<br>', goog.i18n.bidi.Dir.NEUTRAL);

//third_party/javascript/closure/html/uncheckedconversions.js
/**
 * @license
 * Copyright The Closure Library Authors.
 * SPDX-License-Identifier: Apache-2.0
 */

/**
 * @fileoverview Unchecked conversions to create values of goog.html types from
 * plain strings.  Use of these functions could potentially result in instances
 * of goog.html types that violate their type contracts, and hence result in
 * security vulnerabilties.
 *
 * Therefore, all uses of the methods herein must be carefully security
 * reviewed.  Avoid use of the methods in this file whenever possible; instead
 * prefer to create instances of goog.html types using inherently safe builders
 * or template systems.
 *
 * MOE:begin_intracomment_strip
 * See http://go/safehtml-unchecked for guidelines on using these functions.
 * MOE:end_intracomment_strip
 *
 * MOE:begin_intracomment_strip
 * MAINTAINERS: Use of these functions is detected with a Tricorder analyzer.
 * If adding functions here also add them to analyzer's list at
 * j/c/g/devtools/staticanalysis/pipeline/analyzers/shared/SafeHtmlAnalyzers.java.
 * MOE:end_intracomment_strip
 */


goog.provide('goog.html.uncheckedconversions');

goog.require('goog.asserts');
goog.require('goog.html.SafeHtml');
goog.require('goog.html.SafeScript');
goog.require('goog.html.SafeStyle');
goog.require('goog.html.SafeStyleSheet');
goog.require('goog.html.SafeUrl');
goog.require('goog.html.TrustedResourceUrl');
goog.require('goog.string.Const');
goog.require('goog.string.internal');


/**
 * Performs an "unchecked conversion" to SafeHtml from a plain string that is
 * known to satisfy the SafeHtml type contract.
 *
 * IMPORTANT: Uses of this method must be carefully security-reviewed to ensure
 * that the value of `html` satisfies the SafeHtml type contract in all
 * possible program states.
 *
 * MOE:begin_intracomment_strip
 * See http://go/safehtml-unchecked for guidelines on using these functions.
 * MOE:end_intracomment_strip
 *
 * @param {!goog.string.Const} justification A constant string explaining why
 *     this use of this method is safe. May include a security review ticket
 *     number.
 * @param {string} html A string that is claimed to adhere to the SafeHtml
 *     contract.
 * @param {?goog.i18n.bidi.Dir=} opt_dir The optional directionality of the
 *     SafeHtml to be constructed. A null or undefined value signifies an
 *     unknown directionality.
 * @return {!goog.html.SafeHtml} The value of html, wrapped in a SafeHtml
 *     object.
 */
goog.html.uncheckedconversions.safeHtmlFromStringKnownToSatisfyTypeContract =
    function(justification, html, opt_dir) {
  // unwrap() called inside an assert so that justification can be optimized
  // away in production code.
  goog.asserts.assertString(
      goog.string.Const.unwrap(justification), 'must provide justification');
  goog.asserts.assert(
      !goog.string.internal.isEmptyOrWhitespace(
          goog.string.Const.unwrap(justification)),
      'must provide non-empty justification');
  return goog.html.SafeHtml.createSafeHtmlSecurityPrivateDoNotAccessOrElse(
      html, opt_dir || null);
};


/**
 * Performs an "unchecked conversion" to SafeScript from a plain string that is
 * known to satisfy the SafeScript type contract.
 *
 * IMPORTANT: Uses of this method must be carefully security-reviewed to ensure
 * that the value of `script` satisfies the SafeScript type contract in
 * all possible program states.
 *
 * MOE:begin_intracomment_strip
 * See http://go/safehtml-unchecked for guidelines on using these functions.
 * MOE:end_intracomment_strip
 *
 * @param {!goog.string.Const} justification A constant string explaining why
 *     this use of this method is safe. May include a security review ticket
 *     number.
 * @param {string} script The string to wrap as a SafeScript.
 * @return {!goog.html.SafeScript} The value of `script`, wrapped in a
 *     SafeScript object.
 */
goog.html.uncheckedconversions.safeScriptFromStringKnownToSatisfyTypeContract =
    function(justification, script) {
  // unwrap() called inside an assert so that justification can be optimized
  // away in production code.
  goog.asserts.assertString(
      goog.string.Const.unwrap(justification), 'must provide justification');
  goog.asserts.assert(
      !goog.string.internal.isEmptyOrWhitespace(
          goog.string.Const.unwrap(justification)),
      'must provide non-empty justification');
  return goog.html.SafeScript.createSafeScriptSecurityPrivateDoNotAccessOrElse(
      script);
};


/**
 * Performs an "unchecked conversion" to SafeStyle from a plain string that is
 * known to satisfy the SafeStyle type contract.
 *
 * IMPORTANT: Uses of this method must be carefully security-reviewed to ensure
 * that the value of `style` satisfies the SafeStyle type contract in all
 * possible program states.
 *
 * MOE:begin_intracomment_strip
 * See http://go/safehtml-unchecked for guidelines on using these functions.
 * MOE:end_intracomment_strip
 *
 * @param {!goog.string.Const} justification A constant string explaining why
 *     this use of this method is safe. May include a security review ticket
 *     number.
 * @param {string} style The string to wrap as a SafeStyle.
 * @return {!goog.html.SafeStyle} The value of `style`, wrapped in a
 *     SafeStyle object.
 */
goog.html.uncheckedconversions.safeStyleFromStringKnownToSatisfyTypeContract =
    function(justification, style) {
  // unwrap() called inside an assert so that justification can be optimized
  // away in production code.
  goog.asserts.assertString(
      goog.string.Const.unwrap(justification), 'must provide justification');
  goog.asserts.assert(
      !goog.string.internal.isEmptyOrWhitespace(
          goog.string.Const.unwrap(justification)),
      'must provide non-empty justification');
  return goog.html.SafeStyle.createSafeStyleSecurityPrivateDoNotAccessOrElse(
      style);
};


/**
 * Performs an "unchecked conversion" to SafeStyleSheet from a plain string
 * that is known to satisfy the SafeStyleSheet type contract.
 *
 * IMPORTANT: Uses of this method must be carefully security-reviewed to ensure
 * that the value of `styleSheet` satisfies the SafeStyleSheet type
 * contract in all possible program states.
 *
 * MOE:begin_intracomment_strip
 * See http://go/safehtml-unchecked for guidelines on using these functions.
 * MOE:end_intracomment_strip
 *
 * @param {!goog.string.Const} justification A constant string explaining why
 *     this use of this method is safe. May include a security review ticket
 *     number.
 * @param {string} styleSheet The string to wrap as a SafeStyleSheet.
 * @return {!goog.html.SafeStyleSheet} The value of `styleSheet`, wrapped
 *     in a SafeStyleSheet object.
 */
goog.html.uncheckedconversions
    .safeStyleSheetFromStringKnownToSatisfyTypeContract = function(
    justification, styleSheet) {
  // unwrap() called inside an assert so that justification can be optimized
  // away in production code.
  goog.asserts.assertString(
      goog.string.Const.unwrap(justification), 'must provide justification');
  goog.asserts.assert(
      !goog.string.internal.isEmptyOrWhitespace(
          goog.string.Const.unwrap(justification)),
      'must provide non-empty justification');
  return goog.html.SafeStyleSheet
      .createSafeStyleSheetSecurityPrivateDoNotAccessOrElse(styleSheet);
};


/**
 * Performs an "unchecked conversion" to SafeUrl from a plain string that is
 * known to satisfy the SafeUrl type contract.
 *
 * IMPORTANT: Uses of this method must be carefully security-reviewed to ensure
 * that the value of `url` satisfies the SafeUrl type contract in all
 * possible program states.
 *
 * MOE:begin_intracomment_strip
 * See http://go/safehtml-unchecked for guidelines on using these functions.
 * MOE:end_intracomment_strip
 *
 * @param {!goog.string.Const} justification A constant string explaining why
 *     this use of this method is safe. May include a security review ticket
 *     number.
 * @param {string} url The string to wrap as a SafeUrl.
 * @return {!goog.html.SafeUrl} The value of `url`, wrapped in a SafeUrl
 *     object.
 */
goog.html.uncheckedconversions.safeUrlFromStringKnownToSatisfyTypeContract =
    function(justification, url) {
  // unwrap() called inside an assert so that justification can be optimized
  // away in production code.
  goog.asserts.assertString(
      goog.string.Const.unwrap(justification), 'must provide justification');
  goog.asserts.assert(
      !goog.string.internal.isEmptyOrWhitespace(
          goog.string.Const.unwrap(justification)),
      'must provide non-empty justification');
  return goog.html.SafeUrl.createSafeUrlSecurityPrivateDoNotAccessOrElse(url);
};


/**
 * Performs an "unchecked conversion" to TrustedResourceUrl from a plain string
 * that is known to satisfy the TrustedResourceUrl type contract.
 *
 * IMPORTANT: Uses of this method must be carefully security-reviewed to ensure
 * that the value of `url` satisfies the TrustedResourceUrl type contract
 * in all possible program states.
 *
 * MOE:begin_intracomment_strip
 * See http://go/safehtml-unchecked for guidelines on using these functions.
 * MOE:end_intracomment_strip
 *
 * @param {!goog.string.Const} justification A constant string explaining why
 *     this use of this method is safe. May include a security review ticket
 *     number.
 * @param {string} url The string to wrap as a TrustedResourceUrl.
 * @return {!goog.html.TrustedResourceUrl} The value of `url`, wrapped in
 *     a TrustedResourceUrl object.
 */
goog.html.uncheckedconversions
    .trustedResourceUrlFromStringKnownToSatisfyTypeContract = function(
    justification, url) {
  // unwrap() called inside an assert so that justification can be optimized
  // away in production code.
  goog.asserts.assertString(
      goog.string.Const.unwrap(justification), 'must provide justification');
  goog.asserts.assert(
      !goog.string.internal.isEmptyOrWhitespace(
          goog.string.Const.unwrap(justification)),
      'must provide non-empty justification');
  return goog.html.TrustedResourceUrl
      .createTrustedResourceUrlSecurityPrivateDoNotAccessOrElse(url);
};

//third_party/javascript/closure/dom/safe.js
/**
 * @license
 * Copyright The Closure Library Authors.
 * SPDX-License-Identifier: Apache-2.0
 */

/**
 * @fileoverview Type-safe wrappers for unsafe DOM APIs.
 *
 * This file provides type-safe wrappers for DOM APIs that can result in
 * cross-site scripting (XSS) vulnerabilities, if the API is supplied with
 * untrusted (attacker-controlled) input.  Instead of plain strings, the type
 * safe wrappers consume values of types from the goog.html package whose
 * contract promises that values are safe to use in the corresponding context.
 *
 * Hence, a program that exclusively uses the wrappers in this file (i.e., whose
 * only reference to security-sensitive raw DOM APIs are in this file) is
 * guaranteed to be free of XSS due to incorrect use of such DOM APIs (modulo
 * correctness of code that produces values of the respective goog.html types,
 * and absent code that violates type safety).
 *
 * For example, assigning to an element's .innerHTML property a string that is
 * derived (even partially) from untrusted input typically results in an XSS
 * vulnerability. The type-safe wrapper goog.dom.safe.setInnerHtml consumes a
 * value of type goog.html.SafeHtml, whose contract states that using its values
 * in a HTML context will not result in XSS. Hence a program that is free of
 * direct assignments to any element's innerHTML property (with the exception of
 * the assignment to .innerHTML in this file) is guaranteed to be free of XSS
 * due to assignment of untrusted strings to the innerHTML property.
 */

goog.provide('goog.dom.safe');
goog.provide('goog.dom.safe.InsertAdjacentHtmlPosition');

goog.require('goog.asserts');
goog.require('goog.dom.asserts');
goog.require('goog.functions');
goog.require('goog.html.SafeHtml');
goog.require('goog.html.SafeScript');
goog.require('goog.html.SafeStyle');
goog.require('goog.html.SafeUrl');
goog.require('goog.html.TrustedResourceUrl');
goog.require('goog.html.uncheckedconversions');
goog.require('goog.string.Const');
goog.require('goog.string.internal');


/** @enum {string} */
goog.dom.safe.InsertAdjacentHtmlPosition = {
  AFTERBEGIN: 'afterbegin',
  AFTEREND: 'afterend',
  BEFOREBEGIN: 'beforebegin',
  BEFOREEND: 'beforeend'
};


/**
 * Inserts known-safe HTML into a Node, at the specified position.
 * @param {!Node} node The node on which to call insertAdjacentHTML.
 * @param {!goog.dom.safe.InsertAdjacentHtmlPosition} position Position where
 *     to insert the HTML.
 * @param {!goog.html.SafeHtml} html The known-safe HTML to insert.
 */
goog.dom.safe.insertAdjacentHtml = function(node, position, html) {
  node.insertAdjacentHTML(position, goog.html.SafeHtml.unwrapTrustedHTML(html));
};


/**
 * Tags not allowed in goog.dom.safe.setInnerHtml.
 * @private @const {!Object<string, boolean>}
 */
goog.dom.safe.SET_INNER_HTML_DISALLOWED_TAGS_ = {
  'MATH': true,
  'SCRIPT': true,
  'STYLE': true,
  'SVG': true,
  'TEMPLATE': true
};


/**
 * Whether assigning to innerHTML results in a non-spec-compliant clean-up. Used
 * to define goog.dom.safe.unsafeSetInnerHtmlDoNotUseOrElse.
 *
 * <p>As mentioned in https://stackoverflow.com/questions/28741528, re-rendering
 * an element in IE by setting innerHTML causes IE to recursively disconnect all
 * parent/children connections that were in the previous contents of the
 * element. Unfortunately, this can unexpectedly result in confusing cases where
 * a function is run (typically asynchronously) on element that has since
 * disconnected from the DOM but assumes the presence of its children. A simple
 * workaround is to remove all children first. Testing on IE11 via
 * https://jsperf.com/innerhtml-vs-removechild/239, removeChild seems to be
 * ~10x faster than innerHTML='' for a large number of children (perhaps due
 * to the latter's recursive behavior), implying that this workaround would
 * not hurt performance and might actually improve it.
 * @return {boolean}
 * @private
 */
goog.dom.safe.isInnerHtmlCleanupRecursive_ =
    goog.functions.cacheReturnValue(function() {
      // `document` missing in some test frameworks.
      if (goog.DEBUG && typeof document === 'undefined') {
        return false;
      }
      // Create 3 nested <div>s without using innerHTML.
      // We're not chaining the appendChilds in one call,  as this breaks
      // in a DocumentFragment.
      var div = document.createElement('div');
      var childDiv = document.createElement('div');
      childDiv.appendChild(document.createElement('div'));
      div.appendChild(childDiv);
      // `firstChild` is null in Google Js Test.
      if (goog.DEBUG && !div.firstChild) {
        return false;
      }
      var innerChild = div.firstChild.firstChild;
      div.innerHTML =
          goog.html.SafeHtml.unwrapTrustedHTML(goog.html.SafeHtml.EMPTY);
      return !innerChild.parentElement;
    });


/**
 * Assigns HTML to an element's innerHTML property. Helper to use only here and
 * in soy.js.
 * @param {?Element} elem The element whose innerHTML is to be assigned to.
 * @param {!goog.html.SafeHtml} html
 */
goog.dom.safe.unsafeSetInnerHtmlDoNotUseOrElse = function(elem, html) {
  // See comment above goog.dom.safe.isInnerHtmlCleanupRecursive_.
  if (goog.dom.safe.isInnerHtmlCleanupRecursive_()) {
    while (elem.lastChild) {
      elem.removeChild(elem.lastChild);
    }
  }
  elem.innerHTML = goog.html.SafeHtml.unwrapTrustedHTML(html);
};


/**
 * Assigns known-safe HTML to an element's innerHTML property.
 * @param {!Element} elem The element whose innerHTML is to be assigned to.
 * @param {!goog.html.SafeHtml} html The known-safe HTML to assign.
 * @throws {Error} If called with one of these tags: math, script, style, svg,
 *     template.
 */
goog.dom.safe.setInnerHtml = function(elem, html) {
  if (goog.asserts.ENABLE_ASSERTS) {
    var tagName = elem.tagName.toUpperCase();
    if (goog.dom.safe.SET_INNER_HTML_DISALLOWED_TAGS_[tagName]) {
      throw new Error(
          'goog.dom.safe.setInnerHtml cannot be used to set content of ' +
          elem.tagName + '.');
    }
  }

  goog.dom.safe.unsafeSetInnerHtmlDoNotUseOrElse(elem, html);
};


/**
 * Assigns known-safe HTML to an element's outerHTML property.
 * @param {!Element} elem The element whose outerHTML is to be assigned to.
 * @param {!goog.html.SafeHtml} html The known-safe HTML to assign.
 */
goog.dom.safe.setOuterHtml = function(elem, html) {
  elem.outerHTML = goog.html.SafeHtml.unwrapTrustedHTML(html);
};


/**
 * Safely assigns a URL a form element's action property.
 *
 * If url is of type goog.html.SafeUrl, its value is unwrapped and assigned to
 * form's action property.  If url is of type string however, it is first
 * sanitized using goog.html.SafeUrl.sanitize.
 *
 * Example usage:
 *   goog.dom.safe.setFormElementAction(formEl, url);
 * which is a safe alternative to
 *   formEl.action = url;
 * The latter can result in XSS vulnerabilities if url is a
 * user-/attacker-controlled value.
 *
 * @param {!Element} form The form element whose action property
 *     is to be assigned to.
 * @param {string|!goog.html.SafeUrl} url The URL to assign.
 * @see goog.html.SafeUrl#sanitize
 */
goog.dom.safe.setFormElementAction = function(form, url) {
  /** @type {!goog.html.SafeUrl} */
  var safeUrl;
  if (url instanceof goog.html.SafeUrl) {
    safeUrl = url;
  } else {
    safeUrl = goog.html.SafeUrl.sanitizeAssertUnchanged(url);
  }
  goog.dom.asserts.assertIsHTMLFormElement(form).action =
      goog.html.SafeUrl.unwrap(safeUrl);
};

/**
 * Safely assigns a URL to a button element's formaction property.
 *
 * If url is of type goog.html.SafeUrl, its value is unwrapped and assigned to
 * button's formaction property.  If url is of type string however, it is first
 * sanitized using goog.html.SafeUrl.sanitize.
 *
 * Example usage:
 *   goog.dom.safe.setButtonFormAction(buttonEl, url);
 * which is a safe alternative to
 *   buttonEl.action = url;
 * The latter can result in XSS vulnerabilities if url is a
 * user-/attacker-controlled value.
 *
 * @param {!Element} button The button element whose action property
 *     is to be assigned to.
 * @param {string|!goog.html.SafeUrl} url The URL to assign.
 * @see goog.html.SafeUrl#sanitize
 */
goog.dom.safe.setButtonFormAction = function(button, url) {
  /** @type {!goog.html.SafeUrl} */
  var safeUrl;
  if (url instanceof goog.html.SafeUrl) {
    safeUrl = url;
  } else {
    safeUrl = goog.html.SafeUrl.sanitizeAssertUnchanged(url);
  }
  goog.dom.asserts.assertIsHTMLButtonElement(button).formAction =
      goog.html.SafeUrl.unwrap(safeUrl);
};
/**
 * Safely assigns a URL to an input element's formaction property.
 *
 * If url is of type goog.html.SafeUrl, its value is unwrapped and assigned to
 * input's formaction property.  If url is of type string however, it is first
 * sanitized using goog.html.SafeUrl.sanitize.
 *
 * Example usage:
 *   goog.dom.safe.setInputFormAction(inputEl, url);
 * which is a safe alternative to
 *   inputEl.action = url;
 * The latter can result in XSS vulnerabilities if url is a
 * user-/attacker-controlled value.
 *
 * @param {!Element} input The input element whose action property
 *     is to be assigned to.
 * @param {string|!goog.html.SafeUrl} url The URL to assign.
 * @see goog.html.SafeUrl#sanitize
 */
goog.dom.safe.setInputFormAction = function(input, url) {
  /** @type {!goog.html.SafeUrl} */
  var safeUrl;
  if (url instanceof goog.html.SafeUrl) {
    safeUrl = url;
  } else {
    safeUrl = goog.html.SafeUrl.sanitizeAssertUnchanged(url);
  }
  goog.dom.asserts.assertIsHTMLInputElement(input).formAction =
      goog.html.SafeUrl.unwrap(safeUrl);
};

/**
 * Sets the given element's style property to the contents of the provided
 * SafeStyle object.
 * @param {!Element} elem
 * @param {!goog.html.SafeStyle} style
 */
goog.dom.safe.setStyle = function(elem, style) {
  elem.style.cssText = goog.html.SafeStyle.unwrap(style);
};


/**
 * Writes known-safe HTML to a document.
 * @param {!Document} doc The document to be written to.
 * @param {!goog.html.SafeHtml} html The known-safe HTML to assign.
 */
goog.dom.safe.documentWrite = function(doc, html) {
  doc.write(goog.html.SafeHtml.unwrapTrustedHTML(html));
};


/**
 * Safely assigns a URL to an anchor element's href property.
 *
 * If url is of type goog.html.SafeUrl, its value is unwrapped and assigned to
 * anchor's href property.  If url is of type string however, it is first
 * sanitized using goog.html.SafeUrl.sanitize.
 *
 * Example usage:
 *   goog.dom.safe.setAnchorHref(anchorEl, url);
 * which is a safe alternative to
 *   anchorEl.href = url;
 * The latter can result in XSS vulnerabilities if url is a
 * user-/attacker-controlled value.
 *
 * @param {!HTMLAnchorElement} anchor The anchor element whose href property
 *     is to be assigned to.
 * @param {string|!goog.html.SafeUrl} url The URL to assign.
 * @see goog.html.SafeUrl#sanitize
 */
goog.dom.safe.setAnchorHref = function(anchor, url) {
  goog.dom.asserts.assertIsHTMLAnchorElement(anchor);
  /** @type {!goog.html.SafeUrl} */
  var safeUrl;
  if (url instanceof goog.html.SafeUrl) {
    safeUrl = url;
  } else {
    safeUrl = goog.html.SafeUrl.sanitizeAssertUnchanged(url);
  }
  anchor.href = goog.html.SafeUrl.unwrap(safeUrl);
};


/**
 * Safely assigns a URL to an image element's src property.
 *
 * If url is of type goog.html.SafeUrl, its value is unwrapped and assigned to
 * image's src property.  If url is of type string however, it is first
 * sanitized using goog.html.SafeUrl.sanitize.
 *
 * @param {!HTMLImageElement} imageElement The image element whose src property
 *     is to be assigned to.
 * @param {string|!goog.html.SafeUrl} url The URL to assign.
 * @see goog.html.SafeUrl#sanitize
 */
goog.dom.safe.setImageSrc = function(imageElement, url) {
  goog.dom.asserts.assertIsHTMLImageElement(imageElement);
  /** @type {!goog.html.SafeUrl} */
  var safeUrl;
  if (url instanceof goog.html.SafeUrl) {
    safeUrl = url;
  } else {
    var allowDataUrl = /^data:image\//i.test(url);
    safeUrl = goog.html.SafeUrl.sanitizeAssertUnchanged(url, allowDataUrl);
  }
  imageElement.src = goog.html.SafeUrl.unwrap(safeUrl);
};

/**
 * Safely assigns a URL to a audio element's src property.
 *
 * If url is of type goog.html.SafeUrl, its value is unwrapped and assigned to
 * audio's src property.  If url is of type string however, it is first
 * sanitized using goog.html.SafeUrl.sanitize.
 *
 * @param {!HTMLAudioElement} audioElement The audio element whose src property
 *     is to be assigned to.
 * @param {string|!goog.html.SafeUrl} url The URL to assign.
 * @see goog.html.SafeUrl#sanitize
 */
goog.dom.safe.setAudioSrc = function(audioElement, url) {
  goog.dom.asserts.assertIsHTMLAudioElement(audioElement);
  /** @type {!goog.html.SafeUrl} */
  var safeUrl;
  if (url instanceof goog.html.SafeUrl) {
    safeUrl = url;
  } else {
    var allowDataUrl = /^data:audio\//i.test(url);
    safeUrl = goog.html.SafeUrl.sanitizeAssertUnchanged(url, allowDataUrl);
  }
  audioElement.src = goog.html.SafeUrl.unwrap(safeUrl);
};

/**
 * Safely assigns a URL to a video element's src property.
 *
 * If url is of type goog.html.SafeUrl, its value is unwrapped and assigned to
 * video's src property.  If url is of type string however, it is first
 * sanitized using goog.html.SafeUrl.sanitize.
 *
 * @param {!HTMLVideoElement} videoElement The video element whose src property
 *     is to be assigned to.
 * @param {string|!goog.html.SafeUrl} url The URL to assign.
 * @see goog.html.SafeUrl#sanitize
 */
goog.dom.safe.setVideoSrc = function(videoElement, url) {
  goog.dom.asserts.assertIsHTMLVideoElement(videoElement);
  /** @type {!goog.html.SafeUrl} */
  var safeUrl;
  if (url instanceof goog.html.SafeUrl) {
    safeUrl = url;
  } else {
    var allowDataUrl = /^data:video\//i.test(url);
    safeUrl = goog.html.SafeUrl.sanitizeAssertUnchanged(url, allowDataUrl);
  }
  videoElement.src = goog.html.SafeUrl.unwrap(safeUrl);
};

/**
 * Safely assigns a URL to an embed element's src property.
 *
 * Example usage:
 *   goog.dom.safe.setEmbedSrc(embedEl, url);
 * which is a safe alternative to
 *   embedEl.src = url;
 * The latter can result in loading untrusted code unless it is ensured that
 * the URL refers to a trustworthy resource.
 *
 * @param {!HTMLEmbedElement} embed The embed element whose src property
 *     is to be assigned to.
 * @param {!goog.html.TrustedResourceUrl} url The URL to assign.
 */
goog.dom.safe.setEmbedSrc = function(embed, url) {
  goog.dom.asserts.assertIsHTMLEmbedElement(embed);
  embed.src = goog.html.TrustedResourceUrl.unwrapTrustedScriptURL(url);
};


/**
 * Safely assigns a URL to a frame element's src property.
 *
 * Example usage:
 *   goog.dom.safe.setFrameSrc(frameEl, url);
 * which is a safe alternative to
 *   frameEl.src = url;
 * The latter can result in loading untrusted code unless it is ensured that
 * the URL refers to a trustworthy resource.
 *
 * @param {!HTMLFrameElement} frame The frame element whose src property
 *     is to be assigned to.
 * @param {!goog.html.TrustedResourceUrl} url The URL to assign.
 */
goog.dom.safe.setFrameSrc = function(frame, url) {
  goog.dom.asserts.assertIsHTMLFrameElement(frame);
  frame.src = goog.html.TrustedResourceUrl.unwrap(url);
};


/**
 * Safely assigns a URL to an iframe element's src property.
 *
 * Example usage:
 *   goog.dom.safe.setIframeSrc(iframeEl, url);
 * which is a safe alternative to
 *   iframeEl.src = url;
 * The latter can result in loading untrusted code unless it is ensured that
 * the URL refers to a trustworthy resource.
 *
 * @param {!HTMLIFrameElement} iframe The iframe element whose src property
 *     is to be assigned to.
 * @param {!goog.html.TrustedResourceUrl} url The URL to assign.
 */
goog.dom.safe.setIframeSrc = function(iframe, url) {
  goog.dom.asserts.assertIsHTMLIFrameElement(iframe);
  iframe.src = goog.html.TrustedResourceUrl.unwrap(url);
};


/**
 * Safely assigns HTML to an iframe element's srcdoc property.
 *
 * Example usage:
 *   goog.dom.safe.setIframeSrcdoc(iframeEl, safeHtml);
 * which is a safe alternative to
 *   iframeEl.srcdoc = html;
 * The latter can result in loading untrusted code.
 *
 * @param {!HTMLIFrameElement} iframe The iframe element whose srcdoc property
 *     is to be assigned to.
 * @param {!goog.html.SafeHtml} html The HTML to assign.
 */
goog.dom.safe.setIframeSrcdoc = function(iframe, html) {
  goog.dom.asserts.assertIsHTMLIFrameElement(iframe);
  iframe.srcdoc = goog.html.SafeHtml.unwrapTrustedHTML(html);
};


/**
 * Safely sets a link element's href and rel properties. Whether or not
 * the URL assigned to href has to be a goog.html.TrustedResourceUrl
 * depends on the value of the rel property. If rel contains "stylesheet"
 * then a TrustedResourceUrl is required.
 *
 * Example usage:
 *   goog.dom.safe.setLinkHrefAndRel(linkEl, url, 'stylesheet');
 * which is a safe alternative to
 *   linkEl.rel = 'stylesheet';
 *   linkEl.href = url;
 * The latter can result in loading untrusted code unless it is ensured that
 * the URL refers to a trustworthy resource.
 *
 * @param {!HTMLLinkElement} link The link element whose href property
 *     is to be assigned to.
 * @param {string|!goog.html.SafeUrl|!goog.html.TrustedResourceUrl} url The URL
 *     to assign to the href property. Must be a TrustedResourceUrl if the
 *     value assigned to rel contains "stylesheet". A string value is
 *     sanitized with goog.html.SafeUrl.sanitize.
 * @param {string} rel The value to assign to the rel property.
 * @throws {Error} if rel contains "stylesheet" and url is not a
 *     TrustedResourceUrl
 * @see goog.html.SafeUrl#sanitize
 */
goog.dom.safe.setLinkHrefAndRel = function(link, url, rel) {
  goog.dom.asserts.assertIsHTMLLinkElement(link);
  link.rel = rel;
  if (goog.string.internal.caseInsensitiveContains(rel, 'stylesheet')) {
    goog.asserts.assert(
        url instanceof goog.html.TrustedResourceUrl,
        'URL must be TrustedResourceUrl because "rel" contains "stylesheet"');
    link.href = goog.html.TrustedResourceUrl.unwrap(url);
  } else if (url instanceof goog.html.TrustedResourceUrl) {
    link.href = goog.html.TrustedResourceUrl.unwrap(url);
  } else if (url instanceof goog.html.SafeUrl) {
    link.href = goog.html.SafeUrl.unwrap(url);
  } else {  // string
    // SafeUrl.sanitize must return legitimate SafeUrl when passed a string.
    link.href = goog.html.SafeUrl.unwrap(
        goog.html.SafeUrl.sanitizeAssertUnchanged(url));
  }
};


/**
 * Safely assigns a URL to an object element's data property.
 *
 * Example usage:
 *   goog.dom.safe.setObjectData(objectEl, url);
 * which is a safe alternative to
 *   objectEl.data = url;
 * The latter can result in loading untrusted code unless setit is ensured that
 * the URL refers to a trustworthy resource.
 *
 * @param {!HTMLObjectElement} object The object element whose data property
 *     is to be assigned to.
 * @param {!goog.html.TrustedResourceUrl} url The URL to assign.
 */
goog.dom.safe.setObjectData = function(object, url) {
  goog.dom.asserts.assertIsHTMLObjectElement(object);
  object.data = goog.html.TrustedResourceUrl.unwrapTrustedScriptURL(url);
};


/**
 * Safely assigns a URL to a script element's src property.
 *
 * Example usage:
 *   goog.dom.safe.setScriptSrc(scriptEl, url);
 * which is a safe alternative to
 *   scriptEl.src = url;
 * The latter can result in loading untrusted code unless it is ensured that
 * the URL refers to a trustworthy resource.
 *
 * @param {!HTMLScriptElement} script The script element whose src property
 *     is to be assigned to.
 * @param {!goog.html.TrustedResourceUrl} url The URL to assign.
 */
goog.dom.safe.setScriptSrc = function(script, url) {
  goog.dom.asserts.assertIsHTMLScriptElement(script);
  script.src = goog.html.TrustedResourceUrl.unwrapTrustedScriptURL(url);

  // If CSP nonces are used, propagate them to dynamically created scripts.
  // This is necessary to allow nonce-based CSPs without 'strict-dynamic'.
  var nonce = goog.getScriptNonce();
  if (nonce) {
    script.setAttribute('nonce', nonce);
  }
};


/**
 * Safely assigns a value to a script element's content.
 *
 * Example usage:
 *   goog.dom.safe.setScriptContent(scriptEl, content);
 * which is a safe alternative to
 *   scriptEl.text = content;
 * The latter can result in executing untrusted code unless it is ensured that
 * the code is loaded from a trustworthy resource.
 *
 * @param {!HTMLScriptElement} script The script element whose content is being
 *     set.
 * @param {!goog.html.SafeScript} content The content to assign.
 */
goog.dom.safe.setScriptContent = function(script, content) {
  goog.dom.asserts.assertIsHTMLScriptElement(script);
  script.text = goog.html.SafeScript.unwrapTrustedScript(content);

  // If CSP nonces are used, propagate them to dynamically created scripts.
  // This is necessary to allow nonce-based CSPs without 'strict-dynamic'.
  var nonce = goog.getScriptNonce();
  if (nonce) {
    script.setAttribute('nonce', nonce);
  }
};


/**
 * Safely assigns a URL to a Location object's href property.
 *
 * If url is of type goog.html.SafeUrl, its value is unwrapped and assigned to
 * loc's href property.  If url is of type string however, it is first sanitized
 * using goog.html.SafeUrl.sanitize.
 *
 * Example usage:
 *   goog.dom.safe.setLocationHref(document.location, redirectUrl);
 * which is a safe alternative to
 *   document.location.href = redirectUrl;
 * The latter can result in XSS vulnerabilities if redirectUrl is a
 * user-/attacker-controlled value.
 *
 * @param {!Location} loc The Location object whose href property is to be
 *     assigned to.
 * @param {string|!goog.html.SafeUrl} url The URL to assign.
 * @see goog.html.SafeUrl#sanitize
 */
goog.dom.safe.setLocationHref = function(loc, url) {
  goog.dom.asserts.assertIsLocation(loc);
  /** @type {!goog.html.SafeUrl} */
  var safeUrl;
  if (url instanceof goog.html.SafeUrl) {
    safeUrl = url;
  } else {
    safeUrl = goog.html.SafeUrl.sanitizeAssertUnchanged(url);
  }
  loc.href = goog.html.SafeUrl.unwrap(safeUrl);
};

/**
 * Safely assigns the URL of a Location object.
 *
 * If url is of type goog.html.SafeUrl, its value is unwrapped and
 * passed to Location#assign. If url is of type string however, it is
 * first sanitized using goog.html.SafeUrl.sanitize.
 *
 * Example usage:
 *   goog.dom.safe.assignLocation(document.location, newUrl);
 * which is a safe alternative to
 *   document.location.assign(newUrl);
 * The latter can result in XSS vulnerabilities if newUrl is a
 * user-/attacker-controlled value.
 *
 * This has the same behaviour as setLocationHref, however some test
 * mock Location.assign instead of a property assignment.
 *
 * @param {!Location} loc The Location object which is to be assigned.
 * @param {string|!goog.html.SafeUrl} url The URL to assign.
 * @see goog.html.SafeUrl#sanitize
 */
goog.dom.safe.assignLocation = function(loc, url) {
  goog.dom.asserts.assertIsLocation(loc);
  /** @type {!goog.html.SafeUrl} */
  var safeUrl;
  if (url instanceof goog.html.SafeUrl) {
    safeUrl = url;
  } else {
    safeUrl = goog.html.SafeUrl.sanitizeAssertUnchanged(url);
  }
  loc.assign(goog.html.SafeUrl.unwrap(safeUrl));
};


/**
 * Safely replaces the URL of a Location object.
 *
 * If url is of type goog.html.SafeUrl, its value is unwrapped and
 * passed to Location#replace. If url is of type string however, it is
 * first sanitized using goog.html.SafeUrl.sanitize.
 *
 * Example usage:
 *   goog.dom.safe.replaceLocation(document.location, newUrl);
 * which is a safe alternative to
 *   document.location.replace(newUrl);
 * The latter can result in XSS vulnerabilities if newUrl is a
 * user-/attacker-controlled value.
 *
 * @param {!Location} loc The Location object which is to be replaced.
 * @param {string|!goog.html.SafeUrl} url The URL to assign.
 * @see goog.html.SafeUrl#sanitize
 */
goog.dom.safe.replaceLocation = function(loc, url) {
  /** @type {!goog.html.SafeUrl} */
  var safeUrl;
  if (url instanceof goog.html.SafeUrl) {
    safeUrl = url;
  } else {
    safeUrl = goog.html.SafeUrl.sanitizeAssertUnchanged(url);
  }
  loc.replace(goog.html.SafeUrl.unwrap(safeUrl));
};


/**
 * Safely opens a URL in a new window (via window.open).
 *
 * If url is of type goog.html.SafeUrl, its value is unwrapped and passed in to
 * window.open.  If url is of type string however, it is first sanitized
 * using goog.html.SafeUrl.sanitize.
 *
 * Note that this function does not prevent leakages via the referer that is
 * sent by window.open. It is advised to only use this to open 1st party URLs.
 *
 * Example usage:
 *   goog.dom.safe.openInWindow(url);
 * which is a safe alternative to
 *   window.open(url);
 * The latter can result in XSS vulnerabilities if redirectUrl is a
 * user-/attacker-controlled value.
 *
 * @param {string|!goog.html.SafeUrl} url The URL to open.
 * @param {Window=} opt_openerWin Window of which to call the .open() method.
 *     Defaults to the global window.
 * @param {!goog.string.Const=} opt_name Name of the window to open in. Can be
 *     _top, etc as allowed by window.open().
 * @param {string=} opt_specs Comma-separated list of specifications, same as
 *     in window.open().
 * @param {boolean=} opt_replace Whether to replace the current entry in browser
 *     history, same as in window.open().
 * @return {Window} Window the url was opened in.
 */
goog.dom.safe.openInWindow = function(
    url, opt_openerWin, opt_name, opt_specs, opt_replace) {
  /** @type {!goog.html.SafeUrl} */
  var safeUrl;
  if (url instanceof goog.html.SafeUrl) {
    safeUrl = url;
  } else {
    safeUrl = goog.html.SafeUrl.sanitizeAssertUnchanged(url);
  }
  var win = opt_openerWin || goog.global;
  return win.open(
      goog.html.SafeUrl.unwrap(safeUrl),
      // If opt_name is undefined, simply passing that in to open() causes IE to
      // reuse the current window instead of opening a new one. Thus we pass ''
      // in instead, which according to spec opens a new window. See
      // https://html.spec.whatwg.org/multipage/browsers.html#dom-open .
      opt_name ? goog.string.Const.unwrap(opt_name) : '', opt_specs,
      opt_replace);
};


/**
 * Parses the HTML as 'text/html'.
 * @param {!DOMParser} parser
 * @param {!goog.html.SafeHtml} html The HTML to be parsed.
 * @return {?Document}
 */
goog.dom.safe.parseFromStringHtml = function(parser, html) {
  return goog.dom.safe.parseFromString(parser, html, 'text/html');
};


/**
 * Parses the string.
 * @param {!DOMParser} parser
 * @param {!goog.html.SafeHtml} content Note: We don't have a special type for
 *     XML od SVG supported by this function so we use SafeHtml.
 * @param {string} type
 * @return {?Document}
 */
goog.dom.safe.parseFromString = function(parser, content, type) {
  return parser.parseFromString(
      goog.html.SafeHtml.unwrapTrustedHTML(content), type);
};


/**
 * Safely creates an HTMLImageElement from a Blob.
 *
 * Example usage:
 *     goog.dom.safe.createImageFromBlob(blob);
 * which is a safe alternative to
 *     image.src = createObjectUrl(blob)
 * The latter can result in executing malicious same-origin scripts from a bad
 * Blob.
 * @param {!Blob} blob The blob to create the image from.
 * @return {!HTMLImageElement} The image element created from the blob.
 * @throws {!Error} If called with a Blob with a MIME type other than image/.*.
 */
goog.dom.safe.createImageFromBlob = function(blob) {
  // Any image/* MIME type is accepted as safe.
  if (!/^image\/.*/g.test(blob.type)) {
    throw new Error(
        'goog.dom.safe.createImageFromBlob only accepts MIME type image/.*.');
  }
  var objectUrl = goog.global.URL.createObjectURL(blob);
  var image = new goog.global.Image();
  image.onload = function() {
    goog.global.URL.revokeObjectURL(objectUrl);
  };
  goog.dom.safe.setImageSrc(
      image,
      goog.html.uncheckedconversions
          .safeUrlFromStringKnownToSatisfyTypeContract(
              goog.string.Const.from('Image blob URL.'), objectUrl));
  return image;
};

//third_party/javascript/closure/string/string.js
/**
 * @license
 * Copyright The Closure Library Authors.
 * SPDX-License-Identifier: Apache-2.0
 */

/**
 * @fileoverview Utilities for string manipulation.
 */


/**
 * Namespace for string utilities
 */
goog.provide('goog.string');
goog.provide('goog.string.Unicode');

goog.require('goog.dom.safe');
goog.require('goog.html.uncheckedconversions');
goog.require('goog.string.Const');
goog.require('goog.string.internal');


/**
 * @define {boolean} Enables HTML escaping of lowercase letter "e" which helps
 * with detection of double-escaping as this letter is frequently used.
 */
goog.string.DETECT_DOUBLE_ESCAPING =
    goog.define('goog.string.DETECT_DOUBLE_ESCAPING', false);


/**
 * @define {boolean} Whether to force non-dom html unescaping.
 */
goog.string.FORCE_NON_DOM_HTML_UNESCAPING =
    goog.define('goog.string.FORCE_NON_DOM_HTML_UNESCAPING', false);


/**
 * Common Unicode string characters.
 * @enum {string}
 */
goog.string.Unicode = {
  NBSP: '\xa0'
};


/**
 * Fast prefix-checker.
 * @param {string} str The string to check.
 * @param {string} prefix A string to look for at the start of `str`.
 * @return {boolean} True if `str` begins with `prefix`.
 */
goog.string.startsWith = goog.string.internal.startsWith;


/**
 * Fast suffix-checker.
 * @param {string} str The string to check.
 * @param {string} suffix A string to look for at the end of `str`.
 * @return {boolean} True if `str` ends with `suffix`.
 */
goog.string.endsWith = goog.string.internal.endsWith;


/**
 * Case-insensitive prefix-checker.
 * @param {string} str The string to check.
 * @param {string} prefix  A string to look for at the end of `str`.
 * @return {boolean} True if `str` begins with `prefix` (ignoring
 *     case).
 */
goog.string.caseInsensitiveStartsWith =
    goog.string.internal.caseInsensitiveStartsWith;


/**
 * Case-insensitive suffix-checker.
 * @param {string} str The string to check.
 * @param {string} suffix A string to look for at the end of `str`.
 * @return {boolean} True if `str` ends with `suffix` (ignoring
 *     case).
 */
goog.string.caseInsensitiveEndsWith =
    goog.string.internal.caseInsensitiveEndsWith;


/**
 * Case-insensitive equality checker.
 * @param {string} str1 First string to check.
 * @param {string} str2 Second string to check.
 * @return {boolean} True if `str1` and `str2` are the same string,
 *     ignoring case.
 */
goog.string.caseInsensitiveEquals = goog.string.internal.caseInsensitiveEquals;


/**
 * Does simple python-style string substitution.
 * subs("foo%s hot%s", "bar", "dog") becomes "foobar hotdog".
 * @param {string} str The string containing the pattern.
 * @param {...*} var_args The items to substitute into the pattern.
 * @return {string} A copy of `str` in which each occurrence of
 *     {@code %s} has been replaced an argument from `var_args`.
 */
goog.string.subs = function(str, var_args) {
  var splitParts = str.split('%s');
  var returnString = '';

  var subsArguments = Array.prototype.slice.call(arguments, 1);
  while (subsArguments.length &&
         // Replace up to the last split part. We are inserting in the
         // positions between split parts.
         splitParts.length > 1) {
    returnString += splitParts.shift() + subsArguments.shift();
  }

  return returnString + splitParts.join('%s');  // Join unused '%s'
};


/**
 * Converts multiple whitespace chars (spaces, non-breaking-spaces, new lines
 * and tabs) to a single space, and strips leading and trailing whitespace.
 * @param {string} str Input string.
 * @return {string} A copy of `str` with collapsed whitespace.
 */
goog.string.collapseWhitespace = function(str) {
  // Since IE doesn't include non-breaking-space (0xa0) in their \s character
  // class (as required by section 7.2 of the ECMAScript spec), we explicitly
  // include it in the regexp to enforce consistent cross-browser behavior.
  return str.replace(/[\s\xa0]+/g, ' ').replace(/^\s+|\s+$/g, '');
};


/**
 * Checks if a string is empty or contains only whitespaces.
 * @param {string} str The string to check.
 * @return {boolean} Whether `str` is empty or whitespace only.
 */
goog.string.isEmptyOrWhitespace = goog.string.internal.isEmptyOrWhitespace;


/**
 * Checks if a string is empty.
 * @param {string} str The string to check.
 * @return {boolean} Whether `str` is empty.
 */
goog.string.isEmptyString = function(str) {
  return str.length == 0;
};


/**
 * Checks if a string is empty or contains only whitespaces.
 *
 * @param {string} str The string to check.
 * @return {boolean} Whether `str` is empty or whitespace only.
 * @deprecated Use goog.string.isEmptyOrWhitespace instead.
 */
goog.string.isEmpty = goog.string.isEmptyOrWhitespace;


/**
 * Checks if a string is null, undefined, empty or contains only whitespaces.
 * @param {*} str The string to check.
 * @return {boolean} Whether `str` is null, undefined, empty, or
 *     whitespace only.
 * @deprecated Use goog.string.isEmptyOrWhitespace(goog.string.makeSafe(str))
 *     instead.
 */
goog.string.isEmptyOrWhitespaceSafe = function(str) {
  return goog.string.isEmptyOrWhitespace(goog.string.makeSafe(str));
};


/**
 * Checks if a string is null, undefined, empty or contains only whitespaces.
 *
 * @param {*} str The string to check.
 * @return {boolean} Whether `str` is null, undefined, empty, or
 *     whitespace only.
 * @deprecated Use goog.string.isEmptyOrWhitespace instead.
 */
goog.string.isEmptySafe = goog.string.isEmptyOrWhitespaceSafe;


/**
 * Checks if a string is all breaking whitespace.
 * @param {string} str The string to check.
 * @return {boolean} Whether the string is all breaking whitespace.
 */
goog.string.isBreakingWhitespace = function(str) {
  return !/[^\t\n\r ]/.test(str);
};


/**
 * Checks if a string contains all letters.
 * @param {string} str string to check.
 * @return {boolean} True if `str` consists entirely of letters.
 */
goog.string.isAlpha = function(str) {
  return !/[^a-zA-Z]/.test(str);
};


/**
 * Checks if a string contains only numbers.
 * @param {*} str string to check. If not a string, it will be
 *     casted to one.
 * @return {boolean} True if `str` is numeric.
 */
goog.string.isNumeric = function(str) {
  return !/[^0-9]/.test(str);
};


/**
 * Checks if a string contains only numbers or letters.
 * @param {string} str string to check.
 * @return {boolean} True if `str` is alphanumeric.
 */
goog.string.isAlphaNumeric = function(str) {
  return !/[^a-zA-Z0-9]/.test(str);
};


/**
 * Checks if a character is a space character.
 * @param {string} ch Character to check.
 * @return {boolean} True if `ch` is a space.
 */
goog.string.isSpace = function(ch) {
  return ch == ' ';
};


/**
 * Checks if a character is a valid unicode character.
 * @param {string} ch Character to check.
 * @return {boolean} True if `ch` is a valid unicode character.
 */
goog.string.isUnicodeChar = function(ch) {
  return ch.length == 1 && ch >= ' ' && ch <= '~' ||
      ch >= '\u0080' && ch <= '\uFFFD';
};


/**
 * Takes a string and replaces newlines with a space. Multiple lines are
 * replaced with a single space.
 * @param {string} str The string from which to strip newlines.
 * @return {string} A copy of `str` stripped of newlines.
 */
goog.string.stripNewlines = function(str) {
  return str.replace(/(\r\n|\r|\n)+/g, ' ');
};


/**
 * Replaces Windows and Mac new lines with unix style: \r or \r\n with \n.
 * @param {string} str The string to in which to canonicalize newlines.
 * @return {string} `str` A copy of {@code} with canonicalized newlines.
 */
goog.string.canonicalizeNewlines = function(str) {
  return str.replace(/(\r\n|\r|\n)/g, '\n');
};


/**
 * Normalizes whitespace in a string, replacing all whitespace chars with
 * a space.
 * @param {string} str The string in which to normalize whitespace.
 * @return {string} A copy of `str` with all whitespace normalized.
 */
goog.string.normalizeWhitespace = function(str) {
  return str.replace(/\xa0|\s/g, ' ');
};


/**
 * Normalizes spaces in a string, replacing all consecutive spaces and tabs
 * with a single space. Replaces non-breaking space with a space.
 * @param {string} str The string in which to normalize spaces.
 * @return {string} A copy of `str` with all consecutive spaces and tabs
 *    replaced with a single space.
 */
goog.string.normalizeSpaces = function(str) {
  return str.replace(/\xa0|[ \t]+/g, ' ');
};


/**
 * Removes the breaking spaces from the left and right of the string and
 * collapses the sequences of breaking spaces in the middle into single spaces.
 * The original and the result strings render the same way in HTML.
 * @param {string} str A string in which to collapse spaces.
 * @return {string} Copy of the string with normalized breaking spaces.
 */
goog.string.collapseBreakingSpaces = function(str) {
  return str.replace(/[\t\r\n ]+/g, ' ')
      .replace(/^[\t\r\n ]+|[\t\r\n ]+$/g, '');
};


/**
 * Trims white spaces to the left and right of a string.
 * @param {string} str The string to trim.
 * @return {string} A trimmed copy of `str`.
 */
goog.string.trim = goog.string.internal.trim;


/**
 * Trims whitespaces at the left end of a string.
 * @param {string} str The string to left trim.
 * @return {string} A trimmed copy of `str`.
 */
goog.string.trimLeft = function(str) {
  // Since IE doesn't include non-breaking-space (0xa0) in their \s character
  // class (as required by section 7.2 of the ECMAScript spec), we explicitly
  // include it in the regexp to enforce consistent cross-browser behavior.
  return str.replace(/^[\s\xa0]+/, '');
};


/**
 * Trims whitespaces at the right end of a string.
 * @param {string} str The string to right trim.
 * @return {string} A trimmed copy of `str`.
 */
goog.string.trimRight = function(str) {
  // Since IE doesn't include non-breaking-space (0xa0) in their \s character
  // class (as required by section 7.2 of the ECMAScript spec), we explicitly
  // include it in the regexp to enforce consistent cross-browser behavior.
  return str.replace(/[\s\xa0]+$/, '');
};


/**
 * A string comparator that ignores case.
 * -1 = str1 less than str2
 *  0 = str1 equals str2
 *  1 = str1 greater than str2
 *
 * @param {string} str1 The string to compare.
 * @param {string} str2 The string to compare `str1` to.
 * @return {number} The comparator result, as described above.
 */
goog.string.caseInsensitiveCompare =
    goog.string.internal.caseInsensitiveCompare;


/**
 * Compares two strings interpreting their numeric substrings as numbers.
 *
 * @param {string} str1 First string.
 * @param {string} str2 Second string.
 * @param {!RegExp} tokenizerRegExp Splits a string into substrings of
 *     non-negative integers, non-numeric characters and optionally fractional
 *     numbers starting with a decimal point.
 * @return {number} Negative if str1 < str2, 0 is str1 == str2, positive if
 *     str1 > str2.
 * @private
 */
goog.string.numberAwareCompare_ = function(str1, str2, tokenizerRegExp) {
  if (str1 == str2) {
    return 0;
  }
  if (!str1) {
    return -1;
  }
  if (!str2) {
    return 1;
  }

  // Using match to split the entire string ahead of time turns out to be faster
  // for most inputs than using RegExp.exec or iterating over each character.
  var tokens1 = str1.toLowerCase().match(tokenizerRegExp);
  var tokens2 = str2.toLowerCase().match(tokenizerRegExp);

  var count = Math.min(tokens1.length, tokens2.length);

  for (var i = 0; i < count; i++) {
    var a = tokens1[i];
    var b = tokens2[i];

    // Compare pairs of tokens, returning if one token sorts before the other.
    if (a != b) {
      // Only if both tokens are integers is a special comparison required.
      // Decimal numbers are sorted as strings (e.g., '.09' < '.1').
      var num1 = parseInt(a, 10);
      if (!isNaN(num1)) {
        var num2 = parseInt(b, 10);
        if (!isNaN(num2) && num1 - num2) {
          return num1 - num2;
        }
      }
      return a < b ? -1 : 1;
    }
  }

  // If one string is a substring of the other, the shorter string sorts first.
  if (tokens1.length != tokens2.length) {
    return tokens1.length - tokens2.length;
  }

  // The two strings must be equivalent except for case (perfect equality is
  // tested at the head of the function.) Revert to default ASCII string
  // comparison to stabilize the sort.
  return str1 < str2 ? -1 : 1;
};


/**
 * String comparison function that handles non-negative integer numbers in a
 * way humans might expect. Using this function, the string 'File 2.jpg' sorts
 * before 'File 10.jpg', and 'Version 1.9' before 'Version 1.10'. The comparison
 * is mostly case-insensitive, though strings that are identical except for case
 * are sorted with the upper-case strings before lower-case.
 *
 * This comparison function is up to 50x slower than either the default or the
 * case-insensitive compare. It should not be used in time-critical code, but
 * should be fast enough to sort several hundred short strings (like filenames)
 * with a reasonable delay.
 *
 * @param {string} str1 The string to compare in a numerically sensitive way.
 * @param {string} str2 The string to compare `str1` to.
 * @return {number} less than 0 if str1 < str2, 0 if str1 == str2, greater than
 *     0 if str1 > str2.
 */
goog.string.intAwareCompare = function(str1, str2) {
  return goog.string.numberAwareCompare_(str1, str2, /\d+|\D+/g);
};


/**
 * String comparison function that handles non-negative integer and fractional
 * numbers in a way humans might expect. Using this function, the string
 * 'File 2.jpg' sorts before 'File 10.jpg', and '3.14' before '3.2'. Equivalent
 * to {@link goog.string.intAwareCompare} apart from the way how it interprets
 * dots.
 *
 * @param {string} str1 The string to compare in a numerically sensitive way.
 * @param {string} str2 The string to compare `str1` to.
 * @return {number} less than 0 if str1 < str2, 0 if str1 == str2, greater than
 *     0 if str1 > str2.
 */
goog.string.floatAwareCompare = function(str1, str2) {
  return goog.string.numberAwareCompare_(str1, str2, /\d+|\.\d+|\D+/g);
};


/**
 * Alias for {@link goog.string.floatAwareCompare}.
 *
 * @param {string} str1
 * @param {string} str2
 * @return {number}
 */
goog.string.numerateCompare = goog.string.floatAwareCompare;


/**
 * URL-encodes a string
 * @param {*} str The string to url-encode.
 * @return {string} An encoded copy of `str` that is safe for urls.
 *     Note that '#', ':', and other characters used to delimit portions
 *     of URLs *will* be encoded.
 */
goog.string.urlEncode = function(str) {
  return encodeURIComponent(String(str));
};


/**
 * URL-decodes the string. We need to specially handle '+'s because
 * the javascript library doesn't convert them to spaces.
 * @param {string} str The string to url decode.
 * @return {string} The decoded `str`.
 */
goog.string.urlDecode = function(str) {
  return decodeURIComponent(str.replace(/\+/g, ' '));
};


/**
 * Converts \n to <br>s or <br />s.
 * @param {string} str The string in which to convert newlines.
 * @param {boolean=} opt_xml Whether to use XML compatible tags.
 * @return {string} A copy of `str` with converted newlines.
 */
goog.string.newLineToBr = goog.string.internal.newLineToBr;


/**
 * Escapes double quote '"' and single quote '\'' characters in addition to
 * '&', '<', and '>' so that a string can be included in an HTML tag attribute
 * value within double or single quotes.
 *
 * It should be noted that > doesn't need to be escaped for the HTML or XML to
 * be valid, but it has been decided to escape it for consistency with other
 * implementations.
 *
 * With goog.string.DETECT_DOUBLE_ESCAPING, this function escapes also the
 * lowercase letter "e".
 *
 * NOTE(pupius):
 * HtmlEscape is often called during the generation of large blocks of HTML.
 * Using statics for the regular expressions and strings is an optimization
 * that can more than half the amount of time IE spends in this function for
 * large apps, since strings and regexes both contribute to GC allocations.
 *
 * Testing for the presence of a character before escaping increases the number
 * of function calls, but actually provides a speed increase for the average
 * case -- since the average case often doesn't require the escaping of all 4
 * characters and indexOf() is much cheaper than replace().
 * The worst case does suffer slightly from the additional calls, therefore the
 * opt_isLikelyToContainHtmlChars option has been included for situations
 * where all 4 HTML entities are very likely to be present and need escaping.
 *
 * Some benchmarks (times tended to fluctuate +-0.05ms):
 *                                     FireFox                     IE6
 * (no chars / average (mix of cases) / all 4 chars)
 * no checks                     0.13 / 0.22 / 0.22         0.23 / 0.53 / 0.80
 * indexOf                       0.08 / 0.17 / 0.26         0.22 / 0.54 / 0.84
 * indexOf + re test             0.07 / 0.17 / 0.28         0.19 / 0.50 / 0.85
 *
 * An additional advantage of checking if replace actually needs to be called
 * is a reduction in the number of object allocations, so as the size of the
 * application grows the difference between the various methods would increase.
 *
 * @param {string} str string to be escaped.
 * @param {boolean=} opt_isLikelyToContainHtmlChars Don't perform a check to see
 *     if the character needs replacing - use this option if you expect each of
 *     the characters to appear often. Leave false if you expect few html
 *     characters to occur in your strings, such as if you are escaping HTML.
 * @return {string} An escaped copy of `str`.
 */
goog.string.htmlEscape = function(str, opt_isLikelyToContainHtmlChars) {
  str = goog.string.internal.htmlEscape(str, opt_isLikelyToContainHtmlChars);
  if (goog.string.DETECT_DOUBLE_ESCAPING) {
    str = str.replace(goog.string.E_RE_, '&#101;');
  }
  return str;
};


/**
 * Regular expression that matches a lowercase letter "e", for use in escaping.
 * @const {!RegExp}
 * @private
 */
goog.string.E_RE_ = /e/g;


/**
 * Unescapes an HTML string.
 *
 * @param {string} str The string to unescape.
 * @return {string} An unescaped copy of `str`.
 */
goog.string.unescapeEntities = function(str) {
  if (goog.string.contains(str, '&')) {
    // We are careful not to use a DOM if we do not have one or we explicitly
    // requested non-DOM html unescaping.
    if (!goog.string.FORCE_NON_DOM_HTML_UNESCAPING &&
        'document' in goog.global) {
      return goog.string.unescapeEntitiesUsingDom_(str);
    } else {
      // Fall back on pure XML entities
      return goog.string.unescapePureXmlEntities_(str);
    }
  }
  return str;
};


/**
 * Unescapes a HTML string using the provided document.
 *
 * @param {string} str The string to unescape.
 * @param {!Document} document A document to use in escaping the string.
 * @return {string} An unescaped copy of `str`.
 */
goog.string.unescapeEntitiesWithDocument = function(str, document) {
  if (goog.string.contains(str, '&')) {
    return goog.string.unescapeEntitiesUsingDom_(str, document);
  }
  return str;
};


/**
 * Unescapes an HTML string using a DOM to resolve non-XML, non-numeric
 * entities. This function is XSS-safe and whitespace-preserving.
 * @private
 * @param {string} str The string to unescape.
 * @param {Document=} opt_document An optional document to use for creating
 *     elements. If this is not specified then the default window.document
 *     will be used.
 * @return {string} The unescaped `str` string.
 */
goog.string.unescapeEntitiesUsingDom_ = function(str, opt_document) {
  /** @type {!Object<string, string>} */
  var seen = {'&amp;': '&', '&lt;': '<', '&gt;': '>', '&quot;': '"'};
  /** @type {!Element} */
  var div;
  if (opt_document) {
    div = opt_document.createElement('div');
  } else {
    div = goog.global.document.createElement('div');
  }
  // Match as many valid entity characters as possible. If the actual entity
  // happens to be shorter, it will still work as innerHTML will return the
  // trailing characters unchanged. Since the entity characters do not include
  // open angle bracket, there is no chance of XSS from the innerHTML use.
  // Since no whitespace is passed to innerHTML, whitespace is preserved.
  return str.replace(goog.string.HTML_ENTITY_PATTERN_, function(s, entity) {
    // Check for cached entity.
    var value = seen[s];
    if (value) {
      return value;
    }
    // Check for numeric entity.
    if (entity.charAt(0) == '#') {
      // Prefix with 0 so that hex entities (e.g. &#x10) parse as hex numbers.
      var n = Number('0' + entity.substr(1));
      if (!isNaN(n)) {
        value = String.fromCharCode(n);
      }
    }
    // Fall back to innerHTML otherwise.
    if (!value) {
      // Append a non-entity character to avoid a bug in Webkit that parses
      // an invalid entity at the end of innerHTML text as the empty string.
      goog.dom.safe.setInnerHtml(
          div,
          goog.html.uncheckedconversions
              .safeHtmlFromStringKnownToSatisfyTypeContract(
                  goog.string.Const.from('Single HTML entity.'), s + ' '));
      // Then remove the trailing character from the result.
      value = div.firstChild.nodeValue.slice(0, -1);
    }
    // Cache and return.
    return seen[s] = value;
  });
};


/**
 * Unescapes XML entities.
 * @private
 * @param {string} str The string to unescape.
 * @return {string} An unescaped copy of `str`.
 */
goog.string.unescapePureXmlEntities_ = function(str) {
  return str.replace(/&([^;]+);/g, function(s, entity) {
    switch (entity) {
      case 'amp':
        return '&';
      case 'lt':
        return '<';
      case 'gt':
        return '>';
      case 'quot':
        return '"';
      default:
        if (entity.charAt(0) == '#') {
          // Prefix with 0 so that hex entities (e.g. &#x10) parse as hex.
          var n = Number('0' + entity.substr(1));
          if (!isNaN(n)) {
            return String.fromCharCode(n);
          }
        }
        // For invalid entities we just return the entity
        return s;
    }
  });
};


/**
 * Regular expression that matches an HTML entity.
 * See also HTML5: Tokenization / Tokenizing character references.
 * @private
 * @type {!RegExp}
 */
goog.string.HTML_ENTITY_PATTERN_ = /&([^;\s<&]+);?/g;


/**
 * Do escaping of whitespace to preserve spatial formatting. We use character
 * entity #160 to make it safer for xml.
 * @param {string} str The string in which to escape whitespace.
 * @param {boolean=} opt_xml Whether to use XML compatible tags.
 * @return {string} An escaped copy of `str`.
 */
goog.string.whitespaceEscape = function(str, opt_xml) {
  // This doesn't use goog.string.preserveSpaces for backwards compatibility.
  return goog.string.newLineToBr(str.replace(/  /g, ' &#160;'), opt_xml);
};


/**
 * Preserve spaces that would be otherwise collapsed in HTML by replacing them
 * with non-breaking space Unicode characters.
 * @param {string} str The string in which to preserve whitespace.
 * @return {string} A copy of `str` with preserved whitespace.
 */
goog.string.preserveSpaces = function(str) {
  return str.replace(/(^|[\n ]) /g, '$1' + goog.string.Unicode.NBSP);
};


/**
 * Strip quote characters around a string.  The second argument is a string of
 * characters to treat as quotes.  This can be a single character or a string of
 * multiple character and in that case each of those are treated as possible
 * quote characters. For example:
 *
 * <pre>
 * goog.string.stripQuotes('"abc"', '"`') --> 'abc'
 * goog.string.stripQuotes('`abc`', '"`') --> 'abc'
 * </pre>
 *
 * @param {string} str The string to strip.
 * @param {string} quoteChars The quote characters to strip.
 * @return {string} A copy of `str` without the quotes.
 */
goog.string.stripQuotes = function(str, quoteChars) {
  var length = quoteChars.length;
  for (var i = 0; i < length; i++) {
    var quoteChar = length == 1 ? quoteChars : quoteChars.charAt(i);
    if (str.charAt(0) == quoteChar && str.charAt(str.length - 1) == quoteChar) {
      return str.substring(1, str.length - 1);
    }
  }
  return str;
};


/**
 * Truncates a string to a certain length and adds '...' if necessary.  The
 * length also accounts for the ellipsis, so a maximum length of 10 and a string
 * 'Hello World!' produces 'Hello W...'.
 * @param {string} str The string to truncate.
 * @param {number} chars Max number of characters.
 * @param {boolean=} opt_protectEscapedCharacters Whether to protect escaped
 *     characters from being cut off in the middle.
 * @return {string} The truncated `str` string.
 */
goog.string.truncate = function(str, chars, opt_protectEscapedCharacters) {
  if (opt_protectEscapedCharacters) {
    str = goog.string.unescapeEntities(str);
  }

  if (str.length > chars) {
    str = str.substring(0, chars - 3) + '...';
  }

  if (opt_protectEscapedCharacters) {
    str = goog.string.htmlEscape(str);
  }

  return str;
};


/**
 * Truncate a string in the middle, adding "..." if necessary,
 * and favoring the beginning of the string.
 * @param {string} str The string to truncate the middle of.
 * @param {number} chars Max number of characters.
 * @param {boolean=} opt_protectEscapedCharacters Whether to protect escaped
 *     characters from being cutoff in the middle.
 * @param {number=} opt_trailingChars Optional number of trailing characters to
 *     leave at the end of the string, instead of truncating as close to the
 *     middle as possible.
 * @return {string} A truncated copy of `str`.
 */
goog.string.truncateMiddle = function(
    str, chars, opt_protectEscapedCharacters, opt_trailingChars) {
  if (opt_protectEscapedCharacters) {
    str = goog.string.unescapeEntities(str);
  }

  if (opt_trailingChars && str.length > chars) {
    if (opt_trailingChars > chars) {
      opt_trailingChars = chars;
    }
    var endPoint = str.length - opt_trailingChars;
    var startPoint = chars - opt_trailingChars;
    str = str.substring(0, startPoint) + '...' + str.substring(endPoint);
  } else if (str.length > chars) {
    // Favor the beginning of the string:
    var half = Math.floor(chars / 2);
    var endPos = str.length - half;
    half += chars % 2;
    str = str.substring(0, half) + '...' + str.substring(endPos);
  }

  if (opt_protectEscapedCharacters) {
    str = goog.string.htmlEscape(str);
  }

  return str;
};


/**
 * Special chars that need to be escaped for goog.string.quote.
 * @private {!Object<string, string>}
 */
goog.string.specialEscapeChars_ = {
  '\0': '\\0',
  '\b': '\\b',
  '\f': '\\f',
  '\n': '\\n',
  '\r': '\\r',
  '\t': '\\t',
  '\x0B': '\\x0B',  // '\v' is not supported in JScript
  '"': '\\"',
  '\\': '\\\\',
  // To support the use case of embedding quoted strings inside of script
  // tags, we have to make sure HTML comments and opening/closing script tags do
  // not appear in the resulting string. The specific strings that must be
  // escaped are documented at:
  // https://html.spec.whatwg.org/multipage/scripting.html#restrictions-for-contents-of-script-elements
  '<': '\\u003C'  // NOTE: JSON.parse crashes on '\\x3c'.
};


/**
 * Character mappings used internally for goog.string.escapeChar.
 * @private {!Object<string, string>}
 */
goog.string.jsEscapeCache_ = {
  '\'': '\\\''
};


/**
 * Encloses a string in double quotes and escapes characters so that the
 * string is a valid JS string. The resulting string is safe to embed in
 * `<script>` tags as "<" is escaped.
 * @param {string} s The string to quote.
 * @return {string} A copy of `s` surrounded by double quotes.
 */
goog.string.quote = function(s) {
  s = String(s);
  var sb = ['"'];
  for (var i = 0; i < s.length; i++) {
    var ch = s.charAt(i);
    var cc = ch.charCodeAt(0);
    sb[i + 1] = goog.string.specialEscapeChars_[ch] ||
        ((cc > 31 && cc < 127) ? ch : goog.string.escapeChar(ch));
  }
  sb.push('"');
  return sb.join('');
};


/**
 * Takes a string and returns the escaped string for that input string.
 * @param {string} str The string to escape.
 * @return {string} An escaped string representing `str`.
 */
goog.string.escapeString = function(str) {
  var sb = [];
  for (var i = 0; i < str.length; i++) {
    sb[i] = goog.string.escapeChar(str.charAt(i));
  }
  return sb.join('');
};


/**
 * Takes a character and returns the escaped string for that character. For
 * example escapeChar(String.fromCharCode(15)) -> "\\x0E".
 * @param {string} c The character to escape.
 * @return {string} An escaped string representing `c`.
 */
goog.string.escapeChar = function(c) {
  if (c in goog.string.jsEscapeCache_) {
    return goog.string.jsEscapeCache_[c];
  }

  if (c in goog.string.specialEscapeChars_) {
    return goog.string.jsEscapeCache_[c] = goog.string.specialEscapeChars_[c];
  }

  var rv = c;
  var cc = c.charCodeAt(0);
  if (cc > 31 && cc < 127) {
    rv = c;
  } else {
    // tab is 9 but handled above
    if (cc < 256) {
      rv = '\\x';
      if (cc < 16 || cc > 256) {
        rv += '0';
      }
    } else {
      rv = '\\u';
      if (cc < 4096) {  // \u1000
        rv += '0';
      }
    }
    rv += cc.toString(16).toUpperCase();
  }

  return goog.string.jsEscapeCache_[c] = rv;
};


/**
 * Determines whether a string contains a substring.
 * @param {string} str The string to search.
 * @param {string} subString The substring to search for.
 * @return {boolean} Whether `str` contains `subString`.
 */
goog.string.contains = goog.string.internal.contains;


/**
 * Determines whether a string contains a substring, ignoring case.
 * @param {string} str The string to search.
 * @param {string} subString The substring to search for.
 * @return {boolean} Whether `str` contains `subString`.
 */
goog.string.caseInsensitiveContains =
    goog.string.internal.caseInsensitiveContains;


/**
 * Returns the non-overlapping occurrences of ss in s.
 * If either s or ss evalutes to false, then returns zero.
 * @param {string} s The string to look in.
 * @param {string} ss The string to look for.
 * @return {number} Number of occurrences of ss in s.
 */
goog.string.countOf = function(s, ss) {
  return s && ss ? s.split(ss).length - 1 : 0;
};


/**
 * Removes a substring of a specified length at a specific
 * index in a string.
 * @param {string} s The base string from which to remove.
 * @param {number} index The index at which to remove the substring.
 * @param {number} stringLength The length of the substring to remove.
 * @return {string} A copy of `s` with the substring removed or the full
 *     string if nothing is removed or the input is invalid.
 */
goog.string.removeAt = function(s, index, stringLength) {
  var resultStr = s;
  // If the index is greater or equal to 0 then remove substring
  if (index >= 0 && index < s.length && stringLength > 0) {
    resultStr = s.substr(0, index) +
        s.substr(index + stringLength, s.length - index - stringLength);
  }
  return resultStr;
};


/**
 * Removes the first occurrence of a substring from a string.
 * @param {string} str The base string from which to remove.
 * @param {string} substr The string to remove.
 * @return {string} A copy of `str` with `substr` removed or the
 *     full string if nothing is removed.
 */
goog.string.remove = function(str, substr) {
  return str.replace(substr, '');
};


/**
 *  Removes all occurrences of a substring from a string.
 *  @param {string} s The base string from which to remove.
 *  @param {string} ss The string to remove.
 *  @return {string} A copy of `s` with `ss` removed or the full
 *      string if nothing is removed.
 */
goog.string.removeAll = function(s, ss) {
  var re = new RegExp(goog.string.regExpEscape(ss), 'g');
  return s.replace(re, '');
};


/**
 *  Replaces all occurrences of a substring of a string with a new substring.
 *  @param {string} s The base string from which to remove.
 *  @param {string} ss The string to replace.
 *  @param {string} replacement The replacement string.
 *  @return {string} A copy of `s` with `ss` replaced by
 *      `replacement` or the original string if nothing is replaced.
 */
goog.string.replaceAll = function(s, ss, replacement) {
  var re = new RegExp(goog.string.regExpEscape(ss), 'g');
  return s.replace(re, replacement.replace(/\$/g, '$$$$'));
};


/**
 * Escapes characters in the string that are not safe to use in a RegExp.
 * @param {*} s The string to escape. If not a string, it will be casted
 *     to one.
 * @return {string} A RegExp safe, escaped copy of `s`.
 */
goog.string.regExpEscape = function(s) {
  return String(s)
      .replace(/([-()\[\]{}+?*.$\^|,:#<!\\])/g, '\\$1')
      .replace(/\x08/g, '\\x08');
};


/**
 * Repeats a string n times.
 * @param {string} string The string to repeat.
 * @param {number} length The number of times to repeat.
 * @return {string} A string containing `length` repetitions of
 *     `string`.
 */
goog.string.repeat = (String.prototype.repeat) ? function(string, length) {
  // The native method is over 100 times faster than the alternative.
  return string.repeat(length);
} : function(string, length) {
  return new Array(length + 1).join(string);
};


/**
 * Pads number to given length and optionally rounds it to a given precision.
 * For example:
 * <pre>padNumber(1.25, 2, 3) -> '01.250'
 * padNumber(1.25, 2) -> '01.25'
 * padNumber(1.25, 2, 1) -> '01.3'
 * padNumber(1.25, 0) -> '1.25'</pre>
 *
 * @param {number} num The number to pad.
 * @param {number} length The desired length.
 * @param {number=} opt_precision The desired precision.
 * @return {string} `num` as a string with the given options.
 */
goog.string.padNumber = function(num, length, opt_precision) {
  var s =
      (opt_precision !== undefined) ? num.toFixed(opt_precision) : String(num);
  var index = s.indexOf('.');
  if (index == -1) {
    index = s.length;
  }
  return goog.string.repeat('0', Math.max(0, length - index)) + s;
};


/**
 * Returns a string representation of the given object, with
 * null and undefined being returned as the empty string.
 *
 * @param {*} obj The object to convert.
 * @return {string} A string representation of the `obj`.
 */
goog.string.makeSafe = function(obj) {
  return obj == null ? '' : String(obj);
};


/**
 * Concatenates string expressions. This is useful
 * since some browsers are very inefficient when it comes to using plus to
 * concat strings. Be careful when using null and undefined here since
 * these will not be included in the result. If you need to represent these
 * be sure to cast the argument to a String first.
 * For example:
 * <pre>buildString('a', 'b', 'c', 'd') -> 'abcd'
 * buildString(null, undefined) -> ''
 * </pre>
 * @param {...*} var_args A list of strings to concatenate. If not a string,
 *     it will be casted to one.
 * @return {string} The concatenation of `var_args`.
 */
goog.string.buildString = function(var_args) {
  return Array.prototype.join.call(arguments, '');
};


/**
 * Returns a string with at least 64-bits of randomness.
 *
 * Doesn't trust JavaScript's random function entirely. Uses a combination of
 * random and current timestamp, and then encodes the string in base-36 to
 * make it shorter.
 *
 * @return {string} A random string, e.g. sn1s7vb4gcic.
 */
goog.string.getRandomString = function() {
  var x = 2147483648;
  return Math.floor(Math.random() * x).toString(36) +
      Math.abs(Math.floor(Math.random() * x) ^ goog.now()).toString(36);
};


/**
 * Compares two version numbers.
 *
 * @param {string|number} version1 Version of first item.
 * @param {string|number} version2 Version of second item.
 *
 * @return {number}  1 if `version1` is higher.
 *                   0 if arguments are equal.
 *                  -1 if `version2` is higher.
 */
goog.string.compareVersions = goog.string.internal.compareVersions;


/**
 * String hash function similar to java.lang.String.hashCode().
 * The hash code for a string is computed as
 * s[0] * 31 ^ (n - 1) + s[1] * 31 ^ (n - 2) + ... + s[n - 1],
 * where s[i] is the ith character of the string and n is the length of
 * the string. We mod the result to make it between 0 (inclusive) and 2^32
 * (exclusive).
 * @param {string} str A string.
 * @return {number} Hash value for `str`, between 0 (inclusive) and 2^32
 *  (exclusive). The empty string returns 0.
 */
goog.string.hashCode = function(str) {
  var result = 0;
  for (var i = 0; i < str.length; ++i) {
    // Normalize to 4 byte range, 0 ... 2^32.
    result = (31 * result + str.charCodeAt(i)) >>> 0;
  }
  return result;
};


/**
 * The most recent unique ID. |0 is equivalent to Math.floor in this case.
 * @type {number}
 * @private
 */
goog.string.uniqueStringCounter_ = Math.random() * 0x80000000 | 0;


/**
 * Generates and returns a string which is unique in the current document.
 * This is useful, for example, to create unique IDs for DOM elements.
 * @return {string} A unique id.
 */
goog.string.createUniqueString = function() {
  return 'goog_' + goog.string.uniqueStringCounter_++;
};


/**
 * Converts the supplied string to a number, which may be Infinity or NaN.
 * This function strips whitespace: (toNumber(' 123') === 123)
 * This function accepts scientific notation: (toNumber('1e1') === 10)
 *
 * This is better than JavaScript's built-in conversions because, sadly:
 *     (Number(' ') === 0) and (parseFloat('123a') === 123)
 *
 * @param {string} str The string to convert.
 * @return {number} The number the supplied string represents, or NaN.
 */
goog.string.toNumber = function(str) {
  var num = Number(str);
  if (num == 0 && goog.string.isEmptyOrWhitespace(str)) {
    return NaN;
  }
  return num;
};


/**
 * Returns whether the given string is lower camel case (e.g. "isFooBar").
 *
 * Note that this assumes the string is entirely letters.
 * @see http://en.wikipedia.org/wiki/CamelCase#Variations_and_synonyms
 *
 * @param {string} str String to test.
 * @return {boolean} Whether the string is lower camel case.
 */
goog.string.isLowerCamelCase = function(str) {
  return /^[a-z]+([A-Z][a-z]*)*$/.test(str);
};


/**
 * Returns whether the given string is upper camel case (e.g. "FooBarBaz").
 *
 * Note that this assumes the string is entirely letters.
 * @see http://en.wikipedia.org/wiki/CamelCase#Variations_and_synonyms
 *
 * @param {string} str String to test.
 * @return {boolean} Whether the string is upper camel case.
 */
goog.string.isUpperCamelCase = function(str) {
  return /^([A-Z][a-z]*)+$/.test(str);
};


/**
 * Converts a string from selector-case to camelCase (e.g. from
 * "multi-part-string" to "multiPartString"), useful for converting
 * CSS selectors and HTML dataset keys to their equivalent JS properties.
 * @param {string} str The string in selector-case form.
 * @return {string} The string in camelCase form.
 */
goog.string.toCamelCase = function(str) {
  return String(str).replace(/\-([a-z])/g, function(all, match) {
    return match.toUpperCase();
  });
};


/**
 * Converts a string from camelCase to selector-case (e.g. from
 * "multiPartString" to "multi-part-string"), useful for converting JS
 * style and dataset properties to equivalent CSS selectors and HTML keys.
 * @param {string} str The string in camelCase form.
 * @return {string} The string in selector-case form.
 */
goog.string.toSelectorCase = function(str) {
  return String(str).replace(/([A-Z])/g, '-$1').toLowerCase();
};


/**
 * Converts a string into TitleCase. First character of the string is always
 * capitalized in addition to the first letter of every subsequent word.
 * Words are delimited by one or more whitespaces by default. Custom delimiters
 * can optionally be specified to replace the default, which doesn't preserve
 * whitespace delimiters and instead must be explicitly included if needed.
 *
 * Default delimiter => " ":
 *    goog.string.toTitleCase('oneTwoThree')    => 'OneTwoThree'
 *    goog.string.toTitleCase('one two three')  => 'One Two Three'
 *    goog.string.toTitleCase('  one   two   ') => '  One   Two   '
 *    goog.string.toTitleCase('one_two_three')  => 'One_two_three'
 *    goog.string.toTitleCase('one-two-three')  => 'One-two-three'
 *
 * Custom delimiter => "_-.":
 *    goog.string.toTitleCase('oneTwoThree', '_-.')       => 'OneTwoThree'
 *    goog.string.toTitleCase('one two three', '_-.')     => 'One two three'
 *    goog.string.toTitleCase('  one   two   ', '_-.')    => '  one   two   '
 *    goog.string.toTitleCase('one_two_three', '_-.')     => 'One_Two_Three'
 *    goog.string.toTitleCase('one-two-three', '_-.')     => 'One-Two-Three'
 *    goog.string.toTitleCase('one...two...three', '_-.') => 'One...Two...Three'
 *    goog.string.toTitleCase('one. two. three', '_-.')   => 'One. two. three'
 *    goog.string.toTitleCase('one-two.three', '_-.')     => 'One-Two.Three'
 *
 * @param {string} str String value in camelCase form.
 * @param {string=} opt_delimiters Custom delimiter character set used to
 *      distinguish words in the string value. Each character represents a
 *      single delimiter. When provided, default whitespace delimiter is
 *      overridden and must be explicitly included if needed.
 * @return {string} String value in TitleCase form.
 */
goog.string.toTitleCase = function(str, opt_delimiters) {
  var delimiters = (typeof opt_delimiters === 'string') ?
      goog.string.regExpEscape(opt_delimiters) :
      '\\s';

  // For IE8, we need to prevent using an empty character set. Otherwise,
  // incorrect matching will occur.
  delimiters = delimiters ? '|[' + delimiters + ']+' : '';

  var regexp = new RegExp('(^' + delimiters + ')([a-z])', 'g');
  return str.replace(regexp, function(all, p1, p2) {
    return p1 + p2.toUpperCase();
  });
};


/**
 * Capitalizes a string, i.e. converts the first letter to uppercase
 * and all other letters to lowercase, e.g.:
 *
 * goog.string.capitalize('one')     => 'One'
 * goog.string.capitalize('ONE')     => 'One'
 * goog.string.capitalize('one two') => 'One two'
 *
 * Note that this function does not trim initial whitespace.
 *
 * @param {string} str String value to capitalize.
 * @return {string} String value with first letter in uppercase.
 */
goog.string.capitalize = function(str) {
  return String(str.charAt(0)).toUpperCase() +
      String(str.substr(1)).toLowerCase();
};


/**
 * Parse a string in decimal or hexidecimal ('0xFFFF') form.
 *
 * To parse a particular radix, please use parseInt(string, radix) directly. See
 * https://developer.mozilla.org/en/JavaScript/Reference/Global_Objects/parseInt
 *
 * This is a wrapper for the built-in parseInt function that will only parse
 * numbers as base 10 or base 16.  Some JS implementations assume strings
 * starting with "0" are intended to be octal. ES3 allowed but discouraged
 * this behavior. ES5 forbids it.  This function emulates the ES5 behavior.
 *
 * For more information, see Mozilla JS Reference: http://goo.gl/8RiFj
 *
 * @param {string|number|null|undefined} value The value to be parsed.
 * @return {number} The number, parsed. If the string failed to parse, this
 *     will be NaN.
 */
goog.string.parseInt = function(value) {
  // Force finite numbers to strings.
  if (isFinite(value)) {
    value = String(value);
  }

  if (typeof value === 'string') {
    // If the string starts with '0x' or '-0x', parse as hex.
    return /^\s*-?0x/i.test(value) ? parseInt(value, 16) : parseInt(value, 10);
  }

  return NaN;
};


/**
 * Splits a string on a separator a limited number of times.
 *
 * This implementation is more similar to Python or Java, where the limit
 * parameter specifies the maximum number of splits rather than truncating
 * the number of results.
 *
 * See http://docs.python.org/2/library/stdtypes.html#str.split
 * See JavaDoc: http://goo.gl/F2AsY
 * See Mozilla reference: http://goo.gl/dZdZs
 *
 * @param {string} str String to split.
 * @param {string} separator The separator.
 * @param {number} limit The limit to the number of splits. The resulting array
 *     will have a maximum length of limit+1.  Negative numbers are the same
 *     as zero.
 * @return {!Array<string>} The string, split.
 */
goog.string.splitLimit = function(str, separator, limit) {
  var parts = str.split(separator);
  var returnVal = [];

  // Only continue doing this while we haven't hit the limit and we have
  // parts left.
  while (limit > 0 && parts.length) {
    returnVal.push(parts.shift());
    limit--;
  }

  // If there are remaining parts, append them to the end.
  if (parts.length) {
    returnVal.push(parts.join(separator));
  }

  return returnVal;
};


/**
 * Finds the characters to the right of the last instance of any separator
 *
 * This function is similar to goog.string.path.baseName, except it can take a
 * list of characters to split the string on. It will return the rightmost
 * grouping of characters to the right of any separator as a left-to-right
 * oriented string.
 *
 * @see goog.string.path.baseName
 * @param {string} str The string
 * @param {string|!Array<string>} separators A list of separator characters
 * @return {string} The last part of the string with respect to the separators
 */
goog.string.lastComponent = function(str, separators) {
  if (!separators) {
    return str;
  } else if (typeof separators == 'string') {
    separators = [separators];
  }

  var lastSeparatorIndex = -1;
  for (var i = 0; i < separators.length; i++) {
    if (separators[i] == '') {
      continue;
    }
    var currentSeparatorIndex = str.lastIndexOf(separators[i]);
    if (currentSeparatorIndex > lastSeparatorIndex) {
      lastSeparatorIndex = currentSeparatorIndex;
    }
  }
  if (lastSeparatorIndex == -1) {
    return str;
  }
  return str.slice(lastSeparatorIndex + 1);
};


/**
 * Computes the Levenshtein edit distance between two strings.
 * @param {string} a
 * @param {string} b
 * @return {number} The edit distance between the two strings.
 */
goog.string.editDistance = function(a, b) {
  var v0 = [];
  var v1 = [];

  if (a == b) {
    return 0;
  }

  if (!a.length || !b.length) {
    return Math.max(a.length, b.length);
  }

  for (var i = 0; i < b.length + 1; i++) {
    v0[i] = i;
  }

  for (var i = 0; i < a.length; i++) {
    v1[0] = i + 1;

    for (var j = 0; j < b.length; j++) {
      var cost = Number(a[i] != b[j]);
      // Cost for the substring is the minimum of adding one character, removing
      // one character, or a swap.
      v1[j + 1] = Math.min(v1[j] + 1, v0[j + 1] + 1, v0[j] + cost);
    }

    for (var j = 0; j < v0.length; j++) {
      v0[j] = v1[j];
    }
  }

  return v1[b.length];
};

//third_party/javascript/closure/labs/useragent/platform.js
/**
 * @license
 * Copyright The Closure Library Authors.
 * SPDX-License-Identifier: Apache-2.0
 */

/**
 * @fileoverview Closure user agent platform detection.
 * @see <a href="http://www.useragentstring.com/">User agent strings</a>
 * For more information on browser brand, rendering engine, or device see the
 * other sub-namespaces in goog.labs.userAgent (browser, engine, and device
 * respectively).
 */

goog.provide('goog.labs.userAgent.platform');

goog.require('goog.labs.userAgent.util');
goog.require('goog.string');


/**
 * @return {boolean} Whether the platform is Android.
 */
goog.labs.userAgent.platform.isAndroid = function() {
  return goog.labs.userAgent.util.matchUserAgent('Android');
};


/**
 * @return {boolean} Whether the platform is iPod.
 */
goog.labs.userAgent.platform.isIpod = function() {
  return goog.labs.userAgent.util.matchUserAgent('iPod');
};


/**
 * @return {boolean} Whether the platform is iPhone.
 */
goog.labs.userAgent.platform.isIphone = function() {
  return goog.labs.userAgent.util.matchUserAgent('iPhone') &&
      !goog.labs.userAgent.util.matchUserAgent('iPod') &&
      !goog.labs.userAgent.util.matchUserAgent('iPad');
};


/**
 * @return {boolean} Whether the platform is iPad.
 */
goog.labs.userAgent.platform.isIpad = function() {
  return goog.labs.userAgent.util.matchUserAgent('iPad');
};


/**
 * @return {boolean} Whether the platform is iOS.
 */
goog.labs.userAgent.platform.isIos = function() {
  return goog.labs.userAgent.platform.isIphone() ||
      goog.labs.userAgent.platform.isIpad() ||
      goog.labs.userAgent.platform.isIpod();
};


/**
 * @return {boolean} Whether the platform is Mac.
 */
goog.labs.userAgent.platform.isMacintosh = function() {
  return goog.labs.userAgent.util.matchUserAgent('Macintosh');
};


/**
 * Note: ChromeOS is not considered to be Linux as it does not report itself
 * as Linux in the user agent string.
 * @return {boolean} Whether the platform is Linux.
 */
goog.labs.userAgent.platform.isLinux = function() {
  return goog.labs.userAgent.util.matchUserAgent('Linux');
};


/**
 * @return {boolean} Whether the platform is Windows.
 */
goog.labs.userAgent.platform.isWindows = function() {
  return goog.labs.userAgent.util.matchUserAgent('Windows');
};


/**
 * @return {boolean} Whether the platform is ChromeOS.
 */
goog.labs.userAgent.platform.isChromeOS = function() {
  return goog.labs.userAgent.util.matchUserAgent('CrOS');
};

/**
 * @return {boolean} Whether the platform is Chromecast.
 */
goog.labs.userAgent.platform.isChromecast = function() {
  return goog.labs.userAgent.util.matchUserAgent('CrKey');
};

/**
 * @return {boolean} Whether the platform is KaiOS.
 */
goog.labs.userAgent.platform.isKaiOS = function() {
  return goog.labs.userAgent.util.matchUserAgentIgnoreCase('KaiOS');
};

/**
 * The version of the platform. We only determine the version for Windows,
 * Mac, and Chrome OS. It doesn't make much sense on Linux. For Windows, we only
 * look at the NT version. Non-NT-based versions (e.g. 95, 98, etc.) are given
 * version 0.0.
 *
 * @return {string} The platform version or empty string if version cannot be
 *     determined.
 */
goog.labs.userAgent.platform.getVersion = function() {
  var userAgentString = goog.labs.userAgent.util.getUserAgent();
  var version = '', re;
  if (goog.labs.userAgent.platform.isWindows()) {
    re = /Windows (?:NT|Phone) ([0-9.]+)/;
    var match = re.exec(userAgentString);
    if (match) {
      version = match[1];
    } else {
      version = '0.0';
    }
  } else if (goog.labs.userAgent.platform.isIos()) {
    re = /(?:iPhone|iPod|iPad|CPU)\s+OS\s+(\S+)/;
    var match = re.exec(userAgentString);
    // Report the version as x.y.z and not x_y_z
    version = match && match[1].replace(/_/g, '.');
  } else if (goog.labs.userAgent.platform.isMacintosh()) {
    re = /Mac OS X ([0-9_.]+)/;
    var match = re.exec(userAgentString);
    // Note: some old versions of Camino do not report an OSX version.
    // Default to 10.
    version = match ? match[1].replace(/_/g, '.') : '10';
  } else if (goog.labs.userAgent.platform.isKaiOS()) {
    re = /(?:KaiOS)\/(\S+)/i;
    var match = re.exec(userAgentString);
    version = match && match[1];
  } else if (goog.labs.userAgent.platform.isAndroid()) {
    re = /Android\s+([^\);]+)(\)|;)/;
    var match = re.exec(userAgentString);
    version = match && match[1];
  } else if (goog.labs.userAgent.platform.isChromeOS()) {
    re = /(?:CrOS\s+(?:i686|x86_64)\s+([0-9.]+))/;
    var match = re.exec(userAgentString);
    version = match && match[1];
  }
  return version || '';
};


/**
 * @param {string|number} version The version to check.
 * @return {boolean} Whether the browser version is higher or the same as the
 *     given version.
 */
goog.labs.userAgent.platform.isVersionOrHigher = function(version) {
  return goog.string.compareVersions(
             goog.labs.userAgent.platform.getVersion(), version) >= 0;
};

//third_party/javascript/closure/labs/useragent/engine.js
/**
 * @license
 * Copyright The Closure Library Authors.
 * SPDX-License-Identifier: Apache-2.0
 */

/**
 * @fileoverview Closure user agent detection.
 * @see http://en.wikipedia.org/wiki/User_agent
 * For more information on browser brand, platform, or device see the other
 * sub-namespaces in goog.labs.userAgent (browser, platform, and device).
 */

goog.provide('goog.labs.userAgent.engine');

goog.require('goog.array');
goog.require('goog.labs.userAgent.util');
goog.require('goog.string');


/**
 * @return {boolean} Whether the rendering engine is Presto.
 */
goog.labs.userAgent.engine.isPresto = function() {
  return goog.labs.userAgent.util.matchUserAgent('Presto');
};


/**
 * @return {boolean} Whether the rendering engine is Trident.
 */
goog.labs.userAgent.engine.isTrident = function() {
  // IE only started including the Trident token in IE8.
  return goog.labs.userAgent.util.matchUserAgent('Trident') ||
      goog.labs.userAgent.util.matchUserAgent('MSIE');
};


/**
 * @return {boolean} Whether the rendering engine is EdgeHTML.
 */
goog.labs.userAgent.engine.isEdge = function() {
  return goog.labs.userAgent.util.matchUserAgent('Edge');
};


/**
 * @return {boolean} Whether the rendering engine is WebKit. This will return
 * true for Chrome, Blink-based Opera (15+), Edge Chromium and Safari.
 */
goog.labs.userAgent.engine.isWebKit = function() {
  return goog.labs.userAgent.util.matchUserAgentIgnoreCase('WebKit') &&
      !goog.labs.userAgent.engine.isEdge();
};


/**
 * @return {boolean} Whether the rendering engine is Gecko.
 */
goog.labs.userAgent.engine.isGecko = function() {
  return goog.labs.userAgent.util.matchUserAgent('Gecko') &&
      !goog.labs.userAgent.engine.isWebKit() &&
      !goog.labs.userAgent.engine.isTrident() &&
      !goog.labs.userAgent.engine.isEdge();
};


/**
 * @return {string} The rendering engine's version or empty string if version
 *     can't be determined.
 */
goog.labs.userAgent.engine.getVersion = function() {
  var userAgentString = goog.labs.userAgent.util.getUserAgent();
  if (userAgentString) {
    var tuples = goog.labs.userAgent.util.extractVersionTuples(userAgentString);

    var engineTuple = goog.labs.userAgent.engine.getEngineTuple_(tuples);
    if (engineTuple) {
      // In Gecko, the version string is either in the browser info or the
      // Firefox version.  See Gecko user agent string reference:
      // http://goo.gl/mULqa
      if (engineTuple[0] == 'Gecko') {
        return goog.labs.userAgent.engine.getVersionForKey_(tuples, 'Firefox');
      }

      return engineTuple[1];
    }

    // MSIE has only one version identifier, and the Trident version is
    // specified in the parenthetical. IE Edge is covered in the engine tuple
    // detection.
    var browserTuple = tuples[0];
    var info;
    if (browserTuple && (info = browserTuple[2])) {
      var match = /Trident\/([^\s;]+)/.exec(info);
      if (match) {
        return match[1];
      }
    }
  }
  return '';
};


/**
 * @param {!Array<!Array<string>>} tuples Extracted version tuples.
 * @return {!Array<string>|undefined} The engine tuple or undefined if not
 *     found.
 * @private
 */
goog.labs.userAgent.engine.getEngineTuple_ = function(tuples) {
  if (!goog.labs.userAgent.engine.isEdge()) {
    return tuples[1];
  }
  for (var i = 0; i < tuples.length; i++) {
    var tuple = tuples[i];
    if (tuple[0] == 'Edge') {
      return tuple;
    }
  }
};


/**
 * @param {string|number} version The version to check.
 * @return {boolean} Whether the rendering engine version is higher or the same
 *     as the given version.
 */
goog.labs.userAgent.engine.isVersionOrHigher = function(version) {
  return goog.string.compareVersions(
             goog.labs.userAgent.engine.getVersion(), version) >= 0;
};


/**
 * @param {!Array<!Array<string>>} tuples Version tuples.
 * @param {string} key The key to look for.
 * @return {string} The version string of the given key, if present.
 *     Otherwise, the empty string.
 * @private
 */
goog.labs.userAgent.engine.getVersionForKey_ = function(tuples, key) {
  // TODO(nnaze): Move to util if useful elsewhere.

  var pair = goog.array.find(tuples, function(pair) { return key == pair[0]; });

  return pair && pair[1] || '';
};

//third_party/javascript/closure/reflect/reflect.js
/**
 * @license
 * Copyright The Closure Library Authors.
 * SPDX-License-Identifier: Apache-2.0
 */

/**
 * @fileoverview Useful compiler idioms.
 */

goog.provide('goog.reflect');


/**
 * Syntax for object literal casts.
 * @see http://go/jscompiler-renaming
 * @see https://goo.gl/CRs09P
 *
 * Use this if you have an object literal whose keys need to have the same names
 * as the properties of some class even after they are renamed by the compiler.
 *
 * @param {!Function} type Type to cast to.
 * @param {Object} object Object literal to cast.
 * @return {Object} The object literal.
 */
goog.reflect.object = function(type, object) {
  return object;
};

/**
 * Syntax for renaming property strings.
 * @see http://go/jscompiler-renaming
 * @see https://goo.gl/CRs09P
 *
 * Use this if you have an need to access a property as a string, but want
 * to also have the property renamed by the compiler. In contrast to
 * goog.reflect.object, this method takes an instance of an object.
 *
 * Properties must be simple names (not qualified names).
 *
 * @param {string} prop Name of the property
 * @param {!Object} object Instance of the object whose type will be used
 *     for renaming
 * @return {string} The renamed property.
 */
goog.reflect.objectProperty = function(prop, object) {
  return prop;
};

/**
 * To assert to the compiler that an operation is needed when it would
 * otherwise be stripped. For example:
 * <code>
 *     // Force a layout
 *     goog.reflect.sinkValue(dialog.offsetHeight);
 * </code>
 * @param {T} x
 * @return {T}
 * @template T
 */
goog.reflect.sinkValue = function(x) {
  goog.reflect.sinkValue[' '](x);
  return x;
};


/**
 * The compiler should optimize this function away iff no one ever uses
 * goog.reflect.sinkValue.
 */
goog.reflect.sinkValue[' '] = goog.nullFunction;


/**
 * Check if a property can be accessed without throwing an exception.
 * @param {Object} obj The owner of the property.
 * @param {string} prop The property name.
 * @return {boolean} Whether the property is accessible. Will also return true
 *     if obj is null.
 */
goog.reflect.canAccessProperty = function(obj, prop) {

  try {
    goog.reflect.sinkValue(obj[prop]);
    return true;
  } catch (e) {
  }
  return false;
};


/**
 * Retrieves a value from a cache given a key. The compiler provides special
 * consideration for this call such that it is generally considered side-effect
 * free. However, if the `opt_keyFn` or `valueFn` have side-effects
 * then the entire call is considered to have side-effects.
 *
 * Conventionally storing the value on the cache would be considered a
 * side-effect and preclude unused calls from being pruned, ie. even if
 * the value was never used, it would still always be stored in the cache.
 *
 * Providing a side-effect free `valueFn` and `opt_keyFn`
 * allows unused calls to `goog.reflect.cache` to be pruned.
 *
 * @param {!Object<K, V>} cacheObj The object that contains the cached values.
 * @param {?} key The key to lookup in the cache. If it is not string or number
 *     then a `opt_keyFn` should be provided. The key is also used as the
 *     parameter to the `valueFn`.
 * @param {function(?):V} valueFn The value provider to use to calculate the
 *     value to store in the cache. This function should be side-effect free
 *     to take advantage of the optimization.
 * @param {function(?):K=} opt_keyFn The key provider to determine the cache
 *     map key. This should be used if the given key is not a string or number.
 *     If not provided then the given key is used. This function should be
 *     side-effect free to take advantage of the optimization.
 * @return {V} The cached or calculated value.
 * @template K
 * @template V
 */
goog.reflect.cache = function(cacheObj, key, valueFn, opt_keyFn) {
  const storedKey = opt_keyFn ? opt_keyFn(key) : key;

  if (Object.prototype.hasOwnProperty.call(cacheObj, storedKey)) {
    return cacheObj[storedKey];
  }

  return (cacheObj[storedKey] = valueFn(key));
};

//third_party/javascript/closure/useragent/useragent.js
/**
 * @license
 * Copyright The Closure Library Authors.
 * SPDX-License-Identifier: Apache-2.0
 */

/**
 * @fileoverview Rendering engine detection.
 * @see <a href="http://www.useragentstring.com/">User agent strings</a>
 * For information on the browser brand (such as Safari versus Chrome), see
 * goog.userAgent.product.
 * @see ../demos/useragent.html
 */

goog.provide('goog.userAgent');

goog.require('goog.labs.userAgent.browser');
goog.require('goog.labs.userAgent.engine');
goog.require('goog.labs.userAgent.platform');
goog.require('goog.labs.userAgent.util');
goog.require('goog.reflect');
goog.require('goog.string');


/**
 * @define {boolean} Whether we know at compile-time that the browser is IE.
 */
goog.userAgent.ASSUME_IE = goog.define('goog.userAgent.ASSUME_IE', false);


/**
 * @define {boolean} Whether we know at compile-time that the browser is EDGE,
 * referring to EdgeHTML based Edge.
 */
goog.userAgent.ASSUME_EDGE = goog.define('goog.userAgent.ASSUME_EDGE', false);


/**
 * @define {boolean} Whether we know at compile-time that the browser is GECKO.
 */
goog.userAgent.ASSUME_GECKO = goog.define('goog.userAgent.ASSUME_GECKO', false);


/**
 * @define {boolean} Whether we know at compile-time that the browser is WEBKIT.
 */
goog.userAgent.ASSUME_WEBKIT =
    goog.define('goog.userAgent.ASSUME_WEBKIT', false);


/**
 * @define {boolean} Whether we know at compile-time that the browser is a
 *     mobile device running WebKit e.g. iPhone or Android.
 */
goog.userAgent.ASSUME_MOBILE_WEBKIT =
    goog.define('goog.userAgent.ASSUME_MOBILE_WEBKIT', false);


/**
 * @define {boolean} Whether we know at compile-time that the browser is OPERA,
 * referring to Presto-based Opera.
 */
goog.userAgent.ASSUME_OPERA = goog.define('goog.userAgent.ASSUME_OPERA', false);


/**
 * @define {boolean} Whether the
 *     `goog.userAgent.isVersionOrHigher`
 *     function will return true for any version.
 */
goog.userAgent.ASSUME_ANY_VERSION =
    goog.define('goog.userAgent.ASSUME_ANY_VERSION', false);


/**
 * Whether we know the browser engine at compile-time.
 * @type {boolean}
 * @private
 */
goog.userAgent.BROWSER_KNOWN_ = goog.userAgent.ASSUME_IE ||
    goog.userAgent.ASSUME_EDGE || goog.userAgent.ASSUME_GECKO ||
    goog.userAgent.ASSUME_MOBILE_WEBKIT || goog.userAgent.ASSUME_WEBKIT ||
    goog.userAgent.ASSUME_OPERA;


/**
 * Returns the userAgent string for the current browser.
 *
 * @return {string} The userAgent string.
 */
goog.userAgent.getUserAgentString = function() {
  return goog.labs.userAgent.util.getUserAgent();
};


/**
 * @return {?Navigator} The native navigator object.
 */
goog.userAgent.getNavigatorTyped = function() {
  // Need a local navigator reference instead of using the global one,
  // to avoid the rare case where they reference different objects.
  // (in a WorkerPool, for example).
  return goog.global['navigator'] || null;
};


/**
 * TODO(nnaze): Change type to "Navigator" and update compilation targets.
 * @return {?Object} The native navigator object.
 */
goog.userAgent.getNavigator = function() {
  return goog.userAgent.getNavigatorTyped();
};


/**
 * Whether the user agent is Presto-based Opera.
 * @type {boolean}
 */
goog.userAgent.OPERA = goog.userAgent.BROWSER_KNOWN_ ?
    goog.userAgent.ASSUME_OPERA :
    goog.labs.userAgent.browser.isOpera();


/**
 * Whether the user agent is Internet Explorer.
 * @type {boolean}
 */
goog.userAgent.IE = goog.userAgent.BROWSER_KNOWN_ ?
    goog.userAgent.ASSUME_IE :
    goog.labs.userAgent.browser.isIE();


/**
 * Whether the user agent is Microsoft Edge (EdgeHTML based).
 * @type {boolean}
 */
goog.userAgent.EDGE = goog.userAgent.BROWSER_KNOWN_ ?
    goog.userAgent.ASSUME_EDGE :
    goog.labs.userAgent.engine.isEdge();


/**
 * Whether the user agent is MS Internet Explorer or MS Edge (EdgeHTML based).
 * @type {boolean}
 */
goog.userAgent.EDGE_OR_IE = goog.userAgent.EDGE || goog.userAgent.IE;


/**
 * Whether the user agent is Gecko. Gecko is the rendering engine used by
 * Mozilla, Firefox, and others.
 * @type {boolean}
 */
goog.userAgent.GECKO = goog.userAgent.BROWSER_KNOWN_ ?
    goog.userAgent.ASSUME_GECKO :
    goog.labs.userAgent.engine.isGecko();


/**
 * Whether the user agent is WebKit. WebKit is the rendering engine that
 * Safari, Edge Chromium, Opera Chromium, Android and others use.
 * @type {boolean}
 */
goog.userAgent.WEBKIT = goog.userAgent.BROWSER_KNOWN_ ?
    goog.userAgent.ASSUME_WEBKIT || goog.userAgent.ASSUME_MOBILE_WEBKIT :
    goog.labs.userAgent.engine.isWebKit();


/**
 * Whether the user agent is running on a mobile device.
 *
 * This is a separate function so that the logic can be tested.
 *
 * TODO(nnaze): Investigate swapping in goog.labs.userAgent.device.isMobile().
 *
 * @return {boolean} Whether the user agent is running on a mobile device.
 * @private
 */
goog.userAgent.isMobile_ = function() {
  return goog.userAgent.WEBKIT &&
      goog.labs.userAgent.util.matchUserAgent('Mobile');
};


/**
 * Whether the user agent is running on a mobile device.
 *
 * TODO(nnaze): Consider deprecating MOBILE when labs.userAgent
 *   is promoted as the gecko/webkit logic is likely inaccurate.
 *
 * @type {boolean}
 */
goog.userAgent.MOBILE =
    goog.userAgent.ASSUME_MOBILE_WEBKIT || goog.userAgent.isMobile_();


/**
 * Used while transitioning code to use WEBKIT instead.
 * @type {boolean}
 * @deprecated Use {@link goog.userAgent.product.SAFARI} instead.
 * TODO(nicksantos): Delete this from goog.userAgent.
 */
goog.userAgent.SAFARI = goog.userAgent.WEBKIT;


/**
 * @return {string} the platform (operating system) the user agent is running
 *     on. Default to empty string because navigator.platform may not be defined
 *     (on Rhino, for example).
 * @private
 */
goog.userAgent.determinePlatform_ = function() {
  var navigator = goog.userAgent.getNavigatorTyped();
  return navigator && navigator.platform || '';
};


/**
 * The platform (operating system) the user agent is running on. Default to
 * empty string because navigator.platform may not be defined (on Rhino, for
 * example).
 * @type {string}
 */
goog.userAgent.PLATFORM = goog.userAgent.determinePlatform_();


/**
 * @define {boolean} Whether the user agent is running on a Macintosh operating
 *     system.
 */
goog.userAgent.ASSUME_MAC = goog.define('goog.userAgent.ASSUME_MAC', false);


/**
 * @define {boolean} Whether the user agent is running on a Windows operating
 *     system.
 */
goog.userAgent.ASSUME_WINDOWS =
    goog.define('goog.userAgent.ASSUME_WINDOWS', false);


/**
 * @define {boolean} Whether the user agent is running on a Linux operating
 *     system.
 */
goog.userAgent.ASSUME_LINUX = goog.define('goog.userAgent.ASSUME_LINUX', false);


/**
 * @define {boolean} Whether the user agent is running on a X11 windowing
 *     system.
 */
goog.userAgent.ASSUME_X11 = goog.define('goog.userAgent.ASSUME_X11', false);


/**
 * @define {boolean} Whether the user agent is running on Android.
 */
goog.userAgent.ASSUME_ANDROID =
    goog.define('goog.userAgent.ASSUME_ANDROID', false);


/**
 * @define {boolean} Whether the user agent is running on an iPhone.
 */
goog.userAgent.ASSUME_IPHONE =
    goog.define('goog.userAgent.ASSUME_IPHONE', false);


/**
 * @define {boolean} Whether the user agent is running on an iPad.
 */
goog.userAgent.ASSUME_IPAD = goog.define('goog.userAgent.ASSUME_IPAD', false);


/**
 * @define {boolean} Whether the user agent is running on an iPod.
 */
goog.userAgent.ASSUME_IPOD = goog.define('goog.userAgent.ASSUME_IPOD', false);


/**
 * @define {boolean} Whether the user agent is running on KaiOS.
 */
goog.userAgent.ASSUME_KAIOS = goog.define('goog.userAgent.ASSUME_KAIOS', false);


/**
 * @type {boolean}
 * @private
 */
goog.userAgent.PLATFORM_KNOWN_ = goog.userAgent.ASSUME_MAC ||
    goog.userAgent.ASSUME_WINDOWS || goog.userAgent.ASSUME_LINUX ||
    goog.userAgent.ASSUME_X11 || goog.userAgent.ASSUME_ANDROID ||
    goog.userAgent.ASSUME_IPHONE || goog.userAgent.ASSUME_IPAD ||
    goog.userAgent.ASSUME_IPOD;


/**
 * Whether the user agent is running on a Macintosh operating system.
 * @type {boolean}
 */
goog.userAgent.MAC = goog.userAgent.PLATFORM_KNOWN_ ?
    goog.userAgent.ASSUME_MAC :
    goog.labs.userAgent.platform.isMacintosh();


/**
 * Whether the user agent is running on a Windows operating system.
 * @type {boolean}
 */
goog.userAgent.WINDOWS = goog.userAgent.PLATFORM_KNOWN_ ?
    goog.userAgent.ASSUME_WINDOWS :
    goog.labs.userAgent.platform.isWindows();


/**
 * Whether the user agent is Linux per the legacy behavior of
 * goog.userAgent.LINUX, which considered ChromeOS to also be
 * Linux.
 * @return {boolean}
 * @private
 */
goog.userAgent.isLegacyLinux_ = function() {
  return goog.labs.userAgent.platform.isLinux() ||
      goog.labs.userAgent.platform.isChromeOS();
};


/**
 * Whether the user agent is running on a Linux operating system.
 *
 * Note that goog.userAgent.LINUX considers ChromeOS to be Linux,
 * while goog.labs.userAgent.platform considers ChromeOS and
 * Linux to be different OSes.
 *
 * @type {boolean}
 */
goog.userAgent.LINUX = goog.userAgent.PLATFORM_KNOWN_ ?
    goog.userAgent.ASSUME_LINUX :
    goog.userAgent.isLegacyLinux_();


/**
 * @return {boolean} Whether the user agent is an X11 windowing system.
 * @private
 */
goog.userAgent.isX11_ = function() {
  var navigator = goog.userAgent.getNavigatorTyped();
  return !!navigator &&
      goog.string.contains(navigator['appVersion'] || '', 'X11');
};


/**
 * Whether the user agent is running on a X11 windowing system.
 * @type {boolean}
 */
goog.userAgent.X11 = goog.userAgent.PLATFORM_KNOWN_ ?
    goog.userAgent.ASSUME_X11 :
    goog.userAgent.isX11_();


/**
 * Whether the user agent is running on Android.
 * @type {boolean}
 */
goog.userAgent.ANDROID = goog.userAgent.PLATFORM_KNOWN_ ?
    goog.userAgent.ASSUME_ANDROID :
    goog.labs.userAgent.platform.isAndroid();


/**
 * Whether the user agent is running on an iPhone.
 * @type {boolean}
 */
goog.userAgent.IPHONE = goog.userAgent.PLATFORM_KNOWN_ ?
    goog.userAgent.ASSUME_IPHONE :
    goog.labs.userAgent.platform.isIphone();


/**
 * Whether the user agent is running on an iPad.
 * @type {boolean}
 */
goog.userAgent.IPAD = goog.userAgent.PLATFORM_KNOWN_ ?
    goog.userAgent.ASSUME_IPAD :
    goog.labs.userAgent.platform.isIpad();


/**
 * Whether the user agent is running on an iPod.
 * @type {boolean}
 */
goog.userAgent.IPOD = goog.userAgent.PLATFORM_KNOWN_ ?
    goog.userAgent.ASSUME_IPOD :
    goog.labs.userAgent.platform.isIpod();


/**
 * Whether the user agent is running on iOS.
 * @type {boolean}
 */
goog.userAgent.IOS = goog.userAgent.PLATFORM_KNOWN_ ?
    (goog.userAgent.ASSUME_IPHONE || goog.userAgent.ASSUME_IPAD ||
     goog.userAgent.ASSUME_IPOD) :
    goog.labs.userAgent.platform.isIos();

/**
 * Whether the user agent is running on KaiOS.
 * @type {boolean}
 */
goog.userAgent.KAIOS = goog.userAgent.PLATFORM_KNOWN_ ?
    goog.userAgent.ASSUME_KAIOS :
    goog.labs.userAgent.platform.isKaiOS();


/**
 * @return {string} The string that describes the version number of the user
 *     agent.
 * @private
 */
goog.userAgent.determineVersion_ = function() {
  // All browsers have different ways to detect the version and they all have
  // different naming schemes.
  // version is a string rather than a number because it may contain 'b', 'a',
  // and so on.
  var version = '';
  var arr = goog.userAgent.getVersionRegexResult_();
  if (arr) {
    version = arr ? arr[1] : '';
  }

  if (goog.userAgent.IE) {
    // IE9 can be in document mode 9 but be reporting an inconsistent user agent
    // version.  If it is identifying as a version lower than 9 we take the
    // documentMode as the version instead.  IE8 has similar behavior.
    // It is recommended to set the X-UA-Compatible header to ensure that IE9
    // uses documentMode 9.
    var docMode = goog.userAgent.getDocumentMode_();
    if (docMode != null && docMode > parseFloat(version)) {
      return String(docMode);
    }
  }

  return version;
};


/**
 * @return {?IArrayLike<string>|undefined} The version regex matches from
 *     parsing the user
 *     agent string. These regex statements must be executed inline so they can
 *     be compiled out by the closure compiler with the rest of the useragent
 *     detection logic when ASSUME_* is specified.
 * @private
 */
goog.userAgent.getVersionRegexResult_ = function() {
  var userAgent = goog.userAgent.getUserAgentString();
  if (goog.userAgent.GECKO) {
    return /rv\:([^\);]+)(\)|;)/.exec(userAgent);
  }
  if (goog.userAgent.EDGE) {
    return /Edge\/([\d\.]+)/.exec(userAgent);
  }
  if (goog.userAgent.IE) {
    return /\b(?:MSIE|rv)[: ]([^\);]+)(\)|;)/.exec(userAgent);
  }
  if (goog.userAgent.WEBKIT) {
    // WebKit/125.4
    return /WebKit\/(\S+)/.exec(userAgent);
  }
  if (goog.userAgent.OPERA) {
    // If none of the above browsers were detected but the browser is Opera, the
    // only string that is of interest is 'Version/<number>'.
    return /(?:Version)[ \/]?(\S+)/.exec(userAgent);
  }
  return undefined;
};


/**
 * @return {number|undefined} Returns the document mode (for testing).
 * @private
 */
goog.userAgent.getDocumentMode_ = function() {
  // NOTE(pupius): goog.userAgent may be used in context where there is no DOM.
  var doc = goog.global['document'];
  return doc ? doc['documentMode'] : undefined;
};


/**
 * The version of the user agent. This is a string because it might contain
 * 'b' (as in beta) as well as multiple dots.
 * @type {string}
 */
goog.userAgent.VERSION = goog.userAgent.determineVersion_();


/**
 * Compares two version numbers.
 *
 * @param {string} v1 Version of first item.
 * @param {string} v2 Version of second item.
 *
 * @return {number}  1 if first argument is higher
 *                   0 if arguments are equal
 *                  -1 if second argument is higher.
 * @deprecated Use goog.string.compareVersions.
 */
goog.userAgent.compare = function(v1, v2) {
  return goog.string.compareVersions(v1, v2);
};


/**
 * Cache for {@link goog.userAgent.isVersionOrHigher}.
 * Calls to compareVersions are surprisingly expensive and, as a browser's
 * version number is unlikely to change during a session, we cache the results.
 * @const
 * @private
 */
goog.userAgent.isVersionOrHigherCache_ = {};


/**
 * Whether the user agent version is higher or the same as the given version.
 * NOTE: When checking the version numbers for Firefox and Safari, be sure to
 * use the engine's version, not the browser's version number.  For example,
 * Firefox 3.0 corresponds to Gecko 1.9 and Safari 3.0 to Webkit 522.11.
 * Opera and Internet Explorer versions match the product release number.<br>
 * @see <a href="http://en.wikipedia.org/wiki/Safari_version_history">
 *     Webkit</a>
 * @see <a href="http://en.wikipedia.org/wiki/Gecko_engine">Gecko</a>
 *
 * @param {string|number} version The version to check.
 * @return {boolean} Whether the user agent version is higher or the same as
 *     the given version.
 */
goog.userAgent.isVersionOrHigher = function(version) {
  return goog.userAgent.ASSUME_ANY_VERSION ||
      goog.reflect.cache(
          goog.userAgent.isVersionOrHigherCache_, version, function() {
            return goog.string.compareVersions(
                       goog.userAgent.VERSION, version) >= 0;
          });
};


/**
 * Deprecated alias to `goog.userAgent.isVersionOrHigher`.
 * @param {string|number} version The version to check.
 * @return {boolean} Whether the user agent version is higher or the same as
 *     the given version.
 * @deprecated Use goog.userAgent.isVersionOrHigher().
 */
goog.userAgent.isVersion = goog.userAgent.isVersionOrHigher;


/**
 * Whether the IE effective document mode is higher or the same as the given
 * document mode version.
 * NOTE: Only for IE, return false for another browser.
 *
 * @param {number} documentMode The document mode version to check.
 * @return {boolean} Whether the IE effective document mode is higher or the
 *     same as the given version.
 */
goog.userAgent.isDocumentModeOrHigher = function(documentMode) {
  return Number(goog.userAgent.DOCUMENT_MODE) >= documentMode;
};


/**
 * Deprecated alias to `goog.userAgent.isDocumentModeOrHigher`.
 * @param {number} version The version to check.
 * @return {boolean} Whether the IE effective document mode is higher or the
 *      same as the given version.
 * @deprecated Use goog.userAgent.isDocumentModeOrHigher().
 */
goog.userAgent.isDocumentMode = goog.userAgent.isDocumentModeOrHigher;


/**
 * For IE version < 7, documentMode is undefined, so attempt to use the
 * CSS1Compat property to see if we are in standards mode. If we are in
 * standards mode, treat the browser version as the document mode. Otherwise,
 * IE is emulating version 5.
 *
 * NOTE(2019/05/31): Support for IE < 7 is long gone, so this is now simplified.
 * It returns document.documentMode for IE and undefined for everything else.
 *
 * @type {number|undefined}
 * @const
 */
goog.userAgent.DOCUMENT_MODE = (function() {
  var doc = goog.global['document'];
  if (!doc || !goog.userAgent.IE) return undefined;
  // This must be an IE user agent.
  var documentMode = goog.userAgent.getDocumentMode_();
  if (documentMode) return documentMode;
  // The user agent version string begins with the major version.
  // Parse the major version and truncate anything following.
  var ieVersion = parseInt(goog.userAgent.VERSION, 10);
  return ieVersion || undefined;
})();

//third_party/javascript/closure/useragent/product.js
/**
 * @license
 * Copyright The Closure Library Authors.
 * SPDX-License-Identifier: Apache-2.0
 */

/**
 * @fileoverview Detects the specific browser and not just the rendering engine.
 */

goog.provide('goog.userAgent.product');

goog.require('goog.labs.userAgent.browser');
goog.require('goog.labs.userAgent.platform');
goog.require('goog.userAgent');


/**
 * @define {boolean} Whether the code is running on the Firefox web browser.
 */
goog.userAgent.product.ASSUME_FIREFOX =
    goog.define('goog.userAgent.product.ASSUME_FIREFOX', false);


/**
 * @define {boolean} Whether we know at compile-time that the product is an
 *     iPhone.
 */
goog.userAgent.product.ASSUME_IPHONE =
    goog.define('goog.userAgent.product.ASSUME_IPHONE', false);


/**
 * @define {boolean} Whether we know at compile-time that the product is an
 *     iPad.
 */
goog.userAgent.product.ASSUME_IPAD =
    goog.define('goog.userAgent.product.ASSUME_IPAD', false);


/**
 * @define {boolean} Whether we know at compile-time that the product is an
 *     AOSP browser or WebView inside a pre KitKat Android phone or tablet.
 */
goog.userAgent.product.ASSUME_ANDROID =
    goog.define('goog.userAgent.product.ASSUME_ANDROID', false);


/**
 * @define {boolean} Whether the code is running on the Chrome web browser on
 * any platform or AOSP browser or WebView in a KitKat+ Android phone or tablet.
 */
goog.userAgent.product.ASSUME_CHROME =
    goog.define('goog.userAgent.product.ASSUME_CHROME', false);


/**
 * @define {boolean} Whether the code is running on the Safari web browser.
 */
goog.userAgent.product.ASSUME_SAFARI =
    goog.define('goog.userAgent.product.ASSUME_SAFARI', false);


/**
 * Whether we know the product type at compile-time.
 * @type {boolean}
 * @private
 */
goog.userAgent.product.PRODUCT_KNOWN_ = goog.userAgent.ASSUME_IE ||
    goog.userAgent.ASSUME_EDGE || goog.userAgent.ASSUME_OPERA ||
    goog.userAgent.product.ASSUME_FIREFOX ||
    goog.userAgent.product.ASSUME_IPHONE ||
    goog.userAgent.product.ASSUME_IPAD ||
    goog.userAgent.product.ASSUME_ANDROID ||
    goog.userAgent.product.ASSUME_CHROME ||
    goog.userAgent.product.ASSUME_SAFARI;


/**
 * Whether the code is running on the Opera web browser.
 * @type {boolean}
 */
goog.userAgent.product.OPERA = goog.userAgent.OPERA;


/**
 * Whether the code is running on an IE web browser.
 * @type {boolean}
 */
goog.userAgent.product.IE = goog.userAgent.IE;


/**
 * Whether the code is running on an Edge web browser (EdgeHTML based).
 * @type {boolean}
 */
goog.userAgent.product.EDGE = goog.userAgent.EDGE;


/**
 * Whether the code is running on the Firefox web browser.
 * @type {boolean}
 */
goog.userAgent.product.FIREFOX = goog.userAgent.product.PRODUCT_KNOWN_ ?
    goog.userAgent.product.ASSUME_FIREFOX :
    goog.labs.userAgent.browser.isFirefox();


/**
 * Whether the user agent is an iPhone or iPod (as in iPod touch).
 * @return {boolean}
 * @private
 */
goog.userAgent.product.isIphoneOrIpod_ = function() {
  return goog.labs.userAgent.platform.isIphone() ||
      goog.labs.userAgent.platform.isIpod();
};


/**
 * Whether the code is running on an iPhone or iPod touch.
 *
 * iPod touch is considered an iPhone for legacy reasons.
 * @type {boolean}
 */
goog.userAgent.product.IPHONE = goog.userAgent.product.PRODUCT_KNOWN_ ?
    goog.userAgent.product.ASSUME_IPHONE :
    goog.userAgent.product.isIphoneOrIpod_();


/**
 * Whether the code is running on an iPad.
 * @type {boolean}
 */
goog.userAgent.product.IPAD = goog.userAgent.product.PRODUCT_KNOWN_ ?
    goog.userAgent.product.ASSUME_IPAD :
    goog.labs.userAgent.platform.isIpad();


/**
 * Whether the code is running on AOSP browser or WebView inside
 * a pre KitKat Android phone or tablet.
 * @type {boolean}
 */
goog.userAgent.product.ANDROID = goog.userAgent.product.PRODUCT_KNOWN_ ?
    goog.userAgent.product.ASSUME_ANDROID :
    goog.labs.userAgent.browser.isAndroidBrowser();


/**
 * Whether the code is running on any Chromium-based web browser on any platform
 * or AOSP browser or WebView in a KitKat+ Android phone or tablet.
 * @type {boolean}
 */
goog.userAgent.product.CHROME = goog.userAgent.product.PRODUCT_KNOWN_ ?
    goog.userAgent.product.ASSUME_CHROME :
    goog.labs.userAgent.browser.isChrome();


/**
 * @return {boolean} Whether the browser is Safari on desktop.
 * @private
 */
goog.userAgent.product.isSafariDesktop_ = function() {
  return goog.labs.userAgent.browser.isSafari() &&
      !goog.labs.userAgent.platform.isIos();
};


/**
 * Whether the code is running on the desktop Safari web browser.
 * Note: the legacy behavior here is only true for Safari not running
 * on iOS.
 * @type {boolean}
 */
goog.userAgent.product.SAFARI = goog.userAgent.product.PRODUCT_KNOWN_ ?
    goog.userAgent.product.ASSUME_SAFARI :
    goog.userAgent.product.isSafariDesktop_();

//third_party/javascript/closure/crypt/base64.js
/**
 * @license
 * Copyright The Closure Library Authors.
 * SPDX-License-Identifier: Apache-2.0
 */

/**
 * @fileoverview Base64 en/decoding. Not much to say here except that we
 * work with decoded values in arrays of bytes. By "byte" I mean a number
 * in [0, 255].
 */

goog.provide('goog.crypt.base64');

goog.require('goog.asserts');
goog.require('goog.crypt');
goog.require('goog.string');
goog.require('goog.userAgent');
goog.require('goog.userAgent.product');

/**
 * Default alphabet, shared between alphabets. Only 62 characters.
 * @private {string}
 */
goog.crypt.base64.DEFAULT_ALPHABET_COMMON_ = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ' +
    'abcdefghijklmnopqrstuvwxyz' +
    '0123456789';


/**
 * Alphabet characters for Alphabet.DEFAULT encoding.
 * For characters without padding, please consider using
 * `goog.crypt.baseN.BASE_64` instead.
 *
 * @type {string}
 */
goog.crypt.base64.ENCODED_VALS =
    goog.crypt.base64.DEFAULT_ALPHABET_COMMON_ + '+/=';


/**
 * Alphabet characters for Alphabet.WEBSAFE_DOT_PADDING encoding.
 * The dot padding is no Internet Standard, according to RFC 4686.
 * https://tools.ietf.org/html/rfc4648
 * For characters without padding, please consider using
 * `goog.crypt.baseN.BASE_64_URL_SAFE` instead.
 *
 * @type {string}
 */
goog.crypt.base64.ENCODED_VALS_WEBSAFE =
    goog.crypt.base64.DEFAULT_ALPHABET_COMMON_ + '-_.';


/**
 * Alphabets for Base64 encoding
 * Alphabets with no padding character are for encoding without padding.
 * About the alphabets, please refer to RFC 4686.
 * https://tools.ietf.org/html/rfc4648
 * @enum {number}
 */
goog.crypt.base64.Alphabet = {
  DEFAULT: 0,
  NO_PADDING: 1,
  WEBSAFE: 2,
  WEBSAFE_DOT_PADDING: 3,
  WEBSAFE_NO_PADDING: 4,
};


/**
 * Padding chars for Base64 encoding
 * @const {string}
 * @private
 */
goog.crypt.base64.paddingChars_ = '=.';


/**
 * Check if a character is a padding character
 *
 * @param {string} char
 * @return {boolean}
 * @private
 */
goog.crypt.base64.isPadding_ = function(char) {
  return goog.string.contains(goog.crypt.base64.paddingChars_, char);
};


// Static lookup maps, lazily populated by init_()

/**
 * For each `Alphabet`, maps from bytes to characters.
 *
 * @see https://jsperf.com/char-lookups
 * @type {!Object<!goog.crypt.base64.Alphabet, !Array<string>>}
 * @private
 */
goog.crypt.base64.byteToCharMaps_ = {};

/**
 * Maps characters to bytes.
 *
 * This map is used for all alphabets since, across alphabets, common chars
 * always map to the same byte.
 *
 * `null` indicates `init` has not yet been called.
 *
 * @type {?Object<string, number>}
 * @private
 */
goog.crypt.base64.charToByteMap_ = null;


/**
 * White list of implementations with known-good native atob and btoa functions.
 * Listing these explicitly (via the ASSUME_* wrappers) benefits dead-code
 * removal in per-browser compilations.
 * @private {boolean}
 */
goog.crypt.base64.ASSUME_NATIVE_SUPPORT_ = goog.userAgent.GECKO ||
    (goog.userAgent.WEBKIT && !goog.userAgent.product.SAFARI) ||
    goog.userAgent.OPERA;


/**
 * Does this browser have a working btoa function?
 * @private {boolean}
 */
goog.crypt.base64.HAS_NATIVE_ENCODE_ =
    goog.crypt.base64.ASSUME_NATIVE_SUPPORT_ ||
    typeof(goog.global.btoa) == 'function';


/**
 * Does this browser have a working atob function?
 * We blacklist known-bad implementations:
 *  - IE (10+) added atob() but it does not tolerate whitespace on the input.
 * @private {boolean}
 */
goog.crypt.base64.HAS_NATIVE_DECODE_ =
    goog.crypt.base64.ASSUME_NATIVE_SUPPORT_ ||
    (!goog.userAgent.product.SAFARI && !goog.userAgent.IE &&
     typeof(goog.global.atob) == 'function');


/**
 * Base64-encode an array of bytes.
 *
 * @param {Array<number>|Uint8Array} input An array of bytes (numbers with
 *     value in [0, 255]) to encode.
 * @param {!goog.crypt.base64.Alphabet=} alphabet Base 64 alphabet to
 *     use in encoding. Alphabet.DEFAULT is used by default.
 * @return {string} The base64 encoded string.
 */
goog.crypt.base64.encodeByteArray = function(input, alphabet) {
  // Assert avoids runtime dependency on goog.isArrayLike, which helps reduce
  // size of jscompiler output, and which yields slight performance increase.
  goog.asserts.assert(
      goog.isArrayLike(input), 'encodeByteArray takes an array as a parameter');

  if (alphabet === undefined) {
    alphabet = goog.crypt.base64.Alphabet.DEFAULT;
  }

  goog.crypt.base64.init_();

  var byteToCharMap = goog.crypt.base64.byteToCharMaps_[alphabet];

  var output = [];

  for (var i = 0; i < input.length; i += 3) {
    var byte1 = input[i];
    var haveByte2 = i + 1 < input.length;
    var byte2 = haveByte2 ? input[i + 1] : 0;
    var haveByte3 = i + 2 < input.length;
    var byte3 = haveByte3 ? input[i + 2] : 0;

    var outByte1 = byte1 >> 2;
    var outByte2 = ((byte1 & 0x03) << 4) | (byte2 >> 4);
    var outByte3 = ((byte2 & 0x0F) << 2) | (byte3 >> 6);
    var outByte4 = byte3 & 0x3F;

    if (!haveByte3) {
      outByte4 = 64;

      if (!haveByte2) {
        outByte3 = 64;
      }
    }

    output.push(
        byteToCharMap[outByte1], byteToCharMap[outByte2],
        byteToCharMap[outByte3] || '', byteToCharMap[outByte4] || '');
  }

  return output.join('');
};


/**
 * Base64-encode a string.
 *
 * @param {string} input A string to encode.
 * @param {!goog.crypt.base64.Alphabet=} alphabet Base 64 alphabet to
 *     use in encoding. Alphabet.DEFAULT is used by default.
 * @return {string} The base64 encoded string.
 */
goog.crypt.base64.encodeString = function(input, alphabet) {
  // Shortcut for browsers that implement
  // a native base64 encoder in the form of "btoa/atob"
  if (goog.crypt.base64.HAS_NATIVE_ENCODE_ && !alphabet) {
    return goog.global.btoa(input);
  }
  return goog.crypt.base64.encodeByteArray(
      goog.crypt.stringToByteArray(input), alphabet);
};


/**
 * Base64-decode a string.
 *
 * @param {string} input Input to decode. Any whitespace is ignored, and the
 *     input maybe encoded with either supported alphabet (or a mix thereof).
 * @param {boolean=} useCustomDecoder True indicates the custom decoder is used,
 *     which supports alternative alphabets. Note that passing false may still
 *     use the custom decoder on browsers without native support.
 * @return {string} string representing the decoded value.
 */
goog.crypt.base64.decodeString = function(input, useCustomDecoder) {
  // Shortcut for browsers that implement
  // a native base64 encoder in the form of "btoa/atob"
  if (goog.crypt.base64.HAS_NATIVE_DECODE_ && !useCustomDecoder) {
    return goog.global.atob(input);
  }
  var output = '';
  function pushByte(b) {
    output += String.fromCharCode(b);
  }

  goog.crypt.base64.decodeStringInternal_(input, pushByte);

  return output;
};


/**
 * Base64-decode a string to an Array of numbers.
 *
 * In base-64 decoding, groups of four characters are converted into three
 * bytes.  If the encoder did not apply padding, the input length may not
 * be a multiple of 4.
 *
 * In this case, the last group will have fewer than 4 characters, and
 * padding will be inferred.  If the group has one or two characters, it decodes
 * to one byte.  If the group has three characters, it decodes to two bytes.
 *
 * @param {string} input Input to decode. Any whitespace is ignored, and the
 *     input maybe encoded with either supported alphabet (or a mix thereof).
 * @param {boolean=} opt_ignored Unused parameter, retained for compatibility.
 * @return {!Array<number>} bytes representing the decoded value.
 */
goog.crypt.base64.decodeStringToByteArray = function(input, opt_ignored) {
  var output = [];
  function pushByte(b) { output.push(b); }

  goog.crypt.base64.decodeStringInternal_(input, pushByte);

  return output;
};


/**
 * Base64-decode a string to a Uint8Array.
 *
 * Note that Uint8Array is not supported on older browsers, e.g. IE < 10.
 * @see http://caniuse.com/uint8array
 *
 * In base-64 decoding, groups of four characters are converted into three
 * bytes.  If the encoder did not apply padding, the input length may not
 * be a multiple of 4.
 *
 * In this case, the last group will have fewer than 4 characters, and
 * padding will be inferred.  If the group has one or two characters, it decodes
 * to one byte.  If the group has three characters, it decodes to two bytes.
 *
 * @param {string} input Input to decode. Any whitespace is ignored, and the
 *     input maybe encoded with either supported alphabet (or a mix thereof).
 * @return {!Uint8Array} bytes representing the decoded value.
 */
goog.crypt.base64.decodeStringToUint8Array = function(input) {
  goog.asserts.assert(
      !goog.userAgent.IE || goog.userAgent.isVersionOrHigher('10'),
      'Browser does not support typed arrays');
  var len = input.length;
  // Approximate the length of the array needed for output.
  // Our method varies according to the format of the input, which we can
  // consider in three categories:
  //   A) well-formed with proper padding
  //   B) well-formed without any padding
  //   C) not-well-formed, either with extra whitespace in the middle or with
  //      extra padding characters.
  //
  //  In the case of (A), (length * 3 / 4) will result in an integer number of
  //  bytes evenly divisible by 3, and we need only subtract bytes according to
  //  the padding observed.
  //
  //  In the case of (B), (length * 3 / 4) will result in a non-integer number
  //  of bytes, or not evenly divisible by 3. (If the result is evenly divisible
  //  by 3, it's well-formed with the proper amount of padding [0 padding]).
  //  This approximation can become exact by rounding down.
  //
  //  In the case of (C), the only way to get the length is to walk the full
  //  length of the string to consider each character. This is handled by
  //  tracking the number of bytes added to the array and using subarray to
  //  trim the array back down to size.
  var approxByteLength = len * 3 / 4;
  if (approxByteLength % 3) {
    // The string isn't complete, either because it didn't include padding, or
    // because it has extra white space.
    // In either case, we won't generate more bytes than are completely encoded,
    // so rounding down is appropriate to have a buffer at least as large as
    // output.
    approxByteLength = Math.floor(approxByteLength);
  } else if (goog.crypt.base64.isPadding_(input[len - 1])) {
    // The string has a round length, and has some padding.
    // Reduce the byte length according to the quantity of padding.
    if (goog.crypt.base64.isPadding_(input[len - 2])) {
      approxByteLength -= 2;
    } else {
      approxByteLength -= 1;
    }
  }
  var output = new Uint8Array(approxByteLength);
  var outLen = 0;
  function pushByte(b) {
    output[outLen++] = b;
  }

  goog.crypt.base64.decodeStringInternal_(input, pushByte);

  // Return a subarray to handle the case that input included extra whitespace
  // or extra padding and approxByteLength was incorrect.
  return output.subarray(0, outLen);
};


/**
 * @param {string} input Input to decode.
 * @param {function(number):void} pushByte result accumulator.
 * @private
 */
goog.crypt.base64.decodeStringInternal_ = function(input, pushByte) {
  goog.crypt.base64.init_();

  var nextCharIndex = 0;
  /**
   * @param {number} default_val Used for end-of-input.
   * @return {number} The next 6-bit value, or the default for end-of-input.
   */
  function getByte(default_val) {
    while (nextCharIndex < input.length) {
      var ch = input.charAt(nextCharIndex++);
      var b = goog.crypt.base64.charToByteMap_[ch];
      if (b != null) {
        return b;  // Common case: decoded the char.
      }
      if (!goog.string.isEmptyOrWhitespace(ch)) {
        throw new Error('Unknown base64 encoding at char: ' + ch);
      }
      // We encountered whitespace: loop around to the next input char.
    }
    return default_val;  // No more input remaining.
  }

  while (true) {
    var byte1 = getByte(-1);
    var byte2 = getByte(0);
    var byte3 = getByte(64);
    var byte4 = getByte(64);

    // The common case is that all four bytes are present, so if we have byte4
    // we can skip over the truncated input special case handling.
    if (byte4 === 64) {
      if (byte1 === -1) {
        return;  // Terminal case: no input left to decode.
      }
      // Here we know an intermediate number of bytes are missing.
      // The defaults for byte2, byte3 and byte4 apply the inferred padding
      // rules per the public API documentation. i.e: 1 byte
      // missing should yield 2 bytes of output, but 2 or 3 missing bytes yield
      // a single byte of output. (Recall that 64 corresponds the padding char).
    }

    var outByte1 = (byte1 << 2) | (byte2 >> 4);
    pushByte(outByte1);

    if (byte3 != 64) {
      var outByte2 = ((byte2 << 4) & 0xF0) | (byte3 >> 2);
      pushByte(outByte2);

      if (byte4 != 64) {
        var outByte3 = ((byte3 << 6) & 0xC0) | byte4;
        pushByte(outByte3);
      }
    }
  }
};


/**
 * Lazy static initialization function. Called before
 * accessing any of the static map variables.
 * @private
 */
goog.crypt.base64.init_ = function() {
  if (goog.crypt.base64.charToByteMap_) {
    return;
  }
  goog.crypt.base64.charToByteMap_ = {};

  // We want quick mappings back and forth, so we precompute encoding maps.

  /** @type {!Array<string>} */
  var commonChars = goog.crypt.base64.DEFAULT_ALPHABET_COMMON_.split('');
  var specialChars = [
    '+/=',  // DEFAULT
    '+/',   // NO_PADDING
    '-_=',  // WEBSAFE
    '-_.',  // WEBSAFE_DOT_PADDING
    '-_',   // WEBSAFE_NO_PADDING
  ];

  for (var i = 0; i < 5; i++) {
    // `i` is each value of the `goog.crypt.base64.Alphabet` enum
    var chars = commonChars.concat(specialChars[i].split(''));

    // Sets byte-to-char map
    goog.crypt.base64
        .byteToCharMaps_[/** @type {!goog.crypt.base64.Alphabet} */ (i)] =
        chars;

    // Sets char-to-byte map
    for (var j = 0; j < chars.length; j++) {
      var char = chars[j];

      var existingByte = goog.crypt.base64.charToByteMap_[char];
      if (existingByte === undefined) {
        goog.crypt.base64.charToByteMap_[char] = j;
      } else {
        goog.asserts.assert(existingByte === j);
      }
    }
  }
};

//javascript/apps/jspb/binary/utils.js
/**
 * @fileoverview This file contains helper code used by jspb.BinaryReader
 * and BinaryWriter.
 *
 * @author aappleby@google.com (Austin Appleby)
 */

goog.provide('jspb.utils');

goog.require('goog.asserts');
goog.require('goog.crypt');
goog.require('goog.crypt.base64');
goog.require('goog.string');
goog.require('jspb.BinaryConstants');


/**
 * Javascript can't natively handle 64-bit data types, so to manipulate them we
 * have to split them into two 32-bit halves and do the math manually.
 *
 * Instead of instantiating and passing small structures around to do this, we
 * instead just use two global temporary values. This one stores the low 32
 * bits of a split value - for example, if the original value was a 64-bit
 * integer, this temporary value will contain the low 32 bits of that integer.
 * If the original value was a double, this temporary value will contain the
 * low 32 bits of the binary representation of that double, etcetera.
 * @type {number}
 */
jspb.utils.split64Low = 0;


/**
 * And correspondingly, this temporary variable will contain the high 32 bits
 * of whatever value was split.
 * @type {number}
 */
jspb.utils.split64High = 0;


/**
 * Splits an unsigned Javascript integer into two 32-bit halves and stores it
 * in the temp values above.
 * @param {number} value The number to split.
 */
jspb.utils.splitUint64 = function(value) {
  // Extract low 32 bits and high 32 bits as unsigned integers.
  var lowBits = value >>> 0;
  var highBits = Math.floor((value - lowBits) /
                            jspb.BinaryConstants.TWO_TO_32) >>> 0;

  jspb.utils.split64Low = lowBits;
  jspb.utils.split64High = highBits;
};


/**
 * Splits a signed Javascript integer into two 32-bit halves and stores it in
 * the temp values above.
 * @param {number} value The number to split.
 */
jspb.utils.splitInt64 = function(value) {
  // Convert to sign-magnitude representation.
  var sign = (value < 0);
  value = Math.abs(value);

  // Extract low 32 bits and high 32 bits as unsigned integers.
  var lowBits = value >>> 0;
  var highBits = Math.floor((value - lowBits) /
                            jspb.BinaryConstants.TWO_TO_32);
  highBits = highBits >>> 0;

  // Perform two's complement conversion if the sign bit was set.
  if (sign) {
    highBits = ~highBits >>> 0;
    lowBits = ~lowBits >>> 0;
    lowBits += 1;
    if (lowBits > 0xFFFFFFFF) {
      lowBits = 0;
      highBits++;
      if (highBits > 0xFFFFFFFF) highBits = 0;
    }
  }

  jspb.utils.split64Low = lowBits;
  jspb.utils.split64High = highBits;
};


/**
 * Convers a signed Javascript integer into zigzag format, splits it into two
 * 32-bit halves, and stores it in the temp values above.
 * @param {number} value The number to split.
 */
jspb.utils.splitZigzag64 = function(value) {
  // Convert to sign-magnitude and scale by 2 before we split the value.
  var sign = (value < 0);
  value = Math.abs(value) * 2;

  jspb.utils.splitUint64(value);
  var lowBits = jspb.utils.split64Low;
  var highBits = jspb.utils.split64High;

  // If the value is negative, subtract 1 from the split representation so we
  // don't lose the sign bit due to precision issues.
  if (sign) {
    if (lowBits == 0) {
      if (highBits == 0) {
        lowBits = 0xFFFFFFFF;
        highBits = 0xFFFFFFFF;
      } else {
        highBits--;
        lowBits = 0xFFFFFFFF;
      }
    } else {
      lowBits--;
    }
  }

  jspb.utils.split64Low = lowBits;
  jspb.utils.split64High = highBits;
};


/**
 * Converts a floating-point number into 32-bit IEEE representation and stores
 * it in the temp values above.
 * @param {number} value
 */
jspb.utils.splitFloat32 = function(value) {
  var sign = (value < 0) ? 1 : 0;
  value = sign ? -value : value;
  var exp;
  var mant;

  // Handle zeros.
  if (value === 0) {
    if ((1 / value) > 0) {
      // Positive zero.
      jspb.utils.split64High = 0;
      jspb.utils.split64Low = 0x00000000;
    } else {
      // Negative zero.
      jspb.utils.split64High = 0;
      jspb.utils.split64Low = 0x80000000;
    }
    return;
  }

  // Handle nans.
  if (isNaN(value)) {
    jspb.utils.split64High = 0;
    jspb.utils.split64Low = 0x7FFFFFFF;
    return;
  }

  // Handle infinities.
  if (value > jspb.BinaryConstants.FLOAT32_MAX) {
    jspb.utils.split64High = 0;
    jspb.utils.split64Low = ((sign << 31) | (0x7F800000)) >>> 0;
    return;
  }

  // Handle denormals.
  if (value < jspb.BinaryConstants.FLOAT32_MIN) {
    // Number is a denormal.
    mant = Math.round(value / Math.pow(2, -149));
    jspb.utils.split64High = 0;
    jspb.utils.split64Low = ((sign << 31) | mant) >>> 0;
    return;
  }

  exp = Math.floor(Math.log(value) / Math.LN2);
  mant = value * Math.pow(2, -exp);
  mant = Math.round(mant * jspb.BinaryConstants.TWO_TO_23) & 0x7FFFFF;

  jspb.utils.split64High = 0;
  jspb.utils.split64Low = ((sign << 31) | ((exp + 127) << 23) | mant) >>> 0;
};


/**
 * Converts a floating-point number into 64-bit IEEE representation and stores
 * it in the temp values above.
 * @param {number} value
 */
jspb.utils.splitFloat64 = function(value) {
  var sign = (value < 0) ? 1 : 0;
  value = sign ? -value : value;

  // Handle zeros.
  if (value === 0) {
    if ((1 / value) > 0) {
      // Positive zero.
      jspb.utils.split64High = 0x00000000;
      jspb.utils.split64Low = 0x00000000;
    } else {
      // Negative zero.
      jspb.utils.split64High = 0x80000000;
      jspb.utils.split64Low = 0x00000000;
    }
    return;
  }

  // Handle nans.
  if (isNaN(value)) {
    jspb.utils.split64High = 0x7FFFFFFF;
    jspb.utils.split64Low = 0xFFFFFFFF;
    return;
  }

  // Handle infinities.
  if (value > jspb.BinaryConstants.FLOAT64_MAX) {
    jspb.utils.split64High = ((sign << 31) | (0x7FF00000)) >>> 0;
    jspb.utils.split64Low = 0;
    return;
  }

  // Handle denormals.
  if (value < jspb.BinaryConstants.FLOAT64_MIN) {
    // Number is a denormal.
    var mant = value / Math.pow(2, -1074);
    var mantHigh = (mant / jspb.BinaryConstants.TWO_TO_32);
    jspb.utils.split64High = ((sign << 31) | mantHigh) >>> 0;
    jspb.utils.split64Low = (mant >>> 0);
    return;
  }

  // Compute the least significant exponent needed to represent the magnitude of
  // the value by repeadly dividing/multiplying by 2 until the magnitude
  // crosses 2. While tempting to use log math to find the exponent, at the
  // bounadaries of precision, the result can be off by one.
  var maxDoubleExponent = 1023;
  var minDoubleExponent = -1022;
  var x = value;
  var exp = 0;
  if (x >= 2) {
    while (x >= 2 && exp < maxDoubleExponent) {
      exp++;
      x = x / 2;
    }
  } else {
    while (x < 1 && exp > minDoubleExponent) {
      x = x * 2;
      exp--;
    }
  }
  var mant = value * Math.pow(2, -exp);

  var mantHigh = (mant * jspb.BinaryConstants.TWO_TO_20) & 0xFFFFF;
  var mantLow = (mant * jspb.BinaryConstants.TWO_TO_52) >>> 0;

  jspb.utils.split64High =
      ((sign << 31) | ((exp + 1023) << 20) | mantHigh) >>> 0;
  jspb.utils.split64Low = mantLow;
};


/**
 * Converts an 8-character hash string into two 32-bit numbers and stores them
 * in the temp values above.
 * @param {string} hash
 */
jspb.utils.splitHash64 = function(hash) {
  var a = hash.charCodeAt(0);
  var b = hash.charCodeAt(1);
  var c = hash.charCodeAt(2);
  var d = hash.charCodeAt(3);
  var e = hash.charCodeAt(4);
  var f = hash.charCodeAt(5);
  var g = hash.charCodeAt(6);
  var h = hash.charCodeAt(7);

  jspb.utils.split64Low = (a + (b << 8) + (c << 16) + (d << 24)) >>> 0;
  jspb.utils.split64High = (e + (f << 8) + (g << 16) + (h << 24)) >>> 0;
};


/**
 * Joins two 32-bit values into a 64-bit unsigned integer. Precision will be
 * lost if the result is greater than 2^52.
 * @param {number} bitsLow
 * @param {number} bitsHigh
 * @return {number}
 */
jspb.utils.joinUint64 = function(bitsLow, bitsHigh) {
  return bitsHigh * jspb.BinaryConstants.TWO_TO_32 + (bitsLow >>> 0);
};


/**
 * Joins two 32-bit values into a 64-bit signed integer. Precision will be lost
 * if the result is greater than 2^52.
 * @param {number} bitsLow
 * @param {number} bitsHigh
 * @return {number}
 */
jspb.utils.joinInt64 = function(bitsLow, bitsHigh) {
  // If the high bit is set, do a manual two's complement conversion.
  var sign = (bitsHigh & 0x80000000);
  if (sign) {
    bitsLow = (~bitsLow + 1) >>> 0;
    bitsHigh = ~bitsHigh >>> 0;
    if (bitsLow == 0) {
      bitsHigh = (bitsHigh + 1) >>> 0;
    }
  }

  var result = jspb.utils.joinUint64(bitsLow, bitsHigh);
  return sign ? -result : result;
};

/**
 * Converts split 64-bit values from standard two's complement encoding to
 * zig-zag encoding. Invokes the provided function to produce final result.
 *
 * @param {number} bitsLow
 * @param {number} bitsHigh
 * @param {function(number, number): T} convert Conversion function to produce
 *     the result value, takes parameters (lowBits, highBits).
 * @return {T}
 * @template T
 */
jspb.utils.toZigzag64 = function(bitsLow, bitsHigh, convert) {
  // See
  // https://engdoc.corp.google.com/eng/howto/protocolbuffers/developerguide/encoding.shtml?cl=head#types
  // 64-bit math is: (n << 1) ^ (n >> 63)
  //
  // To do this in 32 bits, we can get a 32-bit sign-flipping mask from the
  // high word.
  // Then we can operate on each word individually, with the addition of the
  // "carry" to get the most significant bit from the low word into the high
  // word.
  var signFlipMask = bitsHigh >> 31;
  bitsHigh = (bitsHigh << 1 | bitsLow >>> 31) ^ signFlipMask;
  bitsLow = (bitsLow << 1) ^ signFlipMask;
  return convert(bitsLow, bitsHigh);
};


/**
 * Joins two 32-bit values into a 64-bit unsigned integer and applies zigzag
 * decoding. Precision will be lost if the result is greater than 2^52.
 * @param {number} bitsLow
 * @param {number} bitsHigh
 * @return {number}
 */
jspb.utils.joinZigzag64 = function(bitsLow, bitsHigh) {
  return jspb.utils.fromZigzag64(bitsLow, bitsHigh, jspb.utils.joinInt64);
};


/**
 * Converts split 64-bit values from zigzag encoding to standard two's
 * complement encoding. Invokes the provided function to produce final result.
 *
 * @param {number} bitsLow
 * @param {number} bitsHigh
 * @param {function(number, number): T} convert Conversion function to produce
 *     the result value, takes parameters (lowBits, highBits).
 * @return {T}
 * @template T
 */
jspb.utils.fromZigzag64 = function(bitsLow, bitsHigh, convert) {
  // 64 bit math is:
  //   signmask = (zigzag & 1) ? -1 : 0;
  //   twosComplement = (zigzag >> 1) ^ signmask;
  //
  // To work with 32 bit, we can operate on both but "carry" the lowest bit
  // from the high word by shifting it up 31 bits to be the most significant bit
  // of the low word.
  var signFlipMask = -(bitsLow & 1);
  bitsLow = ((bitsLow >>> 1) | (bitsHigh << 31)) ^ signFlipMask;
  bitsHigh = (bitsHigh >>> 1) ^ signFlipMask;
  return convert(bitsLow, bitsHigh);
};


/**
 * Joins two 32-bit values into a 32-bit IEEE floating point number and
 * converts it back into a Javascript number.
 * @param {number} bitsLow The low 32 bits of the binary number;
 * @param {number} bitsHigh The high 32 bits of the binary number.
 * @return {number}
 */
jspb.utils.joinFloat32 = function(bitsLow, bitsHigh) {
  var sign = ((bitsLow >> 31) * 2 + 1);
  var exp = (bitsLow >>> 23) & 0xFF;
  var mant = bitsLow & 0x7FFFFF;

  if (exp == 0xFF) {
    if (mant) {
      return NaN;
    } else {
      return sign * Infinity;
    }
  }

  if (exp == 0) {
    // Denormal.
    return sign * Math.pow(2, -149) * mant;
  } else {
    return sign * Math.pow(2, exp - 150) *
           (mant + Math.pow(2, 23));
  }
};


/**
 * Joins two 32-bit values into a 64-bit IEEE floating point number and
 * converts it back into a Javascript number.
 * @param {number} bitsLow The low 32 bits of the binary number;
 * @param {number} bitsHigh The high 32 bits of the binary number.
 * @return {number}
 */
jspb.utils.joinFloat64 = function(bitsLow, bitsHigh) {
  var sign = ((bitsHigh >> 31) * 2 + 1);
  var exp = (bitsHigh >>> 20) & 0x7FF;
  var mant = jspb.BinaryConstants.TWO_TO_32 * (bitsHigh & 0xFFFFF) + bitsLow;

  if (exp == 0x7FF) {
    if (mant) {
      return NaN;
    } else {
      return sign * Infinity;
    }
  }

  if (exp == 0) {
    // Denormal.
    return sign * Math.pow(2, -1074) * mant;
  } else {
    return sign * Math.pow(2, exp - 1075) *
           (mant + jspb.BinaryConstants.TWO_TO_52);
  }
};


/**
 * Joins two 32-bit values into an 8-character hash string.
 * @param {number} bitsLow
 * @param {number} bitsHigh
 * @return {string}
 */
jspb.utils.joinHash64 = function(bitsLow, bitsHigh) {
  var a = (bitsLow >>> 0) & 0xFF;
  var b = (bitsLow >>> 8) & 0xFF;
  var c = (bitsLow >>> 16) & 0xFF;
  var d = (bitsLow >>> 24) & 0xFF;
  var e = (bitsHigh >>> 0) & 0xFF;
  var f = (bitsHigh >>> 8) & 0xFF;
  var g = (bitsHigh >>> 16) & 0xFF;
  var h = (bitsHigh >>> 24) & 0xFF;

  return String.fromCharCode(a, b, c, d, e, f, g, h);
};

/**
 * Individual digits for number->string conversion.
 * @const {!Array<string>}
 */
jspb.utils.DIGITS = [
  '0', '1', '2', '3', '4', '5', '6', '7',
  '8', '9', 'a', 'b', 'c', 'd', 'e', 'f'
];

/** @const @private {number} '0' */
jspb.utils.ZERO_CHAR_CODE_ = 48;

/** @const @private {number} 'a' */
jspb.utils.A_CHAR_CODE_ = 97;

/**
 * Losslessly converts a 64-bit unsigned integer in 32:32 split representation
 * into a decimal string.
 * @param {number} bitsLow The low 32 bits of the binary number;
 * @param {number} bitsHigh The high 32 bits of the binary number.
 * @return {string} The binary number represented as a string.
 */
jspb.utils.joinUnsignedDecimalString = function(bitsLow, bitsHigh) {
  // Skip the expensive conversion if the number is small enough to use the
  // built-in conversions.
  if (bitsHigh <= 0x1FFFFF) {
    return '' + (jspb.BinaryConstants.TWO_TO_32 * bitsHigh + bitsLow);
  }

  // What this code is doing is essentially converting the input number from
  // base-2 to base-1e7, which allows us to represent the 64-bit range with
  // only 3 (very large) digits. Those digits are then trivial to convert to
  // a base-10 string.

  // The magic numbers used here are -
  // 2^24 = 16777216 = (1,6777216) in base-1e7.
  // 2^48 = 281474976710656 = (2,8147497,6710656) in base-1e7.

  // Split 32:32 representation into 16:24:24 representation so our
  // intermediate digits don't overflow.
  var low = bitsLow & 0xFFFFFF;
  var mid = (((bitsLow >>> 24) | (bitsHigh << 8)) >>> 0) & 0xFFFFFF;
  var high = (bitsHigh >> 16) & 0xFFFF;

  // Assemble our three base-1e7 digits, ignoring carries. The maximum
  // value in a digit at this step is representable as a 48-bit integer, which
  // can be stored in a 64-bit floating point number.
  var digitA = low + (mid * 6777216) + (high * 6710656);
  var digitB = mid + (high * 8147497);
  var digitC = (high * 2);

  // Apply carries from A to B and from B to C.
  var base = 10000000;
  if (digitA >= base) {
    digitB += Math.floor(digitA / base);
    digitA %= base;
  }

  if (digitB >= base) {
    digitC += Math.floor(digitB / base);
    digitB %= base;
  }

  // Convert base-1e7 digits to base-10, with optional leading zeroes.
  function decimalFrom1e7(digit1e7, needLeadingZeros) {
    var partial = digit1e7 ? String(digit1e7) : '';
    if (needLeadingZeros) {
      return '0000000'.slice(partial.length) + partial;
    }
    return partial;
  }

  return decimalFrom1e7(digitC, /*needLeadingZeros=*/ 0) +
      decimalFrom1e7(digitB, /*needLeadingZeros=*/ digitC) +
      // If the final 1e7 digit didn't need leading zeros, we would have
      // returned via the trivial code path at the top.
      decimalFrom1e7(digitA, /*needLeadingZeros=*/ 1);
};


/**
 * Losslessly converts a 64-bit signed integer in 32:32 split representation
 * into a decimal string.
 * @param {number} bitsLow The low 32 bits of the binary number;
 * @param {number} bitsHigh The high 32 bits of the binary number.
 * @return {string} The binary number represented as a string.
 */
jspb.utils.joinSignedDecimalString = function(bitsLow, bitsHigh) {
  // If we're treating the input as a signed value and the high bit is set, do
  // a manual two's complement conversion before the decimal conversion.
  var negative = (bitsHigh & 0x80000000);
  if (negative) {
    bitsLow = (~bitsLow + 1) >>> 0;
    var carry = (bitsLow == 0) ? 1 : 0;
    bitsHigh = (~bitsHigh + carry) >>> 0;
  }

  var result = jspb.utils.joinUnsignedDecimalString(bitsLow, bitsHigh);
  return negative ? '-' + result : result;
};


/**
 * Convert an 8-character hash string representing either a signed or unsigned
 * 64-bit integer into its decimal representation without losing accuracy.
 * @param {string} hash The hash string to convert.
 * @param {boolean} signed True if we should treat the hash string as encoding
 *     a signed integer.
 * @return {string}
 */
jspb.utils.hash64ToDecimalString = function(hash, signed) {
  jspb.utils.splitHash64(hash);
  var bitsLow = jspb.utils.split64Low;
  var bitsHigh = jspb.utils.split64High;
  return signed ?
      jspb.utils.joinSignedDecimalString(bitsLow, bitsHigh) :
      jspb.utils.joinUnsignedDecimalString(bitsLow, bitsHigh);
};


/**
 * Converts an array of 8-character hash strings into their decimal
 * representations.
 * @param {!Array<string>} hashes The array of hash strings to convert.
 * @param {boolean} signed True if we should treat the hash string as encoding
 *     a signed integer.
 * @return {!Array<string>}
 */
jspb.utils.hash64ArrayToDecimalStrings = function(hashes, signed) {
  var result = new Array(hashes.length);
  for (var i = 0; i < hashes.length; i++) {
    result[i] = jspb.utils.hash64ToDecimalString(hashes[i], signed);
  }
  return result;
};


/**
 * Converts a signed or unsigned decimal string into its hash string
 * representation.
 * @param {string} dec
 * @return {string}
 */
jspb.utils.decimalStringToHash64 = function(dec) {
  goog.asserts.assert(dec.length > 0);

  // Check for minus sign.
  var minus = false;
  if (dec[0] === '-') {
    minus = true;
    dec = dec.slice(1);
  }

  // Store result as a byte array.
  var resultBytes = [0, 0, 0, 0, 0, 0, 0, 0];

  // Set result to m*result + c.
  function muladd(m, c) {
    for (var i = 0; i < 8 && (m !== 1 || c > 0); i++) {
      var r = m * resultBytes[i] + c;
      resultBytes[i] = r & 0xFF;
      c = r >>> 8;
    }
  }

  // Negate the result bits.
  function neg() {
    for (var i = 0; i < 8; i++) {
      resultBytes[i] = (~resultBytes[i]) & 0xFF;
    }
  }

  // For each decimal digit, set result to 10*result + digit.
  for (var i = 0; i < dec.length; i++) {
    muladd(10, dec.charCodeAt(i) - jspb.utils.ZERO_CHAR_CODE_);
  }

  // If there's a minus sign, convert into two's complement.
  if (minus) {
    neg();
    muladd(1, 1);
  }

  return goog.crypt.byteArrayToString(resultBytes);
};


/**
 * Converts a signed or unsigned decimal string into two 32-bit halves, and
 * stores them in the temp variables listed above.
 * @param {string} value The decimal string to convert.
 */
jspb.utils.splitDecimalString = function(value) {
  jspb.utils.splitHash64(jspb.utils.decimalStringToHash64(value));
};

/**
 * @param {number} nibble A 4-bit integer.
 * @return {string}
 * @private
 */
jspb.utils.toHexDigit_ = function(nibble) {
  return String.fromCharCode(
      nibble < 10 ? jspb.utils.ZERO_CHAR_CODE_ + nibble :
                    jspb.utils.A_CHAR_CODE_ - 10 + nibble);
};

/**
 * @param {number} hexCharCode
 * @return {number}
 * @private
 */
jspb.utils.fromHexCharCode_ = function(hexCharCode) {
  if (hexCharCode >= jspb.utils.A_CHAR_CODE_) {
    return hexCharCode - jspb.utils.A_CHAR_CODE_ + 10;
  }
  return hexCharCode - jspb.utils.ZERO_CHAR_CODE_;
};

/**
 * Converts an 8-character hash string into its hexadecimal representation.
 * @param {string} hash
 * @return {string}
 */
jspb.utils.hash64ToHexString = function(hash) {
  var temp = new Array(18);
  temp[0] = '0';
  temp[1] = 'x';

  for (var i = 0; i < 8; i++) {
    var c = hash.charCodeAt(7 - i);
    temp[i * 2 + 2] = jspb.utils.toHexDigit_(c >> 4);
    temp[i * 2 + 3] = jspb.utils.toHexDigit_(c & 0xF);
  }

  var result = temp.join('');
  return result;
};


/**
 * Converts a '0x<16 digits>' hex string into its hash string representation.
 * @param {string} hex
 * @return {string}
 */
jspb.utils.hexStringToHash64 = function(hex) {
  hex = hex.toLowerCase();
  goog.asserts.assert(hex.length == 18);
  goog.asserts.assert(hex[0] == '0');
  goog.asserts.assert(hex[1] == 'x');

  var result = '';
  for (var i = 0; i < 8; i++) {
    var hi = jspb.utils.fromHexCharCode_(hex.charCodeAt(i * 2 + 2));
    var lo = jspb.utils.fromHexCharCode_(hex.charCodeAt(i * 2 + 3));
    result = String.fromCharCode(hi * 16 + lo) + result;
  }

  return result;
};


/**
 * Convert an 8-character hash string representing either a signed or unsigned
 * 64-bit integer into a Javascript number. Will lose accuracy if the result is
 * larger than 2^52.
 * @param {string} hash The hash string to convert.
 * @param {boolean} signed True if the has should be interpreted as a signed
 *     number.
 * @return {number}
 */
jspb.utils.hash64ToNumber = function(hash, signed) {
  jspb.utils.splitHash64(hash);
  var bitsLow = jspb.utils.split64Low;
  var bitsHigh = jspb.utils.split64High;
  return signed ? jspb.utils.joinInt64(bitsLow, bitsHigh) :
                  jspb.utils.joinUint64(bitsLow, bitsHigh);
};


/**
 * Convert a Javascript number into an 8-character hash string. Will lose
 * precision if the value is non-integral or greater than 2^64.
 * @param {number} value The integer to convert.
 * @return {string}
 */
jspb.utils.numberToHash64 = function(value) {
  jspb.utils.splitInt64(value);
  return jspb.utils.joinHash64(jspb.utils.split64Low,
                                  jspb.utils.split64High);
};


/**
 * Counts the number of contiguous varints in a buffer.
 * @param {!Uint8Array} buffer The buffer to scan.
 * @param {number} start The starting point in the buffer to scan.
 * @param {number} end The end point in the buffer to scan.
 * @return {number} The number of varints in the buffer.
 */
jspb.utils.countVarints = function(buffer, start, end) {
  // Count how many high bits of each byte were set in the buffer.
  var count = 0;
  for (var i = start; i < end; i++) {
    count += buffer[i] >> 7;
  }

  // The number of varints in the buffer equals the size of the buffer minus
  // the number of non-terminal bytes in the buffer (those with the high bit
  // set).
  return (end - start) - count;
};


/**
 * Counts the number of contiguous varint fields with the given field number in
 * the buffer.
 * @param {!Uint8Array} buffer The buffer to scan.
 * @param {number} start The starting point in the buffer to scan.
 * @param {number} end The end point in the buffer to scan.
 * @param {number} field The field number to count.
 * @return {number} The number of matching fields in the buffer.
 */
jspb.utils.countVarintFields = function(buffer, start, end, field) {
  var count = 0;
  var cursor = start;
  var tag = field * 8 + jspb.BinaryConstants.WireType.VARINT;

  if (tag < 128) {
    // Single-byte field tag, we can use a slightly quicker count.
    while (cursor < end) {
      // Skip the field tag, or exit if we find a non-matching tag.
      if (buffer[cursor++] != tag) return count;

      // Field tag matches, we've found a valid field.
      count++;

      // Skip the varint.
      while (1) {
        var x = buffer[cursor++];
        if ((x & 0x80) == 0) break;
      }
    }
  } else {
    while (cursor < end) {
      // Skip the field tag, or exit if we find a non-matching tag.
      var temp = tag;
      while (temp > 128) {
        if (buffer[cursor] != ((temp & 0x7F) | 0x80)) return count;
        cursor++;
        temp >>= 7;
      }
      if (buffer[cursor++] != temp) return count;

      // Field tag matches, we've found a valid field.
      count++;

      // Skip the varint.
      while (1) {
        var x = buffer[cursor++];
        if ((x & 0x80) == 0) break;
      }
    }
  }
  return count;
};


/**
 * Counts the number of contiguous fixed32 fields with the given tag in the
 * buffer.
 * @param {!Uint8Array} buffer The buffer to scan.
 * @param {number} start The starting point in the buffer to scan.
 * @param {number} end The end point in the buffer to scan.
 * @param {number} tag The tag value to count.
 * @param {number} stride The number of bytes to skip per field.
 * @return {number} The number of fields with a matching tag in the buffer.
 * @private
 */
jspb.utils.countFixedFields_ =
    function(buffer, start, end, tag, stride) {
  var count = 0;
  var cursor = start;

  if (tag < 128) {
    // Single-byte field tag, we can use a slightly quicker count.
    while (cursor < end) {
      // Skip the field tag, or exit if we find a non-matching tag.
      if (buffer[cursor++] != tag) return count;

      // Field tag matches, we've found a valid field.
      count++;

      // Skip the value.
      cursor += stride;
    }
  } else {
    while (cursor < end) {
      // Skip the field tag, or exit if we find a non-matching tag.
      var temp = tag;
      while (temp > 128) {
        if (buffer[cursor++] != ((temp & 0x7F) | 0x80)) return count;
        temp >>= 7;
      }
      if (buffer[cursor++] != temp) return count;

      // Field tag matches, we've found a valid field.
      count++;

      // Skip the value.
      cursor += stride;
    }
  }
  return count;
};


/**
 * Counts the number of contiguous fixed32 fields with the given field number
 * in the buffer.
 * @param {!Uint8Array} buffer The buffer to scan.
 * @param {number} start The starting point in the buffer to scan.
 * @param {number} end The end point in the buffer to scan.
 * @param {number} field The field number to count.
 * @return {number} The number of matching fields in the buffer.
 */
jspb.utils.countFixed32Fields = function(buffer, start, end, field) {
  var tag = field * 8 + jspb.BinaryConstants.WireType.FIXED32;
  return jspb.utils.countFixedFields_(buffer, start, end, tag, 4);
};


/**
 * Counts the number of contiguous fixed64 fields with the given field number
 * in the buffer.
 * @param {!Uint8Array} buffer The buffer to scan.
 * @param {number} start The starting point in the buffer to scan.
 * @param {number} end The end point in the buffer to scan.
 * @param {number} field The field number to count
 * @return {number} The number of matching fields in the buffer.
 */
jspb.utils.countFixed64Fields = function(buffer, start, end, field) {
  var tag = field * 8 + jspb.BinaryConstants.WireType.FIXED64;
  return jspb.utils.countFixedFields_(buffer, start, end, tag, 8);
};


/**
 * Counts the number of contiguous delimited fields with the given field number
 * in the buffer.
 * @param {!Uint8Array} buffer The buffer to scan.
 * @param {number} start The starting point in the buffer to scan.
 * @param {number} end The end point in the buffer to scan.
 * @param {number} field The field number to count.
 * @return {number} The number of matching fields in the buffer.
 */
jspb.utils.countDelimitedFields = function(buffer, start, end, field) {
  var count = 0;
  var cursor = start;
  var tag = field * 8 + jspb.BinaryConstants.WireType.DELIMITED;

  while (cursor < end) {
    // Skip the field tag, or exit if we find a non-matching tag.
    var temp = tag;
    while (temp > 128) {
      if (buffer[cursor++] != ((temp & 0x7F) | 0x80)) return count;
      temp >>= 7;
    }
    if (buffer[cursor++] != temp) return count;

    // Field tag matches, we've found a valid field.
    count++;

    // Decode the length prefix.
    var length = 0;
    var shift = 1;
    while (1) {
      temp = buffer[cursor++];
      length += (temp & 0x7f) * shift;
      shift *= 128;
      if ((temp & 0x80) == 0) break;
    }

    // Advance the cursor past the blob.
    cursor += length;
  }
  return count;
};


/**
 * String-ify bytes for text format. Should be optimized away in non-debug.
 * The returned string uses \xXX escapes for all values and is itself quoted.
 * [1, 31] serializes to '"\x01\x1f"'.
 * @param {jspb.ByteSource} byteSource The bytes to serialize.
 * @return {string} Stringified bytes for text format.
 */
jspb.utils.debugBytesToTextFormat = function(byteSource) {
  var s = '"';
  if (byteSource) {
    var bytes = jspb.utils.byteSourceToUint8Array(byteSource);
    for (var i = 0; i < bytes.length; i++) {
      s += '\\x';
      if (bytes[i] < 16) s += '0';
      s += bytes[i].toString(16);
    }
  }
  return s + '"';
};


/**
 * String-ify a scalar for text format. Should be optimized away in non-debug.
 * @param {string|number|boolean} scalar The scalar to stringify.
 * @return {string} Stringified scalar for text format.
 */
jspb.utils.debugScalarToTextFormat = function(scalar) {
  if (typeof scalar === 'string') {
    return goog.string.quote(scalar);
  } else {
    return scalar.toString();
  }
};


/**
 * Utility function: convert a string with codepoints 0--255 inclusive to a
 * Uint8Array. If any codepoints greater than 255 exist in the string, throws an
 * exception.
 * @param {string} str
 * @return {!Uint8Array}
 */
jspb.utils.stringToByteArray = function(str) {
  var arr = new Uint8Array(str.length);
  for (var i = 0; i < str.length; i++) {
    var codepoint = str.charCodeAt(i);
    if (codepoint > 255) {
      throw new Error('Conversion error: string contains codepoint ' +
                      'outside of byte range');
    }
    arr[i] = codepoint;
  }
  return arr;
};


/**
 * Converts any type defined in jspb.ByteSource into a Uint8Array.
 * @param {!jspb.ByteSource} data
 * @return {!Uint8Array}
 * @suppress {invalidCasts}
 */
jspb.utils.byteSourceToUint8Array = function(data) {
  if (data.constructor === Uint8Array) {
    return /** @type {!Uint8Array} */(data);
  }

  if (data.constructor === ArrayBuffer) {
    data = /** @type {!ArrayBuffer} */(data);
    return /** @type {!Uint8Array} */(new Uint8Array(data));
  }

  // BEGIN PROTOBUF-OPENSOURCE
  // if (typeof Buffer != 'undefined' && data.constructor === Buffer) {
  //   return /** @type {!Uint8Array} */ (
  //       new Uint8Array(/** @type {?} */ (data)));
  // }
  // END PROTOBUF-OPENSOURCE

  if (data.constructor === Array) {
    data = /** @type {!Array<number>} */(data);
    return /** @type {!Uint8Array} */(new Uint8Array(data));
  }

  if (data.constructor === String) {
    data = /** @type {string} */(data);
    return goog.crypt.base64.decodeStringToUint8Array(data);
  }

  goog.asserts.fail('Type not convertible to Uint8Array.');
  return /** @type {!Uint8Array} */(new Uint8Array(0));
};

//javascript/apps/jspb/binary/decoder.js
/**
 * @fileoverview This file contains utilities for decoding primitive values
 * (signed and unsigned integers, varints, booleans, enums, hashes, strings,
 * and raw bytes) embedded in Uint8Arrays into their corresponding Javascript
 * types.
 *
 * Major caveat - Javascript is unable to accurately represent integers larger
 * than 2^53 due to its use of a double-precision floating point format or all
 * numbers. If you need to guarantee that 64-bit values survive with all bits
 * intact, you _must_ read them using one of the Hash64 methods, which return
 * an 8-character string.
 *
 * @author aappleby@google.com (Austin Appleby)
 */

goog.provide('jspb.BinaryDecoder');

goog.require('goog.asserts');
goog.require('goog.crypt');
goog.require('jspb.utils');



/**
 * BinaryDecoder implements the decoders for all the wire types specified in
 * https://developers.google.com/protocol-buffers/docs/encoding.
 *
 * @param {jspb.ByteSource=} opt_bytes The bytes we're reading from.
 * @param {number=} opt_start The optional offset to start reading at.
 * @param {number=} opt_length The optional length of the block to read -
 *     we'll throw an assertion if we go off the end of the block.
 * @constructor
 * @struct
 */
jspb.BinaryDecoder = function(opt_bytes, opt_start, opt_length) {
  /**
   * Typed byte-wise view of the source buffer.
   * @private {?Uint8Array}
   */
  this.bytes_ = null;

  /**
   * Start point of the block to read.
   * @private {number}
   */
  this.start_ = 0;

  /**
   * End point of the block to read.
   * @private {number}
   */
  this.end_ = 0;

  /**
   * Current read location in bytes_.
   * @private {number}
   */
  this.cursor_ = 0;

  /**
   * Set to true if this decoder encountered an error due to corrupt data.
   * @private {boolean}
   */
  this.error_ = false;

  if (opt_bytes) {
    this.setBlock(opt_bytes, opt_start, opt_length);
  }
};


/**
 * Global pool of BinaryDecoder instances.
 * @private {!Array<!jspb.BinaryDecoder>}
 */
jspb.BinaryDecoder.instanceCache_ = [];


/**
 * Pops an instance off the instance cache, or creates one if the cache is
 * empty.
 * @param {jspb.ByteSource=} opt_bytes The bytes we're reading from.
 * @param {number=} opt_start The optional offset to start reading at.
 * @param {number=} opt_length The optional length of the block to read -
 *     we'll throw an assertion if we go off the end of the block.
 * @return {!jspb.BinaryDecoder}
 */
jspb.BinaryDecoder.alloc = function(opt_bytes, opt_start, opt_length) {
  if (jspb.BinaryDecoder.instanceCache_.length) {
    var newDecoder = jspb.BinaryDecoder.instanceCache_.pop();
    if (opt_bytes) {
      newDecoder.setBlock(opt_bytes, opt_start, opt_length);
    }
    return newDecoder;
  } else {
    return new jspb.BinaryDecoder(opt_bytes, opt_start, opt_length);
  }
};


/**
 * Puts this instance back in the instance cache.
 */
jspb.BinaryDecoder.prototype.free = function() {
  this.clear();
  if (jspb.BinaryDecoder.instanceCache_.length < 100) {
    jspb.BinaryDecoder.instanceCache_.push(this);
  }
};


/**
 * Makes a copy of this decoder.
 * @return {!jspb.BinaryDecoder}
 */
jspb.BinaryDecoder.prototype.clone = function() {
  return jspb.BinaryDecoder.alloc(this.bytes_,
      this.start_, this.end_ - this.start_);
};


/**
 * Clears the decoder.
 */
jspb.BinaryDecoder.prototype.clear = function() {
  this.bytes_ = null;
  this.start_ = 0;
  this.end_ = 0;
  this.cursor_ = 0;
  this.error_ = false;
};


/**
 * Returns the raw buffer.
 * @return {?Uint8Array} The raw buffer.
 */
jspb.BinaryDecoder.prototype.getBuffer = function() {
  return this.bytes_;
};


/**
 * Changes the block of bytes we're decoding.
 * @param {!jspb.ByteSource} data The bytes we're reading from.
 * @param {number=} opt_start The optional offset to start reading at.
 * @param {number=} opt_length The optional length of the block to read -
 *     we'll throw an assertion if we go off the end of the block.
 */
jspb.BinaryDecoder.prototype.setBlock =
    function(data, opt_start, opt_length) {
  this.bytes_ = jspb.utils.byteSourceToUint8Array(data);
  this.start_ = (opt_start !== undefined) ? opt_start : 0;
  this.end_ = (opt_length !== undefined) ? this.start_ + opt_length :
                                           this.bytes_.length;
  this.cursor_ = this.start_;
};


/**
 * @return {number}
 */
jspb.BinaryDecoder.prototype.getEnd = function() {
  return this.end_;
};


/**
 * @param {number} end
 */
jspb.BinaryDecoder.prototype.setEnd = function(end) {
  this.end_ = end;
};


/**
 * Moves the read cursor back to the start of the block.
 */
jspb.BinaryDecoder.prototype.reset = function() {
  this.cursor_ = this.start_;
};


/**
 * Returns the internal read cursor.
 * @return {number} The internal read cursor.
 */
jspb.BinaryDecoder.prototype.getCursor = function() {
  return this.cursor_;
};


/**
 * Returns the internal read cursor.
 * @param {number} cursor The new cursor.
 */
jspb.BinaryDecoder.prototype.setCursor = function(cursor) {
  this.cursor_ = cursor;
};


/**
 * Advances the stream cursor by the given number of bytes.
 * @param {number} count The number of bytes to advance by.
 */
jspb.BinaryDecoder.prototype.advance = function(count) {
  this.cursor_ += count;
  goog.asserts.assert(this.cursor_ <= this.end_);
};


/**
 * Returns true if this decoder is at the end of the block.
 * @return {boolean}
 */
jspb.BinaryDecoder.prototype.atEnd = function() {
  return this.cursor_ == this.end_;
};


/**
 * Returns true if this decoder is at the end of the block.
 * @return {boolean}
 */
jspb.BinaryDecoder.prototype.pastEnd = function() {
  return this.cursor_ > this.end_;
};


/**
 * Returns true if this decoder encountered an error due to corrupt data.
 * @return {boolean}
 */
jspb.BinaryDecoder.prototype.getError = function() {
  return this.error_ ||
         (this.cursor_ < 0) ||
         (this.cursor_ > this.end_);
};


/**
 * Reads an unsigned varint from the binary stream and invokes the conversion
 * function with the value in two signed 32 bit integers to produce the result.
 * Since this does not convert the value to a number, no precision is lost.
 *
 * It's possible for an unsigned varint to be incorrectly encoded - more than
 * 64 bits' worth of data could be present. If this happens, this method will
 * throw an error.
 *
 * Decoding varints requires doing some funny base-128 math - for more
 * details on the format, see
 * https://developers.google.com/protocol-buffers/docs/encoding
 *
 * @param {function(number, number): T} convert Conversion function to produce
 *     the result value, takes parameters (lowBits, highBits).
 * @return {T}
 * @template T
 */
jspb.BinaryDecoder.prototype.readSplitVarint64 = function(convert) {
  var temp = 128;
  var lowBits = 0;
  var highBits = 0;

  // Read the first four bytes of the varint, stopping at the terminator if we
  // see it.
  for (var i = 0; i < 4 && temp >= 128; i++) {
    temp = this.bytes_[this.cursor_++];
    lowBits |= (temp & 0x7F) << (i * 7);
  }

  if (temp >= 128) {
    // Read the fifth byte, which straddles the low and high dwords.
    temp = this.bytes_[this.cursor_++];
    lowBits |= (temp & 0x7F) << 28;
    highBits |= (temp & 0x7F) >> 4;
  }

  if (temp >= 128) {
    // Read the sixth through tenth byte.
    for (var i = 0; i < 5 && temp >= 128; i++) {
      temp = this.bytes_[this.cursor_++];
      highBits |= (temp & 0x7F) << (i * 7 + 3);
    }
  }

  if (temp < 128) {
    return convert(lowBits >>> 0, highBits >>> 0);
  }

  // If we did not see the terminator, the encoding was invalid.
  goog.asserts.fail('Failed to read varint, encoding is invalid.');
  this.error_ = true;
};


/**
 * Reads a signed zigzag encoded varint from the binary stream and invokes
 * the conversion function with the value in two signed 32 bit integers to
 * produce the result. Since this does not convert the value to a number, no
 * precision is lost.
 *
 * It's possible for an unsigned varint to be incorrectly encoded - more than
 * 64 bits' worth of data could be present. If this happens, this method will
 * throw an error.
 *
 * Zigzag encoding is a modification of varint encoding that reduces the
 * storage overhead for small negative integers - for more details on the
 * format, see https://developers.google.com/protocol-buffers/docs/encoding
 *
 * @param {function(number, number): T} convert Conversion function to produce
 *     the result value, takes parameters (lowBits, highBits).
 * @return {T}
 * @template T
 */
jspb.BinaryDecoder.prototype.readSplitZigzagVarint64 = function(convert) {
  return this.readSplitVarint64(function(low, high) {
    return jspb.utils.fromZigzag64(low, high, convert);
  });
};


/**
 * Reads a 64-bit fixed-width value from the stream and invokes the conversion
 * function with the value in two signed 32 bit integers to produce the result.
 * Since this does not convert the value to a number, no precision is lost.
 *
 * @param {function(number, number): T} convert Conversion function to produce
 *     the result value, takes parameters (lowBits, highBits).
 * @return {T}
 * @template T
 */
jspb.BinaryDecoder.prototype.readSplitFixed64 = function(convert) {
  var bytes = this.bytes_;
  var cursor = this.cursor_;
  this.cursor_ += 8;
  var lowBits = 0;
  var highBits = 0;
  for (var i = cursor + 7; i >= cursor; i--) {
    lowBits = (lowBits << 8) | bytes[i];
    highBits = (highBits << 8) | bytes[i + 4];
  }
  return convert(lowBits, highBits);
};


/**
 * Skips over a varint in the block without decoding it.
 */
jspb.BinaryDecoder.prototype.skipVarint = function() {
  while (this.bytes_[this.cursor_] & 0x80) {
    this.cursor_++;
  }
  this.cursor_++;
};


/**
 * Skips backwards over a varint in the block - to do this correctly, we have
 * to know the value we're skipping backwards over or things are ambiguous.
 * @param {number} value The varint value to unskip.
 */
jspb.BinaryDecoder.prototype.unskipVarint = function(value) {
  while (value > 128) {
    this.cursor_--;
    value = value >>> 7;
  }
  this.cursor_--;
};


/**
 * Reads a 32-bit varint from the binary stream. Due to a quirk of the encoding
 * format and Javascript's handling of bitwise math, this actually works
 * correctly for both signed and unsigned 32-bit varints.
 *
 * This function is called vastly more frequently than any other in
 * BinaryDecoder, so it has been unrolled and tweaked for performance.
 *
 * If there are more than 32 bits of data in the varint, it _must_ be due to
 * sign-extension. If we're in debug mode and the high 32 bits don't match the
 * expected sign extension, this method will throw an error.
 *
 * Decoding varints requires doing some funny base-128 math - for more
 * details on the format, see
 * https://developers.google.com/protocol-buffers/docs/encoding
 *
 * @return {number} The decoded unsigned 32-bit varint.
 */
jspb.BinaryDecoder.prototype.readUnsignedVarint32 = function() {
  var temp;
  var bytes = this.bytes_;

  temp = bytes[this.cursor_ + 0];
  var x = (temp & 0x7F);
  if (temp < 128) {
    this.cursor_ += 1;
    goog.asserts.assert(this.cursor_ <= this.end_);
    return x;
  }

  temp = bytes[this.cursor_ + 1];
  x |= (temp & 0x7F) << 7;
  if (temp < 128) {
    this.cursor_ += 2;
    goog.asserts.assert(this.cursor_ <= this.end_);
    return x;
  }

  temp = bytes[this.cursor_ + 2];
  x |= (temp & 0x7F) << 14;
  if (temp < 128) {
    this.cursor_ += 3;
    goog.asserts.assert(this.cursor_ <= this.end_);
    return x;
  }

  temp = bytes[this.cursor_ + 3];
  x |= (temp & 0x7F) << 21;
  if (temp < 128) {
    this.cursor_ += 4;
    goog.asserts.assert(this.cursor_ <= this.end_);
    return x;
  }

  temp = bytes[this.cursor_ + 4];
  x |= (temp & 0x0F) << 28;
  if (temp < 128) {
    // We're reading the high bits of an unsigned varint. The byte we just read
    // also contains bits 33 through 35, which we're going to discard.
    this.cursor_ += 5;
    goog.asserts.assert(this.cursor_ <= this.end_);
    return x >>> 0;
  }

  // If we get here, we need to truncate coming bytes. However we need to make
  // sure cursor place is correct.
  this.cursor_ += 5;
  if (bytes[this.cursor_++] >= 128 &&
      bytes[this.cursor_++] >= 128 &&
      bytes[this.cursor_++] >= 128 &&
      bytes[this.cursor_++] >= 128 &&
      bytes[this.cursor_++] >= 128) {
    // If we get here, the varint is too long.
    goog.asserts.assert(false);
  }

  goog.asserts.assert(this.cursor_ <= this.end_);
  return x;
};


/**
 * The readUnsignedVarint32 above deals with signed 32-bit varints correctly,
 * so this is just an alias.
 *
 * @return {number} The decoded signed 32-bit varint.
 */
jspb.BinaryDecoder.prototype.readSignedVarint32 =
    jspb.BinaryDecoder.prototype.readUnsignedVarint32;


/**
 * Reads a 32-bit unsigned variant and returns its value as a string.
 *
 * @return {string} The decoded unsigned 32-bit varint as a string.
 */
jspb.BinaryDecoder.prototype.readUnsignedVarint32String = function() {
  // 32-bit integers fit in JavaScript numbers without loss of precision, so
  // string variants of 32-bit varint readers can simply delegate then convert
  // to string.
  var value = this.readUnsignedVarint32();
  return value.toString();
};


/**
 * Reads a 32-bit signed variant and returns its value as a string.
 *
 * @return {string} The decoded signed 32-bit varint as a string.
 */
jspb.BinaryDecoder.prototype.readSignedVarint32String = function() {
  // 32-bit integers fit in JavaScript numbers without loss of precision, so
  // string variants of 32-bit varint readers can simply delegate then convert
  // to string.
  var value = this.readSignedVarint32();
  return value.toString();
};


/**
 * Reads a signed, zigzag-encoded 32-bit varint from the binary stream.
 *
 * Zigzag encoding is a modification of varint encoding that reduces the
 * storage overhead for small negative integers - for more details on the
 * format, see https://developers.google.com/protocol-buffers/docs/encoding
 *
 * @return {number} The decoded signed, zigzag-encoded 32-bit varint.
 */
jspb.BinaryDecoder.prototype.readZigzagVarint32 = function() {
  var result = this.readUnsignedVarint32();
  return (result >>> 1) ^ - (result & 1);
};


/**
 * Reads an unsigned 64-bit varint from the binary stream. Note that since
 * Javascript represents all numbers as double-precision floats, there will be
 * precision lost if the absolute value of the varint is larger than 2^53.
 *
 * @return {number} The decoded unsigned varint. Precision will be lost if the
 *     integer exceeds 2^53.
 */
jspb.BinaryDecoder.prototype.readUnsignedVarint64 = function() {
  return this.readSplitVarint64(jspb.utils.joinUint64);
};


/**
 * Reads an unsigned 64-bit varint from the binary stream and returns the value
 * as a decimal string.
 *
 * @return {string} The decoded unsigned varint as a decimal string.
 */
jspb.BinaryDecoder.prototype.readUnsignedVarint64String = function() {
  return this.readSplitVarint64(jspb.utils.joinUnsignedDecimalString);
};


/**
 * Reads a signed 64-bit varint from the binary stream. Note that since
 * Javascript represents all numbers as double-precision floats, there will be
 * precision lost if the absolute value of the varint is larger than 2^53.
 *
 * @return {number} The decoded signed varint. Precision will be lost if the
 *     integer exceeds 2^53.
 */
jspb.BinaryDecoder.prototype.readSignedVarint64 = function() {
  return this.readSplitVarint64(jspb.utils.joinInt64);
};


/**
 * Reads an signed 64-bit varint from the binary stream and returns the value
 * as a decimal string.
 *
 * @return {string} The decoded signed varint as a decimal string.
 */
jspb.BinaryDecoder.prototype.readSignedVarint64String = function() {
  return this.readSplitVarint64(jspb.utils.joinSignedDecimalString);
};


/**
 * Reads a signed, zigzag-encoded 64-bit varint from the binary stream. Note
 * that since Javascript represents all numbers as double-precision floats,
 * there will be precision lost if the absolute value of the varint is larger
 * than 2^53.
 *
 * Zigzag encoding is a modification of varint encoding that reduces the
 * storage overhead for small negative integers - for more details on the
 * format, see https://developers.google.com/protocol-buffers/docs/encoding
 *
 * @return {number} The decoded zigzag varint. Precision will be lost if the
 *     integer exceeds 2^53.
 */
jspb.BinaryDecoder.prototype.readZigzagVarint64 = function() {
  return this.readSplitVarint64(jspb.utils.joinZigzag64);
};


/**
 * Reads a signed, zigzag-encoded 64-bit varint from the binary stream
 * losslessly and returns it as an 8-character Unicode string for use as a hash
 * table key.
 *
 * Zigzag encoding is a modification of varint encoding that reduces the
 * storage overhead for small negative integers - for more details on the
 * format, see https://developers.google.com/protocol-buffers/docs/encoding
 *
 * @return {string} The decoded zigzag varint in hash64 format.
 */
jspb.BinaryDecoder.prototype.readZigzagVarintHash64 = function() {
  return this.readSplitZigzagVarint64(jspb.utils.joinHash64);
};


/**
 * Reads a signed, zigzag-encoded 64-bit varint from the binary stream and
 * returns its value as a string.
 *
 * Zigzag encoding is a modification of varint encoding that reduces the
 * storage overhead for small negative integers - for more details on the
 * format, see https://developers.google.com/protocol-buffers/docs/encoding
 *
 * @return {string} The decoded signed, zigzag-encoded 64-bit varint as a
 * string.
 */
jspb.BinaryDecoder.prototype.readZigzagVarint64String = function() {
  return this.readSplitZigzagVarint64(jspb.utils.joinSignedDecimalString);
};


/**
 * Reads a raw unsigned 8-bit integer from the binary stream.
 *
 * @return {number} The unsigned 8-bit integer read from the binary stream.
 */
jspb.BinaryDecoder.prototype.readUint8 = function() {
  var a = this.bytes_[this.cursor_ + 0];
  this.cursor_ += 1;
  goog.asserts.assert(this.cursor_ <= this.end_);
  return a;
};


/**
 * Reads a raw unsigned 16-bit integer from the binary stream.
 *
 * @return {number} The unsigned 16-bit integer read from the binary stream.
 */
jspb.BinaryDecoder.prototype.readUint16 = function() {
  var a = this.bytes_[this.cursor_ + 0];
  var b = this.bytes_[this.cursor_ + 1];
  this.cursor_ += 2;
  goog.asserts.assert(this.cursor_ <= this.end_);
  return (a << 0) | (b << 8);
};


/**
 * Reads a raw unsigned 32-bit integer from the binary stream.
 *
 * @return {number} The unsigned 32-bit integer read from the binary stream.
 */
jspb.BinaryDecoder.prototype.readUint32 = function() {
  var a = this.bytes_[this.cursor_ + 0];
  var b = this.bytes_[this.cursor_ + 1];
  var c = this.bytes_[this.cursor_ + 2];
  var d = this.bytes_[this.cursor_ + 3];
  this.cursor_ += 4;
  goog.asserts.assert(this.cursor_ <= this.end_);
  return ((a << 0) | (b << 8) | (c << 16) | (d << 24)) >>> 0;
};


/**
 * Reads a raw unsigned 64-bit integer from the binary stream. Note that since
 * Javascript represents all numbers as double-precision floats, there will be
 * precision lost if the absolute value of the integer is larger than 2^53.
 *
 * @return {number} The unsigned 64-bit integer read from the binary stream.
 *     Precision will be lost if the integer exceeds 2^53.
 */
jspb.BinaryDecoder.prototype.readUint64 = function() {
  var bitsLow = this.readUint32();
  var bitsHigh = this.readUint32();
  return jspb.utils.joinUint64(bitsLow, bitsHigh);
};


/**
 * Reads a raw unsigned 64-bit integer from the binary stream. Note that since
 * Javascript represents all numbers as double-precision floats, there will be
 * precision lost if the absolute value of the integer is larger than 2^53.
 *
 * @return {string} The unsigned 64-bit integer read from the binary stream.
 */
jspb.BinaryDecoder.prototype.readUint64String = function() {
  var bitsLow = this.readUint32();
  var bitsHigh = this.readUint32();
  return jspb.utils.joinUnsignedDecimalString(bitsLow, bitsHigh);
};


/**
 * Reads a raw signed 8-bit integer from the binary stream.
 *
 * @return {number} The signed 8-bit integer read from the binary stream.
 */
jspb.BinaryDecoder.prototype.readInt8 = function() {
  var a = this.bytes_[this.cursor_ + 0];
  this.cursor_ += 1;
  goog.asserts.assert(this.cursor_ <= this.end_);
  return (a << 24) >> 24;
};


/**
 * Reads a raw signed 16-bit integer from the binary stream.
 *
 * @return {number} The signed 16-bit integer read from the binary stream.
 */
jspb.BinaryDecoder.prototype.readInt16 = function() {
  var a = this.bytes_[this.cursor_ + 0];
  var b = this.bytes_[this.cursor_ + 1];
  this.cursor_ += 2;
  goog.asserts.assert(this.cursor_ <= this.end_);
  return (((a << 0) | (b << 8)) << 16) >> 16;
};


/**
 * Reads a raw signed 32-bit integer from the binary stream.
 *
 * @return {number} The signed 32-bit integer read from the binary stream.
 */
jspb.BinaryDecoder.prototype.readInt32 = function() {
  var a = this.bytes_[this.cursor_ + 0];
  var b = this.bytes_[this.cursor_ + 1];
  var c = this.bytes_[this.cursor_ + 2];
  var d = this.bytes_[this.cursor_ + 3];
  this.cursor_ += 4;
  goog.asserts.assert(this.cursor_ <= this.end_);
  return (a << 0) | (b << 8) | (c << 16) | (d << 24);
};


/**
 * Reads a raw signed 64-bit integer from the binary stream. Note that since
 * Javascript represents all numbers as double-precision floats, there will be
 * precision lost if the absolute vlaue of the integer is larger than 2^53.
 *
 * @return {number} The signed 64-bit integer read from the binary stream.
 *     Precision will be lost if the integer exceeds 2^53.
 */
jspb.BinaryDecoder.prototype.readInt64 = function() {
  var bitsLow = this.readUint32();
  var bitsHigh = this.readUint32();
  return jspb.utils.joinInt64(bitsLow, bitsHigh);
};


/**
 * Reads a raw signed 64-bit integer from the binary stream and returns it as a
 * string.
 *
 * @return {string} The signed 64-bit integer read from the binary stream.
 *     Precision will be lost if the integer exceeds 2^53.
 */
jspb.BinaryDecoder.prototype.readInt64String = function() {
  var bitsLow = this.readUint32();
  var bitsHigh = this.readUint32();
  return jspb.utils.joinSignedDecimalString(bitsLow, bitsHigh);
};


/**
 * Reads a 32-bit floating-point number from the binary stream, using the
 * temporary buffer to realign the data.
 *
 * @return {number} The float read from the binary stream.
 */
jspb.BinaryDecoder.prototype.readFloat = function() {
  var bitsLow = this.readUint32();
  var bitsHigh = 0;
  return jspb.utils.joinFloat32(bitsLow, bitsHigh);
};


/**
 * Reads a 64-bit floating-point number from the binary stream, using the
 * temporary buffer to realign the data.
 *
 * @return {number} The double read from the binary stream.
 */
jspb.BinaryDecoder.prototype.readDouble = function() {
  var bitsLow = this.readUint32();
  var bitsHigh = this.readUint32();
  return jspb.utils.joinFloat64(bitsLow, bitsHigh);
};


/**
 * Reads a boolean value from the binary stream.
 * @return {boolean} The boolean read from the binary stream.
 */
jspb.BinaryDecoder.prototype.readBool = function() {
  return !!this.bytes_[this.cursor_++];
};


/**
 * Reads an enum value from the binary stream, which are always encoded as
 * signed varints.
 * @return {number} The enum value read from the binary stream.
 */
jspb.BinaryDecoder.prototype.readEnum = function() {
  return this.readSignedVarint32();
};


/**
 * Reads and parses a UTF-8 encoded unicode string from the stream.
 * The code is inspired by maps.vectortown.parse.StreamedDataViewReader.
 * Supports codepoints from U+0000 up to U+10FFFF.
 * (http://en.wikipedia.org/wiki/UTF-8).
 * @param {number} length The length of the string to read.
 * @return {string} The decoded string.
 */
jspb.BinaryDecoder.prototype.readString = function(length) {
  var bytes = this.bytes_;
  var cursor = this.cursor_;
  var end = cursor + length;
  var codeUnits = [];

  var result = '';
  while (cursor < end) {
    var c = bytes[cursor++];
    if (c < 128) { // Regular 7-bit ASCII.
      codeUnits.push(c);
    } else if (c < 192) {
      // UTF-8 continuation mark. We are out of sync. This
      // might happen if we attempted to read a character
      // with more than four bytes.
      continue;
    } else if (c < 224) { // UTF-8 with two bytes.
      var c2 = bytes[cursor++];
      codeUnits.push(((c & 31) << 6) | (c2 & 63));
    } else if (c < 240) { // UTF-8 with three bytes.
      var c2 = bytes[cursor++];
      var c3 = bytes[cursor++];
      codeUnits.push(((c & 15) << 12) | ((c2 & 63) << 6) | (c3 & 63));
    } else if (c < 248) { // UTF-8 with 4 bytes.
      var c2 = bytes[cursor++];
      var c3 = bytes[cursor++];
      var c4 = bytes[cursor++];
      // Characters written on 4 bytes have 21 bits for a codepoint.
      // We can't fit that on 16bit characters, so we use surrogates.
      var codepoint = ((c & 7) << 18) | ((c2 & 63) << 12) | ((c3 & 63) << 6) | (c4 & 63);
      // Surrogates formula from wikipedia.
      // 1. Subtract 0x10000 from codepoint
      codepoint -= 0x10000;
      // 2. Split this into the high 10-bit value and the low 10-bit value
      // 3. Add 0xD800 to the high value to form the high surrogate
      // 4. Add 0xDC00 to the low value to form the low surrogate:
      var low = (codepoint & 1023) + 0xDC00;
      var high = ((codepoint >> 10) & 1023) + 0xD800;
      codeUnits.push(high, low);
    }

    // Avoid exceeding the maximum stack size when calling `apply`.
    if (codeUnits.length >= 8192) {
      result += String.fromCharCode.apply(null, codeUnits);
      codeUnits.length = 0;
    }
  }
  result += goog.crypt.byteArrayToString(codeUnits);
  this.cursor_ = cursor;
  return result;
};


/**
 * Reads and parses a UTF-8 encoded unicode string (with length prefix) from
 * the stream.
 * @return {string} The decoded string.
 */
jspb.BinaryDecoder.prototype.readStringWithLength = function() {
  var length = this.readUnsignedVarint32();
  return this.readString(length);
};


/**
 * Reads a block of raw bytes from the binary stream.
 *
 * @param {number} length The number of bytes to read.
 * @return {!Uint8Array} The decoded block of bytes, or an empty block if the
 *     length was invalid.
 */
jspb.BinaryDecoder.prototype.readBytes = function(length) {
  if (length < 0 ||
      this.cursor_ + length > this.bytes_.length) {
    this.error_ = true;
    goog.asserts.fail('Invalid byte length!');
    return new Uint8Array(0);
  }

  var result = this.bytes_.subarray(this.cursor_, this.cursor_ + length);

  this.cursor_ += length;
  goog.asserts.assert(this.cursor_ <= this.end_);
  return result;
};


/**
 * Reads a 64-bit varint from the stream and returns it as an 8-character
 * Unicode string for use as a hash table key.
 *
 * @return {string} The hash value.
 */
jspb.BinaryDecoder.prototype.readVarintHash64 = function() {
  return this.readSplitVarint64(jspb.utils.joinHash64);
};


/**
 * Reads a 64-bit fixed-width value from the stream and returns it as an
 * 8-character Unicode string for use as a hash table key.
 *
 * @return {string} The hash value.
 */
jspb.BinaryDecoder.prototype.readFixedHash64 = function() {
  var bytes = this.bytes_;
  var cursor = this.cursor_;

  var a = bytes[cursor + 0];
  var b = bytes[cursor + 1];
  var c = bytes[cursor + 2];
  var d = bytes[cursor + 3];
  var e = bytes[cursor + 4];
  var f = bytes[cursor + 5];
  var g = bytes[cursor + 6];
  var h = bytes[cursor + 7];

  this.cursor_ += 8;

  return String.fromCharCode(a, b, c, d, e, f, g, h);
};

//javascript/apps/jspb/binary/encoder.js
/**
 * @fileoverview BinaryEncode defines methods for encoding Javascript values
 * into arrays of bytes compatible with the Protocol Buffer wire format.
 *
 * @author aappleby@google.com (Austin Appleby)
 */

goog.provide('jspb.BinaryEncoder');

goog.require('goog.asserts');
goog.require('jspb.BinaryConstants');
goog.require('jspb.utils');



/**
 * BinaryEncoder implements encoders for all the wire types specified in
 * https://developers.google.com/protocol-buffers/docs/encoding.
 *
 * @constructor
 * @struct
 */
jspb.BinaryEncoder = function() {
  /** @private {!Array<number>} */
  this.buffer_ = [];
};


/**
 * @return {number}
 */
jspb.BinaryEncoder.prototype.length = function() {
  return this.buffer_.length;
};


/**
 * @return {!Array<number>}
 */
jspb.BinaryEncoder.prototype.end = function() {
  var buffer = this.buffer_;
  this.buffer_ = [];
  return buffer;
};


/**
 * Encodes a 64-bit integer in 32:32 split representation into its wire-format
 * varint representation and stores it in the buffer.
 * @param {number} lowBits The low 32 bits of the int.
 * @param {number} highBits The high 32 bits of the int.
 */
jspb.BinaryEncoder.prototype.writeSplitVarint64 = function(lowBits, highBits) {
  goog.asserts.assert(lowBits == Math.floor(lowBits));
  goog.asserts.assert(highBits == Math.floor(highBits));
  goog.asserts.assert((lowBits >= 0) &&
                      (lowBits < jspb.BinaryConstants.TWO_TO_32));
  goog.asserts.assert((highBits >= 0) &&
                      (highBits < jspb.BinaryConstants.TWO_TO_32));

  // Break the binary representation into chunks of 7 bits, set the 8th bit
  // in each chunk if it's not the final chunk, and append to the result.
  while (highBits > 0 || lowBits > 127) {
    this.buffer_.push((lowBits & 0x7f) | 0x80);
    lowBits = ((lowBits >>> 7) | (highBits << 25)) >>> 0;
    highBits = highBits >>> 7;
  }
  this.buffer_.push(lowBits);
};


/**
 * Encodes a 64-bit integer in 32:32 split representation into its wire-format
 * fixed representation and stores it in the buffer.
 * @param {number} lowBits The low 32 bits of the int.
 * @param {number} highBits The high 32 bits of the int.
 */
jspb.BinaryEncoder.prototype.writeSplitFixed64 = function(lowBits, highBits) {
  goog.asserts.assert(lowBits == Math.floor(lowBits));
  goog.asserts.assert(highBits == Math.floor(highBits));
  goog.asserts.assert((lowBits >= 0) &&
                      (lowBits < jspb.BinaryConstants.TWO_TO_32));
  goog.asserts.assert((highBits >= 0) &&
                      (highBits < jspb.BinaryConstants.TWO_TO_32));
  this.writeUint32(lowBits);
  this.writeUint32(highBits);
};


/**
 * Encodes a 32-bit unsigned integer into its wire-format varint representation
 * and stores it in the buffer.
 * @param {number} value The integer to convert.
 */
jspb.BinaryEncoder.prototype.writeUnsignedVarint32 = function(value) {
  goog.asserts.assert(value == Math.floor(value));
  goog.asserts.assert((value >= 0) &&
                      (value < jspb.BinaryConstants.TWO_TO_32));

  while (value > 127) {
    this.buffer_.push((value & 0x7f) | 0x80);
    value = value >>> 7;
  }

  this.buffer_.push(value);
};


/**
 * Encodes a 32-bit signed integer into its wire-format varint representation
 * and stores it in the buffer.
 * @param {number} value The integer to convert.
 */
jspb.BinaryEncoder.prototype.writeSignedVarint32 = function(value) {
  goog.asserts.assert(value == Math.floor(value));
  goog.asserts.assert((value >= -jspb.BinaryConstants.TWO_TO_31) &&
                      (value < jspb.BinaryConstants.TWO_TO_31));

  // Use the unsigned version if the value is not negative.
  if (value >= 0) {
    this.writeUnsignedVarint32(value);
    return;
  }

  // Write nine bytes with a _signed_ right shift so we preserve the sign bit.
  for (var i = 0; i < 9; i++) {
    this.buffer_.push((value & 0x7f) | 0x80);
    value = value >> 7;
  }

  // The above loop writes out 63 bits, so the last byte is always the sign bit
  // which is always set for negative numbers.
  this.buffer_.push(1);
};


/**
 * Encodes a 64-bit unsigned integer into its wire-format varint representation
 * and stores it in the buffer. Integers that are not representable in 64 bits
 * will be truncated.
 * @param {number} value The integer to convert.
 */
jspb.BinaryEncoder.prototype.writeUnsignedVarint64 = function(value) {
  goog.asserts.assert(value == Math.floor(value));
  goog.asserts.assert((value >= 0) &&
                      (value < jspb.BinaryConstants.TWO_TO_64));
  jspb.utils.splitInt64(value);
  this.writeSplitVarint64(jspb.utils.split64Low,
                          jspb.utils.split64High);
};


/**
 * Encodes a 64-bit signed integer into its wire-format varint representation
 * and stores it in the buffer. Integers that are not representable in 64 bits
 * will be truncated.
 * @param {number} value The integer to convert.
 */
jspb.BinaryEncoder.prototype.writeSignedVarint64 = function(value) {
  goog.asserts.assert(value == Math.floor(value));
  goog.asserts.assert((value >= -jspb.BinaryConstants.TWO_TO_63) &&
                      (value < jspb.BinaryConstants.TWO_TO_63));
  jspb.utils.splitInt64(value);
  this.writeSplitVarint64(jspb.utils.split64Low,
                          jspb.utils.split64High);
};


/**
 * Encodes a JavaScript integer into its wire-format, zigzag-encoded varint
 * representation and stores it in the buffer.
 * @param {number} value The integer to convert.
 */
jspb.BinaryEncoder.prototype.writeZigzagVarint32 = function(value) {
  goog.asserts.assert(value == Math.floor(value));
  goog.asserts.assert((value >= -jspb.BinaryConstants.TWO_TO_31) &&
                      (value < jspb.BinaryConstants.TWO_TO_31));
  this.writeUnsignedVarint32(((value << 1) ^ (value >> 31)) >>> 0);
};


/**
 * Encodes a JavaScript integer into its wire-format, zigzag-encoded varint
 * representation and stores it in the buffer. Integers not representable in 64
 * bits will be truncated.
 * @param {number} value The integer to convert.
 */
jspb.BinaryEncoder.prototype.writeZigzagVarint64 = function(value) {
  goog.asserts.assert(value == Math.floor(value));
  goog.asserts.assert((value >= -jspb.BinaryConstants.TWO_TO_63) &&
                      (value < jspb.BinaryConstants.TWO_TO_63));
  jspb.utils.splitZigzag64(value);
  this.writeSplitVarint64(jspb.utils.split64Low,
                          jspb.utils.split64High);
};


/**
 * Encodes a JavaScript decimal string into its wire-format, zigzag-encoded
 * varint representation and stores it in the buffer. Integers not representable
 * in 64 bits will be truncated.
 * @param {string} value The integer to convert.
 */
jspb.BinaryEncoder.prototype.writeZigzagVarint64String = function(value) {
  this.writeZigzagVarintHash64(jspb.utils.decimalStringToHash64(value));
};


/**
 * Writes a 64-bit hash string (8 characters @ 8 bits of data each) to the
 * buffer as a zigzag varint.
 * @param {string} hash The hash to write.
 */
jspb.BinaryEncoder.prototype.writeZigzagVarintHash64 = function(hash) {
  var self = this;
  jspb.utils.splitHash64(hash);
  jspb.utils.toZigzag64(
      jspb.utils.split64Low, jspb.utils.split64High, function(lo, hi) {
        self.writeSplitVarint64(lo >>> 0, hi >>> 0);
      });
};


/**
 * Writes an 8-bit unsigned integer to the buffer. Numbers outside the range
 * [0,2^8) will be truncated.
 * @param {number} value The value to write.
 */
jspb.BinaryEncoder.prototype.writeUint8 = function(value) {
  goog.asserts.assert(value == Math.floor(value));
  goog.asserts.assert((value >= 0) && (value < 256));
  this.buffer_.push((value >>> 0) & 0xFF);
};


/**
 * Writes a 16-bit unsigned integer to the buffer. Numbers outside the
 * range [0,2^16) will be truncated.
 * @param {number} value The value to write.
 */
jspb.BinaryEncoder.prototype.writeUint16 = function(value) {
  goog.asserts.assert(value == Math.floor(value));
  goog.asserts.assert((value >= 0) && (value < 65536));
  this.buffer_.push((value >>> 0) & 0xFF);
  this.buffer_.push((value >>> 8) & 0xFF);
};


/**
 * Writes a 32-bit unsigned integer to the buffer. Numbers outside the
 * range [0,2^32) will be truncated.
 * @param {number} value The value to write.
 */
jspb.BinaryEncoder.prototype.writeUint32 = function(value) {
  goog.asserts.assert(value == Math.floor(value));
  goog.asserts.assert((value >= 0) &&
                      (value < jspb.BinaryConstants.TWO_TO_32));
  this.buffer_.push((value >>> 0) & 0xFF);
  this.buffer_.push((value >>> 8) & 0xFF);
  this.buffer_.push((value >>> 16) & 0xFF);
  this.buffer_.push((value >>> 24) & 0xFF);
};


/**
 * Writes a 64-bit unsigned integer to the buffer. Numbers outside the
 * range [0,2^64) will be truncated.
 * @param {number} value The value to write.
 */
jspb.BinaryEncoder.prototype.writeUint64 = function(value) {
  goog.asserts.assert(value == Math.floor(value));
  goog.asserts.assert((value >= 0) &&
                      (value < jspb.BinaryConstants.TWO_TO_64));
  jspb.utils.splitUint64(value);
  this.writeUint32(jspb.utils.split64Low);
  this.writeUint32(jspb.utils.split64High);
};


/**
 * Writes an 8-bit integer to the buffer. Numbers outside the range
 * [-2^7,2^7) will be truncated.
 * @param {number} value The value to write.
 */
jspb.BinaryEncoder.prototype.writeInt8 = function(value) {
  goog.asserts.assert(value == Math.floor(value));
  goog.asserts.assert((value >= -128) && (value < 128));
  this.buffer_.push((value >>> 0) & 0xFF);
};


/**
 * Writes a 16-bit integer to the buffer. Numbers outside the range
 * [-2^15,2^15) will be truncated.
 * @param {number} value The value to write.
 */
jspb.BinaryEncoder.prototype.writeInt16 = function(value) {
  goog.asserts.assert(value == Math.floor(value));
  goog.asserts.assert((value >= -32768) && (value < 32768));
  this.buffer_.push((value >>> 0) & 0xFF);
  this.buffer_.push((value >>> 8) & 0xFF);
};


/**
 * Writes a 32-bit integer to the buffer. Numbers outside the range
 * [-2^31,2^31) will be truncated.
 * @param {number} value The value to write.
 */
jspb.BinaryEncoder.prototype.writeInt32 = function(value) {
  goog.asserts.assert(value == Math.floor(value));
  goog.asserts.assert((value >= -jspb.BinaryConstants.TWO_TO_31) &&
                      (value < jspb.BinaryConstants.TWO_TO_31));
  this.buffer_.push((value >>> 0) & 0xFF);
  this.buffer_.push((value >>> 8) & 0xFF);
  this.buffer_.push((value >>> 16) & 0xFF);
  this.buffer_.push((value >>> 24) & 0xFF);
};


/**
 * Writes a 64-bit integer to the buffer. Numbers outside the range
 * [-2^63,2^63) will be truncated.
 * @param {number} value The value to write.
 */
jspb.BinaryEncoder.prototype.writeInt64 = function(value) {
  goog.asserts.assert(value == Math.floor(value));
  goog.asserts.assert((value >= -jspb.BinaryConstants.TWO_TO_63) &&
                      (value < jspb.BinaryConstants.TWO_TO_63));
  jspb.utils.splitInt64(value);
  this.writeSplitFixed64(jspb.utils.split64Low, jspb.utils.split64High);
};


/**
 * Writes a 64-bit integer decimal strings to the buffer. Numbers outside the
 * range [-2^63,2^63) will be truncated.
 * @param {string} value The value to write.
 */
jspb.BinaryEncoder.prototype.writeInt64String = function(value) {
  goog.asserts.assert(value == Math.floor(value));
  goog.asserts.assert((+value >= -jspb.BinaryConstants.TWO_TO_63) &&
                      (+value < jspb.BinaryConstants.TWO_TO_63));
  jspb.utils.splitHash64(jspb.utils.decimalStringToHash64(value));
  this.writeSplitFixed64(jspb.utils.split64Low, jspb.utils.split64High);
};


/**
 * Writes a single-precision floating point value to the buffer. Numbers
 * requiring more than 32 bits of precision will be truncated.
 * @param {number} value The value to write.
 */
jspb.BinaryEncoder.prototype.writeFloat = function(value) {
  goog.asserts.assert(
      value === Infinity || value === -Infinity || isNaN(value) ||
      ((value >= -jspb.BinaryConstants.FLOAT32_MAX) &&
       (value <= jspb.BinaryConstants.FLOAT32_MAX)));
  jspb.utils.splitFloat32(value);
  this.writeUint32(jspb.utils.split64Low);
};


/**
 * Writes a double-precision floating point value to the buffer. As this is
 * the native format used by JavaScript, no precision will be lost.
 * @param {number} value The value to write.
 */
jspb.BinaryEncoder.prototype.writeDouble = function(value) {
  goog.asserts.assert(
      value === Infinity || value === -Infinity || isNaN(value) ||
      ((value >= -jspb.BinaryConstants.FLOAT64_MAX) &&
       (value <= jspb.BinaryConstants.FLOAT64_MAX)));
  jspb.utils.splitFloat64(value);
  this.writeUint32(jspb.utils.split64Low);
  this.writeUint32(jspb.utils.split64High);
};


/**
 * Writes a boolean value to the buffer as a varint. We allow numbers as input
 * because the JSPB code generator uses 0/1 instead of true/false to save space
 * in the string representation of the proto.
 * @param {boolean|number} value The value to write.
 */
jspb.BinaryEncoder.prototype.writeBool = function(value) {
  goog.asserts.assert(typeof value === 'boolean' || typeof value === 'number');
  this.buffer_.push(value ? 1 : 0);
};


/**
 * Writes an enum value to the buffer as a varint.
 * @param {number} value The value to write.
 */
jspb.BinaryEncoder.prototype.writeEnum = function(value) {
  goog.asserts.assert(value == Math.floor(value));
  goog.asserts.assert((value >= -jspb.BinaryConstants.TWO_TO_31) &&
                      (value < jspb.BinaryConstants.TWO_TO_31));
  this.writeSignedVarint32(value);
};


/**
 * Writes an arbitrary byte array to the buffer.
 * @param {!Uint8Array} bytes The array of bytes to write.
 */
jspb.BinaryEncoder.prototype.writeBytes = function(bytes) {
  this.buffer_.push.apply(this.buffer_, bytes);
};


/**
 * Writes a 64-bit hash string (8 characters @ 8 bits of data each) to the
 * buffer as a varint.
 * @param {string} hash The hash to write.
 */
jspb.BinaryEncoder.prototype.writeVarintHash64 = function(hash) {
  jspb.utils.splitHash64(hash);
  this.writeSplitVarint64(jspb.utils.split64Low,
                          jspb.utils.split64High);
};


/**
 * Writes a 64-bit hash string (8 characters @ 8 bits of data each) to the
 * buffer as a fixed64.
 * @param {string} hash The hash to write.
 */
jspb.BinaryEncoder.prototype.writeFixedHash64 = function(hash) {
  jspb.utils.splitHash64(hash);
  this.writeUint32(jspb.utils.split64Low);
  this.writeUint32(jspb.utils.split64High);
};


/**
 * Writes a UTF16 Javascript string to the buffer encoded as UTF8.
 * TODO(aappleby): Add support for surrogate pairs, reject unpaired surrogates.
 * @param {string} value The string to write.
 * @return {number} The number of bytes used to encode the string.
 */
jspb.BinaryEncoder.prototype.writeString = function(value) {
  var oldLength = this.buffer_.length;

  for (var i = 0; i < value.length; i++) {

    var c = value.charCodeAt(i);

    if (c < 128) {
      this.buffer_.push(c);
    } else if (c < 2048) {
      this.buffer_.push((c >> 6) | 192);
      this.buffer_.push((c & 63) | 128);
    } else if (c < 65536) {
      // Look for surrogates
      if (c >= 0xD800 && c <= 0xDBFF && i + 1 < value.length) {
        var second = value.charCodeAt(i + 1);
        if (second >= 0xDC00 && second <= 0xDFFF) { // low surrogate
          // http://mathiasbynens.be/notes/javascript-encoding#surrogate-formulae
          c = (c - 0xD800) * 0x400 + second - 0xDC00 + 0x10000;

          this.buffer_.push((c >> 18) | 240);
          this.buffer_.push(((c >> 12) & 63 ) | 128);
          this.buffer_.push(((c >> 6) & 63) | 128);
          this.buffer_.push((c & 63) | 128);
          i++;
        }
      }
      else {
        this.buffer_.push((c >> 12) | 224);
        this.buffer_.push(((c >> 6) & 63) | 128);
        this.buffer_.push((c & 63) | 128);
      }
    }
  }

  var length = this.buffer_.length - oldLength;
  return length;
};

//javascript/apps/jspb/binary/reader.js
/**
 * @fileoverview This file contains utilities for converting binary,
 * wire-format protocol buffers into Javascript data structures.
 *
 * jspb's BinaryReader class wraps the BinaryDecoder class to add methods
 * that understand the protocol buffer syntax and can do the type checking and
 * bookkeeping necessary to parse trees of nested messages.
 *
 * Major caveat - Users of this library _must_ keep their Javascript proto
 * parsing code in sync with the original .proto file - presumably you'll be
 * using the typed jspb code generator, but if you bypass that you'll need
 * to keep things in sync by hand.
 *
 * @author aappleby@google.com (Austin Appleby)
 */

goog.provide('jspb.BinaryReader');

goog.require('goog.asserts');
goog.require('jspb.BinaryConstants');
goog.require('jspb.BinaryDecoder');
goog.require('jspb.utils');



/**
 * BinaryReader implements the decoders for all the wire types specified in
 * https://developers.google.com/protocol-buffers/docs/encoding.
 *
 * @param {jspb.ByteSource=} opt_bytes The bytes we're reading from.
 * @param {number=} opt_start The optional offset to start reading at.
 * @param {number=} opt_length The optional length of the block to read -
 *     we'll throw an assertion if we go off the end of the block.
 * @constructor
 * @struct
 */
jspb.BinaryReader = function(opt_bytes, opt_start, opt_length) {
  /**
   * Wire-format decoder.
   * @private {!jspb.BinaryDecoder}
   */
  this.decoder_ = jspb.BinaryDecoder.alloc(opt_bytes, opt_start, opt_length);

  /**
   * Cursor immediately before the field tag.
   * @private {number}
   */
  this.fieldCursor_ = this.decoder_.getCursor();

  /**
   * Field number of the next field in the buffer, filled in by nextField().
   * @private {number}
   */
  this.nextField_ = jspb.BinaryConstants.INVALID_FIELD_NUMBER;

  /**
   * Wire type of the next proto field in the buffer, filled in by
   * nextField().
   * @private {jspb.BinaryConstants.WireType}
   */
  this.nextWireType_ = jspb.BinaryConstants.WireType.INVALID;

  /**
   * Set to true if this reader encountered an error due to corrupt data.
   * @private {boolean}
   */
  this.error_ = false;

  /**
   * User-defined reader callbacks.
   * @private {?Object<string, function(!jspb.BinaryReader):*>}
   */
  this.readCallbacks_ = null;
};


/**
 * Global pool of BinaryReader instances.
 * @private {!Array<!jspb.BinaryReader>}
 */
jspb.BinaryReader.instanceCache_ = [];


/**
 * Pops an instance off the instance cache, or creates one if the cache is
 * empty.
 * @param {jspb.ByteSource=} opt_bytes The bytes we're reading from.
 * @param {number=} opt_start The optional offset to start reading at.
 * @param {number=} opt_length The optional length of the block to read -
 *     we'll throw an assertion if we go off the end of the block.
 * @return {!jspb.BinaryReader}
 */
jspb.BinaryReader.alloc =
    function(opt_bytes, opt_start, opt_length) {
  if (jspb.BinaryReader.instanceCache_.length) {
    var newReader = jspb.BinaryReader.instanceCache_.pop();
    if (opt_bytes) {
      newReader.decoder_.setBlock(opt_bytes, opt_start, opt_length);
    }
    return newReader;
  } else {
    return new jspb.BinaryReader(opt_bytes, opt_start, opt_length);
  }
};


/**
 * Alias for the above method.
 * @param {jspb.ByteSource=} opt_bytes The bytes we're reading from.
 * @param {number=} opt_start The optional offset to start reading at.
 * @param {number=} opt_length The optional length of the block to read -
 *     we'll throw an assertion if we go off the end of the block.
 * @return {!jspb.BinaryReader}
 */
jspb.BinaryReader.prototype.alloc = jspb.BinaryReader.alloc;


/**
 * Puts this instance back in the instance cache.
 */
jspb.BinaryReader.prototype.free = function() {
  this.decoder_.clear();
  this.nextField_ = jspb.BinaryConstants.INVALID_FIELD_NUMBER;
  this.nextWireType_ = jspb.BinaryConstants.WireType.INVALID;
  this.error_ = false;
  this.readCallbacks_ = null;

  if (jspb.BinaryReader.instanceCache_.length < 100) {
    jspb.BinaryReader.instanceCache_.push(this);
  }
};


/**
 * Returns the cursor immediately before the current field's tag.
 * @return {number} The internal read cursor.
 */
jspb.BinaryReader.prototype.getFieldCursor = function() {
  return this.fieldCursor_;
};


/**
 * Returns the internal read cursor.
 * @return {number} The internal read cursor.
 */
jspb.BinaryReader.prototype.getCursor = function() {
  return this.decoder_.getCursor();
};


/**
 * Returns the raw buffer.
 * @return {?Uint8Array} The raw buffer.
 */
jspb.BinaryReader.prototype.getBuffer = function() {
  return this.decoder_.getBuffer();
};


/**
 * @return {number} The field number of the next field in the buffer, or
 *     INVALID_FIELD_NUMBER if there is no next field.
 */
jspb.BinaryReader.prototype.getFieldNumber = function() {
  return this.nextField_;
};


/**
 * @return {jspb.BinaryConstants.WireType} The wire type of the next field
 *     in the stream, or WireType.INVALID if there is no next field.
 */
jspb.BinaryReader.prototype.getWireType = function() {
  return this.nextWireType_;
};


/**
 * @return {boolean} Whether the current wire type is an end-group tag. Used as
 * an exit condition in decoder loops in generated code.
 */
jspb.BinaryReader.prototype.isEndGroup = function() {
  return this.nextWireType_ == jspb.BinaryConstants.WireType.END_GROUP;
};


/**
 * Returns true if this reader hit an error due to corrupt data.
 * @return {boolean}
 */
jspb.BinaryReader.prototype.getError = function() {
  return this.error_ || this.decoder_.getError();
};


/**
 * Points this reader at a new block of bytes.
 * @param {!Uint8Array} bytes The block of bytes we're reading from.
 * @param {number} start The offset to start reading at.
 * @param {number} length The length of the block to read.
 */
jspb.BinaryReader.prototype.setBlock = function(bytes, start, length) {
  this.decoder_.setBlock(bytes, start, length);
  this.nextField_ = jspb.BinaryConstants.INVALID_FIELD_NUMBER;
  this.nextWireType_ = jspb.BinaryConstants.WireType.INVALID;
};


/**
 * Rewinds the stream cursor to the beginning of the buffer and resets all
 * internal state.
 */
jspb.BinaryReader.prototype.reset = function() {
  this.decoder_.reset();
  this.nextField_ = jspb.BinaryConstants.INVALID_FIELD_NUMBER;
  this.nextWireType_ = jspb.BinaryConstants.WireType.INVALID;
};


/**
 * Advances the stream cursor by the given number of bytes.
 * @param {number} count The number of bytes to advance by.
 */
jspb.BinaryReader.prototype.advance = function(count) {
  this.decoder_.advance(count);
};


/**
 * Reads the next field header in the stream if there is one, returns true if
 * we saw a valid field header or false if we've read the whole stream.
 * Throws an error if we encountered a deprecated START_GROUP/END_GROUP field.
 * @return {boolean} True if the stream contains more fields.
 */
jspb.BinaryReader.prototype.nextField = function() {
  // If we're at the end of the block, there are no more fields.
  if (this.decoder_.atEnd()) {
    return false;
  }

  // If we hit an error decoding the previous field, stop now before we
  // try to decode anything else
  if (this.getError()) {
    goog.asserts.fail('Decoder hit an error');
    return false;
  }

  // Otherwise just read the header of the next field.
  this.fieldCursor_ = this.decoder_.getCursor();
  var header = this.decoder_.readUnsignedVarint32();

  var nextField = header >>> 3;
  var nextWireType = /** @type {jspb.BinaryConstants.WireType} */
      (header & 0x7);

  // If the wire type isn't one of the valid ones, something's broken.
  if (nextWireType != jspb.BinaryConstants.WireType.VARINT &&
      nextWireType != jspb.BinaryConstants.WireType.FIXED32 &&
      nextWireType != jspb.BinaryConstants.WireType.FIXED64 &&
      nextWireType != jspb.BinaryConstants.WireType.DELIMITED &&
      nextWireType != jspb.BinaryConstants.WireType.START_GROUP &&
      nextWireType != jspb.BinaryConstants.WireType.END_GROUP) {
    goog.asserts.fail(
        'Invalid wire type: %s (at position %s)', nextWireType,
        this.fieldCursor_);
    this.error_ = true;
    return false;
  }

  this.nextField_ = nextField;
  this.nextWireType_ = nextWireType;

  return true;
};


/**
 * Winds the reader back to just before this field's header.
 */
jspb.BinaryReader.prototype.unskipHeader = function() {
  this.decoder_.unskipVarint((this.nextField_ << 3) | this.nextWireType_);
};


/**
 * Skips all contiguous fields whose header matches the one we just read.
 */
jspb.BinaryReader.prototype.skipMatchingFields = function() {
  var field = this.nextField_;
  this.unskipHeader();

  while (this.nextField() && (this.getFieldNumber() == field)) {
    this.skipField();
  }

  if (!this.decoder_.atEnd()) {
    this.unskipHeader();
  }
};


/**
 * Skips over the next varint field in the binary stream.
 */
jspb.BinaryReader.prototype.skipVarintField = function() {
  if (this.nextWireType_ != jspb.BinaryConstants.WireType.VARINT) {
    goog.asserts.fail('Invalid wire type for skipVarintField');
    this.skipField();
    return;
  }

  this.decoder_.skipVarint();
};


/**
 * Skips over the next delimited field in the binary stream.
 */
jspb.BinaryReader.prototype.skipDelimitedField = function() {
  if (this.nextWireType_ != jspb.BinaryConstants.WireType.DELIMITED) {
    goog.asserts.fail('Invalid wire type for skipDelimitedField');
    this.skipField();
    return;
  }

  var length = this.decoder_.readUnsignedVarint32();
  this.decoder_.advance(length);
};


/**
 * Skips over the next fixed32 field in the binary stream.
 */
jspb.BinaryReader.prototype.skipFixed32Field = function() {
  if (this.nextWireType_ != jspb.BinaryConstants.WireType.FIXED32) {
    goog.asserts.fail('Invalid wire type for skipFixed32Field');
    this.skipField();
    return;
  }

  this.decoder_.advance(4);
};


/**
 * Skips over the next fixed64 field in the binary stream.
 */
jspb.BinaryReader.prototype.skipFixed64Field = function() {
  if (this.nextWireType_ != jspb.BinaryConstants.WireType.FIXED64) {
    goog.asserts.fail('Invalid wire type for skipFixed64Field');
    this.skipField();
    return;
  }

  this.decoder_.advance(8);
};


/**
 * Skips over the next group field in the binary stream.
 */
jspb.BinaryReader.prototype.skipGroup = function() {
  var previousField = this.nextField_;
  do {
    if (!this.nextField()) {
      goog.asserts.fail('Unmatched start-group tag: stream EOF');
      this.error_ = true;
      return;
    }
    if (this.nextWireType_ ==
               jspb.BinaryConstants.WireType.END_GROUP) {
      // Group end: check that it matches top-of-stack.
      if (this.nextField_ != previousField) {
        goog.asserts.fail('Unmatched end-group tag');
        this.error_ = true;
        return;
      }
      return;
    }
    this.skipField();
  } while (true);
};


/**
 * Skips over the next field in the binary stream - this is useful if we're
 * decoding a message that contain unknown fields.
 */
jspb.BinaryReader.prototype.skipField = function() {
  switch (this.nextWireType_) {
    case jspb.BinaryConstants.WireType.VARINT:
      this.skipVarintField();
      break;
    case jspb.BinaryConstants.WireType.FIXED64:
      this.skipFixed64Field();
      break;
    case jspb.BinaryConstants.WireType.DELIMITED:
      this.skipDelimitedField();
      break;
    case jspb.BinaryConstants.WireType.FIXED32:
      this.skipFixed32Field();
      break;
    case jspb.BinaryConstants.WireType.START_GROUP:
      this.skipGroup();
      break;
    default:
      this.error_ = true;
      goog.asserts.fail('Invalid wire encoding for field.');
  }
};


/**
 * Registers a user-defined read callback.
 * @param {string} callbackName
 * @param {function(!jspb.BinaryReader):*} callback
 */
jspb.BinaryReader.prototype.registerReadCallback = function(
    callbackName, callback) {
  if (this.readCallbacks_ === null) {
    this.readCallbacks_ = {};
  }
  goog.asserts.assert(!this.readCallbacks_[callbackName]);
  this.readCallbacks_[callbackName] = callback;
};


/**
 * Runs a registered read callback.
 * @param {string} callbackName The name the callback is registered under.
 * @return {*} The value returned by the callback.
 */
jspb.BinaryReader.prototype.runReadCallback = function(callbackName) {
  goog.asserts.assert(this.readCallbacks_ !== null);
  var callback = this.readCallbacks_[callbackName];
  goog.asserts.assert(callback);
  return callback(this);
};


/**
 * Reads a field of any valid non-message type from the binary stream.
 * @param {jspb.BinaryConstants.FieldType} fieldType
 * @return {!jspb.AnyFieldType}
 */
jspb.BinaryReader.prototype.readAny = function(fieldType) {
  this.nextWireType_ = jspb.BinaryConstants.FieldTypeToWireType(fieldType);
  var fieldTypes = jspb.BinaryConstants.FieldType;
  switch (fieldType) {
    case fieldTypes.DOUBLE:
      return this.readDouble();
    case fieldTypes.FLOAT:
      return this.readFloat();
    case fieldTypes.INT64:
      return this.readInt64();
    case fieldTypes.UINT64:
      return this.readUint64();
    case fieldTypes.INT32:
      return this.readInt32();
    case fieldTypes.FIXED64:
      return this.readFixed64();
    case fieldTypes.FIXED32:
      return this.readFixed32();
    case fieldTypes.BOOL:
      return this.readBool();
    case fieldTypes.STRING:
      return this.readString();
    case fieldTypes.GROUP:
      goog.asserts.fail('Group field type not supported in readAny()');
    case fieldTypes.MESSAGE:
      goog.asserts.fail('Message field type not supported in readAny()');
    case fieldTypes.BYTES:
      return this.readBytes();
    case fieldTypes.UINT32:
      return this.readUint32();
    case fieldTypes.ENUM:
      return this.readEnum();
    case fieldTypes.SFIXED32:
      return this.readSfixed32();
    case fieldTypes.SFIXED64:
      return this.readSfixed64();
    case fieldTypes.SINT32:
      return this.readSint32();
    case fieldTypes.SINT64:
      return this.readSint64();
    case fieldTypes.FHASH64:
      return this.readFixedHash64();
    case fieldTypes.VHASH64:
      return this.readVarintHash64();
    default:
      goog.asserts.fail('Invalid field type in readAny()');
  }
  return 0;
};


/**
 * Deserialize a proto into the provided message object using the provided
 * reader function. This function is templated as we currently have one client
 * who is using manual deserialization instead of the code-generated versions.
 * @template T
 * @param {T} message
 * @param {function(T, !jspb.BinaryReader)} reader
 */
jspb.BinaryReader.prototype.readMessage = function(message, reader) {
  goog.asserts.assert(
      this.nextWireType_ == jspb.BinaryConstants.WireType.DELIMITED);

  // Save the current endpoint of the decoder and move it to the end of the
  // embedded message.
  var oldEnd = this.decoder_.getEnd();
  var length = this.decoder_.readUnsignedVarint32();
  var newEnd = this.decoder_.getCursor() + length;
  this.decoder_.setEnd(newEnd);

  // Deserialize the embedded message.
  reader(message, this);

  // Advance the decoder past the embedded message and restore the endpoint.
  this.decoder_.setCursor(newEnd);
  this.decoder_.setEnd(oldEnd);
};


/**
 * Deserialize a proto into the provided message object using the provided
 * reader function, assuming that the message is serialized as a group
 * with the given tag.
 * @template T
 * @param {number} field
 * @param {T} message
 * @param {function(T, !jspb.BinaryReader)} reader
 */
jspb.BinaryReader.prototype.readGroup =
    function(field, message, reader) {
  // Ensure that the wire type is correct.
  goog.asserts.assert(
      this.nextWireType_ == jspb.BinaryConstants.WireType.START_GROUP);
  // Ensure that the field number is correct.
  goog.asserts.assert(this.nextField_ == field);

  // Deserialize the message. The deserialization will stop at an END_GROUP tag.
  reader(message, this);

  if (!this.error_ &&
      this.nextWireType_ != jspb.BinaryConstants.WireType.END_GROUP) {
    goog.asserts.fail('Group submessage did not end with an END_GROUP tag');
    this.error_ = true;
  }
};


/**
 * Return a decoder that wraps the current delimited field.
 * @return {!jspb.BinaryDecoder}
 */
jspb.BinaryReader.prototype.getFieldDecoder = function() {
  goog.asserts.assert(
      this.nextWireType_ == jspb.BinaryConstants.WireType.DELIMITED);

  var length = this.decoder_.readUnsignedVarint32();
  var start = this.decoder_.getCursor();
  var end = start + length;

  var innerDecoder =
      jspb.BinaryDecoder.alloc(this.decoder_.getBuffer(), start, length);
  this.decoder_.setCursor(end);
  return innerDecoder;
};


/**
 * Reads a signed 32-bit integer field from the binary stream, or throws an
 * error if the next field in the stream is not of the correct wire type.
 *
 * @return {number} The value of the signed 32-bit integer field.
 */
jspb.BinaryReader.prototype.readInt32 = function() {
  goog.asserts.assert(
      this.nextWireType_ == jspb.BinaryConstants.WireType.VARINT);
  return this.decoder_.readSignedVarint32();
};


/**
 * Reads a signed 32-bit integer field from the binary stream, or throws an
 * error if the next field in the stream is not of the correct wire type.
 *
 * Returns the value as a string.
 *
 * @return {string} The value of the signed 32-bit integer field as a decimal
 * string.
 */
jspb.BinaryReader.prototype.readInt32String = function() {
  goog.asserts.assert(
      this.nextWireType_ == jspb.BinaryConstants.WireType.VARINT);
  return this.decoder_.readSignedVarint32String();
};


/**
 * Reads a signed 64-bit integer field from the binary stream, or throws an
 * error if the next field in the stream is not of the correct wire type.
 *
 * @return {number} The value of the signed 64-bit integer field.
 */
jspb.BinaryReader.prototype.readInt64 = function() {
  goog.asserts.assert(
      this.nextWireType_ == jspb.BinaryConstants.WireType.VARINT);
  return this.decoder_.readSignedVarint64();
};


/**
 * Reads a signed 64-bit integer field from the binary stream, or throws an
 * error if the next field in the stream is not of the correct wire type.
 *
 * Returns the value as a string.
 *
 * @return {string} The value of the signed 64-bit integer field as a decimal
 * string.
 */
jspb.BinaryReader.prototype.readInt64String = function() {
  goog.asserts.assert(
      this.nextWireType_ == jspb.BinaryConstants.WireType.VARINT);
  return this.decoder_.readSignedVarint64String();
};


/**
 * Reads an unsigned 32-bit integer field from the binary stream, or throws an
 * error if the next field in the stream is not of the correct wire type.
 *
 * @return {number} The value of the unsigned 32-bit integer field.
 */
jspb.BinaryReader.prototype.readUint32 = function() {
  goog.asserts.assert(
      this.nextWireType_ == jspb.BinaryConstants.WireType.VARINT);
  return this.decoder_.readUnsignedVarint32();
};


/**
 * Reads an unsigned 32-bit integer field from the binary stream, or throws an
 * error if the next field in the stream is not of the correct wire type.
 *
 * Returns the value as a string.
 *
 * @return {string} The value of the unsigned 32-bit integer field as a decimal
 * string.
 */
jspb.BinaryReader.prototype.readUint32String = function() {
  goog.asserts.assert(
      this.nextWireType_ == jspb.BinaryConstants.WireType.VARINT);
  return this.decoder_.readUnsignedVarint32String();
};


/**
 * Reads an unsigned 64-bit integer field from the binary stream, or throws an
 * error if the next field in the stream is not of the correct wire type.
 *
 * @return {number} The value of the unsigned 64-bit integer field.
 */
jspb.BinaryReader.prototype.readUint64 = function() {
  goog.asserts.assert(
      this.nextWireType_ == jspb.BinaryConstants.WireType.VARINT);
  return this.decoder_.readUnsignedVarint64();
};


/**
 * Reads an unsigned 64-bit integer field from the binary stream, or throws an
 * error if the next field in the stream is not of the correct wire type.
 *
 * Returns the value as a string.
 *
 * @return {string} The value of the unsigned 64-bit integer field as a decimal
 * string.
 */
jspb.BinaryReader.prototype.readUint64String = function() {
  goog.asserts.assert(
      this.nextWireType_ == jspb.BinaryConstants.WireType.VARINT);
  return this.decoder_.readUnsignedVarint64String();
};


/**
 * Reads a signed zigzag-encoded 32-bit integer field from the binary stream,
 * or throws an error if the next field in the stream is not of the correct
 * wire type.
 *
 * @return {number} The value of the signed 32-bit integer field.
 */
jspb.BinaryReader.prototype.readSint32 = function() {
  goog.asserts.assert(
      this.nextWireType_ == jspb.BinaryConstants.WireType.VARINT);
  return this.decoder_.readZigzagVarint32();
};


/**
 * Reads a signed zigzag-encoded 64-bit integer field from the binary stream,
 * or throws an error if the next field in the stream is not of the correct
 * wire type.
 *
 * @return {number} The value of the signed 64-bit integer field.
 */
jspb.BinaryReader.prototype.readSint64 = function() {
  goog.asserts.assert(
      this.nextWireType_ == jspb.BinaryConstants.WireType.VARINT);
  return this.decoder_.readZigzagVarint64();
};


/**
 * Reads a signed zigzag-encoded 64-bit integer field from the binary stream,
 * or throws an error if the next field in the stream is not of the correct
 * wire type.
 *
 * @return {string} The value of the signed 64-bit integer field as a decimal string.
 */
jspb.BinaryReader.prototype.readSint64String = function() {
  goog.asserts.assert(
      this.nextWireType_ == jspb.BinaryConstants.WireType.VARINT);
  return this.decoder_.readZigzagVarint64String();
};


/**
 * Reads an unsigned 32-bit fixed-length integer fiield from the binary stream,
 * or throws an error if the next field in the stream is not of the correct
 * wire type.
 *
 * @return {number} The value of the double field.
 */
jspb.BinaryReader.prototype.readFixed32 = function() {
  goog.asserts.assert(
      this.nextWireType_ == jspb.BinaryConstants.WireType.FIXED32);
  return this.decoder_.readUint32();
};


/**
 * Reads an unsigned 64-bit fixed-length integer fiield from the binary stream,
 * or throws an error if the next field in the stream is not of the correct
 * wire type.
 *
 * @return {number} The value of the float field.
 */
jspb.BinaryReader.prototype.readFixed64 = function() {
  goog.asserts.assert(
      this.nextWireType_ == jspb.BinaryConstants.WireType.FIXED64);
  return this.decoder_.readUint64();
};


/**
 * Reads a signed 64-bit integer field from the binary stream as a string, or
 * throws an error if the next field in the stream is not of the correct wire
 * type.
 *
 * Returns the value as a string.
 *
 * @return {string} The value of the unsigned 64-bit integer field as a decimal
 * string.
 */
jspb.BinaryReader.prototype.readFixed64String = function() {
  goog.asserts.assert(
      this.nextWireType_ == jspb.BinaryConstants.WireType.FIXED64);
  return this.decoder_.readUint64String();
};


/**
 * Reads a signed 32-bit fixed-length integer fiield from the binary stream, or
 * throws an error if the next field in the stream is not of the correct wire
 * type.
 *
 * @return {number} The value of the signed 32-bit integer field.
 */
jspb.BinaryReader.prototype.readSfixed32 = function() {
  goog.asserts.assert(
      this.nextWireType_ == jspb.BinaryConstants.WireType.FIXED32);
  return this.decoder_.readInt32();
};


/**
 * Reads a signed 32-bit fixed-length integer fiield from the binary stream, or
 * throws an error if the next field in the stream is not of the correct wire
 * type.
 *
 * @return {string} The value of the signed 32-bit integer field as a decimal
 * string.
 */
jspb.BinaryReader.prototype.readSfixed32String = function() {
  goog.asserts.assert(
      this.nextWireType_ == jspb.BinaryConstants.WireType.FIXED32);
  return this.decoder_.readInt32().toString();
};


/**
 * Reads a signed 64-bit fixed-length integer fiield from the binary stream, or
 * throws an error if the next field in the stream is not of the correct wire
 * type.
 *
 * @return {number} The value of the sfixed64 field.
 */
jspb.BinaryReader.prototype.readSfixed64 = function() {
  goog.asserts.assert(
      this.nextWireType_ == jspb.BinaryConstants.WireType.FIXED64);
  return this.decoder_.readInt64();
};


/**
 * Reads a signed 64-bit fixed-length integer fiield from the binary stream, or
 * throws an error if the next field in the stream is not of the correct wire
 * type.
 *
 * Returns the value as a string.
 *
 * @return {string} The value of the sfixed64 field as a decimal string.
 */
jspb.BinaryReader.prototype.readSfixed64String = function() {
  goog.asserts.assert(
      this.nextWireType_ == jspb.BinaryConstants.WireType.FIXED64);
  return this.decoder_.readInt64String();
};


/**
 * Reads a 32-bit floating-point field from the binary stream, or throws an
 * error if the next field in the stream is not of the correct wire type.
 *
 * @return {number} The value of the float field.
 */
jspb.BinaryReader.prototype.readFloat = function() {
  goog.asserts.assert(
      this.nextWireType_ == jspb.BinaryConstants.WireType.FIXED32);
  return this.decoder_.readFloat();
};


/**
 * Reads a 64-bit floating-point field from the binary stream, or throws an
 * error if the next field in the stream is not of the correct wire type.
 *
 * @return {number} The value of the double field.
 */
jspb.BinaryReader.prototype.readDouble = function() {
  goog.asserts.assert(
      this.nextWireType_ == jspb.BinaryConstants.WireType.FIXED64);
  return this.decoder_.readDouble();
};


/**
 * Reads a boolean field from the binary stream, or throws an error if the next
 * field in the stream is not of the correct wire type.
 *
 * @return {boolean} The value of the boolean field.
 */
jspb.BinaryReader.prototype.readBool = function() {
  goog.asserts.assert(
      this.nextWireType_ == jspb.BinaryConstants.WireType.VARINT);
  return !!this.decoder_.readUnsignedVarint32();
};


/**
 * Reads an enum field from the binary stream, or throws an error if the next
 * field in the stream is not of the correct wire type.
 *
 * @return {number} The value of the enum field.
 */
jspb.BinaryReader.prototype.readEnum = function() {
  goog.asserts.assert(
      this.nextWireType_ == jspb.BinaryConstants.WireType.VARINT);
  return this.decoder_.readSignedVarint64();
};


/**
 * Reads a string field from the binary stream, or throws an error if the next
 * field in the stream is not of the correct wire type.
 *
 * @return {string} The value of the string field.
 */
jspb.BinaryReader.prototype.readString = function() {
  goog.asserts.assert(
      this.nextWireType_ == jspb.BinaryConstants.WireType.DELIMITED);
  var length = this.decoder_.readUnsignedVarint32();
  return this.decoder_.readString(length);
};


/**
 * Reads a length-prefixed block of bytes from the binary stream, or returns
 * null if the next field in the stream has an invalid length value.
 *
 * @return {!Uint8Array} The block of bytes.
 */
jspb.BinaryReader.prototype.readBytes = function() {
  goog.asserts.assert(
      this.nextWireType_ == jspb.BinaryConstants.WireType.DELIMITED);
  var length = this.decoder_.readUnsignedVarint32();
  return this.decoder_.readBytes(length);
};


/**
 * Reads a 64-bit varint or fixed64 field from the stream and returns it as an
 * 8-character Unicode string for use as a hash table key, or throws an error
 * if the next field in the stream is not of the correct wire type.
 *
 * @return {string} The hash value.
 */
jspb.BinaryReader.prototype.readVarintHash64 = function() {
  goog.asserts.assert(
      this.nextWireType_ == jspb.BinaryConstants.WireType.VARINT);
  return this.decoder_.readVarintHash64();
};


/**
 * Reads an sint64 field from the stream and returns it as an 8-character
 * Unicode string for use as a hash table key, or throws an error if the next
 * field in the stream is not of the correct wire type.
 *
 * @return {string} The hash value.
 */
jspb.BinaryReader.prototype.readSintHash64 = function() {
  goog.asserts.assert(
      this.nextWireType_ == jspb.BinaryConstants.WireType.VARINT);
  return this.decoder_.readZigzagVarintHash64();
};


/**
 * Reads a 64-bit varint field from the stream and invokes `convert` to produce
 * the return value, or throws an error if the next field in the stream is not
 * of the correct wire type.
 *
 * @param {function(number, number): T} convert Conversion function to produce
 *     the result value, takes parameters (lowBits, highBits).
 * @return {T}
 * @template T
 */
jspb.BinaryReader.prototype.readSplitVarint64 = function(convert) {
  goog.asserts.assert(
      this.nextWireType_ == jspb.BinaryConstants.WireType.VARINT);
  return this.decoder_.readSplitVarint64(convert);
};


/**
 * Reads a 64-bit zig-zag varint field from the stream and invokes `convert` to
 * produce the return value, or throws an error if the next field in the stream
 * is not of the correct wire type.
 *
 * @param {function(number, number): T} convert Conversion function to produce
 *     the result value, takes parameters (lowBits, highBits).
 * @return {T}
 * @template T
 */
jspb.BinaryReader.prototype.readSplitZigzagVarint64 = function(convert) {
  goog.asserts.assert(
      this.nextWireType_ == jspb.BinaryConstants.WireType.VARINT);
  return this.decoder_.readSplitVarint64(function(lowBits, highBits) {
    return jspb.utils.fromZigzag64(lowBits, highBits, convert);
  });
};


/**
 * Reads a 64-bit varint or fixed64 field from the stream and returns it as a
 * 8-character Unicode string for use as a hash table key, or throws an error
 * if the next field in the stream is not of the correct wire type.
 *
 * @return {string} The hash value.
 */
jspb.BinaryReader.prototype.readFixedHash64 = function() {
  goog.asserts.assert(
      this.nextWireType_ == jspb.BinaryConstants.WireType.FIXED64);
  return this.decoder_.readFixedHash64();
};


/**
 * Reads a 64-bit fixed64 field from the stream and invokes `convert`
 * to produce the return value, or throws an error if the next field in the
 * stream is not of the correct wire type.
 *
 * @param {function(number, number): T} convert Conversion function to produce
 *     the result value, takes parameters (lowBits, highBits).
 * @return {T}
 * @template T
 */
jspb.BinaryReader.prototype.readSplitFixed64 = function(convert) {
  goog.asserts.assert(
      this.nextWireType_ == jspb.BinaryConstants.WireType.FIXED64);
  return this.decoder_.readSplitFixed64(convert);
};


/**
 * Reads a packed scalar field using the supplied raw reader function.
 * @param {function(this:jspb.BinaryDecoder)} decodeMethod
 * @return {!Array}
 * @private
 */
jspb.BinaryReader.prototype.readPackedField_ = function(decodeMethod) {
  goog.asserts.assert(
      this.nextWireType_ == jspb.BinaryConstants.WireType.DELIMITED);
  var length = this.decoder_.readUnsignedVarint32();
  var end = this.decoder_.getCursor() + length;
  var result = [];
  while (this.decoder_.getCursor() < end) {
    // TODO(aappleby): .call is slow
    result.push(decodeMethod.call(this.decoder_));
  }
  return result;
};


/**
 * Reads a packed int32 field, which consists of a length header and a list of
 * signed varints.
 * @return {!Array<number>}
 */
jspb.BinaryReader.prototype.readPackedInt32 = function() {
  return this.readPackedField_(this.decoder_.readSignedVarint32);
};


/**
 * Reads a packed int32 field, which consists of a length header and a list of
 * signed varints. Returns a list of strings.
 * @return {!Array<string>}
 */
jspb.BinaryReader.prototype.readPackedInt32String = function() {
  return this.readPackedField_(this.decoder_.readSignedVarint32String);
};


/**
 * Reads a packed int64 field, which consists of a length header and a list of
 * signed varints.
 * @return {!Array<number>}
 */
jspb.BinaryReader.prototype.readPackedInt64 = function() {
  return this.readPackedField_(this.decoder_.readSignedVarint64);
};


/**
 * Reads a packed int64 field, which consists of a length header and a list of
 * signed varints. Returns a list of strings.
 * @return {!Array<string>}
 */
jspb.BinaryReader.prototype.readPackedInt64String = function() {
  return this.readPackedField_(this.decoder_.readSignedVarint64String);
};


/**
 * Reads a packed uint32 field, which consists of a length header and a list of
 * unsigned varints.
 * @return {!Array<number>}
 */
jspb.BinaryReader.prototype.readPackedUint32 = function() {
  return this.readPackedField_(this.decoder_.readUnsignedVarint32);
};


/**
 * Reads a packed uint32 field, which consists of a length header and a list of
 * unsigned varints. Returns a list of strings.
 * @return {!Array<string>}
 */
jspb.BinaryReader.prototype.readPackedUint32String = function() {
  return this.readPackedField_(this.decoder_.readUnsignedVarint32String);
};


/**
 * Reads a packed uint64 field, which consists of a length header and a list of
 * unsigned varints.
 * @return {!Array<number>}
 */
jspb.BinaryReader.prototype.readPackedUint64 = function() {
  return this.readPackedField_(this.decoder_.readUnsignedVarint64);
};


/**
 * Reads a packed uint64 field, which consists of a length header and a list of
 * unsigned varints. Returns a list of strings.
 * @return {!Array<string>}
 */
jspb.BinaryReader.prototype.readPackedUint64String = function() {
  return this.readPackedField_(this.decoder_.readUnsignedVarint64String);
};


/**
 * Reads a packed sint32 field, which consists of a length header and a list of
 * zigzag varints.
 * @return {!Array<number>}
 */
jspb.BinaryReader.prototype.readPackedSint32 = function() {
  return this.readPackedField_(this.decoder_.readZigzagVarint32);
};


/**
 * Reads a packed sint64 field, which consists of a length header and a list of
 * zigzag varints.
 * @return {!Array<number>}
 */
jspb.BinaryReader.prototype.readPackedSint64 = function() {
  return this.readPackedField_(this.decoder_.readZigzagVarint64);
};


/**
 * Reads a packed sint64 field, which consists of a length header and a list of
 * zigzag varints.  Returns a list of strings.
 * @return {!Array<string>}
 */
jspb.BinaryReader.prototype.readPackedSint64String = function() {
  return this.readPackedField_(this.decoder_.readZigzagVarint64String);
};


/**
 * Reads a packed fixed32 field, which consists of a length header and a list
 * of unsigned 32-bit ints.
 * @return {!Array<number>}
 */
jspb.BinaryReader.prototype.readPackedFixed32 = function() {
  return this.readPackedField_(this.decoder_.readUint32);
};


/**
 * Reads a packed fixed64 field, which consists of a length header and a list
 * of unsigned 64-bit ints.
 * @return {!Array<number>}
 */
jspb.BinaryReader.prototype.readPackedFixed64 = function() {
  return this.readPackedField_(this.decoder_.readUint64);
};


/**
 * Reads a packed fixed64 field, which consists of a length header and a list
 * of unsigned 64-bit ints.  Returns a list of strings.
 * @return {!Array<number>}
 */
jspb.BinaryReader.prototype.readPackedFixed64String = function() {
  return this.readPackedField_(this.decoder_.readUint64String);
};


/**
 * Reads a packed sfixed32 field, which consists of a length header and a list
 * of 32-bit ints.
 * @return {!Array<number>}
 */
jspb.BinaryReader.prototype.readPackedSfixed32 = function() {
  return this.readPackedField_(this.decoder_.readInt32);
};


/**
 * Reads a packed sfixed64 field, which consists of a length header and a list
 * of 64-bit ints.
 * @return {!Array<number>}
 */
jspb.BinaryReader.prototype.readPackedSfixed64 = function() {
  return this.readPackedField_(this.decoder_.readInt64);
};


/**
 * Reads a packed sfixed64 field, which consists of a length header and a list
 * of 64-bit ints.  Returns a list of strings.
 * @return {!Array<string>}
 */
jspb.BinaryReader.prototype.readPackedSfixed64String = function() {
  return this.readPackedField_(this.decoder_.readInt64String);
};


/**
 * Reads a packed float field, which consists of a length header and a list of
 * floats.
 * @return {!Array<number>}
 */
jspb.BinaryReader.prototype.readPackedFloat = function() {
  return this.readPackedField_(this.decoder_.readFloat);
};


/**
 * Reads a packed double field, which consists of a length header and a list of
 * doubles.
 * @return {!Array<number>}
 */
jspb.BinaryReader.prototype.readPackedDouble = function() {
  return this.readPackedField_(this.decoder_.readDouble);
};


/**
 * Reads a packed bool field, which consists of a length header and a list of
 * unsigned varints.
 * @return {!Array<boolean>}
 */
jspb.BinaryReader.prototype.readPackedBool = function() {
  return this.readPackedField_(this.decoder_.readBool);
};


/**
 * Reads a packed enum field, which consists of a length header and a list of
 * unsigned varints.
 * @return {!Array<number>}
 */
jspb.BinaryReader.prototype.readPackedEnum = function() {
  return this.readPackedField_(this.decoder_.readEnum);
};


/**
 * Reads a packed varint hash64 field, which consists of a length header and a
 * list of varint hash64s.
 * @return {!Array<string>}
 */
jspb.BinaryReader.prototype.readPackedVarintHash64 = function() {
  return this.readPackedField_(this.decoder_.readVarintHash64);
};


/**
 * Reads a packed fixed hash64 field, which consists of a length header and a
 * list of fixed hash64s.
 * @return {!Array<string>}
 */
jspb.BinaryReader.prototype.readPackedFixedHash64 = function() {
  return this.readPackedField_(this.decoder_.readFixedHash64);
};

//javascript/apps/jspb/binary/writer.js
/**
 * @fileoverview This file contains utilities for encoding Javascript objects
 * into binary, wire-format protocol buffers (in the form of Uint8Arrays) that
 * a server can consume directly.
 *
 * jspb's BinaryWriter class defines methods for efficiently encoding
 * Javascript objects into binary, wire-format protocol buffers and supports
 * all the fundamental field types used in protocol buffers.
 *
 * Major caveat 1 - Users of this library _must_ keep their Javascript proto
 * parsing code in sync with the original .proto file - presumably you'll be
 * using the typed jspb code generator, but if you bypass that you'll need
 * to keep things in sync by hand.
 *
 * Major caveat 2 - Javascript is unable to accurately represent integers
 * larger than 2^53 due to its use of a double-precision floating point format
 * for all numbers. BinaryWriter does not make any special effort to preserve
 * precision for values above this limit - if you need to pass 64-bit integers
 * (hash codes, for example) between the client and server without precision
 * loss, do _not_ use this library.
 *
 * Major caveat 3 - This class uses typed arrays and must not be used on older
 * browsers that do not support them.
 *
 * @author aappleby@google.com (Austin Appleby)
 */

goog.provide('jspb.BinaryWriter');

goog.require('goog.asserts');
goog.require('goog.crypt.base64');
goog.require('jspb.BinaryConstants');
goog.require('jspb.BinaryEncoder');
goog.require('jspb.arith.Int64');
goog.require('jspb.arith.UInt64');
goog.require('jspb.utils');



/**
 * BinaryWriter implements encoders for all the wire types specified in
 * https://developers.google.com/protocol-buffers/docs/encoding.
 *
 * @constructor
 * @struct
 */
jspb.BinaryWriter = function() {
  /**
   * Blocks of serialized data that will be concatenated once all messages have
   * been written.
   * @private {!Array<!Uint8Array|!Array<number>>}
   */
  this.blocks_ = [];

  /**
   * Total number of bytes in the blocks_ array. Does _not_ include bytes in
   * the encoder below.
   * @private {number}
   */
  this.totalLength_ = 0;

  /**
   * Binary encoder holding pieces of a message that we're still serializing.
   * When we get to a stopping point (either the start of a new submessage, or
   * when we need to append a raw Uint8Array), the encoder's buffer will be
   * added to the block array above and the encoder will be reset.
   * @private {!jspb.BinaryEncoder}
   */
  this.encoder_ = new jspb.BinaryEncoder();

  /**
   * A stack of bookmarks containing the parent blocks for each message started
   * via beginSubMessage(), needed as bookkeeping for endSubMessage().
   * TODO(aappleby): Deprecated, users should be calling writeMessage().
   * @private {!Array<!Array<number>>}
   */
  this.bookmarks_ = [];
};


/**
 * Append a typed array of bytes onto the buffer.
 *
 * @param {!Uint8Array} arr The byte array to append.
 * @private
 */
jspb.BinaryWriter.prototype.appendUint8Array_ = function(arr) {
  var temp = this.encoder_.end();
  this.blocks_.push(temp);
  this.blocks_.push(arr);
  this.totalLength_ += temp.length + arr.length;
};


/**
 * Begins a new message by writing the field header and returning a bookmark
 * which we will use to patch in the message length to in endDelimited_ below.
 * @param {number} field
 * @return {!Array<number>}
 * @private
 */
jspb.BinaryWriter.prototype.beginDelimited_ = function(field) {
  this.writeFieldHeader_(field, jspb.BinaryConstants.WireType.DELIMITED);
  var bookmark = this.encoder_.end();
  this.blocks_.push(bookmark);
  this.totalLength_ += bookmark.length;
  bookmark.push(this.totalLength_);
  return bookmark;
};


/**
 * Ends a message by encoding the _change_ in length of the buffer to the
 * parent block and adds the number of bytes needed to encode that length to
 * the total byte length.
 * @param {!Array<number>} bookmark
 * @private
 */
jspb.BinaryWriter.prototype.endDelimited_ = function(bookmark) {
  var oldLength = bookmark.pop();
  var messageLength = this.totalLength_ + this.encoder_.length() - oldLength;
  goog.asserts.assert(messageLength >= 0);

  while (messageLength > 127) {
    bookmark.push((messageLength & 0x7f) | 0x80);
    messageLength = messageLength >>> 7;
    this.totalLength_++;
  }

  bookmark.push(messageLength);
  this.totalLength_++;
};


/**
 * Writes a pre-serialized message to the buffer.
 * @param {!Uint8Array} bytes The array of bytes to write.
 * @param {number} start The start of the range to write.
 * @param {number} end The end of the range to write.
 */
jspb.BinaryWriter.prototype.writeSerializedMessage = function(
    bytes, start, end) {
  this.appendUint8Array_(bytes.subarray(start, end));
};


/**
 * Writes a pre-serialized message to the buffer if the message and endpoints
 * are non-null.
 * @param {?Uint8Array} bytes The array of bytes to write.
 * @param {?number} start The start of the range to write.
 * @param {?number} end The end of the range to write.
 */
jspb.BinaryWriter.prototype.maybeWriteSerializedMessage = function(
    bytes, start, end) {
  if (bytes != null && start != null && end != null) {
    this.writeSerializedMessage(bytes, start, end);
  }
};


/**
 * Resets the writer, throwing away any accumulated buffers.
 */
jspb.BinaryWriter.prototype.reset = function() {
  this.blocks_ = [];
  this.encoder_.end();
  this.totalLength_ = 0;
  this.bookmarks_ = [];
};


/**
 * Converts the encoded data into a Uint8Array.
 * @return {!Uint8Array}
 */
jspb.BinaryWriter.prototype.getResultBuffer = function() {
  goog.asserts.assert(this.bookmarks_.length == 0);

  var flat = new Uint8Array(this.totalLength_ + this.encoder_.length());

  var blocks = this.blocks_;
  var blockCount = blocks.length;
  var offset = 0;

  for (var i = 0; i < blockCount; i++) {
    var block = blocks[i];
    flat.set(block, offset);
    offset += block.length;
  }

  var tail = this.encoder_.end();
  flat.set(tail, offset);
  offset += tail.length;

  // Post condition: `flattened` must have had every byte written.
  goog.asserts.assert(offset == flat.length);

  // Replace our block list with the flattened block, which lets GC reclaim
  // the temp blocks sooner.
  this.blocks_ = [flat];

  return flat;
};


/**
 * Converts the encoded data into a base64-encoded string.
 * @param {!goog.crypt.base64.Alphabet=} alphabet Which flavor of base64 to use.
 * @return {string}
 */
jspb.BinaryWriter.prototype.getResultBase64String = function(alphabet) {
  return goog.crypt.base64.encodeByteArray(this.getResultBuffer(), alphabet);
};


/**
 * Begins a new sub-message. The client must call endSubMessage() when they're
 * done.
 * TODO(aappleby): Deprecated. Move callers to writeMessage().
 * @param {number} field The field number of the sub-message.
 */
jspb.BinaryWriter.prototype.beginSubMessage = function(field) {
  this.bookmarks_.push(this.beginDelimited_(field));
};


/**
 * Finishes a sub-message and packs it into the parent messages' buffer.
 * TODO(aappleby): Deprecated. Move callers to writeMessage().
 */
jspb.BinaryWriter.prototype.endSubMessage = function() {
  goog.asserts.assert(this.bookmarks_.length >= 0);
  this.endDelimited_(this.bookmarks_.pop());
};


/**
 * Encodes a (field number, wire type) tuple into a wire-format field header
 * and stores it in the buffer as a varint.
 * @param {number} field The field number.
 * @param {number} wireType The wire-type of the field, as specified in the
 *     protocol buffer documentation.
 * @private
 */
jspb.BinaryWriter.prototype.writeFieldHeader_ =
    function(field, wireType) {
  goog.asserts.assert(field >= 1 && field == Math.floor(field));
  var x = field * 8 + wireType;
  this.encoder_.writeUnsignedVarint32(x);
};


/**
 * Writes a field of any valid scalar type to the binary stream.
 * @param {jspb.BinaryConstants.FieldType} fieldType
 * @param {number} field
 * @param {jspb.AnyFieldType} value
 */
jspb.BinaryWriter.prototype.writeAny = function(fieldType, field, value) {
  var fieldTypes = jspb.BinaryConstants.FieldType;
  switch (fieldType) {
    case fieldTypes.DOUBLE:
      this.writeDouble(field, /** @type {number} */(value));
      return;
    case fieldTypes.FLOAT:
      this.writeFloat(field, /** @type {number} */(value));
      return;
    case fieldTypes.INT64:
      this.writeInt64(field, /** @type {number} */(value));
      return;
    case fieldTypes.UINT64:
      this.writeUint64(field, /** @type {number} */(value));
      return;
    case fieldTypes.INT32:
      this.writeInt32(field, /** @type {number} */(value));
      return;
    case fieldTypes.FIXED64:
      this.writeFixed64(field, /** @type {number} */(value));
      return;
    case fieldTypes.FIXED32:
      this.writeFixed32(field, /** @type {number} */(value));
      return;
    case fieldTypes.BOOL:
      this.writeBool(field, /** @type {boolean} */(value));
      return;
    case fieldTypes.STRING:
      this.writeString(field, /** @type {string} */(value));
      return;
    case fieldTypes.GROUP:
      goog.asserts.fail('Group field type not supported in writeAny()');
      return;
    case fieldTypes.MESSAGE:
      goog.asserts.fail('Message field type not supported in writeAny()');
      return;
    case fieldTypes.BYTES:
      this.writeBytes(field, /** @type {?Uint8Array} */(value));
      return;
    case fieldTypes.UINT32:
      this.writeUint32(field, /** @type {number} */(value));
      return;
    case fieldTypes.ENUM:
      this.writeEnum(field, /** @type {number} */(value));
      return;
    case fieldTypes.SFIXED32:
      this.writeSfixed32(field, /** @type {number} */(value));
      return;
    case fieldTypes.SFIXED64:
      this.writeSfixed64(field, /** @type {number} */(value));
      return;
    case fieldTypes.SINT32:
      this.writeSint32(field, /** @type {number} */(value));
      return;
    case fieldTypes.SINT64:
      this.writeSint64(field, /** @type {number} */(value));
      return;
    case fieldTypes.FHASH64:
      this.writeFixedHash64(field, /** @type {string} */(value));
      return;
    case fieldTypes.VHASH64:
      this.writeVarintHash64(field, /** @type {string} */(value));
      return;
    default:
      goog.asserts.fail('Invalid field type in writeAny()');
      return;
  }
};


/**
 * Writes a varint field to the buffer without range checking.
 * @param {number} field The field number.
 * @param {number?} value The value to write.
 * @private
 */
jspb.BinaryWriter.prototype.writeUnsignedVarint32_ = function(field, value) {
  if (value == null) return;
  this.writeFieldHeader_(field, jspb.BinaryConstants.WireType.VARINT);
  this.encoder_.writeUnsignedVarint32(value);
};


/**
 * Writes a varint field to the buffer without range checking.
 * @param {number} field The field number.
 * @param {number?} value The value to write.
 * @private
 */
jspb.BinaryWriter.prototype.writeSignedVarint32_ = function(field, value) {
  if (value == null) return;
  this.writeFieldHeader_(field, jspb.BinaryConstants.WireType.VARINT);
  this.encoder_.writeSignedVarint32(value);
};


/**
 * Writes a varint field to the buffer without range checking.
 * @param {number} field The field number.
 * @param {number?} value The value to write.
 * @private
 */
jspb.BinaryWriter.prototype.writeUnsignedVarint64_ = function(field, value) {
  if (value == null) return;
  this.writeFieldHeader_(field, jspb.BinaryConstants.WireType.VARINT);
  this.encoder_.writeUnsignedVarint64(value);
};


/**
 * Writes a varint field to the buffer without range checking.
 * @param {number} field The field number.
 * @param {number?} value The value to write.
 * @private
 */
jspb.BinaryWriter.prototype.writeSignedVarint64_ = function(field, value) {
  if (value == null) return;
  this.writeFieldHeader_(field, jspb.BinaryConstants.WireType.VARINT);
  this.encoder_.writeSignedVarint64(value);
};


/**
 * Writes a zigzag varint field to the buffer without range checking.
 * @param {number} field The field number.
 * @param {number?} value The value to write.
 * @private
 */
jspb.BinaryWriter.prototype.writeZigzagVarint32_ = function(field, value) {
  if (value == null) return;
  this.writeFieldHeader_(field, jspb.BinaryConstants.WireType.VARINT);
  this.encoder_.writeZigzagVarint32(value);
};


/**
 * Writes a zigzag varint field to the buffer without range checking.
 * @param {number} field The field number.
 * @param {number?} value The value to write.
 * @private
 */
jspb.BinaryWriter.prototype.writeZigzagVarint64_ = function(field, value) {
  if (value == null) return;
  this.writeFieldHeader_(field, jspb.BinaryConstants.WireType.VARINT);
  this.encoder_.writeZigzagVarint64(value);
};


/**
 * Writes a zigzag varint field to the buffer without range checking.
 * @param {number} field The field number.
 * @param {string?} value The value to write.
 * @private
 */
jspb.BinaryWriter.prototype.writeZigzagVarint64String_ = function(
    field, value) {
  if (value == null) return;
  this.writeFieldHeader_(field, jspb.BinaryConstants.WireType.VARINT);
  this.encoder_.writeZigzagVarint64String(value);
};


/**
 * Writes a zigzag varint field to the buffer without range checking.
 * @param {number} field The field number.
 * @param {string?} value The value to write.
 * @private
 */
jspb.BinaryWriter.prototype.writeZigzagVarintHash64_ = function(field, value) {
  if (value == null) return;
  this.writeFieldHeader_(field, jspb.BinaryConstants.WireType.VARINT);
  this.encoder_.writeZigzagVarintHash64(value);
};


/**
 * Writes an int32 field to the buffer. Numbers outside the range [-2^31,2^31)
 * will be truncated.
 * @param {number} field The field number.
 * @param {number?} value The value to write.
 */
jspb.BinaryWriter.prototype.writeInt32 = function(field, value) {
  if (value == null) return;
  goog.asserts.assert((value >= -jspb.BinaryConstants.TWO_TO_31) &&
                      (value < jspb.BinaryConstants.TWO_TO_31));
  this.writeSignedVarint32_(field, value);
};


/**
 * Writes an int32 field represented as a string to the buffer. Numbers outside
 * the range [-2^31,2^31) will be truncated.
 * @param {number} field The field number.
 * @param {string?} value The value to write.
 */
jspb.BinaryWriter.prototype.writeInt32String = function(field, value) {
  if (value == null) return;
  var intValue = /** {number} */ parseInt(value, 10);
  goog.asserts.assert((intValue >= -jspb.BinaryConstants.TWO_TO_31) &&
                      (intValue < jspb.BinaryConstants.TWO_TO_31));
  this.writeSignedVarint32_(field, intValue);
};


/**
 * Writes an int64 field to the buffer. Numbers outside the range [-2^63,2^63)
 * will be truncated.
 * @param {number} field The field number.
 * @param {number?} value The value to write.
 */
jspb.BinaryWriter.prototype.writeInt64 = function(field, value) {
  if (value == null) return;
  goog.asserts.assert((value >= -jspb.BinaryConstants.TWO_TO_63) &&
                      (value < jspb.BinaryConstants.TWO_TO_63));
  this.writeSignedVarint64_(field, value);
};


/**
 * Writes a int64 field (with value as a string) to the buffer.
 * @param {number} field The field number.
 * @param {string?} value The value to write.
 */
jspb.BinaryWriter.prototype.writeInt64String = function(field, value) {
  if (value == null) return;
  var num = jspb.arith.Int64.fromString(value);
  this.writeFieldHeader_(field, jspb.BinaryConstants.WireType.VARINT);
  this.encoder_.writeSplitVarint64(num.lo, num.hi);
};


/**
 * Writes a uint32 field to the buffer. Numbers outside the range [0,2^32)
 * will be truncated.
 * @param {number} field The field number.
 * @param {number?} value The value to write.
 */
jspb.BinaryWriter.prototype.writeUint32 = function(field, value) {
  if (value == null) return;
  goog.asserts.assert((value >= 0) &&
                      (value < jspb.BinaryConstants.TWO_TO_32));
  this.writeUnsignedVarint32_(field, value);
};


/**
 * Writes a uint32 field represented as a string to the buffer. Numbers outside
 * the range [0,2^32) will be truncated.
 * @param {number} field The field number.
 * @param {string?} value The value to write.
 */
jspb.BinaryWriter.prototype.writeUint32String = function(field, value) {
  if (value == null) return;
  var intValue = /** {number} */ parseInt(value, 10);
  goog.asserts.assert((intValue >= 0) &&
                      (intValue < jspb.BinaryConstants.TWO_TO_32));
  this.writeUnsignedVarint32_(field, intValue);
};


/**
 * Writes a uint64 field to the buffer. Numbers outside the range [0,2^64)
 * will be truncated.
 * @param {number} field The field number.
 * @param {number?} value The value to write.
 */
jspb.BinaryWriter.prototype.writeUint64 = function(field, value) {
  if (value == null) return;
  goog.asserts.assert((value >= 0) &&
                      (value < jspb.BinaryConstants.TWO_TO_64));
  this.writeUnsignedVarint64_(field, value);
};


/**
 * Writes a uint64 field (with value as a string) to the buffer.
 * @param {number} field The field number.
 * @param {string?} value The value to write.
 */
jspb.BinaryWriter.prototype.writeUint64String = function(field, value) {
  if (value == null) return;
  var num = jspb.arith.UInt64.fromString(value);
  this.writeFieldHeader_(field, jspb.BinaryConstants.WireType.VARINT);
  this.encoder_.writeSplitVarint64(num.lo, num.hi);
};


/**
 * Writes an sint32 field to the buffer. Numbers outside the range [-2^31,2^31)
 * will be truncated.
 * @param {number} field The field number.
 * @param {number?} value The value to write.
 */
jspb.BinaryWriter.prototype.writeSint32 = function(field, value) {
  if (value == null) return;
  goog.asserts.assert((value >= -jspb.BinaryConstants.TWO_TO_31) &&
                      (value < jspb.BinaryConstants.TWO_TO_31));
  this.writeZigzagVarint32_(field, value);
};


/**
 * Writes an sint64 field to the buffer. Numbers outside the range [-2^63,2^63)
 * will be truncated.
 * @param {number} field The field number.
 * @param {number?} value The value to write.
 */
jspb.BinaryWriter.prototype.writeSint64 = function(field, value) {
  if (value == null) return;
  goog.asserts.assert((value >= -jspb.BinaryConstants.TWO_TO_63) &&
                      (value < jspb.BinaryConstants.TWO_TO_63));
  this.writeZigzagVarint64_(field, value);
};


/**
 * Writes an sint64 field to the buffer from a hash64 encoded value. Numbers
 * outside the range [-2^63,2^63) will be truncated.
 * @param {number} field The field number.
 * @param {string?} value The hash64 string to write.
 */
jspb.BinaryWriter.prototype.writeSintHash64 = function(field, value) {
  if (value == null) return;
  this.writeZigzagVarintHash64_(field, value);
};


/**
 * Writes an sint64 field to the buffer. Numbers outside the range [-2^63,2^63)
 * will be truncated.
 * @param {number} field The field number.
 * @param {string?} value The decimal string to write.
 */
jspb.BinaryWriter.prototype.writeSint64String = function(field, value) {
  if (value == null) return;
  this.writeZigzagVarint64String_(field, value);
};


/**
 * Writes a fixed32 field to the buffer. Numbers outside the range [0,2^32)
 * will be truncated.
 * @param {number} field The field number.
 * @param {number?} value The value to write.
 */
jspb.BinaryWriter.prototype.writeFixed32 = function(field, value) {
  if (value == null) return;
  goog.asserts.assert((value >= 0) &&
                      (value < jspb.BinaryConstants.TWO_TO_32));
  this.writeFieldHeader_(field, jspb.BinaryConstants.WireType.FIXED32);
  this.encoder_.writeUint32(value);
};


/**
 * Writes a fixed64 field to the buffer. Numbers outside the range [0,2^64)
 * will be truncated.
 * @param {number} field The field number.
 * @param {number?} value The value to write.
 */
jspb.BinaryWriter.prototype.writeFixed64 = function(field, value) {
  if (value == null) return;
  goog.asserts.assert((value >= 0) &&
                      (value < jspb.BinaryConstants.TWO_TO_64));
  this.writeFieldHeader_(field, jspb.BinaryConstants.WireType.FIXED64);
  this.encoder_.writeUint64(value);
};


/**
 * Writes a fixed64 field (with value as a string) to the buffer.
 * @param {number} field The field number.
 * @param {string?} value The value to write.
 */
jspb.BinaryWriter.prototype.writeFixed64String = function(field, value) {
  if (value == null) return;
  var num = jspb.arith.UInt64.fromString(value);
  this.writeFieldHeader_(field, jspb.BinaryConstants.WireType.FIXED64);
  this.encoder_.writeSplitFixed64(num.lo, num.hi);
};


/**
 * Writes a sfixed32 field to the buffer. Numbers outside the range
 * [-2^31,2^31) will be truncated.
 * @param {number} field The field number.
 * @param {number?} value The value to write.
 */
jspb.BinaryWriter.prototype.writeSfixed32 = function(field, value) {
  if (value == null) return;
  goog.asserts.assert((value >= -jspb.BinaryConstants.TWO_TO_31) &&
                      (value < jspb.BinaryConstants.TWO_TO_31));
  this.writeFieldHeader_(field, jspb.BinaryConstants.WireType.FIXED32);
  this.encoder_.writeInt32(value);
};


/**
 * Writes a sfixed64 field to the buffer. Numbers outside the range
 * [-2^63,2^63) will be truncated.
 * @param {number} field The field number.
 * @param {number?} value The value to write.
 */
jspb.BinaryWriter.prototype.writeSfixed64 = function(field, value) {
  if (value == null) return;
  goog.asserts.assert((value >= -jspb.BinaryConstants.TWO_TO_63) &&
                      (value < jspb.BinaryConstants.TWO_TO_63));
  this.writeFieldHeader_(field, jspb.BinaryConstants.WireType.FIXED64);
  this.encoder_.writeInt64(value);
};


/**
 * Writes a sfixed64 string field to the buffer. Numbers outside the range
 * [-2^63,2^63) will be truncated.
 * @param {number} field The field number.
 * @param {string?} value The value to write.
 */
jspb.BinaryWriter.prototype.writeSfixed64String = function(field, value) {
  if (value == null) return;
  var num = jspb.arith.Int64.fromString(value);
  this.writeFieldHeader_(field, jspb.BinaryConstants.WireType.FIXED64);
  this.encoder_.writeSplitFixed64(num.lo, num.hi);
};


/**
 * Writes a single-precision floating point field to the buffer. Numbers
 * requiring more than 32 bits of precision will be truncated.
 * @param {number} field The field number.
 * @param {number?} value The value to write.
 */
jspb.BinaryWriter.prototype.writeFloat = function(field, value) {
  if (value == null) return;
  this.writeFieldHeader_(field, jspb.BinaryConstants.WireType.FIXED32);
  this.encoder_.writeFloat(value);
};


/**
 * Writes a double-precision floating point field to the buffer. As this is the
 * native format used by JavaScript, no precision will be lost.
 * @param {number} field The field number.
 * @param {number?} value The value to write.
 */
jspb.BinaryWriter.prototype.writeDouble = function(field, value) {
  if (value == null) return;
  this.writeFieldHeader_(field, jspb.BinaryConstants.WireType.FIXED64);
  this.encoder_.writeDouble(value);
};


/**
 * Writes a boolean field to the buffer. We allow numbers as input
 * because the JSPB code generator uses 0/1 instead of true/false to save space
 * in the string representation of the proto.
 * @param {number} field The field number.
 * @param {boolean?|number?} value The value to write.
 */
jspb.BinaryWriter.prototype.writeBool = function(field, value) {
  if (value == null) return;
  goog.asserts.assert(typeof value === 'boolean' || typeof value === 'number');
  this.writeFieldHeader_(field, jspb.BinaryConstants.WireType.VARINT);
  this.encoder_.writeBool(value);
};


/**
 * Writes an enum field to the buffer.
 * @param {number} field The field number.
 * @param {number?} value The value to write.
 */
jspb.BinaryWriter.prototype.writeEnum = function(field, value) {
  if (value == null) return;
  goog.asserts.assert((value >= -jspb.BinaryConstants.TWO_TO_31) &&
                      (value < jspb.BinaryConstants.TWO_TO_31));
  this.writeFieldHeader_(field, jspb.BinaryConstants.WireType.VARINT);
  this.encoder_.writeSignedVarint32(value);
};


/**
 * Writes a string field to the buffer.
 * @param {number} field The field number.
 * @param {string?} value The string to write.
 */
jspb.BinaryWriter.prototype.writeString = function(field, value) {
  if (value == null) return;
  var bookmark = this.beginDelimited_(field);
  this.encoder_.writeString(value);
  this.endDelimited_(bookmark);
};


/**
 * Writes an arbitrary byte field to the buffer. Note - to match the behavior
 * of the C++ implementation, empty byte arrays _are_ serialized.
 * @param {number} field The field number.
 * @param {?jspb.ByteSource} value The array of bytes to write.
 */
jspb.BinaryWriter.prototype.writeBytes = function(field, value) {
  if (value == null) return;
  var bytes = jspb.utils.byteSourceToUint8Array(value);
  this.writeFieldHeader_(field, jspb.BinaryConstants.WireType.DELIMITED);
  this.encoder_.writeUnsignedVarint32(bytes.length);
  this.appendUint8Array_(bytes);
};


/**
 * Writes a message to the buffer.
 * @param {number} field The field number.
 * @param {?MessageType} value The message to write.
 * @param {function(MessageTypeNonNull, !jspb.BinaryWriter)} writerCallback
 *     Will be invoked with the value to write and the writer to write it with.
 * @template MessageType
 * Use go/closure-ttl to declare a non-nullable version of MessageType.  Replace
 * the null in blah|null with none.  This is necessary because the compiler will
 * infer MessageType to be nullable if the value parameter is nullable.
 * @template MessageTypeNonNull :=
 *     cond(isUnknown(MessageType), unknown(),
 *       mapunion(MessageType, (X) =>
 *         cond(eq(X, 'null'), none(), X)))
 * =:
 */
jspb.BinaryWriter.prototype.writeMessage = function(
    field, value, writerCallback) {
  if (value == null) return;
  var bookmark = this.beginDelimited_(field);
  writerCallback(value, this);
  this.endDelimited_(bookmark);
};


/**
 * Writes a message set extension to the buffer.
 * @param {number} field The field number for the extension.
 * @param {?MessageType} value The extension message object to write. Note that
 *     message set can only have extensions with type of optional message.
 * @param {function(!MessageTypeNonNull, !jspb.BinaryWriter)} writerCallback
 *     Will be invoked with the value to write and the writer to write it with.
 * @template MessageType
 * Use go/closure-ttl to declare a non-nullable version of MessageType.  Replace
 * the null in blah|null with none.  This is necessary because the compiler will
 * infer MessageType to be nullable if the value parameter is nullable.
 * @template MessageTypeNonNull :=
 *     cond(isUnknown(MessageType), unknown(),
 *       mapunion(MessageType, (X) =>
 *         cond(eq(X, 'null'), none(), X)))
 * =:
 */
jspb.BinaryWriter.prototype.writeMessageSet = function(
    field, value, writerCallback) {
  if (value == null) return;
  // The wire format for a message set is defined by
  // google3/net/proto/message_set.proto
  this.writeFieldHeader_(1, jspb.BinaryConstants.WireType.START_GROUP);
  this.writeFieldHeader_(2, jspb.BinaryConstants.WireType.VARINT);
  this.encoder_.writeSignedVarint32(field);
  var bookmark = this.beginDelimited_(3);
  writerCallback(value, this);
  this.endDelimited_(bookmark);
  this.writeFieldHeader_(1, jspb.BinaryConstants.WireType.END_GROUP);
};


/**
 * Writes a group message to the buffer.
 *
 * @param {number} field The field number.
 * @param {?MessageType} value The message to write, wrapped with START_GROUP /
 *     END_GROUP tags. Will be a no-op if 'value' is null.
 * @param {function(MessageTypeNonNull, !jspb.BinaryWriter)} writerCallback
 *     Will be invoked with the value to write and the writer to write it with.
 * @template MessageType
 * Use go/closure-ttl to declare a non-nullable version of MessageType.  Replace
 * the null in blah|null with none.  This is necessary because the compiler will
 * infer MessageType to be nullable if the value parameter is nullable.
 * @template MessageTypeNonNull :=
 *     cond(isUnknown(MessageType), unknown(),
 *       mapunion(MessageType, (X) =>
 *         cond(eq(X, 'null'), none(), X)))
 * =:
 */
jspb.BinaryWriter.prototype.writeGroup = function(
    field, value, writerCallback) {
  if (value == null) return;
  this.writeFieldHeader_(field, jspb.BinaryConstants.WireType.START_GROUP);
  writerCallback(value, this);
  this.writeFieldHeader_(field, jspb.BinaryConstants.WireType.END_GROUP);
};


/**
 * Writes a 64-bit hash string field (8 characters @ 8 bits of data each) to
 * the buffer.
 * @param {number} field The field number.
 * @param {string?} value The hash string.
 */
jspb.BinaryWriter.prototype.writeFixedHash64 = function(field, value) {
  if (value == null) return;
  goog.asserts.assert(value.length == 8);
  this.writeFieldHeader_(field, jspb.BinaryConstants.WireType.FIXED64);
  this.encoder_.writeFixedHash64(value);
};


/**
 * Writes a 64-bit hash string field (8 characters @ 8 bits of data each) to
 * the buffer.
 * @param {number} field The field number.
 * @param {string?} value The hash string.
 */
jspb.BinaryWriter.prototype.writeVarintHash64 = function(field, value) {
  if (value == null) return;
  goog.asserts.assert(value.length == 8);
  this.writeFieldHeader_(field, jspb.BinaryConstants.WireType.VARINT);
  this.encoder_.writeVarintHash64(value);
};


/**
 * Writes a 64-bit field to the buffer as a fixed64.
 * @param {number} field The field number.
 * @param {number} lowBits The low 32 bits.
 * @param {number} highBits The high 32 bits.
 */
jspb.BinaryWriter.prototype.writeSplitFixed64 = function(
    field, lowBits, highBits) {
  this.writeFieldHeader_(field, jspb.BinaryConstants.WireType.FIXED64);
  this.encoder_.writeSplitFixed64(lowBits, highBits);
};


/**
 * Writes a 64-bit field to the buffer as a varint.
 * @param {number} field The field number.
 * @param {number} lowBits The low 32 bits.
 * @param {number} highBits The high 32 bits.
 */
jspb.BinaryWriter.prototype.writeSplitVarint64 = function(
    field, lowBits, highBits) {
  this.writeFieldHeader_(field, jspb.BinaryConstants.WireType.VARINT);
  this.encoder_.writeSplitVarint64(lowBits, highBits);
};


/**
 * Writes a 64-bit field to the buffer as a zigzag encoded varint.
 * @param {number} field The field number.
 * @param {number} lowBits The low 32 bits.
 * @param {number} highBits The high 32 bits.
 */
jspb.BinaryWriter.prototype.writeSplitZigzagVarint64 = function(
    field, lowBits, highBits) {
  this.writeFieldHeader_(field, jspb.BinaryConstants.WireType.VARINT);
  var encoder = this.encoder_;
  jspb.utils.toZigzag64(lowBits, highBits, function(lowBits, highBits) {
    encoder.writeSplitVarint64(lowBits >>> 0, highBits >>> 0);
  });
};


/**
 * Writes an array of numbers to the buffer as a repeated 32-bit int field.
 * @param {number} field The field number.
 * @param {?Array<number>} value The array of ints to write.
 */
jspb.BinaryWriter.prototype.writeRepeatedInt32 = function(field, value) {
  if (value == null) return;
  for (var i = 0; i < value.length; i++) {
    this.writeSignedVarint32_(field, value[i]);
  }
};


/**
 * Writes an array of numbers formatted as strings to the buffer as a repeated
 * 32-bit int field.
 * @param {number} field The field number.
 * @param {?Array<string>} value The array of ints to write.
 */
jspb.BinaryWriter.prototype.writeRepeatedInt32String = function(field, value) {
  if (value == null) return;
  for (var i = 0; i < value.length; i++) {
    this.writeInt32String(field, value[i]);
  }
};


/**
 * Writes an array of numbers to the buffer as a repeated 64-bit int field.
 * @param {number} field The field number.
 * @param {?Array<number>} value The array of ints to write.
 */
jspb.BinaryWriter.prototype.writeRepeatedInt64 = function(field, value) {
  if (value == null) return;
  for (var i = 0; i < value.length; i++) {
    this.writeSignedVarint64_(field, value[i]);
  }
};


/**
 * Writes an array of 64-bit values to the buffer as a fixed64.
 * @param {number} field The field number.
 * @param {?Array<T>} value The value.
 * @param {function(T): number} lo Function to get low bits.
 * @param {function(T): number} hi Function to get high bits.
 * @template T
 */
jspb.BinaryWriter.prototype.writeRepeatedSplitFixed64 = function(
    field, value, lo, hi) {
  if (value == null) return;
  for (var i = 0; i < value.length; i++) {
    this.writeSplitFixed64(field, lo(value[i]), hi(value[i]));
  }
};


/**
 * Writes an array of 64-bit values to the buffer as a varint.
 * @param {number} field The field number.
 * @param {?Array<T>} value The value.
 * @param {function(T): number} lo Function to get low bits.
 * @param {function(T): number} hi Function to get high bits.
 * @template T
 */
jspb.BinaryWriter.prototype.writeRepeatedSplitVarint64 = function(
    field, value, lo, hi) {
  if (value == null) return;
  for (var i = 0; i < value.length; i++) {
    this.writeSplitVarint64(field, lo(value[i]), hi(value[i]));
  }
};


/**
 * Writes an array of 64-bit values to the buffer as a zigzag varint.
 * @param {number} field The field number.
 * @param {?Array<T>} value The value.
 * @param {function(T): number} lo Function to get low bits.
 * @param {function(T): number} hi Function to get high bits.
 * @template T
 */
jspb.BinaryWriter.prototype.writeRepeatedSplitZigzagVarint64 = function(
    field, value, lo, hi) {
  if (value == null) return;
  for (var i = 0; i < value.length; i++) {
    this.writeSplitZigzagVarint64(field, lo(value[i]), hi(value[i]));
  }
};


/**
 * Writes an array of numbers formatted as strings to the buffer as a repeated
 * 64-bit int field.
 * @param {number} field The field number.
 * @param {?Array<string>} value The array of ints to write.
 */
jspb.BinaryWriter.prototype.writeRepeatedInt64String = function(field, value) {
  if (value == null) return;
  for (var i = 0; i < value.length; i++) {
    this.writeInt64String(field, value[i]);
  }
};


/**
 * Writes an array numbers to the buffer as a repeated unsigned 32-bit int
 *     field.
 * @param {number} field The field number.
 * @param {?Array<number>} value The array of ints to write.
 */
jspb.BinaryWriter.prototype.writeRepeatedUint32 = function(field, value) {
  if (value == null) return;
  for (var i = 0; i < value.length; i++) {
    this.writeUnsignedVarint32_(field, value[i]);
  }
};


/**
 * Writes an array of numbers formatted as strings to the buffer as a repeated
 * unsigned 32-bit int field.
 * @param {number} field The field number.
 * @param {?Array<string>} value The array of ints to write.
 */
jspb.BinaryWriter.prototype.writeRepeatedUint32String = function(field, value) {
  if (value == null) return;
  for (var i = 0; i < value.length; i++) {
    this.writeUint32String(field, value[i]);
  }
};


/**
 * Writes an array numbers to the buffer as a repeated unsigned 64-bit int
 *     field.
 * @param {number} field The field number.
 * @param {?Array<number>} value The array of ints to write.
 */
jspb.BinaryWriter.prototype.writeRepeatedUint64 = function(field, value) {
  if (value == null) return;
  for (var i = 0; i < value.length; i++) {
    this.writeUnsignedVarint64_(field, value[i]);
  }
};


/**
 * Writes an array of numbers formatted as strings to the buffer as a repeated
 * unsigned 64-bit int field.
 * @param {number} field The field number.
 * @param {?Array<string>} value The array of ints to write.
 */
jspb.BinaryWriter.prototype.writeRepeatedUint64String = function(field, value) {
  if (value == null) return;
  for (var i = 0; i < value.length; i++) {
    this.writeUint64String(field, value[i]);
  }
};


/**
 * Writes an array numbers to the buffer as a repeated signed 32-bit int field.
 * @param {number} field The field number.
 * @param {?Array<number>} value The array of ints to write.
 */
jspb.BinaryWriter.prototype.writeRepeatedSint32 = function(field, value) {
  if (value == null) return;
  for (var i = 0; i < value.length; i++) {
    this.writeZigzagVarint32_(field, value[i]);
  }
};


/**
 * Writes an array numbers to the buffer as a repeated signed 64-bit int field.
 * @param {number} field The field number.
 * @param {?Array<number>} value The array of ints to write.
 */
jspb.BinaryWriter.prototype.writeRepeatedSint64 = function(field, value) {
  if (value == null) return;
  for (var i = 0; i < value.length; i++) {
    this.writeZigzagVarint64_(field, value[i]);
  }
};


/**
 * Writes an array numbers to the buffer as a repeated signed 64-bit int field.
 * @param {number} field The field number.
 * @param {?Array<string>} value The array of ints to write.
 */
jspb.BinaryWriter.prototype.writeRepeatedSint64String = function(field, value) {
  if (value == null) return;
  for (var i = 0; i < value.length; i++) {
    this.writeZigzagVarint64String_(field, value[i]);
  }
};


/**
 * Writes an array of hash64 strings to the buffer as a repeated signed 64-bit
 * int field.
 * @param {number} field The field number.
 * @param {?Array<string>} value The array of ints to write.
 */
jspb.BinaryWriter.prototype.writeRepeatedSintHash64 = function(field, value) {
  if (value == null) return;
  for (var i = 0; i < value.length; i++) {
    this.writeZigzagVarintHash64_(field, value[i]);
  }
};


/**
 * Writes an array of numbers to the buffer as a repeated fixed32 field. This
 * works for both signed and unsigned fixed32s.
 * @param {number} field The field number.
 * @param {?Array<number>} value The array of ints to write.
 */
jspb.BinaryWriter.prototype.writeRepeatedFixed32 = function(field, value) {
  if (value == null) return;
  for (var i = 0; i < value.length; i++) {
    this.writeFixed32(field, value[i]);
  }
};


/**
 * Writes an array of numbers to the buffer as a repeated fixed64 field. This
 * works for both signed and unsigned fixed64s.
 * @param {number} field The field number.
 * @param {?Array<number>} value The array of ints to write.
 */
jspb.BinaryWriter.prototype.writeRepeatedFixed64 = function(field, value) {
  if (value == null) return;
  for (var i = 0; i < value.length; i++) {
    this.writeFixed64(field, value[i]);
  }
};


/**
 * Writes an array of numbers to the buffer as a repeated fixed64 field. This
 * works for both signed and unsigned fixed64s.
 * @param {number} field The field number.
 * @param {?Array<string>} value The array of decimal strings to write.
 */
jspb.BinaryWriter.prototype.writeRepeatedFixed64String = function(
    field, value) {
  if (value == null) return;
  for (var i = 0; i < value.length; i++) {
    this.writeFixed64String(field, value[i]);
  }
};


/**
 * Writes an array of numbers to the buffer as a repeated sfixed32 field.
 * @param {number} field The field number.
 * @param {?Array<number>} value The array of ints to write.
 */
jspb.BinaryWriter.prototype.writeRepeatedSfixed32 = function(field, value) {
  if (value == null) return;
  for (var i = 0; i < value.length; i++) {
    this.writeSfixed32(field, value[i]);
  }
};


/**
 * Writes an array of numbers to the buffer as a repeated sfixed64 field.
 * @param {number} field The field number.
 * @param {?Array<number>} value The array of ints to write.
 */
jspb.BinaryWriter.prototype.writeRepeatedSfixed64 = function(field, value) {
  if (value == null) return;
  for (var i = 0; i < value.length; i++) {
    this.writeSfixed64(field, value[i]);
  }
};


/**
 * Writes an array of decimal strings to the buffer as a repeated sfixed64
 * field.
 * @param {number} field The field number.
 * @param {?Array<string>} value The array of decimal strings to write.
 */
jspb.BinaryWriter.prototype.writeRepeatedSfixed64String = function(field, value) {
  if (value == null) return;
  for (var i = 0; i < value.length; i++) {
    this.writeSfixed64String(field, value[i]);
  }
};


/**
 * Writes an array of numbers to the buffer as a repeated float field.
 * @param {number} field The field number.
 * @param {?Array<number>} value The array of ints to write.
 */
jspb.BinaryWriter.prototype.writeRepeatedFloat = function(field, value) {
  if (value == null) return;
  for (var i = 0; i < value.length; i++) {
    this.writeFloat(field, value[i]);
  }
};


/**
 * Writes an array of numbers to the buffer as a repeated double field.
 * @param {number} field The field number.
 * @param {?Array<number>} value The array of ints to write.
 */
jspb.BinaryWriter.prototype.writeRepeatedDouble = function(field, value) {
  if (value == null) return;
  for (var i = 0; i < value.length; i++) {
    this.writeDouble(field, value[i]);
  }
};


/**
 * Writes an array of booleans to the buffer as a repeated bool field.
 * @param {number} field The field number.
 * @param {?Array<boolean>} value The array of ints to write.
 */
jspb.BinaryWriter.prototype.writeRepeatedBool = function(field, value) {
  if (value == null) return;
  for (var i = 0; i < value.length; i++) {
    this.writeBool(field, value[i]);
  }
};


/**
 * Writes an array of enums to the buffer as a repeated enum field.
 * @param {number} field The field number.
 * @param {?Array<number>} value The array of ints to write.
 */
jspb.BinaryWriter.prototype.writeRepeatedEnum = function(field, value) {
  if (value == null) return;
  for (var i = 0; i < value.length; i++) {
    this.writeEnum(field, value[i]);
  }
};


/**
 * Writes an array of strings to the buffer as a repeated string field.
 * @param {number} field The field number.
 * @param {?Array<string>} value The array of strings to write.
 */
jspb.BinaryWriter.prototype.writeRepeatedString = function(field, value) {
  if (value == null) return;
  for (var i = 0; i < value.length; i++) {
    this.writeString(field, value[i]);
  }
};


/**
 * Writes an array of arbitrary byte fields to the buffer.
 * @param {number} field The field number.
 * @param {?Array<!jspb.ByteSource>} value The arrays of arrays of bytes to
 *     write.
 */
jspb.BinaryWriter.prototype.writeRepeatedBytes = function(field, value) {
  if (value == null) return;
  for (var i = 0; i < value.length; i++) {
    this.writeBytes(field, value[i]);
  }
};


/**
 * Writes an array of messages to the buffer.
 * @template MessageType
 * @param {number} field The field number.
 * @param {?Array<MessageType>} value The array of messages to
 *    write.
 * @param {function(MessageType, !jspb.BinaryWriter)} writerCallback
 *     Will be invoked with the value to write and the writer to write it with.
 */
jspb.BinaryWriter.prototype.writeRepeatedMessage = function(
    field, value, writerCallback) {
  if (value == null) return;
  for (var i = 0; i < value.length; i++) {
    var bookmark = this.beginDelimited_(field);
    writerCallback(value[i], this);
    this.endDelimited_(bookmark);
  }
};


/**
 * Writes an array of group messages to the buffer.
 * @template MessageType
 * @param {number} field The field number.
 * @param {?Array<MessageType>} value The array of messages to
 *    write.
 * @param {function(MessageType, !jspb.BinaryWriter)} writerCallback
 *     Will be invoked with the value to write and the writer to write it with.
 */
jspb.BinaryWriter.prototype.writeRepeatedGroup = function(
    field, value, writerCallback) {
  if (value == null) return;
  for (var i = 0; i < value.length; i++) {
    this.writeFieldHeader_(field, jspb.BinaryConstants.WireType.START_GROUP);
    writerCallback(value[i], this);
    this.writeFieldHeader_(field, jspb.BinaryConstants.WireType.END_GROUP);
  }
};


/**
 * Writes a 64-bit hash string field (8 characters @ 8 bits of data each) to
 * the buffer.
 * @param {number} field The field number.
 * @param {?Array<string>} value The array of hashes to write.
 */
jspb.BinaryWriter.prototype.writeRepeatedFixedHash64 =
    function(field, value) {
  if (value == null) return;
  for (var i = 0; i < value.length; i++) {
    this.writeFixedHash64(field, value[i]);
  }
};


/**
 * Writes a repeated 64-bit hash string field (8 characters @ 8 bits of data
 * each) to the buffer.
 * @param {number} field The field number.
 * @param {?Array<string>} value The array of hashes to write.
 */
jspb.BinaryWriter.prototype.writeRepeatedVarintHash64 =
    function(field, value) {
  if (value == null) return;
  for (var i = 0; i < value.length; i++) {
    this.writeVarintHash64(field, value[i]);
  }
};


/**
 * Writes an array of numbers to the buffer as a packed 32-bit int field.
 * @param {number} field The field number.
 * @param {?Array<number>} value The array of ints to write.
 */
jspb.BinaryWriter.prototype.writePackedInt32 = function(field, value) {
  if (value == null || !value.length) return;
  var bookmark = this.beginDelimited_(field);
  for (var i = 0; i < value.length; i++) {
    this.encoder_.writeSignedVarint32(value[i]);
  }
  this.endDelimited_(bookmark);
};


/**
 * Writes an array of numbers represented as strings to the buffer as a packed
 * 32-bit int field.
 * @param {number} field
 * @param {?Array<string>} value
 */
jspb.BinaryWriter.prototype.writePackedInt32String = function(field, value) {
  if (value == null || !value.length) return;
  var bookmark = this.beginDelimited_(field);
  for (var i = 0; i < value.length; i++) {
    this.encoder_.writeSignedVarint32(parseInt(value[i], 10));
  }
  this.endDelimited_(bookmark);
};


/**
 * Writes an array of numbers to the buffer as a packed 64-bit int field.
 * @param {number} field The field number.
 * @param {?Array<number>} value The array of ints to write.
 */
jspb.BinaryWriter.prototype.writePackedInt64 = function(field, value) {
  if (value == null || !value.length) return;
  var bookmark = this.beginDelimited_(field);
  for (var i = 0; i < value.length; i++) {
    this.encoder_.writeSignedVarint64(value[i]);
  }
  this.endDelimited_(bookmark);
};


/**
 * Writes an array of 64-bit values to the buffer as a fixed64.
 * @param {number} field The field number.
 * @param {?Array<T>} value The value.
 * @param {function(T): number} lo Function to get low bits.
 * @param {function(T): number} hi Function to get high bits.
 * @template T
 */
jspb.BinaryWriter.prototype.writePackedSplitFixed64 = function(
    field, value, lo, hi) {
  if (value == null) return;
  var bookmark = this.beginDelimited_(field);
  for (var i = 0; i < value.length; i++) {
    this.encoder_.writeSplitFixed64(lo(value[i]), hi(value[i]));
  }
  this.endDelimited_(bookmark);
};


/**
 * Writes an array of 64-bit values to the buffer as a varint.
 * @param {number} field The field number.
 * @param {?Array<T>} value The value.
 * @param {function(T): number} lo Function to get low bits.
 * @param {function(T): number} hi Function to get high bits.
 * @template T
 */
jspb.BinaryWriter.prototype.writePackedSplitVarint64 = function(
    field, value, lo, hi) {
  if (value == null) return;
  var bookmark = this.beginDelimited_(field);
  for (var i = 0; i < value.length; i++) {
    this.encoder_.writeSplitVarint64(lo(value[i]), hi(value[i]));
  }
  this.endDelimited_(bookmark);
};


/**
 * Writes an array of 64-bit values to the buffer as a zigzag varint.
 * @param {number} field The field number.
 * @param {?Array<T>} value The value.
 * @param {function(T): number} lo Function to get low bits.
 * @param {function(T): number} hi Function to get high bits.
 * @template T
 */
jspb.BinaryWriter.prototype.writePackedSplitZigzagVarint64 = function(
    field, value, lo, hi) {
  if (value == null) return;
  var bookmark = this.beginDelimited_(field);
  var encoder = this.encoder_;
  for (var i = 0; i < value.length; i++) {
    jspb.utils.toZigzag64(
        lo(value[i]), hi(value[i]), function(bitsLow, bitsHigh) {
          encoder.writeSplitVarint64(bitsLow >>> 0, bitsHigh >>> 0);
        });
  }
  this.endDelimited_(bookmark);
};


/**
 * Writes an array of numbers represented as strings to the buffer as a packed
 * 64-bit int field.
 * @param {number} field
 * @param {?Array<string>} value
 */
jspb.BinaryWriter.prototype.writePackedInt64String = function(field, value) {
  if (value == null || !value.length) return;
  var bookmark = this.beginDelimited_(field);
  for (var i = 0; i < value.length; i++) {
    var num = jspb.arith.Int64.fromString(value[i]);
    this.encoder_.writeSplitVarint64(num.lo, num.hi);
  }
  this.endDelimited_(bookmark);
};


/**
 * Writes an array numbers to the buffer as a packed unsigned 32-bit int field.
 * @param {number} field The field number.
 * @param {?Array<number>} value The array of ints to write.
 */
jspb.BinaryWriter.prototype.writePackedUint32 = function(field, value) {
  if (value == null || !value.length) return;
  var bookmark = this.beginDelimited_(field);
  for (var i = 0; i < value.length; i++) {
    this.encoder_.writeUnsignedVarint32(value[i]);
  }
  this.endDelimited_(bookmark);
};


/**
 * Writes an array of numbers represented as strings to the buffer as a packed
 * unsigned 32-bit int field.
 * @param {number} field
 * @param {?Array<string>} value
 */
jspb.BinaryWriter.prototype.writePackedUint32String =
    function(field, value) {
  if (value == null || !value.length) return;
  var bookmark = this.beginDelimited_(field);
  for (var i = 0; i < value.length; i++) {
    this.encoder_.writeUnsignedVarint32(parseInt(value[i], 10));
  }
  this.endDelimited_(bookmark);
};


/**
 * Writes an array numbers to the buffer as a packed unsigned 64-bit int field.
 * @param {number} field The field number.
 * @param {?Array<number>} value The array of ints to write.
 */
jspb.BinaryWriter.prototype.writePackedUint64 = function(field, value) {
  if (value == null || !value.length) return;
  var bookmark = this.beginDelimited_(field);
  for (var i = 0; i < value.length; i++) {
    this.encoder_.writeUnsignedVarint64(value[i]);
  }
  this.endDelimited_(bookmark);
};


/**
 * Writes an array of numbers represented as strings to the buffer as a packed
 * unsigned 64-bit int field.
 * @param {number} field
 * @param {?Array<string>} value
 */
jspb.BinaryWriter.prototype.writePackedUint64String =
    function(field, value) {
  if (value == null || !value.length) return;
  var bookmark = this.beginDelimited_(field);
  for (var i = 0; i < value.length; i++) {
    var num = jspb.arith.UInt64.fromString(value[i]);
    this.encoder_.writeSplitVarint64(num.lo, num.hi);
  }
  this.endDelimited_(bookmark);
};


/**
 * Writes an array numbers to the buffer as a packed signed 32-bit int field.
 * @param {number} field The field number.
 * @param {?Array<number>} value The array of ints to write.
 */
jspb.BinaryWriter.prototype.writePackedSint32 = function(field, value) {
  if (value == null || !value.length) return;
  var bookmark = this.beginDelimited_(field);
  for (var i = 0; i < value.length; i++) {
    this.encoder_.writeZigzagVarint32(value[i]);
  }
  this.endDelimited_(bookmark);
};


/**
 * Writes an array of numbers to the buffer as a packed signed 64-bit int field.
 * @param {number} field The field number.
 * @param {?Array<number>} value The array of ints to write.
 */
jspb.BinaryWriter.prototype.writePackedSint64 = function(field, value) {
  if (value == null || !value.length) return;
  var bookmark = this.beginDelimited_(field);
  for (var i = 0; i < value.length; i++) {
    this.encoder_.writeZigzagVarint64(value[i]);
  }
  this.endDelimited_(bookmark);
};


/**
 * Writes an array of decimal strings to the buffer as a packed signed 64-bit
 * int field.
 * @param {number} field The field number.
 * @param {?Array<string>} value The array of decimal strings to write.
 */
jspb.BinaryWriter.prototype.writePackedSint64String = function(field, value) {
  if (value == null || !value.length) return;
  var bookmark = this.beginDelimited_(field);
  for (var i = 0; i < value.length; i++) {
    this.encoder_.writeZigzagVarintHash64(
        jspb.utils.decimalStringToHash64(value[i]));
  }
  this.endDelimited_(bookmark);
};


/**
 * Writes an array of hash 64 strings to the buffer as a packed signed 64-bit
 * int field.
 * @param {number} field The field number.
 * @param {?Array<string>} value The array of decimal strings to write.
 */
jspb.BinaryWriter.prototype.writePackedSintHash64 = function(field, value) {
  if (value == null || !value.length) return;
  var bookmark = this.beginDelimited_(field);
  for (var i = 0; i < value.length; i++) {
    this.encoder_.writeZigzagVarintHash64(value[i]);
  }
  this.endDelimited_(bookmark);
};


/**
 * Writes an array of numbers to the buffer as a packed fixed32 field.
 * @param {number} field The field number.
 * @param {?Array<number>} value The array of ints to write.
 */
jspb.BinaryWriter.prototype.writePackedFixed32 = function(field, value) {
  if (value == null || !value.length) return;
  this.writeFieldHeader_(field, jspb.BinaryConstants.WireType.DELIMITED);
  this.encoder_.writeUnsignedVarint32(value.length * 4);
  for (var i = 0; i < value.length; i++) {
    this.encoder_.writeUint32(value[i]);
  }
};


/**
 * Writes an array of numbers to the buffer as a packed fixed64 field.
 * @param {number} field The field number.
 * @param {?Array<number>} value The array of ints to write.
 */
jspb.BinaryWriter.prototype.writePackedFixed64 = function(field, value) {
  if (value == null || !value.length) return;
  this.writeFieldHeader_(field, jspb.BinaryConstants.WireType.DELIMITED);
  this.encoder_.writeUnsignedVarint32(value.length * 8);
  for (var i = 0; i < value.length; i++) {
    this.encoder_.writeUint64(value[i]);
  }
};


/**
 * Writes an array of numbers represented as strings to the buffer as a packed
 * fixed64 field.
 * @param {number} field The field number.
 * @param {?Array<string>} value The array of strings to write.
 */
jspb.BinaryWriter.prototype.writePackedFixed64String = function(field, value) {
  if (value == null || !value.length) return;
  this.writeFieldHeader_(field, jspb.BinaryConstants.WireType.DELIMITED);
  this.encoder_.writeUnsignedVarint32(value.length * 8);
  for (var i = 0; i < value.length; i++) {
    var num = jspb.arith.UInt64.fromString(value[i]);
    this.encoder_.writeSplitFixed64(num.lo, num.hi);
  }
};


/**
 * Writes an array of numbers to the buffer as a packed sfixed32 field.
 * @param {number} field The field number.
 * @param {?Array<number>} value The array of ints to write.
 */
jspb.BinaryWriter.prototype.writePackedSfixed32 = function(field, value) {
  if (value == null || !value.length) return;
  this.writeFieldHeader_(field, jspb.BinaryConstants.WireType.DELIMITED);
  this.encoder_.writeUnsignedVarint32(value.length * 4);
  for (var i = 0; i < value.length; i++) {
    this.encoder_.writeInt32(value[i]);
  }
};


/**
 * Writes an array of numbers to the buffer as a packed sfixed64 field.
 * @param {number} field The field number.
 * @param {?Array<number>} value The array of ints to write.
 */
jspb.BinaryWriter.prototype.writePackedSfixed64 = function(field, value) {
  if (value == null || !value.length) return;
  this.writeFieldHeader_(field, jspb.BinaryConstants.WireType.DELIMITED);
  this.encoder_.writeUnsignedVarint32(value.length * 8);
  for (var i = 0; i < value.length; i++) {
    this.encoder_.writeInt64(value[i]);
  }
};


/**
 * Writes an array of numbers to the buffer as a packed sfixed64 field.
 * @param {number} field The field number.
 * @param {?Array<string>} value The array of decimal strings to write.
 */
jspb.BinaryWriter.prototype.writePackedSfixed64String = function(field, value) {
  if (value == null || !value.length) return;
  this.writeFieldHeader_(field, jspb.BinaryConstants.WireType.DELIMITED);
  this.encoder_.writeUnsignedVarint32(value.length * 8);
  for (var i = 0; i < value.length; i++) {
    this.encoder_.writeInt64String(value[i]);
  }
};


/**
 * Writes an array of numbers to the buffer as a packed float field.
 * @param {number} field The field number.
 * @param {?Array<number>} value The array of ints to write.
 */
jspb.BinaryWriter.prototype.writePackedFloat = function(field, value) {
  if (value == null || !value.length) return;
  this.writeFieldHeader_(field, jspb.BinaryConstants.WireType.DELIMITED);
  this.encoder_.writeUnsignedVarint32(value.length * 4);
  for (var i = 0; i < value.length; i++) {
    this.encoder_.writeFloat(value[i]);
  }
};


/**
 * Writes an array of numbers to the buffer as a packed double field.
 * @param {number} field The field number.
 * @param {?Array<number>} value The array of ints to write.
 */
jspb.BinaryWriter.prototype.writePackedDouble = function(field, value) {
  if (value == null || !value.length) return;
  this.writeFieldHeader_(field, jspb.BinaryConstants.WireType.DELIMITED);
  this.encoder_.writeUnsignedVarint32(value.length * 8);
  for (var i = 0; i < value.length; i++) {
    this.encoder_.writeDouble(value[i]);
  }
};


/**
 * Writes an array of booleans to the buffer as a packed bool field.
 * @param {number} field The field number.
 * @param {?Array<boolean>} value The array of ints to write.
 */
jspb.BinaryWriter.prototype.writePackedBool = function(field, value) {
  if (value == null || !value.length) return;
  this.writeFieldHeader_(field, jspb.BinaryConstants.WireType.DELIMITED);
  this.encoder_.writeUnsignedVarint32(value.length);
  for (var i = 0; i < value.length; i++) {
    this.encoder_.writeBool(value[i]);
  }
};


/**
 * Writes an array of enums to the buffer as a packed enum field.
 * @param {number} field The field number.
 * @param {?Array<number>} value The array of ints to write.
 */
jspb.BinaryWriter.prototype.writePackedEnum = function(field, value) {
  if (value == null || !value.length) return;
  var bookmark = this.beginDelimited_(field);
  for (var i = 0; i < value.length; i++) {
    this.encoder_.writeEnum(value[i]);
  }
  this.endDelimited_(bookmark);
};


/**
 * Writes a 64-bit hash string field (8 characters @ 8 bits of data each) to
 * the buffer.
 * @param {number} field The field number.
 * @param {?Array<string>} value The array of hashes to write.
 */
jspb.BinaryWriter.prototype.writePackedFixedHash64 = function(field, value) {
  if (value == null || !value.length) return;
  this.writeFieldHeader_(field, jspb.BinaryConstants.WireType.DELIMITED);
  this.encoder_.writeUnsignedVarint32(value.length * 8);
  for (var i = 0; i < value.length; i++) {
    this.encoder_.writeFixedHash64(value[i]);
  }
};


/**
 * Writes a 64-bit hash string field (8 characters @ 8 bits of data each) to
 * the buffer.
 * @param {number} field The field number.
 * @param {?Array<string>} value The array of hashes to write.
 */
jspb.BinaryWriter.prototype.writePackedVarintHash64 = function(field, value) {
  if (value == null || !value.length) return;
  var bookmark = this.beginDelimited_(field);
  for (var i = 0; i < value.length; i++) {
    this.encoder_.writeVarintHash64(value[i]);
  }
  this.endDelimited_(bookmark);
};

//javascript/apps/jspb/extension_field_info.js
goog.loadModule(function(exports) {'use strict';goog.module('jspb.ExtensionFieldInfo');
goog.module.declareLegacyNamespace();


const Message = goog.requireType('jspb.Message');

/**
 * Stores information for a single extension field.
 *
 * For example, an extension field defined like so:
 *
 *     extend BaseMessage {
 *       optional MyMessage my_field = 123;
 *     }
 *
 * will result in an ExtensionFieldInfo object with these properties:
 *
 *     {
 *       fieldIndex: 123,
 *       fieldName: {my_field_renamed: 0},
 *       ctor: proto.example.MyMessage,
 *       toObjectFn: proto.example.MyMessage.toObject,
 *       isRepeated: 0
 *     }
 *
 * We include `toObjectFn` to allow the JSCompiler to perform dead-code
 * removal on unused toObject() methods.
 *
 * If an extension field is primitive, ctor and toObjectFn will be null.
 * isRepeated should be 0 or 1.
 *
 * binary{Reader,Writer}Fn and (if message type) binaryMessageSerializeFn are
 * always provided. binaryReaderFn and binaryWriterFn are references to the
 * appropriate methods on BinaryReader/BinaryWriter to read/write the value of
 * this extension, and binaryMessageSerializeFn is a reference to the message
 * class's .serializeBinary method, if available.
 * @template T
 */
class ExtensionFieldInfo {
  /**
   *
   *
   * @param {number} fieldNumber
   * @param {?Object} fieldName This has the extension field name as a property.
   * @param {?function(new: Message, ?Array=)} ctor
   * @param {?function((boolean|undefined),!Message):!Object} toObjectFn
   * @param {number} isRepeated
   */
  constructor(fieldNumber, fieldName, ctor, toObjectFn, isRepeated) {
    /** @const {number} */
    this.fieldIndex = fieldNumber;
    /** @const {?Object} */
    this.fieldName = fieldName;
    /** @const ?function(new: Message, ?Array=)*/
    this.ctor = ctor;
    /** @const {?function((boolean|undefined),!Message):!Object} */
    this.toObjectFn = toObjectFn;
    /** @const {number} */
    this.isRepeated = isRepeated;
  }

  /**
   * @return {boolean} Does this field represent a sub Message?
   */
  isMessageType() {
    return !!this.ctor;
  }
}

exports = ExtensionFieldInfo;

;return exports;});

//javascript/apps/jspb/extension_field_binary_info.js
goog.loadModule(function(exports) {'use strict';goog.module('jspb.ExtensionFieldBinaryInfo');
goog.module.declareLegacyNamespace();

const BinaryReader = goog.require('jspb.BinaryReader');
const BinaryWriter = goog.require('jspb.BinaryWriter');
const ExtensionFieldInfo = goog.require('jspb.ExtensionFieldInfo');

/**
 * Stores binary-related information for a single extension field.
 * @template T
 */
class ExtensionFieldBinaryInfo {
  /**
   * @param {!ExtensionFieldInfo<T>} fieldInfo
   * @param {function(this:BinaryReader,number,?,?)} binaryReaderFn
   * @param {function(this:BinaryWriter,number,?)
   *        |function(this:BinaryWriter,number,?,?,?,?,?)} binaryWriterFn
   * @param {function(?,?)=} binaryMessageSerializeFn
   * @param {function(?,?)=} binaryMessageDeserializeFn
   * @param {boolean=} isPacked
   */

  constructor(
      fieldInfo, binaryReaderFn, binaryWriterFn, binaryMessageSerializeFn,
      binaryMessageDeserializeFn, isPacked) {
    /** @const {!ExtensionFieldInfo<T>} */
    this.fieldInfo = fieldInfo;
    /** @const {function(this:BinaryReader,number,?,?)} */
    this.binaryReaderFn = binaryReaderFn;
    /** @const {function(this:BinaryWriter,number,?)|function(this:BinaryWriter,number,?,?,?,?,?)} */
    this.binaryWriterFn = binaryWriterFn;
    /** @const {?function(?,?)|undefined} */
    this.binaryMessageSerializeFn = binaryMessageSerializeFn;
    /** @const {?function(?,?)|undefined} */
    this.binaryMessageDeserializeFn = binaryMessageDeserializeFn;
    /** @const {boolean} */
    this.isPacked = isPacked || false;
  }
}


exports = ExtensionFieldBinaryInfo;

;return exports;});

//javascript/apps/jspb/map.js

goog.provide('jspb.Map');

goog.require('goog.asserts');

goog.requireType('jspb.BinaryReader');
goog.requireType('jspb.BinaryWriter');



/**
 * Constructs a new Map. A Map is a container that is used to implement map
 * fields on message objects. It closely follows the ES6 Map API; however,
 * it is distinct because we do not want to depend on external polyfills or
 * on ES6 itself.
 *
 * This constructor should only be called from generated message code. It is not
 * intended for general use by library consumers.
 *
 * @template K, V
 *
 * @param {!Array<!Array<?>>} arr
 *
 * @param {?function(new:V, ?=)=} opt_valueCtor
 *    The constructor for type V, if type V is a message type.
 *
 * @constructor
 * @struct
 */
jspb.Map = function(arr, opt_valueCtor) {
  /** @const @private */
  this.arr_ = arr;
  /** @const @private */
  this.valueCtor_ = opt_valueCtor;

  /** @type {!Object<string, !jspb.Map.Entry_<K,V>>} @private */
  this.map_ = {};

  /**
   * Is `this.arr_ updated with respect to `this.map_`?
   * @type {boolean}
   */
  this.arrClean = true;

  if (this.arr_.length > 0) {
    this.loadFromArray_();
  }
};


/**
 * Load initial content from underlying array.
 * @private
 */
jspb.Map.prototype.loadFromArray_ = function() {
  for (var i = 0; i < this.arr_.length; i++) {
    var record = this.arr_[i];
    var key = record[0];
    var value = record[1];
    this.map_[key.toString()] = new jspb.Map.Entry_(key, value);
  }
  this.arrClean = true;
};


/**
 * Synchronize content to underlying array, if needed, and return it.
 * @return {!Array<!Array<!Object>>}
 */
jspb.Map.prototype.toArray = function() {
  if (this.arrClean) {
    if (this.valueCtor_) {
      // We need to recursively sync maps in submessages to their arrays.
      var m = this.map_;
      for (var p in m) {
        if (Object.prototype.hasOwnProperty.call(m, p)) {
          var valueWrapper = /** @type {?jspb.Message} */ (m[p].valueWrapper);
          if (valueWrapper) {
            valueWrapper.toArray();
          }
        }
      }
    }
  } else {
    // Delete all elements.
    this.arr_.length = 0;
    var strKeys = this.stringKeys_();
    // Output keys in deterministic (sorted) order.
    strKeys.sort();
    for (var i = 0; i < strKeys.length; i++) {
      var entry = this.map_[strKeys[i]];
      var valueWrapper = /** @type {?jspb.Message} */ (entry.valueWrapper);
      if (valueWrapper) {
        valueWrapper.toArray();
      }
      this.arr_.push([entry.key, entry.value]);
    }
    this.arrClean = true;
  }
  return this.arr_;
};


/**
 * Returns the map formatted as an array of key-value pairs, suitable for the
 * toObject() form of a message.
 *
 * @param {boolean=} includeInstance Whether to include the JSPB instance for
 *    transitional soy proto support: http://goto/soy-param-migration
 * @param {function((boolean|undefined),V):!Object=} valueToObject
 *    The static toObject() method, if V is a message type.
 * @return {!Array<!Array<!Object>>}
 */
jspb.Map.prototype.toObject = function(includeInstance, valueToObject) {
  var rawArray = this.toArray();
  var entries = [];
  for (var i = 0; i < rawArray.length; i++) {
    var entry = this.map_[rawArray[i][0].toString()];
    this.wrapEntry_(entry);
    var valueWrapper = /** @type {V|undefined} */ (entry.valueWrapper);
    if (valueWrapper) {
      goog.asserts.assert(valueToObject);
      entries.push([entry.key, valueToObject(includeInstance, valueWrapper)]);
    } else {
      entries.push([entry.key, entry.value]);
    }
  }
  return entries;
};


/**
 * Returns a Map from the given array of key-value pairs when the values are of
 * message type. The values in the array must match the format returned by their
 * message type's toObject() method.
 *
 * @template K, V
 * @param {!Array<!Array<!Object>>} entries
 * @param {function(new:V,?=)} valueCtor
 *    The constructor for type V.
 * @param {function(!Object):V} valueFromObject
 *    The fromObject function for type V.
 * @return {!jspb.Map<K, V>}
 */
jspb.Map.fromObject = function(entries, valueCtor, valueFromObject) {
  var result = new jspb.Map([], valueCtor);
  for (var i = 0; i < entries.length; i++) {
    var key = entries[i][0];
    var value = valueFromObject(entries[i][1]);
    result.set(key, value);
  }
  return result;
};


/**
 * Helper: an IteratorIterable over an array.
 * @template T
 * @param {!Array<T>} arr the array
 * @implements {IteratorIterable<T>}
 * @constructor @struct
 * @private
 */
jspb.Map.ArrayIteratorIterable_ = function(arr) {
  /** @type {number} @private */
  this.idx_ = 0;

  /** @const @private */
  this.arr_ = arr;
};


/** @override @final */
jspb.Map.ArrayIteratorIterable_.prototype.next = function() {
  if (this.idx_ < this.arr_.length) {
    return {done: false, value: this.arr_[this.idx_++]};
  } else {
    return {done: true, value: undefined};
  }
};

if (typeof(Symbol) != 'undefined') {
  /** @override */
  jspb.Map.ArrayIteratorIterable_.prototype[Symbol.iterator] = function() {
    return this;
  };
}


/**
 * Returns the map's length (number of key/value pairs).
 * @return {number}
 */
jspb.Map.prototype.getLength = function() {
  return this.stringKeys_().length;
};


/**
 * Clears the map.
 */
jspb.Map.prototype.clear = function() {
  this.map_ = {};
  this.arrClean = false;
};


/**
 * Deletes a particular key from the map.
 * N.B.: differs in name from ES6 Map's `delete` because IE8 does not support
 * reserved words as property names.
 * @this {jspb.Map}
 * @param {K} key
 * @return {boolean} Whether any entry with this key was deleted.
 */
jspb.Map.prototype.del = function(key) {
  var keyValue = key.toString();
  var hadKey = this.map_.hasOwnProperty(keyValue);
  delete this.map_[keyValue];
  this.arrClean = false;
  return hadKey;
};


/**
 * Returns an array of [key, value] pairs in the map.
 *
 * This is redundant compared to the plain entries() method, but we provide this
 * to help out Angular 1.x users.  Still evaluating whether this is the best
 * option.
 *
 * @return {!Array<!Array<K|V>>}
 */
jspb.Map.prototype.getEntryList = function() {
  var entries = [];
  var strKeys = this.stringKeys_();
  strKeys.sort();
  for (var i = 0; i < strKeys.length; i++) {
    var entry = this.map_[strKeys[i]];
    entries.push([entry.key, entry.value]);
  }
  return entries;
};


/**
 * Returns an iterator-iterable over [key, value] pairs in the map.
 * Closure compiler sadly doesn't support tuples, ie. Iterator<[K,V]>.
 * @return {!IteratorIterable<!Array<K|V>>} The iterator-iterable.
 */
jspb.Map.prototype.entries = function() {
  var entries = [];
  var strKeys = this.stringKeys_();
  strKeys.sort();
  for (var i = 0; i < strKeys.length; i++) {
    var entry = this.map_[strKeys[i]];
    entries.push([entry.key, this.wrapEntry_(entry)]);
  }
  return new jspb.Map.ArrayIteratorIterable_(entries);
};


/**
 * Returns an iterator-iterable over keys in the map.
 * @return {!IteratorIterable<K>} The iterator-iterable.
 */
jspb.Map.prototype.keys = function() {
  var keys = [];
  var strKeys = this.stringKeys_();
  strKeys.sort();
  for (var i = 0; i < strKeys.length; i++) {
    var entry = this.map_[strKeys[i]];
    keys.push(entry.key);
  }
  return new jspb.Map.ArrayIteratorIterable_(keys);
};


/**
 * Returns an iterator-iterable over values in the map.
 * @return {!IteratorIterable<V>} The iterator-iterable.
 */
jspb.Map.prototype.values = function() {
  var values = [];
  var strKeys = this.stringKeys_();
  strKeys.sort();
  for (var i = 0; i < strKeys.length; i++) {
    var entry = this.map_[strKeys[i]];
    values.push(this.wrapEntry_(entry));
  }
  return new jspb.Map.ArrayIteratorIterable_(values);
};


/**
 * Iterates over entries in the map, calling a function on each.
 * @template T
 * @param {function(this:T, V, K, ?jspb.Map<K, V>)} cb
 * @param {T=} opt_thisArg
 */
jspb.Map.prototype.forEach = function(cb, opt_thisArg) {
  var strKeys = this.stringKeys_();
  strKeys.sort();
  for (var i = 0; i < strKeys.length; i++) {
    var entry = this.map_[strKeys[i]];
    cb.call(opt_thisArg, this.wrapEntry_(entry), entry.key, this);
  }
};


/**
 * Sets a key in the map to the given value.
 * @param {K} key The key
 * @param {V} value The value
 * @return {!jspb.Map<K,V>}
 */
jspb.Map.prototype.set = function(key, value) {
  var entry = new jspb.Map.Entry_(key);
  if (this.valueCtor_) {
    entry.valueWrapper = value;
    // .toArray() on a message returns a reference to the underlying array
    // rather than a copy.
    entry.value = value.toArray();
  } else {
    entry.value = value;
  }
  this.map_[key.toString()] = entry;
  this.arrClean = false;
  return this;
};


/**
 * Helper: lazily construct a wrapper around an entry, if needed, and return the
 * user-visible type.
 * @param {!jspb.Map.Entry_<K,V>} entry
 * @return {V}
 * @private
 */
jspb.Map.prototype.wrapEntry_ = function(entry) {
  if (this.valueCtor_) {
    if (!entry.valueWrapper) {
      entry.valueWrapper = new this.valueCtor_(entry.value);
    }
    return /** @type {V} */ (entry.valueWrapper);
  } else {
    return entry.value;
  }
};


/**
 * Gets the value corresponding to a key in the map.
 * @param {K} key
 * @return {V|undefined} The value, or `undefined` if key not present
 */
jspb.Map.prototype.get = function(key) {
  var keyValue = key.toString();
  var entry = this.map_[keyValue];
  if (entry) {
    return this.wrapEntry_(entry);
  } else {
    return undefined;
  }
};


/**
 * Determines whether the given key is present in the map.
 * @param {K} key
 * @return {boolean} `true` if the key is present
 */
jspb.Map.prototype.has = function(key) {
  var keyValue = key.toString();
  return (keyValue in this.map_);
};


/**
 * Write this Map field in wire format to a BinaryWriter, using the given field
 * number.
 * @param {number} fieldNumber
 * @param {!jspb.BinaryWriter} writer
 * @param {function(this:jspb.BinaryWriter,number,K)} keyWriterFn
 *     The method on BinaryWriter that writes type K to the stream.
 * @param {function(this:jspb.BinaryWriter,number,V,?=)|
 *          function(this:jspb.BinaryWriter,number,V,?)} valueWriterFn
 *     The method on BinaryWriter that writes type V to the stream.  May be
 *     writeMessage, in which case the second callback arg form is used.
 * @param {function(V,!jspb.BinaryWriter)=} opt_valueWriterCallback
 *    The BinaryWriter serialization callback for type V, if V is a message
 *    type.
 */
jspb.Map.prototype.serializeBinary = function(
    fieldNumber, writer, keyWriterFn, valueWriterFn, opt_valueWriterCallback) {
  var strKeys = this.stringKeys_();
  strKeys.sort();
  for (var i = 0; i < strKeys.length; i++) {
    var entry = this.map_[strKeys[i]];
    writer.beginSubMessage(fieldNumber);
    keyWriterFn.call(writer, 1, entry.key);
    if (this.valueCtor_) {
      valueWriterFn.call(writer, 2, this.wrapEntry_(entry),
                         opt_valueWriterCallback);
    } else {
      /** @type {function(this:jspb.BinaryWriter,number,?)} */ (valueWriterFn)
          .call(writer, 2, entry.value);
    }
    writer.endSubMessage();
  }
};


/**
 * Read one key/value message from the given BinaryReader. Compatible as the
 * `reader` callback parameter to jspb.BinaryReader.readMessage, to be called
 * when a key/value pair submessage is encountered. If the Key is undefined,
 * we should default it to 0.
 * @template K, V
 * @param {!jspb.Map} map
 * @param {!jspb.BinaryReader} reader
 * @param {function(this:jspb.BinaryReader):K} keyReaderFn
 *     The method on BinaryReader that reads type K from the stream.
 *
 * @param {function(this:jspb.BinaryReader):V|
 *          function(this:jspb.BinaryReader,V,
 *                  function(V,!jspb.BinaryReader))} valueReaderFn
 *    The method on BinaryReader that reads type V from the stream. May be
 *    readMessage, in which case the second callback arg form is used.
 *
 * @param {?function(V,!jspb.BinaryReader)=} opt_valueReaderCallback
 *    The BinaryReader parsing callback for type V, if V is a message type
 *
 * @param {K=} opt_defaultKey
 *    The default value for the type of map keys. Accepting map entries with
 *    unset keys is required for maps to be backwards compatible with the
 *    repeated message representation described here: goo.gl/zuoLAC
 *
 * @param {V=} opt_defaultValue
 *    The default value for the type of map values. Accepting map entries with
 *    unset values is required for maps to be backwards compatible with the
 *    repeated message representation described here: goo.gl/zuoLAC
 *
 */
jspb.Map.deserializeBinary = function(map, reader, keyReaderFn, valueReaderFn,
                                      opt_valueReaderCallback, opt_defaultKey,
                                      opt_defaultValue) {
  var key = opt_defaultKey;
  var value = opt_defaultValue;

  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();

    if (field == 1) {
      // Key.
      key = keyReaderFn.call(reader);
    } else if (field == 2) {
      // Value.
      if (map.valueCtor_) {
        goog.asserts.assert(opt_valueReaderCallback);
        if (!value) {
          // Old generator still doesn't provide default value message.
          // Need this for backward compatibility.
          value = new map.valueCtor_();
        }
        valueReaderFn.call(reader, value, opt_valueReaderCallback);
      } else {
        value =
            (/** @type {function(this:jspb.BinaryReader):?} */ (valueReaderFn))
                .call(reader);
      }
    }
  }

  goog.asserts.assert(key != undefined);
  goog.asserts.assert(value != undefined);
  map.set(key, value);
};


/**
 * Helper: compute the list of all stringified keys in the underlying Object
 * map.
 * @return {!Array<string>}
 * @private
 */
jspb.Map.prototype.stringKeys_ = function() {
  var m = this.map_;
  var ret = [];
  for (var p in m) {
    if (Object.prototype.hasOwnProperty.call(m, p)) {
      ret.push(p);
    }
  }
  return ret;
};



/**
 * @param {K} key The entry's key.
 * @param {V=} opt_value The entry's value wrapper.
 * @constructor
 * @struct
 * @template K, V
 * @private
 */
jspb.Map.Entry_ = function(key, opt_value) {
  /** @const {K} */
  this.key = key;

  // The JSPB-serializable value.  For primitive types this will be of type V.
  // For message types it will be an array.
  /** @type {V} */
  this.value = opt_value;

  // Only used for submessage values.
  /** @type {V} */
  this.valueWrapper = undefined;
};

//javascript/apps/jspb/message.js
// Copyright 2010 Google Inc. All Rights Reserved.

/**
 * @fileoverview Definition of jspb.Message.
 *
 * @author mwr@google.com (Mark Rawling)
 */
goog.provide('jspb.Message');

goog.require('goog.array');
goog.require('goog.asserts');
goog.require('goog.crypt.base64');
// BEGIN GOOGLE-INTERNAL
goog.require('jspb.BinaryConstants');
// END GOOGLE-INTERNAL
goog.require('jspb.BinaryReader');
goog.require('jspb.ExtensionFieldBinaryInfo');
goog.require('jspb.ExtensionFieldInfo');
goog.require('jspb.Map');
// BEGIN GOOGLE-INTERNAL
// Not needed in compilation units that have no protos with xids.
goog.requireType('xid.String');
// END GOOGLE-INTERNAL


/**
 * Base class for all JsPb messages.
 *
 * Several common methods (toObject, serializeBinary, in particular) are not
 * defined on the prototype to encourage code patterns that minimize code bloat
 * due to otherwise unused code on all protos contained in the project.
 *
 * If you want to call these methods on a generic message, either
 * pass in your instance of method as a parameter:
 *     someFunction(instanceOfKnownProto,
 *                  KnownProtoClass.prototype.serializeBinary);
 * or use a lambda that knows the type:
 *     someFunction(()=>instanceOfKnownProto.serializeBinary());
 * or, if you don't care about code size, just suppress the
 *     WARNING - Property serializeBinary never defined on jspb.Message
 * and call it the intuitive way.
 *
 * @constructor
 * @struct
 */
jspb.Message = function() {
};


/**
 * @define {boolean} Whether to generate toObject methods for objects. Turn
 *     this off, if you do not want toObject to be ever used in your project.
 *     When turning off this flag, consider adding a conformance test that bans
 *     calling toObject. Enabling this will disable the JSCompiler's ability to
 *     dead code eliminate fields used in protocol buffers that are never used
 *     in an application.
 */
jspb.Message.GENERATE_TO_OBJECT =
    goog.define('jspb.Message.GENERATE_TO_OBJECT', true);


/**
 * @define {boolean} Whether to generate fromObject methods for objects. Turn
 *     this off, if you do not want fromObject to be ever used in your project.
 *     When turning off this flag, consider adding a conformance test that bans
 *     calling fromObject. Enabling this might disable the JSCompiler's ability
 *     to dead code eliminate fields used in protocol buffers that are never
 *     used in an application.
 *     By default this is enabled for test code only.
 */
jspb.Message.GENERATE_FROM_OBJECT = goog.define(
    'jspb.Message.GENERATE_FROM_OBJECT', !goog.DISALLOW_TEST_ONLY_CODE);


/**
 * @define {boolean} Whether to generate toString methods for objects. Turn
 *     this off if you do not use toString in your project and want to trim it
 *     from the compiled JS.
 */
jspb.Message.GENERATE_TO_STRING =
    goog.define('jspb.Message.GENERATE_TO_STRING', true);



// TODO(jakubvrana): Turn this off by default.
/**
 * @define {boolean} Disabling the serialization of empty trailing fields
 *     reduces the size of serialized protos. The price is an extra iteration of
 *     the proto before serialization. This is enabled by default to be
 *     backwards compatible. Projects are advised to turn this flag always off.
 */
jspb.Message.SERIALIZE_EMPTY_TRAILING_FIELDS =
    goog.define('jspb.Message.SERIALIZE_EMPTY_TRAILING_FIELDS', true);


/**
 * Does this JavaScript environment support Uint8Aray typed arrays?
 * @type {boolean}
 * @private
 */
jspb.Message.SUPPORTS_UINT8ARRAY_ = (typeof Uint8Array == 'function');


/**
 * The internal data array.
 * @type {!Array}
 * @protected
 */
jspb.Message.prototype.array;


/**
 * Wrappers are the constructed instances of message-type fields. They are built
 * on demand from the raw array data. Includes message fields, repeated message
 * fields and extension message fields. Indexed by field number.
 * @type {Object}
 * @private
 */
jspb.Message.prototype.wrappers_;


/**
 * The object that contains extension fields, if any. This is an object that
 * maps from a proto field number to the field's value.
 * @type {Object}
 * @private
 */
jspb.Message.prototype.extensionObject_;


/**
 * Non-extension fields with a field number at or above the pivot are
 * stored in the extension object (in addition to all extension fields).
 * @type {number}
 * @private
 */
jspb.Message.prototype.pivot_;


/**
 * The JsPb message_id of this proto.
 * @type {string|undefined} the message id or undefined if this message
 *     has no id.
 * @private
 */
jspb.Message.prototype.messageId_;


/**
 * Repeated fields that have been converted to their proper type. This is used
 * for numbers stored as strings (typically "NaN", "Infinity" and "-Infinity")
 * and for booleans stored as numbers (0 or 1).
 * @private {!Object<number,boolean>|undefined}
 */
jspb.Message.prototype.convertedPrimitiveFields_;

/**
 * Repeated fields numbers.
 * @protected {?Array<number>|undefined}
 */
jspb.Message.prototype.repeatedFields;

// BEGIN GOOGLE-INTERNAL
/**
 * The xid of this proto type (The same for all instances of a proto). Provides
 * a way to identify a proto by stable obfuscated name.
 * @see {xid}.
 * Available if {@link jspb.generate_xid} is added as a Message option to
 * a protocol buffer.
 * @const {!xid.String|undefined} The xid or undefined if message is
 *     annotated to generate the xid.
 */
jspb.Message.prototype.messageXid;
// END GOOGLE-INTERNAL


/**
 * Returns the JsPb message_id of this proto.
 * @return {string|undefined} the message id or undefined if this message
 *     has no id.
 */
jspb.Message.prototype.getJsPbMessageId = function() {
  return this.messageId_;
};


/**
 * An offset applied to lookups into this.array to account for the presence or
 * absence of a messageId at position 0. For response messages, this will be 0.
 * Otherwise, it will be -1 so that the first array position is not wasted.
 * @type {number}
 * @private
 */
jspb.Message.prototype.arrayIndexOffset_;


/**
 * Returns the index into msg.array at which the proto field with tag number
 * fieldNumber will be located.
 * @param {!jspb.Message} msg Message for which we're calculating an index.
 * @param {number} fieldNumber The field number.
 * @return {number} The index.
 * @private
 */
jspb.Message.getIndex_ = function(msg, fieldNumber) {
  return fieldNumber + msg.arrayIndexOffset_;
};

// This is only here to ensure we are not back sliding on ES6 requiements for
// protos in g3.
jspb.Message.hiddenES6Property_ = class {};


/**
 * Returns the tag number based on the index in msg.array.
 * @param {!jspb.Message} msg Message for which we're calculating an index.
 * @param {number} index The tag number.
 * @return {number} The field number.
 * @private
 */
jspb.Message.getFieldNumber_ = function(msg, index) {
  return index - msg.arrayIndexOffset_;
};


/**
 * Initializes a JsPb Message.
 * @param {!jspb.Message} msg The JsPb proto to modify.
 * @param {Array|undefined} data An initial data array.
 * @param {string|number} messageId For response messages, the message id or ''
 *     if no message id is specified. For non-response messages, 0.
 * @param {number} suggestedPivot The field number at which to start putting
 *     fields into the extension object. This is only used if data does not
 *     contain an extension object already. -1 if no extension object is
 *     required for this message type.
 * @param {Array<number>} repeatedFields The message's repeated fields.
 * @param {Array<!Array<number>>=} opt_oneofFields The fields belonging to
 *     each of the message's oneof unions.
 * @protected
 */
jspb.Message.initialize = function(
    msg, data, messageId, suggestedPivot, repeatedFields, opt_oneofFields) {
  msg.wrappers_ = null;
  if (!data) {
    data = messageId ? [messageId] : [];
  }
  msg.messageId_ = messageId ? String(messageId) : undefined;
  // If the messageId is 0, this message is not a response message, so we shift
  // array indices down by 1 so as not to waste the first position in the array,
  // which would otherwise go unused.
  msg.arrayIndexOffset_ = messageId === 0 ? -1 : 0;
  msg.array = data;
  jspb.Message.initPivotAndExtensionObject_(msg, suggestedPivot);
  msg.convertedPrimitiveFields_ = {};

  if (!jspb.Message.SERIALIZE_EMPTY_TRAILING_FIELDS) {
    // TODO(jakubvrana): This is same for all instances, move to prototype.
    // TODO(jakubvrana): There are indexOf calls on this in serializtion,
    // consider switching to a set.
    msg.repeatedFields = repeatedFields;
  }

  if (repeatedFields) {
    for (var i = 0; i < repeatedFields.length; i++) {
      var fieldNumber = repeatedFields[i];
      if (fieldNumber < msg.pivot_) {
        var index = jspb.Message.getIndex_(msg, fieldNumber);
        msg.array[index] =
            msg.array[index] || jspb.Message.EMPTY_LIST_SENTINEL_;
      } else {
        jspb.Message.maybeInitEmptyExtensionObject_(msg);
        msg.extensionObject_[fieldNumber] = msg.extensionObject_[fieldNumber] ||
            jspb.Message.EMPTY_LIST_SENTINEL_;
      }
    }
  }

  if (opt_oneofFields && opt_oneofFields.length) {
    // Compute the oneof case for each union. This ensures only one value is
    // set in the union.
    for (var i = 0; i < opt_oneofFields.length; i++) {
      jspb.Message.computeOneofCase(msg, opt_oneofFields[i]);
    }
  }
};


/**
 * Used to mark empty repeated fields. Serializes to null when serialized
 * to JSON.
 * When reading a repeated field readers must check the return value against
 * this value and return and replace it with a new empty array if it is
 * present.
 * @private @const {!Object}
 */
jspb.Message.EMPTY_LIST_SENTINEL_ = goog.DEBUG && Object.freeze ?
    Object.freeze([]) :
    [];


/**
 * Returns true if the provided argument is an extension object.
 * @param {*} o The object to classify as array or not.
 * @return {boolean} True if the provided object is an extension object.
 * @private
 */
jspb.Message.isExtensionObject_ = function(o) {
  // Normal fields are never objects, so we can be sure that if we find an
  // object here, then it's the extension object. However, we must ensure that
  // the object is not an array, since arrays are valid field values (bytes
  // fields can also be array).
  // NOTE(lukestebbing): We avoid looking at .length to avoid a JIT bug
  // in Safari on iOS 8. See the description of CL/86511464 for details.
  return (o !== null && typeof o == 'object' &&
      !Array.isArray(o) &&
      !(jspb.Message.SUPPORTS_UINT8ARRAY_ && o instanceof Uint8Array));
};


/**
 * If the array contains an extension object in its last position, then the
 * object is kept in place and its position is used as the pivot.  If not,
 * decides the pivot of the message based on suggestedPivot without
 * materializing the extension object.
 *
 * @param {!jspb.Message} msg The JsPb proto to modify.
 * @param {number} suggestedPivot See description for initialize().
 * @private
 */
jspb.Message.initPivotAndExtensionObject_ = function(msg, suggestedPivot) {
  // There are 3 variants that need to be dealt with which are the
  // combination of whether there exists an extension object (EO) and
  // whether there is a suggested pivot (SP).
  //
  // EO,    ?    : pivot is the index of the EO
  // no-EO, no-SP: pivot is MAX_INT
  // no-EO, SP   : pivot is the max(lastindex + 1, SP)

  var msgLength = msg.array.length;
  var lastIndex = -1;
  if (msgLength) {
    lastIndex = msgLength - 1;
    var obj = msg.array[lastIndex];
    if (jspb.Message.isExtensionObject_(obj)) {
      msg.pivot_ = jspb.Message.getFieldNumber_(msg, lastIndex);
      msg.extensionObject_ = obj;
      return;
    }
  }

  if (suggestedPivot > -1) {
    // If a extension object is not present, set the pivot value as being
    // after the last value in the array to avoid overwriting values, etc.
    msg.pivot_ = Math.max(
        suggestedPivot, jspb.Message.getFieldNumber_(msg, lastIndex + 1));
    // Avoid changing the shape of the proto with an empty extension object by
    // deferring the materialization of the extension object until the first
    // time a field set into it (may be due to getting a repeated proto field
    // from it, in which case a new empty array is set into it at first).
    msg.extensionObject_ = null;
  } else {
    // suggestedPivot is -1, which means that we don't have an extension object
    // at all, in which case all fields are stored in the array.
    msg.pivot_ = Number.MAX_VALUE;
  }
};


/**
 * Creates an empty extensionObject_ if non exists.
 * @param {!jspb.Message} msg The JsPb proto to modify.
 * @private
 */
jspb.Message.maybeInitEmptyExtensionObject_ = function(msg) {
  var pivotIndex = jspb.Message.getIndex_(msg, msg.pivot_);
  if (!msg.array[pivotIndex]) {
    msg.extensionObject_ = msg.array[pivotIndex] = {};
  }
};


/**
 * Converts a JsPb repeated message field into an object list.
 * @param {!Array<T>} field The repeated message field to be
 *     converted.
 * @param {?function(boolean=): Object|
 *     function((boolean|undefined),T): Object} toObjectFn The toObject
 *     function for this field.  We need to pass this for effective dead code
 *     removal.
 * @param {boolean=} opt_includeInstance Whether to include the JSPB instance
 *     for transitional soy proto support: http://goto/soy-param-migration
 * @return {!Array<Object>} An array of converted message objects.
 * @template T
 */
jspb.Message.toObjectList = function(field, toObjectFn, opt_includeInstance) {
  // Not using goog.array.map in the generated code to keep it small.
  // And not using it here to avoid a function call.
  var result = [];
  for (var i = 0; i < field.length; i++) {
    result[i] = toObjectFn.call(field[i], opt_includeInstance, field[i]);
  }
  return result;
};


/**
 * Adds a proto's extension data to a Soy rendering object.
 * @param {!jspb.Message} proto The proto whose extensions to convert.
 * @param {!Object} obj The Soy object to add converted extension data to.
 * @param {!Object} extensions The proto class' registered extensions.
 * @param {function(this:?, jspb.ExtensionFieldInfo) : *} getExtensionFn
 *     The proto class' getExtension function. Passed for effective dead code
 *     removal.
 * @param {boolean=} opt_includeInstance Whether to include the JSPB instance
 *     for transitional soy proto support: http://goto/soy-param-migration
 */
jspb.Message.toObjectExtension = function(proto, obj, extensions,
    getExtensionFn, opt_includeInstance) {
  for (var fieldNumber in extensions) {
    var fieldInfo = extensions[fieldNumber];
    var value = getExtensionFn.call(proto, fieldInfo);
    if (value != null) {
      for (var name in fieldInfo.fieldName) {
        if (fieldInfo.fieldName.hasOwnProperty(name)) {
          break; // the compiled field name
        }
      }
      if (!fieldInfo.toObjectFn) {
        obj[name] = value;
      } else {
        if (fieldInfo.isRepeated) {
          obj[name] = jspb.Message.toObjectList(
              /** @type {!Array<!jspb.Message>} */ (value),
              fieldInfo.toObjectFn, opt_includeInstance);
        } else {
          obj[name] = fieldInfo.toObjectFn(
              opt_includeInstance, /** @type {!jspb.Message} */ (value));
        }
      }
    }
  }
};


/**
 * Writes a proto's extension data to a binary-format output stream.
 * @param {!jspb.Message} proto The proto whose extensions to convert.
 * @param {*} writer The binary-format writer to write to.
 * @param {!Object} extensions The proto class' registered extensions.
 * @param {function(this:jspb.Message,!jspb.ExtensionFieldInfo) : *} getExtensionFn The proto
 *     class' getExtension function. Passed for effective dead code removal.
 */
jspb.Message.serializeBinaryExtensions = function(proto, writer, extensions,
    getExtensionFn) {
  for (var fieldNumber in extensions) {
    var binaryFieldInfo = extensions[fieldNumber];
    var fieldInfo = binaryFieldInfo.fieldInfo;

    // The old codegen doesn't add the extra fields to ExtensionFieldInfo, so we
    // need to gracefully error-out here rather than produce a null dereference
    // below.
    if (!binaryFieldInfo.binaryWriterFn) {
      throw new Error('Message extension present that was generated ' +
                      'without binary serialization support');
    }
    var value = getExtensionFn.call(proto, fieldInfo);
    if (value != null) {
      if (fieldInfo.isMessageType()) {
        // If the message type of the extension was generated without binary
        // support, there may not be a binary message serializer function, and
        // we can't know when we codegen the extending message that the extended
        // message may require binary support, so we can *only* catch this error
        // here, at runtime (and this decoupled codegen is the whole point of
        // extensions!).
        if (binaryFieldInfo.binaryMessageSerializeFn) {
          binaryFieldInfo.binaryWriterFn.call(writer, fieldInfo.fieldIndex,
              value, binaryFieldInfo.binaryMessageSerializeFn);
        } else {
          throw new Error('Message extension present holding submessage ' +
                          'without binary support enabled, and message is ' +
                          'being serialized to binary format');
        }
      } else {
        binaryFieldInfo.binaryWriterFn.call(
            writer, fieldInfo.fieldIndex, value);
      }
    }
  }
};

// BEGIN GOOGLE-INTERNAL

/**
 * Reads an message set extension from the given reader and, if a valid
 * extension, sets the extension value to the message set message.
 * @param {!jspb.Message} msg A jspb proto.
 * @param {!jspb.BinaryReader} reader
 * @param {!Object} extensions The extensions object.
 * @param {function(this:jspb.Message,!jspb.ExtensionFieldInfo)} getExtensionFn
 * @param {function(this:jspb.Message,!jspb.ExtensionFieldInfo, ?)} setExtensionFn
 */
jspb.Message.readBinaryExtensionMessageSet = function(
    msg, reader, extensions, getExtensionFn, setExtensionFn) {
  // The wire format for a message set is defined by
  // google3/net/proto/message_set.proto
  if (reader.getFieldNumber() == 1
      && reader.getWireType() == jspb.BinaryConstants.WireType.START_GROUP) {
    var fieldNumber = 0;
    var rawBytes = null;
    while (reader.nextField()) {
      //  tag = 0 case
      if (reader.getWireType() == 0 && reader.getFieldNumber() == 0) {
        break;
      }
      if (reader.getWireType() == jspb.BinaryConstants.WireType.VARINT
          && reader.getFieldNumber() == 2) {
        // message set type id case
        fieldNumber = reader.readUint32();
      } else if (reader.getWireType()
                 == jspb.BinaryConstants.WireType.DELIMITED
                 && reader.getFieldNumber() == 3) {
        // message set contents case
        rawBytes = reader.readBytes();
      } else if (reader.getWireType()
                 == jspb.BinaryConstants.WireType.END_GROUP) {
        // message set ending case
        break;
      } else {
        reader.skipField();
      }
    }
    if (reader.getFieldNumber() != 1
        || reader.getWireType() != jspb.BinaryConstants.WireType.END_GROUP
        || rawBytes == null
        || fieldNumber == 0) {
      throw new Error("Malformed binary bytes for message set");
    }
    var binaryFieldInfo = extensions[fieldNumber];
    if (binaryFieldInfo) {
      var fieldInfo = binaryFieldInfo.fieldInfo;
      var newValue = new fieldInfo.ctor();
      binaryFieldInfo.binaryMessageDeserializeFn.call(
          newValue, newValue, new jspb.BinaryReader(rawBytes));
      setExtensionFn.call(msg, fieldInfo, newValue);
    }
  } else {
    // TODO(yilunchong): change delimited type to be restored in unknowns when
    // jspb support unknowns.
    reader.skipField();
  }
};

// END GOOGLE-INTERNAL

/**
 * Reads an extension field from the given reader and, if a valid extension,
 * sets the extension value.
 * @param {!jspb.Message} msg A jspb proto.
 * @param {!jspb.BinaryReader} reader
 * @param {!Object} extensions The extensions object.
 * @param {function(this:jspb.Message,!jspb.ExtensionFieldInfo)} getExtensionFn
 * @param {function(this:jspb.Message,!jspb.ExtensionFieldInfo, ?)} setExtensionFn
 */
jspb.Message.readBinaryExtension = function(msg, reader, extensions,
    getExtensionFn, setExtensionFn) {
  var binaryFieldInfo = extensions[reader.getFieldNumber()];
  if (!binaryFieldInfo) {
    reader.skipField();
    return;
  }
  var fieldInfo = binaryFieldInfo.fieldInfo;
  if (!binaryFieldInfo.binaryReaderFn) {
    throw new Error('Deserializing extension whose generated code does not ' +
                    'support binary format');
  }

  var value;
  if (fieldInfo.isMessageType()) {
    value = new fieldInfo.ctor();
    binaryFieldInfo.binaryReaderFn.call(
        reader, value, binaryFieldInfo.binaryMessageDeserializeFn);
  } else {
    // All other types.
    value = binaryFieldInfo.binaryReaderFn.call(reader);
  }

  if (fieldInfo.isRepeated && !binaryFieldInfo.isPacked) {
    var currentList = getExtensionFn.call(msg, fieldInfo);
    if (!currentList) {
      setExtensionFn.call(msg, fieldInfo, [value]);
    } else {
      currentList.push(value);
    }
  } else {
    setExtensionFn.call(msg, fieldInfo, value);
  }
};


/**
 * Gets the value of a non-extension field.
 * @param {!jspb.Message} msg A jspb proto.
 * @param {number} fieldNumber The field number.
 * @return {string|number|boolean|Uint8Array|Array|null|undefined}
 * The field's value.
 * @protected
 */
jspb.Message.getField = function(msg, fieldNumber) {
  if (fieldNumber < msg.pivot_) {
    var index = jspb.Message.getIndex_(msg, fieldNumber);
    var val = msg.array[index];
    if (val === jspb.Message.EMPTY_LIST_SENTINEL_) {
      return msg.array[index] = [];
    }
    return val;
  } else {
    if (!msg.extensionObject_) {
      return undefined;
    }
    var val = msg.extensionObject_[fieldNumber];
    if (val === jspb.Message.EMPTY_LIST_SENTINEL_) {
      return msg.extensionObject_[fieldNumber] = [];
    }
    return val;
  }
};


/**
 * Gets the value of a non-extension repeated field.
 * @param {!jspb.Message} msg A jspb proto.
 * @param {number} fieldNumber The field number.
 * @return {!Array}
 * The field's value.
 * @protected
 */
jspb.Message.getRepeatedField = function(msg, fieldNumber) {
  return /** @type {!Array} */ (jspb.Message.getField(msg, fieldNumber));
};


/**
 * Gets the value of an optional float or double field.
 * @param {!jspb.Message} msg A jspb proto.
 * @param {number} fieldNumber The field number.
 * @return {?number|undefined} The field's value.
 * @protected
 */
jspb.Message.getOptionalFloatingPointField = function(msg, fieldNumber) {
  var value = jspb.Message.getField(msg, fieldNumber);
  // Converts "NaN", "Infinity" and "-Infinity" to their corresponding numbers.
  return value == null ? value : +value;
};


/**
 * Gets the value of an optional boolean field.
 * @param {!jspb.Message} msg A jspb proto.
 * @param {number} fieldNumber The field number.
 * @return {?boolean|undefined} The field's value.
 * @protected
 */
jspb.Message.getBooleanField = function(msg, fieldNumber) {
  var value = jspb.Message.getField(msg, fieldNumber);
  // TODO(b/122673075): always return null when the value is null-ish.
  return value == null ? (value) : !!value;
};


/**
 * Gets the value of a repeated float or double field.
 * @param {!jspb.Message} msg A jspb proto.
 * @param {number} fieldNumber The field number.
 * @return {!Array<number>} The field's value.
 * @protected
 */
jspb.Message.getRepeatedFloatingPointField = function(msg, fieldNumber) {
  var values = jspb.Message.getRepeatedField(msg, fieldNumber);
  if (!msg.convertedPrimitiveFields_) {
    msg.convertedPrimitiveFields_ = {};
  }
  if (!msg.convertedPrimitiveFields_[fieldNumber]) {
    for (var i = 0; i < values.length; i++) {
      // Converts "NaN", "Infinity" and "-Infinity" to their corresponding
      // numbers.
      values[i] = +values[i];
    }
    msg.convertedPrimitiveFields_[fieldNumber] = true;
  }
  return /** @type {!Array<number>} */ (values);
};

/**
 * Gets the value of a repeated boolean field.
 * @param {!jspb.Message} msg A jspb proto.
 * @param {number} fieldNumber The field number.
 * @return {!Array<boolean>} The field's value.
 * @protected
 */
jspb.Message.getRepeatedBooleanField = function(msg, fieldNumber) {
  var values = jspb.Message.getRepeatedField(msg, fieldNumber);
  if (!msg.convertedPrimitiveFields_) {
    msg.convertedPrimitiveFields_ = {};
  }
  if (!msg.convertedPrimitiveFields_[fieldNumber]) {
    for (var i = 0; i < values.length; i++) {
      // Converts 0 and 1 to their corresponding booleans.
      values[i] = !!values[i];
    }
    msg.convertedPrimitiveFields_[fieldNumber] = true;
  }
  return /** @type {!Array<boolean>} */ (values);
};


/**
 * Coerce a 'bytes' field to a base 64 string.
 * @param {string|Uint8Array|null} value
 * @return {?string} The field's coerced value.
 */
jspb.Message.bytesAsB64 = function(value) {
  if (value == null || typeof value === 'string') {
    return value;
  }
  if (jspb.Message.SUPPORTS_UINT8ARRAY_ && value instanceof Uint8Array) {
    return goog.crypt.base64.encodeByteArray(value);
  }
  goog.asserts.fail('Cannot coerce to b64 string: ' + goog.typeOf(value));
  return null;
};


/**
 * Coerce a 'bytes' field to a Uint8Array byte buffer.
 * Note that Uint8Array is not supported on IE versions before 10 nor on Opera
 * Mini. @see http://caniuse.com/Uint8Array
 * @param {string|Uint8Array|null} value
 * @return {?Uint8Array} The field's coerced value.
 */
jspb.Message.bytesAsU8 = function(value) {
  if (value == null || value instanceof Uint8Array) {
    return value;
  }
  if (typeof value === 'string') {
    return goog.crypt.base64.decodeStringToUint8Array(value);
  }
  goog.asserts.fail('Cannot coerce to Uint8Array: ' + goog.typeOf(value));
  return null;
};


/**
 * Coerce a repeated 'bytes' field to an array of base 64 strings.
 * Note: the returned array should be treated as immutable.
 * @param {!Array<string>|!Array<!Uint8Array>} value
 * @return {!Array<string?>} The field's coerced value.
 */
jspb.Message.bytesListAsB64 = function(value) {
  jspb.Message.assertConsistentTypes_(value);
  if (!value.length || typeof value[0] === 'string') {
    return /** @type {!Array<string>} */ (value);
  }
  return goog.array.map(value, jspb.Message.bytesAsB64);
};


/**
 * Coerce a repeated 'bytes' field to an array of Uint8Array byte buffers.
 * Note: the returned array should be treated as immutable.
 * Note that Uint8Array is not supported on IE versions before 10 nor on Opera
 * Mini. @see http://caniuse.com/Uint8Array
 * @param {!Array<string>|!Array<!Uint8Array>} value
 * @return {!Array<Uint8Array?>} The field's coerced value.
 */
jspb.Message.bytesListAsU8 = function(value) {
  jspb.Message.assertConsistentTypes_(value);
  if (!value.length || value[0] instanceof Uint8Array) {
    return /** @type {!Array<!Uint8Array>} */ (value);
  }
  return goog.array.map(value, jspb.Message.bytesAsU8);
};


/**
 * Asserts that all elements of an array are of the same type.
 * @param {Array?} array The array to test.
 * @private
 */
jspb.Message.assertConsistentTypes_ = function(array) {
  if (goog.DEBUG && array && array.length > 1) {
    var expected = goog.typeOf(array[0]);
    goog.array.forEach(array, function(e) {
      if (goog.typeOf(e) != expected) {
        goog.asserts.fail('Inconsistent type in JSPB repeated field array. ' +
            'Got ' + goog.typeOf(e) + ' expected ' + expected);
      }
    });
  }
};


/**
 * Gets the value of a non-extension primitive field, with proto3 (non-nullable
 * primitives) semantics. Returns `defaultValue` if the field is not otherwise
 * set.
 * @template T
 * @param {!jspb.Message} msg A jspb proto.
 * @param {number} fieldNumber The field number.
 * @param {T} defaultValue The default value.
 * @return {T} The field's value.
 * @protected
 */
jspb.Message.getFieldWithDefault = function(msg, fieldNumber, defaultValue) {
  var value = jspb.Message.getField(msg, fieldNumber);
  if (value == null) {
    return defaultValue;
  } else {
    return value;
  }
};


/**
 * Gets the value of a boolean field, with proto3 (non-nullable primitives)
 * semantics. Returns `defaultValue` if the field is not otherwise set.
 * @template T
 * @param {!jspb.Message} msg A jspb proto.
 * @param {number} fieldNumber The field number.
 * @param {boolean} defaultValue The default value.
 * @return {boolean} The field's value.
 * @protected
 */
jspb.Message.getBooleanFieldWithDefault = function(
    msg, fieldNumber, defaultValue) {
  var value = jspb.Message.getBooleanField(msg, fieldNumber);
  if (value == null) {
    return defaultValue;
  } else {
    return value;
  }
};


/**
 * Gets the value of a floating point field, with proto3 (non-nullable
 * primitives) semantics. Returns `defaultValue` if the field is not otherwise
 * set.
 * @template T
 * @param {!jspb.Message} msg A jspb proto.
 * @param {number} fieldNumber The field number.
 * @param {number} defaultValue The default value.
 * @return {number} The field's value.
 * @protected
 */
jspb.Message.getFloatingPointFieldWithDefault = function(
    msg, fieldNumber, defaultValue) {
  var value = jspb.Message.getOptionalFloatingPointField(msg, fieldNumber);
  if (value == null) {
    return defaultValue;
  } else {
    return value;
  }
};


/**
 * Alias for getFieldWithDefault used by older generated code.
 * @template T
 * @param {!jspb.Message} msg A jspb proto.
 * @param {number} fieldNumber The field number.
 * @param {T} defaultValue The default value.
 * @return {T} The field's value.
 * @protected
 */
jspb.Message.getFieldProto3 = jspb.Message.getFieldWithDefault;


/**
 * Gets the value of a map field, lazily creating the map container if
 * necessary.
 *
 * This should only be called from generated code, because it requires knowledge
 * of serialization/parsing callbacks (which are required by the map at
 * construction time, and the map may be constructed here).
 *
 * @template K, V
 * @param {!jspb.Message} msg
 * @param {number} fieldNumber
 * @param {boolean|undefined} noLazyCreate
 * @param {?=} opt_valueCtor
 * @return {!jspb.Map<K, V>|undefined}
 * @protected
 */
jspb.Message.getMapField = function(msg, fieldNumber, noLazyCreate,
    opt_valueCtor) {
  if (!msg.wrappers_) {
    msg.wrappers_ = {};
  }
  // If we already have a map in the map wrappers, return that.
  if (fieldNumber in msg.wrappers_) {
    return msg.wrappers_[fieldNumber];
  }
  var arr = jspb.Message.getField(msg, fieldNumber);
  // Wrap the underlying elements array with a Map.
  if (!arr) {
    if (noLazyCreate) {
      return undefined;
    }
    arr = [];
    jspb.Message.setField(msg, fieldNumber, arr);
  }
  return msg.wrappers_[fieldNumber] =
      new jspb.Map(
          /** @type {!Array<!Array<!Object>>} */ (arr), opt_valueCtor);
};


/**
 * Sets the value of a non-extension field.
 * @param {T} msg A jspb proto.
 * @param {number} fieldNumber The field number.
 * @param {string|number|boolean|Uint8Array|Array|undefined} value New value
 * @return {T} return msg
 * @template T
 * @protected
 */
jspb.Message.setField = function(msg, fieldNumber, value) {
  // TODO(b/35241823): replace this with a bounded generic when available
  goog.asserts.assertInstanceof(msg, jspb.Message);
  if (fieldNumber < msg.pivot_) {
    msg.array[jspb.Message.getIndex_(msg, fieldNumber)] = value;
  } else {
    jspb.Message.maybeInitEmptyExtensionObject_(msg);
    msg.extensionObject_[fieldNumber] = value;
  }
  return msg;
};


/**
 * Sets the value of a non-extension integer field of a proto3
 * @param {T} msg A jspb proto.
 * @param {number} fieldNumber The field number.
 * @param {number} value New value
 * @return {T} return msg
 * @template T
 * @protected
 */
jspb.Message.setProto3IntField = function(msg, fieldNumber, value) {
  return jspb.Message.setFieldIgnoringDefault_(msg, fieldNumber, value, 0);
};


/**
 * Sets the value of a non-extension floating point field of a proto3
 * @param {T} msg A jspb proto.
 * @param {number} fieldNumber The field number.
 * @param {number} value New value
 * @return {T} return msg
 * @template T
 * @protected
 */
jspb.Message.setProto3FloatField = function(msg, fieldNumber, value) {
  return jspb.Message.setFieldIgnoringDefault_(msg, fieldNumber, value, 0.0);
};


/**
 * Sets the value of a non-extension boolean field of a proto3
 * @param {T} msg A jspb proto.
 * @param {number} fieldNumber The field number.
 * @param {boolean} value New value
 * @return {T} return msg
 * @template T
 * @protected
 */
jspb.Message.setProto3BooleanField = function(msg, fieldNumber, value) {
  return jspb.Message.setFieldIgnoringDefault_(msg, fieldNumber, value, false);
};


/**
 * Sets the value of a non-extension String field of a proto3
 * @param {T} msg A jspb proto.
 * @param {number} fieldNumber The field number.
 * @param {string} value New value
 * @return {T} return msg
 * @template T
 * @protected
 */
jspb.Message.setProto3StringField = function(msg, fieldNumber, value) {
  return jspb.Message.setFieldIgnoringDefault_(msg, fieldNumber, value, '');
};


/**
 * Sets the value of a non-extension Bytes field of a proto3
 * @param {T} msg A jspb proto.
 * @param {number} fieldNumber The field number.
 * @param {!Uint8Array|string} value New value
 * @return {T} return msg
 * @template T
 * @protected
 */
jspb.Message.setProto3BytesField = function(msg, fieldNumber, value) {
  return jspb.Message.setFieldIgnoringDefault_(msg, fieldNumber, value, '');
};


/**
 * Sets the value of a non-extension enum field of a proto3
 * @param {T} msg A jspb proto.
 * @param {number} fieldNumber The field number.
 * @param {number} value New value
 * @return {T} return msg
 * @template T
 * @protected
 */
jspb.Message.setProto3EnumField = function(msg, fieldNumber, value) {
  return jspb.Message.setFieldIgnoringDefault_(msg, fieldNumber, value, 0);
};


/**
 * Sets the value of a non-extension int field of a proto3 that has jstype set
 * to String.
 * @param {T} msg A jspb proto.
 * @param {number} fieldNumber The field number.
 * @param {string} value New value
 * @return {T} return msg
 * @template T
 * @protected
 */
jspb.Message.setProto3StringIntField = function(msg, fieldNumber, value) {
  return jspb.Message.setFieldIgnoringDefault_(msg, fieldNumber, value, '0');
};

/**
 * Sets the value of a non-extension primitive field, with proto3 (non-nullable
 * primitives) semantics of ignoring values that are equal to the type's
 * default.
 * @param {T} msg A jspb proto.
 * @param {number} fieldNumber The field number.
 * @param {!Uint8Array|string|number|boolean|undefined} value New value
 * @param {!Uint8Array|string|number|boolean} defaultValue The default value.
 * @return {T} return msg
 * @template T
 * @private
 */
jspb.Message.setFieldIgnoringDefault_ = function(
    msg, fieldNumber, value, defaultValue) {
  // TODO(b/35241823): replace this with a bounded generic when available
  goog.asserts.assertInstanceof(msg, jspb.Message);
  if (value !== defaultValue) {
    jspb.Message.setField(msg, fieldNumber, value);
  } else if (fieldNumber < msg.pivot_) {
    msg.array[jspb.Message.getIndex_(msg, fieldNumber)] = null;
  } else {
    jspb.Message.maybeInitEmptyExtensionObject_(msg);
    delete msg.extensionObject_[fieldNumber];
  }
  return msg;
};


/**
 * Adds a value to a repeated, primitive field.
 * @param {T} msg A jspb proto.
 * @param {number} fieldNumber The field number.
 * @param {string|number|boolean|!Uint8Array} value New value
 * @param {number=} opt_index Index where to put new value.
 * @return {T} return msg
 * @template T
 * @protected
 */
jspb.Message.addToRepeatedField = function(msg, fieldNumber, value, opt_index) {
  // TODO(b/35241823): replace this with a bounded generic when available
  goog.asserts.assertInstanceof(msg, jspb.Message);
  var arr = jspb.Message.getRepeatedField(msg, fieldNumber);
  if (opt_index != undefined) {
    arr.splice(opt_index, 0, value);
  } else {
    arr.push(value);
  }
  return msg;
};


/**
 * Sets the value of a field in a oneof union and clears all other fields in
 * the union.
 * @param {T} msg A jspb proto.
 * @param {number} fieldNumber The field number.
 * @param {!Array<number>} oneof The fields belonging to the union.
 * @param {string|number|boolean|Uint8Array|Array|undefined} value New value
 * @return {T} return msg
 * @template T
 * @protected
 */
jspb.Message.setOneofField = function(msg, fieldNumber, oneof, value) {
  // TODO(b/35241823): replace this with a bounded generic when available
  goog.asserts.assertInstanceof(msg, jspb.Message);
  var currentCase = jspb.Message.computeOneofCase(msg, oneof);
  if (currentCase && currentCase !== fieldNumber && value !== undefined) {
    if (msg.wrappers_ && currentCase in msg.wrappers_) {
      msg.wrappers_[currentCase] = undefined;
    }
    jspb.Message.setField(msg, currentCase, undefined);
  }
  return jspb.Message.setField(msg, fieldNumber, value);
};


/**
 * Computes the selection in a oneof group for the given message, ensuring
 * only one field is set in the process.
 *
 * According to the protobuf language guide (
 * https://developers.google.com/protocol-buffers/docs/proto#oneof), "if the
 * parser encounters multiple members of the same oneof on the wire, only the
 * last member seen is used in the parsed message." Since JSPB serializes
 * messages to a JSON array, the "last member seen" will always be the field
 * with the greatest field number (directly corresponding to the greatest
 * array index).
 *
 * @param {!jspb.Message} msg A jspb proto.
 * @param {!Array<number>} oneof The field numbers belonging to the union.
 * @return {number} The field number currently set in the union, or 0 if none.
 * @protected
 */
jspb.Message.computeOneofCase = function(msg, oneof) {
  var oneofField;
  var oneofValue;

  for (var i = 0; i < oneof.length; i++) {
    var fieldNumber = oneof[i];
    var value = jspb.Message.getField(msg, fieldNumber);
    if (value != null) {
      oneofField = fieldNumber;
      oneofValue = value;
      jspb.Message.setField(msg, fieldNumber, undefined);
    }
  }

  if (oneofField) {
    // NB: We know the value is unique, so we can call jspb.Message.setField
    // directly instead of jpsb.Message.setOneofField. Also, setOneofField
    // calls this function.
    jspb.Message.setField(msg, oneofField, oneofValue);
    return oneofField;
  }

  return 0;
};


/**
 * Gets and wraps a proto field on access.
 * @param {!jspb.Message} msg A jspb proto.
 * @param {function(new:jspb.Message, Array)} ctor Constructor for the field.
 * @param {number} fieldNumber The field number.
 * @param {number=} opt_required True (1) if this is a required field.
 * @return {jspb.Message} The field as a jspb proto.
 * @protected
 */
jspb.Message.getWrapperField = function(msg, ctor, fieldNumber, opt_required) {
  // TODO(mwr): Consider copying data and/or arrays.
  if (!msg.wrappers_) {
    msg.wrappers_ = {};
  }
  if (!msg.wrappers_[fieldNumber]) {
    var data = /** @type {Array} */ (jspb.Message.getField(msg, fieldNumber));
    if (opt_required || data) {
      // TODO(mwr): Remove existence test for always valid default protos.
      msg.wrappers_[fieldNumber] = new ctor(data);
    }
  }
  return /** @type {jspb.Message} */ (msg.wrappers_[fieldNumber]);
};


/**
 * Gets and wraps a repeated proto field on access.
 * @param {!jspb.Message} msg A jspb proto.
 * @param {function(new:jspb.Message, Array)} ctor Constructor for the field.
 * @param {number} fieldNumber The field number.
 * @return {!Array<!jspb.Message>} The repeated field as an array of protos.
 * @protected
 */
jspb.Message.getRepeatedWrapperField = function(msg, ctor, fieldNumber) {
  jspb.Message.wrapRepeatedField_(msg, ctor, fieldNumber);
  var val = msg.wrappers_[fieldNumber];
  if (val == jspb.Message.EMPTY_LIST_SENTINEL_) {
    val = msg.wrappers_[fieldNumber] = [];
  }
  return /** @type {!Array<!jspb.Message>} */ (val);
};


/**
 * Wraps underlying array into proto message representation if it wasn't done
 * before.
 * @param {!jspb.Message} msg A jspb proto.
 * @param {function(new:jspb.Message, ?Array)} ctor Constructor for the field.
 * @param {number} fieldNumber The field number.
 * @private
 */
jspb.Message.wrapRepeatedField_ = function(msg, ctor, fieldNumber) {
  if (!msg.wrappers_) {
    msg.wrappers_ = {};
  }
  if (!msg.wrappers_[fieldNumber]) {
    var data = jspb.Message.getRepeatedField(msg, fieldNumber);
    for (var wrappers = [], i = 0; i < data.length; i++) {
      wrappers[i] = new ctor(data[i]);
    }
    msg.wrappers_[fieldNumber] = wrappers;
  }
};


/**
 * Sets a proto field and syncs it to the backing array.
 * @param {T} msg A jspb proto.
 * @param {number} fieldNumber The field number.
 * @param {?jspb.Message|?jspb.Map|undefined} value A new value for this proto
 * field.
 * @return {T} the msg
 * @template T
 * @protected
 */
jspb.Message.setWrapperField = function(msg, fieldNumber, value) {
  // TODO(b/35241823): replace this with a bounded generic when available
  goog.asserts.assertInstanceof(msg, jspb.Message);
  if (!msg.wrappers_) {
    msg.wrappers_ = {};
  }
  var data = value ? value.toArray() : value;
  msg.wrappers_[fieldNumber] = value;
  return jspb.Message.setField(msg, fieldNumber, data);
};



/**
 * Sets a proto field in a oneof union and syncs it to the backing array.
 * @param {T} msg A jspb proto.
 * @param {number} fieldNumber The field number.
 * @param {!Array<number>} oneof The fields belonging to the union.
 * @param {jspb.Message|undefined} value A new value for this proto field.
 * @return {T} the msg
 * @template T
 * @protected
 */
jspb.Message.setOneofWrapperField = function(msg, fieldNumber, oneof, value) {
  // TODO(b/35241823): replace this with a bounded generic when available
  goog.asserts.assertInstanceof(msg, jspb.Message);
  if (!msg.wrappers_) {
    msg.wrappers_ = {};
  }
  var data = value ? value.toArray() : value;
  msg.wrappers_[fieldNumber] = value;
  return jspb.Message.setOneofField(msg, fieldNumber, oneof, data);
};


/**
 * Sets a repeated proto field and syncs it to the backing array.
 * @param {T} msg A jspb proto.
 * @param {number} fieldNumber The field number.
 * @param {Array<!jspb.Message>|undefined} value An array of protos.
 * @return {T} the msg
 * @template T
 * @protected
 */
jspb.Message.setRepeatedWrapperField = function(msg, fieldNumber, value) {
  // TODO(b/35241823): replace this with a bounded generic when available
  goog.asserts.assertInstanceof(msg, jspb.Message);
  if (!msg.wrappers_) {
    msg.wrappers_ = {};
  }
  value = value || [];
  for (var data = [], i = 0; i < value.length; i++) {
    data[i] = value[i].toArray();
  }
  msg.wrappers_[fieldNumber] = value;
  return jspb.Message.setField(msg, fieldNumber, data);
};


/**
 * Add a message to a repeated proto field.
 * @param {!jspb.Message} msg A jspb proto.
 * @param {number} fieldNumber The field number.
 * @param {T_CHILD|undefined} value Proto that will be added to the
 *     repeated field.
 * @param {function(new:T_CHILD, ?Array=)} ctor The constructor of the
 *     message type.
 * @param {number|undefined} index Index at which to insert the value.
 * @return {T_CHILD_NOT_UNDEFINED} proto that was inserted to the repeated field
 * @template MessageType
 * Use go/closure-ttl to declare a non-undefined version of T_CHILD. Replace the
 * undefined in blah|undefined with none. This is necessary because the compiler
 * will infer T_CHILD to be |undefined.
 * @template T_CHILD
 * @template T_CHILD_NOT_UNDEFINED :=
 *     cond(isUnknown(T_CHILD), unknown(),
 *       mapunion(T_CHILD, (X) =>
 *         cond(eq(X, 'undefined'), none(), X)))
 * =:
 * @protected
 */
jspb.Message.addToRepeatedWrapperField = function(
    msg, fieldNumber, value, ctor, index) {
  jspb.Message.wrapRepeatedField_(msg, ctor, fieldNumber);
  var wrapperArray = msg.wrappers_[fieldNumber];
  if (!wrapperArray) {
    wrapperArray = msg.wrappers_[fieldNumber] = [];
  }
  var insertedValue = value ? value : new ctor();
  var array = jspb.Message.getRepeatedField(msg, fieldNumber);
  if (index != undefined) {
    wrapperArray.splice(index, 0, insertedValue);
    array.splice(index, 0, insertedValue.toArray());
  } else {
    wrapperArray.push(insertedValue);
    array.push(insertedValue.toArray());
  }
  return insertedValue;
};


/**
 * Converts a JsPb repeated message field into a map. The map will contain
 * protos unless an optional toObject function is given, in which case it will
 * contain objects suitable for Soy rendering.
 * @param {!Array<T>} field The repeated message field to be
 *     converted.
 * @param {function() : string?} mapKeyGetterFn The function to get the key of
 *     the map.
 * @param {?function(boolean=): Object|
 *     function((boolean|undefined),T): Object} opt_toObjectFn The
 *     toObject function for this field. We need to pass this for effective
 *     dead code removal.
 * @param {boolean=} opt_includeInstance Whether to include the JSPB instance
 *     for transitional soy proto support: http://goto/soy-param-migration
 * @return {!Object<string, Object>} A map of proto or Soy objects.
 * @template T
 */
jspb.Message.toMap = function(
    field, mapKeyGetterFn, opt_toObjectFn, opt_includeInstance) {
  var result = {};
  for (var i = 0; i < field.length; i++) {
    result[mapKeyGetterFn.call(field[i])] = opt_toObjectFn ?
        opt_toObjectFn.call(field[i], opt_includeInstance,
            /** @type {!jspb.Message} */ (field[i])) : field[i];
  }
  return result;
};


/**
 * Syncs all map fields' contents back to their underlying arrays.
 * @private
 */
jspb.Message.prototype.syncMapFields_ = function() {
  // This iterates over submessage, map, and repeated fields, which is intended.
  // Submessages can contain maps which also need to be synced.
  //
  // There is a lot of opportunity for optimization here.  For example we could
  // statically determine that some messages have no submessages with maps and
  // optimize this method away for those just by generating one extra static
  // boolean per message type.
  if (this.wrappers_) {
    for (var fieldNumber in this.wrappers_) {
      var val = this.wrappers_[fieldNumber];
      if (goog.isArray(val)) {
        for (var i = 0; i < val.length; i++) {
          if (val[i]) {
            val[i].toArray();
          }
        }
      } else {
        // Works for submessages and maps.
        if (val) {
          val.toArray();
        }
      }
    }
  }
};


/**
 * Returns the internal array of this proto.
 * <p>Note: If you use this array to construct a second proto, the content
 * would then be partially shared between the two protos.
 * @return {!Array} The proto represented as an array.
 */
jspb.Message.prototype.toArray = function() {
  this.syncMapFields_();
  return this.array;
};


// BEGIN GOOGLE-INTERNAL

// We are not releasing the JSPB wire format in open-source until it is
// better standardized.


/**
 * Serializes a JsPb proto for use in server requests.
 * @this {!jspb.Message}
 * @return {string} The serialized proto.
 */
jspb.Message.prototype.serialize = jspb.Message.SUPPORTS_UINT8ARRAY_ ?
    /**
     * @return {string} The serialized proto.
     */
    function() {
      // Monkey-patch Uint8Array to generate JSON in base64 encoding. We do this
      // rather than use stringify's |replacer| to dodge a >50% performance hit.
      // See http://b/26173701#comment4 for more background.
      var old_toJSON = Uint8Array.prototype.toJSON;
      /** @override */
      Uint8Array.prototype.toJSON = function() {
        return goog.crypt.base64.encodeByteArray(this);
      };
      try {
        return JSON.stringify(
            this.array &&
                jspb.Message.prepareForSerialize_(this.toArray(), this),
            jspb.Message.serializeSpecialNumbers_);
      } finally {
        /** @override */
        Uint8Array.prototype.toJSON = old_toJSON;
      }
    } :
    function() {
      return JSON.stringify(
          this.array && jspb.Message.prepareForSerialize_(this.toArray(), this),
          jspb.Message.serializeSpecialNumbers_);
    };


/**
 * Removes empty fields from arrays.
 * @param {!Array} array
 * @param {!jspb.Message|!Array<!jspb.Message>|undefined} msg This is
 *     jspb.Message when serializing messages, Array when serializing nested
 *     repeated fields and undefined if the type is unknown.
 * @return {!Array}
 * @private
 */
jspb.Message.prepareForSerialize_ = function(array, msg) {
  if (jspb.Message.SERIALIZE_EMPTY_TRAILING_FIELDS) {
    return array;
  }
  // To minimize memory allocations, we copy the array only if it's different
  // from the original.
  var result;
  var length = array.length;
  var needsCopy = false;
  var extension;
  for (var i = array.length; i--;) {
    var value = array[i];
    if (Array.isArray(value)) {
      // wrappers_ contain an array of jspb.Message for a repeated field or
      // jspb.Message for a single field.
      var nestedMsg;
      if (Array.isArray(msg)) {
        nestedMsg = msg[i];
      } else if (msg && msg.wrappers_) {
        nestedMsg = msg.wrappers_[jspb.Message.getFieldNumber_(msg, i)];
      } else {
        nestedMsg = undefined;
      }
      value = jspb.Message.prepareForSerialize_(value, nestedMsg);
      if (!value.length && msg) {
        if (Array.isArray(msg)) {
          // We are inside a repeated field of nested messages. It means that
          // value is a message and not a repeated field. We must keep it.
        } else if (msg.repeatedFields) {
          var fieldNumber = jspb.Message.getFieldNumber_(msg, i);
          if (msg.repeatedFields.indexOf(fieldNumber) != -1) {
            // This is a repeated field, we could remove it.
            value = null;
          }
          // Otherwise, it's a nested message, keep it.
        } else {
          // We know nothing about the field, rather keep it.
        }
      }
      if (value != array[i]) {
        needsCopy = true;
      }
    } else if (jspb.Message.isExtensionObject_(value)) {
      extension = jspb.Message.prepareExtensionForSerialize_(
          value, msg && goog.asserts.assertInstanceof(msg, jspb.Message));
      if (extension != value) {
        needsCopy = true;
      }
      length--;
      continue;
    }
    if (value == null && length == i + 1) {
      needsCopy = true;
      length--;
    } else if (needsCopy) {
      if (!result) {
        result = array.slice(0, length);
      }
      result[i] = value;
    }
  }
  if (!needsCopy) {
    return array;
  }
  if (!result) {
    result = array.slice(0, length);
  }
  if (extension) {
    result.push(extension);
  }
  return result;
};


/**
 * Removes empty fields from extension.
 * @param {!Object} extension
 * @param {!jspb.Message|undefined} msg
 * @return {?Object} Extension object with removed empty fields or null if
 *     empty.
 * @private
 */
jspb.Message.prepareExtensionForSerialize_ = function(extension, msg) {
  var result = {};
  var changed = false;
  for (var key in extension) {
    var value = extension[key];
    if (Array.isArray(value)) {
      var prepared = jspb.Message.prepareForSerialize_(
          value, msg && msg.wrappers_ && msg.wrappers_[key]);
      if (prepared.length || !msg || !msg.repeatedFields ||
          msg.repeatedFields.indexOf(+key) == -1) {
        result[key] = prepared;
      }
      if (result[key] != value) {
        changed = true;
      }
    } else if (value != null) {
      result[key] = value;
    } else {
      changed = true;
    }
  }
  if (!changed) {
    // We return the original object if possible.
    return extension;
  }
  // Equivalent to goog.object.isEmpty.
  for (var key in result) {
    return result;
  }
  return null;
};


/**
 * Converts NaN to "NaN", Infinity to "Infinity" and -Infinity to "-Infinity".
 * These numbers are otherwise serialized as null.
 * @param {string} key
 * @param {*} value
 * @return {*} The converted proto message.
 * @private
 */
jspb.Message.serializeSpecialNumbers_ = function(key, value) {
  if (typeof value === 'number') {
    if (isNaN(value) || value === Infinity || value === -Infinity) {
      return String(value);
    }
  }
  return value;
};


/**
 * Deserialize a JsPb string.
 * @param {function(new:JSPB, ?Array=)} ctor The constructor of the
 *     message type.
 * @param {string} data Our serialized data.
 * @return {JSPB} The new message with the serialized data populating
 *     its fields.
 * @template JSPB
 */
jspb.Message.deserializeWithCtor = function(ctor, data) {
  // data should be always non-empty but especially tests are sometimes passing
  // undefined.
  var msg = new ctor(data ? /** @type {?Array} */ (JSON.parse(data)) : null);
  goog.asserts.assertInstanceof(msg, jspb.Message);
  return msg;
};

// END GOOGLE-INTERNAL

if (jspb.Message.GENERATE_TO_STRING) {

/**
 * Creates a string representation of the internal data array of this proto.
 * <p>NOTE: This string is *not* suitable for use in server requests.
 * @return {string} A string representation of this proto.
 * @override
 */
jspb.Message.prototype.toString = function() {
  this.syncMapFields_();
  return this.array.toString();
};

}

/**
 * Gets the value of the extension field from the extended object.
 * @param {jspb.ExtensionFieldInfo<T>} fieldInfo Specifies the field to get.
 * @return {T} The value of the field.
 * @template T
 */
jspb.Message.prototype.getExtension = function(fieldInfo) {
  if (!this.extensionObject_) {
    return undefined;
  }
  if (!this.wrappers_) {
    this.wrappers_ = {};
  }
  var fieldNumber = fieldInfo.fieldIndex;
  if (fieldInfo.isRepeated) {
    if (fieldInfo.isMessageType()) {
      if (!this.wrappers_[fieldNumber]) {
        this.wrappers_[fieldNumber] =
            goog.array.map(this.extensionObject_[fieldNumber] || [],
                function(arr) {
                  return new fieldInfo.ctor(arr);
                });
      }
      return this.wrappers_[fieldNumber];
    } else {
      return this.extensionObject_[fieldNumber];
    }
  } else {
    if (fieldInfo.isMessageType()) {
      if (!this.wrappers_[fieldNumber] && this.extensionObject_[fieldNumber]) {
        this.wrappers_[fieldNumber] = new fieldInfo.ctor(
            /** @type {Array|undefined} */ (
                this.extensionObject_[fieldNumber]));
      }
      return this.wrappers_[fieldNumber];
    } else {
      return this.extensionObject_[fieldNumber];
    }
  }
};


/**
 * Sets the value of the extension field in the extended object.
 * @param {jspb.ExtensionFieldInfo} fieldInfo Specifies the field to set.
 * @param {jspb.Message|string|Uint8Array|number|boolean|Array?} value The value
 *     to set.
 * @return {THIS} For chaining
 * @this {THIS}
 * @template THIS
 */
jspb.Message.prototype.setExtension = function(fieldInfo, value) {
  // Cast self, since the inferred THIS is unknown inside the function body.
  // https://github.com/google/closure-compiler/issues/1411#issuecomment-232442220
  var self = /** @type {!jspb.Message} */ (this);
  if (!self.wrappers_) {
    self.wrappers_ = {};
  }
  jspb.Message.maybeInitEmptyExtensionObject_(self);
  var fieldNumber = fieldInfo.fieldIndex;
  if (fieldInfo.isRepeated) {
    value = value || [];
    if (fieldInfo.isMessageType()) {
      self.wrappers_[fieldNumber] = value;
      self.extensionObject_[fieldNumber] = goog.array.map(
          /** @type {!Array<!jspb.Message>} */ (value), function(msg) {
        return msg.toArray();
      });
    } else {
      self.extensionObject_[fieldNumber] = value;
    }
  } else {
    if (fieldInfo.isMessageType()) {
      self.wrappers_[fieldNumber] = value;
      self.extensionObject_[fieldNumber] =
          value ? /** @type {!jspb.Message} */ (value).toArray() : value;
    } else {
      self.extensionObject_[fieldNumber] = value;
    }
  }
  return self;
};


/**
 * Creates a difference object between two messages.
 *
 * The result will contain the top-level fields of m2 that differ from those of
 * m1 at any level of nesting. No data is cloned, the result object will
 * share its top-level elements with m2 (but not with m1).
 *
 * Note that repeated fields should not have null/undefined elements, but if
 * they do, this operation will treat repeated fields of different length as
 * the same if the only difference between them is due to trailing
 * null/undefined values.
 *
 * @param {!jspb.Message} m1 The first message object.
 * @param {!jspb.Message} m2 The second message object.
 * @return {!jspb.Message} The difference returned as a proto message.
 *     Note that the returned message may be missing required fields. This is
 *     currently tolerated in Js, but would cause an error if you tried to
 *     send such a proto to the server. You can access the raw difference
 *     array with result.toArray().
 * @throws {Error} If the messages are responses with different types.
 */
jspb.Message.difference = function(m1, m2) {
  if (!(m1 instanceof m2.constructor)) {
    throw new Error('Messages have different types.');
  }
  var arr1 = m1.toArray();
  var arr2 = m2.toArray();
  var res = [];
  var start = 0;
  var length = arr1.length > arr2.length ? arr1.length : arr2.length;
  if (m1.getJsPbMessageId()) {
    res[0] = m1.getJsPbMessageId();
    start = 1;
  }
  for (var i = start; i < length; i++) {
    if (!jspb.Message.compareFields(arr1[i], arr2[i])) {
      res[i] = arr2[i];
    }
  }
  return new m1.constructor(res);
};


/**
 * Tests whether two messages are equal.
 * @param {jspb.Message|undefined} m1 The first message object.
 * @param {jspb.Message|undefined} m2 The second message object.
 * @return {boolean} true if both messages are null/undefined, or if both are
 *     of the same type and have the same field values.
 */
jspb.Message.equals = function(m1, m2) {
  return m1 == m2 || (!!(m1 && m2) && (m1 instanceof m2.constructor) &&
      jspb.Message.compareFields(m1.toArray(), m2.toArray()));
};


/**
 * Compares two message extension fields recursively.
 * @param {!Object} extension1 The first field.
 * @param {!Object} extension2 The second field.
 * @return {boolean} true if the extensions are null/undefined, or otherwise
 *     equal.
 */
jspb.Message.compareExtensions = function(extension1, extension2) {
  extension1 = extension1 || {};
  extension2 = extension2 || {};

  var keys = {};
  for (var name in extension1) {
    keys[name] = 0;
  }
  for (var name in extension2) {
    keys[name] = 0;
  }
  for (name in keys) {
    if (!jspb.Message.compareFields(extension1[name], extension2[name])) {
      return false;
    }
  }
  return true;
};


/**
 * Compares two message fields recursively.
 * @param {*} field1 The first field.
 * @param {*} field2 The second field.
 * @return {boolean} true if the fields are null/undefined, or otherwise equal.
 */
jspb.Message.compareFields = function(field1, field2) {
  // If the fields are trivially equal, they're equal.
  if (field1 == field2) return true;

  if (!goog.isObject(field1) || !goog.isObject(field2)) {
    // NaN != NaN so we cover this case.
    if ((typeof field1 === 'number' && isNaN(field1)) ||
        (typeof field2 === 'number' && isNaN(field2))) {
      // One of the fields might be a string 'NaN'.
      return String(field1) == String(field2);
    }
    // If the fields aren't trivially equal and one of them isn't an object,
    // they can't possibly be equal.
    return false;
  }

  // We have two objects. If they're different types, they're not equal.
  field1 = /** @type {!Object} */(field1);
  field2 = /** @type {!Object} */(field2);
  if (field1.constructor != field2.constructor) return false;

  // If both are Uint8Arrays, compare them element-by-element.
  if (jspb.Message.SUPPORTS_UINT8ARRAY_ && field1.constructor === Uint8Array) {
    var bytes1 = /** @type {!Uint8Array} */(field1);
    var bytes2 = /** @type {!Uint8Array} */(field2);
    if (bytes1.length != bytes2.length) return false;
    for (var i = 0; i < bytes1.length; i++) {
      if (bytes1[i] != bytes2[i]) return false;
    }
    return true;
  }

  // If they're both Arrays, compare them element by element except for the
  // optional extension objects at the end, which we compare separately.
  if (field1.constructor === Array) {
    var typedField1 = /** @type {!Array<?>} */ (field1);
    var typedField2 = /** @type {!Array<?>} */ (field2);
    var extension1 = undefined;
    var extension2 = undefined;

    var length = Math.max(typedField1.length, typedField2.length);
    for (var i = 0; i < length; i++) {
      var val1 = typedField1[i];
      var val2 = typedField2[i];

      if (val1 && (val1.constructor == Object)) {
        goog.asserts.assert(extension1 === undefined);
        goog.asserts.assert(i === typedField1.length - 1);
        extension1 = val1;
        val1 = undefined;
      }

      if (val2 && (val2.constructor == Object)) {
        goog.asserts.assert(extension2 === undefined);
        goog.asserts.assert(i === typedField2.length - 1);
        extension2 = val2;
        val2 = undefined;
      }

      if (!jspb.Message.compareFields(val1, val2)) {
        return false;
      }
    }

    if (extension1 || extension2) {
      extension1 = extension1 || {};
      extension2 = extension2 || {};
      return jspb.Message.compareExtensions(extension1, extension2);
    }

    return true;
  }

  // If they're both plain Objects (i.e. extensions), compare them as
  // extensions.
  if (field1.constructor === Object) {
    return jspb.Message.compareExtensions(field1, field2);
  }

  throw new Error('Invalid type in JSPB array');
};


/**
 * Templated, type-safe cloneMessage definition.
 * @return {THIS}
 * @this {THIS}
 * @template THIS
 */
jspb.Message.prototype.cloneMessage = function() {
  return jspb.Message.cloneMessage(/** @type {!jspb.Message} */ (this));
};

/**
 * Alias clone to cloneMessage. goog.object.unsafeClone uses clone to
 * efficiently copy objects. Without this alias, copying jspb messages comes
 * with a large performance penalty.
 * @return {THIS}
 * @this {THIS}
 * @template THIS
 */
jspb.Message.prototype.clone = function() {
  return jspb.Message.cloneMessage(/** @type {!jspb.Message} */ (this));
};

/**
 * Static clone function. NOTE: A type-safe method called "cloneMessage"
 * exists
 * on each generated JsPb class. Do not call this function directly.
 * @param {!jspb.Message} msg A message to clone.
 * @return {!jspb.Message} A deep clone of the given message.
 */
jspb.Message.clone = function(msg) {
  // Although we could include the wrappers, we leave them out here.
  return jspb.Message.cloneMessage(msg);
};


/**
 * @param {!jspb.Message} msg A message to clone.
 * @return {!jspb.Message} A deep clone of the given message.
 * @protected
 */
jspb.Message.cloneMessage = function(msg) {
  // Although we could include the wrappers, we leave them out here.
  return new msg.constructor(jspb.Message.clone_(msg.toArray()));
};


/**
 * Takes 2 messages of the same type and copies the contents of the first
 * message into the second. After this the 2 messages will equals in terms of
 * value semantics but share no state. All data in the destination message will
 * be overridden.
 *
 * @param {MESSAGE} fromMessage Message that will be copied into toMessage.
 * @param {MESSAGE} toMessage Message which will receive a copy of fromMessage
 *     as its contents.
 * @template MESSAGE
 */
jspb.Message.copyInto = function(fromMessage, toMessage) {
  goog.asserts.assertInstanceof(fromMessage, jspb.Message);
  goog.asserts.assertInstanceof(toMessage, jspb.Message);
  goog.asserts.assert(fromMessage.constructor == toMessage.constructor,
      'Copy source and target message should have the same type.');
  var copyOfFrom = jspb.Message.clone(fromMessage);

  var to = toMessage.toArray();
  var from = copyOfFrom.toArray();

  // Empty destination in case it has more values at the end of the array.
  to.length = 0;
  // and then copy everything from the new to the existing message.
  for (var i = 0; i < from.length; i++) {
    to[i] = from[i];
  }

  // This is either null or empty for a fresh copy.
  toMessage.wrappers_ = copyOfFrom.wrappers_;
  // Just a reference into the shared array.
  toMessage.extensionObject_ = copyOfFrom.extensionObject_;
};


/**
 * Helper for cloning an internal JsPb object.
 * @param {!Object} obj A JsPb object, eg, a field, to be cloned.
 * @return {!Object} A clone of the input object.
 * @private
 */
jspb.Message.clone_ = function(obj) {
  var o;
  if (Array.isArray(obj)) {
    // Allocate array of correct size.
    var clonedArray = new Array(obj.length);
    // Use array iteration where possible because it is faster than for-in.
    for (var i = 0; i < obj.length; i++) {
      o = obj[i];
      if (o != null) {
        // NOTE:redundant null check existing for NTI compatibility.
        // see b/70515949
        clonedArray[i] = (typeof o == 'object') ?
            jspb.Message.clone_(goog.asserts.assert(o)) :
            o;
      }
    }
    return clonedArray;
  }
  if (jspb.Message.SUPPORTS_UINT8ARRAY_ && obj instanceof Uint8Array) {
    return new Uint8Array(obj);
  }
  var clone = {};
  for (var key in obj) {
    o = obj[key];
    if (o != null) {
      // NOTE:redundant null check existing for NTI compatibility.
      // see b/70515949
      clone[key] = (typeof o == 'object') ?
          jspb.Message.clone_(goog.asserts.assert(o)) :
          o;
    }
  }
  return clone;
};


/**
 * Registers a JsPb message type id with its constructor.
 * @param {string} id The id for this type of message.
 * @param {Function} constructor The message constructor.
 */
jspb.Message.registerMessageType = function(id, constructor) {
  // This is needed so we can later access messageId directly on the contructor,
  // otherwise it is not available due to 'property collapsing' by the compiler.
  /**
   * @suppress {strictMissingProperties} messageId is not defined on Function
   */
  constructor.messageId = id;
};
/**
 * The extensions registered on MessageSet. This is a map of extension
 * field number to field info object. This should be considered as a
 * private API.
 *
 * This is similar to [jspb class name].extensions object for
 * non-MessageSet. We special case MessageSet so that we do not need
 * to goog.require MessageSet from classes that extends MessageSet.
 *
 * @type {!Object<number, jspb.ExtensionFieldInfo>}
 */
jspb.Message.messageSetExtensions = {};

/**
 * @type {!Object<number, jspb.ExtensionFieldBinaryInfo>}
 */
jspb.Message.messageSetExtensionsBinary = {};

//location/unified/proto/location_descriptor_proto_jspb/activityrecord.jspb.js
// source: location/unified/proto/location_descriptor.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.location.unified.ActivityRecord');
goog.provide('proto.location.unified.ActivityRecord.Type');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.location.unified.ActivityRecord = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.location.unified.ActivityRecord, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.location.unified.ActivityRecord.displayName = 'proto.location.unified.ActivityRecord';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.location.unified.ActivityRecord.prototype.toObject = function(opt_includeInstance) {
  return proto.location.unified.ActivityRecord.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.location.unified.ActivityRecord} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.location.unified.ActivityRecord.toObject = function(includeInstance, msg) {
  var f, obj = {
    type: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    confidence: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of ActivityRecord as accepted by the `fromObject` method.
 * @record
 */
proto.location.unified.ActivityRecord.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.type;

  /** @type {?number|undefined} */
  this.confidence;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.location.unified.ActivityRecord.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.location.unified.ActivityRecord}
 */
proto.location.unified.ActivityRecord.fromObject = function(obj) {
  var msg = new proto.location.unified.ActivityRecord();
  obj.type != null && jspb.Message.setField(msg, 1, obj.type);
  obj.confidence != null && jspb.Message.setField(msg, 2, obj.confidence);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.location.unified.ActivityRecord}
 */
proto.location.unified.ActivityRecord.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.location.unified.ActivityRecord;
  return proto.location.unified.ActivityRecord.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.location.unified.ActivityRecord} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.location.unified.ActivityRecord}
 */
proto.location.unified.ActivityRecord.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {!proto.location.unified.ActivityRecord.Type} */ (reader.readEnum());
      msg.setType(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setConfidence(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.location.unified.ActivityRecord.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.location.unified.ActivityRecord.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.location.unified.ActivityRecord} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.location.unified.ActivityRecord.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {!proto.location.unified.ActivityRecord.Type} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeEnum(
      1,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeInt32(
      2,
      f
    );
  }
};


/**
 * @enum {number}
 */
proto.location.unified.ActivityRecord.Type = {
  IN_VEHICLE: 0,
  ON_BICYCLE: 1,
  ON_FOOT: 2,
  STILL: 3,
  UNKNOWN: 4,
  TILTING: 5,
  EXITING_VEHICLE: 6,
  WALKING: 7,
  RUNNING: 8,
  OFF_BODY: 9,
  TRUSTED_GAIT: 10,
  FLOOR_CHANGE: 11,
  ON_STAIRS: 12,
  ON_ESCALATOR: 13,
  IN_ELEVATOR: 14,
  SLEEPING: 15,
  IN_ROAD_VEHICLE: 16,
  IN_RAIL_VEHICLE: 17,
  IN_TWO_WHEELER_VEHICLE: 18,
  IN_FOUR_WHEELER_VEHICLE: 19,
  IN_CAR: 20,
  IN_BUS: 21,
  EXPERIMENTAL_EXTRA_PERSONAL_VEHICLE: -1000
};

/**
 * optional Type type = 1;
 * @return {!proto.location.unified.ActivityRecord.Type}
 */
proto.location.unified.ActivityRecord.prototype.getType = function() {
  return /** @type {!proto.location.unified.ActivityRecord.Type} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?proto.location.unified.ActivityRecord.Type|undefined} value
 * @return {!proto.location.unified.ActivityRecord} returns this
 */
proto.location.unified.ActivityRecord.prototype.setType = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.ActivityRecord} returns this
 */
proto.location.unified.ActivityRecord.prototype.clearType = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.ActivityRecord.prototype.hasType = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional int32 confidence = 2;
 * @return {?number}
 */
proto.location.unified.ActivityRecord.prototype.getConfidence = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.location.unified.ActivityRecord} returns this
 */
proto.location.unified.ActivityRecord.prototype.setConfidence = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.ActivityRecord} returns this
 */
proto.location.unified.ActivityRecord.prototype.clearConfidence = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.ActivityRecord.prototype.hasConfidence = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.location.unified.ActivityRecord}
 */
proto.location.unified.ActivityRecord.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.location.unified.ActivityRecord, data);
};



//location/unified/proto/location_descriptor_proto_jspb/featureidproto.jspb.js
// source: location/unified/proto/location_descriptor.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.location.unified.FeatureIdProto');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.location.unified.FeatureIdProto = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.location.unified.FeatureIdProto, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.location.unified.FeatureIdProto.displayName = 'proto.location.unified.FeatureIdProto';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.location.unified.FeatureIdProto.prototype.toObject = function(opt_includeInstance) {
  return proto.location.unified.FeatureIdProto.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.location.unified.FeatureIdProto} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.location.unified.FeatureIdProto.toObject = function(includeInstance, msg) {
  var f, obj = {
    cellId: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    fprint: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of FeatureIdProto as accepted by the `fromObject` method.
 * @record
 */
proto.location.unified.FeatureIdProto.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.cellId;

  /** @type {?number|undefined} */
  this.fprint;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.location.unified.FeatureIdProto.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.location.unified.FeatureIdProto}
 */
proto.location.unified.FeatureIdProto.fromObject = function(obj) {
  var msg = new proto.location.unified.FeatureIdProto();
  obj.cellId != null && jspb.Message.setField(msg, 1, obj.cellId);
  obj.fprint != null && jspb.Message.setField(msg, 2, obj.fprint);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.location.unified.FeatureIdProto}
 */
proto.location.unified.FeatureIdProto.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.location.unified.FeatureIdProto;
  return proto.location.unified.FeatureIdProto.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.location.unified.FeatureIdProto} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.location.unified.FeatureIdProto}
 */
proto.location.unified.FeatureIdProto.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {number} */ (reader.readFixed64());
      msg.setCellId(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readFixed64());
      msg.setFprint(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.location.unified.FeatureIdProto.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.location.unified.FeatureIdProto.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.location.unified.FeatureIdProto} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.location.unified.FeatureIdProto.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?number} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeFixed64(
      1,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeFixed64(
      2,
      f
    );
  }
};


/**
 * optional fixed64 cell_id = 1;
 * @return {?number}
 */
proto.location.unified.FeatureIdProto.prototype.getCellId = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.location.unified.FeatureIdProto} returns this
 */
proto.location.unified.FeatureIdProto.prototype.setCellId = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.FeatureIdProto} returns this
 */
proto.location.unified.FeatureIdProto.prototype.clearCellId = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.FeatureIdProto.prototype.hasCellId = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional fixed64 fprint = 2;
 * @return {?number}
 */
proto.location.unified.FeatureIdProto.prototype.getFprint = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.location.unified.FeatureIdProto} returns this
 */
proto.location.unified.FeatureIdProto.prototype.setFprint = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.FeatureIdProto} returns this
 */
proto.location.unified.FeatureIdProto.prototype.clearFprint = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.FeatureIdProto.prototype.hasFprint = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.location.unified.FeatureIdProto}
 */
proto.location.unified.FeatureIdProto.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.location.unified.FeatureIdProto, data);
};



//location/unified/proto/location_descriptor_proto_jspb/fieldofview.jspb.js
// source: location/unified/proto/location_descriptor.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.location.unified.FieldOfView');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.location.unified.FieldOfView = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.location.unified.FieldOfView, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.location.unified.FieldOfView.displayName = 'proto.location.unified.FieldOfView';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.location.unified.FieldOfView.prototype.toObject = function(opt_includeInstance) {
  return proto.location.unified.FieldOfView.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.location.unified.FieldOfView} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.location.unified.FieldOfView.toObject = function(includeInstance, msg) {
  var f, obj = {
    fieldOfViewXDegrees: (f = jspb.Message.getOptionalFloatingPointField(msg, 1)) == null ? undefined : f,
    fieldOfViewYDegrees: (f = jspb.Message.getOptionalFloatingPointField(msg, 2)) == null ? undefined : f,
    screenWidthPixels: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of FieldOfView as accepted by the `fromObject` method.
 * @record
 */
proto.location.unified.FieldOfView.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.fieldOfViewXDegrees;

  /** @type {?number|undefined} */
  this.fieldOfViewYDegrees;

  /** @type {?number|undefined} */
  this.screenWidthPixels;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.location.unified.FieldOfView.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.location.unified.FieldOfView}
 */
proto.location.unified.FieldOfView.fromObject = function(obj) {
  var msg = new proto.location.unified.FieldOfView();
  obj.fieldOfViewXDegrees != null && jspb.Message.setField(msg, 1, obj.fieldOfViewXDegrees);
  obj.fieldOfViewYDegrees != null && jspb.Message.setField(msg, 2, obj.fieldOfViewYDegrees);
  obj.screenWidthPixels != null && jspb.Message.setField(msg, 3, obj.screenWidthPixels);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.location.unified.FieldOfView}
 */
proto.location.unified.FieldOfView.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.location.unified.FieldOfView;
  return proto.location.unified.FieldOfView.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.location.unified.FieldOfView} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.location.unified.FieldOfView}
 */
proto.location.unified.FieldOfView.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setFieldOfViewXDegrees(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setFieldOfViewYDegrees(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setScreenWidthPixels(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.location.unified.FieldOfView.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.location.unified.FieldOfView.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.location.unified.FieldOfView} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.location.unified.FieldOfView.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?number} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeFloat(
      1,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeFloat(
      2,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeInt32(
      3,
      f
    );
  }
};


/**
 * optional float field_of_view_x_degrees = 1;
 * @return {?number}
 */
proto.location.unified.FieldOfView.prototype.getFieldOfViewXDegrees = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 1));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.location.unified.FieldOfView} returns this
 */
proto.location.unified.FieldOfView.prototype.setFieldOfViewXDegrees = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.FieldOfView} returns this
 */
proto.location.unified.FieldOfView.prototype.clearFieldOfViewXDegrees = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.FieldOfView.prototype.hasFieldOfViewXDegrees = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional float field_of_view_y_degrees = 2;
 * @return {?number}
 */
proto.location.unified.FieldOfView.prototype.getFieldOfViewYDegrees = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 2));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.location.unified.FieldOfView} returns this
 */
proto.location.unified.FieldOfView.prototype.setFieldOfViewYDegrees = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.FieldOfView} returns this
 */
proto.location.unified.FieldOfView.prototype.clearFieldOfViewYDegrees = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.FieldOfView.prototype.hasFieldOfViewYDegrees = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional int32 screen_width_pixels = 3;
 * @return {?number}
 */
proto.location.unified.FieldOfView.prototype.getScreenWidthPixels = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.location.unified.FieldOfView} returns this
 */
proto.location.unified.FieldOfView.prototype.setScreenWidthPixels = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.FieldOfView} returns this
 */
proto.location.unified.FieldOfView.prototype.clearScreenWidthPixels = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.FieldOfView.prototype.hasScreenWidthPixels = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.location.unified.FieldOfView}
 */
proto.location.unified.FieldOfView.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.location.unified.FieldOfView, data);
};



//location/unified/proto/location_descriptor_proto_jspb/latlng.jspb.js
// source: location/unified/proto/location_descriptor.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.location.unified.LatLng');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.location.unified.LatLng = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.location.unified.LatLng, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.location.unified.LatLng.displayName = 'proto.location.unified.LatLng';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.location.unified.LatLng.prototype.toObject = function(opt_includeInstance) {
  return proto.location.unified.LatLng.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.location.unified.LatLng} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.location.unified.LatLng.toObject = function(includeInstance, msg) {
  var f, obj = {
    latitudeE7: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    longitudeE7: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of LatLng as accepted by the `fromObject` method.
 * @record
 */
proto.location.unified.LatLng.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.latitudeE7;

  /** @type {?number|undefined} */
  this.longitudeE7;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.location.unified.LatLng.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.location.unified.LatLng}
 */
proto.location.unified.LatLng.fromObject = function(obj) {
  var msg = new proto.location.unified.LatLng();
  obj.latitudeE7 != null && jspb.Message.setField(msg, 1, obj.latitudeE7);
  obj.longitudeE7 != null && jspb.Message.setField(msg, 2, obj.longitudeE7);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.location.unified.LatLng}
 */
proto.location.unified.LatLng.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.location.unified.LatLng;
  return proto.location.unified.LatLng.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.location.unified.LatLng} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.location.unified.LatLng}
 */
proto.location.unified.LatLng.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {number} */ (reader.readSfixed32());
      msg.setLatitudeE7(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readSfixed32());
      msg.setLongitudeE7(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.location.unified.LatLng.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.location.unified.LatLng.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.location.unified.LatLng} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.location.unified.LatLng.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?number} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeSfixed32(
      1,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeSfixed32(
      2,
      f
    );
  }
};


/**
 * optional sfixed32 latitude_e7 = 1;
 * @return {?number}
 */
proto.location.unified.LatLng.prototype.getLatitudeE7 = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.location.unified.LatLng} returns this
 */
proto.location.unified.LatLng.prototype.setLatitudeE7 = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.LatLng} returns this
 */
proto.location.unified.LatLng.prototype.clearLatitudeE7 = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.LatLng.prototype.hasLatitudeE7 = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional sfixed32 longitude_e7 = 2;
 * @return {?number}
 */
proto.location.unified.LatLng.prototype.getLongitudeE7 = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.location.unified.LatLng} returns this
 */
proto.location.unified.LatLng.prototype.setLongitudeE7 = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.LatLng} returns this
 */
proto.location.unified.LatLng.prototype.clearLongitudeE7 = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.LatLng.prototype.hasLongitudeE7 = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.location.unified.LatLng}
 */
proto.location.unified.LatLng.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.location.unified.LatLng, data);
};



//location/unified/proto/location_descriptor_proto_jspb/latlngrect.jspb.js
// source: location/unified/proto/location_descriptor.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.location.unified.LatLngRect');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.location.unified.LatLng');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.location.unified.LatLngRect = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.location.unified.LatLngRect, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.location.unified.LatLngRect.displayName = 'proto.location.unified.LatLngRect';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.location.unified.LatLngRect.prototype.toObject = function(opt_includeInstance) {
  return proto.location.unified.LatLngRect.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.location.unified.LatLngRect} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.location.unified.LatLngRect.toObject = function(includeInstance, msg) {
  var f, obj = {
    lo: (f = msg.getLo()) && proto.location.unified.LatLng.toObject(includeInstance, f),
    hi: (f = msg.getHi()) && proto.location.unified.LatLng.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of LatLngRect as accepted by the `fromObject` method.
 * @record
 */
proto.location.unified.LatLngRect.ObjectFormat = function() {
  /** @type {?proto.location.unified.LatLng.ObjectFormat|undefined} */
  this.lo;

  /** @type {?proto.location.unified.LatLng.ObjectFormat|undefined} */
  this.hi;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.location.unified.LatLngRect.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.location.unified.LatLngRect}
 */
proto.location.unified.LatLngRect.fromObject = function(obj) {
  var msg = new proto.location.unified.LatLngRect();
  obj.lo && jspb.Message.setWrapperField(
      msg, 1, proto.location.unified.LatLng.fromObject(obj.lo));
  obj.hi && jspb.Message.setWrapperField(
      msg, 2, proto.location.unified.LatLng.fromObject(obj.hi));
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.location.unified.LatLngRect}
 */
proto.location.unified.LatLngRect.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.location.unified.LatLngRect;
  return proto.location.unified.LatLngRect.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.location.unified.LatLngRect} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.location.unified.LatLngRect}
 */
proto.location.unified.LatLngRect.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.location.unified.LatLng;
      reader.readMessage(value,proto.location.unified.LatLng.deserializeBinaryFromReader);
      msg.setLo(value);
      break;
    case 2:
      var value = new proto.location.unified.LatLng;
      reader.readMessage(value,proto.location.unified.LatLng.deserializeBinaryFromReader);
      msg.setHi(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.location.unified.LatLngRect.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.location.unified.LatLngRect.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.location.unified.LatLngRect} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.location.unified.LatLngRect.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getLo();
  if (f != null) {
    writer.writeMessage(
      1,
      f,
      proto.location.unified.LatLng.serializeBinaryToWriter
    );
  }
  f = message.getHi();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.location.unified.LatLng.serializeBinaryToWriter
    );
  }
};


/**
 * optional LatLng lo = 1;
 * @return {?proto.location.unified.LatLng}
 */
proto.location.unified.LatLngRect.prototype.getLo = function() {
  return /** @type{?proto.location.unified.LatLng} */ (
    jspb.Message.getWrapperField(this, proto.location.unified.LatLng, 1));
};


/**
 * @param {?proto.location.unified.LatLng|undefined} value
 * @return {!proto.location.unified.LatLngRect} returns this
*/
proto.location.unified.LatLngRect.prototype.setLo = function(value) {
  return jspb.Message.setWrapperField(this, 1, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.location.unified.LatLngRect} returns this
 */
proto.location.unified.LatLngRect.prototype.clearLo = function() {
  return this.setLo(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.LatLngRect.prototype.hasLo = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional LatLng hi = 2;
 * @return {?proto.location.unified.LatLng}
 */
proto.location.unified.LatLngRect.prototype.getHi = function() {
  return /** @type{?proto.location.unified.LatLng} */ (
    jspb.Message.getWrapperField(this, proto.location.unified.LatLng, 2));
};


/**
 * @param {?proto.location.unified.LatLng|undefined} value
 * @return {!proto.location.unified.LatLngRect} returns this
*/
proto.location.unified.LatLngRect.prototype.setHi = function(value) {
  return jspb.Message.setWrapperField(this, 2, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.location.unified.LatLngRect} returns this
 */
proto.location.unified.LatLngRect.prototype.clearHi = function() {
  return this.setHi(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.LatLngRect.prototype.hasHi = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.location.unified.LatLngRect}
 */
proto.location.unified.LatLngRect.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.location.unified.LatLngRect, data);
};



//location/unified/proto/location_descriptor_proto_jspb/personalizedlocationattributes.jspb.js
// source: location/unified/proto/location_descriptor.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.location.unified.PersonalizedLocationAttributes');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.location.unified.PersonalizedLocationAttributes = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.location.unified.PersonalizedLocationAttributes, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.location.unified.PersonalizedLocationAttributes.displayName = 'proto.location.unified.PersonalizedLocationAttributes';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.location.unified.PersonalizedLocationAttributes.prototype.toObject = function(opt_includeInstance) {
  return proto.location.unified.PersonalizedLocationAttributes.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.location.unified.PersonalizedLocationAttributes} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.location.unified.PersonalizedLocationAttributes.toObject = function(includeInstance, msg) {
  var f, obj = {
    ppSupportScore: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    ppRegularityScore: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    ppEiSupportDiff: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,
    ppSupportingDays: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f,
    ppSupportingWeeks: (f = jspb.Message.getField(msg, 5)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of PersonalizedLocationAttributes as accepted by the `fromObject` method.
 * @record
 */
proto.location.unified.PersonalizedLocationAttributes.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.ppSupportScore;

  /** @type {?number|undefined} */
  this.ppRegularityScore;

  /** @type {?number|undefined} */
  this.ppEiSupportDiff;

  /** @type {?number|undefined} */
  this.ppSupportingDays;

  /** @type {?number|undefined} */
  this.ppSupportingWeeks;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.location.unified.PersonalizedLocationAttributes.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.location.unified.PersonalizedLocationAttributes}
 */
proto.location.unified.PersonalizedLocationAttributes.fromObject = function(obj) {
  var msg = new proto.location.unified.PersonalizedLocationAttributes();
  obj.ppSupportScore != null && jspb.Message.setField(msg, 1, obj.ppSupportScore);
  obj.ppRegularityScore != null && jspb.Message.setField(msg, 2, obj.ppRegularityScore);
  obj.ppEiSupportDiff != null && jspb.Message.setField(msg, 3, obj.ppEiSupportDiff);
  obj.ppSupportingDays != null && jspb.Message.setField(msg, 4, obj.ppSupportingDays);
  obj.ppSupportingWeeks != null && jspb.Message.setField(msg, 5, obj.ppSupportingWeeks);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.location.unified.PersonalizedLocationAttributes}
 */
proto.location.unified.PersonalizedLocationAttributes.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.location.unified.PersonalizedLocationAttributes;
  return proto.location.unified.PersonalizedLocationAttributes.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.location.unified.PersonalizedLocationAttributes} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.location.unified.PersonalizedLocationAttributes}
 */
proto.location.unified.PersonalizedLocationAttributes.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setPpSupportScore(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setPpRegularityScore(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setPpEiSupportDiff(value);
      break;
    case 4:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setPpSupportingDays(value);
      break;
    case 5:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setPpSupportingWeeks(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.location.unified.PersonalizedLocationAttributes.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.location.unified.PersonalizedLocationAttributes.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.location.unified.PersonalizedLocationAttributes} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.location.unified.PersonalizedLocationAttributes.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?number} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeInt32(
      1,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeInt32(
      2,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeInt32(
      3,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeInt32(
      4,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeInt32(
      5,
      f
    );
  }
};


/**
 * optional int32 pp_support_score = 1;
 * @return {?number}
 */
proto.location.unified.PersonalizedLocationAttributes.prototype.getPpSupportScore = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.location.unified.PersonalizedLocationAttributes} returns this
 */
proto.location.unified.PersonalizedLocationAttributes.prototype.setPpSupportScore = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.PersonalizedLocationAttributes} returns this
 */
proto.location.unified.PersonalizedLocationAttributes.prototype.clearPpSupportScore = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.PersonalizedLocationAttributes.prototype.hasPpSupportScore = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional int32 pp_regularity_score = 2;
 * @return {?number}
 */
proto.location.unified.PersonalizedLocationAttributes.prototype.getPpRegularityScore = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.location.unified.PersonalizedLocationAttributes} returns this
 */
proto.location.unified.PersonalizedLocationAttributes.prototype.setPpRegularityScore = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.PersonalizedLocationAttributes} returns this
 */
proto.location.unified.PersonalizedLocationAttributes.prototype.clearPpRegularityScore = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.PersonalizedLocationAttributes.prototype.hasPpRegularityScore = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional int32 pp_ei_support_diff = 3;
 * @return {?number}
 */
proto.location.unified.PersonalizedLocationAttributes.prototype.getPpEiSupportDiff = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.location.unified.PersonalizedLocationAttributes} returns this
 */
proto.location.unified.PersonalizedLocationAttributes.prototype.setPpEiSupportDiff = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.PersonalizedLocationAttributes} returns this
 */
proto.location.unified.PersonalizedLocationAttributes.prototype.clearPpEiSupportDiff = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.PersonalizedLocationAttributes.prototype.hasPpEiSupportDiff = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional int32 pp_supporting_days = 4;
 * @return {?number}
 */
proto.location.unified.PersonalizedLocationAttributes.prototype.getPpSupportingDays = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 4));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.location.unified.PersonalizedLocationAttributes} returns this
 */
proto.location.unified.PersonalizedLocationAttributes.prototype.setPpSupportingDays = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.PersonalizedLocationAttributes} returns this
 */
proto.location.unified.PersonalizedLocationAttributes.prototype.clearPpSupportingDays = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.PersonalizedLocationAttributes.prototype.hasPpSupportingDays = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional int32 pp_supporting_weeks = 5;
 * @return {?number}
 */
proto.location.unified.PersonalizedLocationAttributes.prototype.getPpSupportingWeeks = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 5));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.location.unified.PersonalizedLocationAttributes} returns this
 */
proto.location.unified.PersonalizedLocationAttributes.prototype.setPpSupportingWeeks = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.PersonalizedLocationAttributes} returns this
 */
proto.location.unified.PersonalizedLocationAttributes.prototype.clearPpSupportingWeeks = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.PersonalizedLocationAttributes.prototype.hasPpSupportingWeeks = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.location.unified.PersonalizedLocationAttributes}
 */
proto.location.unified.PersonalizedLocationAttributes.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.location.unified.PersonalizedLocationAttributes, data);
};



//location/unified/proto/location_descriptor_proto_jspb/locationattributesproto.jspb.js
// source: location/unified/proto/location_descriptor.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.location.unified.LocationAttributesProto');
goog.provide('proto.location.unified.LocationAttributesProto.CarrierIpType');
goog.provide('proto.location.unified.LocationAttributesProto.Inference');
goog.provide('proto.location.unified.LocationAttributesProto.ManualEntry');
goog.provide('proto.location.unified.LocationAttributesProto.PLMType');
goog.provide('proto.location.unified.LocationAttributesProto.ViewportSearchOptions');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.location.unified.ActivityRecord');
goog.require('proto.location.unified.FieldOfView');
goog.require('proto.location.unified.PersonalizedLocationAttributes');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.location.unified.LocationAttributesProto = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.location.unified.LocationAttributesProto.repeatedFields_, null);
};
goog.inherits(proto.location.unified.LocationAttributesProto, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.location.unified.LocationAttributesProto.displayName = 'proto.location.unified.LocationAttributesProto';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.location.unified.LocationAttributesProto.repeatedFields_ = [11];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.location.unified.LocationAttributesProto.prototype.toObject = function(opt_includeInstance) {
  return proto.location.unified.LocationAttributesProto.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.location.unified.LocationAttributesProto} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.location.unified.LocationAttributesProto.toObject = function(includeInstance, msg) {
  var f, obj = {
    headingDegrees: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    bearingDegrees: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,
    bearingAccuracyDegrees: (f = jspb.Message.getField(msg, 12)) == null ? undefined : f,
    speedKph: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f,
    speedAccuracyKph: (f = jspb.Message.getField(msg, 13)) == null ? undefined : f,
    tiltDegrees: (f = jspb.Message.getField(msg, 5)) == null ? undefined : f,
    rollDegrees: (f = jspb.Message.getField(msg, 6)) == null ? undefined : f,
    altitudeMetersFromGround: (f = jspb.Message.getOptionalFloatingPointField(msg, 7)) == null ? undefined : f,
    fieldOfView: (f = msg.getFieldOfView()) && proto.location.unified.FieldOfView.toObject(includeInstance, f),
    boardedTransitVehicleToken: (f = jspb.Message.getField(msg, 9)) == null ? undefined : f,
    activityRecordList: jspb.Message.toObjectList(msg.getActivityRecordList(),
    proto.location.unified.ActivityRecord.toObject, includeInstance),
    plmType: (f = jspb.Message.getField(msg, 14)) == null ? undefined : f,
    inference: (f = jspb.Message.getField(msg, 15)) == null ? undefined : f,
    manualEntry: (f = jspb.Message.getField(msg, 16)) == null ? undefined : f,
    weekSecondConfidence: (f = jspb.Message.getOptionalFloatingPointField(msg, 17)) == null ? undefined : f,
    ipRangeConfidence: (f = jspb.Message.getOptionalFloatingPointField(msg, 18)) == null ? undefined : f,
    carrierIpType: (f = jspb.Message.getField(msg, 19)) == null ? undefined : f,
    adsConfidence: (f = jspb.Message.getOptionalFloatingPointField(msg, 20)) == null ? undefined : f,
    viewportSearchOptions: (f = jspb.Message.getField(msg, 21)) == null ? undefined : f,
    deviceLocationRatio: (f = jspb.Message.getOptionalFloatingPointField(msg, 10)) == null ? undefined : f,
    plmSourceLocationCount: (f = jspb.Message.getField(msg, 22)) == null ? undefined : f,
    personalizedLocationAttributes: (f = msg.getPersonalizedLocationAttributes()) && proto.location.unified.PersonalizedLocationAttributes.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of LocationAttributesProto as accepted by the `fromObject` method.
 * @record
 */
proto.location.unified.LocationAttributesProto.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.headingDegrees;

  /** @type {?number|undefined} */
  this.bearingDegrees;

  /** @type {?number|undefined} */
  this.bearingAccuracyDegrees;

  /** @type {?number|undefined} */
  this.speedKph;

  /** @type {?number|undefined} */
  this.speedAccuracyKph;

  /** @type {?number|undefined} */
  this.tiltDegrees;

  /** @type {?number|undefined} */
  this.rollDegrees;

  /** @type {?number|undefined} */
  this.altitudeMetersFromGround;

  /** @type {?proto.location.unified.FieldOfView.ObjectFormat|undefined} */
  this.fieldOfView;

  /** @type {?string|undefined} */
  this.boardedTransitVehicleToken;

  /** @type {?Array<!proto.location.unified.ActivityRecord.ObjectFormat>|undefined} */
  this.activityRecordList;

  /** @type {?number|undefined} */
  this.plmType;

  /** @type {?number|undefined} */
  this.inference;

  /** @type {?number|undefined} */
  this.manualEntry;

  /** @type {?number|undefined} */
  this.weekSecondConfidence;

  /** @type {?number|undefined} */
  this.ipRangeConfidence;

  /** @type {?number|undefined} */
  this.carrierIpType;

  /** @type {?number|undefined} */
  this.adsConfidence;

  /** @type {?number|undefined} */
  this.viewportSearchOptions;

  /** @type {?number|undefined} */
  this.deviceLocationRatio;

  /** @type {?number|undefined} */
  this.plmSourceLocationCount;

  /** @type {?proto.location.unified.PersonalizedLocationAttributes.ObjectFormat|undefined} */
  this.personalizedLocationAttributes;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.location.unified.LocationAttributesProto.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.location.unified.LocationAttributesProto}
 */
proto.location.unified.LocationAttributesProto.fromObject = function(obj) {
  var msg = new proto.location.unified.LocationAttributesProto();
  obj.headingDegrees != null && jspb.Message.setField(msg, 2, obj.headingDegrees);
  obj.bearingDegrees != null && jspb.Message.setField(msg, 3, obj.bearingDegrees);
  obj.bearingAccuracyDegrees != null && jspb.Message.setField(msg, 12, obj.bearingAccuracyDegrees);
  obj.speedKph != null && jspb.Message.setField(msg, 4, obj.speedKph);
  obj.speedAccuracyKph != null && jspb.Message.setField(msg, 13, obj.speedAccuracyKph);
  obj.tiltDegrees != null && jspb.Message.setField(msg, 5, obj.tiltDegrees);
  obj.rollDegrees != null && jspb.Message.setField(msg, 6, obj.rollDegrees);
  obj.altitudeMetersFromGround != null && jspb.Message.setField(msg, 7, obj.altitudeMetersFromGround);
  obj.fieldOfView && jspb.Message.setWrapperField(
      msg, 8, proto.location.unified.FieldOfView.fromObject(obj.fieldOfView));
  obj.boardedTransitVehicleToken != null && jspb.Message.setField(msg, 9, obj.boardedTransitVehicleToken);
  obj.activityRecordList && jspb.Message.setRepeatedWrapperField(
      msg, 11, obj.activityRecordList.map(
          proto.location.unified.ActivityRecord.fromObject));
  obj.plmType != null && jspb.Message.setField(msg, 14, obj.plmType);
  obj.inference != null && jspb.Message.setField(msg, 15, obj.inference);
  obj.manualEntry != null && jspb.Message.setField(msg, 16, obj.manualEntry);
  obj.weekSecondConfidence != null && jspb.Message.setField(msg, 17, obj.weekSecondConfidence);
  obj.ipRangeConfidence != null && jspb.Message.setField(msg, 18, obj.ipRangeConfidence);
  obj.carrierIpType != null && jspb.Message.setField(msg, 19, obj.carrierIpType);
  obj.adsConfidence != null && jspb.Message.setField(msg, 20, obj.adsConfidence);
  obj.viewportSearchOptions != null && jspb.Message.setField(msg, 21, obj.viewportSearchOptions);
  obj.deviceLocationRatio != null && jspb.Message.setField(msg, 10, obj.deviceLocationRatio);
  obj.plmSourceLocationCount != null && jspb.Message.setField(msg, 22, obj.plmSourceLocationCount);
  obj.personalizedLocationAttributes && jspb.Message.setWrapperField(
      msg, 23, proto.location.unified.PersonalizedLocationAttributes.fromObject(obj.personalizedLocationAttributes));
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.location.unified.LocationAttributesProto}
 */
proto.location.unified.LocationAttributesProto.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.location.unified.LocationAttributesProto;
  return proto.location.unified.LocationAttributesProto.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.location.unified.LocationAttributesProto} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.location.unified.LocationAttributesProto}
 */
proto.location.unified.LocationAttributesProto.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 2:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setHeadingDegrees(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setBearingDegrees(value);
      break;
    case 12:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setBearingAccuracyDegrees(value);
      break;
    case 4:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setSpeedKph(value);
      break;
    case 13:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setSpeedAccuracyKph(value);
      break;
    case 5:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setTiltDegrees(value);
      break;
    case 6:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setRollDegrees(value);
      break;
    case 7:
      var value = /** @type {number} */ (reader.readDouble());
      msg.setAltitudeMetersFromGround(value);
      break;
    case 8:
      var value = new proto.location.unified.FieldOfView;
      reader.readMessage(value,proto.location.unified.FieldOfView.deserializeBinaryFromReader);
      msg.setFieldOfView(value);
      break;
    case 9:
      var value = /** @type {string} */ (reader.readString());
      msg.setBoardedTransitVehicleToken(value);
      break;
    case 11:
      var value = new proto.location.unified.ActivityRecord;
      reader.readMessage(value,proto.location.unified.ActivityRecord.deserializeBinaryFromReader);
      msg.addActivityRecord(value);
      break;
    case 14:
      var value = /** @type {!proto.location.unified.LocationAttributesProto.PLMType} */ (reader.readEnum());
      msg.setPlmType(value);
      break;
    case 15:
      var value = /** @type {!proto.location.unified.LocationAttributesProto.Inference} */ (reader.readEnum());
      msg.setInference(value);
      break;
    case 16:
      var value = /** @type {!proto.location.unified.LocationAttributesProto.ManualEntry} */ (reader.readEnum());
      msg.setManualEntry(value);
      break;
    case 17:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setWeekSecondConfidence(value);
      break;
    case 18:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setIpRangeConfidence(value);
      break;
    case 19:
      var value = /** @type {!proto.location.unified.LocationAttributesProto.CarrierIpType} */ (reader.readEnum());
      msg.setCarrierIpType(value);
      break;
    case 20:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setAdsConfidence(value);
      break;
    case 21:
      var value = /** @type {!proto.location.unified.LocationAttributesProto.ViewportSearchOptions} */ (reader.readEnum());
      msg.setViewportSearchOptions(value);
      break;
    case 10:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setDeviceLocationRatio(value);
      break;
    case 22:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setPlmSourceLocationCount(value);
      break;
    case 23:
      var value = new proto.location.unified.PersonalizedLocationAttributes;
      reader.readMessage(value,proto.location.unified.PersonalizedLocationAttributes.deserializeBinaryFromReader);
      msg.setPersonalizedLocationAttributes(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.location.unified.LocationAttributesProto.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.location.unified.LocationAttributesProto.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.location.unified.LocationAttributesProto} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.location.unified.LocationAttributesProto.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeInt32(
      2,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeInt32(
      3,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 12));
  if (f != null) {
    writer.writeInt32(
      12,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeInt32(
      4,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 13));
  if (f != null) {
    writer.writeInt32(
      13,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeInt32(
      5,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeInt32(
      6,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 7));
  if (f != null) {
    writer.writeDouble(
      7,
      f
    );
  }
  f = message.getFieldOfView();
  if (f != null) {
    writer.writeMessage(
      8,
      f,
      proto.location.unified.FieldOfView.serializeBinaryToWriter
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 9));
  if (f != null) {
    writer.writeString(
      9,
      f
    );
  }
  f = message.getActivityRecordList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      11,
      f,
      proto.location.unified.ActivityRecord.serializeBinaryToWriter
    );
  }
  f = /** @type {!proto.location.unified.LocationAttributesProto.PLMType} */ (jspb.Message.getField(message, 14));
  if (f != null) {
    writer.writeEnum(
      14,
      f
    );
  }
  f = /** @type {!proto.location.unified.LocationAttributesProto.Inference} */ (jspb.Message.getField(message, 15));
  if (f != null) {
    writer.writeEnum(
      15,
      f
    );
  }
  f = /** @type {!proto.location.unified.LocationAttributesProto.ManualEntry} */ (jspb.Message.getField(message, 16));
  if (f != null) {
    writer.writeEnum(
      16,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 17));
  if (f != null) {
    writer.writeFloat(
      17,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 18));
  if (f != null) {
    writer.writeFloat(
      18,
      f
    );
  }
  f = /** @type {!proto.location.unified.LocationAttributesProto.CarrierIpType} */ (jspb.Message.getField(message, 19));
  if (f != null) {
    writer.writeEnum(
      19,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 20));
  if (f != null) {
    writer.writeFloat(
      20,
      f
    );
  }
  f = /** @type {!proto.location.unified.LocationAttributesProto.ViewportSearchOptions} */ (jspb.Message.getField(message, 21));
  if (f != null) {
    writer.writeEnum(
      21,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 10));
  if (f != null) {
    writer.writeFloat(
      10,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 22));
  if (f != null) {
    writer.writeInt32(
      22,
      f
    );
  }
  f = message.getPersonalizedLocationAttributes();
  if (f != null) {
    writer.writeMessage(
      23,
      f,
      proto.location.unified.PersonalizedLocationAttributes.serializeBinaryToWriter
    );
  }
};


/**
 * @enum {number}
 */
proto.location.unified.LocationAttributesProto.PLMType = {
  PLMTYPE_UNKNOWN: 0,
  PLMTYPE_FIRST: 1,
  PLMTYPE_SECOND: 2,
  PLMTYPE_THIRD: 3,
  PLMTYPE_FOURTH: 4,
  PLMTYPE_FIFTH: 5,
  PLMTYPE_SIXTH: 6,
  PLMTYPE_SEVENTH: 7,
  PLMTYPE_EIGHTH: 8,
  PLMTYPE_NINTH: 9
};

/**
 * @enum {number}
 */
proto.location.unified.LocationAttributesProto.Inference = {
  INFERENCE_NONE: 0,
  INFERENCE_HULK_HEURISTIC: 1,
  INFERENCE_HULK_CLASSIFIED: 2,
  INFERENCE_HULK_UNKNOWN: 3,
  INFERENCE_PLM: 4
};

/**
 * @enum {number}
 */
proto.location.unified.LocationAttributesProto.ManualEntry = {
  MANUAL_ENTRY_NONE: 0,
  MANUAL_ENTRY_UNKNOWN: 1,
  MANUAL_ENTRY: 2,
  MANUAL_ENTRY_DROPPED_PIN: 3,
  MANUAL_ENTRY_HIGH_LEVEL: 4,
  MANUAL_ENTRY_STALE: 5,
  MANUAL_ENTRY_INSUFFICIENT_LOCATIONS: 6
};

/**
 * @enum {number}
 */
proto.location.unified.LocationAttributesProto.CarrierIpType = {
  CARRIER_IP_UNKNOWN: 0,
  CARRIER_IP_NOT_CARRIER: 1,
  CARRIER_IP_MOBILE: 2
};

/**
 * @enum {number}
 */
proto.location.unified.LocationAttributesProto.ViewportSearchOptions = {
  VIEWPORT_SEARCH_OPTIONS_UNSPECIFIED: 0,
  VIEWPORT_SEARCH_OPTIONS_AROUND: 1,
  VIEWPORT_SEARCH_OPTIONS_RESTRICT: 2,
  VIEWPORT_SEARCH_OPTIONS_FILTER_ONLY: 3,
  VIEWPORT_SEARCH_OPTIONS_FORCE_AROUND: 4
};

/**
 * optional int32 heading_degrees = 2;
 * @return {?number}
 */
proto.location.unified.LocationAttributesProto.prototype.getHeadingDegrees = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.location.unified.LocationAttributesProto} returns this
 */
proto.location.unified.LocationAttributesProto.prototype.setHeadingDegrees = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.LocationAttributesProto} returns this
 */
proto.location.unified.LocationAttributesProto.prototype.clearHeadingDegrees = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.LocationAttributesProto.prototype.hasHeadingDegrees = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional int32 bearing_degrees = 3;
 * @return {?number}
 */
proto.location.unified.LocationAttributesProto.prototype.getBearingDegrees = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.location.unified.LocationAttributesProto} returns this
 */
proto.location.unified.LocationAttributesProto.prototype.setBearingDegrees = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.LocationAttributesProto} returns this
 */
proto.location.unified.LocationAttributesProto.prototype.clearBearingDegrees = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.LocationAttributesProto.prototype.hasBearingDegrees = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional int32 bearing_accuracy_degrees = 12;
 * @return {?number}
 */
proto.location.unified.LocationAttributesProto.prototype.getBearingAccuracyDegrees = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 12));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.location.unified.LocationAttributesProto} returns this
 */
proto.location.unified.LocationAttributesProto.prototype.setBearingAccuracyDegrees = function(value) {
  return jspb.Message.setField(this, 12, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.LocationAttributesProto} returns this
 */
proto.location.unified.LocationAttributesProto.prototype.clearBearingAccuracyDegrees = function() {
  return jspb.Message.setField(this, 12, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.LocationAttributesProto.prototype.hasBearingAccuracyDegrees = function() {
  return jspb.Message.getField(this, 12) != null;
};


/**
 * optional int32 speed_kph = 4;
 * @return {?number}
 */
proto.location.unified.LocationAttributesProto.prototype.getSpeedKph = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 4));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.location.unified.LocationAttributesProto} returns this
 */
proto.location.unified.LocationAttributesProto.prototype.setSpeedKph = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.LocationAttributesProto} returns this
 */
proto.location.unified.LocationAttributesProto.prototype.clearSpeedKph = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.LocationAttributesProto.prototype.hasSpeedKph = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional int32 speed_accuracy_kph = 13;
 * @return {?number}
 */
proto.location.unified.LocationAttributesProto.prototype.getSpeedAccuracyKph = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 13));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.location.unified.LocationAttributesProto} returns this
 */
proto.location.unified.LocationAttributesProto.prototype.setSpeedAccuracyKph = function(value) {
  return jspb.Message.setField(this, 13, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.LocationAttributesProto} returns this
 */
proto.location.unified.LocationAttributesProto.prototype.clearSpeedAccuracyKph = function() {
  return jspb.Message.setField(this, 13, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.LocationAttributesProto.prototype.hasSpeedAccuracyKph = function() {
  return jspb.Message.getField(this, 13) != null;
};


/**
 * optional int32 tilt_degrees = 5;
 * @return {?number}
 */
proto.location.unified.LocationAttributesProto.prototype.getTiltDegrees = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 5));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.location.unified.LocationAttributesProto} returns this
 */
proto.location.unified.LocationAttributesProto.prototype.setTiltDegrees = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.LocationAttributesProto} returns this
 */
proto.location.unified.LocationAttributesProto.prototype.clearTiltDegrees = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.LocationAttributesProto.prototype.hasTiltDegrees = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional int32 roll_degrees = 6;
 * @return {?number}
 */
proto.location.unified.LocationAttributesProto.prototype.getRollDegrees = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 6));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.location.unified.LocationAttributesProto} returns this
 */
proto.location.unified.LocationAttributesProto.prototype.setRollDegrees = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.LocationAttributesProto} returns this
 */
proto.location.unified.LocationAttributesProto.prototype.clearRollDegrees = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.LocationAttributesProto.prototype.hasRollDegrees = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional double altitude_meters_from_ground = 7;
 * @return {?number}
 */
proto.location.unified.LocationAttributesProto.prototype.getAltitudeMetersFromGround = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 7));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.location.unified.LocationAttributesProto} returns this
 */
proto.location.unified.LocationAttributesProto.prototype.setAltitudeMetersFromGround = function(value) {
  return jspb.Message.setField(this, 7, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.LocationAttributesProto} returns this
 */
proto.location.unified.LocationAttributesProto.prototype.clearAltitudeMetersFromGround = function() {
  return jspb.Message.setField(this, 7, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.LocationAttributesProto.prototype.hasAltitudeMetersFromGround = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * optional FieldOfView field_of_view = 8;
 * @return {?proto.location.unified.FieldOfView}
 */
proto.location.unified.LocationAttributesProto.prototype.getFieldOfView = function() {
  return /** @type{?proto.location.unified.FieldOfView} */ (
    jspb.Message.getWrapperField(this, proto.location.unified.FieldOfView, 8));
};


/**
 * @param {?proto.location.unified.FieldOfView|undefined} value
 * @return {!proto.location.unified.LocationAttributesProto} returns this
*/
proto.location.unified.LocationAttributesProto.prototype.setFieldOfView = function(value) {
  return jspb.Message.setWrapperField(this, 8, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.location.unified.LocationAttributesProto} returns this
 */
proto.location.unified.LocationAttributesProto.prototype.clearFieldOfView = function() {
  return this.setFieldOfView(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.LocationAttributesProto.prototype.hasFieldOfView = function() {
  return jspb.Message.getField(this, 8) != null;
};


/**
 * optional string boarded_transit_vehicle_token = 9;
 * @return {?string}
 */
proto.location.unified.LocationAttributesProto.prototype.getBoardedTransitVehicleToken = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 9));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.location.unified.LocationAttributesProto} returns this
 */
proto.location.unified.LocationAttributesProto.prototype.setBoardedTransitVehicleToken = function(value) {
  return jspb.Message.setField(this, 9, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.LocationAttributesProto} returns this
 */
proto.location.unified.LocationAttributesProto.prototype.clearBoardedTransitVehicleToken = function() {
  return jspb.Message.setField(this, 9, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.LocationAttributesProto.prototype.hasBoardedTransitVehicleToken = function() {
  return jspb.Message.getField(this, 9) != null;
};


/**
 * repeated ActivityRecord activity_record = 11;
 * @return {!Array<!proto.location.unified.ActivityRecord>}
 */
proto.location.unified.LocationAttributesProto.prototype.getActivityRecordList = function() {
  return /** @type{!Array<!proto.location.unified.ActivityRecord>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.location.unified.ActivityRecord, 11));
};


/**
 * @param {Array<!proto.location.unified.ActivityRecord>|undefined} value
 * @return {!proto.location.unified.LocationAttributesProto} returns this
*/
proto.location.unified.LocationAttributesProto.prototype.setActivityRecordList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 11, value);
};


/**
 * @param {!proto.location.unified.ActivityRecord=} opt_value
 * @param {number=} opt_index
 * @return {!proto.location.unified.ActivityRecord}
 */
proto.location.unified.LocationAttributesProto.prototype.addActivityRecord = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 11, opt_value, proto.location.unified.ActivityRecord, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.location.unified.LocationAttributesProto} returns this
 */
proto.location.unified.LocationAttributesProto.prototype.clearActivityRecordList = function() {
  return this.setActivityRecordList([]);
};


/**
 * optional PLMType plm_type = 14;
 * @return {!proto.location.unified.LocationAttributesProto.PLMType}
 */
proto.location.unified.LocationAttributesProto.prototype.getPlmType = function() {
  return /** @type {!proto.location.unified.LocationAttributesProto.PLMType} */ (jspb.Message.getField(this, 14));
};


/**
 * @param {?proto.location.unified.LocationAttributesProto.PLMType|undefined} value
 * @return {!proto.location.unified.LocationAttributesProto} returns this
 */
proto.location.unified.LocationAttributesProto.prototype.setPlmType = function(value) {
  return jspb.Message.setField(this, 14, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.LocationAttributesProto} returns this
 */
proto.location.unified.LocationAttributesProto.prototype.clearPlmType = function() {
  return jspb.Message.setField(this, 14, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.LocationAttributesProto.prototype.hasPlmType = function() {
  return jspb.Message.getField(this, 14) != null;
};


/**
 * optional Inference inference = 15;
 * @return {!proto.location.unified.LocationAttributesProto.Inference}
 */
proto.location.unified.LocationAttributesProto.prototype.getInference = function() {
  return /** @type {!proto.location.unified.LocationAttributesProto.Inference} */ (jspb.Message.getField(this, 15));
};


/**
 * @param {?proto.location.unified.LocationAttributesProto.Inference|undefined} value
 * @return {!proto.location.unified.LocationAttributesProto} returns this
 */
proto.location.unified.LocationAttributesProto.prototype.setInference = function(value) {
  return jspb.Message.setField(this, 15, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.LocationAttributesProto} returns this
 */
proto.location.unified.LocationAttributesProto.prototype.clearInference = function() {
  return jspb.Message.setField(this, 15, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.LocationAttributesProto.prototype.hasInference = function() {
  return jspb.Message.getField(this, 15) != null;
};


/**
 * optional ManualEntry manual_entry = 16;
 * @return {!proto.location.unified.LocationAttributesProto.ManualEntry}
 */
proto.location.unified.LocationAttributesProto.prototype.getManualEntry = function() {
  return /** @type {!proto.location.unified.LocationAttributesProto.ManualEntry} */ (jspb.Message.getField(this, 16));
};


/**
 * @param {?proto.location.unified.LocationAttributesProto.ManualEntry|undefined} value
 * @return {!proto.location.unified.LocationAttributesProto} returns this
 */
proto.location.unified.LocationAttributesProto.prototype.setManualEntry = function(value) {
  return jspb.Message.setField(this, 16, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.LocationAttributesProto} returns this
 */
proto.location.unified.LocationAttributesProto.prototype.clearManualEntry = function() {
  return jspb.Message.setField(this, 16, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.LocationAttributesProto.prototype.hasManualEntry = function() {
  return jspb.Message.getField(this, 16) != null;
};


/**
 * optional float week_second_confidence = 17;
 * @return {?number}
 */
proto.location.unified.LocationAttributesProto.prototype.getWeekSecondConfidence = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 17));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.location.unified.LocationAttributesProto} returns this
 */
proto.location.unified.LocationAttributesProto.prototype.setWeekSecondConfidence = function(value) {
  return jspb.Message.setField(this, 17, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.LocationAttributesProto} returns this
 */
proto.location.unified.LocationAttributesProto.prototype.clearWeekSecondConfidence = function() {
  return jspb.Message.setField(this, 17, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.LocationAttributesProto.prototype.hasWeekSecondConfidence = function() {
  return jspb.Message.getField(this, 17) != null;
};


/**
 * optional float ip_range_confidence = 18;
 * @return {?number}
 */
proto.location.unified.LocationAttributesProto.prototype.getIpRangeConfidence = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 18));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.location.unified.LocationAttributesProto} returns this
 */
proto.location.unified.LocationAttributesProto.prototype.setIpRangeConfidence = function(value) {
  return jspb.Message.setField(this, 18, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.LocationAttributesProto} returns this
 */
proto.location.unified.LocationAttributesProto.prototype.clearIpRangeConfidence = function() {
  return jspb.Message.setField(this, 18, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.LocationAttributesProto.prototype.hasIpRangeConfidence = function() {
  return jspb.Message.getField(this, 18) != null;
};


/**
 * optional CarrierIpType carrier_ip_type = 19;
 * @return {!proto.location.unified.LocationAttributesProto.CarrierIpType}
 */
proto.location.unified.LocationAttributesProto.prototype.getCarrierIpType = function() {
  return /** @type {!proto.location.unified.LocationAttributesProto.CarrierIpType} */ (jspb.Message.getField(this, 19));
};


/**
 * @param {?proto.location.unified.LocationAttributesProto.CarrierIpType|undefined} value
 * @return {!proto.location.unified.LocationAttributesProto} returns this
 */
proto.location.unified.LocationAttributesProto.prototype.setCarrierIpType = function(value) {
  return jspb.Message.setField(this, 19, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.LocationAttributesProto} returns this
 */
proto.location.unified.LocationAttributesProto.prototype.clearCarrierIpType = function() {
  return jspb.Message.setField(this, 19, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.LocationAttributesProto.prototype.hasCarrierIpType = function() {
  return jspb.Message.getField(this, 19) != null;
};


/**
 * optional float ads_confidence = 20;
 * @return {?number}
 */
proto.location.unified.LocationAttributesProto.prototype.getAdsConfidence = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 20));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.location.unified.LocationAttributesProto} returns this
 */
proto.location.unified.LocationAttributesProto.prototype.setAdsConfidence = function(value) {
  return jspb.Message.setField(this, 20, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.LocationAttributesProto} returns this
 */
proto.location.unified.LocationAttributesProto.prototype.clearAdsConfidence = function() {
  return jspb.Message.setField(this, 20, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.LocationAttributesProto.prototype.hasAdsConfidence = function() {
  return jspb.Message.getField(this, 20) != null;
};


/**
 * optional ViewportSearchOptions viewport_search_options = 21;
 * @return {!proto.location.unified.LocationAttributesProto.ViewportSearchOptions}
 */
proto.location.unified.LocationAttributesProto.prototype.getViewportSearchOptions = function() {
  return /** @type {!proto.location.unified.LocationAttributesProto.ViewportSearchOptions} */ (jspb.Message.getField(this, 21));
};


/**
 * @param {?proto.location.unified.LocationAttributesProto.ViewportSearchOptions|undefined} value
 * @return {!proto.location.unified.LocationAttributesProto} returns this
 */
proto.location.unified.LocationAttributesProto.prototype.setViewportSearchOptions = function(value) {
  return jspb.Message.setField(this, 21, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.LocationAttributesProto} returns this
 */
proto.location.unified.LocationAttributesProto.prototype.clearViewportSearchOptions = function() {
  return jspb.Message.setField(this, 21, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.LocationAttributesProto.prototype.hasViewportSearchOptions = function() {
  return jspb.Message.getField(this, 21) != null;
};


/**
 * optional float device_location_ratio = 10;
 * @return {?number}
 */
proto.location.unified.LocationAttributesProto.prototype.getDeviceLocationRatio = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 10));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.location.unified.LocationAttributesProto} returns this
 */
proto.location.unified.LocationAttributesProto.prototype.setDeviceLocationRatio = function(value) {
  return jspb.Message.setField(this, 10, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.LocationAttributesProto} returns this
 */
proto.location.unified.LocationAttributesProto.prototype.clearDeviceLocationRatio = function() {
  return jspb.Message.setField(this, 10, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.LocationAttributesProto.prototype.hasDeviceLocationRatio = function() {
  return jspb.Message.getField(this, 10) != null;
};


/**
 * optional int32 plm_source_location_count = 22;
 * @return {?number}
 */
proto.location.unified.LocationAttributesProto.prototype.getPlmSourceLocationCount = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 22));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.location.unified.LocationAttributesProto} returns this
 */
proto.location.unified.LocationAttributesProto.prototype.setPlmSourceLocationCount = function(value) {
  return jspb.Message.setField(this, 22, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.LocationAttributesProto} returns this
 */
proto.location.unified.LocationAttributesProto.prototype.clearPlmSourceLocationCount = function() {
  return jspb.Message.setField(this, 22, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.LocationAttributesProto.prototype.hasPlmSourceLocationCount = function() {
  return jspb.Message.getField(this, 22) != null;
};


/**
 * optional PersonalizedLocationAttributes personalized_location_attributes = 23;
 * @return {?proto.location.unified.PersonalizedLocationAttributes}
 */
proto.location.unified.LocationAttributesProto.prototype.getPersonalizedLocationAttributes = function() {
  return /** @type{?proto.location.unified.PersonalizedLocationAttributes} */ (
    jspb.Message.getWrapperField(this, proto.location.unified.PersonalizedLocationAttributes, 23));
};


/**
 * @param {?proto.location.unified.PersonalizedLocationAttributes|undefined} value
 * @return {!proto.location.unified.LocationAttributesProto} returns this
*/
proto.location.unified.LocationAttributesProto.prototype.setPersonalizedLocationAttributes = function(value) {
  return jspb.Message.setWrapperField(this, 23, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.location.unified.LocationAttributesProto} returns this
 */
proto.location.unified.LocationAttributesProto.prototype.clearPersonalizedLocationAttributes = function() {
  return this.setPersonalizedLocationAttributes(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.LocationAttributesProto.prototype.hasPersonalizedLocationAttributes = function() {
  return jspb.Message.getField(this, 23) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.location.unified.LocationAttributesProto}
 */
proto.location.unified.LocationAttributesProto.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.location.unified.LocationAttributesProto, data);
};



//location/unified/proto/location_descriptor_proto_jspb/presenceinterval.jspb.js
// source: location/unified/proto/location_descriptor.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.location.unified.PresenceInterval');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.location.unified.PresenceInterval = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.location.unified.PresenceInterval, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.location.unified.PresenceInterval.displayName = 'proto.location.unified.PresenceInterval';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.location.unified.PresenceInterval.prototype.toObject = function(opt_includeInstance) {
  return proto.location.unified.PresenceInterval.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.location.unified.PresenceInterval} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.location.unified.PresenceInterval.toObject = function(includeInstance, msg) {
  var f, obj = {
    startOffsetSec: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    durationSec: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    confidence: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of PresenceInterval as accepted by the `fromObject` method.
 * @record
 */
proto.location.unified.PresenceInterval.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.startOffsetSec;

  /** @type {?number|undefined} */
  this.durationSec;

  /** @type {?number|undefined} */
  this.confidence;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.location.unified.PresenceInterval.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.location.unified.PresenceInterval}
 */
proto.location.unified.PresenceInterval.fromObject = function(obj) {
  var msg = new proto.location.unified.PresenceInterval();
  obj.startOffsetSec != null && jspb.Message.setField(msg, 1, obj.startOffsetSec);
  obj.durationSec != null && jspb.Message.setField(msg, 2, obj.durationSec);
  obj.confidence != null && jspb.Message.setField(msg, 3, obj.confidence);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.location.unified.PresenceInterval}
 */
proto.location.unified.PresenceInterval.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.location.unified.PresenceInterval;
  return proto.location.unified.PresenceInterval.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.location.unified.PresenceInterval} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.location.unified.PresenceInterval}
 */
proto.location.unified.PresenceInterval.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {number} */ (reader.readUint64());
      msg.setStartOffsetSec(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readUint64());
      msg.setDurationSec(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readUint32());
      msg.setConfidence(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.location.unified.PresenceInterval.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.location.unified.PresenceInterval.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.location.unified.PresenceInterval} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.location.unified.PresenceInterval.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?number} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeUint64(
      1,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeUint64(
      2,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeUint32(
      3,
      f
    );
  }
};


/**
 * optional uint64 start_offset_sec = 1;
 * @return {?number}
 */
proto.location.unified.PresenceInterval.prototype.getStartOffsetSec = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.location.unified.PresenceInterval} returns this
 */
proto.location.unified.PresenceInterval.prototype.setStartOffsetSec = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.PresenceInterval} returns this
 */
proto.location.unified.PresenceInterval.prototype.clearStartOffsetSec = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.PresenceInterval.prototype.hasStartOffsetSec = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional uint64 duration_sec = 2;
 * @return {?number}
 */
proto.location.unified.PresenceInterval.prototype.getDurationSec = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.location.unified.PresenceInterval} returns this
 */
proto.location.unified.PresenceInterval.prototype.setDurationSec = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.PresenceInterval} returns this
 */
proto.location.unified.PresenceInterval.prototype.clearDurationSec = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.PresenceInterval.prototype.hasDurationSec = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional uint32 confidence = 3;
 * @return {?number}
 */
proto.location.unified.PresenceInterval.prototype.getConfidence = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.location.unified.PresenceInterval} returns this
 */
proto.location.unified.PresenceInterval.prototype.setConfidence = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.PresenceInterval} returns this
 */
proto.location.unified.PresenceInterval.prototype.clearConfidence = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.PresenceInterval.prototype.hasConfidence = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.location.unified.PresenceInterval}
 */
proto.location.unified.PresenceInterval.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.location.unified.PresenceInterval, data);
};



//location/unified/proto/location_descriptor_proto_jspb/semanticplace.jspb.js
// source: location/unified/proto/location_descriptor.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.location.unified.SemanticPlace');
goog.provide('proto.location.unified.SemanticPlace.GConceptInstanceProto');
goog.provide('proto.location.unified.SemanticPlace.GConceptInstanceProto.Prominence');
goog.provide('proto.location.unified.SemanticPlace.SemanticPlaceConfidence');
goog.provide('proto.location.unified.SemanticPlace.Source');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.location.unified.FeatureIdProto');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.location.unified.SemanticPlace = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.location.unified.SemanticPlace.repeatedFields_, null);
};
goog.inherits(proto.location.unified.SemanticPlace, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.location.unified.SemanticPlace.displayName = 'proto.location.unified.SemanticPlace';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.location.unified.SemanticPlace.GConceptInstanceProto = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.location.unified.SemanticPlace.GConceptInstanceProto, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.location.unified.SemanticPlace.GConceptInstanceProto.displayName = 'proto.location.unified.SemanticPlace.GConceptInstanceProto';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.location.unified.SemanticPlace.repeatedFields_ = [2];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.location.unified.SemanticPlace.prototype.toObject = function(opt_includeInstance) {
  return proto.location.unified.SemanticPlace.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.location.unified.SemanticPlace} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.location.unified.SemanticPlace.toObject = function(includeInstance, msg) {
  var f, obj = {
    featureId: (f = msg.getFeatureId()) && proto.location.unified.FeatureIdProto.toObject(includeInstance, f),
    gconceptInstanceList: jspb.Message.toObjectList(msg.getGconceptInstanceList(),
    proto.location.unified.SemanticPlace.GConceptInstanceProto.toObject, includeInstance),
    score: (f = jspb.Message.getOptionalFloatingPointField(msg, 3)) == null ? undefined : f,
    confidence: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f,
    source: (f = jspb.Message.getField(msg, 5)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of SemanticPlace as accepted by the `fromObject` method.
 * @record
 */
proto.location.unified.SemanticPlace.ObjectFormat = function() {
  /** @type {?proto.location.unified.FeatureIdProto.ObjectFormat|undefined} */
  this.featureId;

  /** @type {?Array<!proto.location.unified.SemanticPlace.GConceptInstanceProto.ObjectFormat>|undefined} */
  this.gconceptInstanceList;

  /** @type {?number|undefined} */
  this.score;

  /** @type {?number|undefined} */
  this.confidence;

  /** @type {?number|undefined} */
  this.source;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.location.unified.SemanticPlace.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.location.unified.SemanticPlace}
 */
proto.location.unified.SemanticPlace.fromObject = function(obj) {
  var msg = new proto.location.unified.SemanticPlace();
  obj.featureId && jspb.Message.setWrapperField(
      msg, 1, proto.location.unified.FeatureIdProto.fromObject(obj.featureId));
  obj.gconceptInstanceList && jspb.Message.setRepeatedWrapperField(
      msg, 2, obj.gconceptInstanceList.map(
          proto.location.unified.SemanticPlace.GConceptInstanceProto.fromObject));
  obj.score != null && jspb.Message.setField(msg, 3, obj.score);
  obj.confidence != null && jspb.Message.setField(msg, 4, obj.confidence);
  obj.source != null && jspb.Message.setField(msg, 5, obj.source);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.location.unified.SemanticPlace}
 */
proto.location.unified.SemanticPlace.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.location.unified.SemanticPlace;
  return proto.location.unified.SemanticPlace.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.location.unified.SemanticPlace} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.location.unified.SemanticPlace}
 */
proto.location.unified.SemanticPlace.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.location.unified.FeatureIdProto;
      reader.readMessage(value,proto.location.unified.FeatureIdProto.deserializeBinaryFromReader);
      msg.setFeatureId(value);
      break;
    case 2:
      var value = new proto.location.unified.SemanticPlace.GConceptInstanceProto;
      reader.readMessage(value,proto.location.unified.SemanticPlace.GConceptInstanceProto.deserializeBinaryFromReader);
      msg.addGconceptInstance(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setScore(value);
      break;
    case 4:
      var value = /** @type {!proto.location.unified.SemanticPlace.SemanticPlaceConfidence} */ (reader.readEnum());
      msg.setConfidence(value);
      break;
    case 5:
      var value = /** @type {!proto.location.unified.SemanticPlace.Source} */ (reader.readEnum());
      msg.setSource(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.location.unified.SemanticPlace.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.location.unified.SemanticPlace.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.location.unified.SemanticPlace} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.location.unified.SemanticPlace.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getFeatureId();
  if (f != null) {
    writer.writeMessage(
      1,
      f,
      proto.location.unified.FeatureIdProto.serializeBinaryToWriter
    );
  }
  f = message.getGconceptInstanceList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      2,
      f,
      proto.location.unified.SemanticPlace.GConceptInstanceProto.serializeBinaryToWriter
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeFloat(
      3,
      f
    );
  }
  f = /** @type {!proto.location.unified.SemanticPlace.SemanticPlaceConfidence} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeEnum(
      4,
      f
    );
  }
  f = /** @type {!proto.location.unified.SemanticPlace.Source} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeEnum(
      5,
      f
    );
  }
};


/**
 * @enum {number}
 */
proto.location.unified.SemanticPlace.SemanticPlaceConfidence = {
  UNKNOWN_CONFIDENCE: 0,
  LOW_CONFIDENCE: 1,
  MEDIUM_CONFIDENCE: 2,
  HIGH_CONFIDENCE: 3
};

/**
 * @enum {number}
 */
proto.location.unified.SemanticPlace.Source = {
  UNKNOWN: 0,
  HAPPY_HOUR: 1,
  HULK_REAL_TIME: 2,
  ANDROID_CONTEXT: 3
};




if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.location.unified.SemanticPlace.GConceptInstanceProto.prototype.toObject = function(opt_includeInstance) {
  return proto.location.unified.SemanticPlace.GConceptInstanceProto.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.location.unified.SemanticPlace.GConceptInstanceProto} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.location.unified.SemanticPlace.GConceptInstanceProto.toObject = function(includeInstance, msg) {
  var f, obj = {
    gconceptId: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    prominence: jspb.Message.getFieldWithDefault(msg, 2, 0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of GConceptInstanceProto as accepted by the `fromObject` method.
 * @record
 */
proto.location.unified.SemanticPlace.GConceptInstanceProto.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.gconceptId;

  /** @type {?number|undefined} */
  this.prominence;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.location.unified.SemanticPlace.GConceptInstanceProto.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.location.unified.SemanticPlace.GConceptInstanceProto}
 */
proto.location.unified.SemanticPlace.GConceptInstanceProto.fromObject = function(obj) {
  var msg = new proto.location.unified.SemanticPlace.GConceptInstanceProto();
  obj.gconceptId != null && jspb.Message.setField(msg, 1, obj.gconceptId);
  obj.prominence != null && jspb.Message.setField(msg, 2, obj.prominence);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.location.unified.SemanticPlace.GConceptInstanceProto}
 */
proto.location.unified.SemanticPlace.GConceptInstanceProto.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.location.unified.SemanticPlace.GConceptInstanceProto;
  return proto.location.unified.SemanticPlace.GConceptInstanceProto.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.location.unified.SemanticPlace.GConceptInstanceProto} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.location.unified.SemanticPlace.GConceptInstanceProto}
 */
proto.location.unified.SemanticPlace.GConceptInstanceProto.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setGconceptId(value);
      break;
    case 2:
      var value = /** @type {!proto.location.unified.SemanticPlace.GConceptInstanceProto.Prominence} */ (reader.readEnum());
      msg.setProminence(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.location.unified.SemanticPlace.GConceptInstanceProto.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.location.unified.SemanticPlace.GConceptInstanceProto.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.location.unified.SemanticPlace.GConceptInstanceProto} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.location.unified.SemanticPlace.GConceptInstanceProto.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = /** @type {!proto.location.unified.SemanticPlace.GConceptInstanceProto.Prominence} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeEnum(
      2,
      f
    );
  }
};


/**
 * @enum {number}
 */
proto.location.unified.SemanticPlace.GConceptInstanceProto.Prominence = {
  NON_PRIMARY: 0,
  PRIMARY: 1000
};

/**
 * optional string gconcept_id = 1;
 * @return {?string}
 */
proto.location.unified.SemanticPlace.GConceptInstanceProto.prototype.getGconceptId = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.location.unified.SemanticPlace.GConceptInstanceProto} returns this
 */
proto.location.unified.SemanticPlace.GConceptInstanceProto.prototype.setGconceptId = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.SemanticPlace.GConceptInstanceProto} returns this
 */
proto.location.unified.SemanticPlace.GConceptInstanceProto.prototype.clearGconceptId = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.SemanticPlace.GConceptInstanceProto.prototype.hasGconceptId = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional Prominence prominence = 2;
 * @return {!proto.location.unified.SemanticPlace.GConceptInstanceProto.Prominence}
 */
proto.location.unified.SemanticPlace.GConceptInstanceProto.prototype.getProminence = function() {
  return /** @type {!proto.location.unified.SemanticPlace.GConceptInstanceProto.Prominence} */ (jspb.Message.getFieldWithDefault(this, 2, 0));
};


/**
 * @param {?proto.location.unified.SemanticPlace.GConceptInstanceProto.Prominence|undefined} value
 * @return {!proto.location.unified.SemanticPlace.GConceptInstanceProto} returns this
 */
proto.location.unified.SemanticPlace.GConceptInstanceProto.prototype.setProminence = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.SemanticPlace.GConceptInstanceProto} returns this
 */
proto.location.unified.SemanticPlace.GConceptInstanceProto.prototype.clearProminence = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.SemanticPlace.GConceptInstanceProto.prototype.hasProminence = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.location.unified.SemanticPlace.GConceptInstanceProto}
 */
proto.location.unified.SemanticPlace.GConceptInstanceProto.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.location.unified.SemanticPlace.GConceptInstanceProto, data);
};


/**
 * optional FeatureIdProto feature_id = 1;
 * @return {?proto.location.unified.FeatureIdProto}
 */
proto.location.unified.SemanticPlace.prototype.getFeatureId = function() {
  return /** @type{?proto.location.unified.FeatureIdProto} */ (
    jspb.Message.getWrapperField(this, proto.location.unified.FeatureIdProto, 1));
};


/**
 * @param {?proto.location.unified.FeatureIdProto|undefined} value
 * @return {!proto.location.unified.SemanticPlace} returns this
*/
proto.location.unified.SemanticPlace.prototype.setFeatureId = function(value) {
  return jspb.Message.setWrapperField(this, 1, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.location.unified.SemanticPlace} returns this
 */
proto.location.unified.SemanticPlace.prototype.clearFeatureId = function() {
  return this.setFeatureId(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.SemanticPlace.prototype.hasFeatureId = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * repeated GConceptInstanceProto gconcept_instance = 2;
 * @return {!Array<!proto.location.unified.SemanticPlace.GConceptInstanceProto>}
 */
proto.location.unified.SemanticPlace.prototype.getGconceptInstanceList = function() {
  return /** @type{!Array<!proto.location.unified.SemanticPlace.GConceptInstanceProto>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.location.unified.SemanticPlace.GConceptInstanceProto, 2));
};


/**
 * @param {Array<!proto.location.unified.SemanticPlace.GConceptInstanceProto>|undefined} value
 * @return {!proto.location.unified.SemanticPlace} returns this
*/
proto.location.unified.SemanticPlace.prototype.setGconceptInstanceList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 2, value);
};


/**
 * @param {!proto.location.unified.SemanticPlace.GConceptInstanceProto=} opt_value
 * @param {number=} opt_index
 * @return {!proto.location.unified.SemanticPlace.GConceptInstanceProto}
 */
proto.location.unified.SemanticPlace.prototype.addGconceptInstance = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 2, opt_value, proto.location.unified.SemanticPlace.GConceptInstanceProto, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.location.unified.SemanticPlace} returns this
 */
proto.location.unified.SemanticPlace.prototype.clearGconceptInstanceList = function() {
  return this.setGconceptInstanceList([]);
};


/**
 * optional float score = 3;
 * @return {?number}
 */
proto.location.unified.SemanticPlace.prototype.getScore = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 3));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.location.unified.SemanticPlace} returns this
 */
proto.location.unified.SemanticPlace.prototype.setScore = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.SemanticPlace} returns this
 */
proto.location.unified.SemanticPlace.prototype.clearScore = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.SemanticPlace.prototype.hasScore = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional SemanticPlaceConfidence confidence = 4;
 * @return {!proto.location.unified.SemanticPlace.SemanticPlaceConfidence}
 */
proto.location.unified.SemanticPlace.prototype.getConfidence = function() {
  return /** @type {!proto.location.unified.SemanticPlace.SemanticPlaceConfidence} */ (jspb.Message.getField(this, 4));
};


/**
 * @param {?proto.location.unified.SemanticPlace.SemanticPlaceConfidence|undefined} value
 * @return {!proto.location.unified.SemanticPlace} returns this
 */
proto.location.unified.SemanticPlace.prototype.setConfidence = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.SemanticPlace} returns this
 */
proto.location.unified.SemanticPlace.prototype.clearConfidence = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.SemanticPlace.prototype.hasConfidence = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional Source source = 5;
 * @return {!proto.location.unified.SemanticPlace.Source}
 */
proto.location.unified.SemanticPlace.prototype.getSource = function() {
  return /** @type {!proto.location.unified.SemanticPlace.Source} */ (jspb.Message.getField(this, 5));
};


/**
 * @param {?proto.location.unified.SemanticPlace.Source|undefined} value
 * @return {!proto.location.unified.SemanticPlace} returns this
 */
proto.location.unified.SemanticPlace.prototype.setSource = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.SemanticPlace} returns this
 */
proto.location.unified.SemanticPlace.prototype.clearSource = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.SemanticPlace.prototype.hasSource = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.location.unified.SemanticPlace}
 */
proto.location.unified.SemanticPlace.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.location.unified.SemanticPlace, data);
};



//location/unified/proto/location_descriptor_proto_jspb/visiblenetwork.jspb.js
// source: location/unified/proto/location_descriptor.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.location.unified.VisibleNetwork');
goog.provide('proto.location.unified.VisibleNetwork.Cell');
goog.provide('proto.location.unified.VisibleNetwork.Cell.Type');
goog.provide('proto.location.unified.VisibleNetwork.TypeCase');
goog.provide('proto.location.unified.VisibleNetwork.WiFi');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.location.unified.VisibleNetwork = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, proto.location.unified.VisibleNetwork.oneofGroups_);
};
goog.inherits(proto.location.unified.VisibleNetwork, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.location.unified.VisibleNetwork.displayName = 'proto.location.unified.VisibleNetwork';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.location.unified.VisibleNetwork.WiFi = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.location.unified.VisibleNetwork.WiFi, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.location.unified.VisibleNetwork.WiFi.displayName = 'proto.location.unified.VisibleNetwork.WiFi';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.location.unified.VisibleNetwork.Cell = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.location.unified.VisibleNetwork.Cell, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.location.unified.VisibleNetwork.Cell.displayName = 'proto.location.unified.VisibleNetwork.Cell';
}

/**
 * Oneof group definitions for this message. Each group defines the field
 * numbers belonging to that group. When of these fields' value is set, all
 * other fields in the group are cleared. During deserialization, if multiple
 * fields are encountered for a group, only the last value seen will be kept.
 * @private {!Array<!Array<number>>}
 * @const
 */
proto.location.unified.VisibleNetwork.oneofGroups_ = [[1,2]];

/**
 * @enum {number}
 */
proto.location.unified.VisibleNetwork.TypeCase = {
  TYPE_NOT_SET: 0,
  WIFI: 1,
  CELL: 2
};

/**
 * @return {!proto.location.unified.VisibleNetwork.TypeCase}
 */
proto.location.unified.VisibleNetwork.prototype.getTypeCase = function() {
  return /** @type {!proto.location.unified.VisibleNetwork.TypeCase} */(jspb.Message.computeOneofCase(this, proto.location.unified.VisibleNetwork.oneofGroups_[0]));
};



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.location.unified.VisibleNetwork.prototype.toObject = function(opt_includeInstance) {
  return proto.location.unified.VisibleNetwork.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.location.unified.VisibleNetwork} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.location.unified.VisibleNetwork.toObject = function(includeInstance, msg) {
  var f, obj = {
    wifi: (f = msg.getWifi()) && proto.location.unified.VisibleNetwork.WiFi.toObject(includeInstance, f),
    cell: (f = msg.getCell()) && proto.location.unified.VisibleNetwork.Cell.toObject(includeInstance, f),
    connected: (f = jspb.Message.getBooleanField(msg, 3)) == null ? undefined : f,
    timestampMs: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of VisibleNetwork as accepted by the `fromObject` method.
 * @record
 */
proto.location.unified.VisibleNetwork.ObjectFormat = function() {
  /** @type {?proto.location.unified.VisibleNetwork.WiFi.ObjectFormat|undefined} */
  this.wifi;

  /** @type {?proto.location.unified.VisibleNetwork.Cell.ObjectFormat|undefined} */
  this.cell;

  /** @type {?boolean|undefined} */
  this.connected;

  /** @type {?number|undefined} */
  this.timestampMs;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.location.unified.VisibleNetwork.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.location.unified.VisibleNetwork}
 */
proto.location.unified.VisibleNetwork.fromObject = function(obj) {
  var msg = new proto.location.unified.VisibleNetwork();
  obj.wifi && jspb.Message.setWrapperField(
      msg, 1, proto.location.unified.VisibleNetwork.WiFi.fromObject(obj.wifi));
  obj.cell && jspb.Message.setWrapperField(
      msg, 2, proto.location.unified.VisibleNetwork.Cell.fromObject(obj.cell));
  obj.connected != null && jspb.Message.setField(msg, 3, obj.connected);
  obj.timestampMs != null && jspb.Message.setField(msg, 4, obj.timestampMs);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.location.unified.VisibleNetwork}
 */
proto.location.unified.VisibleNetwork.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.location.unified.VisibleNetwork;
  return proto.location.unified.VisibleNetwork.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.location.unified.VisibleNetwork} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.location.unified.VisibleNetwork}
 */
proto.location.unified.VisibleNetwork.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.location.unified.VisibleNetwork.WiFi;
      reader.readMessage(value,proto.location.unified.VisibleNetwork.WiFi.deserializeBinaryFromReader);
      msg.setWifi(value);
      break;
    case 2:
      var value = new proto.location.unified.VisibleNetwork.Cell;
      reader.readMessage(value,proto.location.unified.VisibleNetwork.Cell.deserializeBinaryFromReader);
      msg.setCell(value);
      break;
    case 3:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setConnected(value);
      break;
    case 4:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setTimestampMs(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.location.unified.VisibleNetwork.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.location.unified.VisibleNetwork.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.location.unified.VisibleNetwork} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.location.unified.VisibleNetwork.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getWifi();
  if (f != null) {
    writer.writeMessage(
      1,
      f,
      proto.location.unified.VisibleNetwork.WiFi.serializeBinaryToWriter
    );
  }
  f = message.getCell();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.location.unified.VisibleNetwork.Cell.serializeBinaryToWriter
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeBool(
      3,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeInt64(
      4,
      f
    );
  }
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.location.unified.VisibleNetwork.WiFi.prototype.toObject = function(opt_includeInstance) {
  return proto.location.unified.VisibleNetwork.WiFi.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.location.unified.VisibleNetwork.WiFi} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.location.unified.VisibleNetwork.WiFi.toObject = function(includeInstance, msg) {
  var f, obj = {
    bssid: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    levelDbm: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of WiFi as accepted by the `fromObject` method.
 * @record
 */
proto.location.unified.VisibleNetwork.WiFi.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.bssid;

  /** @type {?number|undefined} */
  this.levelDbm;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.location.unified.VisibleNetwork.WiFi.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.location.unified.VisibleNetwork.WiFi}
 */
proto.location.unified.VisibleNetwork.WiFi.fromObject = function(obj) {
  var msg = new proto.location.unified.VisibleNetwork.WiFi();
  obj.bssid != null && jspb.Message.setField(msg, 1, obj.bssid);
  obj.levelDbm != null && jspb.Message.setField(msg, 2, obj.levelDbm);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.location.unified.VisibleNetwork.WiFi}
 */
proto.location.unified.VisibleNetwork.WiFi.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.location.unified.VisibleNetwork.WiFi;
  return proto.location.unified.VisibleNetwork.WiFi.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.location.unified.VisibleNetwork.WiFi} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.location.unified.VisibleNetwork.WiFi}
 */
proto.location.unified.VisibleNetwork.WiFi.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setBssid(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setLevelDbm(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.location.unified.VisibleNetwork.WiFi.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.location.unified.VisibleNetwork.WiFi.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.location.unified.VisibleNetwork.WiFi} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.location.unified.VisibleNetwork.WiFi.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeInt32(
      2,
      f
    );
  }
};


/**
 * optional string bssid = 1;
 * @return {?string}
 */
proto.location.unified.VisibleNetwork.WiFi.prototype.getBssid = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.location.unified.VisibleNetwork.WiFi} returns this
 */
proto.location.unified.VisibleNetwork.WiFi.prototype.setBssid = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.VisibleNetwork.WiFi} returns this
 */
proto.location.unified.VisibleNetwork.WiFi.prototype.clearBssid = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.VisibleNetwork.WiFi.prototype.hasBssid = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional int32 level_dbm = 2;
 * @return {?number}
 */
proto.location.unified.VisibleNetwork.WiFi.prototype.getLevelDbm = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.location.unified.VisibleNetwork.WiFi} returns this
 */
proto.location.unified.VisibleNetwork.WiFi.prototype.setLevelDbm = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.VisibleNetwork.WiFi} returns this
 */
proto.location.unified.VisibleNetwork.WiFi.prototype.clearLevelDbm = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.VisibleNetwork.WiFi.prototype.hasLevelDbm = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.location.unified.VisibleNetwork.WiFi}
 */
proto.location.unified.VisibleNetwork.WiFi.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.location.unified.VisibleNetwork.WiFi, data);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.location.unified.VisibleNetwork.Cell.prototype.toObject = function(opt_includeInstance) {
  return proto.location.unified.VisibleNetwork.Cell.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.location.unified.VisibleNetwork.Cell} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.location.unified.VisibleNetwork.Cell.toObject = function(includeInstance, msg) {
  var f, obj = {
    type: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    cellId: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    locationAreaCode: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,
    mobileCountryCode: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f,
    mobileNetworkCode: (f = jspb.Message.getField(msg, 5)) == null ? undefined : f,
    primaryScramblingCode: (f = jspb.Message.getField(msg, 6)) == null ? undefined : f,
    physicalCellId: (f = jspb.Message.getField(msg, 7)) == null ? undefined : f,
    trackingAreaCode: (f = jspb.Message.getField(msg, 8)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Cell as accepted by the `fromObject` method.
 * @record
 */
proto.location.unified.VisibleNetwork.Cell.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.type;

  /** @type {?number|undefined} */
  this.cellId;

  /** @type {?number|undefined} */
  this.locationAreaCode;

  /** @type {?number|undefined} */
  this.mobileCountryCode;

  /** @type {?number|undefined} */
  this.mobileNetworkCode;

  /** @type {?number|undefined} */
  this.primaryScramblingCode;

  /** @type {?number|undefined} */
  this.physicalCellId;

  /** @type {?number|undefined} */
  this.trackingAreaCode;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.location.unified.VisibleNetwork.Cell.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.location.unified.VisibleNetwork.Cell}
 */
proto.location.unified.VisibleNetwork.Cell.fromObject = function(obj) {
  var msg = new proto.location.unified.VisibleNetwork.Cell();
  obj.type != null && jspb.Message.setField(msg, 1, obj.type);
  obj.cellId != null && jspb.Message.setField(msg, 2, obj.cellId);
  obj.locationAreaCode != null && jspb.Message.setField(msg, 3, obj.locationAreaCode);
  obj.mobileCountryCode != null && jspb.Message.setField(msg, 4, obj.mobileCountryCode);
  obj.mobileNetworkCode != null && jspb.Message.setField(msg, 5, obj.mobileNetworkCode);
  obj.primaryScramblingCode != null && jspb.Message.setField(msg, 6, obj.primaryScramblingCode);
  obj.physicalCellId != null && jspb.Message.setField(msg, 7, obj.physicalCellId);
  obj.trackingAreaCode != null && jspb.Message.setField(msg, 8, obj.trackingAreaCode);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.location.unified.VisibleNetwork.Cell}
 */
proto.location.unified.VisibleNetwork.Cell.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.location.unified.VisibleNetwork.Cell;
  return proto.location.unified.VisibleNetwork.Cell.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.location.unified.VisibleNetwork.Cell} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.location.unified.VisibleNetwork.Cell}
 */
proto.location.unified.VisibleNetwork.Cell.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {!proto.location.unified.VisibleNetwork.Cell.Type} */ (reader.readEnum());
      msg.setType(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setCellId(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setLocationAreaCode(value);
      break;
    case 4:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setMobileCountryCode(value);
      break;
    case 5:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setMobileNetworkCode(value);
      break;
    case 6:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setPrimaryScramblingCode(value);
      break;
    case 7:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setPhysicalCellId(value);
      break;
    case 8:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setTrackingAreaCode(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.location.unified.VisibleNetwork.Cell.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.location.unified.VisibleNetwork.Cell.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.location.unified.VisibleNetwork.Cell} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.location.unified.VisibleNetwork.Cell.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {!proto.location.unified.VisibleNetwork.Cell.Type} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeEnum(
      1,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeInt32(
      2,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeInt32(
      3,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeInt32(
      4,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeInt32(
      5,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeInt32(
      6,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 7));
  if (f != null) {
    writer.writeInt32(
      7,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 8));
  if (f != null) {
    writer.writeInt32(
      8,
      f
    );
  }
};


/**
 * @enum {number}
 */
proto.location.unified.VisibleNetwork.Cell.Type = {
  UNKNOWN: 0,
  GSM: 1,
  LTE: 2,
  CDMA: 3,
  WCDMA: 4
};

/**
 * optional Type type = 1;
 * @return {!proto.location.unified.VisibleNetwork.Cell.Type}
 */
proto.location.unified.VisibleNetwork.Cell.prototype.getType = function() {
  return /** @type {!proto.location.unified.VisibleNetwork.Cell.Type} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?proto.location.unified.VisibleNetwork.Cell.Type|undefined} value
 * @return {!proto.location.unified.VisibleNetwork.Cell} returns this
 */
proto.location.unified.VisibleNetwork.Cell.prototype.setType = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.VisibleNetwork.Cell} returns this
 */
proto.location.unified.VisibleNetwork.Cell.prototype.clearType = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.VisibleNetwork.Cell.prototype.hasType = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional int32 cell_id = 2;
 * @return {?number}
 */
proto.location.unified.VisibleNetwork.Cell.prototype.getCellId = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.location.unified.VisibleNetwork.Cell} returns this
 */
proto.location.unified.VisibleNetwork.Cell.prototype.setCellId = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.VisibleNetwork.Cell} returns this
 */
proto.location.unified.VisibleNetwork.Cell.prototype.clearCellId = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.VisibleNetwork.Cell.prototype.hasCellId = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional int32 location_area_code = 3;
 * @return {?number}
 */
proto.location.unified.VisibleNetwork.Cell.prototype.getLocationAreaCode = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.location.unified.VisibleNetwork.Cell} returns this
 */
proto.location.unified.VisibleNetwork.Cell.prototype.setLocationAreaCode = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.VisibleNetwork.Cell} returns this
 */
proto.location.unified.VisibleNetwork.Cell.prototype.clearLocationAreaCode = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.VisibleNetwork.Cell.prototype.hasLocationAreaCode = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional int32 mobile_country_code = 4;
 * @return {?number}
 */
proto.location.unified.VisibleNetwork.Cell.prototype.getMobileCountryCode = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 4));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.location.unified.VisibleNetwork.Cell} returns this
 */
proto.location.unified.VisibleNetwork.Cell.prototype.setMobileCountryCode = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.VisibleNetwork.Cell} returns this
 */
proto.location.unified.VisibleNetwork.Cell.prototype.clearMobileCountryCode = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.VisibleNetwork.Cell.prototype.hasMobileCountryCode = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional int32 mobile_network_code = 5;
 * @return {?number}
 */
proto.location.unified.VisibleNetwork.Cell.prototype.getMobileNetworkCode = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 5));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.location.unified.VisibleNetwork.Cell} returns this
 */
proto.location.unified.VisibleNetwork.Cell.prototype.setMobileNetworkCode = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.VisibleNetwork.Cell} returns this
 */
proto.location.unified.VisibleNetwork.Cell.prototype.clearMobileNetworkCode = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.VisibleNetwork.Cell.prototype.hasMobileNetworkCode = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional int32 primary_scrambling_code = 6;
 * @return {?number}
 */
proto.location.unified.VisibleNetwork.Cell.prototype.getPrimaryScramblingCode = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 6));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.location.unified.VisibleNetwork.Cell} returns this
 */
proto.location.unified.VisibleNetwork.Cell.prototype.setPrimaryScramblingCode = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.VisibleNetwork.Cell} returns this
 */
proto.location.unified.VisibleNetwork.Cell.prototype.clearPrimaryScramblingCode = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.VisibleNetwork.Cell.prototype.hasPrimaryScramblingCode = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional int32 physical_cell_id = 7;
 * @return {?number}
 */
proto.location.unified.VisibleNetwork.Cell.prototype.getPhysicalCellId = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 7));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.location.unified.VisibleNetwork.Cell} returns this
 */
proto.location.unified.VisibleNetwork.Cell.prototype.setPhysicalCellId = function(value) {
  return jspb.Message.setField(this, 7, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.VisibleNetwork.Cell} returns this
 */
proto.location.unified.VisibleNetwork.Cell.prototype.clearPhysicalCellId = function() {
  return jspb.Message.setField(this, 7, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.VisibleNetwork.Cell.prototype.hasPhysicalCellId = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * optional int32 tracking_area_code = 8;
 * @return {?number}
 */
proto.location.unified.VisibleNetwork.Cell.prototype.getTrackingAreaCode = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 8));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.location.unified.VisibleNetwork.Cell} returns this
 */
proto.location.unified.VisibleNetwork.Cell.prototype.setTrackingAreaCode = function(value) {
  return jspb.Message.setField(this, 8, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.VisibleNetwork.Cell} returns this
 */
proto.location.unified.VisibleNetwork.Cell.prototype.clearTrackingAreaCode = function() {
  return jspb.Message.setField(this, 8, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.VisibleNetwork.Cell.prototype.hasTrackingAreaCode = function() {
  return jspb.Message.getField(this, 8) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.location.unified.VisibleNetwork.Cell}
 */
proto.location.unified.VisibleNetwork.Cell.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.location.unified.VisibleNetwork.Cell, data);
};


/**
 * optional WiFi wifi = 1;
 * @return {?proto.location.unified.VisibleNetwork.WiFi}
 */
proto.location.unified.VisibleNetwork.prototype.getWifi = function() {
  return /** @type{?proto.location.unified.VisibleNetwork.WiFi} */ (
    jspb.Message.getWrapperField(this, proto.location.unified.VisibleNetwork.WiFi, 1));
};


/**
 * @param {?proto.location.unified.VisibleNetwork.WiFi|undefined} value
 * @return {!proto.location.unified.VisibleNetwork} returns this
*/
proto.location.unified.VisibleNetwork.prototype.setWifi = function(value) {
  return jspb.Message.setOneofWrapperField(this, 1, proto.location.unified.VisibleNetwork.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.location.unified.VisibleNetwork} returns this
 */
proto.location.unified.VisibleNetwork.prototype.clearWifi = function() {
  return this.setWifi(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.VisibleNetwork.prototype.hasWifi = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional Cell cell = 2;
 * @return {?proto.location.unified.VisibleNetwork.Cell}
 */
proto.location.unified.VisibleNetwork.prototype.getCell = function() {
  return /** @type{?proto.location.unified.VisibleNetwork.Cell} */ (
    jspb.Message.getWrapperField(this, proto.location.unified.VisibleNetwork.Cell, 2));
};


/**
 * @param {?proto.location.unified.VisibleNetwork.Cell|undefined} value
 * @return {!proto.location.unified.VisibleNetwork} returns this
*/
proto.location.unified.VisibleNetwork.prototype.setCell = function(value) {
  return jspb.Message.setOneofWrapperField(this, 2, proto.location.unified.VisibleNetwork.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.location.unified.VisibleNetwork} returns this
 */
proto.location.unified.VisibleNetwork.prototype.clearCell = function() {
  return this.setCell(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.VisibleNetwork.prototype.hasCell = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional bool connected = 3;
 * @return {?boolean}
 */
proto.location.unified.VisibleNetwork.prototype.getConnected = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 3));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.location.unified.VisibleNetwork} returns this
 */
proto.location.unified.VisibleNetwork.prototype.setConnected = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.VisibleNetwork} returns this
 */
proto.location.unified.VisibleNetwork.prototype.clearConnected = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.VisibleNetwork.prototype.hasConnected = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional int64 timestamp_ms = 4;
 * @return {?number}
 */
proto.location.unified.VisibleNetwork.prototype.getTimestampMs = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 4));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.location.unified.VisibleNetwork} returns this
 */
proto.location.unified.VisibleNetwork.prototype.setTimestampMs = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.VisibleNetwork} returns this
 */
proto.location.unified.VisibleNetwork.prototype.clearTimestampMs = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.VisibleNetwork.prototype.hasTimestampMs = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.location.unified.VisibleNetwork}
 */
proto.location.unified.VisibleNetwork.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.location.unified.VisibleNetwork, data);
};



//location/unified/proto/location_descriptor_proto_jspb/locationdescriptor.jspb.js
// source: location/unified/proto/location_descriptor.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.location.unified.LocationDescriptor');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.location.unified.FeatureIdProto');
goog.require('proto.location.unified.LatLng');
goog.require('proto.location.unified.LatLngRect');
goog.require('proto.location.unified.LocationAttributesProto');
goog.require('proto.location.unified.PresenceInterval');
goog.require('proto.location.unified.SemanticPlace');
goog.require('proto.location.unified.VisibleNetwork');

goog.forwardDeclare('proto.location.unified.LocationProducer');
goog.forwardDeclare('proto.location.unified.LocationProvenance');
goog.forwardDeclare('proto.location.unified.LocationRole');
goog.forwardDeclare('proto.location.unified.LocationSemantic');
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.location.unified.LocationDescriptor = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.location.unified.LocationDescriptor.repeatedFields_, null);
};
goog.inherits(proto.location.unified.LocationDescriptor, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.location.unified.LocationDescriptor.displayName = 'proto.location.unified.LocationDescriptor';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.location.unified.LocationDescriptor.repeatedFields_ = [21,22,23,24];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.location.unified.LocationDescriptor.prototype.toObject = function(opt_includeInstance) {
  return proto.location.unified.LocationDescriptor.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.location.unified.LocationDescriptor} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.location.unified.LocationDescriptor.toObject = function(includeInstance, msg) {
  var f, obj = {
    role: jspb.Message.getFieldWithDefault(msg, 1, 0),
    producer: jspb.Message.getFieldWithDefault(msg, 2, 0),
    timestamp: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,
    loc: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f,
    latlng: (f = msg.getLatlng()) && proto.location.unified.LatLng.toObject(includeInstance, f),
    latlngSpan: (f = msg.getLatlngSpan()) && proto.location.unified.LatLng.toObject(includeInstance, f),
    rect: (f = msg.getRect()) && proto.location.unified.LatLngRect.toObject(includeInstance, f),
    radius: (f = jspb.Message.getOptionalFloatingPointField(msg, 7)) == null ? undefined : f,
    confidence: jspb.Message.getFieldWithDefault(msg, 8, 100),
    featureId: (f = msg.getFeatureId()) && proto.location.unified.FeatureIdProto.toObject(includeInstance, f),
    mid: (f = jspb.Message.getField(msg, 16)) == null ? undefined : f,
    levelFeatureId: (f = msg.getLevelFeatureId()) && proto.location.unified.FeatureIdProto.toObject(includeInstance, f),
    levelNumber: (f = jspb.Message.getOptionalFloatingPointField(msg, 18)) == null ? undefined : f,
    language: (f = jspb.Message.getField(msg, 11)) == null ? undefined : f,
    provenance: jspb.Message.getFieldWithDefault(msg, 9, 0),
    historicalRole: jspb.Message.getFieldWithDefault(msg, 12, 0),
    historicalProducer: jspb.Message.getFieldWithDefault(msg, 13, 0),
    historicalProminence: (f = jspb.Message.getField(msg, 15)) == null ? undefined : f,
    attributes: (f = msg.getAttributes()) && proto.location.unified.LocationAttributesProto.toObject(includeInstance, f),
    diagnosticInfo: (f = jspb.Message.getField(msg, 20)) == null ? undefined : f,
    semanticList: (f = jspb.Message.getRepeatedField(msg, 21)) == null ? undefined : f,
    semanticPlaceList: jspb.Message.toObjectList(msg.getSemanticPlaceList(),
    proto.location.unified.SemanticPlace.toObject, includeInstance),
    visibleNetworkList: jspb.Message.toObjectList(msg.getVisibleNetworkList(),
    proto.location.unified.VisibleNetwork.toObject, includeInstance),
    presenceIntervalList: jspb.Message.toObjectList(msg.getPresenceIntervalList(),
    proto.location.unified.PresenceInterval.toObject, includeInstance)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of LocationDescriptor as accepted by the `fromObject` method.
 * @record
 */
proto.location.unified.LocationDescriptor.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.role;

  /** @type {?number|undefined} */
  this.producer;

  /** @type {?number|undefined} */
  this.timestamp;

  /** @type {?string|undefined} */
  this.loc;

  /** @type {?proto.location.unified.LatLng.ObjectFormat|undefined} */
  this.latlng;

  /** @type {?proto.location.unified.LatLng.ObjectFormat|undefined} */
  this.latlngSpan;

  /** @type {?proto.location.unified.LatLngRect.ObjectFormat|undefined} */
  this.rect;

  /** @type {?number|undefined} */
  this.radius;

  /** @type {?number|undefined} */
  this.confidence;

  /** @type {?proto.location.unified.FeatureIdProto.ObjectFormat|undefined} */
  this.featureId;

  /** @type {?number|undefined} */
  this.mid;

  /** @type {?proto.location.unified.FeatureIdProto.ObjectFormat|undefined} */
  this.levelFeatureId;

  /** @type {?number|undefined} */
  this.levelNumber;

  /** @type {?string|undefined} */
  this.language;

  /** @type {?number|undefined} */
  this.provenance;

  /** @type {?number|undefined} */
  this.historicalRole;

  /** @type {?number|undefined} */
  this.historicalProducer;

  /** @type {?number|undefined} */
  this.historicalProminence;

  /** @type {?proto.location.unified.LocationAttributesProto.ObjectFormat|undefined} */
  this.attributes;

  /** @type {?string|undefined} */
  this.diagnosticInfo;

  /** @type {?Array<number>|undefined} */
  this.semanticList;

  /** @type {?Array<!proto.location.unified.SemanticPlace.ObjectFormat>|undefined} */
  this.semanticPlaceList;

  /** @type {?Array<!proto.location.unified.VisibleNetwork.ObjectFormat>|undefined} */
  this.visibleNetworkList;

  /** @type {?Array<!proto.location.unified.PresenceInterval.ObjectFormat>|undefined} */
  this.presenceIntervalList;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.location.unified.LocationDescriptor.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.location.unified.LocationDescriptor}
 */
proto.location.unified.LocationDescriptor.fromObject = function(obj) {
  var msg = new proto.location.unified.LocationDescriptor();
  obj.role != null && jspb.Message.setField(msg, 1, obj.role);
  obj.producer != null && jspb.Message.setField(msg, 2, obj.producer);
  obj.timestamp != null && jspb.Message.setField(msg, 3, obj.timestamp);
  obj.loc != null && jspb.Message.setField(msg, 4, obj.loc);
  obj.latlng && jspb.Message.setWrapperField(
      msg, 5, proto.location.unified.LatLng.fromObject(obj.latlng));
  obj.latlngSpan && jspb.Message.setWrapperField(
      msg, 6, proto.location.unified.LatLng.fromObject(obj.latlngSpan));
  obj.rect && jspb.Message.setWrapperField(
      msg, 14, proto.location.unified.LatLngRect.fromObject(obj.rect));
  obj.radius != null && jspb.Message.setField(msg, 7, obj.radius);
  obj.confidence != null && jspb.Message.setField(msg, 8, obj.confidence);
  obj.featureId && jspb.Message.setWrapperField(
      msg, 10, proto.location.unified.FeatureIdProto.fromObject(obj.featureId));
  obj.mid != null && jspb.Message.setField(msg, 16, obj.mid);
  obj.levelFeatureId && jspb.Message.setWrapperField(
      msg, 17, proto.location.unified.FeatureIdProto.fromObject(obj.levelFeatureId));
  obj.levelNumber != null && jspb.Message.setField(msg, 18, obj.levelNumber);
  obj.language != null && jspb.Message.setField(msg, 11, obj.language);
  obj.provenance != null && jspb.Message.setField(msg, 9, obj.provenance);
  obj.historicalRole != null && jspb.Message.setField(msg, 12, obj.historicalRole);
  obj.historicalProducer != null && jspb.Message.setField(msg, 13, obj.historicalProducer);
  obj.historicalProminence != null && jspb.Message.setField(msg, 15, obj.historicalProminence);
  obj.attributes && jspb.Message.setWrapperField(
      msg, 19, proto.location.unified.LocationAttributesProto.fromObject(obj.attributes));
  obj.diagnosticInfo != null && jspb.Message.setField(msg, 20, obj.diagnosticInfo);
  obj.semanticList != null && jspb.Message.setField(msg, 21, obj.semanticList);
  obj.semanticPlaceList && jspb.Message.setRepeatedWrapperField(
      msg, 22, obj.semanticPlaceList.map(
          proto.location.unified.SemanticPlace.fromObject));
  obj.visibleNetworkList && jspb.Message.setRepeatedWrapperField(
      msg, 23, obj.visibleNetworkList.map(
          proto.location.unified.VisibleNetwork.fromObject));
  obj.presenceIntervalList && jspb.Message.setRepeatedWrapperField(
      msg, 24, obj.presenceIntervalList.map(
          proto.location.unified.PresenceInterval.fromObject));
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.location.unified.LocationDescriptor}
 */
proto.location.unified.LocationDescriptor.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.location.unified.LocationDescriptor;
  return proto.location.unified.LocationDescriptor.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.location.unified.LocationDescriptor} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.location.unified.LocationDescriptor}
 */
proto.location.unified.LocationDescriptor.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {!proto.location.unified.LocationRole} */ (reader.readEnum());
      msg.setRole(value);
      break;
    case 2:
      var value = /** @type {!proto.location.unified.LocationProducer} */ (reader.readEnum());
      msg.setProducer(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setTimestamp(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setLoc(value);
      break;
    case 5:
      var value = new proto.location.unified.LatLng;
      reader.readMessage(value,proto.location.unified.LatLng.deserializeBinaryFromReader);
      msg.setLatlng(value);
      break;
    case 6:
      var value = new proto.location.unified.LatLng;
      reader.readMessage(value,proto.location.unified.LatLng.deserializeBinaryFromReader);
      msg.setLatlngSpan(value);
      break;
    case 14:
      var value = new proto.location.unified.LatLngRect;
      reader.readMessage(value,proto.location.unified.LatLngRect.deserializeBinaryFromReader);
      msg.setRect(value);
      break;
    case 7:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setRadius(value);
      break;
    case 8:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setConfidence(value);
      break;
    case 10:
      var value = new proto.location.unified.FeatureIdProto;
      reader.readMessage(value,proto.location.unified.FeatureIdProto.deserializeBinaryFromReader);
      msg.setFeatureId(value);
      break;
    case 16:
      var value = /** @type {number} */ (reader.readUint64());
      msg.setMid(value);
      break;
    case 17:
      var value = new proto.location.unified.FeatureIdProto;
      reader.readMessage(value,proto.location.unified.FeatureIdProto.deserializeBinaryFromReader);
      msg.setLevelFeatureId(value);
      break;
    case 18:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setLevelNumber(value);
      break;
    case 11:
      var value = /** @type {string} */ (reader.readString());
      msg.setLanguage(value);
      break;
    case 9:
      var value = /** @type {!proto.location.unified.LocationProvenance} */ (reader.readEnum());
      msg.setProvenance(value);
      break;
    case 12:
      var value = /** @type {!proto.location.unified.LocationRole} */ (reader.readEnum());
      msg.setHistoricalRole(value);
      break;
    case 13:
      var value = /** @type {!proto.location.unified.LocationProducer} */ (reader.readEnum());
      msg.setHistoricalProducer(value);
      break;
    case 15:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setHistoricalProminence(value);
      break;
    case 19:
      var value = new proto.location.unified.LocationAttributesProto;
      reader.readMessage(value,proto.location.unified.LocationAttributesProto.deserializeBinaryFromReader);
      msg.setAttributes(value);
      break;
    case 20:
      var value = /** @type {string} */ (reader.readString());
      msg.setDiagnosticInfo(value);
      break;
    case 21:
      var value = /** @type {!proto.location.unified.LocationSemantic} */ (reader.readEnum());
      msg.addSemantic(value);
      break;
    case 22:
      var value = new proto.location.unified.SemanticPlace;
      reader.readMessage(value,proto.location.unified.SemanticPlace.deserializeBinaryFromReader);
      msg.addSemanticPlace(value);
      break;
    case 23:
      var value = new proto.location.unified.VisibleNetwork;
      reader.readMessage(value,proto.location.unified.VisibleNetwork.deserializeBinaryFromReader);
      msg.addVisibleNetwork(value);
      break;
    case 24:
      var value = new proto.location.unified.PresenceInterval;
      reader.readMessage(value,proto.location.unified.PresenceInterval.deserializeBinaryFromReader);
      msg.addPresenceInterval(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.location.unified.LocationDescriptor.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.location.unified.LocationDescriptor.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.location.unified.LocationDescriptor} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.location.unified.LocationDescriptor.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {!proto.location.unified.LocationRole} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeEnum(
      1,
      f
    );
  }
  f = /** @type {!proto.location.unified.LocationProducer} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeEnum(
      2,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeInt64(
      3,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeString(
      4,
      f
    );
  }
  f = message.getLatlng();
  if (f != null) {
    writer.writeMessage(
      5,
      f,
      proto.location.unified.LatLng.serializeBinaryToWriter
    );
  }
  f = message.getLatlngSpan();
  if (f != null) {
    writer.writeMessage(
      6,
      f,
      proto.location.unified.LatLng.serializeBinaryToWriter
    );
  }
  f = message.getRect();
  if (f != null) {
    writer.writeMessage(
      14,
      f,
      proto.location.unified.LatLngRect.serializeBinaryToWriter
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 7));
  if (f != null) {
    writer.writeFloat(
      7,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 8));
  if (f != null) {
    writer.writeInt32(
      8,
      f
    );
  }
  f = message.getFeatureId();
  if (f != null) {
    writer.writeMessage(
      10,
      f,
      proto.location.unified.FeatureIdProto.serializeBinaryToWriter
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 16));
  if (f != null) {
    writer.writeUint64(
      16,
      f
    );
  }
  f = message.getLevelFeatureId();
  if (f != null) {
    writer.writeMessage(
      17,
      f,
      proto.location.unified.FeatureIdProto.serializeBinaryToWriter
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 18));
  if (f != null) {
    writer.writeFloat(
      18,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 11));
  if (f != null) {
    writer.writeString(
      11,
      f
    );
  }
  f = /** @type {!proto.location.unified.LocationProvenance} */ (jspb.Message.getField(message, 9));
  if (f != null) {
    writer.writeEnum(
      9,
      f
    );
  }
  f = /** @type {!proto.location.unified.LocationRole} */ (jspb.Message.getField(message, 12));
  if (f != null) {
    writer.writeEnum(
      12,
      f
    );
  }
  f = /** @type {!proto.location.unified.LocationProducer} */ (jspb.Message.getField(message, 13));
  if (f != null) {
    writer.writeEnum(
      13,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 15));
  if (f != null) {
    writer.writeInt32(
      15,
      f
    );
  }
  f = message.getAttributes();
  if (f != null) {
    writer.writeMessage(
      19,
      f,
      proto.location.unified.LocationAttributesProto.serializeBinaryToWriter
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 20));
  if (f != null) {
    writer.writeString(
      20,
      f
    );
  }
  f = message.getSemanticList();
  if (f.length > 0) {
    writer.writeRepeatedEnum(
      21,
      f
    );
  }
  f = message.getSemanticPlaceList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      22,
      f,
      proto.location.unified.SemanticPlace.serializeBinaryToWriter
    );
  }
  f = message.getVisibleNetworkList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      23,
      f,
      proto.location.unified.VisibleNetwork.serializeBinaryToWriter
    );
  }
  f = message.getPresenceIntervalList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      24,
      f,
      proto.location.unified.PresenceInterval.serializeBinaryToWriter
    );
  }
};


/**
 * optional LocationRole role = 1;
 * @return {!proto.location.unified.LocationRole}
 */
proto.location.unified.LocationDescriptor.prototype.getRole = function() {
  return /** @type {!proto.location.unified.LocationRole} */ (jspb.Message.getFieldWithDefault(this, 1, 0));
};


/**
 * @param {?proto.location.unified.LocationRole|undefined} value
 * @return {!proto.location.unified.LocationDescriptor} returns this
 */
proto.location.unified.LocationDescriptor.prototype.setRole = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.LocationDescriptor} returns this
 */
proto.location.unified.LocationDescriptor.prototype.clearRole = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.LocationDescriptor.prototype.hasRole = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional LocationProducer producer = 2;
 * @return {!proto.location.unified.LocationProducer}
 */
proto.location.unified.LocationDescriptor.prototype.getProducer = function() {
  return /** @type {!proto.location.unified.LocationProducer} */ (jspb.Message.getFieldWithDefault(this, 2, 0));
};


/**
 * @param {?proto.location.unified.LocationProducer|undefined} value
 * @return {!proto.location.unified.LocationDescriptor} returns this
 */
proto.location.unified.LocationDescriptor.prototype.setProducer = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.LocationDescriptor} returns this
 */
proto.location.unified.LocationDescriptor.prototype.clearProducer = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.LocationDescriptor.prototype.hasProducer = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional int64 timestamp = 3;
 * @return {?number}
 */
proto.location.unified.LocationDescriptor.prototype.getTimestamp = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.location.unified.LocationDescriptor} returns this
 */
proto.location.unified.LocationDescriptor.prototype.setTimestamp = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.LocationDescriptor} returns this
 */
proto.location.unified.LocationDescriptor.prototype.clearTimestamp = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.LocationDescriptor.prototype.hasTimestamp = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional string loc = 4;
 * @return {?string}
 */
proto.location.unified.LocationDescriptor.prototype.getLoc = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 4));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.location.unified.LocationDescriptor} returns this
 */
proto.location.unified.LocationDescriptor.prototype.setLoc = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.LocationDescriptor} returns this
 */
proto.location.unified.LocationDescriptor.prototype.clearLoc = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.LocationDescriptor.prototype.hasLoc = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional LatLng latlng = 5;
 * @return {?proto.location.unified.LatLng}
 */
proto.location.unified.LocationDescriptor.prototype.getLatlng = function() {
  return /** @type{?proto.location.unified.LatLng} */ (
    jspb.Message.getWrapperField(this, proto.location.unified.LatLng, 5));
};


/**
 * @param {?proto.location.unified.LatLng|undefined} value
 * @return {!proto.location.unified.LocationDescriptor} returns this
*/
proto.location.unified.LocationDescriptor.prototype.setLatlng = function(value) {
  return jspb.Message.setWrapperField(this, 5, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.location.unified.LocationDescriptor} returns this
 */
proto.location.unified.LocationDescriptor.prototype.clearLatlng = function() {
  return this.setLatlng(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.LocationDescriptor.prototype.hasLatlng = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional LatLng latlng_span = 6;
 * @return {?proto.location.unified.LatLng}
 */
proto.location.unified.LocationDescriptor.prototype.getLatlngSpan = function() {
  return /** @type{?proto.location.unified.LatLng} */ (
    jspb.Message.getWrapperField(this, proto.location.unified.LatLng, 6));
};


/**
 * @param {?proto.location.unified.LatLng|undefined} value
 * @return {!proto.location.unified.LocationDescriptor} returns this
*/
proto.location.unified.LocationDescriptor.prototype.setLatlngSpan = function(value) {
  return jspb.Message.setWrapperField(this, 6, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.location.unified.LocationDescriptor} returns this
 */
proto.location.unified.LocationDescriptor.prototype.clearLatlngSpan = function() {
  return this.setLatlngSpan(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.LocationDescriptor.prototype.hasLatlngSpan = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional LatLngRect rect = 14;
 * @return {?proto.location.unified.LatLngRect}
 */
proto.location.unified.LocationDescriptor.prototype.getRect = function() {
  return /** @type{?proto.location.unified.LatLngRect} */ (
    jspb.Message.getWrapperField(this, proto.location.unified.LatLngRect, 14));
};


/**
 * @param {?proto.location.unified.LatLngRect|undefined} value
 * @return {!proto.location.unified.LocationDescriptor} returns this
*/
proto.location.unified.LocationDescriptor.prototype.setRect = function(value) {
  return jspb.Message.setWrapperField(this, 14, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.location.unified.LocationDescriptor} returns this
 */
proto.location.unified.LocationDescriptor.prototype.clearRect = function() {
  return this.setRect(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.LocationDescriptor.prototype.hasRect = function() {
  return jspb.Message.getField(this, 14) != null;
};


/**
 * optional float radius = 7;
 * @return {?number}
 */
proto.location.unified.LocationDescriptor.prototype.getRadius = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 7));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.location.unified.LocationDescriptor} returns this
 */
proto.location.unified.LocationDescriptor.prototype.setRadius = function(value) {
  return jspb.Message.setField(this, 7, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.LocationDescriptor} returns this
 */
proto.location.unified.LocationDescriptor.prototype.clearRadius = function() {
  return jspb.Message.setField(this, 7, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.LocationDescriptor.prototype.hasRadius = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * optional int32 confidence = 8;
 * @return {number}
 */
proto.location.unified.LocationDescriptor.prototype.getConfidence = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 8, 100));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.location.unified.LocationDescriptor} returns this
 */
proto.location.unified.LocationDescriptor.prototype.setConfidence = function(value) {
  return jspb.Message.setField(this, 8, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.LocationDescriptor} returns this
 */
proto.location.unified.LocationDescriptor.prototype.clearConfidence = function() {
  return jspb.Message.setField(this, 8, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.LocationDescriptor.prototype.hasConfidence = function() {
  return jspb.Message.getField(this, 8) != null;
};


/**
 * optional FeatureIdProto feature_id = 10;
 * @return {?proto.location.unified.FeatureIdProto}
 */
proto.location.unified.LocationDescriptor.prototype.getFeatureId = function() {
  return /** @type{?proto.location.unified.FeatureIdProto} */ (
    jspb.Message.getWrapperField(this, proto.location.unified.FeatureIdProto, 10));
};


/**
 * @param {?proto.location.unified.FeatureIdProto|undefined} value
 * @return {!proto.location.unified.LocationDescriptor} returns this
*/
proto.location.unified.LocationDescriptor.prototype.setFeatureId = function(value) {
  return jspb.Message.setWrapperField(this, 10, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.location.unified.LocationDescriptor} returns this
 */
proto.location.unified.LocationDescriptor.prototype.clearFeatureId = function() {
  return this.setFeatureId(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.LocationDescriptor.prototype.hasFeatureId = function() {
  return jspb.Message.getField(this, 10) != null;
};


/**
 * optional uint64 mid = 16;
 * @return {?number}
 */
proto.location.unified.LocationDescriptor.prototype.getMid = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 16));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.location.unified.LocationDescriptor} returns this
 */
proto.location.unified.LocationDescriptor.prototype.setMid = function(value) {
  return jspb.Message.setField(this, 16, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.LocationDescriptor} returns this
 */
proto.location.unified.LocationDescriptor.prototype.clearMid = function() {
  return jspb.Message.setField(this, 16, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.LocationDescriptor.prototype.hasMid = function() {
  return jspb.Message.getField(this, 16) != null;
};


/**
 * optional FeatureIdProto level_feature_id = 17;
 * @return {?proto.location.unified.FeatureIdProto}
 */
proto.location.unified.LocationDescriptor.prototype.getLevelFeatureId = function() {
  return /** @type{?proto.location.unified.FeatureIdProto} */ (
    jspb.Message.getWrapperField(this, proto.location.unified.FeatureIdProto, 17));
};


/**
 * @param {?proto.location.unified.FeatureIdProto|undefined} value
 * @return {!proto.location.unified.LocationDescriptor} returns this
*/
proto.location.unified.LocationDescriptor.prototype.setLevelFeatureId = function(value) {
  return jspb.Message.setWrapperField(this, 17, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.location.unified.LocationDescriptor} returns this
 */
proto.location.unified.LocationDescriptor.prototype.clearLevelFeatureId = function() {
  return this.setLevelFeatureId(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.LocationDescriptor.prototype.hasLevelFeatureId = function() {
  return jspb.Message.getField(this, 17) != null;
};


/**
 * optional float level_number = 18;
 * @return {?number}
 */
proto.location.unified.LocationDescriptor.prototype.getLevelNumber = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 18));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.location.unified.LocationDescriptor} returns this
 */
proto.location.unified.LocationDescriptor.prototype.setLevelNumber = function(value) {
  return jspb.Message.setField(this, 18, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.LocationDescriptor} returns this
 */
proto.location.unified.LocationDescriptor.prototype.clearLevelNumber = function() {
  return jspb.Message.setField(this, 18, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.LocationDescriptor.prototype.hasLevelNumber = function() {
  return jspb.Message.getField(this, 18) != null;
};


/**
 * optional string language = 11;
 * @return {?string}
 */
proto.location.unified.LocationDescriptor.prototype.getLanguage = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 11));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.location.unified.LocationDescriptor} returns this
 */
proto.location.unified.LocationDescriptor.prototype.setLanguage = function(value) {
  return jspb.Message.setField(this, 11, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.LocationDescriptor} returns this
 */
proto.location.unified.LocationDescriptor.prototype.clearLanguage = function() {
  return jspb.Message.setField(this, 11, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.LocationDescriptor.prototype.hasLanguage = function() {
  return jspb.Message.getField(this, 11) != null;
};


/**
 * optional LocationProvenance provenance = 9;
 * @return {!proto.location.unified.LocationProvenance}
 */
proto.location.unified.LocationDescriptor.prototype.getProvenance = function() {
  return /** @type {!proto.location.unified.LocationProvenance} */ (jspb.Message.getFieldWithDefault(this, 9, 0));
};


/**
 * @param {?proto.location.unified.LocationProvenance|undefined} value
 * @return {!proto.location.unified.LocationDescriptor} returns this
 */
proto.location.unified.LocationDescriptor.prototype.setProvenance = function(value) {
  return jspb.Message.setField(this, 9, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.LocationDescriptor} returns this
 */
proto.location.unified.LocationDescriptor.prototype.clearProvenance = function() {
  return jspb.Message.setField(this, 9, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.LocationDescriptor.prototype.hasProvenance = function() {
  return jspb.Message.getField(this, 9) != null;
};


/**
 * optional LocationRole historical_role = 12;
 * @return {!proto.location.unified.LocationRole}
 */
proto.location.unified.LocationDescriptor.prototype.getHistoricalRole = function() {
  return /** @type {!proto.location.unified.LocationRole} */ (jspb.Message.getFieldWithDefault(this, 12, 0));
};


/**
 * @param {?proto.location.unified.LocationRole|undefined} value
 * @return {!proto.location.unified.LocationDescriptor} returns this
 */
proto.location.unified.LocationDescriptor.prototype.setHistoricalRole = function(value) {
  return jspb.Message.setField(this, 12, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.LocationDescriptor} returns this
 */
proto.location.unified.LocationDescriptor.prototype.clearHistoricalRole = function() {
  return jspb.Message.setField(this, 12, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.LocationDescriptor.prototype.hasHistoricalRole = function() {
  return jspb.Message.getField(this, 12) != null;
};


/**
 * optional LocationProducer historical_producer = 13;
 * @return {!proto.location.unified.LocationProducer}
 */
proto.location.unified.LocationDescriptor.prototype.getHistoricalProducer = function() {
  return /** @type {!proto.location.unified.LocationProducer} */ (jspb.Message.getFieldWithDefault(this, 13, 0));
};


/**
 * @param {?proto.location.unified.LocationProducer|undefined} value
 * @return {!proto.location.unified.LocationDescriptor} returns this
 */
proto.location.unified.LocationDescriptor.prototype.setHistoricalProducer = function(value) {
  return jspb.Message.setField(this, 13, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.LocationDescriptor} returns this
 */
proto.location.unified.LocationDescriptor.prototype.clearHistoricalProducer = function() {
  return jspb.Message.setField(this, 13, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.LocationDescriptor.prototype.hasHistoricalProducer = function() {
  return jspb.Message.getField(this, 13) != null;
};


/**
 * optional int32 historical_prominence = 15;
 * @return {?number}
 */
proto.location.unified.LocationDescriptor.prototype.getHistoricalProminence = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 15));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.location.unified.LocationDescriptor} returns this
 */
proto.location.unified.LocationDescriptor.prototype.setHistoricalProminence = function(value) {
  return jspb.Message.setField(this, 15, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.LocationDescriptor} returns this
 */
proto.location.unified.LocationDescriptor.prototype.clearHistoricalProminence = function() {
  return jspb.Message.setField(this, 15, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.LocationDescriptor.prototype.hasHistoricalProminence = function() {
  return jspb.Message.getField(this, 15) != null;
};


/**
 * optional LocationAttributesProto attributes = 19;
 * @return {?proto.location.unified.LocationAttributesProto}
 */
proto.location.unified.LocationDescriptor.prototype.getAttributes = function() {
  return /** @type{?proto.location.unified.LocationAttributesProto} */ (
    jspb.Message.getWrapperField(this, proto.location.unified.LocationAttributesProto, 19));
};


/**
 * @param {?proto.location.unified.LocationAttributesProto|undefined} value
 * @return {!proto.location.unified.LocationDescriptor} returns this
*/
proto.location.unified.LocationDescriptor.prototype.setAttributes = function(value) {
  return jspb.Message.setWrapperField(this, 19, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.location.unified.LocationDescriptor} returns this
 */
proto.location.unified.LocationDescriptor.prototype.clearAttributes = function() {
  return this.setAttributes(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.LocationDescriptor.prototype.hasAttributes = function() {
  return jspb.Message.getField(this, 19) != null;
};


/**
 * optional string diagnostic_info = 20;
 * @return {?string}
 */
proto.location.unified.LocationDescriptor.prototype.getDiagnosticInfo = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 20));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.location.unified.LocationDescriptor} returns this
 */
proto.location.unified.LocationDescriptor.prototype.setDiagnosticInfo = function(value) {
  return jspb.Message.setField(this, 20, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.location.unified.LocationDescriptor} returns this
 */
proto.location.unified.LocationDescriptor.prototype.clearDiagnosticInfo = function() {
  return jspb.Message.setField(this, 20, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.location.unified.LocationDescriptor.prototype.hasDiagnosticInfo = function() {
  return jspb.Message.getField(this, 20) != null;
};


/**
 * repeated LocationSemantic semantic = 21;
 * @return {!Array<!proto.location.unified.LocationSemantic>}
 */
proto.location.unified.LocationDescriptor.prototype.getSemanticList = function() {
  return /** @type {!Array<!proto.location.unified.LocationSemantic>} */ (jspb.Message.getRepeatedField(this, 21));
};


/**
 * @param {Array<!proto.location.unified.LocationSemantic>|undefined} value
 * @return {!proto.location.unified.LocationDescriptor} returns this
 */
proto.location.unified.LocationDescriptor.prototype.setSemanticList = function(value) {
  return jspb.Message.setField(this, 21, value || []);
};


/**
 * @param {!proto.location.unified.LocationSemantic} value
 * @param {number=} opt_index
 * @return {!proto.location.unified.LocationDescriptor} returns this
 */
proto.location.unified.LocationDescriptor.prototype.addSemantic = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 21, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.location.unified.LocationDescriptor} returns this
 */
proto.location.unified.LocationDescriptor.prototype.clearSemanticList = function() {
  return this.setSemanticList([]);
};


/**
 * repeated SemanticPlace semantic_place = 22;
 * @return {!Array<!proto.location.unified.SemanticPlace>}
 */
proto.location.unified.LocationDescriptor.prototype.getSemanticPlaceList = function() {
  return /** @type{!Array<!proto.location.unified.SemanticPlace>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.location.unified.SemanticPlace, 22));
};


/**
 * @param {Array<!proto.location.unified.SemanticPlace>|undefined} value
 * @return {!proto.location.unified.LocationDescriptor} returns this
*/
proto.location.unified.LocationDescriptor.prototype.setSemanticPlaceList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 22, value);
};


/**
 * @param {!proto.location.unified.SemanticPlace=} opt_value
 * @param {number=} opt_index
 * @return {!proto.location.unified.SemanticPlace}
 */
proto.location.unified.LocationDescriptor.prototype.addSemanticPlace = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 22, opt_value, proto.location.unified.SemanticPlace, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.location.unified.LocationDescriptor} returns this
 */
proto.location.unified.LocationDescriptor.prototype.clearSemanticPlaceList = function() {
  return this.setSemanticPlaceList([]);
};


/**
 * repeated VisibleNetwork visible_network = 23;
 * @return {!Array<!proto.location.unified.VisibleNetwork>}
 */
proto.location.unified.LocationDescriptor.prototype.getVisibleNetworkList = function() {
  return /** @type{!Array<!proto.location.unified.VisibleNetwork>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.location.unified.VisibleNetwork, 23));
};


/**
 * @param {Array<!proto.location.unified.VisibleNetwork>|undefined} value
 * @return {!proto.location.unified.LocationDescriptor} returns this
*/
proto.location.unified.LocationDescriptor.prototype.setVisibleNetworkList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 23, value);
};


/**
 * @param {!proto.location.unified.VisibleNetwork=} opt_value
 * @param {number=} opt_index
 * @return {!proto.location.unified.VisibleNetwork}
 */
proto.location.unified.LocationDescriptor.prototype.addVisibleNetwork = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 23, opt_value, proto.location.unified.VisibleNetwork, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.location.unified.LocationDescriptor} returns this
 */
proto.location.unified.LocationDescriptor.prototype.clearVisibleNetworkList = function() {
  return this.setVisibleNetworkList([]);
};


/**
 * repeated PresenceInterval presence_interval = 24;
 * @return {!Array<!proto.location.unified.PresenceInterval>}
 */
proto.location.unified.LocationDescriptor.prototype.getPresenceIntervalList = function() {
  return /** @type{!Array<!proto.location.unified.PresenceInterval>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.location.unified.PresenceInterval, 24));
};


/**
 * @param {Array<!proto.location.unified.PresenceInterval>|undefined} value
 * @return {!proto.location.unified.LocationDescriptor} returns this
*/
proto.location.unified.LocationDescriptor.prototype.setPresenceIntervalList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 24, value);
};


/**
 * @param {!proto.location.unified.PresenceInterval=} opt_value
 * @param {number=} opt_index
 * @return {!proto.location.unified.PresenceInterval}
 */
proto.location.unified.LocationDescriptor.prototype.addPresenceInterval = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 24, opt_value, proto.location.unified.PresenceInterval, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.location.unified.LocationDescriptor} returns this
 */
proto.location.unified.LocationDescriptor.prototype.clearPresenceIntervalList = function() {
  return this.setPresenceIntervalList([]);
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.location.unified.LocationDescriptor}
 */
proto.location.unified.LocationDescriptor.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.location.unified.LocationDescriptor, data);
};



//speech/common/proto/waveform_proto-lib_jspb/waveheader.jspb.js
// source: speech/common/proto/waveform.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.WaveHeader');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.WaveHeader = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.WaveHeader.repeatedFields_, null);
};
goog.inherits(proto.speech.WaveHeader, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.WaveHeader.displayName = 'proto.speech.WaveHeader';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.WaveHeader.repeatedFields_ = [6];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.WaveHeader.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.WaveHeader.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.WaveHeader} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.WaveHeader.toObject = function(includeInstance, msg) {
  var f, obj = {
    sampleRate: (f = jspb.Message.getOptionalFloatingPointField(msg, 1)) == null ? undefined : f,
    bitRate: jspb.Message.getFloatingPointFieldWithDefault(msg, 14, 0.0),
    atomicType: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    atomicSize: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,
    sampleType: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f,
    rank: (f = jspb.Message.getField(msg, 5)) == null ? undefined : f,
    dimensionList: (f = jspb.Message.getRepeatedField(msg, 6)) == null ? undefined : f,
    elementsPerSample: (f = jspb.Message.getField(msg, 13)) == null ? undefined : f,
    totalSamples: (f = jspb.Message.getField(msg, 7)) == null ? undefined : f,
    byteOrder: (f = jspb.Message.getField(msg, 8)) == null ? undefined : f,
    sampleSize: (f = jspb.Message.getField(msg, 9)) == null ? undefined : f,
    details: (f = jspb.Message.getField(msg, 10)) == null ? undefined : f,
    sampleCoding: (f = jspb.Message.getField(msg, 11)) == null ? undefined : f,
    startTime: jspb.Message.getFloatingPointFieldWithDefault(msg, 12, 0.0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of WaveHeader as accepted by the `fromObject` method.
 * @record
 */
proto.speech.WaveHeader.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.sampleRate;

  /** @type {?number|undefined} */
  this.bitRate;

  /** @type {?string|undefined} */
  this.atomicType;

  /** @type {?number|undefined} */
  this.atomicSize;

  /** @type {?string|undefined} */
  this.sampleType;

  /** @type {?number|undefined} */
  this.rank;

  /** @type {?Array<number>|undefined} */
  this.dimensionList;

  /** @type {?number|undefined} */
  this.elementsPerSample;

  /** @type {?number|undefined} */
  this.totalSamples;

  /** @type {?string|undefined} */
  this.byteOrder;

  /** @type {?number|undefined} */
  this.sampleSize;

  /** @type {?string|undefined} */
  this.details;

  /** @type {?string|undefined} */
  this.sampleCoding;

  /** @type {?number|undefined} */
  this.startTime;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.WaveHeader.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.WaveHeader}
 */
proto.speech.WaveHeader.fromObject = function(obj) {
  var msg = new proto.speech.WaveHeader();
  obj.sampleRate != null && jspb.Message.setField(msg, 1, obj.sampleRate);
  obj.bitRate != null && jspb.Message.setField(msg, 14, obj.bitRate);
  obj.atomicType != null && jspb.Message.setField(msg, 2, obj.atomicType);
  obj.atomicSize != null && jspb.Message.setField(msg, 3, obj.atomicSize);
  obj.sampleType != null && jspb.Message.setField(msg, 4, obj.sampleType);
  obj.rank != null && jspb.Message.setField(msg, 5, obj.rank);
  obj.dimensionList != null && jspb.Message.setField(msg, 6, obj.dimensionList);
  obj.elementsPerSample != null && jspb.Message.setField(msg, 13, obj.elementsPerSample);
  obj.totalSamples != null && jspb.Message.setField(msg, 7, obj.totalSamples);
  obj.byteOrder != null && jspb.Message.setField(msg, 8, obj.byteOrder);
  obj.sampleSize != null && jspb.Message.setField(msg, 9, obj.sampleSize);
  obj.details != null && jspb.Message.setField(msg, 10, obj.details);
  obj.sampleCoding != null && jspb.Message.setField(msg, 11, obj.sampleCoding);
  obj.startTime != null && jspb.Message.setField(msg, 12, obj.startTime);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.WaveHeader}
 */
proto.speech.WaveHeader.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.WaveHeader;
  return proto.speech.WaveHeader.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.WaveHeader} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.WaveHeader}
 */
proto.speech.WaveHeader.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setSampleRate(value);
      break;
    case 14:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setBitRate(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setAtomicType(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setAtomicSize(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setSampleType(value);
      break;
    case 5:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setRank(value);
      break;
    case 6:
      var value = /** @type {number} */ (reader.readInt32());
      msg.addDimension(value);
      break;
    case 13:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setElementsPerSample(value);
      break;
    case 7:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setTotalSamples(value);
      break;
    case 8:
      var value = /** @type {string} */ (reader.readString());
      msg.setByteOrder(value);
      break;
    case 9:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setSampleSize(value);
      break;
    case 10:
      var value = /** @type {string} */ (reader.readString());
      msg.setDetails(value);
      break;
    case 11:
      var value = /** @type {string} */ (reader.readString());
      msg.setSampleCoding(value);
      break;
    case 12:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setStartTime(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.WaveHeader.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.WaveHeader.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.WaveHeader} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.WaveHeader.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {number} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeFloat(
      1,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 14));
  if (f != null) {
    writer.writeFloat(
      14,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeInt32(
      3,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeString(
      4,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeInt32(
      5,
      f
    );
  }
  f = message.getDimensionList();
  if (f.length > 0) {
    writer.writeRepeatedInt32(
      6,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 13));
  if (f != null) {
    writer.writeInt32(
      13,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 7));
  if (f != null) {
    writer.writeInt64(
      7,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 8));
  if (f != null) {
    writer.writeString(
      8,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 9));
  if (f != null) {
    writer.writeInt32(
      9,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 10));
  if (f != null) {
    writer.writeString(
      10,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 11));
  if (f != null) {
    writer.writeString(
      11,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 12));
  if (f != null) {
    writer.writeFloat(
      12,
      f
    );
  }
};


/**
 * required float sample_rate = 1;
 * @return {number}
 */
proto.speech.WaveHeader.prototype.getSampleRate = function() {
  return /** @type {number} */ (jspb.Message.getOptionalFloatingPointField(this, 1));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.WaveHeader} returns this
 */
proto.speech.WaveHeader.prototype.setSampleRate = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.WaveHeader} returns this
 */
proto.speech.WaveHeader.prototype.clearSampleRate = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.WaveHeader.prototype.hasSampleRate = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional float bit_rate = 14;
 * @return {number}
 */
proto.speech.WaveHeader.prototype.getBitRate = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 14, 0.0));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.WaveHeader} returns this
 */
proto.speech.WaveHeader.prototype.setBitRate = function(value) {
  return jspb.Message.setField(this, 14, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.WaveHeader} returns this
 */
proto.speech.WaveHeader.prototype.clearBitRate = function() {
  return jspb.Message.setField(this, 14, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.WaveHeader.prototype.hasBitRate = function() {
  return jspb.Message.getField(this, 14) != null;
};


/**
 * required string atomic_type = 2;
 * @return {string}
 */
proto.speech.WaveHeader.prototype.getAtomicType = function() {
  return /** @type {string} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.WaveHeader} returns this
 */
proto.speech.WaveHeader.prototype.setAtomicType = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.WaveHeader} returns this
 */
proto.speech.WaveHeader.prototype.clearAtomicType = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.WaveHeader.prototype.hasAtomicType = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * required int32 atomic_size = 3;
 * @return {number}
 */
proto.speech.WaveHeader.prototype.getAtomicSize = function() {
  return /** @type {number} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.WaveHeader} returns this
 */
proto.speech.WaveHeader.prototype.setAtomicSize = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.WaveHeader} returns this
 */
proto.speech.WaveHeader.prototype.clearAtomicSize = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.WaveHeader.prototype.hasAtomicSize = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * required string sample_type = 4;
 * @return {string}
 */
proto.speech.WaveHeader.prototype.getSampleType = function() {
  return /** @type {string} */ (jspb.Message.getField(this, 4));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.WaveHeader} returns this
 */
proto.speech.WaveHeader.prototype.setSampleType = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.WaveHeader} returns this
 */
proto.speech.WaveHeader.prototype.clearSampleType = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.WaveHeader.prototype.hasSampleType = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * required int32 rank = 5;
 * @return {number}
 */
proto.speech.WaveHeader.prototype.getRank = function() {
  return /** @type {number} */ (jspb.Message.getField(this, 5));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.WaveHeader} returns this
 */
proto.speech.WaveHeader.prototype.setRank = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.WaveHeader} returns this
 */
proto.speech.WaveHeader.prototype.clearRank = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.WaveHeader.prototype.hasRank = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * repeated int32 dimension = 6;
 * @return {!Array<number>}
 */
proto.speech.WaveHeader.prototype.getDimensionList = function() {
  return /** @type {!Array<number>} */ (jspb.Message.getRepeatedField(this, 6));
};


/**
 * @param {Array<number>|undefined} value
 * @return {!proto.speech.WaveHeader} returns this
 */
proto.speech.WaveHeader.prototype.setDimensionList = function(value) {
  return jspb.Message.setField(this, 6, value || []);
};


/**
 * @param {number} value
 * @param {number=} opt_index
 * @return {!proto.speech.WaveHeader} returns this
 */
proto.speech.WaveHeader.prototype.addDimension = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 6, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.WaveHeader} returns this
 */
proto.speech.WaveHeader.prototype.clearDimensionList = function() {
  return this.setDimensionList([]);
};


/**
 * required int32 elements_per_sample = 13;
 * @return {number}
 */
proto.speech.WaveHeader.prototype.getElementsPerSample = function() {
  return /** @type {number} */ (jspb.Message.getField(this, 13));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.WaveHeader} returns this
 */
proto.speech.WaveHeader.prototype.setElementsPerSample = function(value) {
  return jspb.Message.setField(this, 13, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.WaveHeader} returns this
 */
proto.speech.WaveHeader.prototype.clearElementsPerSample = function() {
  return jspb.Message.setField(this, 13, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.WaveHeader.prototype.hasElementsPerSample = function() {
  return jspb.Message.getField(this, 13) != null;
};


/**
 * required int64 total_samples = 7;
 * @return {number}
 */
proto.speech.WaveHeader.prototype.getTotalSamples = function() {
  return /** @type {number} */ (jspb.Message.getField(this, 7));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.WaveHeader} returns this
 */
proto.speech.WaveHeader.prototype.setTotalSamples = function(value) {
  return jspb.Message.setField(this, 7, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.WaveHeader} returns this
 */
proto.speech.WaveHeader.prototype.clearTotalSamples = function() {
  return jspb.Message.setField(this, 7, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.WaveHeader.prototype.hasTotalSamples = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * required string byte_order = 8;
 * @return {string}
 */
proto.speech.WaveHeader.prototype.getByteOrder = function() {
  return /** @type {string} */ (jspb.Message.getField(this, 8));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.WaveHeader} returns this
 */
proto.speech.WaveHeader.prototype.setByteOrder = function(value) {
  return jspb.Message.setField(this, 8, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.WaveHeader} returns this
 */
proto.speech.WaveHeader.prototype.clearByteOrder = function() {
  return jspb.Message.setField(this, 8, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.WaveHeader.prototype.hasByteOrder = function() {
  return jspb.Message.getField(this, 8) != null;
};


/**
 * required int32 sample_size = 9;
 * @return {number}
 */
proto.speech.WaveHeader.prototype.getSampleSize = function() {
  return /** @type {number} */ (jspb.Message.getField(this, 9));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.WaveHeader} returns this
 */
proto.speech.WaveHeader.prototype.setSampleSize = function(value) {
  return jspb.Message.setField(this, 9, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.WaveHeader} returns this
 */
proto.speech.WaveHeader.prototype.clearSampleSize = function() {
  return jspb.Message.setField(this, 9, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.WaveHeader.prototype.hasSampleSize = function() {
  return jspb.Message.getField(this, 9) != null;
};


/**
 * optional string details = 10;
 * @return {?string}
 */
proto.speech.WaveHeader.prototype.getDetails = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 10));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.WaveHeader} returns this
 */
proto.speech.WaveHeader.prototype.setDetails = function(value) {
  return jspb.Message.setField(this, 10, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.WaveHeader} returns this
 */
proto.speech.WaveHeader.prototype.clearDetails = function() {
  return jspb.Message.setField(this, 10, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.WaveHeader.prototype.hasDetails = function() {
  return jspb.Message.getField(this, 10) != null;
};


/**
 * required string sample_coding = 11;
 * @return {string}
 */
proto.speech.WaveHeader.prototype.getSampleCoding = function() {
  return /** @type {string} */ (jspb.Message.getField(this, 11));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.WaveHeader} returns this
 */
proto.speech.WaveHeader.prototype.setSampleCoding = function(value) {
  return jspb.Message.setField(this, 11, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.WaveHeader} returns this
 */
proto.speech.WaveHeader.prototype.clearSampleCoding = function() {
  return jspb.Message.setField(this, 11, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.WaveHeader.prototype.hasSampleCoding = function() {
  return jspb.Message.getField(this, 11) != null;
};


/**
 * optional float start_time = 12;
 * @return {number}
 */
proto.speech.WaveHeader.prototype.getStartTime = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 12, 0.0));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.WaveHeader} returns this
 */
proto.speech.WaveHeader.prototype.setStartTime = function(value) {
  return jspb.Message.setField(this, 12, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.WaveHeader} returns this
 */
proto.speech.WaveHeader.prototype.clearStartTime = function() {
  return jspb.Message.setField(this, 12, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.WaveHeader.prototype.hasStartTime = function() {
  return jspb.Message.getField(this, 12) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.WaveHeader}
 */
proto.speech.WaveHeader.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.WaveHeader, data);
};



//speech/common/proto/waveform_proto-lib_jspb/waveformdata.jspb.js
// source: speech/common/proto/waveform.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.WaveformData');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.WaveHeader');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.WaveformData = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.WaveformData, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.WaveformData.displayName = 'proto.speech.WaveformData';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.WaveformData.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.WaveformData.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.WaveformData} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.WaveformData.toObject = function(includeInstance, msg) {
  var f, obj = {
    header: (f = msg.getHeader()) && proto.speech.WaveHeader.toObject(includeInstance, f),
    data: msg.getData_asB64(),
    version: jspb.Message.getFieldWithDefault(msg, 1, 20070716)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of WaveformData as accepted by the `fromObject` method.
 * @record
 */
proto.speech.WaveformData.ObjectFormat = function() {
  /** @type {?proto.speech.WaveHeader.ObjectFormat|undefined} */
  this.header;

  /** @type {?string|undefined} */
  this.data;

  /** @type {?number|undefined} */
  this.version;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.WaveformData.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.WaveformData}
 */
proto.speech.WaveformData.fromObject = function(obj) {
  var msg = new proto.speech.WaveformData();
  obj.header && jspb.Message.setWrapperField(
      msg, 2, proto.speech.WaveHeader.fromObject(obj.header));
  obj.data != null && jspb.Message.setField(msg, 3, obj.data);
  obj.version != null && jspb.Message.setField(msg, 1, obj.version);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.WaveformData}
 */
proto.speech.WaveformData.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.WaveformData;
  return proto.speech.WaveformData.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.WaveformData} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.WaveformData}
 */
proto.speech.WaveformData.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 2:
      var value = new proto.speech.WaveHeader;
      reader.readMessage(value,proto.speech.WaveHeader.deserializeBinaryFromReader);
      msg.setHeader(value);
      break;
    case 3:
      var value = /** @type {!Uint8Array} */ (reader.readBytes());
      msg.setData(value);
      break;
    case 1:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setVersion(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.WaveformData.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.WaveformData.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.WaveformData} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.WaveformData.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getHeader();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.speech.WaveHeader.serializeBinaryToWriter
    );
  }
  f = /** @type {!(string|Uint8Array)} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeBytes(
      3,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeInt64(
      1,
      f
    );
  }
};


/**
 * required WaveHeader header = 2;
 * @return {!proto.speech.WaveHeader}
 */
proto.speech.WaveformData.prototype.getHeader = function() {
  return /** @type{!proto.speech.WaveHeader} */ (
    jspb.Message.getWrapperField(this, proto.speech.WaveHeader, 2, 1));
};


/**
 * @param {?proto.speech.WaveHeader|undefined} value
 * @return {!proto.speech.WaveformData} returns this
*/
proto.speech.WaveformData.prototype.setHeader = function(value) {
  return jspb.Message.setWrapperField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.WaveformData} returns this
 */
proto.speech.WaveformData.prototype.clearHeader = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.WaveformData.prototype.hasHeader = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * required bytes data = 3;
 * @return {string}
 */
proto.speech.WaveformData.prototype.getData = function() {
  return /** @type {string} */ (jspb.Message.getField(this, 3));
};


/**
 * required bytes data = 3;
 * This is a type-conversion wrapper around `getData()`
 * @return {string}
 */
proto.speech.WaveformData.prototype.getData_asB64 = function() {
  return /** @type {string} */ (jspb.Message.bytesAsB64(
      this.getData()));
};


/**
 * required bytes data = 3;
 * Note that Uint8Array is not supported on all browsers.
 * @see http://caniuse.com/Uint8Array
 * This is a type-conversion wrapper around `getData()`
 * @return {!Uint8Array}
 */
proto.speech.WaveformData.prototype.getData_asU8 = function() {
  return /** @type {!Uint8Array} */ (jspb.Message.bytesAsU8(
      this.getData()));
};


/**
 * @param {?(string|Uint8Array)|undefined} value
 * @return {!proto.speech.WaveformData} returns this
 */
proto.speech.WaveformData.prototype.setData = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.WaveformData} returns this
 */
proto.speech.WaveformData.prototype.clearData = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.WaveformData.prototype.hasData = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional int64 version = 1;
 * @return {number}
 */
proto.speech.WaveformData.prototype.getVersion = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 1, 20070716));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.WaveformData} returns this
 */
proto.speech.WaveformData.prototype.setVersion = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.WaveformData} returns this
 */
proto.speech.WaveformData.prototype.clearVersion = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.WaveformData.prototype.hasVersion = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.WaveformData}
 */
proto.speech.WaveformData.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.WaveformData, data);
};



//speech/greco3/core/loggable_proto_jspb/loggable.jspb.js
// source: speech/greco3/core/loggable.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.greco.Loggable');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.greco.Loggable = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 1, null, null);
};
goog.inherits(proto.greco.Loggable, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.greco.Loggable.displayName = 'proto.greco.Loggable';
}

/**
 * The extensions registered with this message class. This is a map of
 * extension field number to fieldInfo object.
 *
 * For example:
 *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }
 *
 * fieldName contains the JsCompiler renamed field name property so that it
 * works in OPTIMIZED mode.
 *
 * @type {!Object<number, jspb.ExtensionFieldInfo>}
 */
proto.greco.Loggable.extensions = {};


/**
 * The extensions registered with this message class. This is a map of
 * extension field number to fieldInfo object.
 *
 * For example:
 *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }
 *
 * fieldName contains the JsCompiler renamed field name property so that it
 * works in OPTIMIZED mode.
 *
 * @type {!Object<number, jspb.ExtensionFieldBinaryInfo>}
 */
proto.greco.Loggable.extensionsBinary = {};




if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.greco.Loggable.prototype.toObject = function(opt_includeInstance) {
  return proto.greco.Loggable.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.greco.Loggable} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.greco.Loggable.toObject = function(includeInstance, msg) {
  var f, obj = {

  };

  jspb.Message.toObjectExtension(/** @type {!jspb.Message} */ (msg), obj,
      proto.greco.Loggable.extensions, proto.greco.Loggable.prototype.getExtension,
      includeInstance);
  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Loggable as accepted by the `fromObject` method.
 * @record
 */
proto.greco.Loggable.ObjectFormat = function() {
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.greco.Loggable.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.greco.Loggable}
 */
proto.greco.Loggable.fromObject = function(obj) {
  var msg = new proto.greco.Loggable();
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.greco.Loggable}
 */
proto.greco.Loggable.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.greco.Loggable;
  return proto.greco.Loggable.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.greco.Loggable} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.greco.Loggable}
 */
proto.greco.Loggable.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    default:
      jspb.Message.readBinaryExtension(msg, reader,
        proto.greco.Loggable.extensionsBinary,
        proto.greco.Loggable.prototype.getExtension,
        proto.greco.Loggable.prototype.setExtension);
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.greco.Loggable.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.greco.Loggable.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.greco.Loggable} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.greco.Loggable.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  jspb.Message.serializeBinaryExtensions(message, writer,
    proto.greco.Loggable.extensionsBinary, proto.greco.Loggable.prototype.getExtension);
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.greco.Loggable}
 */
proto.greco.Loggable.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.greco.Loggable, data);
};



//speech/greco3/core/params_proto_jspb/params.jspb.js
// source: speech/greco3/core/params.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.greco.Params');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.ExtensionFieldBinaryInfo');
goog.require('jspb.ExtensionFieldInfo');
goog.require('jspb.Message');
goog.require('proto.greco.Loggable');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.greco.Params = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 1, null, null);
};
goog.inherits(proto.greco.Params, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.greco.Params.displayName = 'proto.greco.Params';
}

/**
 * The extensions registered with this message class. This is a map of
 * extension field number to fieldInfo object.
 *
 * For example:
 *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }
 *
 * fieldName contains the JsCompiler renamed field name property so that it
 * works in OPTIMIZED mode.
 *
 * @type {!Object<number, jspb.ExtensionFieldInfo>}
 */
proto.greco.Params.extensions = {};


/**
 * The extensions registered with this message class. This is a map of
 * extension field number to fieldInfo object.
 *
 * For example:
 *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }
 *
 * fieldName contains the JsCompiler renamed field name property so that it
 * works in OPTIMIZED mode.
 *
 * @type {!Object<number, jspb.ExtensionFieldBinaryInfo>}
 */
proto.greco.Params.extensionsBinary = {};




if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.greco.Params.prototype.toObject = function(opt_includeInstance) {
  return proto.greco.Params.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.greco.Params} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.greco.Params.toObject = function(includeInstance, msg) {
  var f, obj = {

  };

  jspb.Message.toObjectExtension(/** @type {!jspb.Message} */ (msg), obj,
      proto.greco.Params.extensions, proto.greco.Params.prototype.getExtension,
      includeInstance);
  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Params as accepted by the `fromObject` method.
 * @record
 */
proto.greco.Params.ObjectFormat = function() {
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.greco.Params.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.greco.Params}
 */
proto.greco.Params.fromObject = function(obj) {
  var msg = new proto.greco.Params();
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.greco.Params}
 */
proto.greco.Params.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.greco.Params;
  return proto.greco.Params.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.greco.Params} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.greco.Params}
 */
proto.greco.Params.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    default:
      jspb.Message.readBinaryExtension(msg, reader,
        proto.greco.Params.extensionsBinary,
        proto.greco.Params.prototype.getExtension,
        proto.greco.Params.prototype.setExtension);
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.greco.Params.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.greco.Params.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.greco.Params} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.greco.Params.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  jspb.Message.serializeBinaryExtensions(message, writer,
    proto.greco.Params.extensionsBinary, proto.greco.Params.prototype.getExtension);
};



/**
 * A tuple of {field number, class constructor} for the extension
 * field named `logId`.
 * @type {!jspb.ExtensionFieldInfo<!proto.greco.Params>}
 */
proto.greco.Params.logId = new jspb.ExtensionFieldInfo(
    29829655,
    {logId: 0},
    proto.greco.Params,
     /** @type {?function((boolean|undefined),!jspb.Message=): !Object} */ (
         proto.greco.Params.toObject),
    0);

proto.greco.Loggable.extensionsBinary[29829655] = new jspb.ExtensionFieldBinaryInfo(
    proto.greco.Params.logId,
    jspb.BinaryReader.prototype.readMessage,
    jspb.BinaryWriter.prototype.writeMessage,
    proto.greco.Params.serializeBinaryToWriter,
    proto.greco.Params.deserializeBinaryFromReader,
    false);
// This registers the extension field with the extended class, so that
// toObject() will function correctly.
proto.greco.Loggable.extensions[29829655] = proto.greco.Params.logId;

/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.greco.Params}
 */
proto.greco.Params.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.greco.Params, data);
};



//speech/tts/engine/utterance/audio_buffer_proto_jspb/audiobuffer.jspb.js
// source: speech/tts/engine/utterance/audio_buffer.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.AudioBuffer');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.AudioBuffer = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.AudioBuffer, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.AudioBuffer.displayName = 'proto.speech.tts.AudioBuffer';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.AudioBuffer.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.AudioBuffer.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.AudioBuffer} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.AudioBuffer.toObject = function(includeInstance, msg) {
  var f, obj = {
    samples: msg.getSamples_asB64(),
    sampleRate: jspb.Message.getFieldWithDefault(msg, 2, 16000)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of AudioBuffer as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.AudioBuffer.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.samples;

  /** @type {?number|undefined} */
  this.sampleRate;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.AudioBuffer.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.AudioBuffer}
 */
proto.speech.tts.AudioBuffer.fromObject = function(obj) {
  var msg = new proto.speech.tts.AudioBuffer();
  obj.samples != null && jspb.Message.setField(msg, 1, obj.samples);
  obj.sampleRate != null && jspb.Message.setField(msg, 2, obj.sampleRate);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.AudioBuffer}
 */
proto.speech.tts.AudioBuffer.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.AudioBuffer;
  return proto.speech.tts.AudioBuffer.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.AudioBuffer} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.AudioBuffer}
 */
proto.speech.tts.AudioBuffer.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {!Uint8Array} */ (reader.readBytes());
      msg.setSamples(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setSampleRate(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.AudioBuffer.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.AudioBuffer.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.AudioBuffer} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.AudioBuffer.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?(string|Uint8Array)} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeBytes(
      1,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeInt32(
      2,
      f
    );
  }
};


/**
 * optional bytes samples = 1;
 * @return {?string}
 */
proto.speech.tts.AudioBuffer.prototype.getSamples = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 1));
};


/**
 * optional bytes samples = 1;
 * This is a type-conversion wrapper around `getSamples()`
 * @return {?string}
 */
proto.speech.tts.AudioBuffer.prototype.getSamples_asB64 = function() {
  return /** @type {?string} */ (jspb.Message.bytesAsB64(
      this.getSamples()));
};


/**
 * optional bytes samples = 1;
 * Note that Uint8Array is not supported on all browsers.
 * @see http://caniuse.com/Uint8Array
 * This is a type-conversion wrapper around `getSamples()`
 * @return {?Uint8Array}
 */
proto.speech.tts.AudioBuffer.prototype.getSamples_asU8 = function() {
  return /** @type {?Uint8Array} */ (jspb.Message.bytesAsU8(
      this.getSamples()));
};


/**
 * @param {?(string|Uint8Array)|undefined} value
 * @return {!proto.speech.tts.AudioBuffer} returns this
 */
proto.speech.tts.AudioBuffer.prototype.setSamples = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.AudioBuffer} returns this
 */
proto.speech.tts.AudioBuffer.prototype.clearSamples = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.AudioBuffer.prototype.hasSamples = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional int32 sample_rate = 2;
 * @return {number}
 */
proto.speech.tts.AudioBuffer.prototype.getSampleRate = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 2, 16000));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.AudioBuffer} returns this
 */
proto.speech.tts.AudioBuffer.prototype.setSampleRate = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.AudioBuffer} returns this
 */
proto.speech.tts.AudioBuffer.prototype.clearSampleRate = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.AudioBuffer.prototype.hasSampleRate = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.AudioBuffer}
 */
proto.speech.tts.AudioBuffer.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.AudioBuffer, data);
};



//speech/tts/engine/pipeline/resources/voice_params_proto_jspb/speakerparams.jspb.js
// source: speech/tts/engine/pipeline/resources/voice_params.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.SpeakerParams');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

goog.forwardDeclare('proto.speech.Voice.LaunchStatus');
goog.forwardDeclare('proto.speech.Voice.Usage');
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.SpeakerParams = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.SpeakerParams.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.SpeakerParams, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.SpeakerParams.displayName = 'proto.speech.tts.SpeakerParams';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.SpeakerParams.repeatedFields_ = [9];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.SpeakerParams.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.SpeakerParams.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.SpeakerParams} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.SpeakerParams.toObject = function(includeInstance, msg) {
  var f, obj = {
    speaker: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    gender: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    wavenetSynthesizerRef: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,
    lstmDurModelRef: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f,
    lstmSpeechModelRef: (f = jspb.Message.getField(msg, 5)) == null ? undefined : f,
    addSpecificationRef: (f = jspb.Message.getField(msg, 6)) == null ? undefined : f,
    outputGain: jspb.Message.getFloatingPointFieldWithDefault(msg, 7, 1.0),
    voiceName: (f = jspb.Message.getField(msg, 8)) == null ? undefined : f,
    aliasList: (f = jspb.Message.getRepeatedField(msg, 9)) == null ? undefined : f,
    usage: (f = jspb.Message.getField(msg, 10)) == null ? undefined : f,
    launchStatus: (f = jspb.Message.getField(msg, 11)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of SpeakerParams as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.SpeakerParams.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.speaker;

  /** @type {?string|undefined} */
  this.gender;

  /** @type {?string|undefined} */
  this.wavenetSynthesizerRef;

  /** @type {?string|undefined} */
  this.lstmDurModelRef;

  /** @type {?string|undefined} */
  this.lstmSpeechModelRef;

  /** @type {?string|undefined} */
  this.addSpecificationRef;

  /** @type {?number|undefined} */
  this.outputGain;

  /** @type {?string|undefined} */
  this.voiceName;

  /** @type {?Array<string>|undefined} */
  this.aliasList;

  /** @type {?number|undefined} */
  this.usage;

  /** @type {?number|undefined} */
  this.launchStatus;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.SpeakerParams.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.SpeakerParams}
 */
proto.speech.tts.SpeakerParams.fromObject = function(obj) {
  var msg = new proto.speech.tts.SpeakerParams();
  obj.speaker != null && jspb.Message.setField(msg, 1, obj.speaker);
  obj.gender != null && jspb.Message.setField(msg, 2, obj.gender);
  obj.wavenetSynthesizerRef != null && jspb.Message.setField(msg, 3, obj.wavenetSynthesizerRef);
  obj.lstmDurModelRef != null && jspb.Message.setField(msg, 4, obj.lstmDurModelRef);
  obj.lstmSpeechModelRef != null && jspb.Message.setField(msg, 5, obj.lstmSpeechModelRef);
  obj.addSpecificationRef != null && jspb.Message.setField(msg, 6, obj.addSpecificationRef);
  obj.outputGain != null && jspb.Message.setField(msg, 7, obj.outputGain);
  obj.voiceName != null && jspb.Message.setField(msg, 8, obj.voiceName);
  obj.aliasList != null && jspb.Message.setField(msg, 9, obj.aliasList);
  obj.usage != null && jspb.Message.setField(msg, 10, obj.usage);
  obj.launchStatus != null && jspb.Message.setField(msg, 11, obj.launchStatus);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.SpeakerParams}
 */
proto.speech.tts.SpeakerParams.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.SpeakerParams;
  return proto.speech.tts.SpeakerParams.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.SpeakerParams} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.SpeakerParams}
 */
proto.speech.tts.SpeakerParams.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setSpeaker(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setGender(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setWavenetSynthesizerRef(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setLstmDurModelRef(value);
      break;
    case 5:
      var value = /** @type {string} */ (reader.readString());
      msg.setLstmSpeechModelRef(value);
      break;
    case 6:
      var value = /** @type {string} */ (reader.readString());
      msg.setAddSpecificationRef(value);
      break;
    case 7:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setOutputGain(value);
      break;
    case 8:
      var value = /** @type {string} */ (reader.readString());
      msg.setVoiceName(value);
      break;
    case 9:
      var value = /** @type {string} */ (reader.readString());
      msg.addAlias(value);
      break;
    case 10:
      var value = /** @type {!proto.speech.Voice.Usage} */ (reader.readEnum());
      msg.setUsage(value);
      break;
    case 11:
      var value = /** @type {!proto.speech.Voice.LaunchStatus} */ (reader.readEnum());
      msg.setLaunchStatus(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.SpeakerParams.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.SpeakerParams.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.SpeakerParams} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.SpeakerParams.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeString(
      3,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeString(
      4,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeString(
      5,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeString(
      6,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 7));
  if (f != null) {
    writer.writeFloat(
      7,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 8));
  if (f != null) {
    writer.writeString(
      8,
      f
    );
  }
  f = message.getAliasList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      9,
      f
    );
  }
  f = /** @type {!proto.speech.Voice.Usage} */ (jspb.Message.getField(message, 10));
  if (f != null) {
    writer.writeEnum(
      10,
      f
    );
  }
  f = /** @type {!proto.speech.Voice.LaunchStatus} */ (jspb.Message.getField(message, 11));
  if (f != null) {
    writer.writeEnum(
      11,
      f
    );
  }
};


/**
 * optional string speaker = 1;
 * @return {?string}
 */
proto.speech.tts.SpeakerParams.prototype.getSpeaker = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.SpeakerParams} returns this
 */
proto.speech.tts.SpeakerParams.prototype.setSpeaker = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.SpeakerParams} returns this
 */
proto.speech.tts.SpeakerParams.prototype.clearSpeaker = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.SpeakerParams.prototype.hasSpeaker = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional string gender = 2;
 * @return {?string}
 */
proto.speech.tts.SpeakerParams.prototype.getGender = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.SpeakerParams} returns this
 */
proto.speech.tts.SpeakerParams.prototype.setGender = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.SpeakerParams} returns this
 */
proto.speech.tts.SpeakerParams.prototype.clearGender = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.SpeakerParams.prototype.hasGender = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional string wavenet_synthesizer_ref = 3;
 * @return {?string}
 */
proto.speech.tts.SpeakerParams.prototype.getWavenetSynthesizerRef = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.SpeakerParams} returns this
 */
proto.speech.tts.SpeakerParams.prototype.setWavenetSynthesizerRef = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.SpeakerParams} returns this
 */
proto.speech.tts.SpeakerParams.prototype.clearWavenetSynthesizerRef = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.SpeakerParams.prototype.hasWavenetSynthesizerRef = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional string lstm_dur_model_ref = 4;
 * @return {?string}
 */
proto.speech.tts.SpeakerParams.prototype.getLstmDurModelRef = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 4));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.SpeakerParams} returns this
 */
proto.speech.tts.SpeakerParams.prototype.setLstmDurModelRef = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.SpeakerParams} returns this
 */
proto.speech.tts.SpeakerParams.prototype.clearLstmDurModelRef = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.SpeakerParams.prototype.hasLstmDurModelRef = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional string lstm_speech_model_ref = 5;
 * @return {?string}
 */
proto.speech.tts.SpeakerParams.prototype.getLstmSpeechModelRef = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 5));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.SpeakerParams} returns this
 */
proto.speech.tts.SpeakerParams.prototype.setLstmSpeechModelRef = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.SpeakerParams} returns this
 */
proto.speech.tts.SpeakerParams.prototype.clearLstmSpeechModelRef = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.SpeakerParams.prototype.hasLstmSpeechModelRef = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional string add_specification_ref = 6;
 * @return {?string}
 */
proto.speech.tts.SpeakerParams.prototype.getAddSpecificationRef = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 6));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.SpeakerParams} returns this
 */
proto.speech.tts.SpeakerParams.prototype.setAddSpecificationRef = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.SpeakerParams} returns this
 */
proto.speech.tts.SpeakerParams.prototype.clearAddSpecificationRef = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.SpeakerParams.prototype.hasAddSpecificationRef = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional float output_gain = 7;
 * @return {number}
 */
proto.speech.tts.SpeakerParams.prototype.getOutputGain = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 7, 1.0));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.SpeakerParams} returns this
 */
proto.speech.tts.SpeakerParams.prototype.setOutputGain = function(value) {
  return jspb.Message.setField(this, 7, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.SpeakerParams} returns this
 */
proto.speech.tts.SpeakerParams.prototype.clearOutputGain = function() {
  return jspb.Message.setField(this, 7, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.SpeakerParams.prototype.hasOutputGain = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * optional string voice_name = 8;
 * @return {?string}
 */
proto.speech.tts.SpeakerParams.prototype.getVoiceName = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 8));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.SpeakerParams} returns this
 */
proto.speech.tts.SpeakerParams.prototype.setVoiceName = function(value) {
  return jspb.Message.setField(this, 8, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.SpeakerParams} returns this
 */
proto.speech.tts.SpeakerParams.prototype.clearVoiceName = function() {
  return jspb.Message.setField(this, 8, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.SpeakerParams.prototype.hasVoiceName = function() {
  return jspb.Message.getField(this, 8) != null;
};


/**
 * repeated string alias = 9;
 * @return {!Array<string>}
 */
proto.speech.tts.SpeakerParams.prototype.getAliasList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 9));
};


/**
 * @param {Array<string>|undefined} value
 * @return {!proto.speech.tts.SpeakerParams} returns this
 */
proto.speech.tts.SpeakerParams.prototype.setAliasList = function(value) {
  return jspb.Message.setField(this, 9, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.SpeakerParams} returns this
 */
proto.speech.tts.SpeakerParams.prototype.addAlias = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 9, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.SpeakerParams} returns this
 */
proto.speech.tts.SpeakerParams.prototype.clearAliasList = function() {
  return this.setAliasList([]);
};


/**
 * optional speech.Voice.Usage usage = 10;
 * @return {!proto.speech.Voice.Usage}
 */
proto.speech.tts.SpeakerParams.prototype.getUsage = function() {
  return /** @type {!proto.speech.Voice.Usage} */ (jspb.Message.getField(this, 10));
};


/**
 * @param {?proto.speech.Voice.Usage|undefined} value
 * @return {!proto.speech.tts.SpeakerParams} returns this
 */
proto.speech.tts.SpeakerParams.prototype.setUsage = function(value) {
  return jspb.Message.setField(this, 10, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.SpeakerParams} returns this
 */
proto.speech.tts.SpeakerParams.prototype.clearUsage = function() {
  return jspb.Message.setField(this, 10, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.SpeakerParams.prototype.hasUsage = function() {
  return jspb.Message.getField(this, 10) != null;
};


/**
 * optional speech.Voice.LaunchStatus launch_status = 11;
 * @return {!proto.speech.Voice.LaunchStatus}
 */
proto.speech.tts.SpeakerParams.prototype.getLaunchStatus = function() {
  return /** @type {!proto.speech.Voice.LaunchStatus} */ (jspb.Message.getField(this, 11));
};


/**
 * @param {?proto.speech.Voice.LaunchStatus|undefined} value
 * @return {!proto.speech.tts.SpeakerParams} returns this
 */
proto.speech.tts.SpeakerParams.prototype.setLaunchStatus = function(value) {
  return jspb.Message.setField(this, 11, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.SpeakerParams} returns this
 */
proto.speech.tts.SpeakerParams.prototype.clearLaunchStatus = function() {
  return jspb.Message.setField(this, 11, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.SpeakerParams.prototype.hasLaunchStatus = function() {
  return jspb.Message.getField(this, 11) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.SpeakerParams}
 */
proto.speech.tts.SpeakerParams.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.SpeakerParams, data);
};



//speech/tts/engine/pipeline/resources/voice_params_proto_jspb/voiceparams.jspb.js
// source: speech/tts/engine/pipeline/resources/voice_params.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.VoiceParams');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.ExtensionFieldBinaryInfo');
goog.require('jspb.ExtensionFieldInfo');
goog.require('jspb.Message');
goog.require('proto.greco.Params');
goog.require('proto.speech.tts.SpeakerParams');

goog.forwardDeclare('proto.speech.Voice.VoicePlatform');
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.VoiceParams = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.VoiceParams.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.VoiceParams, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.VoiceParams.displayName = 'proto.speech.tts.VoiceParams';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.VoiceParams.repeatedFields_ = [14];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.VoiceParams.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.VoiceParams.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.VoiceParams} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.VoiceParams.toObject = function(includeInstance, msg) {
  var f, obj = {
    description: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    language: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    region: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,
    speaker: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f,
    speakersList: jspb.Message.toObjectList(msg.getSpeakersList(),
    proto.speech.tts.SpeakerParams.toObject, includeInstance),
    gender: (f = jspb.Message.getField(msg, 5)) == null ? undefined : f,
    quality: (f = jspb.Message.getField(msg, 6)) == null ? undefined : f,
    sampleRate: (f = jspb.Message.getField(msg, 7)) == null ? undefined : f,
    serverVoiceName: (f = jspb.Message.getField(msg, 8)) == null ? undefined : f,
    voicePlatform: (f = jspb.Message.getField(msg, 13)) == null ? undefined : f,
    supportsSetSampleRate: (f = jspb.Message.getBooleanField(msg, 10)) == null ? undefined : f,
    multiSpeaker: (f = jspb.Message.getBooleanField(msg, 11)) == null ? undefined : f,
    textnormServerRequestServiceName: (f = jspb.Message.getField(msg, 12)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of VoiceParams as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.VoiceParams.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.description;

  /** @type {?string|undefined} */
  this.language;

  /** @type {?string|undefined} */
  this.region;

  /** @type {?string|undefined} */
  this.speaker;

  /** @type {?Array<!proto.speech.tts.SpeakerParams.ObjectFormat>|undefined} */
  this.speakersList;

  /** @type {?string|undefined} */
  this.gender;

  /** @type {?string|undefined} */
  this.quality;

  /** @type {?number|undefined} */
  this.sampleRate;

  /** @type {?string|undefined} */
  this.serverVoiceName;

  /** @type {?number|undefined} */
  this.voicePlatform;

  /** @type {?boolean|undefined} */
  this.supportsSetSampleRate;

  /** @type {?boolean|undefined} */
  this.multiSpeaker;

  /** @type {?string|undefined} */
  this.textnormServerRequestServiceName;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.VoiceParams.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.VoiceParams}
 */
proto.speech.tts.VoiceParams.fromObject = function(obj) {
  var msg = new proto.speech.tts.VoiceParams();
  obj.description != null && jspb.Message.setField(msg, 1, obj.description);
  obj.language != null && jspb.Message.setField(msg, 2, obj.language);
  obj.region != null && jspb.Message.setField(msg, 3, obj.region);
  obj.speaker != null && jspb.Message.setField(msg, 4, obj.speaker);
  obj.speakersList && jspb.Message.setRepeatedWrapperField(
      msg, 14, obj.speakersList.map(
          proto.speech.tts.SpeakerParams.fromObject));
  obj.gender != null && jspb.Message.setField(msg, 5, obj.gender);
  obj.quality != null && jspb.Message.setField(msg, 6, obj.quality);
  obj.sampleRate != null && jspb.Message.setField(msg, 7, obj.sampleRate);
  obj.serverVoiceName != null && jspb.Message.setField(msg, 8, obj.serverVoiceName);
  obj.voicePlatform != null && jspb.Message.setField(msg, 13, obj.voicePlatform);
  obj.supportsSetSampleRate != null && jspb.Message.setField(msg, 10, obj.supportsSetSampleRate);
  obj.multiSpeaker != null && jspb.Message.setField(msg, 11, obj.multiSpeaker);
  obj.textnormServerRequestServiceName != null && jspb.Message.setField(msg, 12, obj.textnormServerRequestServiceName);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.VoiceParams}
 */
proto.speech.tts.VoiceParams.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.VoiceParams;
  return proto.speech.tts.VoiceParams.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.VoiceParams} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.VoiceParams}
 */
proto.speech.tts.VoiceParams.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setDescription(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setLanguage(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setRegion(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setSpeaker(value);
      break;
    case 14:
      var value = new proto.speech.tts.SpeakerParams;
      reader.readMessage(value,proto.speech.tts.SpeakerParams.deserializeBinaryFromReader);
      msg.addSpeakers(value);
      break;
    case 5:
      var value = /** @type {string} */ (reader.readString());
      msg.setGender(value);
      break;
    case 6:
      var value = /** @type {string} */ (reader.readString());
      msg.setQuality(value);
      break;
    case 7:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setSampleRate(value);
      break;
    case 8:
      var value = /** @type {string} */ (reader.readString());
      msg.setServerVoiceName(value);
      break;
    case 13:
      var value = /** @type {!proto.speech.Voice.VoicePlatform} */ (reader.readEnum());
      msg.setVoicePlatform(value);
      break;
    case 10:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setSupportsSetSampleRate(value);
      break;
    case 11:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setMultiSpeaker(value);
      break;
    case 12:
      var value = /** @type {string} */ (reader.readString());
      msg.setTextnormServerRequestServiceName(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.VoiceParams.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.VoiceParams.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.VoiceParams} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.VoiceParams.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeString(
      3,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeString(
      4,
      f
    );
  }
  f = message.getSpeakersList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      14,
      f,
      proto.speech.tts.SpeakerParams.serializeBinaryToWriter
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeString(
      5,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeString(
      6,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 7));
  if (f != null) {
    writer.writeInt32(
      7,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 8));
  if (f != null) {
    writer.writeString(
      8,
      f
    );
  }
  f = /** @type {!proto.speech.Voice.VoicePlatform} */ (jspb.Message.getField(message, 13));
  if (f != null) {
    writer.writeEnum(
      13,
      f
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 10));
  if (f != null) {
    writer.writeBool(
      10,
      f
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 11));
  if (f != null) {
    writer.writeBool(
      11,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 12));
  if (f != null) {
    writer.writeString(
      12,
      f
    );
  }
};



/**
 * A tuple of {field number, class constructor} for the extension
 * field named `id`.
 * @type {!jspb.ExtensionFieldInfo<!proto.speech.tts.VoiceParams>}
 */
proto.speech.tts.VoiceParams.id = new jspb.ExtensionFieldInfo(
    72317013,
    {id: 0},
    proto.speech.tts.VoiceParams,
     /** @type {?function((boolean|undefined),!jspb.Message=): !Object} */ (
         proto.speech.tts.VoiceParams.toObject),
    0);

proto.greco.Params.extensionsBinary[72317013] = new jspb.ExtensionFieldBinaryInfo(
    proto.speech.tts.VoiceParams.id,
    jspb.BinaryReader.prototype.readMessage,
    jspb.BinaryWriter.prototype.writeMessage,
    proto.speech.tts.VoiceParams.serializeBinaryToWriter,
    proto.speech.tts.VoiceParams.deserializeBinaryFromReader,
    false);
// This registers the extension field with the extended class, so that
// toObject() will function correctly.
proto.greco.Params.extensions[72317013] = proto.speech.tts.VoiceParams.id;

/**
 * optional string description = 1;
 * @return {?string}
 */
proto.speech.tts.VoiceParams.prototype.getDescription = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.VoiceParams} returns this
 */
proto.speech.tts.VoiceParams.prototype.setDescription = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.VoiceParams} returns this
 */
proto.speech.tts.VoiceParams.prototype.clearDescription = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.VoiceParams.prototype.hasDescription = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional string language = 2;
 * @return {?string}
 */
proto.speech.tts.VoiceParams.prototype.getLanguage = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.VoiceParams} returns this
 */
proto.speech.tts.VoiceParams.prototype.setLanguage = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.VoiceParams} returns this
 */
proto.speech.tts.VoiceParams.prototype.clearLanguage = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.VoiceParams.prototype.hasLanguage = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional string region = 3;
 * @return {?string}
 */
proto.speech.tts.VoiceParams.prototype.getRegion = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.VoiceParams} returns this
 */
proto.speech.tts.VoiceParams.prototype.setRegion = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.VoiceParams} returns this
 */
proto.speech.tts.VoiceParams.prototype.clearRegion = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.VoiceParams.prototype.hasRegion = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional string speaker = 4;
 * @return {?string}
 */
proto.speech.tts.VoiceParams.prototype.getSpeaker = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 4));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.VoiceParams} returns this
 */
proto.speech.tts.VoiceParams.prototype.setSpeaker = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.VoiceParams} returns this
 */
proto.speech.tts.VoiceParams.prototype.clearSpeaker = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.VoiceParams.prototype.hasSpeaker = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * repeated SpeakerParams speakers = 14;
 * @return {!Array<!proto.speech.tts.SpeakerParams>}
 */
proto.speech.tts.VoiceParams.prototype.getSpeakersList = function() {
  return /** @type{!Array<!proto.speech.tts.SpeakerParams>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.SpeakerParams, 14));
};


/**
 * @param {Array<!proto.speech.tts.SpeakerParams>|undefined} value
 * @return {!proto.speech.tts.VoiceParams} returns this
*/
proto.speech.tts.VoiceParams.prototype.setSpeakersList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 14, value);
};


/**
 * @param {!proto.speech.tts.SpeakerParams=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.SpeakerParams}
 */
proto.speech.tts.VoiceParams.prototype.addSpeakers = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 14, opt_value, proto.speech.tts.SpeakerParams, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.VoiceParams} returns this
 */
proto.speech.tts.VoiceParams.prototype.clearSpeakersList = function() {
  return this.setSpeakersList([]);
};


/**
 * optional string gender = 5;
 * @return {?string}
 */
proto.speech.tts.VoiceParams.prototype.getGender = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 5));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.VoiceParams} returns this
 */
proto.speech.tts.VoiceParams.prototype.setGender = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.VoiceParams} returns this
 */
proto.speech.tts.VoiceParams.prototype.clearGender = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.VoiceParams.prototype.hasGender = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional string quality = 6;
 * @return {?string}
 */
proto.speech.tts.VoiceParams.prototype.getQuality = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 6));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.VoiceParams} returns this
 */
proto.speech.tts.VoiceParams.prototype.setQuality = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.VoiceParams} returns this
 */
proto.speech.tts.VoiceParams.prototype.clearQuality = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.VoiceParams.prototype.hasQuality = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional int32 sample_rate = 7;
 * @return {?number}
 */
proto.speech.tts.VoiceParams.prototype.getSampleRate = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 7));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.VoiceParams} returns this
 */
proto.speech.tts.VoiceParams.prototype.setSampleRate = function(value) {
  return jspb.Message.setField(this, 7, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.VoiceParams} returns this
 */
proto.speech.tts.VoiceParams.prototype.clearSampleRate = function() {
  return jspb.Message.setField(this, 7, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.VoiceParams.prototype.hasSampleRate = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * optional string server_voice_name = 8;
 * @return {?string}
 */
proto.speech.tts.VoiceParams.prototype.getServerVoiceName = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 8));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.VoiceParams} returns this
 */
proto.speech.tts.VoiceParams.prototype.setServerVoiceName = function(value) {
  return jspb.Message.setField(this, 8, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.VoiceParams} returns this
 */
proto.speech.tts.VoiceParams.prototype.clearServerVoiceName = function() {
  return jspb.Message.setField(this, 8, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.VoiceParams.prototype.hasServerVoiceName = function() {
  return jspb.Message.getField(this, 8) != null;
};


/**
 * optional speech.Voice.VoicePlatform voice_platform = 13;
 * @return {!proto.speech.Voice.VoicePlatform}
 */
proto.speech.tts.VoiceParams.prototype.getVoicePlatform = function() {
  return /** @type {!proto.speech.Voice.VoicePlatform} */ (jspb.Message.getField(this, 13));
};


/**
 * @param {?proto.speech.Voice.VoicePlatform|undefined} value
 * @return {!proto.speech.tts.VoiceParams} returns this
 */
proto.speech.tts.VoiceParams.prototype.setVoicePlatform = function(value) {
  return jspb.Message.setField(this, 13, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.VoiceParams} returns this
 */
proto.speech.tts.VoiceParams.prototype.clearVoicePlatform = function() {
  return jspb.Message.setField(this, 13, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.VoiceParams.prototype.hasVoicePlatform = function() {
  return jspb.Message.getField(this, 13) != null;
};


/**
 * optional bool supports_set_sample_rate = 10;
 * @return {?boolean}
 */
proto.speech.tts.VoiceParams.prototype.getSupportsSetSampleRate = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 10));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.VoiceParams} returns this
 */
proto.speech.tts.VoiceParams.prototype.setSupportsSetSampleRate = function(value) {
  return jspb.Message.setField(this, 10, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.VoiceParams} returns this
 */
proto.speech.tts.VoiceParams.prototype.clearSupportsSetSampleRate = function() {
  return jspb.Message.setField(this, 10, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.VoiceParams.prototype.hasSupportsSetSampleRate = function() {
  return jspb.Message.getField(this, 10) != null;
};


/**
 * optional bool multi_speaker = 11;
 * @return {?boolean}
 */
proto.speech.tts.VoiceParams.prototype.getMultiSpeaker = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 11));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.VoiceParams} returns this
 */
proto.speech.tts.VoiceParams.prototype.setMultiSpeaker = function(value) {
  return jspb.Message.setField(this, 11, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.VoiceParams} returns this
 */
proto.speech.tts.VoiceParams.prototype.clearMultiSpeaker = function() {
  return jspb.Message.setField(this, 11, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.VoiceParams.prototype.hasMultiSpeaker = function() {
  return jspb.Message.getField(this, 11) != null;
};


/**
 * optional string textnorm_server_request_service_name = 12;
 * @return {?string}
 */
proto.speech.tts.VoiceParams.prototype.getTextnormServerRequestServiceName = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 12));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.VoiceParams} returns this
 */
proto.speech.tts.VoiceParams.prototype.setTextnormServerRequestServiceName = function(value) {
  return jspb.Message.setField(this, 12, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.VoiceParams} returns this
 */
proto.speech.tts.VoiceParams.prototype.clearTextnormServerRequestServiceName = function() {
  return jspb.Message.setField(this, 12, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.VoiceParams.prototype.hasTextnormServerRequestServiceName = function() {
  return jspb.Message.getField(this, 12) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.VoiceParams}
 */
proto.speech.tts.VoiceParams.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.VoiceParams, data);
};



//speech/tts/engine/utterance/artificial_concatenative_specification_data_proto_jspb/artificialconcatenativespecification.jspb.js
// source: speech/tts/engine/utterance/artificial_concatenative_specification_data.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.ArtificialConcatenativeSpecification');
goog.provide('proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.ArtificialConcatenativeSpecification = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.ArtificialConcatenativeSpecification.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.ArtificialConcatenativeSpecification, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.ArtificialConcatenativeSpecification.displayName = 'proto.speech.tts.ArtificialConcatenativeSpecification';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame.displayName = 'proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.ArtificialConcatenativeSpecification.repeatedFields_ = [4,5];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.ArtificialConcatenativeSpecification.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.ArtificialConcatenativeSpecification.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.ArtificialConcatenativeSpecification} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.ArtificialConcatenativeSpecification.toObject = function(includeInstance, msg) {
  var f, obj = {
    specId: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    startTime: (f = jspb.Message.getOptionalFloatingPointField(msg, 2)) == null ? undefined : f,
    endTime: (f = jspb.Message.getOptionalFloatingPointField(msg, 3)) == null ? undefined : f,
    acousticFramesBeforeList: jspb.Message.toObjectList(msg.getAcousticFramesBeforeList(),
    proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame.toObject, includeInstance),
    acousticFramesAfterList: jspb.Message.toObjectList(msg.getAcousticFramesAfterList(),
    proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame.toObject, includeInstance)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of ArtificialConcatenativeSpecification as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.ArtificialConcatenativeSpecification.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.specId;

  /** @type {?number|undefined} */
  this.startTime;

  /** @type {?number|undefined} */
  this.endTime;

  /** @type {?Array<!proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame.ObjectFormat>|undefined} */
  this.acousticFramesBeforeList;

  /** @type {?Array<!proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame.ObjectFormat>|undefined} */
  this.acousticFramesAfterList;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.ArtificialConcatenativeSpecification.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.ArtificialConcatenativeSpecification}
 */
proto.speech.tts.ArtificialConcatenativeSpecification.fromObject = function(obj) {
  var msg = new proto.speech.tts.ArtificialConcatenativeSpecification();
  obj.specId != null && jspb.Message.setField(msg, 1, obj.specId);
  obj.startTime != null && jspb.Message.setField(msg, 2, obj.startTime);
  obj.endTime != null && jspb.Message.setField(msg, 3, obj.endTime);
  obj.acousticFramesBeforeList && jspb.Message.setRepeatedWrapperField(
      msg, 4, obj.acousticFramesBeforeList.map(
          proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame.fromObject));
  obj.acousticFramesAfterList && jspb.Message.setRepeatedWrapperField(
      msg, 5, obj.acousticFramesAfterList.map(
          proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame.fromObject));
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.ArtificialConcatenativeSpecification}
 */
proto.speech.tts.ArtificialConcatenativeSpecification.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.ArtificialConcatenativeSpecification;
  return proto.speech.tts.ArtificialConcatenativeSpecification.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.ArtificialConcatenativeSpecification} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.ArtificialConcatenativeSpecification}
 */
proto.speech.tts.ArtificialConcatenativeSpecification.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setSpecId(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setStartTime(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setEndTime(value);
      break;
    case 4:
      var value = new proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame;
      reader.readMessage(value,proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame.deserializeBinaryFromReader);
      msg.addAcousticFramesBefore(value);
      break;
    case 5:
      var value = new proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame;
      reader.readMessage(value,proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame.deserializeBinaryFromReader);
      msg.addAcousticFramesAfter(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.ArtificialConcatenativeSpecification.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.ArtificialConcatenativeSpecification.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.ArtificialConcatenativeSpecification} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.ArtificialConcatenativeSpecification.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?number} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeInt32(
      1,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeFloat(
      2,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeFloat(
      3,
      f
    );
  }
  f = message.getAcousticFramesBeforeList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      4,
      f,
      proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame.serializeBinaryToWriter
    );
  }
  f = message.getAcousticFramesAfterList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      5,
      f,
      proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame.serializeBinaryToWriter
    );
  }
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame.toObject = function(includeInstance, msg) {
  var f, obj = {
    acousticFeatureList: (f = jspb.Message.getRepeatedFloatingPointField(msg, 1)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of AcousticFrame as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame.ObjectFormat = function() {
  /** @type {?Array<number>|undefined} */
  this.acousticFeatureList;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame}
 */
proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame.fromObject = function(obj) {
  var msg = new proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame();
  obj.acousticFeatureList != null && jspb.Message.setField(msg, 1, obj.acousticFeatureList);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame}
 */
proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame;
  return proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame}
 */
proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {number} */ (reader.readFloat());
      msg.addAcousticFeature(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getAcousticFeatureList();
  if (f.length > 0) {
    writer.writeRepeatedFloat(
      1,
      f
    );
  }
};


/**
 * repeated float acoustic_feature = 1;
 * @return {!Array<number>}
 */
proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame.prototype.getAcousticFeatureList = function() {
  return /** @type {!Array<number>} */ (jspb.Message.getRepeatedFloatingPointField(this, 1));
};


/**
 * @param {Array<number>|undefined} value
 * @return {!proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame} returns this
 */
proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame.prototype.setAcousticFeatureList = function(value) {
  return jspb.Message.setField(this, 1, value || []);
};


/**
 * @param {number} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame} returns this
 */
proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame.prototype.addAcousticFeature = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 1, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame} returns this
 */
proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame.prototype.clearAcousticFeatureList = function() {
  return this.setAcousticFeatureList([]);
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame}
 */
proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame, data);
};


/**
 * optional int32 spec_id = 1;
 * @return {?number}
 */
proto.speech.tts.ArtificialConcatenativeSpecification.prototype.getSpecId = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.ArtificialConcatenativeSpecification} returns this
 */
proto.speech.tts.ArtificialConcatenativeSpecification.prototype.setSpecId = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.ArtificialConcatenativeSpecification} returns this
 */
proto.speech.tts.ArtificialConcatenativeSpecification.prototype.clearSpecId = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.ArtificialConcatenativeSpecification.prototype.hasSpecId = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional float start_time = 2;
 * @return {?number}
 */
proto.speech.tts.ArtificialConcatenativeSpecification.prototype.getStartTime = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 2));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.ArtificialConcatenativeSpecification} returns this
 */
proto.speech.tts.ArtificialConcatenativeSpecification.prototype.setStartTime = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.ArtificialConcatenativeSpecification} returns this
 */
proto.speech.tts.ArtificialConcatenativeSpecification.prototype.clearStartTime = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.ArtificialConcatenativeSpecification.prototype.hasStartTime = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional float end_time = 3;
 * @return {?number}
 */
proto.speech.tts.ArtificialConcatenativeSpecification.prototype.getEndTime = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 3));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.ArtificialConcatenativeSpecification} returns this
 */
proto.speech.tts.ArtificialConcatenativeSpecification.prototype.setEndTime = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.ArtificialConcatenativeSpecification} returns this
 */
proto.speech.tts.ArtificialConcatenativeSpecification.prototype.clearEndTime = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.ArtificialConcatenativeSpecification.prototype.hasEndTime = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * repeated AcousticFrame acoustic_frames_before = 4;
 * @return {!Array<!proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame>}
 */
proto.speech.tts.ArtificialConcatenativeSpecification.prototype.getAcousticFramesBeforeList = function() {
  return /** @type{!Array<!proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame, 4));
};


/**
 * @param {Array<!proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame>|undefined} value
 * @return {!proto.speech.tts.ArtificialConcatenativeSpecification} returns this
*/
proto.speech.tts.ArtificialConcatenativeSpecification.prototype.setAcousticFramesBeforeList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 4, value);
};


/**
 * @param {!proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame}
 */
proto.speech.tts.ArtificialConcatenativeSpecification.prototype.addAcousticFramesBefore = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 4, opt_value, proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.ArtificialConcatenativeSpecification} returns this
 */
proto.speech.tts.ArtificialConcatenativeSpecification.prototype.clearAcousticFramesBeforeList = function() {
  return this.setAcousticFramesBeforeList([]);
};


/**
 * repeated AcousticFrame acoustic_frames_after = 5;
 * @return {!Array<!proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame>}
 */
proto.speech.tts.ArtificialConcatenativeSpecification.prototype.getAcousticFramesAfterList = function() {
  return /** @type{!Array<!proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame, 5));
};


/**
 * @param {Array<!proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame>|undefined} value
 * @return {!proto.speech.tts.ArtificialConcatenativeSpecification} returns this
*/
proto.speech.tts.ArtificialConcatenativeSpecification.prototype.setAcousticFramesAfterList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 5, value);
};


/**
 * @param {!proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame}
 */
proto.speech.tts.ArtificialConcatenativeSpecification.prototype.addAcousticFramesAfter = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 5, opt_value, proto.speech.tts.ArtificialConcatenativeSpecification.AcousticFrame, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.ArtificialConcatenativeSpecification} returns this
 */
proto.speech.tts.ArtificialConcatenativeSpecification.prototype.clearAcousticFramesAfterList = function() {
  return this.setAcousticFramesAfterList([]);
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.ArtificialConcatenativeSpecification}
 */
proto.speech.tts.ArtificialConcatenativeSpecification.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.ArtificialConcatenativeSpecification, data);
};



//speech/tts/engine/utterance/artificial_concatenative_specification_data_proto_jspb/artificialconcatenativespecificationdata.jspb.js
// source: speech/tts/engine/utterance/artificial_concatenative_specification_data.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.ArtificialConcatenativeSpecificationData');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.ArtificialConcatenativeSpecification');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.ArtificialConcatenativeSpecificationData = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.ArtificialConcatenativeSpecificationData.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.ArtificialConcatenativeSpecificationData, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.ArtificialConcatenativeSpecificationData.displayName = 'proto.speech.tts.ArtificialConcatenativeSpecificationData';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.ArtificialConcatenativeSpecificationData.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.ArtificialConcatenativeSpecificationData.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.ArtificialConcatenativeSpecificationData.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.ArtificialConcatenativeSpecificationData} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.ArtificialConcatenativeSpecificationData.toObject = function(includeInstance, msg) {
  var f, obj = {
    unitSpecificationList: jspb.Message.toObjectList(msg.getUnitSpecificationList(),
    proto.speech.tts.ArtificialConcatenativeSpecification.toObject, includeInstance),
    perUnit: (f = jspb.Message.getBooleanField(msg, 2)) == null ? undefined : f,
    inDomain: (f = jspb.Message.getBooleanField(msg, 3)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of ArtificialConcatenativeSpecificationData as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.ArtificialConcatenativeSpecificationData.ObjectFormat = function() {
  /** @type {?Array<!proto.speech.tts.ArtificialConcatenativeSpecification.ObjectFormat>|undefined} */
  this.unitSpecificationList;

  /** @type {?boolean|undefined} */
  this.perUnit;

  /** @type {?boolean|undefined} */
  this.inDomain;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.ArtificialConcatenativeSpecificationData.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.ArtificialConcatenativeSpecificationData}
 */
proto.speech.tts.ArtificialConcatenativeSpecificationData.fromObject = function(obj) {
  var msg = new proto.speech.tts.ArtificialConcatenativeSpecificationData();
  obj.unitSpecificationList && jspb.Message.setRepeatedWrapperField(
      msg, 1, obj.unitSpecificationList.map(
          proto.speech.tts.ArtificialConcatenativeSpecification.fromObject));
  obj.perUnit != null && jspb.Message.setField(msg, 2, obj.perUnit);
  obj.inDomain != null && jspb.Message.setField(msg, 3, obj.inDomain);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.ArtificialConcatenativeSpecificationData}
 */
proto.speech.tts.ArtificialConcatenativeSpecificationData.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.ArtificialConcatenativeSpecificationData;
  return proto.speech.tts.ArtificialConcatenativeSpecificationData.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.ArtificialConcatenativeSpecificationData} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.ArtificialConcatenativeSpecificationData}
 */
proto.speech.tts.ArtificialConcatenativeSpecificationData.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.speech.tts.ArtificialConcatenativeSpecification;
      reader.readMessage(value,proto.speech.tts.ArtificialConcatenativeSpecification.deserializeBinaryFromReader);
      msg.addUnitSpecification(value);
      break;
    case 2:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setPerUnit(value);
      break;
    case 3:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setInDomain(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.ArtificialConcatenativeSpecificationData.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.ArtificialConcatenativeSpecificationData.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.ArtificialConcatenativeSpecificationData} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.ArtificialConcatenativeSpecificationData.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getUnitSpecificationList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      1,
      f,
      proto.speech.tts.ArtificialConcatenativeSpecification.serializeBinaryToWriter
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeBool(
      2,
      f
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeBool(
      3,
      f
    );
  }
};


/**
 * repeated ArtificialConcatenativeSpecification unit_specification = 1;
 * @return {!Array<!proto.speech.tts.ArtificialConcatenativeSpecification>}
 */
proto.speech.tts.ArtificialConcatenativeSpecificationData.prototype.getUnitSpecificationList = function() {
  return /** @type{!Array<!proto.speech.tts.ArtificialConcatenativeSpecification>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.ArtificialConcatenativeSpecification, 1));
};


/**
 * @param {Array<!proto.speech.tts.ArtificialConcatenativeSpecification>|undefined} value
 * @return {!proto.speech.tts.ArtificialConcatenativeSpecificationData} returns this
*/
proto.speech.tts.ArtificialConcatenativeSpecificationData.prototype.setUnitSpecificationList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 1, value);
};


/**
 * @param {!proto.speech.tts.ArtificialConcatenativeSpecification=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.ArtificialConcatenativeSpecification}
 */
proto.speech.tts.ArtificialConcatenativeSpecificationData.prototype.addUnitSpecification = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.speech.tts.ArtificialConcatenativeSpecification, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.ArtificialConcatenativeSpecificationData} returns this
 */
proto.speech.tts.ArtificialConcatenativeSpecificationData.prototype.clearUnitSpecificationList = function() {
  return this.setUnitSpecificationList([]);
};


/**
 * optional bool per_unit = 2;
 * @return {?boolean}
 */
proto.speech.tts.ArtificialConcatenativeSpecificationData.prototype.getPerUnit = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 2));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.ArtificialConcatenativeSpecificationData} returns this
 */
proto.speech.tts.ArtificialConcatenativeSpecificationData.prototype.setPerUnit = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.ArtificialConcatenativeSpecificationData} returns this
 */
proto.speech.tts.ArtificialConcatenativeSpecificationData.prototype.clearPerUnit = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.ArtificialConcatenativeSpecificationData.prototype.hasPerUnit = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional bool in_domain = 3;
 * @return {?boolean}
 */
proto.speech.tts.ArtificialConcatenativeSpecificationData.prototype.getInDomain = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 3));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.ArtificialConcatenativeSpecificationData} returns this
 */
proto.speech.tts.ArtificialConcatenativeSpecificationData.prototype.setInDomain = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.ArtificialConcatenativeSpecificationData} returns this
 */
proto.speech.tts.ArtificialConcatenativeSpecificationData.prototype.clearInDomain = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.ArtificialConcatenativeSpecificationData.prototype.hasInDomain = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.ArtificialConcatenativeSpecificationData}
 */
proto.speech.tts.ArtificialConcatenativeSpecificationData.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.ArtificialConcatenativeSpecificationData, data);
};



//speech/tts/engine/utterance/coverage_proto_jspb/coverage.jspb.js
// source: speech/tts/engine/utterance/coverage.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.Coverage');
goog.provide('proto.speech.tts.Coverage.Item');
goog.provide('proto.speech.tts.Coverage.Section');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.Coverage = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.Coverage.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.Coverage, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.Coverage.displayName = 'proto.speech.tts.Coverage';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.Coverage.Item = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.Coverage.Item, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.Coverage.Item.displayName = 'proto.speech.tts.Coverage.Item';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.Coverage.Section = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.Coverage.Section.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.Coverage.Section, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.Coverage.Section.displayName = 'proto.speech.tts.Coverage.Section';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.Coverage.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.Coverage.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.Coverage.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.Coverage} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Coverage.toObject = function(includeInstance, msg) {
  var f, obj = {
    sectionsList: jspb.Message.toObjectList(msg.getSectionsList(),
    proto.speech.tts.Coverage.Section.toObject, includeInstance),
    wordIds: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    vuiId: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Coverage as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.Coverage.ObjectFormat = function() {
  /** @type {?Array<!proto.speech.tts.Coverage.Section.ObjectFormat>|undefined} */
  this.sectionsList;

  /** @type {?string|undefined} */
  this.wordIds;

  /** @type {?string|undefined} */
  this.vuiId;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.Coverage.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.Coverage}
 */
proto.speech.tts.Coverage.fromObject = function(obj) {
  var msg = new proto.speech.tts.Coverage();
  obj.sectionsList && jspb.Message.setRepeatedWrapperField(
      msg, 1, obj.sectionsList.map(
          proto.speech.tts.Coverage.Section.fromObject));
  obj.wordIds != null && jspb.Message.setField(msg, 2, obj.wordIds);
  obj.vuiId != null && jspb.Message.setField(msg, 3, obj.vuiId);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.Coverage}
 */
proto.speech.tts.Coverage.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.Coverage;
  return proto.speech.tts.Coverage.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.Coverage} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.Coverage}
 */
proto.speech.tts.Coverage.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.speech.tts.Coverage.Section;
      reader.readMessage(value,proto.speech.tts.Coverage.Section.deserializeBinaryFromReader);
      msg.addSections(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setWordIds(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setVuiId(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.Coverage.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.Coverage.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.Coverage} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Coverage.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getSectionsList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      1,
      f,
      proto.speech.tts.Coverage.Section.serializeBinaryToWriter
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeString(
      3,
      f
    );
  }
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.Coverage.Item.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.Coverage.Item.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.Coverage.Item} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Coverage.Item.toObject = function(includeInstance, msg) {
  var f, obj = {
    name: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    firstWord: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    lastWord: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Item as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.Coverage.Item.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.name;

  /** @type {?number|undefined} */
  this.firstWord;

  /** @type {?number|undefined} */
  this.lastWord;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.Coverage.Item.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.Coverage.Item}
 */
proto.speech.tts.Coverage.Item.fromObject = function(obj) {
  var msg = new proto.speech.tts.Coverage.Item();
  obj.name != null && jspb.Message.setField(msg, 1, obj.name);
  obj.firstWord != null && jspb.Message.setField(msg, 2, obj.firstWord);
  obj.lastWord != null && jspb.Message.setField(msg, 3, obj.lastWord);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.Coverage.Item}
 */
proto.speech.tts.Coverage.Item.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.Coverage.Item;
  return proto.speech.tts.Coverage.Item.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.Coverage.Item} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.Coverage.Item}
 */
proto.speech.tts.Coverage.Item.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setName(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setFirstWord(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setLastWord(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.Coverage.Item.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.Coverage.Item.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.Coverage.Item} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Coverage.Item.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeInt32(
      2,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeInt32(
      3,
      f
    );
  }
};


/**
 * optional string name = 1;
 * @return {?string}
 */
proto.speech.tts.Coverage.Item.prototype.getName = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Coverage.Item} returns this
 */
proto.speech.tts.Coverage.Item.prototype.setName = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Coverage.Item} returns this
 */
proto.speech.tts.Coverage.Item.prototype.clearName = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Coverage.Item.prototype.hasName = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional int32 first_word = 2;
 * @return {?number}
 */
proto.speech.tts.Coverage.Item.prototype.getFirstWord = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Coverage.Item} returns this
 */
proto.speech.tts.Coverage.Item.prototype.setFirstWord = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Coverage.Item} returns this
 */
proto.speech.tts.Coverage.Item.prototype.clearFirstWord = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Coverage.Item.prototype.hasFirstWord = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional int32 last_word = 3;
 * @return {?number}
 */
proto.speech.tts.Coverage.Item.prototype.getLastWord = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Coverage.Item} returns this
 */
proto.speech.tts.Coverage.Item.prototype.setLastWord = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Coverage.Item} returns this
 */
proto.speech.tts.Coverage.Item.prototype.clearLastWord = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Coverage.Item.prototype.hasLastWord = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.Coverage.Item}
 */
proto.speech.tts.Coverage.Item.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.Coverage.Item, data);
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.Coverage.Section.repeatedFields_ = [2];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.Coverage.Section.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.Coverage.Section.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.Coverage.Section} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Coverage.Section.toObject = function(includeInstance, msg) {
  var f, obj = {
    name: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    itemsList: jspb.Message.toObjectList(msg.getItemsList(),
    proto.speech.tts.Coverage.Item.toObject, includeInstance)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Section as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.Coverage.Section.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.name;

  /** @type {?Array<!proto.speech.tts.Coverage.Item.ObjectFormat>|undefined} */
  this.itemsList;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.Coverage.Section.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.Coverage.Section}
 */
proto.speech.tts.Coverage.Section.fromObject = function(obj) {
  var msg = new proto.speech.tts.Coverage.Section();
  obj.name != null && jspb.Message.setField(msg, 1, obj.name);
  obj.itemsList && jspb.Message.setRepeatedWrapperField(
      msg, 2, obj.itemsList.map(
          proto.speech.tts.Coverage.Item.fromObject));
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.Coverage.Section}
 */
proto.speech.tts.Coverage.Section.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.Coverage.Section;
  return proto.speech.tts.Coverage.Section.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.Coverage.Section} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.Coverage.Section}
 */
proto.speech.tts.Coverage.Section.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setName(value);
      break;
    case 2:
      var value = new proto.speech.tts.Coverage.Item;
      reader.readMessage(value,proto.speech.tts.Coverage.Item.deserializeBinaryFromReader);
      msg.addItems(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.Coverage.Section.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.Coverage.Section.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.Coverage.Section} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Coverage.Section.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = message.getItemsList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      2,
      f,
      proto.speech.tts.Coverage.Item.serializeBinaryToWriter
    );
  }
};


/**
 * optional string name = 1;
 * @return {?string}
 */
proto.speech.tts.Coverage.Section.prototype.getName = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Coverage.Section} returns this
 */
proto.speech.tts.Coverage.Section.prototype.setName = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Coverage.Section} returns this
 */
proto.speech.tts.Coverage.Section.prototype.clearName = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Coverage.Section.prototype.hasName = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * repeated Item items = 2;
 * @return {!Array<!proto.speech.tts.Coverage.Item>}
 */
proto.speech.tts.Coverage.Section.prototype.getItemsList = function() {
  return /** @type{!Array<!proto.speech.tts.Coverage.Item>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.Coverage.Item, 2));
};


/**
 * @param {Array<!proto.speech.tts.Coverage.Item>|undefined} value
 * @return {!proto.speech.tts.Coverage.Section} returns this
*/
proto.speech.tts.Coverage.Section.prototype.setItemsList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 2, value);
};


/**
 * @param {!proto.speech.tts.Coverage.Item=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.Coverage.Item}
 */
proto.speech.tts.Coverage.Section.prototype.addItems = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 2, opt_value, proto.speech.tts.Coverage.Item, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.Coverage.Section} returns this
 */
proto.speech.tts.Coverage.Section.prototype.clearItemsList = function() {
  return this.setItemsList([]);
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.Coverage.Section}
 */
proto.speech.tts.Coverage.Section.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.Coverage.Section, data);
};


/**
 * repeated Section sections = 1;
 * @return {!Array<!proto.speech.tts.Coverage.Section>}
 */
proto.speech.tts.Coverage.prototype.getSectionsList = function() {
  return /** @type{!Array<!proto.speech.tts.Coverage.Section>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.Coverage.Section, 1));
};


/**
 * @param {Array<!proto.speech.tts.Coverage.Section>|undefined} value
 * @return {!proto.speech.tts.Coverage} returns this
*/
proto.speech.tts.Coverage.prototype.setSectionsList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 1, value);
};


/**
 * @param {!proto.speech.tts.Coverage.Section=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.Coverage.Section}
 */
proto.speech.tts.Coverage.prototype.addSections = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.speech.tts.Coverage.Section, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.Coverage} returns this
 */
proto.speech.tts.Coverage.prototype.clearSectionsList = function() {
  return this.setSectionsList([]);
};


/**
 * optional string word_ids = 2;
 * @return {?string}
 */
proto.speech.tts.Coverage.prototype.getWordIds = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Coverage} returns this
 */
proto.speech.tts.Coverage.prototype.setWordIds = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Coverage} returns this
 */
proto.speech.tts.Coverage.prototype.clearWordIds = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Coverage.prototype.hasWordIds = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional string vui_id = 3;
 * @return {?string}
 */
proto.speech.tts.Coverage.prototype.getVuiId = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Coverage} returns this
 */
proto.speech.tts.Coverage.prototype.setVuiId = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Coverage} returns this
 */
proto.speech.tts.Coverage.prototype.clearVuiId = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Coverage.prototype.hasVuiId = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.Coverage}
 */
proto.speech.tts.Coverage.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.Coverage, data);
};



//speech/tts/engine/eval/performance_info_proto_jspb/performanceinfo.jspb.js
// source: speech/tts/engine/eval/performance_info.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.PerformanceInfo');
goog.provide('proto.speech.tts.PerformanceInfo.Stats');
goog.provide('proto.speech.tts.PerformanceInfo.Stats.MemoryInfo');
goog.provide('proto.speech.tts.PerformanceInfo.Stats.SpeedInfo');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.PerformanceInfo = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.PerformanceInfo.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.PerformanceInfo, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.PerformanceInfo.displayName = 'proto.speech.tts.PerformanceInfo';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.PerformanceInfo.Stats = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.PerformanceInfo.Stats, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.PerformanceInfo.Stats.displayName = 'proto.speech.tts.PerformanceInfo.Stats';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.PerformanceInfo.Stats.MemoryInfo = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.PerformanceInfo.Stats.MemoryInfo, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.PerformanceInfo.Stats.MemoryInfo.displayName = 'proto.speech.tts.PerformanceInfo.Stats.MemoryInfo';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.PerformanceInfo.Stats.SpeedInfo = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.PerformanceInfo.Stats.SpeedInfo, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.PerformanceInfo.Stats.SpeedInfo.displayName = 'proto.speech.tts.PerformanceInfo.Stats.SpeedInfo';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.PerformanceInfo.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.PerformanceInfo.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.PerformanceInfo.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.PerformanceInfo} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.PerformanceInfo.toObject = function(includeInstance, msg) {
  var f, obj = {
    streamStatsList: jspb.Message.toObjectList(msg.getStreamStatsList(),
    proto.speech.tts.PerformanceInfo.Stats.toObject, includeInstance),
    overallStats: (f = msg.getOverallStats()) && proto.speech.tts.PerformanceInfo.Stats.toObject(includeInstance, f),
    input: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,
    inputId: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f,
    length: (f = jspb.Message.getField(msg, 5)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of PerformanceInfo as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.PerformanceInfo.ObjectFormat = function() {
  /** @type {?Array<!proto.speech.tts.PerformanceInfo.Stats.ObjectFormat>|undefined} */
  this.streamStatsList;

  /** @type {?proto.speech.tts.PerformanceInfo.Stats.ObjectFormat|undefined} */
  this.overallStats;

  /** @type {?string|undefined} */
  this.input;

  /** @type {?number|undefined} */
  this.inputId;

  /** @type {?number|undefined} */
  this.length;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.PerformanceInfo.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.PerformanceInfo}
 */
proto.speech.tts.PerformanceInfo.fromObject = function(obj) {
  var msg = new proto.speech.tts.PerformanceInfo();
  obj.streamStatsList && jspb.Message.setRepeatedWrapperField(
      msg, 1, obj.streamStatsList.map(
          proto.speech.tts.PerformanceInfo.Stats.fromObject));
  obj.overallStats && jspb.Message.setWrapperField(
      msg, 2, proto.speech.tts.PerformanceInfo.Stats.fromObject(obj.overallStats));
  obj.input != null && jspb.Message.setField(msg, 3, obj.input);
  obj.inputId != null && jspb.Message.setField(msg, 4, obj.inputId);
  obj.length != null && jspb.Message.setField(msg, 5, obj.length);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.PerformanceInfo}
 */
proto.speech.tts.PerformanceInfo.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.PerformanceInfo;
  return proto.speech.tts.PerformanceInfo.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.PerformanceInfo} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.PerformanceInfo}
 */
proto.speech.tts.PerformanceInfo.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.speech.tts.PerformanceInfo.Stats;
      reader.readMessage(value,proto.speech.tts.PerformanceInfo.Stats.deserializeBinaryFromReader);
      msg.addStreamStats(value);
      break;
    case 2:
      var value = new proto.speech.tts.PerformanceInfo.Stats;
      reader.readMessage(value,proto.speech.tts.PerformanceInfo.Stats.deserializeBinaryFromReader);
      msg.setOverallStats(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setInput(value);
      break;
    case 4:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setInputId(value);
      break;
    case 5:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setLength(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.PerformanceInfo.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.PerformanceInfo.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.PerformanceInfo} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.PerformanceInfo.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getStreamStatsList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      1,
      f,
      proto.speech.tts.PerformanceInfo.Stats.serializeBinaryToWriter
    );
  }
  f = message.getOverallStats();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.speech.tts.PerformanceInfo.Stats.serializeBinaryToWriter
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeString(
      3,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeInt32(
      4,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeInt32(
      5,
      f
    );
  }
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.PerformanceInfo.Stats.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.PerformanceInfo.Stats.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.PerformanceInfo.Stats} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.PerformanceInfo.Stats.toObject = function(includeInstance, msg) {
  var f, obj = {
    streamName: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    mem: (f = msg.getMem()) && proto.speech.tts.PerformanceInfo.Stats.MemoryInfo.toObject(includeInstance, f),
    time: (f = msg.getTime()) && proto.speech.tts.PerformanceInfo.Stats.SpeedInfo.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Stats as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.PerformanceInfo.Stats.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.streamName;

  /** @type {?proto.speech.tts.PerformanceInfo.Stats.MemoryInfo.ObjectFormat|undefined} */
  this.mem;

  /** @type {?proto.speech.tts.PerformanceInfo.Stats.SpeedInfo.ObjectFormat|undefined} */
  this.time;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.PerformanceInfo.Stats.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.PerformanceInfo.Stats}
 */
proto.speech.tts.PerformanceInfo.Stats.fromObject = function(obj) {
  var msg = new proto.speech.tts.PerformanceInfo.Stats();
  obj.streamName != null && jspb.Message.setField(msg, 1, obj.streamName);
  obj.mem && jspb.Message.setWrapperField(
      msg, 2, proto.speech.tts.PerformanceInfo.Stats.MemoryInfo.fromObject(obj.mem));
  obj.time && jspb.Message.setWrapperField(
      msg, 3, proto.speech.tts.PerformanceInfo.Stats.SpeedInfo.fromObject(obj.time));
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.PerformanceInfo.Stats}
 */
proto.speech.tts.PerformanceInfo.Stats.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.PerformanceInfo.Stats;
  return proto.speech.tts.PerformanceInfo.Stats.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.PerformanceInfo.Stats} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.PerformanceInfo.Stats}
 */
proto.speech.tts.PerformanceInfo.Stats.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setStreamName(value);
      break;
    case 2:
      var value = new proto.speech.tts.PerformanceInfo.Stats.MemoryInfo;
      reader.readMessage(value,proto.speech.tts.PerformanceInfo.Stats.MemoryInfo.deserializeBinaryFromReader);
      msg.setMem(value);
      break;
    case 3:
      var value = new proto.speech.tts.PerformanceInfo.Stats.SpeedInfo;
      reader.readMessage(value,proto.speech.tts.PerformanceInfo.Stats.SpeedInfo.deserializeBinaryFromReader);
      msg.setTime(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.PerformanceInfo.Stats.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.PerformanceInfo.Stats.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.PerformanceInfo.Stats} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.PerformanceInfo.Stats.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = message.getMem();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.speech.tts.PerformanceInfo.Stats.MemoryInfo.serializeBinaryToWriter
    );
  }
  f = message.getTime();
  if (f != null) {
    writer.writeMessage(
      3,
      f,
      proto.speech.tts.PerformanceInfo.Stats.SpeedInfo.serializeBinaryToWriter
    );
  }
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.PerformanceInfo.Stats.MemoryInfo.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.PerformanceInfo.Stats.MemoryInfo.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.PerformanceInfo.Stats.MemoryInfo} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.PerformanceInfo.Stats.MemoryInfo.toObject = function(includeInstance, msg) {
  var f, obj = {
    startMemory: (f = jspb.Message.getOptionalFloatingPointField(msg, 1)) == null ? undefined : f,
    usedMemory: (f = jspb.Message.getOptionalFloatingPointField(msg, 2)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of MemoryInfo as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.PerformanceInfo.Stats.MemoryInfo.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.startMemory;

  /** @type {?number|undefined} */
  this.usedMemory;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.PerformanceInfo.Stats.MemoryInfo.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.PerformanceInfo.Stats.MemoryInfo}
 */
proto.speech.tts.PerformanceInfo.Stats.MemoryInfo.fromObject = function(obj) {
  var msg = new proto.speech.tts.PerformanceInfo.Stats.MemoryInfo();
  obj.startMemory != null && jspb.Message.setField(msg, 1, obj.startMemory);
  obj.usedMemory != null && jspb.Message.setField(msg, 2, obj.usedMemory);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.PerformanceInfo.Stats.MemoryInfo}
 */
proto.speech.tts.PerformanceInfo.Stats.MemoryInfo.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.PerformanceInfo.Stats.MemoryInfo;
  return proto.speech.tts.PerformanceInfo.Stats.MemoryInfo.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.PerformanceInfo.Stats.MemoryInfo} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.PerformanceInfo.Stats.MemoryInfo}
 */
proto.speech.tts.PerformanceInfo.Stats.MemoryInfo.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {number} */ (reader.readDouble());
      msg.setStartMemory(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readDouble());
      msg.setUsedMemory(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.PerformanceInfo.Stats.MemoryInfo.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.PerformanceInfo.Stats.MemoryInfo.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.PerformanceInfo.Stats.MemoryInfo} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.PerformanceInfo.Stats.MemoryInfo.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?number} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeDouble(
      1,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeDouble(
      2,
      f
    );
  }
};


/**
 * optional double start_memory = 1;
 * @return {?number}
 */
proto.speech.tts.PerformanceInfo.Stats.MemoryInfo.prototype.getStartMemory = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 1));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.PerformanceInfo.Stats.MemoryInfo} returns this
 */
proto.speech.tts.PerformanceInfo.Stats.MemoryInfo.prototype.setStartMemory = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.PerformanceInfo.Stats.MemoryInfo} returns this
 */
proto.speech.tts.PerformanceInfo.Stats.MemoryInfo.prototype.clearStartMemory = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.PerformanceInfo.Stats.MemoryInfo.prototype.hasStartMemory = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional double used_memory = 2;
 * @return {?number}
 */
proto.speech.tts.PerformanceInfo.Stats.MemoryInfo.prototype.getUsedMemory = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 2));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.PerformanceInfo.Stats.MemoryInfo} returns this
 */
proto.speech.tts.PerformanceInfo.Stats.MemoryInfo.prototype.setUsedMemory = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.PerformanceInfo.Stats.MemoryInfo} returns this
 */
proto.speech.tts.PerformanceInfo.Stats.MemoryInfo.prototype.clearUsedMemory = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.PerformanceInfo.Stats.MemoryInfo.prototype.hasUsedMemory = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.PerformanceInfo.Stats.MemoryInfo}
 */
proto.speech.tts.PerformanceInfo.Stats.MemoryInfo.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.PerformanceInfo.Stats.MemoryInfo, data);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.PerformanceInfo.Stats.SpeedInfo.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.PerformanceInfo.Stats.SpeedInfo.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.PerformanceInfo.Stats.SpeedInfo} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.PerformanceInfo.Stats.SpeedInfo.toObject = function(includeInstance, msg) {
  var f, obj = {
    duration: (f = jspb.Message.getOptionalFloatingPointField(msg, 1)) == null ? undefined : f,
    timeUsec: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of SpeedInfo as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.PerformanceInfo.Stats.SpeedInfo.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.duration;

  /** @type {?number|undefined} */
  this.timeUsec;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.PerformanceInfo.Stats.SpeedInfo.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.PerformanceInfo.Stats.SpeedInfo}
 */
proto.speech.tts.PerformanceInfo.Stats.SpeedInfo.fromObject = function(obj) {
  var msg = new proto.speech.tts.PerformanceInfo.Stats.SpeedInfo();
  obj.duration != null && jspb.Message.setField(msg, 1, obj.duration);
  obj.timeUsec != null && jspb.Message.setField(msg, 2, obj.timeUsec);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.PerformanceInfo.Stats.SpeedInfo}
 */
proto.speech.tts.PerformanceInfo.Stats.SpeedInfo.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.PerformanceInfo.Stats.SpeedInfo;
  return proto.speech.tts.PerformanceInfo.Stats.SpeedInfo.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.PerformanceInfo.Stats.SpeedInfo} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.PerformanceInfo.Stats.SpeedInfo}
 */
proto.speech.tts.PerformanceInfo.Stats.SpeedInfo.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {number} */ (reader.readDouble());
      msg.setDuration(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setTimeUsec(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.PerformanceInfo.Stats.SpeedInfo.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.PerformanceInfo.Stats.SpeedInfo.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.PerformanceInfo.Stats.SpeedInfo} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.PerformanceInfo.Stats.SpeedInfo.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?number} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeDouble(
      1,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeInt64(
      2,
      f
    );
  }
};


/**
 * optional double duration = 1;
 * @return {?number}
 */
proto.speech.tts.PerformanceInfo.Stats.SpeedInfo.prototype.getDuration = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 1));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.PerformanceInfo.Stats.SpeedInfo} returns this
 */
proto.speech.tts.PerformanceInfo.Stats.SpeedInfo.prototype.setDuration = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.PerformanceInfo.Stats.SpeedInfo} returns this
 */
proto.speech.tts.PerformanceInfo.Stats.SpeedInfo.prototype.clearDuration = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.PerformanceInfo.Stats.SpeedInfo.prototype.hasDuration = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional int64 time_usec = 2;
 * @return {?number}
 */
proto.speech.tts.PerformanceInfo.Stats.SpeedInfo.prototype.getTimeUsec = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.PerformanceInfo.Stats.SpeedInfo} returns this
 */
proto.speech.tts.PerformanceInfo.Stats.SpeedInfo.prototype.setTimeUsec = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.PerformanceInfo.Stats.SpeedInfo} returns this
 */
proto.speech.tts.PerformanceInfo.Stats.SpeedInfo.prototype.clearTimeUsec = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.PerformanceInfo.Stats.SpeedInfo.prototype.hasTimeUsec = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.PerformanceInfo.Stats.SpeedInfo}
 */
proto.speech.tts.PerformanceInfo.Stats.SpeedInfo.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.PerformanceInfo.Stats.SpeedInfo, data);
};


/**
 * optional string stream_name = 1;
 * @return {?string}
 */
proto.speech.tts.PerformanceInfo.Stats.prototype.getStreamName = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.PerformanceInfo.Stats} returns this
 */
proto.speech.tts.PerformanceInfo.Stats.prototype.setStreamName = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.PerformanceInfo.Stats} returns this
 */
proto.speech.tts.PerformanceInfo.Stats.prototype.clearStreamName = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.PerformanceInfo.Stats.prototype.hasStreamName = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional MemoryInfo mem = 2;
 * @return {?proto.speech.tts.PerformanceInfo.Stats.MemoryInfo}
 */
proto.speech.tts.PerformanceInfo.Stats.prototype.getMem = function() {
  return /** @type{?proto.speech.tts.PerformanceInfo.Stats.MemoryInfo} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.PerformanceInfo.Stats.MemoryInfo, 2));
};


/**
 * @param {?proto.speech.tts.PerformanceInfo.Stats.MemoryInfo|undefined} value
 * @return {!proto.speech.tts.PerformanceInfo.Stats} returns this
*/
proto.speech.tts.PerformanceInfo.Stats.prototype.setMem = function(value) {
  return jspb.Message.setWrapperField(this, 2, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.PerformanceInfo.Stats} returns this
 */
proto.speech.tts.PerformanceInfo.Stats.prototype.clearMem = function() {
  return this.setMem(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.PerformanceInfo.Stats.prototype.hasMem = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional SpeedInfo time = 3;
 * @return {?proto.speech.tts.PerformanceInfo.Stats.SpeedInfo}
 */
proto.speech.tts.PerformanceInfo.Stats.prototype.getTime = function() {
  return /** @type{?proto.speech.tts.PerformanceInfo.Stats.SpeedInfo} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.PerformanceInfo.Stats.SpeedInfo, 3));
};


/**
 * @param {?proto.speech.tts.PerformanceInfo.Stats.SpeedInfo|undefined} value
 * @return {!proto.speech.tts.PerformanceInfo.Stats} returns this
*/
proto.speech.tts.PerformanceInfo.Stats.prototype.setTime = function(value) {
  return jspb.Message.setWrapperField(this, 3, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.PerformanceInfo.Stats} returns this
 */
proto.speech.tts.PerformanceInfo.Stats.prototype.clearTime = function() {
  return this.setTime(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.PerformanceInfo.Stats.prototype.hasTime = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.PerformanceInfo.Stats}
 */
proto.speech.tts.PerformanceInfo.Stats.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.PerformanceInfo.Stats, data);
};


/**
 * repeated Stats stream_stats = 1;
 * @return {!Array<!proto.speech.tts.PerformanceInfo.Stats>}
 */
proto.speech.tts.PerformanceInfo.prototype.getStreamStatsList = function() {
  return /** @type{!Array<!proto.speech.tts.PerformanceInfo.Stats>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.PerformanceInfo.Stats, 1));
};


/**
 * @param {Array<!proto.speech.tts.PerformanceInfo.Stats>|undefined} value
 * @return {!proto.speech.tts.PerformanceInfo} returns this
*/
proto.speech.tts.PerformanceInfo.prototype.setStreamStatsList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 1, value);
};


/**
 * @param {!proto.speech.tts.PerformanceInfo.Stats=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.PerformanceInfo.Stats}
 */
proto.speech.tts.PerformanceInfo.prototype.addStreamStats = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.speech.tts.PerformanceInfo.Stats, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.PerformanceInfo} returns this
 */
proto.speech.tts.PerformanceInfo.prototype.clearStreamStatsList = function() {
  return this.setStreamStatsList([]);
};


/**
 * optional Stats overall_stats = 2;
 * @return {?proto.speech.tts.PerformanceInfo.Stats}
 */
proto.speech.tts.PerformanceInfo.prototype.getOverallStats = function() {
  return /** @type{?proto.speech.tts.PerformanceInfo.Stats} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.PerformanceInfo.Stats, 2));
};


/**
 * @param {?proto.speech.tts.PerformanceInfo.Stats|undefined} value
 * @return {!proto.speech.tts.PerformanceInfo} returns this
*/
proto.speech.tts.PerformanceInfo.prototype.setOverallStats = function(value) {
  return jspb.Message.setWrapperField(this, 2, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.PerformanceInfo} returns this
 */
proto.speech.tts.PerformanceInfo.prototype.clearOverallStats = function() {
  return this.setOverallStats(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.PerformanceInfo.prototype.hasOverallStats = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional string input = 3;
 * @return {?string}
 */
proto.speech.tts.PerformanceInfo.prototype.getInput = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.PerformanceInfo} returns this
 */
proto.speech.tts.PerformanceInfo.prototype.setInput = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.PerformanceInfo} returns this
 */
proto.speech.tts.PerformanceInfo.prototype.clearInput = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.PerformanceInfo.prototype.hasInput = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional int32 input_id = 4;
 * @return {?number}
 */
proto.speech.tts.PerformanceInfo.prototype.getInputId = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 4));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.PerformanceInfo} returns this
 */
proto.speech.tts.PerformanceInfo.prototype.setInputId = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.PerformanceInfo} returns this
 */
proto.speech.tts.PerformanceInfo.prototype.clearInputId = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.PerformanceInfo.prototype.hasInputId = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional int32 length = 5;
 * @return {?number}
 */
proto.speech.tts.PerformanceInfo.prototype.getLength = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 5));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.PerformanceInfo} returns this
 */
proto.speech.tts.PerformanceInfo.prototype.setLength = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.PerformanceInfo} returns this
 */
proto.speech.tts.PerformanceInfo.prototype.clearLength = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.PerformanceInfo.prototype.hasLength = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.PerformanceInfo}
 */
proto.speech.tts.PerformanceInfo.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.PerformanceInfo, data);
};



//speech/tts/engine/textnorm/textnorm_diagnostics_proto_jspb/graphvizdiagnostics.jspb.js
// source: speech/tts/engine/textnorm/textnorm_diagnostics.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.GraphvizDiagnostics');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.GraphvizDiagnostics = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.GraphvizDiagnostics, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.GraphvizDiagnostics.displayName = 'proto.speech.tts.GraphvizDiagnostics';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.GraphvizDiagnostics.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.GraphvizDiagnostics.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.GraphvizDiagnostics} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.GraphvizDiagnostics.toObject = function(includeInstance, msg) {
  var f, obj = {
    title: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    dot: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of GraphvizDiagnostics as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.GraphvizDiagnostics.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.title;

  /** @type {?string|undefined} */
  this.dot;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.GraphvizDiagnostics.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.GraphvizDiagnostics}
 */
proto.speech.tts.GraphvizDiagnostics.fromObject = function(obj) {
  var msg = new proto.speech.tts.GraphvizDiagnostics();
  obj.title != null && jspb.Message.setField(msg, 1, obj.title);
  obj.dot != null && jspb.Message.setField(msg, 2, obj.dot);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.GraphvizDiagnostics}
 */
proto.speech.tts.GraphvizDiagnostics.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.GraphvizDiagnostics;
  return proto.speech.tts.GraphvizDiagnostics.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.GraphvizDiagnostics} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.GraphvizDiagnostics}
 */
proto.speech.tts.GraphvizDiagnostics.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setTitle(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setDot(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.GraphvizDiagnostics.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.GraphvizDiagnostics.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.GraphvizDiagnostics} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.GraphvizDiagnostics.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string title = 1;
 * @return {?string}
 */
proto.speech.tts.GraphvizDiagnostics.prototype.getTitle = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.GraphvizDiagnostics} returns this
 */
proto.speech.tts.GraphvizDiagnostics.prototype.setTitle = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.GraphvizDiagnostics} returns this
 */
proto.speech.tts.GraphvizDiagnostics.prototype.clearTitle = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.GraphvizDiagnostics.prototype.hasTitle = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional string dot = 2;
 * @return {?string}
 */
proto.speech.tts.GraphvizDiagnostics.prototype.getDot = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.GraphvizDiagnostics} returns this
 */
proto.speech.tts.GraphvizDiagnostics.prototype.setDot = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.GraphvizDiagnostics} returns this
 */
proto.speech.tts.GraphvizDiagnostics.prototype.clearDot = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.GraphvizDiagnostics.prototype.hasDot = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.GraphvizDiagnostics}
 */
proto.speech.tts.GraphvizDiagnostics.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.GraphvizDiagnostics, data);
};



//speech/tts/engine/textnorm/textnorm_diagnostics_proto_jspb/grmapplicationdiagnostics.jspb.js
// source: speech/tts/engine/textnorm/textnorm_diagnostics.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.GrmApplicationDiagnostics');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.GrmApplicationDiagnostics = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.GrmApplicationDiagnostics.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.GrmApplicationDiagnostics, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.GrmApplicationDiagnostics.displayName = 'proto.speech.tts.GrmApplicationDiagnostics';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.GrmApplicationDiagnostics.repeatedFields_ = [5,6];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.GrmApplicationDiagnostics.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.GrmApplicationDiagnostics.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.GrmApplicationDiagnostics} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.GrmApplicationDiagnostics.toObject = function(includeInstance, msg) {
  var f, obj = {
    rule: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    acceptedInput: (f = jspb.Message.getField(msg, 8)) == null ? undefined : f,
    topInput: (f = jspb.Message.getField(msg, 9)) == null ? undefined : f,
    input: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    result: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,
    accepted: (f = jspb.Message.getBooleanField(msg, 4)) == null ? undefined : f,
    inputPathsList: (f = jspb.Message.getRepeatedField(msg, 5)) == null ? undefined : f,
    outputPathsList: (f = jspb.Message.getRepeatedField(msg, 6)) == null ? undefined : f,
    elapsedTimeMs: (f = jspb.Message.getOptionalFloatingPointField(msg, 7)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of GrmApplicationDiagnostics as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.GrmApplicationDiagnostics.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.rule;

  /** @type {?string|undefined} */
  this.acceptedInput;

  /** @type {?string|undefined} */
  this.topInput;

  /** @type {?string|undefined} */
  this.input;

  /** @type {?string|undefined} */
  this.result;

  /** @type {?boolean|undefined} */
  this.accepted;

  /** @type {?Array<string>|undefined} */
  this.inputPathsList;

  /** @type {?Array<string>|undefined} */
  this.outputPathsList;

  /** @type {?number|undefined} */
  this.elapsedTimeMs;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.GrmApplicationDiagnostics.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.GrmApplicationDiagnostics}
 */
proto.speech.tts.GrmApplicationDiagnostics.fromObject = function(obj) {
  var msg = new proto.speech.tts.GrmApplicationDiagnostics();
  obj.rule != null && jspb.Message.setField(msg, 1, obj.rule);
  obj.acceptedInput != null && jspb.Message.setField(msg, 8, obj.acceptedInput);
  obj.topInput != null && jspb.Message.setField(msg, 9, obj.topInput);
  obj.input != null && jspb.Message.setField(msg, 2, obj.input);
  obj.result != null && jspb.Message.setField(msg, 3, obj.result);
  obj.accepted != null && jspb.Message.setField(msg, 4, obj.accepted);
  obj.inputPathsList != null && jspb.Message.setField(msg, 5, obj.inputPathsList);
  obj.outputPathsList != null && jspb.Message.setField(msg, 6, obj.outputPathsList);
  obj.elapsedTimeMs != null && jspb.Message.setField(msg, 7, obj.elapsedTimeMs);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.GrmApplicationDiagnostics}
 */
proto.speech.tts.GrmApplicationDiagnostics.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.GrmApplicationDiagnostics;
  return proto.speech.tts.GrmApplicationDiagnostics.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.GrmApplicationDiagnostics} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.GrmApplicationDiagnostics}
 */
proto.speech.tts.GrmApplicationDiagnostics.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setRule(value);
      break;
    case 8:
      var value = /** @type {string} */ (reader.readString());
      msg.setAcceptedInput(value);
      break;
    case 9:
      var value = /** @type {string} */ (reader.readString());
      msg.setTopInput(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setInput(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setResult(value);
      break;
    case 4:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setAccepted(value);
      break;
    case 5:
      var value = /** @type {string} */ (reader.readString());
      msg.addInputPaths(value);
      break;
    case 6:
      var value = /** @type {string} */ (reader.readString());
      msg.addOutputPaths(value);
      break;
    case 7:
      var value = /** @type {number} */ (reader.readDouble());
      msg.setElapsedTimeMs(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.GrmApplicationDiagnostics.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.GrmApplicationDiagnostics.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.GrmApplicationDiagnostics} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.GrmApplicationDiagnostics.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 8));
  if (f != null) {
    writer.writeString(
      8,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 9));
  if (f != null) {
    writer.writeString(
      9,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeString(
      3,
      f
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeBool(
      4,
      f
    );
  }
  f = message.getInputPathsList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      5,
      f
    );
  }
  f = message.getOutputPathsList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      6,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 7));
  if (f != null) {
    writer.writeDouble(
      7,
      f
    );
  }
};


/**
 * optional string rule = 1;
 * @return {?string}
 */
proto.speech.tts.GrmApplicationDiagnostics.prototype.getRule = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.GrmApplicationDiagnostics} returns this
 */
proto.speech.tts.GrmApplicationDiagnostics.prototype.setRule = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.GrmApplicationDiagnostics} returns this
 */
proto.speech.tts.GrmApplicationDiagnostics.prototype.clearRule = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.GrmApplicationDiagnostics.prototype.hasRule = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional string accepted_input = 8;
 * @return {?string}
 */
proto.speech.tts.GrmApplicationDiagnostics.prototype.getAcceptedInput = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 8));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.GrmApplicationDiagnostics} returns this
 */
proto.speech.tts.GrmApplicationDiagnostics.prototype.setAcceptedInput = function(value) {
  return jspb.Message.setField(this, 8, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.GrmApplicationDiagnostics} returns this
 */
proto.speech.tts.GrmApplicationDiagnostics.prototype.clearAcceptedInput = function() {
  return jspb.Message.setField(this, 8, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.GrmApplicationDiagnostics.prototype.hasAcceptedInput = function() {
  return jspb.Message.getField(this, 8) != null;
};


/**
 * optional string top_input = 9;
 * @return {?string}
 */
proto.speech.tts.GrmApplicationDiagnostics.prototype.getTopInput = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 9));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.GrmApplicationDiagnostics} returns this
 */
proto.speech.tts.GrmApplicationDiagnostics.prototype.setTopInput = function(value) {
  return jspb.Message.setField(this, 9, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.GrmApplicationDiagnostics} returns this
 */
proto.speech.tts.GrmApplicationDiagnostics.prototype.clearTopInput = function() {
  return jspb.Message.setField(this, 9, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.GrmApplicationDiagnostics.prototype.hasTopInput = function() {
  return jspb.Message.getField(this, 9) != null;
};


/**
 * optional string input = 2;
 * @return {?string}
 */
proto.speech.tts.GrmApplicationDiagnostics.prototype.getInput = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.GrmApplicationDiagnostics} returns this
 */
proto.speech.tts.GrmApplicationDiagnostics.prototype.setInput = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.GrmApplicationDiagnostics} returns this
 */
proto.speech.tts.GrmApplicationDiagnostics.prototype.clearInput = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.GrmApplicationDiagnostics.prototype.hasInput = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional string result = 3;
 * @return {?string}
 */
proto.speech.tts.GrmApplicationDiagnostics.prototype.getResult = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.GrmApplicationDiagnostics} returns this
 */
proto.speech.tts.GrmApplicationDiagnostics.prototype.setResult = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.GrmApplicationDiagnostics} returns this
 */
proto.speech.tts.GrmApplicationDiagnostics.prototype.clearResult = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.GrmApplicationDiagnostics.prototype.hasResult = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional bool accepted = 4;
 * @return {?boolean}
 */
proto.speech.tts.GrmApplicationDiagnostics.prototype.getAccepted = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 4));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.GrmApplicationDiagnostics} returns this
 */
proto.speech.tts.GrmApplicationDiagnostics.prototype.setAccepted = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.GrmApplicationDiagnostics} returns this
 */
proto.speech.tts.GrmApplicationDiagnostics.prototype.clearAccepted = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.GrmApplicationDiagnostics.prototype.hasAccepted = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * repeated string input_paths = 5;
 * @return {!Array<string>}
 */
proto.speech.tts.GrmApplicationDiagnostics.prototype.getInputPathsList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 5));
};


/**
 * @param {Array<string>|undefined} value
 * @return {!proto.speech.tts.GrmApplicationDiagnostics} returns this
 */
proto.speech.tts.GrmApplicationDiagnostics.prototype.setInputPathsList = function(value) {
  return jspb.Message.setField(this, 5, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.GrmApplicationDiagnostics} returns this
 */
proto.speech.tts.GrmApplicationDiagnostics.prototype.addInputPaths = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 5, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.GrmApplicationDiagnostics} returns this
 */
proto.speech.tts.GrmApplicationDiagnostics.prototype.clearInputPathsList = function() {
  return this.setInputPathsList([]);
};


/**
 * repeated string output_paths = 6;
 * @return {!Array<string>}
 */
proto.speech.tts.GrmApplicationDiagnostics.prototype.getOutputPathsList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 6));
};


/**
 * @param {Array<string>|undefined} value
 * @return {!proto.speech.tts.GrmApplicationDiagnostics} returns this
 */
proto.speech.tts.GrmApplicationDiagnostics.prototype.setOutputPathsList = function(value) {
  return jspb.Message.setField(this, 6, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.GrmApplicationDiagnostics} returns this
 */
proto.speech.tts.GrmApplicationDiagnostics.prototype.addOutputPaths = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 6, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.GrmApplicationDiagnostics} returns this
 */
proto.speech.tts.GrmApplicationDiagnostics.prototype.clearOutputPathsList = function() {
  return this.setOutputPathsList([]);
};


/**
 * optional double elapsed_time_ms = 7;
 * @return {?number}
 */
proto.speech.tts.GrmApplicationDiagnostics.prototype.getElapsedTimeMs = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 7));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.GrmApplicationDiagnostics} returns this
 */
proto.speech.tts.GrmApplicationDiagnostics.prototype.setElapsedTimeMs = function(value) {
  return jspb.Message.setField(this, 7, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.GrmApplicationDiagnostics} returns this
 */
proto.speech.tts.GrmApplicationDiagnostics.prototype.clearElapsedTimeMs = function() {
  return jspb.Message.setField(this, 7, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.GrmApplicationDiagnostics.prototype.hasElapsedTimeMs = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.GrmApplicationDiagnostics}
 */
proto.speech.tts.GrmApplicationDiagnostics.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.GrmApplicationDiagnostics, data);
};



//speech/tts/public/lucid/annotation_proto_jspb/crosslingual.jspb.js
// source: speech/tts/public/lucid/annotation.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.lucid.CrossLingual');
goog.provide('proto.speech.tts.lucid.CrossLingual.Style');
goog.provide('proto.speech.tts.lucid.CrossLingual.VerbalizationStyle');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.lucid.CrossLingual = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.lucid.CrossLingual.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.lucid.CrossLingual, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.lucid.CrossLingual.displayName = 'proto.speech.tts.lucid.CrossLingual';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.lucid.CrossLingual.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.lucid.CrossLingual.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.lucid.CrossLingual.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.lucid.CrossLingual} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.CrossLingual.toObject = function(includeInstance, msg) {
  var f, obj = {
    foreignLanguageHintsList: (f = jspb.Message.getRepeatedField(msg, 1)) == null ? undefined : f,
    style: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,
    verbalizationStyle: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of CrossLingual as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.lucid.CrossLingual.ObjectFormat = function() {
  /** @type {?Array<string>|undefined} */
  this.foreignLanguageHintsList;

  /** @type {?number|undefined} */
  this.style;

  /** @type {?number|undefined} */
  this.verbalizationStyle;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.lucid.CrossLingual.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.lucid.CrossLingual}
 */
proto.speech.tts.lucid.CrossLingual.fromObject = function(obj) {
  var msg = new proto.speech.tts.lucid.CrossLingual();
  obj.foreignLanguageHintsList != null && jspb.Message.setField(msg, 1, obj.foreignLanguageHintsList);
  obj.style != null && jspb.Message.setField(msg, 3, obj.style);
  obj.verbalizationStyle != null && jspb.Message.setField(msg, 4, obj.verbalizationStyle);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.lucid.CrossLingual}
 */
proto.speech.tts.lucid.CrossLingual.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.lucid.CrossLingual;
  return proto.speech.tts.lucid.CrossLingual.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.lucid.CrossLingual} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.lucid.CrossLingual}
 */
proto.speech.tts.lucid.CrossLingual.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.addForeignLanguageHints(value);
      break;
    case 3:
      var value = /** @type {!proto.speech.tts.lucid.CrossLingual.Style} */ (reader.readEnum());
      msg.setStyle(value);
      break;
    case 4:
      var value = /** @type {!proto.speech.tts.lucid.CrossLingual.VerbalizationStyle} */ (reader.readEnum());
      msg.setVerbalizationStyle(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.lucid.CrossLingual.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.lucid.CrossLingual.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.lucid.CrossLingual} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.CrossLingual.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getForeignLanguageHintsList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      1,
      f
    );
  }
  f = /** @type {!proto.speech.tts.lucid.CrossLingual.Style} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeEnum(
      3,
      f
    );
  }
  f = /** @type {!proto.speech.tts.lucid.CrossLingual.VerbalizationStyle} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeEnum(
      4,
      f
    );
  }
};


/**
 * @enum {number}
 */
proto.speech.tts.lucid.CrossLingual.Style = {
  UNSPECIFIED_STYLE: 0,
  NATIVE_PRON_PREFERRED: 1,
  FOREIGN_PRON_PREFERRED: 2,
  EXACT_FOREIGN_PRON_PREFERRED: 3,
  VOICE_SWITCH_ALLOWED: 4,
  REGISTER_CHOICE: 5
};

/**
 * @enum {number}
 */
proto.speech.tts.lucid.CrossLingual.VerbalizationStyle = {
  UNSPECIFIED_VERBALIZATION_STYLE: 0,
  NATIVE_VERBALIZATION: 1,
  FOREIGN_VERBALIZATION: 2
};

/**
 * repeated string foreign_language_hints = 1;
 * @return {!Array<string>}
 */
proto.speech.tts.lucid.CrossLingual.prototype.getForeignLanguageHintsList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 1));
};


/**
 * @param {Array<string>|undefined} value
 * @return {!proto.speech.tts.lucid.CrossLingual} returns this
 */
proto.speech.tts.lucid.CrossLingual.prototype.setForeignLanguageHintsList = function(value) {
  return jspb.Message.setField(this, 1, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.lucid.CrossLingual} returns this
 */
proto.speech.tts.lucid.CrossLingual.prototype.addForeignLanguageHints = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 1, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.lucid.CrossLingual} returns this
 */
proto.speech.tts.lucid.CrossLingual.prototype.clearForeignLanguageHintsList = function() {
  return this.setForeignLanguageHintsList([]);
};


/**
 * optional Style style = 3;
 * @return {!proto.speech.tts.lucid.CrossLingual.Style}
 */
proto.speech.tts.lucid.CrossLingual.prototype.getStyle = function() {
  return /** @type {!proto.speech.tts.lucid.CrossLingual.Style} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?proto.speech.tts.lucid.CrossLingual.Style|undefined} value
 * @return {!proto.speech.tts.lucid.CrossLingual} returns this
 */
proto.speech.tts.lucid.CrossLingual.prototype.setStyle = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.CrossLingual} returns this
 */
proto.speech.tts.lucid.CrossLingual.prototype.clearStyle = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.CrossLingual.prototype.hasStyle = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional VerbalizationStyle verbalization_style = 4;
 * @return {!proto.speech.tts.lucid.CrossLingual.VerbalizationStyle}
 */
proto.speech.tts.lucid.CrossLingual.prototype.getVerbalizationStyle = function() {
  return /** @type {!proto.speech.tts.lucid.CrossLingual.VerbalizationStyle} */ (jspb.Message.getField(this, 4));
};


/**
 * @param {?proto.speech.tts.lucid.CrossLingual.VerbalizationStyle|undefined} value
 * @return {!proto.speech.tts.lucid.CrossLingual} returns this
 */
proto.speech.tts.lucid.CrossLingual.prototype.setVerbalizationStyle = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.CrossLingual} returns this
 */
proto.speech.tts.lucid.CrossLingual.prototype.clearVerbalizationStyle = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.CrossLingual.prototype.hasVerbalizationStyle = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.lucid.CrossLingual}
 */
proto.speech.tts.lucid.CrossLingual.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.lucid.CrossLingual, data);
};



//speech/tts/public/lucid/prosody_proto_jspb/novelty.jspb.js
// source: speech/tts/public/lucid/prosody.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.lucid.Novelty');
goog.provide('proto.speech.tts.lucid.Novelty.NoveltyLevel');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.lucid.Novelty = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.lucid.Novelty, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.lucid.Novelty.displayName = 'proto.speech.tts.lucid.Novelty';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.lucid.Novelty.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.lucid.Novelty.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.lucid.Novelty} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.Novelty.toObject = function(includeInstance, msg) {
  var f, obj = {
    level: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Novelty as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.lucid.Novelty.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.level;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.lucid.Novelty.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.lucid.Novelty}
 */
proto.speech.tts.lucid.Novelty.fromObject = function(obj) {
  var msg = new proto.speech.tts.lucid.Novelty();
  obj.level != null && jspb.Message.setField(msg, 3, obj.level);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.lucid.Novelty}
 */
proto.speech.tts.lucid.Novelty.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.lucid.Novelty;
  return proto.speech.tts.lucid.Novelty.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.lucid.Novelty} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.lucid.Novelty}
 */
proto.speech.tts.lucid.Novelty.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 3:
      var value = /** @type {!proto.speech.tts.lucid.Novelty.NoveltyLevel} */ (reader.readEnum());
      msg.setLevel(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.lucid.Novelty.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.lucid.Novelty.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.lucid.Novelty} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.Novelty.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {!proto.speech.tts.lucid.Novelty.NoveltyLevel} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeEnum(
      3,
      f
    );
  }
};


/**
 * @enum {number}
 */
proto.speech.tts.lucid.Novelty.NoveltyLevel = {
  UNSPECIFIED_NOVELTY_LEVEL: 0,
  REPETITION: 1,
  RELATED: 2,
  NOVEL: 3,
  UNEXPECTED: 4
};

/**
 * optional NoveltyLevel level = 3;
 * @return {!proto.speech.tts.lucid.Novelty.NoveltyLevel}
 */
proto.speech.tts.lucid.Novelty.prototype.getLevel = function() {
  return /** @type {!proto.speech.tts.lucid.Novelty.NoveltyLevel} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?proto.speech.tts.lucid.Novelty.NoveltyLevel|undefined} value
 * @return {!proto.speech.tts.lucid.Novelty} returns this
 */
proto.speech.tts.lucid.Novelty.prototype.setLevel = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Novelty} returns this
 */
proto.speech.tts.lucid.Novelty.prototype.clearLevel = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Novelty.prototype.hasLevel = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.lucid.Novelty}
 */
proto.speech.tts.lucid.Novelty.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.lucid.Novelty, data);
};



//speech/tts/public/lucid/prosody_proto_jspb/partialsentenceprosody.jspb.js
// source: speech/tts/public/lucid/prosody.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.lucid.PartialSentenceProsody');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.lucid.PartialSentenceProsody = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.lucid.PartialSentenceProsody, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.lucid.PartialSentenceProsody.displayName = 'proto.speech.tts.lucid.PartialSentenceProsody';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.lucid.PartialSentenceProsody.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.lucid.PartialSentenceProsody.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.lucid.PartialSentenceProsody} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.PartialSentenceProsody.toObject = function(includeInstance, msg) {
  var f, obj = {
    incompleteAtStart: (f = jspb.Message.getBooleanField(msg, 1)) == null ? undefined : f,
    incompleteAtEnd: (f = jspb.Message.getBooleanField(msg, 2)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of PartialSentenceProsody as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.lucid.PartialSentenceProsody.ObjectFormat = function() {
  /** @type {?boolean|undefined} */
  this.incompleteAtStart;

  /** @type {?boolean|undefined} */
  this.incompleteAtEnd;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.lucid.PartialSentenceProsody.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.lucid.PartialSentenceProsody}
 */
proto.speech.tts.lucid.PartialSentenceProsody.fromObject = function(obj) {
  var msg = new proto.speech.tts.lucid.PartialSentenceProsody();
  obj.incompleteAtStart != null && jspb.Message.setField(msg, 1, obj.incompleteAtStart);
  obj.incompleteAtEnd != null && jspb.Message.setField(msg, 2, obj.incompleteAtEnd);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.lucid.PartialSentenceProsody}
 */
proto.speech.tts.lucid.PartialSentenceProsody.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.lucid.PartialSentenceProsody;
  return proto.speech.tts.lucid.PartialSentenceProsody.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.lucid.PartialSentenceProsody} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.lucid.PartialSentenceProsody}
 */
proto.speech.tts.lucid.PartialSentenceProsody.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIncompleteAtStart(value);
      break;
    case 2:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIncompleteAtEnd(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.lucid.PartialSentenceProsody.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.lucid.PartialSentenceProsody.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.lucid.PartialSentenceProsody} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.PartialSentenceProsody.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeBool(
      1,
      f
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeBool(
      2,
      f
    );
  }
};


/**
 * optional bool incomplete_at_start = 1;
 * @return {?boolean}
 */
proto.speech.tts.lucid.PartialSentenceProsody.prototype.getIncompleteAtStart = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 1));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.lucid.PartialSentenceProsody} returns this
 */
proto.speech.tts.lucid.PartialSentenceProsody.prototype.setIncompleteAtStart = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.PartialSentenceProsody} returns this
 */
proto.speech.tts.lucid.PartialSentenceProsody.prototype.clearIncompleteAtStart = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.PartialSentenceProsody.prototype.hasIncompleteAtStart = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional bool incomplete_at_end = 2;
 * @return {?boolean}
 */
proto.speech.tts.lucid.PartialSentenceProsody.prototype.getIncompleteAtEnd = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 2));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.lucid.PartialSentenceProsody} returns this
 */
proto.speech.tts.lucid.PartialSentenceProsody.prototype.setIncompleteAtEnd = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.PartialSentenceProsody} returns this
 */
proto.speech.tts.lucid.PartialSentenceProsody.prototype.clearIncompleteAtEnd = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.PartialSentenceProsody.prototype.hasIncompleteAtEnd = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.lucid.PartialSentenceProsody}
 */
proto.speech.tts.lucid.PartialSentenceProsody.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.lucid.PartialSentenceProsody, data);
};



//speech/tts/public/lucid/prosody_proto_jspb/prominence.jspb.js
// source: speech/tts/public/lucid/prosody.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.lucid.Prominence');
goog.provide('proto.speech.tts.lucid.Prominence.ProminenceLevel');
goog.provide('proto.speech.tts.lucid.Prominence.ProminenceType');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.lucid.Prominence = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.lucid.Prominence, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.lucid.Prominence.displayName = 'proto.speech.tts.lucid.Prominence';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.lucid.Prominence.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.lucid.Prominence.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.lucid.Prominence} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.Prominence.toObject = function(includeInstance, msg) {
  var f, obj = {
    level: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    type: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Prominence as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.lucid.Prominence.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.level;

  /** @type {?number|undefined} */
  this.type;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.lucid.Prominence.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.lucid.Prominence}
 */
proto.speech.tts.lucid.Prominence.fromObject = function(obj) {
  var msg = new proto.speech.tts.lucid.Prominence();
  obj.level != null && jspb.Message.setField(msg, 1, obj.level);
  obj.type != null && jspb.Message.setField(msg, 2, obj.type);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.lucid.Prominence}
 */
proto.speech.tts.lucid.Prominence.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.lucid.Prominence;
  return proto.speech.tts.lucid.Prominence.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.lucid.Prominence} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.lucid.Prominence}
 */
proto.speech.tts.lucid.Prominence.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {!proto.speech.tts.lucid.Prominence.ProminenceLevel} */ (reader.readEnum());
      msg.setLevel(value);
      break;
    case 2:
      var value = /** @type {!proto.speech.tts.lucid.Prominence.ProminenceType} */ (reader.readEnum());
      msg.setType(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.lucid.Prominence.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.lucid.Prominence.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.lucid.Prominence} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.Prominence.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {!proto.speech.tts.lucid.Prominence.ProminenceLevel} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeEnum(
      1,
      f
    );
  }
  f = /** @type {!proto.speech.tts.lucid.Prominence.ProminenceType} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeEnum(
      2,
      f
    );
  }
};


/**
 * @enum {number}
 */
proto.speech.tts.lucid.Prominence.ProminenceLevel = {
  UNSPECIFIED_PROMINENCE_LEVEL: 0,
  DEACCENTED: 1,
  NONE: 2,
  MILD: 3,
  STRONG: 4
};

/**
 * @enum {number}
 */
proto.speech.tts.lucid.Prominence.ProminenceType = {
  UNSPECIFIED_PROMINENCE_TYPE: 0,
  STANDARD: 1,
  EMPHATIC: 2,
  CONTRASTIVE_FIRST: 3,
  CONTRASTIVE_SECOND: 4
};

/**
 * optional ProminenceLevel level = 1;
 * @return {!proto.speech.tts.lucid.Prominence.ProminenceLevel}
 */
proto.speech.tts.lucid.Prominence.prototype.getLevel = function() {
  return /** @type {!proto.speech.tts.lucid.Prominence.ProminenceLevel} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?proto.speech.tts.lucid.Prominence.ProminenceLevel|undefined} value
 * @return {!proto.speech.tts.lucid.Prominence} returns this
 */
proto.speech.tts.lucid.Prominence.prototype.setLevel = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Prominence} returns this
 */
proto.speech.tts.lucid.Prominence.prototype.clearLevel = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Prominence.prototype.hasLevel = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional ProminenceType type = 2;
 * @return {!proto.speech.tts.lucid.Prominence.ProminenceType}
 */
proto.speech.tts.lucid.Prominence.prototype.getType = function() {
  return /** @type {!proto.speech.tts.lucid.Prominence.ProminenceType} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?proto.speech.tts.lucid.Prominence.ProminenceType|undefined} value
 * @return {!proto.speech.tts.lucid.Prominence} returns this
 */
proto.speech.tts.lucid.Prominence.prototype.setType = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Prominence} returns this
 */
proto.speech.tts.lucid.Prominence.prototype.clearType = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Prominence.prototype.hasType = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.lucid.Prominence}
 */
proto.speech.tts.lucid.Prominence.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.lucid.Prominence, data);
};



//speech/tts/public/lucid/prosody_proto_jspb/prosody.jspb.js
// source: speech/tts/public/lucid/prosody.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.lucid.Prosody');
goog.provide('proto.speech.tts.lucid.Prosody.SpeechRate');
goog.provide('proto.speech.tts.lucid.Prosody.SpeechRateSpecCase');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.lucid.Novelty');
goog.require('proto.speech.tts.lucid.PartialSentenceProsody');
goog.require('proto.speech.tts.lucid.Prominence');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.lucid.Prosody = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, proto.speech.tts.lucid.Prosody.oneofGroups_);
};
goog.inherits(proto.speech.tts.lucid.Prosody, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.lucid.Prosody.displayName = 'proto.speech.tts.lucid.Prosody';
}

/**
 * Oneof group definitions for this message. Each group defines the field
 * numbers belonging to that group. When of these fields' value is set, all
 * other fields in the group are cleared. During deserialization, if multiple
 * fields are encountered for a group, only the last value seen will be kept.
 * @private {!Array<!Array<number>>}
 * @const
 */
proto.speech.tts.lucid.Prosody.oneofGroups_ = [[1,4]];

/**
 * @enum {number}
 */
proto.speech.tts.lucid.Prosody.SpeechRateSpecCase = {
  SPEECH_RATE_SPEC_NOT_SET: 0,
  SPEECH_RATE: 1,
  SPEECH_RATE_ENUM: 4
};

/**
 * @return {!proto.speech.tts.lucid.Prosody.SpeechRateSpecCase}
 */
proto.speech.tts.lucid.Prosody.prototype.getSpeechRateSpecCase = function() {
  return /** @type {!proto.speech.tts.lucid.Prosody.SpeechRateSpecCase} */(jspb.Message.computeOneofCase(this, proto.speech.tts.lucid.Prosody.oneofGroups_[0]));
};



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.lucid.Prosody.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.lucid.Prosody.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.lucid.Prosody} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.Prosody.toObject = function(includeInstance, msg) {
  var f, obj = {
    speechRate: (f = jspb.Message.getOptionalFloatingPointField(msg, 1)) == null ? undefined : f,
    speechRateEnum: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f,
    prominence: (f = msg.getProminence()) && proto.speech.tts.lucid.Prominence.toObject(includeInstance, f),
    novelty: (f = msg.getNovelty()) && proto.speech.tts.lucid.Novelty.toObject(includeInstance, f),
    partialSentenceProsody: (f = msg.getPartialSentenceProsody()) && proto.speech.tts.lucid.PartialSentenceProsody.toObject(includeInstance, f),
    f0Scale: (f = jspb.Message.getOptionalFloatingPointField(msg, 6)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Prosody as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.lucid.Prosody.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.speechRate;

  /** @type {?number|undefined} */
  this.speechRateEnum;

  /** @type {?proto.speech.tts.lucid.Prominence.ObjectFormat|undefined} */
  this.prominence;

  /** @type {?proto.speech.tts.lucid.Novelty.ObjectFormat|undefined} */
  this.novelty;

  /** @type {?proto.speech.tts.lucid.PartialSentenceProsody.ObjectFormat|undefined} */
  this.partialSentenceProsody;

  /** @type {?number|undefined} */
  this.f0Scale;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.lucid.Prosody.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.lucid.Prosody}
 */
proto.speech.tts.lucid.Prosody.fromObject = function(obj) {
  var msg = new proto.speech.tts.lucid.Prosody();
  obj.speechRate != null && jspb.Message.setField(msg, 1, obj.speechRate);
  obj.speechRateEnum != null && jspb.Message.setField(msg, 4, obj.speechRateEnum);
  obj.prominence && jspb.Message.setWrapperField(
      msg, 2, proto.speech.tts.lucid.Prominence.fromObject(obj.prominence));
  obj.novelty && jspb.Message.setWrapperField(
      msg, 3, proto.speech.tts.lucid.Novelty.fromObject(obj.novelty));
  obj.partialSentenceProsody && jspb.Message.setWrapperField(
      msg, 5, proto.speech.tts.lucid.PartialSentenceProsody.fromObject(obj.partialSentenceProsody));
  obj.f0Scale != null && jspb.Message.setField(msg, 6, obj.f0Scale);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.lucid.Prosody}
 */
proto.speech.tts.lucid.Prosody.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.lucid.Prosody;
  return proto.speech.tts.lucid.Prosody.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.lucid.Prosody} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.lucid.Prosody}
 */
proto.speech.tts.lucid.Prosody.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setSpeechRate(value);
      break;
    case 4:
      var value = /** @type {!proto.speech.tts.lucid.Prosody.SpeechRate} */ (reader.readEnum());
      msg.setSpeechRateEnum(value);
      break;
    case 2:
      var value = new proto.speech.tts.lucid.Prominence;
      reader.readMessage(value,proto.speech.tts.lucid.Prominence.deserializeBinaryFromReader);
      msg.setProminence(value);
      break;
    case 3:
      var value = new proto.speech.tts.lucid.Novelty;
      reader.readMessage(value,proto.speech.tts.lucid.Novelty.deserializeBinaryFromReader);
      msg.setNovelty(value);
      break;
    case 5:
      var value = new proto.speech.tts.lucid.PartialSentenceProsody;
      reader.readMessage(value,proto.speech.tts.lucid.PartialSentenceProsody.deserializeBinaryFromReader);
      msg.setPartialSentenceProsody(value);
      break;
    case 6:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setF0Scale(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.lucid.Prosody.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.lucid.Prosody.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.lucid.Prosody} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.Prosody.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?number} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeFloat(
      1,
      f
    );
  }
  f = /** @type {!proto.speech.tts.lucid.Prosody.SpeechRate} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeEnum(
      4,
      f
    );
  }
  f = message.getProminence();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.speech.tts.lucid.Prominence.serializeBinaryToWriter
    );
  }
  f = message.getNovelty();
  if (f != null) {
    writer.writeMessage(
      3,
      f,
      proto.speech.tts.lucid.Novelty.serializeBinaryToWriter
    );
  }
  f = message.getPartialSentenceProsody();
  if (f != null) {
    writer.writeMessage(
      5,
      f,
      proto.speech.tts.lucid.PartialSentenceProsody.serializeBinaryToWriter
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeFloat(
      6,
      f
    );
  }
};


/**
 * @enum {number}
 */
proto.speech.tts.lucid.Prosody.SpeechRate = {
  UNSPECIFIED_SPEECH_RATE: 0,
  VERY_SLOW: 1,
  SLOW: 2,
  NORMAL: 3,
  FAST: 4,
  VERY_FAST: 5
};

/**
 * optional float speech_rate = 1;
 * @return {?number}
 */
proto.speech.tts.lucid.Prosody.prototype.getSpeechRate = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 1));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.lucid.Prosody} returns this
 */
proto.speech.tts.lucid.Prosody.prototype.setSpeechRate = function(value) {
  return jspb.Message.setOneofField(this, 1, proto.speech.tts.lucid.Prosody.oneofGroups_[0], value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Prosody} returns this
 */
proto.speech.tts.lucid.Prosody.prototype.clearSpeechRate = function() {
  return jspb.Message.setOneofField(this, 1, proto.speech.tts.lucid.Prosody.oneofGroups_[0], undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Prosody.prototype.hasSpeechRate = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional SpeechRate speech_rate_enum = 4;
 * @return {!proto.speech.tts.lucid.Prosody.SpeechRate}
 */
proto.speech.tts.lucid.Prosody.prototype.getSpeechRateEnum = function() {
  return /** @type {!proto.speech.tts.lucid.Prosody.SpeechRate} */ (jspb.Message.getField(this, 4));
};


/**
 * @param {?proto.speech.tts.lucid.Prosody.SpeechRate|undefined} value
 * @return {!proto.speech.tts.lucid.Prosody} returns this
 */
proto.speech.tts.lucid.Prosody.prototype.setSpeechRateEnum = function(value) {
  return jspb.Message.setOneofField(this, 4, proto.speech.tts.lucid.Prosody.oneofGroups_[0], value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Prosody} returns this
 */
proto.speech.tts.lucid.Prosody.prototype.clearSpeechRateEnum = function() {
  return jspb.Message.setOneofField(this, 4, proto.speech.tts.lucid.Prosody.oneofGroups_[0], undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Prosody.prototype.hasSpeechRateEnum = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional Prominence prominence = 2;
 * @return {?proto.speech.tts.lucid.Prominence}
 */
proto.speech.tts.lucid.Prosody.prototype.getProminence = function() {
  return /** @type{?proto.speech.tts.lucid.Prominence} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.lucid.Prominence, 2));
};


/**
 * @param {?proto.speech.tts.lucid.Prominence|undefined} value
 * @return {!proto.speech.tts.lucid.Prosody} returns this
*/
proto.speech.tts.lucid.Prosody.prototype.setProminence = function(value) {
  return jspb.Message.setWrapperField(this, 2, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.lucid.Prosody} returns this
 */
proto.speech.tts.lucid.Prosody.prototype.clearProminence = function() {
  return this.setProminence(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Prosody.prototype.hasProminence = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional Novelty novelty = 3;
 * @return {?proto.speech.tts.lucid.Novelty}
 */
proto.speech.tts.lucid.Prosody.prototype.getNovelty = function() {
  return /** @type{?proto.speech.tts.lucid.Novelty} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.lucid.Novelty, 3));
};


/**
 * @param {?proto.speech.tts.lucid.Novelty|undefined} value
 * @return {!proto.speech.tts.lucid.Prosody} returns this
*/
proto.speech.tts.lucid.Prosody.prototype.setNovelty = function(value) {
  return jspb.Message.setWrapperField(this, 3, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.lucid.Prosody} returns this
 */
proto.speech.tts.lucid.Prosody.prototype.clearNovelty = function() {
  return this.setNovelty(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Prosody.prototype.hasNovelty = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional PartialSentenceProsody partial_sentence_prosody = 5;
 * @return {?proto.speech.tts.lucid.PartialSentenceProsody}
 */
proto.speech.tts.lucid.Prosody.prototype.getPartialSentenceProsody = function() {
  return /** @type{?proto.speech.tts.lucid.PartialSentenceProsody} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.lucid.PartialSentenceProsody, 5));
};


/**
 * @param {?proto.speech.tts.lucid.PartialSentenceProsody|undefined} value
 * @return {!proto.speech.tts.lucid.Prosody} returns this
*/
proto.speech.tts.lucid.Prosody.prototype.setPartialSentenceProsody = function(value) {
  return jspb.Message.setWrapperField(this, 5, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.lucid.Prosody} returns this
 */
proto.speech.tts.lucid.Prosody.prototype.clearPartialSentenceProsody = function() {
  return this.setPartialSentenceProsody(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Prosody.prototype.hasPartialSentenceProsody = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional float f0_scale = 6;
 * @return {?number}
 */
proto.speech.tts.lucid.Prosody.prototype.getF0Scale = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 6));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.lucid.Prosody} returns this
 */
proto.speech.tts.lucid.Prosody.prototype.setF0Scale = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Prosody} returns this
 */
proto.speech.tts.lucid.Prosody.prototype.clearF0Scale = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Prosody.prototype.hasF0Scale = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.lucid.Prosody}
 */
proto.speech.tts.lucid.Prosody.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.lucid.Prosody, data);
};



//speech/tts/public/lucid/semantics_proto_jspb/listinfo.jspb.js
// source: speech/tts/public/lucid/semantics.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.lucid.ListInfo');
goog.provide('proto.speech.tts.lucid.ListInfo.EntityType');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.lucid.ListInfo = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.lucid.ListInfo, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.lucid.ListInfo.displayName = 'proto.speech.tts.lucid.ListInfo';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.lucid.ListInfo.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.lucid.ListInfo.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.lucid.ListInfo} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.ListInfo.toObject = function(includeInstance, msg) {
  var f, obj = {
    listId: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    type: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of ListInfo as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.lucid.ListInfo.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.listId;

  /** @type {?number|undefined} */
  this.type;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.lucid.ListInfo.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.lucid.ListInfo}
 */
proto.speech.tts.lucid.ListInfo.fromObject = function(obj) {
  var msg = new proto.speech.tts.lucid.ListInfo();
  obj.listId != null && jspb.Message.setField(msg, 1, obj.listId);
  obj.type != null && jspb.Message.setField(msg, 2, obj.type);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.lucid.ListInfo}
 */
proto.speech.tts.lucid.ListInfo.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.lucid.ListInfo;
  return proto.speech.tts.lucid.ListInfo.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.lucid.ListInfo} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.lucid.ListInfo}
 */
proto.speech.tts.lucid.ListInfo.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setListId(value);
      break;
    case 2:
      var value = /** @type {!proto.speech.tts.lucid.ListInfo.EntityType} */ (reader.readEnum());
      msg.setType(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.lucid.ListInfo.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.lucid.ListInfo.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.lucid.ListInfo} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.ListInfo.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?number} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeInt32(
      1,
      f
    );
  }
  f = /** @type {!proto.speech.tts.lucid.ListInfo.EntityType} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeEnum(
      2,
      f
    );
  }
};


/**
 * @enum {number}
 */
proto.speech.tts.lucid.ListInfo.EntityType = {
  UNSPECIFIED_ENTITY_TYPE: 0,
  ENTIRE_LIST: 1,
  LIST_ITEM: 2,
  LIST_ITEM_HEADER: 3,
  CONNECTOR: 4
};

/**
 * optional int32 list_id = 1;
 * @return {?number}
 */
proto.speech.tts.lucid.ListInfo.prototype.getListId = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.lucid.ListInfo} returns this
 */
proto.speech.tts.lucid.ListInfo.prototype.setListId = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.ListInfo} returns this
 */
proto.speech.tts.lucid.ListInfo.prototype.clearListId = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.ListInfo.prototype.hasListId = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional EntityType type = 2;
 * @return {!proto.speech.tts.lucid.ListInfo.EntityType}
 */
proto.speech.tts.lucid.ListInfo.prototype.getType = function() {
  return /** @type {!proto.speech.tts.lucid.ListInfo.EntityType} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?proto.speech.tts.lucid.ListInfo.EntityType|undefined} value
 * @return {!proto.speech.tts.lucid.ListInfo} returns this
 */
proto.speech.tts.lucid.ListInfo.prototype.setType = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.ListInfo} returns this
 */
proto.speech.tts.lucid.ListInfo.prototype.clearType = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.ListInfo.prototype.hasType = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.lucid.ListInfo}
 */
proto.speech.tts.lucid.ListInfo.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.lucid.ListInfo, data);
};



//speech/tts/public/lucid/semantics_proto_jspb/namedentity.jspb.js
// source: speech/tts/public/lucid/semantics.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.lucid.NamedEntity');
goog.provide('proto.speech.tts.lucid.NamedEntity.Domain');
goog.provide('proto.speech.tts.lucid.NamedEntity.SemanticGender');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.lucid.NamedEntity = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.lucid.NamedEntity, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.lucid.NamedEntity.displayName = 'proto.speech.tts.lucid.NamedEntity';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.lucid.NamedEntity.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.lucid.NamedEntity.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.lucid.NamedEntity} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.NamedEntity.toObject = function(includeInstance, msg) {
  var f, obj = {
    type: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    domain: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    confidenceScore: (f = jspb.Message.getOptionalFloatingPointField(msg, 3)) == null ? undefined : f,
    semanticGender: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of NamedEntity as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.lucid.NamedEntity.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.type;

  /** @type {?number|undefined} */
  this.domain;

  /** @type {?number|undefined} */
  this.confidenceScore;

  /** @type {?number|undefined} */
  this.semanticGender;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.lucid.NamedEntity.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.lucid.NamedEntity}
 */
proto.speech.tts.lucid.NamedEntity.fromObject = function(obj) {
  var msg = new proto.speech.tts.lucid.NamedEntity();
  obj.type != null && jspb.Message.setField(msg, 1, obj.type);
  obj.domain != null && jspb.Message.setField(msg, 2, obj.domain);
  obj.confidenceScore != null && jspb.Message.setField(msg, 3, obj.confidenceScore);
  obj.semanticGender != null && jspb.Message.setField(msg, 4, obj.semanticGender);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.lucid.NamedEntity}
 */
proto.speech.tts.lucid.NamedEntity.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.lucid.NamedEntity;
  return proto.speech.tts.lucid.NamedEntity.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.lucid.NamedEntity} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.lucid.NamedEntity}
 */
proto.speech.tts.lucid.NamedEntity.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setType(value);
      break;
    case 2:
      var value = /** @type {!proto.speech.tts.lucid.NamedEntity.Domain} */ (reader.readEnum());
      msg.setDomain(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setConfidenceScore(value);
      break;
    case 4:
      var value = /** @type {!proto.speech.tts.lucid.NamedEntity.SemanticGender} */ (reader.readEnum());
      msg.setSemanticGender(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.lucid.NamedEntity.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.lucid.NamedEntity.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.lucid.NamedEntity} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.NamedEntity.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = /** @type {!proto.speech.tts.lucid.NamedEntity.Domain} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeEnum(
      2,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeFloat(
      3,
      f
    );
  }
  f = /** @type {!proto.speech.tts.lucid.NamedEntity.SemanticGender} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeEnum(
      4,
      f
    );
  }
};


/**
 * @enum {number}
 */
proto.speech.tts.lucid.NamedEntity.Domain = {
  UNSPECIFIED_DOMAIN: 0,
  SAFT_COARSE: 1,
  SAFT_FINE: 2,
  COLLECTIONS: 3
};

/**
 * @enum {number}
 */
proto.speech.tts.lucid.NamedEntity.SemanticGender = {
  UNSPECIFIED_GENDER: 0,
  GENDER_FEMALE: 1,
  GENDER_MALE: 2,
  GENDER_OTHER: 3,
  GENDER_NONE: 4
};

/**
 * optional string type = 1;
 * @return {?string}
 */
proto.speech.tts.lucid.NamedEntity.prototype.getType = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.lucid.NamedEntity} returns this
 */
proto.speech.tts.lucid.NamedEntity.prototype.setType = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.NamedEntity} returns this
 */
proto.speech.tts.lucid.NamedEntity.prototype.clearType = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.NamedEntity.prototype.hasType = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional Domain domain = 2;
 * @return {!proto.speech.tts.lucid.NamedEntity.Domain}
 */
proto.speech.tts.lucid.NamedEntity.prototype.getDomain = function() {
  return /** @type {!proto.speech.tts.lucid.NamedEntity.Domain} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?proto.speech.tts.lucid.NamedEntity.Domain|undefined} value
 * @return {!proto.speech.tts.lucid.NamedEntity} returns this
 */
proto.speech.tts.lucid.NamedEntity.prototype.setDomain = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.NamedEntity} returns this
 */
proto.speech.tts.lucid.NamedEntity.prototype.clearDomain = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.NamedEntity.prototype.hasDomain = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional float confidence_score = 3;
 * @return {?number}
 */
proto.speech.tts.lucid.NamedEntity.prototype.getConfidenceScore = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 3));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.lucid.NamedEntity} returns this
 */
proto.speech.tts.lucid.NamedEntity.prototype.setConfidenceScore = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.NamedEntity} returns this
 */
proto.speech.tts.lucid.NamedEntity.prototype.clearConfidenceScore = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.NamedEntity.prototype.hasConfidenceScore = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional SemanticGender semantic_gender = 4;
 * @return {!proto.speech.tts.lucid.NamedEntity.SemanticGender}
 */
proto.speech.tts.lucid.NamedEntity.prototype.getSemanticGender = function() {
  return /** @type {!proto.speech.tts.lucid.NamedEntity.SemanticGender} */ (jspb.Message.getField(this, 4));
};


/**
 * @param {?proto.speech.tts.lucid.NamedEntity.SemanticGender|undefined} value
 * @return {!proto.speech.tts.lucid.NamedEntity} returns this
 */
proto.speech.tts.lucid.NamedEntity.prototype.setSemanticGender = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.NamedEntity} returns this
 */
proto.speech.tts.lucid.NamedEntity.prototype.clearSemanticGender = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.NamedEntity.prototype.hasSemanticGender = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.lucid.NamedEntity}
 */
proto.speech.tts.lucid.NamedEntity.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.lucid.NamedEntity, data);
};



//speech/tts/public/lucid/semantics_proto_jspb/semantics.jspb.js
// source: speech/tts/public/lucid/semantics.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.lucid.Semantics');
goog.provide('proto.speech.tts.lucid.Semantics.SemioticInfo');
goog.provide('proto.speech.tts.lucid.Semantics.SemioticInfo.HintLevel');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.location.unified.LocationDescriptor');
goog.require('proto.speech.tts.lucid.ListInfo');
goog.require('proto.speech.tts.lucid.NamedEntity');

goog.forwardDeclare('proto.speech.tts.lucid.SemioticType.Type');
goog.forwardDeclare('proto.speech.tts.lucid.SourceInfo.Source');
goog.forwardDeclare('proto.speech.tts.lucid.TextnormLevel');
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.lucid.Semantics = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.lucid.Semantics.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.lucid.Semantics, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.lucid.Semantics.displayName = 'proto.speech.tts.lucid.Semantics';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.lucid.Semantics.SemioticInfo = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.lucid.Semantics.SemioticInfo, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.lucid.Semantics.SemioticInfo.displayName = 'proto.speech.tts.lucid.Semantics.SemioticInfo';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.lucid.Semantics.repeatedFields_ = [1,2,4,5];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.lucid.Semantics.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.lucid.Semantics.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.lucid.Semantics} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.Semantics.toObject = function(includeInstance, msg) {
  var f, obj = {
    semioticInfoList: jspb.Message.toObjectList(msg.getSemioticInfoList(),
    proto.speech.tts.lucid.Semantics.SemioticInfo.toObject, includeInstance),
    sourceInfoList: (f = jspb.Message.getRepeatedField(msg, 2)) == null ? undefined : f,
    location: (f = msg.getLocation()) && proto.location.unified.LocationDescriptor.toObject(includeInstance, f),
    knowledgeGraphMidsList: (f = jspb.Message.getRepeatedField(msg, 4)) == null ? undefined : f,
    namedEntitiesList: jspb.Message.toObjectList(msg.getNamedEntitiesList(),
    proto.speech.tts.lucid.NamedEntity.toObject, includeInstance),
    listInfo: (f = msg.getListInfo()) && proto.speech.tts.lucid.ListInfo.toObject(includeInstance, f),
    textnormLevel: (f = jspb.Message.getField(msg, 7)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Semantics as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.lucid.Semantics.ObjectFormat = function() {
  /** @type {?Array<!proto.speech.tts.lucid.Semantics.SemioticInfo.ObjectFormat>|undefined} */
  this.semioticInfoList;

  /** @type {?Array<number>|undefined} */
  this.sourceInfoList;

  /** @type {?proto.location.unified.LocationDescriptor.ObjectFormat|undefined} */
  this.location;

  /** @type {?Array<string>|undefined} */
  this.knowledgeGraphMidsList;

  /** @type {?Array<!proto.speech.tts.lucid.NamedEntity.ObjectFormat>|undefined} */
  this.namedEntitiesList;

  /** @type {?proto.speech.tts.lucid.ListInfo.ObjectFormat|undefined} */
  this.listInfo;

  /** @type {?number|undefined} */
  this.textnormLevel;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.lucid.Semantics.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.lucid.Semantics}
 */
proto.speech.tts.lucid.Semantics.fromObject = function(obj) {
  var msg = new proto.speech.tts.lucid.Semantics();
  obj.semioticInfoList && jspb.Message.setRepeatedWrapperField(
      msg, 1, obj.semioticInfoList.map(
          proto.speech.tts.lucid.Semantics.SemioticInfo.fromObject));
  obj.sourceInfoList != null && jspb.Message.setField(msg, 2, obj.sourceInfoList);
  obj.location && jspb.Message.setWrapperField(
      msg, 3, proto.location.unified.LocationDescriptor.fromObject(obj.location));
  obj.knowledgeGraphMidsList != null && jspb.Message.setField(msg, 4, obj.knowledgeGraphMidsList);
  obj.namedEntitiesList && jspb.Message.setRepeatedWrapperField(
      msg, 5, obj.namedEntitiesList.map(
          proto.speech.tts.lucid.NamedEntity.fromObject));
  obj.listInfo && jspb.Message.setWrapperField(
      msg, 6, proto.speech.tts.lucid.ListInfo.fromObject(obj.listInfo));
  obj.textnormLevel != null && jspb.Message.setField(msg, 7, obj.textnormLevel);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.lucid.Semantics}
 */
proto.speech.tts.lucid.Semantics.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.lucid.Semantics;
  return proto.speech.tts.lucid.Semantics.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.lucid.Semantics} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.lucid.Semantics}
 */
proto.speech.tts.lucid.Semantics.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.speech.tts.lucid.Semantics.SemioticInfo;
      reader.readMessage(value,proto.speech.tts.lucid.Semantics.SemioticInfo.deserializeBinaryFromReader);
      msg.addSemioticInfo(value);
      break;
    case 2:
      var value = /** @type {!proto.speech.tts.lucid.SourceInfo.Source} */ (reader.readEnum());
      msg.addSourceInfo(value);
      break;
    case 3:
      var value = new proto.location.unified.LocationDescriptor;
      reader.readMessage(value,proto.location.unified.LocationDescriptor.deserializeBinaryFromReader);
      msg.setLocation(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.addKnowledgeGraphMids(value);
      break;
    case 5:
      var value = new proto.speech.tts.lucid.NamedEntity;
      reader.readMessage(value,proto.speech.tts.lucid.NamedEntity.deserializeBinaryFromReader);
      msg.addNamedEntities(value);
      break;
    case 6:
      var value = new proto.speech.tts.lucid.ListInfo;
      reader.readMessage(value,proto.speech.tts.lucid.ListInfo.deserializeBinaryFromReader);
      msg.setListInfo(value);
      break;
    case 7:
      var value = /** @type {!proto.speech.tts.lucid.TextnormLevel} */ (reader.readEnum());
      msg.setTextnormLevel(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.lucid.Semantics.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.lucid.Semantics.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.lucid.Semantics} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.Semantics.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getSemioticInfoList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      1,
      f,
      proto.speech.tts.lucid.Semantics.SemioticInfo.serializeBinaryToWriter
    );
  }
  f = message.getSourceInfoList();
  if (f.length > 0) {
    writer.writeRepeatedEnum(
      2,
      f
    );
  }
  f = message.getLocation();
  if (f != null) {
    writer.writeMessage(
      3,
      f,
      proto.location.unified.LocationDescriptor.serializeBinaryToWriter
    );
  }
  f = message.getKnowledgeGraphMidsList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      4,
      f
    );
  }
  f = message.getNamedEntitiesList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      5,
      f,
      proto.speech.tts.lucid.NamedEntity.serializeBinaryToWriter
    );
  }
  f = message.getListInfo();
  if (f != null) {
    writer.writeMessage(
      6,
      f,
      proto.speech.tts.lucid.ListInfo.serializeBinaryToWriter
    );
  }
  f = /** @type {!proto.speech.tts.lucid.TextnormLevel} */ (jspb.Message.getField(message, 7));
  if (f != null) {
    writer.writeEnum(
      7,
      f
    );
  }
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.lucid.Semantics.SemioticInfo.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.lucid.Semantics.SemioticInfo.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.lucid.Semantics.SemioticInfo} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.Semantics.SemioticInfo.toObject = function(includeInstance, msg) {
  var f, obj = {
    semioticType: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    hintLevel: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of SemioticInfo as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.lucid.Semantics.SemioticInfo.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.semioticType;

  /** @type {?number|undefined} */
  this.hintLevel;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.lucid.Semantics.SemioticInfo.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.lucid.Semantics.SemioticInfo}
 */
proto.speech.tts.lucid.Semantics.SemioticInfo.fromObject = function(obj) {
  var msg = new proto.speech.tts.lucid.Semantics.SemioticInfo();
  obj.semioticType != null && jspb.Message.setField(msg, 1, obj.semioticType);
  obj.hintLevel != null && jspb.Message.setField(msg, 3, obj.hintLevel);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.lucid.Semantics.SemioticInfo}
 */
proto.speech.tts.lucid.Semantics.SemioticInfo.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.lucid.Semantics.SemioticInfo;
  return proto.speech.tts.lucid.Semantics.SemioticInfo.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.lucid.Semantics.SemioticInfo} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.lucid.Semantics.SemioticInfo}
 */
proto.speech.tts.lucid.Semantics.SemioticInfo.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {!proto.speech.tts.lucid.SemioticType.Type} */ (reader.readEnum());
      msg.setSemioticType(value);
      break;
    case 3:
      var value = /** @type {!proto.speech.tts.lucid.Semantics.SemioticInfo.HintLevel} */ (reader.readEnum());
      msg.setHintLevel(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.lucid.Semantics.SemioticInfo.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.lucid.Semantics.SemioticInfo.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.lucid.Semantics.SemioticInfo} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.Semantics.SemioticInfo.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {!proto.speech.tts.lucid.SemioticType.Type} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeEnum(
      1,
      f
    );
  }
  f = /** @type {!proto.speech.tts.lucid.Semantics.SemioticInfo.HintLevel} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeEnum(
      3,
      f
    );
  }
};


/**
 * @enum {number}
 */
proto.speech.tts.lucid.Semantics.SemioticInfo.HintLevel = {
  UNSPECIFIED_HINT_LEVEL: 0,
  POSSIBLY_CONTAINED: 1,
  CERTAINLY_CONTAINED: 2,
  EXACTLY_CONTAINED: 3
};

/**
 * optional SemioticType.Type semiotic_type = 1;
 * @return {!proto.speech.tts.lucid.SemioticType.Type}
 */
proto.speech.tts.lucid.Semantics.SemioticInfo.prototype.getSemioticType = function() {
  return /** @type {!proto.speech.tts.lucid.SemioticType.Type} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?proto.speech.tts.lucid.SemioticType.Type|undefined} value
 * @return {!proto.speech.tts.lucid.Semantics.SemioticInfo} returns this
 */
proto.speech.tts.lucid.Semantics.SemioticInfo.prototype.setSemioticType = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Semantics.SemioticInfo} returns this
 */
proto.speech.tts.lucid.Semantics.SemioticInfo.prototype.clearSemioticType = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Semantics.SemioticInfo.prototype.hasSemioticType = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional HintLevel hint_level = 3;
 * @return {!proto.speech.tts.lucid.Semantics.SemioticInfo.HintLevel}
 */
proto.speech.tts.lucid.Semantics.SemioticInfo.prototype.getHintLevel = function() {
  return /** @type {!proto.speech.tts.lucid.Semantics.SemioticInfo.HintLevel} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?proto.speech.tts.lucid.Semantics.SemioticInfo.HintLevel|undefined} value
 * @return {!proto.speech.tts.lucid.Semantics.SemioticInfo} returns this
 */
proto.speech.tts.lucid.Semantics.SemioticInfo.prototype.setHintLevel = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Semantics.SemioticInfo} returns this
 */
proto.speech.tts.lucid.Semantics.SemioticInfo.prototype.clearHintLevel = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Semantics.SemioticInfo.prototype.hasHintLevel = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.lucid.Semantics.SemioticInfo}
 */
proto.speech.tts.lucid.Semantics.SemioticInfo.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.lucid.Semantics.SemioticInfo, data);
};


/**
 * repeated SemioticInfo semiotic_info = 1;
 * @return {!Array<!proto.speech.tts.lucid.Semantics.SemioticInfo>}
 */
proto.speech.tts.lucid.Semantics.prototype.getSemioticInfoList = function() {
  return /** @type{!Array<!proto.speech.tts.lucid.Semantics.SemioticInfo>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.lucid.Semantics.SemioticInfo, 1));
};


/**
 * @param {Array<!proto.speech.tts.lucid.Semantics.SemioticInfo>|undefined} value
 * @return {!proto.speech.tts.lucid.Semantics} returns this
*/
proto.speech.tts.lucid.Semantics.prototype.setSemioticInfoList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 1, value);
};


/**
 * @param {!proto.speech.tts.lucid.Semantics.SemioticInfo=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.lucid.Semantics.SemioticInfo}
 */
proto.speech.tts.lucid.Semantics.prototype.addSemioticInfo = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.speech.tts.lucid.Semantics.SemioticInfo, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.lucid.Semantics} returns this
 */
proto.speech.tts.lucid.Semantics.prototype.clearSemioticInfoList = function() {
  return this.setSemioticInfoList([]);
};


/**
 * repeated SourceInfo.Source source_info = 2;
 * @return {!Array<!proto.speech.tts.lucid.SourceInfo.Source>}
 */
proto.speech.tts.lucid.Semantics.prototype.getSourceInfoList = function() {
  return /** @type {!Array<!proto.speech.tts.lucid.SourceInfo.Source>} */ (jspb.Message.getRepeatedField(this, 2));
};


/**
 * @param {Array<!proto.speech.tts.lucid.SourceInfo.Source>|undefined} value
 * @return {!proto.speech.tts.lucid.Semantics} returns this
 */
proto.speech.tts.lucid.Semantics.prototype.setSourceInfoList = function(value) {
  return jspb.Message.setField(this, 2, value || []);
};


/**
 * @param {!proto.speech.tts.lucid.SourceInfo.Source} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.lucid.Semantics} returns this
 */
proto.speech.tts.lucid.Semantics.prototype.addSourceInfo = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 2, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.lucid.Semantics} returns this
 */
proto.speech.tts.lucid.Semantics.prototype.clearSourceInfoList = function() {
  return this.setSourceInfoList([]);
};


/**
 * optional location.unified.LocationDescriptor location = 3;
 * @return {?proto.location.unified.LocationDescriptor}
 */
proto.speech.tts.lucid.Semantics.prototype.getLocation = function() {
  return /** @type{?proto.location.unified.LocationDescriptor} */ (
    jspb.Message.getWrapperField(this, proto.location.unified.LocationDescriptor, 3));
};


/**
 * @param {?proto.location.unified.LocationDescriptor|undefined} value
 * @return {!proto.speech.tts.lucid.Semantics} returns this
*/
proto.speech.tts.lucid.Semantics.prototype.setLocation = function(value) {
  return jspb.Message.setWrapperField(this, 3, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.lucid.Semantics} returns this
 */
proto.speech.tts.lucid.Semantics.prototype.clearLocation = function() {
  return this.setLocation(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Semantics.prototype.hasLocation = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * repeated string knowledge_graph_mids = 4;
 * @return {!Array<string>}
 */
proto.speech.tts.lucid.Semantics.prototype.getKnowledgeGraphMidsList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 4));
};


/**
 * @param {Array<string>|undefined} value
 * @return {!proto.speech.tts.lucid.Semantics} returns this
 */
proto.speech.tts.lucid.Semantics.prototype.setKnowledgeGraphMidsList = function(value) {
  return jspb.Message.setField(this, 4, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.lucid.Semantics} returns this
 */
proto.speech.tts.lucid.Semantics.prototype.addKnowledgeGraphMids = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 4, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.lucid.Semantics} returns this
 */
proto.speech.tts.lucid.Semantics.prototype.clearKnowledgeGraphMidsList = function() {
  return this.setKnowledgeGraphMidsList([]);
};


/**
 * repeated NamedEntity named_entities = 5;
 * @return {!Array<!proto.speech.tts.lucid.NamedEntity>}
 */
proto.speech.tts.lucid.Semantics.prototype.getNamedEntitiesList = function() {
  return /** @type{!Array<!proto.speech.tts.lucid.NamedEntity>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.lucid.NamedEntity, 5));
};


/**
 * @param {Array<!proto.speech.tts.lucid.NamedEntity>|undefined} value
 * @return {!proto.speech.tts.lucid.Semantics} returns this
*/
proto.speech.tts.lucid.Semantics.prototype.setNamedEntitiesList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 5, value);
};


/**
 * @param {!proto.speech.tts.lucid.NamedEntity=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.lucid.NamedEntity}
 */
proto.speech.tts.lucid.Semantics.prototype.addNamedEntities = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 5, opt_value, proto.speech.tts.lucid.NamedEntity, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.lucid.Semantics} returns this
 */
proto.speech.tts.lucid.Semantics.prototype.clearNamedEntitiesList = function() {
  return this.setNamedEntitiesList([]);
};


/**
 * optional ListInfo list_info = 6;
 * @return {?proto.speech.tts.lucid.ListInfo}
 */
proto.speech.tts.lucid.Semantics.prototype.getListInfo = function() {
  return /** @type{?proto.speech.tts.lucid.ListInfo} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.lucid.ListInfo, 6));
};


/**
 * @param {?proto.speech.tts.lucid.ListInfo|undefined} value
 * @return {!proto.speech.tts.lucid.Semantics} returns this
*/
proto.speech.tts.lucid.Semantics.prototype.setListInfo = function(value) {
  return jspb.Message.setWrapperField(this, 6, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.lucid.Semantics} returns this
 */
proto.speech.tts.lucid.Semantics.prototype.clearListInfo = function() {
  return this.setListInfo(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Semantics.prototype.hasListInfo = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional TextnormLevel textnorm_level = 7;
 * @return {!proto.speech.tts.lucid.TextnormLevel}
 */
proto.speech.tts.lucid.Semantics.prototype.getTextnormLevel = function() {
  return /** @type {!proto.speech.tts.lucid.TextnormLevel} */ (jspb.Message.getField(this, 7));
};


/**
 * @param {?proto.speech.tts.lucid.TextnormLevel|undefined} value
 * @return {!proto.speech.tts.lucid.Semantics} returns this
 */
proto.speech.tts.lucid.Semantics.prototype.setTextnormLevel = function(value) {
  return jspb.Message.setField(this, 7, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Semantics} returns this
 */
proto.speech.tts.lucid.Semantics.prototype.clearTextnormLevel = function() {
  return jspb.Message.setField(this, 7, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Semantics.prototype.hasTextnormLevel = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.lucid.Semantics}
 */
proto.speech.tts.lucid.Semantics.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.lucid.Semantics, data);
};



//speech/tts/public/lucid/syntax_proto_jspb/syntax.jspb.js
// source: speech/tts/public/lucid/syntax.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.lucid.Syntax');
goog.provide('proto.speech.tts.lucid.Syntax.DependencyData');
goog.provide('proto.speech.tts.lucid.Syntax.MorphophoneticAttribute');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

goog.forwardDeclare('proto.nlp_saft.MorphologyAttrValue.Value');
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.lucid.Syntax = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.lucid.Syntax.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.lucid.Syntax, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.lucid.Syntax.displayName = 'proto.speech.tts.lucid.Syntax';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.lucid.Syntax.DependencyData = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.lucid.Syntax.DependencyData, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.lucid.Syntax.DependencyData.displayName = 'proto.speech.tts.lucid.Syntax.DependencyData';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.lucid.Syntax.repeatedFields_ = [7,8];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.lucid.Syntax.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.lucid.Syntax.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.lucid.Syntax} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.Syntax.toObject = function(includeInstance, msg) {
  var f, obj = {
    spelling: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    baseFormSpelling: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    syntacticCategory: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,
    partOfSpeech: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f,
    partOfSpeechConfidence: (f = jspb.Message.getOptionalFloatingPointField(msg, 5)) == null ? undefined : f,
    dependencyData: (f = msg.getDependencyData()) && proto.speech.tts.lucid.Syntax.DependencyData.toObject(includeInstance, f),
    morphologyAttributesList: (f = jspb.Message.getRepeatedField(msg, 7)) == null ? undefined : f,
    morphophoneticAttributesList: (f = jspb.Message.getRepeatedField(msg, 8)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Syntax as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.lucid.Syntax.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.spelling;

  /** @type {?string|undefined} */
  this.baseFormSpelling;

  /** @type {?string|undefined} */
  this.syntacticCategory;

  /** @type {?string|undefined} */
  this.partOfSpeech;

  /** @type {?number|undefined} */
  this.partOfSpeechConfidence;

  /** @type {?proto.speech.tts.lucid.Syntax.DependencyData.ObjectFormat|undefined} */
  this.dependencyData;

  /** @type {?Array<number>|undefined} */
  this.morphologyAttributesList;

  /** @type {?Array<number>|undefined} */
  this.morphophoneticAttributesList;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.lucid.Syntax.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.lucid.Syntax}
 */
proto.speech.tts.lucid.Syntax.fromObject = function(obj) {
  var msg = new proto.speech.tts.lucid.Syntax();
  obj.spelling != null && jspb.Message.setField(msg, 1, obj.spelling);
  obj.baseFormSpelling != null && jspb.Message.setField(msg, 2, obj.baseFormSpelling);
  obj.syntacticCategory != null && jspb.Message.setField(msg, 3, obj.syntacticCategory);
  obj.partOfSpeech != null && jspb.Message.setField(msg, 4, obj.partOfSpeech);
  obj.partOfSpeechConfidence != null && jspb.Message.setField(msg, 5, obj.partOfSpeechConfidence);
  obj.dependencyData && jspb.Message.setWrapperField(
      msg, 6, proto.speech.tts.lucid.Syntax.DependencyData.fromObject(obj.dependencyData));
  obj.morphologyAttributesList != null && jspb.Message.setField(msg, 7, obj.morphologyAttributesList);
  obj.morphophoneticAttributesList != null && jspb.Message.setField(msg, 8, obj.morphophoneticAttributesList);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.lucid.Syntax}
 */
proto.speech.tts.lucid.Syntax.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.lucid.Syntax;
  return proto.speech.tts.lucid.Syntax.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.lucid.Syntax} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.lucid.Syntax}
 */
proto.speech.tts.lucid.Syntax.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setSpelling(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setBaseFormSpelling(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setSyntacticCategory(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setPartOfSpeech(value);
      break;
    case 5:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setPartOfSpeechConfidence(value);
      break;
    case 6:
      var value = new proto.speech.tts.lucid.Syntax.DependencyData;
      reader.readMessage(value,proto.speech.tts.lucid.Syntax.DependencyData.deserializeBinaryFromReader);
      msg.setDependencyData(value);
      break;
    case 7:
      var value = /** @type {!proto.nlp_saft.MorphologyAttrValue.Value} */ (reader.readEnum());
      msg.addMorphologyAttributes(value);
      break;
    case 8:
      var value = /** @type {!proto.speech.tts.lucid.Syntax.MorphophoneticAttribute} */ (reader.readEnum());
      msg.addMorphophoneticAttributes(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.lucid.Syntax.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.lucid.Syntax.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.lucid.Syntax} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.Syntax.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeString(
      3,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeString(
      4,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeFloat(
      5,
      f
    );
  }
  f = message.getDependencyData();
  if (f != null) {
    writer.writeMessage(
      6,
      f,
      proto.speech.tts.lucid.Syntax.DependencyData.serializeBinaryToWriter
    );
  }
  f = message.getMorphologyAttributesList();
  if (f.length > 0) {
    writer.writeRepeatedEnum(
      7,
      f
    );
  }
  f = message.getMorphophoneticAttributesList();
  if (f.length > 0) {
    writer.writeRepeatedEnum(
      8,
      f
    );
  }
};


/**
 * @enum {number}
 */
proto.speech.tts.lucid.Syntax.MorphophoneticAttribute = {
  UNSPECIFIED_MORPHOPHONETIC_ATTRIBUTE: 0,
  APOCOPE: 1,
  SYNCOPE: 2
};




if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.lucid.Syntax.DependencyData.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.lucid.Syntax.DependencyData.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.lucid.Syntax.DependencyData} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.Syntax.DependencyData.toObject = function(includeInstance, msg) {
  var f, obj = {
    head: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    label: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of DependencyData as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.lucid.Syntax.DependencyData.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.head;

  /** @type {?string|undefined} */
  this.label;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.lucid.Syntax.DependencyData.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.lucid.Syntax.DependencyData}
 */
proto.speech.tts.lucid.Syntax.DependencyData.fromObject = function(obj) {
  var msg = new proto.speech.tts.lucid.Syntax.DependencyData();
  obj.head != null && jspb.Message.setField(msg, 1, obj.head);
  obj.label != null && jspb.Message.setField(msg, 2, obj.label);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.lucid.Syntax.DependencyData}
 */
proto.speech.tts.lucid.Syntax.DependencyData.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.lucid.Syntax.DependencyData;
  return proto.speech.tts.lucid.Syntax.DependencyData.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.lucid.Syntax.DependencyData} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.lucid.Syntax.DependencyData}
 */
proto.speech.tts.lucid.Syntax.DependencyData.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setHead(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setLabel(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.lucid.Syntax.DependencyData.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.lucid.Syntax.DependencyData.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.lucid.Syntax.DependencyData} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.Syntax.DependencyData.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?number} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeInt32(
      1,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional int32 head = 1;
 * @return {?number}
 */
proto.speech.tts.lucid.Syntax.DependencyData.prototype.getHead = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.lucid.Syntax.DependencyData} returns this
 */
proto.speech.tts.lucid.Syntax.DependencyData.prototype.setHead = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Syntax.DependencyData} returns this
 */
proto.speech.tts.lucid.Syntax.DependencyData.prototype.clearHead = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Syntax.DependencyData.prototype.hasHead = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional string label = 2;
 * @return {?string}
 */
proto.speech.tts.lucid.Syntax.DependencyData.prototype.getLabel = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.lucid.Syntax.DependencyData} returns this
 */
proto.speech.tts.lucid.Syntax.DependencyData.prototype.setLabel = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Syntax.DependencyData} returns this
 */
proto.speech.tts.lucid.Syntax.DependencyData.prototype.clearLabel = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Syntax.DependencyData.prototype.hasLabel = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.lucid.Syntax.DependencyData}
 */
proto.speech.tts.lucid.Syntax.DependencyData.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.lucid.Syntax.DependencyData, data);
};


/**
 * optional string spelling = 1;
 * @return {?string}
 */
proto.speech.tts.lucid.Syntax.prototype.getSpelling = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.lucid.Syntax} returns this
 */
proto.speech.tts.lucid.Syntax.prototype.setSpelling = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Syntax} returns this
 */
proto.speech.tts.lucid.Syntax.prototype.clearSpelling = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Syntax.prototype.hasSpelling = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional string base_form_spelling = 2;
 * @return {?string}
 */
proto.speech.tts.lucid.Syntax.prototype.getBaseFormSpelling = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.lucid.Syntax} returns this
 */
proto.speech.tts.lucid.Syntax.prototype.setBaseFormSpelling = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Syntax} returns this
 */
proto.speech.tts.lucid.Syntax.prototype.clearBaseFormSpelling = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Syntax.prototype.hasBaseFormSpelling = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional string syntactic_category = 3;
 * @return {?string}
 */
proto.speech.tts.lucid.Syntax.prototype.getSyntacticCategory = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.lucid.Syntax} returns this
 */
proto.speech.tts.lucid.Syntax.prototype.setSyntacticCategory = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Syntax} returns this
 */
proto.speech.tts.lucid.Syntax.prototype.clearSyntacticCategory = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Syntax.prototype.hasSyntacticCategory = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional string part_of_speech = 4;
 * @return {?string}
 */
proto.speech.tts.lucid.Syntax.prototype.getPartOfSpeech = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 4));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.lucid.Syntax} returns this
 */
proto.speech.tts.lucid.Syntax.prototype.setPartOfSpeech = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Syntax} returns this
 */
proto.speech.tts.lucid.Syntax.prototype.clearPartOfSpeech = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Syntax.prototype.hasPartOfSpeech = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional float part_of_speech_confidence = 5;
 * @return {?number}
 */
proto.speech.tts.lucid.Syntax.prototype.getPartOfSpeechConfidence = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 5));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.lucid.Syntax} returns this
 */
proto.speech.tts.lucid.Syntax.prototype.setPartOfSpeechConfidence = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Syntax} returns this
 */
proto.speech.tts.lucid.Syntax.prototype.clearPartOfSpeechConfidence = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Syntax.prototype.hasPartOfSpeechConfidence = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional DependencyData dependency_data = 6;
 * @return {?proto.speech.tts.lucid.Syntax.DependencyData}
 */
proto.speech.tts.lucid.Syntax.prototype.getDependencyData = function() {
  return /** @type{?proto.speech.tts.lucid.Syntax.DependencyData} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.lucid.Syntax.DependencyData, 6));
};


/**
 * @param {?proto.speech.tts.lucid.Syntax.DependencyData|undefined} value
 * @return {!proto.speech.tts.lucid.Syntax} returns this
*/
proto.speech.tts.lucid.Syntax.prototype.setDependencyData = function(value) {
  return jspb.Message.setWrapperField(this, 6, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.lucid.Syntax} returns this
 */
proto.speech.tts.lucid.Syntax.prototype.clearDependencyData = function() {
  return this.setDependencyData(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Syntax.prototype.hasDependencyData = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * repeated nlp_saft.MorphologyAttrValue.Value morphology_attributes = 7;
 * @return {!Array<!proto.nlp_saft.MorphologyAttrValue.Value>}
 */
proto.speech.tts.lucid.Syntax.prototype.getMorphologyAttributesList = function() {
  return /** @type {!Array<!proto.nlp_saft.MorphologyAttrValue.Value>} */ (jspb.Message.getRepeatedField(this, 7));
};


/**
 * @param {Array<!proto.nlp_saft.MorphologyAttrValue.Value>|undefined} value
 * @return {!proto.speech.tts.lucid.Syntax} returns this
 */
proto.speech.tts.lucid.Syntax.prototype.setMorphologyAttributesList = function(value) {
  return jspb.Message.setField(this, 7, value || []);
};


/**
 * @param {!proto.nlp_saft.MorphologyAttrValue.Value} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.lucid.Syntax} returns this
 */
proto.speech.tts.lucid.Syntax.prototype.addMorphologyAttributes = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 7, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.lucid.Syntax} returns this
 */
proto.speech.tts.lucid.Syntax.prototype.clearMorphologyAttributesList = function() {
  return this.setMorphologyAttributesList([]);
};


/**
 * repeated MorphophoneticAttribute morphophonetic_attributes = 8;
 * @return {!Array<!proto.speech.tts.lucid.Syntax.MorphophoneticAttribute>}
 */
proto.speech.tts.lucid.Syntax.prototype.getMorphophoneticAttributesList = function() {
  return /** @type {!Array<!proto.speech.tts.lucid.Syntax.MorphophoneticAttribute>} */ (jspb.Message.getRepeatedField(this, 8));
};


/**
 * @param {Array<!proto.speech.tts.lucid.Syntax.MorphophoneticAttribute>|undefined} value
 * @return {!proto.speech.tts.lucid.Syntax} returns this
 */
proto.speech.tts.lucid.Syntax.prototype.setMorphophoneticAttributesList = function(value) {
  return jspb.Message.setField(this, 8, value || []);
};


/**
 * @param {!proto.speech.tts.lucid.Syntax.MorphophoneticAttribute} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.lucid.Syntax} returns this
 */
proto.speech.tts.lucid.Syntax.prototype.addMorphophoneticAttributes = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 8, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.lucid.Syntax} returns this
 */
proto.speech.tts.lucid.Syntax.prototype.clearMorphophoneticAttributesList = function() {
  return this.setMorphophoneticAttributesList([]);
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.lucid.Syntax}
 */
proto.speech.tts.lucid.Syntax.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.lucid.Syntax, data);
};



//speech/tts/public/lucid/synthesis_parameters_proto_jspb/audiosource.jspb.js
// source: speech/tts/public/lucid/synthesis_parameters.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.lucid.AudioSource');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.lucid.AudioSource = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.lucid.AudioSource, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.lucid.AudioSource.displayName = 'proto.speech.tts.lucid.AudioSource';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.lucid.AudioSource.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.lucid.AudioSource.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.lucid.AudioSource} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.AudioSource.toObject = function(includeInstance, msg) {
  var f, obj = {
    vuiId: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    stageName: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of AudioSource as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.lucid.AudioSource.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.vuiId;

  /** @type {?string|undefined} */
  this.stageName;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.lucid.AudioSource.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.lucid.AudioSource}
 */
proto.speech.tts.lucid.AudioSource.fromObject = function(obj) {
  var msg = new proto.speech.tts.lucid.AudioSource();
  obj.vuiId != null && jspb.Message.setField(msg, 1, obj.vuiId);
  obj.stageName != null && jspb.Message.setField(msg, 2, obj.stageName);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.lucid.AudioSource}
 */
proto.speech.tts.lucid.AudioSource.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.lucid.AudioSource;
  return proto.speech.tts.lucid.AudioSource.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.lucid.AudioSource} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.lucid.AudioSource}
 */
proto.speech.tts.lucid.AudioSource.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setVuiId(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setStageName(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.lucid.AudioSource.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.lucid.AudioSource.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.lucid.AudioSource} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.AudioSource.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string vui_id = 1;
 * @return {?string}
 */
proto.speech.tts.lucid.AudioSource.prototype.getVuiId = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.lucid.AudioSource} returns this
 */
proto.speech.tts.lucid.AudioSource.prototype.setVuiId = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.AudioSource} returns this
 */
proto.speech.tts.lucid.AudioSource.prototype.clearVuiId = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.AudioSource.prototype.hasVuiId = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional string stage_name = 2;
 * @return {?string}
 */
proto.speech.tts.lucid.AudioSource.prototype.getStageName = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.lucid.AudioSource} returns this
 */
proto.speech.tts.lucid.AudioSource.prototype.setStageName = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.AudioSource} returns this
 */
proto.speech.tts.lucid.AudioSource.prototype.clearStageName = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.AudioSource.prototype.hasStageName = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.lucid.AudioSource}
 */
proto.speech.tts.lucid.AudioSource.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.lucid.AudioSource, data);
};



//speech/tts/public/performance_options_proto_jspb/performancespecification.jspb.js
// source: speech/tts/public/performance_options.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.PerformanceSpecification');
goog.provide('proto.speech.tts.PerformanceSpecification.TextPolicy');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.PerformanceSpecification = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.PerformanceSpecification, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.PerformanceSpecification.displayName = 'proto.speech.tts.PerformanceSpecification';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.PerformanceSpecification.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.PerformanceSpecification.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.PerformanceSpecification} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.PerformanceSpecification.toObject = function(includeInstance, msg) {
  var f, obj = {
    performanceId: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    textPolicy: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    obtainedWithVuiIdMapping: jspb.Message.getBooleanFieldWithDefault(msg, 3, false)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of PerformanceSpecification as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.PerformanceSpecification.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.performanceId;

  /** @type {?number|undefined} */
  this.textPolicy;

  /** @type {?boolean|undefined} */
  this.obtainedWithVuiIdMapping;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.PerformanceSpecification.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.PerformanceSpecification}
 */
proto.speech.tts.PerformanceSpecification.fromObject = function(obj) {
  var msg = new proto.speech.tts.PerformanceSpecification();
  obj.performanceId != null && jspb.Message.setField(msg, 1, obj.performanceId);
  obj.textPolicy != null && jspb.Message.setField(msg, 2, obj.textPolicy);
  obj.obtainedWithVuiIdMapping != null && jspb.Message.setField(msg, 3, obj.obtainedWithVuiIdMapping);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.PerformanceSpecification}
 */
proto.speech.tts.PerformanceSpecification.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.PerformanceSpecification;
  return proto.speech.tts.PerformanceSpecification.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.PerformanceSpecification} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.PerformanceSpecification}
 */
proto.speech.tts.PerformanceSpecification.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {number} */ (reader.readUint64());
      msg.setPerformanceId(value);
      break;
    case 2:
      var value = /** @type {!proto.speech.tts.PerformanceSpecification.TextPolicy} */ (reader.readEnum());
      msg.setTextPolicy(value);
      break;
    case 3:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setObtainedWithVuiIdMapping(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.PerformanceSpecification.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.PerformanceSpecification.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.PerformanceSpecification} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.PerformanceSpecification.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?number} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeUint64(
      1,
      f
    );
  }
  f = /** @type {!proto.speech.tts.PerformanceSpecification.TextPolicy} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeEnum(
      2,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeBool(
      3,
      f
    );
  }
};


/**
 * @enum {number}
 */
proto.speech.tts.PerformanceSpecification.TextPolicy = {
  UNKNOWN_POLICY: 0,
  USE_REQUESTED_TEXT: 1,
  USE_DIRECTOR_TEXT: 2,
  CALCULATE_PROSODY_EMBEDDING: 3
};

/**
 * optional uint64 performance_id = 1;
 * @return {?number}
 */
proto.speech.tts.PerformanceSpecification.prototype.getPerformanceId = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.PerformanceSpecification} returns this
 */
proto.speech.tts.PerformanceSpecification.prototype.setPerformanceId = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.PerformanceSpecification} returns this
 */
proto.speech.tts.PerformanceSpecification.prototype.clearPerformanceId = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.PerformanceSpecification.prototype.hasPerformanceId = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional TextPolicy text_policy = 2;
 * @return {!proto.speech.tts.PerformanceSpecification.TextPolicy}
 */
proto.speech.tts.PerformanceSpecification.prototype.getTextPolicy = function() {
  return /** @type {!proto.speech.tts.PerformanceSpecification.TextPolicy} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?proto.speech.tts.PerformanceSpecification.TextPolicy|undefined} value
 * @return {!proto.speech.tts.PerformanceSpecification} returns this
 */
proto.speech.tts.PerformanceSpecification.prototype.setTextPolicy = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.PerformanceSpecification} returns this
 */
proto.speech.tts.PerformanceSpecification.prototype.clearTextPolicy = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.PerformanceSpecification.prototype.hasTextPolicy = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional bool obtained_with_vui_id_mapping = 3;
 * @return {boolean}
 */
proto.speech.tts.PerformanceSpecification.prototype.getObtainedWithVuiIdMapping = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 3, false));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.PerformanceSpecification} returns this
 */
proto.speech.tts.PerformanceSpecification.prototype.setObtainedWithVuiIdMapping = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.PerformanceSpecification} returns this
 */
proto.speech.tts.PerformanceSpecification.prototype.clearObtainedWithVuiIdMapping = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.PerformanceSpecification.prototype.hasObtainedWithVuiIdMapping = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.PerformanceSpecification}
 */
proto.speech.tts.PerformanceSpecification.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.PerformanceSpecification, data);
};



//speech/tts/public/performance_options_proto_jspb/performanceoptions.jspb.js
// source: speech/tts/public/performance_options.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.PerformanceOptions');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.PerformanceSpecification');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.PerformanceOptions = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.PerformanceOptions, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.PerformanceOptions.displayName = 'proto.speech.tts.PerformanceOptions';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.PerformanceOptions.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.PerformanceOptions.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.PerformanceOptions} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.PerformanceOptions.toObject = function(includeInstance, msg) {
  var f, obj = {
    performanceSpec: (f = msg.getPerformanceSpec()) && proto.speech.tts.PerformanceSpecification.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of PerformanceOptions as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.PerformanceOptions.ObjectFormat = function() {
  /** @type {?proto.speech.tts.PerformanceSpecification.ObjectFormat|undefined} */
  this.performanceSpec;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.PerformanceOptions.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.PerformanceOptions}
 */
proto.speech.tts.PerformanceOptions.fromObject = function(obj) {
  var msg = new proto.speech.tts.PerformanceOptions();
  obj.performanceSpec && jspb.Message.setWrapperField(
      msg, 1, proto.speech.tts.PerformanceSpecification.fromObject(obj.performanceSpec));
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.PerformanceOptions}
 */
proto.speech.tts.PerformanceOptions.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.PerformanceOptions;
  return proto.speech.tts.PerformanceOptions.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.PerformanceOptions} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.PerformanceOptions}
 */
proto.speech.tts.PerformanceOptions.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.speech.tts.PerformanceSpecification;
      reader.readMessage(value,proto.speech.tts.PerformanceSpecification.deserializeBinaryFromReader);
      msg.setPerformanceSpec(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.PerformanceOptions.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.PerformanceOptions.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.PerformanceOptions} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.PerformanceOptions.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getPerformanceSpec();
  if (f != null) {
    writer.writeMessage(
      1,
      f,
      proto.speech.tts.PerformanceSpecification.serializeBinaryToWriter
    );
  }
};


/**
 * optional PerformanceSpecification performance_spec = 1;
 * @return {?proto.speech.tts.PerformanceSpecification}
 */
proto.speech.tts.PerformanceOptions.prototype.getPerformanceSpec = function() {
  return /** @type{?proto.speech.tts.PerformanceSpecification} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.PerformanceSpecification, 1));
};


/**
 * @param {?proto.speech.tts.PerformanceSpecification|undefined} value
 * @return {!proto.speech.tts.PerformanceOptions} returns this
*/
proto.speech.tts.PerformanceOptions.prototype.setPerformanceSpec = function(value) {
  return jspb.Message.setWrapperField(this, 1, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.PerformanceOptions} returns this
 */
proto.speech.tts.PerformanceOptions.prototype.clearPerformanceSpec = function() {
  return this.setPerformanceSpec(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.PerformanceOptions.prototype.hasPerformanceSpec = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.PerformanceOptions}
 */
proto.speech.tts.PerformanceOptions.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.PerformanceOptions, data);
};



//speech/tts/public/speech_morphing_target_proto_jspb/speechmorphingtarget.jspb.js
// source: speech/tts/public/speech_morphing_target.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.SpeechMorphingTarget');
goog.provide('proto.speech.tts.SpeechMorphingTarget.MorphingType');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.SpeechMorphingTarget = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.SpeechMorphingTarget, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.SpeechMorphingTarget.displayName = 'proto.speech.tts.SpeechMorphingTarget';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.SpeechMorphingTarget.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.SpeechMorphingTarget.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.SpeechMorphingTarget} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.SpeechMorphingTarget.toObject = function(includeInstance, msg) {
  var f, obj = {
    type: jspb.Message.getFieldWithDefault(msg, 1, 0),
    reference: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of SpeechMorphingTarget as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.SpeechMorphingTarget.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.type;

  /** @type {?string|undefined} */
  this.reference;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.SpeechMorphingTarget.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.SpeechMorphingTarget}
 */
proto.speech.tts.SpeechMorphingTarget.fromObject = function(obj) {
  var msg = new proto.speech.tts.SpeechMorphingTarget();
  obj.type != null && jspb.Message.setField(msg, 1, obj.type);
  obj.reference != null && jspb.Message.setField(msg, 2, obj.reference);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.SpeechMorphingTarget}
 */
proto.speech.tts.SpeechMorphingTarget.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.SpeechMorphingTarget;
  return proto.speech.tts.SpeechMorphingTarget.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.SpeechMorphingTarget} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.SpeechMorphingTarget}
 */
proto.speech.tts.SpeechMorphingTarget.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {!proto.speech.tts.SpeechMorphingTarget.MorphingType} */ (reader.readEnum());
      msg.setType(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setReference(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.SpeechMorphingTarget.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.SpeechMorphingTarget.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.SpeechMorphingTarget} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.SpeechMorphingTarget.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {!proto.speech.tts.SpeechMorphingTarget.MorphingType} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeEnum(
      1,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * @enum {number}
 */
proto.speech.tts.SpeechMorphingTarget.MorphingType = {
  UNKNOWN_MORPHING_TYPE: 0,
  NONE: 1,
  VOICE_TIMBRE_USING_ODFWW: 2
};

/**
 * optional MorphingType type = 1;
 * @return {!proto.speech.tts.SpeechMorphingTarget.MorphingType}
 */
proto.speech.tts.SpeechMorphingTarget.prototype.getType = function() {
  return /** @type {!proto.speech.tts.SpeechMorphingTarget.MorphingType} */ (jspb.Message.getFieldWithDefault(this, 1, 0));
};


/**
 * @param {?proto.speech.tts.SpeechMorphingTarget.MorphingType|undefined} value
 * @return {!proto.speech.tts.SpeechMorphingTarget} returns this
 */
proto.speech.tts.SpeechMorphingTarget.prototype.setType = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.SpeechMorphingTarget} returns this
 */
proto.speech.tts.SpeechMorphingTarget.prototype.clearType = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.SpeechMorphingTarget.prototype.hasType = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional string reference = 2;
 * @return {?string}
 */
proto.speech.tts.SpeechMorphingTarget.prototype.getReference = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.SpeechMorphingTarget} returns this
 */
proto.speech.tts.SpeechMorphingTarget.prototype.setReference = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.SpeechMorphingTarget} returns this
 */
proto.speech.tts.SpeechMorphingTarget.prototype.clearReference = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.SpeechMorphingTarget.prototype.hasReference = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.SpeechMorphingTarget}
 */
proto.speech.tts.SpeechMorphingTarget.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.SpeechMorphingTarget, data);
};



//speech/tts/public/speech_morphing_target_proto_jspb/speechmorphingtargets.jspb.js
// source: speech/tts/public/speech_morphing_target.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.SpeechMorphingTargets');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.SpeechMorphingTarget');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.SpeechMorphingTargets = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.SpeechMorphingTargets.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.SpeechMorphingTargets, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.SpeechMorphingTargets.displayName = 'proto.speech.tts.SpeechMorphingTargets';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.SpeechMorphingTargets.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.SpeechMorphingTargets.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.SpeechMorphingTargets.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.SpeechMorphingTargets} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.SpeechMorphingTargets.toObject = function(includeInstance, msg) {
  var f, obj = {
    targetList: jspb.Message.toObjectList(msg.getTargetList(),
    proto.speech.tts.SpeechMorphingTarget.toObject, includeInstance)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of SpeechMorphingTargets as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.SpeechMorphingTargets.ObjectFormat = function() {
  /** @type {?Array<!proto.speech.tts.SpeechMorphingTarget.ObjectFormat>|undefined} */
  this.targetList;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.SpeechMorphingTargets.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.SpeechMorphingTargets}
 */
proto.speech.tts.SpeechMorphingTargets.fromObject = function(obj) {
  var msg = new proto.speech.tts.SpeechMorphingTargets();
  obj.targetList && jspb.Message.setRepeatedWrapperField(
      msg, 1, obj.targetList.map(
          proto.speech.tts.SpeechMorphingTarget.fromObject));
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.SpeechMorphingTargets}
 */
proto.speech.tts.SpeechMorphingTargets.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.SpeechMorphingTargets;
  return proto.speech.tts.SpeechMorphingTargets.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.SpeechMorphingTargets} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.SpeechMorphingTargets}
 */
proto.speech.tts.SpeechMorphingTargets.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.speech.tts.SpeechMorphingTarget;
      reader.readMessage(value,proto.speech.tts.SpeechMorphingTarget.deserializeBinaryFromReader);
      msg.addTarget(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.SpeechMorphingTargets.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.SpeechMorphingTargets.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.SpeechMorphingTargets} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.SpeechMorphingTargets.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getTargetList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      1,
      f,
      proto.speech.tts.SpeechMorphingTarget.serializeBinaryToWriter
    );
  }
};


/**
 * repeated SpeechMorphingTarget target = 1;
 * @return {!Array<!proto.speech.tts.SpeechMorphingTarget>}
 */
proto.speech.tts.SpeechMorphingTargets.prototype.getTargetList = function() {
  return /** @type{!Array<!proto.speech.tts.SpeechMorphingTarget>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.SpeechMorphingTarget, 1));
};


/**
 * @param {Array<!proto.speech.tts.SpeechMorphingTarget>|undefined} value
 * @return {!proto.speech.tts.SpeechMorphingTargets} returns this
*/
proto.speech.tts.SpeechMorphingTargets.prototype.setTargetList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 1, value);
};


/**
 * @param {!proto.speech.tts.SpeechMorphingTarget=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.SpeechMorphingTarget}
 */
proto.speech.tts.SpeechMorphingTargets.prototype.addTarget = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.speech.tts.SpeechMorphingTarget, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.SpeechMorphingTargets} returns this
 */
proto.speech.tts.SpeechMorphingTargets.prototype.clearTargetList = function() {
  return this.setTargetList([]);
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.SpeechMorphingTargets}
 */
proto.speech.tts.SpeechMorphingTargets.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.SpeechMorphingTargets, data);
};



//speech/tts/public/lucid/synthesis_parameters_proto_jspb/synthesisparameters.jspb.js
// source: speech/tts/public/lucid/synthesis_parameters.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.lucid.SynthesisParameters');
goog.provide('proto.speech.tts.lucid.SynthesisParameters.Redaction');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.PerformanceOptions');
goog.require('proto.speech.tts.SpeechMorphingTargets');
goog.require('proto.speech.tts.lucid.AudioSource');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.lucid.SynthesisParameters = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.lucid.SynthesisParameters.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.lucid.SynthesisParameters, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.lucid.SynthesisParameters.displayName = 'proto.speech.tts.lucid.SynthesisParameters';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.lucid.SynthesisParameters.repeatedFields_ = [3,4];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.lucid.SynthesisParameters.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.lucid.SynthesisParameters.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.lucid.SynthesisParameters} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.SynthesisParameters.toObject = function(includeInstance, msg) {
  var f, obj = {
    morphing: (f = msg.getMorphing()) && proto.speech.tts.SpeechMorphingTargets.toObject(includeInstance, f),
    outputGain: jspb.Message.getFloatingPointFieldWithDefault(msg, 2, 1.0),
    audioSourcesList: jspb.Message.toObjectList(msg.getAudioSourcesList(),
    proto.speech.tts.lucid.AudioSource.toObject, includeInstance),
    performanceOptions: (f = msg.getPerformanceOptions()) && proto.speech.tts.PerformanceOptions.toObject(includeInstance, f),
    redactionList: (f = jspb.Message.getRepeatedField(msg, 4)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of SynthesisParameters as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.lucid.SynthesisParameters.ObjectFormat = function() {
  /** @type {?proto.speech.tts.SpeechMorphingTargets.ObjectFormat|undefined} */
  this.morphing;

  /** @type {?number|undefined} */
  this.outputGain;

  /** @type {?Array<!proto.speech.tts.lucid.AudioSource.ObjectFormat>|undefined} */
  this.audioSourcesList;

  /** @type {?proto.speech.tts.PerformanceOptions.ObjectFormat|undefined} */
  this.performanceOptions;

  /** @type {?Array<number>|undefined} */
  this.redactionList;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.lucid.SynthesisParameters.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.lucid.SynthesisParameters}
 */
proto.speech.tts.lucid.SynthesisParameters.fromObject = function(obj) {
  var msg = new proto.speech.tts.lucid.SynthesisParameters();
  obj.morphing && jspb.Message.setWrapperField(
      msg, 1, proto.speech.tts.SpeechMorphingTargets.fromObject(obj.morphing));
  obj.outputGain != null && jspb.Message.setField(msg, 2, obj.outputGain);
  obj.audioSourcesList && jspb.Message.setRepeatedWrapperField(
      msg, 3, obj.audioSourcesList.map(
          proto.speech.tts.lucid.AudioSource.fromObject));
  obj.performanceOptions && jspb.Message.setWrapperField(
      msg, 5, proto.speech.tts.PerformanceOptions.fromObject(obj.performanceOptions));
  obj.redactionList != null && jspb.Message.setField(msg, 4, obj.redactionList);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.lucid.SynthesisParameters}
 */
proto.speech.tts.lucid.SynthesisParameters.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.lucid.SynthesisParameters;
  return proto.speech.tts.lucid.SynthesisParameters.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.lucid.SynthesisParameters} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.lucid.SynthesisParameters}
 */
proto.speech.tts.lucid.SynthesisParameters.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.speech.tts.SpeechMorphingTargets;
      reader.readMessage(value,proto.speech.tts.SpeechMorphingTargets.deserializeBinaryFromReader);
      msg.setMorphing(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setOutputGain(value);
      break;
    case 3:
      var value = new proto.speech.tts.lucid.AudioSource;
      reader.readMessage(value,proto.speech.tts.lucid.AudioSource.deserializeBinaryFromReader);
      msg.addAudioSources(value);
      break;
    case 5:
      var value = new proto.speech.tts.PerformanceOptions;
      reader.readMessage(value,proto.speech.tts.PerformanceOptions.deserializeBinaryFromReader);
      msg.setPerformanceOptions(value);
      break;
    case 4:
      var value = /** @type {!proto.speech.tts.lucid.SynthesisParameters.Redaction} */ (reader.readEnum());
      msg.addRedaction(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.lucid.SynthesisParameters.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.lucid.SynthesisParameters.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.lucid.SynthesisParameters} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.SynthesisParameters.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getMorphing();
  if (f != null) {
    writer.writeMessage(
      1,
      f,
      proto.speech.tts.SpeechMorphingTargets.serializeBinaryToWriter
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeFloat(
      2,
      f
    );
  }
  f = message.getAudioSourcesList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      3,
      f,
      proto.speech.tts.lucid.AudioSource.serializeBinaryToWriter
    );
  }
  f = message.getPerformanceOptions();
  if (f != null) {
    writer.writeMessage(
      5,
      f,
      proto.speech.tts.PerformanceOptions.serializeBinaryToWriter
    );
  }
  f = message.getRedactionList();
  if (f.length > 0) {
    writer.writeRepeatedEnum(
      4,
      f
    );
  }
};


/**
 * @enum {number}
 */
proto.speech.tts.lucid.SynthesisParameters.Redaction = {
  UNSPECIFIED_REDACTION: 0,
  BLEEP: 1,
  BOWDLERIZE: 2,
  NO_CELEBRITY: 3
};

/**
 * optional speech.tts.SpeechMorphingTargets morphing = 1;
 * @return {?proto.speech.tts.SpeechMorphingTargets}
 */
proto.speech.tts.lucid.SynthesisParameters.prototype.getMorphing = function() {
  return /** @type{?proto.speech.tts.SpeechMorphingTargets} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.SpeechMorphingTargets, 1));
};


/**
 * @param {?proto.speech.tts.SpeechMorphingTargets|undefined} value
 * @return {!proto.speech.tts.lucid.SynthesisParameters} returns this
*/
proto.speech.tts.lucid.SynthesisParameters.prototype.setMorphing = function(value) {
  return jspb.Message.setWrapperField(this, 1, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.lucid.SynthesisParameters} returns this
 */
proto.speech.tts.lucid.SynthesisParameters.prototype.clearMorphing = function() {
  return this.setMorphing(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.SynthesisParameters.prototype.hasMorphing = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional float output_gain = 2;
 * @return {number}
 */
proto.speech.tts.lucid.SynthesisParameters.prototype.getOutputGain = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 2, 1.0));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.lucid.SynthesisParameters} returns this
 */
proto.speech.tts.lucid.SynthesisParameters.prototype.setOutputGain = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.SynthesisParameters} returns this
 */
proto.speech.tts.lucid.SynthesisParameters.prototype.clearOutputGain = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.SynthesisParameters.prototype.hasOutputGain = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * repeated AudioSource audio_sources = 3;
 * @return {!Array<!proto.speech.tts.lucid.AudioSource>}
 */
proto.speech.tts.lucid.SynthesisParameters.prototype.getAudioSourcesList = function() {
  return /** @type{!Array<!proto.speech.tts.lucid.AudioSource>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.lucid.AudioSource, 3));
};


/**
 * @param {Array<!proto.speech.tts.lucid.AudioSource>|undefined} value
 * @return {!proto.speech.tts.lucid.SynthesisParameters} returns this
*/
proto.speech.tts.lucid.SynthesisParameters.prototype.setAudioSourcesList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 3, value);
};


/**
 * @param {!proto.speech.tts.lucid.AudioSource=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.lucid.AudioSource}
 */
proto.speech.tts.lucid.SynthesisParameters.prototype.addAudioSources = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 3, opt_value, proto.speech.tts.lucid.AudioSource, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.lucid.SynthesisParameters} returns this
 */
proto.speech.tts.lucid.SynthesisParameters.prototype.clearAudioSourcesList = function() {
  return this.setAudioSourcesList([]);
};


/**
 * optional speech.tts.PerformanceOptions performance_options = 5;
 * @return {?proto.speech.tts.PerformanceOptions}
 */
proto.speech.tts.lucid.SynthesisParameters.prototype.getPerformanceOptions = function() {
  return /** @type{?proto.speech.tts.PerformanceOptions} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.PerformanceOptions, 5));
};


/**
 * @param {?proto.speech.tts.PerformanceOptions|undefined} value
 * @return {!proto.speech.tts.lucid.SynthesisParameters} returns this
*/
proto.speech.tts.lucid.SynthesisParameters.prototype.setPerformanceOptions = function(value) {
  return jspb.Message.setWrapperField(this, 5, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.lucid.SynthesisParameters} returns this
 */
proto.speech.tts.lucid.SynthesisParameters.prototype.clearPerformanceOptions = function() {
  return this.setPerformanceOptions(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.SynthesisParameters.prototype.hasPerformanceOptions = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * repeated Redaction redaction = 4;
 * @return {!Array<!proto.speech.tts.lucid.SynthesisParameters.Redaction>}
 */
proto.speech.tts.lucid.SynthesisParameters.prototype.getRedactionList = function() {
  return /** @type {!Array<!proto.speech.tts.lucid.SynthesisParameters.Redaction>} */ (jspb.Message.getRepeatedField(this, 4));
};


/**
 * @param {Array<!proto.speech.tts.lucid.SynthesisParameters.Redaction>|undefined} value
 * @return {!proto.speech.tts.lucid.SynthesisParameters} returns this
 */
proto.speech.tts.lucid.SynthesisParameters.prototype.setRedactionList = function(value) {
  return jspb.Message.setField(this, 4, value || []);
};


/**
 * @param {!proto.speech.tts.lucid.SynthesisParameters.Redaction} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.lucid.SynthesisParameters} returns this
 */
proto.speech.tts.lucid.SynthesisParameters.prototype.addRedaction = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 4, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.lucid.SynthesisParameters} returns this
 */
proto.speech.tts.lucid.SynthesisParameters.prototype.clearRedactionList = function() {
  return this.setRedactionList([]);
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.lucid.SynthesisParameters}
 */
proto.speech.tts.lucid.SynthesisParameters.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.lucid.SynthesisParameters, data);
};



//speech/tts/public/lucid/annotation_proto_jspb/annotation.jspb.js
// source: speech/tts/public/lucid/annotation.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.lucid.Annotation');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.lucid.CrossLingual');
goog.require('proto.speech.tts.lucid.Prosody');
goog.require('proto.speech.tts.lucid.Semantics');
goog.require('proto.speech.tts.lucid.Syntax');
goog.require('proto.speech.tts.lucid.SynthesisParameters');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.lucid.Annotation = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 6, null, null);
};
goog.inherits(proto.speech.tts.lucid.Annotation, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.lucid.Annotation.displayName = 'proto.speech.tts.lucid.Annotation';
}

/**
 * The extensions registered with this message class. This is a map of
 * extension field number to fieldInfo object.
 *
 * For example:
 *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }
 *
 * fieldName contains the JsCompiler renamed field name property so that it
 * works in OPTIMIZED mode.
 *
 * @type {!Object<number, jspb.ExtensionFieldInfo>}
 */
proto.speech.tts.lucid.Annotation.extensions = {};


/**
 * The extensions registered with this message class. This is a map of
 * extension field number to fieldInfo object.
 *
 * For example:
 *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }
 *
 * fieldName contains the JsCompiler renamed field name property so that it
 * works in OPTIMIZED mode.
 *
 * @type {!Object<number, jspb.ExtensionFieldBinaryInfo>}
 */
proto.speech.tts.lucid.Annotation.extensionsBinary = {};




if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.lucid.Annotation.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.lucid.Annotation.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.lucid.Annotation} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.Annotation.toObject = function(includeInstance, msg) {
  var f, obj = {
    semantics: (f = msg.getSemantics()) && proto.speech.tts.lucid.Semantics.toObject(includeInstance, f),
    syntax: (f = msg.getSyntax()) && proto.speech.tts.lucid.Syntax.toObject(includeInstance, f),
    prosody: (f = msg.getProsody()) && proto.speech.tts.lucid.Prosody.toObject(includeInstance, f),
    synthesisParameters: (f = msg.getSynthesisParameters()) && proto.speech.tts.lucid.SynthesisParameters.toObject(includeInstance, f),
    crossLingual: (f = msg.getCrossLingual()) && proto.speech.tts.lucid.CrossLingual.toObject(includeInstance, f)
  };

  jspb.Message.toObjectExtension(/** @type {!jspb.Message} */ (msg), obj,
      proto.speech.tts.lucid.Annotation.extensions, proto.speech.tts.lucid.Annotation.prototype.getExtension,
      includeInstance);
  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Annotation as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.lucid.Annotation.ObjectFormat = function() {
  /** @type {?proto.speech.tts.lucid.Semantics.ObjectFormat|undefined} */
  this.semantics;

  /** @type {?proto.speech.tts.lucid.Syntax.ObjectFormat|undefined} */
  this.syntax;

  /** @type {?proto.speech.tts.lucid.Prosody.ObjectFormat|undefined} */
  this.prosody;

  /** @type {?proto.speech.tts.lucid.SynthesisParameters.ObjectFormat|undefined} */
  this.synthesisParameters;

  /** @type {?proto.speech.tts.lucid.CrossLingual.ObjectFormat|undefined} */
  this.crossLingual;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.lucid.Annotation.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.lucid.Annotation}
 */
proto.speech.tts.lucid.Annotation.fromObject = function(obj) {
  var msg = new proto.speech.tts.lucid.Annotation();
  obj.semantics && jspb.Message.setWrapperField(
      msg, 1, proto.speech.tts.lucid.Semantics.fromObject(obj.semantics));
  obj.syntax && jspb.Message.setWrapperField(
      msg, 2, proto.speech.tts.lucid.Syntax.fromObject(obj.syntax));
  obj.prosody && jspb.Message.setWrapperField(
      msg, 3, proto.speech.tts.lucid.Prosody.fromObject(obj.prosody));
  obj.synthesisParameters && jspb.Message.setWrapperField(
      msg, 4, proto.speech.tts.lucid.SynthesisParameters.fromObject(obj.synthesisParameters));
  obj.crossLingual && jspb.Message.setWrapperField(
      msg, 5, proto.speech.tts.lucid.CrossLingual.fromObject(obj.crossLingual));
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.lucid.Annotation}
 */
proto.speech.tts.lucid.Annotation.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.lucid.Annotation;
  return proto.speech.tts.lucid.Annotation.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.lucid.Annotation} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.lucid.Annotation}
 */
proto.speech.tts.lucid.Annotation.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.speech.tts.lucid.Semantics;
      reader.readMessage(value,proto.speech.tts.lucid.Semantics.deserializeBinaryFromReader);
      msg.setSemantics(value);
      break;
    case 2:
      var value = new proto.speech.tts.lucid.Syntax;
      reader.readMessage(value,proto.speech.tts.lucid.Syntax.deserializeBinaryFromReader);
      msg.setSyntax(value);
      break;
    case 3:
      var value = new proto.speech.tts.lucid.Prosody;
      reader.readMessage(value,proto.speech.tts.lucid.Prosody.deserializeBinaryFromReader);
      msg.setProsody(value);
      break;
    case 4:
      var value = new proto.speech.tts.lucid.SynthesisParameters;
      reader.readMessage(value,proto.speech.tts.lucid.SynthesisParameters.deserializeBinaryFromReader);
      msg.setSynthesisParameters(value);
      break;
    case 5:
      var value = new proto.speech.tts.lucid.CrossLingual;
      reader.readMessage(value,proto.speech.tts.lucid.CrossLingual.deserializeBinaryFromReader);
      msg.setCrossLingual(value);
      break;
    default:
      jspb.Message.readBinaryExtension(msg, reader,
        proto.speech.tts.lucid.Annotation.extensionsBinary,
        proto.speech.tts.lucid.Annotation.prototype.getExtension,
        proto.speech.tts.lucid.Annotation.prototype.setExtension);
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.lucid.Annotation.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.lucid.Annotation.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.lucid.Annotation} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.Annotation.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getSemantics();
  if (f != null) {
    writer.writeMessage(
      1,
      f,
      proto.speech.tts.lucid.Semantics.serializeBinaryToWriter
    );
  }
  f = message.getSyntax();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.speech.tts.lucid.Syntax.serializeBinaryToWriter
    );
  }
  f = message.getProsody();
  if (f != null) {
    writer.writeMessage(
      3,
      f,
      proto.speech.tts.lucid.Prosody.serializeBinaryToWriter
    );
  }
  f = message.getSynthesisParameters();
  if (f != null) {
    writer.writeMessage(
      4,
      f,
      proto.speech.tts.lucid.SynthesisParameters.serializeBinaryToWriter
    );
  }
  f = message.getCrossLingual();
  if (f != null) {
    writer.writeMessage(
      5,
      f,
      proto.speech.tts.lucid.CrossLingual.serializeBinaryToWriter
    );
  }
  jspb.Message.serializeBinaryExtensions(message, writer,
    proto.speech.tts.lucid.Annotation.extensionsBinary, proto.speech.tts.lucid.Annotation.prototype.getExtension);
};


/**
 * optional Semantics semantics = 1;
 * @return {?proto.speech.tts.lucid.Semantics}
 */
proto.speech.tts.lucid.Annotation.prototype.getSemantics = function() {
  return /** @type{?proto.speech.tts.lucid.Semantics} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.lucid.Semantics, 1));
};


/**
 * @param {?proto.speech.tts.lucid.Semantics|undefined} value
 * @return {!proto.speech.tts.lucid.Annotation} returns this
*/
proto.speech.tts.lucid.Annotation.prototype.setSemantics = function(value) {
  return jspb.Message.setWrapperField(this, 1, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.lucid.Annotation} returns this
 */
proto.speech.tts.lucid.Annotation.prototype.clearSemantics = function() {
  return this.setSemantics(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Annotation.prototype.hasSemantics = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional Syntax syntax = 2;
 * @return {?proto.speech.tts.lucid.Syntax}
 */
proto.speech.tts.lucid.Annotation.prototype.getSyntax = function() {
  return /** @type{?proto.speech.tts.lucid.Syntax} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.lucid.Syntax, 2));
};


/**
 * @param {?proto.speech.tts.lucid.Syntax|undefined} value
 * @return {!proto.speech.tts.lucid.Annotation} returns this
*/
proto.speech.tts.lucid.Annotation.prototype.setSyntax = function(value) {
  return jspb.Message.setWrapperField(this, 2, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.lucid.Annotation} returns this
 */
proto.speech.tts.lucid.Annotation.prototype.clearSyntax = function() {
  return this.setSyntax(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Annotation.prototype.hasSyntax = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional Prosody prosody = 3;
 * @return {?proto.speech.tts.lucid.Prosody}
 */
proto.speech.tts.lucid.Annotation.prototype.getProsody = function() {
  return /** @type{?proto.speech.tts.lucid.Prosody} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.lucid.Prosody, 3));
};


/**
 * @param {?proto.speech.tts.lucid.Prosody|undefined} value
 * @return {!proto.speech.tts.lucid.Annotation} returns this
*/
proto.speech.tts.lucid.Annotation.prototype.setProsody = function(value) {
  return jspb.Message.setWrapperField(this, 3, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.lucid.Annotation} returns this
 */
proto.speech.tts.lucid.Annotation.prototype.clearProsody = function() {
  return this.setProsody(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Annotation.prototype.hasProsody = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional SynthesisParameters synthesis_parameters = 4;
 * @return {?proto.speech.tts.lucid.SynthesisParameters}
 */
proto.speech.tts.lucid.Annotation.prototype.getSynthesisParameters = function() {
  return /** @type{?proto.speech.tts.lucid.SynthesisParameters} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.lucid.SynthesisParameters, 4));
};


/**
 * @param {?proto.speech.tts.lucid.SynthesisParameters|undefined} value
 * @return {!proto.speech.tts.lucid.Annotation} returns this
*/
proto.speech.tts.lucid.Annotation.prototype.setSynthesisParameters = function(value) {
  return jspb.Message.setWrapperField(this, 4, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.lucid.Annotation} returns this
 */
proto.speech.tts.lucid.Annotation.prototype.clearSynthesisParameters = function() {
  return this.setSynthesisParameters(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Annotation.prototype.hasSynthesisParameters = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional CrossLingual cross_lingual = 5;
 * @return {?proto.speech.tts.lucid.CrossLingual}
 */
proto.speech.tts.lucid.Annotation.prototype.getCrossLingual = function() {
  return /** @type{?proto.speech.tts.lucid.CrossLingual} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.lucid.CrossLingual, 5));
};


/**
 * @param {?proto.speech.tts.lucid.CrossLingual|undefined} value
 * @return {!proto.speech.tts.lucid.Annotation} returns this
*/
proto.speech.tts.lucid.Annotation.prototype.setCrossLingual = function(value) {
  return jspb.Message.setWrapperField(this, 5, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.lucid.Annotation} returns this
 */
proto.speech.tts.lucid.Annotation.prototype.clearCrossLingual = function() {
  return this.setCrossLingual(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Annotation.prototype.hasCrossLingual = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.lucid.Annotation}
 */
proto.speech.tts.lucid.Annotation.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.lucid.Annotation, data);
};



//speech/tts/public/lucid/annotation_proto_jspb/spanannotation.jspb.js
// source: speech/tts/public/lucid/annotation.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.lucid.SpanAnnotation');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.lucid.Annotation');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.lucid.SpanAnnotation = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.lucid.SpanAnnotation, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.lucid.SpanAnnotation.displayName = 'proto.speech.tts.lucid.SpanAnnotation';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.lucid.SpanAnnotation.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.lucid.SpanAnnotation.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.lucid.SpanAnnotation} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.SpanAnnotation.toObject = function(includeInstance, msg) {
  var f, obj = {
    first: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    last: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    annotation: (f = msg.getAnnotation()) && proto.speech.tts.lucid.Annotation.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of SpanAnnotation as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.lucid.SpanAnnotation.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.first;

  /** @type {?number|undefined} */
  this.last;

  /** @type {?proto.speech.tts.lucid.Annotation.ObjectFormat|undefined} */
  this.annotation;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.lucid.SpanAnnotation.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.lucid.SpanAnnotation}
 */
proto.speech.tts.lucid.SpanAnnotation.fromObject = function(obj) {
  var msg = new proto.speech.tts.lucid.SpanAnnotation();
  obj.first != null && jspb.Message.setField(msg, 1, obj.first);
  obj.last != null && jspb.Message.setField(msg, 2, obj.last);
  obj.annotation && jspb.Message.setWrapperField(
      msg, 3, proto.speech.tts.lucid.Annotation.fromObject(obj.annotation));
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.lucid.SpanAnnotation}
 */
proto.speech.tts.lucid.SpanAnnotation.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.lucid.SpanAnnotation;
  return proto.speech.tts.lucid.SpanAnnotation.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.lucid.SpanAnnotation} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.lucid.SpanAnnotation}
 */
proto.speech.tts.lucid.SpanAnnotation.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setFirst(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setLast(value);
      break;
    case 3:
      var value = new proto.speech.tts.lucid.Annotation;
      reader.readMessage(value,proto.speech.tts.lucid.Annotation.deserializeBinaryFromReader);
      msg.setAnnotation(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.lucid.SpanAnnotation.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.lucid.SpanAnnotation.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.lucid.SpanAnnotation} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.SpanAnnotation.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?number} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeInt32(
      1,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeInt32(
      2,
      f
    );
  }
  f = message.getAnnotation();
  if (f != null) {
    writer.writeMessage(
      3,
      f,
      proto.speech.tts.lucid.Annotation.serializeBinaryToWriter
    );
  }
};


/**
 * optional int32 first = 1;
 * @return {?number}
 */
proto.speech.tts.lucid.SpanAnnotation.prototype.getFirst = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.lucid.SpanAnnotation} returns this
 */
proto.speech.tts.lucid.SpanAnnotation.prototype.setFirst = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.SpanAnnotation} returns this
 */
proto.speech.tts.lucid.SpanAnnotation.prototype.clearFirst = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.SpanAnnotation.prototype.hasFirst = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional int32 last = 2;
 * @return {?number}
 */
proto.speech.tts.lucid.SpanAnnotation.prototype.getLast = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.lucid.SpanAnnotation} returns this
 */
proto.speech.tts.lucid.SpanAnnotation.prototype.setLast = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.SpanAnnotation} returns this
 */
proto.speech.tts.lucid.SpanAnnotation.prototype.clearLast = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.SpanAnnotation.prototype.hasLast = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional Annotation annotation = 3;
 * @return {?proto.speech.tts.lucid.Annotation}
 */
proto.speech.tts.lucid.SpanAnnotation.prototype.getAnnotation = function() {
  return /** @type{?proto.speech.tts.lucid.Annotation} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.lucid.Annotation, 3));
};


/**
 * @param {?proto.speech.tts.lucid.Annotation|undefined} value
 * @return {!proto.speech.tts.lucid.SpanAnnotation} returns this
*/
proto.speech.tts.lucid.SpanAnnotation.prototype.setAnnotation = function(value) {
  return jspb.Message.setWrapperField(this, 3, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.lucid.SpanAnnotation} returns this
 */
proto.speech.tts.lucid.SpanAnnotation.prototype.clearAnnotation = function() {
  return this.setAnnotation(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.SpanAnnotation.prototype.hasAnnotation = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.lucid.SpanAnnotation}
 */
proto.speech.tts.lucid.SpanAnnotation.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.lucid.SpanAnnotation, data);
};



//speech/tts/public/lucid/lucid_markup_proto_jspb/lucidword.jspb.js
// source: speech/tts/public/lucid/lucid_markup.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.lucid.LucidWord');
goog.provide('proto.speech.tts.lucid.LucidWord.PhonologyType');
goog.provide('proto.speech.tts.lucid.LucidWord.PronunciationVariant');
goog.provide('proto.speech.tts.lucid.LucidWord.VariantCase');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.lucid.Annotation');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.lucid.LucidWord = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, proto.speech.tts.lucid.LucidWord.oneofGroups_);
};
goog.inherits(proto.speech.tts.lucid.LucidWord, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.lucid.LucidWord.displayName = 'proto.speech.tts.lucid.LucidWord';
}

/**
 * Oneof group definitions for this message. Each group defines the field
 * numbers belonging to that group. When of these fields' value is set, all
 * other fields in the group are cleared. During deserialization, if multiple
 * fields are encountered for a group, only the last value seen will be kept.
 * @private {!Array<!Array<number>>}
 * @const
 */
proto.speech.tts.lucid.LucidWord.oneofGroups_ = [[4,5]];

/**
 * @enum {number}
 */
proto.speech.tts.lucid.LucidWord.VariantCase = {
  VARIANT_NOT_SET: 0,
  VARIANT_ENUM: 4,
  VARIANT_ID: 5
};

/**
 * @return {!proto.speech.tts.lucid.LucidWord.VariantCase}
 */
proto.speech.tts.lucid.LucidWord.prototype.getVariantCase = function() {
  return /** @type {!proto.speech.tts.lucid.LucidWord.VariantCase} */(jspb.Message.computeOneofCase(this, proto.speech.tts.lucid.LucidWord.oneofGroups_[0]));
};



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.lucid.LucidWord.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.lucid.LucidWord.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.lucid.LucidWord} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.LucidWord.toObject = function(includeInstance, msg) {
  var f, obj = {
    ownIndex: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    spelling: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    wordId: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,
    variantEnum: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f,
    variantId: (f = jspb.Message.getField(msg, 5)) == null ? undefined : f,
    phonologyType: (f = jspb.Message.getField(msg, 8)) == null ? undefined : f,
    ttsPronunciation: (f = jspb.Message.getField(msg, 6)) == null ? undefined : f,
    annotation: (f = msg.getAnnotation()) && proto.speech.tts.lucid.Annotation.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of LucidWord as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.lucid.LucidWord.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.ownIndex;

  /** @type {?string|undefined} */
  this.spelling;

  /** @type {?string|undefined} */
  this.wordId;

  /** @type {?number|undefined} */
  this.variantEnum;

  /** @type {?string|undefined} */
  this.variantId;

  /** @type {?number|undefined} */
  this.phonologyType;

  /** @type {?string|undefined} */
  this.ttsPronunciation;

  /** @type {?proto.speech.tts.lucid.Annotation.ObjectFormat|undefined} */
  this.annotation;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.lucid.LucidWord.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.lucid.LucidWord}
 */
proto.speech.tts.lucid.LucidWord.fromObject = function(obj) {
  var msg = new proto.speech.tts.lucid.LucidWord();
  obj.ownIndex != null && jspb.Message.setField(msg, 1, obj.ownIndex);
  obj.spelling != null && jspb.Message.setField(msg, 2, obj.spelling);
  obj.wordId != null && jspb.Message.setField(msg, 3, obj.wordId);
  obj.variantEnum != null && jspb.Message.setField(msg, 4, obj.variantEnum);
  obj.variantId != null && jspb.Message.setField(msg, 5, obj.variantId);
  obj.phonologyType != null && jspb.Message.setField(msg, 8, obj.phonologyType);
  obj.ttsPronunciation != null && jspb.Message.setField(msg, 6, obj.ttsPronunciation);
  obj.annotation && jspb.Message.setWrapperField(
      msg, 7, proto.speech.tts.lucid.Annotation.fromObject(obj.annotation));
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.lucid.LucidWord}
 */
proto.speech.tts.lucid.LucidWord.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.lucid.LucidWord;
  return proto.speech.tts.lucid.LucidWord.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.lucid.LucidWord} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.lucid.LucidWord}
 */
proto.speech.tts.lucid.LucidWord.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setOwnIndex(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setSpelling(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setWordId(value);
      break;
    case 4:
      var value = /** @type {!proto.speech.tts.lucid.LucidWord.PronunciationVariant} */ (reader.readEnum());
      msg.setVariantEnum(value);
      break;
    case 5:
      var value = /** @type {string} */ (reader.readString());
      msg.setVariantId(value);
      break;
    case 8:
      var value = /** @type {!proto.speech.tts.lucid.LucidWord.PhonologyType} */ (reader.readEnum());
      msg.setPhonologyType(value);
      break;
    case 6:
      var value = /** @type {string} */ (reader.readString());
      msg.setTtsPronunciation(value);
      break;
    case 7:
      var value = new proto.speech.tts.lucid.Annotation;
      reader.readMessage(value,proto.speech.tts.lucid.Annotation.deserializeBinaryFromReader);
      msg.setAnnotation(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.lucid.LucidWord.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.lucid.LucidWord.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.lucid.LucidWord} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.LucidWord.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?number} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeInt32(
      1,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeString(
      3,
      f
    );
  }
  f = /** @type {!proto.speech.tts.lucid.LucidWord.PronunciationVariant} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeEnum(
      4,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeString(
      5,
      f
    );
  }
  f = /** @type {!proto.speech.tts.lucid.LucidWord.PhonologyType} */ (jspb.Message.getField(message, 8));
  if (f != null) {
    writer.writeEnum(
      8,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeString(
      6,
      f
    );
  }
  f = message.getAnnotation();
  if (f != null) {
    writer.writeMessage(
      7,
      f,
      proto.speech.tts.lucid.Annotation.serializeBinaryToWriter
    );
  }
};


/**
 * @enum {number}
 */
proto.speech.tts.lucid.LucidWord.PronunciationVariant = {
  UNSPECIFIED_PRONUNCIATION_VARIANT: 0,
  REDUCED: 1,
  CANONICAL: 2
};

/**
 * @enum {number}
 */
proto.speech.tts.lucid.LucidWord.PhonologyType = {
  UNSPECIFIED_PHONOLOGY_TYPE: 0,
  TTS_NATIVE: 1,
  IPA: 2,
  XSAMPA_STANDARD: 3,
  XSAMPA_PRON_BUFFET: 4,
  TTS_LEGACY: 5,
  JAPANESE_YOMIGANA: 6,
  PINYIN: 7,
  JYUTPING: 8
};

/**
 * optional int32 own_index = 1;
 * @return {?number}
 */
proto.speech.tts.lucid.LucidWord.prototype.getOwnIndex = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.lucid.LucidWord} returns this
 */
proto.speech.tts.lucid.LucidWord.prototype.setOwnIndex = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.LucidWord} returns this
 */
proto.speech.tts.lucid.LucidWord.prototype.clearOwnIndex = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.LucidWord.prototype.hasOwnIndex = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional string spelling = 2;
 * @return {?string}
 */
proto.speech.tts.lucid.LucidWord.prototype.getSpelling = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.lucid.LucidWord} returns this
 */
proto.speech.tts.lucid.LucidWord.prototype.setSpelling = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.LucidWord} returns this
 */
proto.speech.tts.lucid.LucidWord.prototype.clearSpelling = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.LucidWord.prototype.hasSpelling = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional string word_id = 3;
 * @return {?string}
 */
proto.speech.tts.lucid.LucidWord.prototype.getWordId = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.lucid.LucidWord} returns this
 */
proto.speech.tts.lucid.LucidWord.prototype.setWordId = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.LucidWord} returns this
 */
proto.speech.tts.lucid.LucidWord.prototype.clearWordId = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.LucidWord.prototype.hasWordId = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional PronunciationVariant variant_enum = 4;
 * @return {!proto.speech.tts.lucid.LucidWord.PronunciationVariant}
 */
proto.speech.tts.lucid.LucidWord.prototype.getVariantEnum = function() {
  return /** @type {!proto.speech.tts.lucid.LucidWord.PronunciationVariant} */ (jspb.Message.getField(this, 4));
};


/**
 * @param {?proto.speech.tts.lucid.LucidWord.PronunciationVariant|undefined} value
 * @return {!proto.speech.tts.lucid.LucidWord} returns this
 */
proto.speech.tts.lucid.LucidWord.prototype.setVariantEnum = function(value) {
  return jspb.Message.setOneofField(this, 4, proto.speech.tts.lucid.LucidWord.oneofGroups_[0], value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.LucidWord} returns this
 */
proto.speech.tts.lucid.LucidWord.prototype.clearVariantEnum = function() {
  return jspb.Message.setOneofField(this, 4, proto.speech.tts.lucid.LucidWord.oneofGroups_[0], undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.LucidWord.prototype.hasVariantEnum = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional string variant_id = 5;
 * @return {?string}
 */
proto.speech.tts.lucid.LucidWord.prototype.getVariantId = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 5));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.lucid.LucidWord} returns this
 */
proto.speech.tts.lucid.LucidWord.prototype.setVariantId = function(value) {
  return jspb.Message.setOneofField(this, 5, proto.speech.tts.lucid.LucidWord.oneofGroups_[0], value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.LucidWord} returns this
 */
proto.speech.tts.lucid.LucidWord.prototype.clearVariantId = function() {
  return jspb.Message.setOneofField(this, 5, proto.speech.tts.lucid.LucidWord.oneofGroups_[0], undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.LucidWord.prototype.hasVariantId = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional PhonologyType phonology_type = 8;
 * @return {!proto.speech.tts.lucid.LucidWord.PhonologyType}
 */
proto.speech.tts.lucid.LucidWord.prototype.getPhonologyType = function() {
  return /** @type {!proto.speech.tts.lucid.LucidWord.PhonologyType} */ (jspb.Message.getField(this, 8));
};


/**
 * @param {?proto.speech.tts.lucid.LucidWord.PhonologyType|undefined} value
 * @return {!proto.speech.tts.lucid.LucidWord} returns this
 */
proto.speech.tts.lucid.LucidWord.prototype.setPhonologyType = function(value) {
  return jspb.Message.setField(this, 8, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.LucidWord} returns this
 */
proto.speech.tts.lucid.LucidWord.prototype.clearPhonologyType = function() {
  return jspb.Message.setField(this, 8, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.LucidWord.prototype.hasPhonologyType = function() {
  return jspb.Message.getField(this, 8) != null;
};


/**
 * optional string tts_pronunciation = 6;
 * @return {?string}
 */
proto.speech.tts.lucid.LucidWord.prototype.getTtsPronunciation = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 6));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.lucid.LucidWord} returns this
 */
proto.speech.tts.lucid.LucidWord.prototype.setTtsPronunciation = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.LucidWord} returns this
 */
proto.speech.tts.lucid.LucidWord.prototype.clearTtsPronunciation = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.LucidWord.prototype.hasTtsPronunciation = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional Annotation annotation = 7;
 * @return {?proto.speech.tts.lucid.Annotation}
 */
proto.speech.tts.lucid.LucidWord.prototype.getAnnotation = function() {
  return /** @type{?proto.speech.tts.lucid.Annotation} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.lucid.Annotation, 7));
};


/**
 * @param {?proto.speech.tts.lucid.Annotation|undefined} value
 * @return {!proto.speech.tts.lucid.LucidWord} returns this
*/
proto.speech.tts.lucid.LucidWord.prototype.setAnnotation = function(value) {
  return jspb.Message.setWrapperField(this, 7, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.lucid.LucidWord} returns this
 */
proto.speech.tts.lucid.LucidWord.prototype.clearAnnotation = function() {
  return this.setAnnotation(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.LucidWord.prototype.hasAnnotation = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.lucid.LucidWord}
 */
proto.speech.tts.lucid.LucidWord.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.lucid.LucidWord, data);
};



//speech/tts/public/lucid/structured_token_proto_jspb/abbreviation.jspb.js
// source: speech/tts/public/lucid/structured_token.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.lucid.Abbreviation');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.lucid.Abbreviation = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.lucid.Abbreviation, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.lucid.Abbreviation.displayName = 'proto.speech.tts.lucid.Abbreviation';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.lucid.Abbreviation.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.lucid.Abbreviation.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.lucid.Abbreviation} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.Abbreviation.toObject = function(includeInstance, msg) {
  var f, obj = {
    text: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    expansion: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Abbreviation as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.lucid.Abbreviation.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.text;

  /** @type {?string|undefined} */
  this.expansion;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.lucid.Abbreviation.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.lucid.Abbreviation}
 */
proto.speech.tts.lucid.Abbreviation.fromObject = function(obj) {
  var msg = new proto.speech.tts.lucid.Abbreviation();
  obj.text != null && jspb.Message.setField(msg, 1, obj.text);
  obj.expansion != null && jspb.Message.setField(msg, 2, obj.expansion);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.lucid.Abbreviation}
 */
proto.speech.tts.lucid.Abbreviation.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.lucid.Abbreviation;
  return proto.speech.tts.lucid.Abbreviation.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.lucid.Abbreviation} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.lucid.Abbreviation}
 */
proto.speech.tts.lucid.Abbreviation.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setText(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setExpansion(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.lucid.Abbreviation.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.lucid.Abbreviation.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.lucid.Abbreviation} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.Abbreviation.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string text = 1;
 * @return {?string}
 */
proto.speech.tts.lucid.Abbreviation.prototype.getText = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.lucid.Abbreviation} returns this
 */
proto.speech.tts.lucid.Abbreviation.prototype.setText = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Abbreviation} returns this
 */
proto.speech.tts.lucid.Abbreviation.prototype.clearText = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Abbreviation.prototype.hasText = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional string expansion = 2;
 * @return {?string}
 */
proto.speech.tts.lucid.Abbreviation.prototype.getExpansion = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.lucid.Abbreviation} returns this
 */
proto.speech.tts.lucid.Abbreviation.prototype.setExpansion = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Abbreviation} returns this
 */
proto.speech.tts.lucid.Abbreviation.prototype.clearExpansion = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Abbreviation.prototype.hasExpansion = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.lucid.Abbreviation}
 */
proto.speech.tts.lucid.Abbreviation.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.lucid.Abbreviation, data);
};



//speech/tts/public/lucid/structured_token_proto_jspb/chemicalformula.jspb.js
// source: speech/tts/public/lucid/structured_token.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.lucid.ChemicalFormula');
goog.provide('proto.speech.tts.lucid.ChemicalFormula.Unit');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.lucid.ChemicalFormula = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.lucid.ChemicalFormula.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.lucid.ChemicalFormula, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.lucid.ChemicalFormula.displayName = 'proto.speech.tts.lucid.ChemicalFormula';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.lucid.ChemicalFormula.Unit = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.lucid.ChemicalFormula.Unit, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.lucid.ChemicalFormula.Unit.displayName = 'proto.speech.tts.lucid.ChemicalFormula.Unit';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.lucid.ChemicalFormula.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.lucid.ChemicalFormula.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.lucid.ChemicalFormula.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.lucid.ChemicalFormula} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.ChemicalFormula.toObject = function(includeInstance, msg) {
  var f, obj = {
    unitsList: jspb.Message.toObjectList(msg.getUnitsList(),
    proto.speech.tts.lucid.ChemicalFormula.Unit.toObject, includeInstance),
    type: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    charge: (f = jspb.Message.getBooleanField(msg, 3)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of ChemicalFormula as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.lucid.ChemicalFormula.ObjectFormat = function() {
  /** @type {?Array<!proto.speech.tts.lucid.ChemicalFormula.Unit.ObjectFormat>|undefined} */
  this.unitsList;

  /** @type {?string|undefined} */
  this.type;

  /** @type {?boolean|undefined} */
  this.charge;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.lucid.ChemicalFormula.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.lucid.ChemicalFormula}
 */
proto.speech.tts.lucid.ChemicalFormula.fromObject = function(obj) {
  var msg = new proto.speech.tts.lucid.ChemicalFormula();
  obj.unitsList && jspb.Message.setRepeatedWrapperField(
      msg, 1, obj.unitsList.map(
          proto.speech.tts.lucid.ChemicalFormula.Unit.fromObject));
  obj.type != null && jspb.Message.setField(msg, 2, obj.type);
  obj.charge != null && jspb.Message.setField(msg, 3, obj.charge);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.lucid.ChemicalFormula}
 */
proto.speech.tts.lucid.ChemicalFormula.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.lucid.ChemicalFormula;
  return proto.speech.tts.lucid.ChemicalFormula.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.lucid.ChemicalFormula} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.lucid.ChemicalFormula}
 */
proto.speech.tts.lucid.ChemicalFormula.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.speech.tts.lucid.ChemicalFormula.Unit;
      reader.readMessage(value,proto.speech.tts.lucid.ChemicalFormula.Unit.deserializeBinaryFromReader);
      msg.addUnits(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setType(value);
      break;
    case 3:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setCharge(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.lucid.ChemicalFormula.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.lucid.ChemicalFormula.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.lucid.ChemicalFormula} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.ChemicalFormula.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getUnitsList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      1,
      f,
      proto.speech.tts.lucid.ChemicalFormula.Unit.serializeBinaryToWriter
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeBool(
      3,
      f
    );
  }
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.lucid.ChemicalFormula.Unit.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.lucid.ChemicalFormula.Unit.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.lucid.ChemicalFormula.Unit} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.ChemicalFormula.Unit.toObject = function(includeInstance, msg) {
  var f, obj = {
    symbol: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    number: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Unit as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.lucid.ChemicalFormula.Unit.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.symbol;

  /** @type {?string|undefined} */
  this.number;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.lucid.ChemicalFormula.Unit.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.lucid.ChemicalFormula.Unit}
 */
proto.speech.tts.lucid.ChemicalFormula.Unit.fromObject = function(obj) {
  var msg = new proto.speech.tts.lucid.ChemicalFormula.Unit();
  obj.symbol != null && jspb.Message.setField(msg, 1, obj.symbol);
  obj.number != null && jspb.Message.setField(msg, 2, obj.number);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.lucid.ChemicalFormula.Unit}
 */
proto.speech.tts.lucid.ChemicalFormula.Unit.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.lucid.ChemicalFormula.Unit;
  return proto.speech.tts.lucid.ChemicalFormula.Unit.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.lucid.ChemicalFormula.Unit} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.lucid.ChemicalFormula.Unit}
 */
proto.speech.tts.lucid.ChemicalFormula.Unit.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setSymbol(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setNumber(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.lucid.ChemicalFormula.Unit.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.lucid.ChemicalFormula.Unit.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.lucid.ChemicalFormula.Unit} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.ChemicalFormula.Unit.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string symbol = 1;
 * @return {?string}
 */
proto.speech.tts.lucid.ChemicalFormula.Unit.prototype.getSymbol = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.lucid.ChemicalFormula.Unit} returns this
 */
proto.speech.tts.lucid.ChemicalFormula.Unit.prototype.setSymbol = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.ChemicalFormula.Unit} returns this
 */
proto.speech.tts.lucid.ChemicalFormula.Unit.prototype.clearSymbol = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.ChemicalFormula.Unit.prototype.hasSymbol = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional string number = 2;
 * @return {?string}
 */
proto.speech.tts.lucid.ChemicalFormula.Unit.prototype.getNumber = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.lucid.ChemicalFormula.Unit} returns this
 */
proto.speech.tts.lucid.ChemicalFormula.Unit.prototype.setNumber = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.ChemicalFormula.Unit} returns this
 */
proto.speech.tts.lucid.ChemicalFormula.Unit.prototype.clearNumber = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.ChemicalFormula.Unit.prototype.hasNumber = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.lucid.ChemicalFormula.Unit}
 */
proto.speech.tts.lucid.ChemicalFormula.Unit.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.lucid.ChemicalFormula.Unit, data);
};


/**
 * repeated Unit units = 1;
 * @return {!Array<!proto.speech.tts.lucid.ChemicalFormula.Unit>}
 */
proto.speech.tts.lucid.ChemicalFormula.prototype.getUnitsList = function() {
  return /** @type{!Array<!proto.speech.tts.lucid.ChemicalFormula.Unit>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.lucid.ChemicalFormula.Unit, 1));
};


/**
 * @param {Array<!proto.speech.tts.lucid.ChemicalFormula.Unit>|undefined} value
 * @return {!proto.speech.tts.lucid.ChemicalFormula} returns this
*/
proto.speech.tts.lucid.ChemicalFormula.prototype.setUnitsList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 1, value);
};


/**
 * @param {!proto.speech.tts.lucid.ChemicalFormula.Unit=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.lucid.ChemicalFormula.Unit}
 */
proto.speech.tts.lucid.ChemicalFormula.prototype.addUnits = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.speech.tts.lucid.ChemicalFormula.Unit, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.lucid.ChemicalFormula} returns this
 */
proto.speech.tts.lucid.ChemicalFormula.prototype.clearUnitsList = function() {
  return this.setUnitsList([]);
};


/**
 * optional string type = 2;
 * @return {?string}
 */
proto.speech.tts.lucid.ChemicalFormula.prototype.getType = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.lucid.ChemicalFormula} returns this
 */
proto.speech.tts.lucid.ChemicalFormula.prototype.setType = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.ChemicalFormula} returns this
 */
proto.speech.tts.lucid.ChemicalFormula.prototype.clearType = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.ChemicalFormula.prototype.hasType = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional bool charge = 3;
 * @return {?boolean}
 */
proto.speech.tts.lucid.ChemicalFormula.prototype.getCharge = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 3));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.lucid.ChemicalFormula} returns this
 */
proto.speech.tts.lucid.ChemicalFormula.prototype.setCharge = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.ChemicalFormula} returns this
 */
proto.speech.tts.lucid.ChemicalFormula.prototype.clearCharge = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.ChemicalFormula.prototype.hasCharge = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.lucid.ChemicalFormula}
 */
proto.speech.tts.lucid.ChemicalFormula.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.lucid.ChemicalFormula, data);
};



//speech/tts/public/lucid/structured_token_proto_jspb/connector.jspb.js
// source: speech/tts/public/lucid/structured_token.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.lucid.Connector');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.lucid.Connector = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.lucid.Connector, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.lucid.Connector.displayName = 'proto.speech.tts.lucid.Connector';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.lucid.Connector.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.lucid.Connector.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.lucid.Connector} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.Connector.toObject = function(includeInstance, msg) {
  var f, obj = {
    text: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    expansion: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Connector as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.lucid.Connector.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.text;

  /** @type {?string|undefined} */
  this.expansion;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.lucid.Connector.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.lucid.Connector}
 */
proto.speech.tts.lucid.Connector.fromObject = function(obj) {
  var msg = new proto.speech.tts.lucid.Connector();
  obj.text != null && jspb.Message.setField(msg, 1, obj.text);
  obj.expansion != null && jspb.Message.setField(msg, 2, obj.expansion);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.lucid.Connector}
 */
proto.speech.tts.lucid.Connector.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.lucid.Connector;
  return proto.speech.tts.lucid.Connector.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.lucid.Connector} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.lucid.Connector}
 */
proto.speech.tts.lucid.Connector.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setText(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setExpansion(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.lucid.Connector.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.lucid.Connector.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.lucid.Connector} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.Connector.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string text = 1;
 * @return {?string}
 */
proto.speech.tts.lucid.Connector.prototype.getText = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.lucid.Connector} returns this
 */
proto.speech.tts.lucid.Connector.prototype.setText = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Connector} returns this
 */
proto.speech.tts.lucid.Connector.prototype.clearText = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Connector.prototype.hasText = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional string expansion = 2;
 * @return {?string}
 */
proto.speech.tts.lucid.Connector.prototype.getExpansion = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.lucid.Connector} returns this
 */
proto.speech.tts.lucid.Connector.prototype.setExpansion = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Connector} returns this
 */
proto.speech.tts.lucid.Connector.prototype.clearExpansion = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Connector.prototype.hasExpansion = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.lucid.Connector}
 */
proto.speech.tts.lucid.Connector.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.lucid.Connector, data);
};



//speech/tts/public/lucid/structured_token_proto_jspb/date.jspb.js
// source: speech/tts/public/lucid/structured_token.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.lucid.Date');
goog.provide('proto.speech.tts.lucid.Date.Calendar');
goog.provide('proto.speech.tts.lucid.Date.MonthCase');
goog.provide('proto.speech.tts.lucid.Date.Style');
goog.provide('proto.speech.tts.lucid.Date.StyleCase');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

goog.forwardDeclare('proto.speech.tts.lucid.CommonVerbalizationStyles.Style');
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.lucid.Date = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, proto.speech.tts.lucid.Date.oneofGroups_);
};
goog.inherits(proto.speech.tts.lucid.Date, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.lucid.Date.displayName = 'proto.speech.tts.lucid.Date';
}

/**
 * Oneof group definitions for this message. Each group defines the field
 * numbers belonging to that group. When of these fields' value is set, all
 * other fields in the group are cleared. During deserialization, if multiple
 * fields are encountered for a group, only the last value seen will be kept.
 * @private {!Array<!Array<number>>}
 * @const
 */
proto.speech.tts.lucid.Date.oneofGroups_ = [[3,4],[8,9,10]];

/**
 * @enum {number}
 */
proto.speech.tts.lucid.Date.MonthCase = {
  MONTH_NOT_SET: 0,
  MONTH_NUMBER: 3,
  MONTH_NAME: 4
};

/**
 * @return {!proto.speech.tts.lucid.Date.MonthCase}
 */
proto.speech.tts.lucid.Date.prototype.getMonthCase = function() {
  return /** @type {!proto.speech.tts.lucid.Date.MonthCase} */(jspb.Message.computeOneofCase(this, proto.speech.tts.lucid.Date.oneofGroups_[0]));
};

/**
 * @enum {number}
 */
proto.speech.tts.lucid.Date.StyleCase = {
  STYLE_NOT_SET: 0,
  COMMON_STYLE: 8,
  SPECIFIC_STYLE: 9,
  STYLE_ID: 10
};

/**
 * @return {!proto.speech.tts.lucid.Date.StyleCase}
 */
proto.speech.tts.lucid.Date.prototype.getStyleCase = function() {
  return /** @type {!proto.speech.tts.lucid.Date.StyleCase} */(jspb.Message.computeOneofCase(this, proto.speech.tts.lucid.Date.oneofGroups_[1]));
};



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.lucid.Date.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.lucid.Date.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.lucid.Date} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.Date.toObject = function(includeInstance, msg) {
  var f, obj = {
    weekday: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    day: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    monthNumber: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,
    monthName: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f,
    year: (f = jspb.Message.getField(msg, 5)) == null ? undefined : f,
    calendar: (f = jspb.Message.getField(msg, 6)) == null ? undefined : f,
    era: (f = jspb.Message.getField(msg, 7)) == null ? undefined : f,
    commonStyle: (f = jspb.Message.getField(msg, 8)) == null ? undefined : f,
    specificStyle: (f = jspb.Message.getField(msg, 9)) == null ? undefined : f,
    styleId: (f = jspb.Message.getField(msg, 10)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Date as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.lucid.Date.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.weekday;

  /** @type {?number|undefined} */
  this.day;

  /** @type {?number|undefined} */
  this.monthNumber;

  /** @type {?string|undefined} */
  this.monthName;

  /** @type {?number|undefined} */
  this.year;

  /** @type {?number|undefined} */
  this.calendar;

  /** @type {?string|undefined} */
  this.era;

  /** @type {?number|undefined} */
  this.commonStyle;

  /** @type {?number|undefined} */
  this.specificStyle;

  /** @type {?number|undefined} */
  this.styleId;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.lucid.Date.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.lucid.Date}
 */
proto.speech.tts.lucid.Date.fromObject = function(obj) {
  var msg = new proto.speech.tts.lucid.Date();
  obj.weekday != null && jspb.Message.setField(msg, 1, obj.weekday);
  obj.day != null && jspb.Message.setField(msg, 2, obj.day);
  obj.monthNumber != null && jspb.Message.setField(msg, 3, obj.monthNumber);
  obj.monthName != null && jspb.Message.setField(msg, 4, obj.monthName);
  obj.year != null && jspb.Message.setField(msg, 5, obj.year);
  obj.calendar != null && jspb.Message.setField(msg, 6, obj.calendar);
  obj.era != null && jspb.Message.setField(msg, 7, obj.era);
  obj.commonStyle != null && jspb.Message.setField(msg, 8, obj.commonStyle);
  obj.specificStyle != null && jspb.Message.setField(msg, 9, obj.specificStyle);
  obj.styleId != null && jspb.Message.setField(msg, 10, obj.styleId);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.lucid.Date}
 */
proto.speech.tts.lucid.Date.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.lucid.Date;
  return proto.speech.tts.lucid.Date.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.lucid.Date} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.lucid.Date}
 */
proto.speech.tts.lucid.Date.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setWeekday(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setDay(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setMonthNumber(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setMonthName(value);
      break;
    case 5:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setYear(value);
      break;
    case 6:
      var value = /** @type {!proto.speech.tts.lucid.Date.Calendar} */ (reader.readEnum());
      msg.setCalendar(value);
      break;
    case 7:
      var value = /** @type {string} */ (reader.readString());
      msg.setEra(value);
      break;
    case 8:
      var value = /** @type {!proto.speech.tts.lucid.CommonVerbalizationStyles.Style} */ (reader.readEnum());
      msg.setCommonStyle(value);
      break;
    case 9:
      var value = /** @type {!proto.speech.tts.lucid.Date.Style} */ (reader.readEnum());
      msg.setSpecificStyle(value);
      break;
    case 10:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setStyleId(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.lucid.Date.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.lucid.Date.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.lucid.Date} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.Date.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?number} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeInt32(
      1,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeInt32(
      2,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeInt32(
      3,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeString(
      4,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeInt32(
      5,
      f
    );
  }
  f = /** @type {!proto.speech.tts.lucid.Date.Calendar} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeEnum(
      6,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 7));
  if (f != null) {
    writer.writeString(
      7,
      f
    );
  }
  f = /** @type {!proto.speech.tts.lucid.CommonVerbalizationStyles.Style} */ (jspb.Message.getField(message, 8));
  if (f != null) {
    writer.writeEnum(
      8,
      f
    );
  }
  f = /** @type {!proto.speech.tts.lucid.Date.Style} */ (jspb.Message.getField(message, 9));
  if (f != null) {
    writer.writeEnum(
      9,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 10));
  if (f != null) {
    writer.writeInt32(
      10,
      f
    );
  }
};


/**
 * @enum {number}
 */
proto.speech.tts.lucid.Date.Calendar = {
  UNSPECIFIED_CALENDAR: 0,
  GREGORIAN: 1
};

/**
 * @enum {number}
 */
proto.speech.tts.lucid.Date.Style = {
  UNSPECIFIED_STYLE: 0,
  DAY_MONTHNAME_YEAR: 1,
  MONTHNAME_DAY_YEAR: 2,
  DAY_MONTHNUMBER_YEAR: 3,
  MONTHNUMBER_DAY_YEAR: 4
};

/**
 * optional int32 weekday = 1;
 * @return {?number}
 */
proto.speech.tts.lucid.Date.prototype.getWeekday = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.lucid.Date} returns this
 */
proto.speech.tts.lucid.Date.prototype.setWeekday = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Date} returns this
 */
proto.speech.tts.lucid.Date.prototype.clearWeekday = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Date.prototype.hasWeekday = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional int32 day = 2;
 * @return {?number}
 */
proto.speech.tts.lucid.Date.prototype.getDay = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.lucid.Date} returns this
 */
proto.speech.tts.lucid.Date.prototype.setDay = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Date} returns this
 */
proto.speech.tts.lucid.Date.prototype.clearDay = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Date.prototype.hasDay = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional int32 month_number = 3;
 * @return {?number}
 */
proto.speech.tts.lucid.Date.prototype.getMonthNumber = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.lucid.Date} returns this
 */
proto.speech.tts.lucid.Date.prototype.setMonthNumber = function(value) {
  return jspb.Message.setOneofField(this, 3, proto.speech.tts.lucid.Date.oneofGroups_[0], value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Date} returns this
 */
proto.speech.tts.lucid.Date.prototype.clearMonthNumber = function() {
  return jspb.Message.setOneofField(this, 3, proto.speech.tts.lucid.Date.oneofGroups_[0], undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Date.prototype.hasMonthNumber = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional string month_name = 4;
 * @return {?string}
 */
proto.speech.tts.lucid.Date.prototype.getMonthName = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 4));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.lucid.Date} returns this
 */
proto.speech.tts.lucid.Date.prototype.setMonthName = function(value) {
  return jspb.Message.setOneofField(this, 4, proto.speech.tts.lucid.Date.oneofGroups_[0], value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Date} returns this
 */
proto.speech.tts.lucid.Date.prototype.clearMonthName = function() {
  return jspb.Message.setOneofField(this, 4, proto.speech.tts.lucid.Date.oneofGroups_[0], undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Date.prototype.hasMonthName = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional int32 year = 5;
 * @return {?number}
 */
proto.speech.tts.lucid.Date.prototype.getYear = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 5));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.lucid.Date} returns this
 */
proto.speech.tts.lucid.Date.prototype.setYear = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Date} returns this
 */
proto.speech.tts.lucid.Date.prototype.clearYear = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Date.prototype.hasYear = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional Calendar calendar = 6;
 * @return {!proto.speech.tts.lucid.Date.Calendar}
 */
proto.speech.tts.lucid.Date.prototype.getCalendar = function() {
  return /** @type {!proto.speech.tts.lucid.Date.Calendar} */ (jspb.Message.getField(this, 6));
};


/**
 * @param {?proto.speech.tts.lucid.Date.Calendar|undefined} value
 * @return {!proto.speech.tts.lucid.Date} returns this
 */
proto.speech.tts.lucid.Date.prototype.setCalendar = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Date} returns this
 */
proto.speech.tts.lucid.Date.prototype.clearCalendar = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Date.prototype.hasCalendar = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional string era = 7;
 * @return {?string}
 */
proto.speech.tts.lucid.Date.prototype.getEra = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 7));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.lucid.Date} returns this
 */
proto.speech.tts.lucid.Date.prototype.setEra = function(value) {
  return jspb.Message.setField(this, 7, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Date} returns this
 */
proto.speech.tts.lucid.Date.prototype.clearEra = function() {
  return jspb.Message.setField(this, 7, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Date.prototype.hasEra = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * optional CommonVerbalizationStyles.Style common_style = 8;
 * @return {!proto.speech.tts.lucid.CommonVerbalizationStyles.Style}
 */
proto.speech.tts.lucid.Date.prototype.getCommonStyle = function() {
  return /** @type {!proto.speech.tts.lucid.CommonVerbalizationStyles.Style} */ (jspb.Message.getField(this, 8));
};


/**
 * @param {?proto.speech.tts.lucid.CommonVerbalizationStyles.Style|undefined} value
 * @return {!proto.speech.tts.lucid.Date} returns this
 */
proto.speech.tts.lucid.Date.prototype.setCommonStyle = function(value) {
  return jspb.Message.setOneofField(this, 8, proto.speech.tts.lucid.Date.oneofGroups_[1], value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Date} returns this
 */
proto.speech.tts.lucid.Date.prototype.clearCommonStyle = function() {
  return jspb.Message.setOneofField(this, 8, proto.speech.tts.lucid.Date.oneofGroups_[1], undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Date.prototype.hasCommonStyle = function() {
  return jspb.Message.getField(this, 8) != null;
};


/**
 * optional Style specific_style = 9;
 * @return {!proto.speech.tts.lucid.Date.Style}
 */
proto.speech.tts.lucid.Date.prototype.getSpecificStyle = function() {
  return /** @type {!proto.speech.tts.lucid.Date.Style} */ (jspb.Message.getField(this, 9));
};


/**
 * @param {?proto.speech.tts.lucid.Date.Style|undefined} value
 * @return {!proto.speech.tts.lucid.Date} returns this
 */
proto.speech.tts.lucid.Date.prototype.setSpecificStyle = function(value) {
  return jspb.Message.setOneofField(this, 9, proto.speech.tts.lucid.Date.oneofGroups_[1], value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Date} returns this
 */
proto.speech.tts.lucid.Date.prototype.clearSpecificStyle = function() {
  return jspb.Message.setOneofField(this, 9, proto.speech.tts.lucid.Date.oneofGroups_[1], undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Date.prototype.hasSpecificStyle = function() {
  return jspb.Message.getField(this, 9) != null;
};


/**
 * optional int32 style_id = 10;
 * @return {?number}
 */
proto.speech.tts.lucid.Date.prototype.getStyleId = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 10));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.lucid.Date} returns this
 */
proto.speech.tts.lucid.Date.prototype.setStyleId = function(value) {
  return jspb.Message.setOneofField(this, 10, proto.speech.tts.lucid.Date.oneofGroups_[1], value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Date} returns this
 */
proto.speech.tts.lucid.Date.prototype.clearStyleId = function() {
  return jspb.Message.setOneofField(this, 10, proto.speech.tts.lucid.Date.oneofGroups_[1], undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Date.prototype.hasStyleId = function() {
  return jspb.Message.getField(this, 10) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.lucid.Date}
 */
proto.speech.tts.lucid.Date.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.lucid.Date, data);
};



//speech/tts/public/lucid/structured_token_proto_jspb/digits.jspb.js
// source: speech/tts/public/lucid/structured_token.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.lucid.Digits');
goog.provide('proto.speech.tts.lucid.Digits.Style');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.lucid.Digits = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.lucid.Digits.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.lucid.Digits, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.lucid.Digits.displayName = 'proto.speech.tts.lucid.Digits';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.lucid.Digits.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.lucid.Digits.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.lucid.Digits.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.lucid.Digits} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.Digits.toObject = function(includeInstance, msg) {
  var f, obj = {
    chunksList: (f = jspb.Message.getRepeatedField(msg, 1)) == null ? undefined : f,
    style: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Digits as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.lucid.Digits.ObjectFormat = function() {
  /** @type {?Array<string>|undefined} */
  this.chunksList;

  /** @type {?number|undefined} */
  this.style;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.lucid.Digits.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.lucid.Digits}
 */
proto.speech.tts.lucid.Digits.fromObject = function(obj) {
  var msg = new proto.speech.tts.lucid.Digits();
  obj.chunksList != null && jspb.Message.setField(msg, 1, obj.chunksList);
  obj.style != null && jspb.Message.setField(msg, 2, obj.style);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.lucid.Digits}
 */
proto.speech.tts.lucid.Digits.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.lucid.Digits;
  return proto.speech.tts.lucid.Digits.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.lucid.Digits} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.lucid.Digits}
 */
proto.speech.tts.lucid.Digits.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.addChunks(value);
      break;
    case 2:
      var value = /** @type {!proto.speech.tts.lucid.Digits.Style} */ (reader.readEnum());
      msg.setStyle(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.lucid.Digits.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.lucid.Digits.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.lucid.Digits} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.Digits.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getChunksList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      1,
      f
    );
  }
  f = /** @type {!proto.speech.tts.lucid.Digits.Style} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeEnum(
      2,
      f
    );
  }
};


/**
 * @enum {number}
 */
proto.speech.tts.lucid.Digits.Style = {
  UNSPECIFIED_STYLE: 0,
  SEPARATE_DIGITS: 1,
  GROUP_REPETITIONS: 2,
  AS_CARDINAL: 3,
  AS_CARDINAL_GROUPING_ZEROS: 4
};

/**
 * repeated string chunks = 1;
 * @return {!Array<string>}
 */
proto.speech.tts.lucid.Digits.prototype.getChunksList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 1));
};


/**
 * @param {Array<string>|undefined} value
 * @return {!proto.speech.tts.lucid.Digits} returns this
 */
proto.speech.tts.lucid.Digits.prototype.setChunksList = function(value) {
  return jspb.Message.setField(this, 1, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.lucid.Digits} returns this
 */
proto.speech.tts.lucid.Digits.prototype.addChunks = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 1, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.lucid.Digits} returns this
 */
proto.speech.tts.lucid.Digits.prototype.clearChunksList = function() {
  return this.setChunksList([]);
};


/**
 * optional Style style = 2;
 * @return {!proto.speech.tts.lucid.Digits.Style}
 */
proto.speech.tts.lucid.Digits.prototype.getStyle = function() {
  return /** @type {!proto.speech.tts.lucid.Digits.Style} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?proto.speech.tts.lucid.Digits.Style|undefined} value
 * @return {!proto.speech.tts.lucid.Digits} returns this
 */
proto.speech.tts.lucid.Digits.prototype.setStyle = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Digits} returns this
 */
proto.speech.tts.lucid.Digits.prototype.clearStyle = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Digits.prototype.hasStyle = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.lucid.Digits}
 */
proto.speech.tts.lucid.Digits.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.lucid.Digits, data);
};



//speech/tts/public/lucid/structured_token_proto_jspb/duration.jspb.js
// source: speech/tts/public/lucid/structured_token.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.lucid.Duration');
goog.provide('proto.speech.tts.lucid.Duration.StyleCase');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

goog.forwardDeclare('proto.speech.tts.lucid.CommonVerbalizationStyles.Style');
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.lucid.Duration = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, proto.speech.tts.lucid.Duration.oneofGroups_);
};
goog.inherits(proto.speech.tts.lucid.Duration, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.lucid.Duration.displayName = 'proto.speech.tts.lucid.Duration';
}

/**
 * Oneof group definitions for this message. Each group defines the field
 * numbers belonging to that group. When of these fields' value is set, all
 * other fields in the group are cleared. During deserialization, if multiple
 * fields are encountered for a group, only the last value seen will be kept.
 * @private {!Array<!Array<number>>}
 * @const
 */
proto.speech.tts.lucid.Duration.oneofGroups_ = [[5,6]];

/**
 * @enum {number}
 */
proto.speech.tts.lucid.Duration.StyleCase = {
  STYLE_NOT_SET: 0,
  COMMON_STYLE: 5,
  STYLE_ID: 6
};

/**
 * @return {!proto.speech.tts.lucid.Duration.StyleCase}
 */
proto.speech.tts.lucid.Duration.prototype.getStyleCase = function() {
  return /** @type {!proto.speech.tts.lucid.Duration.StyleCase} */(jspb.Message.computeOneofCase(this, proto.speech.tts.lucid.Duration.oneofGroups_[0]));
};



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.lucid.Duration.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.lucid.Duration.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.lucid.Duration} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.Duration.toObject = function(includeInstance, msg) {
  var f, obj = {
    hours: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    minutes: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    seconds: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,
    milliseconds: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f,
    commonStyle: (f = jspb.Message.getField(msg, 5)) == null ? undefined : f,
    styleId: (f = jspb.Message.getField(msg, 6)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Duration as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.lucid.Duration.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.hours;

  /** @type {?number|undefined} */
  this.minutes;

  /** @type {?number|undefined} */
  this.seconds;

  /** @type {?number|undefined} */
  this.milliseconds;

  /** @type {?number|undefined} */
  this.commonStyle;

  /** @type {?number|undefined} */
  this.styleId;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.lucid.Duration.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.lucid.Duration}
 */
proto.speech.tts.lucid.Duration.fromObject = function(obj) {
  var msg = new proto.speech.tts.lucid.Duration();
  obj.hours != null && jspb.Message.setField(msg, 1, obj.hours);
  obj.minutes != null && jspb.Message.setField(msg, 2, obj.minutes);
  obj.seconds != null && jspb.Message.setField(msg, 3, obj.seconds);
  obj.milliseconds != null && jspb.Message.setField(msg, 4, obj.milliseconds);
  obj.commonStyle != null && jspb.Message.setField(msg, 5, obj.commonStyle);
  obj.styleId != null && jspb.Message.setField(msg, 6, obj.styleId);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.lucid.Duration}
 */
proto.speech.tts.lucid.Duration.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.lucid.Duration;
  return proto.speech.tts.lucid.Duration.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.lucid.Duration} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.lucid.Duration}
 */
proto.speech.tts.lucid.Duration.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setHours(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setMinutes(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setSeconds(value);
      break;
    case 4:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setMilliseconds(value);
      break;
    case 5:
      var value = /** @type {!proto.speech.tts.lucid.CommonVerbalizationStyles.Style} */ (reader.readEnum());
      msg.setCommonStyle(value);
      break;
    case 6:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setStyleId(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.lucid.Duration.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.lucid.Duration.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.lucid.Duration} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.Duration.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?number} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeInt32(
      1,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeInt32(
      2,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeInt32(
      3,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeInt32(
      4,
      f
    );
  }
  f = /** @type {!proto.speech.tts.lucid.CommonVerbalizationStyles.Style} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeEnum(
      5,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeInt32(
      6,
      f
    );
  }
};


/**
 * optional int32 hours = 1;
 * @return {?number}
 */
proto.speech.tts.lucid.Duration.prototype.getHours = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.lucid.Duration} returns this
 */
proto.speech.tts.lucid.Duration.prototype.setHours = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Duration} returns this
 */
proto.speech.tts.lucid.Duration.prototype.clearHours = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Duration.prototype.hasHours = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional int32 minutes = 2;
 * @return {?number}
 */
proto.speech.tts.lucid.Duration.prototype.getMinutes = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.lucid.Duration} returns this
 */
proto.speech.tts.lucid.Duration.prototype.setMinutes = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Duration} returns this
 */
proto.speech.tts.lucid.Duration.prototype.clearMinutes = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Duration.prototype.hasMinutes = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional int32 seconds = 3;
 * @return {?number}
 */
proto.speech.tts.lucid.Duration.prototype.getSeconds = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.lucid.Duration} returns this
 */
proto.speech.tts.lucid.Duration.prototype.setSeconds = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Duration} returns this
 */
proto.speech.tts.lucid.Duration.prototype.clearSeconds = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Duration.prototype.hasSeconds = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional int32 milliseconds = 4;
 * @return {?number}
 */
proto.speech.tts.lucid.Duration.prototype.getMilliseconds = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 4));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.lucid.Duration} returns this
 */
proto.speech.tts.lucid.Duration.prototype.setMilliseconds = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Duration} returns this
 */
proto.speech.tts.lucid.Duration.prototype.clearMilliseconds = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Duration.prototype.hasMilliseconds = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional CommonVerbalizationStyles.Style common_style = 5;
 * @return {!proto.speech.tts.lucid.CommonVerbalizationStyles.Style}
 */
proto.speech.tts.lucid.Duration.prototype.getCommonStyle = function() {
  return /** @type {!proto.speech.tts.lucid.CommonVerbalizationStyles.Style} */ (jspb.Message.getField(this, 5));
};


/**
 * @param {?proto.speech.tts.lucid.CommonVerbalizationStyles.Style|undefined} value
 * @return {!proto.speech.tts.lucid.Duration} returns this
 */
proto.speech.tts.lucid.Duration.prototype.setCommonStyle = function(value) {
  return jspb.Message.setOneofField(this, 5, proto.speech.tts.lucid.Duration.oneofGroups_[0], value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Duration} returns this
 */
proto.speech.tts.lucid.Duration.prototype.clearCommonStyle = function() {
  return jspb.Message.setOneofField(this, 5, proto.speech.tts.lucid.Duration.oneofGroups_[0], undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Duration.prototype.hasCommonStyle = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional int32 style_id = 6;
 * @return {?number}
 */
proto.speech.tts.lucid.Duration.prototype.getStyleId = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 6));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.lucid.Duration} returns this
 */
proto.speech.tts.lucid.Duration.prototype.setStyleId = function(value) {
  return jspb.Message.setOneofField(this, 6, proto.speech.tts.lucid.Duration.oneofGroups_[0], value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Duration} returns this
 */
proto.speech.tts.lucid.Duration.prototype.clearStyleId = function() {
  return jspb.Message.setOneofField(this, 6, proto.speech.tts.lucid.Duration.oneofGroups_[0], undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Duration.prototype.hasStyleId = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.lucid.Duration}
 */
proto.speech.tts.lucid.Duration.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.lucid.Duration, data);
};



//speech/tts/public/lucid/structured_token_proto_jspb/electronic.jspb.js
// source: speech/tts/public/lucid/structured_token.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.lucid.Electronic');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.lucid.Electronic = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.lucid.Electronic, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.lucid.Electronic.displayName = 'proto.speech.tts.lucid.Electronic';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.lucid.Electronic.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.lucid.Electronic.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.lucid.Electronic} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.Electronic.toObject = function(includeInstance, msg) {
  var f, obj = {
    protocol: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    username: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    password: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,
    domain: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f,
    port: (f = jspb.Message.getField(msg, 5)) == null ? undefined : f,
    path: (f = jspb.Message.getField(msg, 6)) == null ? undefined : f,
    queryString: (f = jspb.Message.getField(msg, 7)) == null ? undefined : f,
    fragmentId: (f = jspb.Message.getField(msg, 8)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Electronic as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.lucid.Electronic.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.protocol;

  /** @type {?string|undefined} */
  this.username;

  /** @type {?string|undefined} */
  this.password;

  /** @type {?string|undefined} */
  this.domain;

  /** @type {?number|undefined} */
  this.port;

  /** @type {?string|undefined} */
  this.path;

  /** @type {?string|undefined} */
  this.queryString;

  /** @type {?string|undefined} */
  this.fragmentId;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.lucid.Electronic.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.lucid.Electronic}
 */
proto.speech.tts.lucid.Electronic.fromObject = function(obj) {
  var msg = new proto.speech.tts.lucid.Electronic();
  obj.protocol != null && jspb.Message.setField(msg, 1, obj.protocol);
  obj.username != null && jspb.Message.setField(msg, 2, obj.username);
  obj.password != null && jspb.Message.setField(msg, 3, obj.password);
  obj.domain != null && jspb.Message.setField(msg, 4, obj.domain);
  obj.port != null && jspb.Message.setField(msg, 5, obj.port);
  obj.path != null && jspb.Message.setField(msg, 6, obj.path);
  obj.queryString != null && jspb.Message.setField(msg, 7, obj.queryString);
  obj.fragmentId != null && jspb.Message.setField(msg, 8, obj.fragmentId);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.lucid.Electronic}
 */
proto.speech.tts.lucid.Electronic.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.lucid.Electronic;
  return proto.speech.tts.lucid.Electronic.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.lucid.Electronic} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.lucid.Electronic}
 */
proto.speech.tts.lucid.Electronic.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setProtocol(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setUsername(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setPassword(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setDomain(value);
      break;
    case 5:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setPort(value);
      break;
    case 6:
      var value = /** @type {string} */ (reader.readString());
      msg.setPath(value);
      break;
    case 7:
      var value = /** @type {string} */ (reader.readString());
      msg.setQueryString(value);
      break;
    case 8:
      var value = /** @type {string} */ (reader.readString());
      msg.setFragmentId(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.lucid.Electronic.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.lucid.Electronic.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.lucid.Electronic} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.Electronic.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeString(
      3,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeString(
      4,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeInt32(
      5,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeString(
      6,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 7));
  if (f != null) {
    writer.writeString(
      7,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 8));
  if (f != null) {
    writer.writeString(
      8,
      f
    );
  }
};


/**
 * optional string protocol = 1;
 * @return {?string}
 */
proto.speech.tts.lucid.Electronic.prototype.getProtocol = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.lucid.Electronic} returns this
 */
proto.speech.tts.lucid.Electronic.prototype.setProtocol = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Electronic} returns this
 */
proto.speech.tts.lucid.Electronic.prototype.clearProtocol = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Electronic.prototype.hasProtocol = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional string username = 2;
 * @return {?string}
 */
proto.speech.tts.lucid.Electronic.prototype.getUsername = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.lucid.Electronic} returns this
 */
proto.speech.tts.lucid.Electronic.prototype.setUsername = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Electronic} returns this
 */
proto.speech.tts.lucid.Electronic.prototype.clearUsername = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Electronic.prototype.hasUsername = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional string password = 3;
 * @return {?string}
 */
proto.speech.tts.lucid.Electronic.prototype.getPassword = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.lucid.Electronic} returns this
 */
proto.speech.tts.lucid.Electronic.prototype.setPassword = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Electronic} returns this
 */
proto.speech.tts.lucid.Electronic.prototype.clearPassword = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Electronic.prototype.hasPassword = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional string domain = 4;
 * @return {?string}
 */
proto.speech.tts.lucid.Electronic.prototype.getDomain = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 4));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.lucid.Electronic} returns this
 */
proto.speech.tts.lucid.Electronic.prototype.setDomain = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Electronic} returns this
 */
proto.speech.tts.lucid.Electronic.prototype.clearDomain = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Electronic.prototype.hasDomain = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional int32 port = 5;
 * @return {?number}
 */
proto.speech.tts.lucid.Electronic.prototype.getPort = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 5));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.lucid.Electronic} returns this
 */
proto.speech.tts.lucid.Electronic.prototype.setPort = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Electronic} returns this
 */
proto.speech.tts.lucid.Electronic.prototype.clearPort = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Electronic.prototype.hasPort = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional string path = 6;
 * @return {?string}
 */
proto.speech.tts.lucid.Electronic.prototype.getPath = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 6));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.lucid.Electronic} returns this
 */
proto.speech.tts.lucid.Electronic.prototype.setPath = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Electronic} returns this
 */
proto.speech.tts.lucid.Electronic.prototype.clearPath = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Electronic.prototype.hasPath = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional string query_string = 7;
 * @return {?string}
 */
proto.speech.tts.lucid.Electronic.prototype.getQueryString = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 7));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.lucid.Electronic} returns this
 */
proto.speech.tts.lucid.Electronic.prototype.setQueryString = function(value) {
  return jspb.Message.setField(this, 7, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Electronic} returns this
 */
proto.speech.tts.lucid.Electronic.prototype.clearQueryString = function() {
  return jspb.Message.setField(this, 7, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Electronic.prototype.hasQueryString = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * optional string fragment_id = 8;
 * @return {?string}
 */
proto.speech.tts.lucid.Electronic.prototype.getFragmentId = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 8));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.lucid.Electronic} returns this
 */
proto.speech.tts.lucid.Electronic.prototype.setFragmentId = function(value) {
  return jspb.Message.setField(this, 8, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Electronic} returns this
 */
proto.speech.tts.lucid.Electronic.prototype.clearFragmentId = function() {
  return jspb.Message.setField(this, 8, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Electronic.prototype.hasFragmentId = function() {
  return jspb.Message.getField(this, 8) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.lucid.Electronic}
 */
proto.speech.tts.lucid.Electronic.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.lucid.Electronic, data);
};



//speech/tts/public/lucid/structured_token_proto_jspb/emoticonemoji.jspb.js
// source: speech/tts/public/lucid/structured_token.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.lucid.EmoticonEmoji');
goog.provide('proto.speech.tts.lucid.EmoticonEmoji.Style');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.lucid.EmoticonEmoji = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.lucid.EmoticonEmoji, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.lucid.EmoticonEmoji.displayName = 'proto.speech.tts.lucid.EmoticonEmoji';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.lucid.EmoticonEmoji.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.lucid.EmoticonEmoji.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.lucid.EmoticonEmoji} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.EmoticonEmoji.toObject = function(includeInstance, msg) {
  var f, obj = {
    text: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    style: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of EmoticonEmoji as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.lucid.EmoticonEmoji.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.text;

  /** @type {?number|undefined} */
  this.style;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.lucid.EmoticonEmoji.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.lucid.EmoticonEmoji}
 */
proto.speech.tts.lucid.EmoticonEmoji.fromObject = function(obj) {
  var msg = new proto.speech.tts.lucid.EmoticonEmoji();
  obj.text != null && jspb.Message.setField(msg, 1, obj.text);
  obj.style != null && jspb.Message.setField(msg, 2, obj.style);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.lucid.EmoticonEmoji}
 */
proto.speech.tts.lucid.EmoticonEmoji.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.lucid.EmoticonEmoji;
  return proto.speech.tts.lucid.EmoticonEmoji.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.lucid.EmoticonEmoji} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.lucid.EmoticonEmoji}
 */
proto.speech.tts.lucid.EmoticonEmoji.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setText(value);
      break;
    case 2:
      var value = /** @type {!proto.speech.tts.lucid.EmoticonEmoji.Style} */ (reader.readEnum());
      msg.setStyle(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.lucid.EmoticonEmoji.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.lucid.EmoticonEmoji.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.lucid.EmoticonEmoji} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.EmoticonEmoji.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = /** @type {!proto.speech.tts.lucid.EmoticonEmoji.Style} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeEnum(
      2,
      f
    );
  }
};


/**
 * @enum {number}
 */
proto.speech.tts.lucid.EmoticonEmoji.Style = {
  UNSPECIFIED_STYLE: 0,
  CONCISE_STYLE: 1,
  VERBOSE_STYLE: 2
};

/**
 * optional string text = 1;
 * @return {?string}
 */
proto.speech.tts.lucid.EmoticonEmoji.prototype.getText = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.lucid.EmoticonEmoji} returns this
 */
proto.speech.tts.lucid.EmoticonEmoji.prototype.setText = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.EmoticonEmoji} returns this
 */
proto.speech.tts.lucid.EmoticonEmoji.prototype.clearText = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.EmoticonEmoji.prototype.hasText = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional Style style = 2;
 * @return {!proto.speech.tts.lucid.EmoticonEmoji.Style}
 */
proto.speech.tts.lucid.EmoticonEmoji.prototype.getStyle = function() {
  return /** @type {!proto.speech.tts.lucid.EmoticonEmoji.Style} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?proto.speech.tts.lucid.EmoticonEmoji.Style|undefined} value
 * @return {!proto.speech.tts.lucid.EmoticonEmoji} returns this
 */
proto.speech.tts.lucid.EmoticonEmoji.prototype.setStyle = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.EmoticonEmoji} returns this
 */
proto.speech.tts.lucid.EmoticonEmoji.prototype.clearStyle = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.EmoticonEmoji.prototype.hasStyle = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.lucid.EmoticonEmoji}
 */
proto.speech.tts.lucid.EmoticonEmoji.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.lucid.EmoticonEmoji, data);
};



//speech/tts/public/lucid/structured_token_proto_jspb/fraction.jspb.js
// source: speech/tts/public/lucid/structured_token.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.lucid.Fraction');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.lucid.Fraction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.lucid.Fraction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.lucid.Fraction.displayName = 'proto.speech.tts.lucid.Fraction';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.lucid.Fraction.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.lucid.Fraction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.lucid.Fraction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.Fraction.toObject = function(includeInstance, msg) {
  var f, obj = {
    integerPart: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    numerator: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    denominator: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Fraction as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.lucid.Fraction.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.integerPart;

  /** @type {?string|undefined} */
  this.numerator;

  /** @type {?string|undefined} */
  this.denominator;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.lucid.Fraction.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.lucid.Fraction}
 */
proto.speech.tts.lucid.Fraction.fromObject = function(obj) {
  var msg = new proto.speech.tts.lucid.Fraction();
  obj.integerPart != null && jspb.Message.setField(msg, 1, obj.integerPart);
  obj.numerator != null && jspb.Message.setField(msg, 2, obj.numerator);
  obj.denominator != null && jspb.Message.setField(msg, 3, obj.denominator);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.lucid.Fraction}
 */
proto.speech.tts.lucid.Fraction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.lucid.Fraction;
  return proto.speech.tts.lucid.Fraction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.lucid.Fraction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.lucid.Fraction}
 */
proto.speech.tts.lucid.Fraction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setIntegerPart(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setNumerator(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setDenominator(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.lucid.Fraction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.lucid.Fraction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.lucid.Fraction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.Fraction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeString(
      3,
      f
    );
  }
};


/**
 * optional string integer_part = 1;
 * @return {?string}
 */
proto.speech.tts.lucid.Fraction.prototype.getIntegerPart = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.lucid.Fraction} returns this
 */
proto.speech.tts.lucid.Fraction.prototype.setIntegerPart = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Fraction} returns this
 */
proto.speech.tts.lucid.Fraction.prototype.clearIntegerPart = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Fraction.prototype.hasIntegerPart = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional string numerator = 2;
 * @return {?string}
 */
proto.speech.tts.lucid.Fraction.prototype.getNumerator = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.lucid.Fraction} returns this
 */
proto.speech.tts.lucid.Fraction.prototype.setNumerator = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Fraction} returns this
 */
proto.speech.tts.lucid.Fraction.prototype.clearNumerator = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Fraction.prototype.hasNumerator = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional string denominator = 3;
 * @return {?string}
 */
proto.speech.tts.lucid.Fraction.prototype.getDenominator = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.lucid.Fraction} returns this
 */
proto.speech.tts.lucid.Fraction.prototype.setDenominator = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Fraction} returns this
 */
proto.speech.tts.lucid.Fraction.prototype.clearDenominator = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Fraction.prototype.hasDenominator = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.lucid.Fraction}
 */
proto.speech.tts.lucid.Fraction.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.lucid.Fraction, data);
};



//speech/tts/public/lucid/structured_token_proto_jspb/lettersequence.jspb.js
// source: speech/tts/public/lucid/structured_token.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.lucid.LetterSequence');
goog.provide('proto.speech.tts.lucid.LetterSequence.Style');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.lucid.LetterSequence = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.lucid.LetterSequence, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.lucid.LetterSequence.displayName = 'proto.speech.tts.lucid.LetterSequence';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.lucid.LetterSequence.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.lucid.LetterSequence.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.lucid.LetterSequence} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.LetterSequence.toObject = function(includeInstance, msg) {
  var f, obj = {
    text: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    style: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of LetterSequence as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.lucid.LetterSequence.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.text;

  /** @type {?number|undefined} */
  this.style;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.lucid.LetterSequence.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.lucid.LetterSequence}
 */
proto.speech.tts.lucid.LetterSequence.fromObject = function(obj) {
  var msg = new proto.speech.tts.lucid.LetterSequence();
  obj.text != null && jspb.Message.setField(msg, 1, obj.text);
  obj.style != null && jspb.Message.setField(msg, 2, obj.style);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.lucid.LetterSequence}
 */
proto.speech.tts.lucid.LetterSequence.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.lucid.LetterSequence;
  return proto.speech.tts.lucid.LetterSequence.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.lucid.LetterSequence} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.lucid.LetterSequence}
 */
proto.speech.tts.lucid.LetterSequence.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setText(value);
      break;
    case 2:
      var value = /** @type {!proto.speech.tts.lucid.LetterSequence.Style} */ (reader.readEnum());
      msg.setStyle(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.lucid.LetterSequence.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.lucid.LetterSequence.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.lucid.LetterSequence} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.LetterSequence.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = /** @type {!proto.speech.tts.lucid.LetterSequence.Style} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeEnum(
      2,
      f
    );
  }
};


/**
 * @enum {number}
 */
proto.speech.tts.lucid.LetterSequence.Style = {
  UNSPECIFIED_STYLE: 0,
  SEPARATE_LETTERS: 1,
  AS_WORD: 2
};

/**
 * optional string text = 1;
 * @return {?string}
 */
proto.speech.tts.lucid.LetterSequence.prototype.getText = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.lucid.LetterSequence} returns this
 */
proto.speech.tts.lucid.LetterSequence.prototype.setText = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.LetterSequence} returns this
 */
proto.speech.tts.lucid.LetterSequence.prototype.clearText = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.LetterSequence.prototype.hasText = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional Style style = 2;
 * @return {!proto.speech.tts.lucid.LetterSequence.Style}
 */
proto.speech.tts.lucid.LetterSequence.prototype.getStyle = function() {
  return /** @type {!proto.speech.tts.lucid.LetterSequence.Style} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?proto.speech.tts.lucid.LetterSequence.Style|undefined} value
 * @return {!proto.speech.tts.lucid.LetterSequence} returns this
 */
proto.speech.tts.lucid.LetterSequence.prototype.setStyle = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.LetterSequence} returns this
 */
proto.speech.tts.lucid.LetterSequence.prototype.clearStyle = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.LetterSequence.prototype.hasStyle = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.lucid.LetterSequence}
 */
proto.speech.tts.lucid.LetterSequence.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.lucid.LetterSequence, data);
};



//speech/tts/public/lucid/structured_token_proto_jspb/number.jspb.js
// source: speech/tts/public/lucid/structured_token.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.lucid.Number');
goog.provide('proto.speech.tts.lucid.Number.ValueCase');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.lucid.Number = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, proto.speech.tts.lucid.Number.oneofGroups_);
};
goog.inherits(proto.speech.tts.lucid.Number, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.lucid.Number.displayName = 'proto.speech.tts.lucid.Number';
}

/**
 * Oneof group definitions for this message. Each group defines the field
 * numbers belonging to that group. When of these fields' value is set, all
 * other fields in the group are cleared. During deserialization, if multiple
 * fields are encountered for a group, only the last value seen will be kept.
 * @private {!Array<!Array<number>>}
 * @const
 */
proto.speech.tts.lucid.Number.oneofGroups_ = [[3,4]];

/**
 * @enum {number}
 */
proto.speech.tts.lucid.Number.ValueCase = {
  VALUE_NOT_SET: 0,
  INTEGER: 3,
  FLOATING_POINT: 4
};

/**
 * @return {!proto.speech.tts.lucid.Number.ValueCase}
 */
proto.speech.tts.lucid.Number.prototype.getValueCase = function() {
  return /** @type {!proto.speech.tts.lucid.Number.ValueCase} */(jspb.Message.computeOneofCase(this, proto.speech.tts.lucid.Number.oneofGroups_[0]));
};



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.lucid.Number.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.lucid.Number.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.lucid.Number} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.Number.toObject = function(includeInstance, msg) {
  var f, obj = {
    integerPart: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    fractionalPart: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    integer: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,
    floatingPoint: (f = jspb.Message.getOptionalFloatingPointField(msg, 4)) == null ? undefined : f,
    minPrecision: (f = jspb.Message.getField(msg, 5)) == null ? undefined : f,
    maxPrecision: jspb.Message.getFieldWithDefault(msg, 6, 6),
    exponent: (f = jspb.Message.getField(msg, 7)) == null ? undefined : f,
    quantity: (f = jspb.Message.getField(msg, 8)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Number as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.lucid.Number.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.integerPart;

  /** @type {?string|undefined} */
  this.fractionalPart;

  /** @type {?number|undefined} */
  this.integer;

  /** @type {?number|undefined} */
  this.floatingPoint;

  /** @type {?number|undefined} */
  this.minPrecision;

  /** @type {?number|undefined} */
  this.maxPrecision;

  /** @type {?string|undefined} */
  this.exponent;

  /** @type {?string|undefined} */
  this.quantity;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.lucid.Number.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.lucid.Number}
 */
proto.speech.tts.lucid.Number.fromObject = function(obj) {
  var msg = new proto.speech.tts.lucid.Number();
  obj.integerPart != null && jspb.Message.setField(msg, 1, obj.integerPart);
  obj.fractionalPart != null && jspb.Message.setField(msg, 2, obj.fractionalPart);
  obj.integer != null && jspb.Message.setField(msg, 3, obj.integer);
  obj.floatingPoint != null && jspb.Message.setField(msg, 4, obj.floatingPoint);
  obj.minPrecision != null && jspb.Message.setField(msg, 5, obj.minPrecision);
  obj.maxPrecision != null && jspb.Message.setField(msg, 6, obj.maxPrecision);
  obj.exponent != null && jspb.Message.setField(msg, 7, obj.exponent);
  obj.quantity != null && jspb.Message.setField(msg, 8, obj.quantity);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.lucid.Number}
 */
proto.speech.tts.lucid.Number.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.lucid.Number;
  return proto.speech.tts.lucid.Number.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.lucid.Number} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.lucid.Number}
 */
proto.speech.tts.lucid.Number.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setIntegerPart(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setFractionalPart(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setInteger(value);
      break;
    case 4:
      var value = /** @type {number} */ (reader.readDouble());
      msg.setFloatingPoint(value);
      break;
    case 5:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setMinPrecision(value);
      break;
    case 6:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setMaxPrecision(value);
      break;
    case 7:
      var value = /** @type {string} */ (reader.readString());
      msg.setExponent(value);
      break;
    case 8:
      var value = /** @type {string} */ (reader.readString());
      msg.setQuantity(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.lucid.Number.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.lucid.Number.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.lucid.Number} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.Number.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeInt64(
      3,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeDouble(
      4,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeInt32(
      5,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeInt32(
      6,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 7));
  if (f != null) {
    writer.writeString(
      7,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 8));
  if (f != null) {
    writer.writeString(
      8,
      f
    );
  }
};


/**
 * optional string integer_part = 1;
 * @return {?string}
 */
proto.speech.tts.lucid.Number.prototype.getIntegerPart = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.lucid.Number} returns this
 */
proto.speech.tts.lucid.Number.prototype.setIntegerPart = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Number} returns this
 */
proto.speech.tts.lucid.Number.prototype.clearIntegerPart = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Number.prototype.hasIntegerPart = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional string fractional_part = 2;
 * @return {?string}
 */
proto.speech.tts.lucid.Number.prototype.getFractionalPart = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.lucid.Number} returns this
 */
proto.speech.tts.lucid.Number.prototype.setFractionalPart = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Number} returns this
 */
proto.speech.tts.lucid.Number.prototype.clearFractionalPart = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Number.prototype.hasFractionalPart = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional int64 integer = 3;
 * @return {?number}
 */
proto.speech.tts.lucid.Number.prototype.getInteger = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.lucid.Number} returns this
 */
proto.speech.tts.lucid.Number.prototype.setInteger = function(value) {
  return jspb.Message.setOneofField(this, 3, proto.speech.tts.lucid.Number.oneofGroups_[0], value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Number} returns this
 */
proto.speech.tts.lucid.Number.prototype.clearInteger = function() {
  return jspb.Message.setOneofField(this, 3, proto.speech.tts.lucid.Number.oneofGroups_[0], undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Number.prototype.hasInteger = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional double floating_point = 4;
 * @return {?number}
 */
proto.speech.tts.lucid.Number.prototype.getFloatingPoint = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 4));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.lucid.Number} returns this
 */
proto.speech.tts.lucid.Number.prototype.setFloatingPoint = function(value) {
  return jspb.Message.setOneofField(this, 4, proto.speech.tts.lucid.Number.oneofGroups_[0], value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Number} returns this
 */
proto.speech.tts.lucid.Number.prototype.clearFloatingPoint = function() {
  return jspb.Message.setOneofField(this, 4, proto.speech.tts.lucid.Number.oneofGroups_[0], undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Number.prototype.hasFloatingPoint = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional int32 min_precision = 5;
 * @return {?number}
 */
proto.speech.tts.lucid.Number.prototype.getMinPrecision = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 5));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.lucid.Number} returns this
 */
proto.speech.tts.lucid.Number.prototype.setMinPrecision = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Number} returns this
 */
proto.speech.tts.lucid.Number.prototype.clearMinPrecision = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Number.prototype.hasMinPrecision = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional int32 max_precision = 6;
 * @return {number}
 */
proto.speech.tts.lucid.Number.prototype.getMaxPrecision = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 6, 6));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.lucid.Number} returns this
 */
proto.speech.tts.lucid.Number.prototype.setMaxPrecision = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Number} returns this
 */
proto.speech.tts.lucid.Number.prototype.clearMaxPrecision = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Number.prototype.hasMaxPrecision = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional string exponent = 7;
 * @return {?string}
 */
proto.speech.tts.lucid.Number.prototype.getExponent = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 7));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.lucid.Number} returns this
 */
proto.speech.tts.lucid.Number.prototype.setExponent = function(value) {
  return jspb.Message.setField(this, 7, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Number} returns this
 */
proto.speech.tts.lucid.Number.prototype.clearExponent = function() {
  return jspb.Message.setField(this, 7, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Number.prototype.hasExponent = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * optional string quantity = 8;
 * @return {?string}
 */
proto.speech.tts.lucid.Number.prototype.getQuantity = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 8));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.lucid.Number} returns this
 */
proto.speech.tts.lucid.Number.prototype.setQuantity = function(value) {
  return jspb.Message.setField(this, 8, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Number} returns this
 */
proto.speech.tts.lucid.Number.prototype.clearQuantity = function() {
  return jspb.Message.setField(this, 8, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Number.prototype.hasQuantity = function() {
  return jspb.Message.getField(this, 8) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.lucid.Number}
 */
proto.speech.tts.lucid.Number.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.lucid.Number, data);
};



//speech/tts/public/lucid/structured_token_proto_jspb/measure.jspb.js
// source: speech/tts/public/lucid/structured_token.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.lucid.Measure');
goog.provide('proto.speech.tts.lucid.Measure.Style');
goog.provide('proto.speech.tts.lucid.Measure.StyleCase');
goog.provide('proto.speech.tts.lucid.Measure.ValueCase');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.lucid.Fraction');
goog.require('proto.speech.tts.lucid.Number');

goog.forwardDeclare('proto.speech.tts.lucid.CommonVerbalizationStyles.Style');
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.lucid.Measure = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, proto.speech.tts.lucid.Measure.oneofGroups_);
};
goog.inherits(proto.speech.tts.lucid.Measure, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.lucid.Measure.displayName = 'proto.speech.tts.lucid.Measure';
}

/**
 * Oneof group definitions for this message. Each group defines the field
 * numbers belonging to that group. When of these fields' value is set, all
 * other fields in the group are cleared. During deserialization, if multiple
 * fields are encountered for a group, only the last value seen will be kept.
 * @private {!Array<!Array<number>>}
 * @const
 */
proto.speech.tts.lucid.Measure.oneofGroups_ = [[1,2],[4,5,6]];

/**
 * @enum {number}
 */
proto.speech.tts.lucid.Measure.ValueCase = {
  VALUE_NOT_SET: 0,
  DECIMAL: 1,
  FRACTION: 2
};

/**
 * @return {!proto.speech.tts.lucid.Measure.ValueCase}
 */
proto.speech.tts.lucid.Measure.prototype.getValueCase = function() {
  return /** @type {!proto.speech.tts.lucid.Measure.ValueCase} */(jspb.Message.computeOneofCase(this, proto.speech.tts.lucid.Measure.oneofGroups_[0]));
};

/**
 * @enum {number}
 */
proto.speech.tts.lucid.Measure.StyleCase = {
  STYLE_NOT_SET: 0,
  COMMON_STYLE: 4,
  SPECIFIC_STYLE: 5,
  STYLE_ID: 6
};

/**
 * @return {!proto.speech.tts.lucid.Measure.StyleCase}
 */
proto.speech.tts.lucid.Measure.prototype.getStyleCase = function() {
  return /** @type {!proto.speech.tts.lucid.Measure.StyleCase} */(jspb.Message.computeOneofCase(this, proto.speech.tts.lucid.Measure.oneofGroups_[1]));
};



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.lucid.Measure.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.lucid.Measure.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.lucid.Measure} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.Measure.toObject = function(includeInstance, msg) {
  var f, obj = {
    decimal: (f = msg.getDecimal()) && proto.speech.tts.lucid.Number.toObject(includeInstance, f),
    fraction: (f = msg.getFraction()) && proto.speech.tts.lucid.Fraction.toObject(includeInstance, f),
    unit: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,
    commonStyle: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f,
    specificStyle: (f = jspb.Message.getField(msg, 5)) == null ? undefined : f,
    styleId: (f = jspb.Message.getField(msg, 6)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Measure as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.lucid.Measure.ObjectFormat = function() {
  /** @type {?proto.speech.tts.lucid.Number.ObjectFormat|undefined} */
  this.decimal;

  /** @type {?proto.speech.tts.lucid.Fraction.ObjectFormat|undefined} */
  this.fraction;

  /** @type {?string|undefined} */
  this.unit;

  /** @type {?number|undefined} */
  this.commonStyle;

  /** @type {?number|undefined} */
  this.specificStyle;

  /** @type {?number|undefined} */
  this.styleId;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.lucid.Measure.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.lucid.Measure}
 */
proto.speech.tts.lucid.Measure.fromObject = function(obj) {
  var msg = new proto.speech.tts.lucid.Measure();
  obj.decimal && jspb.Message.setWrapperField(
      msg, 1, proto.speech.tts.lucid.Number.fromObject(obj.decimal));
  obj.fraction && jspb.Message.setWrapperField(
      msg, 2, proto.speech.tts.lucid.Fraction.fromObject(obj.fraction));
  obj.unit != null && jspb.Message.setField(msg, 3, obj.unit);
  obj.commonStyle != null && jspb.Message.setField(msg, 4, obj.commonStyle);
  obj.specificStyle != null && jspb.Message.setField(msg, 5, obj.specificStyle);
  obj.styleId != null && jspb.Message.setField(msg, 6, obj.styleId);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.lucid.Measure}
 */
proto.speech.tts.lucid.Measure.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.lucid.Measure;
  return proto.speech.tts.lucid.Measure.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.lucid.Measure} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.lucid.Measure}
 */
proto.speech.tts.lucid.Measure.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.speech.tts.lucid.Number;
      reader.readMessage(value,proto.speech.tts.lucid.Number.deserializeBinaryFromReader);
      msg.setDecimal(value);
      break;
    case 2:
      var value = new proto.speech.tts.lucid.Fraction;
      reader.readMessage(value,proto.speech.tts.lucid.Fraction.deserializeBinaryFromReader);
      msg.setFraction(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setUnit(value);
      break;
    case 4:
      var value = /** @type {!proto.speech.tts.lucid.CommonVerbalizationStyles.Style} */ (reader.readEnum());
      msg.setCommonStyle(value);
      break;
    case 5:
      var value = /** @type {!proto.speech.tts.lucid.Measure.Style} */ (reader.readEnum());
      msg.setSpecificStyle(value);
      break;
    case 6:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setStyleId(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.lucid.Measure.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.lucid.Measure.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.lucid.Measure} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.Measure.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getDecimal();
  if (f != null) {
    writer.writeMessage(
      1,
      f,
      proto.speech.tts.lucid.Number.serializeBinaryToWriter
    );
  }
  f = message.getFraction();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.speech.tts.lucid.Fraction.serializeBinaryToWriter
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeString(
      3,
      f
    );
  }
  f = /** @type {!proto.speech.tts.lucid.CommonVerbalizationStyles.Style} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeEnum(
      4,
      f
    );
  }
  f = /** @type {!proto.speech.tts.lucid.Measure.Style} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeEnum(
      5,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeInt32(
      6,
      f
    );
  }
};


/**
 * @enum {number}
 */
proto.speech.tts.lucid.Measure.Style = {
  UNSPECIFIED_STYLE: 0
};

/**
 * optional Number decimal = 1;
 * @return {?proto.speech.tts.lucid.Number}
 */
proto.speech.tts.lucid.Measure.prototype.getDecimal = function() {
  return /** @type{?proto.speech.tts.lucid.Number} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.lucid.Number, 1));
};


/**
 * @param {?proto.speech.tts.lucid.Number|undefined} value
 * @return {!proto.speech.tts.lucid.Measure} returns this
*/
proto.speech.tts.lucid.Measure.prototype.setDecimal = function(value) {
  return jspb.Message.setOneofWrapperField(this, 1, proto.speech.tts.lucid.Measure.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.lucid.Measure} returns this
 */
proto.speech.tts.lucid.Measure.prototype.clearDecimal = function() {
  return this.setDecimal(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Measure.prototype.hasDecimal = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional Fraction fraction = 2;
 * @return {?proto.speech.tts.lucid.Fraction}
 */
proto.speech.tts.lucid.Measure.prototype.getFraction = function() {
  return /** @type{?proto.speech.tts.lucid.Fraction} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.lucid.Fraction, 2));
};


/**
 * @param {?proto.speech.tts.lucid.Fraction|undefined} value
 * @return {!proto.speech.tts.lucid.Measure} returns this
*/
proto.speech.tts.lucid.Measure.prototype.setFraction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 2, proto.speech.tts.lucid.Measure.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.lucid.Measure} returns this
 */
proto.speech.tts.lucid.Measure.prototype.clearFraction = function() {
  return this.setFraction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Measure.prototype.hasFraction = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional string unit = 3;
 * @return {?string}
 */
proto.speech.tts.lucid.Measure.prototype.getUnit = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.lucid.Measure} returns this
 */
proto.speech.tts.lucid.Measure.prototype.setUnit = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Measure} returns this
 */
proto.speech.tts.lucid.Measure.prototype.clearUnit = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Measure.prototype.hasUnit = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional CommonVerbalizationStyles.Style common_style = 4;
 * @return {!proto.speech.tts.lucid.CommonVerbalizationStyles.Style}
 */
proto.speech.tts.lucid.Measure.prototype.getCommonStyle = function() {
  return /** @type {!proto.speech.tts.lucid.CommonVerbalizationStyles.Style} */ (jspb.Message.getField(this, 4));
};


/**
 * @param {?proto.speech.tts.lucid.CommonVerbalizationStyles.Style|undefined} value
 * @return {!proto.speech.tts.lucid.Measure} returns this
 */
proto.speech.tts.lucid.Measure.prototype.setCommonStyle = function(value) {
  return jspb.Message.setOneofField(this, 4, proto.speech.tts.lucid.Measure.oneofGroups_[1], value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Measure} returns this
 */
proto.speech.tts.lucid.Measure.prototype.clearCommonStyle = function() {
  return jspb.Message.setOneofField(this, 4, proto.speech.tts.lucid.Measure.oneofGroups_[1], undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Measure.prototype.hasCommonStyle = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional Style specific_style = 5;
 * @return {!proto.speech.tts.lucid.Measure.Style}
 */
proto.speech.tts.lucid.Measure.prototype.getSpecificStyle = function() {
  return /** @type {!proto.speech.tts.lucid.Measure.Style} */ (jspb.Message.getField(this, 5));
};


/**
 * @param {?proto.speech.tts.lucid.Measure.Style|undefined} value
 * @return {!proto.speech.tts.lucid.Measure} returns this
 */
proto.speech.tts.lucid.Measure.prototype.setSpecificStyle = function(value) {
  return jspb.Message.setOneofField(this, 5, proto.speech.tts.lucid.Measure.oneofGroups_[1], value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Measure} returns this
 */
proto.speech.tts.lucid.Measure.prototype.clearSpecificStyle = function() {
  return jspb.Message.setOneofField(this, 5, proto.speech.tts.lucid.Measure.oneofGroups_[1], undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Measure.prototype.hasSpecificStyle = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional int32 style_id = 6;
 * @return {?number}
 */
proto.speech.tts.lucid.Measure.prototype.getStyleId = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 6));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.lucid.Measure} returns this
 */
proto.speech.tts.lucid.Measure.prototype.setStyleId = function(value) {
  return jspb.Message.setOneofField(this, 6, proto.speech.tts.lucid.Measure.oneofGroups_[1], value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Measure} returns this
 */
proto.speech.tts.lucid.Measure.prototype.clearStyleId = function() {
  return jspb.Message.setOneofField(this, 6, proto.speech.tts.lucid.Measure.oneofGroups_[1], undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Measure.prototype.hasStyleId = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.lucid.Measure}
 */
proto.speech.tts.lucid.Measure.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.lucid.Measure, data);
};



//speech/tts/public/lucid/structured_token_proto_jspb/money.jspb.js
// source: speech/tts/public/lucid/structured_token.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.lucid.Money');
goog.provide('proto.speech.tts.lucid.Money.Style');
goog.provide('proto.speech.tts.lucid.Money.StyleCase');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.lucid.Number');

goog.forwardDeclare('proto.speech.tts.lucid.CommonVerbalizationStyles.Style');
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.lucid.Money = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, proto.speech.tts.lucid.Money.oneofGroups_);
};
goog.inherits(proto.speech.tts.lucid.Money, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.lucid.Money.displayName = 'proto.speech.tts.lucid.Money';
}

/**
 * Oneof group definitions for this message. Each group defines the field
 * numbers belonging to that group. When of these fields' value is set, all
 * other fields in the group are cleared. During deserialization, if multiple
 * fields are encountered for a group, only the last value seen will be kept.
 * @private {!Array<!Array<number>>}
 * @const
 */
proto.speech.tts.lucid.Money.oneofGroups_ = [[4,5,6]];

/**
 * @enum {number}
 */
proto.speech.tts.lucid.Money.StyleCase = {
  STYLE_NOT_SET: 0,
  COMMON_STYLE: 4,
  SPECIFIC_STYLE: 5,
  STYLE_ID: 6
};

/**
 * @return {!proto.speech.tts.lucid.Money.StyleCase}
 */
proto.speech.tts.lucid.Money.prototype.getStyleCase = function() {
  return /** @type {!proto.speech.tts.lucid.Money.StyleCase} */(jspb.Message.computeOneofCase(this, proto.speech.tts.lucid.Money.oneofGroups_[0]));
};



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.lucid.Money.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.lucid.Money.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.lucid.Money} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.Money.toObject = function(includeInstance, msg) {
  var f, obj = {
    amount: (f = msg.getAmount()) && proto.speech.tts.lucid.Number.toObject(includeInstance, f),
    currency: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    commonStyle: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f,
    specificStyle: (f = jspb.Message.getField(msg, 5)) == null ? undefined : f,
    styleId: (f = jspb.Message.getField(msg, 6)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Money as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.lucid.Money.ObjectFormat = function() {
  /** @type {?proto.speech.tts.lucid.Number.ObjectFormat|undefined} */
  this.amount;

  /** @type {?string|undefined} */
  this.currency;

  /** @type {?number|undefined} */
  this.commonStyle;

  /** @type {?number|undefined} */
  this.specificStyle;

  /** @type {?number|undefined} */
  this.styleId;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.lucid.Money.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.lucid.Money}
 */
proto.speech.tts.lucid.Money.fromObject = function(obj) {
  var msg = new proto.speech.tts.lucid.Money();
  obj.amount && jspb.Message.setWrapperField(
      msg, 1, proto.speech.tts.lucid.Number.fromObject(obj.amount));
  obj.currency != null && jspb.Message.setField(msg, 2, obj.currency);
  obj.commonStyle != null && jspb.Message.setField(msg, 4, obj.commonStyle);
  obj.specificStyle != null && jspb.Message.setField(msg, 5, obj.specificStyle);
  obj.styleId != null && jspb.Message.setField(msg, 6, obj.styleId);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.lucid.Money}
 */
proto.speech.tts.lucid.Money.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.lucid.Money;
  return proto.speech.tts.lucid.Money.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.lucid.Money} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.lucid.Money}
 */
proto.speech.tts.lucid.Money.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.speech.tts.lucid.Number;
      reader.readMessage(value,proto.speech.tts.lucid.Number.deserializeBinaryFromReader);
      msg.setAmount(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setCurrency(value);
      break;
    case 4:
      var value = /** @type {!proto.speech.tts.lucid.CommonVerbalizationStyles.Style} */ (reader.readEnum());
      msg.setCommonStyle(value);
      break;
    case 5:
      var value = /** @type {!proto.speech.tts.lucid.Money.Style} */ (reader.readEnum());
      msg.setSpecificStyle(value);
      break;
    case 6:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setStyleId(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.lucid.Money.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.lucid.Money.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.lucid.Money} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.Money.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getAmount();
  if (f != null) {
    writer.writeMessage(
      1,
      f,
      proto.speech.tts.lucid.Number.serializeBinaryToWriter
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
  f = /** @type {!proto.speech.tts.lucid.CommonVerbalizationStyles.Style} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeEnum(
      4,
      f
    );
  }
  f = /** @type {!proto.speech.tts.lucid.Money.Style} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeEnum(
      5,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeInt32(
      6,
      f
    );
  }
};


/**
 * @enum {number}
 */
proto.speech.tts.lucid.Money.Style = {
  UNSPECIFIED_STYLE: 0,
  SAY_FULL_UNAMBIGUOUS_CURRENCY: 1,
  SAY_MAJOR_AND_MINOR_CURRENCY: 2,
  OMIT_MINOR_CURRENCY: 3,
  OMIT_CURRENCY: 4
};

/**
 * optional Number amount = 1;
 * @return {?proto.speech.tts.lucid.Number}
 */
proto.speech.tts.lucid.Money.prototype.getAmount = function() {
  return /** @type{?proto.speech.tts.lucid.Number} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.lucid.Number, 1));
};


/**
 * @param {?proto.speech.tts.lucid.Number|undefined} value
 * @return {!proto.speech.tts.lucid.Money} returns this
*/
proto.speech.tts.lucid.Money.prototype.setAmount = function(value) {
  return jspb.Message.setWrapperField(this, 1, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.lucid.Money} returns this
 */
proto.speech.tts.lucid.Money.prototype.clearAmount = function() {
  return this.setAmount(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Money.prototype.hasAmount = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional string currency = 2;
 * @return {?string}
 */
proto.speech.tts.lucid.Money.prototype.getCurrency = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.lucid.Money} returns this
 */
proto.speech.tts.lucid.Money.prototype.setCurrency = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Money} returns this
 */
proto.speech.tts.lucid.Money.prototype.clearCurrency = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Money.prototype.hasCurrency = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional CommonVerbalizationStyles.Style common_style = 4;
 * @return {!proto.speech.tts.lucid.CommonVerbalizationStyles.Style}
 */
proto.speech.tts.lucid.Money.prototype.getCommonStyle = function() {
  return /** @type {!proto.speech.tts.lucid.CommonVerbalizationStyles.Style} */ (jspb.Message.getField(this, 4));
};


/**
 * @param {?proto.speech.tts.lucid.CommonVerbalizationStyles.Style|undefined} value
 * @return {!proto.speech.tts.lucid.Money} returns this
 */
proto.speech.tts.lucid.Money.prototype.setCommonStyle = function(value) {
  return jspb.Message.setOneofField(this, 4, proto.speech.tts.lucid.Money.oneofGroups_[0], value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Money} returns this
 */
proto.speech.tts.lucid.Money.prototype.clearCommonStyle = function() {
  return jspb.Message.setOneofField(this, 4, proto.speech.tts.lucid.Money.oneofGroups_[0], undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Money.prototype.hasCommonStyle = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional Style specific_style = 5;
 * @return {!proto.speech.tts.lucid.Money.Style}
 */
proto.speech.tts.lucid.Money.prototype.getSpecificStyle = function() {
  return /** @type {!proto.speech.tts.lucid.Money.Style} */ (jspb.Message.getField(this, 5));
};


/**
 * @param {?proto.speech.tts.lucid.Money.Style|undefined} value
 * @return {!proto.speech.tts.lucid.Money} returns this
 */
proto.speech.tts.lucid.Money.prototype.setSpecificStyle = function(value) {
  return jspb.Message.setOneofField(this, 5, proto.speech.tts.lucid.Money.oneofGroups_[0], value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Money} returns this
 */
proto.speech.tts.lucid.Money.prototype.clearSpecificStyle = function() {
  return jspb.Message.setOneofField(this, 5, proto.speech.tts.lucid.Money.oneofGroups_[0], undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Money.prototype.hasSpecificStyle = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional int32 style_id = 6;
 * @return {?number}
 */
proto.speech.tts.lucid.Money.prototype.getStyleId = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 6));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.lucid.Money} returns this
 */
proto.speech.tts.lucid.Money.prototype.setStyleId = function(value) {
  return jspb.Message.setOneofField(this, 6, proto.speech.tts.lucid.Money.oneofGroups_[0], value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Money} returns this
 */
proto.speech.tts.lucid.Money.prototype.clearStyleId = function() {
  return jspb.Message.setOneofField(this, 6, proto.speech.tts.lucid.Money.oneofGroups_[0], undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Money.prototype.hasStyleId = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.lucid.Money}
 */
proto.speech.tts.lucid.Money.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.lucid.Money, data);
};



//speech/tts/public/lucid/structured_token_proto_jspb/ordinal.jspb.js
// source: speech/tts/public/lucid/structured_token.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.lucid.Ordinal');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.lucid.Ordinal = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.lucid.Ordinal, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.lucid.Ordinal.displayName = 'proto.speech.tts.lucid.Ordinal';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.lucid.Ordinal.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.lucid.Ordinal.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.lucid.Ordinal} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.Ordinal.toObject = function(includeInstance, msg) {
  var f, obj = {
    integer: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Ordinal as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.lucid.Ordinal.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.integer;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.lucid.Ordinal.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.lucid.Ordinal}
 */
proto.speech.tts.lucid.Ordinal.fromObject = function(obj) {
  var msg = new proto.speech.tts.lucid.Ordinal();
  obj.integer != null && jspb.Message.setField(msg, 2, obj.integer);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.lucid.Ordinal}
 */
proto.speech.tts.lucid.Ordinal.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.lucid.Ordinal;
  return proto.speech.tts.lucid.Ordinal.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.lucid.Ordinal} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.lucid.Ordinal}
 */
proto.speech.tts.lucid.Ordinal.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setInteger(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.lucid.Ordinal.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.lucid.Ordinal.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.lucid.Ordinal} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.Ordinal.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string integer = 2;
 * @return {?string}
 */
proto.speech.tts.lucid.Ordinal.prototype.getInteger = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.lucid.Ordinal} returns this
 */
proto.speech.tts.lucid.Ordinal.prototype.setInteger = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Ordinal} returns this
 */
proto.speech.tts.lucid.Ordinal.prototype.clearInteger = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Ordinal.prototype.hasInteger = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.lucid.Ordinal}
 */
proto.speech.tts.lucid.Ordinal.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.lucid.Ordinal, data);
};



//speech/tts/public/lucid/structured_token_proto_jspb/punctuationorpause.jspb.js
// source: speech/tts/public/lucid/structured_token.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.lucid.PunctuationOrPause');
goog.provide('proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak');
goog.provide('proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak.BreakStrength');
goog.provide('proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak.BreakType');
goog.provide('proto.speech.tts.lucid.PunctuationOrPause.PunctuationStyle');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.lucid.PunctuationOrPause = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.lucid.PunctuationOrPause, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.lucid.PunctuationOrPause.displayName = 'proto.speech.tts.lucid.PunctuationOrPause';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak.displayName = 'proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.lucid.PunctuationOrPause.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.lucid.PunctuationOrPause.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.lucid.PunctuationOrPause} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.PunctuationOrPause.toObject = function(includeInstance, msg) {
  var f, obj = {
    text: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    punctuationStyle: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    phraseBreak: (f = msg.getPhraseBreak()) && proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of PunctuationOrPause as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.lucid.PunctuationOrPause.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.text;

  /** @type {?number|undefined} */
  this.punctuationStyle;

  /** @type {?proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak.ObjectFormat|undefined} */
  this.phraseBreak;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.lucid.PunctuationOrPause.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.lucid.PunctuationOrPause}
 */
proto.speech.tts.lucid.PunctuationOrPause.fromObject = function(obj) {
  var msg = new proto.speech.tts.lucid.PunctuationOrPause();
  obj.text != null && jspb.Message.setField(msg, 1, obj.text);
  obj.punctuationStyle != null && jspb.Message.setField(msg, 2, obj.punctuationStyle);
  obj.phraseBreak && jspb.Message.setWrapperField(
      msg, 3, proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak.fromObject(obj.phraseBreak));
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.lucid.PunctuationOrPause}
 */
proto.speech.tts.lucid.PunctuationOrPause.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.lucid.PunctuationOrPause;
  return proto.speech.tts.lucid.PunctuationOrPause.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.lucid.PunctuationOrPause} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.lucid.PunctuationOrPause}
 */
proto.speech.tts.lucid.PunctuationOrPause.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setText(value);
      break;
    case 2:
      var value = /** @type {!proto.speech.tts.lucid.PunctuationOrPause.PunctuationStyle} */ (reader.readEnum());
      msg.setPunctuationStyle(value);
      break;
    case 3:
      var value = new proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak;
      reader.readMessage(value,proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak.deserializeBinaryFromReader);
      msg.setPhraseBreak(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.lucid.PunctuationOrPause.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.lucid.PunctuationOrPause.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.lucid.PunctuationOrPause} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.PunctuationOrPause.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = /** @type {!proto.speech.tts.lucid.PunctuationOrPause.PunctuationStyle} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeEnum(
      2,
      f
    );
  }
  f = message.getPhraseBreak();
  if (f != null) {
    writer.writeMessage(
      3,
      f,
      proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak.serializeBinaryToWriter
    );
  }
};


/**
 * @enum {number}
 */
proto.speech.tts.lucid.PunctuationOrPause.PunctuationStyle = {
  UNSPECIFIED_STYLE: 0,
  SILENT: 1,
  SPOKEN: 2
};




if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak.toObject = function(includeInstance, msg) {
  var f, obj = {
    strength: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    durationSeconds: (f = jspb.Message.getOptionalFloatingPointField(msg, 2)) == null ? undefined : f,
    type: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of PhraseBreak as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.strength;

  /** @type {?number|undefined} */
  this.durationSeconds;

  /** @type {?number|undefined} */
  this.type;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak}
 */
proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak.fromObject = function(obj) {
  var msg = new proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak();
  obj.strength != null && jspb.Message.setField(msg, 1, obj.strength);
  obj.durationSeconds != null && jspb.Message.setField(msg, 2, obj.durationSeconds);
  obj.type != null && jspb.Message.setField(msg, 3, obj.type);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak}
 */
proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak;
  return proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak}
 */
proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {!proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak.BreakStrength} */ (reader.readEnum());
      msg.setStrength(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setDurationSeconds(value);
      break;
    case 3:
      var value = /** @type {!proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak.BreakType} */ (reader.readEnum());
      msg.setType(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {!proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak.BreakStrength} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeEnum(
      1,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeFloat(
      2,
      f
    );
  }
  f = /** @type {!proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak.BreakType} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeEnum(
      3,
      f
    );
  }
};


/**
 * @enum {number}
 */
proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak.BreakStrength = {
  UNSPECIFIED_BREAK_STRENGTH: 0,
  NO_BREAK: 1,
  MILD_BREAK: 2,
  MODERATE_BREAK: 3,
  STRONG_BREAK: 4,
  EXTRA_STRONG_BREAK: 5,
  PARAGRAPH_BREAK: 6
};

/**
 * @enum {number}
 */
proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak.BreakType = {
  UNSPECIFIED_BREAK_TYPE: 0,
  TONALITY_ONLY: 1,
  DURATION_ONLY: 2,
  DURATION_AND_TONALITY: 3
};

/**
 * optional BreakStrength strength = 1;
 * @return {!proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak.BreakStrength}
 */
proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak.prototype.getStrength = function() {
  return /** @type {!proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak.BreakStrength} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak.BreakStrength|undefined} value
 * @return {!proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak} returns this
 */
proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak.prototype.setStrength = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak} returns this
 */
proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak.prototype.clearStrength = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak.prototype.hasStrength = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional float duration_seconds = 2;
 * @return {?number}
 */
proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak.prototype.getDurationSeconds = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 2));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak} returns this
 */
proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak.prototype.setDurationSeconds = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak} returns this
 */
proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak.prototype.clearDurationSeconds = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak.prototype.hasDurationSeconds = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional BreakType type = 3;
 * @return {!proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak.BreakType}
 */
proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak.prototype.getType = function() {
  return /** @type {!proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak.BreakType} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak.BreakType|undefined} value
 * @return {!proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak} returns this
 */
proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak.prototype.setType = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak} returns this
 */
proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak.prototype.clearType = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak.prototype.hasType = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak}
 */
proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak, data);
};


/**
 * optional string text = 1;
 * @return {?string}
 */
proto.speech.tts.lucid.PunctuationOrPause.prototype.getText = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.lucid.PunctuationOrPause} returns this
 */
proto.speech.tts.lucid.PunctuationOrPause.prototype.setText = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.PunctuationOrPause} returns this
 */
proto.speech.tts.lucid.PunctuationOrPause.prototype.clearText = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.PunctuationOrPause.prototype.hasText = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional PunctuationStyle punctuation_style = 2;
 * @return {!proto.speech.tts.lucid.PunctuationOrPause.PunctuationStyle}
 */
proto.speech.tts.lucid.PunctuationOrPause.prototype.getPunctuationStyle = function() {
  return /** @type {!proto.speech.tts.lucid.PunctuationOrPause.PunctuationStyle} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?proto.speech.tts.lucid.PunctuationOrPause.PunctuationStyle|undefined} value
 * @return {!proto.speech.tts.lucid.PunctuationOrPause} returns this
 */
proto.speech.tts.lucid.PunctuationOrPause.prototype.setPunctuationStyle = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.PunctuationOrPause} returns this
 */
proto.speech.tts.lucid.PunctuationOrPause.prototype.clearPunctuationStyle = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.PunctuationOrPause.prototype.hasPunctuationStyle = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional PhraseBreak phrase_break = 3;
 * @return {?proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak}
 */
proto.speech.tts.lucid.PunctuationOrPause.prototype.getPhraseBreak = function() {
  return /** @type{?proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak, 3));
};


/**
 * @param {?proto.speech.tts.lucid.PunctuationOrPause.PhraseBreak|undefined} value
 * @return {!proto.speech.tts.lucid.PunctuationOrPause} returns this
*/
proto.speech.tts.lucid.PunctuationOrPause.prototype.setPhraseBreak = function(value) {
  return jspb.Message.setWrapperField(this, 3, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.lucid.PunctuationOrPause} returns this
 */
proto.speech.tts.lucid.PunctuationOrPause.prototype.clearPhraseBreak = function() {
  return this.setPhraseBreak(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.PunctuationOrPause.prototype.hasPhraseBreak = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.lucid.PunctuationOrPause}
 */
proto.speech.tts.lucid.PunctuationOrPause.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.lucid.PunctuationOrPause, data);
};



//speech/tts/public/lucid/structured_token_proto_jspb/romannumeral.jspb.js
// source: speech/tts/public/lucid/structured_token.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.lucid.RomanNumeral');
goog.provide('proto.speech.tts.lucid.RomanNumeral.Style');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.lucid.RomanNumeral = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.lucid.RomanNumeral, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.lucid.RomanNumeral.displayName = 'proto.speech.tts.lucid.RomanNumeral';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.lucid.RomanNumeral.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.lucid.RomanNumeral.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.lucid.RomanNumeral} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.RomanNumeral.toObject = function(includeInstance, msg) {
  var f, obj = {
    text: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    style: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of RomanNumeral as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.lucid.RomanNumeral.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.text;

  /** @type {?number|undefined} */
  this.style;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.lucid.RomanNumeral.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.lucid.RomanNumeral}
 */
proto.speech.tts.lucid.RomanNumeral.fromObject = function(obj) {
  var msg = new proto.speech.tts.lucid.RomanNumeral();
  obj.text != null && jspb.Message.setField(msg, 1, obj.text);
  obj.style != null && jspb.Message.setField(msg, 2, obj.style);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.lucid.RomanNumeral}
 */
proto.speech.tts.lucid.RomanNumeral.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.lucid.RomanNumeral;
  return proto.speech.tts.lucid.RomanNumeral.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.lucid.RomanNumeral} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.lucid.RomanNumeral}
 */
proto.speech.tts.lucid.RomanNumeral.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setText(value);
      break;
    case 2:
      var value = /** @type {!proto.speech.tts.lucid.RomanNumeral.Style} */ (reader.readEnum());
      msg.setStyle(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.lucid.RomanNumeral.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.lucid.RomanNumeral.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.lucid.RomanNumeral} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.RomanNumeral.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = /** @type {!proto.speech.tts.lucid.RomanNumeral.Style} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeEnum(
      2,
      f
    );
  }
};


/**
 * @enum {number}
 */
proto.speech.tts.lucid.RomanNumeral.Style = {
  UNSPECIFIED_STYLE: 0,
  AS_CARDINAL: 1,
  AS_ORDINAL: 2,
  AS_DEFINITE_ORDINAL: 3
};

/**
 * optional string text = 1;
 * @return {?string}
 */
proto.speech.tts.lucid.RomanNumeral.prototype.getText = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.lucid.RomanNumeral} returns this
 */
proto.speech.tts.lucid.RomanNumeral.prototype.setText = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.RomanNumeral} returns this
 */
proto.speech.tts.lucid.RomanNumeral.prototype.clearText = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.RomanNumeral.prototype.hasText = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional Style style = 2;
 * @return {!proto.speech.tts.lucid.RomanNumeral.Style}
 */
proto.speech.tts.lucid.RomanNumeral.prototype.getStyle = function() {
  return /** @type {!proto.speech.tts.lucid.RomanNumeral.Style} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?proto.speech.tts.lucid.RomanNumeral.Style|undefined} value
 * @return {!proto.speech.tts.lucid.RomanNumeral} returns this
 */
proto.speech.tts.lucid.RomanNumeral.prototype.setStyle = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.RomanNumeral} returns this
 */
proto.speech.tts.lucid.RomanNumeral.prototype.clearStyle = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.RomanNumeral.prototype.hasStyle = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.lucid.RomanNumeral}
 */
proto.speech.tts.lucid.RomanNumeral.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.lucid.RomanNumeral, data);
};



//speech/tts/public/lucid/structured_token_proto_jspb/spelling.jspb.js
// source: speech/tts/public/lucid/structured_token.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.lucid.Spelling');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.lucid.Spelling = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.lucid.Spelling, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.lucid.Spelling.displayName = 'proto.speech.tts.lucid.Spelling';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.lucid.Spelling.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.lucid.Spelling.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.lucid.Spelling} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.Spelling.toObject = function(includeInstance, msg) {
  var f, obj = {
    graphemes: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    chunkSize: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Spelling as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.lucid.Spelling.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.graphemes;

  /** @type {?number|undefined} */
  this.chunkSize;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.lucid.Spelling.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.lucid.Spelling}
 */
proto.speech.tts.lucid.Spelling.fromObject = function(obj) {
  var msg = new proto.speech.tts.lucid.Spelling();
  obj.graphemes != null && jspb.Message.setField(msg, 1, obj.graphemes);
  obj.chunkSize != null && jspb.Message.setField(msg, 2, obj.chunkSize);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.lucid.Spelling}
 */
proto.speech.tts.lucid.Spelling.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.lucid.Spelling;
  return proto.speech.tts.lucid.Spelling.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.lucid.Spelling} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.lucid.Spelling}
 */
proto.speech.tts.lucid.Spelling.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setGraphemes(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setChunkSize(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.lucid.Spelling.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.lucid.Spelling.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.lucid.Spelling} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.Spelling.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeInt32(
      2,
      f
    );
  }
};


/**
 * optional string graphemes = 1;
 * @return {?string}
 */
proto.speech.tts.lucid.Spelling.prototype.getGraphemes = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.lucid.Spelling} returns this
 */
proto.speech.tts.lucid.Spelling.prototype.setGraphemes = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Spelling} returns this
 */
proto.speech.tts.lucid.Spelling.prototype.clearGraphemes = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Spelling.prototype.hasGraphemes = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional int32 chunk_size = 2;
 * @return {?number}
 */
proto.speech.tts.lucid.Spelling.prototype.getChunkSize = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.lucid.Spelling} returns this
 */
proto.speech.tts.lucid.Spelling.prototype.setChunkSize = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Spelling} returns this
 */
proto.speech.tts.lucid.Spelling.prototype.clearChunkSize = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Spelling.prototype.hasChunkSize = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.lucid.Spelling}
 */
proto.speech.tts.lucid.Spelling.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.lucid.Spelling, data);
};



//speech/tts/public/lucid/structured_token_proto_jspb/telephone.jspb.js
// source: speech/tts/public/lucid/structured_token.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.lucid.Telephone');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.lucid.Telephone = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.lucid.Telephone.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.lucid.Telephone, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.lucid.Telephone.displayName = 'proto.speech.tts.lucid.Telephone';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.lucid.Telephone.repeatedFields_ = [2];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.lucid.Telephone.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.lucid.Telephone.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.lucid.Telephone} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.Telephone.toObject = function(includeInstance, msg) {
  var f, obj = {
    countryCode: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    numberPartsList: (f = jspb.Message.getRepeatedField(msg, 2)) == null ? undefined : f,
    extension: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Telephone as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.lucid.Telephone.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.countryCode;

  /** @type {?Array<string>|undefined} */
  this.numberPartsList;

  /** @type {?string|undefined} */
  this.extension;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.lucid.Telephone.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.lucid.Telephone}
 */
proto.speech.tts.lucid.Telephone.fromObject = function(obj) {
  var msg = new proto.speech.tts.lucid.Telephone();
  obj.countryCode != null && jspb.Message.setField(msg, 1, obj.countryCode);
  obj.numberPartsList != null && jspb.Message.setField(msg, 2, obj.numberPartsList);
  obj.extension != null && jspb.Message.setField(msg, 3, obj.extension);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.lucid.Telephone}
 */
proto.speech.tts.lucid.Telephone.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.lucid.Telephone;
  return proto.speech.tts.lucid.Telephone.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.lucid.Telephone} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.lucid.Telephone}
 */
proto.speech.tts.lucid.Telephone.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setCountryCode(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.addNumberParts(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setExtension$(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.lucid.Telephone.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.lucid.Telephone.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.lucid.Telephone} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.Telephone.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = message.getNumberPartsList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      2,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeString(
      3,
      f
    );
  }
};


/**
 * optional string country_code = 1;
 * @return {?string}
 */
proto.speech.tts.lucid.Telephone.prototype.getCountryCode = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.lucid.Telephone} returns this
 */
proto.speech.tts.lucid.Telephone.prototype.setCountryCode = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Telephone} returns this
 */
proto.speech.tts.lucid.Telephone.prototype.clearCountryCode = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Telephone.prototype.hasCountryCode = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * repeated string number_parts = 2;
 * @return {!Array<string>}
 */
proto.speech.tts.lucid.Telephone.prototype.getNumberPartsList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 2));
};


/**
 * @param {Array<string>|undefined} value
 * @return {!proto.speech.tts.lucid.Telephone} returns this
 */
proto.speech.tts.lucid.Telephone.prototype.setNumberPartsList = function(value) {
  return jspb.Message.setField(this, 2, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.lucid.Telephone} returns this
 */
proto.speech.tts.lucid.Telephone.prototype.addNumberParts = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 2, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.lucid.Telephone} returns this
 */
proto.speech.tts.lucid.Telephone.prototype.clearNumberPartsList = function() {
  return this.setNumberPartsList([]);
};


/**
 * optional string extension = 3;
 * @return {?string}
 */
proto.speech.tts.lucid.Telephone.prototype.getExtension$ = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.lucid.Telephone} returns this
 */
proto.speech.tts.lucid.Telephone.prototype.setExtension$ = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Telephone} returns this
 */
proto.speech.tts.lucid.Telephone.prototype.clearExtension$ = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Telephone.prototype.hasExtension$ = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.lucid.Telephone}
 */
proto.speech.tts.lucid.Telephone.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.lucid.Telephone, data);
};



//speech/tts/public/lucid/structured_token_proto_jspb/time.jspb.js
// source: speech/tts/public/lucid/structured_token.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.lucid.Time');
goog.provide('proto.speech.tts.lucid.Time.Style');
goog.provide('proto.speech.tts.lucid.Time.StyleCase');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

goog.forwardDeclare('proto.speech.tts.lucid.CommonVerbalizationStyles.Style');
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.lucid.Time = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, proto.speech.tts.lucid.Time.oneofGroups_);
};
goog.inherits(proto.speech.tts.lucid.Time, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.lucid.Time.displayName = 'proto.speech.tts.lucid.Time';
}

/**
 * Oneof group definitions for this message. Each group defines the field
 * numbers belonging to that group. When of these fields' value is set, all
 * other fields in the group are cleared. During deserialization, if multiple
 * fields are encountered for a group, only the last value seen will be kept.
 * @private {!Array<!Array<number>>}
 * @const
 */
proto.speech.tts.lucid.Time.oneofGroups_ = [[5,6,7]];

/**
 * @enum {number}
 */
proto.speech.tts.lucid.Time.StyleCase = {
  STYLE_NOT_SET: 0,
  COMMON_STYLE: 5,
  SPECIFIC_STYLE: 6,
  STYLE_ID: 7
};

/**
 * @return {!proto.speech.tts.lucid.Time.StyleCase}
 */
proto.speech.tts.lucid.Time.prototype.getStyleCase = function() {
  return /** @type {!proto.speech.tts.lucid.Time.StyleCase} */(jspb.Message.computeOneofCase(this, proto.speech.tts.lucid.Time.oneofGroups_[0]));
};



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.lucid.Time.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.lucid.Time.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.lucid.Time} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.Time.toObject = function(includeInstance, msg) {
  var f, obj = {
    hours: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    minutes: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    seconds: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,
    zone: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f,
    commonStyle: (f = jspb.Message.getField(msg, 5)) == null ? undefined : f,
    specificStyle: (f = jspb.Message.getField(msg, 6)) == null ? undefined : f,
    styleId: (f = jspb.Message.getField(msg, 7)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Time as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.lucid.Time.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.hours;

  /** @type {?number|undefined} */
  this.minutes;

  /** @type {?number|undefined} */
  this.seconds;

  /** @type {?string|undefined} */
  this.zone;

  /** @type {?number|undefined} */
  this.commonStyle;

  /** @type {?number|undefined} */
  this.specificStyle;

  /** @type {?number|undefined} */
  this.styleId;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.lucid.Time.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.lucid.Time}
 */
proto.speech.tts.lucid.Time.fromObject = function(obj) {
  var msg = new proto.speech.tts.lucid.Time();
  obj.hours != null && jspb.Message.setField(msg, 1, obj.hours);
  obj.minutes != null && jspb.Message.setField(msg, 2, obj.minutes);
  obj.seconds != null && jspb.Message.setField(msg, 3, obj.seconds);
  obj.zone != null && jspb.Message.setField(msg, 4, obj.zone);
  obj.commonStyle != null && jspb.Message.setField(msg, 5, obj.commonStyle);
  obj.specificStyle != null && jspb.Message.setField(msg, 6, obj.specificStyle);
  obj.styleId != null && jspb.Message.setField(msg, 7, obj.styleId);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.lucid.Time}
 */
proto.speech.tts.lucid.Time.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.lucid.Time;
  return proto.speech.tts.lucid.Time.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.lucid.Time} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.lucid.Time}
 */
proto.speech.tts.lucid.Time.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setHours(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setMinutes(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setSeconds(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setZone(value);
      break;
    case 5:
      var value = /** @type {!proto.speech.tts.lucid.CommonVerbalizationStyles.Style} */ (reader.readEnum());
      msg.setCommonStyle(value);
      break;
    case 6:
      var value = /** @type {!proto.speech.tts.lucid.Time.Style} */ (reader.readEnum());
      msg.setSpecificStyle(value);
      break;
    case 7:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setStyleId(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.lucid.Time.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.lucid.Time.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.lucid.Time} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.Time.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?number} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeInt32(
      1,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeInt32(
      2,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeInt32(
      3,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeString(
      4,
      f
    );
  }
  f = /** @type {!proto.speech.tts.lucid.CommonVerbalizationStyles.Style} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeEnum(
      5,
      f
    );
  }
  f = /** @type {!proto.speech.tts.lucid.Time.Style} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeEnum(
      6,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 7));
  if (f != null) {
    writer.writeInt32(
      7,
      f
    );
  }
};


/**
 * @enum {number}
 */
proto.speech.tts.lucid.Time.Style = {
  UNSPECIFIED_STYLE: 0,
  TWENTY_FOUR_HOUR_STYLE: 1,
  TWELVE_HOUR_STYLE_NO_PERIOD: 2,
  TWELVE_HOUR_STYLE_WITH_ABBREVIATED_PERIOD: 3,
  TWELVE_HOUR_STYLE_WITH_VERBOSE_PERIOD: 4
};

/**
 * optional int32 hours = 1;
 * @return {?number}
 */
proto.speech.tts.lucid.Time.prototype.getHours = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.lucid.Time} returns this
 */
proto.speech.tts.lucid.Time.prototype.setHours = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Time} returns this
 */
proto.speech.tts.lucid.Time.prototype.clearHours = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Time.prototype.hasHours = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional int32 minutes = 2;
 * @return {?number}
 */
proto.speech.tts.lucid.Time.prototype.getMinutes = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.lucid.Time} returns this
 */
proto.speech.tts.lucid.Time.prototype.setMinutes = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Time} returns this
 */
proto.speech.tts.lucid.Time.prototype.clearMinutes = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Time.prototype.hasMinutes = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional int32 seconds = 3;
 * @return {?number}
 */
proto.speech.tts.lucid.Time.prototype.getSeconds = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.lucid.Time} returns this
 */
proto.speech.tts.lucid.Time.prototype.setSeconds = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Time} returns this
 */
proto.speech.tts.lucid.Time.prototype.clearSeconds = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Time.prototype.hasSeconds = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional string zone = 4;
 * @return {?string}
 */
proto.speech.tts.lucid.Time.prototype.getZone = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 4));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.lucid.Time} returns this
 */
proto.speech.tts.lucid.Time.prototype.setZone = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Time} returns this
 */
proto.speech.tts.lucid.Time.prototype.clearZone = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Time.prototype.hasZone = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional CommonVerbalizationStyles.Style common_style = 5;
 * @return {!proto.speech.tts.lucid.CommonVerbalizationStyles.Style}
 */
proto.speech.tts.lucid.Time.prototype.getCommonStyle = function() {
  return /** @type {!proto.speech.tts.lucid.CommonVerbalizationStyles.Style} */ (jspb.Message.getField(this, 5));
};


/**
 * @param {?proto.speech.tts.lucid.CommonVerbalizationStyles.Style|undefined} value
 * @return {!proto.speech.tts.lucid.Time} returns this
 */
proto.speech.tts.lucid.Time.prototype.setCommonStyle = function(value) {
  return jspb.Message.setOneofField(this, 5, proto.speech.tts.lucid.Time.oneofGroups_[0], value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Time} returns this
 */
proto.speech.tts.lucid.Time.prototype.clearCommonStyle = function() {
  return jspb.Message.setOneofField(this, 5, proto.speech.tts.lucid.Time.oneofGroups_[0], undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Time.prototype.hasCommonStyle = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional Style specific_style = 6;
 * @return {!proto.speech.tts.lucid.Time.Style}
 */
proto.speech.tts.lucid.Time.prototype.getSpecificStyle = function() {
  return /** @type {!proto.speech.tts.lucid.Time.Style} */ (jspb.Message.getField(this, 6));
};


/**
 * @param {?proto.speech.tts.lucid.Time.Style|undefined} value
 * @return {!proto.speech.tts.lucid.Time} returns this
 */
proto.speech.tts.lucid.Time.prototype.setSpecificStyle = function(value) {
  return jspb.Message.setOneofField(this, 6, proto.speech.tts.lucid.Time.oneofGroups_[0], value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Time} returns this
 */
proto.speech.tts.lucid.Time.prototype.clearSpecificStyle = function() {
  return jspb.Message.setOneofField(this, 6, proto.speech.tts.lucid.Time.oneofGroups_[0], undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Time.prototype.hasSpecificStyle = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional int32 style_id = 7;
 * @return {?number}
 */
proto.speech.tts.lucid.Time.prototype.getStyleId = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 7));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.lucid.Time} returns this
 */
proto.speech.tts.lucid.Time.prototype.setStyleId = function(value) {
  return jspb.Message.setOneofField(this, 7, proto.speech.tts.lucid.Time.oneofGroups_[0], value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Time} returns this
 */
proto.speech.tts.lucid.Time.prototype.clearStyleId = function() {
  return jspb.Message.setOneofField(this, 7, proto.speech.tts.lucid.Time.oneofGroups_[0], undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Time.prototype.hasStyleId = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.lucid.Time}
 */
proto.speech.tts.lucid.Time.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.lucid.Time, data);
};



//speech/tts/public/lucid/structured_token_proto_jspb/verbatim.jspb.js
// source: speech/tts/public/lucid/structured_token.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.lucid.Verbatim');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.lucid.Verbatim = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.lucid.Verbatim, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.lucid.Verbatim.displayName = 'proto.speech.tts.lucid.Verbatim';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.lucid.Verbatim.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.lucid.Verbatim.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.lucid.Verbatim} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.Verbatim.toObject = function(includeInstance, msg) {
  var f, obj = {
    text: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Verbatim as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.lucid.Verbatim.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.text;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.lucid.Verbatim.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.lucid.Verbatim}
 */
proto.speech.tts.lucid.Verbatim.fromObject = function(obj) {
  var msg = new proto.speech.tts.lucid.Verbatim();
  obj.text != null && jspb.Message.setField(msg, 1, obj.text);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.lucid.Verbatim}
 */
proto.speech.tts.lucid.Verbatim.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.lucid.Verbatim;
  return proto.speech.tts.lucid.Verbatim.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.lucid.Verbatim} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.lucid.Verbatim}
 */
proto.speech.tts.lucid.Verbatim.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setText(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.lucid.Verbatim.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.lucid.Verbatim.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.lucid.Verbatim} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.Verbatim.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
};


/**
 * optional string text = 1;
 * @return {?string}
 */
proto.speech.tts.lucid.Verbatim.prototype.getText = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.lucid.Verbatim} returns this
 */
proto.speech.tts.lucid.Verbatim.prototype.setText = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.Verbatim} returns this
 */
proto.speech.tts.lucid.Verbatim.prototype.clearText = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.Verbatim.prototype.hasText = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.lucid.Verbatim}
 */
proto.speech.tts.lucid.Verbatim.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.lucid.Verbatim, data);
};



//speech/tts/public/lucid/lucid_markup_proto_jspb/lucidpiece.jspb.js
// source: speech/tts/public/lucid/lucid_markup.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.lucid.LucidPiece');
goog.provide('proto.speech.tts.lucid.LucidPiece.OriginalPosition');
goog.provide('proto.speech.tts.lucid.LucidPiece.TokenCase');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.lucid.Abbreviation');
goog.require('proto.speech.tts.lucid.Annotation');
goog.require('proto.speech.tts.lucid.ChemicalFormula');
goog.require('proto.speech.tts.lucid.Connector');
goog.require('proto.speech.tts.lucid.Date');
goog.require('proto.speech.tts.lucid.Digits');
goog.require('proto.speech.tts.lucid.Duration');
goog.require('proto.speech.tts.lucid.Electronic');
goog.require('proto.speech.tts.lucid.EmoticonEmoji');
goog.require('proto.speech.tts.lucid.Fraction');
goog.require('proto.speech.tts.lucid.LetterSequence');
goog.require('proto.speech.tts.lucid.LucidWord');
goog.require('proto.speech.tts.lucid.Measure');
goog.require('proto.speech.tts.lucid.Money');
goog.require('proto.speech.tts.lucid.Number');
goog.require('proto.speech.tts.lucid.Ordinal');
goog.require('proto.speech.tts.lucid.PunctuationOrPause');
goog.require('proto.speech.tts.lucid.RomanNumeral');
goog.require('proto.speech.tts.lucid.Spelling');
goog.require('proto.speech.tts.lucid.Telephone');
goog.require('proto.speech.tts.lucid.Time');
goog.require('proto.speech.tts.lucid.Verbatim');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.lucid.LucidPiece = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.lucid.LucidPiece.repeatedFields_, proto.speech.tts.lucid.LucidPiece.oneofGroups_);
};
goog.inherits(proto.speech.tts.lucid.LucidPiece, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.lucid.LucidPiece.displayName = 'proto.speech.tts.lucid.LucidPiece';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.lucid.LucidPiece.OriginalPosition = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.lucid.LucidPiece.OriginalPosition, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.lucid.LucidPiece.OriginalPosition.displayName = 'proto.speech.tts.lucid.LucidPiece.OriginalPosition';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.lucid.LucidPiece.repeatedFields_ = [5];

/**
 * Oneof group definitions for this message. Each group defines the field
 * numbers belonging to that group. When of these fields' value is set, all
 * other fields in the group are cleared. During deserialization, if multiple
 * fields are encountered for a group, only the last value seen will be kept.
 * @private {!Array<!Array<number>>}
 * @const
 */
proto.speech.tts.lucid.LucidPiece.oneofGroups_ = [[100,101,102,103,104,105,106,107,108,109,110,111,112,113,114,115,116,117,118,119,120]];

/**
 * @enum {number}
 */
proto.speech.tts.lucid.LucidPiece.TokenCase = {
  TOKEN_NOT_SET: 0,
  WORD: 100,
  PUNCTUATION_OR_PAUSE: 101,
  VERBATIM: 102,
  SPELLING: 103,
  LETTERS: 104,
  ABBREVIATION: 105,
  ROMAN_NUMERAL: 106,
  NUMBER: 107,
  ORDINAL: 108,
  FRACTION: 109,
  DIGITS: 110,
  TIME: 111,
  DURATION: 112,
  MEASURE: 113,
  DATE: 114,
  TELEPHONE: 115,
  MONEY: 116,
  ELECTRONIC: 117,
  EMOTICON_EMOJI: 118,
  CHEMICAL_FORMULA: 119,
  CONNECTOR: 120
};

/**
 * @return {!proto.speech.tts.lucid.LucidPiece.TokenCase}
 */
proto.speech.tts.lucid.LucidPiece.prototype.getTokenCase = function() {
  return /** @type {!proto.speech.tts.lucid.LucidPiece.TokenCase} */(jspb.Message.computeOneofCase(this, proto.speech.tts.lucid.LucidPiece.oneofGroups_[0]));
};



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.lucid.LucidPiece.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.lucid.LucidPiece.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.lucid.LucidPiece} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.LucidPiece.toObject = function(includeInstance, msg) {
  var f, obj = {
    ownIndex: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    inputTextConnector: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,
    inputText: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    annotation: (f = msg.getAnnotation()) && proto.speech.tts.lucid.Annotation.toObject(includeInstance, f),
    verbalizedWordsList: jspb.Message.toObjectList(msg.getVerbalizedWordsList(),
    proto.speech.tts.lucid.LucidWord.toObject, includeInstance),
    tokenPrefix: (f = jspb.Message.getField(msg, 6)) == null ? undefined : f,
    tokenSuffix: (f = jspb.Message.getField(msg, 7)) == null ? undefined : f,
    originalStartPosition: (f = msg.getOriginalStartPosition()) && proto.speech.tts.lucid.LucidPiece.OriginalPosition.toObject(includeInstance, f),
    word: (f = jspb.Message.getField(msg, 100)) == null ? undefined : f,
    punctuationOrPause: (f = msg.getPunctuationOrPause()) && proto.speech.tts.lucid.PunctuationOrPause.toObject(includeInstance, f),
    verbatim: (f = msg.getVerbatim()) && proto.speech.tts.lucid.Verbatim.toObject(includeInstance, f),
    spelling: (f = msg.getSpelling()) && proto.speech.tts.lucid.Spelling.toObject(includeInstance, f),
    letters: (f = msg.getLetters()) && proto.speech.tts.lucid.LetterSequence.toObject(includeInstance, f),
    abbreviation: (f = msg.getAbbreviation()) && proto.speech.tts.lucid.Abbreviation.toObject(includeInstance, f),
    romanNumeral: (f = msg.getRomanNumeral()) && proto.speech.tts.lucid.RomanNumeral.toObject(includeInstance, f),
    number: (f = msg.getNumber()) && proto.speech.tts.lucid.Number.toObject(includeInstance, f),
    ordinal: (f = msg.getOrdinal()) && proto.speech.tts.lucid.Ordinal.toObject(includeInstance, f),
    fraction: (f = msg.getFraction()) && proto.speech.tts.lucid.Fraction.toObject(includeInstance, f),
    digits: (f = msg.getDigits()) && proto.speech.tts.lucid.Digits.toObject(includeInstance, f),
    time: (f = msg.getTime()) && proto.speech.tts.lucid.Time.toObject(includeInstance, f),
    duration: (f = msg.getDuration()) && proto.speech.tts.lucid.Duration.toObject(includeInstance, f),
    measure: (f = msg.getMeasure()) && proto.speech.tts.lucid.Measure.toObject(includeInstance, f),
    date: (f = msg.getDate()) && proto.speech.tts.lucid.Date.toObject(includeInstance, f),
    telephone: (f = msg.getTelephone()) && proto.speech.tts.lucid.Telephone.toObject(includeInstance, f),
    money: (f = msg.getMoney()) && proto.speech.tts.lucid.Money.toObject(includeInstance, f),
    electronic: (f = msg.getElectronic()) && proto.speech.tts.lucid.Electronic.toObject(includeInstance, f),
    emoticonEmoji: (f = msg.getEmoticonEmoji()) && proto.speech.tts.lucid.EmoticonEmoji.toObject(includeInstance, f),
    chemicalFormula: (f = msg.getChemicalFormula()) && proto.speech.tts.lucid.ChemicalFormula.toObject(includeInstance, f),
    connector: (f = msg.getConnector()) && proto.speech.tts.lucid.Connector.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of LucidPiece as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.lucid.LucidPiece.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.ownIndex;

  /** @type {?string|undefined} */
  this.inputTextConnector;

  /** @type {?string|undefined} */
  this.inputText;

  /** @type {?proto.speech.tts.lucid.Annotation.ObjectFormat|undefined} */
  this.annotation;

  /** @type {?Array<!proto.speech.tts.lucid.LucidWord.ObjectFormat>|undefined} */
  this.verbalizedWordsList;

  /** @type {?string|undefined} */
  this.tokenPrefix;

  /** @type {?string|undefined} */
  this.tokenSuffix;

  /** @type {?proto.speech.tts.lucid.LucidPiece.OriginalPosition.ObjectFormat|undefined} */
  this.originalStartPosition;

  /** @type {?string|undefined} */
  this.word;

  /** @type {?proto.speech.tts.lucid.PunctuationOrPause.ObjectFormat|undefined} */
  this.punctuationOrPause;

  /** @type {?proto.speech.tts.lucid.Verbatim.ObjectFormat|undefined} */
  this.verbatim;

  /** @type {?proto.speech.tts.lucid.Spelling.ObjectFormat|undefined} */
  this.spelling;

  /** @type {?proto.speech.tts.lucid.LetterSequence.ObjectFormat|undefined} */
  this.letters;

  /** @type {?proto.speech.tts.lucid.Abbreviation.ObjectFormat|undefined} */
  this.abbreviation;

  /** @type {?proto.speech.tts.lucid.RomanNumeral.ObjectFormat|undefined} */
  this.romanNumeral;

  /** @type {?proto.speech.tts.lucid.Number.ObjectFormat|undefined} */
  this.number;

  /** @type {?proto.speech.tts.lucid.Ordinal.ObjectFormat|undefined} */
  this.ordinal;

  /** @type {?proto.speech.tts.lucid.Fraction.ObjectFormat|undefined} */
  this.fraction;

  /** @type {?proto.speech.tts.lucid.Digits.ObjectFormat|undefined} */
  this.digits;

  /** @type {?proto.speech.tts.lucid.Time.ObjectFormat|undefined} */
  this.time;

  /** @type {?proto.speech.tts.lucid.Duration.ObjectFormat|undefined} */
  this.duration;

  /** @type {?proto.speech.tts.lucid.Measure.ObjectFormat|undefined} */
  this.measure;

  /** @type {?proto.speech.tts.lucid.Date.ObjectFormat|undefined} */
  this.date;

  /** @type {?proto.speech.tts.lucid.Telephone.ObjectFormat|undefined} */
  this.telephone;

  /** @type {?proto.speech.tts.lucid.Money.ObjectFormat|undefined} */
  this.money;

  /** @type {?proto.speech.tts.lucid.Electronic.ObjectFormat|undefined} */
  this.electronic;

  /** @type {?proto.speech.tts.lucid.EmoticonEmoji.ObjectFormat|undefined} */
  this.emoticonEmoji;

  /** @type {?proto.speech.tts.lucid.ChemicalFormula.ObjectFormat|undefined} */
  this.chemicalFormula;

  /** @type {?proto.speech.tts.lucid.Connector.ObjectFormat|undefined} */
  this.connector;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.lucid.LucidPiece.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.lucid.LucidPiece}
 */
proto.speech.tts.lucid.LucidPiece.fromObject = function(obj) {
  var msg = new proto.speech.tts.lucid.LucidPiece();
  obj.ownIndex != null && jspb.Message.setField(msg, 1, obj.ownIndex);
  obj.inputTextConnector != null && jspb.Message.setField(msg, 3, obj.inputTextConnector);
  obj.inputText != null && jspb.Message.setField(msg, 2, obj.inputText);
  obj.annotation && jspb.Message.setWrapperField(
      msg, 4, proto.speech.tts.lucid.Annotation.fromObject(obj.annotation));
  obj.verbalizedWordsList && jspb.Message.setRepeatedWrapperField(
      msg, 5, obj.verbalizedWordsList.map(
          proto.speech.tts.lucid.LucidWord.fromObject));
  obj.tokenPrefix != null && jspb.Message.setField(msg, 6, obj.tokenPrefix);
  obj.tokenSuffix != null && jspb.Message.setField(msg, 7, obj.tokenSuffix);
  obj.originalStartPosition && jspb.Message.setWrapperField(
      msg, 8, proto.speech.tts.lucid.LucidPiece.OriginalPosition.fromObject(obj.originalStartPosition));
  obj.word != null && jspb.Message.setField(msg, 100, obj.word);
  obj.punctuationOrPause && jspb.Message.setWrapperField(
      msg, 101, proto.speech.tts.lucid.PunctuationOrPause.fromObject(obj.punctuationOrPause));
  obj.verbatim && jspb.Message.setWrapperField(
      msg, 102, proto.speech.tts.lucid.Verbatim.fromObject(obj.verbatim));
  obj.spelling && jspb.Message.setWrapperField(
      msg, 103, proto.speech.tts.lucid.Spelling.fromObject(obj.spelling));
  obj.letters && jspb.Message.setWrapperField(
      msg, 104, proto.speech.tts.lucid.LetterSequence.fromObject(obj.letters));
  obj.abbreviation && jspb.Message.setWrapperField(
      msg, 105, proto.speech.tts.lucid.Abbreviation.fromObject(obj.abbreviation));
  obj.romanNumeral && jspb.Message.setWrapperField(
      msg, 106, proto.speech.tts.lucid.RomanNumeral.fromObject(obj.romanNumeral));
  obj.number && jspb.Message.setWrapperField(
      msg, 107, proto.speech.tts.lucid.Number.fromObject(obj.number));
  obj.ordinal && jspb.Message.setWrapperField(
      msg, 108, proto.speech.tts.lucid.Ordinal.fromObject(obj.ordinal));
  obj.fraction && jspb.Message.setWrapperField(
      msg, 109, proto.speech.tts.lucid.Fraction.fromObject(obj.fraction));
  obj.digits && jspb.Message.setWrapperField(
      msg, 110, proto.speech.tts.lucid.Digits.fromObject(obj.digits));
  obj.time && jspb.Message.setWrapperField(
      msg, 111, proto.speech.tts.lucid.Time.fromObject(obj.time));
  obj.duration && jspb.Message.setWrapperField(
      msg, 112, proto.speech.tts.lucid.Duration.fromObject(obj.duration));
  obj.measure && jspb.Message.setWrapperField(
      msg, 113, proto.speech.tts.lucid.Measure.fromObject(obj.measure));
  obj.date && jspb.Message.setWrapperField(
      msg, 114, proto.speech.tts.lucid.Date.fromObject(obj.date));
  obj.telephone && jspb.Message.setWrapperField(
      msg, 115, proto.speech.tts.lucid.Telephone.fromObject(obj.telephone));
  obj.money && jspb.Message.setWrapperField(
      msg, 116, proto.speech.tts.lucid.Money.fromObject(obj.money));
  obj.electronic && jspb.Message.setWrapperField(
      msg, 117, proto.speech.tts.lucid.Electronic.fromObject(obj.electronic));
  obj.emoticonEmoji && jspb.Message.setWrapperField(
      msg, 118, proto.speech.tts.lucid.EmoticonEmoji.fromObject(obj.emoticonEmoji));
  obj.chemicalFormula && jspb.Message.setWrapperField(
      msg, 119, proto.speech.tts.lucid.ChemicalFormula.fromObject(obj.chemicalFormula));
  obj.connector && jspb.Message.setWrapperField(
      msg, 120, proto.speech.tts.lucid.Connector.fromObject(obj.connector));
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.lucid.LucidPiece}
 */
proto.speech.tts.lucid.LucidPiece.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.lucid.LucidPiece;
  return proto.speech.tts.lucid.LucidPiece.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.lucid.LucidPiece} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.lucid.LucidPiece}
 */
proto.speech.tts.lucid.LucidPiece.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setOwnIndex(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setInputTextConnector(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setInputText(value);
      break;
    case 4:
      var value = new proto.speech.tts.lucid.Annotation;
      reader.readMessage(value,proto.speech.tts.lucid.Annotation.deserializeBinaryFromReader);
      msg.setAnnotation(value);
      break;
    case 5:
      var value = new proto.speech.tts.lucid.LucidWord;
      reader.readMessage(value,proto.speech.tts.lucid.LucidWord.deserializeBinaryFromReader);
      msg.addVerbalizedWords(value);
      break;
    case 6:
      var value = /** @type {string} */ (reader.readString());
      msg.setTokenPrefix(value);
      break;
    case 7:
      var value = /** @type {string} */ (reader.readString());
      msg.setTokenSuffix(value);
      break;
    case 8:
      var value = new proto.speech.tts.lucid.LucidPiece.OriginalPosition;
      reader.readMessage(value,proto.speech.tts.lucid.LucidPiece.OriginalPosition.deserializeBinaryFromReader);
      msg.setOriginalStartPosition(value);
      break;
    case 100:
      var value = /** @type {string} */ (reader.readString());
      msg.setWord(value);
      break;
    case 101:
      var value = new proto.speech.tts.lucid.PunctuationOrPause;
      reader.readMessage(value,proto.speech.tts.lucid.PunctuationOrPause.deserializeBinaryFromReader);
      msg.setPunctuationOrPause(value);
      break;
    case 102:
      var value = new proto.speech.tts.lucid.Verbatim;
      reader.readMessage(value,proto.speech.tts.lucid.Verbatim.deserializeBinaryFromReader);
      msg.setVerbatim(value);
      break;
    case 103:
      var value = new proto.speech.tts.lucid.Spelling;
      reader.readMessage(value,proto.speech.tts.lucid.Spelling.deserializeBinaryFromReader);
      msg.setSpelling(value);
      break;
    case 104:
      var value = new proto.speech.tts.lucid.LetterSequence;
      reader.readMessage(value,proto.speech.tts.lucid.LetterSequence.deserializeBinaryFromReader);
      msg.setLetters(value);
      break;
    case 105:
      var value = new proto.speech.tts.lucid.Abbreviation;
      reader.readMessage(value,proto.speech.tts.lucid.Abbreviation.deserializeBinaryFromReader);
      msg.setAbbreviation(value);
      break;
    case 106:
      var value = new proto.speech.tts.lucid.RomanNumeral;
      reader.readMessage(value,proto.speech.tts.lucid.RomanNumeral.deserializeBinaryFromReader);
      msg.setRomanNumeral(value);
      break;
    case 107:
      var value = new proto.speech.tts.lucid.Number;
      reader.readMessage(value,proto.speech.tts.lucid.Number.deserializeBinaryFromReader);
      msg.setNumber(value);
      break;
    case 108:
      var value = new proto.speech.tts.lucid.Ordinal;
      reader.readMessage(value,proto.speech.tts.lucid.Ordinal.deserializeBinaryFromReader);
      msg.setOrdinal(value);
      break;
    case 109:
      var value = new proto.speech.tts.lucid.Fraction;
      reader.readMessage(value,proto.speech.tts.lucid.Fraction.deserializeBinaryFromReader);
      msg.setFraction(value);
      break;
    case 110:
      var value = new proto.speech.tts.lucid.Digits;
      reader.readMessage(value,proto.speech.tts.lucid.Digits.deserializeBinaryFromReader);
      msg.setDigits(value);
      break;
    case 111:
      var value = new proto.speech.tts.lucid.Time;
      reader.readMessage(value,proto.speech.tts.lucid.Time.deserializeBinaryFromReader);
      msg.setTime(value);
      break;
    case 112:
      var value = new proto.speech.tts.lucid.Duration;
      reader.readMessage(value,proto.speech.tts.lucid.Duration.deserializeBinaryFromReader);
      msg.setDuration(value);
      break;
    case 113:
      var value = new proto.speech.tts.lucid.Measure;
      reader.readMessage(value,proto.speech.tts.lucid.Measure.deserializeBinaryFromReader);
      msg.setMeasure(value);
      break;
    case 114:
      var value = new proto.speech.tts.lucid.Date;
      reader.readMessage(value,proto.speech.tts.lucid.Date.deserializeBinaryFromReader);
      msg.setDate(value);
      break;
    case 115:
      var value = new proto.speech.tts.lucid.Telephone;
      reader.readMessage(value,proto.speech.tts.lucid.Telephone.deserializeBinaryFromReader);
      msg.setTelephone(value);
      break;
    case 116:
      var value = new proto.speech.tts.lucid.Money;
      reader.readMessage(value,proto.speech.tts.lucid.Money.deserializeBinaryFromReader);
      msg.setMoney(value);
      break;
    case 117:
      var value = new proto.speech.tts.lucid.Electronic;
      reader.readMessage(value,proto.speech.tts.lucid.Electronic.deserializeBinaryFromReader);
      msg.setElectronic(value);
      break;
    case 118:
      var value = new proto.speech.tts.lucid.EmoticonEmoji;
      reader.readMessage(value,proto.speech.tts.lucid.EmoticonEmoji.deserializeBinaryFromReader);
      msg.setEmoticonEmoji(value);
      break;
    case 119:
      var value = new proto.speech.tts.lucid.ChemicalFormula;
      reader.readMessage(value,proto.speech.tts.lucid.ChemicalFormula.deserializeBinaryFromReader);
      msg.setChemicalFormula(value);
      break;
    case 120:
      var value = new proto.speech.tts.lucid.Connector;
      reader.readMessage(value,proto.speech.tts.lucid.Connector.deserializeBinaryFromReader);
      msg.setConnector(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.lucid.LucidPiece.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.lucid.LucidPiece.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.lucid.LucidPiece} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.LucidPiece.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?number} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeInt32(
      1,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeString(
      3,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
  f = message.getAnnotation();
  if (f != null) {
    writer.writeMessage(
      4,
      f,
      proto.speech.tts.lucid.Annotation.serializeBinaryToWriter
    );
  }
  f = message.getVerbalizedWordsList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      5,
      f,
      proto.speech.tts.lucid.LucidWord.serializeBinaryToWriter
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeString(
      6,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 7));
  if (f != null) {
    writer.writeString(
      7,
      f
    );
  }
  f = message.getOriginalStartPosition();
  if (f != null) {
    writer.writeMessage(
      8,
      f,
      proto.speech.tts.lucid.LucidPiece.OriginalPosition.serializeBinaryToWriter
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 100));
  if (f != null) {
    writer.writeString(
      100,
      f
    );
  }
  f = message.getPunctuationOrPause();
  if (f != null) {
    writer.writeMessage(
      101,
      f,
      proto.speech.tts.lucid.PunctuationOrPause.serializeBinaryToWriter
    );
  }
  f = message.getVerbatim();
  if (f != null) {
    writer.writeMessage(
      102,
      f,
      proto.speech.tts.lucid.Verbatim.serializeBinaryToWriter
    );
  }
  f = message.getSpelling();
  if (f != null) {
    writer.writeMessage(
      103,
      f,
      proto.speech.tts.lucid.Spelling.serializeBinaryToWriter
    );
  }
  f = message.getLetters();
  if (f != null) {
    writer.writeMessage(
      104,
      f,
      proto.speech.tts.lucid.LetterSequence.serializeBinaryToWriter
    );
  }
  f = message.getAbbreviation();
  if (f != null) {
    writer.writeMessage(
      105,
      f,
      proto.speech.tts.lucid.Abbreviation.serializeBinaryToWriter
    );
  }
  f = message.getRomanNumeral();
  if (f != null) {
    writer.writeMessage(
      106,
      f,
      proto.speech.tts.lucid.RomanNumeral.serializeBinaryToWriter
    );
  }
  f = message.getNumber();
  if (f != null) {
    writer.writeMessage(
      107,
      f,
      proto.speech.tts.lucid.Number.serializeBinaryToWriter
    );
  }
  f = message.getOrdinal();
  if (f != null) {
    writer.writeMessage(
      108,
      f,
      proto.speech.tts.lucid.Ordinal.serializeBinaryToWriter
    );
  }
  f = message.getFraction();
  if (f != null) {
    writer.writeMessage(
      109,
      f,
      proto.speech.tts.lucid.Fraction.serializeBinaryToWriter
    );
  }
  f = message.getDigits();
  if (f != null) {
    writer.writeMessage(
      110,
      f,
      proto.speech.tts.lucid.Digits.serializeBinaryToWriter
    );
  }
  f = message.getTime();
  if (f != null) {
    writer.writeMessage(
      111,
      f,
      proto.speech.tts.lucid.Time.serializeBinaryToWriter
    );
  }
  f = message.getDuration();
  if (f != null) {
    writer.writeMessage(
      112,
      f,
      proto.speech.tts.lucid.Duration.serializeBinaryToWriter
    );
  }
  f = message.getMeasure();
  if (f != null) {
    writer.writeMessage(
      113,
      f,
      proto.speech.tts.lucid.Measure.serializeBinaryToWriter
    );
  }
  f = message.getDate();
  if (f != null) {
    writer.writeMessage(
      114,
      f,
      proto.speech.tts.lucid.Date.serializeBinaryToWriter
    );
  }
  f = message.getTelephone();
  if (f != null) {
    writer.writeMessage(
      115,
      f,
      proto.speech.tts.lucid.Telephone.serializeBinaryToWriter
    );
  }
  f = message.getMoney();
  if (f != null) {
    writer.writeMessage(
      116,
      f,
      proto.speech.tts.lucid.Money.serializeBinaryToWriter
    );
  }
  f = message.getElectronic();
  if (f != null) {
    writer.writeMessage(
      117,
      f,
      proto.speech.tts.lucid.Electronic.serializeBinaryToWriter
    );
  }
  f = message.getEmoticonEmoji();
  if (f != null) {
    writer.writeMessage(
      118,
      f,
      proto.speech.tts.lucid.EmoticonEmoji.serializeBinaryToWriter
    );
  }
  f = message.getChemicalFormula();
  if (f != null) {
    writer.writeMessage(
      119,
      f,
      proto.speech.tts.lucid.ChemicalFormula.serializeBinaryToWriter
    );
  }
  f = message.getConnector();
  if (f != null) {
    writer.writeMessage(
      120,
      f,
      proto.speech.tts.lucid.Connector.serializeBinaryToWriter
    );
  }
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.lucid.LucidPiece.OriginalPosition.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.lucid.LucidPiece.OriginalPosition.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.lucid.LucidPiece.OriginalPosition} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.LucidPiece.OriginalPosition.toObject = function(includeInstance, msg) {
  var f, obj = {
    contentIndex: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    pieceIndex: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    byteIndex: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,
    timepointName: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of OriginalPosition as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.lucid.LucidPiece.OriginalPosition.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.contentIndex;

  /** @type {?number|undefined} */
  this.pieceIndex;

  /** @type {?number|undefined} */
  this.byteIndex;

  /** @type {?string|undefined} */
  this.timepointName;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.lucid.LucidPiece.OriginalPosition.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.lucid.LucidPiece.OriginalPosition}
 */
proto.speech.tts.lucid.LucidPiece.OriginalPosition.fromObject = function(obj) {
  var msg = new proto.speech.tts.lucid.LucidPiece.OriginalPosition();
  obj.contentIndex != null && jspb.Message.setField(msg, 1, obj.contentIndex);
  obj.pieceIndex != null && jspb.Message.setField(msg, 2, obj.pieceIndex);
  obj.byteIndex != null && jspb.Message.setField(msg, 3, obj.byteIndex);
  obj.timepointName != null && jspb.Message.setField(msg, 4, obj.timepointName);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.lucid.LucidPiece.OriginalPosition}
 */
proto.speech.tts.lucid.LucidPiece.OriginalPosition.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.lucid.LucidPiece.OriginalPosition;
  return proto.speech.tts.lucid.LucidPiece.OriginalPosition.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.lucid.LucidPiece.OriginalPosition} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.lucid.LucidPiece.OriginalPosition}
 */
proto.speech.tts.lucid.LucidPiece.OriginalPosition.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setContentIndex(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setPieceIndex(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setByteIndex(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setTimepointName(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.lucid.LucidPiece.OriginalPosition.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.lucid.LucidPiece.OriginalPosition.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.lucid.LucidPiece.OriginalPosition} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.LucidPiece.OriginalPosition.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?number} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeInt32(
      1,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeInt32(
      2,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeInt32(
      3,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeString(
      4,
      f
    );
  }
};


/**
 * optional int32 content_index = 1;
 * @return {?number}
 */
proto.speech.tts.lucid.LucidPiece.OriginalPosition.prototype.getContentIndex = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.lucid.LucidPiece.OriginalPosition} returns this
 */
proto.speech.tts.lucid.LucidPiece.OriginalPosition.prototype.setContentIndex = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.LucidPiece.OriginalPosition} returns this
 */
proto.speech.tts.lucid.LucidPiece.OriginalPosition.prototype.clearContentIndex = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.LucidPiece.OriginalPosition.prototype.hasContentIndex = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional int32 piece_index = 2;
 * @return {?number}
 */
proto.speech.tts.lucid.LucidPiece.OriginalPosition.prototype.getPieceIndex = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.lucid.LucidPiece.OriginalPosition} returns this
 */
proto.speech.tts.lucid.LucidPiece.OriginalPosition.prototype.setPieceIndex = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.LucidPiece.OriginalPosition} returns this
 */
proto.speech.tts.lucid.LucidPiece.OriginalPosition.prototype.clearPieceIndex = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.LucidPiece.OriginalPosition.prototype.hasPieceIndex = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional int32 byte_index = 3;
 * @return {?number}
 */
proto.speech.tts.lucid.LucidPiece.OriginalPosition.prototype.getByteIndex = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.lucid.LucidPiece.OriginalPosition} returns this
 */
proto.speech.tts.lucid.LucidPiece.OriginalPosition.prototype.setByteIndex = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.LucidPiece.OriginalPosition} returns this
 */
proto.speech.tts.lucid.LucidPiece.OriginalPosition.prototype.clearByteIndex = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.LucidPiece.OriginalPosition.prototype.hasByteIndex = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional string timepoint_name = 4;
 * @return {?string}
 */
proto.speech.tts.lucid.LucidPiece.OriginalPosition.prototype.getTimepointName = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 4));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.lucid.LucidPiece.OriginalPosition} returns this
 */
proto.speech.tts.lucid.LucidPiece.OriginalPosition.prototype.setTimepointName = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.LucidPiece.OriginalPosition} returns this
 */
proto.speech.tts.lucid.LucidPiece.OriginalPosition.prototype.clearTimepointName = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.LucidPiece.OriginalPosition.prototype.hasTimepointName = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.lucid.LucidPiece.OriginalPosition}
 */
proto.speech.tts.lucid.LucidPiece.OriginalPosition.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.lucid.LucidPiece.OriginalPosition, data);
};


/**
 * optional int32 own_index = 1;
 * @return {?number}
 */
proto.speech.tts.lucid.LucidPiece.prototype.getOwnIndex = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
 */
proto.speech.tts.lucid.LucidPiece.prototype.setOwnIndex = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
 */
proto.speech.tts.lucid.LucidPiece.prototype.clearOwnIndex = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.LucidPiece.prototype.hasOwnIndex = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional string input_text_connector = 3;
 * @return {?string}
 */
proto.speech.tts.lucid.LucidPiece.prototype.getInputTextConnector = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
 */
proto.speech.tts.lucid.LucidPiece.prototype.setInputTextConnector = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
 */
proto.speech.tts.lucid.LucidPiece.prototype.clearInputTextConnector = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.LucidPiece.prototype.hasInputTextConnector = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional string input_text = 2;
 * @return {?string}
 */
proto.speech.tts.lucid.LucidPiece.prototype.getInputText = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
 */
proto.speech.tts.lucid.LucidPiece.prototype.setInputText = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
 */
proto.speech.tts.lucid.LucidPiece.prototype.clearInputText = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.LucidPiece.prototype.hasInputText = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional Annotation annotation = 4;
 * @return {?proto.speech.tts.lucid.Annotation}
 */
proto.speech.tts.lucid.LucidPiece.prototype.getAnnotation = function() {
  return /** @type{?proto.speech.tts.lucid.Annotation} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.lucid.Annotation, 4));
};


/**
 * @param {?proto.speech.tts.lucid.Annotation|undefined} value
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
*/
proto.speech.tts.lucid.LucidPiece.prototype.setAnnotation = function(value) {
  return jspb.Message.setWrapperField(this, 4, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
 */
proto.speech.tts.lucid.LucidPiece.prototype.clearAnnotation = function() {
  return this.setAnnotation(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.LucidPiece.prototype.hasAnnotation = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * repeated LucidWord verbalized_words = 5;
 * @return {!Array<!proto.speech.tts.lucid.LucidWord>}
 */
proto.speech.tts.lucid.LucidPiece.prototype.getVerbalizedWordsList = function() {
  return /** @type{!Array<!proto.speech.tts.lucid.LucidWord>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.lucid.LucidWord, 5));
};


/**
 * @param {Array<!proto.speech.tts.lucid.LucidWord>|undefined} value
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
*/
proto.speech.tts.lucid.LucidPiece.prototype.setVerbalizedWordsList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 5, value);
};


/**
 * @param {!proto.speech.tts.lucid.LucidWord=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.lucid.LucidWord}
 */
proto.speech.tts.lucid.LucidPiece.prototype.addVerbalizedWords = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 5, opt_value, proto.speech.tts.lucid.LucidWord, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
 */
proto.speech.tts.lucid.LucidPiece.prototype.clearVerbalizedWordsList = function() {
  return this.setVerbalizedWordsList([]);
};


/**
 * optional string token_prefix = 6;
 * @return {?string}
 */
proto.speech.tts.lucid.LucidPiece.prototype.getTokenPrefix = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 6));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
 */
proto.speech.tts.lucid.LucidPiece.prototype.setTokenPrefix = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
 */
proto.speech.tts.lucid.LucidPiece.prototype.clearTokenPrefix = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.LucidPiece.prototype.hasTokenPrefix = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional string token_suffix = 7;
 * @return {?string}
 */
proto.speech.tts.lucid.LucidPiece.prototype.getTokenSuffix = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 7));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
 */
proto.speech.tts.lucid.LucidPiece.prototype.setTokenSuffix = function(value) {
  return jspb.Message.setField(this, 7, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
 */
proto.speech.tts.lucid.LucidPiece.prototype.clearTokenSuffix = function() {
  return jspb.Message.setField(this, 7, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.LucidPiece.prototype.hasTokenSuffix = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * optional OriginalPosition original_start_position = 8;
 * @return {?proto.speech.tts.lucid.LucidPiece.OriginalPosition}
 */
proto.speech.tts.lucid.LucidPiece.prototype.getOriginalStartPosition = function() {
  return /** @type{?proto.speech.tts.lucid.LucidPiece.OriginalPosition} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.lucid.LucidPiece.OriginalPosition, 8));
};


/**
 * @param {?proto.speech.tts.lucid.LucidPiece.OriginalPosition|undefined} value
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
*/
proto.speech.tts.lucid.LucidPiece.prototype.setOriginalStartPosition = function(value) {
  return jspb.Message.setWrapperField(this, 8, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
 */
proto.speech.tts.lucid.LucidPiece.prototype.clearOriginalStartPosition = function() {
  return this.setOriginalStartPosition(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.LucidPiece.prototype.hasOriginalStartPosition = function() {
  return jspb.Message.getField(this, 8) != null;
};


/**
 * optional string word = 100;
 * @return {?string}
 */
proto.speech.tts.lucid.LucidPiece.prototype.getWord = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 100));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
 */
proto.speech.tts.lucid.LucidPiece.prototype.setWord = function(value) {
  return jspb.Message.setOneofField(this, 100, proto.speech.tts.lucid.LucidPiece.oneofGroups_[0], value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
 */
proto.speech.tts.lucid.LucidPiece.prototype.clearWord = function() {
  return jspb.Message.setOneofField(this, 100, proto.speech.tts.lucid.LucidPiece.oneofGroups_[0], undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.LucidPiece.prototype.hasWord = function() {
  return jspb.Message.getField(this, 100) != null;
};


/**
 * optional PunctuationOrPause punctuation_or_pause = 101;
 * @return {?proto.speech.tts.lucid.PunctuationOrPause}
 */
proto.speech.tts.lucid.LucidPiece.prototype.getPunctuationOrPause = function() {
  return /** @type{?proto.speech.tts.lucid.PunctuationOrPause} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.lucid.PunctuationOrPause, 101));
};


/**
 * @param {?proto.speech.tts.lucid.PunctuationOrPause|undefined} value
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
*/
proto.speech.tts.lucid.LucidPiece.prototype.setPunctuationOrPause = function(value) {
  return jspb.Message.setOneofWrapperField(this, 101, proto.speech.tts.lucid.LucidPiece.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
 */
proto.speech.tts.lucid.LucidPiece.prototype.clearPunctuationOrPause = function() {
  return this.setPunctuationOrPause(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.LucidPiece.prototype.hasPunctuationOrPause = function() {
  return jspb.Message.getField(this, 101) != null;
};


/**
 * optional Verbatim verbatim = 102;
 * @return {?proto.speech.tts.lucid.Verbatim}
 */
proto.speech.tts.lucid.LucidPiece.prototype.getVerbatim = function() {
  return /** @type{?proto.speech.tts.lucid.Verbatim} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.lucid.Verbatim, 102));
};


/**
 * @param {?proto.speech.tts.lucid.Verbatim|undefined} value
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
*/
proto.speech.tts.lucid.LucidPiece.prototype.setVerbatim = function(value) {
  return jspb.Message.setOneofWrapperField(this, 102, proto.speech.tts.lucid.LucidPiece.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
 */
proto.speech.tts.lucid.LucidPiece.prototype.clearVerbatim = function() {
  return this.setVerbatim(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.LucidPiece.prototype.hasVerbatim = function() {
  return jspb.Message.getField(this, 102) != null;
};


/**
 * optional Spelling spelling = 103;
 * @return {?proto.speech.tts.lucid.Spelling}
 */
proto.speech.tts.lucid.LucidPiece.prototype.getSpelling = function() {
  return /** @type{?proto.speech.tts.lucid.Spelling} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.lucid.Spelling, 103));
};


/**
 * @param {?proto.speech.tts.lucid.Spelling|undefined} value
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
*/
proto.speech.tts.lucid.LucidPiece.prototype.setSpelling = function(value) {
  return jspb.Message.setOneofWrapperField(this, 103, proto.speech.tts.lucid.LucidPiece.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
 */
proto.speech.tts.lucid.LucidPiece.prototype.clearSpelling = function() {
  return this.setSpelling(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.LucidPiece.prototype.hasSpelling = function() {
  return jspb.Message.getField(this, 103) != null;
};


/**
 * optional LetterSequence letters = 104;
 * @return {?proto.speech.tts.lucid.LetterSequence}
 */
proto.speech.tts.lucid.LucidPiece.prototype.getLetters = function() {
  return /** @type{?proto.speech.tts.lucid.LetterSequence} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.lucid.LetterSequence, 104));
};


/**
 * @param {?proto.speech.tts.lucid.LetterSequence|undefined} value
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
*/
proto.speech.tts.lucid.LucidPiece.prototype.setLetters = function(value) {
  return jspb.Message.setOneofWrapperField(this, 104, proto.speech.tts.lucid.LucidPiece.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
 */
proto.speech.tts.lucid.LucidPiece.prototype.clearLetters = function() {
  return this.setLetters(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.LucidPiece.prototype.hasLetters = function() {
  return jspb.Message.getField(this, 104) != null;
};


/**
 * optional Abbreviation abbreviation = 105;
 * @return {?proto.speech.tts.lucid.Abbreviation}
 */
proto.speech.tts.lucid.LucidPiece.prototype.getAbbreviation = function() {
  return /** @type{?proto.speech.tts.lucid.Abbreviation} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.lucid.Abbreviation, 105));
};


/**
 * @param {?proto.speech.tts.lucid.Abbreviation|undefined} value
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
*/
proto.speech.tts.lucid.LucidPiece.prototype.setAbbreviation = function(value) {
  return jspb.Message.setOneofWrapperField(this, 105, proto.speech.tts.lucid.LucidPiece.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
 */
proto.speech.tts.lucid.LucidPiece.prototype.clearAbbreviation = function() {
  return this.setAbbreviation(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.LucidPiece.prototype.hasAbbreviation = function() {
  return jspb.Message.getField(this, 105) != null;
};


/**
 * optional RomanNumeral roman_numeral = 106;
 * @return {?proto.speech.tts.lucid.RomanNumeral}
 */
proto.speech.tts.lucid.LucidPiece.prototype.getRomanNumeral = function() {
  return /** @type{?proto.speech.tts.lucid.RomanNumeral} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.lucid.RomanNumeral, 106));
};


/**
 * @param {?proto.speech.tts.lucid.RomanNumeral|undefined} value
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
*/
proto.speech.tts.lucid.LucidPiece.prototype.setRomanNumeral = function(value) {
  return jspb.Message.setOneofWrapperField(this, 106, proto.speech.tts.lucid.LucidPiece.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
 */
proto.speech.tts.lucid.LucidPiece.prototype.clearRomanNumeral = function() {
  return this.setRomanNumeral(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.LucidPiece.prototype.hasRomanNumeral = function() {
  return jspb.Message.getField(this, 106) != null;
};


/**
 * optional Number number = 107;
 * @return {?proto.speech.tts.lucid.Number}
 */
proto.speech.tts.lucid.LucidPiece.prototype.getNumber = function() {
  return /** @type{?proto.speech.tts.lucid.Number} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.lucid.Number, 107));
};


/**
 * @param {?proto.speech.tts.lucid.Number|undefined} value
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
*/
proto.speech.tts.lucid.LucidPiece.prototype.setNumber = function(value) {
  return jspb.Message.setOneofWrapperField(this, 107, proto.speech.tts.lucid.LucidPiece.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
 */
proto.speech.tts.lucid.LucidPiece.prototype.clearNumber = function() {
  return this.setNumber(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.LucidPiece.prototype.hasNumber = function() {
  return jspb.Message.getField(this, 107) != null;
};


/**
 * optional Ordinal ordinal = 108;
 * @return {?proto.speech.tts.lucid.Ordinal}
 */
proto.speech.tts.lucid.LucidPiece.prototype.getOrdinal = function() {
  return /** @type{?proto.speech.tts.lucid.Ordinal} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.lucid.Ordinal, 108));
};


/**
 * @param {?proto.speech.tts.lucid.Ordinal|undefined} value
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
*/
proto.speech.tts.lucid.LucidPiece.prototype.setOrdinal = function(value) {
  return jspb.Message.setOneofWrapperField(this, 108, proto.speech.tts.lucid.LucidPiece.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
 */
proto.speech.tts.lucid.LucidPiece.prototype.clearOrdinal = function() {
  return this.setOrdinal(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.LucidPiece.prototype.hasOrdinal = function() {
  return jspb.Message.getField(this, 108) != null;
};


/**
 * optional Fraction fraction = 109;
 * @return {?proto.speech.tts.lucid.Fraction}
 */
proto.speech.tts.lucid.LucidPiece.prototype.getFraction = function() {
  return /** @type{?proto.speech.tts.lucid.Fraction} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.lucid.Fraction, 109));
};


/**
 * @param {?proto.speech.tts.lucid.Fraction|undefined} value
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
*/
proto.speech.tts.lucid.LucidPiece.prototype.setFraction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 109, proto.speech.tts.lucid.LucidPiece.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
 */
proto.speech.tts.lucid.LucidPiece.prototype.clearFraction = function() {
  return this.setFraction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.LucidPiece.prototype.hasFraction = function() {
  return jspb.Message.getField(this, 109) != null;
};


/**
 * optional Digits digits = 110;
 * @return {?proto.speech.tts.lucid.Digits}
 */
proto.speech.tts.lucid.LucidPiece.prototype.getDigits = function() {
  return /** @type{?proto.speech.tts.lucid.Digits} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.lucid.Digits, 110));
};


/**
 * @param {?proto.speech.tts.lucid.Digits|undefined} value
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
*/
proto.speech.tts.lucid.LucidPiece.prototype.setDigits = function(value) {
  return jspb.Message.setOneofWrapperField(this, 110, proto.speech.tts.lucid.LucidPiece.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
 */
proto.speech.tts.lucid.LucidPiece.prototype.clearDigits = function() {
  return this.setDigits(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.LucidPiece.prototype.hasDigits = function() {
  return jspb.Message.getField(this, 110) != null;
};


/**
 * optional Time time = 111;
 * @return {?proto.speech.tts.lucid.Time}
 */
proto.speech.tts.lucid.LucidPiece.prototype.getTime = function() {
  return /** @type{?proto.speech.tts.lucid.Time} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.lucid.Time, 111));
};


/**
 * @param {?proto.speech.tts.lucid.Time|undefined} value
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
*/
proto.speech.tts.lucid.LucidPiece.prototype.setTime = function(value) {
  return jspb.Message.setOneofWrapperField(this, 111, proto.speech.tts.lucid.LucidPiece.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
 */
proto.speech.tts.lucid.LucidPiece.prototype.clearTime = function() {
  return this.setTime(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.LucidPiece.prototype.hasTime = function() {
  return jspb.Message.getField(this, 111) != null;
};


/**
 * optional Duration duration = 112;
 * @return {?proto.speech.tts.lucid.Duration}
 */
proto.speech.tts.lucid.LucidPiece.prototype.getDuration = function() {
  return /** @type{?proto.speech.tts.lucid.Duration} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.lucid.Duration, 112));
};


/**
 * @param {?proto.speech.tts.lucid.Duration|undefined} value
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
*/
proto.speech.tts.lucid.LucidPiece.prototype.setDuration = function(value) {
  return jspb.Message.setOneofWrapperField(this, 112, proto.speech.tts.lucid.LucidPiece.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
 */
proto.speech.tts.lucid.LucidPiece.prototype.clearDuration = function() {
  return this.setDuration(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.LucidPiece.prototype.hasDuration = function() {
  return jspb.Message.getField(this, 112) != null;
};


/**
 * optional Measure measure = 113;
 * @return {?proto.speech.tts.lucid.Measure}
 */
proto.speech.tts.lucid.LucidPiece.prototype.getMeasure = function() {
  return /** @type{?proto.speech.tts.lucid.Measure} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.lucid.Measure, 113));
};


/**
 * @param {?proto.speech.tts.lucid.Measure|undefined} value
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
*/
proto.speech.tts.lucid.LucidPiece.prototype.setMeasure = function(value) {
  return jspb.Message.setOneofWrapperField(this, 113, proto.speech.tts.lucid.LucidPiece.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
 */
proto.speech.tts.lucid.LucidPiece.prototype.clearMeasure = function() {
  return this.setMeasure(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.LucidPiece.prototype.hasMeasure = function() {
  return jspb.Message.getField(this, 113) != null;
};


/**
 * optional Date date = 114;
 * @return {?proto.speech.tts.lucid.Date}
 */
proto.speech.tts.lucid.LucidPiece.prototype.getDate = function() {
  return /** @type{?proto.speech.tts.lucid.Date} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.lucid.Date, 114));
};


/**
 * @param {?proto.speech.tts.lucid.Date|undefined} value
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
*/
proto.speech.tts.lucid.LucidPiece.prototype.setDate = function(value) {
  return jspb.Message.setOneofWrapperField(this, 114, proto.speech.tts.lucid.LucidPiece.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
 */
proto.speech.tts.lucid.LucidPiece.prototype.clearDate = function() {
  return this.setDate(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.LucidPiece.prototype.hasDate = function() {
  return jspb.Message.getField(this, 114) != null;
};


/**
 * optional Telephone telephone = 115;
 * @return {?proto.speech.tts.lucid.Telephone}
 */
proto.speech.tts.lucid.LucidPiece.prototype.getTelephone = function() {
  return /** @type{?proto.speech.tts.lucid.Telephone} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.lucid.Telephone, 115));
};


/**
 * @param {?proto.speech.tts.lucid.Telephone|undefined} value
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
*/
proto.speech.tts.lucid.LucidPiece.prototype.setTelephone = function(value) {
  return jspb.Message.setOneofWrapperField(this, 115, proto.speech.tts.lucid.LucidPiece.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
 */
proto.speech.tts.lucid.LucidPiece.prototype.clearTelephone = function() {
  return this.setTelephone(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.LucidPiece.prototype.hasTelephone = function() {
  return jspb.Message.getField(this, 115) != null;
};


/**
 * optional Money money = 116;
 * @return {?proto.speech.tts.lucid.Money}
 */
proto.speech.tts.lucid.LucidPiece.prototype.getMoney = function() {
  return /** @type{?proto.speech.tts.lucid.Money} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.lucid.Money, 116));
};


/**
 * @param {?proto.speech.tts.lucid.Money|undefined} value
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
*/
proto.speech.tts.lucid.LucidPiece.prototype.setMoney = function(value) {
  return jspb.Message.setOneofWrapperField(this, 116, proto.speech.tts.lucid.LucidPiece.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
 */
proto.speech.tts.lucid.LucidPiece.prototype.clearMoney = function() {
  return this.setMoney(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.LucidPiece.prototype.hasMoney = function() {
  return jspb.Message.getField(this, 116) != null;
};


/**
 * optional Electronic electronic = 117;
 * @return {?proto.speech.tts.lucid.Electronic}
 */
proto.speech.tts.lucid.LucidPiece.prototype.getElectronic = function() {
  return /** @type{?proto.speech.tts.lucid.Electronic} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.lucid.Electronic, 117));
};


/**
 * @param {?proto.speech.tts.lucid.Electronic|undefined} value
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
*/
proto.speech.tts.lucid.LucidPiece.prototype.setElectronic = function(value) {
  return jspb.Message.setOneofWrapperField(this, 117, proto.speech.tts.lucid.LucidPiece.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
 */
proto.speech.tts.lucid.LucidPiece.prototype.clearElectronic = function() {
  return this.setElectronic(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.LucidPiece.prototype.hasElectronic = function() {
  return jspb.Message.getField(this, 117) != null;
};


/**
 * optional EmoticonEmoji emoticon_emoji = 118;
 * @return {?proto.speech.tts.lucid.EmoticonEmoji}
 */
proto.speech.tts.lucid.LucidPiece.prototype.getEmoticonEmoji = function() {
  return /** @type{?proto.speech.tts.lucid.EmoticonEmoji} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.lucid.EmoticonEmoji, 118));
};


/**
 * @param {?proto.speech.tts.lucid.EmoticonEmoji|undefined} value
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
*/
proto.speech.tts.lucid.LucidPiece.prototype.setEmoticonEmoji = function(value) {
  return jspb.Message.setOneofWrapperField(this, 118, proto.speech.tts.lucid.LucidPiece.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
 */
proto.speech.tts.lucid.LucidPiece.prototype.clearEmoticonEmoji = function() {
  return this.setEmoticonEmoji(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.LucidPiece.prototype.hasEmoticonEmoji = function() {
  return jspb.Message.getField(this, 118) != null;
};


/**
 * optional ChemicalFormula chemical_formula = 119;
 * @return {?proto.speech.tts.lucid.ChemicalFormula}
 */
proto.speech.tts.lucid.LucidPiece.prototype.getChemicalFormula = function() {
  return /** @type{?proto.speech.tts.lucid.ChemicalFormula} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.lucid.ChemicalFormula, 119));
};


/**
 * @param {?proto.speech.tts.lucid.ChemicalFormula|undefined} value
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
*/
proto.speech.tts.lucid.LucidPiece.prototype.setChemicalFormula = function(value) {
  return jspb.Message.setOneofWrapperField(this, 119, proto.speech.tts.lucid.LucidPiece.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
 */
proto.speech.tts.lucid.LucidPiece.prototype.clearChemicalFormula = function() {
  return this.setChemicalFormula(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.LucidPiece.prototype.hasChemicalFormula = function() {
  return jspb.Message.getField(this, 119) != null;
};


/**
 * optional Connector connector = 120;
 * @return {?proto.speech.tts.lucid.Connector}
 */
proto.speech.tts.lucid.LucidPiece.prototype.getConnector = function() {
  return /** @type{?proto.speech.tts.lucid.Connector} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.lucid.Connector, 120));
};


/**
 * @param {?proto.speech.tts.lucid.Connector|undefined} value
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
*/
proto.speech.tts.lucid.LucidPiece.prototype.setConnector = function(value) {
  return jspb.Message.setOneofWrapperField(this, 120, proto.speech.tts.lucid.LucidPiece.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.lucid.LucidPiece} returns this
 */
proto.speech.tts.lucid.LucidPiece.prototype.clearConnector = function() {
  return this.setConnector(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.LucidPiece.prototype.hasConnector = function() {
  return jspb.Message.getField(this, 120) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.lucid.LucidPiece}
 */
proto.speech.tts.lucid.LucidPiece.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.lucid.LucidPiece, data);
};



//speech/tts/public/lucid/lucid_markup_proto_jspb/lucidutterance.jspb.js
// source: speech/tts/public/lucid/lucid_markup.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.lucid.LucidUtterance');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.lucid.Annotation');
goog.require('proto.speech.tts.lucid.LucidPiece');
goog.require('proto.speech.tts.lucid.SpanAnnotation');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.lucid.LucidUtterance = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 4, proto.speech.tts.lucid.LucidUtterance.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.lucid.LucidUtterance, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.lucid.LucidUtterance.displayName = 'proto.speech.tts.lucid.LucidUtterance';
}

/**
 * The extensions registered with this message class. This is a map of
 * extension field number to fieldInfo object.
 *
 * For example:
 *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }
 *
 * fieldName contains the JsCompiler renamed field name property so that it
 * works in OPTIMIZED mode.
 *
 * @type {!Object<number, jspb.ExtensionFieldInfo>}
 */
proto.speech.tts.lucid.LucidUtterance.extensions = {};


/**
 * The extensions registered with this message class. This is a map of
 * extension field number to fieldInfo object.
 *
 * For example:
 *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }
 *
 * fieldName contains the JsCompiler renamed field name property so that it
 * works in OPTIMIZED mode.
 *
 * @type {!Object<number, jspb.ExtensionFieldBinaryInfo>}
 */
proto.speech.tts.lucid.LucidUtterance.extensionsBinary = {};


/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.lucid.LucidUtterance.repeatedFields_ = [1,3];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.lucid.LucidUtterance.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.lucid.LucidUtterance.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.lucid.LucidUtterance} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.LucidUtterance.toObject = function(includeInstance, msg) {
  var f, obj = {
    piecesList: jspb.Message.toObjectList(msg.getPiecesList(),
    proto.speech.tts.lucid.LucidPiece.toObject, includeInstance),
    annotation: (f = msg.getAnnotation()) && proto.speech.tts.lucid.Annotation.toObject(includeInstance, f),
    spanAnnotationsList: jspb.Message.toObjectList(msg.getSpanAnnotationsList(),
    proto.speech.tts.lucid.SpanAnnotation.toObject, includeInstance)
  };

  jspb.Message.toObjectExtension(/** @type {!jspb.Message} */ (msg), obj,
      proto.speech.tts.lucid.LucidUtterance.extensions, proto.speech.tts.lucid.LucidUtterance.prototype.getExtension,
      includeInstance);
  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of LucidUtterance as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.lucid.LucidUtterance.ObjectFormat = function() {
  /** @type {?Array<!proto.speech.tts.lucid.LucidPiece.ObjectFormat>|undefined} */
  this.piecesList;

  /** @type {?proto.speech.tts.lucid.Annotation.ObjectFormat|undefined} */
  this.annotation;

  /** @type {?Array<!proto.speech.tts.lucid.SpanAnnotation.ObjectFormat>|undefined} */
  this.spanAnnotationsList;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.lucid.LucidUtterance.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.lucid.LucidUtterance}
 */
proto.speech.tts.lucid.LucidUtterance.fromObject = function(obj) {
  var msg = new proto.speech.tts.lucid.LucidUtterance();
  obj.piecesList && jspb.Message.setRepeatedWrapperField(
      msg, 1, obj.piecesList.map(
          proto.speech.tts.lucid.LucidPiece.fromObject));
  obj.annotation && jspb.Message.setWrapperField(
      msg, 2, proto.speech.tts.lucid.Annotation.fromObject(obj.annotation));
  obj.spanAnnotationsList && jspb.Message.setRepeatedWrapperField(
      msg, 3, obj.spanAnnotationsList.map(
          proto.speech.tts.lucid.SpanAnnotation.fromObject));
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.lucid.LucidUtterance}
 */
proto.speech.tts.lucid.LucidUtterance.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.lucid.LucidUtterance;
  return proto.speech.tts.lucid.LucidUtterance.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.lucid.LucidUtterance} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.lucid.LucidUtterance}
 */
proto.speech.tts.lucid.LucidUtterance.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.speech.tts.lucid.LucidPiece;
      reader.readMessage(value,proto.speech.tts.lucid.LucidPiece.deserializeBinaryFromReader);
      msg.addPieces(value);
      break;
    case 2:
      var value = new proto.speech.tts.lucid.Annotation;
      reader.readMessage(value,proto.speech.tts.lucid.Annotation.deserializeBinaryFromReader);
      msg.setAnnotation(value);
      break;
    case 3:
      var value = new proto.speech.tts.lucid.SpanAnnotation;
      reader.readMessage(value,proto.speech.tts.lucid.SpanAnnotation.deserializeBinaryFromReader);
      msg.addSpanAnnotations(value);
      break;
    default:
      jspb.Message.readBinaryExtension(msg, reader,
        proto.speech.tts.lucid.LucidUtterance.extensionsBinary,
        proto.speech.tts.lucid.LucidUtterance.prototype.getExtension,
        proto.speech.tts.lucid.LucidUtterance.prototype.setExtension);
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.lucid.LucidUtterance.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.lucid.LucidUtterance.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.lucid.LucidUtterance} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.LucidUtterance.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getPiecesList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      1,
      f,
      proto.speech.tts.lucid.LucidPiece.serializeBinaryToWriter
    );
  }
  f = message.getAnnotation();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.speech.tts.lucid.Annotation.serializeBinaryToWriter
    );
  }
  f = message.getSpanAnnotationsList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      3,
      f,
      proto.speech.tts.lucid.SpanAnnotation.serializeBinaryToWriter
    );
  }
  jspb.Message.serializeBinaryExtensions(message, writer,
    proto.speech.tts.lucid.LucidUtterance.extensionsBinary, proto.speech.tts.lucid.LucidUtterance.prototype.getExtension);
};


/**
 * repeated LucidPiece pieces = 1;
 * @return {!Array<!proto.speech.tts.lucid.LucidPiece>}
 */
proto.speech.tts.lucid.LucidUtterance.prototype.getPiecesList = function() {
  return /** @type{!Array<!proto.speech.tts.lucid.LucidPiece>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.lucid.LucidPiece, 1));
};


/**
 * @param {Array<!proto.speech.tts.lucid.LucidPiece>|undefined} value
 * @return {!proto.speech.tts.lucid.LucidUtterance} returns this
*/
proto.speech.tts.lucid.LucidUtterance.prototype.setPiecesList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 1, value);
};


/**
 * @param {!proto.speech.tts.lucid.LucidPiece=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.lucid.LucidPiece}
 */
proto.speech.tts.lucid.LucidUtterance.prototype.addPieces = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.speech.tts.lucid.LucidPiece, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.lucid.LucidUtterance} returns this
 */
proto.speech.tts.lucid.LucidUtterance.prototype.clearPiecesList = function() {
  return this.setPiecesList([]);
};


/**
 * optional Annotation annotation = 2;
 * @return {?proto.speech.tts.lucid.Annotation}
 */
proto.speech.tts.lucid.LucidUtterance.prototype.getAnnotation = function() {
  return /** @type{?proto.speech.tts.lucid.Annotation} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.lucid.Annotation, 2));
};


/**
 * @param {?proto.speech.tts.lucid.Annotation|undefined} value
 * @return {!proto.speech.tts.lucid.LucidUtterance} returns this
*/
proto.speech.tts.lucid.LucidUtterance.prototype.setAnnotation = function(value) {
  return jspb.Message.setWrapperField(this, 2, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.lucid.LucidUtterance} returns this
 */
proto.speech.tts.lucid.LucidUtterance.prototype.clearAnnotation = function() {
  return this.setAnnotation(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.LucidUtterance.prototype.hasAnnotation = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * repeated SpanAnnotation span_annotations = 3;
 * @return {!Array<!proto.speech.tts.lucid.SpanAnnotation>}
 */
proto.speech.tts.lucid.LucidUtterance.prototype.getSpanAnnotationsList = function() {
  return /** @type{!Array<!proto.speech.tts.lucid.SpanAnnotation>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.lucid.SpanAnnotation, 3));
};


/**
 * @param {Array<!proto.speech.tts.lucid.SpanAnnotation>|undefined} value
 * @return {!proto.speech.tts.lucid.LucidUtterance} returns this
*/
proto.speech.tts.lucid.LucidUtterance.prototype.setSpanAnnotationsList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 3, value);
};


/**
 * @param {!proto.speech.tts.lucid.SpanAnnotation=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.lucid.SpanAnnotation}
 */
proto.speech.tts.lucid.LucidUtterance.prototype.addSpanAnnotations = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 3, opt_value, proto.speech.tts.lucid.SpanAnnotation, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.lucid.LucidUtterance} returns this
 */
proto.speech.tts.lucid.LucidUtterance.prototype.clearSpanAnnotationsList = function() {
  return this.setSpanAnnotationsList([]);
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.lucid.LucidUtterance}
 */
proto.speech.tts.lucid.LucidUtterance.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.lucid.LucidUtterance, data);
};



//speech/tts/public/lucid/lucid_markup_proto_jspb/multisentencetext.jspb.js
// source: speech/tts/public/lucid/lucid_markup.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.lucid.MultiSentenceText');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.lucid.Annotation');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.lucid.MultiSentenceText = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 3, null, null);
};
goog.inherits(proto.speech.tts.lucid.MultiSentenceText, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.lucid.MultiSentenceText.displayName = 'proto.speech.tts.lucid.MultiSentenceText';
}

/**
 * The extensions registered with this message class. This is a map of
 * extension field number to fieldInfo object.
 *
 * For example:
 *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }
 *
 * fieldName contains the JsCompiler renamed field name property so that it
 * works in OPTIMIZED mode.
 *
 * @type {!Object<number, jspb.ExtensionFieldInfo>}
 */
proto.speech.tts.lucid.MultiSentenceText.extensions = {};


/**
 * The extensions registered with this message class. This is a map of
 * extension field number to fieldInfo object.
 *
 * For example:
 *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }
 *
 * fieldName contains the JsCompiler renamed field name property so that it
 * works in OPTIMIZED mode.
 *
 * @type {!Object<number, jspb.ExtensionFieldBinaryInfo>}
 */
proto.speech.tts.lucid.MultiSentenceText.extensionsBinary = {};




if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.lucid.MultiSentenceText.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.lucid.MultiSentenceText.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.lucid.MultiSentenceText} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.MultiSentenceText.toObject = function(includeInstance, msg) {
  var f, obj = {
    text: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    annotation: (f = msg.getAnnotation()) && proto.speech.tts.lucid.Annotation.toObject(includeInstance, f)
  };

  jspb.Message.toObjectExtension(/** @type {!jspb.Message} */ (msg), obj,
      proto.speech.tts.lucid.MultiSentenceText.extensions, proto.speech.tts.lucid.MultiSentenceText.prototype.getExtension,
      includeInstance);
  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of MultiSentenceText as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.lucid.MultiSentenceText.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.text;

  /** @type {?proto.speech.tts.lucid.Annotation.ObjectFormat|undefined} */
  this.annotation;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.lucid.MultiSentenceText.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.lucid.MultiSentenceText}
 */
proto.speech.tts.lucid.MultiSentenceText.fromObject = function(obj) {
  var msg = new proto.speech.tts.lucid.MultiSentenceText();
  obj.text != null && jspb.Message.setField(msg, 1, obj.text);
  obj.annotation && jspb.Message.setWrapperField(
      msg, 2, proto.speech.tts.lucid.Annotation.fromObject(obj.annotation));
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.lucid.MultiSentenceText}
 */
proto.speech.tts.lucid.MultiSentenceText.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.lucid.MultiSentenceText;
  return proto.speech.tts.lucid.MultiSentenceText.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.lucid.MultiSentenceText} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.lucid.MultiSentenceText}
 */
proto.speech.tts.lucid.MultiSentenceText.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setText(value);
      break;
    case 2:
      var value = new proto.speech.tts.lucid.Annotation;
      reader.readMessage(value,proto.speech.tts.lucid.Annotation.deserializeBinaryFromReader);
      msg.setAnnotation(value);
      break;
    default:
      jspb.Message.readBinaryExtension(msg, reader,
        proto.speech.tts.lucid.MultiSentenceText.extensionsBinary,
        proto.speech.tts.lucid.MultiSentenceText.prototype.getExtension,
        proto.speech.tts.lucid.MultiSentenceText.prototype.setExtension);
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.lucid.MultiSentenceText.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.lucid.MultiSentenceText.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.lucid.MultiSentenceText} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.MultiSentenceText.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = message.getAnnotation();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.speech.tts.lucid.Annotation.serializeBinaryToWriter
    );
  }
  jspb.Message.serializeBinaryExtensions(message, writer,
    proto.speech.tts.lucid.MultiSentenceText.extensionsBinary, proto.speech.tts.lucid.MultiSentenceText.prototype.getExtension);
};


/**
 * optional string text = 1;
 * @return {?string}
 */
proto.speech.tts.lucid.MultiSentenceText.prototype.getText = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.lucid.MultiSentenceText} returns this
 */
proto.speech.tts.lucid.MultiSentenceText.prototype.setText = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.MultiSentenceText} returns this
 */
proto.speech.tts.lucid.MultiSentenceText.prototype.clearText = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.MultiSentenceText.prototype.hasText = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional Annotation annotation = 2;
 * @return {?proto.speech.tts.lucid.Annotation}
 */
proto.speech.tts.lucid.MultiSentenceText.prototype.getAnnotation = function() {
  return /** @type{?proto.speech.tts.lucid.Annotation} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.lucid.Annotation, 2));
};


/**
 * @param {?proto.speech.tts.lucid.Annotation|undefined} value
 * @return {!proto.speech.tts.lucid.MultiSentenceText} returns this
*/
proto.speech.tts.lucid.MultiSentenceText.prototype.setAnnotation = function(value) {
  return jspb.Message.setWrapperField(this, 2, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.lucid.MultiSentenceText} returns this
 */
proto.speech.tts.lucid.MultiSentenceText.prototype.clearAnnotation = function() {
  return this.setAnnotation(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.MultiSentenceText.prototype.hasAnnotation = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.lucid.MultiSentenceText}
 */
proto.speech.tts.lucid.MultiSentenceText.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.lucid.MultiSentenceText, data);
};



//speech/tts/engine/textnorm/textnorm_diagnostics_proto_jspb/lucidstatediagnostics.jspb.js
// source: speech/tts/engine/textnorm/textnorm_diagnostics.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.LucidStateDiagnostics');
goog.provide('proto.speech.tts.LucidStateDiagnostics.DiagnosticCase');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.lucid.LucidUtterance');
goog.require('proto.speech.tts.lucid.MultiSentenceText');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.LucidStateDiagnostics = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, proto.speech.tts.LucidStateDiagnostics.oneofGroups_);
};
goog.inherits(proto.speech.tts.LucidStateDiagnostics, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.LucidStateDiagnostics.displayName = 'proto.speech.tts.LucidStateDiagnostics';
}

/**
 * Oneof group definitions for this message. Each group defines the field
 * numbers belonging to that group. When of these fields' value is set, all
 * other fields in the group are cleared. During deserialization, if multiple
 * fields are encountered for a group, only the last value seen will be kept.
 * @private {!Array<!Array<number>>}
 * @const
 */
proto.speech.tts.LucidStateDiagnostics.oneofGroups_ = [[3,4]];

/**
 * @enum {number}
 */
proto.speech.tts.LucidStateDiagnostics.DiagnosticCase = {
  DIAGNOSTIC_NOT_SET: 0,
  UTTERANCE: 3,
  MULTI_SENTENCE: 4
};

/**
 * @return {!proto.speech.tts.LucidStateDiagnostics.DiagnosticCase}
 */
proto.speech.tts.LucidStateDiagnostics.prototype.getDiagnosticCase = function() {
  return /** @type {!proto.speech.tts.LucidStateDiagnostics.DiagnosticCase} */(jspb.Message.computeOneofCase(this, proto.speech.tts.LucidStateDiagnostics.oneofGroups_[0]));
};



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.LucidStateDiagnostics.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.LucidStateDiagnostics.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.LucidStateDiagnostics} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.LucidStateDiagnostics.toObject = function(includeInstance, msg) {
  var f, obj = {
    stage: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    utterance: (f = msg.getUtterance()) && proto.speech.tts.lucid.LucidUtterance.toObject(includeInstance, f),
    multiSentence: (f = msg.getMultiSentence()) && proto.speech.tts.lucid.MultiSentenceText.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of LucidStateDiagnostics as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.LucidStateDiagnostics.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.stage;

  /** @type {?proto.speech.tts.lucid.LucidUtterance.ObjectFormat|undefined} */
  this.utterance;

  /** @type {?proto.speech.tts.lucid.MultiSentenceText.ObjectFormat|undefined} */
  this.multiSentence;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.LucidStateDiagnostics.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.LucidStateDiagnostics}
 */
proto.speech.tts.LucidStateDiagnostics.fromObject = function(obj) {
  var msg = new proto.speech.tts.LucidStateDiagnostics();
  obj.stage != null && jspb.Message.setField(msg, 2, obj.stage);
  obj.utterance && jspb.Message.setWrapperField(
      msg, 3, proto.speech.tts.lucid.LucidUtterance.fromObject(obj.utterance));
  obj.multiSentence && jspb.Message.setWrapperField(
      msg, 4, proto.speech.tts.lucid.MultiSentenceText.fromObject(obj.multiSentence));
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.LucidStateDiagnostics}
 */
proto.speech.tts.LucidStateDiagnostics.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.LucidStateDiagnostics;
  return proto.speech.tts.LucidStateDiagnostics.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.LucidStateDiagnostics} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.LucidStateDiagnostics}
 */
proto.speech.tts.LucidStateDiagnostics.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setStage(value);
      break;
    case 3:
      var value = new proto.speech.tts.lucid.LucidUtterance;
      reader.readMessage(value,proto.speech.tts.lucid.LucidUtterance.deserializeBinaryFromReader);
      msg.setUtterance(value);
      break;
    case 4:
      var value = new proto.speech.tts.lucid.MultiSentenceText;
      reader.readMessage(value,proto.speech.tts.lucid.MultiSentenceText.deserializeBinaryFromReader);
      msg.setMultiSentence(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.LucidStateDiagnostics.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.LucidStateDiagnostics.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.LucidStateDiagnostics} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.LucidStateDiagnostics.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
  f = message.getUtterance();
  if (f != null) {
    writer.writeMessage(
      3,
      f,
      proto.speech.tts.lucid.LucidUtterance.serializeBinaryToWriter
    );
  }
  f = message.getMultiSentence();
  if (f != null) {
    writer.writeMessage(
      4,
      f,
      proto.speech.tts.lucid.MultiSentenceText.serializeBinaryToWriter
    );
  }
};


/**
 * optional string stage = 2;
 * @return {?string}
 */
proto.speech.tts.LucidStateDiagnostics.prototype.getStage = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.LucidStateDiagnostics} returns this
 */
proto.speech.tts.LucidStateDiagnostics.prototype.setStage = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.LucidStateDiagnostics} returns this
 */
proto.speech.tts.LucidStateDiagnostics.prototype.clearStage = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.LucidStateDiagnostics.prototype.hasStage = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional lucid.LucidUtterance utterance = 3;
 * @return {?proto.speech.tts.lucid.LucidUtterance}
 */
proto.speech.tts.LucidStateDiagnostics.prototype.getUtterance = function() {
  return /** @type{?proto.speech.tts.lucid.LucidUtterance} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.lucid.LucidUtterance, 3));
};


/**
 * @param {?proto.speech.tts.lucid.LucidUtterance|undefined} value
 * @return {!proto.speech.tts.LucidStateDiagnostics} returns this
*/
proto.speech.tts.LucidStateDiagnostics.prototype.setUtterance = function(value) {
  return jspb.Message.setOneofWrapperField(this, 3, proto.speech.tts.LucidStateDiagnostics.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.LucidStateDiagnostics} returns this
 */
proto.speech.tts.LucidStateDiagnostics.prototype.clearUtterance = function() {
  return this.setUtterance(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.LucidStateDiagnostics.prototype.hasUtterance = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional lucid.MultiSentenceText multi_sentence = 4;
 * @return {?proto.speech.tts.lucid.MultiSentenceText}
 */
proto.speech.tts.LucidStateDiagnostics.prototype.getMultiSentence = function() {
  return /** @type{?proto.speech.tts.lucid.MultiSentenceText} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.lucid.MultiSentenceText, 4));
};


/**
 * @param {?proto.speech.tts.lucid.MultiSentenceText|undefined} value
 * @return {!proto.speech.tts.LucidStateDiagnostics} returns this
*/
proto.speech.tts.LucidStateDiagnostics.prototype.setMultiSentence = function(value) {
  return jspb.Message.setOneofWrapperField(this, 4, proto.speech.tts.LucidStateDiagnostics.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.LucidStateDiagnostics} returns this
 */
proto.speech.tts.LucidStateDiagnostics.prototype.clearMultiSentence = function() {
  return this.setMultiSentence(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.LucidStateDiagnostics.prototype.hasMultiSentence = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.LucidStateDiagnostics}
 */
proto.speech.tts.LucidStateDiagnostics.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.LucidStateDiagnostics, data);
};



//speech/tts/engine/utterance/links_proto_jspb/links.jspb.js
// source: speech/tts/engine/utterance/links.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.Links');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.Links = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.Links, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.Links.displayName = 'proto.speech.tts.Links';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.Links.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.Links.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.Links} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Links.toObject = function(includeInstance, msg) {
  var f, obj = {
    ownIndex: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    parent: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    firstChild: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,
    lastChild: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Links as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.Links.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.ownIndex;

  /** @type {?number|undefined} */
  this.parent;

  /** @type {?number|undefined} */
  this.firstChild;

  /** @type {?number|undefined} */
  this.lastChild;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.Links.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.Links}
 */
proto.speech.tts.Links.fromObject = function(obj) {
  var msg = new proto.speech.tts.Links();
  obj.ownIndex != null && jspb.Message.setField(msg, 1, obj.ownIndex);
  obj.parent != null && jspb.Message.setField(msg, 2, obj.parent);
  obj.firstChild != null && jspb.Message.setField(msg, 3, obj.firstChild);
  obj.lastChild != null && jspb.Message.setField(msg, 4, obj.lastChild);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.Links}
 */
proto.speech.tts.Links.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.Links;
  return proto.speech.tts.Links.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.Links} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.Links}
 */
proto.speech.tts.Links.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setOwnIndex(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setParent(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setFirstChild(value);
      break;
    case 4:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setLastChild(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.Links.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.Links.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.Links} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Links.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?number} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeInt32(
      1,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeInt32(
      2,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeInt32(
      3,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeInt32(
      4,
      f
    );
  }
};


/**
 * optional int32 own_index = 1;
 * @return {?number}
 */
proto.speech.tts.Links.prototype.getOwnIndex = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Links} returns this
 */
proto.speech.tts.Links.prototype.setOwnIndex = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Links} returns this
 */
proto.speech.tts.Links.prototype.clearOwnIndex = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Links.prototype.hasOwnIndex = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional int32 parent = 2;
 * @return {?number}
 */
proto.speech.tts.Links.prototype.getParent = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Links} returns this
 */
proto.speech.tts.Links.prototype.setParent = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Links} returns this
 */
proto.speech.tts.Links.prototype.clearParent = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Links.prototype.hasParent = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional int32 first_child = 3;
 * @return {?number}
 */
proto.speech.tts.Links.prototype.getFirstChild = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Links} returns this
 */
proto.speech.tts.Links.prototype.setFirstChild = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Links} returns this
 */
proto.speech.tts.Links.prototype.clearFirstChild = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Links.prototype.hasFirstChild = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional int32 last_child = 4;
 * @return {?number}
 */
proto.speech.tts.Links.prototype.getLastChild = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 4));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Links} returns this
 */
proto.speech.tts.Links.prototype.setLastChild = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Links} returns this
 */
proto.speech.tts.Links.prototype.clearLastChild = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Links.prototype.hasLastChild = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.Links}
 */
proto.speech.tts.Links.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.Links, data);
};



//speech/tts/engine/utterance/linguistic_structure_proto_jspb/phoneme.jspb.js
// source: speech/tts/engine/utterance/phoneme.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.Phoneme');
goog.provide('proto.speech.tts.Phoneme.AlignerStatistics');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.Links');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.Phoneme = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 10, proto.speech.tts.Phoneme.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.Phoneme, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.Phoneme.displayName = 'proto.speech.tts.Phoneme';
}

/**
 * The extensions registered with this message class. This is a map of
 * extension field number to fieldInfo object.
 *
 * For example:
 *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }
 *
 * fieldName contains the JsCompiler renamed field name property so that it
 * works in OPTIMIZED mode.
 *
 * @type {!Object<number, jspb.ExtensionFieldInfo>}
 */
proto.speech.tts.Phoneme.extensions = {};


/**
 * The extensions registered with this message class. This is a map of
 * extension field number to fieldInfo object.
 *
 * For example:
 *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }
 *
 * fieldName contains the JsCompiler renamed field name property so that it
 * works in OPTIMIZED mode.
 *
 * @type {!Object<number, jspb.ExtensionFieldBinaryInfo>}
 */
proto.speech.tts.Phoneme.extensionsBinary = {};

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.Phoneme.AlignerStatistics = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.Phoneme.AlignerStatistics, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.Phoneme.AlignerStatistics.displayName = 'proto.speech.tts.Phoneme.AlignerStatistics';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.Phoneme.repeatedFields_ = [7];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.Phoneme.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.Phoneme.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.Phoneme} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Phoneme.toObject = function(includeInstance, msg) {
  var f, obj = {
    links: (f = msg.getLinks()) && proto.speech.tts.Links.toObject(includeInstance, f),
    name: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    vowel: (f = jspb.Message.getBooleanField(msg, 3)) == null ? undefined : f,
    start: (f = jspb.Message.getOptionalFloatingPointField(msg, 4)) == null ? undefined : f,
    end: (f = jspb.Message.getOptionalFloatingPointField(msg, 5)) == null ? undefined : f,
    firstFrame: (f = jspb.Message.getField(msg, 8)) == null ? undefined : f,
    lastFrame: (f = jspb.Message.getField(msg, 9)) == null ? undefined : f,
    alignerStats: (f = msg.getAlignerStats()) && proto.speech.tts.Phoneme.AlignerStatistics.toObject(includeInstance, f),
    alignerStateBoundariesList: (f = jspb.Message.getRepeatedFloatingPointField(msg, 7)) == null ? undefined : f
  };

  jspb.Message.toObjectExtension(/** @type {!jspb.Message} */ (msg), obj,
      proto.speech.tts.Phoneme.extensions, proto.speech.tts.Phoneme.prototype.getExtension,
      includeInstance);
  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Phoneme as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.Phoneme.ObjectFormat = function() {
  /** @type {?proto.speech.tts.Links.ObjectFormat|undefined} */
  this.links;

  /** @type {?string|undefined} */
  this.name;

  /** @type {?boolean|undefined} */
  this.vowel;

  /** @type {?number|undefined} */
  this.start;

  /** @type {?number|undefined} */
  this.end;

  /** @type {?number|undefined} */
  this.firstFrame;

  /** @type {?number|undefined} */
  this.lastFrame;

  /** @type {?proto.speech.tts.Phoneme.AlignerStatistics.ObjectFormat|undefined} */
  this.alignerStats;

  /** @type {?Array<number>|undefined} */
  this.alignerStateBoundariesList;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.Phoneme.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.Phoneme}
 */
proto.speech.tts.Phoneme.fromObject = function(obj) {
  var msg = new proto.speech.tts.Phoneme();
  obj.links && jspb.Message.setWrapperField(
      msg, 1, proto.speech.tts.Links.fromObject(obj.links));
  obj.name != null && jspb.Message.setField(msg, 2, obj.name);
  obj.vowel != null && jspb.Message.setField(msg, 3, obj.vowel);
  obj.start != null && jspb.Message.setField(msg, 4, obj.start);
  obj.end != null && jspb.Message.setField(msg, 5, obj.end);
  obj.firstFrame != null && jspb.Message.setField(msg, 8, obj.firstFrame);
  obj.lastFrame != null && jspb.Message.setField(msg, 9, obj.lastFrame);
  obj.alignerStats && jspb.Message.setWrapperField(
      msg, 6, proto.speech.tts.Phoneme.AlignerStatistics.fromObject(obj.alignerStats));
  obj.alignerStateBoundariesList != null && jspb.Message.setField(msg, 7, obj.alignerStateBoundariesList);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.Phoneme}
 */
proto.speech.tts.Phoneme.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.Phoneme;
  return proto.speech.tts.Phoneme.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.Phoneme} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.Phoneme}
 */
proto.speech.tts.Phoneme.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.speech.tts.Links;
      reader.readMessage(value,proto.speech.tts.Links.deserializeBinaryFromReader);
      msg.setLinks(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setName(value);
      break;
    case 3:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setVowel(value);
      break;
    case 4:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setStart(value);
      break;
    case 5:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setEnd(value);
      break;
    case 8:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setFirstFrame(value);
      break;
    case 9:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setLastFrame(value);
      break;
    case 6:
      var value = new proto.speech.tts.Phoneme.AlignerStatistics;
      reader.readMessage(value,proto.speech.tts.Phoneme.AlignerStatistics.deserializeBinaryFromReader);
      msg.setAlignerStats(value);
      break;
    case 7:
      var value = /** @type {number} */ (reader.readFloat());
      msg.addAlignerStateBoundaries(value);
      break;
    default:
      jspb.Message.readBinaryExtension(msg, reader,
        proto.speech.tts.Phoneme.extensionsBinary,
        proto.speech.tts.Phoneme.prototype.getExtension,
        proto.speech.tts.Phoneme.prototype.setExtension);
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.Phoneme.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.Phoneme.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.Phoneme} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Phoneme.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getLinks();
  if (f != null) {
    writer.writeMessage(
      1,
      f,
      proto.speech.tts.Links.serializeBinaryToWriter
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeBool(
      3,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeFloat(
      4,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeFloat(
      5,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 8));
  if (f != null) {
    writer.writeInt32(
      8,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 9));
  if (f != null) {
    writer.writeInt32(
      9,
      f
    );
  }
  f = message.getAlignerStats();
  if (f != null) {
    writer.writeMessage(
      6,
      f,
      proto.speech.tts.Phoneme.AlignerStatistics.serializeBinaryToWriter
    );
  }
  f = message.getAlignerStateBoundariesList();
  if (f.length > 0) {
    writer.writeRepeatedFloat(
      7,
      f
    );
  }
  jspb.Message.serializeBinaryExtensions(message, writer,
    proto.speech.tts.Phoneme.extensionsBinary, proto.speech.tts.Phoneme.prototype.getExtension);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.Phoneme.AlignerStatistics.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.Phoneme.AlignerStatistics.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.Phoneme.AlignerStatistics} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Phoneme.AlignerStatistics.toObject = function(includeInstance, msg) {
  var f, obj = {
    absZDuration: (f = jspb.Message.getOptionalFloatingPointField(msg, 1)) == null ? undefined : f,
    zScore: (f = jspb.Message.getOptionalFloatingPointField(msg, 2)) == null ? undefined : f,
    phoneScore: (f = jspb.Message.getOptionalFloatingPointField(msg, 3)) == null ? undefined : f,
    cutTime: (f = jspb.Message.getOptionalFloatingPointField(msg, 4)) == null ? undefined : f,
    score: (f = jspb.Message.getOptionalFloatingPointField(msg, 5)) == null ? undefined : f,
    maxFrameEnergy: (f = jspb.Message.getOptionalFloatingPointField(msg, 6)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of AlignerStatistics as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.Phoneme.AlignerStatistics.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.absZDuration;

  /** @type {?number|undefined} */
  this.zScore;

  /** @type {?number|undefined} */
  this.phoneScore;

  /** @type {?number|undefined} */
  this.cutTime;

  /** @type {?number|undefined} */
  this.score;

  /** @type {?number|undefined} */
  this.maxFrameEnergy;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.Phoneme.AlignerStatistics.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.Phoneme.AlignerStatistics}
 */
proto.speech.tts.Phoneme.AlignerStatistics.fromObject = function(obj) {
  var msg = new proto.speech.tts.Phoneme.AlignerStatistics();
  obj.absZDuration != null && jspb.Message.setField(msg, 1, obj.absZDuration);
  obj.zScore != null && jspb.Message.setField(msg, 2, obj.zScore);
  obj.phoneScore != null && jspb.Message.setField(msg, 3, obj.phoneScore);
  obj.cutTime != null && jspb.Message.setField(msg, 4, obj.cutTime);
  obj.score != null && jspb.Message.setField(msg, 5, obj.score);
  obj.maxFrameEnergy != null && jspb.Message.setField(msg, 6, obj.maxFrameEnergy);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.Phoneme.AlignerStatistics}
 */
proto.speech.tts.Phoneme.AlignerStatistics.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.Phoneme.AlignerStatistics;
  return proto.speech.tts.Phoneme.AlignerStatistics.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.Phoneme.AlignerStatistics} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.Phoneme.AlignerStatistics}
 */
proto.speech.tts.Phoneme.AlignerStatistics.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setAbsZDuration(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setZScore(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setPhoneScore(value);
      break;
    case 4:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setCutTime(value);
      break;
    case 5:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setScore(value);
      break;
    case 6:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setMaxFrameEnergy(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.Phoneme.AlignerStatistics.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.Phoneme.AlignerStatistics.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.Phoneme.AlignerStatistics} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Phoneme.AlignerStatistics.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?number} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeFloat(
      1,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeFloat(
      2,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeFloat(
      3,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeFloat(
      4,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeFloat(
      5,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeFloat(
      6,
      f
    );
  }
};


/**
 * optional float abs_z_duration = 1;
 * @return {?number}
 */
proto.speech.tts.Phoneme.AlignerStatistics.prototype.getAbsZDuration = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 1));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Phoneme.AlignerStatistics} returns this
 */
proto.speech.tts.Phoneme.AlignerStatistics.prototype.setAbsZDuration = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Phoneme.AlignerStatistics} returns this
 */
proto.speech.tts.Phoneme.AlignerStatistics.prototype.clearAbsZDuration = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Phoneme.AlignerStatistics.prototype.hasAbsZDuration = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional float z_score = 2;
 * @return {?number}
 */
proto.speech.tts.Phoneme.AlignerStatistics.prototype.getZScore = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 2));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Phoneme.AlignerStatistics} returns this
 */
proto.speech.tts.Phoneme.AlignerStatistics.prototype.setZScore = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Phoneme.AlignerStatistics} returns this
 */
proto.speech.tts.Phoneme.AlignerStatistics.prototype.clearZScore = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Phoneme.AlignerStatistics.prototype.hasZScore = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional float phone_score = 3;
 * @return {?number}
 */
proto.speech.tts.Phoneme.AlignerStatistics.prototype.getPhoneScore = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 3));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Phoneme.AlignerStatistics} returns this
 */
proto.speech.tts.Phoneme.AlignerStatistics.prototype.setPhoneScore = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Phoneme.AlignerStatistics} returns this
 */
proto.speech.tts.Phoneme.AlignerStatistics.prototype.clearPhoneScore = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Phoneme.AlignerStatistics.prototype.hasPhoneScore = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional float cut_time = 4;
 * @return {?number}
 */
proto.speech.tts.Phoneme.AlignerStatistics.prototype.getCutTime = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 4));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Phoneme.AlignerStatistics} returns this
 */
proto.speech.tts.Phoneme.AlignerStatistics.prototype.setCutTime = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Phoneme.AlignerStatistics} returns this
 */
proto.speech.tts.Phoneme.AlignerStatistics.prototype.clearCutTime = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Phoneme.AlignerStatistics.prototype.hasCutTime = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional float score = 5;
 * @return {?number}
 */
proto.speech.tts.Phoneme.AlignerStatistics.prototype.getScore = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 5));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Phoneme.AlignerStatistics} returns this
 */
proto.speech.tts.Phoneme.AlignerStatistics.prototype.setScore = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Phoneme.AlignerStatistics} returns this
 */
proto.speech.tts.Phoneme.AlignerStatistics.prototype.clearScore = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Phoneme.AlignerStatistics.prototype.hasScore = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional float max_frame_energy = 6;
 * @return {?number}
 */
proto.speech.tts.Phoneme.AlignerStatistics.prototype.getMaxFrameEnergy = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 6));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Phoneme.AlignerStatistics} returns this
 */
proto.speech.tts.Phoneme.AlignerStatistics.prototype.setMaxFrameEnergy = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Phoneme.AlignerStatistics} returns this
 */
proto.speech.tts.Phoneme.AlignerStatistics.prototype.clearMaxFrameEnergy = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Phoneme.AlignerStatistics.prototype.hasMaxFrameEnergy = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.Phoneme.AlignerStatistics}
 */
proto.speech.tts.Phoneme.AlignerStatistics.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.Phoneme.AlignerStatistics, data);
};


/**
 * optional Links links = 1;
 * @return {?proto.speech.tts.Links}
 */
proto.speech.tts.Phoneme.prototype.getLinks = function() {
  return /** @type{?proto.speech.tts.Links} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Links, 1));
};


/**
 * @param {?proto.speech.tts.Links|undefined} value
 * @return {!proto.speech.tts.Phoneme} returns this
*/
proto.speech.tts.Phoneme.prototype.setLinks = function(value) {
  return jspb.Message.setWrapperField(this, 1, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Phoneme} returns this
 */
proto.speech.tts.Phoneme.prototype.clearLinks = function() {
  return this.setLinks(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Phoneme.prototype.hasLinks = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional string name = 2;
 * @return {?string}
 */
proto.speech.tts.Phoneme.prototype.getName = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Phoneme} returns this
 */
proto.speech.tts.Phoneme.prototype.setName = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Phoneme} returns this
 */
proto.speech.tts.Phoneme.prototype.clearName = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Phoneme.prototype.hasName = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional bool vowel = 3;
 * @return {?boolean}
 */
proto.speech.tts.Phoneme.prototype.getVowel = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 3));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.Phoneme} returns this
 */
proto.speech.tts.Phoneme.prototype.setVowel = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Phoneme} returns this
 */
proto.speech.tts.Phoneme.prototype.clearVowel = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Phoneme.prototype.hasVowel = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional float start = 4;
 * @return {?number}
 */
proto.speech.tts.Phoneme.prototype.getStart = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 4));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Phoneme} returns this
 */
proto.speech.tts.Phoneme.prototype.setStart = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Phoneme} returns this
 */
proto.speech.tts.Phoneme.prototype.clearStart = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Phoneme.prototype.hasStart = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional float end = 5;
 * @return {?number}
 */
proto.speech.tts.Phoneme.prototype.getEnd = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 5));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Phoneme} returns this
 */
proto.speech.tts.Phoneme.prototype.setEnd = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Phoneme} returns this
 */
proto.speech.tts.Phoneme.prototype.clearEnd = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Phoneme.prototype.hasEnd = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional int32 first_frame = 8;
 * @return {?number}
 */
proto.speech.tts.Phoneme.prototype.getFirstFrame = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 8));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Phoneme} returns this
 */
proto.speech.tts.Phoneme.prototype.setFirstFrame = function(value) {
  return jspb.Message.setField(this, 8, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Phoneme} returns this
 */
proto.speech.tts.Phoneme.prototype.clearFirstFrame = function() {
  return jspb.Message.setField(this, 8, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Phoneme.prototype.hasFirstFrame = function() {
  return jspb.Message.getField(this, 8) != null;
};


/**
 * optional int32 last_frame = 9;
 * @return {?number}
 */
proto.speech.tts.Phoneme.prototype.getLastFrame = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 9));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Phoneme} returns this
 */
proto.speech.tts.Phoneme.prototype.setLastFrame = function(value) {
  return jspb.Message.setField(this, 9, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Phoneme} returns this
 */
proto.speech.tts.Phoneme.prototype.clearLastFrame = function() {
  return jspb.Message.setField(this, 9, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Phoneme.prototype.hasLastFrame = function() {
  return jspb.Message.getField(this, 9) != null;
};


/**
 * optional AlignerStatistics aligner_stats = 6;
 * @return {?proto.speech.tts.Phoneme.AlignerStatistics}
 */
proto.speech.tts.Phoneme.prototype.getAlignerStats = function() {
  return /** @type{?proto.speech.tts.Phoneme.AlignerStatistics} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Phoneme.AlignerStatistics, 6));
};


/**
 * @param {?proto.speech.tts.Phoneme.AlignerStatistics|undefined} value
 * @return {!proto.speech.tts.Phoneme} returns this
*/
proto.speech.tts.Phoneme.prototype.setAlignerStats = function(value) {
  return jspb.Message.setWrapperField(this, 6, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Phoneme} returns this
 */
proto.speech.tts.Phoneme.prototype.clearAlignerStats = function() {
  return this.setAlignerStats(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Phoneme.prototype.hasAlignerStats = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * repeated float aligner_state_boundaries = 7;
 * @return {!Array<number>}
 */
proto.speech.tts.Phoneme.prototype.getAlignerStateBoundariesList = function() {
  return /** @type {!Array<number>} */ (jspb.Message.getRepeatedFloatingPointField(this, 7));
};


/**
 * @param {Array<number>|undefined} value
 * @return {!proto.speech.tts.Phoneme} returns this
 */
proto.speech.tts.Phoneme.prototype.setAlignerStateBoundariesList = function(value) {
  return jspb.Message.setField(this, 7, value || []);
};


/**
 * @param {number} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.Phoneme} returns this
 */
proto.speech.tts.Phoneme.prototype.addAlignerStateBoundaries = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 7, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.Phoneme} returns this
 */
proto.speech.tts.Phoneme.prototype.clearAlignerStateBoundariesList = function() {
  return this.setAlignerStateBoundariesList([]);
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.Phoneme}
 */
proto.speech.tts.Phoneme.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.Phoneme, data);
};



//speech/tts/engine/utterance/linguistic_structure_proto_jspb/syllable.jspb.js
// source: speech/tts/engine/utterance/syllable.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.Syllable');
goog.provide('proto.speech.tts.Syllable.ContourDirection');
goog.provide('proto.speech.tts.Syllable.ContourExtreme');
goog.provide('proto.speech.tts.Syllable.ContourStart');
goog.provide('proto.speech.tts.Syllable.ContourValues');
goog.provide('proto.speech.tts.Syllable.ContourValues.Position');
goog.provide('proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.Links');

goog.forwardDeclare('proto.speech.tts.Stress');
goog.forwardDeclare('proto.speech.tts.Tone');
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.Syllable = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 10, proto.speech.tts.Syllable.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.Syllable, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.Syllable.displayName = 'proto.speech.tts.Syllable';
}

/**
 * The extensions registered with this message class. This is a map of
 * extension field number to fieldInfo object.
 *
 * For example:
 *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }
 *
 * fieldName contains the JsCompiler renamed field name property so that it
 * works in OPTIMIZED mode.
 *
 * @type {!Object<number, jspb.ExtensionFieldInfo>}
 */
proto.speech.tts.Syllable.extensions = {};


/**
 * The extensions registered with this message class. This is a map of
 * extension field number to fieldInfo object.
 *
 * For example:
 *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }
 *
 * fieldName contains the JsCompiler renamed field name property so that it
 * works in OPTIMIZED mode.
 *
 * @type {!Object<number, jspb.ExtensionFieldBinaryInfo>}
 */
proto.speech.tts.Syllable.extensionsBinary = {};

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.Syllable.ContourValues = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.Syllable.ContourValues, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.Syllable.ContourValues.displayName = 'proto.speech.tts.Syllable.ContourValues';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures.displayName = 'proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.Syllable.repeatedFields_ = [8];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.Syllable.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.Syllable.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.Syllable} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Syllable.toObject = function(includeInstance, msg) {
  var f, obj = {
    links: (f = msg.getLinks()) && proto.speech.tts.Links.toObject(includeInstance, f),
    name: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    stress: jspb.Message.getFieldWithDefault(msg, 3, -1),
    tone: jspb.Message.getFieldWithDefault(msg, 4, -1),
    contourStart: (f = jspb.Message.getField(msg, 5)) == null ? undefined : f,
    contourDirection: (f = jspb.Message.getField(msg, 6)) == null ? undefined : f,
    contourExtreme: (f = jspb.Message.getField(msg, 7)) == null ? undefined : f,
    contoursList: jspb.Message.toObjectList(msg.getContoursList(),
    proto.speech.tts.Syllable.ContourValues.toObject, includeInstance),
    japaneseLinguisticFeatures: (f = msg.getJapaneseLinguisticFeatures()) && proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures.toObject(includeInstance, f)
  };

  jspb.Message.toObjectExtension(/** @type {!jspb.Message} */ (msg), obj,
      proto.speech.tts.Syllable.extensions, proto.speech.tts.Syllable.prototype.getExtension,
      includeInstance);
  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Syllable as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.Syllable.ObjectFormat = function() {
  /** @type {?proto.speech.tts.Links.ObjectFormat|undefined} */
  this.links;

  /** @type {?string|undefined} */
  this.name;

  /** @type {?number|undefined} */
  this.stress;

  /** @type {?number|undefined} */
  this.tone;

  /** @type {?number|undefined} */
  this.contourStart;

  /** @type {?number|undefined} */
  this.contourDirection;

  /** @type {?number|undefined} */
  this.contourExtreme;

  /** @type {?Array<!proto.speech.tts.Syllable.ContourValues.ObjectFormat>|undefined} */
  this.contoursList;

  /** @type {?proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures.ObjectFormat|undefined} */
  this.japaneseLinguisticFeatures;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.Syllable.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.Syllable}
 */
proto.speech.tts.Syllable.fromObject = function(obj) {
  var msg = new proto.speech.tts.Syllable();
  obj.links && jspb.Message.setWrapperField(
      msg, 1, proto.speech.tts.Links.fromObject(obj.links));
  obj.name != null && jspb.Message.setField(msg, 2, obj.name);
  obj.stress != null && jspb.Message.setField(msg, 3, obj.stress);
  obj.tone != null && jspb.Message.setField(msg, 4, obj.tone);
  obj.contourStart != null && jspb.Message.setField(msg, 5, obj.contourStart);
  obj.contourDirection != null && jspb.Message.setField(msg, 6, obj.contourDirection);
  obj.contourExtreme != null && jspb.Message.setField(msg, 7, obj.contourExtreme);
  obj.contoursList && jspb.Message.setRepeatedWrapperField(
      msg, 8, obj.contoursList.map(
          proto.speech.tts.Syllable.ContourValues.fromObject));
  obj.japaneseLinguisticFeatures && jspb.Message.setWrapperField(
      msg, 9, proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures.fromObject(obj.japaneseLinguisticFeatures));
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.Syllable}
 */
proto.speech.tts.Syllable.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.Syllable;
  return proto.speech.tts.Syllable.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.Syllable} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.Syllable}
 */
proto.speech.tts.Syllable.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.speech.tts.Links;
      reader.readMessage(value,proto.speech.tts.Links.deserializeBinaryFromReader);
      msg.setLinks(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setName(value);
      break;
    case 3:
      var value = /** @type {!proto.speech.tts.Stress} */ (reader.readEnum());
      msg.setStress(value);
      break;
    case 4:
      var value = /** @type {!proto.speech.tts.Tone} */ (reader.readEnum());
      msg.setTone(value);
      break;
    case 5:
      var value = /** @type {!proto.speech.tts.Syllable.ContourStart} */ (reader.readEnum());
      msg.setContourStart(value);
      break;
    case 6:
      var value = /** @type {!proto.speech.tts.Syllable.ContourDirection} */ (reader.readEnum());
      msg.setContourDirection(value);
      break;
    case 7:
      var value = /** @type {!proto.speech.tts.Syllable.ContourExtreme} */ (reader.readEnum());
      msg.setContourExtreme(value);
      break;
    case 8:
      var value = new proto.speech.tts.Syllable.ContourValues;
      reader.readMessage(value,proto.speech.tts.Syllable.ContourValues.deserializeBinaryFromReader);
      msg.addContours(value);
      break;
    case 9:
      var value = new proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures;
      reader.readMessage(value,proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures.deserializeBinaryFromReader);
      msg.setJapaneseLinguisticFeatures(value);
      break;
    default:
      jspb.Message.readBinaryExtension(msg, reader,
        proto.speech.tts.Syllable.extensionsBinary,
        proto.speech.tts.Syllable.prototype.getExtension,
        proto.speech.tts.Syllable.prototype.setExtension);
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.Syllable.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.Syllable.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.Syllable} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Syllable.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getLinks();
  if (f != null) {
    writer.writeMessage(
      1,
      f,
      proto.speech.tts.Links.serializeBinaryToWriter
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
  f = /** @type {!proto.speech.tts.Stress} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeEnum(
      3,
      f
    );
  }
  f = /** @type {!proto.speech.tts.Tone} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeEnum(
      4,
      f
    );
  }
  f = /** @type {!proto.speech.tts.Syllable.ContourStart} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeEnum(
      5,
      f
    );
  }
  f = /** @type {!proto.speech.tts.Syllable.ContourDirection} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeEnum(
      6,
      f
    );
  }
  f = /** @type {!proto.speech.tts.Syllable.ContourExtreme} */ (jspb.Message.getField(message, 7));
  if (f != null) {
    writer.writeEnum(
      7,
      f
    );
  }
  f = message.getContoursList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      8,
      f,
      proto.speech.tts.Syllable.ContourValues.serializeBinaryToWriter
    );
  }
  f = message.getJapaneseLinguisticFeatures();
  if (f != null) {
    writer.writeMessage(
      9,
      f,
      proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures.serializeBinaryToWriter
    );
  }
  jspb.Message.serializeBinaryExtensions(message, writer,
    proto.speech.tts.Syllable.extensionsBinary, proto.speech.tts.Syllable.prototype.getExtension);
};


/**
 * @enum {number}
 */
proto.speech.tts.Syllable.ContourStart = {
  UNVOICED_START: 0,
  LOW: 1,
  MEDIUM: 2,
  HIGH: 3
};

/**
 * @enum {number}
 */
proto.speech.tts.Syllable.ContourDirection = {
  UNVOICED_DIRECTION: 0,
  DOWN: 1,
  STRAIGHT: 2,
  UP: 3
};

/**
 * @enum {number}
 */
proto.speech.tts.Syllable.ContourExtreme = {
  UNVOICED_EXTREME: 0,
  POSITIVE: 1,
  NOEXTREME: 2,
  NEGATIVE: 3
};




if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.Syllable.ContourValues.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.Syllable.ContourValues.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.Syllable.ContourValues} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Syllable.ContourValues.toObject = function(includeInstance, msg) {
  var f, obj = {
    position: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    value: (f = jspb.Message.getOptionalFloatingPointField(msg, 2)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of ContourValues as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.Syllable.ContourValues.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.position;

  /** @type {?number|undefined} */
  this.value;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.Syllable.ContourValues.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.Syllable.ContourValues}
 */
proto.speech.tts.Syllable.ContourValues.fromObject = function(obj) {
  var msg = new proto.speech.tts.Syllable.ContourValues();
  obj.position != null && jspb.Message.setField(msg, 1, obj.position);
  obj.value != null && jspb.Message.setField(msg, 2, obj.value);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.Syllable.ContourValues}
 */
proto.speech.tts.Syllable.ContourValues.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.Syllable.ContourValues;
  return proto.speech.tts.Syllable.ContourValues.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.Syllable.ContourValues} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.Syllable.ContourValues}
 */
proto.speech.tts.Syllable.ContourValues.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {!proto.speech.tts.Syllable.ContourValues.Position} */ (reader.readEnum());
      msg.setPosition(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setValue(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.Syllable.ContourValues.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.Syllable.ContourValues.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.Syllable.ContourValues} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Syllable.ContourValues.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {!proto.speech.tts.Syllable.ContourValues.Position} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeEnum(
      1,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeFloat(
      2,
      f
    );
  }
};


/**
 * @enum {number}
 */
proto.speech.tts.Syllable.ContourValues.Position = {
  START: 0,
  END: 1,
  EXTREME_VALUE: 2,
  EXTREME_POSITION: 3
};

/**
 * optional Position position = 1;
 * @return {!proto.speech.tts.Syllable.ContourValues.Position}
 */
proto.speech.tts.Syllable.ContourValues.prototype.getPosition = function() {
  return /** @type {!proto.speech.tts.Syllable.ContourValues.Position} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?proto.speech.tts.Syllable.ContourValues.Position|undefined} value
 * @return {!proto.speech.tts.Syllable.ContourValues} returns this
 */
proto.speech.tts.Syllable.ContourValues.prototype.setPosition = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Syllable.ContourValues} returns this
 */
proto.speech.tts.Syllable.ContourValues.prototype.clearPosition = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Syllable.ContourValues.prototype.hasPosition = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional float value = 2;
 * @return {?number}
 */
proto.speech.tts.Syllable.ContourValues.prototype.getValue = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 2));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Syllable.ContourValues} returns this
 */
proto.speech.tts.Syllable.ContourValues.prototype.setValue = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Syllable.ContourValues} returns this
 */
proto.speech.tts.Syllable.ContourValues.prototype.clearValue = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Syllable.ContourValues.prototype.hasValue = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.Syllable.ContourValues}
 */
proto.speech.tts.Syllable.ContourValues.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.Syllable.ContourValues, data);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures.toObject = function(includeInstance, msg) {
  var f, obj = {
    numberOfPrecedingMoraeInPitchAccentPhrase: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    numberOfSucceedingMoraeInPitchAccentPhrase: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    numberOfPrecedingMoraeToPitchAccentNuclear: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,
    numberOfSucceedingMoraeToPitchAccentNuclear: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of JapaneseSyllableLinguisticFeatures as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.numberOfPrecedingMoraeInPitchAccentPhrase;

  /** @type {?number|undefined} */
  this.numberOfSucceedingMoraeInPitchAccentPhrase;

  /** @type {?number|undefined} */
  this.numberOfPrecedingMoraeToPitchAccentNuclear;

  /** @type {?number|undefined} */
  this.numberOfSucceedingMoraeToPitchAccentNuclear;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures}
 */
proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures.fromObject = function(obj) {
  var msg = new proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures();
  obj.numberOfPrecedingMoraeInPitchAccentPhrase != null && jspb.Message.setField(msg, 1, obj.numberOfPrecedingMoraeInPitchAccentPhrase);
  obj.numberOfSucceedingMoraeInPitchAccentPhrase != null && jspb.Message.setField(msg, 2, obj.numberOfSucceedingMoraeInPitchAccentPhrase);
  obj.numberOfPrecedingMoraeToPitchAccentNuclear != null && jspb.Message.setField(msg, 3, obj.numberOfPrecedingMoraeToPitchAccentNuclear);
  obj.numberOfSucceedingMoraeToPitchAccentNuclear != null && jspb.Message.setField(msg, 4, obj.numberOfSucceedingMoraeToPitchAccentNuclear);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures}
 */
proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures;
  return proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures}
 */
proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setNumberOfPrecedingMoraeInPitchAccentPhrase(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setNumberOfSucceedingMoraeInPitchAccentPhrase(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setNumberOfPrecedingMoraeToPitchAccentNuclear(value);
      break;
    case 4:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setNumberOfSucceedingMoraeToPitchAccentNuclear(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?number} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeInt32(
      1,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeInt32(
      2,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeInt32(
      3,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeInt32(
      4,
      f
    );
  }
};


/**
 * optional int32 number_of_preceding_morae_in_pitch_accent_phrase = 1;
 * @return {?number}
 */
proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures.prototype.getNumberOfPrecedingMoraeInPitchAccentPhrase = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures} returns this
 */
proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures.prototype.setNumberOfPrecedingMoraeInPitchAccentPhrase = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures} returns this
 */
proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures.prototype.clearNumberOfPrecedingMoraeInPitchAccentPhrase = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures.prototype.hasNumberOfPrecedingMoraeInPitchAccentPhrase = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional int32 number_of_succeeding_morae_in_pitch_accent_phrase = 2;
 * @return {?number}
 */
proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures.prototype.getNumberOfSucceedingMoraeInPitchAccentPhrase = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures} returns this
 */
proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures.prototype.setNumberOfSucceedingMoraeInPitchAccentPhrase = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures} returns this
 */
proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures.prototype.clearNumberOfSucceedingMoraeInPitchAccentPhrase = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures.prototype.hasNumberOfSucceedingMoraeInPitchAccentPhrase = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional int32 number_of_preceding_morae_to_pitch_accent_nuclear = 3;
 * @return {?number}
 */
proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures.prototype.getNumberOfPrecedingMoraeToPitchAccentNuclear = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures} returns this
 */
proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures.prototype.setNumberOfPrecedingMoraeToPitchAccentNuclear = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures} returns this
 */
proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures.prototype.clearNumberOfPrecedingMoraeToPitchAccentNuclear = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures.prototype.hasNumberOfPrecedingMoraeToPitchAccentNuclear = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional int32 number_of_succeeding_morae_to_pitch_accent_nuclear = 4;
 * @return {?number}
 */
proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures.prototype.getNumberOfSucceedingMoraeToPitchAccentNuclear = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 4));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures} returns this
 */
proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures.prototype.setNumberOfSucceedingMoraeToPitchAccentNuclear = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures} returns this
 */
proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures.prototype.clearNumberOfSucceedingMoraeToPitchAccentNuclear = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures.prototype.hasNumberOfSucceedingMoraeToPitchAccentNuclear = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures}
 */
proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures, data);
};


/**
 * optional Links links = 1;
 * @return {?proto.speech.tts.Links}
 */
proto.speech.tts.Syllable.prototype.getLinks = function() {
  return /** @type{?proto.speech.tts.Links} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Links, 1));
};


/**
 * @param {?proto.speech.tts.Links|undefined} value
 * @return {!proto.speech.tts.Syllable} returns this
*/
proto.speech.tts.Syllable.prototype.setLinks = function(value) {
  return jspb.Message.setWrapperField(this, 1, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Syllable} returns this
 */
proto.speech.tts.Syllable.prototype.clearLinks = function() {
  return this.setLinks(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Syllable.prototype.hasLinks = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional string name = 2;
 * @return {?string}
 */
proto.speech.tts.Syllable.prototype.getName = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Syllable} returns this
 */
proto.speech.tts.Syllable.prototype.setName = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Syllable} returns this
 */
proto.speech.tts.Syllable.prototype.clearName = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Syllable.prototype.hasName = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional Stress stress = 3;
 * @return {!proto.speech.tts.Stress}
 */
proto.speech.tts.Syllable.prototype.getStress = function() {
  return /** @type {!proto.speech.tts.Stress} */ (jspb.Message.getFieldWithDefault(this, 3, -1));
};


/**
 * @param {?proto.speech.tts.Stress|undefined} value
 * @return {!proto.speech.tts.Syllable} returns this
 */
proto.speech.tts.Syllable.prototype.setStress = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Syllable} returns this
 */
proto.speech.tts.Syllable.prototype.clearStress = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Syllable.prototype.hasStress = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional Tone tone = 4;
 * @return {!proto.speech.tts.Tone}
 */
proto.speech.tts.Syllable.prototype.getTone = function() {
  return /** @type {!proto.speech.tts.Tone} */ (jspb.Message.getFieldWithDefault(this, 4, -1));
};


/**
 * @param {?proto.speech.tts.Tone|undefined} value
 * @return {!proto.speech.tts.Syllable} returns this
 */
proto.speech.tts.Syllable.prototype.setTone = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Syllable} returns this
 */
proto.speech.tts.Syllable.prototype.clearTone = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Syllable.prototype.hasTone = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional ContourStart contour_start = 5;
 * @return {!proto.speech.tts.Syllable.ContourStart}
 */
proto.speech.tts.Syllable.prototype.getContourStart = function() {
  return /** @type {!proto.speech.tts.Syllable.ContourStart} */ (jspb.Message.getField(this, 5));
};


/**
 * @param {?proto.speech.tts.Syllable.ContourStart|undefined} value
 * @return {!proto.speech.tts.Syllable} returns this
 */
proto.speech.tts.Syllable.prototype.setContourStart = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Syllable} returns this
 */
proto.speech.tts.Syllable.prototype.clearContourStart = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Syllable.prototype.hasContourStart = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional ContourDirection contour_direction = 6;
 * @return {!proto.speech.tts.Syllable.ContourDirection}
 */
proto.speech.tts.Syllable.prototype.getContourDirection = function() {
  return /** @type {!proto.speech.tts.Syllable.ContourDirection} */ (jspb.Message.getField(this, 6));
};


/**
 * @param {?proto.speech.tts.Syllable.ContourDirection|undefined} value
 * @return {!proto.speech.tts.Syllable} returns this
 */
proto.speech.tts.Syllable.prototype.setContourDirection = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Syllable} returns this
 */
proto.speech.tts.Syllable.prototype.clearContourDirection = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Syllable.prototype.hasContourDirection = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional ContourExtreme contour_extreme = 7;
 * @return {!proto.speech.tts.Syllable.ContourExtreme}
 */
proto.speech.tts.Syllable.prototype.getContourExtreme = function() {
  return /** @type {!proto.speech.tts.Syllable.ContourExtreme} */ (jspb.Message.getField(this, 7));
};


/**
 * @param {?proto.speech.tts.Syllable.ContourExtreme|undefined} value
 * @return {!proto.speech.tts.Syllable} returns this
 */
proto.speech.tts.Syllable.prototype.setContourExtreme = function(value) {
  return jspb.Message.setField(this, 7, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Syllable} returns this
 */
proto.speech.tts.Syllable.prototype.clearContourExtreme = function() {
  return jspb.Message.setField(this, 7, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Syllable.prototype.hasContourExtreme = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * repeated ContourValues contours = 8;
 * @return {!Array<!proto.speech.tts.Syllable.ContourValues>}
 */
proto.speech.tts.Syllable.prototype.getContoursList = function() {
  return /** @type{!Array<!proto.speech.tts.Syllable.ContourValues>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.Syllable.ContourValues, 8));
};


/**
 * @param {Array<!proto.speech.tts.Syllable.ContourValues>|undefined} value
 * @return {!proto.speech.tts.Syllable} returns this
*/
proto.speech.tts.Syllable.prototype.setContoursList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 8, value);
};


/**
 * @param {!proto.speech.tts.Syllable.ContourValues=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.Syllable.ContourValues}
 */
proto.speech.tts.Syllable.prototype.addContours = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 8, opt_value, proto.speech.tts.Syllable.ContourValues, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.Syllable} returns this
 */
proto.speech.tts.Syllable.prototype.clearContoursList = function() {
  return this.setContoursList([]);
};


/**
 * optional JapaneseSyllableLinguisticFeatures japanese_linguistic_features = 9;
 * @return {?proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures}
 */
proto.speech.tts.Syllable.prototype.getJapaneseLinguisticFeatures = function() {
  return /** @type{?proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures, 9));
};


/**
 * @param {?proto.speech.tts.Syllable.JapaneseSyllableLinguisticFeatures|undefined} value
 * @return {!proto.speech.tts.Syllable} returns this
*/
proto.speech.tts.Syllable.prototype.setJapaneseLinguisticFeatures = function(value) {
  return jspb.Message.setWrapperField(this, 9, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Syllable} returns this
 */
proto.speech.tts.Syllable.prototype.clearJapaneseLinguisticFeatures = function() {
  return this.setJapaneseLinguisticFeatures(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Syllable.prototype.hasJapaneseLinguisticFeatures = function() {
  return jspb.Message.getField(this, 9) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.Syllable}
 */
proto.speech.tts.Syllable.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.Syllable, data);
};



//speech/tts/engine/phonology/lexicon/lexicon_id_proto_jspb/lexiconid.jspb.js
// source: speech/tts/engine/phonology/lexicon/lexicon_id.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.LexiconId');
goog.provide('proto.speech.tts.LexiconId.TypeCase');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.LexiconId = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, proto.speech.tts.LexiconId.oneofGroups_);
};
goog.inherits(proto.speech.tts.LexiconId, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.LexiconId.displayName = 'proto.speech.tts.LexiconId';
}

/**
 * Oneof group definitions for this message. Each group defines the field
 * numbers belonging to that group. When of these fields' value is set, all
 * other fields in the group are cleared. During deserialization, if multiple
 * fields are encountered for a group, only the last value seen will be kept.
 * @private {!Array<!Array<number>>}
 * @const
 */
proto.speech.tts.LexiconId.oneofGroups_ = [[1,2,3,4]];

/**
 * @enum {number}
 */
proto.speech.tts.LexiconId.TypeCase = {
  TYPE_NOT_SET: 0,
  PRIMARY_LEXICON: 1,
  USER_LEXICON: 2,
  SECONDARY_LEXICON: 3,
  CROSS_LINGUAL_LEXICON: 4
};

/**
 * @return {!proto.speech.tts.LexiconId.TypeCase}
 */
proto.speech.tts.LexiconId.prototype.getTypeCase = function() {
  return /** @type {!proto.speech.tts.LexiconId.TypeCase} */(jspb.Message.computeOneofCase(this, proto.speech.tts.LexiconId.oneofGroups_[0]));
};



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.LexiconId.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.LexiconId.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.LexiconId} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.LexiconId.toObject = function(includeInstance, msg) {
  var f, obj = {
    primaryLexicon: jspb.Message.getBooleanFieldWithDefault(msg, 1, false),
    userLexicon: jspb.Message.getBooleanFieldWithDefault(msg, 2, false),
    secondaryLexicon: jspb.Message.getFieldWithDefault(msg, 3, ""),
    crossLingualLexicon: jspb.Message.getFieldWithDefault(msg, 4, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of LexiconId as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.LexiconId.ObjectFormat = function() {
  /** @type {?boolean|undefined} */
  this.primaryLexicon;

  /** @type {?boolean|undefined} */
  this.userLexicon;

  /** @type {?string|undefined} */
  this.secondaryLexicon;

  /** @type {?string|undefined} */
  this.crossLingualLexicon;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.LexiconId.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.LexiconId}
 */
proto.speech.tts.LexiconId.fromObject = function(obj) {
  var msg = new proto.speech.tts.LexiconId();
  obj.primaryLexicon != null && jspb.Message.setField(msg, 1, obj.primaryLexicon);
  obj.userLexicon != null && jspb.Message.setField(msg, 2, obj.userLexicon);
  obj.secondaryLexicon != null && jspb.Message.setField(msg, 3, obj.secondaryLexicon);
  obj.crossLingualLexicon != null && jspb.Message.setField(msg, 4, obj.crossLingualLexicon);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.LexiconId}
 */
proto.speech.tts.LexiconId.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.LexiconId;
  return proto.speech.tts.LexiconId.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.LexiconId} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.LexiconId}
 */
proto.speech.tts.LexiconId.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setPrimaryLexicon(value);
      break;
    case 2:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setUserLexicon(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setSecondaryLexicon(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setCrossLingualLexicon(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.LexiconId.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.LexiconId.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.LexiconId} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.LexiconId.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {boolean} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeBool(
      1,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeBool(
      2,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeString(
      3,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeString(
      4,
      f
    );
  }
};


/**
 * optional bool primary_lexicon = 1;
 * @return {boolean}
 */
proto.speech.tts.LexiconId.prototype.getPrimaryLexicon = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 1, false));
};


/**
 * @param {boolean} value
 * @return {!proto.speech.tts.LexiconId} returns this
 */
proto.speech.tts.LexiconId.prototype.setPrimaryLexicon = function(value) {
  return jspb.Message.setOneofField(this, 1, proto.speech.tts.LexiconId.oneofGroups_[0], value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.LexiconId} returns this
 */
proto.speech.tts.LexiconId.prototype.clearPrimaryLexicon = function() {
  return jspb.Message.setOneofField(this, 1, proto.speech.tts.LexiconId.oneofGroups_[0], undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.LexiconId.prototype.hasPrimaryLexicon = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional bool user_lexicon = 2;
 * @return {boolean}
 */
proto.speech.tts.LexiconId.prototype.getUserLexicon = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));
};


/**
 * @param {boolean} value
 * @return {!proto.speech.tts.LexiconId} returns this
 */
proto.speech.tts.LexiconId.prototype.setUserLexicon = function(value) {
  return jspb.Message.setOneofField(this, 2, proto.speech.tts.LexiconId.oneofGroups_[0], value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.LexiconId} returns this
 */
proto.speech.tts.LexiconId.prototype.clearUserLexicon = function() {
  return jspb.Message.setOneofField(this, 2, proto.speech.tts.LexiconId.oneofGroups_[0], undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.LexiconId.prototype.hasUserLexicon = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional string secondary_lexicon = 3;
 * @return {string}
 */
proto.speech.tts.LexiconId.prototype.getSecondaryLexicon = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, ""));
};


/**
 * @param {string} value
 * @return {!proto.speech.tts.LexiconId} returns this
 */
proto.speech.tts.LexiconId.prototype.setSecondaryLexicon = function(value) {
  return jspb.Message.setOneofField(this, 3, proto.speech.tts.LexiconId.oneofGroups_[0], value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.LexiconId} returns this
 */
proto.speech.tts.LexiconId.prototype.clearSecondaryLexicon = function() {
  return jspb.Message.setOneofField(this, 3, proto.speech.tts.LexiconId.oneofGroups_[0], undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.LexiconId.prototype.hasSecondaryLexicon = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional string cross_lingual_lexicon = 4;
 * @return {string}
 */
proto.speech.tts.LexiconId.prototype.getCrossLingualLexicon = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, ""));
};


/**
 * @param {string} value
 * @return {!proto.speech.tts.LexiconId} returns this
 */
proto.speech.tts.LexiconId.prototype.setCrossLingualLexicon = function(value) {
  return jspb.Message.setOneofField(this, 4, proto.speech.tts.LexiconId.oneofGroups_[0], value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.LexiconId} returns this
 */
proto.speech.tts.LexiconId.prototype.clearCrossLingualLexicon = function() {
  return jspb.Message.setOneofField(this, 4, proto.speech.tts.LexiconId.oneofGroups_[0], undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.LexiconId.prototype.hasCrossLingualLexicon = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.LexiconId}
 */
proto.speech.tts.LexiconId.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.LexiconId, data);
};



//speech/tts/engine/utterance/linguistic_structure_proto_jspb/dependencydata.jspb.js
// source: speech/tts/engine/utterance/syntax.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.DependencyData');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.DependencyData = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.DependencyData, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.DependencyData.displayName = 'proto.speech.tts.DependencyData';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.DependencyData.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.DependencyData.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.DependencyData} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.DependencyData.toObject = function(includeInstance, msg) {
  var f, obj = {
    head: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    label: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    depth: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,
    offsetFromHead: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f,
    sizeToLeftLowerBound: (f = jspb.Message.getField(msg, 5)) == null ? undefined : f,
    sizeToLeftUpperBound: (f = jspb.Message.getField(msg, 6)) == null ? undefined : f,
    sizeToRightLowerBound: (f = jspb.Message.getField(msg, 7)) == null ? undefined : f,
    sizeToRightUpperBound: (f = jspb.Message.getField(msg, 8)) == null ? undefined : f,
    crossingLinksToRight: (f = jspb.Message.getField(msg, 9)) == null ? undefined : f,
    boundaryStrengthToRight: (f = jspb.Message.getOptionalFloatingPointField(msg, 10)) == null ? undefined : f,
    linkStepsToRight: (f = jspb.Message.getField(msg, 11)) == null ? undefined : f,
    weightedCrossingLinksToRight: (f = jspb.Message.getOptionalFloatingPointField(msg, 12)) == null ? undefined : f,
    weightedBoundaryStrengthToRight: (f = jspb.Message.getOptionalFloatingPointField(msg, 13)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of DependencyData as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.DependencyData.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.head;

  /** @type {?string|undefined} */
  this.label;

  /** @type {?number|undefined} */
  this.depth;

  /** @type {?number|undefined} */
  this.offsetFromHead;

  /** @type {?number|undefined} */
  this.sizeToLeftLowerBound;

  /** @type {?number|undefined} */
  this.sizeToLeftUpperBound;

  /** @type {?number|undefined} */
  this.sizeToRightLowerBound;

  /** @type {?number|undefined} */
  this.sizeToRightUpperBound;

  /** @type {?number|undefined} */
  this.crossingLinksToRight;

  /** @type {?number|undefined} */
  this.boundaryStrengthToRight;

  /** @type {?number|undefined} */
  this.linkStepsToRight;

  /** @type {?number|undefined} */
  this.weightedCrossingLinksToRight;

  /** @type {?number|undefined} */
  this.weightedBoundaryStrengthToRight;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.DependencyData.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.DependencyData}
 */
proto.speech.tts.DependencyData.fromObject = function(obj) {
  var msg = new proto.speech.tts.DependencyData();
  obj.head != null && jspb.Message.setField(msg, 1, obj.head);
  obj.label != null && jspb.Message.setField(msg, 2, obj.label);
  obj.depth != null && jspb.Message.setField(msg, 3, obj.depth);
  obj.offsetFromHead != null && jspb.Message.setField(msg, 4, obj.offsetFromHead);
  obj.sizeToLeftLowerBound != null && jspb.Message.setField(msg, 5, obj.sizeToLeftLowerBound);
  obj.sizeToLeftUpperBound != null && jspb.Message.setField(msg, 6, obj.sizeToLeftUpperBound);
  obj.sizeToRightLowerBound != null && jspb.Message.setField(msg, 7, obj.sizeToRightLowerBound);
  obj.sizeToRightUpperBound != null && jspb.Message.setField(msg, 8, obj.sizeToRightUpperBound);
  obj.crossingLinksToRight != null && jspb.Message.setField(msg, 9, obj.crossingLinksToRight);
  obj.boundaryStrengthToRight != null && jspb.Message.setField(msg, 10, obj.boundaryStrengthToRight);
  obj.linkStepsToRight != null && jspb.Message.setField(msg, 11, obj.linkStepsToRight);
  obj.weightedCrossingLinksToRight != null && jspb.Message.setField(msg, 12, obj.weightedCrossingLinksToRight);
  obj.weightedBoundaryStrengthToRight != null && jspb.Message.setField(msg, 13, obj.weightedBoundaryStrengthToRight);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.DependencyData}
 */
proto.speech.tts.DependencyData.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.DependencyData;
  return proto.speech.tts.DependencyData.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.DependencyData} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.DependencyData}
 */
proto.speech.tts.DependencyData.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {number} */ (reader.readUint32());
      msg.setHead(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setLabel(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readUint32());
      msg.setDepth(value);
      break;
    case 4:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setOffsetFromHead(value);
      break;
    case 5:
      var value = /** @type {number} */ (reader.readUint32());
      msg.setSizeToLeftLowerBound(value);
      break;
    case 6:
      var value = /** @type {number} */ (reader.readUint32());
      msg.setSizeToLeftUpperBound(value);
      break;
    case 7:
      var value = /** @type {number} */ (reader.readUint32());
      msg.setSizeToRightLowerBound(value);
      break;
    case 8:
      var value = /** @type {number} */ (reader.readUint32());
      msg.setSizeToRightUpperBound(value);
      break;
    case 9:
      var value = /** @type {number} */ (reader.readUint32());
      msg.setCrossingLinksToRight(value);
      break;
    case 10:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setBoundaryStrengthToRight(value);
      break;
    case 11:
      var value = /** @type {number} */ (reader.readUint32());
      msg.setLinkStepsToRight(value);
      break;
    case 12:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setWeightedCrossingLinksToRight(value);
      break;
    case 13:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setWeightedBoundaryStrengthToRight(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.DependencyData.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.DependencyData.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.DependencyData} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.DependencyData.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?number} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeUint32(
      1,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeUint32(
      3,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeInt32(
      4,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeUint32(
      5,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeUint32(
      6,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 7));
  if (f != null) {
    writer.writeUint32(
      7,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 8));
  if (f != null) {
    writer.writeUint32(
      8,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 9));
  if (f != null) {
    writer.writeUint32(
      9,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 10));
  if (f != null) {
    writer.writeFloat(
      10,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 11));
  if (f != null) {
    writer.writeUint32(
      11,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 12));
  if (f != null) {
    writer.writeFloat(
      12,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 13));
  if (f != null) {
    writer.writeFloat(
      13,
      f
    );
  }
};


/**
 * optional uint32 head = 1;
 * @return {?number}
 */
proto.speech.tts.DependencyData.prototype.getHead = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.DependencyData} returns this
 */
proto.speech.tts.DependencyData.prototype.setHead = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.DependencyData} returns this
 */
proto.speech.tts.DependencyData.prototype.clearHead = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.DependencyData.prototype.hasHead = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional string label = 2;
 * @return {?string}
 */
proto.speech.tts.DependencyData.prototype.getLabel = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.DependencyData} returns this
 */
proto.speech.tts.DependencyData.prototype.setLabel = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.DependencyData} returns this
 */
proto.speech.tts.DependencyData.prototype.clearLabel = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.DependencyData.prototype.hasLabel = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional uint32 depth = 3;
 * @return {?number}
 */
proto.speech.tts.DependencyData.prototype.getDepth = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.DependencyData} returns this
 */
proto.speech.tts.DependencyData.prototype.setDepth = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.DependencyData} returns this
 */
proto.speech.tts.DependencyData.prototype.clearDepth = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.DependencyData.prototype.hasDepth = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional int32 offset_from_head = 4;
 * @return {?number}
 */
proto.speech.tts.DependencyData.prototype.getOffsetFromHead = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 4));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.DependencyData} returns this
 */
proto.speech.tts.DependencyData.prototype.setOffsetFromHead = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.DependencyData} returns this
 */
proto.speech.tts.DependencyData.prototype.clearOffsetFromHead = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.DependencyData.prototype.hasOffsetFromHead = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional uint32 size_to_left_lower_bound = 5;
 * @return {?number}
 */
proto.speech.tts.DependencyData.prototype.getSizeToLeftLowerBound = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 5));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.DependencyData} returns this
 */
proto.speech.tts.DependencyData.prototype.setSizeToLeftLowerBound = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.DependencyData} returns this
 */
proto.speech.tts.DependencyData.prototype.clearSizeToLeftLowerBound = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.DependencyData.prototype.hasSizeToLeftLowerBound = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional uint32 size_to_left_upper_bound = 6;
 * @return {?number}
 */
proto.speech.tts.DependencyData.prototype.getSizeToLeftUpperBound = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 6));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.DependencyData} returns this
 */
proto.speech.tts.DependencyData.prototype.setSizeToLeftUpperBound = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.DependencyData} returns this
 */
proto.speech.tts.DependencyData.prototype.clearSizeToLeftUpperBound = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.DependencyData.prototype.hasSizeToLeftUpperBound = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional uint32 size_to_right_lower_bound = 7;
 * @return {?number}
 */
proto.speech.tts.DependencyData.prototype.getSizeToRightLowerBound = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 7));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.DependencyData} returns this
 */
proto.speech.tts.DependencyData.prototype.setSizeToRightLowerBound = function(value) {
  return jspb.Message.setField(this, 7, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.DependencyData} returns this
 */
proto.speech.tts.DependencyData.prototype.clearSizeToRightLowerBound = function() {
  return jspb.Message.setField(this, 7, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.DependencyData.prototype.hasSizeToRightLowerBound = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * optional uint32 size_to_right_upper_bound = 8;
 * @return {?number}
 */
proto.speech.tts.DependencyData.prototype.getSizeToRightUpperBound = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 8));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.DependencyData} returns this
 */
proto.speech.tts.DependencyData.prototype.setSizeToRightUpperBound = function(value) {
  return jspb.Message.setField(this, 8, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.DependencyData} returns this
 */
proto.speech.tts.DependencyData.prototype.clearSizeToRightUpperBound = function() {
  return jspb.Message.setField(this, 8, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.DependencyData.prototype.hasSizeToRightUpperBound = function() {
  return jspb.Message.getField(this, 8) != null;
};


/**
 * optional uint32 crossing_links_to_right = 9;
 * @return {?number}
 */
proto.speech.tts.DependencyData.prototype.getCrossingLinksToRight = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 9));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.DependencyData} returns this
 */
proto.speech.tts.DependencyData.prototype.setCrossingLinksToRight = function(value) {
  return jspb.Message.setField(this, 9, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.DependencyData} returns this
 */
proto.speech.tts.DependencyData.prototype.clearCrossingLinksToRight = function() {
  return jspb.Message.setField(this, 9, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.DependencyData.prototype.hasCrossingLinksToRight = function() {
  return jspb.Message.getField(this, 9) != null;
};


/**
 * optional float boundary_strength_to_right = 10;
 * @return {?number}
 */
proto.speech.tts.DependencyData.prototype.getBoundaryStrengthToRight = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 10));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.DependencyData} returns this
 */
proto.speech.tts.DependencyData.prototype.setBoundaryStrengthToRight = function(value) {
  return jspb.Message.setField(this, 10, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.DependencyData} returns this
 */
proto.speech.tts.DependencyData.prototype.clearBoundaryStrengthToRight = function() {
  return jspb.Message.setField(this, 10, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.DependencyData.prototype.hasBoundaryStrengthToRight = function() {
  return jspb.Message.getField(this, 10) != null;
};


/**
 * optional uint32 link_steps_to_right = 11;
 * @return {?number}
 */
proto.speech.tts.DependencyData.prototype.getLinkStepsToRight = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 11));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.DependencyData} returns this
 */
proto.speech.tts.DependencyData.prototype.setLinkStepsToRight = function(value) {
  return jspb.Message.setField(this, 11, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.DependencyData} returns this
 */
proto.speech.tts.DependencyData.prototype.clearLinkStepsToRight = function() {
  return jspb.Message.setField(this, 11, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.DependencyData.prototype.hasLinkStepsToRight = function() {
  return jspb.Message.getField(this, 11) != null;
};


/**
 * optional float weighted_crossing_links_to_right = 12;
 * @return {?number}
 */
proto.speech.tts.DependencyData.prototype.getWeightedCrossingLinksToRight = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 12));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.DependencyData} returns this
 */
proto.speech.tts.DependencyData.prototype.setWeightedCrossingLinksToRight = function(value) {
  return jspb.Message.setField(this, 12, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.DependencyData} returns this
 */
proto.speech.tts.DependencyData.prototype.clearWeightedCrossingLinksToRight = function() {
  return jspb.Message.setField(this, 12, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.DependencyData.prototype.hasWeightedCrossingLinksToRight = function() {
  return jspb.Message.getField(this, 12) != null;
};


/**
 * optional float weighted_boundary_strength_to_right = 13;
 * @return {?number}
 */
proto.speech.tts.DependencyData.prototype.getWeightedBoundaryStrengthToRight = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 13));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.DependencyData} returns this
 */
proto.speech.tts.DependencyData.prototype.setWeightedBoundaryStrengthToRight = function(value) {
  return jspb.Message.setField(this, 13, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.DependencyData} returns this
 */
proto.speech.tts.DependencyData.prototype.clearWeightedBoundaryStrengthToRight = function() {
  return jspb.Message.setField(this, 13, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.DependencyData.prototype.hasWeightedBoundaryStrengthToRight = function() {
  return jspb.Message.getField(this, 13) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.DependencyData}
 */
proto.speech.tts.DependencyData.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.DependencyData, data);
};



//speech/tts/engine/utterance/linguistic_structure_proto_jspb/syntax.jspb.js
// source: speech/tts/engine/utterance/syntax.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.Syntax');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.DependencyData');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.Syntax = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.Syntax.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.Syntax, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.Syntax.displayName = 'proto.speech.tts.Syntax';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.Syntax.repeatedFields_ = [16,12,15];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.Syntax.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.Syntax.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.Syntax} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Syntax.toObject = function(includeInstance, msg) {
  var f, obj = {
    name: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    deprecatedSaftLemma: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    lemmasList: (f = jspb.Message.getRepeatedField(msg, 16)) == null ? undefined : f,
    category: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f,
    tag: (f = jspb.Message.getField(msg, 5)) == null ? undefined : f,
    rosettaAnalyzerFeaturesList: (f = jspb.Message.getRepeatedField(msg, 12)) == null ? undefined : f,
    dependencyData: (f = msg.getDependencyData()) && proto.speech.tts.DependencyData.toObject(includeInstance, f),
    containsNucleus: (f = jspb.Message.getBooleanField(msg, 14)) == null ? undefined : f,
    saftAnalyzerFeaturesList: (f = jspb.Message.getRepeatedField(msg, 15)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Syntax as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.Syntax.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.name;

  /** @type {?string|undefined} */
  this.deprecatedSaftLemma;

  /** @type {?Array<string>|undefined} */
  this.lemmasList;

  /** @type {?string|undefined} */
  this.category;

  /** @type {?string|undefined} */
  this.tag;

  /** @type {?Array<string>|undefined} */
  this.rosettaAnalyzerFeaturesList;

  /** @type {?proto.speech.tts.DependencyData.ObjectFormat|undefined} */
  this.dependencyData;

  /** @type {?boolean|undefined} */
  this.containsNucleus;

  /** @type {?Array<string>|undefined} */
  this.saftAnalyzerFeaturesList;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.Syntax.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.Syntax}
 */
proto.speech.tts.Syntax.fromObject = function(obj) {
  var msg = new proto.speech.tts.Syntax();
  obj.name != null && jspb.Message.setField(msg, 1, obj.name);
  obj.deprecatedSaftLemma != null && jspb.Message.setField(msg, 2, obj.deprecatedSaftLemma);
  obj.lemmasList != null && jspb.Message.setField(msg, 16, obj.lemmasList);
  obj.category != null && jspb.Message.setField(msg, 4, obj.category);
  obj.tag != null && jspb.Message.setField(msg, 5, obj.tag);
  obj.rosettaAnalyzerFeaturesList != null && jspb.Message.setField(msg, 12, obj.rosettaAnalyzerFeaturesList);
  obj.dependencyData && jspb.Message.setWrapperField(
      msg, 13, proto.speech.tts.DependencyData.fromObject(obj.dependencyData));
  obj.containsNucleus != null && jspb.Message.setField(msg, 14, obj.containsNucleus);
  obj.saftAnalyzerFeaturesList != null && jspb.Message.setField(msg, 15, obj.saftAnalyzerFeaturesList);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.Syntax}
 */
proto.speech.tts.Syntax.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.Syntax;
  return proto.speech.tts.Syntax.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.Syntax} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.Syntax}
 */
proto.speech.tts.Syntax.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setName(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setDeprecatedSaftLemma(value);
      break;
    case 16:
      var value = /** @type {string} */ (reader.readString());
      msg.addLemmas(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setCategory(value);
      break;
    case 5:
      var value = /** @type {string} */ (reader.readString());
      msg.setTag(value);
      break;
    case 12:
      var value = /** @type {string} */ (reader.readString());
      msg.addRosettaAnalyzerFeatures(value);
      break;
    case 13:
      var value = new proto.speech.tts.DependencyData;
      reader.readMessage(value,proto.speech.tts.DependencyData.deserializeBinaryFromReader);
      msg.setDependencyData(value);
      break;
    case 14:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setContainsNucleus(value);
      break;
    case 15:
      var value = /** @type {string} */ (reader.readString());
      msg.addSaftAnalyzerFeatures(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.Syntax.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.Syntax.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.Syntax} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Syntax.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
  f = message.getLemmasList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      16,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeString(
      4,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeString(
      5,
      f
    );
  }
  f = message.getRosettaAnalyzerFeaturesList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      12,
      f
    );
  }
  f = message.getDependencyData();
  if (f != null) {
    writer.writeMessage(
      13,
      f,
      proto.speech.tts.DependencyData.serializeBinaryToWriter
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 14));
  if (f != null) {
    writer.writeBool(
      14,
      f
    );
  }
  f = message.getSaftAnalyzerFeaturesList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      15,
      f
    );
  }
};


/**
 * optional string name = 1;
 * @return {?string}
 */
proto.speech.tts.Syntax.prototype.getName = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Syntax} returns this
 */
proto.speech.tts.Syntax.prototype.setName = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Syntax} returns this
 */
proto.speech.tts.Syntax.prototype.clearName = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Syntax.prototype.hasName = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional string deprecated_saft_lemma = 2;
 * @return {?string}
 */
proto.speech.tts.Syntax.prototype.getDeprecatedSaftLemma = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Syntax} returns this
 */
proto.speech.tts.Syntax.prototype.setDeprecatedSaftLemma = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Syntax} returns this
 */
proto.speech.tts.Syntax.prototype.clearDeprecatedSaftLemma = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Syntax.prototype.hasDeprecatedSaftLemma = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * repeated string lemmas = 16;
 * @return {!Array<string>}
 */
proto.speech.tts.Syntax.prototype.getLemmasList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 16));
};


/**
 * @param {Array<string>|undefined} value
 * @return {!proto.speech.tts.Syntax} returns this
 */
proto.speech.tts.Syntax.prototype.setLemmasList = function(value) {
  return jspb.Message.setField(this, 16, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.Syntax} returns this
 */
proto.speech.tts.Syntax.prototype.addLemmas = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 16, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.Syntax} returns this
 */
proto.speech.tts.Syntax.prototype.clearLemmasList = function() {
  return this.setLemmasList([]);
};


/**
 * optional string category = 4;
 * @return {?string}
 */
proto.speech.tts.Syntax.prototype.getCategory = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 4));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Syntax} returns this
 */
proto.speech.tts.Syntax.prototype.setCategory = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Syntax} returns this
 */
proto.speech.tts.Syntax.prototype.clearCategory = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Syntax.prototype.hasCategory = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional string tag = 5;
 * @return {?string}
 */
proto.speech.tts.Syntax.prototype.getTag = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 5));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Syntax} returns this
 */
proto.speech.tts.Syntax.prototype.setTag = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Syntax} returns this
 */
proto.speech.tts.Syntax.prototype.clearTag = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Syntax.prototype.hasTag = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * repeated string rosetta_analyzer_features = 12;
 * @return {!Array<string>}
 */
proto.speech.tts.Syntax.prototype.getRosettaAnalyzerFeaturesList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 12));
};


/**
 * @param {Array<string>|undefined} value
 * @return {!proto.speech.tts.Syntax} returns this
 */
proto.speech.tts.Syntax.prototype.setRosettaAnalyzerFeaturesList = function(value) {
  return jspb.Message.setField(this, 12, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.Syntax} returns this
 */
proto.speech.tts.Syntax.prototype.addRosettaAnalyzerFeatures = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 12, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.Syntax} returns this
 */
proto.speech.tts.Syntax.prototype.clearRosettaAnalyzerFeaturesList = function() {
  return this.setRosettaAnalyzerFeaturesList([]);
};


/**
 * optional DependencyData dependency_data = 13;
 * @return {?proto.speech.tts.DependencyData}
 */
proto.speech.tts.Syntax.prototype.getDependencyData = function() {
  return /** @type{?proto.speech.tts.DependencyData} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.DependencyData, 13));
};


/**
 * @param {?proto.speech.tts.DependencyData|undefined} value
 * @return {!proto.speech.tts.Syntax} returns this
*/
proto.speech.tts.Syntax.prototype.setDependencyData = function(value) {
  return jspb.Message.setWrapperField(this, 13, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Syntax} returns this
 */
proto.speech.tts.Syntax.prototype.clearDependencyData = function() {
  return this.setDependencyData(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Syntax.prototype.hasDependencyData = function() {
  return jspb.Message.getField(this, 13) != null;
};


/**
 * optional bool contains_nucleus = 14;
 * @return {?boolean}
 */
proto.speech.tts.Syntax.prototype.getContainsNucleus = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 14));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.Syntax} returns this
 */
proto.speech.tts.Syntax.prototype.setContainsNucleus = function(value) {
  return jspb.Message.setField(this, 14, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Syntax} returns this
 */
proto.speech.tts.Syntax.prototype.clearContainsNucleus = function() {
  return jspb.Message.setField(this, 14, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Syntax.prototype.hasContainsNucleus = function() {
  return jspb.Message.getField(this, 14) != null;
};


/**
 * repeated string saft_analyzer_features = 15;
 * @return {!Array<string>}
 */
proto.speech.tts.Syntax.prototype.getSaftAnalyzerFeaturesList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 15));
};


/**
 * @param {Array<string>|undefined} value
 * @return {!proto.speech.tts.Syntax} returns this
 */
proto.speech.tts.Syntax.prototype.setSaftAnalyzerFeaturesList = function(value) {
  return jspb.Message.setField(this, 15, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.Syntax} returns this
 */
proto.speech.tts.Syntax.prototype.addSaftAnalyzerFeatures = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 15, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.Syntax} returns this
 */
proto.speech.tts.Syntax.prototype.clearSaftAnalyzerFeaturesList = function() {
  return this.setSaftAnalyzerFeaturesList([]);
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.Syntax}
 */
proto.speech.tts.Syntax.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.Syntax, data);
};



//speech/tts/public/morphosyntactic_feature_vector_proto_jspb/morphosyntacticfeaturevector.jspb.js
// source: speech/tts/public/morphosyntactic_feature_vector.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.MorphosyntacticFeatureVector');
goog.provide('proto.speech.tts.MorphosyntacticFeatureVector.Case');
goog.provide('proto.speech.tts.MorphosyntacticFeatureVector.DeclensionType');
goog.provide('proto.speech.tts.MorphosyntacticFeatureVector.Definiteness');
goog.provide('proto.speech.tts.MorphosyntacticFeatureVector.Gender');
goog.provide('proto.speech.tts.MorphosyntacticFeatureVector.Number');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.MorphosyntacticFeatureVector = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.MorphosyntacticFeatureVector, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.MorphosyntacticFeatureVector.displayName = 'proto.speech.tts.MorphosyntacticFeatureVector';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.MorphosyntacticFeatureVector.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.MorphosyntacticFeatureVector.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.MorphosyntacticFeatureVector} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.MorphosyntacticFeatureVector.toObject = function(includeInstance, msg) {
  var f, obj = {
    caseMarker: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    number: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    gender: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,
    declension: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f,
    definiteness: (f = jspb.Message.getField(msg, 5)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of MorphosyntacticFeatureVector as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.MorphosyntacticFeatureVector.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.caseMarker;

  /** @type {?number|undefined} */
  this.number;

  /** @type {?number|undefined} */
  this.gender;

  /** @type {?number|undefined} */
  this.declension;

  /** @type {?number|undefined} */
  this.definiteness;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.MorphosyntacticFeatureVector.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.MorphosyntacticFeatureVector}
 */
proto.speech.tts.MorphosyntacticFeatureVector.fromObject = function(obj) {
  var msg = new proto.speech.tts.MorphosyntacticFeatureVector();
  obj.caseMarker != null && jspb.Message.setField(msg, 1, obj.caseMarker);
  obj.number != null && jspb.Message.setField(msg, 2, obj.number);
  obj.gender != null && jspb.Message.setField(msg, 3, obj.gender);
  obj.declension != null && jspb.Message.setField(msg, 4, obj.declension);
  obj.definiteness != null && jspb.Message.setField(msg, 5, obj.definiteness);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.MorphosyntacticFeatureVector}
 */
proto.speech.tts.MorphosyntacticFeatureVector.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.MorphosyntacticFeatureVector;
  return proto.speech.tts.MorphosyntacticFeatureVector.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.MorphosyntacticFeatureVector} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.MorphosyntacticFeatureVector}
 */
proto.speech.tts.MorphosyntacticFeatureVector.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {!proto.speech.tts.MorphosyntacticFeatureVector.Case} */ (reader.readEnum());
      msg.setCaseMarker(value);
      break;
    case 2:
      var value = /** @type {!proto.speech.tts.MorphosyntacticFeatureVector.Number} */ (reader.readEnum());
      msg.setNumber(value);
      break;
    case 3:
      var value = /** @type {!proto.speech.tts.MorphosyntacticFeatureVector.Gender} */ (reader.readEnum());
      msg.setGender(value);
      break;
    case 4:
      var value = /** @type {!proto.speech.tts.MorphosyntacticFeatureVector.DeclensionType} */ (reader.readEnum());
      msg.setDeclension(value);
      break;
    case 5:
      var value = /** @type {!proto.speech.tts.MorphosyntacticFeatureVector.Definiteness} */ (reader.readEnum());
      msg.setDefiniteness(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.MorphosyntacticFeatureVector.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.MorphosyntacticFeatureVector.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.MorphosyntacticFeatureVector} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.MorphosyntacticFeatureVector.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {!proto.speech.tts.MorphosyntacticFeatureVector.Case} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeEnum(
      1,
      f
    );
  }
  f = /** @type {!proto.speech.tts.MorphosyntacticFeatureVector.Number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeEnum(
      2,
      f
    );
  }
  f = /** @type {!proto.speech.tts.MorphosyntacticFeatureVector.Gender} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeEnum(
      3,
      f
    );
  }
  f = /** @type {!proto.speech.tts.MorphosyntacticFeatureVector.DeclensionType} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeEnum(
      4,
      f
    );
  }
  f = /** @type {!proto.speech.tts.MorphosyntacticFeatureVector.Definiteness} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeEnum(
      5,
      f
    );
  }
};


/**
 * @enum {number}
 */
proto.speech.tts.MorphosyntacticFeatureVector.Case = {
  UNKNOWN_CASE: 0,
  NOMINATIVE: 1,
  ERGATIVE: 2,
  ABSOLUTIVE: 3,
  ACCUSATIVE: 4,
  BENEFACTIVE: 5,
  DATIVE: 6,
  MALEFACTIVE: 7,
  GENITIVE: 8,
  POSSESSED: 9,
  LOCATIVE: 10,
  PREPOSITIONAL: 10,
  ALLATIVE: 11,
  CONTALLATIVE: 12,
  ILLATIVE: 13,
  INALLATIVE: 14,
  INTERALLATIVE: 15,
  SUBALLATIVE: 16,
  SUPERALLATIVE: 17,
  ABLATIVE: 18,
  CONTABLATIVE: 19,
  DELATIVE: 20,
  ELATIVE: 21,
  INABLATIVE: 22,
  INTERABLATIVE: 23,
  SUBABLATIVE: 24,
  SUPERABLATIVE: 25,
  CONTLATIVE: 26,
  INTERLATIVE: 27,
  INTERTRANSLATIVE: 28,
  INTRANSLATIVE: 29,
  LATIVE: 30,
  PERLATIVE: 31,
  SUBLATIVE: 32,
  SUBTRANSLATIVE: 33,
  SUPERLATIVE: 34,
  SUPERTRANSLATIVE: 35,
  TRANSLATIVE: 36,
  CONTERMINATIVE: 37,
  INTERMINATIVE: 38,
  INTERTERMINATIVE: 39,
  SUBTERMINATIVE: 40,
  SUPERTERMINATIVE: 41,
  ABESSIVE: 42,
  ADESSIVE: 43,
  ESSIVE: 44,
  INESSIVE: 45,
  INTERESSIVE: 46,
  SUBESSIVE: 47,
  SUPERESSIVE: 48,
  INSTRUMENTAL: 49,
  COMITATIVE: 50,
  PARTITIVE: 51,
  VOCATIVE: 52,
  AVERSIVE: 53
};

/**
 * @enum {number}
 */
proto.speech.tts.MorphosyntacticFeatureVector.Number = {
  UNKNOWN_NUMBER: 0,
  SINGULAR: 1,
  DUAL: 2,
  GREATER_PAUCAL: 3,
  GREATER_PLURAL: 4,
  MULTAL: 5,
  PLURAL: 6,
  TRIAL: 7,
  GENERAL: 8,
  PAUCAL: 9
};

/**
 * @enum {number}
 */
proto.speech.tts.MorphosyntacticFeatureVector.Gender = {
  UNKNOWN_GENDER: 0,
  MASCULINE: 1,
  FEMININE: 2,
  NEUTER: 3,
  HUMAN: 4,
  ANIMATE: 5,
  INANIMATE: 6,
  VEGETABLE: 7,
  COMMON: 8,
  APOCOPE: 9
};

/**
 * @enum {number}
 */
proto.speech.tts.MorphosyntacticFeatureVector.DeclensionType = {
  UNKNOWN_DECLENSION: 0,
  WEAK: 1,
  STRONG: 2,
  MIXED: 3
};

/**
 * @enum {number}
 */
proto.speech.tts.MorphosyntacticFeatureVector.Definiteness = {
  UNKNOWN_DEFINITENESS: 0,
  DEFINITE: 1,
  INDEFINITE: 2
};

/**
 * optional Case case_marker = 1;
 * @return {!proto.speech.tts.MorphosyntacticFeatureVector.Case}
 */
proto.speech.tts.MorphosyntacticFeatureVector.prototype.getCaseMarker = function() {
  return /** @type {!proto.speech.tts.MorphosyntacticFeatureVector.Case} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?proto.speech.tts.MorphosyntacticFeatureVector.Case|undefined} value
 * @return {!proto.speech.tts.MorphosyntacticFeatureVector} returns this
 */
proto.speech.tts.MorphosyntacticFeatureVector.prototype.setCaseMarker = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.MorphosyntacticFeatureVector} returns this
 */
proto.speech.tts.MorphosyntacticFeatureVector.prototype.clearCaseMarker = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.MorphosyntacticFeatureVector.prototype.hasCaseMarker = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional Number number = 2;
 * @return {!proto.speech.tts.MorphosyntacticFeatureVector.Number}
 */
proto.speech.tts.MorphosyntacticFeatureVector.prototype.getNumber = function() {
  return /** @type {!proto.speech.tts.MorphosyntacticFeatureVector.Number} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?proto.speech.tts.MorphosyntacticFeatureVector.Number|undefined} value
 * @return {!proto.speech.tts.MorphosyntacticFeatureVector} returns this
 */
proto.speech.tts.MorphosyntacticFeatureVector.prototype.setNumber = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.MorphosyntacticFeatureVector} returns this
 */
proto.speech.tts.MorphosyntacticFeatureVector.prototype.clearNumber = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.MorphosyntacticFeatureVector.prototype.hasNumber = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional Gender gender = 3;
 * @return {!proto.speech.tts.MorphosyntacticFeatureVector.Gender}
 */
proto.speech.tts.MorphosyntacticFeatureVector.prototype.getGender = function() {
  return /** @type {!proto.speech.tts.MorphosyntacticFeatureVector.Gender} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?proto.speech.tts.MorphosyntacticFeatureVector.Gender|undefined} value
 * @return {!proto.speech.tts.MorphosyntacticFeatureVector} returns this
 */
proto.speech.tts.MorphosyntacticFeatureVector.prototype.setGender = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.MorphosyntacticFeatureVector} returns this
 */
proto.speech.tts.MorphosyntacticFeatureVector.prototype.clearGender = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.MorphosyntacticFeatureVector.prototype.hasGender = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional DeclensionType declension = 4;
 * @return {!proto.speech.tts.MorphosyntacticFeatureVector.DeclensionType}
 */
proto.speech.tts.MorphosyntacticFeatureVector.prototype.getDeclension = function() {
  return /** @type {!proto.speech.tts.MorphosyntacticFeatureVector.DeclensionType} */ (jspb.Message.getField(this, 4));
};


/**
 * @param {?proto.speech.tts.MorphosyntacticFeatureVector.DeclensionType|undefined} value
 * @return {!proto.speech.tts.MorphosyntacticFeatureVector} returns this
 */
proto.speech.tts.MorphosyntacticFeatureVector.prototype.setDeclension = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.MorphosyntacticFeatureVector} returns this
 */
proto.speech.tts.MorphosyntacticFeatureVector.prototype.clearDeclension = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.MorphosyntacticFeatureVector.prototype.hasDeclension = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional Definiteness definiteness = 5;
 * @return {!proto.speech.tts.MorphosyntacticFeatureVector.Definiteness}
 */
proto.speech.tts.MorphosyntacticFeatureVector.prototype.getDefiniteness = function() {
  return /** @type {!proto.speech.tts.MorphosyntacticFeatureVector.Definiteness} */ (jspb.Message.getField(this, 5));
};


/**
 * @param {?proto.speech.tts.MorphosyntacticFeatureVector.Definiteness|undefined} value
 * @return {!proto.speech.tts.MorphosyntacticFeatureVector} returns this
 */
proto.speech.tts.MorphosyntacticFeatureVector.prototype.setDefiniteness = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.MorphosyntacticFeatureVector} returns this
 */
proto.speech.tts.MorphosyntacticFeatureVector.prototype.clearDefiniteness = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.MorphosyntacticFeatureVector.prototype.hasDefiniteness = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.MorphosyntacticFeatureVector}
 */
proto.speech.tts.MorphosyntacticFeatureVector.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.MorphosyntacticFeatureVector, data);
};



//speech/tts/public/address_proto_jspb/thoroughfare.jspb.js
// source: speech/tts/public/address.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.Thoroughfare');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.MorphosyntacticFeatureVector');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.Thoroughfare = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.Thoroughfare.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.Thoroughfare, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.Thoroughfare.displayName = 'proto.speech.tts.Thoroughfare';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.Thoroughfare.repeatedFields_ = [8];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.Thoroughfare.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.Thoroughfare.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.Thoroughfare} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Thoroughfare.toObject = function(includeInstance, msg) {
  var f, obj = {
    number: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    dependentName: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    name: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,
    premises: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f,
    subpremises: (f = jspb.Message.getField(msg, 5)) == null ? undefined : f,
    morphosyntacticFeatures: (f = jspb.Message.getField(msg, 6)) == null ? undefined : f,
    structuredFeatures: (f = msg.getStructuredFeatures()) && proto.speech.tts.MorphosyntacticFeatureVector.toObject(includeInstance, f),
    preserveOrder: jspb.Message.getBooleanFieldWithDefault(msg, 7, true),
    fieldOrderList: (f = jspb.Message.getRepeatedField(msg, 8)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Thoroughfare as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.Thoroughfare.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.number;

  /** @type {?string|undefined} */
  this.dependentName;

  /** @type {?string|undefined} */
  this.name;

  /** @type {?string|undefined} */
  this.premises;

  /** @type {?string|undefined} */
  this.subpremises;

  /** @type {?string|undefined} */
  this.morphosyntacticFeatures;

  /** @type {?proto.speech.tts.MorphosyntacticFeatureVector.ObjectFormat|undefined} */
  this.structuredFeatures;

  /** @type {?boolean|undefined} */
  this.preserveOrder;

  /** @type {?Array<string>|undefined} */
  this.fieldOrderList;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.Thoroughfare.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.Thoroughfare}
 */
proto.speech.tts.Thoroughfare.fromObject = function(obj) {
  var msg = new proto.speech.tts.Thoroughfare();
  obj.number != null && jspb.Message.setField(msg, 1, obj.number);
  obj.dependentName != null && jspb.Message.setField(msg, 2, obj.dependentName);
  obj.name != null && jspb.Message.setField(msg, 3, obj.name);
  obj.premises != null && jspb.Message.setField(msg, 4, obj.premises);
  obj.subpremises != null && jspb.Message.setField(msg, 5, obj.subpremises);
  obj.morphosyntacticFeatures != null && jspb.Message.setField(msg, 6, obj.morphosyntacticFeatures);
  obj.structuredFeatures && jspb.Message.setWrapperField(
      msg, 9, proto.speech.tts.MorphosyntacticFeatureVector.fromObject(obj.structuredFeatures));
  obj.preserveOrder != null && jspb.Message.setField(msg, 7, obj.preserveOrder);
  obj.fieldOrderList != null && jspb.Message.setField(msg, 8, obj.fieldOrderList);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.Thoroughfare}
 */
proto.speech.tts.Thoroughfare.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.Thoroughfare;
  return proto.speech.tts.Thoroughfare.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.Thoroughfare} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.Thoroughfare}
 */
proto.speech.tts.Thoroughfare.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setNumber(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setDependentName(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setName(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setPremises(value);
      break;
    case 5:
      var value = /** @type {string} */ (reader.readString());
      msg.setSubpremises(value);
      break;
    case 6:
      var value = /** @type {string} */ (reader.readString());
      msg.setMorphosyntacticFeatures(value);
      break;
    case 9:
      var value = new proto.speech.tts.MorphosyntacticFeatureVector;
      reader.readMessage(value,proto.speech.tts.MorphosyntacticFeatureVector.deserializeBinaryFromReader);
      msg.setStructuredFeatures(value);
      break;
    case 7:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setPreserveOrder(value);
      break;
    case 8:
      var value = /** @type {string} */ (reader.readString());
      msg.addFieldOrder(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.Thoroughfare.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.Thoroughfare.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.Thoroughfare} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Thoroughfare.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeString(
      3,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeString(
      4,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeString(
      5,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeString(
      6,
      f
    );
  }
  f = message.getStructuredFeatures();
  if (f != null) {
    writer.writeMessage(
      9,
      f,
      proto.speech.tts.MorphosyntacticFeatureVector.serializeBinaryToWriter
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 7));
  if (f != null) {
    writer.writeBool(
      7,
      f
    );
  }
  f = message.getFieldOrderList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      8,
      f
    );
  }
};


/**
 * optional string number = 1;
 * @return {?string}
 */
proto.speech.tts.Thoroughfare.prototype.getNumber = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Thoroughfare} returns this
 */
proto.speech.tts.Thoroughfare.prototype.setNumber = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Thoroughfare} returns this
 */
proto.speech.tts.Thoroughfare.prototype.clearNumber = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Thoroughfare.prototype.hasNumber = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional string dependent_name = 2;
 * @return {?string}
 */
proto.speech.tts.Thoroughfare.prototype.getDependentName = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Thoroughfare} returns this
 */
proto.speech.tts.Thoroughfare.prototype.setDependentName = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Thoroughfare} returns this
 */
proto.speech.tts.Thoroughfare.prototype.clearDependentName = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Thoroughfare.prototype.hasDependentName = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional string name = 3;
 * @return {?string}
 */
proto.speech.tts.Thoroughfare.prototype.getName = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Thoroughfare} returns this
 */
proto.speech.tts.Thoroughfare.prototype.setName = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Thoroughfare} returns this
 */
proto.speech.tts.Thoroughfare.prototype.clearName = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Thoroughfare.prototype.hasName = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional string premises = 4;
 * @return {?string}
 */
proto.speech.tts.Thoroughfare.prototype.getPremises = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 4));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Thoroughfare} returns this
 */
proto.speech.tts.Thoroughfare.prototype.setPremises = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Thoroughfare} returns this
 */
proto.speech.tts.Thoroughfare.prototype.clearPremises = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Thoroughfare.prototype.hasPremises = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional string subpremises = 5;
 * @return {?string}
 */
proto.speech.tts.Thoroughfare.prototype.getSubpremises = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 5));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Thoroughfare} returns this
 */
proto.speech.tts.Thoroughfare.prototype.setSubpremises = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Thoroughfare} returns this
 */
proto.speech.tts.Thoroughfare.prototype.clearSubpremises = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Thoroughfare.prototype.hasSubpremises = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional string morphosyntactic_features = 6;
 * @return {?string}
 */
proto.speech.tts.Thoroughfare.prototype.getMorphosyntacticFeatures = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 6));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Thoroughfare} returns this
 */
proto.speech.tts.Thoroughfare.prototype.setMorphosyntacticFeatures = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Thoroughfare} returns this
 */
proto.speech.tts.Thoroughfare.prototype.clearMorphosyntacticFeatures = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Thoroughfare.prototype.hasMorphosyntacticFeatures = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional MorphosyntacticFeatureVector structured_features = 9;
 * @return {?proto.speech.tts.MorphosyntacticFeatureVector}
 */
proto.speech.tts.Thoroughfare.prototype.getStructuredFeatures = function() {
  return /** @type{?proto.speech.tts.MorphosyntacticFeatureVector} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.MorphosyntacticFeatureVector, 9));
};


/**
 * @param {?proto.speech.tts.MorphosyntacticFeatureVector|undefined} value
 * @return {!proto.speech.tts.Thoroughfare} returns this
*/
proto.speech.tts.Thoroughfare.prototype.setStructuredFeatures = function(value) {
  return jspb.Message.setWrapperField(this, 9, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Thoroughfare} returns this
 */
proto.speech.tts.Thoroughfare.prototype.clearStructuredFeatures = function() {
  return this.setStructuredFeatures(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Thoroughfare.prototype.hasStructuredFeatures = function() {
  return jspb.Message.getField(this, 9) != null;
};


/**
 * optional bool preserve_order = 7;
 * @return {boolean}
 */
proto.speech.tts.Thoroughfare.prototype.getPreserveOrder = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 7, true));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.Thoroughfare} returns this
 */
proto.speech.tts.Thoroughfare.prototype.setPreserveOrder = function(value) {
  return jspb.Message.setField(this, 7, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Thoroughfare} returns this
 */
proto.speech.tts.Thoroughfare.prototype.clearPreserveOrder = function() {
  return jspb.Message.setField(this, 7, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Thoroughfare.prototype.hasPreserveOrder = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * repeated string field_order = 8;
 * @return {!Array<string>}
 */
proto.speech.tts.Thoroughfare.prototype.getFieldOrderList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 8));
};


/**
 * @param {Array<string>|undefined} value
 * @return {!proto.speech.tts.Thoroughfare} returns this
 */
proto.speech.tts.Thoroughfare.prototype.setFieldOrderList = function(value) {
  return jspb.Message.setField(this, 8, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.Thoroughfare} returns this
 */
proto.speech.tts.Thoroughfare.prototype.addFieldOrder = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 8, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.Thoroughfare} returns this
 */
proto.speech.tts.Thoroughfare.prototype.clearFieldOrderList = function() {
  return this.setFieldOrderList([]);
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.Thoroughfare}
 */
proto.speech.tts.Thoroughfare.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.Thoroughfare, data);
};



//speech/tts/public/address_proto_jspb/address.jspb.js
// source: speech/tts/public/address.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.Address');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.MorphosyntacticFeatureVector');
goog.require('proto.speech.tts.Thoroughfare');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.Address = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.Address.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.Address, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.Address.displayName = 'proto.speech.tts.Address';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.Address.repeatedFields_ = [12];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.Address.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.Address.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.Address} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Address.toObject = function(includeInstance, msg) {
  var f, obj = {
    thoroughfare: (f = msg.getThoroughfare()) && proto.speech.tts.Thoroughfare.toObject(includeInstance, f),
    dependentLocality: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    locality: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,
    subadminArea: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f,
    adminArea: (f = jspb.Message.getField(msg, 5)) == null ? undefined : f,
    countryName: (f = jspb.Message.getField(msg, 6)) == null ? undefined : f,
    postCode: (f = jspb.Message.getField(msg, 7)) == null ? undefined : f,
    poBox: (f = jspb.Message.getField(msg, 8)) == null ? undefined : f,
    morphosyntacticFeatures: (f = jspb.Message.getField(msg, 9)) == null ? undefined : f,
    structuredFeatures: (f = msg.getStructuredFeatures()) && proto.speech.tts.MorphosyntacticFeatureVector.toObject(includeInstance, f),
    preserveOrder: jspb.Message.getBooleanFieldWithDefault(msg, 10, true),
    codeSwitch: (f = jspb.Message.getField(msg, 11)) == null ? undefined : f,
    fieldOrderList: (f = jspb.Message.getRepeatedField(msg, 12)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Address as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.Address.ObjectFormat = function() {
  /** @type {?proto.speech.tts.Thoroughfare.ObjectFormat|undefined} */
  this.thoroughfare;

  /** @type {?string|undefined} */
  this.dependentLocality;

  /** @type {?string|undefined} */
  this.locality;

  /** @type {?string|undefined} */
  this.subadminArea;

  /** @type {?string|undefined} */
  this.adminArea;

  /** @type {?string|undefined} */
  this.countryName;

  /** @type {?string|undefined} */
  this.postCode;

  /** @type {?string|undefined} */
  this.poBox;

  /** @type {?string|undefined} */
  this.morphosyntacticFeatures;

  /** @type {?proto.speech.tts.MorphosyntacticFeatureVector.ObjectFormat|undefined} */
  this.structuredFeatures;

  /** @type {?boolean|undefined} */
  this.preserveOrder;

  /** @type {?string|undefined} */
  this.codeSwitch;

  /** @type {?Array<string>|undefined} */
  this.fieldOrderList;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.Address.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.Address}
 */
proto.speech.tts.Address.fromObject = function(obj) {
  var msg = new proto.speech.tts.Address();
  obj.thoroughfare && jspb.Message.setWrapperField(
      msg, 1, proto.speech.tts.Thoroughfare.fromObject(obj.thoroughfare));
  obj.dependentLocality != null && jspb.Message.setField(msg, 2, obj.dependentLocality);
  obj.locality != null && jspb.Message.setField(msg, 3, obj.locality);
  obj.subadminArea != null && jspb.Message.setField(msg, 4, obj.subadminArea);
  obj.adminArea != null && jspb.Message.setField(msg, 5, obj.adminArea);
  obj.countryName != null && jspb.Message.setField(msg, 6, obj.countryName);
  obj.postCode != null && jspb.Message.setField(msg, 7, obj.postCode);
  obj.poBox != null && jspb.Message.setField(msg, 8, obj.poBox);
  obj.morphosyntacticFeatures != null && jspb.Message.setField(msg, 9, obj.morphosyntacticFeatures);
  obj.structuredFeatures && jspb.Message.setWrapperField(
      msg, 13, proto.speech.tts.MorphosyntacticFeatureVector.fromObject(obj.structuredFeatures));
  obj.preserveOrder != null && jspb.Message.setField(msg, 10, obj.preserveOrder);
  obj.codeSwitch != null && jspb.Message.setField(msg, 11, obj.codeSwitch);
  obj.fieldOrderList != null && jspb.Message.setField(msg, 12, obj.fieldOrderList);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.Address}
 */
proto.speech.tts.Address.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.Address;
  return proto.speech.tts.Address.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.Address} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.Address}
 */
proto.speech.tts.Address.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.speech.tts.Thoroughfare;
      reader.readMessage(value,proto.speech.tts.Thoroughfare.deserializeBinaryFromReader);
      msg.setThoroughfare(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setDependentLocality(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setLocality(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setSubadminArea(value);
      break;
    case 5:
      var value = /** @type {string} */ (reader.readString());
      msg.setAdminArea(value);
      break;
    case 6:
      var value = /** @type {string} */ (reader.readString());
      msg.setCountryName(value);
      break;
    case 7:
      var value = /** @type {string} */ (reader.readString());
      msg.setPostCode(value);
      break;
    case 8:
      var value = /** @type {string} */ (reader.readString());
      msg.setPoBox(value);
      break;
    case 9:
      var value = /** @type {string} */ (reader.readString());
      msg.setMorphosyntacticFeatures(value);
      break;
    case 13:
      var value = new proto.speech.tts.MorphosyntacticFeatureVector;
      reader.readMessage(value,proto.speech.tts.MorphosyntacticFeatureVector.deserializeBinaryFromReader);
      msg.setStructuredFeatures(value);
      break;
    case 10:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setPreserveOrder(value);
      break;
    case 11:
      var value = /** @type {string} */ (reader.readString());
      msg.setCodeSwitch(value);
      break;
    case 12:
      var value = /** @type {string} */ (reader.readString());
      msg.addFieldOrder(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.Address.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.Address.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.Address} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Address.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getThoroughfare();
  if (f != null) {
    writer.writeMessage(
      1,
      f,
      proto.speech.tts.Thoroughfare.serializeBinaryToWriter
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeString(
      3,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeString(
      4,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeString(
      5,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeString(
      6,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 7));
  if (f != null) {
    writer.writeString(
      7,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 8));
  if (f != null) {
    writer.writeString(
      8,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 9));
  if (f != null) {
    writer.writeString(
      9,
      f
    );
  }
  f = message.getStructuredFeatures();
  if (f != null) {
    writer.writeMessage(
      13,
      f,
      proto.speech.tts.MorphosyntacticFeatureVector.serializeBinaryToWriter
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 10));
  if (f != null) {
    writer.writeBool(
      10,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 11));
  if (f != null) {
    writer.writeString(
      11,
      f
    );
  }
  f = message.getFieldOrderList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      12,
      f
    );
  }
};


/**
 * optional Thoroughfare thoroughfare = 1;
 * @return {?proto.speech.tts.Thoroughfare}
 */
proto.speech.tts.Address.prototype.getThoroughfare = function() {
  return /** @type{?proto.speech.tts.Thoroughfare} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Thoroughfare, 1));
};


/**
 * @param {?proto.speech.tts.Thoroughfare|undefined} value
 * @return {!proto.speech.tts.Address} returns this
*/
proto.speech.tts.Address.prototype.setThoroughfare = function(value) {
  return jspb.Message.setWrapperField(this, 1, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Address} returns this
 */
proto.speech.tts.Address.prototype.clearThoroughfare = function() {
  return this.setThoroughfare(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Address.prototype.hasThoroughfare = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional string dependent_locality = 2;
 * @return {?string}
 */
proto.speech.tts.Address.prototype.getDependentLocality = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Address} returns this
 */
proto.speech.tts.Address.prototype.setDependentLocality = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Address} returns this
 */
proto.speech.tts.Address.prototype.clearDependentLocality = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Address.prototype.hasDependentLocality = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional string locality = 3;
 * @return {?string}
 */
proto.speech.tts.Address.prototype.getLocality = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Address} returns this
 */
proto.speech.tts.Address.prototype.setLocality = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Address} returns this
 */
proto.speech.tts.Address.prototype.clearLocality = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Address.prototype.hasLocality = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional string subadmin_area = 4;
 * @return {?string}
 */
proto.speech.tts.Address.prototype.getSubadminArea = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 4));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Address} returns this
 */
proto.speech.tts.Address.prototype.setSubadminArea = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Address} returns this
 */
proto.speech.tts.Address.prototype.clearSubadminArea = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Address.prototype.hasSubadminArea = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional string admin_area = 5;
 * @return {?string}
 */
proto.speech.tts.Address.prototype.getAdminArea = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 5));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Address} returns this
 */
proto.speech.tts.Address.prototype.setAdminArea = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Address} returns this
 */
proto.speech.tts.Address.prototype.clearAdminArea = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Address.prototype.hasAdminArea = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional string country_name = 6;
 * @return {?string}
 */
proto.speech.tts.Address.prototype.getCountryName = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 6));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Address} returns this
 */
proto.speech.tts.Address.prototype.setCountryName = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Address} returns this
 */
proto.speech.tts.Address.prototype.clearCountryName = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Address.prototype.hasCountryName = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional string post_code = 7;
 * @return {?string}
 */
proto.speech.tts.Address.prototype.getPostCode = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 7));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Address} returns this
 */
proto.speech.tts.Address.prototype.setPostCode = function(value) {
  return jspb.Message.setField(this, 7, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Address} returns this
 */
proto.speech.tts.Address.prototype.clearPostCode = function() {
  return jspb.Message.setField(this, 7, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Address.prototype.hasPostCode = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * optional string po_box = 8;
 * @return {?string}
 */
proto.speech.tts.Address.prototype.getPoBox = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 8));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Address} returns this
 */
proto.speech.tts.Address.prototype.setPoBox = function(value) {
  return jspb.Message.setField(this, 8, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Address} returns this
 */
proto.speech.tts.Address.prototype.clearPoBox = function() {
  return jspb.Message.setField(this, 8, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Address.prototype.hasPoBox = function() {
  return jspb.Message.getField(this, 8) != null;
};


/**
 * optional string morphosyntactic_features = 9;
 * @return {?string}
 */
proto.speech.tts.Address.prototype.getMorphosyntacticFeatures = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 9));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Address} returns this
 */
proto.speech.tts.Address.prototype.setMorphosyntacticFeatures = function(value) {
  return jspb.Message.setField(this, 9, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Address} returns this
 */
proto.speech.tts.Address.prototype.clearMorphosyntacticFeatures = function() {
  return jspb.Message.setField(this, 9, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Address.prototype.hasMorphosyntacticFeatures = function() {
  return jspb.Message.getField(this, 9) != null;
};


/**
 * optional MorphosyntacticFeatureVector structured_features = 13;
 * @return {?proto.speech.tts.MorphosyntacticFeatureVector}
 */
proto.speech.tts.Address.prototype.getStructuredFeatures = function() {
  return /** @type{?proto.speech.tts.MorphosyntacticFeatureVector} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.MorphosyntacticFeatureVector, 13));
};


/**
 * @param {?proto.speech.tts.MorphosyntacticFeatureVector|undefined} value
 * @return {!proto.speech.tts.Address} returns this
*/
proto.speech.tts.Address.prototype.setStructuredFeatures = function(value) {
  return jspb.Message.setWrapperField(this, 13, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Address} returns this
 */
proto.speech.tts.Address.prototype.clearStructuredFeatures = function() {
  return this.setStructuredFeatures(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Address.prototype.hasStructuredFeatures = function() {
  return jspb.Message.getField(this, 13) != null;
};


/**
 * optional bool preserve_order = 10;
 * @return {boolean}
 */
proto.speech.tts.Address.prototype.getPreserveOrder = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 10, true));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.Address} returns this
 */
proto.speech.tts.Address.prototype.setPreserveOrder = function(value) {
  return jspb.Message.setField(this, 10, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Address} returns this
 */
proto.speech.tts.Address.prototype.clearPreserveOrder = function() {
  return jspb.Message.setField(this, 10, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Address.prototype.hasPreserveOrder = function() {
  return jspb.Message.getField(this, 10) != null;
};


/**
 * optional string code_switch = 11;
 * @return {?string}
 */
proto.speech.tts.Address.prototype.getCodeSwitch = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 11));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Address} returns this
 */
proto.speech.tts.Address.prototype.setCodeSwitch = function(value) {
  return jspb.Message.setField(this, 11, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Address} returns this
 */
proto.speech.tts.Address.prototype.clearCodeSwitch = function() {
  return jspb.Message.setField(this, 11, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Address.prototype.hasCodeSwitch = function() {
  return jspb.Message.getField(this, 11) != null;
};


/**
 * repeated string field_order = 12;
 * @return {!Array<string>}
 */
proto.speech.tts.Address.prototype.getFieldOrderList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 12));
};


/**
 * @param {Array<string>|undefined} value
 * @return {!proto.speech.tts.Address} returns this
 */
proto.speech.tts.Address.prototype.setFieldOrderList = function(value) {
  return jspb.Message.setField(this, 12, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.Address} returns this
 */
proto.speech.tts.Address.prototype.addFieldOrder = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 12, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.Address} returns this
 */
proto.speech.tts.Address.prototype.clearFieldOrderList = function() {
  return this.setFieldOrderList([]);
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.Address}
 */
proto.speech.tts.Address.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.Address, data);
};



//speech/tts/public/semiotic_classes_proto_jspb/abbreviation.jspb.js
// source: speech/tts/public/semiotic_classes.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.Abbreviation');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.MorphosyntacticFeatureVector');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.Abbreviation = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.Abbreviation, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.Abbreviation.displayName = 'proto.speech.tts.Abbreviation';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.Abbreviation.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.Abbreviation.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.Abbreviation} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Abbreviation.toObject = function(includeInstance, msg) {
  var f, obj = {
    text: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    morphosyntacticFeatures: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    structuredFeatures: (f = msg.getStructuredFeatures()) && proto.speech.tts.MorphosyntacticFeatureVector.toObject(includeInstance, f),
    codeSwitch: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Abbreviation as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.Abbreviation.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.text;

  /** @type {?string|undefined} */
  this.morphosyntacticFeatures;

  /** @type {?proto.speech.tts.MorphosyntacticFeatureVector.ObjectFormat|undefined} */
  this.structuredFeatures;

  /** @type {?string|undefined} */
  this.codeSwitch;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.Abbreviation.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.Abbreviation}
 */
proto.speech.tts.Abbreviation.fromObject = function(obj) {
  var msg = new proto.speech.tts.Abbreviation();
  obj.text != null && jspb.Message.setField(msg, 1, obj.text);
  obj.morphosyntacticFeatures != null && jspb.Message.setField(msg, 2, obj.morphosyntacticFeatures);
  obj.structuredFeatures && jspb.Message.setWrapperField(
      msg, 4, proto.speech.tts.MorphosyntacticFeatureVector.fromObject(obj.structuredFeatures));
  obj.codeSwitch != null && jspb.Message.setField(msg, 3, obj.codeSwitch);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.Abbreviation}
 */
proto.speech.tts.Abbreviation.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.Abbreviation;
  return proto.speech.tts.Abbreviation.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.Abbreviation} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.Abbreviation}
 */
proto.speech.tts.Abbreviation.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setText(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setMorphosyntacticFeatures(value);
      break;
    case 4:
      var value = new proto.speech.tts.MorphosyntacticFeatureVector;
      reader.readMessage(value,proto.speech.tts.MorphosyntacticFeatureVector.deserializeBinaryFromReader);
      msg.setStructuredFeatures(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setCodeSwitch(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.Abbreviation.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.Abbreviation.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.Abbreviation} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Abbreviation.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
  f = message.getStructuredFeatures();
  if (f != null) {
    writer.writeMessage(
      4,
      f,
      proto.speech.tts.MorphosyntacticFeatureVector.serializeBinaryToWriter
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeString(
      3,
      f
    );
  }
};


/**
 * required string text = 1;
 * @return {string}
 */
proto.speech.tts.Abbreviation.prototype.getText = function() {
  return /** @type {string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Abbreviation} returns this
 */
proto.speech.tts.Abbreviation.prototype.setText = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Abbreviation} returns this
 */
proto.speech.tts.Abbreviation.prototype.clearText = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Abbreviation.prototype.hasText = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional string morphosyntactic_features = 2;
 * @return {?string}
 */
proto.speech.tts.Abbreviation.prototype.getMorphosyntacticFeatures = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Abbreviation} returns this
 */
proto.speech.tts.Abbreviation.prototype.setMorphosyntacticFeatures = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Abbreviation} returns this
 */
proto.speech.tts.Abbreviation.prototype.clearMorphosyntacticFeatures = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Abbreviation.prototype.hasMorphosyntacticFeatures = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional MorphosyntacticFeatureVector structured_features = 4;
 * @return {?proto.speech.tts.MorphosyntacticFeatureVector}
 */
proto.speech.tts.Abbreviation.prototype.getStructuredFeatures = function() {
  return /** @type{?proto.speech.tts.MorphosyntacticFeatureVector} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.MorphosyntacticFeatureVector, 4));
};


/**
 * @param {?proto.speech.tts.MorphosyntacticFeatureVector|undefined} value
 * @return {!proto.speech.tts.Abbreviation} returns this
*/
proto.speech.tts.Abbreviation.prototype.setStructuredFeatures = function(value) {
  return jspb.Message.setWrapperField(this, 4, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Abbreviation} returns this
 */
proto.speech.tts.Abbreviation.prototype.clearStructuredFeatures = function() {
  return this.setStructuredFeatures(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Abbreviation.prototype.hasStructuredFeatures = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional string code_switch = 3;
 * @return {?string}
 */
proto.speech.tts.Abbreviation.prototype.getCodeSwitch = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Abbreviation} returns this
 */
proto.speech.tts.Abbreviation.prototype.setCodeSwitch = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Abbreviation} returns this
 */
proto.speech.tts.Abbreviation.prototype.clearCodeSwitch = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Abbreviation.prototype.hasCodeSwitch = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.Abbreviation}
 */
proto.speech.tts.Abbreviation.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.Abbreviation, data);
};



//speech/tts/public/semiotic_classes_proto_jspb/cardinal.jspb.js
// source: speech/tts/public/semiotic_classes.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.Cardinal');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.MorphosyntacticFeatureVector');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.Cardinal = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.Cardinal.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.Cardinal, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.Cardinal.displayName = 'proto.speech.tts.Cardinal';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.Cardinal.repeatedFields_ = [5];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.Cardinal.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.Cardinal.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.Cardinal} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Cardinal.toObject = function(includeInstance, msg) {
  var f, obj = {
    integer: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    morphosyntacticFeatures: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    structuredFeatures: (f = msg.getStructuredFeatures()) && proto.speech.tts.MorphosyntacticFeatureVector.toObject(includeInstance, f),
    preserveOrder: (f = jspb.Message.getBooleanField(msg, 3)) == null ? undefined : f,
    codeSwitch: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f,
    fieldOrderList: (f = jspb.Message.getRepeatedField(msg, 5)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Cardinal as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.Cardinal.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.integer;

  /** @type {?string|undefined} */
  this.morphosyntacticFeatures;

  /** @type {?proto.speech.tts.MorphosyntacticFeatureVector.ObjectFormat|undefined} */
  this.structuredFeatures;

  /** @type {?boolean|undefined} */
  this.preserveOrder;

  /** @type {?string|undefined} */
  this.codeSwitch;

  /** @type {?Array<string>|undefined} */
  this.fieldOrderList;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.Cardinal.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.Cardinal}
 */
proto.speech.tts.Cardinal.fromObject = function(obj) {
  var msg = new proto.speech.tts.Cardinal();
  obj.integer != null && jspb.Message.setField(msg, 1, obj.integer);
  obj.morphosyntacticFeatures != null && jspb.Message.setField(msg, 2, obj.morphosyntacticFeatures);
  obj.structuredFeatures && jspb.Message.setWrapperField(
      msg, 6, proto.speech.tts.MorphosyntacticFeatureVector.fromObject(obj.structuredFeatures));
  obj.preserveOrder != null && jspb.Message.setField(msg, 3, obj.preserveOrder);
  obj.codeSwitch != null && jspb.Message.setField(msg, 4, obj.codeSwitch);
  obj.fieldOrderList != null && jspb.Message.setField(msg, 5, obj.fieldOrderList);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.Cardinal}
 */
proto.speech.tts.Cardinal.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.Cardinal;
  return proto.speech.tts.Cardinal.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.Cardinal} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.Cardinal}
 */
proto.speech.tts.Cardinal.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setInteger(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setMorphosyntacticFeatures(value);
      break;
    case 6:
      var value = new proto.speech.tts.MorphosyntacticFeatureVector;
      reader.readMessage(value,proto.speech.tts.MorphosyntacticFeatureVector.deserializeBinaryFromReader);
      msg.setStructuredFeatures(value);
      break;
    case 3:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setPreserveOrder(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setCodeSwitch(value);
      break;
    case 5:
      var value = /** @type {string} */ (reader.readString());
      msg.addFieldOrder(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.Cardinal.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.Cardinal.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.Cardinal} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Cardinal.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
  f = message.getStructuredFeatures();
  if (f != null) {
    writer.writeMessage(
      6,
      f,
      proto.speech.tts.MorphosyntacticFeatureVector.serializeBinaryToWriter
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeBool(
      3,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeString(
      4,
      f
    );
  }
  f = message.getFieldOrderList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      5,
      f
    );
  }
};


/**
 * required string integer = 1;
 * @return {string}
 */
proto.speech.tts.Cardinal.prototype.getInteger = function() {
  return /** @type {string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Cardinal} returns this
 */
proto.speech.tts.Cardinal.prototype.setInteger = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Cardinal} returns this
 */
proto.speech.tts.Cardinal.prototype.clearInteger = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Cardinal.prototype.hasInteger = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional string morphosyntactic_features = 2;
 * @return {?string}
 */
proto.speech.tts.Cardinal.prototype.getMorphosyntacticFeatures = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Cardinal} returns this
 */
proto.speech.tts.Cardinal.prototype.setMorphosyntacticFeatures = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Cardinal} returns this
 */
proto.speech.tts.Cardinal.prototype.clearMorphosyntacticFeatures = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Cardinal.prototype.hasMorphosyntacticFeatures = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional MorphosyntacticFeatureVector structured_features = 6;
 * @return {?proto.speech.tts.MorphosyntacticFeatureVector}
 */
proto.speech.tts.Cardinal.prototype.getStructuredFeatures = function() {
  return /** @type{?proto.speech.tts.MorphosyntacticFeatureVector} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.MorphosyntacticFeatureVector, 6));
};


/**
 * @param {?proto.speech.tts.MorphosyntacticFeatureVector|undefined} value
 * @return {!proto.speech.tts.Cardinal} returns this
*/
proto.speech.tts.Cardinal.prototype.setStructuredFeatures = function(value) {
  return jspb.Message.setWrapperField(this, 6, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Cardinal} returns this
 */
proto.speech.tts.Cardinal.prototype.clearStructuredFeatures = function() {
  return this.setStructuredFeatures(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Cardinal.prototype.hasStructuredFeatures = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional bool preserve_order = 3;
 * @return {?boolean}
 */
proto.speech.tts.Cardinal.prototype.getPreserveOrder = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 3));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.Cardinal} returns this
 */
proto.speech.tts.Cardinal.prototype.setPreserveOrder = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Cardinal} returns this
 */
proto.speech.tts.Cardinal.prototype.clearPreserveOrder = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Cardinal.prototype.hasPreserveOrder = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional string code_switch = 4;
 * @return {?string}
 */
proto.speech.tts.Cardinal.prototype.getCodeSwitch = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 4));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Cardinal} returns this
 */
proto.speech.tts.Cardinal.prototype.setCodeSwitch = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Cardinal} returns this
 */
proto.speech.tts.Cardinal.prototype.clearCodeSwitch = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Cardinal.prototype.hasCodeSwitch = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * repeated string field_order = 5;
 * @return {!Array<string>}
 */
proto.speech.tts.Cardinal.prototype.getFieldOrderList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 5));
};


/**
 * @param {Array<string>|undefined} value
 * @return {!proto.speech.tts.Cardinal} returns this
 */
proto.speech.tts.Cardinal.prototype.setFieldOrderList = function(value) {
  return jspb.Message.setField(this, 5, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.Cardinal} returns this
 */
proto.speech.tts.Cardinal.prototype.addFieldOrder = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 5, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.Cardinal} returns this
 */
proto.speech.tts.Cardinal.prototype.clearFieldOrderList = function() {
  return this.setFieldOrderList([]);
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.Cardinal}
 */
proto.speech.tts.Cardinal.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.Cardinal, data);
};



//speech/tts/public/semiotic_classes_proto_jspb/chemicalformula.jspb.js
// source: speech/tts/public/semiotic_classes.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.ChemicalFormula');
goog.provide('proto.speech.tts.ChemicalFormula.Unit');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.ChemicalFormula = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.ChemicalFormula.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.ChemicalFormula, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.ChemicalFormula.displayName = 'proto.speech.tts.ChemicalFormula';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.ChemicalFormula.Unit = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.ChemicalFormula.Unit, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.ChemicalFormula.Unit.displayName = 'proto.speech.tts.ChemicalFormula.Unit';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.ChemicalFormula.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.ChemicalFormula.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.ChemicalFormula.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.ChemicalFormula} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.ChemicalFormula.toObject = function(includeInstance, msg) {
  var f, obj = {
    unitList: jspb.Message.toObjectList(msg.getUnitList(),
    proto.speech.tts.ChemicalFormula.Unit.toObject, includeInstance),
    type: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    charge: (f = jspb.Message.getBooleanField(msg, 3)) == null ? undefined : f,
    codeSwitch: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of ChemicalFormula as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.ChemicalFormula.ObjectFormat = function() {
  /** @type {?Array<!proto.speech.tts.ChemicalFormula.Unit.ObjectFormat>|undefined} */
  this.unitList;

  /** @type {?string|undefined} */
  this.type;

  /** @type {?boolean|undefined} */
  this.charge;

  /** @type {?string|undefined} */
  this.codeSwitch;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.ChemicalFormula.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.ChemicalFormula}
 */
proto.speech.tts.ChemicalFormula.fromObject = function(obj) {
  var msg = new proto.speech.tts.ChemicalFormula();
  obj.unitList && jspb.Message.setRepeatedWrapperField(
      msg, 1, obj.unitList.map(
          proto.speech.tts.ChemicalFormula.Unit.fromObject));
  obj.type != null && jspb.Message.setField(msg, 2, obj.type);
  obj.charge != null && jspb.Message.setField(msg, 3, obj.charge);
  obj.codeSwitch != null && jspb.Message.setField(msg, 4, obj.codeSwitch);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.ChemicalFormula}
 */
proto.speech.tts.ChemicalFormula.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.ChemicalFormula;
  return proto.speech.tts.ChemicalFormula.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.ChemicalFormula} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.ChemicalFormula}
 */
proto.speech.tts.ChemicalFormula.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.speech.tts.ChemicalFormula.Unit;
      reader.readMessage(value,proto.speech.tts.ChemicalFormula.Unit.deserializeBinaryFromReader);
      msg.addUnit(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setType(value);
      break;
    case 3:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setCharge(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setCodeSwitch(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.ChemicalFormula.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.ChemicalFormula.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.ChemicalFormula} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.ChemicalFormula.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getUnitList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      1,
      f,
      proto.speech.tts.ChemicalFormula.Unit.serializeBinaryToWriter
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeBool(
      3,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeString(
      4,
      f
    );
  }
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.ChemicalFormula.Unit.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.ChemicalFormula.Unit.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.ChemicalFormula.Unit} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.ChemicalFormula.Unit.toObject = function(includeInstance, msg) {
  var f, obj = {
    symbol: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    numUnits: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Unit as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.ChemicalFormula.Unit.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.symbol;

  /** @type {?string|undefined} */
  this.numUnits;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.ChemicalFormula.Unit.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.ChemicalFormula.Unit}
 */
proto.speech.tts.ChemicalFormula.Unit.fromObject = function(obj) {
  var msg = new proto.speech.tts.ChemicalFormula.Unit();
  obj.symbol != null && jspb.Message.setField(msg, 1, obj.symbol);
  obj.numUnits != null && jspb.Message.setField(msg, 2, obj.numUnits);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.ChemicalFormula.Unit}
 */
proto.speech.tts.ChemicalFormula.Unit.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.ChemicalFormula.Unit;
  return proto.speech.tts.ChemicalFormula.Unit.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.ChemicalFormula.Unit} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.ChemicalFormula.Unit}
 */
proto.speech.tts.ChemicalFormula.Unit.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setSymbol(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setNumUnits(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.ChemicalFormula.Unit.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.ChemicalFormula.Unit.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.ChemicalFormula.Unit} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.ChemicalFormula.Unit.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string symbol = 1;
 * @return {?string}
 */
proto.speech.tts.ChemicalFormula.Unit.prototype.getSymbol = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.ChemicalFormula.Unit} returns this
 */
proto.speech.tts.ChemicalFormula.Unit.prototype.setSymbol = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.ChemicalFormula.Unit} returns this
 */
proto.speech.tts.ChemicalFormula.Unit.prototype.clearSymbol = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.ChemicalFormula.Unit.prototype.hasSymbol = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional string num_units = 2;
 * @return {?string}
 */
proto.speech.tts.ChemicalFormula.Unit.prototype.getNumUnits = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.ChemicalFormula.Unit} returns this
 */
proto.speech.tts.ChemicalFormula.Unit.prototype.setNumUnits = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.ChemicalFormula.Unit} returns this
 */
proto.speech.tts.ChemicalFormula.Unit.prototype.clearNumUnits = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.ChemicalFormula.Unit.prototype.hasNumUnits = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.ChemicalFormula.Unit}
 */
proto.speech.tts.ChemicalFormula.Unit.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.ChemicalFormula.Unit, data);
};


/**
 * repeated Unit unit = 1;
 * @return {!Array<!proto.speech.tts.ChemicalFormula.Unit>}
 */
proto.speech.tts.ChemicalFormula.prototype.getUnitList = function() {
  return /** @type{!Array<!proto.speech.tts.ChemicalFormula.Unit>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.ChemicalFormula.Unit, 1));
};


/**
 * @param {Array<!proto.speech.tts.ChemicalFormula.Unit>|undefined} value
 * @return {!proto.speech.tts.ChemicalFormula} returns this
*/
proto.speech.tts.ChemicalFormula.prototype.setUnitList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 1, value);
};


/**
 * @param {!proto.speech.tts.ChemicalFormula.Unit=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.ChemicalFormula.Unit}
 */
proto.speech.tts.ChemicalFormula.prototype.addUnit = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.speech.tts.ChemicalFormula.Unit, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.ChemicalFormula} returns this
 */
proto.speech.tts.ChemicalFormula.prototype.clearUnitList = function() {
  return this.setUnitList([]);
};


/**
 * optional string type = 2;
 * @return {?string}
 */
proto.speech.tts.ChemicalFormula.prototype.getType = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.ChemicalFormula} returns this
 */
proto.speech.tts.ChemicalFormula.prototype.setType = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.ChemicalFormula} returns this
 */
proto.speech.tts.ChemicalFormula.prototype.clearType = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.ChemicalFormula.prototype.hasType = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional bool charge = 3;
 * @return {?boolean}
 */
proto.speech.tts.ChemicalFormula.prototype.getCharge = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 3));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.ChemicalFormula} returns this
 */
proto.speech.tts.ChemicalFormula.prototype.setCharge = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.ChemicalFormula} returns this
 */
proto.speech.tts.ChemicalFormula.prototype.clearCharge = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.ChemicalFormula.prototype.hasCharge = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional string code_switch = 4;
 * @return {?string}
 */
proto.speech.tts.ChemicalFormula.prototype.getCodeSwitch = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 4));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.ChemicalFormula} returns this
 */
proto.speech.tts.ChemicalFormula.prototype.setCodeSwitch = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.ChemicalFormula} returns this
 */
proto.speech.tts.ChemicalFormula.prototype.clearCodeSwitch = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.ChemicalFormula.prototype.hasCodeSwitch = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.ChemicalFormula}
 */
proto.speech.tts.ChemicalFormula.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.ChemicalFormula, data);
};



//speech/tts/public/semiotic_classes_proto_jspb/connector.jspb.js
// source: speech/tts/public/semiotic_classes.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.Connector');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.MorphosyntacticFeatureVector');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.Connector = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.Connector, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.Connector.displayName = 'proto.speech.tts.Connector';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.Connector.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.Connector.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.Connector} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Connector.toObject = function(includeInstance, msg) {
  var f, obj = {
    type: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    morphosyntacticFeatures: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    structuredFeatures: (f = msg.getStructuredFeatures()) && proto.speech.tts.MorphosyntacticFeatureVector.toObject(includeInstance, f),
    codeSwitch: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Connector as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.Connector.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.type;

  /** @type {?string|undefined} */
  this.morphosyntacticFeatures;

  /** @type {?proto.speech.tts.MorphosyntacticFeatureVector.ObjectFormat|undefined} */
  this.structuredFeatures;

  /** @type {?string|undefined} */
  this.codeSwitch;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.Connector.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.Connector}
 */
proto.speech.tts.Connector.fromObject = function(obj) {
  var msg = new proto.speech.tts.Connector();
  obj.type != null && jspb.Message.setField(msg, 1, obj.type);
  obj.morphosyntacticFeatures != null && jspb.Message.setField(msg, 2, obj.morphosyntacticFeatures);
  obj.structuredFeatures && jspb.Message.setWrapperField(
      msg, 4, proto.speech.tts.MorphosyntacticFeatureVector.fromObject(obj.structuredFeatures));
  obj.codeSwitch != null && jspb.Message.setField(msg, 3, obj.codeSwitch);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.Connector}
 */
proto.speech.tts.Connector.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.Connector;
  return proto.speech.tts.Connector.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.Connector} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.Connector}
 */
proto.speech.tts.Connector.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setType(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setMorphosyntacticFeatures(value);
      break;
    case 4:
      var value = new proto.speech.tts.MorphosyntacticFeatureVector;
      reader.readMessage(value,proto.speech.tts.MorphosyntacticFeatureVector.deserializeBinaryFromReader);
      msg.setStructuredFeatures(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setCodeSwitch(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.Connector.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.Connector.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.Connector} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Connector.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
  f = message.getStructuredFeatures();
  if (f != null) {
    writer.writeMessage(
      4,
      f,
      proto.speech.tts.MorphosyntacticFeatureVector.serializeBinaryToWriter
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeString(
      3,
      f
    );
  }
};


/**
 * optional string type = 1;
 * @return {?string}
 */
proto.speech.tts.Connector.prototype.getType = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Connector} returns this
 */
proto.speech.tts.Connector.prototype.setType = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Connector} returns this
 */
proto.speech.tts.Connector.prototype.clearType = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Connector.prototype.hasType = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional string morphosyntactic_features = 2;
 * @return {?string}
 */
proto.speech.tts.Connector.prototype.getMorphosyntacticFeatures = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Connector} returns this
 */
proto.speech.tts.Connector.prototype.setMorphosyntacticFeatures = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Connector} returns this
 */
proto.speech.tts.Connector.prototype.clearMorphosyntacticFeatures = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Connector.prototype.hasMorphosyntacticFeatures = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional MorphosyntacticFeatureVector structured_features = 4;
 * @return {?proto.speech.tts.MorphosyntacticFeatureVector}
 */
proto.speech.tts.Connector.prototype.getStructuredFeatures = function() {
  return /** @type{?proto.speech.tts.MorphosyntacticFeatureVector} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.MorphosyntacticFeatureVector, 4));
};


/**
 * @param {?proto.speech.tts.MorphosyntacticFeatureVector|undefined} value
 * @return {!proto.speech.tts.Connector} returns this
*/
proto.speech.tts.Connector.prototype.setStructuredFeatures = function(value) {
  return jspb.Message.setWrapperField(this, 4, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Connector} returns this
 */
proto.speech.tts.Connector.prototype.clearStructuredFeatures = function() {
  return this.setStructuredFeatures(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Connector.prototype.hasStructuredFeatures = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional string code_switch = 3;
 * @return {?string}
 */
proto.speech.tts.Connector.prototype.getCodeSwitch = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Connector} returns this
 */
proto.speech.tts.Connector.prototype.setCodeSwitch = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Connector} returns this
 */
proto.speech.tts.Connector.prototype.clearCodeSwitch = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Connector.prototype.hasCodeSwitch = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.Connector}
 */
proto.speech.tts.Connector.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.Connector, data);
};



//speech/tts/public/semiotic_classes_proto_jspb/date.jspb.js
// source: speech/tts/public/semiotic_classes.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.Date');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.MorphosyntacticFeatureVector');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.Date = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.Date.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.Date, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.Date.displayName = 'proto.speech.tts.Date';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.Date.repeatedFields_ = [12];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.Date.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.Date.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.Date} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Date.toObject = function(includeInstance, msg) {
  var f, obj = {
    weekday: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    day: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    dayIndicator: (f = jspb.Message.getField(msg, 14)) == null ? undefined : f,
    month: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,
    monthIndicator: (f = jspb.Message.getField(msg, 15)) == null ? undefined : f,
    year: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f,
    yearIndicator: (f = jspb.Message.getField(msg, 16)) == null ? undefined : f,
    style: (f = jspb.Message.getField(msg, 5)) == null ? undefined : f,
    text: (f = jspb.Message.getField(msg, 6)) == null ? undefined : f,
    shortYear: (f = jspb.Message.getBooleanField(msg, 7)) == null ? undefined : f,
    era: (f = jspb.Message.getField(msg, 8)) == null ? undefined : f,
    morphosyntacticFeatures: (f = jspb.Message.getField(msg, 9)) == null ? undefined : f,
    structuredFeatures: (f = msg.getStructuredFeatures()) && proto.speech.tts.MorphosyntacticFeatureVector.toObject(includeInstance, f),
    preserveOrder: (f = jspb.Message.getBooleanField(msg, 10)) == null ? undefined : f,
    codeSwitch: (f = jspb.Message.getField(msg, 11)) == null ? undefined : f,
    fieldOrderList: (f = jspb.Message.getRepeatedField(msg, 12)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Date as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.Date.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.weekday;

  /** @type {?string|undefined} */
  this.day;

  /** @type {?string|undefined} */
  this.dayIndicator;

  /** @type {?string|undefined} */
  this.month;

  /** @type {?string|undefined} */
  this.monthIndicator;

  /** @type {?string|undefined} */
  this.year;

  /** @type {?string|undefined} */
  this.yearIndicator;

  /** @type {?number|undefined} */
  this.style;

  /** @type {?string|undefined} */
  this.text;

  /** @type {?boolean|undefined} */
  this.shortYear;

  /** @type {?string|undefined} */
  this.era;

  /** @type {?string|undefined} */
  this.morphosyntacticFeatures;

  /** @type {?proto.speech.tts.MorphosyntacticFeatureVector.ObjectFormat|undefined} */
  this.structuredFeatures;

  /** @type {?boolean|undefined} */
  this.preserveOrder;

  /** @type {?string|undefined} */
  this.codeSwitch;

  /** @type {?Array<string>|undefined} */
  this.fieldOrderList;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.Date.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.Date}
 */
proto.speech.tts.Date.fromObject = function(obj) {
  var msg = new proto.speech.tts.Date();
  obj.weekday != null && jspb.Message.setField(msg, 1, obj.weekday);
  obj.day != null && jspb.Message.setField(msg, 2, obj.day);
  obj.dayIndicator != null && jspb.Message.setField(msg, 14, obj.dayIndicator);
  obj.month != null && jspb.Message.setField(msg, 3, obj.month);
  obj.monthIndicator != null && jspb.Message.setField(msg, 15, obj.monthIndicator);
  obj.year != null && jspb.Message.setField(msg, 4, obj.year);
  obj.yearIndicator != null && jspb.Message.setField(msg, 16, obj.yearIndicator);
  obj.style != null && jspb.Message.setField(msg, 5, obj.style);
  obj.text != null && jspb.Message.setField(msg, 6, obj.text);
  obj.shortYear != null && jspb.Message.setField(msg, 7, obj.shortYear);
  obj.era != null && jspb.Message.setField(msg, 8, obj.era);
  obj.morphosyntacticFeatures != null && jspb.Message.setField(msg, 9, obj.morphosyntacticFeatures);
  obj.structuredFeatures && jspb.Message.setWrapperField(
      msg, 13, proto.speech.tts.MorphosyntacticFeatureVector.fromObject(obj.structuredFeatures));
  obj.preserveOrder != null && jspb.Message.setField(msg, 10, obj.preserveOrder);
  obj.codeSwitch != null && jspb.Message.setField(msg, 11, obj.codeSwitch);
  obj.fieldOrderList != null && jspb.Message.setField(msg, 12, obj.fieldOrderList);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.Date}
 */
proto.speech.tts.Date.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.Date;
  return proto.speech.tts.Date.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.Date} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.Date}
 */
proto.speech.tts.Date.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setWeekday(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setDay(value);
      break;
    case 14:
      var value = /** @type {string} */ (reader.readString());
      msg.setDayIndicator(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setMonth(value);
      break;
    case 15:
      var value = /** @type {string} */ (reader.readString());
      msg.setMonthIndicator(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setYear(value);
      break;
    case 16:
      var value = /** @type {string} */ (reader.readString());
      msg.setYearIndicator(value);
      break;
    case 5:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setStyle(value);
      break;
    case 6:
      var value = /** @type {string} */ (reader.readString());
      msg.setText(value);
      break;
    case 7:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setShortYear(value);
      break;
    case 8:
      var value = /** @type {string} */ (reader.readString());
      msg.setEra(value);
      break;
    case 9:
      var value = /** @type {string} */ (reader.readString());
      msg.setMorphosyntacticFeatures(value);
      break;
    case 13:
      var value = new proto.speech.tts.MorphosyntacticFeatureVector;
      reader.readMessage(value,proto.speech.tts.MorphosyntacticFeatureVector.deserializeBinaryFromReader);
      msg.setStructuredFeatures(value);
      break;
    case 10:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setPreserveOrder(value);
      break;
    case 11:
      var value = /** @type {string} */ (reader.readString());
      msg.setCodeSwitch(value);
      break;
    case 12:
      var value = /** @type {string} */ (reader.readString());
      msg.addFieldOrder(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.Date.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.Date.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.Date} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Date.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 14));
  if (f != null) {
    writer.writeString(
      14,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeString(
      3,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 15));
  if (f != null) {
    writer.writeString(
      15,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeString(
      4,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 16));
  if (f != null) {
    writer.writeString(
      16,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeInt32(
      5,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeString(
      6,
      f
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 7));
  if (f != null) {
    writer.writeBool(
      7,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 8));
  if (f != null) {
    writer.writeString(
      8,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 9));
  if (f != null) {
    writer.writeString(
      9,
      f
    );
  }
  f = message.getStructuredFeatures();
  if (f != null) {
    writer.writeMessage(
      13,
      f,
      proto.speech.tts.MorphosyntacticFeatureVector.serializeBinaryToWriter
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 10));
  if (f != null) {
    writer.writeBool(
      10,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 11));
  if (f != null) {
    writer.writeString(
      11,
      f
    );
  }
  f = message.getFieldOrderList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      12,
      f
    );
  }
};


/**
 * optional string weekday = 1;
 * @return {?string}
 */
proto.speech.tts.Date.prototype.getWeekday = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Date} returns this
 */
proto.speech.tts.Date.prototype.setWeekday = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Date} returns this
 */
proto.speech.tts.Date.prototype.clearWeekday = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Date.prototype.hasWeekday = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional string day = 2;
 * @return {?string}
 */
proto.speech.tts.Date.prototype.getDay = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Date} returns this
 */
proto.speech.tts.Date.prototype.setDay = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Date} returns this
 */
proto.speech.tts.Date.prototype.clearDay = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Date.prototype.hasDay = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional string day_indicator = 14;
 * @return {?string}
 */
proto.speech.tts.Date.prototype.getDayIndicator = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 14));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Date} returns this
 */
proto.speech.tts.Date.prototype.setDayIndicator = function(value) {
  return jspb.Message.setField(this, 14, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Date} returns this
 */
proto.speech.tts.Date.prototype.clearDayIndicator = function() {
  return jspb.Message.setField(this, 14, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Date.prototype.hasDayIndicator = function() {
  return jspb.Message.getField(this, 14) != null;
};


/**
 * optional string month = 3;
 * @return {?string}
 */
proto.speech.tts.Date.prototype.getMonth = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Date} returns this
 */
proto.speech.tts.Date.prototype.setMonth = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Date} returns this
 */
proto.speech.tts.Date.prototype.clearMonth = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Date.prototype.hasMonth = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional string month_indicator = 15;
 * @return {?string}
 */
proto.speech.tts.Date.prototype.getMonthIndicator = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 15));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Date} returns this
 */
proto.speech.tts.Date.prototype.setMonthIndicator = function(value) {
  return jspb.Message.setField(this, 15, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Date} returns this
 */
proto.speech.tts.Date.prototype.clearMonthIndicator = function() {
  return jspb.Message.setField(this, 15, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Date.prototype.hasMonthIndicator = function() {
  return jspb.Message.getField(this, 15) != null;
};


/**
 * optional string year = 4;
 * @return {?string}
 */
proto.speech.tts.Date.prototype.getYear = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 4));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Date} returns this
 */
proto.speech.tts.Date.prototype.setYear = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Date} returns this
 */
proto.speech.tts.Date.prototype.clearYear = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Date.prototype.hasYear = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional string year_indicator = 16;
 * @return {?string}
 */
proto.speech.tts.Date.prototype.getYearIndicator = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 16));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Date} returns this
 */
proto.speech.tts.Date.prototype.setYearIndicator = function(value) {
  return jspb.Message.setField(this, 16, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Date} returns this
 */
proto.speech.tts.Date.prototype.clearYearIndicator = function() {
  return jspb.Message.setField(this, 16, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Date.prototype.hasYearIndicator = function() {
  return jspb.Message.getField(this, 16) != null;
};


/**
 * optional int32 style = 5;
 * @return {?number}
 */
proto.speech.tts.Date.prototype.getStyle = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 5));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Date} returns this
 */
proto.speech.tts.Date.prototype.setStyle = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Date} returns this
 */
proto.speech.tts.Date.prototype.clearStyle = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Date.prototype.hasStyle = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional string text = 6;
 * @return {?string}
 */
proto.speech.tts.Date.prototype.getText = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 6));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Date} returns this
 */
proto.speech.tts.Date.prototype.setText = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Date} returns this
 */
proto.speech.tts.Date.prototype.clearText = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Date.prototype.hasText = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional bool short_year = 7;
 * @return {?boolean}
 */
proto.speech.tts.Date.prototype.getShortYear = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 7));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.Date} returns this
 */
proto.speech.tts.Date.prototype.setShortYear = function(value) {
  return jspb.Message.setField(this, 7, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Date} returns this
 */
proto.speech.tts.Date.prototype.clearShortYear = function() {
  return jspb.Message.setField(this, 7, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Date.prototype.hasShortYear = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * optional string era = 8;
 * @return {?string}
 */
proto.speech.tts.Date.prototype.getEra = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 8));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Date} returns this
 */
proto.speech.tts.Date.prototype.setEra = function(value) {
  return jspb.Message.setField(this, 8, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Date} returns this
 */
proto.speech.tts.Date.prototype.clearEra = function() {
  return jspb.Message.setField(this, 8, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Date.prototype.hasEra = function() {
  return jspb.Message.getField(this, 8) != null;
};


/**
 * optional string morphosyntactic_features = 9;
 * @return {?string}
 */
proto.speech.tts.Date.prototype.getMorphosyntacticFeatures = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 9));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Date} returns this
 */
proto.speech.tts.Date.prototype.setMorphosyntacticFeatures = function(value) {
  return jspb.Message.setField(this, 9, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Date} returns this
 */
proto.speech.tts.Date.prototype.clearMorphosyntacticFeatures = function() {
  return jspb.Message.setField(this, 9, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Date.prototype.hasMorphosyntacticFeatures = function() {
  return jspb.Message.getField(this, 9) != null;
};


/**
 * optional MorphosyntacticFeatureVector structured_features = 13;
 * @return {?proto.speech.tts.MorphosyntacticFeatureVector}
 */
proto.speech.tts.Date.prototype.getStructuredFeatures = function() {
  return /** @type{?proto.speech.tts.MorphosyntacticFeatureVector} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.MorphosyntacticFeatureVector, 13));
};


/**
 * @param {?proto.speech.tts.MorphosyntacticFeatureVector|undefined} value
 * @return {!proto.speech.tts.Date} returns this
*/
proto.speech.tts.Date.prototype.setStructuredFeatures = function(value) {
  return jspb.Message.setWrapperField(this, 13, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Date} returns this
 */
proto.speech.tts.Date.prototype.clearStructuredFeatures = function() {
  return this.setStructuredFeatures(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Date.prototype.hasStructuredFeatures = function() {
  return jspb.Message.getField(this, 13) != null;
};


/**
 * optional bool preserve_order = 10;
 * @return {?boolean}
 */
proto.speech.tts.Date.prototype.getPreserveOrder = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 10));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.Date} returns this
 */
proto.speech.tts.Date.prototype.setPreserveOrder = function(value) {
  return jspb.Message.setField(this, 10, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Date} returns this
 */
proto.speech.tts.Date.prototype.clearPreserveOrder = function() {
  return jspb.Message.setField(this, 10, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Date.prototype.hasPreserveOrder = function() {
  return jspb.Message.getField(this, 10) != null;
};


/**
 * optional string code_switch = 11;
 * @return {?string}
 */
proto.speech.tts.Date.prototype.getCodeSwitch = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 11));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Date} returns this
 */
proto.speech.tts.Date.prototype.setCodeSwitch = function(value) {
  return jspb.Message.setField(this, 11, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Date} returns this
 */
proto.speech.tts.Date.prototype.clearCodeSwitch = function() {
  return jspb.Message.setField(this, 11, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Date.prototype.hasCodeSwitch = function() {
  return jspb.Message.getField(this, 11) != null;
};


/**
 * repeated string field_order = 12;
 * @return {!Array<string>}
 */
proto.speech.tts.Date.prototype.getFieldOrderList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 12));
};


/**
 * @param {Array<string>|undefined} value
 * @return {!proto.speech.tts.Date} returns this
 */
proto.speech.tts.Date.prototype.setFieldOrderList = function(value) {
  return jspb.Message.setField(this, 12, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.Date} returns this
 */
proto.speech.tts.Date.prototype.addFieldOrder = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 12, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.Date} returns this
 */
proto.speech.tts.Date.prototype.clearFieldOrderList = function() {
  return this.setFieldOrderList([]);
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.Date}
 */
proto.speech.tts.Date.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.Date, data);
};



//speech/tts/public/semiotic_classes_proto_jspb/decimal.jspb.js
// source: speech/tts/public/semiotic_classes.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.Decimal');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.MorphosyntacticFeatureVector');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.Decimal = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.Decimal.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.Decimal, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.Decimal.displayName = 'proto.speech.tts.Decimal';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.Decimal.repeatedFields_ = [10];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.Decimal.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.Decimal.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.Decimal} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Decimal.toObject = function(includeInstance, msg) {
  var f, obj = {
    negative: (f = jspb.Message.getBooleanField(msg, 1)) == null ? undefined : f,
    integerPart: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    fractionalPart: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,
    quantity: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f,
    exponent: (f = jspb.Message.getField(msg, 5)) == null ? undefined : f,
    style: (f = jspb.Message.getField(msg, 6)) == null ? undefined : f,
    morphosyntacticFeatures: (f = jspb.Message.getField(msg, 7)) == null ? undefined : f,
    structuredFeatures: (f = msg.getStructuredFeatures()) && proto.speech.tts.MorphosyntacticFeatureVector.toObject(includeInstance, f),
    preserveOrder: (f = jspb.Message.getBooleanField(msg, 8)) == null ? undefined : f,
    codeSwitch: (f = jspb.Message.getField(msg, 9)) == null ? undefined : f,
    fieldOrderList: (f = jspb.Message.getRepeatedField(msg, 10)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Decimal as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.Decimal.ObjectFormat = function() {
  /** @type {?boolean|undefined} */
  this.negative;

  /** @type {?string|undefined} */
  this.integerPart;

  /** @type {?string|undefined} */
  this.fractionalPart;

  /** @type {?string|undefined} */
  this.quantity;

  /** @type {?string|undefined} */
  this.exponent;

  /** @type {?number|undefined} */
  this.style;

  /** @type {?string|undefined} */
  this.morphosyntacticFeatures;

  /** @type {?proto.speech.tts.MorphosyntacticFeatureVector.ObjectFormat|undefined} */
  this.structuredFeatures;

  /** @type {?boolean|undefined} */
  this.preserveOrder;

  /** @type {?string|undefined} */
  this.codeSwitch;

  /** @type {?Array<string>|undefined} */
  this.fieldOrderList;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.Decimal.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.Decimal}
 */
proto.speech.tts.Decimal.fromObject = function(obj) {
  var msg = new proto.speech.tts.Decimal();
  obj.negative != null && jspb.Message.setField(msg, 1, obj.negative);
  obj.integerPart != null && jspb.Message.setField(msg, 2, obj.integerPart);
  obj.fractionalPart != null && jspb.Message.setField(msg, 3, obj.fractionalPart);
  obj.quantity != null && jspb.Message.setField(msg, 4, obj.quantity);
  obj.exponent != null && jspb.Message.setField(msg, 5, obj.exponent);
  obj.style != null && jspb.Message.setField(msg, 6, obj.style);
  obj.morphosyntacticFeatures != null && jspb.Message.setField(msg, 7, obj.morphosyntacticFeatures);
  obj.structuredFeatures && jspb.Message.setWrapperField(
      msg, 11, proto.speech.tts.MorphosyntacticFeatureVector.fromObject(obj.structuredFeatures));
  obj.preserveOrder != null && jspb.Message.setField(msg, 8, obj.preserveOrder);
  obj.codeSwitch != null && jspb.Message.setField(msg, 9, obj.codeSwitch);
  obj.fieldOrderList != null && jspb.Message.setField(msg, 10, obj.fieldOrderList);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.Decimal}
 */
proto.speech.tts.Decimal.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.Decimal;
  return proto.speech.tts.Decimal.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.Decimal} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.Decimal}
 */
proto.speech.tts.Decimal.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setNegative(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setIntegerPart(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setFractionalPart(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setQuantity(value);
      break;
    case 5:
      var value = /** @type {string} */ (reader.readString());
      msg.setExponent(value);
      break;
    case 6:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setStyle(value);
      break;
    case 7:
      var value = /** @type {string} */ (reader.readString());
      msg.setMorphosyntacticFeatures(value);
      break;
    case 11:
      var value = new proto.speech.tts.MorphosyntacticFeatureVector;
      reader.readMessage(value,proto.speech.tts.MorphosyntacticFeatureVector.deserializeBinaryFromReader);
      msg.setStructuredFeatures(value);
      break;
    case 8:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setPreserveOrder(value);
      break;
    case 9:
      var value = /** @type {string} */ (reader.readString());
      msg.setCodeSwitch(value);
      break;
    case 10:
      var value = /** @type {string} */ (reader.readString());
      msg.addFieldOrder(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.Decimal.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.Decimal.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.Decimal} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Decimal.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeBool(
      1,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeString(
      3,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeString(
      4,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeString(
      5,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeInt32(
      6,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 7));
  if (f != null) {
    writer.writeString(
      7,
      f
    );
  }
  f = message.getStructuredFeatures();
  if (f != null) {
    writer.writeMessage(
      11,
      f,
      proto.speech.tts.MorphosyntacticFeatureVector.serializeBinaryToWriter
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 8));
  if (f != null) {
    writer.writeBool(
      8,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 9));
  if (f != null) {
    writer.writeString(
      9,
      f
    );
  }
  f = message.getFieldOrderList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      10,
      f
    );
  }
};


/**
 * optional bool negative = 1;
 * @return {?boolean}
 */
proto.speech.tts.Decimal.prototype.getNegative = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 1));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.Decimal} returns this
 */
proto.speech.tts.Decimal.prototype.setNegative = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Decimal} returns this
 */
proto.speech.tts.Decimal.prototype.clearNegative = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Decimal.prototype.hasNegative = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional string integer_part = 2;
 * @return {?string}
 */
proto.speech.tts.Decimal.prototype.getIntegerPart = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Decimal} returns this
 */
proto.speech.tts.Decimal.prototype.setIntegerPart = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Decimal} returns this
 */
proto.speech.tts.Decimal.prototype.clearIntegerPart = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Decimal.prototype.hasIntegerPart = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional string fractional_part = 3;
 * @return {?string}
 */
proto.speech.tts.Decimal.prototype.getFractionalPart = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Decimal} returns this
 */
proto.speech.tts.Decimal.prototype.setFractionalPart = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Decimal} returns this
 */
proto.speech.tts.Decimal.prototype.clearFractionalPart = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Decimal.prototype.hasFractionalPart = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional string quantity = 4;
 * @return {?string}
 */
proto.speech.tts.Decimal.prototype.getQuantity = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 4));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Decimal} returns this
 */
proto.speech.tts.Decimal.prototype.setQuantity = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Decimal} returns this
 */
proto.speech.tts.Decimal.prototype.clearQuantity = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Decimal.prototype.hasQuantity = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional string exponent = 5;
 * @return {?string}
 */
proto.speech.tts.Decimal.prototype.getExponent = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 5));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Decimal} returns this
 */
proto.speech.tts.Decimal.prototype.setExponent = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Decimal} returns this
 */
proto.speech.tts.Decimal.prototype.clearExponent = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Decimal.prototype.hasExponent = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional int32 style = 6;
 * @return {?number}
 */
proto.speech.tts.Decimal.prototype.getStyle = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 6));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Decimal} returns this
 */
proto.speech.tts.Decimal.prototype.setStyle = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Decimal} returns this
 */
proto.speech.tts.Decimal.prototype.clearStyle = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Decimal.prototype.hasStyle = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional string morphosyntactic_features = 7;
 * @return {?string}
 */
proto.speech.tts.Decimal.prototype.getMorphosyntacticFeatures = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 7));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Decimal} returns this
 */
proto.speech.tts.Decimal.prototype.setMorphosyntacticFeatures = function(value) {
  return jspb.Message.setField(this, 7, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Decimal} returns this
 */
proto.speech.tts.Decimal.prototype.clearMorphosyntacticFeatures = function() {
  return jspb.Message.setField(this, 7, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Decimal.prototype.hasMorphosyntacticFeatures = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * optional MorphosyntacticFeatureVector structured_features = 11;
 * @return {?proto.speech.tts.MorphosyntacticFeatureVector}
 */
proto.speech.tts.Decimal.prototype.getStructuredFeatures = function() {
  return /** @type{?proto.speech.tts.MorphosyntacticFeatureVector} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.MorphosyntacticFeatureVector, 11));
};


/**
 * @param {?proto.speech.tts.MorphosyntacticFeatureVector|undefined} value
 * @return {!proto.speech.tts.Decimal} returns this
*/
proto.speech.tts.Decimal.prototype.setStructuredFeatures = function(value) {
  return jspb.Message.setWrapperField(this, 11, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Decimal} returns this
 */
proto.speech.tts.Decimal.prototype.clearStructuredFeatures = function() {
  return this.setStructuredFeatures(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Decimal.prototype.hasStructuredFeatures = function() {
  return jspb.Message.getField(this, 11) != null;
};


/**
 * optional bool preserve_order = 8;
 * @return {?boolean}
 */
proto.speech.tts.Decimal.prototype.getPreserveOrder = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 8));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.Decimal} returns this
 */
proto.speech.tts.Decimal.prototype.setPreserveOrder = function(value) {
  return jspb.Message.setField(this, 8, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Decimal} returns this
 */
proto.speech.tts.Decimal.prototype.clearPreserveOrder = function() {
  return jspb.Message.setField(this, 8, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Decimal.prototype.hasPreserveOrder = function() {
  return jspb.Message.getField(this, 8) != null;
};


/**
 * optional string code_switch = 9;
 * @return {?string}
 */
proto.speech.tts.Decimal.prototype.getCodeSwitch = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 9));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Decimal} returns this
 */
proto.speech.tts.Decimal.prototype.setCodeSwitch = function(value) {
  return jspb.Message.setField(this, 9, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Decimal} returns this
 */
proto.speech.tts.Decimal.prototype.clearCodeSwitch = function() {
  return jspb.Message.setField(this, 9, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Decimal.prototype.hasCodeSwitch = function() {
  return jspb.Message.getField(this, 9) != null;
};


/**
 * repeated string field_order = 10;
 * @return {!Array<string>}
 */
proto.speech.tts.Decimal.prototype.getFieldOrderList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 10));
};


/**
 * @param {Array<string>|undefined} value
 * @return {!proto.speech.tts.Decimal} returns this
 */
proto.speech.tts.Decimal.prototype.setFieldOrderList = function(value) {
  return jspb.Message.setField(this, 10, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.Decimal} returns this
 */
proto.speech.tts.Decimal.prototype.addFieldOrder = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 10, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.Decimal} returns this
 */
proto.speech.tts.Decimal.prototype.clearFieldOrderList = function() {
  return this.setFieldOrderList([]);
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.Decimal}
 */
proto.speech.tts.Decimal.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.Decimal, data);
};



//speech/tts/public/semiotic_classes_proto_jspb/fraction.jspb.js
// source: speech/tts/public/semiotic_classes.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.Fraction');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.MorphosyntacticFeatureVector');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.Fraction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.Fraction.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.Fraction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.Fraction.displayName = 'proto.speech.tts.Fraction';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.Fraction.repeatedFields_ = [9];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.Fraction.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.Fraction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.Fraction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Fraction.toObject = function(includeInstance, msg) {
  var f, obj = {
    integerPart: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    numerator: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    denominator: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,
    style: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f,
    morphosyntacticFeatures: (f = jspb.Message.getField(msg, 5)) == null ? undefined : f,
    structuredFeatures: (f = msg.getStructuredFeatures()) && proto.speech.tts.MorphosyntacticFeatureVector.toObject(includeInstance, f),
    preserveOrder: (f = jspb.Message.getBooleanField(msg, 6)) == null ? undefined : f,
    codeSwitch: (f = jspb.Message.getField(msg, 7)) == null ? undefined : f,
    negative: (f = jspb.Message.getBooleanField(msg, 8)) == null ? undefined : f,
    fieldOrderList: (f = jspb.Message.getRepeatedField(msg, 9)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Fraction as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.Fraction.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.integerPart;

  /** @type {?string|undefined} */
  this.numerator;

  /** @type {?string|undefined} */
  this.denominator;

  /** @type {?number|undefined} */
  this.style;

  /** @type {?string|undefined} */
  this.morphosyntacticFeatures;

  /** @type {?proto.speech.tts.MorphosyntacticFeatureVector.ObjectFormat|undefined} */
  this.structuredFeatures;

  /** @type {?boolean|undefined} */
  this.preserveOrder;

  /** @type {?string|undefined} */
  this.codeSwitch;

  /** @type {?boolean|undefined} */
  this.negative;

  /** @type {?Array<string>|undefined} */
  this.fieldOrderList;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.Fraction.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.Fraction}
 */
proto.speech.tts.Fraction.fromObject = function(obj) {
  var msg = new proto.speech.tts.Fraction();
  obj.integerPart != null && jspb.Message.setField(msg, 1, obj.integerPart);
  obj.numerator != null && jspb.Message.setField(msg, 2, obj.numerator);
  obj.denominator != null && jspb.Message.setField(msg, 3, obj.denominator);
  obj.style != null && jspb.Message.setField(msg, 4, obj.style);
  obj.morphosyntacticFeatures != null && jspb.Message.setField(msg, 5, obj.morphosyntacticFeatures);
  obj.structuredFeatures && jspb.Message.setWrapperField(
      msg, 10, proto.speech.tts.MorphosyntacticFeatureVector.fromObject(obj.structuredFeatures));
  obj.preserveOrder != null && jspb.Message.setField(msg, 6, obj.preserveOrder);
  obj.codeSwitch != null && jspb.Message.setField(msg, 7, obj.codeSwitch);
  obj.negative != null && jspb.Message.setField(msg, 8, obj.negative);
  obj.fieldOrderList != null && jspb.Message.setField(msg, 9, obj.fieldOrderList);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.Fraction}
 */
proto.speech.tts.Fraction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.Fraction;
  return proto.speech.tts.Fraction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.Fraction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.Fraction}
 */
proto.speech.tts.Fraction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setIntegerPart(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setNumerator(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setDenominator(value);
      break;
    case 4:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setStyle(value);
      break;
    case 5:
      var value = /** @type {string} */ (reader.readString());
      msg.setMorphosyntacticFeatures(value);
      break;
    case 10:
      var value = new proto.speech.tts.MorphosyntacticFeatureVector;
      reader.readMessage(value,proto.speech.tts.MorphosyntacticFeatureVector.deserializeBinaryFromReader);
      msg.setStructuredFeatures(value);
      break;
    case 6:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setPreserveOrder(value);
      break;
    case 7:
      var value = /** @type {string} */ (reader.readString());
      msg.setCodeSwitch(value);
      break;
    case 8:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setNegative(value);
      break;
    case 9:
      var value = /** @type {string} */ (reader.readString());
      msg.addFieldOrder(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.Fraction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.Fraction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.Fraction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Fraction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeString(
      3,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeInt32(
      4,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeString(
      5,
      f
    );
  }
  f = message.getStructuredFeatures();
  if (f != null) {
    writer.writeMessage(
      10,
      f,
      proto.speech.tts.MorphosyntacticFeatureVector.serializeBinaryToWriter
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeBool(
      6,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 7));
  if (f != null) {
    writer.writeString(
      7,
      f
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 8));
  if (f != null) {
    writer.writeBool(
      8,
      f
    );
  }
  f = message.getFieldOrderList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      9,
      f
    );
  }
};


/**
 * optional string integer_part = 1;
 * @return {?string}
 */
proto.speech.tts.Fraction.prototype.getIntegerPart = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Fraction} returns this
 */
proto.speech.tts.Fraction.prototype.setIntegerPart = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Fraction} returns this
 */
proto.speech.tts.Fraction.prototype.clearIntegerPart = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Fraction.prototype.hasIntegerPart = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * required string numerator = 2;
 * @return {string}
 */
proto.speech.tts.Fraction.prototype.getNumerator = function() {
  return /** @type {string} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Fraction} returns this
 */
proto.speech.tts.Fraction.prototype.setNumerator = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Fraction} returns this
 */
proto.speech.tts.Fraction.prototype.clearNumerator = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Fraction.prototype.hasNumerator = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * required string denominator = 3;
 * @return {string}
 */
proto.speech.tts.Fraction.prototype.getDenominator = function() {
  return /** @type {string} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Fraction} returns this
 */
proto.speech.tts.Fraction.prototype.setDenominator = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Fraction} returns this
 */
proto.speech.tts.Fraction.prototype.clearDenominator = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Fraction.prototype.hasDenominator = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional int32 style = 4;
 * @return {?number}
 */
proto.speech.tts.Fraction.prototype.getStyle = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 4));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Fraction} returns this
 */
proto.speech.tts.Fraction.prototype.setStyle = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Fraction} returns this
 */
proto.speech.tts.Fraction.prototype.clearStyle = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Fraction.prototype.hasStyle = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional string morphosyntactic_features = 5;
 * @return {?string}
 */
proto.speech.tts.Fraction.prototype.getMorphosyntacticFeatures = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 5));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Fraction} returns this
 */
proto.speech.tts.Fraction.prototype.setMorphosyntacticFeatures = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Fraction} returns this
 */
proto.speech.tts.Fraction.prototype.clearMorphosyntacticFeatures = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Fraction.prototype.hasMorphosyntacticFeatures = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional MorphosyntacticFeatureVector structured_features = 10;
 * @return {?proto.speech.tts.MorphosyntacticFeatureVector}
 */
proto.speech.tts.Fraction.prototype.getStructuredFeatures = function() {
  return /** @type{?proto.speech.tts.MorphosyntacticFeatureVector} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.MorphosyntacticFeatureVector, 10));
};


/**
 * @param {?proto.speech.tts.MorphosyntacticFeatureVector|undefined} value
 * @return {!proto.speech.tts.Fraction} returns this
*/
proto.speech.tts.Fraction.prototype.setStructuredFeatures = function(value) {
  return jspb.Message.setWrapperField(this, 10, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Fraction} returns this
 */
proto.speech.tts.Fraction.prototype.clearStructuredFeatures = function() {
  return this.setStructuredFeatures(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Fraction.prototype.hasStructuredFeatures = function() {
  return jspb.Message.getField(this, 10) != null;
};


/**
 * optional bool preserve_order = 6;
 * @return {?boolean}
 */
proto.speech.tts.Fraction.prototype.getPreserveOrder = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 6));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.Fraction} returns this
 */
proto.speech.tts.Fraction.prototype.setPreserveOrder = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Fraction} returns this
 */
proto.speech.tts.Fraction.prototype.clearPreserveOrder = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Fraction.prototype.hasPreserveOrder = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional string code_switch = 7;
 * @return {?string}
 */
proto.speech.tts.Fraction.prototype.getCodeSwitch = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 7));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Fraction} returns this
 */
proto.speech.tts.Fraction.prototype.setCodeSwitch = function(value) {
  return jspb.Message.setField(this, 7, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Fraction} returns this
 */
proto.speech.tts.Fraction.prototype.clearCodeSwitch = function() {
  return jspb.Message.setField(this, 7, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Fraction.prototype.hasCodeSwitch = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * optional bool negative = 8;
 * @return {?boolean}
 */
proto.speech.tts.Fraction.prototype.getNegative = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 8));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.Fraction} returns this
 */
proto.speech.tts.Fraction.prototype.setNegative = function(value) {
  return jspb.Message.setField(this, 8, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Fraction} returns this
 */
proto.speech.tts.Fraction.prototype.clearNegative = function() {
  return jspb.Message.setField(this, 8, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Fraction.prototype.hasNegative = function() {
  return jspb.Message.getField(this, 8) != null;
};


/**
 * repeated string field_order = 9;
 * @return {!Array<string>}
 */
proto.speech.tts.Fraction.prototype.getFieldOrderList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 9));
};


/**
 * @param {Array<string>|undefined} value
 * @return {!proto.speech.tts.Fraction} returns this
 */
proto.speech.tts.Fraction.prototype.setFieldOrderList = function(value) {
  return jspb.Message.setField(this, 9, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.Fraction} returns this
 */
proto.speech.tts.Fraction.prototype.addFieldOrder = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 9, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.Fraction} returns this
 */
proto.speech.tts.Fraction.prototype.clearFieldOrderList = function() {
  return this.setFieldOrderList([]);
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.Fraction}
 */
proto.speech.tts.Fraction.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.Fraction, data);
};



//speech/tts/public/semiotic_classes_proto_jspb/measure.jspb.js
// source: speech/tts/public/semiotic_classes.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.Measure');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.Cardinal');
goog.require('proto.speech.tts.Decimal');
goog.require('proto.speech.tts.Fraction');
goog.require('proto.speech.tts.MorphosyntacticFeatureVector');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.Measure = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.Measure.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.Measure, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.Measure.displayName = 'proto.speech.tts.Measure';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.Measure.repeatedFields_ = [9];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.Measure.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.Measure.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.Measure} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Measure.toObject = function(includeInstance, msg) {
  var f, obj = {
    decimal: (f = msg.getDecimal()) && proto.speech.tts.Decimal.toObject(includeInstance, f),
    fraction: (f = msg.getFraction()) && proto.speech.tts.Fraction.toObject(includeInstance, f),
    cardinal: (f = msg.getCardinal()) && proto.speech.tts.Cardinal.toObject(includeInstance, f),
    units: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f,
    style: (f = jspb.Message.getField(msg, 5)) == null ? undefined : f,
    morphosyntacticFeatures: (f = jspb.Message.getField(msg, 6)) == null ? undefined : f,
    structuredFeatures: (f = msg.getStructuredFeatures()) && proto.speech.tts.MorphosyntacticFeatureVector.toObject(includeInstance, f),
    preserveOrder: (f = jspb.Message.getBooleanField(msg, 7)) == null ? undefined : f,
    codeSwitch: (f = jspb.Message.getField(msg, 8)) == null ? undefined : f,
    fieldOrderList: (f = jspb.Message.getRepeatedField(msg, 9)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Measure as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.Measure.ObjectFormat = function() {
  /** @type {?proto.speech.tts.Decimal.ObjectFormat|undefined} */
  this.decimal;

  /** @type {?proto.speech.tts.Fraction.ObjectFormat|undefined} */
  this.fraction;

  /** @type {?proto.speech.tts.Cardinal.ObjectFormat|undefined} */
  this.cardinal;

  /** @type {?string|undefined} */
  this.units;

  /** @type {?number|undefined} */
  this.style;

  /** @type {?string|undefined} */
  this.morphosyntacticFeatures;

  /** @type {?proto.speech.tts.MorphosyntacticFeatureVector.ObjectFormat|undefined} */
  this.structuredFeatures;

  /** @type {?boolean|undefined} */
  this.preserveOrder;

  /** @type {?string|undefined} */
  this.codeSwitch;

  /** @type {?Array<string>|undefined} */
  this.fieldOrderList;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.Measure.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.Measure}
 */
proto.speech.tts.Measure.fromObject = function(obj) {
  var msg = new proto.speech.tts.Measure();
  obj.decimal && jspb.Message.setWrapperField(
      msg, 1, proto.speech.tts.Decimal.fromObject(obj.decimal));
  obj.fraction && jspb.Message.setWrapperField(
      msg, 2, proto.speech.tts.Fraction.fromObject(obj.fraction));
  obj.cardinal && jspb.Message.setWrapperField(
      msg, 3, proto.speech.tts.Cardinal.fromObject(obj.cardinal));
  obj.units != null && jspb.Message.setField(msg, 4, obj.units);
  obj.style != null && jspb.Message.setField(msg, 5, obj.style);
  obj.morphosyntacticFeatures != null && jspb.Message.setField(msg, 6, obj.morphosyntacticFeatures);
  obj.structuredFeatures && jspb.Message.setWrapperField(
      msg, 10, proto.speech.tts.MorphosyntacticFeatureVector.fromObject(obj.structuredFeatures));
  obj.preserveOrder != null && jspb.Message.setField(msg, 7, obj.preserveOrder);
  obj.codeSwitch != null && jspb.Message.setField(msg, 8, obj.codeSwitch);
  obj.fieldOrderList != null && jspb.Message.setField(msg, 9, obj.fieldOrderList);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.Measure}
 */
proto.speech.tts.Measure.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.Measure;
  return proto.speech.tts.Measure.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.Measure} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.Measure}
 */
proto.speech.tts.Measure.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.speech.tts.Decimal;
      reader.readMessage(value,proto.speech.tts.Decimal.deserializeBinaryFromReader);
      msg.setDecimal(value);
      break;
    case 2:
      var value = new proto.speech.tts.Fraction;
      reader.readMessage(value,proto.speech.tts.Fraction.deserializeBinaryFromReader);
      msg.setFraction(value);
      break;
    case 3:
      var value = new proto.speech.tts.Cardinal;
      reader.readMessage(value,proto.speech.tts.Cardinal.deserializeBinaryFromReader);
      msg.setCardinal(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setUnits(value);
      break;
    case 5:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setStyle(value);
      break;
    case 6:
      var value = /** @type {string} */ (reader.readString());
      msg.setMorphosyntacticFeatures(value);
      break;
    case 10:
      var value = new proto.speech.tts.MorphosyntacticFeatureVector;
      reader.readMessage(value,proto.speech.tts.MorphosyntacticFeatureVector.deserializeBinaryFromReader);
      msg.setStructuredFeatures(value);
      break;
    case 7:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setPreserveOrder(value);
      break;
    case 8:
      var value = /** @type {string} */ (reader.readString());
      msg.setCodeSwitch(value);
      break;
    case 9:
      var value = /** @type {string} */ (reader.readString());
      msg.addFieldOrder(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.Measure.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.Measure.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.Measure} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Measure.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getDecimal();
  if (f != null) {
    writer.writeMessage(
      1,
      f,
      proto.speech.tts.Decimal.serializeBinaryToWriter
    );
  }
  f = message.getFraction();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.speech.tts.Fraction.serializeBinaryToWriter
    );
  }
  f = message.getCardinal();
  if (f != null) {
    writer.writeMessage(
      3,
      f,
      proto.speech.tts.Cardinal.serializeBinaryToWriter
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeString(
      4,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeInt32(
      5,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeString(
      6,
      f
    );
  }
  f = message.getStructuredFeatures();
  if (f != null) {
    writer.writeMessage(
      10,
      f,
      proto.speech.tts.MorphosyntacticFeatureVector.serializeBinaryToWriter
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 7));
  if (f != null) {
    writer.writeBool(
      7,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 8));
  if (f != null) {
    writer.writeString(
      8,
      f
    );
  }
  f = message.getFieldOrderList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      9,
      f
    );
  }
};


/**
 * optional Decimal decimal = 1;
 * @return {?proto.speech.tts.Decimal}
 */
proto.speech.tts.Measure.prototype.getDecimal = function() {
  return /** @type{?proto.speech.tts.Decimal} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Decimal, 1));
};


/**
 * @param {?proto.speech.tts.Decimal|undefined} value
 * @return {!proto.speech.tts.Measure} returns this
*/
proto.speech.tts.Measure.prototype.setDecimal = function(value) {
  return jspb.Message.setWrapperField(this, 1, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Measure} returns this
 */
proto.speech.tts.Measure.prototype.clearDecimal = function() {
  return this.setDecimal(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Measure.prototype.hasDecimal = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional Fraction fraction = 2;
 * @return {?proto.speech.tts.Fraction}
 */
proto.speech.tts.Measure.prototype.getFraction = function() {
  return /** @type{?proto.speech.tts.Fraction} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Fraction, 2));
};


/**
 * @param {?proto.speech.tts.Fraction|undefined} value
 * @return {!proto.speech.tts.Measure} returns this
*/
proto.speech.tts.Measure.prototype.setFraction = function(value) {
  return jspb.Message.setWrapperField(this, 2, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Measure} returns this
 */
proto.speech.tts.Measure.prototype.clearFraction = function() {
  return this.setFraction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Measure.prototype.hasFraction = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional Cardinal cardinal = 3;
 * @return {?proto.speech.tts.Cardinal}
 */
proto.speech.tts.Measure.prototype.getCardinal = function() {
  return /** @type{?proto.speech.tts.Cardinal} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Cardinal, 3));
};


/**
 * @param {?proto.speech.tts.Cardinal|undefined} value
 * @return {!proto.speech.tts.Measure} returns this
*/
proto.speech.tts.Measure.prototype.setCardinal = function(value) {
  return jspb.Message.setWrapperField(this, 3, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Measure} returns this
 */
proto.speech.tts.Measure.prototype.clearCardinal = function() {
  return this.setCardinal(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Measure.prototype.hasCardinal = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional string units = 4;
 * @return {?string}
 */
proto.speech.tts.Measure.prototype.getUnits = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 4));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Measure} returns this
 */
proto.speech.tts.Measure.prototype.setUnits = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Measure} returns this
 */
proto.speech.tts.Measure.prototype.clearUnits = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Measure.prototype.hasUnits = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional int32 style = 5;
 * @return {?number}
 */
proto.speech.tts.Measure.prototype.getStyle = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 5));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Measure} returns this
 */
proto.speech.tts.Measure.prototype.setStyle = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Measure} returns this
 */
proto.speech.tts.Measure.prototype.clearStyle = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Measure.prototype.hasStyle = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional string morphosyntactic_features = 6;
 * @return {?string}
 */
proto.speech.tts.Measure.prototype.getMorphosyntacticFeatures = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 6));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Measure} returns this
 */
proto.speech.tts.Measure.prototype.setMorphosyntacticFeatures = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Measure} returns this
 */
proto.speech.tts.Measure.prototype.clearMorphosyntacticFeatures = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Measure.prototype.hasMorphosyntacticFeatures = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional MorphosyntacticFeatureVector structured_features = 10;
 * @return {?proto.speech.tts.MorphosyntacticFeatureVector}
 */
proto.speech.tts.Measure.prototype.getStructuredFeatures = function() {
  return /** @type{?proto.speech.tts.MorphosyntacticFeatureVector} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.MorphosyntacticFeatureVector, 10));
};


/**
 * @param {?proto.speech.tts.MorphosyntacticFeatureVector|undefined} value
 * @return {!proto.speech.tts.Measure} returns this
*/
proto.speech.tts.Measure.prototype.setStructuredFeatures = function(value) {
  return jspb.Message.setWrapperField(this, 10, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Measure} returns this
 */
proto.speech.tts.Measure.prototype.clearStructuredFeatures = function() {
  return this.setStructuredFeatures(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Measure.prototype.hasStructuredFeatures = function() {
  return jspb.Message.getField(this, 10) != null;
};


/**
 * optional bool preserve_order = 7;
 * @return {?boolean}
 */
proto.speech.tts.Measure.prototype.getPreserveOrder = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 7));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.Measure} returns this
 */
proto.speech.tts.Measure.prototype.setPreserveOrder = function(value) {
  return jspb.Message.setField(this, 7, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Measure} returns this
 */
proto.speech.tts.Measure.prototype.clearPreserveOrder = function() {
  return jspb.Message.setField(this, 7, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Measure.prototype.hasPreserveOrder = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * optional string code_switch = 8;
 * @return {?string}
 */
proto.speech.tts.Measure.prototype.getCodeSwitch = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 8));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Measure} returns this
 */
proto.speech.tts.Measure.prototype.setCodeSwitch = function(value) {
  return jspb.Message.setField(this, 8, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Measure} returns this
 */
proto.speech.tts.Measure.prototype.clearCodeSwitch = function() {
  return jspb.Message.setField(this, 8, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Measure.prototype.hasCodeSwitch = function() {
  return jspb.Message.getField(this, 8) != null;
};


/**
 * repeated string field_order = 9;
 * @return {!Array<string>}
 */
proto.speech.tts.Measure.prototype.getFieldOrderList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 9));
};


/**
 * @param {Array<string>|undefined} value
 * @return {!proto.speech.tts.Measure} returns this
 */
proto.speech.tts.Measure.prototype.setFieldOrderList = function(value) {
  return jspb.Message.setField(this, 9, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.Measure} returns this
 */
proto.speech.tts.Measure.prototype.addFieldOrder = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 9, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.Measure} returns this
 */
proto.speech.tts.Measure.prototype.clearFieldOrderList = function() {
  return this.setFieldOrderList([]);
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.Measure}
 */
proto.speech.tts.Measure.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.Measure, data);
};



//speech/tts/public/semiotic_classes_proto_jspb/money.jspb.js
// source: speech/tts/public/semiotic_classes.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.Money');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.Decimal');
goog.require('proto.speech.tts.MorphosyntacticFeatureVector');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.Money = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.Money.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.Money, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.Money.displayName = 'proto.speech.tts.Money';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.Money.repeatedFields_ = [8];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.Money.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.Money.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.Money} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Money.toObject = function(includeInstance, msg) {
  var f, obj = {
    amount: (f = msg.getAmount()) && proto.speech.tts.Decimal.toObject(includeInstance, f),
    quantity: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    stringQuantity: (f = jspb.Message.getField(msg, 10)) == null ? undefined : f,
    currency: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,
    style: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f,
    morphosyntacticFeatures: (f = jspb.Message.getField(msg, 5)) == null ? undefined : f,
    structuredFeatures: (f = msg.getStructuredFeatures()) && proto.speech.tts.MorphosyntacticFeatureVector.toObject(includeInstance, f),
    preserveOrder: (f = jspb.Message.getBooleanField(msg, 6)) == null ? undefined : f,
    codeSwitch: (f = jspb.Message.getField(msg, 7)) == null ? undefined : f,
    fieldOrderList: (f = jspb.Message.getRepeatedField(msg, 8)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Money as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.Money.ObjectFormat = function() {
  /** @type {?proto.speech.tts.Decimal.ObjectFormat|undefined} */
  this.amount;

  /** @type {?number|undefined} */
  this.quantity;

  /** @type {?string|undefined} */
  this.stringQuantity;

  /** @type {?string|undefined} */
  this.currency;

  /** @type {?number|undefined} */
  this.style;

  /** @type {?string|undefined} */
  this.morphosyntacticFeatures;

  /** @type {?proto.speech.tts.MorphosyntacticFeatureVector.ObjectFormat|undefined} */
  this.structuredFeatures;

  /** @type {?boolean|undefined} */
  this.preserveOrder;

  /** @type {?string|undefined} */
  this.codeSwitch;

  /** @type {?Array<string>|undefined} */
  this.fieldOrderList;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.Money.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.Money}
 */
proto.speech.tts.Money.fromObject = function(obj) {
  var msg = new proto.speech.tts.Money();
  obj.amount && jspb.Message.setWrapperField(
      msg, 1, proto.speech.tts.Decimal.fromObject(obj.amount));
  obj.quantity != null && jspb.Message.setField(msg, 2, obj.quantity);
  obj.stringQuantity != null && jspb.Message.setField(msg, 10, obj.stringQuantity);
  obj.currency != null && jspb.Message.setField(msg, 3, obj.currency);
  obj.style != null && jspb.Message.setField(msg, 4, obj.style);
  obj.morphosyntacticFeatures != null && jspb.Message.setField(msg, 5, obj.morphosyntacticFeatures);
  obj.structuredFeatures && jspb.Message.setWrapperField(
      msg, 9, proto.speech.tts.MorphosyntacticFeatureVector.fromObject(obj.structuredFeatures));
  obj.preserveOrder != null && jspb.Message.setField(msg, 6, obj.preserveOrder);
  obj.codeSwitch != null && jspb.Message.setField(msg, 7, obj.codeSwitch);
  obj.fieldOrderList != null && jspb.Message.setField(msg, 8, obj.fieldOrderList);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.Money}
 */
proto.speech.tts.Money.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.Money;
  return proto.speech.tts.Money.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.Money} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.Money}
 */
proto.speech.tts.Money.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.speech.tts.Decimal;
      reader.readMessage(value,proto.speech.tts.Decimal.deserializeBinaryFromReader);
      msg.setAmount(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setQuantity(value);
      break;
    case 10:
      var value = /** @type {string} */ (reader.readString());
      msg.setStringQuantity(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setCurrency(value);
      break;
    case 4:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setStyle(value);
      break;
    case 5:
      var value = /** @type {string} */ (reader.readString());
      msg.setMorphosyntacticFeatures(value);
      break;
    case 9:
      var value = new proto.speech.tts.MorphosyntacticFeatureVector;
      reader.readMessage(value,proto.speech.tts.MorphosyntacticFeatureVector.deserializeBinaryFromReader);
      msg.setStructuredFeatures(value);
      break;
    case 6:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setPreserveOrder(value);
      break;
    case 7:
      var value = /** @type {string} */ (reader.readString());
      msg.setCodeSwitch(value);
      break;
    case 8:
      var value = /** @type {string} */ (reader.readString());
      msg.addFieldOrder(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.Money.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.Money.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.Money} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Money.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getAmount();
  if (f != null) {
    writer.writeMessage(
      1,
      f,
      proto.speech.tts.Decimal.serializeBinaryToWriter
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeInt64(
      2,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 10));
  if (f != null) {
    writer.writeString(
      10,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeString(
      3,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeInt32(
      4,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeString(
      5,
      f
    );
  }
  f = message.getStructuredFeatures();
  if (f != null) {
    writer.writeMessage(
      9,
      f,
      proto.speech.tts.MorphosyntacticFeatureVector.serializeBinaryToWriter
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeBool(
      6,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 7));
  if (f != null) {
    writer.writeString(
      7,
      f
    );
  }
  f = message.getFieldOrderList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      8,
      f
    );
  }
};


/**
 * required Decimal amount = 1;
 * @return {!proto.speech.tts.Decimal}
 */
proto.speech.tts.Money.prototype.getAmount = function() {
  return /** @type{!proto.speech.tts.Decimal} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Decimal, 1, 1));
};


/**
 * @param {?proto.speech.tts.Decimal|undefined} value
 * @return {!proto.speech.tts.Money} returns this
*/
proto.speech.tts.Money.prototype.setAmount = function(value) {
  return jspb.Message.setWrapperField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Money} returns this
 */
proto.speech.tts.Money.prototype.clearAmount = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Money.prototype.hasAmount = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional int64 quantity = 2;
 * @return {?number}
 */
proto.speech.tts.Money.prototype.getQuantity = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Money} returns this
 */
proto.speech.tts.Money.prototype.setQuantity = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Money} returns this
 */
proto.speech.tts.Money.prototype.clearQuantity = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Money.prototype.hasQuantity = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional string string_quantity = 10;
 * @return {?string}
 */
proto.speech.tts.Money.prototype.getStringQuantity = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 10));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Money} returns this
 */
proto.speech.tts.Money.prototype.setStringQuantity = function(value) {
  return jspb.Message.setField(this, 10, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Money} returns this
 */
proto.speech.tts.Money.prototype.clearStringQuantity = function() {
  return jspb.Message.setField(this, 10, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Money.prototype.hasStringQuantity = function() {
  return jspb.Message.getField(this, 10) != null;
};


/**
 * required string currency = 3;
 * @return {string}
 */
proto.speech.tts.Money.prototype.getCurrency = function() {
  return /** @type {string} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Money} returns this
 */
proto.speech.tts.Money.prototype.setCurrency = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Money} returns this
 */
proto.speech.tts.Money.prototype.clearCurrency = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Money.prototype.hasCurrency = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional int32 style = 4;
 * @return {?number}
 */
proto.speech.tts.Money.prototype.getStyle = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 4));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Money} returns this
 */
proto.speech.tts.Money.prototype.setStyle = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Money} returns this
 */
proto.speech.tts.Money.prototype.clearStyle = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Money.prototype.hasStyle = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional string morphosyntactic_features = 5;
 * @return {?string}
 */
proto.speech.tts.Money.prototype.getMorphosyntacticFeatures = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 5));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Money} returns this
 */
proto.speech.tts.Money.prototype.setMorphosyntacticFeatures = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Money} returns this
 */
proto.speech.tts.Money.prototype.clearMorphosyntacticFeatures = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Money.prototype.hasMorphosyntacticFeatures = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional MorphosyntacticFeatureVector structured_features = 9;
 * @return {?proto.speech.tts.MorphosyntacticFeatureVector}
 */
proto.speech.tts.Money.prototype.getStructuredFeatures = function() {
  return /** @type{?proto.speech.tts.MorphosyntacticFeatureVector} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.MorphosyntacticFeatureVector, 9));
};


/**
 * @param {?proto.speech.tts.MorphosyntacticFeatureVector|undefined} value
 * @return {!proto.speech.tts.Money} returns this
*/
proto.speech.tts.Money.prototype.setStructuredFeatures = function(value) {
  return jspb.Message.setWrapperField(this, 9, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Money} returns this
 */
proto.speech.tts.Money.prototype.clearStructuredFeatures = function() {
  return this.setStructuredFeatures(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Money.prototype.hasStructuredFeatures = function() {
  return jspb.Message.getField(this, 9) != null;
};


/**
 * optional bool preserve_order = 6;
 * @return {?boolean}
 */
proto.speech.tts.Money.prototype.getPreserveOrder = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 6));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.Money} returns this
 */
proto.speech.tts.Money.prototype.setPreserveOrder = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Money} returns this
 */
proto.speech.tts.Money.prototype.clearPreserveOrder = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Money.prototype.hasPreserveOrder = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional string code_switch = 7;
 * @return {?string}
 */
proto.speech.tts.Money.prototype.getCodeSwitch = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 7));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Money} returns this
 */
proto.speech.tts.Money.prototype.setCodeSwitch = function(value) {
  return jspb.Message.setField(this, 7, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Money} returns this
 */
proto.speech.tts.Money.prototype.clearCodeSwitch = function() {
  return jspb.Message.setField(this, 7, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Money.prototype.hasCodeSwitch = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * repeated string field_order = 8;
 * @return {!Array<string>}
 */
proto.speech.tts.Money.prototype.getFieldOrderList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 8));
};


/**
 * @param {Array<string>|undefined} value
 * @return {!proto.speech.tts.Money} returns this
 */
proto.speech.tts.Money.prototype.setFieldOrderList = function(value) {
  return jspb.Message.setField(this, 8, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.Money} returns this
 */
proto.speech.tts.Money.prototype.addFieldOrder = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 8, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.Money} returns this
 */
proto.speech.tts.Money.prototype.clearFieldOrderList = function() {
  return this.setFieldOrderList([]);
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.Money}
 */
proto.speech.tts.Money.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.Money, data);
};



//speech/tts/public/semiotic_classes_proto_jspb/ordinal.jspb.js
// source: speech/tts/public/semiotic_classes.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.Ordinal');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.MorphosyntacticFeatureVector');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.Ordinal = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.Ordinal.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.Ordinal, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.Ordinal.displayName = 'proto.speech.tts.Ordinal';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.Ordinal.repeatedFields_ = [5];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.Ordinal.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.Ordinal.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.Ordinal} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Ordinal.toObject = function(includeInstance, msg) {
  var f, obj = {
    integer: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    morphosyntacticFeatures: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    structuredFeatures: (f = msg.getStructuredFeatures()) && proto.speech.tts.MorphosyntacticFeatureVector.toObject(includeInstance, f),
    preserveOrder: (f = jspb.Message.getBooleanField(msg, 3)) == null ? undefined : f,
    codeSwitch: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f,
    fieldOrderList: (f = jspb.Message.getRepeatedField(msg, 5)) == null ? undefined : f,
    definiteArticle: (f = jspb.Message.getBooleanField(msg, 7)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Ordinal as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.Ordinal.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.integer;

  /** @type {?string|undefined} */
  this.morphosyntacticFeatures;

  /** @type {?proto.speech.tts.MorphosyntacticFeatureVector.ObjectFormat|undefined} */
  this.structuredFeatures;

  /** @type {?boolean|undefined} */
  this.preserveOrder;

  /** @type {?string|undefined} */
  this.codeSwitch;

  /** @type {?Array<string>|undefined} */
  this.fieldOrderList;

  /** @type {?boolean|undefined} */
  this.definiteArticle;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.Ordinal.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.Ordinal}
 */
proto.speech.tts.Ordinal.fromObject = function(obj) {
  var msg = new proto.speech.tts.Ordinal();
  obj.integer != null && jspb.Message.setField(msg, 1, obj.integer);
  obj.morphosyntacticFeatures != null && jspb.Message.setField(msg, 2, obj.morphosyntacticFeatures);
  obj.structuredFeatures && jspb.Message.setWrapperField(
      msg, 6, proto.speech.tts.MorphosyntacticFeatureVector.fromObject(obj.structuredFeatures));
  obj.preserveOrder != null && jspb.Message.setField(msg, 3, obj.preserveOrder);
  obj.codeSwitch != null && jspb.Message.setField(msg, 4, obj.codeSwitch);
  obj.fieldOrderList != null && jspb.Message.setField(msg, 5, obj.fieldOrderList);
  obj.definiteArticle != null && jspb.Message.setField(msg, 7, obj.definiteArticle);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.Ordinal}
 */
proto.speech.tts.Ordinal.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.Ordinal;
  return proto.speech.tts.Ordinal.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.Ordinal} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.Ordinal}
 */
proto.speech.tts.Ordinal.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setInteger(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setMorphosyntacticFeatures(value);
      break;
    case 6:
      var value = new proto.speech.tts.MorphosyntacticFeatureVector;
      reader.readMessage(value,proto.speech.tts.MorphosyntacticFeatureVector.deserializeBinaryFromReader);
      msg.setStructuredFeatures(value);
      break;
    case 3:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setPreserveOrder(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setCodeSwitch(value);
      break;
    case 5:
      var value = /** @type {string} */ (reader.readString());
      msg.addFieldOrder(value);
      break;
    case 7:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setDefiniteArticle(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.Ordinal.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.Ordinal.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.Ordinal} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Ordinal.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
  f = message.getStructuredFeatures();
  if (f != null) {
    writer.writeMessage(
      6,
      f,
      proto.speech.tts.MorphosyntacticFeatureVector.serializeBinaryToWriter
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeBool(
      3,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeString(
      4,
      f
    );
  }
  f = message.getFieldOrderList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      5,
      f
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 7));
  if (f != null) {
    writer.writeBool(
      7,
      f
    );
  }
};


/**
 * required string integer = 1;
 * @return {string}
 */
proto.speech.tts.Ordinal.prototype.getInteger = function() {
  return /** @type {string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Ordinal} returns this
 */
proto.speech.tts.Ordinal.prototype.setInteger = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Ordinal} returns this
 */
proto.speech.tts.Ordinal.prototype.clearInteger = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Ordinal.prototype.hasInteger = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional string morphosyntactic_features = 2;
 * @return {?string}
 */
proto.speech.tts.Ordinal.prototype.getMorphosyntacticFeatures = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Ordinal} returns this
 */
proto.speech.tts.Ordinal.prototype.setMorphosyntacticFeatures = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Ordinal} returns this
 */
proto.speech.tts.Ordinal.prototype.clearMorphosyntacticFeatures = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Ordinal.prototype.hasMorphosyntacticFeatures = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional MorphosyntacticFeatureVector structured_features = 6;
 * @return {?proto.speech.tts.MorphosyntacticFeatureVector}
 */
proto.speech.tts.Ordinal.prototype.getStructuredFeatures = function() {
  return /** @type{?proto.speech.tts.MorphosyntacticFeatureVector} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.MorphosyntacticFeatureVector, 6));
};


/**
 * @param {?proto.speech.tts.MorphosyntacticFeatureVector|undefined} value
 * @return {!proto.speech.tts.Ordinal} returns this
*/
proto.speech.tts.Ordinal.prototype.setStructuredFeatures = function(value) {
  return jspb.Message.setWrapperField(this, 6, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Ordinal} returns this
 */
proto.speech.tts.Ordinal.prototype.clearStructuredFeatures = function() {
  return this.setStructuredFeatures(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Ordinal.prototype.hasStructuredFeatures = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional bool preserve_order = 3;
 * @return {?boolean}
 */
proto.speech.tts.Ordinal.prototype.getPreserveOrder = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 3));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.Ordinal} returns this
 */
proto.speech.tts.Ordinal.prototype.setPreserveOrder = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Ordinal} returns this
 */
proto.speech.tts.Ordinal.prototype.clearPreserveOrder = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Ordinal.prototype.hasPreserveOrder = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional string code_switch = 4;
 * @return {?string}
 */
proto.speech.tts.Ordinal.prototype.getCodeSwitch = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 4));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Ordinal} returns this
 */
proto.speech.tts.Ordinal.prototype.setCodeSwitch = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Ordinal} returns this
 */
proto.speech.tts.Ordinal.prototype.clearCodeSwitch = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Ordinal.prototype.hasCodeSwitch = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * repeated string field_order = 5;
 * @return {!Array<string>}
 */
proto.speech.tts.Ordinal.prototype.getFieldOrderList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 5));
};


/**
 * @param {Array<string>|undefined} value
 * @return {!proto.speech.tts.Ordinal} returns this
 */
proto.speech.tts.Ordinal.prototype.setFieldOrderList = function(value) {
  return jspb.Message.setField(this, 5, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.Ordinal} returns this
 */
proto.speech.tts.Ordinal.prototype.addFieldOrder = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 5, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.Ordinal} returns this
 */
proto.speech.tts.Ordinal.prototype.clearFieldOrderList = function() {
  return this.setFieldOrderList([]);
};


/**
 * optional bool definite_article = 7;
 * @return {?boolean}
 */
proto.speech.tts.Ordinal.prototype.getDefiniteArticle = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 7));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.Ordinal} returns this
 */
proto.speech.tts.Ordinal.prototype.setDefiniteArticle = function(value) {
  return jspb.Message.setField(this, 7, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Ordinal} returns this
 */
proto.speech.tts.Ordinal.prototype.clearDefiniteArticle = function() {
  return jspb.Message.setField(this, 7, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Ordinal.prototype.hasDefiniteArticle = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.Ordinal}
 */
proto.speech.tts.Ordinal.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.Ordinal, data);
};



//speech/tts/public/semiotic_classes_proto_jspb/time.jspb.js
// source: speech/tts/public/semiotic_classes.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.Time');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.MorphosyntacticFeatureVector');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.Time = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.Time.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.Time, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.Time.displayName = 'proto.speech.tts.Time';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.Time.repeatedFields_ = [12];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.Time.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.Time.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.Time} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Time.toObject = function(includeInstance, msg) {
  var f, obj = {
    hours: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    minutes: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    seconds: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,
    milliseconds: (f = jspb.Message.getField(msg, 8)) == null ? undefined : f,
    speakPeriod: (f = jspb.Message.getBooleanField(msg, 4)) == null ? undefined : f,
    suffix: (f = jspb.Message.getField(msg, 5)) == null ? undefined : f,
    style: (f = jspb.Message.getField(msg, 6)) == null ? undefined : f,
    zone: (f = jspb.Message.getField(msg, 7)) == null ? undefined : f,
    morphosyntacticFeatures: (f = jspb.Message.getField(msg, 9)) == null ? undefined : f,
    structuredFeatures: (f = msg.getStructuredFeatures()) && proto.speech.tts.MorphosyntacticFeatureVector.toObject(includeInstance, f),
    preserveOrder: (f = jspb.Message.getBooleanField(msg, 10)) == null ? undefined : f,
    codeSwitch: (f = jspb.Message.getField(msg, 11)) == null ? undefined : f,
    fieldOrderList: (f = jspb.Message.getRepeatedField(msg, 12)) == null ? undefined : f,
    duration: (f = jspb.Message.getBooleanField(msg, 14)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Time as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.Time.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.hours;

  /** @type {?number|undefined} */
  this.minutes;

  /** @type {?number|undefined} */
  this.seconds;

  /** @type {?number|undefined} */
  this.milliseconds;

  /** @type {?boolean|undefined} */
  this.speakPeriod;

  /** @type {?string|undefined} */
  this.suffix;

  /** @type {?number|undefined} */
  this.style;

  /** @type {?string|undefined} */
  this.zone;

  /** @type {?string|undefined} */
  this.morphosyntacticFeatures;

  /** @type {?proto.speech.tts.MorphosyntacticFeatureVector.ObjectFormat|undefined} */
  this.structuredFeatures;

  /** @type {?boolean|undefined} */
  this.preserveOrder;

  /** @type {?string|undefined} */
  this.codeSwitch;

  /** @type {?Array<string>|undefined} */
  this.fieldOrderList;

  /** @type {?boolean|undefined} */
  this.duration;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.Time.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.Time}
 */
proto.speech.tts.Time.fromObject = function(obj) {
  var msg = new proto.speech.tts.Time();
  obj.hours != null && jspb.Message.setField(msg, 1, obj.hours);
  obj.minutes != null && jspb.Message.setField(msg, 2, obj.minutes);
  obj.seconds != null && jspb.Message.setField(msg, 3, obj.seconds);
  obj.milliseconds != null && jspb.Message.setField(msg, 8, obj.milliseconds);
  obj.speakPeriod != null && jspb.Message.setField(msg, 4, obj.speakPeriod);
  obj.suffix != null && jspb.Message.setField(msg, 5, obj.suffix);
  obj.style != null && jspb.Message.setField(msg, 6, obj.style);
  obj.zone != null && jspb.Message.setField(msg, 7, obj.zone);
  obj.morphosyntacticFeatures != null && jspb.Message.setField(msg, 9, obj.morphosyntacticFeatures);
  obj.structuredFeatures && jspb.Message.setWrapperField(
      msg, 13, proto.speech.tts.MorphosyntacticFeatureVector.fromObject(obj.structuredFeatures));
  obj.preserveOrder != null && jspb.Message.setField(msg, 10, obj.preserveOrder);
  obj.codeSwitch != null && jspb.Message.setField(msg, 11, obj.codeSwitch);
  obj.fieldOrderList != null && jspb.Message.setField(msg, 12, obj.fieldOrderList);
  obj.duration != null && jspb.Message.setField(msg, 14, obj.duration);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.Time}
 */
proto.speech.tts.Time.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.Time;
  return proto.speech.tts.Time.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.Time} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.Time}
 */
proto.speech.tts.Time.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setHours(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setMinutes(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setSeconds(value);
      break;
    case 8:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setMilliseconds(value);
      break;
    case 4:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setSpeakPeriod(value);
      break;
    case 5:
      var value = /** @type {string} */ (reader.readString());
      msg.setSuffix(value);
      break;
    case 6:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setStyle(value);
      break;
    case 7:
      var value = /** @type {string} */ (reader.readString());
      msg.setZone(value);
      break;
    case 9:
      var value = /** @type {string} */ (reader.readString());
      msg.setMorphosyntacticFeatures(value);
      break;
    case 13:
      var value = new proto.speech.tts.MorphosyntacticFeatureVector;
      reader.readMessage(value,proto.speech.tts.MorphosyntacticFeatureVector.deserializeBinaryFromReader);
      msg.setStructuredFeatures(value);
      break;
    case 10:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setPreserveOrder(value);
      break;
    case 11:
      var value = /** @type {string} */ (reader.readString());
      msg.setCodeSwitch(value);
      break;
    case 12:
      var value = /** @type {string} */ (reader.readString());
      msg.addFieldOrder(value);
      break;
    case 14:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setDuration(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.Time.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.Time.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.Time} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Time.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?number} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeInt32(
      1,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeInt32(
      2,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeInt32(
      3,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 8));
  if (f != null) {
    writer.writeInt32(
      8,
      f
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeBool(
      4,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeString(
      5,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeInt32(
      6,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 7));
  if (f != null) {
    writer.writeString(
      7,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 9));
  if (f != null) {
    writer.writeString(
      9,
      f
    );
  }
  f = message.getStructuredFeatures();
  if (f != null) {
    writer.writeMessage(
      13,
      f,
      proto.speech.tts.MorphosyntacticFeatureVector.serializeBinaryToWriter
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 10));
  if (f != null) {
    writer.writeBool(
      10,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 11));
  if (f != null) {
    writer.writeString(
      11,
      f
    );
  }
  f = message.getFieldOrderList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      12,
      f
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 14));
  if (f != null) {
    writer.writeBool(
      14,
      f
    );
  }
};


/**
 * optional int32 hours = 1;
 * @return {?number}
 */
proto.speech.tts.Time.prototype.getHours = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Time} returns this
 */
proto.speech.tts.Time.prototype.setHours = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Time} returns this
 */
proto.speech.tts.Time.prototype.clearHours = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Time.prototype.hasHours = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional int32 minutes = 2;
 * @return {?number}
 */
proto.speech.tts.Time.prototype.getMinutes = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Time} returns this
 */
proto.speech.tts.Time.prototype.setMinutes = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Time} returns this
 */
proto.speech.tts.Time.prototype.clearMinutes = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Time.prototype.hasMinutes = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional int32 seconds = 3;
 * @return {?number}
 */
proto.speech.tts.Time.prototype.getSeconds = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Time} returns this
 */
proto.speech.tts.Time.prototype.setSeconds = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Time} returns this
 */
proto.speech.tts.Time.prototype.clearSeconds = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Time.prototype.hasSeconds = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional int32 milliseconds = 8;
 * @return {?number}
 */
proto.speech.tts.Time.prototype.getMilliseconds = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 8));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Time} returns this
 */
proto.speech.tts.Time.prototype.setMilliseconds = function(value) {
  return jspb.Message.setField(this, 8, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Time} returns this
 */
proto.speech.tts.Time.prototype.clearMilliseconds = function() {
  return jspb.Message.setField(this, 8, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Time.prototype.hasMilliseconds = function() {
  return jspb.Message.getField(this, 8) != null;
};


/**
 * optional bool speak_period = 4;
 * @return {?boolean}
 */
proto.speech.tts.Time.prototype.getSpeakPeriod = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 4));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.Time} returns this
 */
proto.speech.tts.Time.prototype.setSpeakPeriod = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Time} returns this
 */
proto.speech.tts.Time.prototype.clearSpeakPeriod = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Time.prototype.hasSpeakPeriod = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional string suffix = 5;
 * @return {?string}
 */
proto.speech.tts.Time.prototype.getSuffix = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 5));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Time} returns this
 */
proto.speech.tts.Time.prototype.setSuffix = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Time} returns this
 */
proto.speech.tts.Time.prototype.clearSuffix = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Time.prototype.hasSuffix = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional int32 style = 6;
 * @return {?number}
 */
proto.speech.tts.Time.prototype.getStyle = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 6));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Time} returns this
 */
proto.speech.tts.Time.prototype.setStyle = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Time} returns this
 */
proto.speech.tts.Time.prototype.clearStyle = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Time.prototype.hasStyle = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional string zone = 7;
 * @return {?string}
 */
proto.speech.tts.Time.prototype.getZone = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 7));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Time} returns this
 */
proto.speech.tts.Time.prototype.setZone = function(value) {
  return jspb.Message.setField(this, 7, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Time} returns this
 */
proto.speech.tts.Time.prototype.clearZone = function() {
  return jspb.Message.setField(this, 7, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Time.prototype.hasZone = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * optional string morphosyntactic_features = 9;
 * @return {?string}
 */
proto.speech.tts.Time.prototype.getMorphosyntacticFeatures = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 9));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Time} returns this
 */
proto.speech.tts.Time.prototype.setMorphosyntacticFeatures = function(value) {
  return jspb.Message.setField(this, 9, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Time} returns this
 */
proto.speech.tts.Time.prototype.clearMorphosyntacticFeatures = function() {
  return jspb.Message.setField(this, 9, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Time.prototype.hasMorphosyntacticFeatures = function() {
  return jspb.Message.getField(this, 9) != null;
};


/**
 * optional MorphosyntacticFeatureVector structured_features = 13;
 * @return {?proto.speech.tts.MorphosyntacticFeatureVector}
 */
proto.speech.tts.Time.prototype.getStructuredFeatures = function() {
  return /** @type{?proto.speech.tts.MorphosyntacticFeatureVector} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.MorphosyntacticFeatureVector, 13));
};


/**
 * @param {?proto.speech.tts.MorphosyntacticFeatureVector|undefined} value
 * @return {!proto.speech.tts.Time} returns this
*/
proto.speech.tts.Time.prototype.setStructuredFeatures = function(value) {
  return jspb.Message.setWrapperField(this, 13, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Time} returns this
 */
proto.speech.tts.Time.prototype.clearStructuredFeatures = function() {
  return this.setStructuredFeatures(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Time.prototype.hasStructuredFeatures = function() {
  return jspb.Message.getField(this, 13) != null;
};


/**
 * optional bool preserve_order = 10;
 * @return {?boolean}
 */
proto.speech.tts.Time.prototype.getPreserveOrder = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 10));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.Time} returns this
 */
proto.speech.tts.Time.prototype.setPreserveOrder = function(value) {
  return jspb.Message.setField(this, 10, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Time} returns this
 */
proto.speech.tts.Time.prototype.clearPreserveOrder = function() {
  return jspb.Message.setField(this, 10, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Time.prototype.hasPreserveOrder = function() {
  return jspb.Message.getField(this, 10) != null;
};


/**
 * optional string code_switch = 11;
 * @return {?string}
 */
proto.speech.tts.Time.prototype.getCodeSwitch = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 11));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Time} returns this
 */
proto.speech.tts.Time.prototype.setCodeSwitch = function(value) {
  return jspb.Message.setField(this, 11, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Time} returns this
 */
proto.speech.tts.Time.prototype.clearCodeSwitch = function() {
  return jspb.Message.setField(this, 11, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Time.prototype.hasCodeSwitch = function() {
  return jspb.Message.getField(this, 11) != null;
};


/**
 * repeated string field_order = 12;
 * @return {!Array<string>}
 */
proto.speech.tts.Time.prototype.getFieldOrderList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 12));
};


/**
 * @param {Array<string>|undefined} value
 * @return {!proto.speech.tts.Time} returns this
 */
proto.speech.tts.Time.prototype.setFieldOrderList = function(value) {
  return jspb.Message.setField(this, 12, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.Time} returns this
 */
proto.speech.tts.Time.prototype.addFieldOrder = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 12, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.Time} returns this
 */
proto.speech.tts.Time.prototype.clearFieldOrderList = function() {
  return this.setFieldOrderList([]);
};


/**
 * optional bool duration = 14;
 * @return {?boolean}
 */
proto.speech.tts.Time.prototype.getDuration = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 14));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.Time} returns this
 */
proto.speech.tts.Time.prototype.setDuration = function(value) {
  return jspb.Message.setField(this, 14, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Time} returns this
 */
proto.speech.tts.Time.prototype.clearDuration = function() {
  return jspb.Message.setField(this, 14, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Time.prototype.hasDuration = function() {
  return jspb.Message.getField(this, 14) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.Time}
 */
proto.speech.tts.Time.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.Time, data);
};



//speech/tts/public/semiotic_classes_proto_jspb/duplicator.jspb.js
// source: speech/tts/public/semiotic_classes.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.Duplicator');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.Date');
goog.require('proto.speech.tts.Decimal');
goog.require('proto.speech.tts.Fraction');
goog.require('proto.speech.tts.Measure');
goog.require('proto.speech.tts.Money');
goog.require('proto.speech.tts.Ordinal');
goog.require('proto.speech.tts.Time');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.Duplicator = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.Duplicator.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.Duplicator, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.Duplicator.displayName = 'proto.speech.tts.Duplicator';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.Duplicator.repeatedFields_ = [1,2,3,4,5,6,7];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.Duplicator.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.Duplicator.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.Duplicator} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Duplicator.toObject = function(includeInstance, msg) {
  var f, obj = {
    dateList: jspb.Message.toObjectList(msg.getDateList(),
    proto.speech.tts.Date.toObject, includeInstance),
    decimalList: jspb.Message.toObjectList(msg.getDecimalList(),
    proto.speech.tts.Decimal.toObject, includeInstance),
    fractionList: jspb.Message.toObjectList(msg.getFractionList(),
    proto.speech.tts.Fraction.toObject, includeInstance),
    measureList: jspb.Message.toObjectList(msg.getMeasureList(),
    proto.speech.tts.Measure.toObject, includeInstance),
    moneyList: jspb.Message.toObjectList(msg.getMoneyList(),
    proto.speech.tts.Money.toObject, includeInstance),
    ordinalList: jspb.Message.toObjectList(msg.getOrdinalList(),
    proto.speech.tts.Ordinal.toObject, includeInstance),
    timeList: jspb.Message.toObjectList(msg.getTimeList(),
    proto.speech.tts.Time.toObject, includeInstance)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Duplicator as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.Duplicator.ObjectFormat = function() {
  /** @type {?Array<!proto.speech.tts.Date.ObjectFormat>|undefined} */
  this.dateList;

  /** @type {?Array<!proto.speech.tts.Decimal.ObjectFormat>|undefined} */
  this.decimalList;

  /** @type {?Array<!proto.speech.tts.Fraction.ObjectFormat>|undefined} */
  this.fractionList;

  /** @type {?Array<!proto.speech.tts.Measure.ObjectFormat>|undefined} */
  this.measureList;

  /** @type {?Array<!proto.speech.tts.Money.ObjectFormat>|undefined} */
  this.moneyList;

  /** @type {?Array<!proto.speech.tts.Ordinal.ObjectFormat>|undefined} */
  this.ordinalList;

  /** @type {?Array<!proto.speech.tts.Time.ObjectFormat>|undefined} */
  this.timeList;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.Duplicator.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.Duplicator}
 */
proto.speech.tts.Duplicator.fromObject = function(obj) {
  var msg = new proto.speech.tts.Duplicator();
  obj.dateList && jspb.Message.setRepeatedWrapperField(
      msg, 1, obj.dateList.map(
          proto.speech.tts.Date.fromObject));
  obj.decimalList && jspb.Message.setRepeatedWrapperField(
      msg, 2, obj.decimalList.map(
          proto.speech.tts.Decimal.fromObject));
  obj.fractionList && jspb.Message.setRepeatedWrapperField(
      msg, 3, obj.fractionList.map(
          proto.speech.tts.Fraction.fromObject));
  obj.measureList && jspb.Message.setRepeatedWrapperField(
      msg, 4, obj.measureList.map(
          proto.speech.tts.Measure.fromObject));
  obj.moneyList && jspb.Message.setRepeatedWrapperField(
      msg, 5, obj.moneyList.map(
          proto.speech.tts.Money.fromObject));
  obj.ordinalList && jspb.Message.setRepeatedWrapperField(
      msg, 6, obj.ordinalList.map(
          proto.speech.tts.Ordinal.fromObject));
  obj.timeList && jspb.Message.setRepeatedWrapperField(
      msg, 7, obj.timeList.map(
          proto.speech.tts.Time.fromObject));
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.Duplicator}
 */
proto.speech.tts.Duplicator.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.Duplicator;
  return proto.speech.tts.Duplicator.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.Duplicator} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.Duplicator}
 */
proto.speech.tts.Duplicator.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.speech.tts.Date;
      reader.readMessage(value,proto.speech.tts.Date.deserializeBinaryFromReader);
      msg.addDate(value);
      break;
    case 2:
      var value = new proto.speech.tts.Decimal;
      reader.readMessage(value,proto.speech.tts.Decimal.deserializeBinaryFromReader);
      msg.addDecimal(value);
      break;
    case 3:
      var value = new proto.speech.tts.Fraction;
      reader.readMessage(value,proto.speech.tts.Fraction.deserializeBinaryFromReader);
      msg.addFraction(value);
      break;
    case 4:
      var value = new proto.speech.tts.Measure;
      reader.readMessage(value,proto.speech.tts.Measure.deserializeBinaryFromReader);
      msg.addMeasure(value);
      break;
    case 5:
      var value = new proto.speech.tts.Money;
      reader.readMessage(value,proto.speech.tts.Money.deserializeBinaryFromReader);
      msg.addMoney(value);
      break;
    case 6:
      var value = new proto.speech.tts.Ordinal;
      reader.readMessage(value,proto.speech.tts.Ordinal.deserializeBinaryFromReader);
      msg.addOrdinal(value);
      break;
    case 7:
      var value = new proto.speech.tts.Time;
      reader.readMessage(value,proto.speech.tts.Time.deserializeBinaryFromReader);
      msg.addTime(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.Duplicator.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.Duplicator.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.Duplicator} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Duplicator.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getDateList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      1,
      f,
      proto.speech.tts.Date.serializeBinaryToWriter
    );
  }
  f = message.getDecimalList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      2,
      f,
      proto.speech.tts.Decimal.serializeBinaryToWriter
    );
  }
  f = message.getFractionList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      3,
      f,
      proto.speech.tts.Fraction.serializeBinaryToWriter
    );
  }
  f = message.getMeasureList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      4,
      f,
      proto.speech.tts.Measure.serializeBinaryToWriter
    );
  }
  f = message.getMoneyList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      5,
      f,
      proto.speech.tts.Money.serializeBinaryToWriter
    );
  }
  f = message.getOrdinalList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      6,
      f,
      proto.speech.tts.Ordinal.serializeBinaryToWriter
    );
  }
  f = message.getTimeList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      7,
      f,
      proto.speech.tts.Time.serializeBinaryToWriter
    );
  }
};


/**
 * repeated Date date = 1;
 * @return {!Array<!proto.speech.tts.Date>}
 */
proto.speech.tts.Duplicator.prototype.getDateList = function() {
  return /** @type{!Array<!proto.speech.tts.Date>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.Date, 1));
};


/**
 * @param {Array<!proto.speech.tts.Date>|undefined} value
 * @return {!proto.speech.tts.Duplicator} returns this
*/
proto.speech.tts.Duplicator.prototype.setDateList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 1, value);
};


/**
 * @param {!proto.speech.tts.Date=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.Date}
 */
proto.speech.tts.Duplicator.prototype.addDate = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.speech.tts.Date, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.Duplicator} returns this
 */
proto.speech.tts.Duplicator.prototype.clearDateList = function() {
  return this.setDateList([]);
};


/**
 * repeated Decimal decimal = 2;
 * @return {!Array<!proto.speech.tts.Decimal>}
 */
proto.speech.tts.Duplicator.prototype.getDecimalList = function() {
  return /** @type{!Array<!proto.speech.tts.Decimal>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.Decimal, 2));
};


/**
 * @param {Array<!proto.speech.tts.Decimal>|undefined} value
 * @return {!proto.speech.tts.Duplicator} returns this
*/
proto.speech.tts.Duplicator.prototype.setDecimalList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 2, value);
};


/**
 * @param {!proto.speech.tts.Decimal=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.Decimal}
 */
proto.speech.tts.Duplicator.prototype.addDecimal = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 2, opt_value, proto.speech.tts.Decimal, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.Duplicator} returns this
 */
proto.speech.tts.Duplicator.prototype.clearDecimalList = function() {
  return this.setDecimalList([]);
};


/**
 * repeated Fraction fraction = 3;
 * @return {!Array<!proto.speech.tts.Fraction>}
 */
proto.speech.tts.Duplicator.prototype.getFractionList = function() {
  return /** @type{!Array<!proto.speech.tts.Fraction>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.Fraction, 3));
};


/**
 * @param {Array<!proto.speech.tts.Fraction>|undefined} value
 * @return {!proto.speech.tts.Duplicator} returns this
*/
proto.speech.tts.Duplicator.prototype.setFractionList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 3, value);
};


/**
 * @param {!proto.speech.tts.Fraction=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.Fraction}
 */
proto.speech.tts.Duplicator.prototype.addFraction = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 3, opt_value, proto.speech.tts.Fraction, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.Duplicator} returns this
 */
proto.speech.tts.Duplicator.prototype.clearFractionList = function() {
  return this.setFractionList([]);
};


/**
 * repeated Measure measure = 4;
 * @return {!Array<!proto.speech.tts.Measure>}
 */
proto.speech.tts.Duplicator.prototype.getMeasureList = function() {
  return /** @type{!Array<!proto.speech.tts.Measure>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.Measure, 4));
};


/**
 * @param {Array<!proto.speech.tts.Measure>|undefined} value
 * @return {!proto.speech.tts.Duplicator} returns this
*/
proto.speech.tts.Duplicator.prototype.setMeasureList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 4, value);
};


/**
 * @param {!proto.speech.tts.Measure=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.Measure}
 */
proto.speech.tts.Duplicator.prototype.addMeasure = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 4, opt_value, proto.speech.tts.Measure, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.Duplicator} returns this
 */
proto.speech.tts.Duplicator.prototype.clearMeasureList = function() {
  return this.setMeasureList([]);
};


/**
 * repeated Money money = 5;
 * @return {!Array<!proto.speech.tts.Money>}
 */
proto.speech.tts.Duplicator.prototype.getMoneyList = function() {
  return /** @type{!Array<!proto.speech.tts.Money>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.Money, 5));
};


/**
 * @param {Array<!proto.speech.tts.Money>|undefined} value
 * @return {!proto.speech.tts.Duplicator} returns this
*/
proto.speech.tts.Duplicator.prototype.setMoneyList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 5, value);
};


/**
 * @param {!proto.speech.tts.Money=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.Money}
 */
proto.speech.tts.Duplicator.prototype.addMoney = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 5, opt_value, proto.speech.tts.Money, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.Duplicator} returns this
 */
proto.speech.tts.Duplicator.prototype.clearMoneyList = function() {
  return this.setMoneyList([]);
};


/**
 * repeated Ordinal ordinal = 6;
 * @return {!Array<!proto.speech.tts.Ordinal>}
 */
proto.speech.tts.Duplicator.prototype.getOrdinalList = function() {
  return /** @type{!Array<!proto.speech.tts.Ordinal>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.Ordinal, 6));
};


/**
 * @param {Array<!proto.speech.tts.Ordinal>|undefined} value
 * @return {!proto.speech.tts.Duplicator} returns this
*/
proto.speech.tts.Duplicator.prototype.setOrdinalList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 6, value);
};


/**
 * @param {!proto.speech.tts.Ordinal=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.Ordinal}
 */
proto.speech.tts.Duplicator.prototype.addOrdinal = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 6, opt_value, proto.speech.tts.Ordinal, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.Duplicator} returns this
 */
proto.speech.tts.Duplicator.prototype.clearOrdinalList = function() {
  return this.setOrdinalList([]);
};


/**
 * repeated Time time = 7;
 * @return {!Array<!proto.speech.tts.Time>}
 */
proto.speech.tts.Duplicator.prototype.getTimeList = function() {
  return /** @type{!Array<!proto.speech.tts.Time>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.Time, 7));
};


/**
 * @param {Array<!proto.speech.tts.Time>|undefined} value
 * @return {!proto.speech.tts.Duplicator} returns this
*/
proto.speech.tts.Duplicator.prototype.setTimeList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 7, value);
};


/**
 * @param {!proto.speech.tts.Time=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.Time}
 */
proto.speech.tts.Duplicator.prototype.addTime = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 7, opt_value, proto.speech.tts.Time, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.Duplicator} returns this
 */
proto.speech.tts.Duplicator.prototype.clearTimeList = function() {
  return this.setTimeList([]);
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.Duplicator}
 */
proto.speech.tts.Duplicator.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.Duplicator, data);
};



//speech/tts/public/semiotic_classes_proto_jspb/electronic.jspb.js
// source: speech/tts/public/semiotic_classes.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.Electronic');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.MorphosyntacticFeatureVector');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.Electronic = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.Electronic.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.Electronic, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.Electronic.displayName = 'proto.speech.tts.Electronic';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.Electronic.repeatedFields_ = [12];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.Electronic.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.Electronic.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.Electronic} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Electronic.toObject = function(includeInstance, msg) {
  var f, obj = {
    protocol: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    username: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    password: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,
    domain: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f,
    port: (f = jspb.Message.getField(msg, 5)) == null ? undefined : f,
    path: (f = jspb.Message.getField(msg, 6)) == null ? undefined : f,
    queryString: (f = jspb.Message.getField(msg, 7)) == null ? undefined : f,
    fragmentId: (f = jspb.Message.getField(msg, 8)) == null ? undefined : f,
    morphosyntacticFeatures: (f = jspb.Message.getField(msg, 9)) == null ? undefined : f,
    structuredFeatures: (f = msg.getStructuredFeatures()) && proto.speech.tts.MorphosyntacticFeatureVector.toObject(includeInstance, f),
    preserveOrder: (f = jspb.Message.getBooleanField(msg, 10)) == null ? undefined : f,
    codeSwitch: (f = jspb.Message.getField(msg, 11)) == null ? undefined : f,
    fieldOrderList: (f = jspb.Message.getRepeatedField(msg, 12)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Electronic as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.Electronic.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.protocol;

  /** @type {?string|undefined} */
  this.username;

  /** @type {?string|undefined} */
  this.password;

  /** @type {?string|undefined} */
  this.domain;

  /** @type {?number|undefined} */
  this.port;

  /** @type {?string|undefined} */
  this.path;

  /** @type {?string|undefined} */
  this.queryString;

  /** @type {?string|undefined} */
  this.fragmentId;

  /** @type {?string|undefined} */
  this.morphosyntacticFeatures;

  /** @type {?proto.speech.tts.MorphosyntacticFeatureVector.ObjectFormat|undefined} */
  this.structuredFeatures;

  /** @type {?boolean|undefined} */
  this.preserveOrder;

  /** @type {?string|undefined} */
  this.codeSwitch;

  /** @type {?Array<string>|undefined} */
  this.fieldOrderList;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.Electronic.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.Electronic}
 */
proto.speech.tts.Electronic.fromObject = function(obj) {
  var msg = new proto.speech.tts.Electronic();
  obj.protocol != null && jspb.Message.setField(msg, 1, obj.protocol);
  obj.username != null && jspb.Message.setField(msg, 2, obj.username);
  obj.password != null && jspb.Message.setField(msg, 3, obj.password);
  obj.domain != null && jspb.Message.setField(msg, 4, obj.domain);
  obj.port != null && jspb.Message.setField(msg, 5, obj.port);
  obj.path != null && jspb.Message.setField(msg, 6, obj.path);
  obj.queryString != null && jspb.Message.setField(msg, 7, obj.queryString);
  obj.fragmentId != null && jspb.Message.setField(msg, 8, obj.fragmentId);
  obj.morphosyntacticFeatures != null && jspb.Message.setField(msg, 9, obj.morphosyntacticFeatures);
  obj.structuredFeatures && jspb.Message.setWrapperField(
      msg, 13, proto.speech.tts.MorphosyntacticFeatureVector.fromObject(obj.structuredFeatures));
  obj.preserveOrder != null && jspb.Message.setField(msg, 10, obj.preserveOrder);
  obj.codeSwitch != null && jspb.Message.setField(msg, 11, obj.codeSwitch);
  obj.fieldOrderList != null && jspb.Message.setField(msg, 12, obj.fieldOrderList);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.Electronic}
 */
proto.speech.tts.Electronic.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.Electronic;
  return proto.speech.tts.Electronic.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.Electronic} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.Electronic}
 */
proto.speech.tts.Electronic.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setProtocol(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setUsername(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setPassword(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setDomain(value);
      break;
    case 5:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setPort(value);
      break;
    case 6:
      var value = /** @type {string} */ (reader.readString());
      msg.setPath(value);
      break;
    case 7:
      var value = /** @type {string} */ (reader.readString());
      msg.setQueryString(value);
      break;
    case 8:
      var value = /** @type {string} */ (reader.readString());
      msg.setFragmentId(value);
      break;
    case 9:
      var value = /** @type {string} */ (reader.readString());
      msg.setMorphosyntacticFeatures(value);
      break;
    case 13:
      var value = new proto.speech.tts.MorphosyntacticFeatureVector;
      reader.readMessage(value,proto.speech.tts.MorphosyntacticFeatureVector.deserializeBinaryFromReader);
      msg.setStructuredFeatures(value);
      break;
    case 10:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setPreserveOrder(value);
      break;
    case 11:
      var value = /** @type {string} */ (reader.readString());
      msg.setCodeSwitch(value);
      break;
    case 12:
      var value = /** @type {string} */ (reader.readString());
      msg.addFieldOrder(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.Electronic.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.Electronic.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.Electronic} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Electronic.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeString(
      3,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeString(
      4,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeInt32(
      5,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeString(
      6,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 7));
  if (f != null) {
    writer.writeString(
      7,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 8));
  if (f != null) {
    writer.writeString(
      8,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 9));
  if (f != null) {
    writer.writeString(
      9,
      f
    );
  }
  f = message.getStructuredFeatures();
  if (f != null) {
    writer.writeMessage(
      13,
      f,
      proto.speech.tts.MorphosyntacticFeatureVector.serializeBinaryToWriter
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 10));
  if (f != null) {
    writer.writeBool(
      10,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 11));
  if (f != null) {
    writer.writeString(
      11,
      f
    );
  }
  f = message.getFieldOrderList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      12,
      f
    );
  }
};


/**
 * optional string protocol = 1;
 * @return {?string}
 */
proto.speech.tts.Electronic.prototype.getProtocol = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Electronic} returns this
 */
proto.speech.tts.Electronic.prototype.setProtocol = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Electronic} returns this
 */
proto.speech.tts.Electronic.prototype.clearProtocol = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Electronic.prototype.hasProtocol = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional string username = 2;
 * @return {?string}
 */
proto.speech.tts.Electronic.prototype.getUsername = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Electronic} returns this
 */
proto.speech.tts.Electronic.prototype.setUsername = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Electronic} returns this
 */
proto.speech.tts.Electronic.prototype.clearUsername = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Electronic.prototype.hasUsername = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional string password = 3;
 * @return {?string}
 */
proto.speech.tts.Electronic.prototype.getPassword = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Electronic} returns this
 */
proto.speech.tts.Electronic.prototype.setPassword = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Electronic} returns this
 */
proto.speech.tts.Electronic.prototype.clearPassword = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Electronic.prototype.hasPassword = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional string domain = 4;
 * @return {?string}
 */
proto.speech.tts.Electronic.prototype.getDomain = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 4));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Electronic} returns this
 */
proto.speech.tts.Electronic.prototype.setDomain = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Electronic} returns this
 */
proto.speech.tts.Electronic.prototype.clearDomain = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Electronic.prototype.hasDomain = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional int32 port = 5;
 * @return {?number}
 */
proto.speech.tts.Electronic.prototype.getPort = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 5));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Electronic} returns this
 */
proto.speech.tts.Electronic.prototype.setPort = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Electronic} returns this
 */
proto.speech.tts.Electronic.prototype.clearPort = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Electronic.prototype.hasPort = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional string path = 6;
 * @return {?string}
 */
proto.speech.tts.Electronic.prototype.getPath = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 6));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Electronic} returns this
 */
proto.speech.tts.Electronic.prototype.setPath = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Electronic} returns this
 */
proto.speech.tts.Electronic.prototype.clearPath = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Electronic.prototype.hasPath = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional string query_string = 7;
 * @return {?string}
 */
proto.speech.tts.Electronic.prototype.getQueryString = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 7));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Electronic} returns this
 */
proto.speech.tts.Electronic.prototype.setQueryString = function(value) {
  return jspb.Message.setField(this, 7, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Electronic} returns this
 */
proto.speech.tts.Electronic.prototype.clearQueryString = function() {
  return jspb.Message.setField(this, 7, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Electronic.prototype.hasQueryString = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * optional string fragment_id = 8;
 * @return {?string}
 */
proto.speech.tts.Electronic.prototype.getFragmentId = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 8));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Electronic} returns this
 */
proto.speech.tts.Electronic.prototype.setFragmentId = function(value) {
  return jspb.Message.setField(this, 8, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Electronic} returns this
 */
proto.speech.tts.Electronic.prototype.clearFragmentId = function() {
  return jspb.Message.setField(this, 8, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Electronic.prototype.hasFragmentId = function() {
  return jspb.Message.getField(this, 8) != null;
};


/**
 * optional string morphosyntactic_features = 9;
 * @return {?string}
 */
proto.speech.tts.Electronic.prototype.getMorphosyntacticFeatures = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 9));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Electronic} returns this
 */
proto.speech.tts.Electronic.prototype.setMorphosyntacticFeatures = function(value) {
  return jspb.Message.setField(this, 9, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Electronic} returns this
 */
proto.speech.tts.Electronic.prototype.clearMorphosyntacticFeatures = function() {
  return jspb.Message.setField(this, 9, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Electronic.prototype.hasMorphosyntacticFeatures = function() {
  return jspb.Message.getField(this, 9) != null;
};


/**
 * optional MorphosyntacticFeatureVector structured_features = 13;
 * @return {?proto.speech.tts.MorphosyntacticFeatureVector}
 */
proto.speech.tts.Electronic.prototype.getStructuredFeatures = function() {
  return /** @type{?proto.speech.tts.MorphosyntacticFeatureVector} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.MorphosyntacticFeatureVector, 13));
};


/**
 * @param {?proto.speech.tts.MorphosyntacticFeatureVector|undefined} value
 * @return {!proto.speech.tts.Electronic} returns this
*/
proto.speech.tts.Electronic.prototype.setStructuredFeatures = function(value) {
  return jspb.Message.setWrapperField(this, 13, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Electronic} returns this
 */
proto.speech.tts.Electronic.prototype.clearStructuredFeatures = function() {
  return this.setStructuredFeatures(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Electronic.prototype.hasStructuredFeatures = function() {
  return jspb.Message.getField(this, 13) != null;
};


/**
 * optional bool preserve_order = 10;
 * @return {?boolean}
 */
proto.speech.tts.Electronic.prototype.getPreserveOrder = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 10));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.Electronic} returns this
 */
proto.speech.tts.Electronic.prototype.setPreserveOrder = function(value) {
  return jspb.Message.setField(this, 10, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Electronic} returns this
 */
proto.speech.tts.Electronic.prototype.clearPreserveOrder = function() {
  return jspb.Message.setField(this, 10, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Electronic.prototype.hasPreserveOrder = function() {
  return jspb.Message.getField(this, 10) != null;
};


/**
 * optional string code_switch = 11;
 * @return {?string}
 */
proto.speech.tts.Electronic.prototype.getCodeSwitch = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 11));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Electronic} returns this
 */
proto.speech.tts.Electronic.prototype.setCodeSwitch = function(value) {
  return jspb.Message.setField(this, 11, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Electronic} returns this
 */
proto.speech.tts.Electronic.prototype.clearCodeSwitch = function() {
  return jspb.Message.setField(this, 11, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Electronic.prototype.hasCodeSwitch = function() {
  return jspb.Message.getField(this, 11) != null;
};


/**
 * repeated string field_order = 12;
 * @return {!Array<string>}
 */
proto.speech.tts.Electronic.prototype.getFieldOrderList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 12));
};


/**
 * @param {Array<string>|undefined} value
 * @return {!proto.speech.tts.Electronic} returns this
 */
proto.speech.tts.Electronic.prototype.setFieldOrderList = function(value) {
  return jspb.Message.setField(this, 12, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.Electronic} returns this
 */
proto.speech.tts.Electronic.prototype.addFieldOrder = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 12, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.Electronic} returns this
 */
proto.speech.tts.Electronic.prototype.clearFieldOrderList = function() {
  return this.setFieldOrderList([]);
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.Electronic}
 */
proto.speech.tts.Electronic.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.Electronic, data);
};



//speech/tts/public/semiotic_classes_proto_jspb/emoticonemoji.jspb.js
// source: speech/tts/public/semiotic_classes.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.EmoticonEmoji');
goog.provide('proto.speech.tts.EmoticonEmoji.Type');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.EmoticonEmoji = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.EmoticonEmoji, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.EmoticonEmoji.displayName = 'proto.speech.tts.EmoticonEmoji';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.EmoticonEmoji.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.EmoticonEmoji.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.EmoticonEmoji} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.EmoticonEmoji.toObject = function(includeInstance, msg) {
  var f, obj = {
    text: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    occurrences: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    type: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,
    codeSwitch: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of EmoticonEmoji as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.EmoticonEmoji.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.text;

  /** @type {?number|undefined} */
  this.occurrences;

  /** @type {?number|undefined} */
  this.type;

  /** @type {?string|undefined} */
  this.codeSwitch;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.EmoticonEmoji.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.EmoticonEmoji}
 */
proto.speech.tts.EmoticonEmoji.fromObject = function(obj) {
  var msg = new proto.speech.tts.EmoticonEmoji();
  obj.text != null && jspb.Message.setField(msg, 1, obj.text);
  obj.occurrences != null && jspb.Message.setField(msg, 2, obj.occurrences);
  obj.type != null && jspb.Message.setField(msg, 3, obj.type);
  obj.codeSwitch != null && jspb.Message.setField(msg, 4, obj.codeSwitch);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.EmoticonEmoji}
 */
proto.speech.tts.EmoticonEmoji.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.EmoticonEmoji;
  return proto.speech.tts.EmoticonEmoji.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.EmoticonEmoji} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.EmoticonEmoji}
 */
proto.speech.tts.EmoticonEmoji.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setText(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setOccurrences(value);
      break;
    case 3:
      var value = /** @type {!proto.speech.tts.EmoticonEmoji.Type} */ (reader.readEnum());
      msg.setType(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setCodeSwitch(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.EmoticonEmoji.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.EmoticonEmoji.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.EmoticonEmoji} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.EmoticonEmoji.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeInt32(
      2,
      f
    );
  }
  f = /** @type {!proto.speech.tts.EmoticonEmoji.Type} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeEnum(
      3,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeString(
      4,
      f
    );
  }
};


/**
 * @enum {number}
 */
proto.speech.tts.EmoticonEmoji.Type = {
  TYPE_UNSPECIFIED: 0,
  EMOTICONS: 1,
  EMOJIS: 2
};

/**
 * optional string text = 1;
 * @return {?string}
 */
proto.speech.tts.EmoticonEmoji.prototype.getText = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.EmoticonEmoji} returns this
 */
proto.speech.tts.EmoticonEmoji.prototype.setText = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.EmoticonEmoji} returns this
 */
proto.speech.tts.EmoticonEmoji.prototype.clearText = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.EmoticonEmoji.prototype.hasText = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional int32 occurrences = 2;
 * @return {?number}
 */
proto.speech.tts.EmoticonEmoji.prototype.getOccurrences = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.EmoticonEmoji} returns this
 */
proto.speech.tts.EmoticonEmoji.prototype.setOccurrences = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.EmoticonEmoji} returns this
 */
proto.speech.tts.EmoticonEmoji.prototype.clearOccurrences = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.EmoticonEmoji.prototype.hasOccurrences = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional Type type = 3;
 * @return {!proto.speech.tts.EmoticonEmoji.Type}
 */
proto.speech.tts.EmoticonEmoji.prototype.getType = function() {
  return /** @type {!proto.speech.tts.EmoticonEmoji.Type} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?proto.speech.tts.EmoticonEmoji.Type|undefined} value
 * @return {!proto.speech.tts.EmoticonEmoji} returns this
 */
proto.speech.tts.EmoticonEmoji.prototype.setType = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.EmoticonEmoji} returns this
 */
proto.speech.tts.EmoticonEmoji.prototype.clearType = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.EmoticonEmoji.prototype.hasType = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional string code_switch = 4;
 * @return {?string}
 */
proto.speech.tts.EmoticonEmoji.prototype.getCodeSwitch = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 4));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.EmoticonEmoji} returns this
 */
proto.speech.tts.EmoticonEmoji.prototype.setCodeSwitch = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.EmoticonEmoji} returns this
 */
proto.speech.tts.EmoticonEmoji.prototype.clearCodeSwitch = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.EmoticonEmoji.prototype.hasCodeSwitch = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.EmoticonEmoji}
 */
proto.speech.tts.EmoticonEmoji.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.EmoticonEmoji, data);
};



//speech/tts/public/semiotic_classes_proto_jspb/spelling.jspb.js
// source: speech/tts/public/semiotic_classes.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.Spelling');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.Spelling = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.Spelling, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.Spelling.displayName = 'proto.speech.tts.Spelling';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.Spelling.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.Spelling.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.Spelling} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Spelling.toObject = function(includeInstance, msg) {
  var f, obj = {
    graphemes: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    chunkSize: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    expressCapitalization: (f = jspb.Message.getBooleanField(msg, 3)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Spelling as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.Spelling.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.graphemes;

  /** @type {?number|undefined} */
  this.chunkSize;

  /** @type {?boolean|undefined} */
  this.expressCapitalization;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.Spelling.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.Spelling}
 */
proto.speech.tts.Spelling.fromObject = function(obj) {
  var msg = new proto.speech.tts.Spelling();
  obj.graphemes != null && jspb.Message.setField(msg, 1, obj.graphemes);
  obj.chunkSize != null && jspb.Message.setField(msg, 2, obj.chunkSize);
  obj.expressCapitalization != null && jspb.Message.setField(msg, 3, obj.expressCapitalization);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.Spelling}
 */
proto.speech.tts.Spelling.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.Spelling;
  return proto.speech.tts.Spelling.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.Spelling} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.Spelling}
 */
proto.speech.tts.Spelling.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setGraphemes(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setChunkSize(value);
      break;
    case 3:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setExpressCapitalization(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.Spelling.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.Spelling.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.Spelling} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Spelling.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeInt32(
      2,
      f
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeBool(
      3,
      f
    );
  }
};


/**
 * optional string graphemes = 1;
 * @return {?string}
 */
proto.speech.tts.Spelling.prototype.getGraphemes = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Spelling} returns this
 */
proto.speech.tts.Spelling.prototype.setGraphemes = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Spelling} returns this
 */
proto.speech.tts.Spelling.prototype.clearGraphemes = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Spelling.prototype.hasGraphemes = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional int32 chunk_size = 2;
 * @return {?number}
 */
proto.speech.tts.Spelling.prototype.getChunkSize = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Spelling} returns this
 */
proto.speech.tts.Spelling.prototype.setChunkSize = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Spelling} returns this
 */
proto.speech.tts.Spelling.prototype.clearChunkSize = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Spelling.prototype.hasChunkSize = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional bool express_capitalization = 3;
 * @return {?boolean}
 */
proto.speech.tts.Spelling.prototype.getExpressCapitalization = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 3));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.Spelling} returns this
 */
proto.speech.tts.Spelling.prototype.setExpressCapitalization = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Spelling} returns this
 */
proto.speech.tts.Spelling.prototype.clearExpressCapitalization = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Spelling.prototype.hasExpressCapitalization = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.Spelling}
 */
proto.speech.tts.Spelling.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.Spelling, data);
};



//speech/tts/public/semiotic_classes_proto_jspb/telephone.jspb.js
// source: speech/tts/public/semiotic_classes.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.Telephone');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.MorphosyntacticFeatureVector');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.Telephone = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.Telephone.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.Telephone, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.Telephone.displayName = 'proto.speech.tts.Telephone';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.Telephone.repeatedFields_ = [2,8];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.Telephone.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.Telephone.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.Telephone} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Telephone.toObject = function(includeInstance, msg) {
  var f, obj = {
    countryCode: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    numberPartList: (f = jspb.Message.getRepeatedField(msg, 2)) == null ? undefined : f,
    extension: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,
    style: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f,
    morphosyntacticFeatures: (f = jspb.Message.getField(msg, 5)) == null ? undefined : f,
    structuredFeatures: (f = msg.getStructuredFeatures()) && proto.speech.tts.MorphosyntacticFeatureVector.toObject(includeInstance, f),
    preserveOrder: (f = jspb.Message.getBooleanField(msg, 6)) == null ? undefined : f,
    codeSwitch: (f = jspb.Message.getField(msg, 7)) == null ? undefined : f,
    fieldOrderList: (f = jspb.Message.getRepeatedField(msg, 8)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Telephone as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.Telephone.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.countryCode;

  /** @type {?Array<string>|undefined} */
  this.numberPartList;

  /** @type {?string|undefined} */
  this.extension;

  /** @type {?number|undefined} */
  this.style;

  /** @type {?string|undefined} */
  this.morphosyntacticFeatures;

  /** @type {?proto.speech.tts.MorphosyntacticFeatureVector.ObjectFormat|undefined} */
  this.structuredFeatures;

  /** @type {?boolean|undefined} */
  this.preserveOrder;

  /** @type {?string|undefined} */
  this.codeSwitch;

  /** @type {?Array<string>|undefined} */
  this.fieldOrderList;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.Telephone.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.Telephone}
 */
proto.speech.tts.Telephone.fromObject = function(obj) {
  var msg = new proto.speech.tts.Telephone();
  obj.countryCode != null && jspb.Message.setField(msg, 1, obj.countryCode);
  obj.numberPartList != null && jspb.Message.setField(msg, 2, obj.numberPartList);
  obj.extension != null && jspb.Message.setField(msg, 3, obj.extension);
  obj.style != null && jspb.Message.setField(msg, 4, obj.style);
  obj.morphosyntacticFeatures != null && jspb.Message.setField(msg, 5, obj.morphosyntacticFeatures);
  obj.structuredFeatures && jspb.Message.setWrapperField(
      msg, 9, proto.speech.tts.MorphosyntacticFeatureVector.fromObject(obj.structuredFeatures));
  obj.preserveOrder != null && jspb.Message.setField(msg, 6, obj.preserveOrder);
  obj.codeSwitch != null && jspb.Message.setField(msg, 7, obj.codeSwitch);
  obj.fieldOrderList != null && jspb.Message.setField(msg, 8, obj.fieldOrderList);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.Telephone}
 */
proto.speech.tts.Telephone.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.Telephone;
  return proto.speech.tts.Telephone.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.Telephone} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.Telephone}
 */
proto.speech.tts.Telephone.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setCountryCode(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.addNumberPart(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setExtension$(value);
      break;
    case 4:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setStyle(value);
      break;
    case 5:
      var value = /** @type {string} */ (reader.readString());
      msg.setMorphosyntacticFeatures(value);
      break;
    case 9:
      var value = new proto.speech.tts.MorphosyntacticFeatureVector;
      reader.readMessage(value,proto.speech.tts.MorphosyntacticFeatureVector.deserializeBinaryFromReader);
      msg.setStructuredFeatures(value);
      break;
    case 6:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setPreserveOrder(value);
      break;
    case 7:
      var value = /** @type {string} */ (reader.readString());
      msg.setCodeSwitch(value);
      break;
    case 8:
      var value = /** @type {string} */ (reader.readString());
      msg.addFieldOrder(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.Telephone.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.Telephone.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.Telephone} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Telephone.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = message.getNumberPartList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      2,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeString(
      3,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeInt32(
      4,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeString(
      5,
      f
    );
  }
  f = message.getStructuredFeatures();
  if (f != null) {
    writer.writeMessage(
      9,
      f,
      proto.speech.tts.MorphosyntacticFeatureVector.serializeBinaryToWriter
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeBool(
      6,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 7));
  if (f != null) {
    writer.writeString(
      7,
      f
    );
  }
  f = message.getFieldOrderList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      8,
      f
    );
  }
};


/**
 * optional string country_code = 1;
 * @return {?string}
 */
proto.speech.tts.Telephone.prototype.getCountryCode = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Telephone} returns this
 */
proto.speech.tts.Telephone.prototype.setCountryCode = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Telephone} returns this
 */
proto.speech.tts.Telephone.prototype.clearCountryCode = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Telephone.prototype.hasCountryCode = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * repeated string number_part = 2;
 * @return {!Array<string>}
 */
proto.speech.tts.Telephone.prototype.getNumberPartList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 2));
};


/**
 * @param {Array<string>|undefined} value
 * @return {!proto.speech.tts.Telephone} returns this
 */
proto.speech.tts.Telephone.prototype.setNumberPartList = function(value) {
  return jspb.Message.setField(this, 2, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.Telephone} returns this
 */
proto.speech.tts.Telephone.prototype.addNumberPart = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 2, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.Telephone} returns this
 */
proto.speech.tts.Telephone.prototype.clearNumberPartList = function() {
  return this.setNumberPartList([]);
};


/**
 * optional string extension = 3;
 * @return {?string}
 */
proto.speech.tts.Telephone.prototype.getExtension$ = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Telephone} returns this
 */
proto.speech.tts.Telephone.prototype.setExtension$ = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Telephone} returns this
 */
proto.speech.tts.Telephone.prototype.clearExtension$ = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Telephone.prototype.hasExtension$ = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional int32 style = 4;
 * @return {?number}
 */
proto.speech.tts.Telephone.prototype.getStyle = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 4));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Telephone} returns this
 */
proto.speech.tts.Telephone.prototype.setStyle = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Telephone} returns this
 */
proto.speech.tts.Telephone.prototype.clearStyle = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Telephone.prototype.hasStyle = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional string morphosyntactic_features = 5;
 * @return {?string}
 */
proto.speech.tts.Telephone.prototype.getMorphosyntacticFeatures = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 5));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Telephone} returns this
 */
proto.speech.tts.Telephone.prototype.setMorphosyntacticFeatures = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Telephone} returns this
 */
proto.speech.tts.Telephone.prototype.clearMorphosyntacticFeatures = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Telephone.prototype.hasMorphosyntacticFeatures = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional MorphosyntacticFeatureVector structured_features = 9;
 * @return {?proto.speech.tts.MorphosyntacticFeatureVector}
 */
proto.speech.tts.Telephone.prototype.getStructuredFeatures = function() {
  return /** @type{?proto.speech.tts.MorphosyntacticFeatureVector} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.MorphosyntacticFeatureVector, 9));
};


/**
 * @param {?proto.speech.tts.MorphosyntacticFeatureVector|undefined} value
 * @return {!proto.speech.tts.Telephone} returns this
*/
proto.speech.tts.Telephone.prototype.setStructuredFeatures = function(value) {
  return jspb.Message.setWrapperField(this, 9, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Telephone} returns this
 */
proto.speech.tts.Telephone.prototype.clearStructuredFeatures = function() {
  return this.setStructuredFeatures(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Telephone.prototype.hasStructuredFeatures = function() {
  return jspb.Message.getField(this, 9) != null;
};


/**
 * optional bool preserve_order = 6;
 * @return {?boolean}
 */
proto.speech.tts.Telephone.prototype.getPreserveOrder = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 6));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.Telephone} returns this
 */
proto.speech.tts.Telephone.prototype.setPreserveOrder = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Telephone} returns this
 */
proto.speech.tts.Telephone.prototype.clearPreserveOrder = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Telephone.prototype.hasPreserveOrder = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional string code_switch = 7;
 * @return {?string}
 */
proto.speech.tts.Telephone.prototype.getCodeSwitch = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 7));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Telephone} returns this
 */
proto.speech.tts.Telephone.prototype.setCodeSwitch = function(value) {
  return jspb.Message.setField(this, 7, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Telephone} returns this
 */
proto.speech.tts.Telephone.prototype.clearCodeSwitch = function() {
  return jspb.Message.setField(this, 7, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Telephone.prototype.hasCodeSwitch = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * repeated string field_order = 8;
 * @return {!Array<string>}
 */
proto.speech.tts.Telephone.prototype.getFieldOrderList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 8));
};


/**
 * @param {Array<string>|undefined} value
 * @return {!proto.speech.tts.Telephone} returns this
 */
proto.speech.tts.Telephone.prototype.setFieldOrderList = function(value) {
  return jspb.Message.setField(this, 8, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.Telephone} returns this
 */
proto.speech.tts.Telephone.prototype.addFieldOrder = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 8, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.Telephone} returns this
 */
proto.speech.tts.Telephone.prototype.clearFieldOrderList = function() {
  return this.setFieldOrderList([]);
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.Telephone}
 */
proto.speech.tts.Telephone.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.Telephone, data);
};



//speech/tts/public/advanced_voice_mod_proto_jspb/timevaluefunctionsamples.jspb.js
// source: speech/tts/public/advanced_voice_mod.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.TimeValueFunctionSamples');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.TimeValueFunctionSamples = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.TimeValueFunctionSamples, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.TimeValueFunctionSamples.displayName = 'proto.speech.tts.TimeValueFunctionSamples';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.TimeValueFunctionSamples.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.TimeValueFunctionSamples.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.TimeValueFunctionSamples} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.TimeValueFunctionSamples.toObject = function(includeInstance, msg) {
  var f, obj = {
    time: (f = jspb.Message.getOptionalFloatingPointField(msg, 1)) == null ? undefined : f,
    value: (f = jspb.Message.getOptionalFloatingPointField(msg, 2)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of TimeValueFunctionSamples as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.TimeValueFunctionSamples.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.time;

  /** @type {?number|undefined} */
  this.value;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.TimeValueFunctionSamples.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.TimeValueFunctionSamples}
 */
proto.speech.tts.TimeValueFunctionSamples.fromObject = function(obj) {
  var msg = new proto.speech.tts.TimeValueFunctionSamples();
  obj.time != null && jspb.Message.setField(msg, 1, obj.time);
  obj.value != null && jspb.Message.setField(msg, 2, obj.value);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.TimeValueFunctionSamples}
 */
proto.speech.tts.TimeValueFunctionSamples.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.TimeValueFunctionSamples;
  return proto.speech.tts.TimeValueFunctionSamples.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.TimeValueFunctionSamples} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.TimeValueFunctionSamples}
 */
proto.speech.tts.TimeValueFunctionSamples.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setTime(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setValue(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.TimeValueFunctionSamples.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.TimeValueFunctionSamples.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.TimeValueFunctionSamples} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.TimeValueFunctionSamples.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {number} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeFloat(
      1,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeFloat(
      2,
      f
    );
  }
};


/**
 * required float time = 1;
 * @return {number}
 */
proto.speech.tts.TimeValueFunctionSamples.prototype.getTime = function() {
  return /** @type {number} */ (jspb.Message.getOptionalFloatingPointField(this, 1));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.TimeValueFunctionSamples} returns this
 */
proto.speech.tts.TimeValueFunctionSamples.prototype.setTime = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.TimeValueFunctionSamples} returns this
 */
proto.speech.tts.TimeValueFunctionSamples.prototype.clearTime = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.TimeValueFunctionSamples.prototype.hasTime = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * required float value = 2;
 * @return {number}
 */
proto.speech.tts.TimeValueFunctionSamples.prototype.getValue = function() {
  return /** @type {number} */ (jspb.Message.getOptionalFloatingPointField(this, 2));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.TimeValueFunctionSamples} returns this
 */
proto.speech.tts.TimeValueFunctionSamples.prototype.setValue = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.TimeValueFunctionSamples} returns this
 */
proto.speech.tts.TimeValueFunctionSamples.prototype.clearValue = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.TimeValueFunctionSamples.prototype.hasValue = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.TimeValueFunctionSamples}
 */
proto.speech.tts.TimeValueFunctionSamples.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.TimeValueFunctionSamples, data);
};



//speech/tts/public/advanced_voice_mod_proto_jspb/copycatparams.jspb.js
// source: speech/tts/public/advanced_voice_mod.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.CopyCatParams');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.TimeValueFunctionSamples');

goog.forwardDeclare('proto.speech.tts.TimeAddressingType');
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.CopyCatParams = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.CopyCatParams.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.CopyCatParams, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.CopyCatParams.displayName = 'proto.speech.tts.CopyCatParams';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.CopyCatParams.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.CopyCatParams.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.CopyCatParams.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.CopyCatParams} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.CopyCatParams.toObject = function(includeInstance, msg) {
  var f, obj = {
    sampleList: jspb.Message.toObjectList(msg.getSampleList(),
    proto.speech.tts.TimeValueFunctionSamples.toObject, includeInstance),
    timeAddressing: jspb.Message.getFieldWithDefault(msg, 2, 0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of CopyCatParams as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.CopyCatParams.ObjectFormat = function() {
  /** @type {?Array<!proto.speech.tts.TimeValueFunctionSamples.ObjectFormat>|undefined} */
  this.sampleList;

  /** @type {?number|undefined} */
  this.timeAddressing;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.CopyCatParams.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.CopyCatParams}
 */
proto.speech.tts.CopyCatParams.fromObject = function(obj) {
  var msg = new proto.speech.tts.CopyCatParams();
  obj.sampleList && jspb.Message.setRepeatedWrapperField(
      msg, 1, obj.sampleList.map(
          proto.speech.tts.TimeValueFunctionSamples.fromObject));
  obj.timeAddressing != null && jspb.Message.setField(msg, 2, obj.timeAddressing);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.CopyCatParams}
 */
proto.speech.tts.CopyCatParams.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.CopyCatParams;
  return proto.speech.tts.CopyCatParams.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.CopyCatParams} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.CopyCatParams}
 */
proto.speech.tts.CopyCatParams.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.speech.tts.TimeValueFunctionSamples;
      reader.readMessage(value,proto.speech.tts.TimeValueFunctionSamples.deserializeBinaryFromReader);
      msg.addSample(value);
      break;
    case 2:
      var value = /** @type {!proto.speech.tts.TimeAddressingType} */ (reader.readEnum());
      msg.setTimeAddressing(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.CopyCatParams.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.CopyCatParams.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.CopyCatParams} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.CopyCatParams.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getSampleList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      1,
      f,
      proto.speech.tts.TimeValueFunctionSamples.serializeBinaryToWriter
    );
  }
  f = /** @type {!proto.speech.tts.TimeAddressingType} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeEnum(
      2,
      f
    );
  }
};


/**
 * repeated TimeValueFunctionSamples sample = 1;
 * @return {!Array<!proto.speech.tts.TimeValueFunctionSamples>}
 */
proto.speech.tts.CopyCatParams.prototype.getSampleList = function() {
  return /** @type{!Array<!proto.speech.tts.TimeValueFunctionSamples>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.TimeValueFunctionSamples, 1));
};


/**
 * @param {Array<!proto.speech.tts.TimeValueFunctionSamples>|undefined} value
 * @return {!proto.speech.tts.CopyCatParams} returns this
*/
proto.speech.tts.CopyCatParams.prototype.setSampleList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 1, value);
};


/**
 * @param {!proto.speech.tts.TimeValueFunctionSamples=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.TimeValueFunctionSamples}
 */
proto.speech.tts.CopyCatParams.prototype.addSample = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.speech.tts.TimeValueFunctionSamples, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.CopyCatParams} returns this
 */
proto.speech.tts.CopyCatParams.prototype.clearSampleList = function() {
  return this.setSampleList([]);
};


/**
 * optional TimeAddressingType time_addressing = 2;
 * @return {!proto.speech.tts.TimeAddressingType}
 */
proto.speech.tts.CopyCatParams.prototype.getTimeAddressing = function() {
  return /** @type {!proto.speech.tts.TimeAddressingType} */ (jspb.Message.getFieldWithDefault(this, 2, 0));
};


/**
 * @param {?proto.speech.tts.TimeAddressingType|undefined} value
 * @return {!proto.speech.tts.CopyCatParams} returns this
 */
proto.speech.tts.CopyCatParams.prototype.setTimeAddressing = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.CopyCatParams} returns this
 */
proto.speech.tts.CopyCatParams.prototype.clearTimeAddressing = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.CopyCatParams.prototype.hasTimeAddressing = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.CopyCatParams}
 */
proto.speech.tts.CopyCatParams.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.CopyCatParams, data);
};



//speech/tts/public/advanced_voice_mod_proto_jspb/meanmodelparams.jspb.js
// source: speech/tts/public/advanced_voice_mod.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.MeanModelParams');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.MeanModelParams = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.MeanModelParams, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.MeanModelParams.displayName = 'proto.speech.tts.MeanModelParams';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.MeanModelParams.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.MeanModelParams.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.MeanModelParams} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.MeanModelParams.toObject = function(includeInstance, msg) {
  var f, obj = {
    mean: (f = jspb.Message.getOptionalFloatingPointField(msg, 1)) == null ? undefined : f,
    deviation: jspb.Message.getFloatingPointFieldWithDefault(msg, 2, 1.0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of MeanModelParams as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.MeanModelParams.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.mean;

  /** @type {?number|undefined} */
  this.deviation;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.MeanModelParams.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.MeanModelParams}
 */
proto.speech.tts.MeanModelParams.fromObject = function(obj) {
  var msg = new proto.speech.tts.MeanModelParams();
  obj.mean != null && jspb.Message.setField(msg, 1, obj.mean);
  obj.deviation != null && jspb.Message.setField(msg, 2, obj.deviation);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.MeanModelParams}
 */
proto.speech.tts.MeanModelParams.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.MeanModelParams;
  return proto.speech.tts.MeanModelParams.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.MeanModelParams} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.MeanModelParams}
 */
proto.speech.tts.MeanModelParams.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setMean(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setDeviation(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.MeanModelParams.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.MeanModelParams.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.MeanModelParams} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.MeanModelParams.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {number} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeFloat(
      1,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeFloat(
      2,
      f
    );
  }
};


/**
 * required float mean = 1;
 * @return {number}
 */
proto.speech.tts.MeanModelParams.prototype.getMean = function() {
  return /** @type {number} */ (jspb.Message.getOptionalFloatingPointField(this, 1));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.MeanModelParams} returns this
 */
proto.speech.tts.MeanModelParams.prototype.setMean = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.MeanModelParams} returns this
 */
proto.speech.tts.MeanModelParams.prototype.clearMean = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.MeanModelParams.prototype.hasMean = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional float deviation = 2;
 * @return {number}
 */
proto.speech.tts.MeanModelParams.prototype.getDeviation = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 2, 1.0));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.MeanModelParams} returns this
 */
proto.speech.tts.MeanModelParams.prototype.setDeviation = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.MeanModelParams} returns this
 */
proto.speech.tts.MeanModelParams.prototype.clearDeviation = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.MeanModelParams.prototype.hasDeviation = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.MeanModelParams}
 */
proto.speech.tts.MeanModelParams.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.MeanModelParams, data);
};



//speech/tts/public/advanced_voice_mod_proto_jspb/piecewisemodelparams.jspb.js
// source: speech/tts/public/advanced_voice_mod.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.PiecewiseModelParams');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.TimeValueFunctionSamples');

goog.forwardDeclare('proto.speech.tts.TimeAddressingType');
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.PiecewiseModelParams = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.PiecewiseModelParams.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.PiecewiseModelParams, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.PiecewiseModelParams.displayName = 'proto.speech.tts.PiecewiseModelParams';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.PiecewiseModelParams.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.PiecewiseModelParams.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.PiecewiseModelParams.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.PiecewiseModelParams} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.PiecewiseModelParams.toObject = function(includeInstance, msg) {
  var f, obj = {
    sampleList: jspb.Message.toObjectList(msg.getSampleList(),
    proto.speech.tts.TimeValueFunctionSamples.toObject, includeInstance),
    deviation: jspb.Message.getFloatingPointFieldWithDefault(msg, 2, 1.0),
    timeAddressing: jspb.Message.getFieldWithDefault(msg, 3, 0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of PiecewiseModelParams as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.PiecewiseModelParams.ObjectFormat = function() {
  /** @type {?Array<!proto.speech.tts.TimeValueFunctionSamples.ObjectFormat>|undefined} */
  this.sampleList;

  /** @type {?number|undefined} */
  this.deviation;

  /** @type {?number|undefined} */
  this.timeAddressing;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.PiecewiseModelParams.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.PiecewiseModelParams}
 */
proto.speech.tts.PiecewiseModelParams.fromObject = function(obj) {
  var msg = new proto.speech.tts.PiecewiseModelParams();
  obj.sampleList && jspb.Message.setRepeatedWrapperField(
      msg, 1, obj.sampleList.map(
          proto.speech.tts.TimeValueFunctionSamples.fromObject));
  obj.deviation != null && jspb.Message.setField(msg, 2, obj.deviation);
  obj.timeAddressing != null && jspb.Message.setField(msg, 3, obj.timeAddressing);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.PiecewiseModelParams}
 */
proto.speech.tts.PiecewiseModelParams.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.PiecewiseModelParams;
  return proto.speech.tts.PiecewiseModelParams.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.PiecewiseModelParams} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.PiecewiseModelParams}
 */
proto.speech.tts.PiecewiseModelParams.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.speech.tts.TimeValueFunctionSamples;
      reader.readMessage(value,proto.speech.tts.TimeValueFunctionSamples.deserializeBinaryFromReader);
      msg.addSample(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setDeviation(value);
      break;
    case 3:
      var value = /** @type {!proto.speech.tts.TimeAddressingType} */ (reader.readEnum());
      msg.setTimeAddressing(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.PiecewiseModelParams.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.PiecewiseModelParams.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.PiecewiseModelParams} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.PiecewiseModelParams.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getSampleList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      1,
      f,
      proto.speech.tts.TimeValueFunctionSamples.serializeBinaryToWriter
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeFloat(
      2,
      f
    );
  }
  f = /** @type {!proto.speech.tts.TimeAddressingType} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeEnum(
      3,
      f
    );
  }
};


/**
 * repeated TimeValueFunctionSamples sample = 1;
 * @return {!Array<!proto.speech.tts.TimeValueFunctionSamples>}
 */
proto.speech.tts.PiecewiseModelParams.prototype.getSampleList = function() {
  return /** @type{!Array<!proto.speech.tts.TimeValueFunctionSamples>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.TimeValueFunctionSamples, 1));
};


/**
 * @param {Array<!proto.speech.tts.TimeValueFunctionSamples>|undefined} value
 * @return {!proto.speech.tts.PiecewiseModelParams} returns this
*/
proto.speech.tts.PiecewiseModelParams.prototype.setSampleList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 1, value);
};


/**
 * @param {!proto.speech.tts.TimeValueFunctionSamples=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.TimeValueFunctionSamples}
 */
proto.speech.tts.PiecewiseModelParams.prototype.addSample = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.speech.tts.TimeValueFunctionSamples, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.PiecewiseModelParams} returns this
 */
proto.speech.tts.PiecewiseModelParams.prototype.clearSampleList = function() {
  return this.setSampleList([]);
};


/**
 * optional float deviation = 2;
 * @return {number}
 */
proto.speech.tts.PiecewiseModelParams.prototype.getDeviation = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 2, 1.0));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.PiecewiseModelParams} returns this
 */
proto.speech.tts.PiecewiseModelParams.prototype.setDeviation = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.PiecewiseModelParams} returns this
 */
proto.speech.tts.PiecewiseModelParams.prototype.clearDeviation = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.PiecewiseModelParams.prototype.hasDeviation = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional TimeAddressingType time_addressing = 3;
 * @return {!proto.speech.tts.TimeAddressingType}
 */
proto.speech.tts.PiecewiseModelParams.prototype.getTimeAddressing = function() {
  return /** @type {!proto.speech.tts.TimeAddressingType} */ (jspb.Message.getFieldWithDefault(this, 3, 0));
};


/**
 * @param {?proto.speech.tts.TimeAddressingType|undefined} value
 * @return {!proto.speech.tts.PiecewiseModelParams} returns this
 */
proto.speech.tts.PiecewiseModelParams.prototype.setTimeAddressing = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.PiecewiseModelParams} returns this
 */
proto.speech.tts.PiecewiseModelParams.prototype.clearTimeAddressing = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.PiecewiseModelParams.prototype.hasTimeAddressing = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.PiecewiseModelParams}
 */
proto.speech.tts.PiecewiseModelParams.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.PiecewiseModelParams, data);
};



//speech/tts/public/advanced_voice_mod_proto_jspb/advancedvoicemod.jspb.js
// source: speech/tts/public/advanced_voice_mod.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.AdvancedVoiceMod');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.CopyCatParams');
goog.require('proto.speech.tts.MeanModelParams');
goog.require('proto.speech.tts.PiecewiseModelParams');
goog.require('proto.speech.tts.SpeechMorphingTargets');
goog.require('proto.speech.tts.TimeValueFunctionSamples');

goog.forwardDeclare('proto.speech.tts.VoiceModFactorType');
goog.forwardDeclare('proto.speech.tts.VoiceModIntervalType');
goog.forwardDeclare('proto.speech.tts.VoiceModLinguisticItemType');
goog.forwardDeclare('proto.speech.tts.VoiceModTransformType');
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.AdvancedVoiceMod = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.AdvancedVoiceMod.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.AdvancedVoiceMod, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.AdvancedVoiceMod.displayName = 'proto.speech.tts.AdvancedVoiceMod';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.AdvancedVoiceMod.repeatedFields_ = [8];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.AdvancedVoiceMod.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.AdvancedVoiceMod.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.AdvancedVoiceMod} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.AdvancedVoiceMod.toObject = function(includeInstance, msg) {
  var f, obj = {
    wordIndex: jspb.Message.getFieldWithDefault(msg, 1, -1),
    item: jspb.Message.getFieldWithDefault(msg, 2, 2),
    itemIndex: jspb.Message.getFieldWithDefault(msg, 3, -1),
    interval: jspb.Message.getFieldWithDefault(msg, 4, 0),
    spanId: jspb.Message.getFieldWithDefault(msg, 5, 0),
    transform: jspb.Message.getFieldWithDefault(msg, 6, 0),
    factor: (f = jspb.Message.getField(msg, 7)) == null ? undefined : f,
    operatorParamsList: jspb.Message.toObjectList(msg.getOperatorParamsList(),
    proto.speech.tts.TimeValueFunctionSamples.toObject, includeInstance),
    meanParams: (f = msg.getMeanParams()) && proto.speech.tts.MeanModelParams.toObject(includeInstance, f),
    piecewiseParams: (f = msg.getPiecewiseParams()) && proto.speech.tts.PiecewiseModelParams.toObject(includeInstance, f),
    copycatParams: (f = msg.getCopycatParams()) && proto.speech.tts.CopyCatParams.toObject(includeInstance, f),
    morphing: (f = msg.getMorphing()) && proto.speech.tts.SpeechMorphingTargets.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of AdvancedVoiceMod as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.AdvancedVoiceMod.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.wordIndex;

  /** @type {?number|undefined} */
  this.item;

  /** @type {?number|undefined} */
  this.itemIndex;

  /** @type {?number|undefined} */
  this.interval;

  /** @type {?number|undefined} */
  this.spanId;

  /** @type {?number|undefined} */
  this.transform;

  /** @type {?number|undefined} */
  this.factor;

  /** @type {?Array<!proto.speech.tts.TimeValueFunctionSamples.ObjectFormat>|undefined} */
  this.operatorParamsList;

  /** @type {?proto.speech.tts.MeanModelParams.ObjectFormat|undefined} */
  this.meanParams;

  /** @type {?proto.speech.tts.PiecewiseModelParams.ObjectFormat|undefined} */
  this.piecewiseParams;

  /** @type {?proto.speech.tts.CopyCatParams.ObjectFormat|undefined} */
  this.copycatParams;

  /** @type {?proto.speech.tts.SpeechMorphingTargets.ObjectFormat|undefined} */
  this.morphing;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.AdvancedVoiceMod.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.AdvancedVoiceMod}
 */
proto.speech.tts.AdvancedVoiceMod.fromObject = function(obj) {
  var msg = new proto.speech.tts.AdvancedVoiceMod();
  obj.wordIndex != null && jspb.Message.setField(msg, 1, obj.wordIndex);
  obj.item != null && jspb.Message.setField(msg, 2, obj.item);
  obj.itemIndex != null && jspb.Message.setField(msg, 3, obj.itemIndex);
  obj.interval != null && jspb.Message.setField(msg, 4, obj.interval);
  obj.spanId != null && jspb.Message.setField(msg, 5, obj.spanId);
  obj.transform != null && jspb.Message.setField(msg, 6, obj.transform);
  obj.factor != null && jspb.Message.setField(msg, 7, obj.factor);
  obj.operatorParamsList && jspb.Message.setRepeatedWrapperField(
      msg, 8, obj.operatorParamsList.map(
          proto.speech.tts.TimeValueFunctionSamples.fromObject));
  obj.meanParams && jspb.Message.setWrapperField(
      msg, 9, proto.speech.tts.MeanModelParams.fromObject(obj.meanParams));
  obj.piecewiseParams && jspb.Message.setWrapperField(
      msg, 10, proto.speech.tts.PiecewiseModelParams.fromObject(obj.piecewiseParams));
  obj.copycatParams && jspb.Message.setWrapperField(
      msg, 11, proto.speech.tts.CopyCatParams.fromObject(obj.copycatParams));
  obj.morphing && jspb.Message.setWrapperField(
      msg, 12, proto.speech.tts.SpeechMorphingTargets.fromObject(obj.morphing));
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.AdvancedVoiceMod}
 */
proto.speech.tts.AdvancedVoiceMod.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.AdvancedVoiceMod;
  return proto.speech.tts.AdvancedVoiceMod.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.AdvancedVoiceMod} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.AdvancedVoiceMod}
 */
proto.speech.tts.AdvancedVoiceMod.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setWordIndex(value);
      break;
    case 2:
      var value = /** @type {!proto.speech.tts.VoiceModLinguisticItemType} */ (reader.readEnum());
      msg.setItem(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setItemIndex(value);
      break;
    case 4:
      var value = /** @type {!proto.speech.tts.VoiceModIntervalType} */ (reader.readEnum());
      msg.setInterval(value);
      break;
    case 5:
      var value = /** @type {number} */ (reader.readUint32());
      msg.setSpanId(value);
      break;
    case 6:
      var value = /** @type {!proto.speech.tts.VoiceModTransformType} */ (reader.readEnum());
      msg.setTransform(value);
      break;
    case 7:
      var value = /** @type {!proto.speech.tts.VoiceModFactorType} */ (reader.readEnum());
      msg.setFactor(value);
      break;
    case 8:
      var value = new proto.speech.tts.TimeValueFunctionSamples;
      reader.readMessage(value,proto.speech.tts.TimeValueFunctionSamples.deserializeBinaryFromReader);
      msg.addOperatorParams(value);
      break;
    case 9:
      var value = new proto.speech.tts.MeanModelParams;
      reader.readMessage(value,proto.speech.tts.MeanModelParams.deserializeBinaryFromReader);
      msg.setMeanParams(value);
      break;
    case 10:
      var value = new proto.speech.tts.PiecewiseModelParams;
      reader.readMessage(value,proto.speech.tts.PiecewiseModelParams.deserializeBinaryFromReader);
      msg.setPiecewiseParams(value);
      break;
    case 11:
      var value = new proto.speech.tts.CopyCatParams;
      reader.readMessage(value,proto.speech.tts.CopyCatParams.deserializeBinaryFromReader);
      msg.setCopycatParams(value);
      break;
    case 12:
      var value = new proto.speech.tts.SpeechMorphingTargets;
      reader.readMessage(value,proto.speech.tts.SpeechMorphingTargets.deserializeBinaryFromReader);
      msg.setMorphing(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.AdvancedVoiceMod.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.AdvancedVoiceMod.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.AdvancedVoiceMod} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.AdvancedVoiceMod.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {number} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeInt32(
      1,
      f
    );
  }
  f = /** @type {!proto.speech.tts.VoiceModLinguisticItemType} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeEnum(
      2,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeInt32(
      3,
      f
    );
  }
  f = /** @type {!proto.speech.tts.VoiceModIntervalType} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeEnum(
      4,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeUint32(
      5,
      f
    );
  }
  f = /** @type {!proto.speech.tts.VoiceModTransformType} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeEnum(
      6,
      f
    );
  }
  f = /** @type {!proto.speech.tts.VoiceModFactorType} */ (jspb.Message.getField(message, 7));
  if (f != null) {
    writer.writeEnum(
      7,
      f
    );
  }
  f = message.getOperatorParamsList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      8,
      f,
      proto.speech.tts.TimeValueFunctionSamples.serializeBinaryToWriter
    );
  }
  f = message.getMeanParams();
  if (f != null) {
    writer.writeMessage(
      9,
      f,
      proto.speech.tts.MeanModelParams.serializeBinaryToWriter
    );
  }
  f = message.getPiecewiseParams();
  if (f != null) {
    writer.writeMessage(
      10,
      f,
      proto.speech.tts.PiecewiseModelParams.serializeBinaryToWriter
    );
  }
  f = message.getCopycatParams();
  if (f != null) {
    writer.writeMessage(
      11,
      f,
      proto.speech.tts.CopyCatParams.serializeBinaryToWriter
    );
  }
  f = message.getMorphing();
  if (f != null) {
    writer.writeMessage(
      12,
      f,
      proto.speech.tts.SpeechMorphingTargets.serializeBinaryToWriter
    );
  }
};


/**
 * optional int32 word_index = 1;
 * @return {number}
 */
proto.speech.tts.AdvancedVoiceMod.prototype.getWordIndex = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 1, -1));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.AdvancedVoiceMod} returns this
 */
proto.speech.tts.AdvancedVoiceMod.prototype.setWordIndex = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.AdvancedVoiceMod} returns this
 */
proto.speech.tts.AdvancedVoiceMod.prototype.clearWordIndex = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.AdvancedVoiceMod.prototype.hasWordIndex = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional VoiceModLinguisticItemType item = 2;
 * @return {!proto.speech.tts.VoiceModLinguisticItemType}
 */
proto.speech.tts.AdvancedVoiceMod.prototype.getItem = function() {
  return /** @type {!proto.speech.tts.VoiceModLinguisticItemType} */ (jspb.Message.getFieldWithDefault(this, 2, 2));
};


/**
 * @param {?proto.speech.tts.VoiceModLinguisticItemType|undefined} value
 * @return {!proto.speech.tts.AdvancedVoiceMod} returns this
 */
proto.speech.tts.AdvancedVoiceMod.prototype.setItem = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.AdvancedVoiceMod} returns this
 */
proto.speech.tts.AdvancedVoiceMod.prototype.clearItem = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.AdvancedVoiceMod.prototype.hasItem = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional int32 item_index = 3;
 * @return {number}
 */
proto.speech.tts.AdvancedVoiceMod.prototype.getItemIndex = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 3, -1));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.AdvancedVoiceMod} returns this
 */
proto.speech.tts.AdvancedVoiceMod.prototype.setItemIndex = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.AdvancedVoiceMod} returns this
 */
proto.speech.tts.AdvancedVoiceMod.prototype.clearItemIndex = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.AdvancedVoiceMod.prototype.hasItemIndex = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional VoiceModIntervalType interval = 4;
 * @return {!proto.speech.tts.VoiceModIntervalType}
 */
proto.speech.tts.AdvancedVoiceMod.prototype.getInterval = function() {
  return /** @type {!proto.speech.tts.VoiceModIntervalType} */ (jspb.Message.getFieldWithDefault(this, 4, 0));
};


/**
 * @param {?proto.speech.tts.VoiceModIntervalType|undefined} value
 * @return {!proto.speech.tts.AdvancedVoiceMod} returns this
 */
proto.speech.tts.AdvancedVoiceMod.prototype.setInterval = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.AdvancedVoiceMod} returns this
 */
proto.speech.tts.AdvancedVoiceMod.prototype.clearInterval = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.AdvancedVoiceMod.prototype.hasInterval = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional uint32 span_id = 5;
 * @return {number}
 */
proto.speech.tts.AdvancedVoiceMod.prototype.getSpanId = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 5, 0));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.AdvancedVoiceMod} returns this
 */
proto.speech.tts.AdvancedVoiceMod.prototype.setSpanId = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.AdvancedVoiceMod} returns this
 */
proto.speech.tts.AdvancedVoiceMod.prototype.clearSpanId = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.AdvancedVoiceMod.prototype.hasSpanId = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional VoiceModTransformType transform = 6;
 * @return {!proto.speech.tts.VoiceModTransformType}
 */
proto.speech.tts.AdvancedVoiceMod.prototype.getTransform = function() {
  return /** @type {!proto.speech.tts.VoiceModTransformType} */ (jspb.Message.getFieldWithDefault(this, 6, 0));
};


/**
 * @param {?proto.speech.tts.VoiceModTransformType|undefined} value
 * @return {!proto.speech.tts.AdvancedVoiceMod} returns this
 */
proto.speech.tts.AdvancedVoiceMod.prototype.setTransform = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.AdvancedVoiceMod} returns this
 */
proto.speech.tts.AdvancedVoiceMod.prototype.clearTransform = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.AdvancedVoiceMod.prototype.hasTransform = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional VoiceModFactorType factor = 7;
 * @return {!proto.speech.tts.VoiceModFactorType}
 */
proto.speech.tts.AdvancedVoiceMod.prototype.getFactor = function() {
  return /** @type {!proto.speech.tts.VoiceModFactorType} */ (jspb.Message.getField(this, 7));
};


/**
 * @param {?proto.speech.tts.VoiceModFactorType|undefined} value
 * @return {!proto.speech.tts.AdvancedVoiceMod} returns this
 */
proto.speech.tts.AdvancedVoiceMod.prototype.setFactor = function(value) {
  return jspb.Message.setField(this, 7, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.AdvancedVoiceMod} returns this
 */
proto.speech.tts.AdvancedVoiceMod.prototype.clearFactor = function() {
  return jspb.Message.setField(this, 7, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.AdvancedVoiceMod.prototype.hasFactor = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * repeated TimeValueFunctionSamples operator_params = 8;
 * @return {!Array<!proto.speech.tts.TimeValueFunctionSamples>}
 */
proto.speech.tts.AdvancedVoiceMod.prototype.getOperatorParamsList = function() {
  return /** @type{!Array<!proto.speech.tts.TimeValueFunctionSamples>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.TimeValueFunctionSamples, 8));
};


/**
 * @param {Array<!proto.speech.tts.TimeValueFunctionSamples>|undefined} value
 * @return {!proto.speech.tts.AdvancedVoiceMod} returns this
*/
proto.speech.tts.AdvancedVoiceMod.prototype.setOperatorParamsList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 8, value);
};


/**
 * @param {!proto.speech.tts.TimeValueFunctionSamples=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.TimeValueFunctionSamples}
 */
proto.speech.tts.AdvancedVoiceMod.prototype.addOperatorParams = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 8, opt_value, proto.speech.tts.TimeValueFunctionSamples, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.AdvancedVoiceMod} returns this
 */
proto.speech.tts.AdvancedVoiceMod.prototype.clearOperatorParamsList = function() {
  return this.setOperatorParamsList([]);
};


/**
 * optional MeanModelParams mean_params = 9;
 * @return {?proto.speech.tts.MeanModelParams}
 */
proto.speech.tts.AdvancedVoiceMod.prototype.getMeanParams = function() {
  return /** @type{?proto.speech.tts.MeanModelParams} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.MeanModelParams, 9));
};


/**
 * @param {?proto.speech.tts.MeanModelParams|undefined} value
 * @return {!proto.speech.tts.AdvancedVoiceMod} returns this
*/
proto.speech.tts.AdvancedVoiceMod.prototype.setMeanParams = function(value) {
  return jspb.Message.setWrapperField(this, 9, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.AdvancedVoiceMod} returns this
 */
proto.speech.tts.AdvancedVoiceMod.prototype.clearMeanParams = function() {
  return this.setMeanParams(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.AdvancedVoiceMod.prototype.hasMeanParams = function() {
  return jspb.Message.getField(this, 9) != null;
};


/**
 * optional PiecewiseModelParams piecewise_params = 10;
 * @return {?proto.speech.tts.PiecewiseModelParams}
 */
proto.speech.tts.AdvancedVoiceMod.prototype.getPiecewiseParams = function() {
  return /** @type{?proto.speech.tts.PiecewiseModelParams} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.PiecewiseModelParams, 10));
};


/**
 * @param {?proto.speech.tts.PiecewiseModelParams|undefined} value
 * @return {!proto.speech.tts.AdvancedVoiceMod} returns this
*/
proto.speech.tts.AdvancedVoiceMod.prototype.setPiecewiseParams = function(value) {
  return jspb.Message.setWrapperField(this, 10, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.AdvancedVoiceMod} returns this
 */
proto.speech.tts.AdvancedVoiceMod.prototype.clearPiecewiseParams = function() {
  return this.setPiecewiseParams(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.AdvancedVoiceMod.prototype.hasPiecewiseParams = function() {
  return jspb.Message.getField(this, 10) != null;
};


/**
 * optional CopyCatParams copycat_params = 11;
 * @return {?proto.speech.tts.CopyCatParams}
 */
proto.speech.tts.AdvancedVoiceMod.prototype.getCopycatParams = function() {
  return /** @type{?proto.speech.tts.CopyCatParams} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.CopyCatParams, 11));
};


/**
 * @param {?proto.speech.tts.CopyCatParams|undefined} value
 * @return {!proto.speech.tts.AdvancedVoiceMod} returns this
*/
proto.speech.tts.AdvancedVoiceMod.prototype.setCopycatParams = function(value) {
  return jspb.Message.setWrapperField(this, 11, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.AdvancedVoiceMod} returns this
 */
proto.speech.tts.AdvancedVoiceMod.prototype.clearCopycatParams = function() {
  return this.setCopycatParams(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.AdvancedVoiceMod.prototype.hasCopycatParams = function() {
  return jspb.Message.getField(this, 11) != null;
};


/**
 * optional SpeechMorphingTargets morphing = 12;
 * @return {?proto.speech.tts.SpeechMorphingTargets}
 */
proto.speech.tts.AdvancedVoiceMod.prototype.getMorphing = function() {
  return /** @type{?proto.speech.tts.SpeechMorphingTargets} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.SpeechMorphingTargets, 12));
};


/**
 * @param {?proto.speech.tts.SpeechMorphingTargets|undefined} value
 * @return {!proto.speech.tts.AdvancedVoiceMod} returns this
*/
proto.speech.tts.AdvancedVoiceMod.prototype.setMorphing = function(value) {
  return jspb.Message.setWrapperField(this, 12, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.AdvancedVoiceMod} returns this
 */
proto.speech.tts.AdvancedVoiceMod.prototype.clearMorphing = function() {
  return this.setMorphing(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.AdvancedVoiceMod.prototype.hasMorphing = function() {
  return jspb.Message.getField(this, 12) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.AdvancedVoiceMod}
 */
proto.speech.tts.AdvancedVoiceMod.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.AdvancedVoiceMod, data);
};



//speech/tts/public/voice_mod_proto_jspb/voicemodpair.jspb.js
// source: speech/tts/public/voice_mod.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.VoiceModPair');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.VoiceModPair = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.VoiceModPair, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.VoiceModPair.displayName = 'proto.speech.tts.VoiceModPair';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.VoiceModPair.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.VoiceModPair.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.VoiceModPair} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.VoiceModPair.toObject = function(includeInstance, msg) {
  var f, obj = {
    t: (f = jspb.Message.getOptionalFloatingPointField(msg, 1)) == null ? undefined : f,
    v: (f = jspb.Message.getOptionalFloatingPointField(msg, 2)) == null ? undefined : f,
    deviation: jspb.Message.getFloatingPointFieldWithDefault(msg, 3, 1.0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of VoiceModPair as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.VoiceModPair.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.t;

  /** @type {?number|undefined} */
  this.v;

  /** @type {?number|undefined} */
  this.deviation;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.VoiceModPair.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.VoiceModPair}
 */
proto.speech.tts.VoiceModPair.fromObject = function(obj) {
  var msg = new proto.speech.tts.VoiceModPair();
  obj.t != null && jspb.Message.setField(msg, 1, obj.t);
  obj.v != null && jspb.Message.setField(msg, 2, obj.v);
  obj.deviation != null && jspb.Message.setField(msg, 3, obj.deviation);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.VoiceModPair}
 */
proto.speech.tts.VoiceModPair.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.VoiceModPair;
  return proto.speech.tts.VoiceModPair.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.VoiceModPair} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.VoiceModPair}
 */
proto.speech.tts.VoiceModPair.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setT(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setV(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setDeviation(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.VoiceModPair.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.VoiceModPair.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.VoiceModPair} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.VoiceModPair.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {number} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeFloat(
      1,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeFloat(
      2,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeFloat(
      3,
      f
    );
  }
};


/**
 * required float t = 1;
 * @return {number}
 */
proto.speech.tts.VoiceModPair.prototype.getT = function() {
  return /** @type {number} */ (jspb.Message.getOptionalFloatingPointField(this, 1));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.VoiceModPair} returns this
 */
proto.speech.tts.VoiceModPair.prototype.setT = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.VoiceModPair} returns this
 */
proto.speech.tts.VoiceModPair.prototype.clearT = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.VoiceModPair.prototype.hasT = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * required float v = 2;
 * @return {number}
 */
proto.speech.tts.VoiceModPair.prototype.getV = function() {
  return /** @type {number} */ (jspb.Message.getOptionalFloatingPointField(this, 2));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.VoiceModPair} returns this
 */
proto.speech.tts.VoiceModPair.prototype.setV = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.VoiceModPair} returns this
 */
proto.speech.tts.VoiceModPair.prototype.clearV = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.VoiceModPair.prototype.hasV = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional float deviation = 3;
 * @return {number}
 */
proto.speech.tts.VoiceModPair.prototype.getDeviation = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 3, 1.0));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.VoiceModPair} returns this
 */
proto.speech.tts.VoiceModPair.prototype.setDeviation = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.VoiceModPair} returns this
 */
proto.speech.tts.VoiceModPair.prototype.clearDeviation = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.VoiceModPair.prototype.hasDeviation = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.VoiceModPair}
 */
proto.speech.tts.VoiceModPair.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.VoiceModPair, data);
};



//speech/tts/public/voice_mod_proto_jspb/voicemod.jspb.js
// source: speech/tts/public/voice_mod.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.VoiceMod');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.AdvancedVoiceMod');
goog.require('proto.speech.tts.VoiceModPair');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.VoiceMod = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.VoiceMod.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.VoiceMod, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.VoiceMod.displayName = 'proto.speech.tts.VoiceMod';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.VoiceMod.repeatedFields_ = [1,5];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.VoiceMod.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.VoiceMod.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.VoiceMod} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.VoiceMod.toObject = function(includeInstance, msg) {
  var f, obj = {
    f0TargetList: jspb.Message.toObjectList(msg.getF0TargetList(),
    proto.speech.tts.VoiceModPair.toObject, includeInstance),
    f0Scale: (f = jspb.Message.getOptionalFloatingPointField(msg, 2)) == null ? undefined : f,
    durScale: (f = jspb.Message.getOptionalFloatingPointField(msg, 3)) == null ? undefined : f,
    wordsPerMinute: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f,
    advancedList: jspb.Message.toObjectList(msg.getAdvancedList(),
    proto.speech.tts.AdvancedVoiceMod.toObject, includeInstance),
    f0DeviationScaling: jspb.Message.getFloatingPointFieldWithDefault(msg, 6, 1.0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of VoiceMod as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.VoiceMod.ObjectFormat = function() {
  /** @type {?Array<!proto.speech.tts.VoiceModPair.ObjectFormat>|undefined} */
  this.f0TargetList;

  /** @type {?number|undefined} */
  this.f0Scale;

  /** @type {?number|undefined} */
  this.durScale;

  /** @type {?number|undefined} */
  this.wordsPerMinute;

  /** @type {?Array<!proto.speech.tts.AdvancedVoiceMod.ObjectFormat>|undefined} */
  this.advancedList;

  /** @type {?number|undefined} */
  this.f0DeviationScaling;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.VoiceMod.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.VoiceMod}
 */
proto.speech.tts.VoiceMod.fromObject = function(obj) {
  var msg = new proto.speech.tts.VoiceMod();
  obj.f0TargetList && jspb.Message.setRepeatedWrapperField(
      msg, 1, obj.f0TargetList.map(
          proto.speech.tts.VoiceModPair.fromObject));
  obj.f0Scale != null && jspb.Message.setField(msg, 2, obj.f0Scale);
  obj.durScale != null && jspb.Message.setField(msg, 3, obj.durScale);
  obj.wordsPerMinute != null && jspb.Message.setField(msg, 4, obj.wordsPerMinute);
  obj.advancedList && jspb.Message.setRepeatedWrapperField(
      msg, 5, obj.advancedList.map(
          proto.speech.tts.AdvancedVoiceMod.fromObject));
  obj.f0DeviationScaling != null && jspb.Message.setField(msg, 6, obj.f0DeviationScaling);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.VoiceMod}
 */
proto.speech.tts.VoiceMod.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.VoiceMod;
  return proto.speech.tts.VoiceMod.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.VoiceMod} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.VoiceMod}
 */
proto.speech.tts.VoiceMod.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.speech.tts.VoiceModPair;
      reader.readMessage(value,proto.speech.tts.VoiceModPair.deserializeBinaryFromReader);
      msg.addF0Target(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setF0Scale(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setDurScale(value);
      break;
    case 4:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setWordsPerMinute(value);
      break;
    case 5:
      var value = new proto.speech.tts.AdvancedVoiceMod;
      reader.readMessage(value,proto.speech.tts.AdvancedVoiceMod.deserializeBinaryFromReader);
      msg.addAdvanced(value);
      break;
    case 6:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setF0DeviationScaling(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.VoiceMod.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.VoiceMod.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.VoiceMod} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.VoiceMod.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getF0TargetList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      1,
      f,
      proto.speech.tts.VoiceModPair.serializeBinaryToWriter
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeFloat(
      2,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeFloat(
      3,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeInt32(
      4,
      f
    );
  }
  f = message.getAdvancedList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      5,
      f,
      proto.speech.tts.AdvancedVoiceMod.serializeBinaryToWriter
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeFloat(
      6,
      f
    );
  }
};


/**
 * repeated VoiceModPair f0_target = 1;
 * @return {!Array<!proto.speech.tts.VoiceModPair>}
 */
proto.speech.tts.VoiceMod.prototype.getF0TargetList = function() {
  return /** @type{!Array<!proto.speech.tts.VoiceModPair>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.VoiceModPair, 1));
};


/**
 * @param {Array<!proto.speech.tts.VoiceModPair>|undefined} value
 * @return {!proto.speech.tts.VoiceMod} returns this
*/
proto.speech.tts.VoiceMod.prototype.setF0TargetList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 1, value);
};


/**
 * @param {!proto.speech.tts.VoiceModPair=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.VoiceModPair}
 */
proto.speech.tts.VoiceMod.prototype.addF0Target = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.speech.tts.VoiceModPair, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.VoiceMod} returns this
 */
proto.speech.tts.VoiceMod.prototype.clearF0TargetList = function() {
  return this.setF0TargetList([]);
};


/**
 * optional float f0_scale = 2;
 * @return {?number}
 */
proto.speech.tts.VoiceMod.prototype.getF0Scale = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 2));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.VoiceMod} returns this
 */
proto.speech.tts.VoiceMod.prototype.setF0Scale = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.VoiceMod} returns this
 */
proto.speech.tts.VoiceMod.prototype.clearF0Scale = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.VoiceMod.prototype.hasF0Scale = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional float dur_scale = 3;
 * @return {?number}
 */
proto.speech.tts.VoiceMod.prototype.getDurScale = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 3));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.VoiceMod} returns this
 */
proto.speech.tts.VoiceMod.prototype.setDurScale = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.VoiceMod} returns this
 */
proto.speech.tts.VoiceMod.prototype.clearDurScale = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.VoiceMod.prototype.hasDurScale = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional int32 words_per_minute = 4;
 * @return {?number}
 */
proto.speech.tts.VoiceMod.prototype.getWordsPerMinute = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 4));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.VoiceMod} returns this
 */
proto.speech.tts.VoiceMod.prototype.setWordsPerMinute = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.VoiceMod} returns this
 */
proto.speech.tts.VoiceMod.prototype.clearWordsPerMinute = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.VoiceMod.prototype.hasWordsPerMinute = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * repeated AdvancedVoiceMod advanced = 5;
 * @return {!Array<!proto.speech.tts.AdvancedVoiceMod>}
 */
proto.speech.tts.VoiceMod.prototype.getAdvancedList = function() {
  return /** @type{!Array<!proto.speech.tts.AdvancedVoiceMod>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.AdvancedVoiceMod, 5));
};


/**
 * @param {Array<!proto.speech.tts.AdvancedVoiceMod>|undefined} value
 * @return {!proto.speech.tts.VoiceMod} returns this
*/
proto.speech.tts.VoiceMod.prototype.setAdvancedList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 5, value);
};


/**
 * @param {!proto.speech.tts.AdvancedVoiceMod=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.AdvancedVoiceMod}
 */
proto.speech.tts.VoiceMod.prototype.addAdvanced = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 5, opt_value, proto.speech.tts.AdvancedVoiceMod, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.VoiceMod} returns this
 */
proto.speech.tts.VoiceMod.prototype.clearAdvancedList = function() {
  return this.setAdvancedList([]);
};


/**
 * optional float f0_deviation_scaling = 6;
 * @return {number}
 */
proto.speech.tts.VoiceMod.prototype.getF0DeviationScaling = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 6, 1.0));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.VoiceMod} returns this
 */
proto.speech.tts.VoiceMod.prototype.setF0DeviationScaling = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.VoiceMod} returns this
 */
proto.speech.tts.VoiceMod.prototype.clearF0DeviationScaling = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.VoiceMod.prototype.hasF0DeviationScaling = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.VoiceMod}
 */
proto.speech.tts.VoiceMod.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.VoiceMod, data);
};



//speech/tts/engine/utterance/linguistic_structure_proto_jspb/token.jspb.js
// source: speech/tts/engine/utterance/token.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.Token');
goog.provide('proto.speech.tts.Token.Type');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.Abbreviation');
goog.require('proto.speech.tts.Address');
goog.require('proto.speech.tts.Cardinal');
goog.require('proto.speech.tts.ChemicalFormula');
goog.require('proto.speech.tts.Connector');
goog.require('proto.speech.tts.Date');
goog.require('proto.speech.tts.Decimal');
goog.require('proto.speech.tts.Duplicator');
goog.require('proto.speech.tts.Electronic');
goog.require('proto.speech.tts.EmoticonEmoji');
goog.require('proto.speech.tts.Fraction');
goog.require('proto.speech.tts.LexiconId');
goog.require('proto.speech.tts.Links');
goog.require('proto.speech.tts.Measure');
goog.require('proto.speech.tts.Money');
goog.require('proto.speech.tts.Ordinal');
goog.require('proto.speech.tts.Spelling');
goog.require('proto.speech.tts.Syntax');
goog.require('proto.speech.tts.Telephone');
goog.require('proto.speech.tts.Time');
goog.require('proto.speech.tts.VoiceMod');

goog.forwardDeclare('proto.speech.tts.PauseLength');
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.Token = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 66, null, null);
};
goog.inherits(proto.speech.tts.Token, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.Token.displayName = 'proto.speech.tts.Token';
}

/**
 * The extensions registered with this message class. This is a map of
 * extension field number to fieldInfo object.
 *
 * For example:
 *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }
 *
 * fieldName contains the JsCompiler renamed field name property so that it
 * works in OPTIMIZED mode.
 *
 * @type {!Object<number, jspb.ExtensionFieldInfo>}
 */
proto.speech.tts.Token.extensions = {};


/**
 * The extensions registered with this message class. This is a map of
 * extension field number to fieldInfo object.
 *
 * For example:
 *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }
 *
 * fieldName contains the JsCompiler renamed field name property so that it
 * works in OPTIMIZED mode.
 *
 * @type {!Object<number, jspb.ExtensionFieldBinaryInfo>}
 */
proto.speech.tts.Token.extensionsBinary = {};




if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.Token.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.Token.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.Token} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Token.toObject = function(includeInstance, msg) {
  var f, obj = {
    links: (f = msg.getLinks()) && proto.speech.tts.Links.toObject(includeInstance, f),
    startIndex: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    endIndex: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,
    name: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f,
    type: (f = jspb.Message.getField(msg, 5)) == null ? undefined : f,
    lexiconId: (f = msg.getLexiconId()) && proto.speech.tts.LexiconId.toObject(includeInstance, f),
    wordid: (f = jspb.Message.getField(msg, 6)) == null ? undefined : f,
    variant: (f = jspb.Message.getField(msg, 7)) == null ? undefined : f,
    spelling: (f = jspb.Message.getField(msg, 8)) == null ? undefined : f,
    phraseBreak: (f = jspb.Message.getBooleanField(msg, 9)) == null ? undefined : f,
    pauseDuration: (f = jspb.Message.getOptionalFloatingPointField(msg, 10)) == null ? undefined : f,
    pauseLength: jspb.Message.getFieldWithDefault(msg, 11, 0),
    spellingWithStress: (f = jspb.Message.getField(msg, 12)) == null ? undefined : f,
    skip: (f = jspb.Message.getBooleanField(msg, 13)) == null ? undefined : f,
    nextSpace: (f = jspb.Message.getBooleanField(msg, 14)) == null ? undefined : f,
    cardinal: (f = msg.getCardinal()) && proto.speech.tts.Cardinal.toObject(includeInstance, f),
    ordinal: (f = msg.getOrdinal()) && proto.speech.tts.Ordinal.toObject(includeInstance, f),
    digit: (f = jspb.Message.getField(msg, 22)) == null ? undefined : f,
    decimal: (f = msg.getDecimal()) && proto.speech.tts.Decimal.toObject(includeInstance, f),
    fraction: (f = msg.getFraction()) && proto.speech.tts.Fraction.toObject(includeInstance, f),
    time: (f = msg.getTime()) && proto.speech.tts.Time.toObject(includeInstance, f),
    measure: (f = msg.getMeasure()) && proto.speech.tts.Measure.toObject(includeInstance, f),
    percent: (f = msg.getPercent()) && proto.speech.tts.Decimal.toObject(includeInstance, f),
    date: (f = msg.getDate()) && proto.speech.tts.Date.toObject(includeInstance, f),
    address: (f = msg.getAddress()) && proto.speech.tts.Address.toObject(includeInstance, f),
    telephone: (f = msg.getTelephone()) && proto.speech.tts.Telephone.toObject(includeInstance, f),
    money: (f = msg.getMoney()) && proto.speech.tts.Money.toObject(includeInstance, f),
    electronic: (f = msg.getElectronic()) && proto.speech.tts.Electronic.toObject(includeInstance, f),
    verbatim: (f = jspb.Message.getField(msg, 33)) == null ? undefined : f,
    letters: (f = jspb.Message.getField(msg, 34)) == null ? undefined : f,
    connector: (f = msg.getConnector()) && proto.speech.tts.Connector.toObject(includeInstance, f),
    concept: (f = jspb.Message.getField(msg, 36)) == null ? undefined : f,
    abbreviation: (f = msg.getAbbreviation()) && proto.speech.tts.Abbreviation.toObject(includeInstance, f),
    duplicator: (f = msg.getDuplicator()) && proto.speech.tts.Duplicator.toObject(includeInstance, f),
    chemicalFormula: (f = msg.getChemicalFormula()) && proto.speech.tts.ChemicalFormula.toObject(includeInstance, f),
    emoticonEmoji: (f = msg.getEmoticonEmoji()) && proto.speech.tts.EmoticonEmoji.toObject(includeInstance, f),
    syntax: (f = msg.getSyntax()) && proto.speech.tts.Syntax.toObject(includeInstance, f),
    voiceMod: (f = msg.getVoiceMod()) && proto.speech.tts.VoiceMod.toObject(includeInstance, f),
    userFeatures: (f = jspb.Message.getField(msg, 52)) == null ? undefined : f,
    conciseEmoji: (f = jspb.Message.getBooleanField(msg, 53)) == null ? undefined : f,
    detectedLanguage: (f = jspb.Message.getField(msg, 54)) == null ? undefined : f,
    languageDetectionProbability: (f = jspb.Message.getOptionalFloatingPointField(msg, 65)) == null ? undefined : f,
    bleep: (f = jspb.Message.getBooleanField(msg, 55)) == null ? undefined : f,
    spell: (f = msg.getSpell()) && proto.speech.tts.Spelling.toObject(includeInstance, f),
    prefix: (f = jspb.Message.getField(msg, 57)) == null ? undefined : f,
    pitchAccentPhraseStart: (f = jspb.Message.getBooleanField(msg, 58)) == null ? undefined : f,
    homographResolvedVia: (f = jspb.Message.getField(msg, 59)) == null ? undefined : f,
    suffix: (f = jspb.Message.getField(msg, 60)) == null ? undefined : f,
    foreignScript: (f = jspb.Message.getField(msg, 61)) == null ? undefined : f,
    verbalizationStyle: (f = jspb.Message.getField(msg, 63)) == null ? undefined : f,
    wordHasUnknownGraphemes: (f = jspb.Message.getBooleanField(msg, 64)) == null ? undefined : f
  };

  jspb.Message.toObjectExtension(/** @type {!jspb.Message} */ (msg), obj,
      proto.speech.tts.Token.extensions, proto.speech.tts.Token.prototype.getExtension,
      includeInstance);
  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Token as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.Token.ObjectFormat = function() {
  /** @type {?proto.speech.tts.Links.ObjectFormat|undefined} */
  this.links;

  /** @type {?number|undefined} */
  this.startIndex;

  /** @type {?number|undefined} */
  this.endIndex;

  /** @type {?string|undefined} */
  this.name;

  /** @type {?number|undefined} */
  this.type;

  /** @type {?proto.speech.tts.LexiconId.ObjectFormat|undefined} */
  this.lexiconId;

  /** @type {?string|undefined} */
  this.wordid;

  /** @type {?string|undefined} */
  this.variant;

  /** @type {?string|undefined} */
  this.spelling;

  /** @type {?boolean|undefined} */
  this.phraseBreak;

  /** @type {?number|undefined} */
  this.pauseDuration;

  /** @type {?number|undefined} */
  this.pauseLength;

  /** @type {?string|undefined} */
  this.spellingWithStress;

  /** @type {?boolean|undefined} */
  this.skip;

  /** @type {?boolean|undefined} */
  this.nextSpace;

  /** @type {?proto.speech.tts.Cardinal.ObjectFormat|undefined} */
  this.cardinal;

  /** @type {?proto.speech.tts.Ordinal.ObjectFormat|undefined} */
  this.ordinal;

  /** @type {?string|undefined} */
  this.digit;

  /** @type {?proto.speech.tts.Decimal.ObjectFormat|undefined} */
  this.decimal;

  /** @type {?proto.speech.tts.Fraction.ObjectFormat|undefined} */
  this.fraction;

  /** @type {?proto.speech.tts.Time.ObjectFormat|undefined} */
  this.time;

  /** @type {?proto.speech.tts.Measure.ObjectFormat|undefined} */
  this.measure;

  /** @type {?proto.speech.tts.Decimal.ObjectFormat|undefined} */
  this.percent;

  /** @type {?proto.speech.tts.Date.ObjectFormat|undefined} */
  this.date;

  /** @type {?proto.speech.tts.Address.ObjectFormat|undefined} */
  this.address;

  /** @type {?proto.speech.tts.Telephone.ObjectFormat|undefined} */
  this.telephone;

  /** @type {?proto.speech.tts.Money.ObjectFormat|undefined} */
  this.money;

  /** @type {?proto.speech.tts.Electronic.ObjectFormat|undefined} */
  this.electronic;

  /** @type {?string|undefined} */
  this.verbatim;

  /** @type {?string|undefined} */
  this.letters;

  /** @type {?proto.speech.tts.Connector.ObjectFormat|undefined} */
  this.connector;

  /** @type {?string|undefined} */
  this.concept;

  /** @type {?proto.speech.tts.Abbreviation.ObjectFormat|undefined} */
  this.abbreviation;

  /** @type {?proto.speech.tts.Duplicator.ObjectFormat|undefined} */
  this.duplicator;

  /** @type {?proto.speech.tts.ChemicalFormula.ObjectFormat|undefined} */
  this.chemicalFormula;

  /** @type {?proto.speech.tts.EmoticonEmoji.ObjectFormat|undefined} */
  this.emoticonEmoji;

  /** @type {?proto.speech.tts.Syntax.ObjectFormat|undefined} */
  this.syntax;

  /** @type {?proto.speech.tts.VoiceMod.ObjectFormat|undefined} */
  this.voiceMod;

  /** @type {?string|undefined} */
  this.userFeatures;

  /** @type {?boolean|undefined} */
  this.conciseEmoji;

  /** @type {?string|undefined} */
  this.detectedLanguage;

  /** @type {?number|undefined} */
  this.languageDetectionProbability;

  /** @type {?boolean|undefined} */
  this.bleep;

  /** @type {?proto.speech.tts.Spelling.ObjectFormat|undefined} */
  this.spell;

  /** @type {?string|undefined} */
  this.prefix;

  /** @type {?boolean|undefined} */
  this.pitchAccentPhraseStart;

  /** @type {?string|undefined} */
  this.homographResolvedVia;

  /** @type {?string|undefined} */
  this.suffix;

  /** @type {?string|undefined} */
  this.foreignScript;

  /** @type {?string|undefined} */
  this.verbalizationStyle;

  /** @type {?boolean|undefined} */
  this.wordHasUnknownGraphemes;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.Token.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.Token}
 */
proto.speech.tts.Token.fromObject = function(obj) {
  var msg = new proto.speech.tts.Token();
  obj.links && jspb.Message.setWrapperField(
      msg, 1, proto.speech.tts.Links.fromObject(obj.links));
  obj.startIndex != null && jspb.Message.setField(msg, 2, obj.startIndex);
  obj.endIndex != null && jspb.Message.setField(msg, 3, obj.endIndex);
  obj.name != null && jspb.Message.setField(msg, 4, obj.name);
  obj.type != null && jspb.Message.setField(msg, 5, obj.type);
  obj.lexiconId && jspb.Message.setWrapperField(
      msg, 62, proto.speech.tts.LexiconId.fromObject(obj.lexiconId));
  obj.wordid != null && jspb.Message.setField(msg, 6, obj.wordid);
  obj.variant != null && jspb.Message.setField(msg, 7, obj.variant);
  obj.spelling != null && jspb.Message.setField(msg, 8, obj.spelling);
  obj.phraseBreak != null && jspb.Message.setField(msg, 9, obj.phraseBreak);
  obj.pauseDuration != null && jspb.Message.setField(msg, 10, obj.pauseDuration);
  obj.pauseLength != null && jspb.Message.setField(msg, 11, obj.pauseLength);
  obj.spellingWithStress != null && jspb.Message.setField(msg, 12, obj.spellingWithStress);
  obj.skip != null && jspb.Message.setField(msg, 13, obj.skip);
  obj.nextSpace != null && jspb.Message.setField(msg, 14, obj.nextSpace);
  obj.cardinal && jspb.Message.setWrapperField(
      msg, 20, proto.speech.tts.Cardinal.fromObject(obj.cardinal));
  obj.ordinal && jspb.Message.setWrapperField(
      msg, 21, proto.speech.tts.Ordinal.fromObject(obj.ordinal));
  obj.digit != null && jspb.Message.setField(msg, 22, obj.digit);
  obj.decimal && jspb.Message.setWrapperField(
      msg, 23, proto.speech.tts.Decimal.fromObject(obj.decimal));
  obj.fraction && jspb.Message.setWrapperField(
      msg, 24, proto.speech.tts.Fraction.fromObject(obj.fraction));
  obj.time && jspb.Message.setWrapperField(
      msg, 25, proto.speech.tts.Time.fromObject(obj.time));
  obj.measure && jspb.Message.setWrapperField(
      msg, 26, proto.speech.tts.Measure.fromObject(obj.measure));
  obj.percent && jspb.Message.setWrapperField(
      msg, 27, proto.speech.tts.Decimal.fromObject(obj.percent));
  obj.date && jspb.Message.setWrapperField(
      msg, 28, proto.speech.tts.Date.fromObject(obj.date));
  obj.address && jspb.Message.setWrapperField(
      msg, 29, proto.speech.tts.Address.fromObject(obj.address));
  obj.telephone && jspb.Message.setWrapperField(
      msg, 30, proto.speech.tts.Telephone.fromObject(obj.telephone));
  obj.money && jspb.Message.setWrapperField(
      msg, 31, proto.speech.tts.Money.fromObject(obj.money));
  obj.electronic && jspb.Message.setWrapperField(
      msg, 32, proto.speech.tts.Electronic.fromObject(obj.electronic));
  obj.verbatim != null && jspb.Message.setField(msg, 33, obj.verbatim);
  obj.letters != null && jspb.Message.setField(msg, 34, obj.letters);
  obj.connector && jspb.Message.setWrapperField(
      msg, 35, proto.speech.tts.Connector.fromObject(obj.connector));
  obj.concept != null && jspb.Message.setField(msg, 36, obj.concept);
  obj.abbreviation && jspb.Message.setWrapperField(
      msg, 37, proto.speech.tts.Abbreviation.fromObject(obj.abbreviation));
  obj.duplicator && jspb.Message.setWrapperField(
      msg, 38, proto.speech.tts.Duplicator.fromObject(obj.duplicator));
  obj.chemicalFormula && jspb.Message.setWrapperField(
      msg, 39, proto.speech.tts.ChemicalFormula.fromObject(obj.chemicalFormula));
  obj.emoticonEmoji && jspb.Message.setWrapperField(
      msg, 40, proto.speech.tts.EmoticonEmoji.fromObject(obj.emoticonEmoji));
  obj.syntax && jspb.Message.setWrapperField(
      msg, 50, proto.speech.tts.Syntax.fromObject(obj.syntax));
  obj.voiceMod && jspb.Message.setWrapperField(
      msg, 51, proto.speech.tts.VoiceMod.fromObject(obj.voiceMod));
  obj.userFeatures != null && jspb.Message.setField(msg, 52, obj.userFeatures);
  obj.conciseEmoji != null && jspb.Message.setField(msg, 53, obj.conciseEmoji);
  obj.detectedLanguage != null && jspb.Message.setField(msg, 54, obj.detectedLanguage);
  obj.languageDetectionProbability != null && jspb.Message.setField(msg, 65, obj.languageDetectionProbability);
  obj.bleep != null && jspb.Message.setField(msg, 55, obj.bleep);
  obj.spell && jspb.Message.setWrapperField(
      msg, 56, proto.speech.tts.Spelling.fromObject(obj.spell));
  obj.prefix != null && jspb.Message.setField(msg, 57, obj.prefix);
  obj.pitchAccentPhraseStart != null && jspb.Message.setField(msg, 58, obj.pitchAccentPhraseStart);
  obj.homographResolvedVia != null && jspb.Message.setField(msg, 59, obj.homographResolvedVia);
  obj.suffix != null && jspb.Message.setField(msg, 60, obj.suffix);
  obj.foreignScript != null && jspb.Message.setField(msg, 61, obj.foreignScript);
  obj.verbalizationStyle != null && jspb.Message.setField(msg, 63, obj.verbalizationStyle);
  obj.wordHasUnknownGraphemes != null && jspb.Message.setField(msg, 64, obj.wordHasUnknownGraphemes);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.Token}
 */
proto.speech.tts.Token.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.Token;
  return proto.speech.tts.Token.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.Token} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.Token}
 */
proto.speech.tts.Token.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.speech.tts.Links;
      reader.readMessage(value,proto.speech.tts.Links.deserializeBinaryFromReader);
      msg.setLinks(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readUint32());
      msg.setStartIndex(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readUint32());
      msg.setEndIndex(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setName(value);
      break;
    case 5:
      var value = /** @type {!proto.speech.tts.Token.Type} */ (reader.readEnum());
      msg.setType(value);
      break;
    case 62:
      var value = new proto.speech.tts.LexiconId;
      reader.readMessage(value,proto.speech.tts.LexiconId.deserializeBinaryFromReader);
      msg.setLexiconId(value);
      break;
    case 6:
      var value = /** @type {string} */ (reader.readString());
      msg.setWordid(value);
      break;
    case 7:
      var value = /** @type {string} */ (reader.readString());
      msg.setVariant(value);
      break;
    case 8:
      var value = /** @type {string} */ (reader.readString());
      msg.setSpelling(value);
      break;
    case 9:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setPhraseBreak(value);
      break;
    case 10:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setPauseDuration(value);
      break;
    case 11:
      var value = /** @type {!proto.speech.tts.PauseLength} */ (reader.readEnum());
      msg.setPauseLength(value);
      break;
    case 12:
      var value = /** @type {string} */ (reader.readString());
      msg.setSpellingWithStress(value);
      break;
    case 13:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setSkip(value);
      break;
    case 14:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setNextSpace(value);
      break;
    case 20:
      var value = new proto.speech.tts.Cardinal;
      reader.readMessage(value,proto.speech.tts.Cardinal.deserializeBinaryFromReader);
      msg.setCardinal(value);
      break;
    case 21:
      var value = new proto.speech.tts.Ordinal;
      reader.readMessage(value,proto.speech.tts.Ordinal.deserializeBinaryFromReader);
      msg.setOrdinal(value);
      break;
    case 22:
      var value = /** @type {string} */ (reader.readString());
      msg.setDigit(value);
      break;
    case 23:
      var value = new proto.speech.tts.Decimal;
      reader.readMessage(value,proto.speech.tts.Decimal.deserializeBinaryFromReader);
      msg.setDecimal(value);
      break;
    case 24:
      var value = new proto.speech.tts.Fraction;
      reader.readMessage(value,proto.speech.tts.Fraction.deserializeBinaryFromReader);
      msg.setFraction(value);
      break;
    case 25:
      var value = new proto.speech.tts.Time;
      reader.readMessage(value,proto.speech.tts.Time.deserializeBinaryFromReader);
      msg.setTime(value);
      break;
    case 26:
      var value = new proto.speech.tts.Measure;
      reader.readMessage(value,proto.speech.tts.Measure.deserializeBinaryFromReader);
      msg.setMeasure(value);
      break;
    case 27:
      var value = new proto.speech.tts.Decimal;
      reader.readMessage(value,proto.speech.tts.Decimal.deserializeBinaryFromReader);
      msg.setPercent(value);
      break;
    case 28:
      var value = new proto.speech.tts.Date;
      reader.readMessage(value,proto.speech.tts.Date.deserializeBinaryFromReader);
      msg.setDate(value);
      break;
    case 29:
      var value = new proto.speech.tts.Address;
      reader.readMessage(value,proto.speech.tts.Address.deserializeBinaryFromReader);
      msg.setAddress(value);
      break;
    case 30:
      var value = new proto.speech.tts.Telephone;
      reader.readMessage(value,proto.speech.tts.Telephone.deserializeBinaryFromReader);
      msg.setTelephone(value);
      break;
    case 31:
      var value = new proto.speech.tts.Money;
      reader.readMessage(value,proto.speech.tts.Money.deserializeBinaryFromReader);
      msg.setMoney(value);
      break;
    case 32:
      var value = new proto.speech.tts.Electronic;
      reader.readMessage(value,proto.speech.tts.Electronic.deserializeBinaryFromReader);
      msg.setElectronic(value);
      break;
    case 33:
      var value = /** @type {string} */ (reader.readString());
      msg.setVerbatim(value);
      break;
    case 34:
      var value = /** @type {string} */ (reader.readString());
      msg.setLetters(value);
      break;
    case 35:
      var value = new proto.speech.tts.Connector;
      reader.readMessage(value,proto.speech.tts.Connector.deserializeBinaryFromReader);
      msg.setConnector(value);
      break;
    case 36:
      var value = /** @type {string} */ (reader.readString());
      msg.setConcept(value);
      break;
    case 37:
      var value = new proto.speech.tts.Abbreviation;
      reader.readMessage(value,proto.speech.tts.Abbreviation.deserializeBinaryFromReader);
      msg.setAbbreviation(value);
      break;
    case 38:
      var value = new proto.speech.tts.Duplicator;
      reader.readMessage(value,proto.speech.tts.Duplicator.deserializeBinaryFromReader);
      msg.setDuplicator(value);
      break;
    case 39:
      var value = new proto.speech.tts.ChemicalFormula;
      reader.readMessage(value,proto.speech.tts.ChemicalFormula.deserializeBinaryFromReader);
      msg.setChemicalFormula(value);
      break;
    case 40:
      var value = new proto.speech.tts.EmoticonEmoji;
      reader.readMessage(value,proto.speech.tts.EmoticonEmoji.deserializeBinaryFromReader);
      msg.setEmoticonEmoji(value);
      break;
    case 50:
      var value = new proto.speech.tts.Syntax;
      reader.readMessage(value,proto.speech.tts.Syntax.deserializeBinaryFromReader);
      msg.setSyntax(value);
      break;
    case 51:
      var value = new proto.speech.tts.VoiceMod;
      reader.readMessage(value,proto.speech.tts.VoiceMod.deserializeBinaryFromReader);
      msg.setVoiceMod(value);
      break;
    case 52:
      var value = /** @type {string} */ (reader.readString());
      msg.setUserFeatures(value);
      break;
    case 53:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setConciseEmoji(value);
      break;
    case 54:
      var value = /** @type {string} */ (reader.readString());
      msg.setDetectedLanguage(value);
      break;
    case 65:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setLanguageDetectionProbability(value);
      break;
    case 55:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setBleep(value);
      break;
    case 56:
      var value = new proto.speech.tts.Spelling;
      reader.readMessage(value,proto.speech.tts.Spelling.deserializeBinaryFromReader);
      msg.setSpell(value);
      break;
    case 57:
      var value = /** @type {string} */ (reader.readString());
      msg.setPrefix(value);
      break;
    case 58:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setPitchAccentPhraseStart(value);
      break;
    case 59:
      var value = /** @type {string} */ (reader.readString());
      msg.setHomographResolvedVia(value);
      break;
    case 60:
      var value = /** @type {string} */ (reader.readString());
      msg.setSuffix(value);
      break;
    case 61:
      var value = /** @type {string} */ (reader.readString());
      msg.setForeignScript(value);
      break;
    case 63:
      var value = /** @type {string} */ (reader.readString());
      msg.setVerbalizationStyle(value);
      break;
    case 64:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setWordHasUnknownGraphemes(value);
      break;
    default:
      jspb.Message.readBinaryExtension(msg, reader,
        proto.speech.tts.Token.extensionsBinary,
        proto.speech.tts.Token.prototype.getExtension,
        proto.speech.tts.Token.prototype.setExtension);
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.Token.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.Token.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.Token} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Token.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getLinks();
  if (f != null) {
    writer.writeMessage(
      1,
      f,
      proto.speech.tts.Links.serializeBinaryToWriter
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeUint32(
      2,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeUint32(
      3,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeString(
      4,
      f
    );
  }
  f = /** @type {!proto.speech.tts.Token.Type} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeEnum(
      5,
      f
    );
  }
  f = message.getLexiconId();
  if (f != null) {
    writer.writeMessage(
      62,
      f,
      proto.speech.tts.LexiconId.serializeBinaryToWriter
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeString(
      6,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 7));
  if (f != null) {
    writer.writeString(
      7,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 8));
  if (f != null) {
    writer.writeString(
      8,
      f
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 9));
  if (f != null) {
    writer.writeBool(
      9,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 10));
  if (f != null) {
    writer.writeFloat(
      10,
      f
    );
  }
  f = /** @type {!proto.speech.tts.PauseLength} */ (jspb.Message.getField(message, 11));
  if (f != null) {
    writer.writeEnum(
      11,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 12));
  if (f != null) {
    writer.writeString(
      12,
      f
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 13));
  if (f != null) {
    writer.writeBool(
      13,
      f
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 14));
  if (f != null) {
    writer.writeBool(
      14,
      f
    );
  }
  f = message.getCardinal();
  if (f != null) {
    writer.writeMessage(
      20,
      f,
      proto.speech.tts.Cardinal.serializeBinaryToWriter
    );
  }
  f = message.getOrdinal();
  if (f != null) {
    writer.writeMessage(
      21,
      f,
      proto.speech.tts.Ordinal.serializeBinaryToWriter
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 22));
  if (f != null) {
    writer.writeString(
      22,
      f
    );
  }
  f = message.getDecimal();
  if (f != null) {
    writer.writeMessage(
      23,
      f,
      proto.speech.tts.Decimal.serializeBinaryToWriter
    );
  }
  f = message.getFraction();
  if (f != null) {
    writer.writeMessage(
      24,
      f,
      proto.speech.tts.Fraction.serializeBinaryToWriter
    );
  }
  f = message.getTime();
  if (f != null) {
    writer.writeMessage(
      25,
      f,
      proto.speech.tts.Time.serializeBinaryToWriter
    );
  }
  f = message.getMeasure();
  if (f != null) {
    writer.writeMessage(
      26,
      f,
      proto.speech.tts.Measure.serializeBinaryToWriter
    );
  }
  f = message.getPercent();
  if (f != null) {
    writer.writeMessage(
      27,
      f,
      proto.speech.tts.Decimal.serializeBinaryToWriter
    );
  }
  f = message.getDate();
  if (f != null) {
    writer.writeMessage(
      28,
      f,
      proto.speech.tts.Date.serializeBinaryToWriter
    );
  }
  f = message.getAddress();
  if (f != null) {
    writer.writeMessage(
      29,
      f,
      proto.speech.tts.Address.serializeBinaryToWriter
    );
  }
  f = message.getTelephone();
  if (f != null) {
    writer.writeMessage(
      30,
      f,
      proto.speech.tts.Telephone.serializeBinaryToWriter
    );
  }
  f = message.getMoney();
  if (f != null) {
    writer.writeMessage(
      31,
      f,
      proto.speech.tts.Money.serializeBinaryToWriter
    );
  }
  f = message.getElectronic();
  if (f != null) {
    writer.writeMessage(
      32,
      f,
      proto.speech.tts.Electronic.serializeBinaryToWriter
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 33));
  if (f != null) {
    writer.writeString(
      33,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 34));
  if (f != null) {
    writer.writeString(
      34,
      f
    );
  }
  f = message.getConnector();
  if (f != null) {
    writer.writeMessage(
      35,
      f,
      proto.speech.tts.Connector.serializeBinaryToWriter
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 36));
  if (f != null) {
    writer.writeString(
      36,
      f
    );
  }
  f = message.getAbbreviation();
  if (f != null) {
    writer.writeMessage(
      37,
      f,
      proto.speech.tts.Abbreviation.serializeBinaryToWriter
    );
  }
  f = message.getDuplicator();
  if (f != null) {
    writer.writeMessage(
      38,
      f,
      proto.speech.tts.Duplicator.serializeBinaryToWriter
    );
  }
  f = message.getChemicalFormula();
  if (f != null) {
    writer.writeMessage(
      39,
      f,
      proto.speech.tts.ChemicalFormula.serializeBinaryToWriter
    );
  }
  f = message.getEmoticonEmoji();
  if (f != null) {
    writer.writeMessage(
      40,
      f,
      proto.speech.tts.EmoticonEmoji.serializeBinaryToWriter
    );
  }
  f = message.getSyntax();
  if (f != null) {
    writer.writeMessage(
      50,
      f,
      proto.speech.tts.Syntax.serializeBinaryToWriter
    );
  }
  f = message.getVoiceMod();
  if (f != null) {
    writer.writeMessage(
      51,
      f,
      proto.speech.tts.VoiceMod.serializeBinaryToWriter
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 52));
  if (f != null) {
    writer.writeString(
      52,
      f
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 53));
  if (f != null) {
    writer.writeBool(
      53,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 54));
  if (f != null) {
    writer.writeString(
      54,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 65));
  if (f != null) {
    writer.writeFloat(
      65,
      f
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 55));
  if (f != null) {
    writer.writeBool(
      55,
      f
    );
  }
  f = message.getSpell();
  if (f != null) {
    writer.writeMessage(
      56,
      f,
      proto.speech.tts.Spelling.serializeBinaryToWriter
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 57));
  if (f != null) {
    writer.writeString(
      57,
      f
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 58));
  if (f != null) {
    writer.writeBool(
      58,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 59));
  if (f != null) {
    writer.writeString(
      59,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 60));
  if (f != null) {
    writer.writeString(
      60,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 61));
  if (f != null) {
    writer.writeString(
      61,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 63));
  if (f != null) {
    writer.writeString(
      63,
      f
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 64));
  if (f != null) {
    writer.writeBool(
      64,
      f
    );
  }
  jspb.Message.serializeBinaryExtensions(message, writer,
    proto.speech.tts.Token.extensionsBinary, proto.speech.tts.Token.prototype.getExtension);
};


/**
 * @enum {number}
 */
proto.speech.tts.Token.Type = {
  UNKNOWN: 0,
  WORD: 1,
  SEMIOTIC_CLASS: 2,
  HOMOGRAPH: 3,
  EXL: 4,
  PUNCT: 5,
  WORD_NEEDS_VERBALIZATION: 6,
  FOREIGN: 7
};

/**
 * optional Links links = 1;
 * @return {?proto.speech.tts.Links}
 */
proto.speech.tts.Token.prototype.getLinks = function() {
  return /** @type{?proto.speech.tts.Links} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Links, 1));
};


/**
 * @param {?proto.speech.tts.Links|undefined} value
 * @return {!proto.speech.tts.Token} returns this
*/
proto.speech.tts.Token.prototype.setLinks = function(value) {
  return jspb.Message.setWrapperField(this, 1, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.clearLinks = function() {
  return this.setLinks(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Token.prototype.hasLinks = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional uint32 start_index = 2;
 * @return {?number}
 */
proto.speech.tts.Token.prototype.getStartIndex = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.setStartIndex = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.clearStartIndex = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Token.prototype.hasStartIndex = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional uint32 end_index = 3;
 * @return {?number}
 */
proto.speech.tts.Token.prototype.getEndIndex = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.setEndIndex = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.clearEndIndex = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Token.prototype.hasEndIndex = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional string name = 4;
 * @return {?string}
 */
proto.speech.tts.Token.prototype.getName = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 4));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.setName = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.clearName = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Token.prototype.hasName = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional Type type = 5;
 * @return {!proto.speech.tts.Token.Type}
 */
proto.speech.tts.Token.prototype.getType = function() {
  return /** @type {!proto.speech.tts.Token.Type} */ (jspb.Message.getField(this, 5));
};


/**
 * @param {?proto.speech.tts.Token.Type|undefined} value
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.setType = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.clearType = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Token.prototype.hasType = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional LexiconId lexicon_id = 62;
 * @return {?proto.speech.tts.LexiconId}
 */
proto.speech.tts.Token.prototype.getLexiconId = function() {
  return /** @type{?proto.speech.tts.LexiconId} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.LexiconId, 62));
};


/**
 * @param {?proto.speech.tts.LexiconId|undefined} value
 * @return {!proto.speech.tts.Token} returns this
*/
proto.speech.tts.Token.prototype.setLexiconId = function(value) {
  return jspb.Message.setWrapperField(this, 62, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.clearLexiconId = function() {
  return this.setLexiconId(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Token.prototype.hasLexiconId = function() {
  return jspb.Message.getField(this, 62) != null;
};


/**
 * optional string wordid = 6;
 * @return {?string}
 */
proto.speech.tts.Token.prototype.getWordid = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 6));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.setWordid = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.clearWordid = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Token.prototype.hasWordid = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional string variant = 7;
 * @return {?string}
 */
proto.speech.tts.Token.prototype.getVariant = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 7));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.setVariant = function(value) {
  return jspb.Message.setField(this, 7, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.clearVariant = function() {
  return jspb.Message.setField(this, 7, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Token.prototype.hasVariant = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * optional string spelling = 8;
 * @return {?string}
 */
proto.speech.tts.Token.prototype.getSpelling = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 8));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.setSpelling = function(value) {
  return jspb.Message.setField(this, 8, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.clearSpelling = function() {
  return jspb.Message.setField(this, 8, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Token.prototype.hasSpelling = function() {
  return jspb.Message.getField(this, 8) != null;
};


/**
 * optional bool phrase_break = 9;
 * @return {?boolean}
 */
proto.speech.tts.Token.prototype.getPhraseBreak = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 9));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.setPhraseBreak = function(value) {
  return jspb.Message.setField(this, 9, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.clearPhraseBreak = function() {
  return jspb.Message.setField(this, 9, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Token.prototype.hasPhraseBreak = function() {
  return jspb.Message.getField(this, 9) != null;
};


/**
 * optional float pause_duration = 10;
 * @return {?number}
 */
proto.speech.tts.Token.prototype.getPauseDuration = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 10));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.setPauseDuration = function(value) {
  return jspb.Message.setField(this, 10, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.clearPauseDuration = function() {
  return jspb.Message.setField(this, 10, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Token.prototype.hasPauseDuration = function() {
  return jspb.Message.getField(this, 10) != null;
};


/**
 * optional PauseLength pause_length = 11;
 * @return {!proto.speech.tts.PauseLength}
 */
proto.speech.tts.Token.prototype.getPauseLength = function() {
  return /** @type {!proto.speech.tts.PauseLength} */ (jspb.Message.getFieldWithDefault(this, 11, 0));
};


/**
 * @param {?proto.speech.tts.PauseLength|undefined} value
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.setPauseLength = function(value) {
  return jspb.Message.setField(this, 11, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.clearPauseLength = function() {
  return jspb.Message.setField(this, 11, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Token.prototype.hasPauseLength = function() {
  return jspb.Message.getField(this, 11) != null;
};


/**
 * optional string spelling_with_stress = 12;
 * @return {?string}
 */
proto.speech.tts.Token.prototype.getSpellingWithStress = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 12));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.setSpellingWithStress = function(value) {
  return jspb.Message.setField(this, 12, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.clearSpellingWithStress = function() {
  return jspb.Message.setField(this, 12, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Token.prototype.hasSpellingWithStress = function() {
  return jspb.Message.getField(this, 12) != null;
};


/**
 * optional bool skip = 13;
 * @return {?boolean}
 */
proto.speech.tts.Token.prototype.getSkip = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 13));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.setSkip = function(value) {
  return jspb.Message.setField(this, 13, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.clearSkip = function() {
  return jspb.Message.setField(this, 13, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Token.prototype.hasSkip = function() {
  return jspb.Message.getField(this, 13) != null;
};


/**
 * optional bool next_space = 14;
 * @return {?boolean}
 */
proto.speech.tts.Token.prototype.getNextSpace = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 14));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.setNextSpace = function(value) {
  return jspb.Message.setField(this, 14, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.clearNextSpace = function() {
  return jspb.Message.setField(this, 14, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Token.prototype.hasNextSpace = function() {
  return jspb.Message.getField(this, 14) != null;
};


/**
 * optional Cardinal cardinal = 20;
 * @return {?proto.speech.tts.Cardinal}
 */
proto.speech.tts.Token.prototype.getCardinal = function() {
  return /** @type{?proto.speech.tts.Cardinal} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Cardinal, 20));
};


/**
 * @param {?proto.speech.tts.Cardinal|undefined} value
 * @return {!proto.speech.tts.Token} returns this
*/
proto.speech.tts.Token.prototype.setCardinal = function(value) {
  return jspb.Message.setWrapperField(this, 20, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.clearCardinal = function() {
  return this.setCardinal(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Token.prototype.hasCardinal = function() {
  return jspb.Message.getField(this, 20) != null;
};


/**
 * optional Ordinal ordinal = 21;
 * @return {?proto.speech.tts.Ordinal}
 */
proto.speech.tts.Token.prototype.getOrdinal = function() {
  return /** @type{?proto.speech.tts.Ordinal} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Ordinal, 21));
};


/**
 * @param {?proto.speech.tts.Ordinal|undefined} value
 * @return {!proto.speech.tts.Token} returns this
*/
proto.speech.tts.Token.prototype.setOrdinal = function(value) {
  return jspb.Message.setWrapperField(this, 21, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.clearOrdinal = function() {
  return this.setOrdinal(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Token.prototype.hasOrdinal = function() {
  return jspb.Message.getField(this, 21) != null;
};


/**
 * optional string digit = 22;
 * @return {?string}
 */
proto.speech.tts.Token.prototype.getDigit = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 22));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.setDigit = function(value) {
  return jspb.Message.setField(this, 22, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.clearDigit = function() {
  return jspb.Message.setField(this, 22, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Token.prototype.hasDigit = function() {
  return jspb.Message.getField(this, 22) != null;
};


/**
 * optional Decimal decimal = 23;
 * @return {?proto.speech.tts.Decimal}
 */
proto.speech.tts.Token.prototype.getDecimal = function() {
  return /** @type{?proto.speech.tts.Decimal} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Decimal, 23));
};


/**
 * @param {?proto.speech.tts.Decimal|undefined} value
 * @return {!proto.speech.tts.Token} returns this
*/
proto.speech.tts.Token.prototype.setDecimal = function(value) {
  return jspb.Message.setWrapperField(this, 23, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.clearDecimal = function() {
  return this.setDecimal(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Token.prototype.hasDecimal = function() {
  return jspb.Message.getField(this, 23) != null;
};


/**
 * optional Fraction fraction = 24;
 * @return {?proto.speech.tts.Fraction}
 */
proto.speech.tts.Token.prototype.getFraction = function() {
  return /** @type{?proto.speech.tts.Fraction} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Fraction, 24));
};


/**
 * @param {?proto.speech.tts.Fraction|undefined} value
 * @return {!proto.speech.tts.Token} returns this
*/
proto.speech.tts.Token.prototype.setFraction = function(value) {
  return jspb.Message.setWrapperField(this, 24, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.clearFraction = function() {
  return this.setFraction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Token.prototype.hasFraction = function() {
  return jspb.Message.getField(this, 24) != null;
};


/**
 * optional Time time = 25;
 * @return {?proto.speech.tts.Time}
 */
proto.speech.tts.Token.prototype.getTime = function() {
  return /** @type{?proto.speech.tts.Time} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Time, 25));
};


/**
 * @param {?proto.speech.tts.Time|undefined} value
 * @return {!proto.speech.tts.Token} returns this
*/
proto.speech.tts.Token.prototype.setTime = function(value) {
  return jspb.Message.setWrapperField(this, 25, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.clearTime = function() {
  return this.setTime(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Token.prototype.hasTime = function() {
  return jspb.Message.getField(this, 25) != null;
};


/**
 * optional Measure measure = 26;
 * @return {?proto.speech.tts.Measure}
 */
proto.speech.tts.Token.prototype.getMeasure = function() {
  return /** @type{?proto.speech.tts.Measure} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Measure, 26));
};


/**
 * @param {?proto.speech.tts.Measure|undefined} value
 * @return {!proto.speech.tts.Token} returns this
*/
proto.speech.tts.Token.prototype.setMeasure = function(value) {
  return jspb.Message.setWrapperField(this, 26, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.clearMeasure = function() {
  return this.setMeasure(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Token.prototype.hasMeasure = function() {
  return jspb.Message.getField(this, 26) != null;
};


/**
 * optional Decimal percent = 27;
 * @return {?proto.speech.tts.Decimal}
 */
proto.speech.tts.Token.prototype.getPercent = function() {
  return /** @type{?proto.speech.tts.Decimal} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Decimal, 27));
};


/**
 * @param {?proto.speech.tts.Decimal|undefined} value
 * @return {!proto.speech.tts.Token} returns this
*/
proto.speech.tts.Token.prototype.setPercent = function(value) {
  return jspb.Message.setWrapperField(this, 27, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.clearPercent = function() {
  return this.setPercent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Token.prototype.hasPercent = function() {
  return jspb.Message.getField(this, 27) != null;
};


/**
 * optional Date date = 28;
 * @return {?proto.speech.tts.Date}
 */
proto.speech.tts.Token.prototype.getDate = function() {
  return /** @type{?proto.speech.tts.Date} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Date, 28));
};


/**
 * @param {?proto.speech.tts.Date|undefined} value
 * @return {!proto.speech.tts.Token} returns this
*/
proto.speech.tts.Token.prototype.setDate = function(value) {
  return jspb.Message.setWrapperField(this, 28, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.clearDate = function() {
  return this.setDate(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Token.prototype.hasDate = function() {
  return jspb.Message.getField(this, 28) != null;
};


/**
 * optional Address address = 29;
 * @return {?proto.speech.tts.Address}
 */
proto.speech.tts.Token.prototype.getAddress = function() {
  return /** @type{?proto.speech.tts.Address} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Address, 29));
};


/**
 * @param {?proto.speech.tts.Address|undefined} value
 * @return {!proto.speech.tts.Token} returns this
*/
proto.speech.tts.Token.prototype.setAddress = function(value) {
  return jspb.Message.setWrapperField(this, 29, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.clearAddress = function() {
  return this.setAddress(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Token.prototype.hasAddress = function() {
  return jspb.Message.getField(this, 29) != null;
};


/**
 * optional Telephone telephone = 30;
 * @return {?proto.speech.tts.Telephone}
 */
proto.speech.tts.Token.prototype.getTelephone = function() {
  return /** @type{?proto.speech.tts.Telephone} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Telephone, 30));
};


/**
 * @param {?proto.speech.tts.Telephone|undefined} value
 * @return {!proto.speech.tts.Token} returns this
*/
proto.speech.tts.Token.prototype.setTelephone = function(value) {
  return jspb.Message.setWrapperField(this, 30, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.clearTelephone = function() {
  return this.setTelephone(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Token.prototype.hasTelephone = function() {
  return jspb.Message.getField(this, 30) != null;
};


/**
 * optional Money money = 31;
 * @return {?proto.speech.tts.Money}
 */
proto.speech.tts.Token.prototype.getMoney = function() {
  return /** @type{?proto.speech.tts.Money} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Money, 31));
};


/**
 * @param {?proto.speech.tts.Money|undefined} value
 * @return {!proto.speech.tts.Token} returns this
*/
proto.speech.tts.Token.prototype.setMoney = function(value) {
  return jspb.Message.setWrapperField(this, 31, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.clearMoney = function() {
  return this.setMoney(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Token.prototype.hasMoney = function() {
  return jspb.Message.getField(this, 31) != null;
};


/**
 * optional Electronic electronic = 32;
 * @return {?proto.speech.tts.Electronic}
 */
proto.speech.tts.Token.prototype.getElectronic = function() {
  return /** @type{?proto.speech.tts.Electronic} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Electronic, 32));
};


/**
 * @param {?proto.speech.tts.Electronic|undefined} value
 * @return {!proto.speech.tts.Token} returns this
*/
proto.speech.tts.Token.prototype.setElectronic = function(value) {
  return jspb.Message.setWrapperField(this, 32, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.clearElectronic = function() {
  return this.setElectronic(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Token.prototype.hasElectronic = function() {
  return jspb.Message.getField(this, 32) != null;
};


/**
 * optional string verbatim = 33;
 * @return {?string}
 */
proto.speech.tts.Token.prototype.getVerbatim = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 33));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.setVerbatim = function(value) {
  return jspb.Message.setField(this, 33, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.clearVerbatim = function() {
  return jspb.Message.setField(this, 33, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Token.prototype.hasVerbatim = function() {
  return jspb.Message.getField(this, 33) != null;
};


/**
 * optional string letters = 34;
 * @return {?string}
 */
proto.speech.tts.Token.prototype.getLetters = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 34));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.setLetters = function(value) {
  return jspb.Message.setField(this, 34, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.clearLetters = function() {
  return jspb.Message.setField(this, 34, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Token.prototype.hasLetters = function() {
  return jspb.Message.getField(this, 34) != null;
};


/**
 * optional Connector connector = 35;
 * @return {?proto.speech.tts.Connector}
 */
proto.speech.tts.Token.prototype.getConnector = function() {
  return /** @type{?proto.speech.tts.Connector} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Connector, 35));
};


/**
 * @param {?proto.speech.tts.Connector|undefined} value
 * @return {!proto.speech.tts.Token} returns this
*/
proto.speech.tts.Token.prototype.setConnector = function(value) {
  return jspb.Message.setWrapperField(this, 35, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.clearConnector = function() {
  return this.setConnector(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Token.prototype.hasConnector = function() {
  return jspb.Message.getField(this, 35) != null;
};


/**
 * optional string concept = 36;
 * @return {?string}
 */
proto.speech.tts.Token.prototype.getConcept = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 36));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.setConcept = function(value) {
  return jspb.Message.setField(this, 36, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.clearConcept = function() {
  return jspb.Message.setField(this, 36, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Token.prototype.hasConcept = function() {
  return jspb.Message.getField(this, 36) != null;
};


/**
 * optional Abbreviation abbreviation = 37;
 * @return {?proto.speech.tts.Abbreviation}
 */
proto.speech.tts.Token.prototype.getAbbreviation = function() {
  return /** @type{?proto.speech.tts.Abbreviation} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Abbreviation, 37));
};


/**
 * @param {?proto.speech.tts.Abbreviation|undefined} value
 * @return {!proto.speech.tts.Token} returns this
*/
proto.speech.tts.Token.prototype.setAbbreviation = function(value) {
  return jspb.Message.setWrapperField(this, 37, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.clearAbbreviation = function() {
  return this.setAbbreviation(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Token.prototype.hasAbbreviation = function() {
  return jspb.Message.getField(this, 37) != null;
};


/**
 * optional Duplicator duplicator = 38;
 * @return {?proto.speech.tts.Duplicator}
 */
proto.speech.tts.Token.prototype.getDuplicator = function() {
  return /** @type{?proto.speech.tts.Duplicator} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Duplicator, 38));
};


/**
 * @param {?proto.speech.tts.Duplicator|undefined} value
 * @return {!proto.speech.tts.Token} returns this
*/
proto.speech.tts.Token.prototype.setDuplicator = function(value) {
  return jspb.Message.setWrapperField(this, 38, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.clearDuplicator = function() {
  return this.setDuplicator(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Token.prototype.hasDuplicator = function() {
  return jspb.Message.getField(this, 38) != null;
};


/**
 * optional ChemicalFormula chemical_formula = 39;
 * @return {?proto.speech.tts.ChemicalFormula}
 */
proto.speech.tts.Token.prototype.getChemicalFormula = function() {
  return /** @type{?proto.speech.tts.ChemicalFormula} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.ChemicalFormula, 39));
};


/**
 * @param {?proto.speech.tts.ChemicalFormula|undefined} value
 * @return {!proto.speech.tts.Token} returns this
*/
proto.speech.tts.Token.prototype.setChemicalFormula = function(value) {
  return jspb.Message.setWrapperField(this, 39, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.clearChemicalFormula = function() {
  return this.setChemicalFormula(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Token.prototype.hasChemicalFormula = function() {
  return jspb.Message.getField(this, 39) != null;
};


/**
 * optional EmoticonEmoji emoticon_emoji = 40;
 * @return {?proto.speech.tts.EmoticonEmoji}
 */
proto.speech.tts.Token.prototype.getEmoticonEmoji = function() {
  return /** @type{?proto.speech.tts.EmoticonEmoji} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.EmoticonEmoji, 40));
};


/**
 * @param {?proto.speech.tts.EmoticonEmoji|undefined} value
 * @return {!proto.speech.tts.Token} returns this
*/
proto.speech.tts.Token.prototype.setEmoticonEmoji = function(value) {
  return jspb.Message.setWrapperField(this, 40, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.clearEmoticonEmoji = function() {
  return this.setEmoticonEmoji(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Token.prototype.hasEmoticonEmoji = function() {
  return jspb.Message.getField(this, 40) != null;
};


/**
 * optional Syntax syntax = 50;
 * @return {?proto.speech.tts.Syntax}
 */
proto.speech.tts.Token.prototype.getSyntax = function() {
  return /** @type{?proto.speech.tts.Syntax} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Syntax, 50));
};


/**
 * @param {?proto.speech.tts.Syntax|undefined} value
 * @return {!proto.speech.tts.Token} returns this
*/
proto.speech.tts.Token.prototype.setSyntax = function(value) {
  return jspb.Message.setWrapperField(this, 50, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.clearSyntax = function() {
  return this.setSyntax(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Token.prototype.hasSyntax = function() {
  return jspb.Message.getField(this, 50) != null;
};


/**
 * optional VoiceMod voice_mod = 51;
 * @return {?proto.speech.tts.VoiceMod}
 */
proto.speech.tts.Token.prototype.getVoiceMod = function() {
  return /** @type{?proto.speech.tts.VoiceMod} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.VoiceMod, 51));
};


/**
 * @param {?proto.speech.tts.VoiceMod|undefined} value
 * @return {!proto.speech.tts.Token} returns this
*/
proto.speech.tts.Token.prototype.setVoiceMod = function(value) {
  return jspb.Message.setWrapperField(this, 51, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.clearVoiceMod = function() {
  return this.setVoiceMod(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Token.prototype.hasVoiceMod = function() {
  return jspb.Message.getField(this, 51) != null;
};


/**
 * optional string user_features = 52;
 * @return {?string}
 */
proto.speech.tts.Token.prototype.getUserFeatures = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 52));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.setUserFeatures = function(value) {
  return jspb.Message.setField(this, 52, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.clearUserFeatures = function() {
  return jspb.Message.setField(this, 52, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Token.prototype.hasUserFeatures = function() {
  return jspb.Message.getField(this, 52) != null;
};


/**
 * optional bool concise_emoji = 53;
 * @return {?boolean}
 */
proto.speech.tts.Token.prototype.getConciseEmoji = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 53));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.setConciseEmoji = function(value) {
  return jspb.Message.setField(this, 53, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.clearConciseEmoji = function() {
  return jspb.Message.setField(this, 53, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Token.prototype.hasConciseEmoji = function() {
  return jspb.Message.getField(this, 53) != null;
};


/**
 * optional string detected_language = 54;
 * @return {?string}
 */
proto.speech.tts.Token.prototype.getDetectedLanguage = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 54));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.setDetectedLanguage = function(value) {
  return jspb.Message.setField(this, 54, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.clearDetectedLanguage = function() {
  return jspb.Message.setField(this, 54, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Token.prototype.hasDetectedLanguage = function() {
  return jspb.Message.getField(this, 54) != null;
};


/**
 * optional float language_detection_probability = 65;
 * @return {?number}
 */
proto.speech.tts.Token.prototype.getLanguageDetectionProbability = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 65));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.setLanguageDetectionProbability = function(value) {
  return jspb.Message.setField(this, 65, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.clearLanguageDetectionProbability = function() {
  return jspb.Message.setField(this, 65, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Token.prototype.hasLanguageDetectionProbability = function() {
  return jspb.Message.getField(this, 65) != null;
};


/**
 * optional bool bleep = 55;
 * @return {?boolean}
 */
proto.speech.tts.Token.prototype.getBleep = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 55));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.setBleep = function(value) {
  return jspb.Message.setField(this, 55, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.clearBleep = function() {
  return jspb.Message.setField(this, 55, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Token.prototype.hasBleep = function() {
  return jspb.Message.getField(this, 55) != null;
};


/**
 * optional Spelling spell = 56;
 * @return {?proto.speech.tts.Spelling}
 */
proto.speech.tts.Token.prototype.getSpell = function() {
  return /** @type{?proto.speech.tts.Spelling} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Spelling, 56));
};


/**
 * @param {?proto.speech.tts.Spelling|undefined} value
 * @return {!proto.speech.tts.Token} returns this
*/
proto.speech.tts.Token.prototype.setSpell = function(value) {
  return jspb.Message.setWrapperField(this, 56, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.clearSpell = function() {
  return this.setSpell(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Token.prototype.hasSpell = function() {
  return jspb.Message.getField(this, 56) != null;
};


/**
 * optional string prefix = 57;
 * @return {?string}
 */
proto.speech.tts.Token.prototype.getPrefix = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 57));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.setPrefix = function(value) {
  return jspb.Message.setField(this, 57, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.clearPrefix = function() {
  return jspb.Message.setField(this, 57, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Token.prototype.hasPrefix = function() {
  return jspb.Message.getField(this, 57) != null;
};


/**
 * optional bool pitch_accent_phrase_start = 58;
 * @return {?boolean}
 */
proto.speech.tts.Token.prototype.getPitchAccentPhraseStart = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 58));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.setPitchAccentPhraseStart = function(value) {
  return jspb.Message.setField(this, 58, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.clearPitchAccentPhraseStart = function() {
  return jspb.Message.setField(this, 58, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Token.prototype.hasPitchAccentPhraseStart = function() {
  return jspb.Message.getField(this, 58) != null;
};


/**
 * optional string homograph_resolved_via = 59;
 * @return {?string}
 */
proto.speech.tts.Token.prototype.getHomographResolvedVia = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 59));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.setHomographResolvedVia = function(value) {
  return jspb.Message.setField(this, 59, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.clearHomographResolvedVia = function() {
  return jspb.Message.setField(this, 59, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Token.prototype.hasHomographResolvedVia = function() {
  return jspb.Message.getField(this, 59) != null;
};


/**
 * optional string suffix = 60;
 * @return {?string}
 */
proto.speech.tts.Token.prototype.getSuffix = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 60));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.setSuffix = function(value) {
  return jspb.Message.setField(this, 60, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.clearSuffix = function() {
  return jspb.Message.setField(this, 60, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Token.prototype.hasSuffix = function() {
  return jspb.Message.getField(this, 60) != null;
};


/**
 * optional string foreign_script = 61;
 * @return {?string}
 */
proto.speech.tts.Token.prototype.getForeignScript = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 61));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.setForeignScript = function(value) {
  return jspb.Message.setField(this, 61, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.clearForeignScript = function() {
  return jspb.Message.setField(this, 61, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Token.prototype.hasForeignScript = function() {
  return jspb.Message.getField(this, 61) != null;
};


/**
 * optional string verbalization_style = 63;
 * @return {?string}
 */
proto.speech.tts.Token.prototype.getVerbalizationStyle = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 63));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.setVerbalizationStyle = function(value) {
  return jspb.Message.setField(this, 63, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.clearVerbalizationStyle = function() {
  return jspb.Message.setField(this, 63, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Token.prototype.hasVerbalizationStyle = function() {
  return jspb.Message.getField(this, 63) != null;
};


/**
 * optional bool word_has_unknown_graphemes = 64;
 * @return {?boolean}
 */
proto.speech.tts.Token.prototype.getWordHasUnknownGraphemes = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 64));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.setWordHasUnknownGraphemes = function(value) {
  return jspb.Message.setField(this, 64, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Token} returns this
 */
proto.speech.tts.Token.prototype.clearWordHasUnknownGraphemes = function() {
  return jspb.Message.setField(this, 64, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Token.prototype.hasWordHasUnknownGraphemes = function() {
  return jspb.Message.getField(this, 64) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.Token}
 */
proto.speech.tts.Token.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.Token, data);
};



//speech/tts/engine/utterance/features_proto_jspb/floatvector.jspb.js
// source: speech/tts/engine/utterance/features.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.FloatVector');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.FloatVector = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.FloatVector.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.FloatVector, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.FloatVector.displayName = 'proto.speech.tts.FloatVector';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.FloatVector.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.FloatVector.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.FloatVector.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.FloatVector} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.FloatVector.toObject = function(includeInstance, msg) {
  var f, obj = {
    valueList: (f = jspb.Message.getRepeatedFloatingPointField(msg, 1)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of FloatVector as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.FloatVector.ObjectFormat = function() {
  /** @type {?Array<number>|undefined} */
  this.valueList;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.FloatVector.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.FloatVector}
 */
proto.speech.tts.FloatVector.fromObject = function(obj) {
  var msg = new proto.speech.tts.FloatVector();
  obj.valueList != null && jspb.Message.setField(msg, 1, obj.valueList);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.FloatVector}
 */
proto.speech.tts.FloatVector.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.FloatVector;
  return proto.speech.tts.FloatVector.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.FloatVector} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.FloatVector}
 */
proto.speech.tts.FloatVector.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {!Array<number>} */ (reader.readPackedFloat());
      msg.setValueList(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.FloatVector.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.FloatVector.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.FloatVector} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.FloatVector.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getValueList();
  if (f.length > 0) {
    writer.writePackedFloat(
      1,
      f
    );
  }
};


/**
 * repeated float value = 1;
 * @return {!Array<number>}
 */
proto.speech.tts.FloatVector.prototype.getValueList = function() {
  return /** @type {!Array<number>} */ (jspb.Message.getRepeatedFloatingPointField(this, 1));
};


/**
 * @param {Array<number>|undefined} value
 * @return {!proto.speech.tts.FloatVector} returns this
 */
proto.speech.tts.FloatVector.prototype.setValueList = function(value) {
  return jspb.Message.setField(this, 1, value || []);
};


/**
 * @param {number} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.FloatVector} returns this
 */
proto.speech.tts.FloatVector.prototype.addValue = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 1, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.FloatVector} returns this
 */
proto.speech.tts.FloatVector.prototype.clearValueList = function() {
  return this.setValueList([]);
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.FloatVector}
 */
proto.speech.tts.FloatVector.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.FloatVector, data);
};



//speech/tts/engine/utterance/linguistic_structure_proto_jspb/externalwordvectors.jspb.js
// source: speech/tts/engine/utterance/external_word_vectors.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.ExternalWordVectors');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.FloatVector');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.ExternalWordVectors = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.ExternalWordVectors, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.ExternalWordVectors.displayName = 'proto.speech.tts.ExternalWordVectors';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.ExternalWordVectors.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.ExternalWordVectors.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.ExternalWordVectors} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.ExternalWordVectors.toObject = function(includeInstance, msg) {
  var f, obj = {
    brainWordEmbedding: (f = msg.getBrainWordEmbedding()) && proto.speech.tts.FloatVector.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of ExternalWordVectors as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.ExternalWordVectors.ObjectFormat = function() {
  /** @type {?proto.speech.tts.FloatVector.ObjectFormat|undefined} */
  this.brainWordEmbedding;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.ExternalWordVectors.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.ExternalWordVectors}
 */
proto.speech.tts.ExternalWordVectors.fromObject = function(obj) {
  var msg = new proto.speech.tts.ExternalWordVectors();
  obj.brainWordEmbedding && jspb.Message.setWrapperField(
      msg, 1, proto.speech.tts.FloatVector.fromObject(obj.brainWordEmbedding));
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.ExternalWordVectors}
 */
proto.speech.tts.ExternalWordVectors.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.ExternalWordVectors;
  return proto.speech.tts.ExternalWordVectors.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.ExternalWordVectors} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.ExternalWordVectors}
 */
proto.speech.tts.ExternalWordVectors.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.speech.tts.FloatVector;
      reader.readMessage(value,proto.speech.tts.FloatVector.deserializeBinaryFromReader);
      msg.setBrainWordEmbedding(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.ExternalWordVectors.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.ExternalWordVectors.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.ExternalWordVectors} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.ExternalWordVectors.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getBrainWordEmbedding();
  if (f != null) {
    writer.writeMessage(
      1,
      f,
      proto.speech.tts.FloatVector.serializeBinaryToWriter
    );
  }
};


/**
 * optional FloatVector brain_word_embedding = 1;
 * @return {?proto.speech.tts.FloatVector}
 */
proto.speech.tts.ExternalWordVectors.prototype.getBrainWordEmbedding = function() {
  return /** @type{?proto.speech.tts.FloatVector} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.FloatVector, 1));
};


/**
 * @param {?proto.speech.tts.FloatVector|undefined} value
 * @return {!proto.speech.tts.ExternalWordVectors} returns this
*/
proto.speech.tts.ExternalWordVectors.prototype.setBrainWordEmbedding = function(value) {
  return jspb.Message.setWrapperField(this, 1, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.ExternalWordVectors} returns this
 */
proto.speech.tts.ExternalWordVectors.prototype.clearBrainWordEmbedding = function() {
  return this.setBrainWordEmbedding(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.ExternalWordVectors.prototype.hasBrainWordEmbedding = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.ExternalWordVectors}
 */
proto.speech.tts.ExternalWordVectors.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.ExternalWordVectors, data);
};



//speech/tts/engine/utterance/lm_score_proto_jspb/lmscore.jspb.js
// source: speech/tts/engine/utterance/lm_score.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.LmScore');
goog.provide('proto.speech.tts.LmScore.Type');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.LmScore = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.LmScore, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.LmScore.displayName = 'proto.speech.tts.LmScore';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.LmScore.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.LmScore.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.LmScore} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.LmScore.toObject = function(includeInstance, msg) {
  var f, obj = {
    type: jspb.Message.getFieldWithDefault(msg, 1, 1),
    unigram: (f = jspb.Message.getOptionalFloatingPointField(msg, 2)) == null ? undefined : f,
    bigram: (f = jspb.Message.getOptionalFloatingPointField(msg, 3)) == null ? undefined : f,
    trigram: (f = jspb.Message.getOptionalFloatingPointField(msg, 4)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of LmScore as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.LmScore.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.type;

  /** @type {?number|undefined} */
  this.unigram;

  /** @type {?number|undefined} */
  this.bigram;

  /** @type {?number|undefined} */
  this.trigram;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.LmScore.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.LmScore}
 */
proto.speech.tts.LmScore.fromObject = function(obj) {
  var msg = new proto.speech.tts.LmScore();
  obj.type != null && jspb.Message.setField(msg, 1, obj.type);
  obj.unigram != null && jspb.Message.setField(msg, 2, obj.unigram);
  obj.bigram != null && jspb.Message.setField(msg, 3, obj.bigram);
  obj.trigram != null && jspb.Message.setField(msg, 4, obj.trigram);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.LmScore}
 */
proto.speech.tts.LmScore.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.LmScore;
  return proto.speech.tts.LmScore.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.LmScore} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.LmScore}
 */
proto.speech.tts.LmScore.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {!proto.speech.tts.LmScore.Type} */ (reader.readEnum());
      msg.setType(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setUnigram(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setBigram(value);
      break;
    case 4:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setTrigram(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.LmScore.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.LmScore.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.LmScore} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.LmScore.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {!proto.speech.tts.LmScore.Type} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeEnum(
      1,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeFloat(
      2,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeFloat(
      3,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeFloat(
      4,
      f
    );
  }
};


/**
 * @enum {number}
 */
proto.speech.tts.LmScore.Type = {
  PRODLM: 1
};

/**
 * optional Type type = 1;
 * @return {!proto.speech.tts.LmScore.Type}
 */
proto.speech.tts.LmScore.prototype.getType = function() {
  return /** @type {!proto.speech.tts.LmScore.Type} */ (jspb.Message.getFieldWithDefault(this, 1, 1));
};


/**
 * @param {?proto.speech.tts.LmScore.Type|undefined} value
 * @return {!proto.speech.tts.LmScore} returns this
 */
proto.speech.tts.LmScore.prototype.setType = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.LmScore} returns this
 */
proto.speech.tts.LmScore.prototype.clearType = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.LmScore.prototype.hasType = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional float unigram = 2;
 * @return {?number}
 */
proto.speech.tts.LmScore.prototype.getUnigram = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 2));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.LmScore} returns this
 */
proto.speech.tts.LmScore.prototype.setUnigram = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.LmScore} returns this
 */
proto.speech.tts.LmScore.prototype.clearUnigram = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.LmScore.prototype.hasUnigram = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional float bigram = 3;
 * @return {?number}
 */
proto.speech.tts.LmScore.prototype.getBigram = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 3));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.LmScore} returns this
 */
proto.speech.tts.LmScore.prototype.setBigram = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.LmScore} returns this
 */
proto.speech.tts.LmScore.prototype.clearBigram = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.LmScore.prototype.hasBigram = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional float trigram = 4;
 * @return {?number}
 */
proto.speech.tts.LmScore.prototype.getTrigram = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 4));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.LmScore} returns this
 */
proto.speech.tts.LmScore.prototype.setTrigram = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.LmScore} returns this
 */
proto.speech.tts.LmScore.prototype.clearTrigram = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.LmScore.prototype.hasTrigram = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.LmScore}
 */
proto.speech.tts.LmScore.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.LmScore, data);
};



//speech/tts/public/prosody_proto_jspb/prominence.jspb.js
// source: speech/tts/public/prosody.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.Prominence');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

goog.forwardDeclare('proto.speech.tts.ProminenceLevel');
goog.forwardDeclare('proto.speech.tts.ProminenceType');
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.Prominence = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.Prominence, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.Prominence.displayName = 'proto.speech.tts.Prominence';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.Prominence.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.Prominence.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.Prominence} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Prominence.toObject = function(includeInstance, msg) {
  var f, obj = {
    prominenceLevel: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    prominenceType: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Prominence as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.Prominence.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.prominenceLevel;

  /** @type {?number|undefined} */
  this.prominenceType;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.Prominence.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.Prominence}
 */
proto.speech.tts.Prominence.fromObject = function(obj) {
  var msg = new proto.speech.tts.Prominence();
  obj.prominenceLevel != null && jspb.Message.setField(msg, 1, obj.prominenceLevel);
  obj.prominenceType != null && jspb.Message.setField(msg, 2, obj.prominenceType);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.Prominence}
 */
proto.speech.tts.Prominence.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.Prominence;
  return proto.speech.tts.Prominence.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.Prominence} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.Prominence}
 */
proto.speech.tts.Prominence.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {!proto.speech.tts.ProminenceLevel} */ (reader.readEnum());
      msg.setProminenceLevel(value);
      break;
    case 2:
      var value = /** @type {!proto.speech.tts.ProminenceType} */ (reader.readEnum());
      msg.setProminenceType(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.Prominence.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.Prominence.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.Prominence} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Prominence.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {!proto.speech.tts.ProminenceLevel} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeEnum(
      1,
      f
    );
  }
  f = /** @type {!proto.speech.tts.ProminenceType} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeEnum(
      2,
      f
    );
  }
};


/**
 * optional ProminenceLevel prominence_level = 1;
 * @return {!proto.speech.tts.ProminenceLevel}
 */
proto.speech.tts.Prominence.prototype.getProminenceLevel = function() {
  return /** @type {!proto.speech.tts.ProminenceLevel} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?proto.speech.tts.ProminenceLevel|undefined} value
 * @return {!proto.speech.tts.Prominence} returns this
 */
proto.speech.tts.Prominence.prototype.setProminenceLevel = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Prominence} returns this
 */
proto.speech.tts.Prominence.prototype.clearProminenceLevel = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Prominence.prototype.hasProminenceLevel = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional ProminenceType prominence_type = 2;
 * @return {!proto.speech.tts.ProminenceType}
 */
proto.speech.tts.Prominence.prototype.getProminenceType = function() {
  return /** @type {!proto.speech.tts.ProminenceType} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?proto.speech.tts.ProminenceType|undefined} value
 * @return {!proto.speech.tts.Prominence} returns this
 */
proto.speech.tts.Prominence.prototype.setProminenceType = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Prominence} returns this
 */
proto.speech.tts.Prominence.prototype.clearProminenceType = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Prominence.prototype.hasProminenceType = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.Prominence}
 */
proto.speech.tts.Prominence.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.Prominence, data);
};



//speech/tts/engine/utterance/linguistic_structure_proto_jspb/word.jspb.js
// source: speech/tts/engine/utterance/word.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.Word');
goog.provide('proto.speech.tts.Word.EntityInfo');
goog.provide('proto.speech.tts.Word.EntityInfo.Mention');
goog.provide('proto.speech.tts.Word.EntityInfo.Position');
goog.provide('proto.speech.tts.Word.ListInfo');
goog.provide('proto.speech.tts.Word.ListInfo.Position');
goog.provide('proto.speech.tts.Word.PauseOrigin');
goog.provide('proto.speech.tts.Word.PhraseBoundaryMarker');
goog.provide('proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker');
goog.provide('proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker');
goog.provide('proto.speech.tts.Word.ShortPauseAnnotation');
goog.provide('proto.speech.tts.Word.SlotFilling');
goog.provide('proto.speech.tts.Word.SlotFilling.Position');
goog.provide('proto.speech.tts.Word.SlotFilling.SlotFillingType');
goog.provide('proto.speech.tts.Word.Type');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.ExternalWordVectors');
goog.require('proto.speech.tts.LexiconId');
goog.require('proto.speech.tts.Links');
goog.require('proto.speech.tts.LmScore');
goog.require('proto.speech.tts.Prominence');
goog.require('proto.speech.tts.Syntax');
goog.require('proto.speech.tts.VoiceMod');

goog.forwardDeclare('proto.speech.tts.IntonationType');
goog.forwardDeclare('proto.speech.tts.LexiconProto.Entry.Syntax');
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.Word = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 67, null, null);
};
goog.inherits(proto.speech.tts.Word, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.Word.displayName = 'proto.speech.tts.Word';
}

/**
 * The extensions registered with this message class. This is a map of
 * extension field number to fieldInfo object.
 *
 * For example:
 *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }
 *
 * fieldName contains the JsCompiler renamed field name property so that it
 * works in OPTIMIZED mode.
 *
 * @type {!Object<number, jspb.ExtensionFieldInfo>}
 */
proto.speech.tts.Word.extensions = {};


/**
 * The extensions registered with this message class. This is a map of
 * extension field number to fieldInfo object.
 *
 * For example:
 *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }
 *
 * fieldName contains the JsCompiler renamed field name property so that it
 * works in OPTIMIZED mode.
 *
 * @type {!Object<number, jspb.ExtensionFieldBinaryInfo>}
 */
proto.speech.tts.Word.extensionsBinary = {};

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.Word.PhraseBoundaryMarker = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.Word.PhraseBoundaryMarker, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.Word.PhraseBoundaryMarker.displayName = 'proto.speech.tts.Word.PhraseBoundaryMarker';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker.displayName = 'proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker.displayName = 'proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.Word.SlotFilling = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.Word.SlotFilling, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.Word.SlotFilling.displayName = 'proto.speech.tts.Word.SlotFilling';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.Word.EntityInfo = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.Word.EntityInfo, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.Word.EntityInfo.displayName = 'proto.speech.tts.Word.EntityInfo';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.Word.ListInfo = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.Word.ListInfo, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.Word.ListInfo.displayName = 'proto.speech.tts.Word.ListInfo';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.Word.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.Word.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.Word} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Word.toObject = function(includeInstance, msg) {
  var f, obj = {
    links: (f = msg.getLinks()) && proto.speech.tts.Links.toObject(includeInstance, f),
    type: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    lexiconId: (f = msg.getLexiconId()) && proto.speech.tts.LexiconId.toObject(includeInstance, f),
    id: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,
    variant: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f,
    variantForced: jspb.Message.getBooleanFieldWithDefault(msg, 5, false),
    spelling: (f = jspb.Message.getField(msg, 7)) == null ? undefined : f,
    lexicalSyntax: (f = jspb.Message.getField(msg, 8)) == null ? undefined : f,
    syntax: (f = msg.getSyntax()) && proto.speech.tts.Syntax.toObject(includeInstance, f),
    pauseLength: (f = jspb.Message.getOptionalFloatingPointField(msg, 10)) == null ? undefined : f,
    intonationType: (f = jspb.Message.getField(msg, 11)) == null ? undefined : f,
    precedesPause: (f = jspb.Message.getBooleanField(msg, 12)) == null ? undefined : f,
    spellingWithStress: (f = jspb.Message.getField(msg, 14)) == null ? undefined : f,
    prominence: (f = msg.getProminence()) && proto.speech.tts.Prominence.toObject(includeInstance, f),
    lmScore: (f = msg.getLmScore()) && proto.speech.tts.LmScore.toObject(includeInstance, f),
    pitchAccent: (f = jspb.Message.getField(msg, 17)) == null ? undefined : f,
    pitchAccentPhraseStart: (f = jspb.Message.getBooleanField(msg, 18)) == null ? undefined : f,
    externalWordVectors: (f = msg.getExternalWordVectors()) && proto.speech.tts.ExternalWordVectors.toObject(includeInstance, f),
    userFeatures: (f = jspb.Message.getField(msg, 20)) == null ? undefined : f,
    shortPauseStatus: (f = jspb.Message.getField(msg, 21)) == null ? undefined : f,
    ratedProminence: (f = jspb.Message.getOptionalFloatingPointField(msg, 22)) == null ? undefined : f,
    predictedProminence: (f = jspb.Message.getOptionalFloatingPointField(msg, 61)) == null ? undefined : f,
    phraseBoundary: (f = msg.getPhraseBoundary()) && proto.speech.tts.Word.PhraseBoundaryMarker.toObject(includeInstance, f),
    slotFilling: (f = msg.getSlotFilling()) && proto.speech.tts.Word.SlotFilling.toObject(includeInstance, f),
    voiceMod: (f = msg.getVoiceMod()) && proto.speech.tts.VoiceMod.toObject(includeInstance, f),
    detectedLanguage: (f = jspb.Message.getField(msg, 51)) == null ? undefined : f,
    bleep: (f = jspb.Message.getBooleanField(msg, 55)) == null ? undefined : f,
    skip: (f = jspb.Message.getBooleanField(msg, 56)) == null ? undefined : f,
    entityInfo: (f = msg.getEntityInfo()) && proto.speech.tts.Word.EntityInfo.toObject(includeInstance, f),
    isQuoted: (f = jspb.Message.getBooleanField(msg, 58)) == null ? undefined : f,
    homographResolvedVia: (f = jspb.Message.getField(msg, 59)) == null ? undefined : f,
    listInfo: (f = msg.getListInfo()) && proto.speech.tts.Word.ListInfo.toObject(includeInstance, f),
    isGenerated: (f = jspb.Message.getBooleanField(msg, 62)) == null ? undefined : f,
    sourcePronunciation: (f = jspb.Message.getField(msg, 64)) == null ? undefined : f,
    pauseOrigin: (f = jspb.Message.getField(msg, 65)) == null ? undefined : f,
    relativeOutputGain: jspb.Message.getFloatingPointFieldWithDefault(msg, 66, 1.0)
  };

  jspb.Message.toObjectExtension(/** @type {!jspb.Message} */ (msg), obj,
      proto.speech.tts.Word.extensions, proto.speech.tts.Word.prototype.getExtension,
      includeInstance);
  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Word as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.Word.ObjectFormat = function() {
  /** @type {?proto.speech.tts.Links.ObjectFormat|undefined} */
  this.links;

  /** @type {?number|undefined} */
  this.type;

  /** @type {?proto.speech.tts.LexiconId.ObjectFormat|undefined} */
  this.lexiconId;

  /** @type {?string|undefined} */
  this.id;

  /** @type {?string|undefined} */
  this.variant;

  /** @type {?boolean|undefined} */
  this.variantForced;

  /** @type {?string|undefined} */
  this.spelling;

  /** @type {?number|undefined} */
  this.lexicalSyntax;

  /** @type {?proto.speech.tts.Syntax.ObjectFormat|undefined} */
  this.syntax;

  /** @type {?number|undefined} */
  this.pauseLength;

  /** @type {?number|undefined} */
  this.intonationType;

  /** @type {?boolean|undefined} */
  this.precedesPause;

  /** @type {?string|undefined} */
  this.spellingWithStress;

  /** @type {?proto.speech.tts.Prominence.ObjectFormat|undefined} */
  this.prominence;

  /** @type {?proto.speech.tts.LmScore.ObjectFormat|undefined} */
  this.lmScore;

  /** @type {?number|undefined} */
  this.pitchAccent;

  /** @type {?boolean|undefined} */
  this.pitchAccentPhraseStart;

  /** @type {?proto.speech.tts.ExternalWordVectors.ObjectFormat|undefined} */
  this.externalWordVectors;

  /** @type {?string|undefined} */
  this.userFeatures;

  /** @type {?number|undefined} */
  this.shortPauseStatus;

  /** @type {?number|undefined} */
  this.ratedProminence;

  /** @type {?number|undefined} */
  this.predictedProminence;

  /** @type {?proto.speech.tts.Word.PhraseBoundaryMarker.ObjectFormat|undefined} */
  this.phraseBoundary;

  /** @type {?proto.speech.tts.Word.SlotFilling.ObjectFormat|undefined} */
  this.slotFilling;

  /** @type {?proto.speech.tts.VoiceMod.ObjectFormat|undefined} */
  this.voiceMod;

  /** @type {?string|undefined} */
  this.detectedLanguage;

  /** @type {?boolean|undefined} */
  this.bleep;

  /** @type {?boolean|undefined} */
  this.skip;

  /** @type {?proto.speech.tts.Word.EntityInfo.ObjectFormat|undefined} */
  this.entityInfo;

  /** @type {?boolean|undefined} */
  this.isQuoted;

  /** @type {?string|undefined} */
  this.homographResolvedVia;

  /** @type {?proto.speech.tts.Word.ListInfo.ObjectFormat|undefined} */
  this.listInfo;

  /** @type {?boolean|undefined} */
  this.isGenerated;

  /** @type {?string|undefined} */
  this.sourcePronunciation;

  /** @type {?number|undefined} */
  this.pauseOrigin;

  /** @type {?number|undefined} */
  this.relativeOutputGain;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.Word.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.Word}
 */
proto.speech.tts.Word.fromObject = function(obj) {
  var msg = new proto.speech.tts.Word();
  obj.links && jspb.Message.setWrapperField(
      msg, 1, proto.speech.tts.Links.fromObject(obj.links));
  obj.type != null && jspb.Message.setField(msg, 2, obj.type);
  obj.lexiconId && jspb.Message.setWrapperField(
      msg, 63, proto.speech.tts.LexiconId.fromObject(obj.lexiconId));
  obj.id != null && jspb.Message.setField(msg, 3, obj.id);
  obj.variant != null && jspb.Message.setField(msg, 4, obj.variant);
  obj.variantForced != null && jspb.Message.setField(msg, 5, obj.variantForced);
  obj.spelling != null && jspb.Message.setField(msg, 7, obj.spelling);
  obj.lexicalSyntax != null && jspb.Message.setField(msg, 8, obj.lexicalSyntax);
  obj.syntax && jspb.Message.setWrapperField(
      msg, 9, proto.speech.tts.Syntax.fromObject(obj.syntax));
  obj.pauseLength != null && jspb.Message.setField(msg, 10, obj.pauseLength);
  obj.intonationType != null && jspb.Message.setField(msg, 11, obj.intonationType);
  obj.precedesPause != null && jspb.Message.setField(msg, 12, obj.precedesPause);
  obj.spellingWithStress != null && jspb.Message.setField(msg, 14, obj.spellingWithStress);
  obj.prominence && jspb.Message.setWrapperField(
      msg, 15, proto.speech.tts.Prominence.fromObject(obj.prominence));
  obj.lmScore && jspb.Message.setWrapperField(
      msg, 16, proto.speech.tts.LmScore.fromObject(obj.lmScore));
  obj.pitchAccent != null && jspb.Message.setField(msg, 17, obj.pitchAccent);
  obj.pitchAccentPhraseStart != null && jspb.Message.setField(msg, 18, obj.pitchAccentPhraseStart);
  obj.externalWordVectors && jspb.Message.setWrapperField(
      msg, 19, proto.speech.tts.ExternalWordVectors.fromObject(obj.externalWordVectors));
  obj.userFeatures != null && jspb.Message.setField(msg, 20, obj.userFeatures);
  obj.shortPauseStatus != null && jspb.Message.setField(msg, 21, obj.shortPauseStatus);
  obj.ratedProminence != null && jspb.Message.setField(msg, 22, obj.ratedProminence);
  obj.predictedProminence != null && jspb.Message.setField(msg, 61, obj.predictedProminence);
  obj.phraseBoundary && jspb.Message.setWrapperField(
      msg, 23, proto.speech.tts.Word.PhraseBoundaryMarker.fromObject(obj.phraseBoundary));
  obj.slotFilling && jspb.Message.setWrapperField(
      msg, 24, proto.speech.tts.Word.SlotFilling.fromObject(obj.slotFilling));
  obj.voiceMod && jspb.Message.setWrapperField(
      msg, 50, proto.speech.tts.VoiceMod.fromObject(obj.voiceMod));
  obj.detectedLanguage != null && jspb.Message.setField(msg, 51, obj.detectedLanguage);
  obj.bleep != null && jspb.Message.setField(msg, 55, obj.bleep);
  obj.skip != null && jspb.Message.setField(msg, 56, obj.skip);
  obj.entityInfo && jspb.Message.setWrapperField(
      msg, 57, proto.speech.tts.Word.EntityInfo.fromObject(obj.entityInfo));
  obj.isQuoted != null && jspb.Message.setField(msg, 58, obj.isQuoted);
  obj.homographResolvedVia != null && jspb.Message.setField(msg, 59, obj.homographResolvedVia);
  obj.listInfo && jspb.Message.setWrapperField(
      msg, 60, proto.speech.tts.Word.ListInfo.fromObject(obj.listInfo));
  obj.isGenerated != null && jspb.Message.setField(msg, 62, obj.isGenerated);
  obj.sourcePronunciation != null && jspb.Message.setField(msg, 64, obj.sourcePronunciation);
  obj.pauseOrigin != null && jspb.Message.setField(msg, 65, obj.pauseOrigin);
  obj.relativeOutputGain != null && jspb.Message.setField(msg, 66, obj.relativeOutputGain);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.Word}
 */
proto.speech.tts.Word.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.Word;
  return proto.speech.tts.Word.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.Word} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.Word}
 */
proto.speech.tts.Word.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.speech.tts.Links;
      reader.readMessage(value,proto.speech.tts.Links.deserializeBinaryFromReader);
      msg.setLinks(value);
      break;
    case 2:
      var value = /** @type {!proto.speech.tts.Word.Type} */ (reader.readEnum());
      msg.setType(value);
      break;
    case 63:
      var value = new proto.speech.tts.LexiconId;
      reader.readMessage(value,proto.speech.tts.LexiconId.deserializeBinaryFromReader);
      msg.setLexiconId(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setId(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setVariant(value);
      break;
    case 5:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setVariantForced(value);
      break;
    case 7:
      var value = /** @type {string} */ (reader.readString());
      msg.setSpelling(value);
      break;
    case 8:
      var value = /** @type {!proto.speech.tts.LexiconProto.Entry.Syntax} */ (reader.readEnum());
      msg.setLexicalSyntax(value);
      break;
    case 9:
      var value = new proto.speech.tts.Syntax;
      reader.readMessage(value,proto.speech.tts.Syntax.deserializeBinaryFromReader);
      msg.setSyntax(value);
      break;
    case 10:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setPauseLength(value);
      break;
    case 11:
      var value = /** @type {!proto.speech.tts.IntonationType} */ (reader.readEnum());
      msg.setIntonationType(value);
      break;
    case 12:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setPrecedesPause(value);
      break;
    case 14:
      var value = /** @type {string} */ (reader.readString());
      msg.setSpellingWithStress(value);
      break;
    case 15:
      var value = new proto.speech.tts.Prominence;
      reader.readMessage(value,proto.speech.tts.Prominence.deserializeBinaryFromReader);
      msg.setProminence(value);
      break;
    case 16:
      var value = new proto.speech.tts.LmScore;
      reader.readMessage(value,proto.speech.tts.LmScore.deserializeBinaryFromReader);
      msg.setLmScore(value);
      break;
    case 17:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setPitchAccent(value);
      break;
    case 18:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setPitchAccentPhraseStart(value);
      break;
    case 19:
      var value = new proto.speech.tts.ExternalWordVectors;
      reader.readMessage(value,proto.speech.tts.ExternalWordVectors.deserializeBinaryFromReader);
      msg.setExternalWordVectors(value);
      break;
    case 20:
      var value = /** @type {string} */ (reader.readString());
      msg.setUserFeatures(value);
      break;
    case 21:
      var value = /** @type {!proto.speech.tts.Word.ShortPauseAnnotation} */ (reader.readEnum());
      msg.setShortPauseStatus(value);
      break;
    case 22:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setRatedProminence(value);
      break;
    case 61:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setPredictedProminence(value);
      break;
    case 23:
      var value = new proto.speech.tts.Word.PhraseBoundaryMarker;
      reader.readMessage(value,proto.speech.tts.Word.PhraseBoundaryMarker.deserializeBinaryFromReader);
      msg.setPhraseBoundary(value);
      break;
    case 24:
      var value = new proto.speech.tts.Word.SlotFilling;
      reader.readMessage(value,proto.speech.tts.Word.SlotFilling.deserializeBinaryFromReader);
      msg.setSlotFilling(value);
      break;
    case 50:
      var value = new proto.speech.tts.VoiceMod;
      reader.readMessage(value,proto.speech.tts.VoiceMod.deserializeBinaryFromReader);
      msg.setVoiceMod(value);
      break;
    case 51:
      var value = /** @type {string} */ (reader.readString());
      msg.setDetectedLanguage(value);
      break;
    case 55:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setBleep(value);
      break;
    case 56:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setSkip(value);
      break;
    case 57:
      var value = new proto.speech.tts.Word.EntityInfo;
      reader.readMessage(value,proto.speech.tts.Word.EntityInfo.deserializeBinaryFromReader);
      msg.setEntityInfo(value);
      break;
    case 58:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsQuoted(value);
      break;
    case 59:
      var value = /** @type {string} */ (reader.readString());
      msg.setHomographResolvedVia(value);
      break;
    case 60:
      var value = new proto.speech.tts.Word.ListInfo;
      reader.readMessage(value,proto.speech.tts.Word.ListInfo.deserializeBinaryFromReader);
      msg.setListInfo(value);
      break;
    case 62:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsGenerated(value);
      break;
    case 64:
      var value = /** @type {string} */ (reader.readString());
      msg.setSourcePronunciation(value);
      break;
    case 65:
      var value = /** @type {!proto.speech.tts.Word.PauseOrigin} */ (reader.readEnum());
      msg.setPauseOrigin(value);
      break;
    case 66:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setRelativeOutputGain(value);
      break;
    default:
      jspb.Message.readBinaryExtension(msg, reader,
        proto.speech.tts.Word.extensionsBinary,
        proto.speech.tts.Word.prototype.getExtension,
        proto.speech.tts.Word.prototype.setExtension);
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.Word.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.Word.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.Word} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Word.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getLinks();
  if (f != null) {
    writer.writeMessage(
      1,
      f,
      proto.speech.tts.Links.serializeBinaryToWriter
    );
  }
  f = /** @type {!proto.speech.tts.Word.Type} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeEnum(
      2,
      f
    );
  }
  f = message.getLexiconId();
  if (f != null) {
    writer.writeMessage(
      63,
      f,
      proto.speech.tts.LexiconId.serializeBinaryToWriter
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeString(
      3,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeString(
      4,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeBool(
      5,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 7));
  if (f != null) {
    writer.writeString(
      7,
      f
    );
  }
  f = /** @type {!proto.speech.tts.LexiconProto.Entry.Syntax} */ (jspb.Message.getField(message, 8));
  if (f != null) {
    writer.writeEnum(
      8,
      f
    );
  }
  f = message.getSyntax();
  if (f != null) {
    writer.writeMessage(
      9,
      f,
      proto.speech.tts.Syntax.serializeBinaryToWriter
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 10));
  if (f != null) {
    writer.writeFloat(
      10,
      f
    );
  }
  f = /** @type {!proto.speech.tts.IntonationType} */ (jspb.Message.getField(message, 11));
  if (f != null) {
    writer.writeEnum(
      11,
      f
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 12));
  if (f != null) {
    writer.writeBool(
      12,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 14));
  if (f != null) {
    writer.writeString(
      14,
      f
    );
  }
  f = message.getProminence();
  if (f != null) {
    writer.writeMessage(
      15,
      f,
      proto.speech.tts.Prominence.serializeBinaryToWriter
    );
  }
  f = message.getLmScore();
  if (f != null) {
    writer.writeMessage(
      16,
      f,
      proto.speech.tts.LmScore.serializeBinaryToWriter
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 17));
  if (f != null) {
    writer.writeInt32(
      17,
      f
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 18));
  if (f != null) {
    writer.writeBool(
      18,
      f
    );
  }
  f = message.getExternalWordVectors();
  if (f != null) {
    writer.writeMessage(
      19,
      f,
      proto.speech.tts.ExternalWordVectors.serializeBinaryToWriter
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 20));
  if (f != null) {
    writer.writeString(
      20,
      f
    );
  }
  f = /** @type {!proto.speech.tts.Word.ShortPauseAnnotation} */ (jspb.Message.getField(message, 21));
  if (f != null) {
    writer.writeEnum(
      21,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 22));
  if (f != null) {
    writer.writeFloat(
      22,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 61));
  if (f != null) {
    writer.writeFloat(
      61,
      f
    );
  }
  f = message.getPhraseBoundary();
  if (f != null) {
    writer.writeMessage(
      23,
      f,
      proto.speech.tts.Word.PhraseBoundaryMarker.serializeBinaryToWriter
    );
  }
  f = message.getSlotFilling();
  if (f != null) {
    writer.writeMessage(
      24,
      f,
      proto.speech.tts.Word.SlotFilling.serializeBinaryToWriter
    );
  }
  f = message.getVoiceMod();
  if (f != null) {
    writer.writeMessage(
      50,
      f,
      proto.speech.tts.VoiceMod.serializeBinaryToWriter
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 51));
  if (f != null) {
    writer.writeString(
      51,
      f
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 55));
  if (f != null) {
    writer.writeBool(
      55,
      f
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 56));
  if (f != null) {
    writer.writeBool(
      56,
      f
    );
  }
  f = message.getEntityInfo();
  if (f != null) {
    writer.writeMessage(
      57,
      f,
      proto.speech.tts.Word.EntityInfo.serializeBinaryToWriter
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 58));
  if (f != null) {
    writer.writeBool(
      58,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 59));
  if (f != null) {
    writer.writeString(
      59,
      f
    );
  }
  f = message.getListInfo();
  if (f != null) {
    writer.writeMessage(
      60,
      f,
      proto.speech.tts.Word.ListInfo.serializeBinaryToWriter
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 62));
  if (f != null) {
    writer.writeBool(
      62,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 64));
  if (f != null) {
    writer.writeString(
      64,
      f
    );
  }
  f = /** @type {!proto.speech.tts.Word.PauseOrigin} */ (jspb.Message.getField(message, 65));
  if (f != null) {
    writer.writeEnum(
      65,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 66));
  if (f != null) {
    writer.writeFloat(
      66,
      f
    );
  }
  jspb.Message.serializeBinaryExtensions(message, writer,
    proto.speech.tts.Word.extensionsBinary, proto.speech.tts.Word.prototype.getExtension);
};


/**
 * @enum {number}
 */
proto.speech.tts.Word.Type = {
  WORD: 1,
  HOMOGRAPH: 3,
  EXL: 4
};

/**
 * @enum {number}
 */
proto.speech.tts.Word.ShortPauseAnnotation = {
  MAYBE_SHORT_PAUSE: 0,
  FORCE_SHORT_PAUSE: 1,
  FORCE_NO_SHORT_PAUSE: 2
};

/**
 * @enum {number}
 */
proto.speech.tts.Word.PauseOrigin = {
  PAUSE_ORIGIN_UNSPECIFIED: 0,
  CLIENT_PROVIDED: 1,
  DEFAULT: 2
};




if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.Word.PhraseBoundaryMarker.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.Word.PhraseBoundaryMarker.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.Word.PhraseBoundaryMarker} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Word.PhraseBoundaryMarker.toObject = function(includeInstance, msg) {
  var f, obj = {
    textMarker: (f = msg.getTextMarker()) && proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker.toObject(includeInstance, f),
    waveMarker: (f = msg.getWaveMarker()) && proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of PhraseBoundaryMarker as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.Word.PhraseBoundaryMarker.ObjectFormat = function() {
  /** @type {?proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker.ObjectFormat|undefined} */
  this.textMarker;

  /** @type {?proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker.ObjectFormat|undefined} */
  this.waveMarker;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.Word.PhraseBoundaryMarker.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.Word.PhraseBoundaryMarker}
 */
proto.speech.tts.Word.PhraseBoundaryMarker.fromObject = function(obj) {
  var msg = new proto.speech.tts.Word.PhraseBoundaryMarker();
  obj.textMarker && jspb.Message.setWrapperField(
      msg, 1, proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker.fromObject(obj.textMarker));
  obj.waveMarker && jspb.Message.setWrapperField(
      msg, 2, proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker.fromObject(obj.waveMarker));
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.Word.PhraseBoundaryMarker}
 */
proto.speech.tts.Word.PhraseBoundaryMarker.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.Word.PhraseBoundaryMarker;
  return proto.speech.tts.Word.PhraseBoundaryMarker.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.Word.PhraseBoundaryMarker} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.Word.PhraseBoundaryMarker}
 */
proto.speech.tts.Word.PhraseBoundaryMarker.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker;
      reader.readMessage(value,proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker.deserializeBinaryFromReader);
      msg.setTextMarker(value);
      break;
    case 2:
      var value = new proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker;
      reader.readMessage(value,proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker.deserializeBinaryFromReader);
      msg.setWaveMarker(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.Word.PhraseBoundaryMarker.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.Word.PhraseBoundaryMarker.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.Word.PhraseBoundaryMarker} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Word.PhraseBoundaryMarker.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getTextMarker();
  if (f != null) {
    writer.writeMessage(
      1,
      f,
      proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker.serializeBinaryToWriter
    );
  }
  f = message.getWaveMarker();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker.serializeBinaryToWriter
    );
  }
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker.toObject = function(includeInstance, msg) {
  var f, obj = {
    requestedLevel: (f = jspb.Message.getOptionalFloatingPointField(msg, 1)) == null ? undefined : f,
    ewokRatedLevel: (f = jspb.Message.getOptionalFloatingPointField(msg, 2)) == null ? undefined : f,
    predictedLevel: (f = jspb.Message.getOptionalFloatingPointField(msg, 3)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of TextMarker as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.requestedLevel;

  /** @type {?number|undefined} */
  this.ewokRatedLevel;

  /** @type {?number|undefined} */
  this.predictedLevel;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker}
 */
proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker.fromObject = function(obj) {
  var msg = new proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker();
  obj.requestedLevel != null && jspb.Message.setField(msg, 1, obj.requestedLevel);
  obj.ewokRatedLevel != null && jspb.Message.setField(msg, 2, obj.ewokRatedLevel);
  obj.predictedLevel != null && jspb.Message.setField(msg, 3, obj.predictedLevel);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker}
 */
proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker;
  return proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker}
 */
proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setRequestedLevel(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setEwokRatedLevel(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setPredictedLevel(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?number} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeFloat(
      1,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeFloat(
      2,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeFloat(
      3,
      f
    );
  }
};


/**
 * optional float requested_level = 1;
 * @return {?number}
 */
proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker.prototype.getRequestedLevel = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 1));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker} returns this
 */
proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker.prototype.setRequestedLevel = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker} returns this
 */
proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker.prototype.clearRequestedLevel = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker.prototype.hasRequestedLevel = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional float ewok_rated_level = 2;
 * @return {?number}
 */
proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker.prototype.getEwokRatedLevel = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 2));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker} returns this
 */
proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker.prototype.setEwokRatedLevel = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker} returns this
 */
proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker.prototype.clearEwokRatedLevel = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker.prototype.hasEwokRatedLevel = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional float predicted_level = 3;
 * @return {?number}
 */
proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker.prototype.getPredictedLevel = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 3));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker} returns this
 */
proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker.prototype.setPredictedLevel = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker} returns this
 */
proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker.prototype.clearPredictedLevel = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker.prototype.hasPredictedLevel = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker}
 */
proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker, data);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker.toObject = function(includeInstance, msg) {
  var f, obj = {
    forcedRatedLevel: (f = jspb.Message.getOptionalFloatingPointField(msg, 1)) == null ? undefined : f,
    ewokRatedLevel: (f = jspb.Message.getOptionalFloatingPointField(msg, 2)) == null ? undefined : f,
    detectedLevel: (f = jspb.Message.getOptionalFloatingPointField(msg, 3)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of WaveMarker as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.forcedRatedLevel;

  /** @type {?number|undefined} */
  this.ewokRatedLevel;

  /** @type {?number|undefined} */
  this.detectedLevel;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker}
 */
proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker.fromObject = function(obj) {
  var msg = new proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker();
  obj.forcedRatedLevel != null && jspb.Message.setField(msg, 1, obj.forcedRatedLevel);
  obj.ewokRatedLevel != null && jspb.Message.setField(msg, 2, obj.ewokRatedLevel);
  obj.detectedLevel != null && jspb.Message.setField(msg, 3, obj.detectedLevel);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker}
 */
proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker;
  return proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker}
 */
proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setForcedRatedLevel(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setEwokRatedLevel(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setDetectedLevel(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?number} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeFloat(
      1,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeFloat(
      2,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeFloat(
      3,
      f
    );
  }
};


/**
 * optional float forced_rated_level = 1;
 * @return {?number}
 */
proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker.prototype.getForcedRatedLevel = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 1));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker} returns this
 */
proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker.prototype.setForcedRatedLevel = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker} returns this
 */
proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker.prototype.clearForcedRatedLevel = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker.prototype.hasForcedRatedLevel = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional float ewok_rated_level = 2;
 * @return {?number}
 */
proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker.prototype.getEwokRatedLevel = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 2));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker} returns this
 */
proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker.prototype.setEwokRatedLevel = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker} returns this
 */
proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker.prototype.clearEwokRatedLevel = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker.prototype.hasEwokRatedLevel = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional float detected_level = 3;
 * @return {?number}
 */
proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker.prototype.getDetectedLevel = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 3));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker} returns this
 */
proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker.prototype.setDetectedLevel = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker} returns this
 */
proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker.prototype.clearDetectedLevel = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker.prototype.hasDetectedLevel = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker}
 */
proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker, data);
};


/**
 * optional TextMarker text_marker = 1;
 * @return {?proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker}
 */
proto.speech.tts.Word.PhraseBoundaryMarker.prototype.getTextMarker = function() {
  return /** @type{?proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker, 1));
};


/**
 * @param {?proto.speech.tts.Word.PhraseBoundaryMarker.TextMarker|undefined} value
 * @return {!proto.speech.tts.Word.PhraseBoundaryMarker} returns this
*/
proto.speech.tts.Word.PhraseBoundaryMarker.prototype.setTextMarker = function(value) {
  return jspb.Message.setWrapperField(this, 1, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Word.PhraseBoundaryMarker} returns this
 */
proto.speech.tts.Word.PhraseBoundaryMarker.prototype.clearTextMarker = function() {
  return this.setTextMarker(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Word.PhraseBoundaryMarker.prototype.hasTextMarker = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional WaveMarker wave_marker = 2;
 * @return {?proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker}
 */
proto.speech.tts.Word.PhraseBoundaryMarker.prototype.getWaveMarker = function() {
  return /** @type{?proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker, 2));
};


/**
 * @param {?proto.speech.tts.Word.PhraseBoundaryMarker.WaveMarker|undefined} value
 * @return {!proto.speech.tts.Word.PhraseBoundaryMarker} returns this
*/
proto.speech.tts.Word.PhraseBoundaryMarker.prototype.setWaveMarker = function(value) {
  return jspb.Message.setWrapperField(this, 2, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Word.PhraseBoundaryMarker} returns this
 */
proto.speech.tts.Word.PhraseBoundaryMarker.prototype.clearWaveMarker = function() {
  return this.setWaveMarker(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Word.PhraseBoundaryMarker.prototype.hasWaveMarker = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.Word.PhraseBoundaryMarker}
 */
proto.speech.tts.Word.PhraseBoundaryMarker.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.Word.PhraseBoundaryMarker, data);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.Word.SlotFilling.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.Word.SlotFilling.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.Word.SlotFilling} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Word.SlotFilling.toObject = function(includeInstance, msg) {
  var f, obj = {
    type: jspb.Message.getFieldWithDefault(msg, 1, 0),
    spellingGroupPosition: jspb.Message.getFieldWithDefault(msg, 2, 0),
    spellingLetterPosition: jspb.Message.getFieldWithDefault(msg, 3, 0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of SlotFilling as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.Word.SlotFilling.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.type;

  /** @type {?number|undefined} */
  this.spellingGroupPosition;

  /** @type {?number|undefined} */
  this.spellingLetterPosition;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.Word.SlotFilling.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.Word.SlotFilling}
 */
proto.speech.tts.Word.SlotFilling.fromObject = function(obj) {
  var msg = new proto.speech.tts.Word.SlotFilling();
  obj.type != null && jspb.Message.setField(msg, 1, obj.type);
  obj.spellingGroupPosition != null && jspb.Message.setField(msg, 2, obj.spellingGroupPosition);
  obj.spellingLetterPosition != null && jspb.Message.setField(msg, 3, obj.spellingLetterPosition);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.Word.SlotFilling}
 */
proto.speech.tts.Word.SlotFilling.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.Word.SlotFilling;
  return proto.speech.tts.Word.SlotFilling.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.Word.SlotFilling} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.Word.SlotFilling}
 */
proto.speech.tts.Word.SlotFilling.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {!proto.speech.tts.Word.SlotFilling.SlotFillingType} */ (reader.readEnum());
      msg.setType(value);
      break;
    case 2:
      var value = /** @type {!proto.speech.tts.Word.SlotFilling.Position} */ (reader.readEnum());
      msg.setSpellingGroupPosition(value);
      break;
    case 3:
      var value = /** @type {!proto.speech.tts.Word.SlotFilling.Position} */ (reader.readEnum());
      msg.setSpellingLetterPosition(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.Word.SlotFilling.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.Word.SlotFilling.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.Word.SlotFilling} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Word.SlotFilling.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {!proto.speech.tts.Word.SlotFilling.SlotFillingType} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeEnum(
      1,
      f
    );
  }
  f = /** @type {!proto.speech.tts.Word.SlotFilling.Position} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeEnum(
      2,
      f
    );
  }
  f = /** @type {!proto.speech.tts.Word.SlotFilling.Position} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeEnum(
      3,
      f
    );
  }
};


/**
 * @enum {number}
 */
proto.speech.tts.Word.SlotFilling.SlotFillingType = {
  NO_PLAYBACK: 0,
  SPELLING: 1,
  NON_PHONOTACTIC: 2
};

/**
 * @enum {number}
 */
proto.speech.tts.Word.SlotFilling.Position = {
  NONE: 0,
  START: 1,
  MIDDLE: 2,
  END: 3
};

/**
 * optional SlotFillingType type = 1;
 * @return {!proto.speech.tts.Word.SlotFilling.SlotFillingType}
 */
proto.speech.tts.Word.SlotFilling.prototype.getType = function() {
  return /** @type {!proto.speech.tts.Word.SlotFilling.SlotFillingType} */ (jspb.Message.getFieldWithDefault(this, 1, 0));
};


/**
 * @param {?proto.speech.tts.Word.SlotFilling.SlotFillingType|undefined} value
 * @return {!proto.speech.tts.Word.SlotFilling} returns this
 */
proto.speech.tts.Word.SlotFilling.prototype.setType = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Word.SlotFilling} returns this
 */
proto.speech.tts.Word.SlotFilling.prototype.clearType = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Word.SlotFilling.prototype.hasType = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional Position spelling_group_position = 2;
 * @return {!proto.speech.tts.Word.SlotFilling.Position}
 */
proto.speech.tts.Word.SlotFilling.prototype.getSpellingGroupPosition = function() {
  return /** @type {!proto.speech.tts.Word.SlotFilling.Position} */ (jspb.Message.getFieldWithDefault(this, 2, 0));
};


/**
 * @param {?proto.speech.tts.Word.SlotFilling.Position|undefined} value
 * @return {!proto.speech.tts.Word.SlotFilling} returns this
 */
proto.speech.tts.Word.SlotFilling.prototype.setSpellingGroupPosition = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Word.SlotFilling} returns this
 */
proto.speech.tts.Word.SlotFilling.prototype.clearSpellingGroupPosition = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Word.SlotFilling.prototype.hasSpellingGroupPosition = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional Position spelling_letter_position = 3;
 * @return {!proto.speech.tts.Word.SlotFilling.Position}
 */
proto.speech.tts.Word.SlotFilling.prototype.getSpellingLetterPosition = function() {
  return /** @type {!proto.speech.tts.Word.SlotFilling.Position} */ (jspb.Message.getFieldWithDefault(this, 3, 0));
};


/**
 * @param {?proto.speech.tts.Word.SlotFilling.Position|undefined} value
 * @return {!proto.speech.tts.Word.SlotFilling} returns this
 */
proto.speech.tts.Word.SlotFilling.prototype.setSpellingLetterPosition = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Word.SlotFilling} returns this
 */
proto.speech.tts.Word.SlotFilling.prototype.clearSpellingLetterPosition = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Word.SlotFilling.prototype.hasSpellingLetterPosition = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.Word.SlotFilling}
 */
proto.speech.tts.Word.SlotFilling.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.Word.SlotFilling, data);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.Word.EntityInfo.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.Word.EntityInfo.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.Word.EntityInfo} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Word.EntityInfo.toObject = function(includeInstance, msg) {
  var f, obj = {
    index: jspb.Message.getFieldWithDefault(msg, 1, -1),
    mention: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    position: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of EntityInfo as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.Word.EntityInfo.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.index;

  /** @type {?number|undefined} */
  this.mention;

  /** @type {?number|undefined} */
  this.position;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.Word.EntityInfo.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.Word.EntityInfo}
 */
proto.speech.tts.Word.EntityInfo.fromObject = function(obj) {
  var msg = new proto.speech.tts.Word.EntityInfo();
  obj.index != null && jspb.Message.setField(msg, 1, obj.index);
  obj.mention != null && jspb.Message.setField(msg, 2, obj.mention);
  obj.position != null && jspb.Message.setField(msg, 3, obj.position);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.Word.EntityInfo}
 */
proto.speech.tts.Word.EntityInfo.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.Word.EntityInfo;
  return proto.speech.tts.Word.EntityInfo.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.Word.EntityInfo} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.Word.EntityInfo}
 */
proto.speech.tts.Word.EntityInfo.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setIndex(value);
      break;
    case 2:
      var value = /** @type {!proto.speech.tts.Word.EntityInfo.Mention} */ (reader.readEnum());
      msg.setMention(value);
      break;
    case 3:
      var value = /** @type {!proto.speech.tts.Word.EntityInfo.Position} */ (reader.readEnum());
      msg.setPosition(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.Word.EntityInfo.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.Word.EntityInfo.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.Word.EntityInfo} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Word.EntityInfo.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {number} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeInt32(
      1,
      f
    );
  }
  f = /** @type {!proto.speech.tts.Word.EntityInfo.Mention} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeEnum(
      2,
      f
    );
  }
  f = /** @type {!proto.speech.tts.Word.EntityInfo.Position} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeEnum(
      3,
      f
    );
  }
};


/**
 * @enum {number}
 */
proto.speech.tts.Word.EntityInfo.Mention = {
  UNSPECIFIED_MENTION: 0,
  INITIAL_MENTION: 1,
  REPEAT_MENTION: 2
};

/**
 * @enum {number}
 */
proto.speech.tts.Word.EntityInfo.Position = {
  UNSPECIFIED_POSITION: 0,
  INITIAL_POSITION: 1,
  MIDDLE_POSITION: 2,
  FINAL_POSITION: 3,
  INITIAL_FINAL_POSITION: 4
};

/**
 * optional int32 index = 1;
 * @return {number}
 */
proto.speech.tts.Word.EntityInfo.prototype.getIndex = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 1, -1));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Word.EntityInfo} returns this
 */
proto.speech.tts.Word.EntityInfo.prototype.setIndex = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Word.EntityInfo} returns this
 */
proto.speech.tts.Word.EntityInfo.prototype.clearIndex = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Word.EntityInfo.prototype.hasIndex = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional Mention mention = 2;
 * @return {!proto.speech.tts.Word.EntityInfo.Mention}
 */
proto.speech.tts.Word.EntityInfo.prototype.getMention = function() {
  return /** @type {!proto.speech.tts.Word.EntityInfo.Mention} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?proto.speech.tts.Word.EntityInfo.Mention|undefined} value
 * @return {!proto.speech.tts.Word.EntityInfo} returns this
 */
proto.speech.tts.Word.EntityInfo.prototype.setMention = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Word.EntityInfo} returns this
 */
proto.speech.tts.Word.EntityInfo.prototype.clearMention = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Word.EntityInfo.prototype.hasMention = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional Position position = 3;
 * @return {!proto.speech.tts.Word.EntityInfo.Position}
 */
proto.speech.tts.Word.EntityInfo.prototype.getPosition = function() {
  return /** @type {!proto.speech.tts.Word.EntityInfo.Position} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?proto.speech.tts.Word.EntityInfo.Position|undefined} value
 * @return {!proto.speech.tts.Word.EntityInfo} returns this
 */
proto.speech.tts.Word.EntityInfo.prototype.setPosition = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Word.EntityInfo} returns this
 */
proto.speech.tts.Word.EntityInfo.prototype.clearPosition = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Word.EntityInfo.prototype.hasPosition = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.Word.EntityInfo}
 */
proto.speech.tts.Word.EntityInfo.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.Word.EntityInfo, data);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.Word.ListInfo.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.Word.ListInfo.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.Word.ListInfo} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Word.ListInfo.toObject = function(includeInstance, msg) {
  var f, obj = {
    itemPosition: jspb.Message.getFieldWithDefault(msg, 1, 0),
    wordPosition: jspb.Message.getFieldWithDefault(msg, 2, 0),
    middleItemIndex: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,
    middleWordIndex: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of ListInfo as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.Word.ListInfo.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.itemPosition;

  /** @type {?number|undefined} */
  this.wordPosition;

  /** @type {?number|undefined} */
  this.middleItemIndex;

  /** @type {?number|undefined} */
  this.middleWordIndex;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.Word.ListInfo.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.Word.ListInfo}
 */
proto.speech.tts.Word.ListInfo.fromObject = function(obj) {
  var msg = new proto.speech.tts.Word.ListInfo();
  obj.itemPosition != null && jspb.Message.setField(msg, 1, obj.itemPosition);
  obj.wordPosition != null && jspb.Message.setField(msg, 2, obj.wordPosition);
  obj.middleItemIndex != null && jspb.Message.setField(msg, 3, obj.middleItemIndex);
  obj.middleWordIndex != null && jspb.Message.setField(msg, 4, obj.middleWordIndex);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.Word.ListInfo}
 */
proto.speech.tts.Word.ListInfo.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.Word.ListInfo;
  return proto.speech.tts.Word.ListInfo.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.Word.ListInfo} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.Word.ListInfo}
 */
proto.speech.tts.Word.ListInfo.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {!proto.speech.tts.Word.ListInfo.Position} */ (reader.readEnum());
      msg.setItemPosition(value);
      break;
    case 2:
      var value = /** @type {!proto.speech.tts.Word.ListInfo.Position} */ (reader.readEnum());
      msg.setWordPosition(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setMiddleItemIndex(value);
      break;
    case 4:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setMiddleWordIndex(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.Word.ListInfo.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.Word.ListInfo.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.Word.ListInfo} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Word.ListInfo.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {!proto.speech.tts.Word.ListInfo.Position} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeEnum(
      1,
      f
    );
  }
  f = /** @type {!proto.speech.tts.Word.ListInfo.Position} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeEnum(
      2,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeInt32(
      3,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeInt32(
      4,
      f
    );
  }
};


/**
 * @enum {number}
 */
proto.speech.tts.Word.ListInfo.Position = {
  UNSPECIFIED: 0,
  INITIAL: 1,
  MIDDLE: 2,
  FINAL: 3,
  INITIAL_FINAL: 4,
  CONNECTOR: 5
};

/**
 * optional Position item_position = 1;
 * @return {!proto.speech.tts.Word.ListInfo.Position}
 */
proto.speech.tts.Word.ListInfo.prototype.getItemPosition = function() {
  return /** @type {!proto.speech.tts.Word.ListInfo.Position} */ (jspb.Message.getFieldWithDefault(this, 1, 0));
};


/**
 * @param {?proto.speech.tts.Word.ListInfo.Position|undefined} value
 * @return {!proto.speech.tts.Word.ListInfo} returns this
 */
proto.speech.tts.Word.ListInfo.prototype.setItemPosition = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Word.ListInfo} returns this
 */
proto.speech.tts.Word.ListInfo.prototype.clearItemPosition = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Word.ListInfo.prototype.hasItemPosition = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional Position word_position = 2;
 * @return {!proto.speech.tts.Word.ListInfo.Position}
 */
proto.speech.tts.Word.ListInfo.prototype.getWordPosition = function() {
  return /** @type {!proto.speech.tts.Word.ListInfo.Position} */ (jspb.Message.getFieldWithDefault(this, 2, 0));
};


/**
 * @param {?proto.speech.tts.Word.ListInfo.Position|undefined} value
 * @return {!proto.speech.tts.Word.ListInfo} returns this
 */
proto.speech.tts.Word.ListInfo.prototype.setWordPosition = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Word.ListInfo} returns this
 */
proto.speech.tts.Word.ListInfo.prototype.clearWordPosition = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Word.ListInfo.prototype.hasWordPosition = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional int32 middle_item_index = 3;
 * @return {?number}
 */
proto.speech.tts.Word.ListInfo.prototype.getMiddleItemIndex = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Word.ListInfo} returns this
 */
proto.speech.tts.Word.ListInfo.prototype.setMiddleItemIndex = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Word.ListInfo} returns this
 */
proto.speech.tts.Word.ListInfo.prototype.clearMiddleItemIndex = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Word.ListInfo.prototype.hasMiddleItemIndex = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional int32 middle_word_index = 4;
 * @return {?number}
 */
proto.speech.tts.Word.ListInfo.prototype.getMiddleWordIndex = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 4));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Word.ListInfo} returns this
 */
proto.speech.tts.Word.ListInfo.prototype.setMiddleWordIndex = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Word.ListInfo} returns this
 */
proto.speech.tts.Word.ListInfo.prototype.clearMiddleWordIndex = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Word.ListInfo.prototype.hasMiddleWordIndex = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.Word.ListInfo}
 */
proto.speech.tts.Word.ListInfo.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.Word.ListInfo, data);
};


/**
 * optional Links links = 1;
 * @return {?proto.speech.tts.Links}
 */
proto.speech.tts.Word.prototype.getLinks = function() {
  return /** @type{?proto.speech.tts.Links} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Links, 1));
};


/**
 * @param {?proto.speech.tts.Links|undefined} value
 * @return {!proto.speech.tts.Word} returns this
*/
proto.speech.tts.Word.prototype.setLinks = function(value) {
  return jspb.Message.setWrapperField(this, 1, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.clearLinks = function() {
  return this.setLinks(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Word.prototype.hasLinks = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional Type type = 2;
 * @return {!proto.speech.tts.Word.Type}
 */
proto.speech.tts.Word.prototype.getType = function() {
  return /** @type {!proto.speech.tts.Word.Type} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?proto.speech.tts.Word.Type|undefined} value
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.setType = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.clearType = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Word.prototype.hasType = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional LexiconId lexicon_id = 63;
 * @return {?proto.speech.tts.LexiconId}
 */
proto.speech.tts.Word.prototype.getLexiconId = function() {
  return /** @type{?proto.speech.tts.LexiconId} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.LexiconId, 63));
};


/**
 * @param {?proto.speech.tts.LexiconId|undefined} value
 * @return {!proto.speech.tts.Word} returns this
*/
proto.speech.tts.Word.prototype.setLexiconId = function(value) {
  return jspb.Message.setWrapperField(this, 63, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.clearLexiconId = function() {
  return this.setLexiconId(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Word.prototype.hasLexiconId = function() {
  return jspb.Message.getField(this, 63) != null;
};


/**
 * optional string id = 3;
 * @return {?string}
 */
proto.speech.tts.Word.prototype.getId = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.setId = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.clearId = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Word.prototype.hasId = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional string variant = 4;
 * @return {?string}
 */
proto.speech.tts.Word.prototype.getVariant = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 4));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.setVariant = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.clearVariant = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Word.prototype.hasVariant = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional bool variant_forced = 5;
 * @return {boolean}
 */
proto.speech.tts.Word.prototype.getVariantForced = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 5, false));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.setVariantForced = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.clearVariantForced = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Word.prototype.hasVariantForced = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional string spelling = 7;
 * @return {?string}
 */
proto.speech.tts.Word.prototype.getSpelling = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 7));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.setSpelling = function(value) {
  return jspb.Message.setField(this, 7, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.clearSpelling = function() {
  return jspb.Message.setField(this, 7, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Word.prototype.hasSpelling = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * optional LexiconProto.Entry.Syntax lexical_syntax = 8;
 * @return {!proto.speech.tts.LexiconProto.Entry.Syntax}
 */
proto.speech.tts.Word.prototype.getLexicalSyntax = function() {
  return /** @type {!proto.speech.tts.LexiconProto.Entry.Syntax} */ (jspb.Message.getField(this, 8));
};


/**
 * @param {?proto.speech.tts.LexiconProto.Entry.Syntax|undefined} value
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.setLexicalSyntax = function(value) {
  return jspb.Message.setField(this, 8, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.clearLexicalSyntax = function() {
  return jspb.Message.setField(this, 8, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Word.prototype.hasLexicalSyntax = function() {
  return jspb.Message.getField(this, 8) != null;
};


/**
 * optional Syntax syntax = 9;
 * @return {?proto.speech.tts.Syntax}
 */
proto.speech.tts.Word.prototype.getSyntax = function() {
  return /** @type{?proto.speech.tts.Syntax} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Syntax, 9));
};


/**
 * @param {?proto.speech.tts.Syntax|undefined} value
 * @return {!proto.speech.tts.Word} returns this
*/
proto.speech.tts.Word.prototype.setSyntax = function(value) {
  return jspb.Message.setWrapperField(this, 9, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.clearSyntax = function() {
  return this.setSyntax(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Word.prototype.hasSyntax = function() {
  return jspb.Message.getField(this, 9) != null;
};


/**
 * optional float pause_length = 10;
 * @return {?number}
 */
proto.speech.tts.Word.prototype.getPauseLength = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 10));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.setPauseLength = function(value) {
  return jspb.Message.setField(this, 10, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.clearPauseLength = function() {
  return jspb.Message.setField(this, 10, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Word.prototype.hasPauseLength = function() {
  return jspb.Message.getField(this, 10) != null;
};


/**
 * optional IntonationType intonation_type = 11;
 * @return {!proto.speech.tts.IntonationType}
 */
proto.speech.tts.Word.prototype.getIntonationType = function() {
  return /** @type {!proto.speech.tts.IntonationType} */ (jspb.Message.getField(this, 11));
};


/**
 * @param {?proto.speech.tts.IntonationType|undefined} value
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.setIntonationType = function(value) {
  return jspb.Message.setField(this, 11, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.clearIntonationType = function() {
  return jspb.Message.setField(this, 11, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Word.prototype.hasIntonationType = function() {
  return jspb.Message.getField(this, 11) != null;
};


/**
 * optional bool precedes_pause = 12;
 * @return {?boolean}
 */
proto.speech.tts.Word.prototype.getPrecedesPause = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 12));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.setPrecedesPause = function(value) {
  return jspb.Message.setField(this, 12, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.clearPrecedesPause = function() {
  return jspb.Message.setField(this, 12, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Word.prototype.hasPrecedesPause = function() {
  return jspb.Message.getField(this, 12) != null;
};


/**
 * optional string spelling_with_stress = 14;
 * @return {?string}
 */
proto.speech.tts.Word.prototype.getSpellingWithStress = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 14));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.setSpellingWithStress = function(value) {
  return jspb.Message.setField(this, 14, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.clearSpellingWithStress = function() {
  return jspb.Message.setField(this, 14, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Word.prototype.hasSpellingWithStress = function() {
  return jspb.Message.getField(this, 14) != null;
};


/**
 * optional Prominence prominence = 15;
 * @return {?proto.speech.tts.Prominence}
 */
proto.speech.tts.Word.prototype.getProminence = function() {
  return /** @type{?proto.speech.tts.Prominence} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Prominence, 15));
};


/**
 * @param {?proto.speech.tts.Prominence|undefined} value
 * @return {!proto.speech.tts.Word} returns this
*/
proto.speech.tts.Word.prototype.setProminence = function(value) {
  return jspb.Message.setWrapperField(this, 15, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.clearProminence = function() {
  return this.setProminence(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Word.prototype.hasProminence = function() {
  return jspb.Message.getField(this, 15) != null;
};


/**
 * optional LmScore lm_score = 16;
 * @return {?proto.speech.tts.LmScore}
 */
proto.speech.tts.Word.prototype.getLmScore = function() {
  return /** @type{?proto.speech.tts.LmScore} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.LmScore, 16));
};


/**
 * @param {?proto.speech.tts.LmScore|undefined} value
 * @return {!proto.speech.tts.Word} returns this
*/
proto.speech.tts.Word.prototype.setLmScore = function(value) {
  return jspb.Message.setWrapperField(this, 16, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.clearLmScore = function() {
  return this.setLmScore(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Word.prototype.hasLmScore = function() {
  return jspb.Message.getField(this, 16) != null;
};


/**
 * optional int32 pitch_accent = 17;
 * @return {?number}
 */
proto.speech.tts.Word.prototype.getPitchAccent = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 17));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.setPitchAccent = function(value) {
  return jspb.Message.setField(this, 17, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.clearPitchAccent = function() {
  return jspb.Message.setField(this, 17, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Word.prototype.hasPitchAccent = function() {
  return jspb.Message.getField(this, 17) != null;
};


/**
 * optional bool pitch_accent_phrase_start = 18;
 * @return {?boolean}
 */
proto.speech.tts.Word.prototype.getPitchAccentPhraseStart = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 18));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.setPitchAccentPhraseStart = function(value) {
  return jspb.Message.setField(this, 18, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.clearPitchAccentPhraseStart = function() {
  return jspb.Message.setField(this, 18, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Word.prototype.hasPitchAccentPhraseStart = function() {
  return jspb.Message.getField(this, 18) != null;
};


/**
 * optional ExternalWordVectors external_word_vectors = 19;
 * @return {?proto.speech.tts.ExternalWordVectors}
 */
proto.speech.tts.Word.prototype.getExternalWordVectors = function() {
  return /** @type{?proto.speech.tts.ExternalWordVectors} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.ExternalWordVectors, 19));
};


/**
 * @param {?proto.speech.tts.ExternalWordVectors|undefined} value
 * @return {!proto.speech.tts.Word} returns this
*/
proto.speech.tts.Word.prototype.setExternalWordVectors = function(value) {
  return jspb.Message.setWrapperField(this, 19, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.clearExternalWordVectors = function() {
  return this.setExternalWordVectors(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Word.prototype.hasExternalWordVectors = function() {
  return jspb.Message.getField(this, 19) != null;
};


/**
 * optional string user_features = 20;
 * @return {?string}
 */
proto.speech.tts.Word.prototype.getUserFeatures = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 20));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.setUserFeatures = function(value) {
  return jspb.Message.setField(this, 20, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.clearUserFeatures = function() {
  return jspb.Message.setField(this, 20, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Word.prototype.hasUserFeatures = function() {
  return jspb.Message.getField(this, 20) != null;
};


/**
 * optional ShortPauseAnnotation short_pause_status = 21;
 * @return {!proto.speech.tts.Word.ShortPauseAnnotation}
 */
proto.speech.tts.Word.prototype.getShortPauseStatus = function() {
  return /** @type {!proto.speech.tts.Word.ShortPauseAnnotation} */ (jspb.Message.getField(this, 21));
};


/**
 * @param {?proto.speech.tts.Word.ShortPauseAnnotation|undefined} value
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.setShortPauseStatus = function(value) {
  return jspb.Message.setField(this, 21, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.clearShortPauseStatus = function() {
  return jspb.Message.setField(this, 21, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Word.prototype.hasShortPauseStatus = function() {
  return jspb.Message.getField(this, 21) != null;
};


/**
 * optional float rated_prominence = 22;
 * @return {?number}
 */
proto.speech.tts.Word.prototype.getRatedProminence = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 22));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.setRatedProminence = function(value) {
  return jspb.Message.setField(this, 22, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.clearRatedProminence = function() {
  return jspb.Message.setField(this, 22, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Word.prototype.hasRatedProminence = function() {
  return jspb.Message.getField(this, 22) != null;
};


/**
 * optional float predicted_prominence = 61;
 * @return {?number}
 */
proto.speech.tts.Word.prototype.getPredictedProminence = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 61));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.setPredictedProminence = function(value) {
  return jspb.Message.setField(this, 61, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.clearPredictedProminence = function() {
  return jspb.Message.setField(this, 61, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Word.prototype.hasPredictedProminence = function() {
  return jspb.Message.getField(this, 61) != null;
};


/**
 * optional PhraseBoundaryMarker phrase_boundary = 23;
 * @return {?proto.speech.tts.Word.PhraseBoundaryMarker}
 */
proto.speech.tts.Word.prototype.getPhraseBoundary = function() {
  return /** @type{?proto.speech.tts.Word.PhraseBoundaryMarker} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Word.PhraseBoundaryMarker, 23));
};


/**
 * @param {?proto.speech.tts.Word.PhraseBoundaryMarker|undefined} value
 * @return {!proto.speech.tts.Word} returns this
*/
proto.speech.tts.Word.prototype.setPhraseBoundary = function(value) {
  return jspb.Message.setWrapperField(this, 23, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.clearPhraseBoundary = function() {
  return this.setPhraseBoundary(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Word.prototype.hasPhraseBoundary = function() {
  return jspb.Message.getField(this, 23) != null;
};


/**
 * optional SlotFilling slot_filling = 24;
 * @return {?proto.speech.tts.Word.SlotFilling}
 */
proto.speech.tts.Word.prototype.getSlotFilling = function() {
  return /** @type{?proto.speech.tts.Word.SlotFilling} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Word.SlotFilling, 24));
};


/**
 * @param {?proto.speech.tts.Word.SlotFilling|undefined} value
 * @return {!proto.speech.tts.Word} returns this
*/
proto.speech.tts.Word.prototype.setSlotFilling = function(value) {
  return jspb.Message.setWrapperField(this, 24, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.clearSlotFilling = function() {
  return this.setSlotFilling(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Word.prototype.hasSlotFilling = function() {
  return jspb.Message.getField(this, 24) != null;
};


/**
 * optional VoiceMod voice_mod = 50;
 * @return {?proto.speech.tts.VoiceMod}
 */
proto.speech.tts.Word.prototype.getVoiceMod = function() {
  return /** @type{?proto.speech.tts.VoiceMod} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.VoiceMod, 50));
};


/**
 * @param {?proto.speech.tts.VoiceMod|undefined} value
 * @return {!proto.speech.tts.Word} returns this
*/
proto.speech.tts.Word.prototype.setVoiceMod = function(value) {
  return jspb.Message.setWrapperField(this, 50, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.clearVoiceMod = function() {
  return this.setVoiceMod(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Word.prototype.hasVoiceMod = function() {
  return jspb.Message.getField(this, 50) != null;
};


/**
 * optional string detected_language = 51;
 * @return {?string}
 */
proto.speech.tts.Word.prototype.getDetectedLanguage = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 51));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.setDetectedLanguage = function(value) {
  return jspb.Message.setField(this, 51, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.clearDetectedLanguage = function() {
  return jspb.Message.setField(this, 51, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Word.prototype.hasDetectedLanguage = function() {
  return jspb.Message.getField(this, 51) != null;
};


/**
 * optional bool bleep = 55;
 * @return {?boolean}
 */
proto.speech.tts.Word.prototype.getBleep = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 55));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.setBleep = function(value) {
  return jspb.Message.setField(this, 55, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.clearBleep = function() {
  return jspb.Message.setField(this, 55, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Word.prototype.hasBleep = function() {
  return jspb.Message.getField(this, 55) != null;
};


/**
 * optional bool skip = 56;
 * @return {?boolean}
 */
proto.speech.tts.Word.prototype.getSkip = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 56));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.setSkip = function(value) {
  return jspb.Message.setField(this, 56, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.clearSkip = function() {
  return jspb.Message.setField(this, 56, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Word.prototype.hasSkip = function() {
  return jspb.Message.getField(this, 56) != null;
};


/**
 * optional EntityInfo entity_info = 57;
 * @return {?proto.speech.tts.Word.EntityInfo}
 */
proto.speech.tts.Word.prototype.getEntityInfo = function() {
  return /** @type{?proto.speech.tts.Word.EntityInfo} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Word.EntityInfo, 57));
};


/**
 * @param {?proto.speech.tts.Word.EntityInfo|undefined} value
 * @return {!proto.speech.tts.Word} returns this
*/
proto.speech.tts.Word.prototype.setEntityInfo = function(value) {
  return jspb.Message.setWrapperField(this, 57, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.clearEntityInfo = function() {
  return this.setEntityInfo(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Word.prototype.hasEntityInfo = function() {
  return jspb.Message.getField(this, 57) != null;
};


/**
 * optional bool is_quoted = 58;
 * @return {?boolean}
 */
proto.speech.tts.Word.prototype.getIsQuoted = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 58));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.setIsQuoted = function(value) {
  return jspb.Message.setField(this, 58, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.clearIsQuoted = function() {
  return jspb.Message.setField(this, 58, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Word.prototype.hasIsQuoted = function() {
  return jspb.Message.getField(this, 58) != null;
};


/**
 * optional string homograph_resolved_via = 59;
 * @return {?string}
 */
proto.speech.tts.Word.prototype.getHomographResolvedVia = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 59));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.setHomographResolvedVia = function(value) {
  return jspb.Message.setField(this, 59, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.clearHomographResolvedVia = function() {
  return jspb.Message.setField(this, 59, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Word.prototype.hasHomographResolvedVia = function() {
  return jspb.Message.getField(this, 59) != null;
};


/**
 * optional ListInfo list_info = 60;
 * @return {?proto.speech.tts.Word.ListInfo}
 */
proto.speech.tts.Word.prototype.getListInfo = function() {
  return /** @type{?proto.speech.tts.Word.ListInfo} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Word.ListInfo, 60));
};


/**
 * @param {?proto.speech.tts.Word.ListInfo|undefined} value
 * @return {!proto.speech.tts.Word} returns this
*/
proto.speech.tts.Word.prototype.setListInfo = function(value) {
  return jspb.Message.setWrapperField(this, 60, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.clearListInfo = function() {
  return this.setListInfo(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Word.prototype.hasListInfo = function() {
  return jspb.Message.getField(this, 60) != null;
};


/**
 * optional bool is_generated = 62;
 * @return {?boolean}
 */
proto.speech.tts.Word.prototype.getIsGenerated = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 62));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.setIsGenerated = function(value) {
  return jspb.Message.setField(this, 62, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.clearIsGenerated = function() {
  return jspb.Message.setField(this, 62, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Word.prototype.hasIsGenerated = function() {
  return jspb.Message.getField(this, 62) != null;
};


/**
 * optional string source_pronunciation = 64;
 * @return {?string}
 */
proto.speech.tts.Word.prototype.getSourcePronunciation = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 64));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.setSourcePronunciation = function(value) {
  return jspb.Message.setField(this, 64, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.clearSourcePronunciation = function() {
  return jspb.Message.setField(this, 64, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Word.prototype.hasSourcePronunciation = function() {
  return jspb.Message.getField(this, 64) != null;
};


/**
 * optional PauseOrigin pause_origin = 65;
 * @return {!proto.speech.tts.Word.PauseOrigin}
 */
proto.speech.tts.Word.prototype.getPauseOrigin = function() {
  return /** @type {!proto.speech.tts.Word.PauseOrigin} */ (jspb.Message.getField(this, 65));
};


/**
 * @param {?proto.speech.tts.Word.PauseOrigin|undefined} value
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.setPauseOrigin = function(value) {
  return jspb.Message.setField(this, 65, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.clearPauseOrigin = function() {
  return jspb.Message.setField(this, 65, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Word.prototype.hasPauseOrigin = function() {
  return jspb.Message.getField(this, 65) != null;
};


/**
 * optional float relative_output_gain = 66;
 * @return {number}
 */
proto.speech.tts.Word.prototype.getRelativeOutputGain = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 66, 1.0));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.setRelativeOutputGain = function(value) {
  return jspb.Message.setField(this, 66, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Word} returns this
 */
proto.speech.tts.Word.prototype.clearRelativeOutputGain = function() {
  return jspb.Message.setField(this, 66, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Word.prototype.hasRelativeOutputGain = function() {
  return jspb.Message.getField(this, 66) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.Word}
 */
proto.speech.tts.Word.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.Word, data);
};



//speech/tts/engine/utterance/linguistic_structure_proto_jspb/linguisticstructure.jspb.js
// source: speech/tts/engine/utterance/linguistic_structure.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.LinguisticStructure');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.Phoneme');
goog.require('proto.speech.tts.Syllable');
goog.require('proto.speech.tts.Token');
goog.require('proto.speech.tts.Word');

goog.forwardDeclare('proto.speech.tts.PhonemeInventoryType');
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.LinguisticStructure = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 7, proto.speech.tts.LinguisticStructure.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.LinguisticStructure, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.LinguisticStructure.displayName = 'proto.speech.tts.LinguisticStructure';
}

/**
 * The extensions registered with this message class. This is a map of
 * extension field number to fieldInfo object.
 *
 * For example:
 *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }
 *
 * fieldName contains the JsCompiler renamed field name property so that it
 * works in OPTIMIZED mode.
 *
 * @type {!Object<number, jspb.ExtensionFieldInfo>}
 */
proto.speech.tts.LinguisticStructure.extensions = {};


/**
 * The extensions registered with this message class. This is a map of
 * extension field number to fieldInfo object.
 *
 * For example:
 *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }
 *
 * fieldName contains the JsCompiler renamed field name property so that it
 * works in OPTIMIZED mode.
 *
 * @type {!Object<number, jspb.ExtensionFieldBinaryInfo>}
 */
proto.speech.tts.LinguisticStructure.extensionsBinary = {};


/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.LinguisticStructure.repeatedFields_ = [1,2,3,4];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.LinguisticStructure.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.LinguisticStructure.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.LinguisticStructure} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.LinguisticStructure.toObject = function(includeInstance, msg) {
  var f, obj = {
    tokensList: jspb.Message.toObjectList(msg.getTokensList(),
    proto.speech.tts.Token.toObject, includeInstance),
    wordsList: jspb.Message.toObjectList(msg.getWordsList(),
    proto.speech.tts.Word.toObject, includeInstance),
    syllablesList: jspb.Message.toObjectList(msg.getSyllablesList(),
    proto.speech.tts.Syllable.toObject, includeInstance),
    phonemesList: jspb.Message.toObjectList(msg.getPhonemesList(),
    proto.speech.tts.Phoneme.toObject, includeInstance),
    phonemeInventoryType: (f = jspb.Message.getField(msg, 6)) == null ? undefined : f
  };

  jspb.Message.toObjectExtension(/** @type {!jspb.Message} */ (msg), obj,
      proto.speech.tts.LinguisticStructure.extensions, proto.speech.tts.LinguisticStructure.prototype.getExtension,
      includeInstance);
  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of LinguisticStructure as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.LinguisticStructure.ObjectFormat = function() {
  /** @type {?Array<!proto.speech.tts.Token.ObjectFormat>|undefined} */
  this.tokensList;

  /** @type {?Array<!proto.speech.tts.Word.ObjectFormat>|undefined} */
  this.wordsList;

  /** @type {?Array<!proto.speech.tts.Syllable.ObjectFormat>|undefined} */
  this.syllablesList;

  /** @type {?Array<!proto.speech.tts.Phoneme.ObjectFormat>|undefined} */
  this.phonemesList;

  /** @type {?number|undefined} */
  this.phonemeInventoryType;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.LinguisticStructure.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.LinguisticStructure}
 */
proto.speech.tts.LinguisticStructure.fromObject = function(obj) {
  var msg = new proto.speech.tts.LinguisticStructure();
  obj.tokensList && jspb.Message.setRepeatedWrapperField(
      msg, 1, obj.tokensList.map(
          proto.speech.tts.Token.fromObject));
  obj.wordsList && jspb.Message.setRepeatedWrapperField(
      msg, 2, obj.wordsList.map(
          proto.speech.tts.Word.fromObject));
  obj.syllablesList && jspb.Message.setRepeatedWrapperField(
      msg, 3, obj.syllablesList.map(
          proto.speech.tts.Syllable.fromObject));
  obj.phonemesList && jspb.Message.setRepeatedWrapperField(
      msg, 4, obj.phonemesList.map(
          proto.speech.tts.Phoneme.fromObject));
  obj.phonemeInventoryType != null && jspb.Message.setField(msg, 6, obj.phonemeInventoryType);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.LinguisticStructure}
 */
proto.speech.tts.LinguisticStructure.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.LinguisticStructure;
  return proto.speech.tts.LinguisticStructure.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.LinguisticStructure} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.LinguisticStructure}
 */
proto.speech.tts.LinguisticStructure.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.speech.tts.Token;
      reader.readMessage(value,proto.speech.tts.Token.deserializeBinaryFromReader);
      msg.addTokens(value);
      break;
    case 2:
      var value = new proto.speech.tts.Word;
      reader.readMessage(value,proto.speech.tts.Word.deserializeBinaryFromReader);
      msg.addWords(value);
      break;
    case 3:
      var value = new proto.speech.tts.Syllable;
      reader.readMessage(value,proto.speech.tts.Syllable.deserializeBinaryFromReader);
      msg.addSyllables(value);
      break;
    case 4:
      var value = new proto.speech.tts.Phoneme;
      reader.readMessage(value,proto.speech.tts.Phoneme.deserializeBinaryFromReader);
      msg.addPhonemes(value);
      break;
    case 6:
      var value = /** @type {!proto.speech.tts.PhonemeInventoryType} */ (reader.readEnum());
      msg.setPhonemeInventoryType(value);
      break;
    default:
      jspb.Message.readBinaryExtension(msg, reader,
        proto.speech.tts.LinguisticStructure.extensionsBinary,
        proto.speech.tts.LinguisticStructure.prototype.getExtension,
        proto.speech.tts.LinguisticStructure.prototype.setExtension);
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.LinguisticStructure.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.LinguisticStructure.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.LinguisticStructure} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.LinguisticStructure.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getTokensList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      1,
      f,
      proto.speech.tts.Token.serializeBinaryToWriter
    );
  }
  f = message.getWordsList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      2,
      f,
      proto.speech.tts.Word.serializeBinaryToWriter
    );
  }
  f = message.getSyllablesList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      3,
      f,
      proto.speech.tts.Syllable.serializeBinaryToWriter
    );
  }
  f = message.getPhonemesList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      4,
      f,
      proto.speech.tts.Phoneme.serializeBinaryToWriter
    );
  }
  f = /** @type {!proto.speech.tts.PhonemeInventoryType} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeEnum(
      6,
      f
    );
  }
  jspb.Message.serializeBinaryExtensions(message, writer,
    proto.speech.tts.LinguisticStructure.extensionsBinary, proto.speech.tts.LinguisticStructure.prototype.getExtension);
};


/**
 * repeated Token tokens = 1;
 * @return {!Array<!proto.speech.tts.Token>}
 */
proto.speech.tts.LinguisticStructure.prototype.getTokensList = function() {
  return /** @type{!Array<!proto.speech.tts.Token>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.Token, 1));
};


/**
 * @param {Array<!proto.speech.tts.Token>|undefined} value
 * @return {!proto.speech.tts.LinguisticStructure} returns this
*/
proto.speech.tts.LinguisticStructure.prototype.setTokensList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 1, value);
};


/**
 * @param {!proto.speech.tts.Token=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.Token}
 */
proto.speech.tts.LinguisticStructure.prototype.addTokens = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.speech.tts.Token, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.LinguisticStructure} returns this
 */
proto.speech.tts.LinguisticStructure.prototype.clearTokensList = function() {
  return this.setTokensList([]);
};


/**
 * repeated Word words = 2;
 * @return {!Array<!proto.speech.tts.Word>}
 */
proto.speech.tts.LinguisticStructure.prototype.getWordsList = function() {
  return /** @type{!Array<!proto.speech.tts.Word>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.Word, 2));
};


/**
 * @param {Array<!proto.speech.tts.Word>|undefined} value
 * @return {!proto.speech.tts.LinguisticStructure} returns this
*/
proto.speech.tts.LinguisticStructure.prototype.setWordsList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 2, value);
};


/**
 * @param {!proto.speech.tts.Word=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.Word}
 */
proto.speech.tts.LinguisticStructure.prototype.addWords = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 2, opt_value, proto.speech.tts.Word, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.LinguisticStructure} returns this
 */
proto.speech.tts.LinguisticStructure.prototype.clearWordsList = function() {
  return this.setWordsList([]);
};


/**
 * repeated Syllable syllables = 3;
 * @return {!Array<!proto.speech.tts.Syllable>}
 */
proto.speech.tts.LinguisticStructure.prototype.getSyllablesList = function() {
  return /** @type{!Array<!proto.speech.tts.Syllable>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.Syllable, 3));
};


/**
 * @param {Array<!proto.speech.tts.Syllable>|undefined} value
 * @return {!proto.speech.tts.LinguisticStructure} returns this
*/
proto.speech.tts.LinguisticStructure.prototype.setSyllablesList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 3, value);
};


/**
 * @param {!proto.speech.tts.Syllable=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.Syllable}
 */
proto.speech.tts.LinguisticStructure.prototype.addSyllables = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 3, opt_value, proto.speech.tts.Syllable, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.LinguisticStructure} returns this
 */
proto.speech.tts.LinguisticStructure.prototype.clearSyllablesList = function() {
  return this.setSyllablesList([]);
};


/**
 * repeated Phoneme phonemes = 4;
 * @return {!Array<!proto.speech.tts.Phoneme>}
 */
proto.speech.tts.LinguisticStructure.prototype.getPhonemesList = function() {
  return /** @type{!Array<!proto.speech.tts.Phoneme>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.Phoneme, 4));
};


/**
 * @param {Array<!proto.speech.tts.Phoneme>|undefined} value
 * @return {!proto.speech.tts.LinguisticStructure} returns this
*/
proto.speech.tts.LinguisticStructure.prototype.setPhonemesList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 4, value);
};


/**
 * @param {!proto.speech.tts.Phoneme=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.Phoneme}
 */
proto.speech.tts.LinguisticStructure.prototype.addPhonemes = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 4, opt_value, proto.speech.tts.Phoneme, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.LinguisticStructure} returns this
 */
proto.speech.tts.LinguisticStructure.prototype.clearPhonemesList = function() {
  return this.setPhonemesList([]);
};


/**
 * optional PhonemeInventoryType phoneme_inventory_type = 6;
 * @return {!proto.speech.tts.PhonemeInventoryType}
 */
proto.speech.tts.LinguisticStructure.prototype.getPhonemeInventoryType = function() {
  return /** @type {!proto.speech.tts.PhonemeInventoryType} */ (jspb.Message.getField(this, 6));
};


/**
 * @param {?proto.speech.tts.PhonemeInventoryType|undefined} value
 * @return {!proto.speech.tts.LinguisticStructure} returns this
 */
proto.speech.tts.LinguisticStructure.prototype.setPhonemeInventoryType = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.LinguisticStructure} returns this
 */
proto.speech.tts.LinguisticStructure.prototype.clearPhonemeInventoryType = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.LinguisticStructure.prototype.hasPhonemeInventoryType = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.LinguisticStructure}
 */
proto.speech.tts.LinguisticStructure.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.LinguisticStructure, data);
};



//speech/tts/engine/utterance/text_normalization_data_proto_jspb/textnormalizationdata.jspb.js
// source: speech/tts/engine/utterance/text_normalization_data.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.TextNormalizationData');
goog.provide('proto.speech.tts.TextNormalizationData.SpanInfo');
goog.provide('proto.speech.tts.TextNormalizationData.TextSpan');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.TextNormalizationData = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.TextNormalizationData.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.TextNormalizationData, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.TextNormalizationData.displayName = 'proto.speech.tts.TextNormalizationData';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.TextNormalizationData.TextSpan = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.TextNormalizationData.TextSpan, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.TextNormalizationData.TextSpan.displayName = 'proto.speech.tts.TextNormalizationData.TextSpan';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.TextNormalizationData.SpanInfo = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.TextNormalizationData.SpanInfo.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.TextNormalizationData.SpanInfo, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.TextNormalizationData.SpanInfo.displayName = 'proto.speech.tts.TextNormalizationData.SpanInfo';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.TextNormalizationData.repeatedFields_ = [4,2];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.TextNormalizationData.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.TextNormalizationData.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.TextNormalizationData} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.TextNormalizationData.toObject = function(includeInstance, msg) {
  var f, obj = {
    verbalizeGrmDecoderOutput: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    spanInfosList: jspb.Message.toObjectList(msg.getSpanInfosList(),
    proto.speech.tts.TextNormalizationData.SpanInfo.toObject, includeInstance),
    invalidAlignment: (f = jspb.Message.getBooleanField(msg, 5)) == null ? undefined : f,
    byteSpansInOriginalTextList: jspb.Message.toObjectList(msg.getByteSpansInOriginalTextList(),
    proto.speech.tts.TextNormalizationData.TextSpan.toObject, includeInstance)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of TextNormalizationData as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.TextNormalizationData.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.verbalizeGrmDecoderOutput;

  /** @type {?Array<!proto.speech.tts.TextNormalizationData.SpanInfo.ObjectFormat>|undefined} */
  this.spanInfosList;

  /** @type {?boolean|undefined} */
  this.invalidAlignment;

  /** @type {?Array<!proto.speech.tts.TextNormalizationData.TextSpan.ObjectFormat>|undefined} */
  this.byteSpansInOriginalTextList;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.TextNormalizationData.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.TextNormalizationData}
 */
proto.speech.tts.TextNormalizationData.fromObject = function(obj) {
  var msg = new proto.speech.tts.TextNormalizationData();
  obj.verbalizeGrmDecoderOutput != null && jspb.Message.setField(msg, 1, obj.verbalizeGrmDecoderOutput);
  obj.spanInfosList && jspb.Message.setRepeatedWrapperField(
      msg, 4, obj.spanInfosList.map(
          proto.speech.tts.TextNormalizationData.SpanInfo.fromObject));
  obj.invalidAlignment != null && jspb.Message.setField(msg, 5, obj.invalidAlignment);
  obj.byteSpansInOriginalTextList && jspb.Message.setRepeatedWrapperField(
      msg, 2, obj.byteSpansInOriginalTextList.map(
          proto.speech.tts.TextNormalizationData.TextSpan.fromObject));
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.TextNormalizationData}
 */
proto.speech.tts.TextNormalizationData.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.TextNormalizationData;
  return proto.speech.tts.TextNormalizationData.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.TextNormalizationData} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.TextNormalizationData}
 */
proto.speech.tts.TextNormalizationData.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setVerbalizeGrmDecoderOutput(value);
      break;
    case 4:
      var value = new proto.speech.tts.TextNormalizationData.SpanInfo;
      reader.readMessage(value,proto.speech.tts.TextNormalizationData.SpanInfo.deserializeBinaryFromReader);
      msg.addSpanInfos(value);
      break;
    case 5:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setInvalidAlignment(value);
      break;
    case 2:
      var value = new proto.speech.tts.TextNormalizationData.TextSpan;
      reader.readMessage(value,proto.speech.tts.TextNormalizationData.TextSpan.deserializeBinaryFromReader);
      msg.addByteSpansInOriginalText(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.TextNormalizationData.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.TextNormalizationData.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.TextNormalizationData} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.TextNormalizationData.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = message.getSpanInfosList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      4,
      f,
      proto.speech.tts.TextNormalizationData.SpanInfo.serializeBinaryToWriter
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeBool(
      5,
      f
    );
  }
  f = message.getByteSpansInOriginalTextList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      2,
      f,
      proto.speech.tts.TextNormalizationData.TextSpan.serializeBinaryToWriter
    );
  }
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.TextNormalizationData.TextSpan.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.TextNormalizationData.TextSpan.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.TextNormalizationData.TextSpan} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.TextNormalizationData.TextSpan.toObject = function(includeInstance, msg) {
  var f, obj = {
    b: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    e: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    parent: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of TextSpan as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.TextNormalizationData.TextSpan.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.b;

  /** @type {?number|undefined} */
  this.e;

  /** @type {?number|undefined} */
  this.parent;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.TextNormalizationData.TextSpan.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.TextNormalizationData.TextSpan}
 */
proto.speech.tts.TextNormalizationData.TextSpan.fromObject = function(obj) {
  var msg = new proto.speech.tts.TextNormalizationData.TextSpan();
  obj.b != null && jspb.Message.setField(msg, 1, obj.b);
  obj.e != null && jspb.Message.setField(msg, 2, obj.e);
  obj.parent != null && jspb.Message.setField(msg, 3, obj.parent);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.TextNormalizationData.TextSpan}
 */
proto.speech.tts.TextNormalizationData.TextSpan.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.TextNormalizationData.TextSpan;
  return proto.speech.tts.TextNormalizationData.TextSpan.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.TextNormalizationData.TextSpan} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.TextNormalizationData.TextSpan}
 */
proto.speech.tts.TextNormalizationData.TextSpan.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setB(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setE(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setParent(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.TextNormalizationData.TextSpan.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.TextNormalizationData.TextSpan.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.TextNormalizationData.TextSpan} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.TextNormalizationData.TextSpan.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {number} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeInt32(
      1,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeInt32(
      2,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeInt32(
      3,
      f
    );
  }
};


/**
 * required int32 b = 1;
 * @return {number}
 */
proto.speech.tts.TextNormalizationData.TextSpan.prototype.getB = function() {
  return /** @type {number} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.TextNormalizationData.TextSpan} returns this
 */
proto.speech.tts.TextNormalizationData.TextSpan.prototype.setB = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.TextNormalizationData.TextSpan} returns this
 */
proto.speech.tts.TextNormalizationData.TextSpan.prototype.clearB = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.TextNormalizationData.TextSpan.prototype.hasB = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * required int32 e = 2;
 * @return {number}
 */
proto.speech.tts.TextNormalizationData.TextSpan.prototype.getE = function() {
  return /** @type {number} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.TextNormalizationData.TextSpan} returns this
 */
proto.speech.tts.TextNormalizationData.TextSpan.prototype.setE = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.TextNormalizationData.TextSpan} returns this
 */
proto.speech.tts.TextNormalizationData.TextSpan.prototype.clearE = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.TextNormalizationData.TextSpan.prototype.hasE = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional int32 parent = 3;
 * @return {?number}
 */
proto.speech.tts.TextNormalizationData.TextSpan.prototype.getParent = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.TextNormalizationData.TextSpan} returns this
 */
proto.speech.tts.TextNormalizationData.TextSpan.prototype.setParent = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.TextNormalizationData.TextSpan} returns this
 */
proto.speech.tts.TextNormalizationData.TextSpan.prototype.clearParent = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.TextNormalizationData.TextSpan.prototype.hasParent = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.TextNormalizationData.TextSpan}
 */
proto.speech.tts.TextNormalizationData.TextSpan.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.TextNormalizationData.TextSpan, data);
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.TextNormalizationData.SpanInfo.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.TextNormalizationData.SpanInfo.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.TextNormalizationData.SpanInfo.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.TextNormalizationData.SpanInfo} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.TextNormalizationData.SpanInfo.toObject = function(includeInstance, msg) {
  var f, obj = {
    byteSpansInOriginalTextList: jspb.Message.toObjectList(msg.getByteSpansInOriginalTextList(),
    proto.speech.tts.TextNormalizationData.TextSpan.toObject, includeInstance)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of SpanInfo as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.TextNormalizationData.SpanInfo.ObjectFormat = function() {
  /** @type {?Array<!proto.speech.tts.TextNormalizationData.TextSpan.ObjectFormat>|undefined} */
  this.byteSpansInOriginalTextList;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.TextNormalizationData.SpanInfo.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.TextNormalizationData.SpanInfo}
 */
proto.speech.tts.TextNormalizationData.SpanInfo.fromObject = function(obj) {
  var msg = new proto.speech.tts.TextNormalizationData.SpanInfo();
  obj.byteSpansInOriginalTextList && jspb.Message.setRepeatedWrapperField(
      msg, 1, obj.byteSpansInOriginalTextList.map(
          proto.speech.tts.TextNormalizationData.TextSpan.fromObject));
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.TextNormalizationData.SpanInfo}
 */
proto.speech.tts.TextNormalizationData.SpanInfo.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.TextNormalizationData.SpanInfo;
  return proto.speech.tts.TextNormalizationData.SpanInfo.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.TextNormalizationData.SpanInfo} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.TextNormalizationData.SpanInfo}
 */
proto.speech.tts.TextNormalizationData.SpanInfo.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.speech.tts.TextNormalizationData.TextSpan;
      reader.readMessage(value,proto.speech.tts.TextNormalizationData.TextSpan.deserializeBinaryFromReader);
      msg.addByteSpansInOriginalText(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.TextNormalizationData.SpanInfo.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.TextNormalizationData.SpanInfo.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.TextNormalizationData.SpanInfo} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.TextNormalizationData.SpanInfo.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getByteSpansInOriginalTextList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      1,
      f,
      proto.speech.tts.TextNormalizationData.TextSpan.serializeBinaryToWriter
    );
  }
};


/**
 * repeated TextSpan byte_spans_in_original_text = 1;
 * @return {!Array<!proto.speech.tts.TextNormalizationData.TextSpan>}
 */
proto.speech.tts.TextNormalizationData.SpanInfo.prototype.getByteSpansInOriginalTextList = function() {
  return /** @type{!Array<!proto.speech.tts.TextNormalizationData.TextSpan>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.TextNormalizationData.TextSpan, 1));
};


/**
 * @param {Array<!proto.speech.tts.TextNormalizationData.TextSpan>|undefined} value
 * @return {!proto.speech.tts.TextNormalizationData.SpanInfo} returns this
*/
proto.speech.tts.TextNormalizationData.SpanInfo.prototype.setByteSpansInOriginalTextList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 1, value);
};


/**
 * @param {!proto.speech.tts.TextNormalizationData.TextSpan=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.TextNormalizationData.TextSpan}
 */
proto.speech.tts.TextNormalizationData.SpanInfo.prototype.addByteSpansInOriginalText = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.speech.tts.TextNormalizationData.TextSpan, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.TextNormalizationData.SpanInfo} returns this
 */
proto.speech.tts.TextNormalizationData.SpanInfo.prototype.clearByteSpansInOriginalTextList = function() {
  return this.setByteSpansInOriginalTextList([]);
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.TextNormalizationData.SpanInfo}
 */
proto.speech.tts.TextNormalizationData.SpanInfo.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.TextNormalizationData.SpanInfo, data);
};


/**
 * optional string verbalize_grm_decoder_output = 1;
 * @return {?string}
 */
proto.speech.tts.TextNormalizationData.prototype.getVerbalizeGrmDecoderOutput = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.TextNormalizationData} returns this
 */
proto.speech.tts.TextNormalizationData.prototype.setVerbalizeGrmDecoderOutput = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.TextNormalizationData} returns this
 */
proto.speech.tts.TextNormalizationData.prototype.clearVerbalizeGrmDecoderOutput = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.TextNormalizationData.prototype.hasVerbalizeGrmDecoderOutput = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * repeated SpanInfo span_infos = 4;
 * @return {!Array<!proto.speech.tts.TextNormalizationData.SpanInfo>}
 */
proto.speech.tts.TextNormalizationData.prototype.getSpanInfosList = function() {
  return /** @type{!Array<!proto.speech.tts.TextNormalizationData.SpanInfo>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.TextNormalizationData.SpanInfo, 4));
};


/**
 * @param {Array<!proto.speech.tts.TextNormalizationData.SpanInfo>|undefined} value
 * @return {!proto.speech.tts.TextNormalizationData} returns this
*/
proto.speech.tts.TextNormalizationData.prototype.setSpanInfosList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 4, value);
};


/**
 * @param {!proto.speech.tts.TextNormalizationData.SpanInfo=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.TextNormalizationData.SpanInfo}
 */
proto.speech.tts.TextNormalizationData.prototype.addSpanInfos = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 4, opt_value, proto.speech.tts.TextNormalizationData.SpanInfo, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.TextNormalizationData} returns this
 */
proto.speech.tts.TextNormalizationData.prototype.clearSpanInfosList = function() {
  return this.setSpanInfosList([]);
};


/**
 * optional bool invalid_alignment = 5;
 * @return {?boolean}
 */
proto.speech.tts.TextNormalizationData.prototype.getInvalidAlignment = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 5));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.TextNormalizationData} returns this
 */
proto.speech.tts.TextNormalizationData.prototype.setInvalidAlignment = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.TextNormalizationData} returns this
 */
proto.speech.tts.TextNormalizationData.prototype.clearInvalidAlignment = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.TextNormalizationData.prototype.hasInvalidAlignment = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * repeated TextSpan byte_spans_in_original_text = 2;
 * @return {!Array<!proto.speech.tts.TextNormalizationData.TextSpan>}
 */
proto.speech.tts.TextNormalizationData.prototype.getByteSpansInOriginalTextList = function() {
  return /** @type{!Array<!proto.speech.tts.TextNormalizationData.TextSpan>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.TextNormalizationData.TextSpan, 2));
};


/**
 * @param {Array<!proto.speech.tts.TextNormalizationData.TextSpan>|undefined} value
 * @return {!proto.speech.tts.TextNormalizationData} returns this
*/
proto.speech.tts.TextNormalizationData.prototype.setByteSpansInOriginalTextList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 2, value);
};


/**
 * @param {!proto.speech.tts.TextNormalizationData.TextSpan=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.TextNormalizationData.TextSpan}
 */
proto.speech.tts.TextNormalizationData.prototype.addByteSpansInOriginalText = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 2, opt_value, proto.speech.tts.TextNormalizationData.TextSpan, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.TextNormalizationData} returns this
 */
proto.speech.tts.TextNormalizationData.prototype.clearByteSpansInOriginalTextList = function() {
  return this.setByteSpansInOriginalTextList([]);
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.TextNormalizationData}
 */
proto.speech.tts.TextNormalizationData.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.TextNormalizationData, data);
};



//speech/tts/public/lexicon_proto_jspb/lexiconproto.jspb.js
// source: speech/tts/public/lexicon.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.LexiconProto');
goog.provide('proto.speech.tts.LexiconProto.CrossLingualInfo');
goog.provide('proto.speech.tts.LexiconProto.Entry');
goog.provide('proto.speech.tts.LexiconProto.Entry.Syntax');
goog.provide('proto.speech.tts.LexiconProto.ExpandedPronunciation');
goog.provide('proto.speech.tts.LexiconProto.Morphology');
goog.provide('proto.speech.tts.LexiconProto.Morphology.Animacy');
goog.provide('proto.speech.tts.LexiconProto.Morphology.Gender');
goog.provide('proto.speech.tts.LexiconProto.Morphology.Number');
goog.provide('proto.speech.tts.LexiconProto.Morphology.PartOfSpeech');
goog.provide('proto.speech.tts.LexiconProto.Pronunciation');
goog.provide('proto.speech.tts.LexiconProto.Pronunciation.PronunciationType');
goog.provide('proto.speech.tts.LexiconProto.Spelling');
goog.provide('proto.speech.tts.LexiconProto.Syllable');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.LexiconProto = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.LexiconProto.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.LexiconProto, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.LexiconProto.displayName = 'proto.speech.tts.LexiconProto';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.LexiconProto.Pronunciation = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.LexiconProto.Pronunciation, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.LexiconProto.Pronunciation.displayName = 'proto.speech.tts.LexiconProto.Pronunciation';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.LexiconProto.Spelling = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.LexiconProto.Spelling, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.LexiconProto.Spelling.displayName = 'proto.speech.tts.LexiconProto.Spelling';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.LexiconProto.ExpandedPronunciation = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.LexiconProto.ExpandedPronunciation.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.LexiconProto.ExpandedPronunciation, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.LexiconProto.ExpandedPronunciation.displayName = 'proto.speech.tts.LexiconProto.ExpandedPronunciation';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.LexiconProto.Syllable = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.LexiconProto.Syllable.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.LexiconProto.Syllable, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.LexiconProto.Syllable.displayName = 'proto.speech.tts.LexiconProto.Syllable';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.LexiconProto.Morphology = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.LexiconProto.Morphology, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.LexiconProto.Morphology.displayName = 'proto.speech.tts.LexiconProto.Morphology';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.LexiconProto.Entry = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.LexiconProto.Entry.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.LexiconProto.Entry, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.LexiconProto.Entry.displayName = 'proto.speech.tts.LexiconProto.Entry';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.LexiconProto.CrossLingualInfo = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.LexiconProto.CrossLingualInfo, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.LexiconProto.CrossLingualInfo.displayName = 'proto.speech.tts.LexiconProto.CrossLingualInfo';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.LexiconProto.repeatedFields_ = [2];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.LexiconProto.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.LexiconProto.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.LexiconProto} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.LexiconProto.toObject = function(includeInstance, msg) {
  var f, obj = {
    header: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    wList: jspb.Message.toObjectList(msg.getWList(),
    proto.speech.tts.LexiconProto.Entry.toObject, includeInstance),
    legalVowels: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,
    crossLingual: (f = msg.getCrossLingual()) && proto.speech.tts.LexiconProto.CrossLingualInfo.toObject(includeInstance, f),
    isTsampa: (f = jspb.Message.getBooleanField(msg, 5)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of LexiconProto as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.LexiconProto.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.header;

  /** @type {?Array<!proto.speech.tts.LexiconProto.Entry.ObjectFormat>|undefined} */
  this.wList;

  /** @type {?string|undefined} */
  this.legalVowels;

  /** @type {?proto.speech.tts.LexiconProto.CrossLingualInfo.ObjectFormat|undefined} */
  this.crossLingual;

  /** @type {?boolean|undefined} */
  this.isTsampa;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.LexiconProto.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.LexiconProto}
 */
proto.speech.tts.LexiconProto.fromObject = function(obj) {
  var msg = new proto.speech.tts.LexiconProto();
  obj.header != null && jspb.Message.setField(msg, 1, obj.header);
  obj.wList && jspb.Message.setRepeatedWrapperField(
      msg, 2, obj.wList.map(
          proto.speech.tts.LexiconProto.Entry.fromObject));
  obj.legalVowels != null && jspb.Message.setField(msg, 3, obj.legalVowels);
  obj.crossLingual && jspb.Message.setWrapperField(
      msg, 4, proto.speech.tts.LexiconProto.CrossLingualInfo.fromObject(obj.crossLingual));
  obj.isTsampa != null && jspb.Message.setField(msg, 5, obj.isTsampa);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.LexiconProto}
 */
proto.speech.tts.LexiconProto.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.LexiconProto;
  return proto.speech.tts.LexiconProto.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.LexiconProto} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.LexiconProto}
 */
proto.speech.tts.LexiconProto.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setHeader(value);
      break;
    case 2:
      var value = new proto.speech.tts.LexiconProto.Entry;
      reader.readMessage(value,proto.speech.tts.LexiconProto.Entry.deserializeBinaryFromReader);
      msg.addW(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setLegalVowels(value);
      break;
    case 4:
      var value = new proto.speech.tts.LexiconProto.CrossLingualInfo;
      reader.readMessage(value,proto.speech.tts.LexiconProto.CrossLingualInfo.deserializeBinaryFromReader);
      msg.setCrossLingual(value);
      break;
    case 5:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsTsampa(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.LexiconProto.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.LexiconProto.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.LexiconProto} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.LexiconProto.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = message.getWList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      2,
      f,
      proto.speech.tts.LexiconProto.Entry.serializeBinaryToWriter
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeString(
      3,
      f
    );
  }
  f = message.getCrossLingual();
  if (f != null) {
    writer.writeMessage(
      4,
      f,
      proto.speech.tts.LexiconProto.CrossLingualInfo.serializeBinaryToWriter
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeBool(
      5,
      f
    );
  }
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.LexiconProto.Pronunciation.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.LexiconProto.Pronunciation.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.LexiconProto.Pronunciation} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.LexiconProto.Pronunciation.toObject = function(includeInstance, msg) {
  var f, obj = {
    id: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    v: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    t: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Pronunciation as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.LexiconProto.Pronunciation.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.id;

  /** @type {?string|undefined} */
  this.v;

  /** @type {?number|undefined} */
  this.t;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.LexiconProto.Pronunciation.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.LexiconProto.Pronunciation}
 */
proto.speech.tts.LexiconProto.Pronunciation.fromObject = function(obj) {
  var msg = new proto.speech.tts.LexiconProto.Pronunciation();
  obj.id != null && jspb.Message.setField(msg, 1, obj.id);
  obj.v != null && jspb.Message.setField(msg, 2, obj.v);
  obj.t != null && jspb.Message.setField(msg, 3, obj.t);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.LexiconProto.Pronunciation}
 */
proto.speech.tts.LexiconProto.Pronunciation.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.LexiconProto.Pronunciation;
  return proto.speech.tts.LexiconProto.Pronunciation.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.LexiconProto.Pronunciation} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.LexiconProto.Pronunciation}
 */
proto.speech.tts.LexiconProto.Pronunciation.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setId(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setV(value);
      break;
    case 3:
      var value = /** @type {!proto.speech.tts.LexiconProto.Pronunciation.PronunciationType} */ (reader.readEnum());
      msg.setT(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.LexiconProto.Pronunciation.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.LexiconProto.Pronunciation.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.LexiconProto.Pronunciation} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.LexiconProto.Pronunciation.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
  f = /** @type {!proto.speech.tts.LexiconProto.Pronunciation.PronunciationType} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeEnum(
      3,
      f
    );
  }
};


/**
 * @enum {number}
 */
proto.speech.tts.LexiconProto.Pronunciation.PronunciationType = {
  UNKNOWN: 0,
  CANONICAL: 1,
  REDUCED: 2,
  ALT1: 3,
  ALT2: 4,
  NOT_SUITABLE_FOR_TTS: 5,
  CANONICAL_FROM_PRON_LEARNING: 6
};

/**
 * required string id = 1;
 * @return {string}
 */
proto.speech.tts.LexiconProto.Pronunciation.prototype.getId = function() {
  return /** @type {string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.LexiconProto.Pronunciation} returns this
 */
proto.speech.tts.LexiconProto.Pronunciation.prototype.setId = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.LexiconProto.Pronunciation} returns this
 */
proto.speech.tts.LexiconProto.Pronunciation.prototype.clearId = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.LexiconProto.Pronunciation.prototype.hasId = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * required string v = 2;
 * @return {string}
 */
proto.speech.tts.LexiconProto.Pronunciation.prototype.getV = function() {
  return /** @type {string} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.LexiconProto.Pronunciation} returns this
 */
proto.speech.tts.LexiconProto.Pronunciation.prototype.setV = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.LexiconProto.Pronunciation} returns this
 */
proto.speech.tts.LexiconProto.Pronunciation.prototype.clearV = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.LexiconProto.Pronunciation.prototype.hasV = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional PronunciationType t = 3;
 * @return {!proto.speech.tts.LexiconProto.Pronunciation.PronunciationType}
 */
proto.speech.tts.LexiconProto.Pronunciation.prototype.getT = function() {
  return /** @type {!proto.speech.tts.LexiconProto.Pronunciation.PronunciationType} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?proto.speech.tts.LexiconProto.Pronunciation.PronunciationType|undefined} value
 * @return {!proto.speech.tts.LexiconProto.Pronunciation} returns this
 */
proto.speech.tts.LexiconProto.Pronunciation.prototype.setT = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.LexiconProto.Pronunciation} returns this
 */
proto.speech.tts.LexiconProto.Pronunciation.prototype.clearT = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.LexiconProto.Pronunciation.prototype.hasT = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.LexiconProto.Pronunciation}
 */
proto.speech.tts.LexiconProto.Pronunciation.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.LexiconProto.Pronunciation, data);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.LexiconProto.Spelling.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.LexiconProto.Spelling.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.LexiconProto.Spelling} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.LexiconProto.Spelling.toObject = function(includeInstance, msg) {
  var f, obj = {
    id: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    v: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Spelling as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.LexiconProto.Spelling.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.id;

  /** @type {?string|undefined} */
  this.v;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.LexiconProto.Spelling.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.LexiconProto.Spelling}
 */
proto.speech.tts.LexiconProto.Spelling.fromObject = function(obj) {
  var msg = new proto.speech.tts.LexiconProto.Spelling();
  obj.id != null && jspb.Message.setField(msg, 1, obj.id);
  obj.v != null && jspb.Message.setField(msg, 2, obj.v);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.LexiconProto.Spelling}
 */
proto.speech.tts.LexiconProto.Spelling.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.LexiconProto.Spelling;
  return proto.speech.tts.LexiconProto.Spelling.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.LexiconProto.Spelling} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.LexiconProto.Spelling}
 */
proto.speech.tts.LexiconProto.Spelling.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setId(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setV(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.LexiconProto.Spelling.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.LexiconProto.Spelling.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.LexiconProto.Spelling} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.LexiconProto.Spelling.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * required string id = 1;
 * @return {string}
 */
proto.speech.tts.LexiconProto.Spelling.prototype.getId = function() {
  return /** @type {string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.LexiconProto.Spelling} returns this
 */
proto.speech.tts.LexiconProto.Spelling.prototype.setId = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.LexiconProto.Spelling} returns this
 */
proto.speech.tts.LexiconProto.Spelling.prototype.clearId = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.LexiconProto.Spelling.prototype.hasId = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * required string v = 2;
 * @return {string}
 */
proto.speech.tts.LexiconProto.Spelling.prototype.getV = function() {
  return /** @type {string} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.LexiconProto.Spelling} returns this
 */
proto.speech.tts.LexiconProto.Spelling.prototype.setV = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.LexiconProto.Spelling} returns this
 */
proto.speech.tts.LexiconProto.Spelling.prototype.clearV = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.LexiconProto.Spelling.prototype.hasV = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.LexiconProto.Spelling}
 */
proto.speech.tts.LexiconProto.Spelling.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.LexiconProto.Spelling, data);
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.LexiconProto.ExpandedPronunciation.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.LexiconProto.ExpandedPronunciation.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.LexiconProto.ExpandedPronunciation.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.LexiconProto.ExpandedPronunciation} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.LexiconProto.ExpandedPronunciation.toObject = function(includeInstance, msg) {
  var f, obj = {
    syllableList: jspb.Message.toObjectList(msg.getSyllableList(),
    proto.speech.tts.LexiconProto.Syllable.toObject, includeInstance)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of ExpandedPronunciation as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.LexiconProto.ExpandedPronunciation.ObjectFormat = function() {
  /** @type {?Array<!proto.speech.tts.LexiconProto.Syllable.ObjectFormat>|undefined} */
  this.syllableList;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.LexiconProto.ExpandedPronunciation.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.LexiconProto.ExpandedPronunciation}
 */
proto.speech.tts.LexiconProto.ExpandedPronunciation.fromObject = function(obj) {
  var msg = new proto.speech.tts.LexiconProto.ExpandedPronunciation();
  obj.syllableList && jspb.Message.setRepeatedWrapperField(
      msg, 1, obj.syllableList.map(
          proto.speech.tts.LexiconProto.Syllable.fromObject));
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.LexiconProto.ExpandedPronunciation}
 */
proto.speech.tts.LexiconProto.ExpandedPronunciation.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.LexiconProto.ExpandedPronunciation;
  return proto.speech.tts.LexiconProto.ExpandedPronunciation.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.LexiconProto.ExpandedPronunciation} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.LexiconProto.ExpandedPronunciation}
 */
proto.speech.tts.LexiconProto.ExpandedPronunciation.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.speech.tts.LexiconProto.Syllable;
      reader.readMessage(value,proto.speech.tts.LexiconProto.Syllable.deserializeBinaryFromReader);
      msg.addSyllable(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.LexiconProto.ExpandedPronunciation.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.LexiconProto.ExpandedPronunciation.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.LexiconProto.ExpandedPronunciation} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.LexiconProto.ExpandedPronunciation.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getSyllableList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      1,
      f,
      proto.speech.tts.LexiconProto.Syllable.serializeBinaryToWriter
    );
  }
};


/**
 * repeated Syllable syllable = 1;
 * @return {!Array<!proto.speech.tts.LexiconProto.Syllable>}
 */
proto.speech.tts.LexiconProto.ExpandedPronunciation.prototype.getSyllableList = function() {
  return /** @type{!Array<!proto.speech.tts.LexiconProto.Syllable>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.LexiconProto.Syllable, 1));
};


/**
 * @param {Array<!proto.speech.tts.LexiconProto.Syllable>|undefined} value
 * @return {!proto.speech.tts.LexiconProto.ExpandedPronunciation} returns this
*/
proto.speech.tts.LexiconProto.ExpandedPronunciation.prototype.setSyllableList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 1, value);
};


/**
 * @param {!proto.speech.tts.LexiconProto.Syllable=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.LexiconProto.Syllable}
 */
proto.speech.tts.LexiconProto.ExpandedPronunciation.prototype.addSyllable = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.speech.tts.LexiconProto.Syllable, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.LexiconProto.ExpandedPronunciation} returns this
 */
proto.speech.tts.LexiconProto.ExpandedPronunciation.prototype.clearSyllableList = function() {
  return this.setSyllableList([]);
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.LexiconProto.ExpandedPronunciation}
 */
proto.speech.tts.LexiconProto.ExpandedPronunciation.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.LexiconProto.ExpandedPronunciation, data);
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.LexiconProto.Syllable.repeatedFields_ = [3,4];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.LexiconProto.Syllable.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.LexiconProto.Syllable.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.LexiconProto.Syllable} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.LexiconProto.Syllable.toObject = function(includeInstance, msg) {
  var f, obj = {
    stress: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    vowel: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    onsetList: (f = jspb.Message.getRepeatedField(msg, 3)) == null ? undefined : f,
    codaList: (f = jspb.Message.getRepeatedField(msg, 4)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Syllable as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.LexiconProto.Syllable.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.stress;

  /** @type {?string|undefined} */
  this.vowel;

  /** @type {?Array<string>|undefined} */
  this.onsetList;

  /** @type {?Array<string>|undefined} */
  this.codaList;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.LexiconProto.Syllable.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.LexiconProto.Syllable}
 */
proto.speech.tts.LexiconProto.Syllable.fromObject = function(obj) {
  var msg = new proto.speech.tts.LexiconProto.Syllable();
  obj.stress != null && jspb.Message.setField(msg, 1, obj.stress);
  obj.vowel != null && jspb.Message.setField(msg, 2, obj.vowel);
  obj.onsetList != null && jspb.Message.setField(msg, 3, obj.onsetList);
  obj.codaList != null && jspb.Message.setField(msg, 4, obj.codaList);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.LexiconProto.Syllable}
 */
proto.speech.tts.LexiconProto.Syllable.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.LexiconProto.Syllable;
  return proto.speech.tts.LexiconProto.Syllable.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.LexiconProto.Syllable} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.LexiconProto.Syllable}
 */
proto.speech.tts.LexiconProto.Syllable.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setStress(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setVowel(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.addOnset(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.addCoda(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.LexiconProto.Syllable.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.LexiconProto.Syllable.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.LexiconProto.Syllable} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.LexiconProto.Syllable.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {number} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeInt32(
      1,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
  f = message.getOnsetList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      3,
      f
    );
  }
  f = message.getCodaList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      4,
      f
    );
  }
};


/**
 * required int32 stress = 1;
 * @return {number}
 */
proto.speech.tts.LexiconProto.Syllable.prototype.getStress = function() {
  return /** @type {number} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.LexiconProto.Syllable} returns this
 */
proto.speech.tts.LexiconProto.Syllable.prototype.setStress = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.LexiconProto.Syllable} returns this
 */
proto.speech.tts.LexiconProto.Syllable.prototype.clearStress = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.LexiconProto.Syllable.prototype.hasStress = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * required string vowel = 2;
 * @return {string}
 */
proto.speech.tts.LexiconProto.Syllable.prototype.getVowel = function() {
  return /** @type {string} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.LexiconProto.Syllable} returns this
 */
proto.speech.tts.LexiconProto.Syllable.prototype.setVowel = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.LexiconProto.Syllable} returns this
 */
proto.speech.tts.LexiconProto.Syllable.prototype.clearVowel = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.LexiconProto.Syllable.prototype.hasVowel = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * repeated string onset = 3;
 * @return {!Array<string>}
 */
proto.speech.tts.LexiconProto.Syllable.prototype.getOnsetList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 3));
};


/**
 * @param {Array<string>|undefined} value
 * @return {!proto.speech.tts.LexiconProto.Syllable} returns this
 */
proto.speech.tts.LexiconProto.Syllable.prototype.setOnsetList = function(value) {
  return jspb.Message.setField(this, 3, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.LexiconProto.Syllable} returns this
 */
proto.speech.tts.LexiconProto.Syllable.prototype.addOnset = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 3, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.LexiconProto.Syllable} returns this
 */
proto.speech.tts.LexiconProto.Syllable.prototype.clearOnsetList = function() {
  return this.setOnsetList([]);
};


/**
 * repeated string coda = 4;
 * @return {!Array<string>}
 */
proto.speech.tts.LexiconProto.Syllable.prototype.getCodaList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 4));
};


/**
 * @param {Array<string>|undefined} value
 * @return {!proto.speech.tts.LexiconProto.Syllable} returns this
 */
proto.speech.tts.LexiconProto.Syllable.prototype.setCodaList = function(value) {
  return jspb.Message.setField(this, 4, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.LexiconProto.Syllable} returns this
 */
proto.speech.tts.LexiconProto.Syllable.prototype.addCoda = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 4, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.LexiconProto.Syllable} returns this
 */
proto.speech.tts.LexiconProto.Syllable.prototype.clearCodaList = function() {
  return this.setCodaList([]);
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.LexiconProto.Syllable}
 */
proto.speech.tts.LexiconProto.Syllable.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.LexiconProto.Syllable, data);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.LexiconProto.Morphology.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.LexiconProto.Morphology.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.LexiconProto.Morphology} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.LexiconProto.Morphology.toObject = function(includeInstance, msg) {
  var f, obj = {
    pos: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    gender: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    animacy: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,
    number: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Morphology as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.LexiconProto.Morphology.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.pos;

  /** @type {?number|undefined} */
  this.gender;

  /** @type {?number|undefined} */
  this.animacy;

  /** @type {?number|undefined} */
  this.number;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.LexiconProto.Morphology.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.LexiconProto.Morphology}
 */
proto.speech.tts.LexiconProto.Morphology.fromObject = function(obj) {
  var msg = new proto.speech.tts.LexiconProto.Morphology();
  obj.pos != null && jspb.Message.setField(msg, 1, obj.pos);
  obj.gender != null && jspb.Message.setField(msg, 2, obj.gender);
  obj.animacy != null && jspb.Message.setField(msg, 3, obj.animacy);
  obj.number != null && jspb.Message.setField(msg, 4, obj.number);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.LexiconProto.Morphology}
 */
proto.speech.tts.LexiconProto.Morphology.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.LexiconProto.Morphology;
  return proto.speech.tts.LexiconProto.Morphology.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.LexiconProto.Morphology} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.LexiconProto.Morphology}
 */
proto.speech.tts.LexiconProto.Morphology.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {!proto.speech.tts.LexiconProto.Morphology.PartOfSpeech} */ (reader.readEnum());
      msg.setPos(value);
      break;
    case 2:
      var value = /** @type {!proto.speech.tts.LexiconProto.Morphology.Gender} */ (reader.readEnum());
      msg.setGender(value);
      break;
    case 3:
      var value = /** @type {!proto.speech.tts.LexiconProto.Morphology.Animacy} */ (reader.readEnum());
      msg.setAnimacy(value);
      break;
    case 4:
      var value = /** @type {!proto.speech.tts.LexiconProto.Morphology.Number} */ (reader.readEnum());
      msg.setNumber(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.LexiconProto.Morphology.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.LexiconProto.Morphology.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.LexiconProto.Morphology} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.LexiconProto.Morphology.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {!proto.speech.tts.LexiconProto.Morphology.PartOfSpeech} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeEnum(
      1,
      f
    );
  }
  f = /** @type {!proto.speech.tts.LexiconProto.Morphology.Gender} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeEnum(
      2,
      f
    );
  }
  f = /** @type {!proto.speech.tts.LexiconProto.Morphology.Animacy} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeEnum(
      3,
      f
    );
  }
  f = /** @type {!proto.speech.tts.LexiconProto.Morphology.Number} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeEnum(
      4,
      f
    );
  }
};


/**
 * @enum {number}
 */
proto.speech.tts.LexiconProto.Morphology.PartOfSpeech = {
  UNKNOWN_POS: 0,
  NOUN: 1,
  PRONOUN: 2,
  ADJECTIVE: 3,
  VERB: 4,
  ADVERB: 5,
  PREPOSITION: 6,
  CONJUNCTION: 7,
  INTERJECTION: 8
};

/**
 * @enum {number}
 */
proto.speech.tts.LexiconProto.Morphology.Gender = {
  UNKNOWN_GENDER: 0,
  NEU: 1,
  MAS: 2,
  FEM: 3
};

/**
 * @enum {number}
 */
proto.speech.tts.LexiconProto.Morphology.Animacy = {
  UNKNOWN_ANIMACY: 0,
  ANIMATE: 1,
  INANIMATE: 2
};

/**
 * @enum {number}
 */
proto.speech.tts.LexiconProto.Morphology.Number = {
  UNKNOWN_NUMBER: 0,
  SINGLE: 1,
  DUAL: 2,
  PLU: 3
};

/**
 * optional PartOfSpeech pos = 1;
 * @return {!proto.speech.tts.LexiconProto.Morphology.PartOfSpeech}
 */
proto.speech.tts.LexiconProto.Morphology.prototype.getPos = function() {
  return /** @type {!proto.speech.tts.LexiconProto.Morphology.PartOfSpeech} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?proto.speech.tts.LexiconProto.Morphology.PartOfSpeech|undefined} value
 * @return {!proto.speech.tts.LexiconProto.Morphology} returns this
 */
proto.speech.tts.LexiconProto.Morphology.prototype.setPos = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.LexiconProto.Morphology} returns this
 */
proto.speech.tts.LexiconProto.Morphology.prototype.clearPos = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.LexiconProto.Morphology.prototype.hasPos = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional Gender gender = 2;
 * @return {!proto.speech.tts.LexiconProto.Morphology.Gender}
 */
proto.speech.tts.LexiconProto.Morphology.prototype.getGender = function() {
  return /** @type {!proto.speech.tts.LexiconProto.Morphology.Gender} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?proto.speech.tts.LexiconProto.Morphology.Gender|undefined} value
 * @return {!proto.speech.tts.LexiconProto.Morphology} returns this
 */
proto.speech.tts.LexiconProto.Morphology.prototype.setGender = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.LexiconProto.Morphology} returns this
 */
proto.speech.tts.LexiconProto.Morphology.prototype.clearGender = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.LexiconProto.Morphology.prototype.hasGender = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional Animacy animacy = 3;
 * @return {!proto.speech.tts.LexiconProto.Morphology.Animacy}
 */
proto.speech.tts.LexiconProto.Morphology.prototype.getAnimacy = function() {
  return /** @type {!proto.speech.tts.LexiconProto.Morphology.Animacy} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?proto.speech.tts.LexiconProto.Morphology.Animacy|undefined} value
 * @return {!proto.speech.tts.LexiconProto.Morphology} returns this
 */
proto.speech.tts.LexiconProto.Morphology.prototype.setAnimacy = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.LexiconProto.Morphology} returns this
 */
proto.speech.tts.LexiconProto.Morphology.prototype.clearAnimacy = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.LexiconProto.Morphology.prototype.hasAnimacy = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional Number number = 4;
 * @return {!proto.speech.tts.LexiconProto.Morphology.Number}
 */
proto.speech.tts.LexiconProto.Morphology.prototype.getNumber = function() {
  return /** @type {!proto.speech.tts.LexiconProto.Morphology.Number} */ (jspb.Message.getField(this, 4));
};


/**
 * @param {?proto.speech.tts.LexiconProto.Morphology.Number|undefined} value
 * @return {!proto.speech.tts.LexiconProto.Morphology} returns this
 */
proto.speech.tts.LexiconProto.Morphology.prototype.setNumber = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.LexiconProto.Morphology} returns this
 */
proto.speech.tts.LexiconProto.Morphology.prototype.clearNumber = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.LexiconProto.Morphology.prototype.hasNumber = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.LexiconProto.Morphology}
 */
proto.speech.tts.LexiconProto.Morphology.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.LexiconProto.Morphology, data);
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.LexiconProto.Entry.repeatedFields_ = [2,3,7];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.LexiconProto.Entry.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.LexiconProto.Entry.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.LexiconProto.Entry} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.LexiconProto.Entry.toObject = function(includeInstance, msg) {
  var f, obj = {
    id: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    pList: jspb.Message.toObjectList(msg.getPList(),
    proto.speech.tts.LexiconProto.Pronunciation.toObject, includeInstance),
    sList: jspb.Message.toObjectList(msg.getSList(),
    proto.speech.tts.LexiconProto.Spelling.toObject, includeInstance),
    c: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f,
    syn: jspb.Message.getFieldWithDefault(msg, 5, 0),
    m: (f = msg.getM()) && proto.speech.tts.LexiconProto.Morphology.toObject(includeInstance, f),
    altList: (f = jspb.Message.getRepeatedField(msg, 7)) == null ? undefined : f,
    obs: (f = jspb.Message.getBooleanField(msg, 8)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Entry as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.LexiconProto.Entry.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.id;

  /** @type {?Array<!proto.speech.tts.LexiconProto.Pronunciation.ObjectFormat>|undefined} */
  this.pList;

  /** @type {?Array<!proto.speech.tts.LexiconProto.Spelling.ObjectFormat>|undefined} */
  this.sList;

  /** @type {?string|undefined} */
  this.c;

  /** @type {?number|undefined} */
  this.syn;

  /** @type {?proto.speech.tts.LexiconProto.Morphology.ObjectFormat|undefined} */
  this.m;

  /** @type {?Array<string>|undefined} */
  this.altList;

  /** @type {?boolean|undefined} */
  this.obs;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.LexiconProto.Entry.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.LexiconProto.Entry}
 */
proto.speech.tts.LexiconProto.Entry.fromObject = function(obj) {
  var msg = new proto.speech.tts.LexiconProto.Entry();
  obj.id != null && jspb.Message.setField(msg, 1, obj.id);
  obj.pList && jspb.Message.setRepeatedWrapperField(
      msg, 2, obj.pList.map(
          proto.speech.tts.LexiconProto.Pronunciation.fromObject));
  obj.sList && jspb.Message.setRepeatedWrapperField(
      msg, 3, obj.sList.map(
          proto.speech.tts.LexiconProto.Spelling.fromObject));
  obj.c != null && jspb.Message.setField(msg, 4, obj.c);
  obj.syn != null && jspb.Message.setField(msg, 5, obj.syn);
  obj.m && jspb.Message.setWrapperField(
      msg, 6, proto.speech.tts.LexiconProto.Morphology.fromObject(obj.m));
  obj.altList != null && jspb.Message.setField(msg, 7, obj.altList);
  obj.obs != null && jspb.Message.setField(msg, 8, obj.obs);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.LexiconProto.Entry}
 */
proto.speech.tts.LexiconProto.Entry.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.LexiconProto.Entry;
  return proto.speech.tts.LexiconProto.Entry.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.LexiconProto.Entry} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.LexiconProto.Entry}
 */
proto.speech.tts.LexiconProto.Entry.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setId(value);
      break;
    case 2:
      var value = new proto.speech.tts.LexiconProto.Pronunciation;
      reader.readMessage(value,proto.speech.tts.LexiconProto.Pronunciation.deserializeBinaryFromReader);
      msg.addP(value);
      break;
    case 3:
      var value = new proto.speech.tts.LexiconProto.Spelling;
      reader.readMessage(value,proto.speech.tts.LexiconProto.Spelling.deserializeBinaryFromReader);
      msg.addS(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setC(value);
      break;
    case 5:
      var value = /** @type {!proto.speech.tts.LexiconProto.Entry.Syntax} */ (reader.readEnum());
      msg.setSyn(value);
      break;
    case 6:
      var value = new proto.speech.tts.LexiconProto.Morphology;
      reader.readMessage(value,proto.speech.tts.LexiconProto.Morphology.deserializeBinaryFromReader);
      msg.setM(value);
      break;
    case 7:
      var value = /** @type {string} */ (reader.readString());
      msg.addAlt(value);
      break;
    case 8:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setObs(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.LexiconProto.Entry.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.LexiconProto.Entry.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.LexiconProto.Entry} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.LexiconProto.Entry.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = message.getPList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      2,
      f,
      proto.speech.tts.LexiconProto.Pronunciation.serializeBinaryToWriter
    );
  }
  f = message.getSList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      3,
      f,
      proto.speech.tts.LexiconProto.Spelling.serializeBinaryToWriter
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeString(
      4,
      f
    );
  }
  f = /** @type {!proto.speech.tts.LexiconProto.Entry.Syntax} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeEnum(
      5,
      f
    );
  }
  f = message.getM();
  if (f != null) {
    writer.writeMessage(
      6,
      f,
      proto.speech.tts.LexiconProto.Morphology.serializeBinaryToWriter
    );
  }
  f = message.getAltList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      7,
      f
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 8));
  if (f != null) {
    writer.writeBool(
      8,
      f
    );
  }
};


/**
 * @enum {number}
 */
proto.speech.tts.LexiconProto.Entry.Syntax = {
  UNSPECIFIED: 0,
  FUNCTION: 1,
  CONTENT: 2,
  NAME: 3,
  PLACE: 4,
  FOREIGN: 5,
  FIRST_NAME: 6,
  LAST_NAME: 7,
  FUNCTION_NR: 8,
  FUNCTION_AR: 9,
  FOREIGN_FUNCTION_AR: 10,
  SILENCE: 11,
  SPELLING_EXCEPTION: 12,
  LSEQ_EXCEPTION: 13,
  LETTER: 14,
  FOREIGN_LETTER: 15,
  NON_PHONOTACTIC: 16
};

/**
 * required string id = 1;
 * @return {string}
 */
proto.speech.tts.LexiconProto.Entry.prototype.getId = function() {
  return /** @type {string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.LexiconProto.Entry} returns this
 */
proto.speech.tts.LexiconProto.Entry.prototype.setId = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.LexiconProto.Entry} returns this
 */
proto.speech.tts.LexiconProto.Entry.prototype.clearId = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.LexiconProto.Entry.prototype.hasId = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * repeated Pronunciation p = 2;
 * @return {!Array<!proto.speech.tts.LexiconProto.Pronunciation>}
 */
proto.speech.tts.LexiconProto.Entry.prototype.getPList = function() {
  return /** @type{!Array<!proto.speech.tts.LexiconProto.Pronunciation>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.LexiconProto.Pronunciation, 2));
};


/**
 * @param {Array<!proto.speech.tts.LexiconProto.Pronunciation>|undefined} value
 * @return {!proto.speech.tts.LexiconProto.Entry} returns this
*/
proto.speech.tts.LexiconProto.Entry.prototype.setPList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 2, value);
};


/**
 * @param {!proto.speech.tts.LexiconProto.Pronunciation=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.LexiconProto.Pronunciation}
 */
proto.speech.tts.LexiconProto.Entry.prototype.addP = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 2, opt_value, proto.speech.tts.LexiconProto.Pronunciation, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.LexiconProto.Entry} returns this
 */
proto.speech.tts.LexiconProto.Entry.prototype.clearPList = function() {
  return this.setPList([]);
};


/**
 * repeated Spelling s = 3;
 * @return {!Array<!proto.speech.tts.LexiconProto.Spelling>}
 */
proto.speech.tts.LexiconProto.Entry.prototype.getSList = function() {
  return /** @type{!Array<!proto.speech.tts.LexiconProto.Spelling>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.LexiconProto.Spelling, 3));
};


/**
 * @param {Array<!proto.speech.tts.LexiconProto.Spelling>|undefined} value
 * @return {!proto.speech.tts.LexiconProto.Entry} returns this
*/
proto.speech.tts.LexiconProto.Entry.prototype.setSList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 3, value);
};


/**
 * @param {!proto.speech.tts.LexiconProto.Spelling=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.LexiconProto.Spelling}
 */
proto.speech.tts.LexiconProto.Entry.prototype.addS = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 3, opt_value, proto.speech.tts.LexiconProto.Spelling, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.LexiconProto.Entry} returns this
 */
proto.speech.tts.LexiconProto.Entry.prototype.clearSList = function() {
  return this.setSList([]);
};


/**
 * required string c = 4;
 * @return {string}
 */
proto.speech.tts.LexiconProto.Entry.prototype.getC = function() {
  return /** @type {string} */ (jspb.Message.getField(this, 4));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.LexiconProto.Entry} returns this
 */
proto.speech.tts.LexiconProto.Entry.prototype.setC = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.LexiconProto.Entry} returns this
 */
proto.speech.tts.LexiconProto.Entry.prototype.clearC = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.LexiconProto.Entry.prototype.hasC = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional Syntax syn = 5;
 * @return {!proto.speech.tts.LexiconProto.Entry.Syntax}
 */
proto.speech.tts.LexiconProto.Entry.prototype.getSyn = function() {
  return /** @type {!proto.speech.tts.LexiconProto.Entry.Syntax} */ (jspb.Message.getFieldWithDefault(this, 5, 0));
};


/**
 * @param {?proto.speech.tts.LexiconProto.Entry.Syntax|undefined} value
 * @return {!proto.speech.tts.LexiconProto.Entry} returns this
 */
proto.speech.tts.LexiconProto.Entry.prototype.setSyn = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.LexiconProto.Entry} returns this
 */
proto.speech.tts.LexiconProto.Entry.prototype.clearSyn = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.LexiconProto.Entry.prototype.hasSyn = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional Morphology m = 6;
 * @return {?proto.speech.tts.LexiconProto.Morphology}
 */
proto.speech.tts.LexiconProto.Entry.prototype.getM = function() {
  return /** @type{?proto.speech.tts.LexiconProto.Morphology} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.LexiconProto.Morphology, 6));
};


/**
 * @param {?proto.speech.tts.LexiconProto.Morphology|undefined} value
 * @return {!proto.speech.tts.LexiconProto.Entry} returns this
*/
proto.speech.tts.LexiconProto.Entry.prototype.setM = function(value) {
  return jspb.Message.setWrapperField(this, 6, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.LexiconProto.Entry} returns this
 */
proto.speech.tts.LexiconProto.Entry.prototype.clearM = function() {
  return this.setM(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.LexiconProto.Entry.prototype.hasM = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * repeated string alt = 7;
 * @return {!Array<string>}
 */
proto.speech.tts.LexiconProto.Entry.prototype.getAltList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 7));
};


/**
 * @param {Array<string>|undefined} value
 * @return {!proto.speech.tts.LexiconProto.Entry} returns this
 */
proto.speech.tts.LexiconProto.Entry.prototype.setAltList = function(value) {
  return jspb.Message.setField(this, 7, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.LexiconProto.Entry} returns this
 */
proto.speech.tts.LexiconProto.Entry.prototype.addAlt = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 7, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.LexiconProto.Entry} returns this
 */
proto.speech.tts.LexiconProto.Entry.prototype.clearAltList = function() {
  return this.setAltList([]);
};


/**
 * optional bool obs = 8;
 * @return {?boolean}
 */
proto.speech.tts.LexiconProto.Entry.prototype.getObs = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 8));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.LexiconProto.Entry} returns this
 */
proto.speech.tts.LexiconProto.Entry.prototype.setObs = function(value) {
  return jspb.Message.setField(this, 8, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.LexiconProto.Entry} returns this
 */
proto.speech.tts.LexiconProto.Entry.prototype.clearObs = function() {
  return jspb.Message.setField(this, 8, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.LexiconProto.Entry.prototype.hasObs = function() {
  return jspb.Message.getField(this, 8) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.LexiconProto.Entry}
 */
proto.speech.tts.LexiconProto.Entry.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.LexiconProto.Entry, data);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.LexiconProto.CrossLingualInfo.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.LexiconProto.CrossLingualInfo.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.LexiconProto.CrossLingualInfo} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.LexiconProto.CrossLingualInfo.toObject = function(includeInstance, msg) {
  var f, obj = {
    foreignLanguage: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    foreignRegion: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    nativeLanguage: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,
    nativeRegion: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of CrossLingualInfo as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.LexiconProto.CrossLingualInfo.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.foreignLanguage;

  /** @type {?string|undefined} */
  this.foreignRegion;

  /** @type {?string|undefined} */
  this.nativeLanguage;

  /** @type {?string|undefined} */
  this.nativeRegion;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.LexiconProto.CrossLingualInfo.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.LexiconProto.CrossLingualInfo}
 */
proto.speech.tts.LexiconProto.CrossLingualInfo.fromObject = function(obj) {
  var msg = new proto.speech.tts.LexiconProto.CrossLingualInfo();
  obj.foreignLanguage != null && jspb.Message.setField(msg, 1, obj.foreignLanguage);
  obj.foreignRegion != null && jspb.Message.setField(msg, 2, obj.foreignRegion);
  obj.nativeLanguage != null && jspb.Message.setField(msg, 3, obj.nativeLanguage);
  obj.nativeRegion != null && jspb.Message.setField(msg, 4, obj.nativeRegion);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.LexiconProto.CrossLingualInfo}
 */
proto.speech.tts.LexiconProto.CrossLingualInfo.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.LexiconProto.CrossLingualInfo;
  return proto.speech.tts.LexiconProto.CrossLingualInfo.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.LexiconProto.CrossLingualInfo} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.LexiconProto.CrossLingualInfo}
 */
proto.speech.tts.LexiconProto.CrossLingualInfo.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setForeignLanguage(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setForeignRegion(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setNativeLanguage(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setNativeRegion(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.LexiconProto.CrossLingualInfo.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.LexiconProto.CrossLingualInfo.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.LexiconProto.CrossLingualInfo} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.LexiconProto.CrossLingualInfo.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeString(
      3,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeString(
      4,
      f
    );
  }
};


/**
 * optional string foreign_language = 1;
 * @return {?string}
 */
proto.speech.tts.LexiconProto.CrossLingualInfo.prototype.getForeignLanguage = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.LexiconProto.CrossLingualInfo} returns this
 */
proto.speech.tts.LexiconProto.CrossLingualInfo.prototype.setForeignLanguage = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.LexiconProto.CrossLingualInfo} returns this
 */
proto.speech.tts.LexiconProto.CrossLingualInfo.prototype.clearForeignLanguage = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.LexiconProto.CrossLingualInfo.prototype.hasForeignLanguage = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional string foreign_region = 2;
 * @return {?string}
 */
proto.speech.tts.LexiconProto.CrossLingualInfo.prototype.getForeignRegion = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.LexiconProto.CrossLingualInfo} returns this
 */
proto.speech.tts.LexiconProto.CrossLingualInfo.prototype.setForeignRegion = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.LexiconProto.CrossLingualInfo} returns this
 */
proto.speech.tts.LexiconProto.CrossLingualInfo.prototype.clearForeignRegion = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.LexiconProto.CrossLingualInfo.prototype.hasForeignRegion = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional string native_language = 3;
 * @return {?string}
 */
proto.speech.tts.LexiconProto.CrossLingualInfo.prototype.getNativeLanguage = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.LexiconProto.CrossLingualInfo} returns this
 */
proto.speech.tts.LexiconProto.CrossLingualInfo.prototype.setNativeLanguage = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.LexiconProto.CrossLingualInfo} returns this
 */
proto.speech.tts.LexiconProto.CrossLingualInfo.prototype.clearNativeLanguage = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.LexiconProto.CrossLingualInfo.prototype.hasNativeLanguage = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional string native_region = 4;
 * @return {?string}
 */
proto.speech.tts.LexiconProto.CrossLingualInfo.prototype.getNativeRegion = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 4));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.LexiconProto.CrossLingualInfo} returns this
 */
proto.speech.tts.LexiconProto.CrossLingualInfo.prototype.setNativeRegion = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.LexiconProto.CrossLingualInfo} returns this
 */
proto.speech.tts.LexiconProto.CrossLingualInfo.prototype.clearNativeRegion = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.LexiconProto.CrossLingualInfo.prototype.hasNativeRegion = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.LexiconProto.CrossLingualInfo}
 */
proto.speech.tts.LexiconProto.CrossLingualInfo.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.LexiconProto.CrossLingualInfo, data);
};


/**
 * required string header = 1;
 * @return {string}
 */
proto.speech.tts.LexiconProto.prototype.getHeader = function() {
  return /** @type {string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.LexiconProto} returns this
 */
proto.speech.tts.LexiconProto.prototype.setHeader = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.LexiconProto} returns this
 */
proto.speech.tts.LexiconProto.prototype.clearHeader = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.LexiconProto.prototype.hasHeader = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * repeated Entry w = 2;
 * @return {!Array<!proto.speech.tts.LexiconProto.Entry>}
 */
proto.speech.tts.LexiconProto.prototype.getWList = function() {
  return /** @type{!Array<!proto.speech.tts.LexiconProto.Entry>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.LexiconProto.Entry, 2));
};


/**
 * @param {Array<!proto.speech.tts.LexiconProto.Entry>|undefined} value
 * @return {!proto.speech.tts.LexiconProto} returns this
*/
proto.speech.tts.LexiconProto.prototype.setWList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 2, value);
};


/**
 * @param {!proto.speech.tts.LexiconProto.Entry=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.LexiconProto.Entry}
 */
proto.speech.tts.LexiconProto.prototype.addW = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 2, opt_value, proto.speech.tts.LexiconProto.Entry, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.LexiconProto} returns this
 */
proto.speech.tts.LexiconProto.prototype.clearWList = function() {
  return this.setWList([]);
};


/**
 * optional string legal_vowels = 3;
 * @return {?string}
 */
proto.speech.tts.LexiconProto.prototype.getLegalVowels = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.LexiconProto} returns this
 */
proto.speech.tts.LexiconProto.prototype.setLegalVowels = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.LexiconProto} returns this
 */
proto.speech.tts.LexiconProto.prototype.clearLegalVowels = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.LexiconProto.prototype.hasLegalVowels = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional CrossLingualInfo cross_lingual = 4;
 * @return {?proto.speech.tts.LexiconProto.CrossLingualInfo}
 */
proto.speech.tts.LexiconProto.prototype.getCrossLingual = function() {
  return /** @type{?proto.speech.tts.LexiconProto.CrossLingualInfo} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.LexiconProto.CrossLingualInfo, 4));
};


/**
 * @param {?proto.speech.tts.LexiconProto.CrossLingualInfo|undefined} value
 * @return {!proto.speech.tts.LexiconProto} returns this
*/
proto.speech.tts.LexiconProto.prototype.setCrossLingual = function(value) {
  return jspb.Message.setWrapperField(this, 4, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.LexiconProto} returns this
 */
proto.speech.tts.LexiconProto.prototype.clearCrossLingual = function() {
  return this.setCrossLingual(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.LexiconProto.prototype.hasCrossLingual = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional bool is_tsampa = 5;
 * @return {?boolean}
 */
proto.speech.tts.LexiconProto.prototype.getIsTsampa = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 5));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.LexiconProto} returns this
 */
proto.speech.tts.LexiconProto.prototype.setIsTsampa = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.LexiconProto} returns this
 */
proto.speech.tts.LexiconProto.prototype.clearIsTsampa = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.LexiconProto.prototype.hasIsTsampa = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.LexiconProto}
 */
proto.speech.tts.LexiconProto.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.LexiconProto, data);
};



//speech/tts/public/tts_markup_proto_jspb/partialsentenceprosody.jspb.js
// source: speech/tts/public/tts_markup.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.PartialSentenceProsody');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.PartialSentenceProsody = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.PartialSentenceProsody, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.PartialSentenceProsody.displayName = 'proto.speech.tts.PartialSentenceProsody';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.PartialSentenceProsody.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.PartialSentenceProsody.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.PartialSentenceProsody} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.PartialSentenceProsody.toObject = function(includeInstance, msg) {
  var f, obj = {
    incompleteAtStart: (f = jspb.Message.getBooleanField(msg, 1)) == null ? undefined : f,
    incompleteAtEnd: (f = jspb.Message.getBooleanField(msg, 2)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of PartialSentenceProsody as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.PartialSentenceProsody.ObjectFormat = function() {
  /** @type {?boolean|undefined} */
  this.incompleteAtStart;

  /** @type {?boolean|undefined} */
  this.incompleteAtEnd;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.PartialSentenceProsody.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.PartialSentenceProsody}
 */
proto.speech.tts.PartialSentenceProsody.fromObject = function(obj) {
  var msg = new proto.speech.tts.PartialSentenceProsody();
  obj.incompleteAtStart != null && jspb.Message.setField(msg, 1, obj.incompleteAtStart);
  obj.incompleteAtEnd != null && jspb.Message.setField(msg, 2, obj.incompleteAtEnd);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.PartialSentenceProsody}
 */
proto.speech.tts.PartialSentenceProsody.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.PartialSentenceProsody;
  return proto.speech.tts.PartialSentenceProsody.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.PartialSentenceProsody} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.PartialSentenceProsody}
 */
proto.speech.tts.PartialSentenceProsody.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIncompleteAtStart(value);
      break;
    case 2:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIncompleteAtEnd(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.PartialSentenceProsody.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.PartialSentenceProsody.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.PartialSentenceProsody} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.PartialSentenceProsody.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeBool(
      1,
      f
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeBool(
      2,
      f
    );
  }
};


/**
 * optional bool incomplete_at_start = 1;
 * @return {?boolean}
 */
proto.speech.tts.PartialSentenceProsody.prototype.getIncompleteAtStart = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 1));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.PartialSentenceProsody} returns this
 */
proto.speech.tts.PartialSentenceProsody.prototype.setIncompleteAtStart = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.PartialSentenceProsody} returns this
 */
proto.speech.tts.PartialSentenceProsody.prototype.clearIncompleteAtStart = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.PartialSentenceProsody.prototype.hasIncompleteAtStart = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional bool incomplete_at_end = 2;
 * @return {?boolean}
 */
proto.speech.tts.PartialSentenceProsody.prototype.getIncompleteAtEnd = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 2));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.PartialSentenceProsody} returns this
 */
proto.speech.tts.PartialSentenceProsody.prototype.setIncompleteAtEnd = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.PartialSentenceProsody} returns this
 */
proto.speech.tts.PartialSentenceProsody.prototype.clearIncompleteAtEnd = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.PartialSentenceProsody.prototype.hasIncompleteAtEnd = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.PartialSentenceProsody}
 */
proto.speech.tts.PartialSentenceProsody.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.PartialSentenceProsody, data);
};



//speech/tts/public/tts_markup_proto_jspb/audiosource.jspb.js
// source: speech/tts/public/tts_markup.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.AudioSource');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.AudioSource = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.AudioSource, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.AudioSource.displayName = 'proto.speech.tts.AudioSource';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.AudioSource.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.AudioSource.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.AudioSource} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.AudioSource.toObject = function(includeInstance, msg) {
  var f, obj = {
    vuiId: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    stageName: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of AudioSource as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.AudioSource.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.vuiId;

  /** @type {?string|undefined} */
  this.stageName;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.AudioSource.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.AudioSource}
 */
proto.speech.tts.AudioSource.fromObject = function(obj) {
  var msg = new proto.speech.tts.AudioSource();
  obj.vuiId != null && jspb.Message.setField(msg, 1, obj.vuiId);
  obj.stageName != null && jspb.Message.setField(msg, 2, obj.stageName);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.AudioSource}
 */
proto.speech.tts.AudioSource.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.AudioSource;
  return proto.speech.tts.AudioSource.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.AudioSource} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.AudioSource}
 */
proto.speech.tts.AudioSource.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setVuiId(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setStageName(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.AudioSource.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.AudioSource.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.AudioSource} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.AudioSource.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string vui_id = 1;
 * @return {?string}
 */
proto.speech.tts.AudioSource.prototype.getVuiId = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.AudioSource} returns this
 */
proto.speech.tts.AudioSource.prototype.setVuiId = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.AudioSource} returns this
 */
proto.speech.tts.AudioSource.prototype.clearVuiId = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.AudioSource.prototype.hasVuiId = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional string stage_name = 2;
 * @return {?string}
 */
proto.speech.tts.AudioSource.prototype.getStageName = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.AudioSource} returns this
 */
proto.speech.tts.AudioSource.prototype.setStageName = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.AudioSource} returns this
 */
proto.speech.tts.AudioSource.prototype.clearStageName = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.AudioSource.prototype.hasStageName = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.AudioSource}
 */
proto.speech.tts.AudioSource.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.AudioSource, data);
};



//speech/tts/public/tts_markup_proto_jspb/crosslingual.jspb.js
// source: speech/tts/public/tts_markup.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.CrossLingual');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.CrossLingual = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.CrossLingual.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.CrossLingual, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.CrossLingual.displayName = 'proto.speech.tts.CrossLingual';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.CrossLingual.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.CrossLingual.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.CrossLingual.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.CrossLingual} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.CrossLingual.toObject = function(includeInstance, msg) {
  var f, obj = {
    foreignLanguageHintList: (f = jspb.Message.getRepeatedField(msg, 1)) == null ? undefined : f,
    foreignLanguageOverride: (f = jspb.Message.getBooleanField(msg, 2)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of CrossLingual as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.CrossLingual.ObjectFormat = function() {
  /** @type {?Array<string>|undefined} */
  this.foreignLanguageHintList;

  /** @type {?boolean|undefined} */
  this.foreignLanguageOverride;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.CrossLingual.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.CrossLingual}
 */
proto.speech.tts.CrossLingual.fromObject = function(obj) {
  var msg = new proto.speech.tts.CrossLingual();
  obj.foreignLanguageHintList != null && jspb.Message.setField(msg, 1, obj.foreignLanguageHintList);
  obj.foreignLanguageOverride != null && jspb.Message.setField(msg, 2, obj.foreignLanguageOverride);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.CrossLingual}
 */
proto.speech.tts.CrossLingual.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.CrossLingual;
  return proto.speech.tts.CrossLingual.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.CrossLingual} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.CrossLingual}
 */
proto.speech.tts.CrossLingual.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.addForeignLanguageHint(value);
      break;
    case 2:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setForeignLanguageOverride(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.CrossLingual.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.CrossLingual.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.CrossLingual} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.CrossLingual.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getForeignLanguageHintList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      1,
      f
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeBool(
      2,
      f
    );
  }
};


/**
 * repeated string foreign_language_hint = 1;
 * @return {!Array<string>}
 */
proto.speech.tts.CrossLingual.prototype.getForeignLanguageHintList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 1));
};


/**
 * @param {Array<string>|undefined} value
 * @return {!proto.speech.tts.CrossLingual} returns this
 */
proto.speech.tts.CrossLingual.prototype.setForeignLanguageHintList = function(value) {
  return jspb.Message.setField(this, 1, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.CrossLingual} returns this
 */
proto.speech.tts.CrossLingual.prototype.addForeignLanguageHint = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 1, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.CrossLingual} returns this
 */
proto.speech.tts.CrossLingual.prototype.clearForeignLanguageHintList = function() {
  return this.setForeignLanguageHintList([]);
};


/**
 * optional bool foreign_language_override = 2;
 * @return {?boolean}
 */
proto.speech.tts.CrossLingual.prototype.getForeignLanguageOverride = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 2));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.CrossLingual} returns this
 */
proto.speech.tts.CrossLingual.prototype.setForeignLanguageOverride = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.CrossLingual} returns this
 */
proto.speech.tts.CrossLingual.prototype.clearForeignLanguageOverride = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.CrossLingual.prototype.hasForeignLanguageOverride = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.CrossLingual}
 */
proto.speech.tts.CrossLingual.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.CrossLingual, data);
};



//speech/tts/public/tts_markup_proto_jspb/markupword.jspb.js
// source: speech/tts/public/tts_markup.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.MarkupWord');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.MarkupWord = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.MarkupWord, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.MarkupWord.displayName = 'proto.speech.tts.MarkupWord';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.MarkupWord.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.MarkupWord.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.MarkupWord} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.MarkupWord.toObject = function(includeInstance, msg) {
  var f, obj = {
    id: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    variant: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    tokenName: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,
    pitchAccentPhraseStart: (f = jspb.Message.getBooleanField(msg, 4)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of MarkupWord as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.MarkupWord.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.id;

  /** @type {?string|undefined} */
  this.variant;

  /** @type {?string|undefined} */
  this.tokenName;

  /** @type {?boolean|undefined} */
  this.pitchAccentPhraseStart;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.MarkupWord.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.MarkupWord}
 */
proto.speech.tts.MarkupWord.fromObject = function(obj) {
  var msg = new proto.speech.tts.MarkupWord();
  obj.id != null && jspb.Message.setField(msg, 1, obj.id);
  obj.variant != null && jspb.Message.setField(msg, 2, obj.variant);
  obj.tokenName != null && jspb.Message.setField(msg, 3, obj.tokenName);
  obj.pitchAccentPhraseStart != null && jspb.Message.setField(msg, 4, obj.pitchAccentPhraseStart);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.MarkupWord}
 */
proto.speech.tts.MarkupWord.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.MarkupWord;
  return proto.speech.tts.MarkupWord.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.MarkupWord} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.MarkupWord}
 */
proto.speech.tts.MarkupWord.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setId(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setVariant(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setTokenName(value);
      break;
    case 4:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setPitchAccentPhraseStart(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.MarkupWord.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.MarkupWord.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.MarkupWord} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.MarkupWord.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeString(
      3,
      f
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeBool(
      4,
      f
    );
  }
};


/**
 * optional string id = 1;
 * @return {?string}
 */
proto.speech.tts.MarkupWord.prototype.getId = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.MarkupWord} returns this
 */
proto.speech.tts.MarkupWord.prototype.setId = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.MarkupWord} returns this
 */
proto.speech.tts.MarkupWord.prototype.clearId = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.MarkupWord.prototype.hasId = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional string variant = 2;
 * @return {?string}
 */
proto.speech.tts.MarkupWord.prototype.getVariant = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.MarkupWord} returns this
 */
proto.speech.tts.MarkupWord.prototype.setVariant = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.MarkupWord} returns this
 */
proto.speech.tts.MarkupWord.prototype.clearVariant = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.MarkupWord.prototype.hasVariant = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional string token_name = 3;
 * @return {?string}
 */
proto.speech.tts.MarkupWord.prototype.getTokenName = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.MarkupWord} returns this
 */
proto.speech.tts.MarkupWord.prototype.setTokenName = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.MarkupWord} returns this
 */
proto.speech.tts.MarkupWord.prototype.clearTokenName = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.MarkupWord.prototype.hasTokenName = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional bool pitch_accent_phrase_start = 4;
 * @return {?boolean}
 */
proto.speech.tts.MarkupWord.prototype.getPitchAccentPhraseStart = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 4));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.MarkupWord} returns this
 */
proto.speech.tts.MarkupWord.prototype.setPitchAccentPhraseStart = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.MarkupWord} returns this
 */
proto.speech.tts.MarkupWord.prototype.clearPitchAccentPhraseStart = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.MarkupWord.prototype.hasPitchAccentPhraseStart = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.MarkupWord}
 */
proto.speech.tts.MarkupWord.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.MarkupWord, data);
};



//speech/tts/public/prosody_proto_jspb/emotiondimensions.jspb.js
// source: speech/tts/public/prosody.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.EmotionDimensions');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.EmotionDimensions = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.EmotionDimensions, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.EmotionDimensions.displayName = 'proto.speech.tts.EmotionDimensions';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.EmotionDimensions.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.EmotionDimensions.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.EmotionDimensions} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.EmotionDimensions.toObject = function(includeInstance, msg) {
  var f, obj = {
    arousal: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    valence: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of EmotionDimensions as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.EmotionDimensions.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.arousal;

  /** @type {?number|undefined} */
  this.valence;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.EmotionDimensions.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.EmotionDimensions}
 */
proto.speech.tts.EmotionDimensions.fromObject = function(obj) {
  var msg = new proto.speech.tts.EmotionDimensions();
  obj.arousal != null && jspb.Message.setField(msg, 1, obj.arousal);
  obj.valence != null && jspb.Message.setField(msg, 2, obj.valence);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.EmotionDimensions}
 */
proto.speech.tts.EmotionDimensions.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.EmotionDimensions;
  return proto.speech.tts.EmotionDimensions.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.EmotionDimensions} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.EmotionDimensions}
 */
proto.speech.tts.EmotionDimensions.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setArousal(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setValence(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.EmotionDimensions.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.EmotionDimensions.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.EmotionDimensions} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.EmotionDimensions.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?number} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeInt32(
      1,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeInt32(
      2,
      f
    );
  }
};


/**
 * optional int32 arousal = 1;
 * @return {?number}
 */
proto.speech.tts.EmotionDimensions.prototype.getArousal = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.EmotionDimensions} returns this
 */
proto.speech.tts.EmotionDimensions.prototype.setArousal = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.EmotionDimensions} returns this
 */
proto.speech.tts.EmotionDimensions.prototype.clearArousal = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.EmotionDimensions.prototype.hasArousal = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional int32 valence = 2;
 * @return {?number}
 */
proto.speech.tts.EmotionDimensions.prototype.getValence = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.EmotionDimensions} returns this
 */
proto.speech.tts.EmotionDimensions.prototype.setValence = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.EmotionDimensions} returns this
 */
proto.speech.tts.EmotionDimensions.prototype.clearValence = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.EmotionDimensions.prototype.hasValence = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.EmotionDimensions}
 */
proto.speech.tts.EmotionDimensions.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.EmotionDimensions, data);
};



//speech/tts/public/tts_markup_proto_jspb/prosodicfeatures.jspb.js
// source: speech/tts/public/tts_markup.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.ProsodicFeatures');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.EmotionDimensions');
goog.require('proto.speech.tts.Prominence');

goog.forwardDeclare('proto.speech.tts.IntonationType');
goog.forwardDeclare('proto.speech.tts.PhonationMode');
goog.forwardDeclare('proto.speech.tts.SpeakingStyle');
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.ProsodicFeatures = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.ProsodicFeatures.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.ProsodicFeatures, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.ProsodicFeatures.displayName = 'proto.speech.tts.ProsodicFeatures';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.ProsodicFeatures.repeatedFields_ = [9];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.ProsodicFeatures.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.ProsodicFeatures.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.ProsodicFeatures} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.ProsodicFeatures.toObject = function(includeInstance, msg) {
  var f, obj = {
    containsAccent: (f = jspb.Message.getBooleanField(msg, 1)) == null ? undefined : f,
    containsNucleus: (f = jspb.Message.getBooleanField(msg, 2)) == null ? undefined : f,
    intonationType: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,
    prominence: (f = msg.getProminence()) && proto.speech.tts.Prominence.toObject(includeInstance, f),
    containsPause: (f = jspb.Message.getBooleanField(msg, 5)) == null ? undefined : f,
    precedesPause: (f = jspb.Message.getBooleanField(msg, 6)) == null ? undefined : f,
    ratedProminence: (f = jspb.Message.getOptionalFloatingPointField(msg, 7)) == null ? undefined : f,
    isExpressive: (f = jspb.Message.getBooleanField(msg, 8)) == null ? undefined : f,
    vuiProsodyStyleList: (f = jspb.Message.getRepeatedField(msg, 9)) == null ? undefined : f,
    emotionDimensions: (f = msg.getEmotionDimensions()) && proto.speech.tts.EmotionDimensions.toObject(includeInstance, f),
    speakingStyle: (f = jspb.Message.getField(msg, 11)) == null ? undefined : f,
    phonationMode: (f = jspb.Message.getField(msg, 12)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of ProsodicFeatures as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.ProsodicFeatures.ObjectFormat = function() {
  /** @type {?boolean|undefined} */
  this.containsAccent;

  /** @type {?boolean|undefined} */
  this.containsNucleus;

  /** @type {?number|undefined} */
  this.intonationType;

  /** @type {?proto.speech.tts.Prominence.ObjectFormat|undefined} */
  this.prominence;

  /** @type {?boolean|undefined} */
  this.containsPause;

  /** @type {?boolean|undefined} */
  this.precedesPause;

  /** @type {?number|undefined} */
  this.ratedProminence;

  /** @type {?boolean|undefined} */
  this.isExpressive;

  /** @type {?Array<string>|undefined} */
  this.vuiProsodyStyleList;

  /** @type {?proto.speech.tts.EmotionDimensions.ObjectFormat|undefined} */
  this.emotionDimensions;

  /** @type {?number|undefined} */
  this.speakingStyle;

  /** @type {?number|undefined} */
  this.phonationMode;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.ProsodicFeatures.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.ProsodicFeatures}
 */
proto.speech.tts.ProsodicFeatures.fromObject = function(obj) {
  var msg = new proto.speech.tts.ProsodicFeatures();
  obj.containsAccent != null && jspb.Message.setField(msg, 1, obj.containsAccent);
  obj.containsNucleus != null && jspb.Message.setField(msg, 2, obj.containsNucleus);
  obj.intonationType != null && jspb.Message.setField(msg, 3, obj.intonationType);
  obj.prominence && jspb.Message.setWrapperField(
      msg, 4, proto.speech.tts.Prominence.fromObject(obj.prominence));
  obj.containsPause != null && jspb.Message.setField(msg, 5, obj.containsPause);
  obj.precedesPause != null && jspb.Message.setField(msg, 6, obj.precedesPause);
  obj.ratedProminence != null && jspb.Message.setField(msg, 7, obj.ratedProminence);
  obj.isExpressive != null && jspb.Message.setField(msg, 8, obj.isExpressive);
  obj.vuiProsodyStyleList != null && jspb.Message.setField(msg, 9, obj.vuiProsodyStyleList);
  obj.emotionDimensions && jspb.Message.setWrapperField(
      msg, 10, proto.speech.tts.EmotionDimensions.fromObject(obj.emotionDimensions));
  obj.speakingStyle != null && jspb.Message.setField(msg, 11, obj.speakingStyle);
  obj.phonationMode != null && jspb.Message.setField(msg, 12, obj.phonationMode);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.ProsodicFeatures}
 */
proto.speech.tts.ProsodicFeatures.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.ProsodicFeatures;
  return proto.speech.tts.ProsodicFeatures.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.ProsodicFeatures} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.ProsodicFeatures}
 */
proto.speech.tts.ProsodicFeatures.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setContainsAccent(value);
      break;
    case 2:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setContainsNucleus(value);
      break;
    case 3:
      var value = /** @type {!proto.speech.tts.IntonationType} */ (reader.readEnum());
      msg.setIntonationType(value);
      break;
    case 4:
      var value = new proto.speech.tts.Prominence;
      reader.readMessage(value,proto.speech.tts.Prominence.deserializeBinaryFromReader);
      msg.setProminence(value);
      break;
    case 5:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setContainsPause(value);
      break;
    case 6:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setPrecedesPause(value);
      break;
    case 7:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setRatedProminence(value);
      break;
    case 8:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsExpressive(value);
      break;
    case 9:
      var value = /** @type {string} */ (reader.readString());
      msg.addVuiProsodyStyle(value);
      break;
    case 10:
      var value = new proto.speech.tts.EmotionDimensions;
      reader.readMessage(value,proto.speech.tts.EmotionDimensions.deserializeBinaryFromReader);
      msg.setEmotionDimensions(value);
      break;
    case 11:
      var value = /** @type {!proto.speech.tts.SpeakingStyle} */ (reader.readEnum());
      msg.setSpeakingStyle(value);
      break;
    case 12:
      var value = /** @type {!proto.speech.tts.PhonationMode} */ (reader.readEnum());
      msg.setPhonationMode(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.ProsodicFeatures.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.ProsodicFeatures.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.ProsodicFeatures} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.ProsodicFeatures.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeBool(
      1,
      f
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeBool(
      2,
      f
    );
  }
  f = /** @type {!proto.speech.tts.IntonationType} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeEnum(
      3,
      f
    );
  }
  f = message.getProminence();
  if (f != null) {
    writer.writeMessage(
      4,
      f,
      proto.speech.tts.Prominence.serializeBinaryToWriter
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeBool(
      5,
      f
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeBool(
      6,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 7));
  if (f != null) {
    writer.writeFloat(
      7,
      f
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 8));
  if (f != null) {
    writer.writeBool(
      8,
      f
    );
  }
  f = message.getVuiProsodyStyleList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      9,
      f
    );
  }
  f = message.getEmotionDimensions();
  if (f != null) {
    writer.writeMessage(
      10,
      f,
      proto.speech.tts.EmotionDimensions.serializeBinaryToWriter
    );
  }
  f = /** @type {!proto.speech.tts.SpeakingStyle} */ (jspb.Message.getField(message, 11));
  if (f != null) {
    writer.writeEnum(
      11,
      f
    );
  }
  f = /** @type {!proto.speech.tts.PhonationMode} */ (jspb.Message.getField(message, 12));
  if (f != null) {
    writer.writeEnum(
      12,
      f
    );
  }
};


/**
 * optional bool contains_accent = 1;
 * @return {?boolean}
 */
proto.speech.tts.ProsodicFeatures.prototype.getContainsAccent = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 1));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.ProsodicFeatures} returns this
 */
proto.speech.tts.ProsodicFeatures.prototype.setContainsAccent = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.ProsodicFeatures} returns this
 */
proto.speech.tts.ProsodicFeatures.prototype.clearContainsAccent = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.ProsodicFeatures.prototype.hasContainsAccent = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional bool contains_nucleus = 2;
 * @return {?boolean}
 */
proto.speech.tts.ProsodicFeatures.prototype.getContainsNucleus = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 2));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.ProsodicFeatures} returns this
 */
proto.speech.tts.ProsodicFeatures.prototype.setContainsNucleus = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.ProsodicFeatures} returns this
 */
proto.speech.tts.ProsodicFeatures.prototype.clearContainsNucleus = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.ProsodicFeatures.prototype.hasContainsNucleus = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional IntonationType intonation_type = 3;
 * @return {!proto.speech.tts.IntonationType}
 */
proto.speech.tts.ProsodicFeatures.prototype.getIntonationType = function() {
  return /** @type {!proto.speech.tts.IntonationType} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?proto.speech.tts.IntonationType|undefined} value
 * @return {!proto.speech.tts.ProsodicFeatures} returns this
 */
proto.speech.tts.ProsodicFeatures.prototype.setIntonationType = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.ProsodicFeatures} returns this
 */
proto.speech.tts.ProsodicFeatures.prototype.clearIntonationType = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.ProsodicFeatures.prototype.hasIntonationType = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional Prominence prominence = 4;
 * @return {?proto.speech.tts.Prominence}
 */
proto.speech.tts.ProsodicFeatures.prototype.getProminence = function() {
  return /** @type{?proto.speech.tts.Prominence} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Prominence, 4));
};


/**
 * @param {?proto.speech.tts.Prominence|undefined} value
 * @return {!proto.speech.tts.ProsodicFeatures} returns this
*/
proto.speech.tts.ProsodicFeatures.prototype.setProminence = function(value) {
  return jspb.Message.setWrapperField(this, 4, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.ProsodicFeatures} returns this
 */
proto.speech.tts.ProsodicFeatures.prototype.clearProminence = function() {
  return this.setProminence(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.ProsodicFeatures.prototype.hasProminence = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional bool contains_pause = 5;
 * @return {?boolean}
 */
proto.speech.tts.ProsodicFeatures.prototype.getContainsPause = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 5));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.ProsodicFeatures} returns this
 */
proto.speech.tts.ProsodicFeatures.prototype.setContainsPause = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.ProsodicFeatures} returns this
 */
proto.speech.tts.ProsodicFeatures.prototype.clearContainsPause = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.ProsodicFeatures.prototype.hasContainsPause = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional bool precedes_pause = 6;
 * @return {?boolean}
 */
proto.speech.tts.ProsodicFeatures.prototype.getPrecedesPause = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 6));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.ProsodicFeatures} returns this
 */
proto.speech.tts.ProsodicFeatures.prototype.setPrecedesPause = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.ProsodicFeatures} returns this
 */
proto.speech.tts.ProsodicFeatures.prototype.clearPrecedesPause = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.ProsodicFeatures.prototype.hasPrecedesPause = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional float rated_prominence = 7;
 * @return {?number}
 */
proto.speech.tts.ProsodicFeatures.prototype.getRatedProminence = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 7));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.ProsodicFeatures} returns this
 */
proto.speech.tts.ProsodicFeatures.prototype.setRatedProminence = function(value) {
  return jspb.Message.setField(this, 7, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.ProsodicFeatures} returns this
 */
proto.speech.tts.ProsodicFeatures.prototype.clearRatedProminence = function() {
  return jspb.Message.setField(this, 7, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.ProsodicFeatures.prototype.hasRatedProminence = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * optional bool is_expressive = 8;
 * @return {?boolean}
 */
proto.speech.tts.ProsodicFeatures.prototype.getIsExpressive = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 8));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.ProsodicFeatures} returns this
 */
proto.speech.tts.ProsodicFeatures.prototype.setIsExpressive = function(value) {
  return jspb.Message.setField(this, 8, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.ProsodicFeatures} returns this
 */
proto.speech.tts.ProsodicFeatures.prototype.clearIsExpressive = function() {
  return jspb.Message.setField(this, 8, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.ProsodicFeatures.prototype.hasIsExpressive = function() {
  return jspb.Message.getField(this, 8) != null;
};


/**
 * repeated string vui_prosody_style = 9;
 * @return {!Array<string>}
 */
proto.speech.tts.ProsodicFeatures.prototype.getVuiProsodyStyleList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 9));
};


/**
 * @param {Array<string>|undefined} value
 * @return {!proto.speech.tts.ProsodicFeatures} returns this
 */
proto.speech.tts.ProsodicFeatures.prototype.setVuiProsodyStyleList = function(value) {
  return jspb.Message.setField(this, 9, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.ProsodicFeatures} returns this
 */
proto.speech.tts.ProsodicFeatures.prototype.addVuiProsodyStyle = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 9, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.ProsodicFeatures} returns this
 */
proto.speech.tts.ProsodicFeatures.prototype.clearVuiProsodyStyleList = function() {
  return this.setVuiProsodyStyleList([]);
};


/**
 * optional EmotionDimensions emotion_dimensions = 10;
 * @return {?proto.speech.tts.EmotionDimensions}
 */
proto.speech.tts.ProsodicFeatures.prototype.getEmotionDimensions = function() {
  return /** @type{?proto.speech.tts.EmotionDimensions} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.EmotionDimensions, 10));
};


/**
 * @param {?proto.speech.tts.EmotionDimensions|undefined} value
 * @return {!proto.speech.tts.ProsodicFeatures} returns this
*/
proto.speech.tts.ProsodicFeatures.prototype.setEmotionDimensions = function(value) {
  return jspb.Message.setWrapperField(this, 10, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.ProsodicFeatures} returns this
 */
proto.speech.tts.ProsodicFeatures.prototype.clearEmotionDimensions = function() {
  return this.setEmotionDimensions(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.ProsodicFeatures.prototype.hasEmotionDimensions = function() {
  return jspb.Message.getField(this, 10) != null;
};


/**
 * optional SpeakingStyle speaking_style = 11;
 * @return {!proto.speech.tts.SpeakingStyle}
 */
proto.speech.tts.ProsodicFeatures.prototype.getSpeakingStyle = function() {
  return /** @type {!proto.speech.tts.SpeakingStyle} */ (jspb.Message.getField(this, 11));
};


/**
 * @param {?proto.speech.tts.SpeakingStyle|undefined} value
 * @return {!proto.speech.tts.ProsodicFeatures} returns this
 */
proto.speech.tts.ProsodicFeatures.prototype.setSpeakingStyle = function(value) {
  return jspb.Message.setField(this, 11, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.ProsodicFeatures} returns this
 */
proto.speech.tts.ProsodicFeatures.prototype.clearSpeakingStyle = function() {
  return jspb.Message.setField(this, 11, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.ProsodicFeatures.prototype.hasSpeakingStyle = function() {
  return jspb.Message.getField(this, 11) != null;
};


/**
 * optional PhonationMode phonation_mode = 12;
 * @return {!proto.speech.tts.PhonationMode}
 */
proto.speech.tts.ProsodicFeatures.prototype.getPhonationMode = function() {
  return /** @type {!proto.speech.tts.PhonationMode} */ (jspb.Message.getField(this, 12));
};


/**
 * @param {?proto.speech.tts.PhonationMode|undefined} value
 * @return {!proto.speech.tts.ProsodicFeatures} returns this
 */
proto.speech.tts.ProsodicFeatures.prototype.setPhonationMode = function(value) {
  return jspb.Message.setField(this, 12, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.ProsodicFeatures} returns this
 */
proto.speech.tts.ProsodicFeatures.prototype.clearPhonationMode = function() {
  return jspb.Message.setField(this, 12, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.ProsodicFeatures.prototype.hasPhonationMode = function() {
  return jspb.Message.getField(this, 12) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.ProsodicFeatures}
 */
proto.speech.tts.ProsodicFeatures.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.ProsodicFeatures, data);
};



//speech/tts/public/tts_markup_proto_jspb/synthesizerselection.jspb.js
// source: speech/tts/public/tts_markup.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.SynthesizerSelection');
goog.provide('proto.speech.tts.SynthesizerSelection.Method');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.SynthesizerSelection = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.SynthesizerSelection, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.SynthesizerSelection.displayName = 'proto.speech.tts.SynthesizerSelection';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.SynthesizerSelection.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.SynthesizerSelection.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.SynthesizerSelection} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.SynthesizerSelection.toObject = function(includeInstance, msg) {
  var f, obj = {
    method: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of SynthesizerSelection as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.SynthesizerSelection.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.method;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.SynthesizerSelection.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.SynthesizerSelection}
 */
proto.speech.tts.SynthesizerSelection.fromObject = function(obj) {
  var msg = new proto.speech.tts.SynthesizerSelection();
  obj.method != null && jspb.Message.setField(msg, 1, obj.method);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.SynthesizerSelection}
 */
proto.speech.tts.SynthesizerSelection.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.SynthesizerSelection;
  return proto.speech.tts.SynthesizerSelection.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.SynthesizerSelection} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.SynthesizerSelection}
 */
proto.speech.tts.SynthesizerSelection.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {!proto.speech.tts.SynthesizerSelection.Method} */ (reader.readEnum());
      msg.setMethod(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.SynthesizerSelection.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.SynthesizerSelection.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.SynthesizerSelection} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.SynthesizerSelection.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {!proto.speech.tts.SynthesizerSelection.Method} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeEnum(
      1,
      f
    );
  }
};


/**
 * @enum {number}
 */
proto.speech.tts.SynthesizerSelection.Method = {
  AUTOMATIC: 0,
  UNIT_SELECTION: 1,
  PARAMETRIC: 2
};

/**
 * optional Method method = 1;
 * @return {!proto.speech.tts.SynthesizerSelection.Method}
 */
proto.speech.tts.SynthesizerSelection.prototype.getMethod = function() {
  return /** @type {!proto.speech.tts.SynthesizerSelection.Method} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?proto.speech.tts.SynthesizerSelection.Method|undefined} value
 * @return {!proto.speech.tts.SynthesizerSelection} returns this
 */
proto.speech.tts.SynthesizerSelection.prototype.setMethod = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.SynthesizerSelection} returns this
 */
proto.speech.tts.SynthesizerSelection.prototype.clearMethod = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.SynthesizerSelection.prototype.hasMethod = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.SynthesizerSelection}
 */
proto.speech.tts.SynthesizerSelection.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.SynthesizerSelection, data);
};



//speech/tts/public/tts_markup_proto_jspb/verbalinfo.jspb.js
// source: speech/tts/public/tts_markup.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.VerbalInfo');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.VerbalInfo = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.VerbalInfo, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.VerbalInfo.displayName = 'proto.speech.tts.VerbalInfo';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.VerbalInfo.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.VerbalInfo.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.VerbalInfo} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.VerbalInfo.toObject = function(includeInstance, msg) {
  var f, obj = {
    text: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    pronunciation: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of VerbalInfo as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.VerbalInfo.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.text;

  /** @type {?string|undefined} */
  this.pronunciation;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.VerbalInfo.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.VerbalInfo}
 */
proto.speech.tts.VerbalInfo.fromObject = function(obj) {
  var msg = new proto.speech.tts.VerbalInfo();
  obj.text != null && jspb.Message.setField(msg, 1, obj.text);
  obj.pronunciation != null && jspb.Message.setField(msg, 2, obj.pronunciation);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.VerbalInfo}
 */
proto.speech.tts.VerbalInfo.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.VerbalInfo;
  return proto.speech.tts.VerbalInfo.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.VerbalInfo} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.VerbalInfo}
 */
proto.speech.tts.VerbalInfo.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setText(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setPronunciation(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.VerbalInfo.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.VerbalInfo.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.VerbalInfo} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.VerbalInfo.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string text = 1;
 * @return {?string}
 */
proto.speech.tts.VerbalInfo.prototype.getText = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.VerbalInfo} returns this
 */
proto.speech.tts.VerbalInfo.prototype.setText = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.VerbalInfo} returns this
 */
proto.speech.tts.VerbalInfo.prototype.clearText = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.VerbalInfo.prototype.hasText = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional string pronunciation = 2;
 * @return {?string}
 */
proto.speech.tts.VerbalInfo.prototype.getPronunciation = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.VerbalInfo} returns this
 */
proto.speech.tts.VerbalInfo.prototype.setPronunciation = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.VerbalInfo} returns this
 */
proto.speech.tts.VerbalInfo.prototype.clearPronunciation = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.VerbalInfo.prototype.hasPronunciation = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.VerbalInfo}
 */
proto.speech.tts.VerbalInfo.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.VerbalInfo, data);
};



//speech/tts/public/tts_markup_proto_jspb/say.jspb.js
// source: speech/tts/public/tts_markup.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.Say');
goog.provide('proto.speech.tts.Say.WhatToSayCase');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.location.unified.LocationDescriptor');
goog.require('proto.speech.tts.Abbreviation');
goog.require('proto.speech.tts.Address');
goog.require('proto.speech.tts.AudioSource');
goog.require('proto.speech.tts.Cardinal');
goog.require('proto.speech.tts.ChemicalFormula');
goog.require('proto.speech.tts.Connector');
goog.require('proto.speech.tts.CrossLingual');
goog.require('proto.speech.tts.Date');
goog.require('proto.speech.tts.Decimal');
goog.require('proto.speech.tts.Electronic');
goog.require('proto.speech.tts.EmoticonEmoji');
goog.require('proto.speech.tts.Fraction');
goog.require('proto.speech.tts.MarkupWord');
goog.require('proto.speech.tts.Measure');
goog.require('proto.speech.tts.Money');
goog.require('proto.speech.tts.MorphosyntacticFeatureVector');
goog.require('proto.speech.tts.Ordinal');
goog.require('proto.speech.tts.ProsodicFeatures');
goog.require('proto.speech.tts.Spelling');
goog.require('proto.speech.tts.SynthesizerSelection');
goog.require('proto.speech.tts.Telephone');
goog.require('proto.speech.tts.Time');
goog.require('proto.speech.tts.VerbalInfo');
goog.require('proto.speech.tts.VoiceMod');

goog.forwardDeclare('proto.speech.tts.TextnormLevel');
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.Say = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 42, proto.speech.tts.Say.repeatedFields_, proto.speech.tts.Say.oneofGroups_);
};
goog.inherits(proto.speech.tts.Say, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.Say.displayName = 'proto.speech.tts.Say';
}

/**
 * The extensions registered with this message class. This is a map of
 * extension field number to fieldInfo object.
 *
 * For example:
 *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }
 *
 * fieldName contains the JsCompiler renamed field name property so that it
 * works in OPTIMIZED mode.
 *
 * @type {!Object<number, jspb.ExtensionFieldInfo>}
 */
proto.speech.tts.Say.extensions = {};


/**
 * The extensions registered with this message class. This is a map of
 * extension field number to fieldInfo object.
 *
 * For example:
 *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }
 *
 * fieldName contains the JsCompiler renamed field name property so that it
 * works in OPTIMIZED mode.
 *
 * @type {!Object<number, jspb.ExtensionFieldBinaryInfo>}
 */
proto.speech.tts.Say.extensionsBinary = {};


/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.Say.repeatedFields_ = [17,34,26];

/**
 * Oneof group definitions for this message. Each group defines the field
 * numbers belonging to that group. When of these fields' value is set, all
 * other fields in the group are cleared. During deserialization, if multiple
 * fields are encountered for a group, only the last value seen will be kept.
 * @private {!Array<!Array<number>>}
 * @const
 */
proto.speech.tts.Say.oneofGroups_ = [[1,3,4,5,6,15,7,8,9,10,11,12,13,14,16,18,19,21,32,25,36,37,41]];

/**
 * @enum {number}
 */
proto.speech.tts.Say.WhatToSayCase = {
  WHAT_TO_SAY_NOT_SET: 0,
  TEXT: 1,
  CARDINAL: 3,
  ORDINAL: 4,
  DECIMAL: 5,
  FRACTION: 6,
  DIGIT: 15,
  TIME: 7,
  MEASURE: 8,
  DATE: 9,
  ADDRESS: 10,
  TELEPHONE: 11,
  MONEY: 12,
  ELECTRONIC: 13,
  CHEMICAL_FORMULA: 14,
  WORDS: 16,
  LETTERS: 18,
  VERBATIM: 19,
  PAUSE: 21,
  SPELL: 32,
  VERBAL_INFO: 25,
  CONNECTOR: 36,
  ABBREVIATION: 37,
  EMOTICON_EMOJI: 41
};

/**
 * @return {!proto.speech.tts.Say.WhatToSayCase}
 */
proto.speech.tts.Say.prototype.getWhatToSayCase = function() {
  return /** @type {!proto.speech.tts.Say.WhatToSayCase} */(jspb.Message.computeOneofCase(this, proto.speech.tts.Say.oneofGroups_[0]));
};



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.Say.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.Say.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.Say} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Say.toObject = function(includeInstance, msg) {
  var f, obj = {
    text: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    cardinal: (f = msg.getCardinal()) && proto.speech.tts.Cardinal.toObject(includeInstance, f),
    ordinal: (f = msg.getOrdinal()) && proto.speech.tts.Ordinal.toObject(includeInstance, f),
    decimal: (f = msg.getDecimal()) && proto.speech.tts.Decimal.toObject(includeInstance, f),
    fraction: (f = msg.getFraction()) && proto.speech.tts.Fraction.toObject(includeInstance, f),
    digit: (f = jspb.Message.getField(msg, 15)) == null ? undefined : f,
    time: (f = msg.getTime()) && proto.speech.tts.Time.toObject(includeInstance, f),
    measure: (f = msg.getMeasure()) && proto.speech.tts.Measure.toObject(includeInstance, f),
    date: (f = msg.getDate()) && proto.speech.tts.Date.toObject(includeInstance, f),
    address: (f = msg.getAddress()) && proto.speech.tts.Address.toObject(includeInstance, f),
    telephone: (f = msg.getTelephone()) && proto.speech.tts.Telephone.toObject(includeInstance, f),
    money: (f = msg.getMoney()) && proto.speech.tts.Money.toObject(includeInstance, f),
    electronic: (f = msg.getElectronic()) && proto.speech.tts.Electronic.toObject(includeInstance, f),
    chemicalFormula: (f = msg.getChemicalFormula()) && proto.speech.tts.ChemicalFormula.toObject(includeInstance, f),
    words: (f = jspb.Message.getField(msg, 16)) == null ? undefined : f,
    letters: (f = jspb.Message.getField(msg, 18)) == null ? undefined : f,
    verbatim: (f = jspb.Message.getField(msg, 19)) == null ? undefined : f,
    pause: (f = jspb.Message.getOptionalFloatingPointField(msg, 21)) == null ? undefined : f,
    spell: (f = msg.getSpell()) && proto.speech.tts.Spelling.toObject(includeInstance, f),
    verbalInfo: (f = msg.getVerbalInfo()) && proto.speech.tts.VerbalInfo.toObject(includeInstance, f),
    connector: (f = msg.getConnector()) && proto.speech.tts.Connector.toObject(includeInstance, f),
    abbreviation: (f = msg.getAbbreviation()) && proto.speech.tts.Abbreviation.toObject(includeInstance, f),
    emoticonEmoji: (f = msg.getEmoticonEmoji()) && proto.speech.tts.EmoticonEmoji.toObject(includeInstance, f),
    wordSequenceList: jspb.Message.toObjectList(msg.getWordSequenceList(),
    proto.speech.tts.MarkupWord.toObject, includeInstance),
    voicemod: (f = msg.getVoicemod()) && proto.speech.tts.VoiceMod.toObject(includeInstance, f),
    morphosyntacticFeatures: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    structuredFeatures: (f = msg.getStructuredFeatures()) && proto.speech.tts.MorphosyntacticFeatureVector.toObject(includeInstance, f),
    prosodicFeatures: (f = msg.getProsodicFeatures()) && proto.speech.tts.ProsodicFeatures.toObject(includeInstance, f),
    location: (f = msg.getLocation()) && proto.location.unified.LocationDescriptor.toObject(includeInstance, f),
    knowledgeGraphMidList: (f = jspb.Message.getRepeatedField(msg, 34)) == null ? undefined : f,
    conciseEmoji: (f = jspb.Message.getBooleanField(msg, 24)) == null ? undefined : f,
    audioSourceList: jspb.Message.toObjectList(msg.getAudioSourceList(),
    proto.speech.tts.AudioSource.toObject, includeInstance),
    crossLingual: (f = msg.getCrossLingual()) && proto.speech.tts.CrossLingual.toObject(includeInstance, f),
    foreignScript: (f = jspb.Message.getField(msg, 39)) == null ? undefined : f,
    textnormLevel: (f = jspb.Message.getField(msg, 29)) == null ? undefined : f,
    synthesizerSelection: (f = msg.getSynthesizerSelection()) && proto.speech.tts.SynthesizerSelection.toObject(includeInstance, f),
    bleep: (f = jspb.Message.getBooleanField(msg, 31)) == null ? undefined : f,
    prefix: (f = jspb.Message.getField(msg, 33)) == null ? undefined : f,
    suffix: (f = jspb.Message.getField(msg, 35)) == null ? undefined : f,
    timepointName: (f = jspb.Message.getField(msg, 38)) == null ? undefined : f,
    noCelebrity: (f = jspb.Message.getBooleanField(msg, 40)) == null ? undefined : f
  };

  jspb.Message.toObjectExtension(/** @type {!jspb.Message} */ (msg), obj,
      proto.speech.tts.Say.extensions, proto.speech.tts.Say.prototype.getExtension,
      includeInstance);
  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Say as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.Say.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.text;

  /** @type {?proto.speech.tts.Cardinal.ObjectFormat|undefined} */
  this.cardinal;

  /** @type {?proto.speech.tts.Ordinal.ObjectFormat|undefined} */
  this.ordinal;

  /** @type {?proto.speech.tts.Decimal.ObjectFormat|undefined} */
  this.decimal;

  /** @type {?proto.speech.tts.Fraction.ObjectFormat|undefined} */
  this.fraction;

  /** @type {?string|undefined} */
  this.digit;

  /** @type {?proto.speech.tts.Time.ObjectFormat|undefined} */
  this.time;

  /** @type {?proto.speech.tts.Measure.ObjectFormat|undefined} */
  this.measure;

  /** @type {?proto.speech.tts.Date.ObjectFormat|undefined} */
  this.date;

  /** @type {?proto.speech.tts.Address.ObjectFormat|undefined} */
  this.address;

  /** @type {?proto.speech.tts.Telephone.ObjectFormat|undefined} */
  this.telephone;

  /** @type {?proto.speech.tts.Money.ObjectFormat|undefined} */
  this.money;

  /** @type {?proto.speech.tts.Electronic.ObjectFormat|undefined} */
  this.electronic;

  /** @type {?proto.speech.tts.ChemicalFormula.ObjectFormat|undefined} */
  this.chemicalFormula;

  /** @type {?string|undefined} */
  this.words;

  /** @type {?string|undefined} */
  this.letters;

  /** @type {?string|undefined} */
  this.verbatim;

  /** @type {?number|undefined} */
  this.pause;

  /** @type {?proto.speech.tts.Spelling.ObjectFormat|undefined} */
  this.spell;

  /** @type {?proto.speech.tts.VerbalInfo.ObjectFormat|undefined} */
  this.verbalInfo;

  /** @type {?proto.speech.tts.Connector.ObjectFormat|undefined} */
  this.connector;

  /** @type {?proto.speech.tts.Abbreviation.ObjectFormat|undefined} */
  this.abbreviation;

  /** @type {?proto.speech.tts.EmoticonEmoji.ObjectFormat|undefined} */
  this.emoticonEmoji;

  /** @type {?Array<!proto.speech.tts.MarkupWord.ObjectFormat>|undefined} */
  this.wordSequenceList;

  /** @type {?proto.speech.tts.VoiceMod.ObjectFormat|undefined} */
  this.voicemod;

  /** @type {?string|undefined} */
  this.morphosyntacticFeatures;

  /** @type {?proto.speech.tts.MorphosyntacticFeatureVector.ObjectFormat|undefined} */
  this.structuredFeatures;

  /** @type {?proto.speech.tts.ProsodicFeatures.ObjectFormat|undefined} */
  this.prosodicFeatures;

  /** @type {?proto.location.unified.LocationDescriptor.ObjectFormat|undefined} */
  this.location;

  /** @type {?Array<string>|undefined} */
  this.knowledgeGraphMidList;

  /** @type {?boolean|undefined} */
  this.conciseEmoji;

  /** @type {?Array<!proto.speech.tts.AudioSource.ObjectFormat>|undefined} */
  this.audioSourceList;

  /** @type {?proto.speech.tts.CrossLingual.ObjectFormat|undefined} */
  this.crossLingual;

  /** @type {?string|undefined} */
  this.foreignScript;

  /** @type {?number|undefined} */
  this.textnormLevel;

  /** @type {?proto.speech.tts.SynthesizerSelection.ObjectFormat|undefined} */
  this.synthesizerSelection;

  /** @type {?boolean|undefined} */
  this.bleep;

  /** @type {?string|undefined} */
  this.prefix;

  /** @type {?string|undefined} */
  this.suffix;

  /** @type {?string|undefined} */
  this.timepointName;

  /** @type {?boolean|undefined} */
  this.noCelebrity;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.Say.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.Say}
 */
proto.speech.tts.Say.fromObject = function(obj) {
  var msg = new proto.speech.tts.Say();
  obj.text != null && jspb.Message.setField(msg, 1, obj.text);
  obj.cardinal && jspb.Message.setWrapperField(
      msg, 3, proto.speech.tts.Cardinal.fromObject(obj.cardinal));
  obj.ordinal && jspb.Message.setWrapperField(
      msg, 4, proto.speech.tts.Ordinal.fromObject(obj.ordinal));
  obj.decimal && jspb.Message.setWrapperField(
      msg, 5, proto.speech.tts.Decimal.fromObject(obj.decimal));
  obj.fraction && jspb.Message.setWrapperField(
      msg, 6, proto.speech.tts.Fraction.fromObject(obj.fraction));
  obj.digit != null && jspb.Message.setField(msg, 15, obj.digit);
  obj.time && jspb.Message.setWrapperField(
      msg, 7, proto.speech.tts.Time.fromObject(obj.time));
  obj.measure && jspb.Message.setWrapperField(
      msg, 8, proto.speech.tts.Measure.fromObject(obj.measure));
  obj.date && jspb.Message.setWrapperField(
      msg, 9, proto.speech.tts.Date.fromObject(obj.date));
  obj.address && jspb.Message.setWrapperField(
      msg, 10, proto.speech.tts.Address.fromObject(obj.address));
  obj.telephone && jspb.Message.setWrapperField(
      msg, 11, proto.speech.tts.Telephone.fromObject(obj.telephone));
  obj.money && jspb.Message.setWrapperField(
      msg, 12, proto.speech.tts.Money.fromObject(obj.money));
  obj.electronic && jspb.Message.setWrapperField(
      msg, 13, proto.speech.tts.Electronic.fromObject(obj.electronic));
  obj.chemicalFormula && jspb.Message.setWrapperField(
      msg, 14, proto.speech.tts.ChemicalFormula.fromObject(obj.chemicalFormula));
  obj.words != null && jspb.Message.setField(msg, 16, obj.words);
  obj.letters != null && jspb.Message.setField(msg, 18, obj.letters);
  obj.verbatim != null && jspb.Message.setField(msg, 19, obj.verbatim);
  obj.pause != null && jspb.Message.setField(msg, 21, obj.pause);
  obj.spell && jspb.Message.setWrapperField(
      msg, 32, proto.speech.tts.Spelling.fromObject(obj.spell));
  obj.verbalInfo && jspb.Message.setWrapperField(
      msg, 25, proto.speech.tts.VerbalInfo.fromObject(obj.verbalInfo));
  obj.connector && jspb.Message.setWrapperField(
      msg, 36, proto.speech.tts.Connector.fromObject(obj.connector));
  obj.abbreviation && jspb.Message.setWrapperField(
      msg, 37, proto.speech.tts.Abbreviation.fromObject(obj.abbreviation));
  obj.emoticonEmoji && jspb.Message.setWrapperField(
      msg, 41, proto.speech.tts.EmoticonEmoji.fromObject(obj.emoticonEmoji));
  obj.wordSequenceList && jspb.Message.setRepeatedWrapperField(
      msg, 17, obj.wordSequenceList.map(
          proto.speech.tts.MarkupWord.fromObject));
  obj.voicemod && jspb.Message.setWrapperField(
      msg, 20, proto.speech.tts.VoiceMod.fromObject(obj.voicemod));
  obj.morphosyntacticFeatures != null && jspb.Message.setField(msg, 2, obj.morphosyntacticFeatures);
  obj.structuredFeatures && jspb.Message.setWrapperField(
      msg, 28, proto.speech.tts.MorphosyntacticFeatureVector.fromObject(obj.structuredFeatures));
  obj.prosodicFeatures && jspb.Message.setWrapperField(
      msg, 22, proto.speech.tts.ProsodicFeatures.fromObject(obj.prosodicFeatures));
  obj.location && jspb.Message.setWrapperField(
      msg, 23, proto.location.unified.LocationDescriptor.fromObject(obj.location));
  obj.knowledgeGraphMidList != null && jspb.Message.setField(msg, 34, obj.knowledgeGraphMidList);
  obj.conciseEmoji != null && jspb.Message.setField(msg, 24, obj.conciseEmoji);
  obj.audioSourceList && jspb.Message.setRepeatedWrapperField(
      msg, 26, obj.audioSourceList.map(
          proto.speech.tts.AudioSource.fromObject));
  obj.crossLingual && jspb.Message.setWrapperField(
      msg, 27, proto.speech.tts.CrossLingual.fromObject(obj.crossLingual));
  obj.foreignScript != null && jspb.Message.setField(msg, 39, obj.foreignScript);
  obj.textnormLevel != null && jspb.Message.setField(msg, 29, obj.textnormLevel);
  obj.synthesizerSelection && jspb.Message.setWrapperField(
      msg, 30, proto.speech.tts.SynthesizerSelection.fromObject(obj.synthesizerSelection));
  obj.bleep != null && jspb.Message.setField(msg, 31, obj.bleep);
  obj.prefix != null && jspb.Message.setField(msg, 33, obj.prefix);
  obj.suffix != null && jspb.Message.setField(msg, 35, obj.suffix);
  obj.timepointName != null && jspb.Message.setField(msg, 38, obj.timepointName);
  obj.noCelebrity != null && jspb.Message.setField(msg, 40, obj.noCelebrity);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.Say}
 */
proto.speech.tts.Say.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.Say;
  return proto.speech.tts.Say.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.Say} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.Say}
 */
proto.speech.tts.Say.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setText(value);
      break;
    case 3:
      var value = new proto.speech.tts.Cardinal;
      reader.readMessage(value,proto.speech.tts.Cardinal.deserializeBinaryFromReader);
      msg.setCardinal(value);
      break;
    case 4:
      var value = new proto.speech.tts.Ordinal;
      reader.readMessage(value,proto.speech.tts.Ordinal.deserializeBinaryFromReader);
      msg.setOrdinal(value);
      break;
    case 5:
      var value = new proto.speech.tts.Decimal;
      reader.readMessage(value,proto.speech.tts.Decimal.deserializeBinaryFromReader);
      msg.setDecimal(value);
      break;
    case 6:
      var value = new proto.speech.tts.Fraction;
      reader.readMessage(value,proto.speech.tts.Fraction.deserializeBinaryFromReader);
      msg.setFraction(value);
      break;
    case 15:
      var value = /** @type {string} */ (reader.readString());
      msg.setDigit(value);
      break;
    case 7:
      var value = new proto.speech.tts.Time;
      reader.readMessage(value,proto.speech.tts.Time.deserializeBinaryFromReader);
      msg.setTime(value);
      break;
    case 8:
      var value = new proto.speech.tts.Measure;
      reader.readMessage(value,proto.speech.tts.Measure.deserializeBinaryFromReader);
      msg.setMeasure(value);
      break;
    case 9:
      var value = new proto.speech.tts.Date;
      reader.readMessage(value,proto.speech.tts.Date.deserializeBinaryFromReader);
      msg.setDate(value);
      break;
    case 10:
      var value = new proto.speech.tts.Address;
      reader.readMessage(value,proto.speech.tts.Address.deserializeBinaryFromReader);
      msg.setAddress(value);
      break;
    case 11:
      var value = new proto.speech.tts.Telephone;
      reader.readMessage(value,proto.speech.tts.Telephone.deserializeBinaryFromReader);
      msg.setTelephone(value);
      break;
    case 12:
      var value = new proto.speech.tts.Money;
      reader.readMessage(value,proto.speech.tts.Money.deserializeBinaryFromReader);
      msg.setMoney(value);
      break;
    case 13:
      var value = new proto.speech.tts.Electronic;
      reader.readMessage(value,proto.speech.tts.Electronic.deserializeBinaryFromReader);
      msg.setElectronic(value);
      break;
    case 14:
      var value = new proto.speech.tts.ChemicalFormula;
      reader.readMessage(value,proto.speech.tts.ChemicalFormula.deserializeBinaryFromReader);
      msg.setChemicalFormula(value);
      break;
    case 16:
      var value = /** @type {string} */ (reader.readString());
      msg.setWords(value);
      break;
    case 18:
      var value = /** @type {string} */ (reader.readString());
      msg.setLetters(value);
      break;
    case 19:
      var value = /** @type {string} */ (reader.readString());
      msg.setVerbatim(value);
      break;
    case 21:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setPause(value);
      break;
    case 32:
      var value = new proto.speech.tts.Spelling;
      reader.readMessage(value,proto.speech.tts.Spelling.deserializeBinaryFromReader);
      msg.setSpell(value);
      break;
    case 25:
      var value = new proto.speech.tts.VerbalInfo;
      reader.readMessage(value,proto.speech.tts.VerbalInfo.deserializeBinaryFromReader);
      msg.setVerbalInfo(value);
      break;
    case 36:
      var value = new proto.speech.tts.Connector;
      reader.readMessage(value,proto.speech.tts.Connector.deserializeBinaryFromReader);
      msg.setConnector(value);
      break;
    case 37:
      var value = new proto.speech.tts.Abbreviation;
      reader.readMessage(value,proto.speech.tts.Abbreviation.deserializeBinaryFromReader);
      msg.setAbbreviation(value);
      break;
    case 41:
      var value = new proto.speech.tts.EmoticonEmoji;
      reader.readMessage(value,proto.speech.tts.EmoticonEmoji.deserializeBinaryFromReader);
      msg.setEmoticonEmoji(value);
      break;
    case 17:
      var value = new proto.speech.tts.MarkupWord;
      reader.readMessage(value,proto.speech.tts.MarkupWord.deserializeBinaryFromReader);
      msg.addWordSequence(value);
      break;
    case 20:
      var value = new proto.speech.tts.VoiceMod;
      reader.readMessage(value,proto.speech.tts.VoiceMod.deserializeBinaryFromReader);
      msg.setVoicemod(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setMorphosyntacticFeatures(value);
      break;
    case 28:
      var value = new proto.speech.tts.MorphosyntacticFeatureVector;
      reader.readMessage(value,proto.speech.tts.MorphosyntacticFeatureVector.deserializeBinaryFromReader);
      msg.setStructuredFeatures(value);
      break;
    case 22:
      var value = new proto.speech.tts.ProsodicFeatures;
      reader.readMessage(value,proto.speech.tts.ProsodicFeatures.deserializeBinaryFromReader);
      msg.setProsodicFeatures(value);
      break;
    case 23:
      var value = new proto.location.unified.LocationDescriptor;
      reader.readMessage(value,proto.location.unified.LocationDescriptor.deserializeBinaryFromReader);
      msg.setLocation(value);
      break;
    case 34:
      var value = /** @type {string} */ (reader.readString());
      msg.addKnowledgeGraphMid(value);
      break;
    case 24:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setConciseEmoji(value);
      break;
    case 26:
      var value = new proto.speech.tts.AudioSource;
      reader.readMessage(value,proto.speech.tts.AudioSource.deserializeBinaryFromReader);
      msg.addAudioSource(value);
      break;
    case 27:
      var value = new proto.speech.tts.CrossLingual;
      reader.readMessage(value,proto.speech.tts.CrossLingual.deserializeBinaryFromReader);
      msg.setCrossLingual(value);
      break;
    case 39:
      var value = /** @type {string} */ (reader.readString());
      msg.setForeignScript(value);
      break;
    case 29:
      var value = /** @type {!proto.speech.tts.TextnormLevel} */ (reader.readEnum());
      msg.setTextnormLevel(value);
      break;
    case 30:
      var value = new proto.speech.tts.SynthesizerSelection;
      reader.readMessage(value,proto.speech.tts.SynthesizerSelection.deserializeBinaryFromReader);
      msg.setSynthesizerSelection(value);
      break;
    case 31:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setBleep(value);
      break;
    case 33:
      var value = /** @type {string} */ (reader.readString());
      msg.setPrefix(value);
      break;
    case 35:
      var value = /** @type {string} */ (reader.readString());
      msg.setSuffix(value);
      break;
    case 38:
      var value = /** @type {string} */ (reader.readString());
      msg.setTimepointName(value);
      break;
    case 40:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setNoCelebrity(value);
      break;
    default:
      jspb.Message.readBinaryExtension(msg, reader,
        proto.speech.tts.Say.extensionsBinary,
        proto.speech.tts.Say.prototype.getExtension,
        proto.speech.tts.Say.prototype.setExtension);
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.Say.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.Say.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.Say} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Say.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = message.getCardinal();
  if (f != null) {
    writer.writeMessage(
      3,
      f,
      proto.speech.tts.Cardinal.serializeBinaryToWriter
    );
  }
  f = message.getOrdinal();
  if (f != null) {
    writer.writeMessage(
      4,
      f,
      proto.speech.tts.Ordinal.serializeBinaryToWriter
    );
  }
  f = message.getDecimal();
  if (f != null) {
    writer.writeMessage(
      5,
      f,
      proto.speech.tts.Decimal.serializeBinaryToWriter
    );
  }
  f = message.getFraction();
  if (f != null) {
    writer.writeMessage(
      6,
      f,
      proto.speech.tts.Fraction.serializeBinaryToWriter
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 15));
  if (f != null) {
    writer.writeString(
      15,
      f
    );
  }
  f = message.getTime();
  if (f != null) {
    writer.writeMessage(
      7,
      f,
      proto.speech.tts.Time.serializeBinaryToWriter
    );
  }
  f = message.getMeasure();
  if (f != null) {
    writer.writeMessage(
      8,
      f,
      proto.speech.tts.Measure.serializeBinaryToWriter
    );
  }
  f = message.getDate();
  if (f != null) {
    writer.writeMessage(
      9,
      f,
      proto.speech.tts.Date.serializeBinaryToWriter
    );
  }
  f = message.getAddress();
  if (f != null) {
    writer.writeMessage(
      10,
      f,
      proto.speech.tts.Address.serializeBinaryToWriter
    );
  }
  f = message.getTelephone();
  if (f != null) {
    writer.writeMessage(
      11,
      f,
      proto.speech.tts.Telephone.serializeBinaryToWriter
    );
  }
  f = message.getMoney();
  if (f != null) {
    writer.writeMessage(
      12,
      f,
      proto.speech.tts.Money.serializeBinaryToWriter
    );
  }
  f = message.getElectronic();
  if (f != null) {
    writer.writeMessage(
      13,
      f,
      proto.speech.tts.Electronic.serializeBinaryToWriter
    );
  }
  f = message.getChemicalFormula();
  if (f != null) {
    writer.writeMessage(
      14,
      f,
      proto.speech.tts.ChemicalFormula.serializeBinaryToWriter
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 16));
  if (f != null) {
    writer.writeString(
      16,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 18));
  if (f != null) {
    writer.writeString(
      18,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 19));
  if (f != null) {
    writer.writeString(
      19,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 21));
  if (f != null) {
    writer.writeFloat(
      21,
      f
    );
  }
  f = message.getSpell();
  if (f != null) {
    writer.writeMessage(
      32,
      f,
      proto.speech.tts.Spelling.serializeBinaryToWriter
    );
  }
  f = message.getVerbalInfo();
  if (f != null) {
    writer.writeMessage(
      25,
      f,
      proto.speech.tts.VerbalInfo.serializeBinaryToWriter
    );
  }
  f = message.getConnector();
  if (f != null) {
    writer.writeMessage(
      36,
      f,
      proto.speech.tts.Connector.serializeBinaryToWriter
    );
  }
  f = message.getAbbreviation();
  if (f != null) {
    writer.writeMessage(
      37,
      f,
      proto.speech.tts.Abbreviation.serializeBinaryToWriter
    );
  }
  f = message.getEmoticonEmoji();
  if (f != null) {
    writer.writeMessage(
      41,
      f,
      proto.speech.tts.EmoticonEmoji.serializeBinaryToWriter
    );
  }
  f = message.getWordSequenceList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      17,
      f,
      proto.speech.tts.MarkupWord.serializeBinaryToWriter
    );
  }
  f = message.getVoicemod();
  if (f != null) {
    writer.writeMessage(
      20,
      f,
      proto.speech.tts.VoiceMod.serializeBinaryToWriter
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
  f = message.getStructuredFeatures();
  if (f != null) {
    writer.writeMessage(
      28,
      f,
      proto.speech.tts.MorphosyntacticFeatureVector.serializeBinaryToWriter
    );
  }
  f = message.getProsodicFeatures();
  if (f != null) {
    writer.writeMessage(
      22,
      f,
      proto.speech.tts.ProsodicFeatures.serializeBinaryToWriter
    );
  }
  f = message.getLocation();
  if (f != null) {
    writer.writeMessage(
      23,
      f,
      proto.location.unified.LocationDescriptor.serializeBinaryToWriter
    );
  }
  f = message.getKnowledgeGraphMidList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      34,
      f
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 24));
  if (f != null) {
    writer.writeBool(
      24,
      f
    );
  }
  f = message.getAudioSourceList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      26,
      f,
      proto.speech.tts.AudioSource.serializeBinaryToWriter
    );
  }
  f = message.getCrossLingual();
  if (f != null) {
    writer.writeMessage(
      27,
      f,
      proto.speech.tts.CrossLingual.serializeBinaryToWriter
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 39));
  if (f != null) {
    writer.writeString(
      39,
      f
    );
  }
  f = /** @type {!proto.speech.tts.TextnormLevel} */ (jspb.Message.getField(message, 29));
  if (f != null) {
    writer.writeEnum(
      29,
      f
    );
  }
  f = message.getSynthesizerSelection();
  if (f != null) {
    writer.writeMessage(
      30,
      f,
      proto.speech.tts.SynthesizerSelection.serializeBinaryToWriter
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 31));
  if (f != null) {
    writer.writeBool(
      31,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 33));
  if (f != null) {
    writer.writeString(
      33,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 35));
  if (f != null) {
    writer.writeString(
      35,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 38));
  if (f != null) {
    writer.writeString(
      38,
      f
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 40));
  if (f != null) {
    writer.writeBool(
      40,
      f
    );
  }
  jspb.Message.serializeBinaryExtensions(message, writer,
    proto.speech.tts.Say.extensionsBinary, proto.speech.tts.Say.prototype.getExtension);
};


/**
 * optional string text = 1;
 * @return {?string}
 */
proto.speech.tts.Say.prototype.getText = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.setText = function(value) {
  return jspb.Message.setOneofField(this, 1, proto.speech.tts.Say.oneofGroups_[0], value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.clearText = function() {
  return jspb.Message.setOneofField(this, 1, proto.speech.tts.Say.oneofGroups_[0], undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Say.prototype.hasText = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional Cardinal cardinal = 3;
 * @return {?proto.speech.tts.Cardinal}
 */
proto.speech.tts.Say.prototype.getCardinal = function() {
  return /** @type{?proto.speech.tts.Cardinal} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Cardinal, 3));
};


/**
 * @param {?proto.speech.tts.Cardinal|undefined} value
 * @return {!proto.speech.tts.Say} returns this
*/
proto.speech.tts.Say.prototype.setCardinal = function(value) {
  return jspb.Message.setOneofWrapperField(this, 3, proto.speech.tts.Say.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.clearCardinal = function() {
  return this.setCardinal(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Say.prototype.hasCardinal = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional Ordinal ordinal = 4;
 * @return {?proto.speech.tts.Ordinal}
 */
proto.speech.tts.Say.prototype.getOrdinal = function() {
  return /** @type{?proto.speech.tts.Ordinal} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Ordinal, 4));
};


/**
 * @param {?proto.speech.tts.Ordinal|undefined} value
 * @return {!proto.speech.tts.Say} returns this
*/
proto.speech.tts.Say.prototype.setOrdinal = function(value) {
  return jspb.Message.setOneofWrapperField(this, 4, proto.speech.tts.Say.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.clearOrdinal = function() {
  return this.setOrdinal(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Say.prototype.hasOrdinal = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional Decimal decimal = 5;
 * @return {?proto.speech.tts.Decimal}
 */
proto.speech.tts.Say.prototype.getDecimal = function() {
  return /** @type{?proto.speech.tts.Decimal} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Decimal, 5));
};


/**
 * @param {?proto.speech.tts.Decimal|undefined} value
 * @return {!proto.speech.tts.Say} returns this
*/
proto.speech.tts.Say.prototype.setDecimal = function(value) {
  return jspb.Message.setOneofWrapperField(this, 5, proto.speech.tts.Say.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.clearDecimal = function() {
  return this.setDecimal(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Say.prototype.hasDecimal = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional Fraction fraction = 6;
 * @return {?proto.speech.tts.Fraction}
 */
proto.speech.tts.Say.prototype.getFraction = function() {
  return /** @type{?proto.speech.tts.Fraction} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Fraction, 6));
};


/**
 * @param {?proto.speech.tts.Fraction|undefined} value
 * @return {!proto.speech.tts.Say} returns this
*/
proto.speech.tts.Say.prototype.setFraction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 6, proto.speech.tts.Say.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.clearFraction = function() {
  return this.setFraction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Say.prototype.hasFraction = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional string digit = 15;
 * @return {?string}
 */
proto.speech.tts.Say.prototype.getDigit = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 15));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.setDigit = function(value) {
  return jspb.Message.setOneofField(this, 15, proto.speech.tts.Say.oneofGroups_[0], value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.clearDigit = function() {
  return jspb.Message.setOneofField(this, 15, proto.speech.tts.Say.oneofGroups_[0], undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Say.prototype.hasDigit = function() {
  return jspb.Message.getField(this, 15) != null;
};


/**
 * optional Time time = 7;
 * @return {?proto.speech.tts.Time}
 */
proto.speech.tts.Say.prototype.getTime = function() {
  return /** @type{?proto.speech.tts.Time} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Time, 7));
};


/**
 * @param {?proto.speech.tts.Time|undefined} value
 * @return {!proto.speech.tts.Say} returns this
*/
proto.speech.tts.Say.prototype.setTime = function(value) {
  return jspb.Message.setOneofWrapperField(this, 7, proto.speech.tts.Say.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.clearTime = function() {
  return this.setTime(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Say.prototype.hasTime = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * optional Measure measure = 8;
 * @return {?proto.speech.tts.Measure}
 */
proto.speech.tts.Say.prototype.getMeasure = function() {
  return /** @type{?proto.speech.tts.Measure} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Measure, 8));
};


/**
 * @param {?proto.speech.tts.Measure|undefined} value
 * @return {!proto.speech.tts.Say} returns this
*/
proto.speech.tts.Say.prototype.setMeasure = function(value) {
  return jspb.Message.setOneofWrapperField(this, 8, proto.speech.tts.Say.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.clearMeasure = function() {
  return this.setMeasure(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Say.prototype.hasMeasure = function() {
  return jspb.Message.getField(this, 8) != null;
};


/**
 * optional Date date = 9;
 * @return {?proto.speech.tts.Date}
 */
proto.speech.tts.Say.prototype.getDate = function() {
  return /** @type{?proto.speech.tts.Date} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Date, 9));
};


/**
 * @param {?proto.speech.tts.Date|undefined} value
 * @return {!proto.speech.tts.Say} returns this
*/
proto.speech.tts.Say.prototype.setDate = function(value) {
  return jspb.Message.setOneofWrapperField(this, 9, proto.speech.tts.Say.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.clearDate = function() {
  return this.setDate(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Say.prototype.hasDate = function() {
  return jspb.Message.getField(this, 9) != null;
};


/**
 * optional Address address = 10;
 * @return {?proto.speech.tts.Address}
 */
proto.speech.tts.Say.prototype.getAddress = function() {
  return /** @type{?proto.speech.tts.Address} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Address, 10));
};


/**
 * @param {?proto.speech.tts.Address|undefined} value
 * @return {!proto.speech.tts.Say} returns this
*/
proto.speech.tts.Say.prototype.setAddress = function(value) {
  return jspb.Message.setOneofWrapperField(this, 10, proto.speech.tts.Say.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.clearAddress = function() {
  return this.setAddress(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Say.prototype.hasAddress = function() {
  return jspb.Message.getField(this, 10) != null;
};


/**
 * optional Telephone telephone = 11;
 * @return {?proto.speech.tts.Telephone}
 */
proto.speech.tts.Say.prototype.getTelephone = function() {
  return /** @type{?proto.speech.tts.Telephone} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Telephone, 11));
};


/**
 * @param {?proto.speech.tts.Telephone|undefined} value
 * @return {!proto.speech.tts.Say} returns this
*/
proto.speech.tts.Say.prototype.setTelephone = function(value) {
  return jspb.Message.setOneofWrapperField(this, 11, proto.speech.tts.Say.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.clearTelephone = function() {
  return this.setTelephone(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Say.prototype.hasTelephone = function() {
  return jspb.Message.getField(this, 11) != null;
};


/**
 * optional Money money = 12;
 * @return {?proto.speech.tts.Money}
 */
proto.speech.tts.Say.prototype.getMoney = function() {
  return /** @type{?proto.speech.tts.Money} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Money, 12));
};


/**
 * @param {?proto.speech.tts.Money|undefined} value
 * @return {!proto.speech.tts.Say} returns this
*/
proto.speech.tts.Say.prototype.setMoney = function(value) {
  return jspb.Message.setOneofWrapperField(this, 12, proto.speech.tts.Say.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.clearMoney = function() {
  return this.setMoney(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Say.prototype.hasMoney = function() {
  return jspb.Message.getField(this, 12) != null;
};


/**
 * optional Electronic electronic = 13;
 * @return {?proto.speech.tts.Electronic}
 */
proto.speech.tts.Say.prototype.getElectronic = function() {
  return /** @type{?proto.speech.tts.Electronic} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Electronic, 13));
};


/**
 * @param {?proto.speech.tts.Electronic|undefined} value
 * @return {!proto.speech.tts.Say} returns this
*/
proto.speech.tts.Say.prototype.setElectronic = function(value) {
  return jspb.Message.setOneofWrapperField(this, 13, proto.speech.tts.Say.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.clearElectronic = function() {
  return this.setElectronic(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Say.prototype.hasElectronic = function() {
  return jspb.Message.getField(this, 13) != null;
};


/**
 * optional ChemicalFormula chemical_formula = 14;
 * @return {?proto.speech.tts.ChemicalFormula}
 */
proto.speech.tts.Say.prototype.getChemicalFormula = function() {
  return /** @type{?proto.speech.tts.ChemicalFormula} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.ChemicalFormula, 14));
};


/**
 * @param {?proto.speech.tts.ChemicalFormula|undefined} value
 * @return {!proto.speech.tts.Say} returns this
*/
proto.speech.tts.Say.prototype.setChemicalFormula = function(value) {
  return jspb.Message.setOneofWrapperField(this, 14, proto.speech.tts.Say.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.clearChemicalFormula = function() {
  return this.setChemicalFormula(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Say.prototype.hasChemicalFormula = function() {
  return jspb.Message.getField(this, 14) != null;
};


/**
 * optional string words = 16;
 * @return {?string}
 */
proto.speech.tts.Say.prototype.getWords = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 16));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.setWords = function(value) {
  return jspb.Message.setOneofField(this, 16, proto.speech.tts.Say.oneofGroups_[0], value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.clearWords = function() {
  return jspb.Message.setOneofField(this, 16, proto.speech.tts.Say.oneofGroups_[0], undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Say.prototype.hasWords = function() {
  return jspb.Message.getField(this, 16) != null;
};


/**
 * optional string letters = 18;
 * @return {?string}
 */
proto.speech.tts.Say.prototype.getLetters = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 18));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.setLetters = function(value) {
  return jspb.Message.setOneofField(this, 18, proto.speech.tts.Say.oneofGroups_[0], value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.clearLetters = function() {
  return jspb.Message.setOneofField(this, 18, proto.speech.tts.Say.oneofGroups_[0], undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Say.prototype.hasLetters = function() {
  return jspb.Message.getField(this, 18) != null;
};


/**
 * optional string verbatim = 19;
 * @return {?string}
 */
proto.speech.tts.Say.prototype.getVerbatim = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 19));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.setVerbatim = function(value) {
  return jspb.Message.setOneofField(this, 19, proto.speech.tts.Say.oneofGroups_[0], value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.clearVerbatim = function() {
  return jspb.Message.setOneofField(this, 19, proto.speech.tts.Say.oneofGroups_[0], undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Say.prototype.hasVerbatim = function() {
  return jspb.Message.getField(this, 19) != null;
};


/**
 * optional float pause = 21;
 * @return {?number}
 */
proto.speech.tts.Say.prototype.getPause = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 21));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.setPause = function(value) {
  return jspb.Message.setOneofField(this, 21, proto.speech.tts.Say.oneofGroups_[0], value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.clearPause = function() {
  return jspb.Message.setOneofField(this, 21, proto.speech.tts.Say.oneofGroups_[0], undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Say.prototype.hasPause = function() {
  return jspb.Message.getField(this, 21) != null;
};


/**
 * optional Spelling spell = 32;
 * @return {?proto.speech.tts.Spelling}
 */
proto.speech.tts.Say.prototype.getSpell = function() {
  return /** @type{?proto.speech.tts.Spelling} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Spelling, 32));
};


/**
 * @param {?proto.speech.tts.Spelling|undefined} value
 * @return {!proto.speech.tts.Say} returns this
*/
proto.speech.tts.Say.prototype.setSpell = function(value) {
  return jspb.Message.setOneofWrapperField(this, 32, proto.speech.tts.Say.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.clearSpell = function() {
  return this.setSpell(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Say.prototype.hasSpell = function() {
  return jspb.Message.getField(this, 32) != null;
};


/**
 * optional VerbalInfo verbal_info = 25;
 * @return {?proto.speech.tts.VerbalInfo}
 */
proto.speech.tts.Say.prototype.getVerbalInfo = function() {
  return /** @type{?proto.speech.tts.VerbalInfo} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.VerbalInfo, 25));
};


/**
 * @param {?proto.speech.tts.VerbalInfo|undefined} value
 * @return {!proto.speech.tts.Say} returns this
*/
proto.speech.tts.Say.prototype.setVerbalInfo = function(value) {
  return jspb.Message.setOneofWrapperField(this, 25, proto.speech.tts.Say.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.clearVerbalInfo = function() {
  return this.setVerbalInfo(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Say.prototype.hasVerbalInfo = function() {
  return jspb.Message.getField(this, 25) != null;
};


/**
 * optional Connector connector = 36;
 * @return {?proto.speech.tts.Connector}
 */
proto.speech.tts.Say.prototype.getConnector = function() {
  return /** @type{?proto.speech.tts.Connector} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Connector, 36));
};


/**
 * @param {?proto.speech.tts.Connector|undefined} value
 * @return {!proto.speech.tts.Say} returns this
*/
proto.speech.tts.Say.prototype.setConnector = function(value) {
  return jspb.Message.setOneofWrapperField(this, 36, proto.speech.tts.Say.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.clearConnector = function() {
  return this.setConnector(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Say.prototype.hasConnector = function() {
  return jspb.Message.getField(this, 36) != null;
};


/**
 * optional Abbreviation abbreviation = 37;
 * @return {?proto.speech.tts.Abbreviation}
 */
proto.speech.tts.Say.prototype.getAbbreviation = function() {
  return /** @type{?proto.speech.tts.Abbreviation} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Abbreviation, 37));
};


/**
 * @param {?proto.speech.tts.Abbreviation|undefined} value
 * @return {!proto.speech.tts.Say} returns this
*/
proto.speech.tts.Say.prototype.setAbbreviation = function(value) {
  return jspb.Message.setOneofWrapperField(this, 37, proto.speech.tts.Say.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.clearAbbreviation = function() {
  return this.setAbbreviation(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Say.prototype.hasAbbreviation = function() {
  return jspb.Message.getField(this, 37) != null;
};


/**
 * optional EmoticonEmoji emoticon_emoji = 41;
 * @return {?proto.speech.tts.EmoticonEmoji}
 */
proto.speech.tts.Say.prototype.getEmoticonEmoji = function() {
  return /** @type{?proto.speech.tts.EmoticonEmoji} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.EmoticonEmoji, 41));
};


/**
 * @param {?proto.speech.tts.EmoticonEmoji|undefined} value
 * @return {!proto.speech.tts.Say} returns this
*/
proto.speech.tts.Say.prototype.setEmoticonEmoji = function(value) {
  return jspb.Message.setOneofWrapperField(this, 41, proto.speech.tts.Say.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.clearEmoticonEmoji = function() {
  return this.setEmoticonEmoji(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Say.prototype.hasEmoticonEmoji = function() {
  return jspb.Message.getField(this, 41) != null;
};


/**
 * repeated MarkupWord word_sequence = 17;
 * @return {!Array<!proto.speech.tts.MarkupWord>}
 */
proto.speech.tts.Say.prototype.getWordSequenceList = function() {
  return /** @type{!Array<!proto.speech.tts.MarkupWord>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.MarkupWord, 17));
};


/**
 * @param {Array<!proto.speech.tts.MarkupWord>|undefined} value
 * @return {!proto.speech.tts.Say} returns this
*/
proto.speech.tts.Say.prototype.setWordSequenceList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 17, value);
};


/**
 * @param {!proto.speech.tts.MarkupWord=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.MarkupWord}
 */
proto.speech.tts.Say.prototype.addWordSequence = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 17, opt_value, proto.speech.tts.MarkupWord, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.clearWordSequenceList = function() {
  return this.setWordSequenceList([]);
};


/**
 * optional VoiceMod voicemod = 20;
 * @return {?proto.speech.tts.VoiceMod}
 */
proto.speech.tts.Say.prototype.getVoicemod = function() {
  return /** @type{?proto.speech.tts.VoiceMod} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.VoiceMod, 20));
};


/**
 * @param {?proto.speech.tts.VoiceMod|undefined} value
 * @return {!proto.speech.tts.Say} returns this
*/
proto.speech.tts.Say.prototype.setVoicemod = function(value) {
  return jspb.Message.setWrapperField(this, 20, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.clearVoicemod = function() {
  return this.setVoicemod(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Say.prototype.hasVoicemod = function() {
  return jspb.Message.getField(this, 20) != null;
};


/**
 * optional string morphosyntactic_features = 2;
 * @return {?string}
 */
proto.speech.tts.Say.prototype.getMorphosyntacticFeatures = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.setMorphosyntacticFeatures = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.clearMorphosyntacticFeatures = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Say.prototype.hasMorphosyntacticFeatures = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional MorphosyntacticFeatureVector structured_features = 28;
 * @return {?proto.speech.tts.MorphosyntacticFeatureVector}
 */
proto.speech.tts.Say.prototype.getStructuredFeatures = function() {
  return /** @type{?proto.speech.tts.MorphosyntacticFeatureVector} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.MorphosyntacticFeatureVector, 28));
};


/**
 * @param {?proto.speech.tts.MorphosyntacticFeatureVector|undefined} value
 * @return {!proto.speech.tts.Say} returns this
*/
proto.speech.tts.Say.prototype.setStructuredFeatures = function(value) {
  return jspb.Message.setWrapperField(this, 28, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.clearStructuredFeatures = function() {
  return this.setStructuredFeatures(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Say.prototype.hasStructuredFeatures = function() {
  return jspb.Message.getField(this, 28) != null;
};


/**
 * optional ProsodicFeatures prosodic_features = 22;
 * @return {?proto.speech.tts.ProsodicFeatures}
 */
proto.speech.tts.Say.prototype.getProsodicFeatures = function() {
  return /** @type{?proto.speech.tts.ProsodicFeatures} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.ProsodicFeatures, 22));
};


/**
 * @param {?proto.speech.tts.ProsodicFeatures|undefined} value
 * @return {!proto.speech.tts.Say} returns this
*/
proto.speech.tts.Say.prototype.setProsodicFeatures = function(value) {
  return jspb.Message.setWrapperField(this, 22, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.clearProsodicFeatures = function() {
  return this.setProsodicFeatures(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Say.prototype.hasProsodicFeatures = function() {
  return jspb.Message.getField(this, 22) != null;
};


/**
 * optional location.unified.LocationDescriptor location = 23;
 * @return {?proto.location.unified.LocationDescriptor}
 */
proto.speech.tts.Say.prototype.getLocation = function() {
  return /** @type{?proto.location.unified.LocationDescriptor} */ (
    jspb.Message.getWrapperField(this, proto.location.unified.LocationDescriptor, 23));
};


/**
 * @param {?proto.location.unified.LocationDescriptor|undefined} value
 * @return {!proto.speech.tts.Say} returns this
*/
proto.speech.tts.Say.prototype.setLocation = function(value) {
  return jspb.Message.setWrapperField(this, 23, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.clearLocation = function() {
  return this.setLocation(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Say.prototype.hasLocation = function() {
  return jspb.Message.getField(this, 23) != null;
};


/**
 * repeated string knowledge_graph_mid = 34;
 * @return {!Array<string>}
 */
proto.speech.tts.Say.prototype.getKnowledgeGraphMidList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 34));
};


/**
 * @param {Array<string>|undefined} value
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.setKnowledgeGraphMidList = function(value) {
  return jspb.Message.setField(this, 34, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.addKnowledgeGraphMid = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 34, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.clearKnowledgeGraphMidList = function() {
  return this.setKnowledgeGraphMidList([]);
};


/**
 * optional bool concise_emoji = 24;
 * @return {?boolean}
 */
proto.speech.tts.Say.prototype.getConciseEmoji = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 24));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.setConciseEmoji = function(value) {
  return jspb.Message.setField(this, 24, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.clearConciseEmoji = function() {
  return jspb.Message.setField(this, 24, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Say.prototype.hasConciseEmoji = function() {
  return jspb.Message.getField(this, 24) != null;
};


/**
 * repeated AudioSource audio_source = 26;
 * @return {!Array<!proto.speech.tts.AudioSource>}
 */
proto.speech.tts.Say.prototype.getAudioSourceList = function() {
  return /** @type{!Array<!proto.speech.tts.AudioSource>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.AudioSource, 26));
};


/**
 * @param {Array<!proto.speech.tts.AudioSource>|undefined} value
 * @return {!proto.speech.tts.Say} returns this
*/
proto.speech.tts.Say.prototype.setAudioSourceList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 26, value);
};


/**
 * @param {!proto.speech.tts.AudioSource=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.AudioSource}
 */
proto.speech.tts.Say.prototype.addAudioSource = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 26, opt_value, proto.speech.tts.AudioSource, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.clearAudioSourceList = function() {
  return this.setAudioSourceList([]);
};


/**
 * optional CrossLingual cross_lingual = 27;
 * @return {?proto.speech.tts.CrossLingual}
 */
proto.speech.tts.Say.prototype.getCrossLingual = function() {
  return /** @type{?proto.speech.tts.CrossLingual} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.CrossLingual, 27));
};


/**
 * @param {?proto.speech.tts.CrossLingual|undefined} value
 * @return {!proto.speech.tts.Say} returns this
*/
proto.speech.tts.Say.prototype.setCrossLingual = function(value) {
  return jspb.Message.setWrapperField(this, 27, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.clearCrossLingual = function() {
  return this.setCrossLingual(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Say.prototype.hasCrossLingual = function() {
  return jspb.Message.getField(this, 27) != null;
};


/**
 * optional string foreign_script = 39;
 * @return {?string}
 */
proto.speech.tts.Say.prototype.getForeignScript = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 39));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.setForeignScript = function(value) {
  return jspb.Message.setField(this, 39, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.clearForeignScript = function() {
  return jspb.Message.setField(this, 39, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Say.prototype.hasForeignScript = function() {
  return jspb.Message.getField(this, 39) != null;
};


/**
 * optional TextnormLevel textnorm_level = 29;
 * @return {!proto.speech.tts.TextnormLevel}
 */
proto.speech.tts.Say.prototype.getTextnormLevel = function() {
  return /** @type {!proto.speech.tts.TextnormLevel} */ (jspb.Message.getField(this, 29));
};


/**
 * @param {?proto.speech.tts.TextnormLevel|undefined} value
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.setTextnormLevel = function(value) {
  return jspb.Message.setField(this, 29, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.clearTextnormLevel = function() {
  return jspb.Message.setField(this, 29, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Say.prototype.hasTextnormLevel = function() {
  return jspb.Message.getField(this, 29) != null;
};


/**
 * optional SynthesizerSelection synthesizer_selection = 30;
 * @return {?proto.speech.tts.SynthesizerSelection}
 */
proto.speech.tts.Say.prototype.getSynthesizerSelection = function() {
  return /** @type{?proto.speech.tts.SynthesizerSelection} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.SynthesizerSelection, 30));
};


/**
 * @param {?proto.speech.tts.SynthesizerSelection|undefined} value
 * @return {!proto.speech.tts.Say} returns this
*/
proto.speech.tts.Say.prototype.setSynthesizerSelection = function(value) {
  return jspb.Message.setWrapperField(this, 30, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.clearSynthesizerSelection = function() {
  return this.setSynthesizerSelection(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Say.prototype.hasSynthesizerSelection = function() {
  return jspb.Message.getField(this, 30) != null;
};


/**
 * optional bool bleep = 31;
 * @return {?boolean}
 */
proto.speech.tts.Say.prototype.getBleep = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 31));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.setBleep = function(value) {
  return jspb.Message.setField(this, 31, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.clearBleep = function() {
  return jspb.Message.setField(this, 31, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Say.prototype.hasBleep = function() {
  return jspb.Message.getField(this, 31) != null;
};


/**
 * optional string prefix = 33;
 * @return {?string}
 */
proto.speech.tts.Say.prototype.getPrefix = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 33));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.setPrefix = function(value) {
  return jspb.Message.setField(this, 33, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.clearPrefix = function() {
  return jspb.Message.setField(this, 33, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Say.prototype.hasPrefix = function() {
  return jspb.Message.getField(this, 33) != null;
};


/**
 * optional string suffix = 35;
 * @return {?string}
 */
proto.speech.tts.Say.prototype.getSuffix = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 35));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.setSuffix = function(value) {
  return jspb.Message.setField(this, 35, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.clearSuffix = function() {
  return jspb.Message.setField(this, 35, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Say.prototype.hasSuffix = function() {
  return jspb.Message.getField(this, 35) != null;
};


/**
 * optional string timepoint_name = 38;
 * @return {?string}
 */
proto.speech.tts.Say.prototype.getTimepointName = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 38));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.setTimepointName = function(value) {
  return jspb.Message.setField(this, 38, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.clearTimepointName = function() {
  return jspb.Message.setField(this, 38, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Say.prototype.hasTimepointName = function() {
  return jspb.Message.getField(this, 38) != null;
};


/**
 * optional bool no_celebrity = 40;
 * @return {?boolean}
 */
proto.speech.tts.Say.prototype.getNoCelebrity = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 40));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.setNoCelebrity = function(value) {
  return jspb.Message.setField(this, 40, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Say} returns this
 */
proto.speech.tts.Say.prototype.clearNoCelebrity = function() {
  return jspb.Message.setField(this, 40, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Say.prototype.hasNoCelebrity = function() {
  return jspb.Message.getField(this, 40) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.Say}
 */
proto.speech.tts.Say.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.Say, data);
};



//speech/tts/public/tts_markup_proto_jspb/context_sentence.jspb.js
// source: speech/tts/public/tts_markup.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.Context');
goog.provide('proto.speech.tts.Sentence');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.LexiconProto.Entry');
goog.require('proto.speech.tts.PartialSentenceProsody');
goog.require('proto.speech.tts.PerformanceOptions');
goog.require('proto.speech.tts.Say');
goog.require('proto.speech.tts.SpeechMorphingTargets');
goog.require('proto.speech.tts.SynthesizerSelection');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.Context = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.Context.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.Context, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.Context.displayName = 'proto.speech.tts.Context';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.Sentence = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 15, proto.speech.tts.Sentence.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.Sentence, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.Sentence.displayName = 'proto.speech.tts.Sentence';
}

/**
 * The extensions registered with this message class. This is a map of
 * extension field number to fieldInfo object.
 *
 * For example:
 *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }
 *
 * fieldName contains the JsCompiler renamed field name property so that it
 * works in OPTIMIZED mode.
 *
 * @type {!Object<number, jspb.ExtensionFieldInfo>}
 */
proto.speech.tts.Sentence.extensions = {};


/**
 * The extensions registered with this message class. This is a map of
 * extension field number to fieldInfo object.
 *
 * For example:
 *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }
 *
 * fieldName contains the JsCompiler renamed field name property so that it
 * works in OPTIMIZED mode.
 *
 * @type {!Object<number, jspb.ExtensionFieldBinaryInfo>}
 */
proto.speech.tts.Sentence.extensionsBinary = {};


/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.Context.repeatedFields_ = [1,2];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.Context.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.Context.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.Context} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Context.toObject = function(includeInstance, msg) {
  var f, obj = {
    previousSentenceList: jspb.Message.toObjectList(msg.getPreviousSentenceList(),
    proto.speech.tts.Sentence.toObject, includeInstance),
    nextSentenceList: jspb.Message.toObjectList(msg.getNextSentenceList(),
    proto.speech.tts.Sentence.toObject, includeInstance)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Context as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.Context.ObjectFormat = function() {
  /** @type {?Array<!proto.speech.tts.Sentence.ObjectFormat>|undefined} */
  this.previousSentenceList;

  /** @type {?Array<!proto.speech.tts.Sentence.ObjectFormat>|undefined} */
  this.nextSentenceList;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.Context.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.Context}
 */
proto.speech.tts.Context.fromObject = function(obj) {
  var msg = new proto.speech.tts.Context();
  obj.previousSentenceList && jspb.Message.setRepeatedWrapperField(
      msg, 1, obj.previousSentenceList.map(
          proto.speech.tts.Sentence.fromObject));
  obj.nextSentenceList && jspb.Message.setRepeatedWrapperField(
      msg, 2, obj.nextSentenceList.map(
          proto.speech.tts.Sentence.fromObject));
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.Context}
 */
proto.speech.tts.Context.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.Context;
  return proto.speech.tts.Context.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.Context} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.Context}
 */
proto.speech.tts.Context.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.speech.tts.Sentence;
      reader.readMessage(value,proto.speech.tts.Sentence.deserializeBinaryFromReader);
      msg.addPreviousSentence(value);
      break;
    case 2:
      var value = new proto.speech.tts.Sentence;
      reader.readMessage(value,proto.speech.tts.Sentence.deserializeBinaryFromReader);
      msg.addNextSentence(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.Context.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.Context.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.Context} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Context.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getPreviousSentenceList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      1,
      f,
      proto.speech.tts.Sentence.serializeBinaryToWriter
    );
  }
  f = message.getNextSentenceList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      2,
      f,
      proto.speech.tts.Sentence.serializeBinaryToWriter
    );
  }
};


/**
 * repeated Sentence previous_sentence = 1;
 * @return {!Array<!proto.speech.tts.Sentence>}
 */
proto.speech.tts.Context.prototype.getPreviousSentenceList = function() {
  return /** @type{!Array<!proto.speech.tts.Sentence>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.Sentence, 1));
};


/**
 * @param {Array<!proto.speech.tts.Sentence>|undefined} value
 * @return {!proto.speech.tts.Context} returns this
*/
proto.speech.tts.Context.prototype.setPreviousSentenceList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 1, value);
};


/**
 * @param {!proto.speech.tts.Sentence=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.Sentence}
 */
proto.speech.tts.Context.prototype.addPreviousSentence = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.speech.tts.Sentence, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.Context} returns this
 */
proto.speech.tts.Context.prototype.clearPreviousSentenceList = function() {
  return this.setPreviousSentenceList([]);
};


/**
 * repeated Sentence next_sentence = 2;
 * @return {!Array<!proto.speech.tts.Sentence>}
 */
proto.speech.tts.Context.prototype.getNextSentenceList = function() {
  return /** @type{!Array<!proto.speech.tts.Sentence>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.Sentence, 2));
};


/**
 * @param {Array<!proto.speech.tts.Sentence>|undefined} value
 * @return {!proto.speech.tts.Context} returns this
*/
proto.speech.tts.Context.prototype.setNextSentenceList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 2, value);
};


/**
 * @param {!proto.speech.tts.Sentence=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.Sentence}
 */
proto.speech.tts.Context.prototype.addNextSentence = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 2, opt_value, proto.speech.tts.Sentence, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.Context} returns this
 */
proto.speech.tts.Context.prototype.clearNextSentenceList = function() {
  return this.setNextSentenceList([]);
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.Context}
 */
proto.speech.tts.Context.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.Context, data);
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.Sentence.repeatedFields_ = [1,2];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.Sentence.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.Sentence.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.Sentence} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Sentence.toObject = function(includeInstance, msg) {
  var f, obj = {
    sayList: jspb.Message.toObjectList(msg.getSayList(),
    proto.speech.tts.Say.toObject, includeInstance),
    entryList: jspb.Message.toObjectList(msg.getEntryList(),
    proto.speech.tts.LexiconProto.Entry.toObject, includeInstance),
    morphing: (f = msg.getMorphing()) && proto.speech.tts.SpeechMorphingTargets.toObject(includeInstance, f),
    synthesizerSelection: (f = msg.getSynthesizerSelection()) && proto.speech.tts.SynthesizerSelection.toObject(includeInstance, f),
    outputGain: jspb.Message.getFloatingPointFieldWithDefault(msg, 8, 1.0),
    automaticSentenceSplitting: jspb.Message.getBooleanFieldWithDefault(msg, 9, true),
    context: (f = msg.getContext()) && proto.speech.tts.Context.toObject(includeInstance, f),
    messageTemplateId: (f = jspb.Message.getField(msg, 11)) == null ? undefined : f,
    performanceOptions: (f = msg.getPerformanceOptions()) && proto.speech.tts.PerformanceOptions.toObject(includeInstance, f),
    bowdlerize: (f = jspb.Message.getBooleanField(msg, 13)) == null ? undefined : f,
    partialSentenceProsody: (f = msg.getPartialSentenceProsody()) && proto.speech.tts.PartialSentenceProsody.toObject(includeInstance, f)
  };

  jspb.Message.toObjectExtension(/** @type {!jspb.Message} */ (msg), obj,
      proto.speech.tts.Sentence.extensions, proto.speech.tts.Sentence.prototype.getExtension,
      includeInstance);
  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Sentence as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.Sentence.ObjectFormat = function() {
  /** @type {?Array<!proto.speech.tts.Say.ObjectFormat>|undefined} */
  this.sayList;

  /** @type {?Array<!proto.speech.tts.LexiconProto.Entry.ObjectFormat>|undefined} */
  this.entryList;

  /** @type {?proto.speech.tts.SpeechMorphingTargets.ObjectFormat|undefined} */
  this.morphing;

  /** @type {?proto.speech.tts.SynthesizerSelection.ObjectFormat|undefined} */
  this.synthesizerSelection;

  /** @type {?number|undefined} */
  this.outputGain;

  /** @type {?boolean|undefined} */
  this.automaticSentenceSplitting;

  /** @type {?proto.speech.tts.Context.ObjectFormat|undefined} */
  this.context;

  /** @type {?string|undefined} */
  this.messageTemplateId;

  /** @type {?proto.speech.tts.PerformanceOptions.ObjectFormat|undefined} */
  this.performanceOptions;

  /** @type {?boolean|undefined} */
  this.bowdlerize;

  /** @type {?proto.speech.tts.PartialSentenceProsody.ObjectFormat|undefined} */
  this.partialSentenceProsody;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.Sentence.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.Sentence}
 */
proto.speech.tts.Sentence.fromObject = function(obj) {
  var msg = new proto.speech.tts.Sentence();
  obj.sayList && jspb.Message.setRepeatedWrapperField(
      msg, 1, obj.sayList.map(
          proto.speech.tts.Say.fromObject));
  obj.entryList && jspb.Message.setRepeatedWrapperField(
      msg, 2, obj.entryList.map(
          proto.speech.tts.LexiconProto.Entry.fromObject));
  obj.morphing && jspb.Message.setWrapperField(
      msg, 6, proto.speech.tts.SpeechMorphingTargets.fromObject(obj.morphing));
  obj.synthesizerSelection && jspb.Message.setWrapperField(
      msg, 7, proto.speech.tts.SynthesizerSelection.fromObject(obj.synthesizerSelection));
  obj.outputGain != null && jspb.Message.setField(msg, 8, obj.outputGain);
  obj.automaticSentenceSplitting != null && jspb.Message.setField(msg, 9, obj.automaticSentenceSplitting);
  obj.context && jspb.Message.setWrapperField(
      msg, 10, proto.speech.tts.Context.fromObject(obj.context));
  obj.messageTemplateId != null && jspb.Message.setField(msg, 11, obj.messageTemplateId);
  obj.performanceOptions && jspb.Message.setWrapperField(
      msg, 12, proto.speech.tts.PerformanceOptions.fromObject(obj.performanceOptions));
  obj.bowdlerize != null && jspb.Message.setField(msg, 13, obj.bowdlerize);
  obj.partialSentenceProsody && jspb.Message.setWrapperField(
      msg, 14, proto.speech.tts.PartialSentenceProsody.fromObject(obj.partialSentenceProsody));
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.Sentence}
 */
proto.speech.tts.Sentence.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.Sentence;
  return proto.speech.tts.Sentence.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.Sentence} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.Sentence}
 */
proto.speech.tts.Sentence.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.speech.tts.Say;
      reader.readMessage(value,proto.speech.tts.Say.deserializeBinaryFromReader);
      msg.addSay(value);
      break;
    case 2:
      var value = new proto.speech.tts.LexiconProto.Entry;
      reader.readMessage(value,proto.speech.tts.LexiconProto.Entry.deserializeBinaryFromReader);
      msg.addEntry(value);
      break;
    case 6:
      var value = new proto.speech.tts.SpeechMorphingTargets;
      reader.readMessage(value,proto.speech.tts.SpeechMorphingTargets.deserializeBinaryFromReader);
      msg.setMorphing(value);
      break;
    case 7:
      var value = new proto.speech.tts.SynthesizerSelection;
      reader.readMessage(value,proto.speech.tts.SynthesizerSelection.deserializeBinaryFromReader);
      msg.setSynthesizerSelection(value);
      break;
    case 8:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setOutputGain(value);
      break;
    case 9:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setAutomaticSentenceSplitting(value);
      break;
    case 10:
      var value = new proto.speech.tts.Context;
      reader.readMessage(value,proto.speech.tts.Context.deserializeBinaryFromReader);
      msg.setContext(value);
      break;
    case 11:
      var value = /** @type {string} */ (reader.readString());
      msg.setMessageTemplateId(value);
      break;
    case 12:
      var value = new proto.speech.tts.PerformanceOptions;
      reader.readMessage(value,proto.speech.tts.PerformanceOptions.deserializeBinaryFromReader);
      msg.setPerformanceOptions(value);
      break;
    case 13:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setBowdlerize(value);
      break;
    case 14:
      var value = new proto.speech.tts.PartialSentenceProsody;
      reader.readMessage(value,proto.speech.tts.PartialSentenceProsody.deserializeBinaryFromReader);
      msg.setPartialSentenceProsody(value);
      break;
    default:
      jspb.Message.readBinaryExtension(msg, reader,
        proto.speech.tts.Sentence.extensionsBinary,
        proto.speech.tts.Sentence.prototype.getExtension,
        proto.speech.tts.Sentence.prototype.setExtension);
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.Sentence.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.Sentence.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.Sentence} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Sentence.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getSayList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      1,
      f,
      proto.speech.tts.Say.serializeBinaryToWriter
    );
  }
  f = message.getEntryList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      2,
      f,
      proto.speech.tts.LexiconProto.Entry.serializeBinaryToWriter
    );
  }
  f = message.getMorphing();
  if (f != null) {
    writer.writeMessage(
      6,
      f,
      proto.speech.tts.SpeechMorphingTargets.serializeBinaryToWriter
    );
  }
  f = message.getSynthesizerSelection();
  if (f != null) {
    writer.writeMessage(
      7,
      f,
      proto.speech.tts.SynthesizerSelection.serializeBinaryToWriter
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 8));
  if (f != null) {
    writer.writeFloat(
      8,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 9));
  if (f != null) {
    writer.writeBool(
      9,
      f
    );
  }
  f = message.getContext();
  if (f != null) {
    writer.writeMessage(
      10,
      f,
      proto.speech.tts.Context.serializeBinaryToWriter
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 11));
  if (f != null) {
    writer.writeString(
      11,
      f
    );
  }
  f = message.getPerformanceOptions();
  if (f != null) {
    writer.writeMessage(
      12,
      f,
      proto.speech.tts.PerformanceOptions.serializeBinaryToWriter
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 13));
  if (f != null) {
    writer.writeBool(
      13,
      f
    );
  }
  f = message.getPartialSentenceProsody();
  if (f != null) {
    writer.writeMessage(
      14,
      f,
      proto.speech.tts.PartialSentenceProsody.serializeBinaryToWriter
    );
  }
  jspb.Message.serializeBinaryExtensions(message, writer,
    proto.speech.tts.Sentence.extensionsBinary, proto.speech.tts.Sentence.prototype.getExtension);
};


/**
 * repeated Say say = 1;
 * @return {!Array<!proto.speech.tts.Say>}
 */
proto.speech.tts.Sentence.prototype.getSayList = function() {
  return /** @type{!Array<!proto.speech.tts.Say>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.Say, 1));
};


/**
 * @param {Array<!proto.speech.tts.Say>|undefined} value
 * @return {!proto.speech.tts.Sentence} returns this
*/
proto.speech.tts.Sentence.prototype.setSayList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 1, value);
};


/**
 * @param {!proto.speech.tts.Say=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.Say}
 */
proto.speech.tts.Sentence.prototype.addSay = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.speech.tts.Say, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.Sentence} returns this
 */
proto.speech.tts.Sentence.prototype.clearSayList = function() {
  return this.setSayList([]);
};


/**
 * repeated LexiconProto.Entry entry = 2;
 * @return {!Array<!proto.speech.tts.LexiconProto.Entry>}
 */
proto.speech.tts.Sentence.prototype.getEntryList = function() {
  return /** @type{!Array<!proto.speech.tts.LexiconProto.Entry>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.LexiconProto.Entry, 2));
};


/**
 * @param {Array<!proto.speech.tts.LexiconProto.Entry>|undefined} value
 * @return {!proto.speech.tts.Sentence} returns this
*/
proto.speech.tts.Sentence.prototype.setEntryList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 2, value);
};


/**
 * @param {!proto.speech.tts.LexiconProto.Entry=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.LexiconProto.Entry}
 */
proto.speech.tts.Sentence.prototype.addEntry = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 2, opt_value, proto.speech.tts.LexiconProto.Entry, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.Sentence} returns this
 */
proto.speech.tts.Sentence.prototype.clearEntryList = function() {
  return this.setEntryList([]);
};


/**
 * optional SpeechMorphingTargets morphing = 6;
 * @return {?proto.speech.tts.SpeechMorphingTargets}
 */
proto.speech.tts.Sentence.prototype.getMorphing = function() {
  return /** @type{?proto.speech.tts.SpeechMorphingTargets} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.SpeechMorphingTargets, 6));
};


/**
 * @param {?proto.speech.tts.SpeechMorphingTargets|undefined} value
 * @return {!proto.speech.tts.Sentence} returns this
*/
proto.speech.tts.Sentence.prototype.setMorphing = function(value) {
  return jspb.Message.setWrapperField(this, 6, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Sentence} returns this
 */
proto.speech.tts.Sentence.prototype.clearMorphing = function() {
  return this.setMorphing(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Sentence.prototype.hasMorphing = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional SynthesizerSelection synthesizer_selection = 7;
 * @return {?proto.speech.tts.SynthesizerSelection}
 */
proto.speech.tts.Sentence.prototype.getSynthesizerSelection = function() {
  return /** @type{?proto.speech.tts.SynthesizerSelection} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.SynthesizerSelection, 7));
};


/**
 * @param {?proto.speech.tts.SynthesizerSelection|undefined} value
 * @return {!proto.speech.tts.Sentence} returns this
*/
proto.speech.tts.Sentence.prototype.setSynthesizerSelection = function(value) {
  return jspb.Message.setWrapperField(this, 7, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Sentence} returns this
 */
proto.speech.tts.Sentence.prototype.clearSynthesizerSelection = function() {
  return this.setSynthesizerSelection(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Sentence.prototype.hasSynthesizerSelection = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * optional float output_gain = 8;
 * @return {number}
 */
proto.speech.tts.Sentence.prototype.getOutputGain = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 8, 1.0));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Sentence} returns this
 */
proto.speech.tts.Sentence.prototype.setOutputGain = function(value) {
  return jspb.Message.setField(this, 8, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Sentence} returns this
 */
proto.speech.tts.Sentence.prototype.clearOutputGain = function() {
  return jspb.Message.setField(this, 8, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Sentence.prototype.hasOutputGain = function() {
  return jspb.Message.getField(this, 8) != null;
};


/**
 * optional bool automatic_sentence_splitting = 9;
 * @return {boolean}
 */
proto.speech.tts.Sentence.prototype.getAutomaticSentenceSplitting = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 9, true));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.Sentence} returns this
 */
proto.speech.tts.Sentence.prototype.setAutomaticSentenceSplitting = function(value) {
  return jspb.Message.setField(this, 9, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Sentence} returns this
 */
proto.speech.tts.Sentence.prototype.clearAutomaticSentenceSplitting = function() {
  return jspb.Message.setField(this, 9, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Sentence.prototype.hasAutomaticSentenceSplitting = function() {
  return jspb.Message.getField(this, 9) != null;
};


/**
 * optional Context context = 10;
 * @return {?proto.speech.tts.Context}
 */
proto.speech.tts.Sentence.prototype.getContext = function() {
  return /** @type{?proto.speech.tts.Context} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Context, 10));
};


/**
 * @param {?proto.speech.tts.Context|undefined} value
 * @return {!proto.speech.tts.Sentence} returns this
*/
proto.speech.tts.Sentence.prototype.setContext = function(value) {
  return jspb.Message.setWrapperField(this, 10, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Sentence} returns this
 */
proto.speech.tts.Sentence.prototype.clearContext = function() {
  return this.setContext(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Sentence.prototype.hasContext = function() {
  return jspb.Message.getField(this, 10) != null;
};


/**
 * optional string message_template_id = 11;
 * @return {?string}
 */
proto.speech.tts.Sentence.prototype.getMessageTemplateId = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 11));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Sentence} returns this
 */
proto.speech.tts.Sentence.prototype.setMessageTemplateId = function(value) {
  return jspb.Message.setField(this, 11, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Sentence} returns this
 */
proto.speech.tts.Sentence.prototype.clearMessageTemplateId = function() {
  return jspb.Message.setField(this, 11, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Sentence.prototype.hasMessageTemplateId = function() {
  return jspb.Message.getField(this, 11) != null;
};


/**
 * optional PerformanceOptions performance_options = 12;
 * @return {?proto.speech.tts.PerformanceOptions}
 */
proto.speech.tts.Sentence.prototype.getPerformanceOptions = function() {
  return /** @type{?proto.speech.tts.PerformanceOptions} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.PerformanceOptions, 12));
};


/**
 * @param {?proto.speech.tts.PerformanceOptions|undefined} value
 * @return {!proto.speech.tts.Sentence} returns this
*/
proto.speech.tts.Sentence.prototype.setPerformanceOptions = function(value) {
  return jspb.Message.setWrapperField(this, 12, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Sentence} returns this
 */
proto.speech.tts.Sentence.prototype.clearPerformanceOptions = function() {
  return this.setPerformanceOptions(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Sentence.prototype.hasPerformanceOptions = function() {
  return jspb.Message.getField(this, 12) != null;
};


/**
 * optional bool bowdlerize = 13;
 * @return {?boolean}
 */
proto.speech.tts.Sentence.prototype.getBowdlerize = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 13));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.Sentence} returns this
 */
proto.speech.tts.Sentence.prototype.setBowdlerize = function(value) {
  return jspb.Message.setField(this, 13, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Sentence} returns this
 */
proto.speech.tts.Sentence.prototype.clearBowdlerize = function() {
  return jspb.Message.setField(this, 13, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Sentence.prototype.hasBowdlerize = function() {
  return jspb.Message.getField(this, 13) != null;
};


/**
 * optional PartialSentenceProsody partial_sentence_prosody = 14;
 * @return {?proto.speech.tts.PartialSentenceProsody}
 */
proto.speech.tts.Sentence.prototype.getPartialSentenceProsody = function() {
  return /** @type{?proto.speech.tts.PartialSentenceProsody} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.PartialSentenceProsody, 14));
};


/**
 * @param {?proto.speech.tts.PartialSentenceProsody|undefined} value
 * @return {!proto.speech.tts.Sentence} returns this
*/
proto.speech.tts.Sentence.prototype.setPartialSentenceProsody = function(value) {
  return jspb.Message.setWrapperField(this, 14, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Sentence} returns this
 */
proto.speech.tts.Sentence.prototype.clearPartialSentenceProsody = function() {
  return this.setPartialSentenceProsody(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Sentence.prototype.hasPartialSentenceProsody = function() {
  return jspb.Message.getField(this, 14) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.Sentence}
 */
proto.speech.tts.Sentence.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.Sentence, data);
};



//speech/tts/engine/textnorm/textnorm_diagnostics_proto_jspb/pipelinestatediagnostics.jspb.js
// source: speech/tts/engine/textnorm/textnorm_diagnostics.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.PipelineStateDiagnostics');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.LinguisticStructure');
goog.require('proto.speech.tts.Sentence');
goog.require('proto.speech.tts.TextNormalizationData');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.PipelineStateDiagnostics = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.PipelineStateDiagnostics, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.PipelineStateDiagnostics.displayName = 'proto.speech.tts.PipelineStateDiagnostics';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.PipelineStateDiagnostics.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.PipelineStateDiagnostics.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.PipelineStateDiagnostics} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.PipelineStateDiagnostics.toObject = function(includeInstance, msg) {
  var f, obj = {
    stage: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    sentence: (f = msg.getSentence()) && proto.speech.tts.Sentence.toObject(includeInstance, f),
    textnormData: (f = msg.getTextnormData()) && proto.speech.tts.TextNormalizationData.toObject(includeInstance, f),
    linguistic: (f = msg.getLinguistic()) && proto.speech.tts.LinguisticStructure.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of PipelineStateDiagnostics as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.PipelineStateDiagnostics.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.stage;

  /** @type {?proto.speech.tts.Sentence.ObjectFormat|undefined} */
  this.sentence;

  /** @type {?proto.speech.tts.TextNormalizationData.ObjectFormat|undefined} */
  this.textnormData;

  /** @type {?proto.speech.tts.LinguisticStructure.ObjectFormat|undefined} */
  this.linguistic;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.PipelineStateDiagnostics.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.PipelineStateDiagnostics}
 */
proto.speech.tts.PipelineStateDiagnostics.fromObject = function(obj) {
  var msg = new proto.speech.tts.PipelineStateDiagnostics();
  obj.stage != null && jspb.Message.setField(msg, 1, obj.stage);
  obj.sentence && jspb.Message.setWrapperField(
      msg, 2, proto.speech.tts.Sentence.fromObject(obj.sentence));
  obj.textnormData && jspb.Message.setWrapperField(
      msg, 3, proto.speech.tts.TextNormalizationData.fromObject(obj.textnormData));
  obj.linguistic && jspb.Message.setWrapperField(
      msg, 4, proto.speech.tts.LinguisticStructure.fromObject(obj.linguistic));
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.PipelineStateDiagnostics}
 */
proto.speech.tts.PipelineStateDiagnostics.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.PipelineStateDiagnostics;
  return proto.speech.tts.PipelineStateDiagnostics.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.PipelineStateDiagnostics} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.PipelineStateDiagnostics}
 */
proto.speech.tts.PipelineStateDiagnostics.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setStage(value);
      break;
    case 2:
      var value = new proto.speech.tts.Sentence;
      reader.readMessage(value,proto.speech.tts.Sentence.deserializeBinaryFromReader);
      msg.setSentence(value);
      break;
    case 3:
      var value = new proto.speech.tts.TextNormalizationData;
      reader.readMessage(value,proto.speech.tts.TextNormalizationData.deserializeBinaryFromReader);
      msg.setTextnormData(value);
      break;
    case 4:
      var value = new proto.speech.tts.LinguisticStructure;
      reader.readMessage(value,proto.speech.tts.LinguisticStructure.deserializeBinaryFromReader);
      msg.setLinguistic(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.PipelineStateDiagnostics.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.PipelineStateDiagnostics.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.PipelineStateDiagnostics} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.PipelineStateDiagnostics.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = message.getSentence();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.speech.tts.Sentence.serializeBinaryToWriter
    );
  }
  f = message.getTextnormData();
  if (f != null) {
    writer.writeMessage(
      3,
      f,
      proto.speech.tts.TextNormalizationData.serializeBinaryToWriter
    );
  }
  f = message.getLinguistic();
  if (f != null) {
    writer.writeMessage(
      4,
      f,
      proto.speech.tts.LinguisticStructure.serializeBinaryToWriter
    );
  }
};


/**
 * optional string stage = 1;
 * @return {?string}
 */
proto.speech.tts.PipelineStateDiagnostics.prototype.getStage = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.PipelineStateDiagnostics} returns this
 */
proto.speech.tts.PipelineStateDiagnostics.prototype.setStage = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.PipelineStateDiagnostics} returns this
 */
proto.speech.tts.PipelineStateDiagnostics.prototype.clearStage = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.PipelineStateDiagnostics.prototype.hasStage = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional Sentence sentence = 2;
 * @return {?proto.speech.tts.Sentence}
 */
proto.speech.tts.PipelineStateDiagnostics.prototype.getSentence = function() {
  return /** @type{?proto.speech.tts.Sentence} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Sentence, 2));
};


/**
 * @param {?proto.speech.tts.Sentence|undefined} value
 * @return {!proto.speech.tts.PipelineStateDiagnostics} returns this
*/
proto.speech.tts.PipelineStateDiagnostics.prototype.setSentence = function(value) {
  return jspb.Message.setWrapperField(this, 2, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.PipelineStateDiagnostics} returns this
 */
proto.speech.tts.PipelineStateDiagnostics.prototype.clearSentence = function() {
  return this.setSentence(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.PipelineStateDiagnostics.prototype.hasSentence = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional TextNormalizationData textnorm_data = 3;
 * @return {?proto.speech.tts.TextNormalizationData}
 */
proto.speech.tts.PipelineStateDiagnostics.prototype.getTextnormData = function() {
  return /** @type{?proto.speech.tts.TextNormalizationData} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.TextNormalizationData, 3));
};


/**
 * @param {?proto.speech.tts.TextNormalizationData|undefined} value
 * @return {!proto.speech.tts.PipelineStateDiagnostics} returns this
*/
proto.speech.tts.PipelineStateDiagnostics.prototype.setTextnormData = function(value) {
  return jspb.Message.setWrapperField(this, 3, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.PipelineStateDiagnostics} returns this
 */
proto.speech.tts.PipelineStateDiagnostics.prototype.clearTextnormData = function() {
  return this.setTextnormData(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.PipelineStateDiagnostics.prototype.hasTextnormData = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional LinguisticStructure linguistic = 4;
 * @return {?proto.speech.tts.LinguisticStructure}
 */
proto.speech.tts.PipelineStateDiagnostics.prototype.getLinguistic = function() {
  return /** @type{?proto.speech.tts.LinguisticStructure} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.LinguisticStructure, 4));
};


/**
 * @param {?proto.speech.tts.LinguisticStructure|undefined} value
 * @return {!proto.speech.tts.PipelineStateDiagnostics} returns this
*/
proto.speech.tts.PipelineStateDiagnostics.prototype.setLinguistic = function(value) {
  return jspb.Message.setWrapperField(this, 4, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.PipelineStateDiagnostics} returns this
 */
proto.speech.tts.PipelineStateDiagnostics.prototype.clearLinguistic = function() {
  return this.setLinguistic(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.PipelineStateDiagnostics.prototype.hasLinguistic = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.PipelineStateDiagnostics}
 */
proto.speech.tts.PipelineStateDiagnostics.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.PipelineStateDiagnostics, data);
};



//speech/tts/engine/textnorm/textnorm_diagnostics_proto_jspb/specificationfeaturediagnostics.jspb.js
// source: speech/tts/engine/textnorm/textnorm_diagnostics.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.SpecificationFeatureDiagnostics');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.SpecificationFeatureDiagnostics = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.SpecificationFeatureDiagnostics.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.SpecificationFeatureDiagnostics, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.SpecificationFeatureDiagnostics.displayName = 'proto.speech.tts.SpecificationFeatureDiagnostics';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.SpecificationFeatureDiagnostics.repeatedFields_ = [2];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.SpecificationFeatureDiagnostics.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.SpecificationFeatureDiagnostics.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.SpecificationFeatureDiagnostics} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.SpecificationFeatureDiagnostics.toObject = function(includeInstance, msg) {
  var f, obj = {
    basetype: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    featureDescriptionList: (f = jspb.Message.getRepeatedField(msg, 2)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of SpecificationFeatureDiagnostics as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.SpecificationFeatureDiagnostics.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.basetype;

  /** @type {?Array<string>|undefined} */
  this.featureDescriptionList;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.SpecificationFeatureDiagnostics.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.SpecificationFeatureDiagnostics}
 */
proto.speech.tts.SpecificationFeatureDiagnostics.fromObject = function(obj) {
  var msg = new proto.speech.tts.SpecificationFeatureDiagnostics();
  obj.basetype != null && jspb.Message.setField(msg, 1, obj.basetype);
  obj.featureDescriptionList != null && jspb.Message.setField(msg, 2, obj.featureDescriptionList);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.SpecificationFeatureDiagnostics}
 */
proto.speech.tts.SpecificationFeatureDiagnostics.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.SpecificationFeatureDiagnostics;
  return proto.speech.tts.SpecificationFeatureDiagnostics.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.SpecificationFeatureDiagnostics} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.SpecificationFeatureDiagnostics}
 */
proto.speech.tts.SpecificationFeatureDiagnostics.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setBasetype(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.addFeatureDescription(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.SpecificationFeatureDiagnostics.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.SpecificationFeatureDiagnostics.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.SpecificationFeatureDiagnostics} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.SpecificationFeatureDiagnostics.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = message.getFeatureDescriptionList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      2,
      f
    );
  }
};


/**
 * optional string basetype = 1;
 * @return {?string}
 */
proto.speech.tts.SpecificationFeatureDiagnostics.prototype.getBasetype = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.SpecificationFeatureDiagnostics} returns this
 */
proto.speech.tts.SpecificationFeatureDiagnostics.prototype.setBasetype = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.SpecificationFeatureDiagnostics} returns this
 */
proto.speech.tts.SpecificationFeatureDiagnostics.prototype.clearBasetype = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.SpecificationFeatureDiagnostics.prototype.hasBasetype = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * repeated string feature_description = 2;
 * @return {!Array<string>}
 */
proto.speech.tts.SpecificationFeatureDiagnostics.prototype.getFeatureDescriptionList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 2));
};


/**
 * @param {Array<string>|undefined} value
 * @return {!proto.speech.tts.SpecificationFeatureDiagnostics} returns this
 */
proto.speech.tts.SpecificationFeatureDiagnostics.prototype.setFeatureDescriptionList = function(value) {
  return jspb.Message.setField(this, 2, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.SpecificationFeatureDiagnostics} returns this
 */
proto.speech.tts.SpecificationFeatureDiagnostics.prototype.addFeatureDescription = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 2, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.SpecificationFeatureDiagnostics} returns this
 */
proto.speech.tts.SpecificationFeatureDiagnostics.prototype.clearFeatureDescriptionList = function() {
  return this.setFeatureDescriptionList([]);
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.SpecificationFeatureDiagnostics}
 */
proto.speech.tts.SpecificationFeatureDiagnostics.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.SpecificationFeatureDiagnostics, data);
};



//speech/tts/engine/textnorm/textnorm_diagnostics_proto_jspb/streamdiagnostics_textnormdiagnostics_textnormoperation.jspb.js
// source: speech/tts/engine/textnorm/textnorm_diagnostics.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.StreamDiagnostics');
goog.provide('proto.speech.tts.TextnormDiagnostics');
goog.provide('proto.speech.tts.TextnormOperation');
goog.provide('proto.speech.tts.TextnormOperation.DiagnosticCase');
goog.provide('proto.speech.tts.TextnormOperation.LogSeverity');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.GraphvizDiagnostics');
goog.require('proto.speech.tts.GrmApplicationDiagnostics');
goog.require('proto.speech.tts.LucidStateDiagnostics');
goog.require('proto.speech.tts.PipelineStateDiagnostics');
goog.require('proto.speech.tts.SpecificationFeatureDiagnostics');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.StreamDiagnostics = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.StreamDiagnostics.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.StreamDiagnostics, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.StreamDiagnostics.displayName = 'proto.speech.tts.StreamDiagnostics';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.TextnormDiagnostics = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.TextnormDiagnostics.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.TextnormDiagnostics, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.TextnormDiagnostics.displayName = 'proto.speech.tts.TextnormDiagnostics';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.TextnormOperation = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, proto.speech.tts.TextnormOperation.oneofGroups_);
};
goog.inherits(proto.speech.tts.TextnormOperation, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.TextnormOperation.displayName = 'proto.speech.tts.TextnormOperation';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.StreamDiagnostics.repeatedFields_ = [2];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.StreamDiagnostics.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.StreamDiagnostics.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.StreamDiagnostics} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.StreamDiagnostics.toObject = function(includeInstance, msg) {
  var f, obj = {
    component: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    operationList: jspb.Message.toObjectList(msg.getOperationList(),
    proto.speech.tts.TextnormOperation.toObject, includeInstance)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of StreamDiagnostics as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.StreamDiagnostics.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.component;

  /** @type {?Array<!proto.speech.tts.TextnormOperation.ObjectFormat>|undefined} */
  this.operationList;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.StreamDiagnostics.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.StreamDiagnostics}
 */
proto.speech.tts.StreamDiagnostics.fromObject = function(obj) {
  var msg = new proto.speech.tts.StreamDiagnostics();
  obj.component != null && jspb.Message.setField(msg, 1, obj.component);
  obj.operationList && jspb.Message.setRepeatedWrapperField(
      msg, 2, obj.operationList.map(
          proto.speech.tts.TextnormOperation.fromObject));
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.StreamDiagnostics}
 */
proto.speech.tts.StreamDiagnostics.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.StreamDiagnostics;
  return proto.speech.tts.StreamDiagnostics.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.StreamDiagnostics} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.StreamDiagnostics}
 */
proto.speech.tts.StreamDiagnostics.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setComponent(value);
      break;
    case 2:
      var value = new proto.speech.tts.TextnormOperation;
      reader.readMessage(value,proto.speech.tts.TextnormOperation.deserializeBinaryFromReader);
      msg.addOperation(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.StreamDiagnostics.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.StreamDiagnostics.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.StreamDiagnostics} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.StreamDiagnostics.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = message.getOperationList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      2,
      f,
      proto.speech.tts.TextnormOperation.serializeBinaryToWriter
    );
  }
};


/**
 * optional string component = 1;
 * @return {?string}
 */
proto.speech.tts.StreamDiagnostics.prototype.getComponent = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.StreamDiagnostics} returns this
 */
proto.speech.tts.StreamDiagnostics.prototype.setComponent = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.StreamDiagnostics} returns this
 */
proto.speech.tts.StreamDiagnostics.prototype.clearComponent = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.StreamDiagnostics.prototype.hasComponent = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * repeated TextnormOperation operation = 2;
 * @return {!Array<!proto.speech.tts.TextnormOperation>}
 */
proto.speech.tts.StreamDiagnostics.prototype.getOperationList = function() {
  return /** @type{!Array<!proto.speech.tts.TextnormOperation>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.TextnormOperation, 2));
};


/**
 * @param {Array<!proto.speech.tts.TextnormOperation>|undefined} value
 * @return {!proto.speech.tts.StreamDiagnostics} returns this
*/
proto.speech.tts.StreamDiagnostics.prototype.setOperationList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 2, value);
};


/**
 * @param {!proto.speech.tts.TextnormOperation=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.TextnormOperation}
 */
proto.speech.tts.StreamDiagnostics.prototype.addOperation = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 2, opt_value, proto.speech.tts.TextnormOperation, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.StreamDiagnostics} returns this
 */
proto.speech.tts.StreamDiagnostics.prototype.clearOperationList = function() {
  return this.setOperationList([]);
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.StreamDiagnostics}
 */
proto.speech.tts.StreamDiagnostics.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.StreamDiagnostics, data);
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.TextnormDiagnostics.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.TextnormDiagnostics.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.TextnormDiagnostics.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.TextnormDiagnostics} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.TextnormDiagnostics.toObject = function(includeInstance, msg) {
  var f, obj = {
    streamDiagnosticsList: jspb.Message.toObjectList(msg.getStreamDiagnosticsList(),
    proto.speech.tts.StreamDiagnostics.toObject, includeInstance)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of TextnormDiagnostics as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.TextnormDiagnostics.ObjectFormat = function() {
  /** @type {?Array<!proto.speech.tts.StreamDiagnostics.ObjectFormat>|undefined} */
  this.streamDiagnosticsList;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.TextnormDiagnostics.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.TextnormDiagnostics}
 */
proto.speech.tts.TextnormDiagnostics.fromObject = function(obj) {
  var msg = new proto.speech.tts.TextnormDiagnostics();
  obj.streamDiagnosticsList && jspb.Message.setRepeatedWrapperField(
      msg, 1, obj.streamDiagnosticsList.map(
          proto.speech.tts.StreamDiagnostics.fromObject));
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.TextnormDiagnostics}
 */
proto.speech.tts.TextnormDiagnostics.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.TextnormDiagnostics;
  return proto.speech.tts.TextnormDiagnostics.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.TextnormDiagnostics} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.TextnormDiagnostics}
 */
proto.speech.tts.TextnormDiagnostics.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.speech.tts.StreamDiagnostics;
      reader.readMessage(value,proto.speech.tts.StreamDiagnostics.deserializeBinaryFromReader);
      msg.addStreamDiagnostics(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.TextnormDiagnostics.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.TextnormDiagnostics.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.TextnormDiagnostics} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.TextnormDiagnostics.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getStreamDiagnosticsList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      1,
      f,
      proto.speech.tts.StreamDiagnostics.serializeBinaryToWriter
    );
  }
};


/**
 * repeated StreamDiagnostics stream_diagnostics = 1;
 * @return {!Array<!proto.speech.tts.StreamDiagnostics>}
 */
proto.speech.tts.TextnormDiagnostics.prototype.getStreamDiagnosticsList = function() {
  return /** @type{!Array<!proto.speech.tts.StreamDiagnostics>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.StreamDiagnostics, 1));
};


/**
 * @param {Array<!proto.speech.tts.StreamDiagnostics>|undefined} value
 * @return {!proto.speech.tts.TextnormDiagnostics} returns this
*/
proto.speech.tts.TextnormDiagnostics.prototype.setStreamDiagnosticsList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 1, value);
};


/**
 * @param {!proto.speech.tts.StreamDiagnostics=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.StreamDiagnostics}
 */
proto.speech.tts.TextnormDiagnostics.prototype.addStreamDiagnostics = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.speech.tts.StreamDiagnostics, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.TextnormDiagnostics} returns this
 */
proto.speech.tts.TextnormDiagnostics.prototype.clearStreamDiagnosticsList = function() {
  return this.setStreamDiagnosticsList([]);
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.TextnormDiagnostics}
 */
proto.speech.tts.TextnormDiagnostics.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.TextnormDiagnostics, data);
};



/**
 * Oneof group definitions for this message. Each group defines the field
 * numbers belonging to that group. When of these fields' value is set, all
 * other fields in the group are cleared. During deserialization, if multiple
 * fields are encountered for a group, only the last value seen will be kept.
 * @private {!Array<!Array<number>>}
 * @const
 */
proto.speech.tts.TextnormOperation.oneofGroups_ = [[1,2,4,5,6,8,7]];

/**
 * @enum {number}
 */
proto.speech.tts.TextnormOperation.DiagnosticCase = {
  DIAGNOSTIC_NOT_SET: 0,
  GRM_APPLICATION_DIAGNOSTIC: 1,
  CODE_DIAGNOSTIC: 2,
  RECURSIVE_DIAGNOSTIC: 4,
  STATE_DIAGNOSTIC: 5,
  SPECIFICATION_DIAGNOSTIC: 6,
  GRAPHVIZ_DIAGNOSTIC: 8,
  LUCID_STATE_DIAGNOSTIC: 7
};

/**
 * @return {!proto.speech.tts.TextnormOperation.DiagnosticCase}
 */
proto.speech.tts.TextnormOperation.prototype.getDiagnosticCase = function() {
  return /** @type {!proto.speech.tts.TextnormOperation.DiagnosticCase} */(jspb.Message.computeOneofCase(this, proto.speech.tts.TextnormOperation.oneofGroups_[0]));
};



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.TextnormOperation.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.TextnormOperation.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.TextnormOperation} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.TextnormOperation.toObject = function(includeInstance, msg) {
  var f, obj = {
    grmApplicationDiagnostic: (f = msg.getGrmApplicationDiagnostic()) && proto.speech.tts.GrmApplicationDiagnostics.toObject(includeInstance, f),
    codeDiagnostic: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    recursiveDiagnostic: (f = msg.getRecursiveDiagnostic()) && proto.speech.tts.TextnormDiagnostics.toObject(includeInstance, f),
    stateDiagnostic: (f = msg.getStateDiagnostic()) && proto.speech.tts.PipelineStateDiagnostics.toObject(includeInstance, f),
    specificationDiagnostic: (f = msg.getSpecificationDiagnostic()) && proto.speech.tts.SpecificationFeatureDiagnostics.toObject(includeInstance, f),
    graphvizDiagnostic: (f = msg.getGraphvizDiagnostic()) && proto.speech.tts.GraphvizDiagnostics.toObject(includeInstance, f),
    lucidStateDiagnostic: (f = msg.getLucidStateDiagnostic()) && proto.speech.tts.LucidStateDiagnostics.toObject(includeInstance, f),
    severity: (f = jspb.Message.getField(msg, 20)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of TextnormOperation as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.TextnormOperation.ObjectFormat = function() {
  /** @type {?proto.speech.tts.GrmApplicationDiagnostics.ObjectFormat|undefined} */
  this.grmApplicationDiagnostic;

  /** @type {?string|undefined} */
  this.codeDiagnostic;

  /** @type {?proto.speech.tts.TextnormDiagnostics.ObjectFormat|undefined} */
  this.recursiveDiagnostic;

  /** @type {?proto.speech.tts.PipelineStateDiagnostics.ObjectFormat|undefined} */
  this.stateDiagnostic;

  /** @type {?proto.speech.tts.SpecificationFeatureDiagnostics.ObjectFormat|undefined} */
  this.specificationDiagnostic;

  /** @type {?proto.speech.tts.GraphvizDiagnostics.ObjectFormat|undefined} */
  this.graphvizDiagnostic;

  /** @type {?proto.speech.tts.LucidStateDiagnostics.ObjectFormat|undefined} */
  this.lucidStateDiagnostic;

  /** @type {?number|undefined} */
  this.severity;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.TextnormOperation.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.TextnormOperation}
 */
proto.speech.tts.TextnormOperation.fromObject = function(obj) {
  var msg = new proto.speech.tts.TextnormOperation();
  obj.grmApplicationDiagnostic && jspb.Message.setWrapperField(
      msg, 1, proto.speech.tts.GrmApplicationDiagnostics.fromObject(obj.grmApplicationDiagnostic));
  obj.codeDiagnostic != null && jspb.Message.setField(msg, 2, obj.codeDiagnostic);
  obj.recursiveDiagnostic && jspb.Message.setWrapperField(
      msg, 4, proto.speech.tts.TextnormDiagnostics.fromObject(obj.recursiveDiagnostic));
  obj.stateDiagnostic && jspb.Message.setWrapperField(
      msg, 5, proto.speech.tts.PipelineStateDiagnostics.fromObject(obj.stateDiagnostic));
  obj.specificationDiagnostic && jspb.Message.setWrapperField(
      msg, 6, proto.speech.tts.SpecificationFeatureDiagnostics.fromObject(obj.specificationDiagnostic));
  obj.graphvizDiagnostic && jspb.Message.setWrapperField(
      msg, 8, proto.speech.tts.GraphvizDiagnostics.fromObject(obj.graphvizDiagnostic));
  obj.lucidStateDiagnostic && jspb.Message.setWrapperField(
      msg, 7, proto.speech.tts.LucidStateDiagnostics.fromObject(obj.lucidStateDiagnostic));
  obj.severity != null && jspb.Message.setField(msg, 20, obj.severity);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.TextnormOperation}
 */
proto.speech.tts.TextnormOperation.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.TextnormOperation;
  return proto.speech.tts.TextnormOperation.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.TextnormOperation} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.TextnormOperation}
 */
proto.speech.tts.TextnormOperation.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.speech.tts.GrmApplicationDiagnostics;
      reader.readMessage(value,proto.speech.tts.GrmApplicationDiagnostics.deserializeBinaryFromReader);
      msg.setGrmApplicationDiagnostic(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setCodeDiagnostic(value);
      break;
    case 4:
      var value = new proto.speech.tts.TextnormDiagnostics;
      reader.readMessage(value,proto.speech.tts.TextnormDiagnostics.deserializeBinaryFromReader);
      msg.setRecursiveDiagnostic(value);
      break;
    case 5:
      var value = new proto.speech.tts.PipelineStateDiagnostics;
      reader.readMessage(value,proto.speech.tts.PipelineStateDiagnostics.deserializeBinaryFromReader);
      msg.setStateDiagnostic(value);
      break;
    case 6:
      var value = new proto.speech.tts.SpecificationFeatureDiagnostics;
      reader.readMessage(value,proto.speech.tts.SpecificationFeatureDiagnostics.deserializeBinaryFromReader);
      msg.setSpecificationDiagnostic(value);
      break;
    case 8:
      var value = new proto.speech.tts.GraphvizDiagnostics;
      reader.readMessage(value,proto.speech.tts.GraphvizDiagnostics.deserializeBinaryFromReader);
      msg.setGraphvizDiagnostic(value);
      break;
    case 7:
      var value = new proto.speech.tts.LucidStateDiagnostics;
      reader.readMessage(value,proto.speech.tts.LucidStateDiagnostics.deserializeBinaryFromReader);
      msg.setLucidStateDiagnostic(value);
      break;
    case 20:
      var value = /** @type {!proto.speech.tts.TextnormOperation.LogSeverity} */ (reader.readEnum());
      msg.setSeverity(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.TextnormOperation.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.TextnormOperation.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.TextnormOperation} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.TextnormOperation.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getGrmApplicationDiagnostic();
  if (f != null) {
    writer.writeMessage(
      1,
      f,
      proto.speech.tts.GrmApplicationDiagnostics.serializeBinaryToWriter
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
  f = message.getRecursiveDiagnostic();
  if (f != null) {
    writer.writeMessage(
      4,
      f,
      proto.speech.tts.TextnormDiagnostics.serializeBinaryToWriter
    );
  }
  f = message.getStateDiagnostic();
  if (f != null) {
    writer.writeMessage(
      5,
      f,
      proto.speech.tts.PipelineStateDiagnostics.serializeBinaryToWriter
    );
  }
  f = message.getSpecificationDiagnostic();
  if (f != null) {
    writer.writeMessage(
      6,
      f,
      proto.speech.tts.SpecificationFeatureDiagnostics.serializeBinaryToWriter
    );
  }
  f = message.getGraphvizDiagnostic();
  if (f != null) {
    writer.writeMessage(
      8,
      f,
      proto.speech.tts.GraphvizDiagnostics.serializeBinaryToWriter
    );
  }
  f = message.getLucidStateDiagnostic();
  if (f != null) {
    writer.writeMessage(
      7,
      f,
      proto.speech.tts.LucidStateDiagnostics.serializeBinaryToWriter
    );
  }
  f = /** @type {!proto.speech.tts.TextnormOperation.LogSeverity} */ (jspb.Message.getField(message, 20));
  if (f != null) {
    writer.writeEnum(
      20,
      f
    );
  }
};


/**
 * @enum {number}
 */
proto.speech.tts.TextnormOperation.LogSeverity = {
  INFO: 0,
  WARNING: 1,
  ERROR: 2
};

/**
 * optional GrmApplicationDiagnostics grm_application_diagnostic = 1;
 * @return {?proto.speech.tts.GrmApplicationDiagnostics}
 */
proto.speech.tts.TextnormOperation.prototype.getGrmApplicationDiagnostic = function() {
  return /** @type{?proto.speech.tts.GrmApplicationDiagnostics} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.GrmApplicationDiagnostics, 1));
};


/**
 * @param {?proto.speech.tts.GrmApplicationDiagnostics|undefined} value
 * @return {!proto.speech.tts.TextnormOperation} returns this
*/
proto.speech.tts.TextnormOperation.prototype.setGrmApplicationDiagnostic = function(value) {
  return jspb.Message.setOneofWrapperField(this, 1, proto.speech.tts.TextnormOperation.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.TextnormOperation} returns this
 */
proto.speech.tts.TextnormOperation.prototype.clearGrmApplicationDiagnostic = function() {
  return this.setGrmApplicationDiagnostic(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.TextnormOperation.prototype.hasGrmApplicationDiagnostic = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional string code_diagnostic = 2;
 * @return {?string}
 */
proto.speech.tts.TextnormOperation.prototype.getCodeDiagnostic = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.TextnormOperation} returns this
 */
proto.speech.tts.TextnormOperation.prototype.setCodeDiagnostic = function(value) {
  return jspb.Message.setOneofField(this, 2, proto.speech.tts.TextnormOperation.oneofGroups_[0], value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.TextnormOperation} returns this
 */
proto.speech.tts.TextnormOperation.prototype.clearCodeDiagnostic = function() {
  return jspb.Message.setOneofField(this, 2, proto.speech.tts.TextnormOperation.oneofGroups_[0], undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.TextnormOperation.prototype.hasCodeDiagnostic = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional TextnormDiagnostics recursive_diagnostic = 4;
 * @return {?proto.speech.tts.TextnormDiagnostics}
 */
proto.speech.tts.TextnormOperation.prototype.getRecursiveDiagnostic = function() {
  return /** @type{?proto.speech.tts.TextnormDiagnostics} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.TextnormDiagnostics, 4));
};


/**
 * @param {?proto.speech.tts.TextnormDiagnostics|undefined} value
 * @return {!proto.speech.tts.TextnormOperation} returns this
*/
proto.speech.tts.TextnormOperation.prototype.setRecursiveDiagnostic = function(value) {
  return jspb.Message.setOneofWrapperField(this, 4, proto.speech.tts.TextnormOperation.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.TextnormOperation} returns this
 */
proto.speech.tts.TextnormOperation.prototype.clearRecursiveDiagnostic = function() {
  return this.setRecursiveDiagnostic(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.TextnormOperation.prototype.hasRecursiveDiagnostic = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional PipelineStateDiagnostics state_diagnostic = 5;
 * @return {?proto.speech.tts.PipelineStateDiagnostics}
 */
proto.speech.tts.TextnormOperation.prototype.getStateDiagnostic = function() {
  return /** @type{?proto.speech.tts.PipelineStateDiagnostics} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.PipelineStateDiagnostics, 5));
};


/**
 * @param {?proto.speech.tts.PipelineStateDiagnostics|undefined} value
 * @return {!proto.speech.tts.TextnormOperation} returns this
*/
proto.speech.tts.TextnormOperation.prototype.setStateDiagnostic = function(value) {
  return jspb.Message.setOneofWrapperField(this, 5, proto.speech.tts.TextnormOperation.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.TextnormOperation} returns this
 */
proto.speech.tts.TextnormOperation.prototype.clearStateDiagnostic = function() {
  return this.setStateDiagnostic(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.TextnormOperation.prototype.hasStateDiagnostic = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional SpecificationFeatureDiagnostics specification_diagnostic = 6;
 * @return {?proto.speech.tts.SpecificationFeatureDiagnostics}
 */
proto.speech.tts.TextnormOperation.prototype.getSpecificationDiagnostic = function() {
  return /** @type{?proto.speech.tts.SpecificationFeatureDiagnostics} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.SpecificationFeatureDiagnostics, 6));
};


/**
 * @param {?proto.speech.tts.SpecificationFeatureDiagnostics|undefined} value
 * @return {!proto.speech.tts.TextnormOperation} returns this
*/
proto.speech.tts.TextnormOperation.prototype.setSpecificationDiagnostic = function(value) {
  return jspb.Message.setOneofWrapperField(this, 6, proto.speech.tts.TextnormOperation.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.TextnormOperation} returns this
 */
proto.speech.tts.TextnormOperation.prototype.clearSpecificationDiagnostic = function() {
  return this.setSpecificationDiagnostic(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.TextnormOperation.prototype.hasSpecificationDiagnostic = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional GraphvizDiagnostics graphviz_diagnostic = 8;
 * @return {?proto.speech.tts.GraphvizDiagnostics}
 */
proto.speech.tts.TextnormOperation.prototype.getGraphvizDiagnostic = function() {
  return /** @type{?proto.speech.tts.GraphvizDiagnostics} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.GraphvizDiagnostics, 8));
};


/**
 * @param {?proto.speech.tts.GraphvizDiagnostics|undefined} value
 * @return {!proto.speech.tts.TextnormOperation} returns this
*/
proto.speech.tts.TextnormOperation.prototype.setGraphvizDiagnostic = function(value) {
  return jspb.Message.setOneofWrapperField(this, 8, proto.speech.tts.TextnormOperation.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.TextnormOperation} returns this
 */
proto.speech.tts.TextnormOperation.prototype.clearGraphvizDiagnostic = function() {
  return this.setGraphvizDiagnostic(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.TextnormOperation.prototype.hasGraphvizDiagnostic = function() {
  return jspb.Message.getField(this, 8) != null;
};


/**
 * optional LucidStateDiagnostics lucid_state_diagnostic = 7;
 * @return {?proto.speech.tts.LucidStateDiagnostics}
 */
proto.speech.tts.TextnormOperation.prototype.getLucidStateDiagnostic = function() {
  return /** @type{?proto.speech.tts.LucidStateDiagnostics} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.LucidStateDiagnostics, 7));
};


/**
 * @param {?proto.speech.tts.LucidStateDiagnostics|undefined} value
 * @return {!proto.speech.tts.TextnormOperation} returns this
*/
proto.speech.tts.TextnormOperation.prototype.setLucidStateDiagnostic = function(value) {
  return jspb.Message.setOneofWrapperField(this, 7, proto.speech.tts.TextnormOperation.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.TextnormOperation} returns this
 */
proto.speech.tts.TextnormOperation.prototype.clearLucidStateDiagnostic = function() {
  return this.setLucidStateDiagnostic(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.TextnormOperation.prototype.hasLucidStateDiagnostic = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * optional LogSeverity severity = 20;
 * @return {!proto.speech.tts.TextnormOperation.LogSeverity}
 */
proto.speech.tts.TextnormOperation.prototype.getSeverity = function() {
  return /** @type {!proto.speech.tts.TextnormOperation.LogSeverity} */ (jspb.Message.getField(this, 20));
};


/**
 * @param {?proto.speech.tts.TextnormOperation.LogSeverity|undefined} value
 * @return {!proto.speech.tts.TextnormOperation} returns this
 */
proto.speech.tts.TextnormOperation.prototype.setSeverity = function(value) {
  return jspb.Message.setField(this, 20, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.TextnormOperation} returns this
 */
proto.speech.tts.TextnormOperation.prototype.clearSeverity = function() {
  return jspb.Message.setField(this, 20, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.TextnormOperation.prototype.hasSeverity = function() {
  return jspb.Message.getField(this, 20) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.TextnormOperation}
 */
proto.speech.tts.TextnormOperation.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.TextnormOperation, data);
};



//speech/tts/engine/utterance/features_proto_jspb/customfeature.jspb.js
// source: speech/tts/engine/utterance/features.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.CustomFeature');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

goog.forwardDeclare('proto.speech.tts.LinguisticFeatureValType');
goog.forwardDeclare('proto.speech.tts.NavigationFeature.FeatureName');
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.CustomFeature = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.CustomFeature, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.CustomFeature.displayName = 'proto.speech.tts.CustomFeature';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.CustomFeature.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.CustomFeature.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.CustomFeature} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.CustomFeature.toObject = function(includeInstance, msg) {
  var f, obj = {
    feature: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    featureEnum: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,
    type: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of CustomFeature as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.CustomFeature.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.feature;

  /** @type {?number|undefined} */
  this.featureEnum;

  /** @type {?number|undefined} */
  this.type;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.CustomFeature.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.CustomFeature}
 */
proto.speech.tts.CustomFeature.fromObject = function(obj) {
  var msg = new proto.speech.tts.CustomFeature();
  obj.feature != null && jspb.Message.setField(msg, 1, obj.feature);
  obj.featureEnum != null && jspb.Message.setField(msg, 3, obj.featureEnum);
  obj.type != null && jspb.Message.setField(msg, 2, obj.type);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.CustomFeature}
 */
proto.speech.tts.CustomFeature.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.CustomFeature;
  return proto.speech.tts.CustomFeature.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.CustomFeature} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.CustomFeature}
 */
proto.speech.tts.CustomFeature.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setFeature(value);
      break;
    case 3:
      var value = /** @type {!proto.speech.tts.NavigationFeature.FeatureName} */ (reader.readEnum());
      msg.setFeatureEnum(value);
      break;
    case 2:
      var value = /** @type {!proto.speech.tts.LinguisticFeatureValType} */ (reader.readEnum());
      msg.setType(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.CustomFeature.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.CustomFeature.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.CustomFeature} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.CustomFeature.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = /** @type {!proto.speech.tts.NavigationFeature.FeatureName} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeEnum(
      3,
      f
    );
  }
  f = /** @type {!proto.speech.tts.LinguisticFeatureValType} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeEnum(
      2,
      f
    );
  }
};


/**
 * optional string feature = 1;
 * @return {?string}
 */
proto.speech.tts.CustomFeature.prototype.getFeature = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.CustomFeature} returns this
 */
proto.speech.tts.CustomFeature.prototype.setFeature = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.CustomFeature} returns this
 */
proto.speech.tts.CustomFeature.prototype.clearFeature = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.CustomFeature.prototype.hasFeature = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional NavigationFeature.FeatureName feature_enum = 3;
 * @return {!proto.speech.tts.NavigationFeature.FeatureName}
 */
proto.speech.tts.CustomFeature.prototype.getFeatureEnum = function() {
  return /** @type {!proto.speech.tts.NavigationFeature.FeatureName} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?proto.speech.tts.NavigationFeature.FeatureName|undefined} value
 * @return {!proto.speech.tts.CustomFeature} returns this
 */
proto.speech.tts.CustomFeature.prototype.setFeatureEnum = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.CustomFeature} returns this
 */
proto.speech.tts.CustomFeature.prototype.clearFeatureEnum = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.CustomFeature.prototype.hasFeatureEnum = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional LinguisticFeatureValType type = 2;
 * @return {!proto.speech.tts.LinguisticFeatureValType}
 */
proto.speech.tts.CustomFeature.prototype.getType = function() {
  return /** @type {!proto.speech.tts.LinguisticFeatureValType} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?proto.speech.tts.LinguisticFeatureValType|undefined} value
 * @return {!proto.speech.tts.CustomFeature} returns this
 */
proto.speech.tts.CustomFeature.prototype.setType = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.CustomFeature} returns this
 */
proto.speech.tts.CustomFeature.prototype.clearType = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.CustomFeature.prototype.hasType = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.CustomFeature}
 */
proto.speech.tts.CustomFeature.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.CustomFeature, data);
};



//speech/tts/engine/utterance/features_proto_jspb/navigationfeature.jspb.js
// source: speech/tts/engine/utterance/features.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.NavigationFeature');
goog.provide('proto.speech.tts.NavigationFeature.DiphoneDirection');
goog.provide('proto.speech.tts.NavigationFeature.FeatureName');
goog.provide('proto.speech.tts.NavigationFeature.StreamName');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

goog.forwardDeclare('proto.speech.tts.LinguisticFeatureValType');
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.NavigationFeature = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.NavigationFeature, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.NavigationFeature.displayName = 'proto.speech.tts.NavigationFeature';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.NavigationFeature.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.NavigationFeature.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.NavigationFeature} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.NavigationFeature.toObject = function(includeInstance, msg) {
  var f, obj = {
    feature: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f,
    featureEnum: (f = jspb.Message.getField(msg, 13)) == null ? undefined : f,
    type: (f = jspb.Message.getField(msg, 9)) == null ? undefined : f,
    stream: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    streamEnum: (f = jspb.Message.getField(msg, 12)) == null ? undefined : f,
    position: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    sPosition: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,
    dependencyParent: jspb.Message.getFieldWithDefault(msg, 7, 0),
    diphoneDirection: jspb.Message.getFieldWithDefault(msg, 6, 0),
    floatVectorIndex: jspb.Message.getFieldWithDefault(msg, 8, -1),
    numValues: (f = jspb.Message.getField(msg, 10)) == null ? undefined : f,
    valuesHash: (f = jspb.Message.getField(msg, 11)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of NavigationFeature as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.NavigationFeature.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.feature;

  /** @type {?number|undefined} */
  this.featureEnum;

  /** @type {?number|undefined} */
  this.type;

  /** @type {?string|undefined} */
  this.stream;

  /** @type {?number|undefined} */
  this.streamEnum;

  /** @type {?number|undefined} */
  this.position;

  /** @type {?number|undefined} */
  this.sPosition;

  /** @type {?number|undefined} */
  this.dependencyParent;

  /** @type {?number|undefined} */
  this.diphoneDirection;

  /** @type {?number|undefined} */
  this.floatVectorIndex;

  /** @type {?number|undefined} */
  this.numValues;

  /** @type {?number|undefined} */
  this.valuesHash;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.NavigationFeature.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.NavigationFeature}
 */
proto.speech.tts.NavigationFeature.fromObject = function(obj) {
  var msg = new proto.speech.tts.NavigationFeature();
  obj.feature != null && jspb.Message.setField(msg, 4, obj.feature);
  obj.featureEnum != null && jspb.Message.setField(msg, 13, obj.featureEnum);
  obj.type != null && jspb.Message.setField(msg, 9, obj.type);
  obj.stream != null && jspb.Message.setField(msg, 1, obj.stream);
  obj.streamEnum != null && jspb.Message.setField(msg, 12, obj.streamEnum);
  obj.position != null && jspb.Message.setField(msg, 2, obj.position);
  obj.sPosition != null && jspb.Message.setField(msg, 3, obj.sPosition);
  obj.dependencyParent != null && jspb.Message.setField(msg, 7, obj.dependencyParent);
  obj.diphoneDirection != null && jspb.Message.setField(msg, 6, obj.diphoneDirection);
  obj.floatVectorIndex != null && jspb.Message.setField(msg, 8, obj.floatVectorIndex);
  obj.numValues != null && jspb.Message.setField(msg, 10, obj.numValues);
  obj.valuesHash != null && jspb.Message.setField(msg, 11, obj.valuesHash);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.NavigationFeature}
 */
proto.speech.tts.NavigationFeature.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.NavigationFeature;
  return proto.speech.tts.NavigationFeature.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.NavigationFeature} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.NavigationFeature}
 */
proto.speech.tts.NavigationFeature.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setFeature(value);
      break;
    case 13:
      var value = /** @type {!proto.speech.tts.NavigationFeature.FeatureName} */ (reader.readEnum());
      msg.setFeatureEnum(value);
      break;
    case 9:
      var value = /** @type {!proto.speech.tts.LinguisticFeatureValType} */ (reader.readEnum());
      msg.setType(value);
      break;
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setStream(value);
      break;
    case 12:
      var value = /** @type {!proto.speech.tts.NavigationFeature.StreamName} */ (reader.readEnum());
      msg.setStreamEnum(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setPosition(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setSPosition(value);
      break;
    case 7:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setDependencyParent(value);
      break;
    case 6:
      var value = /** @type {!proto.speech.tts.NavigationFeature.DiphoneDirection} */ (reader.readEnum());
      msg.setDiphoneDirection(value);
      break;
    case 8:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setFloatVectorIndex(value);
      break;
    case 10:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setNumValues(value);
      break;
    case 11:
      var value = /** @type {number} */ (reader.readUint64());
      msg.setValuesHash(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.NavigationFeature.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.NavigationFeature.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.NavigationFeature} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.NavigationFeature.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?string} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeString(
      4,
      f
    );
  }
  f = /** @type {!proto.speech.tts.NavigationFeature.FeatureName} */ (jspb.Message.getField(message, 13));
  if (f != null) {
    writer.writeEnum(
      13,
      f
    );
  }
  f = /** @type {!proto.speech.tts.LinguisticFeatureValType} */ (jspb.Message.getField(message, 9));
  if (f != null) {
    writer.writeEnum(
      9,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = /** @type {!proto.speech.tts.NavigationFeature.StreamName} */ (jspb.Message.getField(message, 12));
  if (f != null) {
    writer.writeEnum(
      12,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeInt32(
      2,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeInt32(
      3,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 7));
  if (f != null) {
    writer.writeInt32(
      7,
      f
    );
  }
  f = /** @type {!proto.speech.tts.NavigationFeature.DiphoneDirection} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeEnum(
      6,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 8));
  if (f != null) {
    writer.writeInt32(
      8,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 10));
  if (f != null) {
    writer.writeInt32(
      10,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 11));
  if (f != null) {
    writer.writeUint64(
      11,
      f
    );
  }
};


/**
 * @enum {number}
 */
proto.speech.tts.NavigationFeature.StreamName = {
  EMPTY_STREAM: 0,
  PHONEME_STREAM: 1,
  PHRASE_STREAM: 2,
  CUSTOM_STREAM: 3,
  SYLLABLE_STREAM: 4,
  WORD_STREAM: 5,
  TEST_STREAM: 6
};

/**
 * @enum {number}
 */
proto.speech.tts.NavigationFeature.FeatureName = {
  EMPTY_FEATURE: 0,
  BRAIN_WORD_EMBEDDING_FEATURE: 1,
  CLOSED_SYLLABLE_FEATURE: 2,
  DEPENDENCY_BOUNDARY_STRENGTH_TO_RIGHT_FEATURE: 3,
  DEPENDENCY_CROSSING_LINKS_TO_RIGHT_FEATURE: 4,
  DEPENDENCY_DEPTH_FEATURE: 5,
  DEPENDENCY_LABEL_FEATURE: 6,
  DEPENDENCY_LINK_STEPS_TO_RIGHT_FEATURE: 7,
  DEPENDENCY_OFFSET_FROM_HEAD_FEATURE: 8,
  DEPENDENCY_SIZE_TO_LEFT_LOWER_BOUND_FEATURE: 9,
  DEPENDENCY_SIZE_TO_LEFT_UPPER_BOUND_FEATURE: 10,
  DEPENDENCY_SIZE_TO_RIGHT_LOWER_BOUND_FEATURE: 11,
  DEPENDENCY_SIZE_TO_RIGHT_UPPER_BOUND_FEATURE: 12,
  DEPENDENCY_WEIGHTED_BOUNDARY_STRENGTH_TO_RIGHT_FEATURE: 13,
  DEPENDENCY_WEIGHTED_CROSSING_LINKS_TO_RIGHT_FEATURE: 14,
  DIPHONE_POSITION_FEATURE: 15,
  GENDER_FEATURE: 16,
  GENETIC_LANGUAGE_BOTTOM_BRANCH_FEATURE: 17,
  GENETIC_LANGUAGE_FAMILY_FEATURE: 18,
  GENETIC_LANGUAGE_MIDDLE_BRANCH_FEATURE: 19,
  GENETIC_LANGUAGE_TOP_BRANCH_FEATURE: 20,
  HEIGHT_FEATURE: 21,
  ID_FEATURE: 22,
  ID_WORD_TYPE_FEATURE: 23,
  INDEX_FEATURE: 24,
  INDEX_FROM_END_OF_SENTENCE_FEATURE: 25,
  INTONATION_TYPE_FEATURE: 26,
  IS_EVEN_DISTANCE_BEFORE_ACCENT_FEATURE: 27,
  IS_QUOTED: 247,
  LANGUAGE_FEATURE: 28,
  LENGTH_FEATURE: 29,
  LEXICAL_SYNTAX_FEATURE: 30,
  LIST_ITEM_POSITION_FEATURE: 278,
  LIST_WORD_POSITION_FEATURE: 279,
  LIST_MIDDLE_ITEM_INDEX_FEATURE: 280,
  LIST_MIDDLE_WORD_INDEX_FEATURE: 281,
  MANNER_FEATURE: 31,
  NAME_FEATURE: 32,
  NAMED_ENTITY_INDEX_FEATURE: 244,
  NAMED_ENTITY_MENTION_FEATURE: 245,
  NAMED_ENTITY_POSITION_FEATURE: 246,
  NEIGHBORING_ACCENTED_SYLLABLES_FEATURE: 33,
  NEO_UTTERANCE_EMBEDDING_FEATURE: 34,
  NUM_PHONEMES_FEATURE: 35,
  NUM_PHRASES_IN_SENTENCE_FEATURE: 36,
  NUM_PRECEDING_ACCENTED_SYLLABLES_IN_PHRASE_FEATURE: 37,
  NUM_PRECEDING_CATEGORY_CONTENT_WORDS_IN_PHRASE_FEATURE: 38,
  NUM_PRECEDING_CONTENT_WORDS_IN_PHRASE_FEATURE: 39,
  NUM_PRECEDING_MORAE_IN_PITCH_ACCENT_PHRASE_FEATURE: 40,
  NUM_PRECEDING_MORAE_TO_PITCH_ACCENT_NUCLEAR_FEATURE: 41,
  NUM_PRECEDING_PHONEMES_IN_SYLLABLE_FEATURE: 42,
  NUM_PRECEDING_PHONEMES_IN_WORD_FEATURE: 43,
  NUM_PRECEDING_SILENCES_IN_SENTENCE_FEATURE: 44,
  NUM_PRECEDING_STRESSED_SYLLABLES_IN_PHRASE_FEATURE: 45,
  NUM_PRECEDING_SYLLABLES_IN_PHRASE_FEATURE: 46,
  NUM_PRECEDING_SYLLABLES_IN_SENTENCE_FEATURE: 47,
  NUM_PRECEDING_SYLLABLES_IN_WORD_FEATURE: 48,
  NUM_PRECEDING_WORDS_IN_PHRASE_FEATURE: 49,
  NUM_SUCCEEDING_ACCENTED_SYLLABLES_IN_PHRASE_FEATURE: 50,
  NUM_SUCCEEDING_CATEGORY_CONTENT_WORDS_IN_PHRASE_FEATURE: 51,
  NUM_SUCCEEDING_CONTENT_WORDS_IN_PHRASE_FEATURE: 52,
  NUM_SUCCEEDING_MORAE_IN_PITCH_ACCENT_PHRASE_FEATURE: 53,
  NUM_SUCCEEDING_MORAE_TO_PITCH_ACCENT_NUCLEAR_FEATURE: 54,
  NUM_SUCCEEDING_PHONEMES_IN_SYLLABLE_FEATURE: 55,
  NUM_SUCCEEDING_PHONEMES_IN_WORD_FEATURE: 56,
  NUM_SUCCEEDING_SILENCES_IN_SENTENCE_FEATURE: 57,
  NUM_SUCCEEDING_STRESSED_SYLLABLES_IN_PHRASE_FEATURE: 58,
  NUM_SUCCEEDING_SYLLABLES_IN_PHRASE_FEATURE: 59,
  NUM_SUCCEEDING_SYLLABLES_IN_WORD_FEATURE: 60,
  NUM_SUCCEEDING_WORDS_IN_PHRASE_FEATURE: 61,
  NUM_SYLLABLES_FEATURE: 62,
  NUM_SYLLABLES_IN_SENTENCE_FEATURE: 63,
  NUM_SYLLABLES_TO_NEXT_ACCENTED_SYLLABLE_IN_PHRASE_FEATURE: 64,
  NUM_SYLLABLES_TO_NEXT_LEFT_FOOT_BOUNDARY_FEATURE: 65,
  NUM_SYLLABLES_TO_NEXT_RIGHT_FOOT_BOUNDARY_FEATURE: 66,
  NUM_SYLLABLES_TO_NEXT_STRESSED_SYLLABLE_IN_PHRASE_FEATURE: 67,
  NUM_SYLLABLES_TO_PREVIOUS_ACCENTED_SYLLABLE_IN_PHRASE_FEATURE: 68,
  NUM_SYLLABLES_TO_PREVIOUS_LEFT_FOOT_BOUNDARY_FEATURE: 69,
  NUM_SYLLABLES_TO_PREVIOUS_RIGHT_FOOT_BOUNDARY_FEATURE: 70,
  NUM_SYLLABLES_TO_PREVIOUS_STRESSED_SYLLABLE_IN_PHRASE_FEATURE: 71,
  NUM_WORDS_FEATURE: 72,
  NUM_WORDS_IN_SENTENCE_FEATURE: 73,
  NUM_WORDS_TO_NEXT_CATEGORY_CONTENT_WORD_IN_PHRASE_FEATURE: 74,
  NUM_WORDS_TO_NEXT_CONTENT_WORD_IN_PHRASE_FEATURE: 75,
  NUM_WORDS_TO_PREVIOUS_CATEGORY_CONTENT_WORD_IN_PHRASE_FEATURE: 76,
  NUM_WORDS_TO_PREVIOUS_CONTENT_WORD_IN_PHRASE_FEATURE: 77,
  OFFSET_FROM_NUCLEUS_FEATURE: 78,
  ONSET_FEATURE: 79,
  PHONE_POSITION_FEATURE: 80,
  PHON_POS_FEATURE: 81,
  PHRASE_BOUNDARY_LEVEL: 248,
  PHRASING_FEATURE: 82,
  PITCH_ACCENT_PHRASE_START_FEATURE: 83,
  POA_FEATURE: 84,
  POSITION_FEATURE: 85,
  PRECEDING_PUNCTUATION_FEATURE: 86,
  PROMINENCE_LEVEL_FEATURE: 87,
  PROMINENCE_TYPE_FEATURE: 88,
  RAW_INTONATION_TYPE_FEATURE: 89,
  REGION_FEATURE: 90,
  ROUNDED_FEATURE: 91,
  SCHWA_FEATURE: 92,
  SONORANT_POLLUTION_FEATURE: 93,
  SPEAKER_NAME_FEATURE: 94,
  STRESS_FEATURE: 95,
  SUCCEEDING_PUNCTUATION_FEATURE: 96,
  SYLLABLE_PROMINENCE_LEVEL_FEATURE: 97,
  SYNTACTIC_LINKAGE_TO_LEFT_FEATURE: 98,
  SYNTACTIC_LINKAGE_TO_RIGHT_FEATURE: 99,
  SYNTAX_CATEGORY_FEATURE: 100,
  SYNTAX_TAG_FEATURE: 101,
  TONE_FEATURE: 102,
  VARIANT_FEATURE: 103,
  VOICED_FEATURE: 104,
  VOWEL_FEATURE: 105,
  VOWEL_OF_SYLLABLE_FEATURE: 106,
  WORD_LM_SCORE_BIGRAM_FEATURE: 107,
  WORD_LM_SCORE_TRIGRAM_FEATURE: 108,
  WORD_LM_SCORE_UNIGRAM_FEATURE: 109,
  WORD_PITCH_ACCENT_FEATURE: 110,
  WORD_POS_FROM_USER_FEATURES_FEATURE: 111,
  WORD_TYPE_FEATURE: 112,
  WORD_LANGUAGE_FEATURE: 286,
  WORD_REGION_FEATURE: 287,
  GEO_LANGUAGE_UNIT_VECTOR_FEATURE: 113,
  GEO_LANGUAGE_ALL_DISTANCES_FEATURE: 114,
  GEO_CLOSEST_LANGUAGE_1_FEATURE: 115,
  GEO_CLOSEST_LANGUAGE_2_FEATURE: 116,
  GEO_CLOSEST_LANGUAGE_3_FEATURE: 117,
  GEO_CLOSEST_LANGUAGE_4_FEATURE: 118,
  GEO_CLOSEST_LANGUAGE_5_FEATURE: 119,
  RATED_PROMINENCE_FEATURE: 120,
  VUI_ID_1_FEATURE: 121,
  VUI_ID_2_FEATURE: 122,
  VUI_ID_3_FEATURE: 123,
  IS_EXPRESSIVE_FEATURE: 124,
  VUI_ID_PRINCIPAL_FEATURE: 125,
  IS_MULTI_SPEAKER_DATASET_FEATURE: 277,
  EMOTION_DIMENSIONS_AROUSAL_FEATURE: 282,
  EMOTION_DIMENSIONS_VALENCE_FEATURE: 283,
  SPEAKING_STYLE_FEATURE: 284,
  PHONATION_MODE_FEATURE: 285,
  PANPHON_SYLLABIC_0_INT_FEATURE: 126,
  PANPHON_SONORANT_0_INT_FEATURE: 127,
  PANPHON_CONSONANTAL_0_INT_FEATURE: 128,
  PANPHON_CONTINUANT_0_INT_FEATURE: 129,
  PANPHON_DELAYED_RELEASE_0_INT_FEATURE: 130,
  PANPHON_LATERAL_0_INT_FEATURE: 131,
  PANPHON_NASAL_0_INT_FEATURE: 132,
  PANPHON_STRIDENT_0_INT_FEATURE: 133,
  PANPHON_VOICE_0_INT_FEATURE: 134,
  PANPHON_SPREAD_GLOTTIS_0_INT_FEATURE: 135,
  PANPHON_CONSTRICTED_GLOTTIS_0_INT_FEATURE: 136,
  PANPHON_ANTERIOR_0_INT_FEATURE: 137,
  PANPHON_CORONAL_0_INT_FEATURE: 138,
  PANPHON_DISTRIBUTED_0_INT_FEATURE: 139,
  PANPHON_LABIAL_0_INT_FEATURE: 140,
  PANPHON_HIGH_NOT_TONAL_0_INT_FEATURE: 141,
  PANPHON_LOW_NOT_TONAL_0_INT_FEATURE: 142,
  PANPHON_BACK_0_INT_FEATURE: 143,
  PANPHON_ROUND_0_INT_FEATURE: 144,
  PANPHON_VELARIC_0_INT_FEATURE: 145,
  PANPHON_TENSE_0_INT_FEATURE: 146,
  PANPHON_LONG_0_INT_FEATURE: 147,
  PANPHON_SYLLABIC_1_INT_FEATURE: 148,
  PANPHON_SONORANT_1_INT_FEATURE: 149,
  PANPHON_CONSONANTAL_1_INT_FEATURE: 150,
  PANPHON_CONTINUANT_1_INT_FEATURE: 151,
  PANPHON_DELAYED_RELEASE_1_INT_FEATURE: 152,
  PANPHON_LATERAL_1_INT_FEATURE: 153,
  PANPHON_NASAL_1_INT_FEATURE: 154,
  PANPHON_STRIDENT_1_INT_FEATURE: 155,
  PANPHON_VOICE_1_INT_FEATURE: 156,
  PANPHON_SPREAD_GLOTTIS_1_INT_FEATURE: 157,
  PANPHON_CONSTRICTED_GLOTTIS_1_INT_FEATURE: 158,
  PANPHON_ANTERIOR_1_INT_FEATURE: 159,
  PANPHON_CORONAL_1_INT_FEATURE: 160,
  PANPHON_DISTRIBUTED_1_INT_FEATURE: 161,
  PANPHON_LABIAL_1_INT_FEATURE: 162,
  PANPHON_HIGH_NOT_TONAL_1_INT_FEATURE: 163,
  PANPHON_LOW_NOT_TONAL_1_INT_FEATURE: 164,
  PANPHON_BACK_1_INT_FEATURE: 165,
  PANPHON_ROUND_1_INT_FEATURE: 166,
  PANPHON_VELARIC_1_INT_FEATURE: 167,
  PANPHON_TENSE_1_INT_FEATURE: 168,
  PANPHON_LONG_1_INT_FEATURE: 169,
  PHOIBLE_TONE_0_INT_FEATURE: 170,
  PHOIBLE_STRESS_0_INT_FEATURE: 171,
  PHOIBLE_SYLLABIC_0_INT_FEATURE: 172,
  PHOIBLE_SHORT_0_INT_FEATURE: 173,
  PHOIBLE_LONG_0_INT_FEATURE: 174,
  PHOIBLE_CONSONANTAL_0_INT_FEATURE: 175,
  PHOIBLE_SONORANT_0_INT_FEATURE: 176,
  PHOIBLE_CONTINUANT_0_INT_FEATURE: 177,
  PHOIBLE_DELAYED_RELEASE_0_INT_FEATURE: 178,
  PHOIBLE_APPROXIMANT_0_INT_FEATURE: 179,
  PHOIBLE_TAP_0_INT_FEATURE: 180,
  PHOIBLE_TRILL_0_INT_FEATURE: 181,
  PHOIBLE_NASAL_0_INT_FEATURE: 182,
  PHOIBLE_LATERAL_0_INT_FEATURE: 183,
  PHOIBLE_LABIAL_0_INT_FEATURE: 184,
  PHOIBLE_ROUND_0_INT_FEATURE: 185,
  PHOIBLE_LABIODENTAL_0_INT_FEATURE: 186,
  PHOIBLE_CORONAL_0_INT_FEATURE: 187,
  PHOIBLE_ANTERIOR_0_INT_FEATURE: 188,
  PHOIBLE_DISTRIBUTED_0_INT_FEATURE: 189,
  PHOIBLE_STRIDENT_0_INT_FEATURE: 190,
  PHOIBLE_DORSAL_0_INT_FEATURE: 191,
  PHOIBLE_HIGH_0_INT_FEATURE: 192,
  PHOIBLE_LOW_0_INT_FEATURE: 193,
  PHOIBLE_FRONT_0_INT_FEATURE: 194,
  PHOIBLE_BACK_0_INT_FEATURE: 195,
  PHOIBLE_TENSE_0_INT_FEATURE: 196,
  PHOIBLE_RETRACTED_TONGUE_ROOT_0_INT_FEATURE: 197,
  PHOIBLE_ADVANCED_TONGUE_ROOT_0_INT_FEATURE: 198,
  PHOIBLE_PERIODIC_GLOTTAL_SOURCE_0_INT_FEATURE: 199,
  PHOIBLE_EPILARYNGEAL_SOURCE_0_INT_FEATURE: 200,
  PHOIBLE_SPREAD_GLOTTIS_0_INT_FEATURE: 201,
  PHOIBLE_CONSTRICTED_GLOTTIS_0_INT_FEATURE: 202,
  PHOIBLE_FORTIS_0_INT_FEATURE: 203,
  PHOIBLE_RAISED_LARYNX_EJECTIVE_0_INT_FEATURE: 204,
  PHOIBLE_LOWERED_LARYNX_IMPLOSIVE_0_INT_FEATURE: 205,
  PHOIBLE_CLICK_0_INT_FEATURE: 206,
  PHOIBLE_TONE_1_INT_FEATURE: 207,
  PHOIBLE_STRESS_1_INT_FEATURE: 208,
  PHOIBLE_SYLLABIC_1_INT_FEATURE: 209,
  PHOIBLE_SHORT_1_INT_FEATURE: 210,
  PHOIBLE_LONG_1_INT_FEATURE: 211,
  PHOIBLE_CONSONANTAL_1_INT_FEATURE: 212,
  PHOIBLE_SONORANT_1_INT_FEATURE: 213,
  PHOIBLE_CONTINUANT_1_INT_FEATURE: 214,
  PHOIBLE_DELAYED_RELEASE_1_INT_FEATURE: 215,
  PHOIBLE_APPROXIMANT_1_INT_FEATURE: 216,
  PHOIBLE_TAP_1_INT_FEATURE: 217,
  PHOIBLE_TRILL_1_INT_FEATURE: 218,
  PHOIBLE_NASAL_1_INT_FEATURE: 219,
  PHOIBLE_LATERAL_1_INT_FEATURE: 220,
  PHOIBLE_LABIAL_1_INT_FEATURE: 221,
  PHOIBLE_ROUND_1_INT_FEATURE: 222,
  PHOIBLE_LABIODENTAL_1_INT_FEATURE: 223,
  PHOIBLE_CORONAL_1_INT_FEATURE: 224,
  PHOIBLE_ANTERIOR_1_INT_FEATURE: 225,
  PHOIBLE_DISTRIBUTED_1_INT_FEATURE: 226,
  PHOIBLE_STRIDENT_1_INT_FEATURE: 227,
  PHOIBLE_DORSAL_1_INT_FEATURE: 228,
  PHOIBLE_HIGH_1_INT_FEATURE: 229,
  PHOIBLE_LOW_1_INT_FEATURE: 230,
  PHOIBLE_FRONT_1_INT_FEATURE: 231,
  PHOIBLE_BACK_1_INT_FEATURE: 232,
  PHOIBLE_TENSE_1_INT_FEATURE: 233,
  PHOIBLE_RETRACTED_TONGUE_ROOT_1_INT_FEATURE: 234,
  PHOIBLE_ADVANCED_TONGUE_ROOT_1_INT_FEATURE: 235,
  PHOIBLE_PERIODIC_GLOTTAL_SOURCE_1_INT_FEATURE: 236,
  PHOIBLE_EPILARYNGEAL_SOURCE_1_INT_FEATURE: 237,
  PHOIBLE_SPREAD_GLOTTIS_1_INT_FEATURE: 238,
  PHOIBLE_CONSTRICTED_GLOTTIS_1_INT_FEATURE: 239,
  PHOIBLE_FORTIS_1_INT_FEATURE: 240,
  PHOIBLE_RAISED_LARYNX_EJECTIVE_1_INT_FEATURE: 241,
  PHOIBLE_LOWERED_LARYNX_IMPLOSIVE_1_INT_FEATURE: 242,
  PHOIBLE_CLICK_1_INT_FEATURE: 243,
  PHOIBLE_FONETIKODE_GENERAL_CLASS_0_STRING_FEATURE: 250,
  PHOIBLE_FONETIKODE_VOWEL_VERTICAL_OR_TONGUE_HEIGHT_0_STRING_FEATURE: 251,
  PHOIBLE_FONETIKODE_VOWEL_HORIZONTAL_OR_TONGUE_ANTERIORITY_0_STRING_FEATURE: 252,
  PHOIBLE_FONETIKODE_VOWEL_MODIFIERS_0_STRING_FEATURE: 253,
  PHOIBLE_FONETIKODE_CONSONANTAL_POA_0_STRING_FEATURE: 254,
  PHOIBLE_FONETIKODE_CONSONANTAL_MANNER_0_STRING_FEATURE: 255,
  PHOIBLE_FONETIKODE_CONSONANTAL_SEQUENCER_0_STRING_FEATURE: 256,
  PHOIBLE_FONETIKODE_PHONATION_0_STRING_FEATURE: 257,
  PHOIBLE_FONETIKODE_INITIATION_0_STRING_FEATURE: 258,
  PHOIBLE_FONETIKODE_PRIMARY_ARTICULATION_DIACRITICS_0_STRING_FEATURE: 259,
  PHOIBLE_FONETIKODE_SECONDARY_ARTICULATION_DIACRITICS_0_STRING_FEATURE: 260,
  PHOIBLE_FONETIKODE_PROSODIC_PROPERTIES_0_STRING_FEATURE: 261,
  PHOIBLE_FONETIKODE_TONE_MARKERS_0_STRING_FEATURE: 262,
  PHOIBLE_FONETIKODE_GENERAL_CLASS_1_STRING_FEATURE: 263,
  PHOIBLE_FONETIKODE_VOWEL_VERTICAL_OR_TONGUE_HEIGHT_1_STRING_FEATURE: 264,
  PHOIBLE_FONETIKODE_VOWEL_HORIZONTAL_OR_TONGUE_ANTERIORITY_1_STRING_FEATURE: 265,
  PHOIBLE_FONETIKODE_VOWEL_MODIFIERS_1_STRING_FEATURE: 266,
  PHOIBLE_FONETIKODE_CONSONANTAL_POA_1_STRING_FEATURE: 267,
  PHOIBLE_FONETIKODE_CONSONANTAL_MANNER_1_STRING_FEATURE: 268,
  PHOIBLE_FONETIKODE_CONSONANTAL_SEQUENCER_1_STRING_FEATURE: 269,
  PHOIBLE_FONETIKODE_PHONATION_1_STRING_FEATURE: 270,
  PHOIBLE_FONETIKODE_INITIATION_1_STRING_FEATURE: 271,
  PHOIBLE_FONETIKODE_PRIMARY_ARTICULATION_DIACRITICS_1_STRING_FEATURE: 272,
  PHOIBLE_FONETIKODE_SECONDARY_ARTICULATION_DIACRITICS_1_STRING_FEATURE: 273,
  PHOIBLE_FONETIKODE_PROSODIC_PROPERTIES_1_STRING_FEATURE: 274,
  PHOIBLE_FONETIKODE_TONE_MARKERS_1_STRING_FEATURE: 275
};

/**
 * @enum {number}
 */
proto.speech.tts.NavigationFeature.DiphoneDirection = {
  DIPHONE_UNSPECIFIED: 0,
  DIPHONE_LEFT: 1,
  DIPHONE_RIGHT: 2
};

/**
 * optional string feature = 4;
 * @return {?string}
 */
proto.speech.tts.NavigationFeature.prototype.getFeature = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 4));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.NavigationFeature} returns this
 */
proto.speech.tts.NavigationFeature.prototype.setFeature = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.NavigationFeature} returns this
 */
proto.speech.tts.NavigationFeature.prototype.clearFeature = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.NavigationFeature.prototype.hasFeature = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional FeatureName feature_enum = 13;
 * @return {!proto.speech.tts.NavigationFeature.FeatureName}
 */
proto.speech.tts.NavigationFeature.prototype.getFeatureEnum = function() {
  return /** @type {!proto.speech.tts.NavigationFeature.FeatureName} */ (jspb.Message.getField(this, 13));
};


/**
 * @param {?proto.speech.tts.NavigationFeature.FeatureName|undefined} value
 * @return {!proto.speech.tts.NavigationFeature} returns this
 */
proto.speech.tts.NavigationFeature.prototype.setFeatureEnum = function(value) {
  return jspb.Message.setField(this, 13, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.NavigationFeature} returns this
 */
proto.speech.tts.NavigationFeature.prototype.clearFeatureEnum = function() {
  return jspb.Message.setField(this, 13, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.NavigationFeature.prototype.hasFeatureEnum = function() {
  return jspb.Message.getField(this, 13) != null;
};


/**
 * optional LinguisticFeatureValType type = 9;
 * @return {!proto.speech.tts.LinguisticFeatureValType}
 */
proto.speech.tts.NavigationFeature.prototype.getType = function() {
  return /** @type {!proto.speech.tts.LinguisticFeatureValType} */ (jspb.Message.getField(this, 9));
};


/**
 * @param {?proto.speech.tts.LinguisticFeatureValType|undefined} value
 * @return {!proto.speech.tts.NavigationFeature} returns this
 */
proto.speech.tts.NavigationFeature.prototype.setType = function(value) {
  return jspb.Message.setField(this, 9, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.NavigationFeature} returns this
 */
proto.speech.tts.NavigationFeature.prototype.clearType = function() {
  return jspb.Message.setField(this, 9, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.NavigationFeature.prototype.hasType = function() {
  return jspb.Message.getField(this, 9) != null;
};


/**
 * optional string stream = 1;
 * @return {?string}
 */
proto.speech.tts.NavigationFeature.prototype.getStream = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.NavigationFeature} returns this
 */
proto.speech.tts.NavigationFeature.prototype.setStream = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.NavigationFeature} returns this
 */
proto.speech.tts.NavigationFeature.prototype.clearStream = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.NavigationFeature.prototype.hasStream = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional StreamName stream_enum = 12;
 * @return {!proto.speech.tts.NavigationFeature.StreamName}
 */
proto.speech.tts.NavigationFeature.prototype.getStreamEnum = function() {
  return /** @type {!proto.speech.tts.NavigationFeature.StreamName} */ (jspb.Message.getField(this, 12));
};


/**
 * @param {?proto.speech.tts.NavigationFeature.StreamName|undefined} value
 * @return {!proto.speech.tts.NavigationFeature} returns this
 */
proto.speech.tts.NavigationFeature.prototype.setStreamEnum = function(value) {
  return jspb.Message.setField(this, 12, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.NavigationFeature} returns this
 */
proto.speech.tts.NavigationFeature.prototype.clearStreamEnum = function() {
  return jspb.Message.setField(this, 12, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.NavigationFeature.prototype.hasStreamEnum = function() {
  return jspb.Message.getField(this, 12) != null;
};


/**
 * optional int32 position = 2;
 * @return {?number}
 */
proto.speech.tts.NavigationFeature.prototype.getPosition = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.NavigationFeature} returns this
 */
proto.speech.tts.NavigationFeature.prototype.setPosition = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.NavigationFeature} returns this
 */
proto.speech.tts.NavigationFeature.prototype.clearPosition = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.NavigationFeature.prototype.hasPosition = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional int32 s_position = 3;
 * @return {?number}
 */
proto.speech.tts.NavigationFeature.prototype.getSPosition = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.NavigationFeature} returns this
 */
proto.speech.tts.NavigationFeature.prototype.setSPosition = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.NavigationFeature} returns this
 */
proto.speech.tts.NavigationFeature.prototype.clearSPosition = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.NavigationFeature.prototype.hasSPosition = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional int32 dependency_parent = 7;
 * @return {number}
 */
proto.speech.tts.NavigationFeature.prototype.getDependencyParent = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 7, 0));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.NavigationFeature} returns this
 */
proto.speech.tts.NavigationFeature.prototype.setDependencyParent = function(value) {
  return jspb.Message.setField(this, 7, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.NavigationFeature} returns this
 */
proto.speech.tts.NavigationFeature.prototype.clearDependencyParent = function() {
  return jspb.Message.setField(this, 7, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.NavigationFeature.prototype.hasDependencyParent = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * optional DiphoneDirection diphone_direction = 6;
 * @return {!proto.speech.tts.NavigationFeature.DiphoneDirection}
 */
proto.speech.tts.NavigationFeature.prototype.getDiphoneDirection = function() {
  return /** @type {!proto.speech.tts.NavigationFeature.DiphoneDirection} */ (jspb.Message.getFieldWithDefault(this, 6, 0));
};


/**
 * @param {?proto.speech.tts.NavigationFeature.DiphoneDirection|undefined} value
 * @return {!proto.speech.tts.NavigationFeature} returns this
 */
proto.speech.tts.NavigationFeature.prototype.setDiphoneDirection = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.NavigationFeature} returns this
 */
proto.speech.tts.NavigationFeature.prototype.clearDiphoneDirection = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.NavigationFeature.prototype.hasDiphoneDirection = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional int32 float_vector_index = 8;
 * @return {number}
 */
proto.speech.tts.NavigationFeature.prototype.getFloatVectorIndex = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 8, -1));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.NavigationFeature} returns this
 */
proto.speech.tts.NavigationFeature.prototype.setFloatVectorIndex = function(value) {
  return jspb.Message.setField(this, 8, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.NavigationFeature} returns this
 */
proto.speech.tts.NavigationFeature.prototype.clearFloatVectorIndex = function() {
  return jspb.Message.setField(this, 8, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.NavigationFeature.prototype.hasFloatVectorIndex = function() {
  return jspb.Message.getField(this, 8) != null;
};


/**
 * optional int32 num_values = 10;
 * @return {?number}
 */
proto.speech.tts.NavigationFeature.prototype.getNumValues = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 10));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.NavigationFeature} returns this
 */
proto.speech.tts.NavigationFeature.prototype.setNumValues = function(value) {
  return jspb.Message.setField(this, 10, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.NavigationFeature} returns this
 */
proto.speech.tts.NavigationFeature.prototype.clearNumValues = function() {
  return jspb.Message.setField(this, 10, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.NavigationFeature.prototype.hasNumValues = function() {
  return jspb.Message.getField(this, 10) != null;
};


/**
 * optional uint64 values_hash = 11;
 * @return {?number}
 */
proto.speech.tts.NavigationFeature.prototype.getValuesHash = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 11));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.NavigationFeature} returns this
 */
proto.speech.tts.NavigationFeature.prototype.setValuesHash = function(value) {
  return jspb.Message.setField(this, 11, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.NavigationFeature} returns this
 */
proto.speech.tts.NavigationFeature.prototype.clearValuesHash = function() {
  return jspb.Message.setField(this, 11, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.NavigationFeature.prototype.hasValuesHash = function() {
  return jspb.Message.getField(this, 11) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.NavigationFeature}
 */
proto.speech.tts.NavigationFeature.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.NavigationFeature, data);
};



//speech/tts/engine/specification/linguistic_features_proto_jspb/linguisticfeature.jspb.js
// source: speech/tts/engine/specification/linguistic_features.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.LinguisticFeature');
goog.provide('proto.speech.tts.LinguisticFeature.HowToGenerateQuestionsCase');
goog.provide('proto.speech.tts.LinguisticFeature.NumericalBoundaries');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.CustomFeature');
goog.require('proto.speech.tts.NavigationFeature');

goog.forwardDeclare('proto.speech.tts.LinguisticFeatureValType');
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.LinguisticFeature = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.LinguisticFeature.repeatedFields_, proto.speech.tts.LinguisticFeature.oneofGroups_);
};
goog.inherits(proto.speech.tts.LinguisticFeature, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.LinguisticFeature.displayName = 'proto.speech.tts.LinguisticFeature';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.LinguisticFeature.NumericalBoundaries = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.LinguisticFeature.NumericalBoundaries, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.LinguisticFeature.NumericalBoundaries.displayName = 'proto.speech.tts.LinguisticFeature.NumericalBoundaries';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.LinguisticFeature.repeatedFields_ = [8,9,12,13,14];

/**
 * Oneof group definitions for this message. Each group defines the field
 * numbers belonging to that group. When of these fields' value is set, all
 * other fields in the group are cleared. During deserialization, if multiple
 * fields are encountered for a group, only the last value seen will be kept.
 * @private {!Array<!Array<number>>}
 * @const
 */
proto.speech.tts.LinguisticFeature.oneofGroups_ = [[6,7]];

/**
 * @enum {number}
 */
proto.speech.tts.LinguisticFeature.HowToGenerateQuestionsCase = {
  HOW_TO_GENERATE_QUESTIONS_NOT_SET: 0,
  CACHE_VALUES: 6,
  NUMERICAL: 7
};

/**
 * @return {!proto.speech.tts.LinguisticFeature.HowToGenerateQuestionsCase}
 */
proto.speech.tts.LinguisticFeature.prototype.getHowToGenerateQuestionsCase = function() {
  return /** @type {!proto.speech.tts.LinguisticFeature.HowToGenerateQuestionsCase} */(jspb.Message.computeOneofCase(this, proto.speech.tts.LinguisticFeature.oneofGroups_[0]));
};



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.LinguisticFeature.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.LinguisticFeature.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.LinguisticFeature} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.LinguisticFeature.toObject = function(includeInstance, msg) {
  var f, obj = {
    regexpTag: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    type: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,
    nf: (f = msg.getNf()) && proto.speech.tts.NavigationFeature.toObject(includeInstance, f),
    custom: (f = msg.getCustom()) && proto.speech.tts.CustomFeature.toObject(includeInstance, f),
    cacheValues: (f = jspb.Message.getBooleanField(msg, 6)) == null ? undefined : f,
    numerical: (f = msg.getNumerical()) && proto.speech.tts.LinguisticFeature.NumericalBoundaries.toObject(includeInstance, f),
    skipValueList: (f = jspb.Message.getRepeatedField(msg, 8)) == null ? undefined : f,
    skipStringValueList: (f = jspb.Message.getRepeatedField(msg, 9)) == null ? undefined : f,
    useContext: jspb.Message.getBooleanFieldWithDefault(msg, 10, false),
    includeIfLanguageList: (f = jspb.Message.getRepeatedField(msg, 12)) == null ? undefined : f,
    excludeIfLanguageList: (f = jspb.Message.getRepeatedField(msg, 13)) == null ? undefined : f,
    excludeIfDatasetList: (f = jspb.Message.getRepeatedField(msg, 14)) == null ? undefined : f,
    dontPrune: jspb.Message.getBooleanFieldWithDefault(msg, 15, false)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of LinguisticFeature as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.LinguisticFeature.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.regexpTag;

  /** @type {?number|undefined} */
  this.type;

  /** @type {?proto.speech.tts.NavigationFeature.ObjectFormat|undefined} */
  this.nf;

  /** @type {?proto.speech.tts.CustomFeature.ObjectFormat|undefined} */
  this.custom;

  /** @type {?boolean|undefined} */
  this.cacheValues;

  /** @type {?proto.speech.tts.LinguisticFeature.NumericalBoundaries.ObjectFormat|undefined} */
  this.numerical;

  /** @type {?Array<number>|undefined} */
  this.skipValueList;

  /** @type {?Array<string>|undefined} */
  this.skipStringValueList;

  /** @type {?boolean|undefined} */
  this.useContext;

  /** @type {?Array<string>|undefined} */
  this.includeIfLanguageList;

  /** @type {?Array<string>|undefined} */
  this.excludeIfLanguageList;

  /** @type {?Array<string>|undefined} */
  this.excludeIfDatasetList;

  /** @type {?boolean|undefined} */
  this.dontPrune;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.LinguisticFeature.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.LinguisticFeature}
 */
proto.speech.tts.LinguisticFeature.fromObject = function(obj) {
  var msg = new proto.speech.tts.LinguisticFeature();
  obj.regexpTag != null && jspb.Message.setField(msg, 2, obj.regexpTag);
  obj.type != null && jspb.Message.setField(msg, 3, obj.type);
  obj.nf && jspb.Message.setWrapperField(
      msg, 4, proto.speech.tts.NavigationFeature.fromObject(obj.nf));
  obj.custom && jspb.Message.setWrapperField(
      msg, 11, proto.speech.tts.CustomFeature.fromObject(obj.custom));
  obj.cacheValues != null && jspb.Message.setField(msg, 6, obj.cacheValues);
  obj.numerical && jspb.Message.setWrapperField(
      msg, 7, proto.speech.tts.LinguisticFeature.NumericalBoundaries.fromObject(obj.numerical));
  obj.skipValueList != null && jspb.Message.setField(msg, 8, obj.skipValueList);
  obj.skipStringValueList != null && jspb.Message.setField(msg, 9, obj.skipStringValueList);
  obj.useContext != null && jspb.Message.setField(msg, 10, obj.useContext);
  obj.includeIfLanguageList != null && jspb.Message.setField(msg, 12, obj.includeIfLanguageList);
  obj.excludeIfLanguageList != null && jspb.Message.setField(msg, 13, obj.excludeIfLanguageList);
  obj.excludeIfDatasetList != null && jspb.Message.setField(msg, 14, obj.excludeIfDatasetList);
  obj.dontPrune != null && jspb.Message.setField(msg, 15, obj.dontPrune);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.LinguisticFeature}
 */
proto.speech.tts.LinguisticFeature.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.LinguisticFeature;
  return proto.speech.tts.LinguisticFeature.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.LinguisticFeature} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.LinguisticFeature}
 */
proto.speech.tts.LinguisticFeature.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setRegexpTag(value);
      break;
    case 3:
      var value = /** @type {!proto.speech.tts.LinguisticFeatureValType} */ (reader.readEnum());
      msg.setType(value);
      break;
    case 4:
      var value = new proto.speech.tts.NavigationFeature;
      reader.readMessage(value,proto.speech.tts.NavigationFeature.deserializeBinaryFromReader);
      msg.setNf(value);
      break;
    case 11:
      var value = new proto.speech.tts.CustomFeature;
      reader.readMessage(value,proto.speech.tts.CustomFeature.deserializeBinaryFromReader);
      msg.setCustom(value);
      break;
    case 6:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setCacheValues(value);
      break;
    case 7:
      var value = new proto.speech.tts.LinguisticFeature.NumericalBoundaries;
      reader.readMessage(value,proto.speech.tts.LinguisticFeature.NumericalBoundaries.deserializeBinaryFromReader);
      msg.setNumerical(value);
      break;
    case 8:
      var value = /** @type {number} */ (reader.readInt32());
      msg.addSkipValue(value);
      break;
    case 9:
      var value = /** @type {string} */ (reader.readString());
      msg.addSkipStringValue(value);
      break;
    case 10:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setUseContext(value);
      break;
    case 12:
      var value = /** @type {string} */ (reader.readString());
      msg.addIncludeIfLanguage(value);
      break;
    case 13:
      var value = /** @type {string} */ (reader.readString());
      msg.addExcludeIfLanguage(value);
      break;
    case 14:
      var value = /** @type {string} */ (reader.readString());
      msg.addExcludeIfDataset(value);
      break;
    case 15:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setDontPrune(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.LinguisticFeature.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.LinguisticFeature.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.LinguisticFeature} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.LinguisticFeature.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
  f = /** @type {!proto.speech.tts.LinguisticFeatureValType} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeEnum(
      3,
      f
    );
  }
  f = message.getNf();
  if (f != null) {
    writer.writeMessage(
      4,
      f,
      proto.speech.tts.NavigationFeature.serializeBinaryToWriter
    );
  }
  f = message.getCustom();
  if (f != null) {
    writer.writeMessage(
      11,
      f,
      proto.speech.tts.CustomFeature.serializeBinaryToWriter
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeBool(
      6,
      f
    );
  }
  f = message.getNumerical();
  if (f != null) {
    writer.writeMessage(
      7,
      f,
      proto.speech.tts.LinguisticFeature.NumericalBoundaries.serializeBinaryToWriter
    );
  }
  f = message.getSkipValueList();
  if (f.length > 0) {
    writer.writeRepeatedInt32(
      8,
      f
    );
  }
  f = message.getSkipStringValueList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      9,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 10));
  if (f != null) {
    writer.writeBool(
      10,
      f
    );
  }
  f = message.getIncludeIfLanguageList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      12,
      f
    );
  }
  f = message.getExcludeIfLanguageList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      13,
      f
    );
  }
  f = message.getExcludeIfDatasetList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      14,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 15));
  if (f != null) {
    writer.writeBool(
      15,
      f
    );
  }
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.LinguisticFeature.NumericalBoundaries.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.LinguisticFeature.NumericalBoundaries.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.LinguisticFeature.NumericalBoundaries} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.LinguisticFeature.NumericalBoundaries.toObject = function(includeInstance, msg) {
  var f, obj = {
    cacheValues: (f = jspb.Message.getBooleanField(msg, 1)) == null ? undefined : f,
    max: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    min: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,
    categorical: (f = jspb.Message.getBooleanField(msg, 4)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of NumericalBoundaries as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.LinguisticFeature.NumericalBoundaries.ObjectFormat = function() {
  /** @type {?boolean|undefined} */
  this.cacheValues;

  /** @type {?number|undefined} */
  this.max;

  /** @type {?number|undefined} */
  this.min;

  /** @type {?boolean|undefined} */
  this.categorical;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.LinguisticFeature.NumericalBoundaries.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.LinguisticFeature.NumericalBoundaries}
 */
proto.speech.tts.LinguisticFeature.NumericalBoundaries.fromObject = function(obj) {
  var msg = new proto.speech.tts.LinguisticFeature.NumericalBoundaries();
  obj.cacheValues != null && jspb.Message.setField(msg, 1, obj.cacheValues);
  obj.max != null && jspb.Message.setField(msg, 2, obj.max);
  obj.min != null && jspb.Message.setField(msg, 3, obj.min);
  obj.categorical != null && jspb.Message.setField(msg, 4, obj.categorical);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.LinguisticFeature.NumericalBoundaries}
 */
proto.speech.tts.LinguisticFeature.NumericalBoundaries.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.LinguisticFeature.NumericalBoundaries;
  return proto.speech.tts.LinguisticFeature.NumericalBoundaries.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.LinguisticFeature.NumericalBoundaries} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.LinguisticFeature.NumericalBoundaries}
 */
proto.speech.tts.LinguisticFeature.NumericalBoundaries.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setCacheValues(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setMax(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setMin(value);
      break;
    case 4:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setCategorical(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.LinguisticFeature.NumericalBoundaries.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.LinguisticFeature.NumericalBoundaries.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.LinguisticFeature.NumericalBoundaries} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.LinguisticFeature.NumericalBoundaries.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeBool(
      1,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeInt32(
      2,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeInt32(
      3,
      f
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeBool(
      4,
      f
    );
  }
};


/**
 * optional bool cache_values = 1;
 * @return {?boolean}
 */
proto.speech.tts.LinguisticFeature.NumericalBoundaries.prototype.getCacheValues = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 1));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.LinguisticFeature.NumericalBoundaries} returns this
 */
proto.speech.tts.LinguisticFeature.NumericalBoundaries.prototype.setCacheValues = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.LinguisticFeature.NumericalBoundaries} returns this
 */
proto.speech.tts.LinguisticFeature.NumericalBoundaries.prototype.clearCacheValues = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.LinguisticFeature.NumericalBoundaries.prototype.hasCacheValues = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional int32 max = 2;
 * @return {?number}
 */
proto.speech.tts.LinguisticFeature.NumericalBoundaries.prototype.getMax = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.LinguisticFeature.NumericalBoundaries} returns this
 */
proto.speech.tts.LinguisticFeature.NumericalBoundaries.prototype.setMax = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.LinguisticFeature.NumericalBoundaries} returns this
 */
proto.speech.tts.LinguisticFeature.NumericalBoundaries.prototype.clearMax = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.LinguisticFeature.NumericalBoundaries.prototype.hasMax = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional int32 min = 3;
 * @return {?number}
 */
proto.speech.tts.LinguisticFeature.NumericalBoundaries.prototype.getMin = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.LinguisticFeature.NumericalBoundaries} returns this
 */
proto.speech.tts.LinguisticFeature.NumericalBoundaries.prototype.setMin = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.LinguisticFeature.NumericalBoundaries} returns this
 */
proto.speech.tts.LinguisticFeature.NumericalBoundaries.prototype.clearMin = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.LinguisticFeature.NumericalBoundaries.prototype.hasMin = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional bool categorical = 4;
 * @return {?boolean}
 */
proto.speech.tts.LinguisticFeature.NumericalBoundaries.prototype.getCategorical = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 4));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.LinguisticFeature.NumericalBoundaries} returns this
 */
proto.speech.tts.LinguisticFeature.NumericalBoundaries.prototype.setCategorical = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.LinguisticFeature.NumericalBoundaries} returns this
 */
proto.speech.tts.LinguisticFeature.NumericalBoundaries.prototype.clearCategorical = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.LinguisticFeature.NumericalBoundaries.prototype.hasCategorical = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.LinguisticFeature.NumericalBoundaries}
 */
proto.speech.tts.LinguisticFeature.NumericalBoundaries.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.LinguisticFeature.NumericalBoundaries, data);
};


/**
 * optional string regexp_tag = 2;
 * @return {?string}
 */
proto.speech.tts.LinguisticFeature.prototype.getRegexpTag = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.LinguisticFeature} returns this
 */
proto.speech.tts.LinguisticFeature.prototype.setRegexpTag = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.LinguisticFeature} returns this
 */
proto.speech.tts.LinguisticFeature.prototype.clearRegexpTag = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.LinguisticFeature.prototype.hasRegexpTag = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional LinguisticFeatureValType type = 3;
 * @return {!proto.speech.tts.LinguisticFeatureValType}
 */
proto.speech.tts.LinguisticFeature.prototype.getType = function() {
  return /** @type {!proto.speech.tts.LinguisticFeatureValType} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?proto.speech.tts.LinguisticFeatureValType|undefined} value
 * @return {!proto.speech.tts.LinguisticFeature} returns this
 */
proto.speech.tts.LinguisticFeature.prototype.setType = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.LinguisticFeature} returns this
 */
proto.speech.tts.LinguisticFeature.prototype.clearType = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.LinguisticFeature.prototype.hasType = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional NavigationFeature nf = 4;
 * @return {?proto.speech.tts.NavigationFeature}
 */
proto.speech.tts.LinguisticFeature.prototype.getNf = function() {
  return /** @type{?proto.speech.tts.NavigationFeature} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.NavigationFeature, 4));
};


/**
 * @param {?proto.speech.tts.NavigationFeature|undefined} value
 * @return {!proto.speech.tts.LinguisticFeature} returns this
*/
proto.speech.tts.LinguisticFeature.prototype.setNf = function(value) {
  return jspb.Message.setWrapperField(this, 4, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.LinguisticFeature} returns this
 */
proto.speech.tts.LinguisticFeature.prototype.clearNf = function() {
  return this.setNf(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.LinguisticFeature.prototype.hasNf = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional CustomFeature custom = 11;
 * @return {?proto.speech.tts.CustomFeature}
 */
proto.speech.tts.LinguisticFeature.prototype.getCustom = function() {
  return /** @type{?proto.speech.tts.CustomFeature} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.CustomFeature, 11));
};


/**
 * @param {?proto.speech.tts.CustomFeature|undefined} value
 * @return {!proto.speech.tts.LinguisticFeature} returns this
*/
proto.speech.tts.LinguisticFeature.prototype.setCustom = function(value) {
  return jspb.Message.setWrapperField(this, 11, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.LinguisticFeature} returns this
 */
proto.speech.tts.LinguisticFeature.prototype.clearCustom = function() {
  return this.setCustom(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.LinguisticFeature.prototype.hasCustom = function() {
  return jspb.Message.getField(this, 11) != null;
};


/**
 * optional bool cache_values = 6;
 * @return {?boolean}
 */
proto.speech.tts.LinguisticFeature.prototype.getCacheValues = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 6));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.LinguisticFeature} returns this
 */
proto.speech.tts.LinguisticFeature.prototype.setCacheValues = function(value) {
  return jspb.Message.setOneofField(this, 6, proto.speech.tts.LinguisticFeature.oneofGroups_[0], value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.LinguisticFeature} returns this
 */
proto.speech.tts.LinguisticFeature.prototype.clearCacheValues = function() {
  return jspb.Message.setOneofField(this, 6, proto.speech.tts.LinguisticFeature.oneofGroups_[0], undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.LinguisticFeature.prototype.hasCacheValues = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional NumericalBoundaries numerical = 7;
 * @return {?proto.speech.tts.LinguisticFeature.NumericalBoundaries}
 */
proto.speech.tts.LinguisticFeature.prototype.getNumerical = function() {
  return /** @type{?proto.speech.tts.LinguisticFeature.NumericalBoundaries} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.LinguisticFeature.NumericalBoundaries, 7));
};


/**
 * @param {?proto.speech.tts.LinguisticFeature.NumericalBoundaries|undefined} value
 * @return {!proto.speech.tts.LinguisticFeature} returns this
*/
proto.speech.tts.LinguisticFeature.prototype.setNumerical = function(value) {
  return jspb.Message.setOneofWrapperField(this, 7, proto.speech.tts.LinguisticFeature.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.LinguisticFeature} returns this
 */
proto.speech.tts.LinguisticFeature.prototype.clearNumerical = function() {
  return this.setNumerical(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.LinguisticFeature.prototype.hasNumerical = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * repeated int32 skip_value = 8;
 * @return {!Array<number>}
 */
proto.speech.tts.LinguisticFeature.prototype.getSkipValueList = function() {
  return /** @type {!Array<number>} */ (jspb.Message.getRepeatedField(this, 8));
};


/**
 * @param {Array<number>|undefined} value
 * @return {!proto.speech.tts.LinguisticFeature} returns this
 */
proto.speech.tts.LinguisticFeature.prototype.setSkipValueList = function(value) {
  return jspb.Message.setField(this, 8, value || []);
};


/**
 * @param {number} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.LinguisticFeature} returns this
 */
proto.speech.tts.LinguisticFeature.prototype.addSkipValue = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 8, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.LinguisticFeature} returns this
 */
proto.speech.tts.LinguisticFeature.prototype.clearSkipValueList = function() {
  return this.setSkipValueList([]);
};


/**
 * repeated string skip_string_value = 9;
 * @return {!Array<string>}
 */
proto.speech.tts.LinguisticFeature.prototype.getSkipStringValueList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 9));
};


/**
 * @param {Array<string>|undefined} value
 * @return {!proto.speech.tts.LinguisticFeature} returns this
 */
proto.speech.tts.LinguisticFeature.prototype.setSkipStringValueList = function(value) {
  return jspb.Message.setField(this, 9, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.LinguisticFeature} returns this
 */
proto.speech.tts.LinguisticFeature.prototype.addSkipStringValue = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 9, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.LinguisticFeature} returns this
 */
proto.speech.tts.LinguisticFeature.prototype.clearSkipStringValueList = function() {
  return this.setSkipStringValueList([]);
};


/**
 * optional bool use_context = 10;
 * @return {boolean}
 */
proto.speech.tts.LinguisticFeature.prototype.getUseContext = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 10, false));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.LinguisticFeature} returns this
 */
proto.speech.tts.LinguisticFeature.prototype.setUseContext = function(value) {
  return jspb.Message.setField(this, 10, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.LinguisticFeature} returns this
 */
proto.speech.tts.LinguisticFeature.prototype.clearUseContext = function() {
  return jspb.Message.setField(this, 10, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.LinguisticFeature.prototype.hasUseContext = function() {
  return jspb.Message.getField(this, 10) != null;
};


/**
 * repeated string include_if_language = 12;
 * @return {!Array<string>}
 */
proto.speech.tts.LinguisticFeature.prototype.getIncludeIfLanguageList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 12));
};


/**
 * @param {Array<string>|undefined} value
 * @return {!proto.speech.tts.LinguisticFeature} returns this
 */
proto.speech.tts.LinguisticFeature.prototype.setIncludeIfLanguageList = function(value) {
  return jspb.Message.setField(this, 12, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.LinguisticFeature} returns this
 */
proto.speech.tts.LinguisticFeature.prototype.addIncludeIfLanguage = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 12, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.LinguisticFeature} returns this
 */
proto.speech.tts.LinguisticFeature.prototype.clearIncludeIfLanguageList = function() {
  return this.setIncludeIfLanguageList([]);
};


/**
 * repeated string exclude_if_language = 13;
 * @return {!Array<string>}
 */
proto.speech.tts.LinguisticFeature.prototype.getExcludeIfLanguageList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 13));
};


/**
 * @param {Array<string>|undefined} value
 * @return {!proto.speech.tts.LinguisticFeature} returns this
 */
proto.speech.tts.LinguisticFeature.prototype.setExcludeIfLanguageList = function(value) {
  return jspb.Message.setField(this, 13, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.LinguisticFeature} returns this
 */
proto.speech.tts.LinguisticFeature.prototype.addExcludeIfLanguage = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 13, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.LinguisticFeature} returns this
 */
proto.speech.tts.LinguisticFeature.prototype.clearExcludeIfLanguageList = function() {
  return this.setExcludeIfLanguageList([]);
};


/**
 * repeated string exclude_if_dataset = 14;
 * @return {!Array<string>}
 */
proto.speech.tts.LinguisticFeature.prototype.getExcludeIfDatasetList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 14));
};


/**
 * @param {Array<string>|undefined} value
 * @return {!proto.speech.tts.LinguisticFeature} returns this
 */
proto.speech.tts.LinguisticFeature.prototype.setExcludeIfDatasetList = function(value) {
  return jspb.Message.setField(this, 14, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.LinguisticFeature} returns this
 */
proto.speech.tts.LinguisticFeature.prototype.addExcludeIfDataset = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 14, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.LinguisticFeature} returns this
 */
proto.speech.tts.LinguisticFeature.prototype.clearExcludeIfDatasetList = function() {
  return this.setExcludeIfDatasetList([]);
};


/**
 * optional bool dont_prune = 15;
 * @return {boolean}
 */
proto.speech.tts.LinguisticFeature.prototype.getDontPrune = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 15, false));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.LinguisticFeature} returns this
 */
proto.speech.tts.LinguisticFeature.prototype.setDontPrune = function(value) {
  return jspb.Message.setField(this, 15, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.LinguisticFeature} returns this
 */
proto.speech.tts.LinguisticFeature.prototype.clearDontPrune = function() {
  return jspb.Message.setField(this, 15, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.LinguisticFeature.prototype.hasDontPrune = function() {
  return jspb.Message.getField(this, 15) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.LinguisticFeature}
 */
proto.speech.tts.LinguisticFeature.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.LinguisticFeature, data);
};



//speech/tts/engine/specification/linguistic_features_proto_jspb/linguisticfeaturesconfig.jspb.js
// source: speech/tts/engine/specification/linguistic_features.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.LinguisticFeaturesConfig');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.LinguisticFeature');

goog.forwardDeclare('proto.speech.tts.FeatureSet.UnitType');
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.LinguisticFeaturesConfig = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.LinguisticFeaturesConfig.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.LinguisticFeaturesConfig, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.LinguisticFeaturesConfig.displayName = 'proto.speech.tts.LinguisticFeaturesConfig';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.LinguisticFeaturesConfig.repeatedFields_ = [4];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.LinguisticFeaturesConfig.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.LinguisticFeaturesConfig.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.LinguisticFeaturesConfig} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.LinguisticFeaturesConfig.toObject = function(includeInstance, msg) {
  var f, obj = {
    unitType: jspb.Message.getFieldWithDefault(msg, 1, 2),
    contextSize: jspb.Message.getFieldWithDefault(msg, 2, 1),
    isSupraSegmental: jspb.Message.getBooleanFieldWithDefault(msg, 3, false),
    featureList: jspb.Message.toObjectList(msg.getFeatureList(),
    proto.speech.tts.LinguisticFeature.toObject, includeInstance)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of LinguisticFeaturesConfig as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.LinguisticFeaturesConfig.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.unitType;

  /** @type {?number|undefined} */
  this.contextSize;

  /** @type {?boolean|undefined} */
  this.isSupraSegmental;

  /** @type {?Array<!proto.speech.tts.LinguisticFeature.ObjectFormat>|undefined} */
  this.featureList;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.LinguisticFeaturesConfig.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.LinguisticFeaturesConfig}
 */
proto.speech.tts.LinguisticFeaturesConfig.fromObject = function(obj) {
  var msg = new proto.speech.tts.LinguisticFeaturesConfig();
  obj.unitType != null && jspb.Message.setField(msg, 1, obj.unitType);
  obj.contextSize != null && jspb.Message.setField(msg, 2, obj.contextSize);
  obj.isSupraSegmental != null && jspb.Message.setField(msg, 3, obj.isSupraSegmental);
  obj.featureList && jspb.Message.setRepeatedWrapperField(
      msg, 4, obj.featureList.map(
          proto.speech.tts.LinguisticFeature.fromObject));
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.LinguisticFeaturesConfig}
 */
proto.speech.tts.LinguisticFeaturesConfig.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.LinguisticFeaturesConfig;
  return proto.speech.tts.LinguisticFeaturesConfig.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.LinguisticFeaturesConfig} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.LinguisticFeaturesConfig}
 */
proto.speech.tts.LinguisticFeaturesConfig.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {!proto.speech.tts.FeatureSet.UnitType} */ (reader.readEnum());
      msg.setUnitType(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setContextSize(value);
      break;
    case 3:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsSupraSegmental(value);
      break;
    case 4:
      var value = new proto.speech.tts.LinguisticFeature;
      reader.readMessage(value,proto.speech.tts.LinguisticFeature.deserializeBinaryFromReader);
      msg.addFeature(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.LinguisticFeaturesConfig.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.LinguisticFeaturesConfig.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.LinguisticFeaturesConfig} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.LinguisticFeaturesConfig.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {!proto.speech.tts.FeatureSet.UnitType} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeEnum(
      1,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeInt32(
      2,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeBool(
      3,
      f
    );
  }
  f = message.getFeatureList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      4,
      f,
      proto.speech.tts.LinguisticFeature.serializeBinaryToWriter
    );
  }
};


/**
 * optional FeatureSet.UnitType unit_type = 1;
 * @return {!proto.speech.tts.FeatureSet.UnitType}
 */
proto.speech.tts.LinguisticFeaturesConfig.prototype.getUnitType = function() {
  return /** @type {!proto.speech.tts.FeatureSet.UnitType} */ (jspb.Message.getFieldWithDefault(this, 1, 2));
};


/**
 * @param {?proto.speech.tts.FeatureSet.UnitType|undefined} value
 * @return {!proto.speech.tts.LinguisticFeaturesConfig} returns this
 */
proto.speech.tts.LinguisticFeaturesConfig.prototype.setUnitType = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.LinguisticFeaturesConfig} returns this
 */
proto.speech.tts.LinguisticFeaturesConfig.prototype.clearUnitType = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.LinguisticFeaturesConfig.prototype.hasUnitType = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional int32 context_size = 2;
 * @return {number}
 */
proto.speech.tts.LinguisticFeaturesConfig.prototype.getContextSize = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 2, 1));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.LinguisticFeaturesConfig} returns this
 */
proto.speech.tts.LinguisticFeaturesConfig.prototype.setContextSize = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.LinguisticFeaturesConfig} returns this
 */
proto.speech.tts.LinguisticFeaturesConfig.prototype.clearContextSize = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.LinguisticFeaturesConfig.prototype.hasContextSize = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional bool is_supra_segmental = 3;
 * @return {boolean}
 */
proto.speech.tts.LinguisticFeaturesConfig.prototype.getIsSupraSegmental = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 3, false));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.LinguisticFeaturesConfig} returns this
 */
proto.speech.tts.LinguisticFeaturesConfig.prototype.setIsSupraSegmental = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.LinguisticFeaturesConfig} returns this
 */
proto.speech.tts.LinguisticFeaturesConfig.prototype.clearIsSupraSegmental = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.LinguisticFeaturesConfig.prototype.hasIsSupraSegmental = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * repeated LinguisticFeature feature = 4;
 * @return {!Array<!proto.speech.tts.LinguisticFeature>}
 */
proto.speech.tts.LinguisticFeaturesConfig.prototype.getFeatureList = function() {
  return /** @type{!Array<!proto.speech.tts.LinguisticFeature>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.LinguisticFeature, 4));
};


/**
 * @param {Array<!proto.speech.tts.LinguisticFeature>|undefined} value
 * @return {!proto.speech.tts.LinguisticFeaturesConfig} returns this
*/
proto.speech.tts.LinguisticFeaturesConfig.prototype.setFeatureList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 4, value);
};


/**
 * @param {!proto.speech.tts.LinguisticFeature=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.LinguisticFeature}
 */
proto.speech.tts.LinguisticFeaturesConfig.prototype.addFeature = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 4, opt_value, proto.speech.tts.LinguisticFeature, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.LinguisticFeaturesConfig} returns this
 */
proto.speech.tts.LinguisticFeaturesConfig.prototype.clearFeatureList = function() {
  return this.setFeatureList([]);
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.LinguisticFeaturesConfig}
 */
proto.speech.tts.LinguisticFeaturesConfig.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.LinguisticFeaturesConfig, data);
};



//speech/tts/engine/unit_selection/source_diagnostics_proto_jspb/sourcediagnostics.jspb.js
// source: speech/tts/engine/unit_selection/source_diagnostics.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.SourceDiagnostics');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.LinguisticFeaturesConfig');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.SourceDiagnostics = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.SourceDiagnostics.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.SourceDiagnostics, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.SourceDiagnostics.displayName = 'proto.speech.tts.SourceDiagnostics';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.SourceDiagnostics.repeatedFields_ = [2,3];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.SourceDiagnostics.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.SourceDiagnostics.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.SourceDiagnostics} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.SourceDiagnostics.toObject = function(includeInstance, msg) {
  var f, obj = {
    featuresConfig: (f = msg.getFeaturesConfig()) && proto.speech.tts.LinguisticFeaturesConfig.toObject(includeInstance, f),
    pitchmarkTimesSList: (f = jspb.Message.getRepeatedFloatingPointField(msg, 2)) == null ? undefined : f,
    vuiIdList: (f = jspb.Message.getRepeatedField(msg, 3)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of SourceDiagnostics as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.SourceDiagnostics.ObjectFormat = function() {
  /** @type {?proto.speech.tts.LinguisticFeaturesConfig.ObjectFormat|undefined} */
  this.featuresConfig;

  /** @type {?Array<number>|undefined} */
  this.pitchmarkTimesSList;

  /** @type {?Array<string>|undefined} */
  this.vuiIdList;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.SourceDiagnostics.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.SourceDiagnostics}
 */
proto.speech.tts.SourceDiagnostics.fromObject = function(obj) {
  var msg = new proto.speech.tts.SourceDiagnostics();
  obj.featuresConfig && jspb.Message.setWrapperField(
      msg, 1, proto.speech.tts.LinguisticFeaturesConfig.fromObject(obj.featuresConfig));
  obj.pitchmarkTimesSList != null && jspb.Message.setField(msg, 2, obj.pitchmarkTimesSList);
  obj.vuiIdList != null && jspb.Message.setField(msg, 3, obj.vuiIdList);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.SourceDiagnostics}
 */
proto.speech.tts.SourceDiagnostics.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.SourceDiagnostics;
  return proto.speech.tts.SourceDiagnostics.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.SourceDiagnostics} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.SourceDiagnostics}
 */
proto.speech.tts.SourceDiagnostics.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.speech.tts.LinguisticFeaturesConfig;
      reader.readMessage(value,proto.speech.tts.LinguisticFeaturesConfig.deserializeBinaryFromReader);
      msg.setFeaturesConfig(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readFloat());
      msg.addPitchmarkTimesS(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.addVuiId(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.SourceDiagnostics.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.SourceDiagnostics.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.SourceDiagnostics} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.SourceDiagnostics.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getFeaturesConfig();
  if (f != null) {
    writer.writeMessage(
      1,
      f,
      proto.speech.tts.LinguisticFeaturesConfig.serializeBinaryToWriter
    );
  }
  f = message.getPitchmarkTimesSList();
  if (f.length > 0) {
    writer.writeRepeatedFloat(
      2,
      f
    );
  }
  f = message.getVuiIdList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      3,
      f
    );
  }
};


/**
 * optional LinguisticFeaturesConfig features_config = 1;
 * @return {?proto.speech.tts.LinguisticFeaturesConfig}
 */
proto.speech.tts.SourceDiagnostics.prototype.getFeaturesConfig = function() {
  return /** @type{?proto.speech.tts.LinguisticFeaturesConfig} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.LinguisticFeaturesConfig, 1));
};


/**
 * @param {?proto.speech.tts.LinguisticFeaturesConfig|undefined} value
 * @return {!proto.speech.tts.SourceDiagnostics} returns this
*/
proto.speech.tts.SourceDiagnostics.prototype.setFeaturesConfig = function(value) {
  return jspb.Message.setWrapperField(this, 1, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.SourceDiagnostics} returns this
 */
proto.speech.tts.SourceDiagnostics.prototype.clearFeaturesConfig = function() {
  return this.setFeaturesConfig(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.SourceDiagnostics.prototype.hasFeaturesConfig = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * repeated float pitchmark_times_s = 2;
 * @return {!Array<number>}
 */
proto.speech.tts.SourceDiagnostics.prototype.getPitchmarkTimesSList = function() {
  return /** @type {!Array<number>} */ (jspb.Message.getRepeatedFloatingPointField(this, 2));
};


/**
 * @param {Array<number>|undefined} value
 * @return {!proto.speech.tts.SourceDiagnostics} returns this
 */
proto.speech.tts.SourceDiagnostics.prototype.setPitchmarkTimesSList = function(value) {
  return jspb.Message.setField(this, 2, value || []);
};


/**
 * @param {number} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.SourceDiagnostics} returns this
 */
proto.speech.tts.SourceDiagnostics.prototype.addPitchmarkTimesS = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 2, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.SourceDiagnostics} returns this
 */
proto.speech.tts.SourceDiagnostics.prototype.clearPitchmarkTimesSList = function() {
  return this.setPitchmarkTimesSList([]);
};


/**
 * repeated string vui_id = 3;
 * @return {!Array<string>}
 */
proto.speech.tts.SourceDiagnostics.prototype.getVuiIdList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 3));
};


/**
 * @param {Array<string>|undefined} value
 * @return {!proto.speech.tts.SourceDiagnostics} returns this
 */
proto.speech.tts.SourceDiagnostics.prototype.setVuiIdList = function(value) {
  return jspb.Message.setField(this, 3, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.SourceDiagnostics} returns this
 */
proto.speech.tts.SourceDiagnostics.prototype.addVuiId = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 3, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.SourceDiagnostics} returns this
 */
proto.speech.tts.SourceDiagnostics.prototype.clearVuiIdList = function() {
  return this.setVuiIdList([]);
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.SourceDiagnostics}
 */
proto.speech.tts.SourceDiagnostics.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.SourceDiagnostics, data);
};



//speech/tts/engine/unit_selection/lattice_proto_jspb/modelscoreproto.jspb.js
// source: speech/tts/engine/unit_selection/lattice.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.ModelScoreProto');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.ModelScoreProto = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.ModelScoreProto, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.ModelScoreProto.displayName = 'proto.speech.tts.ModelScoreProto';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.ModelScoreProto.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.ModelScoreProto.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.ModelScoreProto} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.ModelScoreProto.toObject = function(includeInstance, msg) {
  var f, obj = {
    modelId: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    modelScore: (f = jspb.Message.getOptionalFloatingPointField(msg, 2)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of ModelScoreProto as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.ModelScoreProto.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.modelId;

  /** @type {?number|undefined} */
  this.modelScore;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.ModelScoreProto.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.ModelScoreProto}
 */
proto.speech.tts.ModelScoreProto.fromObject = function(obj) {
  var msg = new proto.speech.tts.ModelScoreProto();
  obj.modelId != null && jspb.Message.setField(msg, 1, obj.modelId);
  obj.modelScore != null && jspb.Message.setField(msg, 2, obj.modelScore);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.ModelScoreProto}
 */
proto.speech.tts.ModelScoreProto.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.ModelScoreProto;
  return proto.speech.tts.ModelScoreProto.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.ModelScoreProto} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.ModelScoreProto}
 */
proto.speech.tts.ModelScoreProto.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setModelId(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setModelScore(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.ModelScoreProto.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.ModelScoreProto.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.ModelScoreProto} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.ModelScoreProto.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?number} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeInt32(
      1,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeFloat(
      2,
      f
    );
  }
};


/**
 * optional int32 model_id = 1;
 * @return {?number}
 */
proto.speech.tts.ModelScoreProto.prototype.getModelId = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.ModelScoreProto} returns this
 */
proto.speech.tts.ModelScoreProto.prototype.setModelId = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.ModelScoreProto} returns this
 */
proto.speech.tts.ModelScoreProto.prototype.clearModelId = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.ModelScoreProto.prototype.hasModelId = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional float model_score = 2;
 * @return {?number}
 */
proto.speech.tts.ModelScoreProto.prototype.getModelScore = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 2));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.ModelScoreProto} returns this
 */
proto.speech.tts.ModelScoreProto.prototype.setModelScore = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.ModelScoreProto} returns this
 */
proto.speech.tts.ModelScoreProto.prototype.clearModelScore = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.ModelScoreProto.prototype.hasModelScore = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.ModelScoreProto}
 */
proto.speech.tts.ModelScoreProto.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.ModelScoreProto, data);
};



//speech/tts/engine/unit_selection/lattice_proto_jspb/partialpathinfoproto.jspb.js
// source: speech/tts/engine/unit_selection/lattice.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.PartialPathInfoProto');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.PartialPathInfoProto = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.PartialPathInfoProto, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.PartialPathInfoProto.displayName = 'proto.speech.tts.PartialPathInfoProto';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.PartialPathInfoProto.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.PartialPathInfoProto.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.PartialPathInfoProto} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.PartialPathInfoProto.toObject = function(includeInstance, msg) {
  var f, obj = {
    id: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    score: (f = jspb.Message.getOptionalFloatingPointField(msg, 2)) == null ? undefined : f,
    lastJoinCost: (f = jspb.Message.getOptionalFloatingPointField(msg, 3)) == null ? undefined : f,
    previousSlot: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of PartialPathInfoProto as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.PartialPathInfoProto.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.id;

  /** @type {?number|undefined} */
  this.score;

  /** @type {?number|undefined} */
  this.lastJoinCost;

  /** @type {?number|undefined} */
  this.previousSlot;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.PartialPathInfoProto.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.PartialPathInfoProto}
 */
proto.speech.tts.PartialPathInfoProto.fromObject = function(obj) {
  var msg = new proto.speech.tts.PartialPathInfoProto();
  obj.id != null && jspb.Message.setField(msg, 1, obj.id);
  obj.score != null && jspb.Message.setField(msg, 2, obj.score);
  obj.lastJoinCost != null && jspb.Message.setField(msg, 3, obj.lastJoinCost);
  obj.previousSlot != null && jspb.Message.setField(msg, 4, obj.previousSlot);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.PartialPathInfoProto}
 */
proto.speech.tts.PartialPathInfoProto.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.PartialPathInfoProto;
  return proto.speech.tts.PartialPathInfoProto.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.PartialPathInfoProto} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.PartialPathInfoProto}
 */
proto.speech.tts.PartialPathInfoProto.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setId(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setScore(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setLastJoinCost(value);
      break;
    case 4:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setPreviousSlot(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.PartialPathInfoProto.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.PartialPathInfoProto.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.PartialPathInfoProto} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.PartialPathInfoProto.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?number} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeInt32(
      1,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeFloat(
      2,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeFloat(
      3,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeInt32(
      4,
      f
    );
  }
};


/**
 * optional int32 id = 1;
 * @return {?number}
 */
proto.speech.tts.PartialPathInfoProto.prototype.getId = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.PartialPathInfoProto} returns this
 */
proto.speech.tts.PartialPathInfoProto.prototype.setId = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.PartialPathInfoProto} returns this
 */
proto.speech.tts.PartialPathInfoProto.prototype.clearId = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.PartialPathInfoProto.prototype.hasId = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional float score = 2;
 * @return {?number}
 */
proto.speech.tts.PartialPathInfoProto.prototype.getScore = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 2));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.PartialPathInfoProto} returns this
 */
proto.speech.tts.PartialPathInfoProto.prototype.setScore = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.PartialPathInfoProto} returns this
 */
proto.speech.tts.PartialPathInfoProto.prototype.clearScore = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.PartialPathInfoProto.prototype.hasScore = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional float last_join_cost = 3;
 * @return {?number}
 */
proto.speech.tts.PartialPathInfoProto.prototype.getLastJoinCost = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 3));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.PartialPathInfoProto} returns this
 */
proto.speech.tts.PartialPathInfoProto.prototype.setLastJoinCost = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.PartialPathInfoProto} returns this
 */
proto.speech.tts.PartialPathInfoProto.prototype.clearLastJoinCost = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.PartialPathInfoProto.prototype.hasLastJoinCost = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional int32 previous_slot = 4;
 * @return {?number}
 */
proto.speech.tts.PartialPathInfoProto.prototype.getPreviousSlot = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 4));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.PartialPathInfoProto} returns this
 */
proto.speech.tts.PartialPathInfoProto.prototype.setPreviousSlot = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.PartialPathInfoProto} returns this
 */
proto.speech.tts.PartialPathInfoProto.prototype.clearPreviousSlot = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.PartialPathInfoProto.prototype.hasPreviousSlot = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.PartialPathInfoProto}
 */
proto.speech.tts.PartialPathInfoProto.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.PartialPathInfoProto, data);
};



//speech/tts/engine/unit_selection/lattice_proto_jspb/unitslotproto.jspb.js
// source: speech/tts/engine/unit_selection/lattice.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.UnitSlotProto');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.PartialPathInfoProto');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.UnitSlotProto = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.UnitSlotProto.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.UnitSlotProto, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.UnitSlotProto.displayName = 'proto.speech.tts.UnitSlotProto';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.UnitSlotProto.repeatedFields_ = [3];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.UnitSlotProto.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.UnitSlotProto.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.UnitSlotProto} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.UnitSlotProto.toObject = function(includeInstance, msg) {
  var f, obj = {
    unitId: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    targetCost: (f = jspb.Message.getOptionalFloatingPointField(msg, 2)) == null ? undefined : f,
    pathList: jspb.Message.toObjectList(msg.getPathList(),
    proto.speech.tts.PartialPathInfoProto.toObject, includeInstance),
    unitIsArtificial: (f = jspb.Message.getBooleanField(msg, 4)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of UnitSlotProto as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.UnitSlotProto.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.unitId;

  /** @type {?number|undefined} */
  this.targetCost;

  /** @type {?Array<!proto.speech.tts.PartialPathInfoProto.ObjectFormat>|undefined} */
  this.pathList;

  /** @type {?boolean|undefined} */
  this.unitIsArtificial;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.UnitSlotProto.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.UnitSlotProto}
 */
proto.speech.tts.UnitSlotProto.fromObject = function(obj) {
  var msg = new proto.speech.tts.UnitSlotProto();
  obj.unitId != null && jspb.Message.setField(msg, 1, obj.unitId);
  obj.targetCost != null && jspb.Message.setField(msg, 2, obj.targetCost);
  obj.pathList && jspb.Message.setRepeatedWrapperField(
      msg, 3, obj.pathList.map(
          proto.speech.tts.PartialPathInfoProto.fromObject));
  obj.unitIsArtificial != null && jspb.Message.setField(msg, 4, obj.unitIsArtificial);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.UnitSlotProto}
 */
proto.speech.tts.UnitSlotProto.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.UnitSlotProto;
  return proto.speech.tts.UnitSlotProto.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.UnitSlotProto} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.UnitSlotProto}
 */
proto.speech.tts.UnitSlotProto.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setUnitId(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setTargetCost(value);
      break;
    case 3:
      var value = new proto.speech.tts.PartialPathInfoProto;
      reader.readMessage(value,proto.speech.tts.PartialPathInfoProto.deserializeBinaryFromReader);
      msg.addPath(value);
      break;
    case 4:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setUnitIsArtificial(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.UnitSlotProto.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.UnitSlotProto.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.UnitSlotProto} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.UnitSlotProto.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?number} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeInt32(
      1,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeFloat(
      2,
      f
    );
  }
  f = message.getPathList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      3,
      f,
      proto.speech.tts.PartialPathInfoProto.serializeBinaryToWriter
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeBool(
      4,
      f
    );
  }
};


/**
 * optional int32 unit_id = 1;
 * @return {?number}
 */
proto.speech.tts.UnitSlotProto.prototype.getUnitId = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.UnitSlotProto} returns this
 */
proto.speech.tts.UnitSlotProto.prototype.setUnitId = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.UnitSlotProto} returns this
 */
proto.speech.tts.UnitSlotProto.prototype.clearUnitId = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.UnitSlotProto.prototype.hasUnitId = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional float target_cost = 2;
 * @return {?number}
 */
proto.speech.tts.UnitSlotProto.prototype.getTargetCost = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 2));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.UnitSlotProto} returns this
 */
proto.speech.tts.UnitSlotProto.prototype.setTargetCost = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.UnitSlotProto} returns this
 */
proto.speech.tts.UnitSlotProto.prototype.clearTargetCost = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.UnitSlotProto.prototype.hasTargetCost = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * repeated PartialPathInfoProto path = 3;
 * @return {!Array<!proto.speech.tts.PartialPathInfoProto>}
 */
proto.speech.tts.UnitSlotProto.prototype.getPathList = function() {
  return /** @type{!Array<!proto.speech.tts.PartialPathInfoProto>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.PartialPathInfoProto, 3));
};


/**
 * @param {Array<!proto.speech.tts.PartialPathInfoProto>|undefined} value
 * @return {!proto.speech.tts.UnitSlotProto} returns this
*/
proto.speech.tts.UnitSlotProto.prototype.setPathList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 3, value);
};


/**
 * @param {!proto.speech.tts.PartialPathInfoProto=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.PartialPathInfoProto}
 */
proto.speech.tts.UnitSlotProto.prototype.addPath = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 3, opt_value, proto.speech.tts.PartialPathInfoProto, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.UnitSlotProto} returns this
 */
proto.speech.tts.UnitSlotProto.prototype.clearPathList = function() {
  return this.setPathList([]);
};


/**
 * optional bool unit_is_artificial = 4;
 * @return {?boolean}
 */
proto.speech.tts.UnitSlotProto.prototype.getUnitIsArtificial = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 4));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.UnitSlotProto} returns this
 */
proto.speech.tts.UnitSlotProto.prototype.setUnitIsArtificial = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.UnitSlotProto} returns this
 */
proto.speech.tts.UnitSlotProto.prototype.clearUnitIsArtificial = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.UnitSlotProto.prototype.hasUnitIsArtificial = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.UnitSlotProto}
 */
proto.speech.tts.UnitSlotProto.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.UnitSlotProto, data);
};



//speech/tts/engine/unit_selection/lattice_proto_jspb/targetcolumnproto.jspb.js
// source: speech/tts/engine/unit_selection/lattice.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.TargetColumnProto');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.ModelScoreProto');
goog.require('proto.speech.tts.UnitSlotProto');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.TargetColumnProto = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.TargetColumnProto.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.TargetColumnProto, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.TargetColumnProto.displayName = 'proto.speech.tts.TargetColumnProto';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.TargetColumnProto.repeatedFields_ = [2,3];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.TargetColumnProto.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.TargetColumnProto.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.TargetColumnProto} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.TargetColumnProto.toObject = function(includeInstance, msg) {
  var f, obj = {
    name: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    unitSlotList: jspb.Message.toObjectList(msg.getUnitSlotList(),
    proto.speech.tts.UnitSlotProto.toObject, includeInstance),
    modelScoreList: jspb.Message.toObjectList(msg.getModelScoreList(),
    proto.speech.tts.ModelScoreProto.toObject, includeInstance),
    leftPhonemeIsSilence: (f = jspb.Message.getBooleanField(msg, 4)) == null ? undefined : f,
    leftPhonemeIsObstruent: (f = jspb.Message.getBooleanField(msg, 5)) == null ? undefined : f,
    unitIsArtificial: (f = jspb.Message.getBooleanField(msg, 6)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of TargetColumnProto as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.TargetColumnProto.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.name;

  /** @type {?Array<!proto.speech.tts.UnitSlotProto.ObjectFormat>|undefined} */
  this.unitSlotList;

  /** @type {?Array<!proto.speech.tts.ModelScoreProto.ObjectFormat>|undefined} */
  this.modelScoreList;

  /** @type {?boolean|undefined} */
  this.leftPhonemeIsSilence;

  /** @type {?boolean|undefined} */
  this.leftPhonemeIsObstruent;

  /** @type {?boolean|undefined} */
  this.unitIsArtificial;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.TargetColumnProto.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.TargetColumnProto}
 */
proto.speech.tts.TargetColumnProto.fromObject = function(obj) {
  var msg = new proto.speech.tts.TargetColumnProto();
  obj.name != null && jspb.Message.setField(msg, 1, obj.name);
  obj.unitSlotList && jspb.Message.setRepeatedWrapperField(
      msg, 2, obj.unitSlotList.map(
          proto.speech.tts.UnitSlotProto.fromObject));
  obj.modelScoreList && jspb.Message.setRepeatedWrapperField(
      msg, 3, obj.modelScoreList.map(
          proto.speech.tts.ModelScoreProto.fromObject));
  obj.leftPhonemeIsSilence != null && jspb.Message.setField(msg, 4, obj.leftPhonemeIsSilence);
  obj.leftPhonemeIsObstruent != null && jspb.Message.setField(msg, 5, obj.leftPhonemeIsObstruent);
  obj.unitIsArtificial != null && jspb.Message.setField(msg, 6, obj.unitIsArtificial);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.TargetColumnProto}
 */
proto.speech.tts.TargetColumnProto.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.TargetColumnProto;
  return proto.speech.tts.TargetColumnProto.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.TargetColumnProto} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.TargetColumnProto}
 */
proto.speech.tts.TargetColumnProto.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setName(value);
      break;
    case 2:
      var value = new proto.speech.tts.UnitSlotProto;
      reader.readMessage(value,proto.speech.tts.UnitSlotProto.deserializeBinaryFromReader);
      msg.addUnitSlot(value);
      break;
    case 3:
      var value = new proto.speech.tts.ModelScoreProto;
      reader.readMessage(value,proto.speech.tts.ModelScoreProto.deserializeBinaryFromReader);
      msg.addModelScore(value);
      break;
    case 4:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setLeftPhonemeIsSilence(value);
      break;
    case 5:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setLeftPhonemeIsObstruent(value);
      break;
    case 6:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setUnitIsArtificial(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.TargetColumnProto.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.TargetColumnProto.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.TargetColumnProto} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.TargetColumnProto.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = message.getUnitSlotList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      2,
      f,
      proto.speech.tts.UnitSlotProto.serializeBinaryToWriter
    );
  }
  f = message.getModelScoreList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      3,
      f,
      proto.speech.tts.ModelScoreProto.serializeBinaryToWriter
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeBool(
      4,
      f
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeBool(
      5,
      f
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeBool(
      6,
      f
    );
  }
};


/**
 * optional string name = 1;
 * @return {?string}
 */
proto.speech.tts.TargetColumnProto.prototype.getName = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.TargetColumnProto} returns this
 */
proto.speech.tts.TargetColumnProto.prototype.setName = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.TargetColumnProto} returns this
 */
proto.speech.tts.TargetColumnProto.prototype.clearName = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.TargetColumnProto.prototype.hasName = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * repeated UnitSlotProto unit_slot = 2;
 * @return {!Array<!proto.speech.tts.UnitSlotProto>}
 */
proto.speech.tts.TargetColumnProto.prototype.getUnitSlotList = function() {
  return /** @type{!Array<!proto.speech.tts.UnitSlotProto>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.UnitSlotProto, 2));
};


/**
 * @param {Array<!proto.speech.tts.UnitSlotProto>|undefined} value
 * @return {!proto.speech.tts.TargetColumnProto} returns this
*/
proto.speech.tts.TargetColumnProto.prototype.setUnitSlotList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 2, value);
};


/**
 * @param {!proto.speech.tts.UnitSlotProto=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.UnitSlotProto}
 */
proto.speech.tts.TargetColumnProto.prototype.addUnitSlot = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 2, opt_value, proto.speech.tts.UnitSlotProto, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.TargetColumnProto} returns this
 */
proto.speech.tts.TargetColumnProto.prototype.clearUnitSlotList = function() {
  return this.setUnitSlotList([]);
};


/**
 * repeated ModelScoreProto model_score = 3;
 * @return {!Array<!proto.speech.tts.ModelScoreProto>}
 */
proto.speech.tts.TargetColumnProto.prototype.getModelScoreList = function() {
  return /** @type{!Array<!proto.speech.tts.ModelScoreProto>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.ModelScoreProto, 3));
};


/**
 * @param {Array<!proto.speech.tts.ModelScoreProto>|undefined} value
 * @return {!proto.speech.tts.TargetColumnProto} returns this
*/
proto.speech.tts.TargetColumnProto.prototype.setModelScoreList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 3, value);
};


/**
 * @param {!proto.speech.tts.ModelScoreProto=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.ModelScoreProto}
 */
proto.speech.tts.TargetColumnProto.prototype.addModelScore = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 3, opt_value, proto.speech.tts.ModelScoreProto, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.TargetColumnProto} returns this
 */
proto.speech.tts.TargetColumnProto.prototype.clearModelScoreList = function() {
  return this.setModelScoreList([]);
};


/**
 * optional bool left_phoneme_is_silence = 4;
 * @return {?boolean}
 */
proto.speech.tts.TargetColumnProto.prototype.getLeftPhonemeIsSilence = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 4));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.TargetColumnProto} returns this
 */
proto.speech.tts.TargetColumnProto.prototype.setLeftPhonemeIsSilence = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.TargetColumnProto} returns this
 */
proto.speech.tts.TargetColumnProto.prototype.clearLeftPhonemeIsSilence = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.TargetColumnProto.prototype.hasLeftPhonemeIsSilence = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional bool left_phoneme_is_obstruent = 5;
 * @return {?boolean}
 */
proto.speech.tts.TargetColumnProto.prototype.getLeftPhonemeIsObstruent = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 5));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.TargetColumnProto} returns this
 */
proto.speech.tts.TargetColumnProto.prototype.setLeftPhonemeIsObstruent = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.TargetColumnProto} returns this
 */
proto.speech.tts.TargetColumnProto.prototype.clearLeftPhonemeIsObstruent = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.TargetColumnProto.prototype.hasLeftPhonemeIsObstruent = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional bool unit_is_artificial = 6;
 * @return {?boolean}
 */
proto.speech.tts.TargetColumnProto.prototype.getUnitIsArtificial = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 6));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.TargetColumnProto} returns this
 */
proto.speech.tts.TargetColumnProto.prototype.setUnitIsArtificial = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.TargetColumnProto} returns this
 */
proto.speech.tts.TargetColumnProto.prototype.clearUnitIsArtificial = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.TargetColumnProto.prototype.hasUnitIsArtificial = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.TargetColumnProto}
 */
proto.speech.tts.TargetColumnProto.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.TargetColumnProto, data);
};



//speech/tts/engine/unit_selection/lattice_proto_jspb/latticeproto.jspb.js
// source: speech/tts/engine/unit_selection/lattice.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.LatticeProto');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.TargetColumnProto');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.LatticeProto = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.LatticeProto.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.LatticeProto, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.LatticeProto.displayName = 'proto.speech.tts.LatticeProto';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.LatticeProto.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.LatticeProto.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.LatticeProto.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.LatticeProto} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.LatticeProto.toObject = function(includeInstance, msg) {
  var f, obj = {
    targetColumnList: jspb.Message.toObjectList(msg.getTargetColumnList(),
    proto.speech.tts.TargetColumnProto.toObject, includeInstance)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of LatticeProto as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.LatticeProto.ObjectFormat = function() {
  /** @type {?Array<!proto.speech.tts.TargetColumnProto.ObjectFormat>|undefined} */
  this.targetColumnList;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.LatticeProto.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.LatticeProto}
 */
proto.speech.tts.LatticeProto.fromObject = function(obj) {
  var msg = new proto.speech.tts.LatticeProto();
  obj.targetColumnList && jspb.Message.setRepeatedWrapperField(
      msg, 1, obj.targetColumnList.map(
          proto.speech.tts.TargetColumnProto.fromObject));
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.LatticeProto}
 */
proto.speech.tts.LatticeProto.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.LatticeProto;
  return proto.speech.tts.LatticeProto.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.LatticeProto} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.LatticeProto}
 */
proto.speech.tts.LatticeProto.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.speech.tts.TargetColumnProto;
      reader.readMessage(value,proto.speech.tts.TargetColumnProto.deserializeBinaryFromReader);
      msg.addTargetColumn(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.LatticeProto.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.LatticeProto.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.LatticeProto} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.LatticeProto.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getTargetColumnList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      1,
      f,
      proto.speech.tts.TargetColumnProto.serializeBinaryToWriter
    );
  }
};


/**
 * repeated TargetColumnProto target_column = 1;
 * @return {!Array<!proto.speech.tts.TargetColumnProto>}
 */
proto.speech.tts.LatticeProto.prototype.getTargetColumnList = function() {
  return /** @type{!Array<!proto.speech.tts.TargetColumnProto>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.TargetColumnProto, 1));
};


/**
 * @param {Array<!proto.speech.tts.TargetColumnProto>|undefined} value
 * @return {!proto.speech.tts.LatticeProto} returns this
*/
proto.speech.tts.LatticeProto.prototype.setTargetColumnList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 1, value);
};


/**
 * @param {!proto.speech.tts.TargetColumnProto=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.TargetColumnProto}
 */
proto.speech.tts.LatticeProto.prototype.addTargetColumn = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.speech.tts.TargetColumnProto, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.LatticeProto} returns this
 */
proto.speech.tts.LatticeProto.prototype.clearTargetColumnList = function() {
  return this.setTargetColumnList([]);
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.LatticeProto}
 */
proto.speech.tts.LatticeProto.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.LatticeProto, data);
};



//speech/tts/engine/utterance/features_proto_jspb/acousticfeature.jspb.js
// source: speech/tts/engine/utterance/features.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.AcousticFeature');
goog.provide('proto.speech.tts.AcousticFeature.Orientation');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.AcousticFeature = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.AcousticFeature, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.AcousticFeature.displayName = 'proto.speech.tts.AcousticFeature';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.AcousticFeature.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.AcousticFeature.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.AcousticFeature} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.AcousticFeature.toObject = function(includeInstance, msg) {
  var f, obj = {
    filename: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    size: jspb.Message.getFieldWithDefault(msg, 2, 1),
    orient: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,
    feature: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of AcousticFeature as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.AcousticFeature.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.filename;

  /** @type {?number|undefined} */
  this.size;

  /** @type {?number|undefined} */
  this.orient;

  /** @type {?string|undefined} */
  this.feature;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.AcousticFeature.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.AcousticFeature}
 */
proto.speech.tts.AcousticFeature.fromObject = function(obj) {
  var msg = new proto.speech.tts.AcousticFeature();
  obj.filename != null && jspb.Message.setField(msg, 1, obj.filename);
  obj.size != null && jspb.Message.setField(msg, 2, obj.size);
  obj.orient != null && jspb.Message.setField(msg, 3, obj.orient);
  obj.feature != null && jspb.Message.setField(msg, 4, obj.feature);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.AcousticFeature}
 */
proto.speech.tts.AcousticFeature.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.AcousticFeature;
  return proto.speech.tts.AcousticFeature.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.AcousticFeature} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.AcousticFeature}
 */
proto.speech.tts.AcousticFeature.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setFilename(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readUint32());
      msg.setSize(value);
      break;
    case 3:
      var value = /** @type {!proto.speech.tts.AcousticFeature.Orientation} */ (reader.readEnum());
      msg.setOrient(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setFeature(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.AcousticFeature.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.AcousticFeature.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.AcousticFeature} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.AcousticFeature.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeUint32(
      2,
      f
    );
  }
  f = /** @type {!proto.speech.tts.AcousticFeature.Orientation} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeEnum(
      3,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeString(
      4,
      f
    );
  }
};


/**
 * @enum {number}
 */
proto.speech.tts.AcousticFeature.Orientation = {
  EDGE: 1,
  MID: 2
};

/**
 * optional string filename = 1;
 * @return {?string}
 */
proto.speech.tts.AcousticFeature.prototype.getFilename = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.AcousticFeature} returns this
 */
proto.speech.tts.AcousticFeature.prototype.setFilename = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.AcousticFeature} returns this
 */
proto.speech.tts.AcousticFeature.prototype.clearFilename = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.AcousticFeature.prototype.hasFilename = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional uint32 size = 2;
 * @return {number}
 */
proto.speech.tts.AcousticFeature.prototype.getSize = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 2, 1));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.AcousticFeature} returns this
 */
proto.speech.tts.AcousticFeature.prototype.setSize = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.AcousticFeature} returns this
 */
proto.speech.tts.AcousticFeature.prototype.clearSize = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.AcousticFeature.prototype.hasSize = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional Orientation orient = 3;
 * @return {!proto.speech.tts.AcousticFeature.Orientation}
 */
proto.speech.tts.AcousticFeature.prototype.getOrient = function() {
  return /** @type {!proto.speech.tts.AcousticFeature.Orientation} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?proto.speech.tts.AcousticFeature.Orientation|undefined} value
 * @return {!proto.speech.tts.AcousticFeature} returns this
 */
proto.speech.tts.AcousticFeature.prototype.setOrient = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.AcousticFeature} returns this
 */
proto.speech.tts.AcousticFeature.prototype.clearOrient = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.AcousticFeature.prototype.hasOrient = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional string feature = 4;
 * @return {?string}
 */
proto.speech.tts.AcousticFeature.prototype.getFeature = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 4));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.AcousticFeature} returns this
 */
proto.speech.tts.AcousticFeature.prototype.setFeature = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.AcousticFeature} returns this
 */
proto.speech.tts.AcousticFeature.prototype.clearFeature = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.AcousticFeature.prototype.hasFeature = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.AcousticFeature}
 */
proto.speech.tts.AcousticFeature.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.AcousticFeature, data);
};



//speech/tts/engine/utterance/features_proto_jspb/question.jspb.js
// source: speech/tts/engine/utterance/features.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.Question');
goog.provide('proto.speech.tts.Question.Comparison');
goog.provide('proto.speech.tts.Question.Conjunction');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.NavigationFeature');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.Question = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.Question.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.Question, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.Question.displayName = 'proto.speech.tts.Question';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.Question.repeatedFields_ = [6];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.Question.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.Question.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.Question} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Question.toObject = function(includeInstance, msg) {
  var f, obj = {
    nf: (f = msg.getNf()) && proto.speech.tts.NavigationFeature.toObject(includeInstance, f),
    rhs: (f = msg.getRhs()) && proto.speech.tts.NavigationFeature.toObject(includeInstance, f),
    bval: (f = jspb.Message.getBooleanField(msg, 2)) == null ? undefined : f,
    ival: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,
    sval: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f,
    fval: (f = jspb.Message.getOptionalFloatingPointField(msg, 8)) == null ? undefined : f,
    comp: (f = jspb.Message.getField(msg, 5)) == null ? undefined : f,
    qList: jspb.Message.toObjectList(msg.getQList(),
    proto.speech.tts.Question.toObject, includeInstance),
    conj: (f = jspb.Message.getField(msg, 7)) == null ? undefined : f,
    numValues: (f = jspb.Message.getField(msg, 10)) == null ? undefined : f,
    valuesHash: (f = jspb.Message.getField(msg, 11)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Question as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.Question.ObjectFormat = function() {
  /** @type {?proto.speech.tts.NavigationFeature.ObjectFormat|undefined} */
  this.nf;

  /** @type {?proto.speech.tts.NavigationFeature.ObjectFormat|undefined} */
  this.rhs;

  /** @type {?boolean|undefined} */
  this.bval;

  /** @type {?number|undefined} */
  this.ival;

  /** @type {?string|undefined} */
  this.sval;

  /** @type {?number|undefined} */
  this.fval;

  /** @type {?number|undefined} */
  this.comp;

  /** @type {?Array<!proto.speech.tts.Question.ObjectFormat>|undefined} */
  this.qList;

  /** @type {?number|undefined} */
  this.conj;

  /** @type {?number|undefined} */
  this.numValues;

  /** @type {?number|undefined} */
  this.valuesHash;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.Question.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.Question}
 */
proto.speech.tts.Question.fromObject = function(obj) {
  var msg = new proto.speech.tts.Question();
  obj.nf && jspb.Message.setWrapperField(
      msg, 1, proto.speech.tts.NavigationFeature.fromObject(obj.nf));
  obj.rhs && jspb.Message.setWrapperField(
      msg, 9, proto.speech.tts.NavigationFeature.fromObject(obj.rhs));
  obj.bval != null && jspb.Message.setField(msg, 2, obj.bval);
  obj.ival != null && jspb.Message.setField(msg, 3, obj.ival);
  obj.sval != null && jspb.Message.setField(msg, 4, obj.sval);
  obj.fval != null && jspb.Message.setField(msg, 8, obj.fval);
  obj.comp != null && jspb.Message.setField(msg, 5, obj.comp);
  obj.qList && jspb.Message.setRepeatedWrapperField(
      msg, 6, obj.qList.map(
          proto.speech.tts.Question.fromObject));
  obj.conj != null && jspb.Message.setField(msg, 7, obj.conj);
  obj.numValues != null && jspb.Message.setField(msg, 10, obj.numValues);
  obj.valuesHash != null && jspb.Message.setField(msg, 11, obj.valuesHash);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.Question}
 */
proto.speech.tts.Question.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.Question;
  return proto.speech.tts.Question.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.Question} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.Question}
 */
proto.speech.tts.Question.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.speech.tts.NavigationFeature;
      reader.readMessage(value,proto.speech.tts.NavigationFeature.deserializeBinaryFromReader);
      msg.setNf(value);
      break;
    case 9:
      var value = new proto.speech.tts.NavigationFeature;
      reader.readMessage(value,proto.speech.tts.NavigationFeature.deserializeBinaryFromReader);
      msg.setRhs(value);
      break;
    case 2:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setBval(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setIval(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setSval(value);
      break;
    case 8:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setFval(value);
      break;
    case 5:
      var value = /** @type {!proto.speech.tts.Question.Comparison} */ (reader.readEnum());
      msg.setComp(value);
      break;
    case 6:
      var value = new proto.speech.tts.Question;
      reader.readMessage(value,proto.speech.tts.Question.deserializeBinaryFromReader);
      msg.addQ(value);
      break;
    case 7:
      var value = /** @type {!proto.speech.tts.Question.Conjunction} */ (reader.readEnum());
      msg.setConj(value);
      break;
    case 10:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setNumValues(value);
      break;
    case 11:
      var value = /** @type {number} */ (reader.readUint64());
      msg.setValuesHash(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.Question.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.Question.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.Question} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Question.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getNf();
  if (f != null) {
    writer.writeMessage(
      1,
      f,
      proto.speech.tts.NavigationFeature.serializeBinaryToWriter
    );
  }
  f = message.getRhs();
  if (f != null) {
    writer.writeMessage(
      9,
      f,
      proto.speech.tts.NavigationFeature.serializeBinaryToWriter
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeBool(
      2,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeInt32(
      3,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeString(
      4,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 8));
  if (f != null) {
    writer.writeFloat(
      8,
      f
    );
  }
  f = /** @type {!proto.speech.tts.Question.Comparison} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeEnum(
      5,
      f
    );
  }
  f = message.getQList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      6,
      f,
      proto.speech.tts.Question.serializeBinaryToWriter
    );
  }
  f = /** @type {!proto.speech.tts.Question.Conjunction} */ (jspb.Message.getField(message, 7));
  if (f != null) {
    writer.writeEnum(
      7,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 10));
  if (f != null) {
    writer.writeInt32(
      10,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 11));
  if (f != null) {
    writer.writeUint64(
      11,
      f
    );
  }
};


/**
 * @enum {number}
 */
proto.speech.tts.Question.Comparison = {
  EQ: 1,
  LT: 2
};

/**
 * @enum {number}
 */
proto.speech.tts.Question.Conjunction = {
  AND: 1,
  OR: 2
};

/**
 * optional NavigationFeature nf = 1;
 * @return {?proto.speech.tts.NavigationFeature}
 */
proto.speech.tts.Question.prototype.getNf = function() {
  return /** @type{?proto.speech.tts.NavigationFeature} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.NavigationFeature, 1));
};


/**
 * @param {?proto.speech.tts.NavigationFeature|undefined} value
 * @return {!proto.speech.tts.Question} returns this
*/
proto.speech.tts.Question.prototype.setNf = function(value) {
  return jspb.Message.setWrapperField(this, 1, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Question} returns this
 */
proto.speech.tts.Question.prototype.clearNf = function() {
  return this.setNf(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Question.prototype.hasNf = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional NavigationFeature rhs = 9;
 * @return {?proto.speech.tts.NavigationFeature}
 */
proto.speech.tts.Question.prototype.getRhs = function() {
  return /** @type{?proto.speech.tts.NavigationFeature} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.NavigationFeature, 9));
};


/**
 * @param {?proto.speech.tts.NavigationFeature|undefined} value
 * @return {!proto.speech.tts.Question} returns this
*/
proto.speech.tts.Question.prototype.setRhs = function(value) {
  return jspb.Message.setWrapperField(this, 9, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Question} returns this
 */
proto.speech.tts.Question.prototype.clearRhs = function() {
  return this.setRhs(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Question.prototype.hasRhs = function() {
  return jspb.Message.getField(this, 9) != null;
};


/**
 * optional bool bval = 2;
 * @return {?boolean}
 */
proto.speech.tts.Question.prototype.getBval = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 2));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.Question} returns this
 */
proto.speech.tts.Question.prototype.setBval = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Question} returns this
 */
proto.speech.tts.Question.prototype.clearBval = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Question.prototype.hasBval = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional int32 ival = 3;
 * @return {?number}
 */
proto.speech.tts.Question.prototype.getIval = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Question} returns this
 */
proto.speech.tts.Question.prototype.setIval = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Question} returns this
 */
proto.speech.tts.Question.prototype.clearIval = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Question.prototype.hasIval = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional string sval = 4;
 * @return {?string}
 */
proto.speech.tts.Question.prototype.getSval = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 4));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Question} returns this
 */
proto.speech.tts.Question.prototype.setSval = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Question} returns this
 */
proto.speech.tts.Question.prototype.clearSval = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Question.prototype.hasSval = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional float fval = 8;
 * @return {?number}
 */
proto.speech.tts.Question.prototype.getFval = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 8));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Question} returns this
 */
proto.speech.tts.Question.prototype.setFval = function(value) {
  return jspb.Message.setField(this, 8, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Question} returns this
 */
proto.speech.tts.Question.prototype.clearFval = function() {
  return jspb.Message.setField(this, 8, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Question.prototype.hasFval = function() {
  return jspb.Message.getField(this, 8) != null;
};


/**
 * optional Comparison comp = 5;
 * @return {!proto.speech.tts.Question.Comparison}
 */
proto.speech.tts.Question.prototype.getComp = function() {
  return /** @type {!proto.speech.tts.Question.Comparison} */ (jspb.Message.getField(this, 5));
};


/**
 * @param {?proto.speech.tts.Question.Comparison|undefined} value
 * @return {!proto.speech.tts.Question} returns this
 */
proto.speech.tts.Question.prototype.setComp = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Question} returns this
 */
proto.speech.tts.Question.prototype.clearComp = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Question.prototype.hasComp = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * repeated Question q = 6;
 * @return {!Array<!proto.speech.tts.Question>}
 */
proto.speech.tts.Question.prototype.getQList = function() {
  return /** @type{!Array<!proto.speech.tts.Question>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.Question, 6));
};


/**
 * @param {Array<!proto.speech.tts.Question>|undefined} value
 * @return {!proto.speech.tts.Question} returns this
*/
proto.speech.tts.Question.prototype.setQList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 6, value);
};


/**
 * @param {!proto.speech.tts.Question=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.Question}
 */
proto.speech.tts.Question.prototype.addQ = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 6, opt_value, proto.speech.tts.Question, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.Question} returns this
 */
proto.speech.tts.Question.prototype.clearQList = function() {
  return this.setQList([]);
};


/**
 * optional Conjunction conj = 7;
 * @return {!proto.speech.tts.Question.Conjunction}
 */
proto.speech.tts.Question.prototype.getConj = function() {
  return /** @type {!proto.speech.tts.Question.Conjunction} */ (jspb.Message.getField(this, 7));
};


/**
 * @param {?proto.speech.tts.Question.Conjunction|undefined} value
 * @return {!proto.speech.tts.Question} returns this
 */
proto.speech.tts.Question.prototype.setConj = function(value) {
  return jspb.Message.setField(this, 7, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Question} returns this
 */
proto.speech.tts.Question.prototype.clearConj = function() {
  return jspb.Message.setField(this, 7, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Question.prototype.hasConj = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * optional int32 num_values = 10;
 * @return {?number}
 */
proto.speech.tts.Question.prototype.getNumValues = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 10));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Question} returns this
 */
proto.speech.tts.Question.prototype.setNumValues = function(value) {
  return jspb.Message.setField(this, 10, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Question} returns this
 */
proto.speech.tts.Question.prototype.clearNumValues = function() {
  return jspb.Message.setField(this, 10, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Question.prototype.hasNumValues = function() {
  return jspb.Message.getField(this, 10) != null;
};


/**
 * optional uint64 values_hash = 11;
 * @return {?number}
 */
proto.speech.tts.Question.prototype.getValuesHash = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 11));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Question} returns this
 */
proto.speech.tts.Question.prototype.setValuesHash = function(value) {
  return jspb.Message.setField(this, 11, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Question} returns this
 */
proto.speech.tts.Question.prototype.clearValuesHash = function() {
  return jspb.Message.setField(this, 11, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Question.prototype.hasValuesHash = function() {
  return jspb.Message.getField(this, 11) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.Question}
 */
proto.speech.tts.Question.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.Question, data);
};



//speech/tts/engine/utterance/features_proto_jspb/featureset.jspb.js
// source: speech/tts/engine/utterance/features.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.FeatureSet');
goog.provide('proto.speech.tts.FeatureSet.UnitType');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.AcousticFeature');
goog.require('proto.speech.tts.CustomFeature');
goog.require('proto.speech.tts.NavigationFeature');
goog.require('proto.speech.tts.Question');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.FeatureSet = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.FeatureSet.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.FeatureSet, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.FeatureSet.displayName = 'proto.speech.tts.FeatureSet';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.FeatureSet.repeatedFields_ = [1,2,6,4];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.FeatureSet.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.FeatureSet.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.FeatureSet} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.FeatureSet.toObject = function(includeInstance, msg) {
  var f, obj = {
    nfList: jspb.Message.toObjectList(msg.getNfList(),
    proto.speech.tts.NavigationFeature.toObject, includeInstance),
    questionList: jspb.Message.toObjectList(msg.getQuestionList(),
    proto.speech.tts.Question.toObject, includeInstance),
    customList: jspb.Message.toObjectList(msg.getCustomList(),
    proto.speech.tts.CustomFeature.toObject, includeInstance),
    acousticList: jspb.Message.toObjectList(msg.getAcousticList(),
    proto.speech.tts.AcousticFeature.toObject, includeInstance),
    unitType: jspb.Message.getFieldWithDefault(msg, 5, 1)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of FeatureSet as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.FeatureSet.ObjectFormat = function() {
  /** @type {?Array<!proto.speech.tts.NavigationFeature.ObjectFormat>|undefined} */
  this.nfList;

  /** @type {?Array<!proto.speech.tts.Question.ObjectFormat>|undefined} */
  this.questionList;

  /** @type {?Array<!proto.speech.tts.CustomFeature.ObjectFormat>|undefined} */
  this.customList;

  /** @type {?Array<!proto.speech.tts.AcousticFeature.ObjectFormat>|undefined} */
  this.acousticList;

  /** @type {?number|undefined} */
  this.unitType;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.FeatureSet.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.FeatureSet}
 */
proto.speech.tts.FeatureSet.fromObject = function(obj) {
  var msg = new proto.speech.tts.FeatureSet();
  obj.nfList && jspb.Message.setRepeatedWrapperField(
      msg, 1, obj.nfList.map(
          proto.speech.tts.NavigationFeature.fromObject));
  obj.questionList && jspb.Message.setRepeatedWrapperField(
      msg, 2, obj.questionList.map(
          proto.speech.tts.Question.fromObject));
  obj.customList && jspb.Message.setRepeatedWrapperField(
      msg, 6, obj.customList.map(
          proto.speech.tts.CustomFeature.fromObject));
  obj.acousticList && jspb.Message.setRepeatedWrapperField(
      msg, 4, obj.acousticList.map(
          proto.speech.tts.AcousticFeature.fromObject));
  obj.unitType != null && jspb.Message.setField(msg, 5, obj.unitType);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.FeatureSet}
 */
proto.speech.tts.FeatureSet.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.FeatureSet;
  return proto.speech.tts.FeatureSet.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.FeatureSet} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.FeatureSet}
 */
proto.speech.tts.FeatureSet.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.speech.tts.NavigationFeature;
      reader.readMessage(value,proto.speech.tts.NavigationFeature.deserializeBinaryFromReader);
      msg.addNf(value);
      break;
    case 2:
      var value = new proto.speech.tts.Question;
      reader.readMessage(value,proto.speech.tts.Question.deserializeBinaryFromReader);
      msg.addQuestion(value);
      break;
    case 6:
      var value = new proto.speech.tts.CustomFeature;
      reader.readMessage(value,proto.speech.tts.CustomFeature.deserializeBinaryFromReader);
      msg.addCustom(value);
      break;
    case 4:
      var value = new proto.speech.tts.AcousticFeature;
      reader.readMessage(value,proto.speech.tts.AcousticFeature.deserializeBinaryFromReader);
      msg.addAcoustic(value);
      break;
    case 5:
      var value = /** @type {!proto.speech.tts.FeatureSet.UnitType} */ (reader.readEnum());
      msg.setUnitType(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.FeatureSet.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.FeatureSet.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.FeatureSet} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.FeatureSet.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getNfList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      1,
      f,
      proto.speech.tts.NavigationFeature.serializeBinaryToWriter
    );
  }
  f = message.getQuestionList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      2,
      f,
      proto.speech.tts.Question.serializeBinaryToWriter
    );
  }
  f = message.getCustomList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      6,
      f,
      proto.speech.tts.CustomFeature.serializeBinaryToWriter
    );
  }
  f = message.getAcousticList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      4,
      f,
      proto.speech.tts.AcousticFeature.serializeBinaryToWriter
    );
  }
  f = /** @type {!proto.speech.tts.FeatureSet.UnitType} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeEnum(
      5,
      f
    );
  }
};


/**
 * @enum {number}
 */
proto.speech.tts.FeatureSet.UnitType = {
  PHONE: 1,
  DIPHONE: 2
};

/**
 * repeated NavigationFeature nf = 1;
 * @return {!Array<!proto.speech.tts.NavigationFeature>}
 */
proto.speech.tts.FeatureSet.prototype.getNfList = function() {
  return /** @type{!Array<!proto.speech.tts.NavigationFeature>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.NavigationFeature, 1));
};


/**
 * @param {Array<!proto.speech.tts.NavigationFeature>|undefined} value
 * @return {!proto.speech.tts.FeatureSet} returns this
*/
proto.speech.tts.FeatureSet.prototype.setNfList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 1, value);
};


/**
 * @param {!proto.speech.tts.NavigationFeature=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.NavigationFeature}
 */
proto.speech.tts.FeatureSet.prototype.addNf = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.speech.tts.NavigationFeature, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.FeatureSet} returns this
 */
proto.speech.tts.FeatureSet.prototype.clearNfList = function() {
  return this.setNfList([]);
};


/**
 * repeated Question question = 2;
 * @return {!Array<!proto.speech.tts.Question>}
 */
proto.speech.tts.FeatureSet.prototype.getQuestionList = function() {
  return /** @type{!Array<!proto.speech.tts.Question>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.Question, 2));
};


/**
 * @param {Array<!proto.speech.tts.Question>|undefined} value
 * @return {!proto.speech.tts.FeatureSet} returns this
*/
proto.speech.tts.FeatureSet.prototype.setQuestionList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 2, value);
};


/**
 * @param {!proto.speech.tts.Question=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.Question}
 */
proto.speech.tts.FeatureSet.prototype.addQuestion = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 2, opt_value, proto.speech.tts.Question, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.FeatureSet} returns this
 */
proto.speech.tts.FeatureSet.prototype.clearQuestionList = function() {
  return this.setQuestionList([]);
};


/**
 * repeated CustomFeature custom = 6;
 * @return {!Array<!proto.speech.tts.CustomFeature>}
 */
proto.speech.tts.FeatureSet.prototype.getCustomList = function() {
  return /** @type{!Array<!proto.speech.tts.CustomFeature>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.CustomFeature, 6));
};


/**
 * @param {Array<!proto.speech.tts.CustomFeature>|undefined} value
 * @return {!proto.speech.tts.FeatureSet} returns this
*/
proto.speech.tts.FeatureSet.prototype.setCustomList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 6, value);
};


/**
 * @param {!proto.speech.tts.CustomFeature=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.CustomFeature}
 */
proto.speech.tts.FeatureSet.prototype.addCustom = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 6, opt_value, proto.speech.tts.CustomFeature, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.FeatureSet} returns this
 */
proto.speech.tts.FeatureSet.prototype.clearCustomList = function() {
  return this.setCustomList([]);
};


/**
 * repeated AcousticFeature acoustic = 4;
 * @return {!Array<!proto.speech.tts.AcousticFeature>}
 */
proto.speech.tts.FeatureSet.prototype.getAcousticList = function() {
  return /** @type{!Array<!proto.speech.tts.AcousticFeature>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.AcousticFeature, 4));
};


/**
 * @param {Array<!proto.speech.tts.AcousticFeature>|undefined} value
 * @return {!proto.speech.tts.FeatureSet} returns this
*/
proto.speech.tts.FeatureSet.prototype.setAcousticList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 4, value);
};


/**
 * @param {!proto.speech.tts.AcousticFeature=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.AcousticFeature}
 */
proto.speech.tts.FeatureSet.prototype.addAcoustic = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 4, opt_value, proto.speech.tts.AcousticFeature, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.FeatureSet} returns this
 */
proto.speech.tts.FeatureSet.prototype.clearAcousticList = function() {
  return this.setAcousticList([]);
};


/**
 * optional UnitType unit_type = 5;
 * @return {!proto.speech.tts.FeatureSet.UnitType}
 */
proto.speech.tts.FeatureSet.prototype.getUnitType = function() {
  return /** @type {!proto.speech.tts.FeatureSet.UnitType} */ (jspb.Message.getFieldWithDefault(this, 5, 1));
};


/**
 * @param {?proto.speech.tts.FeatureSet.UnitType|undefined} value
 * @return {!proto.speech.tts.FeatureSet} returns this
 */
proto.speech.tts.FeatureSet.prototype.setUnitType = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.FeatureSet} returns this
 */
proto.speech.tts.FeatureSet.prototype.clearUnitType = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.FeatureSet.prototype.hasUnitType = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.FeatureSet}
 */
proto.speech.tts.FeatureSet.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.FeatureSet, data);
};



//speech/tts/engine/unit_selection/unit_selection_diagnostics_proto_jspb/unitselectiondiagnostics.jspb.js
// source: speech/tts/engine/unit_selection/unit_selection_diagnostics.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.UnitSelectionDiagnostics');
goog.provide('proto.speech.tts.UnitSelectionDiagnostics.TargetData');
goog.provide('proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel');
goog.provide('proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.PreselectionMethod');
goog.provide('proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.FeatureSet');
goog.require('proto.speech.tts.LatticeProto');

goog.forwardDeclare('proto.speech.tts.UnitSelectionConfig.Preselection.Barracuda.StreamWeight.StreamId');
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.UnitSelectionDiagnostics = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.UnitSelectionDiagnostics.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.UnitSelectionDiagnostics, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.UnitSelectionDiagnostics.displayName = 'proto.speech.tts.UnitSelectionDiagnostics';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, proto.speech.tts.UnitSelectionDiagnostics.TargetData.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.UnitSelectionDiagnostics.TargetData, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.UnitSelectionDiagnostics.TargetData.displayName = 'proto.speech.tts.UnitSelectionDiagnostics.TargetData';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.displayName = 'proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents.displayName = 'proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.UnitSelectionDiagnostics.repeatedFields_ = [2];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.UnitSelectionDiagnostics.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.UnitSelectionDiagnostics.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.UnitSelectionDiagnostics} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.UnitSelectionDiagnostics.toObject = function(includeInstance, msg) {
  var f, obj = {
    lattice: (f = msg.getLattice()) && proto.speech.tts.LatticeProto.toObject(includeInstance, f),
    targetList: jspb.Message.toObjectList(msg.getTargetList(),
    proto.speech.tts.UnitSelectionDiagnostics.TargetData.toObject, includeInstance),
    featureSet: (f = msg.getFeatureSet()) && proto.speech.tts.FeatureSet.toObject(includeInstance, f),
    predictedMos: (f = jspb.Message.getOptionalFloatingPointField(msg, 4)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of UnitSelectionDiagnostics as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.UnitSelectionDiagnostics.ObjectFormat = function() {
  /** @type {?proto.speech.tts.LatticeProto.ObjectFormat|undefined} */
  this.lattice;

  /** @type {?Array<!proto.speech.tts.UnitSelectionDiagnostics.TargetData.ObjectFormat>|undefined} */
  this.targetList;

  /** @type {?proto.speech.tts.FeatureSet.ObjectFormat|undefined} */
  this.featureSet;

  /** @type {?number|undefined} */
  this.predictedMos;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.UnitSelectionDiagnostics.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.UnitSelectionDiagnostics}
 */
proto.speech.tts.UnitSelectionDiagnostics.fromObject = function(obj) {
  var msg = new proto.speech.tts.UnitSelectionDiagnostics();
  obj.lattice && jspb.Message.setWrapperField(
      msg, 1, proto.speech.tts.LatticeProto.fromObject(obj.lattice));
  obj.targetList && jspb.Message.setRepeatedWrapperField(
      msg, 2, obj.targetList.map(
          proto.speech.tts.UnitSelectionDiagnostics.TargetData.fromObject));
  obj.featureSet && jspb.Message.setWrapperField(
      msg, 3, proto.speech.tts.FeatureSet.fromObject(obj.featureSet));
  obj.predictedMos != null && jspb.Message.setField(msg, 4, obj.predictedMos);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.UnitSelectionDiagnostics}
 */
proto.speech.tts.UnitSelectionDiagnostics.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.UnitSelectionDiagnostics;
  return proto.speech.tts.UnitSelectionDiagnostics.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.UnitSelectionDiagnostics} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.UnitSelectionDiagnostics}
 */
proto.speech.tts.UnitSelectionDiagnostics.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.speech.tts.LatticeProto;
      reader.readMessage(value,proto.speech.tts.LatticeProto.deserializeBinaryFromReader);
      msg.setLattice(value);
      break;
    case 2:
      var value = new proto.speech.tts.UnitSelectionDiagnostics.TargetData;
      reader.readMessage(value,proto.speech.tts.UnitSelectionDiagnostics.TargetData.deserializeBinaryFromReader);
      msg.addTarget(value);
      break;
    case 3:
      var value = new proto.speech.tts.FeatureSet;
      reader.readMessage(value,proto.speech.tts.FeatureSet.deserializeBinaryFromReader);
      msg.setFeatureSet(value);
      break;
    case 4:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setPredictedMos(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.UnitSelectionDiagnostics.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.UnitSelectionDiagnostics.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.UnitSelectionDiagnostics} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.UnitSelectionDiagnostics.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getLattice();
  if (f != null) {
    writer.writeMessage(
      1,
      f,
      proto.speech.tts.LatticeProto.serializeBinaryToWriter
    );
  }
  f = message.getTargetList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      2,
      f,
      proto.speech.tts.UnitSelectionDiagnostics.TargetData.serializeBinaryToWriter
    );
  }
  f = message.getFeatureSet();
  if (f != null) {
    writer.writeMessage(
      3,
      f,
      proto.speech.tts.FeatureSet.serializeBinaryToWriter
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeFloat(
      4,
      f
    );
  }
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.repeatedFields_ = [100];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.UnitSelectionDiagnostics.TargetData.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.UnitSelectionDiagnostics.TargetData} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.toObject = function(includeInstance, msg) {
  var f, obj = {
    ownIndex: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    basetypeName: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    selectedUnitId: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,
    filename: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f,
    modelList: jspb.Message.toObjectList(msg.getModelList(),
    proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.toObject, includeInstance),
    patts1Diagnostics: (f = jspb.Message.getField(msg, 900)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of TargetData as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.ownIndex;

  /** @type {?string|undefined} */
  this.basetypeName;

  /** @type {?number|undefined} */
  this.selectedUnitId;

  /** @type {?string|undefined} */
  this.filename;

  /** @type {?Array<!proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.ObjectFormat>|undefined} */
  this.modelList;

  /** @type {?string|undefined} */
  this.patts1Diagnostics;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.UnitSelectionDiagnostics.TargetData.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.UnitSelectionDiagnostics.TargetData}
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.fromObject = function(obj) {
  var msg = new proto.speech.tts.UnitSelectionDiagnostics.TargetData();
  obj.ownIndex != null && jspb.Message.setField(msg, 1, obj.ownIndex);
  obj.basetypeName != null && jspb.Message.setField(msg, 2, obj.basetypeName);
  obj.selectedUnitId != null && jspb.Message.setField(msg, 3, obj.selectedUnitId);
  obj.filename != null && jspb.Message.setField(msg, 4, obj.filename);
  obj.modelList && jspb.Message.setRepeatedWrapperField(
      msg, 100, obj.modelList.map(
          proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.fromObject));
  obj.patts1Diagnostics != null && jspb.Message.setField(msg, 900, obj.patts1Diagnostics);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.UnitSelectionDiagnostics.TargetData}
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.UnitSelectionDiagnostics.TargetData;
  return proto.speech.tts.UnitSelectionDiagnostics.TargetData.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.UnitSelectionDiagnostics.TargetData} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.UnitSelectionDiagnostics.TargetData}
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setOwnIndex(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setBasetypeName(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setSelectedUnitId(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setFilename(value);
      break;
    case 100:
      var value = new proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel;
      reader.readMessage(value,proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.deserializeBinaryFromReader);
      msg.addModel(value);
      break;
    case 900:
      var value = /** @type {string} */ (reader.readString());
      msg.setPatts1Diagnostics(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.UnitSelectionDiagnostics.TargetData.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.UnitSelectionDiagnostics.TargetData} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?number} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeInt32(
      1,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeInt32(
      3,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeString(
      4,
      f
    );
  }
  f = message.getModelList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      100,
      f,
      proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.serializeBinaryToWriter
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 900));
  if (f != null) {
    writer.writeString(
      900,
      f
    );
  }
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.repeatedFields_ = [3,7];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.toObject = function(includeInstance, msg) {
  var f, obj = {
    id: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    score: (f = jspb.Message.getOptionalFloatingPointField(msg, 2)) == null ? undefined : f,
    preselectedByList: (f = jspb.Message.getRepeatedField(msg, 3)) == null ? undefined : f,
    firstUnitId: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f,
    numAllUnits: (f = jspb.Message.getField(msg, 5)) == null ? undefined : f,
    numPreselectedUnits: (f = jspb.Message.getField(msg, 6)) == null ? undefined : f,
    componentList: jspb.Message.toObjectList(msg.getComponentList(),
    proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents.toObject, includeInstance),
    triphoneMismatchCost: (f = jspb.Message.getOptionalFloatingPointField(msg, 8)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of CandidateModel as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.id;

  /** @type {?number|undefined} */
  this.score;

  /** @type {?Array<number>|undefined} */
  this.preselectedByList;

  /** @type {?number|undefined} */
  this.firstUnitId;

  /** @type {?number|undefined} */
  this.numAllUnits;

  /** @type {?number|undefined} */
  this.numPreselectedUnits;

  /** @type {?Array<!proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents.ObjectFormat>|undefined} */
  this.componentList;

  /** @type {?number|undefined} */
  this.triphoneMismatchCost;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel}
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.fromObject = function(obj) {
  var msg = new proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel();
  obj.id != null && jspb.Message.setField(msg, 1, obj.id);
  obj.score != null && jspb.Message.setField(msg, 2, obj.score);
  obj.preselectedByList != null && jspb.Message.setField(msg, 3, obj.preselectedByList);
  obj.firstUnitId != null && jspb.Message.setField(msg, 4, obj.firstUnitId);
  obj.numAllUnits != null && jspb.Message.setField(msg, 5, obj.numAllUnits);
  obj.numPreselectedUnits != null && jspb.Message.setField(msg, 6, obj.numPreselectedUnits);
  obj.componentList && jspb.Message.setRepeatedWrapperField(
      msg, 7, obj.componentList.map(
          proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents.fromObject));
  obj.triphoneMismatchCost != null && jspb.Message.setField(msg, 8, obj.triphoneMismatchCost);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel}
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel;
  return proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel}
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setId(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setScore(value);
      break;
    case 3:
      var value = /** @type {!proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.PreselectionMethod} */ (reader.readEnum());
      msg.addPreselectedBy(value);
      break;
    case 4:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setFirstUnitId(value);
      break;
    case 5:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setNumAllUnits(value);
      break;
    case 6:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setNumPreselectedUnits(value);
      break;
    case 7:
      var value = new proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents;
      reader.readMessage(value,proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents.deserializeBinaryFromReader);
      msg.addComponent(value);
      break;
    case 8:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setTriphoneMismatchCost(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?number} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeInt32(
      1,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeFloat(
      2,
      f
    );
  }
  f = message.getPreselectedByList();
  if (f.length > 0) {
    writer.writeRepeatedEnum(
      3,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeInt32(
      4,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeInt32(
      5,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeInt32(
      6,
      f
    );
  }
  f = message.getComponentList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      7,
      f,
      proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents.serializeBinaryToWriter
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 8));
  if (f != null) {
    writer.writeFloat(
      8,
      f
    );
  }
};


/**
 * @enum {number}
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.PreselectionMethod = {
  UNKNOWN_PRESELECTION: 0,
  BARRACUDA: 1,
  SCRIPT_MATCHING: 2
};


/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents.repeatedFields_ = [2];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents.toObject = function(includeInstance, msg) {
  var f, obj = {
    streamId: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    scoreList: (f = jspb.Message.getRepeatedFloatingPointField(msg, 2)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of TargetCostComponents as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.streamId;

  /** @type {?Array<number>|undefined} */
  this.scoreList;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents}
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents.fromObject = function(obj) {
  var msg = new proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents();
  obj.streamId != null && jspb.Message.setField(msg, 1, obj.streamId);
  obj.scoreList != null && jspb.Message.setField(msg, 2, obj.scoreList);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents}
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents;
  return proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents}
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {!proto.speech.tts.UnitSelectionConfig.Preselection.Barracuda.StreamWeight.StreamId} */ (reader.readEnum());
      msg.setStreamId(value);
      break;
    case 2:
      var value = /** @type {!Array<number>} */ (reader.readPackedFloat());
      msg.setScoreList(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {!proto.speech.tts.UnitSelectionConfig.Preselection.Barracuda.StreamWeight.StreamId} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeEnum(
      1,
      f
    );
  }
  f = message.getScoreList();
  if (f.length > 0) {
    writer.writePackedFloat(
      2,
      f
    );
  }
};


/**
 * optional UnitSelectionConfig.Preselection.Barracuda.StreamWeight.StreamId stream_id = 1;
 * @return {!proto.speech.tts.UnitSelectionConfig.Preselection.Barracuda.StreamWeight.StreamId}
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents.prototype.getStreamId = function() {
  return /** @type {!proto.speech.tts.UnitSelectionConfig.Preselection.Barracuda.StreamWeight.StreamId} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?proto.speech.tts.UnitSelectionConfig.Preselection.Barracuda.StreamWeight.StreamId|undefined} value
 * @return {!proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents} returns this
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents.prototype.setStreamId = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents} returns this
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents.prototype.clearStreamId = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents.prototype.hasStreamId = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * repeated float score = 2;
 * @return {!Array<number>}
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents.prototype.getScoreList = function() {
  return /** @type {!Array<number>} */ (jspb.Message.getRepeatedFloatingPointField(this, 2));
};


/**
 * @param {Array<number>|undefined} value
 * @return {!proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents} returns this
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents.prototype.setScoreList = function(value) {
  return jspb.Message.setField(this, 2, value || []);
};


/**
 * @param {number} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents} returns this
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents.prototype.addScore = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 2, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents} returns this
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents.prototype.clearScoreList = function() {
  return this.setScoreList([]);
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents}
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents, data);
};


/**
 * optional int32 id = 1;
 * @return {?number}
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.prototype.getId = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel} returns this
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.prototype.setId = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel} returns this
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.prototype.clearId = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.prototype.hasId = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional float score = 2;
 * @return {?number}
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.prototype.getScore = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 2));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel} returns this
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.prototype.setScore = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel} returns this
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.prototype.clearScore = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.prototype.hasScore = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * repeated PreselectionMethod preselected_by = 3;
 * @return {!Array<!proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.PreselectionMethod>}
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.prototype.getPreselectedByList = function() {
  return /** @type {!Array<!proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.PreselectionMethod>} */ (jspb.Message.getRepeatedField(this, 3));
};


/**
 * @param {Array<!proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.PreselectionMethod>|undefined} value
 * @return {!proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel} returns this
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.prototype.setPreselectedByList = function(value) {
  return jspb.Message.setField(this, 3, value || []);
};


/**
 * @param {!proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.PreselectionMethod} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel} returns this
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.prototype.addPreselectedBy = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 3, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel} returns this
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.prototype.clearPreselectedByList = function() {
  return this.setPreselectedByList([]);
};


/**
 * optional int32 first_unit_id = 4;
 * @return {?number}
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.prototype.getFirstUnitId = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 4));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel} returns this
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.prototype.setFirstUnitId = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel} returns this
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.prototype.clearFirstUnitId = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.prototype.hasFirstUnitId = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional int32 num_all_units = 5;
 * @return {?number}
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.prototype.getNumAllUnits = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 5));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel} returns this
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.prototype.setNumAllUnits = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel} returns this
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.prototype.clearNumAllUnits = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.prototype.hasNumAllUnits = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional int32 num_preselected_units = 6;
 * @return {?number}
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.prototype.getNumPreselectedUnits = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 6));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel} returns this
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.prototype.setNumPreselectedUnits = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel} returns this
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.prototype.clearNumPreselectedUnits = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.prototype.hasNumPreselectedUnits = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * repeated TargetCostComponents component = 7;
 * @return {!Array<!proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents>}
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.prototype.getComponentList = function() {
  return /** @type{!Array<!proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents, 7));
};


/**
 * @param {Array<!proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents>|undefined} value
 * @return {!proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel} returns this
*/
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.prototype.setComponentList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 7, value);
};


/**
 * @param {!proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents}
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.prototype.addComponent = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 7, opt_value, proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.TargetCostComponents, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel} returns this
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.prototype.clearComponentList = function() {
  return this.setComponentList([]);
};


/**
 * optional float triphone_mismatch_cost = 8;
 * @return {?number}
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.prototype.getTriphoneMismatchCost = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 8));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel} returns this
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.prototype.setTriphoneMismatchCost = function(value) {
  return jspb.Message.setField(this, 8, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel} returns this
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.prototype.clearTriphoneMismatchCost = function() {
  return jspb.Message.setField(this, 8, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.prototype.hasTriphoneMismatchCost = function() {
  return jspb.Message.getField(this, 8) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel}
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel, data);
};


/**
 * optional int32 own_index = 1;
 * @return {?number}
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.prototype.getOwnIndex = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.UnitSelectionDiagnostics.TargetData} returns this
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.prototype.setOwnIndex = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.UnitSelectionDiagnostics.TargetData} returns this
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.prototype.clearOwnIndex = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.prototype.hasOwnIndex = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional string basetype_name = 2;
 * @return {?string}
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.prototype.getBasetypeName = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.UnitSelectionDiagnostics.TargetData} returns this
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.prototype.setBasetypeName = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.UnitSelectionDiagnostics.TargetData} returns this
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.prototype.clearBasetypeName = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.prototype.hasBasetypeName = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional int32 selected_unit_id = 3;
 * @return {?number}
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.prototype.getSelectedUnitId = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.UnitSelectionDiagnostics.TargetData} returns this
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.prototype.setSelectedUnitId = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.UnitSelectionDiagnostics.TargetData} returns this
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.prototype.clearSelectedUnitId = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.prototype.hasSelectedUnitId = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional string filename = 4;
 * @return {?string}
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.prototype.getFilename = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 4));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.UnitSelectionDiagnostics.TargetData} returns this
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.prototype.setFilename = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.UnitSelectionDiagnostics.TargetData} returns this
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.prototype.clearFilename = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.prototype.hasFilename = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * repeated CandidateModel model = 100;
 * @return {!Array<!proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel>}
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.prototype.getModelList = function() {
  return /** @type{!Array<!proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel, 100));
};


/**
 * @param {Array<!proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel>|undefined} value
 * @return {!proto.speech.tts.UnitSelectionDiagnostics.TargetData} returns this
*/
proto.speech.tts.UnitSelectionDiagnostics.TargetData.prototype.setModelList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 100, value);
};


/**
 * @param {!proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel}
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.prototype.addModel = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 100, opt_value, proto.speech.tts.UnitSelectionDiagnostics.TargetData.CandidateModel, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.UnitSelectionDiagnostics.TargetData} returns this
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.prototype.clearModelList = function() {
  return this.setModelList([]);
};


/**
 * optional string patts1_diagnostics = 900;
 * @return {?string}
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.prototype.getPatts1Diagnostics = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 900));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.UnitSelectionDiagnostics.TargetData} returns this
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.prototype.setPatts1Diagnostics = function(value) {
  return jspb.Message.setField(this, 900, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.UnitSelectionDiagnostics.TargetData} returns this
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.prototype.clearPatts1Diagnostics = function() {
  return jspb.Message.setField(this, 900, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.prototype.hasPatts1Diagnostics = function() {
  return jspb.Message.getField(this, 900) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.UnitSelectionDiagnostics.TargetData}
 */
proto.speech.tts.UnitSelectionDiagnostics.TargetData.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.UnitSelectionDiagnostics.TargetData, data);
};


/**
 * optional LatticeProto lattice = 1;
 * @return {?proto.speech.tts.LatticeProto}
 */
proto.speech.tts.UnitSelectionDiagnostics.prototype.getLattice = function() {
  return /** @type{?proto.speech.tts.LatticeProto} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.LatticeProto, 1));
};


/**
 * @param {?proto.speech.tts.LatticeProto|undefined} value
 * @return {!proto.speech.tts.UnitSelectionDiagnostics} returns this
*/
proto.speech.tts.UnitSelectionDiagnostics.prototype.setLattice = function(value) {
  return jspb.Message.setWrapperField(this, 1, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.UnitSelectionDiagnostics} returns this
 */
proto.speech.tts.UnitSelectionDiagnostics.prototype.clearLattice = function() {
  return this.setLattice(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.UnitSelectionDiagnostics.prototype.hasLattice = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * repeated TargetData target = 2;
 * @return {!Array<!proto.speech.tts.UnitSelectionDiagnostics.TargetData>}
 */
proto.speech.tts.UnitSelectionDiagnostics.prototype.getTargetList = function() {
  return /** @type{!Array<!proto.speech.tts.UnitSelectionDiagnostics.TargetData>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.UnitSelectionDiagnostics.TargetData, 2));
};


/**
 * @param {Array<!proto.speech.tts.UnitSelectionDiagnostics.TargetData>|undefined} value
 * @return {!proto.speech.tts.UnitSelectionDiagnostics} returns this
*/
proto.speech.tts.UnitSelectionDiagnostics.prototype.setTargetList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 2, value);
};


/**
 * @param {!proto.speech.tts.UnitSelectionDiagnostics.TargetData=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.UnitSelectionDiagnostics.TargetData}
 */
proto.speech.tts.UnitSelectionDiagnostics.prototype.addTarget = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 2, opt_value, proto.speech.tts.UnitSelectionDiagnostics.TargetData, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.UnitSelectionDiagnostics} returns this
 */
proto.speech.tts.UnitSelectionDiagnostics.prototype.clearTargetList = function() {
  return this.setTargetList([]);
};


/**
 * optional FeatureSet feature_set = 3;
 * @return {?proto.speech.tts.FeatureSet}
 */
proto.speech.tts.UnitSelectionDiagnostics.prototype.getFeatureSet = function() {
  return /** @type{?proto.speech.tts.FeatureSet} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.FeatureSet, 3));
};


/**
 * @param {?proto.speech.tts.FeatureSet|undefined} value
 * @return {!proto.speech.tts.UnitSelectionDiagnostics} returns this
*/
proto.speech.tts.UnitSelectionDiagnostics.prototype.setFeatureSet = function(value) {
  return jspb.Message.setWrapperField(this, 3, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.UnitSelectionDiagnostics} returns this
 */
proto.speech.tts.UnitSelectionDiagnostics.prototype.clearFeatureSet = function() {
  return this.setFeatureSet(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.UnitSelectionDiagnostics.prototype.hasFeatureSet = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional float predicted_mos = 4;
 * @return {?number}
 */
proto.speech.tts.UnitSelectionDiagnostics.prototype.getPredictedMos = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 4));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.UnitSelectionDiagnostics} returns this
 */
proto.speech.tts.UnitSelectionDiagnostics.prototype.setPredictedMos = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.UnitSelectionDiagnostics} returns this
 */
proto.speech.tts.UnitSelectionDiagnostics.prototype.clearPredictedMos = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.UnitSelectionDiagnostics.prototype.hasPredictedMos = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.UnitSelectionDiagnostics}
 */
proto.speech.tts.UnitSelectionDiagnostics.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.UnitSelectionDiagnostics, data);
};



//speech/tts/engine/wavenet/proto/stitching_options_proto_jspb/stitchingoptions.jspb.js
// source: speech/tts/engine/wavenet/proto/stitching_options.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.StitchingOptions');
goog.provide('proto.speech.tts.StitchingOptions.StitchType');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.StitchingOptions = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.StitchingOptions, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.StitchingOptions.displayName = 'proto.speech.tts.StitchingOptions';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.StitchingOptions.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.StitchingOptions.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.StitchingOptions} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.StitchingOptions.toObject = function(includeInstance, msg) {
  var f, obj = {
    type: jspb.Message.getFieldWithDefault(msg, 1, 2),
    minOffsetSearch: (f = jspb.Message.getField(msg, 10)) == null ? undefined : f,
    maxOffsetSearch: (f = jspb.Message.getField(msg, 12)) == null ? undefined : f,
    desiredOffsetSearchProportion: (f = jspb.Message.getOptionalFloatingPointField(msg, 13)) == null ? undefined : f,
    onlyZeroCrossingOffsets: jspb.Message.getBooleanFieldWithDefault(msg, 14, false),
    effectiveOverlapSamples: (f = jspb.Message.getField(msg, 15)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of StitchingOptions as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.StitchingOptions.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.type;

  /** @type {?number|undefined} */
  this.minOffsetSearch;

  /** @type {?number|undefined} */
  this.maxOffsetSearch;

  /** @type {?number|undefined} */
  this.desiredOffsetSearchProportion;

  /** @type {?boolean|undefined} */
  this.onlyZeroCrossingOffsets;

  /** @type {?number|undefined} */
  this.effectiveOverlapSamples;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.StitchingOptions.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.StitchingOptions}
 */
proto.speech.tts.StitchingOptions.fromObject = function(obj) {
  var msg = new proto.speech.tts.StitchingOptions();
  obj.type != null && jspb.Message.setField(msg, 1, obj.type);
  obj.minOffsetSearch != null && jspb.Message.setField(msg, 10, obj.minOffsetSearch);
  obj.maxOffsetSearch != null && jspb.Message.setField(msg, 12, obj.maxOffsetSearch);
  obj.desiredOffsetSearchProportion != null && jspb.Message.setField(msg, 13, obj.desiredOffsetSearchProportion);
  obj.onlyZeroCrossingOffsets != null && jspb.Message.setField(msg, 14, obj.onlyZeroCrossingOffsets);
  obj.effectiveOverlapSamples != null && jspb.Message.setField(msg, 15, obj.effectiveOverlapSamples);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.StitchingOptions}
 */
proto.speech.tts.StitchingOptions.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.StitchingOptions;
  return proto.speech.tts.StitchingOptions.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.StitchingOptions} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.StitchingOptions}
 */
proto.speech.tts.StitchingOptions.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {!proto.speech.tts.StitchingOptions.StitchType} */ (reader.readEnum());
      msg.setType(value);
      break;
    case 10:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setMinOffsetSearch(value);
      break;
    case 12:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setMaxOffsetSearch(value);
      break;
    case 13:
      var value = /** @type {number} */ (reader.readDouble());
      msg.setDesiredOffsetSearchProportion(value);
      break;
    case 14:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setOnlyZeroCrossingOffsets(value);
      break;
    case 15:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setEffectiveOverlapSamples(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.StitchingOptions.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.StitchingOptions.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.StitchingOptions} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.StitchingOptions.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {!proto.speech.tts.StitchingOptions.StitchType} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeEnum(
      1,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 10));
  if (f != null) {
    writer.writeInt32(
      10,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 12));
  if (f != null) {
    writer.writeInt32(
      12,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 13));
  if (f != null) {
    writer.writeDouble(
      13,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 14));
  if (f != null) {
    writer.writeBool(
      14,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 15));
  if (f != null) {
    writer.writeInt32(
      15,
      f
    );
  }
};


/**
 * @enum {number}
 */
proto.speech.tts.StitchingOptions.StitchType = {
  CHOP_AND_GLUE: 0,
  LINEAR_CROSSFADE: 1,
  BEST_CROSS_CORRELATION_LINEAR_CROSSFADE: 2,
  BEST_CORRELATION_LINEAR_CROSSFADE: 3
};

/**
 * optional StitchType type = 1;
 * @return {!proto.speech.tts.StitchingOptions.StitchType}
 */
proto.speech.tts.StitchingOptions.prototype.getType = function() {
  return /** @type {!proto.speech.tts.StitchingOptions.StitchType} */ (jspb.Message.getFieldWithDefault(this, 1, 2));
};


/**
 * @param {?proto.speech.tts.StitchingOptions.StitchType|undefined} value
 * @return {!proto.speech.tts.StitchingOptions} returns this
 */
proto.speech.tts.StitchingOptions.prototype.setType = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.StitchingOptions} returns this
 */
proto.speech.tts.StitchingOptions.prototype.clearType = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.StitchingOptions.prototype.hasType = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional int32 min_offset_search = 10;
 * @return {?number}
 */
proto.speech.tts.StitchingOptions.prototype.getMinOffsetSearch = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 10));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.StitchingOptions} returns this
 */
proto.speech.tts.StitchingOptions.prototype.setMinOffsetSearch = function(value) {
  return jspb.Message.setField(this, 10, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.StitchingOptions} returns this
 */
proto.speech.tts.StitchingOptions.prototype.clearMinOffsetSearch = function() {
  return jspb.Message.setField(this, 10, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.StitchingOptions.prototype.hasMinOffsetSearch = function() {
  return jspb.Message.getField(this, 10) != null;
};


/**
 * optional int32 max_offset_search = 12;
 * @return {?number}
 */
proto.speech.tts.StitchingOptions.prototype.getMaxOffsetSearch = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 12));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.StitchingOptions} returns this
 */
proto.speech.tts.StitchingOptions.prototype.setMaxOffsetSearch = function(value) {
  return jspb.Message.setField(this, 12, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.StitchingOptions} returns this
 */
proto.speech.tts.StitchingOptions.prototype.clearMaxOffsetSearch = function() {
  return jspb.Message.setField(this, 12, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.StitchingOptions.prototype.hasMaxOffsetSearch = function() {
  return jspb.Message.getField(this, 12) != null;
};


/**
 * optional double desired_offset_search_proportion = 13;
 * @return {?number}
 */
proto.speech.tts.StitchingOptions.prototype.getDesiredOffsetSearchProportion = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 13));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.StitchingOptions} returns this
 */
proto.speech.tts.StitchingOptions.prototype.setDesiredOffsetSearchProportion = function(value) {
  return jspb.Message.setField(this, 13, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.StitchingOptions} returns this
 */
proto.speech.tts.StitchingOptions.prototype.clearDesiredOffsetSearchProportion = function() {
  return jspb.Message.setField(this, 13, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.StitchingOptions.prototype.hasDesiredOffsetSearchProportion = function() {
  return jspb.Message.getField(this, 13) != null;
};


/**
 * optional bool only_zero_crossing_offsets = 14;
 * @return {boolean}
 */
proto.speech.tts.StitchingOptions.prototype.getOnlyZeroCrossingOffsets = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 14, false));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.StitchingOptions} returns this
 */
proto.speech.tts.StitchingOptions.prototype.setOnlyZeroCrossingOffsets = function(value) {
  return jspb.Message.setField(this, 14, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.StitchingOptions} returns this
 */
proto.speech.tts.StitchingOptions.prototype.clearOnlyZeroCrossingOffsets = function() {
  return jspb.Message.setField(this, 14, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.StitchingOptions.prototype.hasOnlyZeroCrossingOffsets = function() {
  return jspb.Message.getField(this, 14) != null;
};


/**
 * optional int32 effective_overlap_samples = 15;
 * @return {?number}
 */
proto.speech.tts.StitchingOptions.prototype.getEffectiveOverlapSamples = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 15));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.StitchingOptions} returns this
 */
proto.speech.tts.StitchingOptions.prototype.setEffectiveOverlapSamples = function(value) {
  return jspb.Message.setField(this, 15, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.StitchingOptions} returns this
 */
proto.speech.tts.StitchingOptions.prototype.clearEffectiveOverlapSamples = function() {
  return jspb.Message.setField(this, 15, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.StitchingOptions.prototype.hasEffectiveOverlapSamples = function() {
  return jspb.Message.getField(this, 15) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.StitchingOptions}
 */
proto.speech.tts.StitchingOptions.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.StitchingOptions, data);
};



//speech/tts/engine/wavenet/proto/wavenet_diagnostics_proto_jspb/chunkinfo.jspb.js
// source: speech/tts/engine/wavenet/proto/wavenet_diagnostics.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.ChunkInfo');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.AudioBuffer');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.ChunkInfo = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.ChunkInfo, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.ChunkInfo.displayName = 'proto.speech.tts.ChunkInfo';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.ChunkInfo.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.ChunkInfo.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.ChunkInfo} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.ChunkInfo.toObject = function(includeInstance, msg) {
  var f, obj = {
    originalAudio: (f = msg.getOriginalAudio()) && proto.speech.tts.AudioBuffer.toObject(includeInstance, f),
    offsetS: (f = jspb.Message.getOptionalFloatingPointField(msg, 2)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of ChunkInfo as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.ChunkInfo.ObjectFormat = function() {
  /** @type {?proto.speech.tts.AudioBuffer.ObjectFormat|undefined} */
  this.originalAudio;

  /** @type {?number|undefined} */
  this.offsetS;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.ChunkInfo.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.ChunkInfo}
 */
proto.speech.tts.ChunkInfo.fromObject = function(obj) {
  var msg = new proto.speech.tts.ChunkInfo();
  obj.originalAudio && jspb.Message.setWrapperField(
      msg, 1, proto.speech.tts.AudioBuffer.fromObject(obj.originalAudio));
  obj.offsetS != null && jspb.Message.setField(msg, 2, obj.offsetS);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.ChunkInfo}
 */
proto.speech.tts.ChunkInfo.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.ChunkInfo;
  return proto.speech.tts.ChunkInfo.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.ChunkInfo} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.ChunkInfo}
 */
proto.speech.tts.ChunkInfo.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.speech.tts.AudioBuffer;
      reader.readMessage(value,proto.speech.tts.AudioBuffer.deserializeBinaryFromReader);
      msg.setOriginalAudio(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setOffsetS(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.ChunkInfo.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.ChunkInfo.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.ChunkInfo} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.ChunkInfo.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getOriginalAudio();
  if (f != null) {
    writer.writeMessage(
      1,
      f,
      proto.speech.tts.AudioBuffer.serializeBinaryToWriter
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeFloat(
      2,
      f
    );
  }
};


/**
 * optional AudioBuffer original_audio = 1;
 * @return {?proto.speech.tts.AudioBuffer}
 */
proto.speech.tts.ChunkInfo.prototype.getOriginalAudio = function() {
  return /** @type{?proto.speech.tts.AudioBuffer} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.AudioBuffer, 1));
};


/**
 * @param {?proto.speech.tts.AudioBuffer|undefined} value
 * @return {!proto.speech.tts.ChunkInfo} returns this
*/
proto.speech.tts.ChunkInfo.prototype.setOriginalAudio = function(value) {
  return jspb.Message.setWrapperField(this, 1, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.ChunkInfo} returns this
 */
proto.speech.tts.ChunkInfo.prototype.clearOriginalAudio = function() {
  return this.setOriginalAudio(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.ChunkInfo.prototype.hasOriginalAudio = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional float offset_s = 2;
 * @return {?number}
 */
proto.speech.tts.ChunkInfo.prototype.getOffsetS = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 2));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.ChunkInfo} returns this
 */
proto.speech.tts.ChunkInfo.prototype.setOffsetS = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.ChunkInfo} returns this
 */
proto.speech.tts.ChunkInfo.prototype.clearOffsetS = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.ChunkInfo.prototype.hasOffsetS = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.ChunkInfo}
 */
proto.speech.tts.ChunkInfo.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.ChunkInfo, data);
};



//speech/tts/engine/wavenet/proto/wavenet_synthesizer_proto_jspb/wavenetfeature.jspb.js
// source: speech/tts/engine/wavenet/proto/wavenet_synthesizer.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.WaveNetFeature');
goog.provide('proto.speech.tts.WaveNetFeature.FeatureSpeed');
goog.provide('proto.speech.tts.WaveNetFeature.GenerationType');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

goog.forwardDeclare('proto.speech.tts.WaveNetLayout');
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.WaveNetFeature = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.WaveNetFeature, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.WaveNetFeature.displayName = 'proto.speech.tts.WaveNetFeature';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.WaveNetFeature.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.WaveNetFeature.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.WaveNetFeature} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.WaveNetFeature.toObject = function(includeInstance, msg) {
  var f, obj = {
    name: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    featureSpeed: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    generationType: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,
    isInteger: (f = jspb.Message.getBooleanField(msg, 4)) == null ? undefined : f,
    depth: (f = jspb.Message.getField(msg, 5)) == null ? undefined : f,
    layout: (f = jspb.Message.getField(msg, 6)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of WaveNetFeature as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.WaveNetFeature.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.name;

  /** @type {?number|undefined} */
  this.featureSpeed;

  /** @type {?number|undefined} */
  this.generationType;

  /** @type {?boolean|undefined} */
  this.isInteger;

  /** @type {?number|undefined} */
  this.depth;

  /** @type {?number|undefined} */
  this.layout;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.WaveNetFeature.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.WaveNetFeature}
 */
proto.speech.tts.WaveNetFeature.fromObject = function(obj) {
  var msg = new proto.speech.tts.WaveNetFeature();
  obj.name != null && jspb.Message.setField(msg, 1, obj.name);
  obj.featureSpeed != null && jspb.Message.setField(msg, 2, obj.featureSpeed);
  obj.generationType != null && jspb.Message.setField(msg, 3, obj.generationType);
  obj.isInteger != null && jspb.Message.setField(msg, 4, obj.isInteger);
  obj.depth != null && jspb.Message.setField(msg, 5, obj.depth);
  obj.layout != null && jspb.Message.setField(msg, 6, obj.layout);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.WaveNetFeature}
 */
proto.speech.tts.WaveNetFeature.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.WaveNetFeature;
  return proto.speech.tts.WaveNetFeature.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.WaveNetFeature} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.WaveNetFeature}
 */
proto.speech.tts.WaveNetFeature.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setName(value);
      break;
    case 2:
      var value = /** @type {!proto.speech.tts.WaveNetFeature.FeatureSpeed} */ (reader.readEnum());
      msg.setFeatureSpeed(value);
      break;
    case 3:
      var value = /** @type {!proto.speech.tts.WaveNetFeature.GenerationType} */ (reader.readEnum());
      msg.setGenerationType(value);
      break;
    case 4:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsInteger(value);
      break;
    case 5:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setDepth(value);
      break;
    case 6:
      var value = /** @type {!proto.speech.tts.WaveNetLayout} */ (reader.readEnum());
      msg.setLayout(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.WaveNetFeature.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.WaveNetFeature.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.WaveNetFeature} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.WaveNetFeature.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = /** @type {!proto.speech.tts.WaveNetFeature.FeatureSpeed} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeEnum(
      2,
      f
    );
  }
  f = /** @type {!proto.speech.tts.WaveNetFeature.GenerationType} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeEnum(
      3,
      f
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeBool(
      4,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeInt32(
      5,
      f
    );
  }
  f = /** @type {!proto.speech.tts.WaveNetLayout} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeEnum(
      6,
      f
    );
  }
};


/**
 * @enum {number}
 */
proto.speech.tts.WaveNetFeature.FeatureSpeed = {
  ONCE: 0,
  LINGUISTIC_SPEED: 1,
  AUDIO_SPEED: 2
};

/**
 * @enum {number}
 */
proto.speech.tts.WaveNetFeature.GenerationType = {
  ZERO: 0,
  NOISE: 1,
  COPY: 2,
  COPY_SPEAKER: 3
};

/**
 * optional string name = 1;
 * @return {?string}
 */
proto.speech.tts.WaveNetFeature.prototype.getName = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.WaveNetFeature} returns this
 */
proto.speech.tts.WaveNetFeature.prototype.setName = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.WaveNetFeature} returns this
 */
proto.speech.tts.WaveNetFeature.prototype.clearName = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.WaveNetFeature.prototype.hasName = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional FeatureSpeed feature_speed = 2;
 * @return {!proto.speech.tts.WaveNetFeature.FeatureSpeed}
 */
proto.speech.tts.WaveNetFeature.prototype.getFeatureSpeed = function() {
  return /** @type {!proto.speech.tts.WaveNetFeature.FeatureSpeed} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?proto.speech.tts.WaveNetFeature.FeatureSpeed|undefined} value
 * @return {!proto.speech.tts.WaveNetFeature} returns this
 */
proto.speech.tts.WaveNetFeature.prototype.setFeatureSpeed = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.WaveNetFeature} returns this
 */
proto.speech.tts.WaveNetFeature.prototype.clearFeatureSpeed = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.WaveNetFeature.prototype.hasFeatureSpeed = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional GenerationType generation_type = 3;
 * @return {!proto.speech.tts.WaveNetFeature.GenerationType}
 */
proto.speech.tts.WaveNetFeature.prototype.getGenerationType = function() {
  return /** @type {!proto.speech.tts.WaveNetFeature.GenerationType} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?proto.speech.tts.WaveNetFeature.GenerationType|undefined} value
 * @return {!proto.speech.tts.WaveNetFeature} returns this
 */
proto.speech.tts.WaveNetFeature.prototype.setGenerationType = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.WaveNetFeature} returns this
 */
proto.speech.tts.WaveNetFeature.prototype.clearGenerationType = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.WaveNetFeature.prototype.hasGenerationType = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional bool is_integer = 4;
 * @return {?boolean}
 */
proto.speech.tts.WaveNetFeature.prototype.getIsInteger = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 4));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.WaveNetFeature} returns this
 */
proto.speech.tts.WaveNetFeature.prototype.setIsInteger = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.WaveNetFeature} returns this
 */
proto.speech.tts.WaveNetFeature.prototype.clearIsInteger = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.WaveNetFeature.prototype.hasIsInteger = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional int32 depth = 5;
 * @return {?number}
 */
proto.speech.tts.WaveNetFeature.prototype.getDepth = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 5));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.WaveNetFeature} returns this
 */
proto.speech.tts.WaveNetFeature.prototype.setDepth = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.WaveNetFeature} returns this
 */
proto.speech.tts.WaveNetFeature.prototype.clearDepth = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.WaveNetFeature.prototype.hasDepth = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional WaveNetLayout layout = 6;
 * @return {!proto.speech.tts.WaveNetLayout}
 */
proto.speech.tts.WaveNetFeature.prototype.getLayout = function() {
  return /** @type {!proto.speech.tts.WaveNetLayout} */ (jspb.Message.getField(this, 6));
};


/**
 * @param {?proto.speech.tts.WaveNetLayout|undefined} value
 * @return {!proto.speech.tts.WaveNetFeature} returns this
 */
proto.speech.tts.WaveNetFeature.prototype.setLayout = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.WaveNetFeature} returns this
 */
proto.speech.tts.WaveNetFeature.prototype.clearLayout = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.WaveNetFeature.prototype.hasLayout = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.WaveNetFeature}
 */
proto.speech.tts.WaveNetFeature.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.WaveNetFeature, data);
};



//speech/tts/engine/wavenet/proto/wavenet_synthesizer_proto_jspb/wavenetmodeloptions.jspb.js
// source: speech/tts/engine/wavenet/proto/wavenet_synthesizer.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.WaveNetModelOptions');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.WaveNetFeature');

goog.forwardDeclare('proto.speech.tts.WaveNetLayout');
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.WaveNetModelOptions = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.WaveNetModelOptions.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.WaveNetModelOptions, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.WaveNetModelOptions.displayName = 'proto.speech.tts.WaveNetModelOptions';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.WaveNetModelOptions.repeatedFields_ = [5,6,7,15,20];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.WaveNetModelOptions.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.WaveNetModelOptions.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.WaveNetModelOptions} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.WaveNetModelOptions.toObject = function(includeInstance, msg) {
  var f, obj = {
    batchSize: jspb.Message.getFieldWithDefault(msg, 1, 1),
    sampleRate: jspb.Message.getFieldWithDefault(msg, 2, 24000),
    outputSamplesPerLinguisticFeature: jspb.Message.getFieldWithDefault(msg, 3, 120),
    muLaw: (f = jspb.Message.getBooleanField(msg, 4)) == null ? undefined : f,
    speakerIdsList: (f = jspb.Message.getRepeatedField(msg, 5)) == null ? undefined : f,
    languageLocaleIdsList: (f = jspb.Message.getRepeatedField(msg, 6)) == null ? undefined : f,
    inputFeaturesList: jspb.Message.toObjectList(msg.getInputFeaturesList(),
    proto.speech.tts.WaveNetFeature.toObject, includeInstance),
    outputLayout: (f = jspb.Message.getField(msg, 8)) == null ? undefined : f,
    numBatchLoops: (f = jspb.Message.getField(msg, 9)) == null ? undefined : f,
    chunkSize: (f = jspb.Message.getField(msg, 10)) == null ? undefined : f,
    overlap: (f = jspb.Message.getField(msg, 11)) == null ? undefined : f,
    conditionPadding: (f = jspb.Message.getField(msg, 16)) == null ? undefined : f,
    modelId: (f = jspb.Message.getField(msg, 12)) == null ? undefined : f,
    exportModelStateKey: (f = jspb.Message.getField(msg, 13)) == null ? undefined : f,
    coherentBatch: jspb.Message.getBooleanFieldWithDefault(msg, 14, false),
    silenceFeatureIndexList: (f = jspb.Message.getRepeatedField(msg, 15)) == null ? undefined : f,
    architectureName: (f = jspb.Message.getField(msg, 18)) == null ? undefined : f,
    customSignatureDefKey: (f = jspb.Message.getField(msg, 19)) == null ? undefined : f,
    conditioningsList: (f = jspb.Message.getRepeatedField(msg, 20)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of WaveNetModelOptions as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.WaveNetModelOptions.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.batchSize;

  /** @type {?number|undefined} */
  this.sampleRate;

  /** @type {?number|undefined} */
  this.outputSamplesPerLinguisticFeature;

  /** @type {?boolean|undefined} */
  this.muLaw;

  /** @type {?Array<string>|undefined} */
  this.speakerIdsList;

  /** @type {?Array<string>|undefined} */
  this.languageLocaleIdsList;

  /** @type {?Array<!proto.speech.tts.WaveNetFeature.ObjectFormat>|undefined} */
  this.inputFeaturesList;

  /** @type {?number|undefined} */
  this.outputLayout;

  /** @type {?number|undefined} */
  this.numBatchLoops;

  /** @type {?number|undefined} */
  this.chunkSize;

  /** @type {?number|undefined} */
  this.overlap;

  /** @type {?number|undefined} */
  this.conditionPadding;

  /** @type {?string|undefined} */
  this.modelId;

  /** @type {?string|undefined} */
  this.exportModelStateKey;

  /** @type {?boolean|undefined} */
  this.coherentBatch;

  /** @type {?Array<number>|undefined} */
  this.silenceFeatureIndexList;

  /** @type {?string|undefined} */
  this.architectureName;

  /** @type {?string|undefined} */
  this.customSignatureDefKey;

  /** @type {?Array<string>|undefined} */
  this.conditioningsList;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.WaveNetModelOptions.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.WaveNetModelOptions}
 */
proto.speech.tts.WaveNetModelOptions.fromObject = function(obj) {
  var msg = new proto.speech.tts.WaveNetModelOptions();
  obj.batchSize != null && jspb.Message.setField(msg, 1, obj.batchSize);
  obj.sampleRate != null && jspb.Message.setField(msg, 2, obj.sampleRate);
  obj.outputSamplesPerLinguisticFeature != null && jspb.Message.setField(msg, 3, obj.outputSamplesPerLinguisticFeature);
  obj.muLaw != null && jspb.Message.setField(msg, 4, obj.muLaw);
  obj.speakerIdsList != null && jspb.Message.setField(msg, 5, obj.speakerIdsList);
  obj.languageLocaleIdsList != null && jspb.Message.setField(msg, 6, obj.languageLocaleIdsList);
  obj.inputFeaturesList && jspb.Message.setRepeatedWrapperField(
      msg, 7, obj.inputFeaturesList.map(
          proto.speech.tts.WaveNetFeature.fromObject));
  obj.outputLayout != null && jspb.Message.setField(msg, 8, obj.outputLayout);
  obj.numBatchLoops != null && jspb.Message.setField(msg, 9, obj.numBatchLoops);
  obj.chunkSize != null && jspb.Message.setField(msg, 10, obj.chunkSize);
  obj.overlap != null && jspb.Message.setField(msg, 11, obj.overlap);
  obj.conditionPadding != null && jspb.Message.setField(msg, 16, obj.conditionPadding);
  obj.modelId != null && jspb.Message.setField(msg, 12, obj.modelId);
  obj.exportModelStateKey != null && jspb.Message.setField(msg, 13, obj.exportModelStateKey);
  obj.coherentBatch != null && jspb.Message.setField(msg, 14, obj.coherentBatch);
  obj.silenceFeatureIndexList != null && jspb.Message.setField(msg, 15, obj.silenceFeatureIndexList);
  obj.architectureName != null && jspb.Message.setField(msg, 18, obj.architectureName);
  obj.customSignatureDefKey != null && jspb.Message.setField(msg, 19, obj.customSignatureDefKey);
  obj.conditioningsList != null && jspb.Message.setField(msg, 20, obj.conditioningsList);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.WaveNetModelOptions}
 */
proto.speech.tts.WaveNetModelOptions.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.WaveNetModelOptions;
  return proto.speech.tts.WaveNetModelOptions.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.WaveNetModelOptions} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.WaveNetModelOptions}
 */
proto.speech.tts.WaveNetModelOptions.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setBatchSize(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setSampleRate(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setOutputSamplesPerLinguisticFeature(value);
      break;
    case 4:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setMuLaw(value);
      break;
    case 5:
      var value = /** @type {string} */ (reader.readString());
      msg.addSpeakerIds(value);
      break;
    case 6:
      var value = /** @type {string} */ (reader.readString());
      msg.addLanguageLocaleIds(value);
      break;
    case 7:
      var value = new proto.speech.tts.WaveNetFeature;
      reader.readMessage(value,proto.speech.tts.WaveNetFeature.deserializeBinaryFromReader);
      msg.addInputFeatures(value);
      break;
    case 8:
      var value = /** @type {!proto.speech.tts.WaveNetLayout} */ (reader.readEnum());
      msg.setOutputLayout(value);
      break;
    case 9:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setNumBatchLoops(value);
      break;
    case 10:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setChunkSize(value);
      break;
    case 11:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setOverlap(value);
      break;
    case 16:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setConditionPadding(value);
      break;
    case 12:
      var value = /** @type {string} */ (reader.readString());
      msg.setModelId(value);
      break;
    case 13:
      var value = /** @type {string} */ (reader.readString());
      msg.setExportModelStateKey(value);
      break;
    case 14:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setCoherentBatch(value);
      break;
    case 15:
      var value = /** @type {number} */ (reader.readInt32());
      msg.addSilenceFeatureIndex(value);
      break;
    case 18:
      var value = /** @type {string} */ (reader.readString());
      msg.setArchitectureName(value);
      break;
    case 19:
      var value = /** @type {string} */ (reader.readString());
      msg.setCustomSignatureDefKey(value);
      break;
    case 20:
      var value = /** @type {string} */ (reader.readString());
      msg.addConditionings(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.WaveNetModelOptions.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.WaveNetModelOptions.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.WaveNetModelOptions} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.WaveNetModelOptions.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {number} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeInt32(
      1,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeInt32(
      2,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeInt32(
      3,
      f
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeBool(
      4,
      f
    );
  }
  f = message.getSpeakerIdsList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      5,
      f
    );
  }
  f = message.getLanguageLocaleIdsList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      6,
      f
    );
  }
  f = message.getInputFeaturesList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      7,
      f,
      proto.speech.tts.WaveNetFeature.serializeBinaryToWriter
    );
  }
  f = /** @type {!proto.speech.tts.WaveNetLayout} */ (jspb.Message.getField(message, 8));
  if (f != null) {
    writer.writeEnum(
      8,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 9));
  if (f != null) {
    writer.writeInt32(
      9,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 10));
  if (f != null) {
    writer.writeInt32(
      10,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 11));
  if (f != null) {
    writer.writeInt32(
      11,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 16));
  if (f != null) {
    writer.writeInt32(
      16,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 12));
  if (f != null) {
    writer.writeString(
      12,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 13));
  if (f != null) {
    writer.writeString(
      13,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 14));
  if (f != null) {
    writer.writeBool(
      14,
      f
    );
  }
  f = message.getSilenceFeatureIndexList();
  if (f.length > 0) {
    writer.writeRepeatedInt32(
      15,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 18));
  if (f != null) {
    writer.writeString(
      18,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 19));
  if (f != null) {
    writer.writeString(
      19,
      f
    );
  }
  f = message.getConditioningsList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      20,
      f
    );
  }
};


/**
 * optional int32 batch_size = 1;
 * @return {number}
 */
proto.speech.tts.WaveNetModelOptions.prototype.getBatchSize = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 1, 1));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.WaveNetModelOptions} returns this
 */
proto.speech.tts.WaveNetModelOptions.prototype.setBatchSize = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.WaveNetModelOptions} returns this
 */
proto.speech.tts.WaveNetModelOptions.prototype.clearBatchSize = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.WaveNetModelOptions.prototype.hasBatchSize = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional int32 sample_rate = 2;
 * @return {number}
 */
proto.speech.tts.WaveNetModelOptions.prototype.getSampleRate = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 2, 24000));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.WaveNetModelOptions} returns this
 */
proto.speech.tts.WaveNetModelOptions.prototype.setSampleRate = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.WaveNetModelOptions} returns this
 */
proto.speech.tts.WaveNetModelOptions.prototype.clearSampleRate = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.WaveNetModelOptions.prototype.hasSampleRate = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional int32 output_samples_per_linguistic_feature = 3;
 * @return {number}
 */
proto.speech.tts.WaveNetModelOptions.prototype.getOutputSamplesPerLinguisticFeature = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 3, 120));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.WaveNetModelOptions} returns this
 */
proto.speech.tts.WaveNetModelOptions.prototype.setOutputSamplesPerLinguisticFeature = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.WaveNetModelOptions} returns this
 */
proto.speech.tts.WaveNetModelOptions.prototype.clearOutputSamplesPerLinguisticFeature = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.WaveNetModelOptions.prototype.hasOutputSamplesPerLinguisticFeature = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional bool mu_law = 4;
 * @return {?boolean}
 */
proto.speech.tts.WaveNetModelOptions.prototype.getMuLaw = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 4));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.WaveNetModelOptions} returns this
 */
proto.speech.tts.WaveNetModelOptions.prototype.setMuLaw = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.WaveNetModelOptions} returns this
 */
proto.speech.tts.WaveNetModelOptions.prototype.clearMuLaw = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.WaveNetModelOptions.prototype.hasMuLaw = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * repeated string speaker_ids = 5;
 * @return {!Array<string>}
 */
proto.speech.tts.WaveNetModelOptions.prototype.getSpeakerIdsList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 5));
};


/**
 * @param {Array<string>|undefined} value
 * @return {!proto.speech.tts.WaveNetModelOptions} returns this
 */
proto.speech.tts.WaveNetModelOptions.prototype.setSpeakerIdsList = function(value) {
  return jspb.Message.setField(this, 5, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.WaveNetModelOptions} returns this
 */
proto.speech.tts.WaveNetModelOptions.prototype.addSpeakerIds = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 5, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.WaveNetModelOptions} returns this
 */
proto.speech.tts.WaveNetModelOptions.prototype.clearSpeakerIdsList = function() {
  return this.setSpeakerIdsList([]);
};


/**
 * repeated string language_locale_ids = 6;
 * @return {!Array<string>}
 */
proto.speech.tts.WaveNetModelOptions.prototype.getLanguageLocaleIdsList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 6));
};


/**
 * @param {Array<string>|undefined} value
 * @return {!proto.speech.tts.WaveNetModelOptions} returns this
 */
proto.speech.tts.WaveNetModelOptions.prototype.setLanguageLocaleIdsList = function(value) {
  return jspb.Message.setField(this, 6, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.WaveNetModelOptions} returns this
 */
proto.speech.tts.WaveNetModelOptions.prototype.addLanguageLocaleIds = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 6, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.WaveNetModelOptions} returns this
 */
proto.speech.tts.WaveNetModelOptions.prototype.clearLanguageLocaleIdsList = function() {
  return this.setLanguageLocaleIdsList([]);
};


/**
 * repeated WaveNetFeature input_features = 7;
 * @return {!Array<!proto.speech.tts.WaveNetFeature>}
 */
proto.speech.tts.WaveNetModelOptions.prototype.getInputFeaturesList = function() {
  return /** @type{!Array<!proto.speech.tts.WaveNetFeature>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.WaveNetFeature, 7));
};


/**
 * @param {Array<!proto.speech.tts.WaveNetFeature>|undefined} value
 * @return {!proto.speech.tts.WaveNetModelOptions} returns this
*/
proto.speech.tts.WaveNetModelOptions.prototype.setInputFeaturesList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 7, value);
};


/**
 * @param {!proto.speech.tts.WaveNetFeature=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.WaveNetFeature}
 */
proto.speech.tts.WaveNetModelOptions.prototype.addInputFeatures = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 7, opt_value, proto.speech.tts.WaveNetFeature, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.WaveNetModelOptions} returns this
 */
proto.speech.tts.WaveNetModelOptions.prototype.clearInputFeaturesList = function() {
  return this.setInputFeaturesList([]);
};


/**
 * optional WaveNetLayout output_layout = 8;
 * @return {!proto.speech.tts.WaveNetLayout}
 */
proto.speech.tts.WaveNetModelOptions.prototype.getOutputLayout = function() {
  return /** @type {!proto.speech.tts.WaveNetLayout} */ (jspb.Message.getField(this, 8));
};


/**
 * @param {?proto.speech.tts.WaveNetLayout|undefined} value
 * @return {!proto.speech.tts.WaveNetModelOptions} returns this
 */
proto.speech.tts.WaveNetModelOptions.prototype.setOutputLayout = function(value) {
  return jspb.Message.setField(this, 8, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.WaveNetModelOptions} returns this
 */
proto.speech.tts.WaveNetModelOptions.prototype.clearOutputLayout = function() {
  return jspb.Message.setField(this, 8, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.WaveNetModelOptions.prototype.hasOutputLayout = function() {
  return jspb.Message.getField(this, 8) != null;
};


/**
 * optional int32 num_batch_loops = 9;
 * @return {?number}
 */
proto.speech.tts.WaveNetModelOptions.prototype.getNumBatchLoops = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 9));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.WaveNetModelOptions} returns this
 */
proto.speech.tts.WaveNetModelOptions.prototype.setNumBatchLoops = function(value) {
  return jspb.Message.setField(this, 9, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.WaveNetModelOptions} returns this
 */
proto.speech.tts.WaveNetModelOptions.prototype.clearNumBatchLoops = function() {
  return jspb.Message.setField(this, 9, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.WaveNetModelOptions.prototype.hasNumBatchLoops = function() {
  return jspb.Message.getField(this, 9) != null;
};


/**
 * optional int32 chunk_size = 10;
 * @return {?number}
 */
proto.speech.tts.WaveNetModelOptions.prototype.getChunkSize = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 10));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.WaveNetModelOptions} returns this
 */
proto.speech.tts.WaveNetModelOptions.prototype.setChunkSize = function(value) {
  return jspb.Message.setField(this, 10, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.WaveNetModelOptions} returns this
 */
proto.speech.tts.WaveNetModelOptions.prototype.clearChunkSize = function() {
  return jspb.Message.setField(this, 10, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.WaveNetModelOptions.prototype.hasChunkSize = function() {
  return jspb.Message.getField(this, 10) != null;
};


/**
 * optional int32 overlap = 11;
 * @return {?number}
 */
proto.speech.tts.WaveNetModelOptions.prototype.getOverlap = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 11));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.WaveNetModelOptions} returns this
 */
proto.speech.tts.WaveNetModelOptions.prototype.setOverlap = function(value) {
  return jspb.Message.setField(this, 11, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.WaveNetModelOptions} returns this
 */
proto.speech.tts.WaveNetModelOptions.prototype.clearOverlap = function() {
  return jspb.Message.setField(this, 11, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.WaveNetModelOptions.prototype.hasOverlap = function() {
  return jspb.Message.getField(this, 11) != null;
};


/**
 * optional int32 condition_padding = 16;
 * @return {?number}
 */
proto.speech.tts.WaveNetModelOptions.prototype.getConditionPadding = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 16));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.WaveNetModelOptions} returns this
 */
proto.speech.tts.WaveNetModelOptions.prototype.setConditionPadding = function(value) {
  return jspb.Message.setField(this, 16, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.WaveNetModelOptions} returns this
 */
proto.speech.tts.WaveNetModelOptions.prototype.clearConditionPadding = function() {
  return jspb.Message.setField(this, 16, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.WaveNetModelOptions.prototype.hasConditionPadding = function() {
  return jspb.Message.getField(this, 16) != null;
};


/**
 * optional string model_id = 12;
 * @return {?string}
 */
proto.speech.tts.WaveNetModelOptions.prototype.getModelId = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 12));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.WaveNetModelOptions} returns this
 */
proto.speech.tts.WaveNetModelOptions.prototype.setModelId = function(value) {
  return jspb.Message.setField(this, 12, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.WaveNetModelOptions} returns this
 */
proto.speech.tts.WaveNetModelOptions.prototype.clearModelId = function() {
  return jspb.Message.setField(this, 12, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.WaveNetModelOptions.prototype.hasModelId = function() {
  return jspb.Message.getField(this, 12) != null;
};


/**
 * optional string export_model_state_key = 13;
 * @return {?string}
 */
proto.speech.tts.WaveNetModelOptions.prototype.getExportModelStateKey = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 13));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.WaveNetModelOptions} returns this
 */
proto.speech.tts.WaveNetModelOptions.prototype.setExportModelStateKey = function(value) {
  return jspb.Message.setField(this, 13, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.WaveNetModelOptions} returns this
 */
proto.speech.tts.WaveNetModelOptions.prototype.clearExportModelStateKey = function() {
  return jspb.Message.setField(this, 13, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.WaveNetModelOptions.prototype.hasExportModelStateKey = function() {
  return jspb.Message.getField(this, 13) != null;
};


/**
 * optional bool coherent_batch = 14;
 * @return {boolean}
 */
proto.speech.tts.WaveNetModelOptions.prototype.getCoherentBatch = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 14, false));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.WaveNetModelOptions} returns this
 */
proto.speech.tts.WaveNetModelOptions.prototype.setCoherentBatch = function(value) {
  return jspb.Message.setField(this, 14, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.WaveNetModelOptions} returns this
 */
proto.speech.tts.WaveNetModelOptions.prototype.clearCoherentBatch = function() {
  return jspb.Message.setField(this, 14, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.WaveNetModelOptions.prototype.hasCoherentBatch = function() {
  return jspb.Message.getField(this, 14) != null;
};


/**
 * repeated int32 silence_feature_index = 15;
 * @return {!Array<number>}
 */
proto.speech.tts.WaveNetModelOptions.prototype.getSilenceFeatureIndexList = function() {
  return /** @type {!Array<number>} */ (jspb.Message.getRepeatedField(this, 15));
};


/**
 * @param {Array<number>|undefined} value
 * @return {!proto.speech.tts.WaveNetModelOptions} returns this
 */
proto.speech.tts.WaveNetModelOptions.prototype.setSilenceFeatureIndexList = function(value) {
  return jspb.Message.setField(this, 15, value || []);
};


/**
 * @param {number} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.WaveNetModelOptions} returns this
 */
proto.speech.tts.WaveNetModelOptions.prototype.addSilenceFeatureIndex = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 15, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.WaveNetModelOptions} returns this
 */
proto.speech.tts.WaveNetModelOptions.prototype.clearSilenceFeatureIndexList = function() {
  return this.setSilenceFeatureIndexList([]);
};


/**
 * optional string architecture_name = 18;
 * @return {?string}
 */
proto.speech.tts.WaveNetModelOptions.prototype.getArchitectureName = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 18));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.WaveNetModelOptions} returns this
 */
proto.speech.tts.WaveNetModelOptions.prototype.setArchitectureName = function(value) {
  return jspb.Message.setField(this, 18, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.WaveNetModelOptions} returns this
 */
proto.speech.tts.WaveNetModelOptions.prototype.clearArchitectureName = function() {
  return jspb.Message.setField(this, 18, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.WaveNetModelOptions.prototype.hasArchitectureName = function() {
  return jspb.Message.getField(this, 18) != null;
};


/**
 * optional string custom_signature_def_key = 19;
 * @return {?string}
 */
proto.speech.tts.WaveNetModelOptions.prototype.getCustomSignatureDefKey = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 19));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.WaveNetModelOptions} returns this
 */
proto.speech.tts.WaveNetModelOptions.prototype.setCustomSignatureDefKey = function(value) {
  return jspb.Message.setField(this, 19, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.WaveNetModelOptions} returns this
 */
proto.speech.tts.WaveNetModelOptions.prototype.clearCustomSignatureDefKey = function() {
  return jspb.Message.setField(this, 19, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.WaveNetModelOptions.prototype.hasCustomSignatureDefKey = function() {
  return jspb.Message.getField(this, 19) != null;
};


/**
 * repeated string conditionings = 20;
 * @return {!Array<string>}
 */
proto.speech.tts.WaveNetModelOptions.prototype.getConditioningsList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 20));
};


/**
 * @param {Array<string>|undefined} value
 * @return {!proto.speech.tts.WaveNetModelOptions} returns this
 */
proto.speech.tts.WaveNetModelOptions.prototype.setConditioningsList = function(value) {
  return jspb.Message.setField(this, 20, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.WaveNetModelOptions} returns this
 */
proto.speech.tts.WaveNetModelOptions.prototype.addConditionings = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 20, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.WaveNetModelOptions} returns this
 */
proto.speech.tts.WaveNetModelOptions.prototype.clearConditioningsList = function() {
  return this.setConditioningsList([]);
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.WaveNetModelOptions}
 */
proto.speech.tts.WaveNetModelOptions.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.WaveNetModelOptions, data);
};



//speech/tts/engine/wavenet/proto/wavenet_synthesizer_proto_jspb/wavenetsynthesisoptions.jspb.js
// source: speech/tts/engine/wavenet/proto/wavenet_synthesizer.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.WaveNetSynthesisOptions');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.StitchingOptions');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.WaveNetSynthesisOptions = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.WaveNetSynthesisOptions, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.WaveNetSynthesisOptions.displayName = 'proto.speech.tts.WaveNetSynthesisOptions';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.WaveNetSynthesisOptions.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.WaveNetSynthesisOptions.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.WaveNetSynthesisOptions} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.WaveNetSynthesisOptions.toObject = function(includeInstance, msg) {
  var f, obj = {
    speakerId: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    languageLocaleId: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    dropStartSamples: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,
    dropEndSamples: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f,
    muteLevel: jspb.Message.getFloatingPointFieldWithDefault(msg, 7, 1.0),
    muteWindowSamples: jspb.Message.getFieldWithDefault(msg, 8, 16),
    scale: jspb.Message.getFloatingPointFieldWithDefault(msg, 9, 1.0),
    stitchingOptions: (f = msg.getStitchingOptions()) && proto.speech.tts.StitchingOptions.toObject(includeInstance, f),
    skipSynth: jspb.Message.getBooleanFieldWithDefault(msg, 11, false)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of WaveNetSynthesisOptions as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.WaveNetSynthesisOptions.ObjectFormat = function() {
  /** @type {?string|undefined} */
  this.speakerId;

  /** @type {?string|undefined} */
  this.languageLocaleId;

  /** @type {?number|undefined} */
  this.dropStartSamples;

  /** @type {?number|undefined} */
  this.dropEndSamples;

  /** @type {?number|undefined} */
  this.muteLevel;

  /** @type {?number|undefined} */
  this.muteWindowSamples;

  /** @type {?number|undefined} */
  this.scale;

  /** @type {?proto.speech.tts.StitchingOptions.ObjectFormat|undefined} */
  this.stitchingOptions;

  /** @type {?boolean|undefined} */
  this.skipSynth;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.WaveNetSynthesisOptions.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.WaveNetSynthesisOptions}
 */
proto.speech.tts.WaveNetSynthesisOptions.fromObject = function(obj) {
  var msg = new proto.speech.tts.WaveNetSynthesisOptions();
  obj.speakerId != null && jspb.Message.setField(msg, 1, obj.speakerId);
  obj.languageLocaleId != null && jspb.Message.setField(msg, 2, obj.languageLocaleId);
  obj.dropStartSamples != null && jspb.Message.setField(msg, 3, obj.dropStartSamples);
  obj.dropEndSamples != null && jspb.Message.setField(msg, 4, obj.dropEndSamples);
  obj.muteLevel != null && jspb.Message.setField(msg, 7, obj.muteLevel);
  obj.muteWindowSamples != null && jspb.Message.setField(msg, 8, obj.muteWindowSamples);
  obj.scale != null && jspb.Message.setField(msg, 9, obj.scale);
  obj.stitchingOptions && jspb.Message.setWrapperField(
      msg, 10, proto.speech.tts.StitchingOptions.fromObject(obj.stitchingOptions));
  obj.skipSynth != null && jspb.Message.setField(msg, 11, obj.skipSynth);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.WaveNetSynthesisOptions}
 */
proto.speech.tts.WaveNetSynthesisOptions.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.WaveNetSynthesisOptions;
  return proto.speech.tts.WaveNetSynthesisOptions.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.WaveNetSynthesisOptions} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.WaveNetSynthesisOptions}
 */
proto.speech.tts.WaveNetSynthesisOptions.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setSpeakerId(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setLanguageLocaleId(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setDropStartSamples(value);
      break;
    case 4:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setDropEndSamples(value);
      break;
    case 7:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setMuteLevel(value);
      break;
    case 8:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setMuteWindowSamples(value);
      break;
    case 9:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setScale(value);
      break;
    case 10:
      var value = new proto.speech.tts.StitchingOptions;
      reader.readMessage(value,proto.speech.tts.StitchingOptions.deserializeBinaryFromReader);
      msg.setStitchingOptions(value);
      break;
    case 11:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setSkipSynth(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.WaveNetSynthesisOptions.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.WaveNetSynthesisOptions.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.WaveNetSynthesisOptions} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.WaveNetSynthesisOptions.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeInt32(
      3,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeInt32(
      4,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 7));
  if (f != null) {
    writer.writeFloat(
      7,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 8));
  if (f != null) {
    writer.writeInt32(
      8,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 9));
  if (f != null) {
    writer.writeFloat(
      9,
      f
    );
  }
  f = message.getStitchingOptions();
  if (f != null) {
    writer.writeMessage(
      10,
      f,
      proto.speech.tts.StitchingOptions.serializeBinaryToWriter
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 11));
  if (f != null) {
    writer.writeBool(
      11,
      f
    );
  }
};


/**
 * optional string speaker_id = 1;
 * @return {?string}
 */
proto.speech.tts.WaveNetSynthesisOptions.prototype.getSpeakerId = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.WaveNetSynthesisOptions} returns this
 */
proto.speech.tts.WaveNetSynthesisOptions.prototype.setSpeakerId = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.WaveNetSynthesisOptions} returns this
 */
proto.speech.tts.WaveNetSynthesisOptions.prototype.clearSpeakerId = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.WaveNetSynthesisOptions.prototype.hasSpeakerId = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional string language_locale_id = 2;
 * @return {?string}
 */
proto.speech.tts.WaveNetSynthesisOptions.prototype.getLanguageLocaleId = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.WaveNetSynthesisOptions} returns this
 */
proto.speech.tts.WaveNetSynthesisOptions.prototype.setLanguageLocaleId = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.WaveNetSynthesisOptions} returns this
 */
proto.speech.tts.WaveNetSynthesisOptions.prototype.clearLanguageLocaleId = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.WaveNetSynthesisOptions.prototype.hasLanguageLocaleId = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional int32 drop_start_samples = 3;
 * @return {?number}
 */
proto.speech.tts.WaveNetSynthesisOptions.prototype.getDropStartSamples = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.WaveNetSynthesisOptions} returns this
 */
proto.speech.tts.WaveNetSynthesisOptions.prototype.setDropStartSamples = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.WaveNetSynthesisOptions} returns this
 */
proto.speech.tts.WaveNetSynthesisOptions.prototype.clearDropStartSamples = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.WaveNetSynthesisOptions.prototype.hasDropStartSamples = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional int32 drop_end_samples = 4;
 * @return {?number}
 */
proto.speech.tts.WaveNetSynthesisOptions.prototype.getDropEndSamples = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 4));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.WaveNetSynthesisOptions} returns this
 */
proto.speech.tts.WaveNetSynthesisOptions.prototype.setDropEndSamples = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.WaveNetSynthesisOptions} returns this
 */
proto.speech.tts.WaveNetSynthesisOptions.prototype.clearDropEndSamples = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.WaveNetSynthesisOptions.prototype.hasDropEndSamples = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional float mute_level = 7;
 * @return {number}
 */
proto.speech.tts.WaveNetSynthesisOptions.prototype.getMuteLevel = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 7, 1.0));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.WaveNetSynthesisOptions} returns this
 */
proto.speech.tts.WaveNetSynthesisOptions.prototype.setMuteLevel = function(value) {
  return jspb.Message.setField(this, 7, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.WaveNetSynthesisOptions} returns this
 */
proto.speech.tts.WaveNetSynthesisOptions.prototype.clearMuteLevel = function() {
  return jspb.Message.setField(this, 7, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.WaveNetSynthesisOptions.prototype.hasMuteLevel = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * optional int32 mute_window_samples = 8;
 * @return {number}
 */
proto.speech.tts.WaveNetSynthesisOptions.prototype.getMuteWindowSamples = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 8, 16));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.WaveNetSynthesisOptions} returns this
 */
proto.speech.tts.WaveNetSynthesisOptions.prototype.setMuteWindowSamples = function(value) {
  return jspb.Message.setField(this, 8, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.WaveNetSynthesisOptions} returns this
 */
proto.speech.tts.WaveNetSynthesisOptions.prototype.clearMuteWindowSamples = function() {
  return jspb.Message.setField(this, 8, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.WaveNetSynthesisOptions.prototype.hasMuteWindowSamples = function() {
  return jspb.Message.getField(this, 8) != null;
};


/**
 * optional float scale = 9;
 * @return {number}
 */
proto.speech.tts.WaveNetSynthesisOptions.prototype.getScale = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 9, 1.0));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.WaveNetSynthesisOptions} returns this
 */
proto.speech.tts.WaveNetSynthesisOptions.prototype.setScale = function(value) {
  return jspb.Message.setField(this, 9, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.WaveNetSynthesisOptions} returns this
 */
proto.speech.tts.WaveNetSynthesisOptions.prototype.clearScale = function() {
  return jspb.Message.setField(this, 9, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.WaveNetSynthesisOptions.prototype.hasScale = function() {
  return jspb.Message.getField(this, 9) != null;
};


/**
 * optional StitchingOptions stitching_options = 10;
 * @return {?proto.speech.tts.StitchingOptions}
 */
proto.speech.tts.WaveNetSynthesisOptions.prototype.getStitchingOptions = function() {
  return /** @type{?proto.speech.tts.StitchingOptions} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.StitchingOptions, 10));
};


/**
 * @param {?proto.speech.tts.StitchingOptions|undefined} value
 * @return {!proto.speech.tts.WaveNetSynthesisOptions} returns this
*/
proto.speech.tts.WaveNetSynthesisOptions.prototype.setStitchingOptions = function(value) {
  return jspb.Message.setWrapperField(this, 10, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.WaveNetSynthesisOptions} returns this
 */
proto.speech.tts.WaveNetSynthesisOptions.prototype.clearStitchingOptions = function() {
  return this.setStitchingOptions(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.WaveNetSynthesisOptions.prototype.hasStitchingOptions = function() {
  return jspb.Message.getField(this, 10) != null;
};


/**
 * optional bool skip_synth = 11;
 * @return {boolean}
 */
proto.speech.tts.WaveNetSynthesisOptions.prototype.getSkipSynth = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 11, false));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.WaveNetSynthesisOptions} returns this
 */
proto.speech.tts.WaveNetSynthesisOptions.prototype.setSkipSynth = function(value) {
  return jspb.Message.setField(this, 11, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.WaveNetSynthesisOptions} returns this
 */
proto.speech.tts.WaveNetSynthesisOptions.prototype.clearSkipSynth = function() {
  return jspb.Message.setField(this, 11, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.WaveNetSynthesisOptions.prototype.hasSkipSynth = function() {
  return jspb.Message.getField(this, 11) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.WaveNetSynthesisOptions}
 */
proto.speech.tts.WaveNetSynthesisOptions.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.WaveNetSynthesisOptions, data);
};



//speech/tts/engine/wavenet/proto/wavenet_diagnostics_proto_jspb/wavenetdiagnostics.jspb.js
// source: speech/tts/engine/wavenet/proto/wavenet_diagnostics.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.WavenetDiagnostics');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.ChunkInfo');
goog.require('proto.speech.tts.StitchingOptions');
goog.require('proto.speech.tts.WaveNetModelOptions');
goog.require('proto.speech.tts.WaveNetSynthesisOptions');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.WavenetDiagnostics = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 7, proto.speech.tts.WavenetDiagnostics.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.WavenetDiagnostics, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.WavenetDiagnostics.displayName = 'proto.speech.tts.WavenetDiagnostics';
}

/**
 * The extensions registered with this message class. This is a map of
 * extension field number to fieldInfo object.
 *
 * For example:
 *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }
 *
 * fieldName contains the JsCompiler renamed field name property so that it
 * works in OPTIMIZED mode.
 *
 * @type {!Object<number, jspb.ExtensionFieldInfo>}
 */
proto.speech.tts.WavenetDiagnostics.extensions = {};


/**
 * The extensions registered with this message class. This is a map of
 * extension field number to fieldInfo object.
 *
 * For example:
 *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }
 *
 * fieldName contains the JsCompiler renamed field name property so that it
 * works in OPTIMIZED mode.
 *
 * @type {!Object<number, jspb.ExtensionFieldBinaryInfo>}
 */
proto.speech.tts.WavenetDiagnostics.extensionsBinary = {};


/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.WavenetDiagnostics.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.WavenetDiagnostics.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.WavenetDiagnostics.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.WavenetDiagnostics} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.WavenetDiagnostics.toObject = function(includeInstance, msg) {
  var f, obj = {
    chunksList: jspb.Message.toObjectList(msg.getChunksList(),
    proto.speech.tts.ChunkInfo.toObject, includeInstance),
    overlapS: (f = jspb.Message.getOptionalFloatingPointField(msg, 2)) == null ? undefined : f,
    sampleRate: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,
    stitchingOptions: (f = msg.getStitchingOptions()) && proto.speech.tts.StitchingOptions.toObject(includeInstance, f),
    modelOptions: (f = msg.getModelOptions()) && proto.speech.tts.WaveNetModelOptions.toObject(includeInstance, f),
    synthesisOptions: (f = msg.getSynthesisOptions()) && proto.speech.tts.WaveNetSynthesisOptions.toObject(includeInstance, f)
  };

  jspb.Message.toObjectExtension(/** @type {!jspb.Message} */ (msg), obj,
      proto.speech.tts.WavenetDiagnostics.extensions, proto.speech.tts.WavenetDiagnostics.prototype.getExtension,
      includeInstance);
  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of WavenetDiagnostics as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.WavenetDiagnostics.ObjectFormat = function() {
  /** @type {?Array<!proto.speech.tts.ChunkInfo.ObjectFormat>|undefined} */
  this.chunksList;

  /** @type {?number|undefined} */
  this.overlapS;

  /** @type {?number|undefined} */
  this.sampleRate;

  /** @type {?proto.speech.tts.StitchingOptions.ObjectFormat|undefined} */
  this.stitchingOptions;

  /** @type {?proto.speech.tts.WaveNetModelOptions.ObjectFormat|undefined} */
  this.modelOptions;

  /** @type {?proto.speech.tts.WaveNetSynthesisOptions.ObjectFormat|undefined} */
  this.synthesisOptions;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.WavenetDiagnostics.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.WavenetDiagnostics}
 */
proto.speech.tts.WavenetDiagnostics.fromObject = function(obj) {
  var msg = new proto.speech.tts.WavenetDiagnostics();
  obj.chunksList && jspb.Message.setRepeatedWrapperField(
      msg, 1, obj.chunksList.map(
          proto.speech.tts.ChunkInfo.fromObject));
  obj.overlapS != null && jspb.Message.setField(msg, 2, obj.overlapS);
  obj.sampleRate != null && jspb.Message.setField(msg, 3, obj.sampleRate);
  obj.stitchingOptions && jspb.Message.setWrapperField(
      msg, 4, proto.speech.tts.StitchingOptions.fromObject(obj.stitchingOptions));
  obj.modelOptions && jspb.Message.setWrapperField(
      msg, 5, proto.speech.tts.WaveNetModelOptions.fromObject(obj.modelOptions));
  obj.synthesisOptions && jspb.Message.setWrapperField(
      msg, 6, proto.speech.tts.WaveNetSynthesisOptions.fromObject(obj.synthesisOptions));
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.WavenetDiagnostics}
 */
proto.speech.tts.WavenetDiagnostics.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.WavenetDiagnostics;
  return proto.speech.tts.WavenetDiagnostics.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.WavenetDiagnostics} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.WavenetDiagnostics}
 */
proto.speech.tts.WavenetDiagnostics.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.speech.tts.ChunkInfo;
      reader.readMessage(value,proto.speech.tts.ChunkInfo.deserializeBinaryFromReader);
      msg.addChunks(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setOverlapS(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setSampleRate(value);
      break;
    case 4:
      var value = new proto.speech.tts.StitchingOptions;
      reader.readMessage(value,proto.speech.tts.StitchingOptions.deserializeBinaryFromReader);
      msg.setStitchingOptions(value);
      break;
    case 5:
      var value = new proto.speech.tts.WaveNetModelOptions;
      reader.readMessage(value,proto.speech.tts.WaveNetModelOptions.deserializeBinaryFromReader);
      msg.setModelOptions(value);
      break;
    case 6:
      var value = new proto.speech.tts.WaveNetSynthesisOptions;
      reader.readMessage(value,proto.speech.tts.WaveNetSynthesisOptions.deserializeBinaryFromReader);
      msg.setSynthesisOptions(value);
      break;
    default:
      jspb.Message.readBinaryExtension(msg, reader,
        proto.speech.tts.WavenetDiagnostics.extensionsBinary,
        proto.speech.tts.WavenetDiagnostics.prototype.getExtension,
        proto.speech.tts.WavenetDiagnostics.prototype.setExtension);
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.WavenetDiagnostics.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.WavenetDiagnostics.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.WavenetDiagnostics} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.WavenetDiagnostics.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getChunksList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      1,
      f,
      proto.speech.tts.ChunkInfo.serializeBinaryToWriter
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeFloat(
      2,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeInt32(
      3,
      f
    );
  }
  f = message.getStitchingOptions();
  if (f != null) {
    writer.writeMessage(
      4,
      f,
      proto.speech.tts.StitchingOptions.serializeBinaryToWriter
    );
  }
  f = message.getModelOptions();
  if (f != null) {
    writer.writeMessage(
      5,
      f,
      proto.speech.tts.WaveNetModelOptions.serializeBinaryToWriter
    );
  }
  f = message.getSynthesisOptions();
  if (f != null) {
    writer.writeMessage(
      6,
      f,
      proto.speech.tts.WaveNetSynthesisOptions.serializeBinaryToWriter
    );
  }
  jspb.Message.serializeBinaryExtensions(message, writer,
    proto.speech.tts.WavenetDiagnostics.extensionsBinary, proto.speech.tts.WavenetDiagnostics.prototype.getExtension);
};


/**
 * repeated ChunkInfo chunks = 1;
 * @return {!Array<!proto.speech.tts.ChunkInfo>}
 */
proto.speech.tts.WavenetDiagnostics.prototype.getChunksList = function() {
  return /** @type{!Array<!proto.speech.tts.ChunkInfo>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.ChunkInfo, 1));
};


/**
 * @param {Array<!proto.speech.tts.ChunkInfo>|undefined} value
 * @return {!proto.speech.tts.WavenetDiagnostics} returns this
*/
proto.speech.tts.WavenetDiagnostics.prototype.setChunksList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 1, value);
};


/**
 * @param {!proto.speech.tts.ChunkInfo=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.ChunkInfo}
 */
proto.speech.tts.WavenetDiagnostics.prototype.addChunks = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.speech.tts.ChunkInfo, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.WavenetDiagnostics} returns this
 */
proto.speech.tts.WavenetDiagnostics.prototype.clearChunksList = function() {
  return this.setChunksList([]);
};


/**
 * optional float overlap_s = 2;
 * @return {?number}
 */
proto.speech.tts.WavenetDiagnostics.prototype.getOverlapS = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 2));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.WavenetDiagnostics} returns this
 */
proto.speech.tts.WavenetDiagnostics.prototype.setOverlapS = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.WavenetDiagnostics} returns this
 */
proto.speech.tts.WavenetDiagnostics.prototype.clearOverlapS = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.WavenetDiagnostics.prototype.hasOverlapS = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional int32 sample_rate = 3;
 * @return {?number}
 */
proto.speech.tts.WavenetDiagnostics.prototype.getSampleRate = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.WavenetDiagnostics} returns this
 */
proto.speech.tts.WavenetDiagnostics.prototype.setSampleRate = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.WavenetDiagnostics} returns this
 */
proto.speech.tts.WavenetDiagnostics.prototype.clearSampleRate = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.WavenetDiagnostics.prototype.hasSampleRate = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional StitchingOptions stitching_options = 4;
 * @return {?proto.speech.tts.StitchingOptions}
 */
proto.speech.tts.WavenetDiagnostics.prototype.getStitchingOptions = function() {
  return /** @type{?proto.speech.tts.StitchingOptions} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.StitchingOptions, 4));
};


/**
 * @param {?proto.speech.tts.StitchingOptions|undefined} value
 * @return {!proto.speech.tts.WavenetDiagnostics} returns this
*/
proto.speech.tts.WavenetDiagnostics.prototype.setStitchingOptions = function(value) {
  return jspb.Message.setWrapperField(this, 4, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.WavenetDiagnostics} returns this
 */
proto.speech.tts.WavenetDiagnostics.prototype.clearStitchingOptions = function() {
  return this.setStitchingOptions(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.WavenetDiagnostics.prototype.hasStitchingOptions = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional WaveNetModelOptions model_options = 5;
 * @return {?proto.speech.tts.WaveNetModelOptions}
 */
proto.speech.tts.WavenetDiagnostics.prototype.getModelOptions = function() {
  return /** @type{?proto.speech.tts.WaveNetModelOptions} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.WaveNetModelOptions, 5));
};


/**
 * @param {?proto.speech.tts.WaveNetModelOptions|undefined} value
 * @return {!proto.speech.tts.WavenetDiagnostics} returns this
*/
proto.speech.tts.WavenetDiagnostics.prototype.setModelOptions = function(value) {
  return jspb.Message.setWrapperField(this, 5, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.WavenetDiagnostics} returns this
 */
proto.speech.tts.WavenetDiagnostics.prototype.clearModelOptions = function() {
  return this.setModelOptions(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.WavenetDiagnostics.prototype.hasModelOptions = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional WaveNetSynthesisOptions synthesis_options = 6;
 * @return {?proto.speech.tts.WaveNetSynthesisOptions}
 */
proto.speech.tts.WavenetDiagnostics.prototype.getSynthesisOptions = function() {
  return /** @type{?proto.speech.tts.WaveNetSynthesisOptions} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.WaveNetSynthesisOptions, 6));
};


/**
 * @param {?proto.speech.tts.WaveNetSynthesisOptions|undefined} value
 * @return {!proto.speech.tts.WavenetDiagnostics} returns this
*/
proto.speech.tts.WavenetDiagnostics.prototype.setSynthesisOptions = function(value) {
  return jspb.Message.setWrapperField(this, 6, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.WavenetDiagnostics} returns this
 */
proto.speech.tts.WavenetDiagnostics.prototype.clearSynthesisOptions = function() {
  return this.setSynthesisOptions(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.WavenetDiagnostics.prototype.hasSynthesisOptions = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.WavenetDiagnostics}
 */
proto.speech.tts.WavenetDiagnostics.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.WavenetDiagnostics, data);
};



//speech/tts/engine/utterance/diagnostics_proto_jspb/diagnostics.jspb.js
// source: speech/tts/engine/utterance/diagnostics.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.Diagnostics');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.PerformanceInfo');
goog.require('proto.speech.tts.SourceDiagnostics');
goog.require('proto.speech.tts.TextnormDiagnostics');
goog.require('proto.speech.tts.UnitSelectionDiagnostics');
goog.require('proto.speech.tts.WavenetDiagnostics');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.Diagnostics = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.Diagnostics, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.Diagnostics.displayName = 'proto.speech.tts.Diagnostics';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.Diagnostics.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.Diagnostics.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.Diagnostics} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Diagnostics.toObject = function(includeInstance, msg) {
  var f, obj = {
    unitSelection: (f = msg.getUnitSelection()) && proto.speech.tts.UnitSelectionDiagnostics.toObject(includeInstance, f),
    performanceInfo: (f = msg.getPerformanceInfo()) && proto.speech.tts.PerformanceInfo.toObject(includeInstance, f),
    textnorm: (f = msg.getTextnorm()) && proto.speech.tts.TextnormDiagnostics.toObject(includeInstance, f),
    source: (f = msg.getSource()) && proto.speech.tts.SourceDiagnostics.toObject(includeInstance, f),
    wavenet: (f = msg.getWavenet()) && proto.speech.tts.WavenetDiagnostics.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Diagnostics as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.Diagnostics.ObjectFormat = function() {
  /** @type {?proto.speech.tts.UnitSelectionDiagnostics.ObjectFormat|undefined} */
  this.unitSelection;

  /** @type {?proto.speech.tts.PerformanceInfo.ObjectFormat|undefined} */
  this.performanceInfo;

  /** @type {?proto.speech.tts.TextnormDiagnostics.ObjectFormat|undefined} */
  this.textnorm;

  /** @type {?proto.speech.tts.SourceDiagnostics.ObjectFormat|undefined} */
  this.source;

  /** @type {?proto.speech.tts.WavenetDiagnostics.ObjectFormat|undefined} */
  this.wavenet;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.Diagnostics.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.Diagnostics}
 */
proto.speech.tts.Diagnostics.fromObject = function(obj) {
  var msg = new proto.speech.tts.Diagnostics();
  obj.unitSelection && jspb.Message.setWrapperField(
      msg, 1, proto.speech.tts.UnitSelectionDiagnostics.fromObject(obj.unitSelection));
  obj.performanceInfo && jspb.Message.setWrapperField(
      msg, 2, proto.speech.tts.PerformanceInfo.fromObject(obj.performanceInfo));
  obj.textnorm && jspb.Message.setWrapperField(
      msg, 3, proto.speech.tts.TextnormDiagnostics.fromObject(obj.textnorm));
  obj.source && jspb.Message.setWrapperField(
      msg, 4, proto.speech.tts.SourceDiagnostics.fromObject(obj.source));
  obj.wavenet && jspb.Message.setWrapperField(
      msg, 5, proto.speech.tts.WavenetDiagnostics.fromObject(obj.wavenet));
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.Diagnostics}
 */
proto.speech.tts.Diagnostics.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.Diagnostics;
  return proto.speech.tts.Diagnostics.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.Diagnostics} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.Diagnostics}
 */
proto.speech.tts.Diagnostics.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.speech.tts.UnitSelectionDiagnostics;
      reader.readMessage(value,proto.speech.tts.UnitSelectionDiagnostics.deserializeBinaryFromReader);
      msg.setUnitSelection(value);
      break;
    case 2:
      var value = new proto.speech.tts.PerformanceInfo;
      reader.readMessage(value,proto.speech.tts.PerformanceInfo.deserializeBinaryFromReader);
      msg.setPerformanceInfo(value);
      break;
    case 3:
      var value = new proto.speech.tts.TextnormDiagnostics;
      reader.readMessage(value,proto.speech.tts.TextnormDiagnostics.deserializeBinaryFromReader);
      msg.setTextnorm(value);
      break;
    case 4:
      var value = new proto.speech.tts.SourceDiagnostics;
      reader.readMessage(value,proto.speech.tts.SourceDiagnostics.deserializeBinaryFromReader);
      msg.setSource(value);
      break;
    case 5:
      var value = new proto.speech.tts.WavenetDiagnostics;
      reader.readMessage(value,proto.speech.tts.WavenetDiagnostics.deserializeBinaryFromReader);
      msg.setWavenet(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.Diagnostics.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.Diagnostics.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.Diagnostics} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Diagnostics.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getUnitSelection();
  if (f != null) {
    writer.writeMessage(
      1,
      f,
      proto.speech.tts.UnitSelectionDiagnostics.serializeBinaryToWriter
    );
  }
  f = message.getPerformanceInfo();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.speech.tts.PerformanceInfo.serializeBinaryToWriter
    );
  }
  f = message.getTextnorm();
  if (f != null) {
    writer.writeMessage(
      3,
      f,
      proto.speech.tts.TextnormDiagnostics.serializeBinaryToWriter
    );
  }
  f = message.getSource();
  if (f != null) {
    writer.writeMessage(
      4,
      f,
      proto.speech.tts.SourceDiagnostics.serializeBinaryToWriter
    );
  }
  f = message.getWavenet();
  if (f != null) {
    writer.writeMessage(
      5,
      f,
      proto.speech.tts.WavenetDiagnostics.serializeBinaryToWriter
    );
  }
};


/**
 * optional UnitSelectionDiagnostics unit_selection = 1;
 * @return {?proto.speech.tts.UnitSelectionDiagnostics}
 */
proto.speech.tts.Diagnostics.prototype.getUnitSelection = function() {
  return /** @type{?proto.speech.tts.UnitSelectionDiagnostics} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.UnitSelectionDiagnostics, 1));
};


/**
 * @param {?proto.speech.tts.UnitSelectionDiagnostics|undefined} value
 * @return {!proto.speech.tts.Diagnostics} returns this
*/
proto.speech.tts.Diagnostics.prototype.setUnitSelection = function(value) {
  return jspb.Message.setWrapperField(this, 1, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Diagnostics} returns this
 */
proto.speech.tts.Diagnostics.prototype.clearUnitSelection = function() {
  return this.setUnitSelection(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Diagnostics.prototype.hasUnitSelection = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional PerformanceInfo performance_info = 2;
 * @return {?proto.speech.tts.PerformanceInfo}
 */
proto.speech.tts.Diagnostics.prototype.getPerformanceInfo = function() {
  return /** @type{?proto.speech.tts.PerformanceInfo} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.PerformanceInfo, 2));
};


/**
 * @param {?proto.speech.tts.PerformanceInfo|undefined} value
 * @return {!proto.speech.tts.Diagnostics} returns this
*/
proto.speech.tts.Diagnostics.prototype.setPerformanceInfo = function(value) {
  return jspb.Message.setWrapperField(this, 2, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Diagnostics} returns this
 */
proto.speech.tts.Diagnostics.prototype.clearPerformanceInfo = function() {
  return this.setPerformanceInfo(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Diagnostics.prototype.hasPerformanceInfo = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional TextnormDiagnostics textnorm = 3;
 * @return {?proto.speech.tts.TextnormDiagnostics}
 */
proto.speech.tts.Diagnostics.prototype.getTextnorm = function() {
  return /** @type{?proto.speech.tts.TextnormDiagnostics} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.TextnormDiagnostics, 3));
};


/**
 * @param {?proto.speech.tts.TextnormDiagnostics|undefined} value
 * @return {!proto.speech.tts.Diagnostics} returns this
*/
proto.speech.tts.Diagnostics.prototype.setTextnorm = function(value) {
  return jspb.Message.setWrapperField(this, 3, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Diagnostics} returns this
 */
proto.speech.tts.Diagnostics.prototype.clearTextnorm = function() {
  return this.setTextnorm(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Diagnostics.prototype.hasTextnorm = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional SourceDiagnostics source = 4;
 * @return {?proto.speech.tts.SourceDiagnostics}
 */
proto.speech.tts.Diagnostics.prototype.getSource = function() {
  return /** @type{?proto.speech.tts.SourceDiagnostics} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.SourceDiagnostics, 4));
};


/**
 * @param {?proto.speech.tts.SourceDiagnostics|undefined} value
 * @return {!proto.speech.tts.Diagnostics} returns this
*/
proto.speech.tts.Diagnostics.prototype.setSource = function(value) {
  return jspb.Message.setWrapperField(this, 4, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Diagnostics} returns this
 */
proto.speech.tts.Diagnostics.prototype.clearSource = function() {
  return this.setSource(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Diagnostics.prototype.hasSource = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional WavenetDiagnostics wavenet = 5;
 * @return {?proto.speech.tts.WavenetDiagnostics}
 */
proto.speech.tts.Diagnostics.prototype.getWavenet = function() {
  return /** @type{?proto.speech.tts.WavenetDiagnostics} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.WavenetDiagnostics, 5));
};


/**
 * @param {?proto.speech.tts.WavenetDiagnostics|undefined} value
 * @return {!proto.speech.tts.Diagnostics} returns this
*/
proto.speech.tts.Diagnostics.prototype.setWavenet = function(value) {
  return jspb.Message.setWrapperField(this, 5, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Diagnostics} returns this
 */
proto.speech.tts.Diagnostics.prototype.clearWavenet = function() {
  return this.setWavenet(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Diagnostics.prototype.hasWavenet = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.Diagnostics}
 */
proto.speech.tts.Diagnostics.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.Diagnostics, data);
};



//speech/tts/engine/utterance/external_utterance_vectors_proto_jspb/externalutterancevectors.jspb.js
// source: speech/tts/engine/utterance/external_utterance_vectors.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.ExternalUtteranceVectors');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.FloatVector');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.ExternalUtteranceVectors = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.ExternalUtteranceVectors, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.ExternalUtteranceVectors.displayName = 'proto.speech.tts.ExternalUtteranceVectors';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.ExternalUtteranceVectors.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.ExternalUtteranceVectors.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.ExternalUtteranceVectors} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.ExternalUtteranceVectors.toObject = function(includeInstance, msg) {
  var f, obj = {
    neoUtteranceEmbedding: (f = msg.getNeoUtteranceEmbedding()) && proto.speech.tts.FloatVector.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of ExternalUtteranceVectors as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.ExternalUtteranceVectors.ObjectFormat = function() {
  /** @type {?proto.speech.tts.FloatVector.ObjectFormat|undefined} */
  this.neoUtteranceEmbedding;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.ExternalUtteranceVectors.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.ExternalUtteranceVectors}
 */
proto.speech.tts.ExternalUtteranceVectors.fromObject = function(obj) {
  var msg = new proto.speech.tts.ExternalUtteranceVectors();
  obj.neoUtteranceEmbedding && jspb.Message.setWrapperField(
      msg, 1, proto.speech.tts.FloatVector.fromObject(obj.neoUtteranceEmbedding));
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.ExternalUtteranceVectors}
 */
proto.speech.tts.ExternalUtteranceVectors.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.ExternalUtteranceVectors;
  return proto.speech.tts.ExternalUtteranceVectors.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.ExternalUtteranceVectors} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.ExternalUtteranceVectors}
 */
proto.speech.tts.ExternalUtteranceVectors.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.speech.tts.FloatVector;
      reader.readMessage(value,proto.speech.tts.FloatVector.deserializeBinaryFromReader);
      msg.setNeoUtteranceEmbedding(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.ExternalUtteranceVectors.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.ExternalUtteranceVectors.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.ExternalUtteranceVectors} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.ExternalUtteranceVectors.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getNeoUtteranceEmbedding();
  if (f != null) {
    writer.writeMessage(
      1,
      f,
      proto.speech.tts.FloatVector.serializeBinaryToWriter
    );
  }
};


/**
 * optional FloatVector neo_utterance_embedding = 1;
 * @return {?proto.speech.tts.FloatVector}
 */
proto.speech.tts.ExternalUtteranceVectors.prototype.getNeoUtteranceEmbedding = function() {
  return /** @type{?proto.speech.tts.FloatVector} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.FloatVector, 1));
};


/**
 * @param {?proto.speech.tts.FloatVector|undefined} value
 * @return {!proto.speech.tts.ExternalUtteranceVectors} returns this
*/
proto.speech.tts.ExternalUtteranceVectors.prototype.setNeoUtteranceEmbedding = function(value) {
  return jspb.Message.setWrapperField(this, 1, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.ExternalUtteranceVectors} returns this
 */
proto.speech.tts.ExternalUtteranceVectors.prototype.clearNeoUtteranceEmbedding = function() {
  return this.setNeoUtteranceEmbedding(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.ExternalUtteranceVectors.prototype.hasNeoUtteranceEmbedding = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.ExternalUtteranceVectors}
 */
proto.speech.tts.ExternalUtteranceVectors.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.ExternalUtteranceVectors, data);
};



//speech/tts/engine/utterance/acoustics_proto_jspb/acousticstreaminfo.jspb.js
// source: speech/tts/engine/utterance/acoustics.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.AcousticStreamInfo');
goog.provide('proto.speech.tts.AcousticStreamInfo.StreamType');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.AcousticStreamInfo = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.AcousticStreamInfo, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.AcousticStreamInfo.displayName = 'proto.speech.tts.AcousticStreamInfo';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.AcousticStreamInfo.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.AcousticStreamInfo.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.AcousticStreamInfo} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.AcousticStreamInfo.toObject = function(includeInstance, msg) {
  var f, obj = {
    type: jspb.Message.getFieldWithDefault(msg, 1, 0),
    size: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of AcousticStreamInfo as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.AcousticStreamInfo.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.type;

  /** @type {?number|undefined} */
  this.size;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.AcousticStreamInfo.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.AcousticStreamInfo}
 */
proto.speech.tts.AcousticStreamInfo.fromObject = function(obj) {
  var msg = new proto.speech.tts.AcousticStreamInfo();
  obj.type != null && jspb.Message.setField(msg, 1, obj.type);
  obj.size != null && jspb.Message.setField(msg, 2, obj.size);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.AcousticStreamInfo}
 */
proto.speech.tts.AcousticStreamInfo.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.AcousticStreamInfo;
  return proto.speech.tts.AcousticStreamInfo.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.AcousticStreamInfo} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.AcousticStreamInfo}
 */
proto.speech.tts.AcousticStreamInfo.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {!proto.speech.tts.AcousticStreamInfo.StreamType} */ (reader.readEnum());
      msg.setType(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setSize(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.AcousticStreamInfo.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.AcousticStreamInfo.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.AcousticStreamInfo} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.AcousticStreamInfo.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {!proto.speech.tts.AcousticStreamInfo.StreamType} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeEnum(
      1,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeInt32(
      2,
      f
    );
  }
};


/**
 * @enum {number}
 */
proto.speech.tts.AcousticStreamInfo.StreamType = {
  UNSPECIFIED: 0,
  DURATION: 1,
  LOG_F0: 2,
  SPECTRA: 3,
  APERIODICITY: 4,
  PHASE: 5,
  VOICING: 6,
  LOG_F0_SPECTRA_APERIODICITY: 7,
  DURATION_LOG_F0_SPECTRA_APERIODICITY: 8,
  SPECTRA_APERIODICITY: 9,
  LOG_F0_VOICING: 10
};

/**
 * optional StreamType type = 1;
 * @return {!proto.speech.tts.AcousticStreamInfo.StreamType}
 */
proto.speech.tts.AcousticStreamInfo.prototype.getType = function() {
  return /** @type {!proto.speech.tts.AcousticStreamInfo.StreamType} */ (jspb.Message.getFieldWithDefault(this, 1, 0));
};


/**
 * @param {?proto.speech.tts.AcousticStreamInfo.StreamType|undefined} value
 * @return {!proto.speech.tts.AcousticStreamInfo} returns this
 */
proto.speech.tts.AcousticStreamInfo.prototype.setType = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.AcousticStreamInfo} returns this
 */
proto.speech.tts.AcousticStreamInfo.prototype.clearType = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.AcousticStreamInfo.prototype.hasType = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional int32 size = 2;
 * @return {?number}
 */
proto.speech.tts.AcousticStreamInfo.prototype.getSize = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.AcousticStreamInfo} returns this
 */
proto.speech.tts.AcousticStreamInfo.prototype.setSize = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.AcousticStreamInfo} returns this
 */
proto.speech.tts.AcousticStreamInfo.prototype.clearSize = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.AcousticStreamInfo.prototype.hasSize = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.AcousticStreamInfo}
 */
proto.speech.tts.AcousticStreamInfo.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.AcousticStreamInfo, data);
};



//speech/tts/engine/utterance/features_proto_jspb/featuredata.jspb.js
// source: speech/tts/engine/utterance/features.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.FeatureData');
goog.provide('proto.speech.tts.FeatureData.ExtraFeatureData');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.FloatVector');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.FeatureData = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.FeatureData.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.FeatureData, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.FeatureData.displayName = 'proto.speech.tts.FeatureData';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.FeatureData.ExtraFeatureData = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.FeatureData.ExtraFeatureData.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.FeatureData.ExtraFeatureData, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.FeatureData.ExtraFeatureData.displayName = 'proto.speech.tts.FeatureData.ExtraFeatureData';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.FeatureData.repeatedFields_ = [1,2,3,4,5];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.FeatureData.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.FeatureData.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.FeatureData} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.FeatureData.toObject = function(includeInstance, msg) {
  var f, obj = {
    boolFeaturesList: (f = jspb.Message.getRepeatedBooleanField(msg, 1)) == null ? undefined : f,
    intFeaturesList: (f = jspb.Message.getRepeatedField(msg, 2)) == null ? undefined : f,
    floatFeaturesList: (f = jspb.Message.getRepeatedFloatingPointField(msg, 3)) == null ? undefined : f,
    stringFeaturesList: (f = jspb.Message.getRepeatedField(msg, 4)) == null ? undefined : f,
    floatVectorFeaturesList: jspb.Message.toObjectList(msg.getFloatVectorFeaturesList(),
    proto.speech.tts.FloatVector.toObject, includeInstance),
    extraFeatures: (f = msg.getExtraFeatures()) && proto.speech.tts.FeatureData.ExtraFeatureData.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of FeatureData as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.FeatureData.ObjectFormat = function() {
  /** @type {?Array<boolean>|undefined} */
  this.boolFeaturesList;

  /** @type {?Array<number>|undefined} */
  this.intFeaturesList;

  /** @type {?Array<number>|undefined} */
  this.floatFeaturesList;

  /** @type {?Array<string>|undefined} */
  this.stringFeaturesList;

  /** @type {?Array<!proto.speech.tts.FloatVector.ObjectFormat>|undefined} */
  this.floatVectorFeaturesList;

  /** @type {?proto.speech.tts.FeatureData.ExtraFeatureData.ObjectFormat|undefined} */
  this.extraFeatures;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.FeatureData.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.FeatureData}
 */
proto.speech.tts.FeatureData.fromObject = function(obj) {
  var msg = new proto.speech.tts.FeatureData();
  obj.boolFeaturesList != null && jspb.Message.setField(msg, 1, obj.boolFeaturesList);
  obj.intFeaturesList != null && jspb.Message.setField(msg, 2, obj.intFeaturesList);
  obj.floatFeaturesList != null && jspb.Message.setField(msg, 3, obj.floatFeaturesList);
  obj.stringFeaturesList != null && jspb.Message.setField(msg, 4, obj.stringFeaturesList);
  obj.floatVectorFeaturesList && jspb.Message.setRepeatedWrapperField(
      msg, 5, obj.floatVectorFeaturesList.map(
          proto.speech.tts.FloatVector.fromObject));
  obj.extraFeatures && jspb.Message.setWrapperField(
      msg, 6, proto.speech.tts.FeatureData.ExtraFeatureData.fromObject(obj.extraFeatures));
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.FeatureData}
 */
proto.speech.tts.FeatureData.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.FeatureData;
  return proto.speech.tts.FeatureData.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.FeatureData} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.FeatureData}
 */
proto.speech.tts.FeatureData.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {!Array<boolean>} */ (reader.readPackedBool());
      msg.setBoolFeaturesList(value);
      break;
    case 2:
      var value = /** @type {!Array<number>} */ (reader.readPackedInt32());
      msg.setIntFeaturesList(value);
      break;
    case 3:
      var value = /** @type {!Array<number>} */ (reader.readPackedFloat());
      msg.setFloatFeaturesList(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.addStringFeatures(value);
      break;
    case 5:
      var value = new proto.speech.tts.FloatVector;
      reader.readMessage(value,proto.speech.tts.FloatVector.deserializeBinaryFromReader);
      msg.addFloatVectorFeatures(value);
      break;
    case 6:
      var value = new proto.speech.tts.FeatureData.ExtraFeatureData;
      reader.readMessage(value,proto.speech.tts.FeatureData.ExtraFeatureData.deserializeBinaryFromReader);
      msg.setExtraFeatures(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.FeatureData.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.FeatureData.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.FeatureData} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.FeatureData.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getBoolFeaturesList();
  if (f.length > 0) {
    writer.writePackedBool(
      1,
      f
    );
  }
  f = message.getIntFeaturesList();
  if (f.length > 0) {
    writer.writePackedInt32(
      2,
      f
    );
  }
  f = message.getFloatFeaturesList();
  if (f.length > 0) {
    writer.writePackedFloat(
      3,
      f
    );
  }
  f = message.getStringFeaturesList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      4,
      f
    );
  }
  f = message.getFloatVectorFeaturesList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      5,
      f,
      proto.speech.tts.FloatVector.serializeBinaryToWriter
    );
  }
  f = message.getExtraFeatures();
  if (f != null) {
    writer.writeMessage(
      6,
      f,
      proto.speech.tts.FeatureData.ExtraFeatureData.serializeBinaryToWriter
    );
  }
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.FeatureData.ExtraFeatureData.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.FeatureData.ExtraFeatureData.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.FeatureData.ExtraFeatureData.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.FeatureData.ExtraFeatureData} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.FeatureData.ExtraFeatureData.toObject = function(includeInstance, msg) {
  var f, obj = {
    durationFeaturesList: (f = jspb.Message.getRepeatedFloatingPointField(msg, 1)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of ExtraFeatureData as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.FeatureData.ExtraFeatureData.ObjectFormat = function() {
  /** @type {?Array<number>|undefined} */
  this.durationFeaturesList;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.FeatureData.ExtraFeatureData.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.FeatureData.ExtraFeatureData}
 */
proto.speech.tts.FeatureData.ExtraFeatureData.fromObject = function(obj) {
  var msg = new proto.speech.tts.FeatureData.ExtraFeatureData();
  obj.durationFeaturesList != null && jspb.Message.setField(msg, 1, obj.durationFeaturesList);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.FeatureData.ExtraFeatureData}
 */
proto.speech.tts.FeatureData.ExtraFeatureData.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.FeatureData.ExtraFeatureData;
  return proto.speech.tts.FeatureData.ExtraFeatureData.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.FeatureData.ExtraFeatureData} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.FeatureData.ExtraFeatureData}
 */
proto.speech.tts.FeatureData.ExtraFeatureData.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {!Array<number>} */ (reader.readPackedFloat());
      msg.setDurationFeaturesList(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.FeatureData.ExtraFeatureData.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.FeatureData.ExtraFeatureData.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.FeatureData.ExtraFeatureData} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.FeatureData.ExtraFeatureData.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getDurationFeaturesList();
  if (f.length > 0) {
    writer.writePackedFloat(
      1,
      f
    );
  }
};


/**
 * repeated float duration_features = 1;
 * @return {!Array<number>}
 */
proto.speech.tts.FeatureData.ExtraFeatureData.prototype.getDurationFeaturesList = function() {
  return /** @type {!Array<number>} */ (jspb.Message.getRepeatedFloatingPointField(this, 1));
};


/**
 * @param {Array<number>|undefined} value
 * @return {!proto.speech.tts.FeatureData.ExtraFeatureData} returns this
 */
proto.speech.tts.FeatureData.ExtraFeatureData.prototype.setDurationFeaturesList = function(value) {
  return jspb.Message.setField(this, 1, value || []);
};


/**
 * @param {number} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.FeatureData.ExtraFeatureData} returns this
 */
proto.speech.tts.FeatureData.ExtraFeatureData.prototype.addDurationFeatures = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 1, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.FeatureData.ExtraFeatureData} returns this
 */
proto.speech.tts.FeatureData.ExtraFeatureData.prototype.clearDurationFeaturesList = function() {
  return this.setDurationFeaturesList([]);
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.FeatureData.ExtraFeatureData}
 */
proto.speech.tts.FeatureData.ExtraFeatureData.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.FeatureData.ExtraFeatureData, data);
};


/**
 * repeated bool bool_features = 1;
 * @return {!Array<boolean>}
 */
proto.speech.tts.FeatureData.prototype.getBoolFeaturesList = function() {
  return /** @type {!Array<boolean>} */ (jspb.Message.getRepeatedBooleanField(this, 1));
};


/**
 * @param {Array<boolean>|undefined} value
 * @return {!proto.speech.tts.FeatureData} returns this
 */
proto.speech.tts.FeatureData.prototype.setBoolFeaturesList = function(value) {
  return jspb.Message.setField(this, 1, value || []);
};


/**
 * @param {boolean} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.FeatureData} returns this
 */
proto.speech.tts.FeatureData.prototype.addBoolFeatures = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 1, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.FeatureData} returns this
 */
proto.speech.tts.FeatureData.prototype.clearBoolFeaturesList = function() {
  return this.setBoolFeaturesList([]);
};


/**
 * repeated int32 int_features = 2;
 * @return {!Array<number>}
 */
proto.speech.tts.FeatureData.prototype.getIntFeaturesList = function() {
  return /** @type {!Array<number>} */ (jspb.Message.getRepeatedField(this, 2));
};


/**
 * @param {Array<number>|undefined} value
 * @return {!proto.speech.tts.FeatureData} returns this
 */
proto.speech.tts.FeatureData.prototype.setIntFeaturesList = function(value) {
  return jspb.Message.setField(this, 2, value || []);
};


/**
 * @param {number} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.FeatureData} returns this
 */
proto.speech.tts.FeatureData.prototype.addIntFeatures = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 2, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.FeatureData} returns this
 */
proto.speech.tts.FeatureData.prototype.clearIntFeaturesList = function() {
  return this.setIntFeaturesList([]);
};


/**
 * repeated float float_features = 3;
 * @return {!Array<number>}
 */
proto.speech.tts.FeatureData.prototype.getFloatFeaturesList = function() {
  return /** @type {!Array<number>} */ (jspb.Message.getRepeatedFloatingPointField(this, 3));
};


/**
 * @param {Array<number>|undefined} value
 * @return {!proto.speech.tts.FeatureData} returns this
 */
proto.speech.tts.FeatureData.prototype.setFloatFeaturesList = function(value) {
  return jspb.Message.setField(this, 3, value || []);
};


/**
 * @param {number} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.FeatureData} returns this
 */
proto.speech.tts.FeatureData.prototype.addFloatFeatures = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 3, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.FeatureData} returns this
 */
proto.speech.tts.FeatureData.prototype.clearFloatFeaturesList = function() {
  return this.setFloatFeaturesList([]);
};


/**
 * repeated string string_features = 4;
 * @return {!Array<string>}
 */
proto.speech.tts.FeatureData.prototype.getStringFeaturesList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 4));
};


/**
 * @param {Array<string>|undefined} value
 * @return {!proto.speech.tts.FeatureData} returns this
 */
proto.speech.tts.FeatureData.prototype.setStringFeaturesList = function(value) {
  return jspb.Message.setField(this, 4, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.FeatureData} returns this
 */
proto.speech.tts.FeatureData.prototype.addStringFeatures = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 4, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.FeatureData} returns this
 */
proto.speech.tts.FeatureData.prototype.clearStringFeaturesList = function() {
  return this.setStringFeaturesList([]);
};


/**
 * repeated FloatVector float_vector_features = 5;
 * @return {!Array<!proto.speech.tts.FloatVector>}
 */
proto.speech.tts.FeatureData.prototype.getFloatVectorFeaturesList = function() {
  return /** @type{!Array<!proto.speech.tts.FloatVector>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.FloatVector, 5));
};


/**
 * @param {Array<!proto.speech.tts.FloatVector>|undefined} value
 * @return {!proto.speech.tts.FeatureData} returns this
*/
proto.speech.tts.FeatureData.prototype.setFloatVectorFeaturesList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 5, value);
};


/**
 * @param {!proto.speech.tts.FloatVector=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.FloatVector}
 */
proto.speech.tts.FeatureData.prototype.addFloatVectorFeatures = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 5, opt_value, proto.speech.tts.FloatVector, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.FeatureData} returns this
 */
proto.speech.tts.FeatureData.prototype.clearFloatVectorFeaturesList = function() {
  return this.setFloatVectorFeaturesList([]);
};


/**
 * optional ExtraFeatureData extra_features = 6;
 * @return {?proto.speech.tts.FeatureData.ExtraFeatureData}
 */
proto.speech.tts.FeatureData.prototype.getExtraFeatures = function() {
  return /** @type{?proto.speech.tts.FeatureData.ExtraFeatureData} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.FeatureData.ExtraFeatureData, 6));
};


/**
 * @param {?proto.speech.tts.FeatureData.ExtraFeatureData|undefined} value
 * @return {!proto.speech.tts.FeatureData} returns this
*/
proto.speech.tts.FeatureData.prototype.setExtraFeatures = function(value) {
  return jspb.Message.setWrapperField(this, 6, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.FeatureData} returns this
 */
proto.speech.tts.FeatureData.prototype.clearExtraFeatures = function() {
  return this.setExtraFeatures(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.FeatureData.prototype.hasExtraFeatures = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.FeatureData}
 */
proto.speech.tts.FeatureData.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.FeatureData, data);
};



//speech/tts/engine/utterance/timing_info_proto_jspb/timinginfo.jspb.js
// source: speech/tts/engine/utterance/timing_info.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.TimingInfo');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.TimingInfo = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.TimingInfo, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.TimingInfo.displayName = 'proto.speech.tts.TimingInfo';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.TimingInfo.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.TimingInfo.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.TimingInfo} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.TimingInfo.toObject = function(includeInstance, msg) {
  var f, obj = {
    startSeconds: (f = jspb.Message.getOptionalFloatingPointField(msg, 1)) == null ? undefined : f,
    midSeconds: (f = jspb.Message.getOptionalFloatingPointField(msg, 2)) == null ? undefined : f,
    endSeconds: (f = jspb.Message.getOptionalFloatingPointField(msg, 3)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of TimingInfo as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.TimingInfo.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.startSeconds;

  /** @type {?number|undefined} */
  this.midSeconds;

  /** @type {?number|undefined} */
  this.endSeconds;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.TimingInfo.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.TimingInfo}
 */
proto.speech.tts.TimingInfo.fromObject = function(obj) {
  var msg = new proto.speech.tts.TimingInfo();
  obj.startSeconds != null && jspb.Message.setField(msg, 1, obj.startSeconds);
  obj.midSeconds != null && jspb.Message.setField(msg, 2, obj.midSeconds);
  obj.endSeconds != null && jspb.Message.setField(msg, 3, obj.endSeconds);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.TimingInfo}
 */
proto.speech.tts.TimingInfo.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.TimingInfo;
  return proto.speech.tts.TimingInfo.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.TimingInfo} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.TimingInfo}
 */
proto.speech.tts.TimingInfo.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setStartSeconds(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setMidSeconds(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setEndSeconds(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.TimingInfo.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.TimingInfo.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.TimingInfo} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.TimingInfo.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?number} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeFloat(
      1,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeFloat(
      2,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeFloat(
      3,
      f
    );
  }
};


/**
 * optional float start_seconds = 1;
 * @return {?number}
 */
proto.speech.tts.TimingInfo.prototype.getStartSeconds = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 1));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.TimingInfo} returns this
 */
proto.speech.tts.TimingInfo.prototype.setStartSeconds = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.TimingInfo} returns this
 */
proto.speech.tts.TimingInfo.prototype.clearStartSeconds = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.TimingInfo.prototype.hasStartSeconds = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional float mid_seconds = 2;
 * @return {?number}
 */
proto.speech.tts.TimingInfo.prototype.getMidSeconds = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 2));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.TimingInfo} returns this
 */
proto.speech.tts.TimingInfo.prototype.setMidSeconds = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.TimingInfo} returns this
 */
proto.speech.tts.TimingInfo.prototype.clearMidSeconds = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.TimingInfo.prototype.hasMidSeconds = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional float end_seconds = 3;
 * @return {?number}
 */
proto.speech.tts.TimingInfo.prototype.getEndSeconds = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 3));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.TimingInfo} returns this
 */
proto.speech.tts.TimingInfo.prototype.setEndSeconds = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.TimingInfo} returns this
 */
proto.speech.tts.TimingInfo.prototype.clearEndSeconds = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.TimingInfo.prototype.hasEndSeconds = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.TimingInfo}
 */
proto.speech.tts.TimingInfo.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.TimingInfo, data);
};



//speech/tts/engine/utterance/specification_proto_jspb/specification.jspb.js
// source: speech/tts/engine/utterance/specification.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.Specification');
goog.provide('proto.speech.tts.Specification.Type');
goog.provide('proto.speech.tts.Specification.UnitEmbedding');
goog.provide('proto.speech.tts.Specification.VoiceBuilding');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.AcousticStreamInfo');
goog.require('proto.speech.tts.FeatureData');
goog.require('proto.speech.tts.Links');
goog.require('proto.speech.tts.TimingInfo');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.Specification = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 11, proto.speech.tts.Specification.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.Specification, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.Specification.displayName = 'proto.speech.tts.Specification';
}

/**
 * The extensions registered with this message class. This is a map of
 * extension field number to fieldInfo object.
 *
 * For example:
 *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }
 *
 * fieldName contains the JsCompiler renamed field name property so that it
 * works in OPTIMIZED mode.
 *
 * @type {!Object<number, jspb.ExtensionFieldInfo>}
 */
proto.speech.tts.Specification.extensions = {};


/**
 * The extensions registered with this message class. This is a map of
 * extension field number to fieldInfo object.
 *
 * For example:
 *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }
 *
 * fieldName contains the JsCompiler renamed field name property so that it
 * works in OPTIMIZED mode.
 *
 * @type {!Object<number, jspb.ExtensionFieldBinaryInfo>}
 */
proto.speech.tts.Specification.extensionsBinary = {};

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.Specification.VoiceBuilding = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.Specification.VoiceBuilding, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.Specification.VoiceBuilding.displayName = 'proto.speech.tts.Specification.VoiceBuilding';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.Specification.UnitEmbedding = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.Specification.UnitEmbedding.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.Specification.UnitEmbedding, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.Specification.UnitEmbedding.displayName = 'proto.speech.tts.Specification.UnitEmbedding';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.Specification.repeatedFields_ = [4];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.Specification.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.Specification.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.Specification} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Specification.toObject = function(includeInstance, msg) {
  var f, obj = {
    links: (f = msg.getLinks()) && proto.speech.tts.Links.toObject(includeInstance, f),
    name: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    fallbackName: (f = jspb.Message.getField(msg, 8)) == null ? undefined : f,
    featureData: (f = msg.getFeatureData()) && proto.speech.tts.FeatureData.toObject(includeInstance, f),
    durationsList: (f = jspb.Message.getRepeatedFloatingPointField(msg, 4)) == null ? undefined : f,
    modelName: (f = jspb.Message.getField(msg, 5)) == null ? undefined : f,
    supraSegmentalModelName: (f = jspb.Message.getField(msg, 6)) == null ? undefined : f,
    voiceBuilding: (f = msg.getVoiceBuilding()) && proto.speech.tts.Specification.VoiceBuilding.toObject(includeInstance, f),
    linguisticEmbedding: (f = msg.getLinguisticEmbedding()) && proto.speech.tts.Specification.UnitEmbedding.toObject(includeInstance, f),
    acousticEmbedding: (f = msg.getAcousticEmbedding()) && proto.speech.tts.Specification.UnitEmbedding.toObject(includeInstance, f)
  };

  jspb.Message.toObjectExtension(/** @type {!jspb.Message} */ (msg), obj,
      proto.speech.tts.Specification.extensions, proto.speech.tts.Specification.prototype.getExtension,
      includeInstance);
  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Specification as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.Specification.ObjectFormat = function() {
  /** @type {?proto.speech.tts.Links.ObjectFormat|undefined} */
  this.links;

  /** @type {?string|undefined} */
  this.name;

  /** @type {?string|undefined} */
  this.fallbackName;

  /** @type {?proto.speech.tts.FeatureData.ObjectFormat|undefined} */
  this.featureData;

  /** @type {?Array<number>|undefined} */
  this.durationsList;

  /** @type {?string|undefined} */
  this.modelName;

  /** @type {?string|undefined} */
  this.supraSegmentalModelName;

  /** @type {?proto.speech.tts.Specification.VoiceBuilding.ObjectFormat|undefined} */
  this.voiceBuilding;

  /** @type {?proto.speech.tts.Specification.UnitEmbedding.ObjectFormat|undefined} */
  this.linguisticEmbedding;

  /** @type {?proto.speech.tts.Specification.UnitEmbedding.ObjectFormat|undefined} */
  this.acousticEmbedding;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.Specification.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.Specification}
 */
proto.speech.tts.Specification.fromObject = function(obj) {
  var msg = new proto.speech.tts.Specification();
  obj.links && jspb.Message.setWrapperField(
      msg, 1, proto.speech.tts.Links.fromObject(obj.links));
  obj.name != null && jspb.Message.setField(msg, 2, obj.name);
  obj.fallbackName != null && jspb.Message.setField(msg, 8, obj.fallbackName);
  obj.featureData && jspb.Message.setWrapperField(
      msg, 3, proto.speech.tts.FeatureData.fromObject(obj.featureData));
  obj.durationsList != null && jspb.Message.setField(msg, 4, obj.durationsList);
  obj.modelName != null && jspb.Message.setField(msg, 5, obj.modelName);
  obj.supraSegmentalModelName != null && jspb.Message.setField(msg, 6, obj.supraSegmentalModelName);
  obj.voiceBuilding && jspb.Message.setWrapperField(
      msg, 7, proto.speech.tts.Specification.VoiceBuilding.fromObject(obj.voiceBuilding));
  obj.linguisticEmbedding && jspb.Message.setWrapperField(
      msg, 9, proto.speech.tts.Specification.UnitEmbedding.fromObject(obj.linguisticEmbedding));
  obj.acousticEmbedding && jspb.Message.setWrapperField(
      msg, 10, proto.speech.tts.Specification.UnitEmbedding.fromObject(obj.acousticEmbedding));
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.Specification}
 */
proto.speech.tts.Specification.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.Specification;
  return proto.speech.tts.Specification.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.Specification} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.Specification}
 */
proto.speech.tts.Specification.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.speech.tts.Links;
      reader.readMessage(value,proto.speech.tts.Links.deserializeBinaryFromReader);
      msg.setLinks(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setName(value);
      break;
    case 8:
      var value = /** @type {string} */ (reader.readString());
      msg.setFallbackName(value);
      break;
    case 3:
      var value = new proto.speech.tts.FeatureData;
      reader.readMessage(value,proto.speech.tts.FeatureData.deserializeBinaryFromReader);
      msg.setFeatureData(value);
      break;
    case 4:
      var value = /** @type {!Array<number>} */ (reader.readPackedFloat());
      msg.setDurationsList(value);
      break;
    case 5:
      var value = /** @type {string} */ (reader.readString());
      msg.setModelName(value);
      break;
    case 6:
      var value = /** @type {string} */ (reader.readString());
      msg.setSupraSegmentalModelName(value);
      break;
    case 7:
      var value = new proto.speech.tts.Specification.VoiceBuilding;
      reader.readMessage(value,proto.speech.tts.Specification.VoiceBuilding.deserializeBinaryFromReader);
      msg.setVoiceBuilding(value);
      break;
    case 9:
      var value = new proto.speech.tts.Specification.UnitEmbedding;
      reader.readMessage(value,proto.speech.tts.Specification.UnitEmbedding.deserializeBinaryFromReader);
      msg.setLinguisticEmbedding(value);
      break;
    case 10:
      var value = new proto.speech.tts.Specification.UnitEmbedding;
      reader.readMessage(value,proto.speech.tts.Specification.UnitEmbedding.deserializeBinaryFromReader);
      msg.setAcousticEmbedding(value);
      break;
    default:
      jspb.Message.readBinaryExtension(msg, reader,
        proto.speech.tts.Specification.extensionsBinary,
        proto.speech.tts.Specification.prototype.getExtension,
        proto.speech.tts.Specification.prototype.setExtension);
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.Specification.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.Specification.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.Specification} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Specification.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getLinks();
  if (f != null) {
    writer.writeMessage(
      1,
      f,
      proto.speech.tts.Links.serializeBinaryToWriter
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 8));
  if (f != null) {
    writer.writeString(
      8,
      f
    );
  }
  f = message.getFeatureData();
  if (f != null) {
    writer.writeMessage(
      3,
      f,
      proto.speech.tts.FeatureData.serializeBinaryToWriter
    );
  }
  f = message.getDurationsList();
  if (f.length > 0) {
    writer.writePackedFloat(
      4,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeString(
      5,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeString(
      6,
      f
    );
  }
  f = message.getVoiceBuilding();
  if (f != null) {
    writer.writeMessage(
      7,
      f,
      proto.speech.tts.Specification.VoiceBuilding.serializeBinaryToWriter
    );
  }
  f = message.getLinguisticEmbedding();
  if (f != null) {
    writer.writeMessage(
      9,
      f,
      proto.speech.tts.Specification.UnitEmbedding.serializeBinaryToWriter
    );
  }
  f = message.getAcousticEmbedding();
  if (f != null) {
    writer.writeMessage(
      10,
      f,
      proto.speech.tts.Specification.UnitEmbedding.serializeBinaryToWriter
    );
  }
  jspb.Message.serializeBinaryExtensions(message, writer,
    proto.speech.tts.Specification.extensionsBinary, proto.speech.tts.Specification.prototype.getExtension);
};


/**
 * @enum {number}
 */
proto.speech.tts.Specification.Type = {
  NONE: -1,
  UNIT_SELECTION: 0,
  PARAMETRIC_SYNTHESIS: 1,
  SUPRASEGMENTAL_SYLLABLE: 2
};




if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.Specification.VoiceBuilding.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.Specification.VoiceBuilding.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.Specification.VoiceBuilding} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Specification.VoiceBuilding.toObject = function(includeInstance, msg) {
  var f, obj = {
    timing: (f = msg.getTiming()) && proto.speech.tts.TimingInfo.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of VoiceBuilding as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.Specification.VoiceBuilding.ObjectFormat = function() {
  /** @type {?proto.speech.tts.TimingInfo.ObjectFormat|undefined} */
  this.timing;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.Specification.VoiceBuilding.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.Specification.VoiceBuilding}
 */
proto.speech.tts.Specification.VoiceBuilding.fromObject = function(obj) {
  var msg = new proto.speech.tts.Specification.VoiceBuilding();
  obj.timing && jspb.Message.setWrapperField(
      msg, 1, proto.speech.tts.TimingInfo.fromObject(obj.timing));
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.Specification.VoiceBuilding}
 */
proto.speech.tts.Specification.VoiceBuilding.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.Specification.VoiceBuilding;
  return proto.speech.tts.Specification.VoiceBuilding.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.Specification.VoiceBuilding} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.Specification.VoiceBuilding}
 */
proto.speech.tts.Specification.VoiceBuilding.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.speech.tts.TimingInfo;
      reader.readMessage(value,proto.speech.tts.TimingInfo.deserializeBinaryFromReader);
      msg.setTiming(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.Specification.VoiceBuilding.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.Specification.VoiceBuilding.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.Specification.VoiceBuilding} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Specification.VoiceBuilding.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getTiming();
  if (f != null) {
    writer.writeMessage(
      1,
      f,
      proto.speech.tts.TimingInfo.serializeBinaryToWriter
    );
  }
};


/**
 * optional TimingInfo timing = 1;
 * @return {?proto.speech.tts.TimingInfo}
 */
proto.speech.tts.Specification.VoiceBuilding.prototype.getTiming = function() {
  return /** @type{?proto.speech.tts.TimingInfo} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.TimingInfo, 1));
};


/**
 * @param {?proto.speech.tts.TimingInfo|undefined} value
 * @return {!proto.speech.tts.Specification.VoiceBuilding} returns this
*/
proto.speech.tts.Specification.VoiceBuilding.prototype.setTiming = function(value) {
  return jspb.Message.setWrapperField(this, 1, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Specification.VoiceBuilding} returns this
 */
proto.speech.tts.Specification.VoiceBuilding.prototype.clearTiming = function() {
  return this.setTiming(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Specification.VoiceBuilding.prototype.hasTiming = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.Specification.VoiceBuilding}
 */
proto.speech.tts.Specification.VoiceBuilding.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.Specification.VoiceBuilding, data);
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.Specification.UnitEmbedding.repeatedFields_ = [1,2,3];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.Specification.UnitEmbedding.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.Specification.UnitEmbedding.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.Specification.UnitEmbedding} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Specification.UnitEmbedding.toObject = function(includeInstance, msg) {
  var f, obj = {
    meanList: (f = jspb.Message.getRepeatedFloatingPointField(msg, 1)) == null ? undefined : f,
    stddevList: (f = jspb.Message.getRepeatedFloatingPointField(msg, 2)) == null ? undefined : f,
    streamInfoList: jspb.Message.toObjectList(msg.getStreamInfoList(),
    proto.speech.tts.AcousticStreamInfo.toObject, includeInstance)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of UnitEmbedding as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.Specification.UnitEmbedding.ObjectFormat = function() {
  /** @type {?Array<number>|undefined} */
  this.meanList;

  /** @type {?Array<number>|undefined} */
  this.stddevList;

  /** @type {?Array<!proto.speech.tts.AcousticStreamInfo.ObjectFormat>|undefined} */
  this.streamInfoList;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.Specification.UnitEmbedding.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.Specification.UnitEmbedding}
 */
proto.speech.tts.Specification.UnitEmbedding.fromObject = function(obj) {
  var msg = new proto.speech.tts.Specification.UnitEmbedding();
  obj.meanList != null && jspb.Message.setField(msg, 1, obj.meanList);
  obj.stddevList != null && jspb.Message.setField(msg, 2, obj.stddevList);
  obj.streamInfoList && jspb.Message.setRepeatedWrapperField(
      msg, 3, obj.streamInfoList.map(
          proto.speech.tts.AcousticStreamInfo.fromObject));
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.Specification.UnitEmbedding}
 */
proto.speech.tts.Specification.UnitEmbedding.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.Specification.UnitEmbedding;
  return proto.speech.tts.Specification.UnitEmbedding.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.Specification.UnitEmbedding} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.Specification.UnitEmbedding}
 */
proto.speech.tts.Specification.UnitEmbedding.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {!Array<number>} */ (reader.readPackedFloat());
      msg.setMeanList(value);
      break;
    case 2:
      var value = /** @type {!Array<number>} */ (reader.readPackedFloat());
      msg.setStddevList(value);
      break;
    case 3:
      var value = new proto.speech.tts.AcousticStreamInfo;
      reader.readMessage(value,proto.speech.tts.AcousticStreamInfo.deserializeBinaryFromReader);
      msg.addStreamInfo(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.Specification.UnitEmbedding.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.Specification.UnitEmbedding.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.Specification.UnitEmbedding} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Specification.UnitEmbedding.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getMeanList();
  if (f.length > 0) {
    writer.writePackedFloat(
      1,
      f
    );
  }
  f = message.getStddevList();
  if (f.length > 0) {
    writer.writePackedFloat(
      2,
      f
    );
  }
  f = message.getStreamInfoList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      3,
      f,
      proto.speech.tts.AcousticStreamInfo.serializeBinaryToWriter
    );
  }
};


/**
 * repeated float mean = 1;
 * @return {!Array<number>}
 */
proto.speech.tts.Specification.UnitEmbedding.prototype.getMeanList = function() {
  return /** @type {!Array<number>} */ (jspb.Message.getRepeatedFloatingPointField(this, 1));
};


/**
 * @param {Array<number>|undefined} value
 * @return {!proto.speech.tts.Specification.UnitEmbedding} returns this
 */
proto.speech.tts.Specification.UnitEmbedding.prototype.setMeanList = function(value) {
  return jspb.Message.setField(this, 1, value || []);
};


/**
 * @param {number} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.Specification.UnitEmbedding} returns this
 */
proto.speech.tts.Specification.UnitEmbedding.prototype.addMean = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 1, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.Specification.UnitEmbedding} returns this
 */
proto.speech.tts.Specification.UnitEmbedding.prototype.clearMeanList = function() {
  return this.setMeanList([]);
};


/**
 * repeated float stddev = 2;
 * @return {!Array<number>}
 */
proto.speech.tts.Specification.UnitEmbedding.prototype.getStddevList = function() {
  return /** @type {!Array<number>} */ (jspb.Message.getRepeatedFloatingPointField(this, 2));
};


/**
 * @param {Array<number>|undefined} value
 * @return {!proto.speech.tts.Specification.UnitEmbedding} returns this
 */
proto.speech.tts.Specification.UnitEmbedding.prototype.setStddevList = function(value) {
  return jspb.Message.setField(this, 2, value || []);
};


/**
 * @param {number} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.Specification.UnitEmbedding} returns this
 */
proto.speech.tts.Specification.UnitEmbedding.prototype.addStddev = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 2, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.Specification.UnitEmbedding} returns this
 */
proto.speech.tts.Specification.UnitEmbedding.prototype.clearStddevList = function() {
  return this.setStddevList([]);
};


/**
 * repeated AcousticStreamInfo stream_info = 3;
 * @return {!Array<!proto.speech.tts.AcousticStreamInfo>}
 */
proto.speech.tts.Specification.UnitEmbedding.prototype.getStreamInfoList = function() {
  return /** @type{!Array<!proto.speech.tts.AcousticStreamInfo>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.AcousticStreamInfo, 3));
};


/**
 * @param {Array<!proto.speech.tts.AcousticStreamInfo>|undefined} value
 * @return {!proto.speech.tts.Specification.UnitEmbedding} returns this
*/
proto.speech.tts.Specification.UnitEmbedding.prototype.setStreamInfoList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 3, value);
};


/**
 * @param {!proto.speech.tts.AcousticStreamInfo=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.AcousticStreamInfo}
 */
proto.speech.tts.Specification.UnitEmbedding.prototype.addStreamInfo = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 3, opt_value, proto.speech.tts.AcousticStreamInfo, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.Specification.UnitEmbedding} returns this
 */
proto.speech.tts.Specification.UnitEmbedding.prototype.clearStreamInfoList = function() {
  return this.setStreamInfoList([]);
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.Specification.UnitEmbedding}
 */
proto.speech.tts.Specification.UnitEmbedding.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.Specification.UnitEmbedding, data);
};


/**
 * optional Links links = 1;
 * @return {?proto.speech.tts.Links}
 */
proto.speech.tts.Specification.prototype.getLinks = function() {
  return /** @type{?proto.speech.tts.Links} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Links, 1));
};


/**
 * @param {?proto.speech.tts.Links|undefined} value
 * @return {!proto.speech.tts.Specification} returns this
*/
proto.speech.tts.Specification.prototype.setLinks = function(value) {
  return jspb.Message.setWrapperField(this, 1, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Specification} returns this
 */
proto.speech.tts.Specification.prototype.clearLinks = function() {
  return this.setLinks(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Specification.prototype.hasLinks = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional string name = 2;
 * @return {?string}
 */
proto.speech.tts.Specification.prototype.getName = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Specification} returns this
 */
proto.speech.tts.Specification.prototype.setName = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Specification} returns this
 */
proto.speech.tts.Specification.prototype.clearName = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Specification.prototype.hasName = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional string fallback_name = 8;
 * @return {?string}
 */
proto.speech.tts.Specification.prototype.getFallbackName = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 8));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Specification} returns this
 */
proto.speech.tts.Specification.prototype.setFallbackName = function(value) {
  return jspb.Message.setField(this, 8, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Specification} returns this
 */
proto.speech.tts.Specification.prototype.clearFallbackName = function() {
  return jspb.Message.setField(this, 8, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Specification.prototype.hasFallbackName = function() {
  return jspb.Message.getField(this, 8) != null;
};


/**
 * optional FeatureData feature_data = 3;
 * @return {?proto.speech.tts.FeatureData}
 */
proto.speech.tts.Specification.prototype.getFeatureData = function() {
  return /** @type{?proto.speech.tts.FeatureData} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.FeatureData, 3));
};


/**
 * @param {?proto.speech.tts.FeatureData|undefined} value
 * @return {!proto.speech.tts.Specification} returns this
*/
proto.speech.tts.Specification.prototype.setFeatureData = function(value) {
  return jspb.Message.setWrapperField(this, 3, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Specification} returns this
 */
proto.speech.tts.Specification.prototype.clearFeatureData = function() {
  return this.setFeatureData(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Specification.prototype.hasFeatureData = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * repeated float durations = 4;
 * @return {!Array<number>}
 */
proto.speech.tts.Specification.prototype.getDurationsList = function() {
  return /** @type {!Array<number>} */ (jspb.Message.getRepeatedFloatingPointField(this, 4));
};


/**
 * @param {Array<number>|undefined} value
 * @return {!proto.speech.tts.Specification} returns this
 */
proto.speech.tts.Specification.prototype.setDurationsList = function(value) {
  return jspb.Message.setField(this, 4, value || []);
};


/**
 * @param {number} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.Specification} returns this
 */
proto.speech.tts.Specification.prototype.addDurations = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 4, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.Specification} returns this
 */
proto.speech.tts.Specification.prototype.clearDurationsList = function() {
  return this.setDurationsList([]);
};


/**
 * optional string model_name = 5;
 * @return {?string}
 */
proto.speech.tts.Specification.prototype.getModelName = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 5));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Specification} returns this
 */
proto.speech.tts.Specification.prototype.setModelName = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Specification} returns this
 */
proto.speech.tts.Specification.prototype.clearModelName = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Specification.prototype.hasModelName = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional string supra_segmental_model_name = 6;
 * @return {?string}
 */
proto.speech.tts.Specification.prototype.getSupraSegmentalModelName = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 6));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Specification} returns this
 */
proto.speech.tts.Specification.prototype.setSupraSegmentalModelName = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Specification} returns this
 */
proto.speech.tts.Specification.prototype.clearSupraSegmentalModelName = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Specification.prototype.hasSupraSegmentalModelName = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional VoiceBuilding voice_building = 7;
 * @return {?proto.speech.tts.Specification.VoiceBuilding}
 */
proto.speech.tts.Specification.prototype.getVoiceBuilding = function() {
  return /** @type{?proto.speech.tts.Specification.VoiceBuilding} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Specification.VoiceBuilding, 7));
};


/**
 * @param {?proto.speech.tts.Specification.VoiceBuilding|undefined} value
 * @return {!proto.speech.tts.Specification} returns this
*/
proto.speech.tts.Specification.prototype.setVoiceBuilding = function(value) {
  return jspb.Message.setWrapperField(this, 7, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Specification} returns this
 */
proto.speech.tts.Specification.prototype.clearVoiceBuilding = function() {
  return this.setVoiceBuilding(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Specification.prototype.hasVoiceBuilding = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * optional UnitEmbedding linguistic_embedding = 9;
 * @return {?proto.speech.tts.Specification.UnitEmbedding}
 */
proto.speech.tts.Specification.prototype.getLinguisticEmbedding = function() {
  return /** @type{?proto.speech.tts.Specification.UnitEmbedding} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Specification.UnitEmbedding, 9));
};


/**
 * @param {?proto.speech.tts.Specification.UnitEmbedding|undefined} value
 * @return {!proto.speech.tts.Specification} returns this
*/
proto.speech.tts.Specification.prototype.setLinguisticEmbedding = function(value) {
  return jspb.Message.setWrapperField(this, 9, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Specification} returns this
 */
proto.speech.tts.Specification.prototype.clearLinguisticEmbedding = function() {
  return this.setLinguisticEmbedding(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Specification.prototype.hasLinguisticEmbedding = function() {
  return jspb.Message.getField(this, 9) != null;
};


/**
 * optional UnitEmbedding acoustic_embedding = 10;
 * @return {?proto.speech.tts.Specification.UnitEmbedding}
 */
proto.speech.tts.Specification.prototype.getAcousticEmbedding = function() {
  return /** @type{?proto.speech.tts.Specification.UnitEmbedding} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Specification.UnitEmbedding, 10));
};


/**
 * @param {?proto.speech.tts.Specification.UnitEmbedding|undefined} value
 * @return {!proto.speech.tts.Specification} returns this
*/
proto.speech.tts.Specification.prototype.setAcousticEmbedding = function(value) {
  return jspb.Message.setWrapperField(this, 10, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Specification} returns this
 */
proto.speech.tts.Specification.prototype.clearAcousticEmbedding = function() {
  return this.setAcousticEmbedding(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Specification.prototype.hasAcousticEmbedding = function() {
  return jspb.Message.getField(this, 10) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.Specification}
 */
proto.speech.tts.Specification.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.Specification, data);
};



//speech/tts/engine/utterance/parametric_synthesis_data_proto_jspb/parametricsynthesisinput.jspb.js
// source: speech/tts/engine/utterance/parametric_synthesis_data.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.ParametricSynthesisInput');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.Specification');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.ParametricSynthesisInput = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.ParametricSynthesisInput.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.ParametricSynthesisInput, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.ParametricSynthesisInput.displayName = 'proto.speech.tts.ParametricSynthesisInput';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.ParametricSynthesisInput.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.ParametricSynthesisInput.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.ParametricSynthesisInput.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.ParametricSynthesisInput} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.ParametricSynthesisInput.toObject = function(includeInstance, msg) {
  var f, obj = {
    targetSpecList: jspb.Message.toObjectList(msg.getTargetSpecList(),
    proto.speech.tts.Specification.toObject, includeInstance)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of ParametricSynthesisInput as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.ParametricSynthesisInput.ObjectFormat = function() {
  /** @type {?Array<!proto.speech.tts.Specification.ObjectFormat>|undefined} */
  this.targetSpecList;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.ParametricSynthesisInput.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.ParametricSynthesisInput}
 */
proto.speech.tts.ParametricSynthesisInput.fromObject = function(obj) {
  var msg = new proto.speech.tts.ParametricSynthesisInput();
  obj.targetSpecList && jspb.Message.setRepeatedWrapperField(
      msg, 1, obj.targetSpecList.map(
          proto.speech.tts.Specification.fromObject));
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.ParametricSynthesisInput}
 */
proto.speech.tts.ParametricSynthesisInput.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.ParametricSynthesisInput;
  return proto.speech.tts.ParametricSynthesisInput.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.ParametricSynthesisInput} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.ParametricSynthesisInput}
 */
proto.speech.tts.ParametricSynthesisInput.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.speech.tts.Specification;
      reader.readMessage(value,proto.speech.tts.Specification.deserializeBinaryFromReader);
      msg.addTargetSpec(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.ParametricSynthesisInput.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.ParametricSynthesisInput.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.ParametricSynthesisInput} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.ParametricSynthesisInput.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getTargetSpecList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      1,
      f,
      proto.speech.tts.Specification.serializeBinaryToWriter
    );
  }
};


/**
 * repeated Specification target_spec = 1;
 * @return {!Array<!proto.speech.tts.Specification>}
 */
proto.speech.tts.ParametricSynthesisInput.prototype.getTargetSpecList = function() {
  return /** @type{!Array<!proto.speech.tts.Specification>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.Specification, 1));
};


/**
 * @param {Array<!proto.speech.tts.Specification>|undefined} value
 * @return {!proto.speech.tts.ParametricSynthesisInput} returns this
*/
proto.speech.tts.ParametricSynthesisInput.prototype.setTargetSpecList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 1, value);
};


/**
 * @param {!proto.speech.tts.Specification=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.Specification}
 */
proto.speech.tts.ParametricSynthesisInput.prototype.addTargetSpec = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.speech.tts.Specification, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.ParametricSynthesisInput} returns this
 */
proto.speech.tts.ParametricSynthesisInput.prototype.clearTargetSpecList = function() {
  return this.setTargetSpecList([]);
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.ParametricSynthesisInput}
 */
proto.speech.tts.ParametricSynthesisInput.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.ParametricSynthesisInput, data);
};



//speech/tts/engine/utterance/parametric_synthesis_data_proto_jspb/parametricsynthesisoutput.jspb.js
// source: speech/tts/engine/utterance/parametric_synthesis_data.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.ParametricSynthesisOutput');
goog.provide('proto.speech.tts.ParametricSynthesisOutput.AcousticFrame');
goog.provide('proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.SpectrogramFrame');
goog.provide('proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.Links');
goog.require('proto.speech.tts.TimingInfo');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.ParametricSynthesisOutput = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.ParametricSynthesisOutput.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.ParametricSynthesisOutput, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.ParametricSynthesisOutput.displayName = 'proto.speech.tts.ParametricSynthesisOutput';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.ParametricSynthesisOutput.AcousticFrame, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.displayName = 'proto.speech.tts.ParametricSynthesisOutput.AcousticFrame';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam.displayName = 'proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.SpectrogramFrame = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.SpectrogramFrame.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.SpectrogramFrame, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.SpectrogramFrame.displayName = 'proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.SpectrogramFrame';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.ParametricSynthesisOutput.repeatedFields_ = [1,2,3,4];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.ParametricSynthesisOutput.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.ParametricSynthesisOutput.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.ParametricSynthesisOutput} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.ParametricSynthesisOutput.toObject = function(includeInstance, msg) {
  var f, obj = {
    frameList: jspb.Message.toObjectList(msg.getFrameList(),
    proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.toObject, includeInstance),
    timingList: jspb.Message.toObjectList(msg.getTimingList(),
    proto.speech.tts.TimingInfo.toObject, includeInstance),
    pitchmarkList: (f = jspb.Message.getRepeatedField(msg, 3)) == null ? undefined : f,
    voicingList: (f = jspb.Message.getRepeatedBooleanField(msg, 4)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of ParametricSynthesisOutput as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.ParametricSynthesisOutput.ObjectFormat = function() {
  /** @type {?Array<!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.ObjectFormat>|undefined} */
  this.frameList;

  /** @type {?Array<!proto.speech.tts.TimingInfo.ObjectFormat>|undefined} */
  this.timingList;

  /** @type {?Array<number>|undefined} */
  this.pitchmarkList;

  /** @type {?Array<boolean>|undefined} */
  this.voicingList;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.ParametricSynthesisOutput.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.ParametricSynthesisOutput}
 */
proto.speech.tts.ParametricSynthesisOutput.fromObject = function(obj) {
  var msg = new proto.speech.tts.ParametricSynthesisOutput();
  obj.frameList && jspb.Message.setRepeatedWrapperField(
      msg, 1, obj.frameList.map(
          proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.fromObject));
  obj.timingList && jspb.Message.setRepeatedWrapperField(
      msg, 2, obj.timingList.map(
          proto.speech.tts.TimingInfo.fromObject));
  obj.pitchmarkList != null && jspb.Message.setField(msg, 3, obj.pitchmarkList);
  obj.voicingList != null && jspb.Message.setField(msg, 4, obj.voicingList);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.ParametricSynthesisOutput}
 */
proto.speech.tts.ParametricSynthesisOutput.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.ParametricSynthesisOutput;
  return proto.speech.tts.ParametricSynthesisOutput.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.ParametricSynthesisOutput} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.ParametricSynthesisOutput}
 */
proto.speech.tts.ParametricSynthesisOutput.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.speech.tts.ParametricSynthesisOutput.AcousticFrame;
      reader.readMessage(value,proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.deserializeBinaryFromReader);
      msg.addFrame(value);
      break;
    case 2:
      var value = new proto.speech.tts.TimingInfo;
      reader.readMessage(value,proto.speech.tts.TimingInfo.deserializeBinaryFromReader);
      msg.addTiming(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readInt32());
      msg.addPitchmark(value);
      break;
    case 4:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.addVoicing(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.ParametricSynthesisOutput.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.ParametricSynthesisOutput.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.ParametricSynthesisOutput} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.ParametricSynthesisOutput.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getFrameList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      1,
      f,
      proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.serializeBinaryToWriter
    );
  }
  f = message.getTimingList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      2,
      f,
      proto.speech.tts.TimingInfo.serializeBinaryToWriter
    );
  }
  f = message.getPitchmarkList();
  if (f.length > 0) {
    writer.writeRepeatedInt32(
      3,
      f
    );
  }
  f = message.getVoicingList();
  if (f.length > 0) {
    writer.writeRepeatedBool(
      4,
      f
    );
  }
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.toObject = function(includeInstance, msg) {
  var f, obj = {
    links: (f = msg.getLinks()) && proto.speech.tts.Links.toObject(includeInstance, f),
    vocoderParam: (f = msg.getVocoderParam()) && proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam.toObject(includeInstance, f),
    numRepetition: jspb.Message.getFieldWithDefault(msg, 3, 1),
    spectrogramFrame: (f = msg.getSpectrogramFrame()) && proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.SpectrogramFrame.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of AcousticFrame as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.ObjectFormat = function() {
  /** @type {?proto.speech.tts.Links.ObjectFormat|undefined} */
  this.links;

  /** @type {?proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam.ObjectFormat|undefined} */
  this.vocoderParam;

  /** @type {?number|undefined} */
  this.numRepetition;

  /** @type {?proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.SpectrogramFrame.ObjectFormat|undefined} */
  this.spectrogramFrame;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame}
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.fromObject = function(obj) {
  var msg = new proto.speech.tts.ParametricSynthesisOutput.AcousticFrame();
  obj.links && jspb.Message.setWrapperField(
      msg, 1, proto.speech.tts.Links.fromObject(obj.links));
  obj.vocoderParam && jspb.Message.setWrapperField(
      msg, 2, proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam.fromObject(obj.vocoderParam));
  obj.numRepetition != null && jspb.Message.setField(msg, 3, obj.numRepetition);
  obj.spectrogramFrame && jspb.Message.setWrapperField(
      msg, 4, proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.SpectrogramFrame.fromObject(obj.spectrogramFrame));
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame}
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.ParametricSynthesisOutput.AcousticFrame;
  return proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame}
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.speech.tts.Links;
      reader.readMessage(value,proto.speech.tts.Links.deserializeBinaryFromReader);
      msg.setLinks(value);
      break;
    case 2:
      var value = new proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam;
      reader.readMessage(value,proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam.deserializeBinaryFromReader);
      msg.setVocoderParam(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setNumRepetition(value);
      break;
    case 4:
      var value = new proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.SpectrogramFrame;
      reader.readMessage(value,proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.SpectrogramFrame.deserializeBinaryFromReader);
      msg.setSpectrogramFrame(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getLinks();
  if (f != null) {
    writer.writeMessage(
      1,
      f,
      proto.speech.tts.Links.serializeBinaryToWriter
    );
  }
  f = message.getVocoderParam();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam.serializeBinaryToWriter
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeInt32(
      3,
      f
    );
  }
  f = message.getSpectrogramFrame();
  if (f != null) {
    writer.writeMessage(
      4,
      f,
      proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.SpectrogramFrame.serializeBinaryToWriter
    );
  }
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam.repeatedFields_ = [3,4];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam.toObject = function(includeInstance, msg) {
  var f, obj = {
    voiced: (f = jspb.Message.getBooleanField(msg, 1)) == null ? undefined : f,
    logF0: (f = jspb.Message.getOptionalFloatingPointField(msg, 2)) == null ? undefined : f,
    amplitudeList: (f = jspb.Message.getRepeatedFloatingPointField(msg, 3)) == null ? undefined : f,
    aperiodicityList: (f = jspb.Message.getRepeatedFloatingPointField(msg, 4)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of VocoderParam as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam.ObjectFormat = function() {
  /** @type {?boolean|undefined} */
  this.voiced;

  /** @type {?number|undefined} */
  this.logF0;

  /** @type {?Array<number>|undefined} */
  this.amplitudeList;

  /** @type {?Array<number>|undefined} */
  this.aperiodicityList;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam}
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam.fromObject = function(obj) {
  var msg = new proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam();
  obj.voiced != null && jspb.Message.setField(msg, 1, obj.voiced);
  obj.logF0 != null && jspb.Message.setField(msg, 2, obj.logF0);
  obj.amplitudeList != null && jspb.Message.setField(msg, 3, obj.amplitudeList);
  obj.aperiodicityList != null && jspb.Message.setField(msg, 4, obj.aperiodicityList);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam}
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam;
  return proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam}
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setVoiced(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setLogF0(value);
      break;
    case 3:
      var value = /** @type {!Array<number>} */ (reader.readPackedFloat());
      msg.setAmplitudeList(value);
      break;
    case 4:
      var value = /** @type {!Array<number>} */ (reader.readPackedFloat());
      msg.setAperiodicityList(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeBool(
      1,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeFloat(
      2,
      f
    );
  }
  f = message.getAmplitudeList();
  if (f.length > 0) {
    writer.writePackedFloat(
      3,
      f
    );
  }
  f = message.getAperiodicityList();
  if (f.length > 0) {
    writer.writePackedFloat(
      4,
      f
    );
  }
};


/**
 * optional bool voiced = 1;
 * @return {?boolean}
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam.prototype.getVoiced = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 1));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam} returns this
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam.prototype.setVoiced = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam} returns this
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam.prototype.clearVoiced = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam.prototype.hasVoiced = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional float log_f0 = 2;
 * @return {?number}
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam.prototype.getLogF0 = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 2));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam} returns this
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam.prototype.setLogF0 = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam} returns this
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam.prototype.clearLogF0 = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam.prototype.hasLogF0 = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * repeated float amplitude = 3;
 * @return {!Array<number>}
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam.prototype.getAmplitudeList = function() {
  return /** @type {!Array<number>} */ (jspb.Message.getRepeatedFloatingPointField(this, 3));
};


/**
 * @param {Array<number>|undefined} value
 * @return {!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam} returns this
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam.prototype.setAmplitudeList = function(value) {
  return jspb.Message.setField(this, 3, value || []);
};


/**
 * @param {number} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam} returns this
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam.prototype.addAmplitude = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 3, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam} returns this
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam.prototype.clearAmplitudeList = function() {
  return this.setAmplitudeList([]);
};


/**
 * repeated float aperiodicity = 4;
 * @return {!Array<number>}
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam.prototype.getAperiodicityList = function() {
  return /** @type {!Array<number>} */ (jspb.Message.getRepeatedFloatingPointField(this, 4));
};


/**
 * @param {Array<number>|undefined} value
 * @return {!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam} returns this
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam.prototype.setAperiodicityList = function(value) {
  return jspb.Message.setField(this, 4, value || []);
};


/**
 * @param {number} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam} returns this
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam.prototype.addAperiodicity = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 4, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam} returns this
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam.prototype.clearAperiodicityList = function() {
  return this.setAperiodicityList([]);
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam}
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam, data);
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.SpectrogramFrame.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.SpectrogramFrame.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.SpectrogramFrame.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.SpectrogramFrame} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.SpectrogramFrame.toObject = function(includeInstance, msg) {
  var f, obj = {
    amplitudeList: (f = jspb.Message.getRepeatedFloatingPointField(msg, 1)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of SpectrogramFrame as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.SpectrogramFrame.ObjectFormat = function() {
  /** @type {?Array<number>|undefined} */
  this.amplitudeList;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.SpectrogramFrame.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.SpectrogramFrame}
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.SpectrogramFrame.fromObject = function(obj) {
  var msg = new proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.SpectrogramFrame();
  obj.amplitudeList != null && jspb.Message.setField(msg, 1, obj.amplitudeList);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.SpectrogramFrame}
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.SpectrogramFrame.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.SpectrogramFrame;
  return proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.SpectrogramFrame.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.SpectrogramFrame} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.SpectrogramFrame}
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.SpectrogramFrame.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {!Array<number>} */ (reader.readPackedFloat());
      msg.setAmplitudeList(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.SpectrogramFrame.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.SpectrogramFrame.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.SpectrogramFrame} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.SpectrogramFrame.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getAmplitudeList();
  if (f.length > 0) {
    writer.writePackedFloat(
      1,
      f
    );
  }
};


/**
 * repeated float amplitude = 1;
 * @return {!Array<number>}
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.SpectrogramFrame.prototype.getAmplitudeList = function() {
  return /** @type {!Array<number>} */ (jspb.Message.getRepeatedFloatingPointField(this, 1));
};


/**
 * @param {Array<number>|undefined} value
 * @return {!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.SpectrogramFrame} returns this
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.SpectrogramFrame.prototype.setAmplitudeList = function(value) {
  return jspb.Message.setField(this, 1, value || []);
};


/**
 * @param {number} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.SpectrogramFrame} returns this
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.SpectrogramFrame.prototype.addAmplitude = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 1, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.SpectrogramFrame} returns this
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.SpectrogramFrame.prototype.clearAmplitudeList = function() {
  return this.setAmplitudeList([]);
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.SpectrogramFrame}
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.SpectrogramFrame.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.SpectrogramFrame, data);
};


/**
 * optional Links links = 1;
 * @return {?proto.speech.tts.Links}
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.prototype.getLinks = function() {
  return /** @type{?proto.speech.tts.Links} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Links, 1));
};


/**
 * @param {?proto.speech.tts.Links|undefined} value
 * @return {!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame} returns this
*/
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.prototype.setLinks = function(value) {
  return jspb.Message.setWrapperField(this, 1, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame} returns this
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.prototype.clearLinks = function() {
  return this.setLinks(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.prototype.hasLinks = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional VocoderParam vocoder_param = 2;
 * @return {?proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam}
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.prototype.getVocoderParam = function() {
  return /** @type{?proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam, 2));
};


/**
 * @param {?proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.VocoderParam|undefined} value
 * @return {!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame} returns this
*/
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.prototype.setVocoderParam = function(value) {
  return jspb.Message.setWrapperField(this, 2, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame} returns this
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.prototype.clearVocoderParam = function() {
  return this.setVocoderParam(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.prototype.hasVocoderParam = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional int32 num_repetition = 3;
 * @return {number}
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.prototype.getNumRepetition = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 3, 1));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame} returns this
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.prototype.setNumRepetition = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame} returns this
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.prototype.clearNumRepetition = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.prototype.hasNumRepetition = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional SpectrogramFrame spectrogram_frame = 4;
 * @return {?proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.SpectrogramFrame}
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.prototype.getSpectrogramFrame = function() {
  return /** @type{?proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.SpectrogramFrame} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.SpectrogramFrame, 4));
};


/**
 * @param {?proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.SpectrogramFrame|undefined} value
 * @return {!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame} returns this
*/
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.prototype.setSpectrogramFrame = function(value) {
  return jspb.Message.setWrapperField(this, 4, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame} returns this
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.prototype.clearSpectrogramFrame = function() {
  return this.setSpectrogramFrame(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.prototype.hasSpectrogramFrame = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame}
 */
proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.ParametricSynthesisOutput.AcousticFrame, data);
};


/**
 * repeated AcousticFrame frame = 1;
 * @return {!Array<!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame>}
 */
proto.speech.tts.ParametricSynthesisOutput.prototype.getFrameList = function() {
  return /** @type{!Array<!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.ParametricSynthesisOutput.AcousticFrame, 1));
};


/**
 * @param {Array<!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame>|undefined} value
 * @return {!proto.speech.tts.ParametricSynthesisOutput} returns this
*/
proto.speech.tts.ParametricSynthesisOutput.prototype.setFrameList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 1, value);
};


/**
 * @param {!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame}
 */
proto.speech.tts.ParametricSynthesisOutput.prototype.addFrame = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.speech.tts.ParametricSynthesisOutput.AcousticFrame, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.ParametricSynthesisOutput} returns this
 */
proto.speech.tts.ParametricSynthesisOutput.prototype.clearFrameList = function() {
  return this.setFrameList([]);
};


/**
 * repeated TimingInfo timing = 2;
 * @return {!Array<!proto.speech.tts.TimingInfo>}
 */
proto.speech.tts.ParametricSynthesisOutput.prototype.getTimingList = function() {
  return /** @type{!Array<!proto.speech.tts.TimingInfo>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.TimingInfo, 2));
};


/**
 * @param {Array<!proto.speech.tts.TimingInfo>|undefined} value
 * @return {!proto.speech.tts.ParametricSynthesisOutput} returns this
*/
proto.speech.tts.ParametricSynthesisOutput.prototype.setTimingList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 2, value);
};


/**
 * @param {!proto.speech.tts.TimingInfo=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.TimingInfo}
 */
proto.speech.tts.ParametricSynthesisOutput.prototype.addTiming = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 2, opt_value, proto.speech.tts.TimingInfo, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.ParametricSynthesisOutput} returns this
 */
proto.speech.tts.ParametricSynthesisOutput.prototype.clearTimingList = function() {
  return this.setTimingList([]);
};


/**
 * repeated int32 pitchmark = 3;
 * @return {!Array<number>}
 */
proto.speech.tts.ParametricSynthesisOutput.prototype.getPitchmarkList = function() {
  return /** @type {!Array<number>} */ (jspb.Message.getRepeatedField(this, 3));
};


/**
 * @param {Array<number>|undefined} value
 * @return {!proto.speech.tts.ParametricSynthesisOutput} returns this
 */
proto.speech.tts.ParametricSynthesisOutput.prototype.setPitchmarkList = function(value) {
  return jspb.Message.setField(this, 3, value || []);
};


/**
 * @param {number} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.ParametricSynthesisOutput} returns this
 */
proto.speech.tts.ParametricSynthesisOutput.prototype.addPitchmark = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 3, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.ParametricSynthesisOutput} returns this
 */
proto.speech.tts.ParametricSynthesisOutput.prototype.clearPitchmarkList = function() {
  return this.setPitchmarkList([]);
};


/**
 * repeated bool voicing = 4;
 * @return {!Array<boolean>}
 */
proto.speech.tts.ParametricSynthesisOutput.prototype.getVoicingList = function() {
  return /** @type {!Array<boolean>} */ (jspb.Message.getRepeatedBooleanField(this, 4));
};


/**
 * @param {Array<boolean>|undefined} value
 * @return {!proto.speech.tts.ParametricSynthesisOutput} returns this
 */
proto.speech.tts.ParametricSynthesisOutput.prototype.setVoicingList = function(value) {
  return jspb.Message.setField(this, 4, value || []);
};


/**
 * @param {boolean} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.ParametricSynthesisOutput} returns this
 */
proto.speech.tts.ParametricSynthesisOutput.prototype.addVoicing = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 4, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.ParametricSynthesisOutput} returns this
 */
proto.speech.tts.ParametricSynthesisOutput.prototype.clearVoicingList = function() {
  return this.setVoicingList([]);
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.ParametricSynthesisOutput}
 */
proto.speech.tts.ParametricSynthesisOutput.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.ParametricSynthesisOutput, data);
};



//speech/tts/engine/utterance/parametric_synthesis_data_proto_jspb/parametricsynthesisdata.jspb.js
// source: speech/tts/engine/utterance/parametric_synthesis_data.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.ParametricSynthesisData');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.ParametricSynthesisInput');
goog.require('proto.speech.tts.ParametricSynthesisOutput');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.ParametricSynthesisData = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 3, proto.speech.tts.ParametricSynthesisData.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.ParametricSynthesisData, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.ParametricSynthesisData.displayName = 'proto.speech.tts.ParametricSynthesisData';
}

/**
 * The extensions registered with this message class. This is a map of
 * extension field number to fieldInfo object.
 *
 * For example:
 *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }
 *
 * fieldName contains the JsCompiler renamed field name property so that it
 * works in OPTIMIZED mode.
 *
 * @type {!Object<number, jspb.ExtensionFieldInfo>}
 */
proto.speech.tts.ParametricSynthesisData.extensions = {};


/**
 * The extensions registered with this message class. This is a map of
 * extension field number to fieldInfo object.
 *
 * For example:
 *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }
 *
 * fieldName contains the JsCompiler renamed field name property so that it
 * works in OPTIMIZED mode.
 *
 * @type {!Object<number, jspb.ExtensionFieldBinaryInfo>}
 */
proto.speech.tts.ParametricSynthesisData.extensionsBinary = {};


/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.ParametricSynthesisData.repeatedFields_ = [2];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.ParametricSynthesisData.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.ParametricSynthesisData.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.ParametricSynthesisData} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.ParametricSynthesisData.toObject = function(includeInstance, msg) {
  var f, obj = {
    input: (f = msg.getInput()) && proto.speech.tts.ParametricSynthesisInput.toObject(includeInstance, f),
    outputList: jspb.Message.toObjectList(msg.getOutputList(),
    proto.speech.tts.ParametricSynthesisOutput.toObject, includeInstance)
  };

  jspb.Message.toObjectExtension(/** @type {!jspb.Message} */ (msg), obj,
      proto.speech.tts.ParametricSynthesisData.extensions, proto.speech.tts.ParametricSynthesisData.prototype.getExtension,
      includeInstance);
  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of ParametricSynthesisData as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.ParametricSynthesisData.ObjectFormat = function() {
  /** @type {?proto.speech.tts.ParametricSynthesisInput.ObjectFormat|undefined} */
  this.input;

  /** @type {?Array<!proto.speech.tts.ParametricSynthesisOutput.ObjectFormat>|undefined} */
  this.outputList;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.ParametricSynthesisData.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.ParametricSynthesisData}
 */
proto.speech.tts.ParametricSynthesisData.fromObject = function(obj) {
  var msg = new proto.speech.tts.ParametricSynthesisData();
  obj.input && jspb.Message.setWrapperField(
      msg, 1, proto.speech.tts.ParametricSynthesisInput.fromObject(obj.input));
  obj.outputList && jspb.Message.setRepeatedWrapperField(
      msg, 2, obj.outputList.map(
          proto.speech.tts.ParametricSynthesisOutput.fromObject));
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.ParametricSynthesisData}
 */
proto.speech.tts.ParametricSynthesisData.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.ParametricSynthesisData;
  return proto.speech.tts.ParametricSynthesisData.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.ParametricSynthesisData} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.ParametricSynthesisData}
 */
proto.speech.tts.ParametricSynthesisData.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.speech.tts.ParametricSynthesisInput;
      reader.readMessage(value,proto.speech.tts.ParametricSynthesisInput.deserializeBinaryFromReader);
      msg.setInput(value);
      break;
    case 2:
      var value = new proto.speech.tts.ParametricSynthesisOutput;
      reader.readMessage(value,proto.speech.tts.ParametricSynthesisOutput.deserializeBinaryFromReader);
      msg.addOutput(value);
      break;
    default:
      jspb.Message.readBinaryExtension(msg, reader,
        proto.speech.tts.ParametricSynthesisData.extensionsBinary,
        proto.speech.tts.ParametricSynthesisData.prototype.getExtension,
        proto.speech.tts.ParametricSynthesisData.prototype.setExtension);
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.ParametricSynthesisData.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.ParametricSynthesisData.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.ParametricSynthesisData} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.ParametricSynthesisData.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getInput();
  if (f != null) {
    writer.writeMessage(
      1,
      f,
      proto.speech.tts.ParametricSynthesisInput.serializeBinaryToWriter
    );
  }
  f = message.getOutputList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      2,
      f,
      proto.speech.tts.ParametricSynthesisOutput.serializeBinaryToWriter
    );
  }
  jspb.Message.serializeBinaryExtensions(message, writer,
    proto.speech.tts.ParametricSynthesisData.extensionsBinary, proto.speech.tts.ParametricSynthesisData.prototype.getExtension);
};


/**
 * optional ParametricSynthesisInput input = 1;
 * @return {?proto.speech.tts.ParametricSynthesisInput}
 */
proto.speech.tts.ParametricSynthesisData.prototype.getInput = function() {
  return /** @type{?proto.speech.tts.ParametricSynthesisInput} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.ParametricSynthesisInput, 1));
};


/**
 * @param {?proto.speech.tts.ParametricSynthesisInput|undefined} value
 * @return {!proto.speech.tts.ParametricSynthesisData} returns this
*/
proto.speech.tts.ParametricSynthesisData.prototype.setInput = function(value) {
  return jspb.Message.setWrapperField(this, 1, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.ParametricSynthesisData} returns this
 */
proto.speech.tts.ParametricSynthesisData.prototype.clearInput = function() {
  return this.setInput(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.ParametricSynthesisData.prototype.hasInput = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * repeated ParametricSynthesisOutput output = 2;
 * @return {!Array<!proto.speech.tts.ParametricSynthesisOutput>}
 */
proto.speech.tts.ParametricSynthesisData.prototype.getOutputList = function() {
  return /** @type{!Array<!proto.speech.tts.ParametricSynthesisOutput>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.ParametricSynthesisOutput, 2));
};


/**
 * @param {Array<!proto.speech.tts.ParametricSynthesisOutput>|undefined} value
 * @return {!proto.speech.tts.ParametricSynthesisData} returns this
*/
proto.speech.tts.ParametricSynthesisData.prototype.setOutputList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 2, value);
};


/**
 * @param {!proto.speech.tts.ParametricSynthesisOutput=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.ParametricSynthesisOutput}
 */
proto.speech.tts.ParametricSynthesisData.prototype.addOutput = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 2, opt_value, proto.speech.tts.ParametricSynthesisOutput, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.ParametricSynthesisData} returns this
 */
proto.speech.tts.ParametricSynthesisData.prototype.clearOutputList = function() {
  return this.setOutputList([]);
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.ParametricSynthesisData}
 */
proto.speech.tts.ParametricSynthesisData.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.ParametricSynthesisData, data);
};



//speech/tts/engine/utterance/playback_data_proto_jspb/playbackdata.jspb.js
// source: speech/tts/engine/utterance/playback_data.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.PlaybackData');
goog.provide('proto.speech.tts.PlaybackData.PlaybackStatus');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.PlaybackData = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.PlaybackData, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.PlaybackData.displayName = 'proto.speech.tts.PlaybackData';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.PlaybackData.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.PlaybackData.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.PlaybackData} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.PlaybackData.toObject = function(includeInstance, msg) {
  var f, obj = {
    status: jspb.Message.getFieldWithDefault(msg, 1, 0),
    lastUnitId: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    stagename: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of PlaybackData as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.PlaybackData.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.status;

  /** @type {?number|undefined} */
  this.lastUnitId;

  /** @type {?string|undefined} */
  this.stagename;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.PlaybackData.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.PlaybackData}
 */
proto.speech.tts.PlaybackData.fromObject = function(obj) {
  var msg = new proto.speech.tts.PlaybackData();
  obj.status != null && jspb.Message.setField(msg, 1, obj.status);
  obj.lastUnitId != null && jspb.Message.setField(msg, 2, obj.lastUnitId);
  obj.stagename != null && jspb.Message.setField(msg, 3, obj.stagename);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.PlaybackData}
 */
proto.speech.tts.PlaybackData.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.PlaybackData;
  return proto.speech.tts.PlaybackData.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.PlaybackData} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.PlaybackData}
 */
proto.speech.tts.PlaybackData.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {!proto.speech.tts.PlaybackData.PlaybackStatus} */ (reader.readEnum());
      msg.setStatus(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setLastUnitId(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setStagename(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.PlaybackData.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.PlaybackData.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.PlaybackData} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.PlaybackData.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {!proto.speech.tts.PlaybackData.PlaybackStatus} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeEnum(
      1,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeInt32(
      2,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeString(
      3,
      f
    );
  }
};


/**
 * @enum {number}
 */
proto.speech.tts.PlaybackData.PlaybackStatus = {
  NO_PLAYBACK: 0,
  PLAYBACK_USING_THIS_UTTERANCE: 1,
  PLAYBACK_USING_OTHER_UTTERANCE: 2,
  GUIDED_SYNTHESIS_DIRECTOR_TEXT: 3,
  GUIDED_SYNTHESIS_REQUEST_TEXT: 4
};

/**
 * optional PlaybackStatus status = 1;
 * @return {!proto.speech.tts.PlaybackData.PlaybackStatus}
 */
proto.speech.tts.PlaybackData.prototype.getStatus = function() {
  return /** @type {!proto.speech.tts.PlaybackData.PlaybackStatus} */ (jspb.Message.getFieldWithDefault(this, 1, 0));
};


/**
 * @param {?proto.speech.tts.PlaybackData.PlaybackStatus|undefined} value
 * @return {!proto.speech.tts.PlaybackData} returns this
 */
proto.speech.tts.PlaybackData.prototype.setStatus = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.PlaybackData} returns this
 */
proto.speech.tts.PlaybackData.prototype.clearStatus = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.PlaybackData.prototype.hasStatus = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional int32 last_unit_id = 2;
 * @return {?number}
 */
proto.speech.tts.PlaybackData.prototype.getLastUnitId = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.PlaybackData} returns this
 */
proto.speech.tts.PlaybackData.prototype.setLastUnitId = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.PlaybackData} returns this
 */
proto.speech.tts.PlaybackData.prototype.clearLastUnitId = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.PlaybackData.prototype.hasLastUnitId = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional string stagename = 3;
 * @return {?string}
 */
proto.speech.tts.PlaybackData.prototype.getStagename = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.PlaybackData} returns this
 */
proto.speech.tts.PlaybackData.prototype.setStagename = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.PlaybackData} returns this
 */
proto.speech.tts.PlaybackData.prototype.clearStagename = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.PlaybackData.prototype.hasStagename = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.PlaybackData}
 */
proto.speech.tts.PlaybackData.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.PlaybackData, data);
};



//speech/tts/engine/utterance/streaming_info_proto_jspb/streaminginfo.jspb.js
// source: speech/tts/engine/utterance/streaming_info.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.StreamingInfo');
goog.provide('proto.speech.tts.StreamingInfo.StreamingState');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.StreamingInfo = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.StreamingInfo, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.StreamingInfo.displayName = 'proto.speech.tts.StreamingInfo';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.StreamingInfo.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.StreamingInfo.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.StreamingInfo} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.StreamingInfo.toObject = function(includeInstance, msg) {
  var f, obj = {
    state: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    chunkNumber: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    numChunks: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of StreamingInfo as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.StreamingInfo.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.state;

  /** @type {?number|undefined} */
  this.chunkNumber;

  /** @type {?number|undefined} */
  this.numChunks;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.StreamingInfo.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.StreamingInfo}
 */
proto.speech.tts.StreamingInfo.fromObject = function(obj) {
  var msg = new proto.speech.tts.StreamingInfo();
  obj.state != null && jspb.Message.setField(msg, 1, obj.state);
  obj.chunkNumber != null && jspb.Message.setField(msg, 2, obj.chunkNumber);
  obj.numChunks != null && jspb.Message.setField(msg, 3, obj.numChunks);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.StreamingInfo}
 */
proto.speech.tts.StreamingInfo.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.StreamingInfo;
  return proto.speech.tts.StreamingInfo.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.StreamingInfo} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.StreamingInfo}
 */
proto.speech.tts.StreamingInfo.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {!proto.speech.tts.StreamingInfo.StreamingState} */ (reader.readEnum());
      msg.setState(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setChunkNumber(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setNumChunks(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.StreamingInfo.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.StreamingInfo.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.StreamingInfo} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.StreamingInfo.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {!proto.speech.tts.StreamingInfo.StreamingState} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeEnum(
      1,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeInt32(
      2,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeInt32(
      3,
      f
    );
  }
};


/**
 * @enum {number}
 */
proto.speech.tts.StreamingInfo.StreamingState = {
  INVALID_STREAMING_STATE: 0,
  STATE_INITIAL: 1,
  STATE_IN_PROGRESS: 2,
  STATE_FAILED: 3,
  STATE_COMPLETED: 4,
  BYPASS_STREAMING_MODULE: 5
};

/**
 * optional StreamingState state = 1;
 * @return {!proto.speech.tts.StreamingInfo.StreamingState}
 */
proto.speech.tts.StreamingInfo.prototype.getState = function() {
  return /** @type {!proto.speech.tts.StreamingInfo.StreamingState} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?proto.speech.tts.StreamingInfo.StreamingState|undefined} value
 * @return {!proto.speech.tts.StreamingInfo} returns this
 */
proto.speech.tts.StreamingInfo.prototype.setState = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.StreamingInfo} returns this
 */
proto.speech.tts.StreamingInfo.prototype.clearState = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.StreamingInfo.prototype.hasState = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional int32 chunk_number = 2;
 * @return {?number}
 */
proto.speech.tts.StreamingInfo.prototype.getChunkNumber = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.StreamingInfo} returns this
 */
proto.speech.tts.StreamingInfo.prototype.setChunkNumber = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.StreamingInfo} returns this
 */
proto.speech.tts.StreamingInfo.prototype.clearChunkNumber = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.StreamingInfo.prototype.hasChunkNumber = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional int32 num_chunks = 3;
 * @return {?number}
 */
proto.speech.tts.StreamingInfo.prototype.getNumChunks = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.StreamingInfo} returns this
 */
proto.speech.tts.StreamingInfo.prototype.setNumChunks = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.StreamingInfo} returns this
 */
proto.speech.tts.StreamingInfo.prototype.clearNumChunks = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.StreamingInfo.prototype.hasNumChunks = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.StreamingInfo}
 */
proto.speech.tts.StreamingInfo.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.StreamingInfo, data);
};



//speech/tts/engine/utterance/unit_selection_data_proto_jspb/unitselectioninput.jspb.js
// source: speech/tts/engine/utterance/unit_selection_data.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.UnitSelectionInput');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.LatticeProto');
goog.require('proto.speech.tts.Specification');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.UnitSelectionInput = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.UnitSelectionInput.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.UnitSelectionInput, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.UnitSelectionInput.displayName = 'proto.speech.tts.UnitSelectionInput';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.UnitSelectionInput.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.UnitSelectionInput.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.UnitSelectionInput.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.UnitSelectionInput} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.UnitSelectionInput.toObject = function(includeInstance, msg) {
  var f, obj = {
    targetSpecList: jspb.Message.toObjectList(msg.getTargetSpecList(),
    proto.speech.tts.Specification.toObject, includeInstance),
    preselection: (f = msg.getPreselection()) && proto.speech.tts.LatticeProto.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of UnitSelectionInput as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.UnitSelectionInput.ObjectFormat = function() {
  /** @type {?Array<!proto.speech.tts.Specification.ObjectFormat>|undefined} */
  this.targetSpecList;

  /** @type {?proto.speech.tts.LatticeProto.ObjectFormat|undefined} */
  this.preselection;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.UnitSelectionInput.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.UnitSelectionInput}
 */
proto.speech.tts.UnitSelectionInput.fromObject = function(obj) {
  var msg = new proto.speech.tts.UnitSelectionInput();
  obj.targetSpecList && jspb.Message.setRepeatedWrapperField(
      msg, 1, obj.targetSpecList.map(
          proto.speech.tts.Specification.fromObject));
  obj.preselection && jspb.Message.setWrapperField(
      msg, 2, proto.speech.tts.LatticeProto.fromObject(obj.preselection));
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.UnitSelectionInput}
 */
proto.speech.tts.UnitSelectionInput.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.UnitSelectionInput;
  return proto.speech.tts.UnitSelectionInput.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.UnitSelectionInput} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.UnitSelectionInput}
 */
proto.speech.tts.UnitSelectionInput.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.speech.tts.Specification;
      reader.readMessage(value,proto.speech.tts.Specification.deserializeBinaryFromReader);
      msg.addTargetSpec(value);
      break;
    case 2:
      var value = new proto.speech.tts.LatticeProto;
      reader.readMessage(value,proto.speech.tts.LatticeProto.deserializeBinaryFromReader);
      msg.setPreselection(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.UnitSelectionInput.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.UnitSelectionInput.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.UnitSelectionInput} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.UnitSelectionInput.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getTargetSpecList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      1,
      f,
      proto.speech.tts.Specification.serializeBinaryToWriter
    );
  }
  f = message.getPreselection();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.speech.tts.LatticeProto.serializeBinaryToWriter
    );
  }
};


/**
 * repeated Specification target_spec = 1;
 * @return {!Array<!proto.speech.tts.Specification>}
 */
proto.speech.tts.UnitSelectionInput.prototype.getTargetSpecList = function() {
  return /** @type{!Array<!proto.speech.tts.Specification>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.Specification, 1));
};


/**
 * @param {Array<!proto.speech.tts.Specification>|undefined} value
 * @return {!proto.speech.tts.UnitSelectionInput} returns this
*/
proto.speech.tts.UnitSelectionInput.prototype.setTargetSpecList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 1, value);
};


/**
 * @param {!proto.speech.tts.Specification=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.Specification}
 */
proto.speech.tts.UnitSelectionInput.prototype.addTargetSpec = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.speech.tts.Specification, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.UnitSelectionInput} returns this
 */
proto.speech.tts.UnitSelectionInput.prototype.clearTargetSpecList = function() {
  return this.setTargetSpecList([]);
};


/**
 * optional LatticeProto preselection = 2;
 * @return {?proto.speech.tts.LatticeProto}
 */
proto.speech.tts.UnitSelectionInput.prototype.getPreselection = function() {
  return /** @type{?proto.speech.tts.LatticeProto} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.LatticeProto, 2));
};


/**
 * @param {?proto.speech.tts.LatticeProto|undefined} value
 * @return {!proto.speech.tts.UnitSelectionInput} returns this
*/
proto.speech.tts.UnitSelectionInput.prototype.setPreselection = function(value) {
  return jspb.Message.setWrapperField(this, 2, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.UnitSelectionInput} returns this
 */
proto.speech.tts.UnitSelectionInput.prototype.clearPreselection = function() {
  return this.setPreselection(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.UnitSelectionInput.prototype.hasPreselection = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.UnitSelectionInput}
 */
proto.speech.tts.UnitSelectionInput.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.UnitSelectionInput, data);
};



//speech/tts/engine/unit_selection/voice/unit_proto_jspb/unit.jspb.js
// source: speech/tts/engine/unit_selection/voice/unit.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.Unit');
goog.provide('proto.speech.tts.Unit.Acoustics');
goog.provide('proto.speech.tts.Unit.Property');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.Unit = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 10, proto.speech.tts.Unit.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.Unit, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.Unit.displayName = 'proto.speech.tts.Unit';
}

/**
 * The extensions registered with this message class. This is a map of
 * extension field number to fieldInfo object.
 *
 * For example:
 *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }
 *
 * fieldName contains the JsCompiler renamed field name property so that it
 * works in OPTIMIZED mode.
 *
 * @type {!Object<number, jspb.ExtensionFieldInfo>}
 */
proto.speech.tts.Unit.extensions = {};


/**
 * The extensions registered with this message class. This is a map of
 * extension field number to fieldInfo object.
 *
 * For example:
 *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }
 *
 * fieldName contains the JsCompiler renamed field name property so that it
 * works in OPTIMIZED mode.
 *
 * @type {!Object<number, jspb.ExtensionFieldBinaryInfo>}
 */
proto.speech.tts.Unit.extensionsBinary = {};

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.Unit.Acoustics = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.Unit.Acoustics.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.Unit.Acoustics, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.Unit.Acoustics.displayName = 'proto.speech.tts.Unit.Acoustics';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.Unit.repeatedFields_ = [6];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.Unit.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.Unit.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.Unit} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Unit.toObject = function(includeInstance, msg) {
  var f, obj = {
    id: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    prevUnitId: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    filenameIndex: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,
    modelId: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f,
    acoustics: (f = msg.getAcoustics()) && proto.speech.tts.Unit.Acoustics.toObject(includeInstance, f),
    propertiesList: (f = jspb.Message.getRepeatedField(msg, 6)) == null ? undefined : f,
    lastUnitIdInLine: (f = jspb.Message.getField(msg, 7)) == null ? undefined : f,
    specificationId: (f = jspb.Message.getField(msg, 8)) == null ? undefined : f,
    cafIndex: (f = jspb.Message.getField(msg, 9)) == null ? undefined : f
  };

  jspb.Message.toObjectExtension(/** @type {!jspb.Message} */ (msg), obj,
      proto.speech.tts.Unit.extensions, proto.speech.tts.Unit.prototype.getExtension,
      includeInstance);
  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Unit as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.Unit.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.id;

  /** @type {?number|undefined} */
  this.prevUnitId;

  /** @type {?number|undefined} */
  this.filenameIndex;

  /** @type {?number|undefined} */
  this.modelId;

  /** @type {?proto.speech.tts.Unit.Acoustics.ObjectFormat|undefined} */
  this.acoustics;

  /** @type {?Array<number>|undefined} */
  this.propertiesList;

  /** @type {?number|undefined} */
  this.lastUnitIdInLine;

  /** @type {?number|undefined} */
  this.specificationId;

  /** @type {?number|undefined} */
  this.cafIndex;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.Unit.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.Unit}
 */
proto.speech.tts.Unit.fromObject = function(obj) {
  var msg = new proto.speech.tts.Unit();
  obj.id != null && jspb.Message.setField(msg, 1, obj.id);
  obj.prevUnitId != null && jspb.Message.setField(msg, 2, obj.prevUnitId);
  obj.filenameIndex != null && jspb.Message.setField(msg, 3, obj.filenameIndex);
  obj.modelId != null && jspb.Message.setField(msg, 4, obj.modelId);
  obj.acoustics && jspb.Message.setWrapperField(
      msg, 5, proto.speech.tts.Unit.Acoustics.fromObject(obj.acoustics));
  obj.propertiesList != null && jspb.Message.setField(msg, 6, obj.propertiesList);
  obj.lastUnitIdInLine != null && jspb.Message.setField(msg, 7, obj.lastUnitIdInLine);
  obj.specificationId != null && jspb.Message.setField(msg, 8, obj.specificationId);
  obj.cafIndex != null && jspb.Message.setField(msg, 9, obj.cafIndex);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.Unit}
 */
proto.speech.tts.Unit.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.Unit;
  return proto.speech.tts.Unit.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.Unit} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.Unit}
 */
proto.speech.tts.Unit.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setId(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setPrevUnitId(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setFilenameIndex(value);
      break;
    case 4:
      var value = /** @type {number} */ (reader.readUint32());
      msg.setModelId(value);
      break;
    case 5:
      var value = new proto.speech.tts.Unit.Acoustics;
      reader.readMessage(value,proto.speech.tts.Unit.Acoustics.deserializeBinaryFromReader);
      msg.setAcoustics(value);
      break;
    case 6:
      var value = /** @type {!proto.speech.tts.Unit.Property} */ (reader.readEnum());
      msg.addProperties(value);
      break;
    case 7:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setLastUnitIdInLine(value);
      break;
    case 8:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setSpecificationId(value);
      break;
    case 9:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setCafIndex(value);
      break;
    default:
      jspb.Message.readBinaryExtension(msg, reader,
        proto.speech.tts.Unit.extensionsBinary,
        proto.speech.tts.Unit.prototype.getExtension,
        proto.speech.tts.Unit.prototype.setExtension);
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.Unit.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.Unit.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.Unit} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Unit.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?number} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeInt32(
      1,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeInt32(
      2,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeInt32(
      3,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeUint32(
      4,
      f
    );
  }
  f = message.getAcoustics();
  if (f != null) {
    writer.writeMessage(
      5,
      f,
      proto.speech.tts.Unit.Acoustics.serializeBinaryToWriter
    );
  }
  f = message.getPropertiesList();
  if (f.length > 0) {
    writer.writeRepeatedEnum(
      6,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 7));
  if (f != null) {
    writer.writeInt32(
      7,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 8));
  if (f != null) {
    writer.writeInt32(
      8,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 9));
  if (f != null) {
    writer.writeInt32(
      9,
      f
    );
  }
  jspb.Message.serializeBinaryExtensions(message, writer,
    proto.speech.tts.Unit.extensionsBinary, proto.speech.tts.Unit.prototype.getExtension);
};


/**
 * @enum {number}
 */
proto.speech.tts.Unit.Property = {
  INVALID_PROPERTY: 0,
  PLAYBACK_ONLY: 1,
  VUI_ID_MATCH_ONLY: 2
};


/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.Unit.Acoustics.repeatedFields_ = [19,20,13,14,15,16,17,18];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.Unit.Acoustics.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.Unit.Acoustics.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.Unit.Acoustics} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Unit.Acoustics.toObject = function(includeInstance, msg) {
  var f, obj = {
    startPitchmark: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    numPitchmarksToMid: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    numPitchmarksToEnd: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,
    durationLeftMs: (f = jspb.Message.getOptionalFloatingPointField(msg, 4)) == null ? undefined : f,
    durationRightMs: (f = jspb.Message.getOptionalFloatingPointField(msg, 5)) == null ? undefined : f,
    f0First: (f = jspb.Message.getOptionalFloatingPointField(msg, 6)) == null ? undefined : f,
    f0Last: (f = jspb.Message.getOptionalFloatingPointField(msg, 7)) == null ? undefined : f,
    f0Mid: (f = jspb.Message.getOptionalFloatingPointField(msg, 8)) == null ? undefined : f,
    f0Leftmid: (f = jspb.Message.getOptionalFloatingPointField(msg, 9)) == null ? undefined : f,
    f0Rightmid: (f = jspb.Message.getOptionalFloatingPointField(msg, 10)) == null ? undefined : f,
    f0DiffFirst: (f = jspb.Message.getOptionalFloatingPointField(msg, 11)) == null ? undefined : f,
    f0DiffLast: (f = jspb.Message.getOptionalFloatingPointField(msg, 12)) == null ? undefined : f,
    intFrameMfcFirstList: (f = jspb.Message.getRepeatedField(msg, 19)) == null ? undefined : f,
    intFrameMfcLastList: (f = jspb.Message.getRepeatedField(msg, 20)) == null ? undefined : f,
    frameMfcFirstList: (f = jspb.Message.getRepeatedFloatingPointField(msg, 13)) == null ? undefined : f,
    frameMfcLastList: (f = jspb.Message.getRepeatedFloatingPointField(msg, 14)) == null ? undefined : f,
    frameDiffMfcFirstList: (f = jspb.Message.getRepeatedFloatingPointField(msg, 15)) == null ? undefined : f,
    frameDiffMfcLastList: (f = jspb.Message.getRepeatedFloatingPointField(msg, 16)) == null ? undefined : f,
    frameMfcLeftList: (f = jspb.Message.getRepeatedFloatingPointField(msg, 17)) == null ? undefined : f,
    frameMfcRightList: (f = jspb.Message.getRepeatedFloatingPointField(msg, 18)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Acoustics as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.Unit.Acoustics.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.startPitchmark;

  /** @type {?number|undefined} */
  this.numPitchmarksToMid;

  /** @type {?number|undefined} */
  this.numPitchmarksToEnd;

  /** @type {?number|undefined} */
  this.durationLeftMs;

  /** @type {?number|undefined} */
  this.durationRightMs;

  /** @type {?number|undefined} */
  this.f0First;

  /** @type {?number|undefined} */
  this.f0Last;

  /** @type {?number|undefined} */
  this.f0Mid;

  /** @type {?number|undefined} */
  this.f0Leftmid;

  /** @type {?number|undefined} */
  this.f0Rightmid;

  /** @type {?number|undefined} */
  this.f0DiffFirst;

  /** @type {?number|undefined} */
  this.f0DiffLast;

  /** @type {?Array<number>|undefined} */
  this.intFrameMfcFirstList;

  /** @type {?Array<number>|undefined} */
  this.intFrameMfcLastList;

  /** @type {?Array<number>|undefined} */
  this.frameMfcFirstList;

  /** @type {?Array<number>|undefined} */
  this.frameMfcLastList;

  /** @type {?Array<number>|undefined} */
  this.frameDiffMfcFirstList;

  /** @type {?Array<number>|undefined} */
  this.frameDiffMfcLastList;

  /** @type {?Array<number>|undefined} */
  this.frameMfcLeftList;

  /** @type {?Array<number>|undefined} */
  this.frameMfcRightList;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.Unit.Acoustics.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.Unit.Acoustics}
 */
proto.speech.tts.Unit.Acoustics.fromObject = function(obj) {
  var msg = new proto.speech.tts.Unit.Acoustics();
  obj.startPitchmark != null && jspb.Message.setField(msg, 1, obj.startPitchmark);
  obj.numPitchmarksToMid != null && jspb.Message.setField(msg, 2, obj.numPitchmarksToMid);
  obj.numPitchmarksToEnd != null && jspb.Message.setField(msg, 3, obj.numPitchmarksToEnd);
  obj.durationLeftMs != null && jspb.Message.setField(msg, 4, obj.durationLeftMs);
  obj.durationRightMs != null && jspb.Message.setField(msg, 5, obj.durationRightMs);
  obj.f0First != null && jspb.Message.setField(msg, 6, obj.f0First);
  obj.f0Last != null && jspb.Message.setField(msg, 7, obj.f0Last);
  obj.f0Mid != null && jspb.Message.setField(msg, 8, obj.f0Mid);
  obj.f0Leftmid != null && jspb.Message.setField(msg, 9, obj.f0Leftmid);
  obj.f0Rightmid != null && jspb.Message.setField(msg, 10, obj.f0Rightmid);
  obj.f0DiffFirst != null && jspb.Message.setField(msg, 11, obj.f0DiffFirst);
  obj.f0DiffLast != null && jspb.Message.setField(msg, 12, obj.f0DiffLast);
  obj.intFrameMfcFirstList != null && jspb.Message.setField(msg, 19, obj.intFrameMfcFirstList);
  obj.intFrameMfcLastList != null && jspb.Message.setField(msg, 20, obj.intFrameMfcLastList);
  obj.frameMfcFirstList != null && jspb.Message.setField(msg, 13, obj.frameMfcFirstList);
  obj.frameMfcLastList != null && jspb.Message.setField(msg, 14, obj.frameMfcLastList);
  obj.frameDiffMfcFirstList != null && jspb.Message.setField(msg, 15, obj.frameDiffMfcFirstList);
  obj.frameDiffMfcLastList != null && jspb.Message.setField(msg, 16, obj.frameDiffMfcLastList);
  obj.frameMfcLeftList != null && jspb.Message.setField(msg, 17, obj.frameMfcLeftList);
  obj.frameMfcRightList != null && jspb.Message.setField(msg, 18, obj.frameMfcRightList);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.Unit.Acoustics}
 */
proto.speech.tts.Unit.Acoustics.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.Unit.Acoustics;
  return proto.speech.tts.Unit.Acoustics.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.Unit.Acoustics} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.Unit.Acoustics}
 */
proto.speech.tts.Unit.Acoustics.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {number} */ (reader.readUint32());
      msg.setStartPitchmark(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readUint32());
      msg.setNumPitchmarksToMid(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readUint32());
      msg.setNumPitchmarksToEnd(value);
      break;
    case 4:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setDurationLeftMs(value);
      break;
    case 5:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setDurationRightMs(value);
      break;
    case 6:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setF0First(value);
      break;
    case 7:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setF0Last(value);
      break;
    case 8:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setF0Mid(value);
      break;
    case 9:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setF0Leftmid(value);
      break;
    case 10:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setF0Rightmid(value);
      break;
    case 11:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setF0DiffFirst(value);
      break;
    case 12:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setF0DiffLast(value);
      break;
    case 19:
      var value = /** @type {number} */ (reader.readSint32());
      msg.addIntFrameMfcFirst(value);
      break;
    case 20:
      var value = /** @type {number} */ (reader.readSint32());
      msg.addIntFrameMfcLast(value);
      break;
    case 13:
      var value = /** @type {number} */ (reader.readFloat());
      msg.addFrameMfcFirst(value);
      break;
    case 14:
      var value = /** @type {number} */ (reader.readFloat());
      msg.addFrameMfcLast(value);
      break;
    case 15:
      var value = /** @type {number} */ (reader.readFloat());
      msg.addFrameDiffMfcFirst(value);
      break;
    case 16:
      var value = /** @type {number} */ (reader.readFloat());
      msg.addFrameDiffMfcLast(value);
      break;
    case 17:
      var value = /** @type {number} */ (reader.readFloat());
      msg.addFrameMfcLeft(value);
      break;
    case 18:
      var value = /** @type {number} */ (reader.readFloat());
      msg.addFrameMfcRight(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.Unit.Acoustics.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.Unit.Acoustics.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.Unit.Acoustics} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Unit.Acoustics.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?number} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeUint32(
      1,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeUint32(
      2,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeUint32(
      3,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeFloat(
      4,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeFloat(
      5,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeFloat(
      6,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 7));
  if (f != null) {
    writer.writeFloat(
      7,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 8));
  if (f != null) {
    writer.writeFloat(
      8,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 9));
  if (f != null) {
    writer.writeFloat(
      9,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 10));
  if (f != null) {
    writer.writeFloat(
      10,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 11));
  if (f != null) {
    writer.writeFloat(
      11,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 12));
  if (f != null) {
    writer.writeFloat(
      12,
      f
    );
  }
  f = message.getIntFrameMfcFirstList();
  if (f.length > 0) {
    writer.writeRepeatedSint32(
      19,
      f
    );
  }
  f = message.getIntFrameMfcLastList();
  if (f.length > 0) {
    writer.writeRepeatedSint32(
      20,
      f
    );
  }
  f = message.getFrameMfcFirstList();
  if (f.length > 0) {
    writer.writeRepeatedFloat(
      13,
      f
    );
  }
  f = message.getFrameMfcLastList();
  if (f.length > 0) {
    writer.writeRepeatedFloat(
      14,
      f
    );
  }
  f = message.getFrameDiffMfcFirstList();
  if (f.length > 0) {
    writer.writeRepeatedFloat(
      15,
      f
    );
  }
  f = message.getFrameDiffMfcLastList();
  if (f.length > 0) {
    writer.writeRepeatedFloat(
      16,
      f
    );
  }
  f = message.getFrameMfcLeftList();
  if (f.length > 0) {
    writer.writeRepeatedFloat(
      17,
      f
    );
  }
  f = message.getFrameMfcRightList();
  if (f.length > 0) {
    writer.writeRepeatedFloat(
      18,
      f
    );
  }
};


/**
 * optional uint32 start_pitchmark = 1;
 * @return {?number}
 */
proto.speech.tts.Unit.Acoustics.prototype.getStartPitchmark = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Unit.Acoustics} returns this
 */
proto.speech.tts.Unit.Acoustics.prototype.setStartPitchmark = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Unit.Acoustics} returns this
 */
proto.speech.tts.Unit.Acoustics.prototype.clearStartPitchmark = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Unit.Acoustics.prototype.hasStartPitchmark = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional uint32 num_pitchmarks_to_mid = 2;
 * @return {?number}
 */
proto.speech.tts.Unit.Acoustics.prototype.getNumPitchmarksToMid = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Unit.Acoustics} returns this
 */
proto.speech.tts.Unit.Acoustics.prototype.setNumPitchmarksToMid = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Unit.Acoustics} returns this
 */
proto.speech.tts.Unit.Acoustics.prototype.clearNumPitchmarksToMid = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Unit.Acoustics.prototype.hasNumPitchmarksToMid = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional uint32 num_pitchmarks_to_end = 3;
 * @return {?number}
 */
proto.speech.tts.Unit.Acoustics.prototype.getNumPitchmarksToEnd = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Unit.Acoustics} returns this
 */
proto.speech.tts.Unit.Acoustics.prototype.setNumPitchmarksToEnd = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Unit.Acoustics} returns this
 */
proto.speech.tts.Unit.Acoustics.prototype.clearNumPitchmarksToEnd = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Unit.Acoustics.prototype.hasNumPitchmarksToEnd = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional float duration_left_ms = 4;
 * @return {?number}
 */
proto.speech.tts.Unit.Acoustics.prototype.getDurationLeftMs = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 4));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Unit.Acoustics} returns this
 */
proto.speech.tts.Unit.Acoustics.prototype.setDurationLeftMs = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Unit.Acoustics} returns this
 */
proto.speech.tts.Unit.Acoustics.prototype.clearDurationLeftMs = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Unit.Acoustics.prototype.hasDurationLeftMs = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional float duration_right_ms = 5;
 * @return {?number}
 */
proto.speech.tts.Unit.Acoustics.prototype.getDurationRightMs = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 5));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Unit.Acoustics} returns this
 */
proto.speech.tts.Unit.Acoustics.prototype.setDurationRightMs = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Unit.Acoustics} returns this
 */
proto.speech.tts.Unit.Acoustics.prototype.clearDurationRightMs = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Unit.Acoustics.prototype.hasDurationRightMs = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional float f0_first = 6;
 * @return {?number}
 */
proto.speech.tts.Unit.Acoustics.prototype.getF0First = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 6));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Unit.Acoustics} returns this
 */
proto.speech.tts.Unit.Acoustics.prototype.setF0First = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Unit.Acoustics} returns this
 */
proto.speech.tts.Unit.Acoustics.prototype.clearF0First = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Unit.Acoustics.prototype.hasF0First = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional float f0_last = 7;
 * @return {?number}
 */
proto.speech.tts.Unit.Acoustics.prototype.getF0Last = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 7));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Unit.Acoustics} returns this
 */
proto.speech.tts.Unit.Acoustics.prototype.setF0Last = function(value) {
  return jspb.Message.setField(this, 7, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Unit.Acoustics} returns this
 */
proto.speech.tts.Unit.Acoustics.prototype.clearF0Last = function() {
  return jspb.Message.setField(this, 7, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Unit.Acoustics.prototype.hasF0Last = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * optional float f0_mid = 8;
 * @return {?number}
 */
proto.speech.tts.Unit.Acoustics.prototype.getF0Mid = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 8));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Unit.Acoustics} returns this
 */
proto.speech.tts.Unit.Acoustics.prototype.setF0Mid = function(value) {
  return jspb.Message.setField(this, 8, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Unit.Acoustics} returns this
 */
proto.speech.tts.Unit.Acoustics.prototype.clearF0Mid = function() {
  return jspb.Message.setField(this, 8, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Unit.Acoustics.prototype.hasF0Mid = function() {
  return jspb.Message.getField(this, 8) != null;
};


/**
 * optional float f0_leftmid = 9;
 * @return {?number}
 */
proto.speech.tts.Unit.Acoustics.prototype.getF0Leftmid = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 9));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Unit.Acoustics} returns this
 */
proto.speech.tts.Unit.Acoustics.prototype.setF0Leftmid = function(value) {
  return jspb.Message.setField(this, 9, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Unit.Acoustics} returns this
 */
proto.speech.tts.Unit.Acoustics.prototype.clearF0Leftmid = function() {
  return jspb.Message.setField(this, 9, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Unit.Acoustics.prototype.hasF0Leftmid = function() {
  return jspb.Message.getField(this, 9) != null;
};


/**
 * optional float f0_rightmid = 10;
 * @return {?number}
 */
proto.speech.tts.Unit.Acoustics.prototype.getF0Rightmid = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 10));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Unit.Acoustics} returns this
 */
proto.speech.tts.Unit.Acoustics.prototype.setF0Rightmid = function(value) {
  return jspb.Message.setField(this, 10, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Unit.Acoustics} returns this
 */
proto.speech.tts.Unit.Acoustics.prototype.clearF0Rightmid = function() {
  return jspb.Message.setField(this, 10, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Unit.Acoustics.prototype.hasF0Rightmid = function() {
  return jspb.Message.getField(this, 10) != null;
};


/**
 * optional float f0_diff_first = 11;
 * @return {?number}
 */
proto.speech.tts.Unit.Acoustics.prototype.getF0DiffFirst = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 11));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Unit.Acoustics} returns this
 */
proto.speech.tts.Unit.Acoustics.prototype.setF0DiffFirst = function(value) {
  return jspb.Message.setField(this, 11, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Unit.Acoustics} returns this
 */
proto.speech.tts.Unit.Acoustics.prototype.clearF0DiffFirst = function() {
  return jspb.Message.setField(this, 11, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Unit.Acoustics.prototype.hasF0DiffFirst = function() {
  return jspb.Message.getField(this, 11) != null;
};


/**
 * optional float f0_diff_last = 12;
 * @return {?number}
 */
proto.speech.tts.Unit.Acoustics.prototype.getF0DiffLast = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 12));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Unit.Acoustics} returns this
 */
proto.speech.tts.Unit.Acoustics.prototype.setF0DiffLast = function(value) {
  return jspb.Message.setField(this, 12, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Unit.Acoustics} returns this
 */
proto.speech.tts.Unit.Acoustics.prototype.clearF0DiffLast = function() {
  return jspb.Message.setField(this, 12, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Unit.Acoustics.prototype.hasF0DiffLast = function() {
  return jspb.Message.getField(this, 12) != null;
};


/**
 * repeated sint32 int_frame_mfc_first = 19;
 * @return {!Array<number>}
 */
proto.speech.tts.Unit.Acoustics.prototype.getIntFrameMfcFirstList = function() {
  return /** @type {!Array<number>} */ (jspb.Message.getRepeatedField(this, 19));
};


/**
 * @param {Array<number>|undefined} value
 * @return {!proto.speech.tts.Unit.Acoustics} returns this
 */
proto.speech.tts.Unit.Acoustics.prototype.setIntFrameMfcFirstList = function(value) {
  return jspb.Message.setField(this, 19, value || []);
};


/**
 * @param {number} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.Unit.Acoustics} returns this
 */
proto.speech.tts.Unit.Acoustics.prototype.addIntFrameMfcFirst = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 19, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.Unit.Acoustics} returns this
 */
proto.speech.tts.Unit.Acoustics.prototype.clearIntFrameMfcFirstList = function() {
  return this.setIntFrameMfcFirstList([]);
};


/**
 * repeated sint32 int_frame_mfc_last = 20;
 * @return {!Array<number>}
 */
proto.speech.tts.Unit.Acoustics.prototype.getIntFrameMfcLastList = function() {
  return /** @type {!Array<number>} */ (jspb.Message.getRepeatedField(this, 20));
};


/**
 * @param {Array<number>|undefined} value
 * @return {!proto.speech.tts.Unit.Acoustics} returns this
 */
proto.speech.tts.Unit.Acoustics.prototype.setIntFrameMfcLastList = function(value) {
  return jspb.Message.setField(this, 20, value || []);
};


/**
 * @param {number} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.Unit.Acoustics} returns this
 */
proto.speech.tts.Unit.Acoustics.prototype.addIntFrameMfcLast = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 20, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.Unit.Acoustics} returns this
 */
proto.speech.tts.Unit.Acoustics.prototype.clearIntFrameMfcLastList = function() {
  return this.setIntFrameMfcLastList([]);
};


/**
 * repeated float frame_mfc_first = 13;
 * @return {!Array<number>}
 */
proto.speech.tts.Unit.Acoustics.prototype.getFrameMfcFirstList = function() {
  return /** @type {!Array<number>} */ (jspb.Message.getRepeatedFloatingPointField(this, 13));
};


/**
 * @param {Array<number>|undefined} value
 * @return {!proto.speech.tts.Unit.Acoustics} returns this
 */
proto.speech.tts.Unit.Acoustics.prototype.setFrameMfcFirstList = function(value) {
  return jspb.Message.setField(this, 13, value || []);
};


/**
 * @param {number} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.Unit.Acoustics} returns this
 */
proto.speech.tts.Unit.Acoustics.prototype.addFrameMfcFirst = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 13, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.Unit.Acoustics} returns this
 */
proto.speech.tts.Unit.Acoustics.prototype.clearFrameMfcFirstList = function() {
  return this.setFrameMfcFirstList([]);
};


/**
 * repeated float frame_mfc_last = 14;
 * @return {!Array<number>}
 */
proto.speech.tts.Unit.Acoustics.prototype.getFrameMfcLastList = function() {
  return /** @type {!Array<number>} */ (jspb.Message.getRepeatedFloatingPointField(this, 14));
};


/**
 * @param {Array<number>|undefined} value
 * @return {!proto.speech.tts.Unit.Acoustics} returns this
 */
proto.speech.tts.Unit.Acoustics.prototype.setFrameMfcLastList = function(value) {
  return jspb.Message.setField(this, 14, value || []);
};


/**
 * @param {number} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.Unit.Acoustics} returns this
 */
proto.speech.tts.Unit.Acoustics.prototype.addFrameMfcLast = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 14, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.Unit.Acoustics} returns this
 */
proto.speech.tts.Unit.Acoustics.prototype.clearFrameMfcLastList = function() {
  return this.setFrameMfcLastList([]);
};


/**
 * repeated float frame_diff_mfc_first = 15;
 * @return {!Array<number>}
 */
proto.speech.tts.Unit.Acoustics.prototype.getFrameDiffMfcFirstList = function() {
  return /** @type {!Array<number>} */ (jspb.Message.getRepeatedFloatingPointField(this, 15));
};


/**
 * @param {Array<number>|undefined} value
 * @return {!proto.speech.tts.Unit.Acoustics} returns this
 */
proto.speech.tts.Unit.Acoustics.prototype.setFrameDiffMfcFirstList = function(value) {
  return jspb.Message.setField(this, 15, value || []);
};


/**
 * @param {number} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.Unit.Acoustics} returns this
 */
proto.speech.tts.Unit.Acoustics.prototype.addFrameDiffMfcFirst = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 15, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.Unit.Acoustics} returns this
 */
proto.speech.tts.Unit.Acoustics.prototype.clearFrameDiffMfcFirstList = function() {
  return this.setFrameDiffMfcFirstList([]);
};


/**
 * repeated float frame_diff_mfc_last = 16;
 * @return {!Array<number>}
 */
proto.speech.tts.Unit.Acoustics.prototype.getFrameDiffMfcLastList = function() {
  return /** @type {!Array<number>} */ (jspb.Message.getRepeatedFloatingPointField(this, 16));
};


/**
 * @param {Array<number>|undefined} value
 * @return {!proto.speech.tts.Unit.Acoustics} returns this
 */
proto.speech.tts.Unit.Acoustics.prototype.setFrameDiffMfcLastList = function(value) {
  return jspb.Message.setField(this, 16, value || []);
};


/**
 * @param {number} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.Unit.Acoustics} returns this
 */
proto.speech.tts.Unit.Acoustics.prototype.addFrameDiffMfcLast = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 16, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.Unit.Acoustics} returns this
 */
proto.speech.tts.Unit.Acoustics.prototype.clearFrameDiffMfcLastList = function() {
  return this.setFrameDiffMfcLastList([]);
};


/**
 * repeated float frame_mfc_left = 17;
 * @return {!Array<number>}
 */
proto.speech.tts.Unit.Acoustics.prototype.getFrameMfcLeftList = function() {
  return /** @type {!Array<number>} */ (jspb.Message.getRepeatedFloatingPointField(this, 17));
};


/**
 * @param {Array<number>|undefined} value
 * @return {!proto.speech.tts.Unit.Acoustics} returns this
 */
proto.speech.tts.Unit.Acoustics.prototype.setFrameMfcLeftList = function(value) {
  return jspb.Message.setField(this, 17, value || []);
};


/**
 * @param {number} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.Unit.Acoustics} returns this
 */
proto.speech.tts.Unit.Acoustics.prototype.addFrameMfcLeft = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 17, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.Unit.Acoustics} returns this
 */
proto.speech.tts.Unit.Acoustics.prototype.clearFrameMfcLeftList = function() {
  return this.setFrameMfcLeftList([]);
};


/**
 * repeated float frame_mfc_right = 18;
 * @return {!Array<number>}
 */
proto.speech.tts.Unit.Acoustics.prototype.getFrameMfcRightList = function() {
  return /** @type {!Array<number>} */ (jspb.Message.getRepeatedFloatingPointField(this, 18));
};


/**
 * @param {Array<number>|undefined} value
 * @return {!proto.speech.tts.Unit.Acoustics} returns this
 */
proto.speech.tts.Unit.Acoustics.prototype.setFrameMfcRightList = function(value) {
  return jspb.Message.setField(this, 18, value || []);
};


/**
 * @param {number} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.Unit.Acoustics} returns this
 */
proto.speech.tts.Unit.Acoustics.prototype.addFrameMfcRight = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 18, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.Unit.Acoustics} returns this
 */
proto.speech.tts.Unit.Acoustics.prototype.clearFrameMfcRightList = function() {
  return this.setFrameMfcRightList([]);
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.Unit.Acoustics}
 */
proto.speech.tts.Unit.Acoustics.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.Unit.Acoustics, data);
};


/**
 * optional int32 id = 1;
 * @return {?number}
 */
proto.speech.tts.Unit.prototype.getId = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Unit} returns this
 */
proto.speech.tts.Unit.prototype.setId = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Unit} returns this
 */
proto.speech.tts.Unit.prototype.clearId = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Unit.prototype.hasId = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional int32 prev_unit_id = 2;
 * @return {?number}
 */
proto.speech.tts.Unit.prototype.getPrevUnitId = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Unit} returns this
 */
proto.speech.tts.Unit.prototype.setPrevUnitId = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Unit} returns this
 */
proto.speech.tts.Unit.prototype.clearPrevUnitId = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Unit.prototype.hasPrevUnitId = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional int32 filename_index = 3;
 * @return {?number}
 */
proto.speech.tts.Unit.prototype.getFilenameIndex = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Unit} returns this
 */
proto.speech.tts.Unit.prototype.setFilenameIndex = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Unit} returns this
 */
proto.speech.tts.Unit.prototype.clearFilenameIndex = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Unit.prototype.hasFilenameIndex = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional uint32 model_id = 4;
 * @return {?number}
 */
proto.speech.tts.Unit.prototype.getModelId = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 4));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Unit} returns this
 */
proto.speech.tts.Unit.prototype.setModelId = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Unit} returns this
 */
proto.speech.tts.Unit.prototype.clearModelId = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Unit.prototype.hasModelId = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional Acoustics acoustics = 5;
 * @return {?proto.speech.tts.Unit.Acoustics}
 */
proto.speech.tts.Unit.prototype.getAcoustics = function() {
  return /** @type{?proto.speech.tts.Unit.Acoustics} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Unit.Acoustics, 5));
};


/**
 * @param {?proto.speech.tts.Unit.Acoustics|undefined} value
 * @return {!proto.speech.tts.Unit} returns this
*/
proto.speech.tts.Unit.prototype.setAcoustics = function(value) {
  return jspb.Message.setWrapperField(this, 5, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Unit} returns this
 */
proto.speech.tts.Unit.prototype.clearAcoustics = function() {
  return this.setAcoustics(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Unit.prototype.hasAcoustics = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * repeated Property properties = 6;
 * @return {!Array<!proto.speech.tts.Unit.Property>}
 */
proto.speech.tts.Unit.prototype.getPropertiesList = function() {
  return /** @type {!Array<!proto.speech.tts.Unit.Property>} */ (jspb.Message.getRepeatedField(this, 6));
};


/**
 * @param {Array<!proto.speech.tts.Unit.Property>|undefined} value
 * @return {!proto.speech.tts.Unit} returns this
 */
proto.speech.tts.Unit.prototype.setPropertiesList = function(value) {
  return jspb.Message.setField(this, 6, value || []);
};


/**
 * @param {!proto.speech.tts.Unit.Property} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.Unit} returns this
 */
proto.speech.tts.Unit.prototype.addProperties = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 6, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.Unit} returns this
 */
proto.speech.tts.Unit.prototype.clearPropertiesList = function() {
  return this.setPropertiesList([]);
};


/**
 * optional int32 last_unit_id_in_line = 7;
 * @return {?number}
 */
proto.speech.tts.Unit.prototype.getLastUnitIdInLine = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 7));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Unit} returns this
 */
proto.speech.tts.Unit.prototype.setLastUnitIdInLine = function(value) {
  return jspb.Message.setField(this, 7, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Unit} returns this
 */
proto.speech.tts.Unit.prototype.clearLastUnitIdInLine = function() {
  return jspb.Message.setField(this, 7, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Unit.prototype.hasLastUnitIdInLine = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * optional int32 specification_id = 8;
 * @return {?number}
 */
proto.speech.tts.Unit.prototype.getSpecificationId = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 8));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Unit} returns this
 */
proto.speech.tts.Unit.prototype.setSpecificationId = function(value) {
  return jspb.Message.setField(this, 8, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Unit} returns this
 */
proto.speech.tts.Unit.prototype.clearSpecificationId = function() {
  return jspb.Message.setField(this, 8, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Unit.prototype.hasSpecificationId = function() {
  return jspb.Message.getField(this, 8) != null;
};


/**
 * optional int32 caf_index = 9;
 * @return {?number}
 */
proto.speech.tts.Unit.prototype.getCafIndex = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 9));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Unit} returns this
 */
proto.speech.tts.Unit.prototype.setCafIndex = function(value) {
  return jspb.Message.setField(this, 9, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Unit} returns this
 */
proto.speech.tts.Unit.prototype.clearCafIndex = function() {
  return jspb.Message.setField(this, 9, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Unit.prototype.hasCafIndex = function() {
  return jspb.Message.getField(this, 9) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.Unit}
 */
proto.speech.tts.Unit.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.Unit, data);
};



//speech/tts/engine/utterance/unit_selection_data_proto_jspb/unitselectionoutput.jspb.js
// source: speech/tts/engine/utterance/unit_selection_data.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.UnitSelectionOutput');
goog.provide('proto.speech.tts.UnitSelectionOutput.SelectedUnit');
goog.provide('proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.Links');
goog.require('proto.speech.tts.TimingInfo');
goog.require('proto.speech.tts.Unit');

goog.forwardDeclare('proto.speech.tts.UnitSelectionCostComponentName');
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.UnitSelectionOutput = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.UnitSelectionOutput.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.UnitSelectionOutput, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.UnitSelectionOutput.displayName = 'proto.speech.tts.UnitSelectionOutput';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.UnitSelectionOutput.SelectedUnit = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.UnitSelectionOutput.SelectedUnit.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.UnitSelectionOutput.SelectedUnit, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.UnitSelectionOutput.SelectedUnit.displayName = 'proto.speech.tts.UnitSelectionOutput.SelectedUnit';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent.displayName = 'proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.UnitSelectionOutput.repeatedFields_ = [2];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.UnitSelectionOutput.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.UnitSelectionOutput.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.UnitSelectionOutput} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.UnitSelectionOutput.toObject = function(includeInstance, msg) {
  var f, obj = {
    pathId: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    selectedList: jspb.Message.toObjectList(msg.getSelectedList(),
    proto.speech.tts.UnitSelectionOutput.SelectedUnit.toObject, includeInstance)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of UnitSelectionOutput as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.UnitSelectionOutput.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.pathId;

  /** @type {?Array<!proto.speech.tts.UnitSelectionOutput.SelectedUnit.ObjectFormat>|undefined} */
  this.selectedList;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.UnitSelectionOutput.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.UnitSelectionOutput}
 */
proto.speech.tts.UnitSelectionOutput.fromObject = function(obj) {
  var msg = new proto.speech.tts.UnitSelectionOutput();
  obj.pathId != null && jspb.Message.setField(msg, 1, obj.pathId);
  obj.selectedList && jspb.Message.setRepeatedWrapperField(
      msg, 2, obj.selectedList.map(
          proto.speech.tts.UnitSelectionOutput.SelectedUnit.fromObject));
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.UnitSelectionOutput}
 */
proto.speech.tts.UnitSelectionOutput.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.UnitSelectionOutput;
  return proto.speech.tts.UnitSelectionOutput.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.UnitSelectionOutput} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.UnitSelectionOutput}
 */
proto.speech.tts.UnitSelectionOutput.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setPathId(value);
      break;
    case 2:
      var value = new proto.speech.tts.UnitSelectionOutput.SelectedUnit;
      reader.readMessage(value,proto.speech.tts.UnitSelectionOutput.SelectedUnit.deserializeBinaryFromReader);
      msg.addSelected(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.UnitSelectionOutput.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.UnitSelectionOutput.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.UnitSelectionOutput} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.UnitSelectionOutput.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?number} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeInt32(
      1,
      f
    );
  }
  f = message.getSelectedList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      2,
      f,
      proto.speech.tts.UnitSelectionOutput.SelectedUnit.serializeBinaryToWriter
    );
  }
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.UnitSelectionOutput.SelectedUnit.repeatedFields_ = [7];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.UnitSelectionOutput.SelectedUnit.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.UnitSelectionOutput.SelectedUnit.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.UnitSelectionOutput.SelectedUnit} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.UnitSelectionOutput.SelectedUnit.toObject = function(includeInstance, msg) {
  var f, obj = {
    links: (f = msg.getLinks()) && proto.speech.tts.Links.toObject(includeInstance, f),
    unit: (f = msg.getUnit()) && proto.speech.tts.Unit.toObject(includeInstance, f),
    timing: (f = msg.getTiming()) && proto.speech.tts.TimingInfo.toObject(includeInstance, f),
    targetCost: (f = jspb.Message.getOptionalFloatingPointField(msg, 4)) == null ? undefined : f,
    joinCost: (f = jspb.Message.getOptionalFloatingPointField(msg, 5)) == null ? undefined : f,
    score: (f = jspb.Message.getOptionalFloatingPointField(msg, 6)) == null ? undefined : f,
    costComponentList: jspb.Message.toObjectList(msg.getCostComponentList(),
    proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent.toObject, includeInstance)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of SelectedUnit as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.UnitSelectionOutput.SelectedUnit.ObjectFormat = function() {
  /** @type {?proto.speech.tts.Links.ObjectFormat|undefined} */
  this.links;

  /** @type {?proto.speech.tts.Unit.ObjectFormat|undefined} */
  this.unit;

  /** @type {?proto.speech.tts.TimingInfo.ObjectFormat|undefined} */
  this.timing;

  /** @type {?number|undefined} */
  this.targetCost;

  /** @type {?number|undefined} */
  this.joinCost;

  /** @type {?number|undefined} */
  this.score;

  /** @type {?Array<!proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent.ObjectFormat>|undefined} */
  this.costComponentList;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.UnitSelectionOutput.SelectedUnit.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.UnitSelectionOutput.SelectedUnit}
 */
proto.speech.tts.UnitSelectionOutput.SelectedUnit.fromObject = function(obj) {
  var msg = new proto.speech.tts.UnitSelectionOutput.SelectedUnit();
  obj.links && jspb.Message.setWrapperField(
      msg, 1, proto.speech.tts.Links.fromObject(obj.links));
  obj.unit && jspb.Message.setWrapperField(
      msg, 2, proto.speech.tts.Unit.fromObject(obj.unit));
  obj.timing && jspb.Message.setWrapperField(
      msg, 3, proto.speech.tts.TimingInfo.fromObject(obj.timing));
  obj.targetCost != null && jspb.Message.setField(msg, 4, obj.targetCost);
  obj.joinCost != null && jspb.Message.setField(msg, 5, obj.joinCost);
  obj.score != null && jspb.Message.setField(msg, 6, obj.score);
  obj.costComponentList && jspb.Message.setRepeatedWrapperField(
      msg, 7, obj.costComponentList.map(
          proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent.fromObject));
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.UnitSelectionOutput.SelectedUnit}
 */
proto.speech.tts.UnitSelectionOutput.SelectedUnit.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.UnitSelectionOutput.SelectedUnit;
  return proto.speech.tts.UnitSelectionOutput.SelectedUnit.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.UnitSelectionOutput.SelectedUnit} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.UnitSelectionOutput.SelectedUnit}
 */
proto.speech.tts.UnitSelectionOutput.SelectedUnit.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.speech.tts.Links;
      reader.readMessage(value,proto.speech.tts.Links.deserializeBinaryFromReader);
      msg.setLinks(value);
      break;
    case 2:
      var value = new proto.speech.tts.Unit;
      reader.readMessage(value,proto.speech.tts.Unit.deserializeBinaryFromReader);
      msg.setUnit(value);
      break;
    case 3:
      var value = new proto.speech.tts.TimingInfo;
      reader.readMessage(value,proto.speech.tts.TimingInfo.deserializeBinaryFromReader);
      msg.setTiming(value);
      break;
    case 4:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setTargetCost(value);
      break;
    case 5:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setJoinCost(value);
      break;
    case 6:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setScore(value);
      break;
    case 7:
      var value = new proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent;
      reader.readMessage(value,proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent.deserializeBinaryFromReader);
      msg.addCostComponent(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.UnitSelectionOutput.SelectedUnit.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.UnitSelectionOutput.SelectedUnit.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.UnitSelectionOutput.SelectedUnit} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.UnitSelectionOutput.SelectedUnit.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getLinks();
  if (f != null) {
    writer.writeMessage(
      1,
      f,
      proto.speech.tts.Links.serializeBinaryToWriter
    );
  }
  f = message.getUnit();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.speech.tts.Unit.serializeBinaryToWriter
    );
  }
  f = message.getTiming();
  if (f != null) {
    writer.writeMessage(
      3,
      f,
      proto.speech.tts.TimingInfo.serializeBinaryToWriter
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeFloat(
      4,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeFloat(
      5,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeFloat(
      6,
      f
    );
  }
  f = message.getCostComponentList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      7,
      f,
      proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent.serializeBinaryToWriter
    );
  }
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent.repeatedFields_ = [2];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent.toObject = function(includeInstance, msg) {
  var f, obj = {
    name: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    costList: (f = jspb.Message.getRepeatedFloatingPointField(msg, 2)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of CostComponent as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.name;

  /** @type {?Array<number>|undefined} */
  this.costList;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent}
 */
proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent.fromObject = function(obj) {
  var msg = new proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent();
  obj.name != null && jspb.Message.setField(msg, 1, obj.name);
  obj.costList != null && jspb.Message.setField(msg, 2, obj.costList);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent}
 */
proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent;
  return proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent}
 */
proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {!proto.speech.tts.UnitSelectionCostComponentName} */ (reader.readEnum());
      msg.setName(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readFloat());
      msg.addCost(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {!proto.speech.tts.UnitSelectionCostComponentName} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeEnum(
      1,
      f
    );
  }
  f = message.getCostList();
  if (f.length > 0) {
    writer.writeRepeatedFloat(
      2,
      f
    );
  }
};


/**
 * optional UnitSelectionCostComponentName name = 1;
 * @return {!proto.speech.tts.UnitSelectionCostComponentName}
 */
proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent.prototype.getName = function() {
  return /** @type {!proto.speech.tts.UnitSelectionCostComponentName} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?proto.speech.tts.UnitSelectionCostComponentName|undefined} value
 * @return {!proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent} returns this
 */
proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent.prototype.setName = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent} returns this
 */
proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent.prototype.clearName = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent.prototype.hasName = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * repeated float cost = 2;
 * @return {!Array<number>}
 */
proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent.prototype.getCostList = function() {
  return /** @type {!Array<number>} */ (jspb.Message.getRepeatedFloatingPointField(this, 2));
};


/**
 * @param {Array<number>|undefined} value
 * @return {!proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent} returns this
 */
proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent.prototype.setCostList = function(value) {
  return jspb.Message.setField(this, 2, value || []);
};


/**
 * @param {number} value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent} returns this
 */
proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent.prototype.addCost = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 2, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent} returns this
 */
proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent.prototype.clearCostList = function() {
  return this.setCostList([]);
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent}
 */
proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent, data);
};


/**
 * optional Links links = 1;
 * @return {?proto.speech.tts.Links}
 */
proto.speech.tts.UnitSelectionOutput.SelectedUnit.prototype.getLinks = function() {
  return /** @type{?proto.speech.tts.Links} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Links, 1));
};


/**
 * @param {?proto.speech.tts.Links|undefined} value
 * @return {!proto.speech.tts.UnitSelectionOutput.SelectedUnit} returns this
*/
proto.speech.tts.UnitSelectionOutput.SelectedUnit.prototype.setLinks = function(value) {
  return jspb.Message.setWrapperField(this, 1, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.UnitSelectionOutput.SelectedUnit} returns this
 */
proto.speech.tts.UnitSelectionOutput.SelectedUnit.prototype.clearLinks = function() {
  return this.setLinks(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.UnitSelectionOutput.SelectedUnit.prototype.hasLinks = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional Unit unit = 2;
 * @return {?proto.speech.tts.Unit}
 */
proto.speech.tts.UnitSelectionOutput.SelectedUnit.prototype.getUnit = function() {
  return /** @type{?proto.speech.tts.Unit} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Unit, 2));
};


/**
 * @param {?proto.speech.tts.Unit|undefined} value
 * @return {!proto.speech.tts.UnitSelectionOutput.SelectedUnit} returns this
*/
proto.speech.tts.UnitSelectionOutput.SelectedUnit.prototype.setUnit = function(value) {
  return jspb.Message.setWrapperField(this, 2, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.UnitSelectionOutput.SelectedUnit} returns this
 */
proto.speech.tts.UnitSelectionOutput.SelectedUnit.prototype.clearUnit = function() {
  return this.setUnit(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.UnitSelectionOutput.SelectedUnit.prototype.hasUnit = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional TimingInfo timing = 3;
 * @return {?proto.speech.tts.TimingInfo}
 */
proto.speech.tts.UnitSelectionOutput.SelectedUnit.prototype.getTiming = function() {
  return /** @type{?proto.speech.tts.TimingInfo} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.TimingInfo, 3));
};


/**
 * @param {?proto.speech.tts.TimingInfo|undefined} value
 * @return {!proto.speech.tts.UnitSelectionOutput.SelectedUnit} returns this
*/
proto.speech.tts.UnitSelectionOutput.SelectedUnit.prototype.setTiming = function(value) {
  return jspb.Message.setWrapperField(this, 3, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.UnitSelectionOutput.SelectedUnit} returns this
 */
proto.speech.tts.UnitSelectionOutput.SelectedUnit.prototype.clearTiming = function() {
  return this.setTiming(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.UnitSelectionOutput.SelectedUnit.prototype.hasTiming = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional float target_cost = 4;
 * @return {?number}
 */
proto.speech.tts.UnitSelectionOutput.SelectedUnit.prototype.getTargetCost = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 4));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.UnitSelectionOutput.SelectedUnit} returns this
 */
proto.speech.tts.UnitSelectionOutput.SelectedUnit.prototype.setTargetCost = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.UnitSelectionOutput.SelectedUnit} returns this
 */
proto.speech.tts.UnitSelectionOutput.SelectedUnit.prototype.clearTargetCost = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.UnitSelectionOutput.SelectedUnit.prototype.hasTargetCost = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional float join_cost = 5;
 * @return {?number}
 */
proto.speech.tts.UnitSelectionOutput.SelectedUnit.prototype.getJoinCost = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 5));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.UnitSelectionOutput.SelectedUnit} returns this
 */
proto.speech.tts.UnitSelectionOutput.SelectedUnit.prototype.setJoinCost = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.UnitSelectionOutput.SelectedUnit} returns this
 */
proto.speech.tts.UnitSelectionOutput.SelectedUnit.prototype.clearJoinCost = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.UnitSelectionOutput.SelectedUnit.prototype.hasJoinCost = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional float score = 6;
 * @return {?number}
 */
proto.speech.tts.UnitSelectionOutput.SelectedUnit.prototype.getScore = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 6));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.UnitSelectionOutput.SelectedUnit} returns this
 */
proto.speech.tts.UnitSelectionOutput.SelectedUnit.prototype.setScore = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.UnitSelectionOutput.SelectedUnit} returns this
 */
proto.speech.tts.UnitSelectionOutput.SelectedUnit.prototype.clearScore = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.UnitSelectionOutput.SelectedUnit.prototype.hasScore = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * repeated CostComponent cost_component = 7;
 * @return {!Array<!proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent>}
 */
proto.speech.tts.UnitSelectionOutput.SelectedUnit.prototype.getCostComponentList = function() {
  return /** @type{!Array<!proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent, 7));
};


/**
 * @param {Array<!proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent>|undefined} value
 * @return {!proto.speech.tts.UnitSelectionOutput.SelectedUnit} returns this
*/
proto.speech.tts.UnitSelectionOutput.SelectedUnit.prototype.setCostComponentList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 7, value);
};


/**
 * @param {!proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent}
 */
proto.speech.tts.UnitSelectionOutput.SelectedUnit.prototype.addCostComponent = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 7, opt_value, proto.speech.tts.UnitSelectionOutput.SelectedUnit.CostComponent, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.UnitSelectionOutput.SelectedUnit} returns this
 */
proto.speech.tts.UnitSelectionOutput.SelectedUnit.prototype.clearCostComponentList = function() {
  return this.setCostComponentList([]);
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.UnitSelectionOutput.SelectedUnit}
 */
proto.speech.tts.UnitSelectionOutput.SelectedUnit.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.UnitSelectionOutput.SelectedUnit, data);
};


/**
 * optional int32 path_id = 1;
 * @return {?number}
 */
proto.speech.tts.UnitSelectionOutput.prototype.getPathId = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.UnitSelectionOutput} returns this
 */
proto.speech.tts.UnitSelectionOutput.prototype.setPathId = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.UnitSelectionOutput} returns this
 */
proto.speech.tts.UnitSelectionOutput.prototype.clearPathId = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.UnitSelectionOutput.prototype.hasPathId = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * repeated SelectedUnit selected = 2;
 * @return {!Array<!proto.speech.tts.UnitSelectionOutput.SelectedUnit>}
 */
proto.speech.tts.UnitSelectionOutput.prototype.getSelectedList = function() {
  return /** @type{!Array<!proto.speech.tts.UnitSelectionOutput.SelectedUnit>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.UnitSelectionOutput.SelectedUnit, 2));
};


/**
 * @param {Array<!proto.speech.tts.UnitSelectionOutput.SelectedUnit>|undefined} value
 * @return {!proto.speech.tts.UnitSelectionOutput} returns this
*/
proto.speech.tts.UnitSelectionOutput.prototype.setSelectedList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 2, value);
};


/**
 * @param {!proto.speech.tts.UnitSelectionOutput.SelectedUnit=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.UnitSelectionOutput.SelectedUnit}
 */
proto.speech.tts.UnitSelectionOutput.prototype.addSelected = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 2, opt_value, proto.speech.tts.UnitSelectionOutput.SelectedUnit, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.UnitSelectionOutput} returns this
 */
proto.speech.tts.UnitSelectionOutput.prototype.clearSelectedList = function() {
  return this.setSelectedList([]);
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.UnitSelectionOutput}
 */
proto.speech.tts.UnitSelectionOutput.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.UnitSelectionOutput, data);
};



//speech/tts/engine/utterance/unit_selection_data_proto_jspb/unitselectiondata.jspb.js
// source: speech/tts/engine/utterance/unit_selection_data.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.UnitSelectionData');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.UnitSelectionInput');
goog.require('proto.speech.tts.UnitSelectionOutput');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.UnitSelectionData = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 3, proto.speech.tts.UnitSelectionData.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.UnitSelectionData, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.UnitSelectionData.displayName = 'proto.speech.tts.UnitSelectionData';
}

/**
 * The extensions registered with this message class. This is a map of
 * extension field number to fieldInfo object.
 *
 * For example:
 *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }
 *
 * fieldName contains the JsCompiler renamed field name property so that it
 * works in OPTIMIZED mode.
 *
 * @type {!Object<number, jspb.ExtensionFieldInfo>}
 */
proto.speech.tts.UnitSelectionData.extensions = {};


/**
 * The extensions registered with this message class. This is a map of
 * extension field number to fieldInfo object.
 *
 * For example:
 *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }
 *
 * fieldName contains the JsCompiler renamed field name property so that it
 * works in OPTIMIZED mode.
 *
 * @type {!Object<number, jspb.ExtensionFieldBinaryInfo>}
 */
proto.speech.tts.UnitSelectionData.extensionsBinary = {};


/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.UnitSelectionData.repeatedFields_ = [2];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.UnitSelectionData.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.UnitSelectionData.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.UnitSelectionData} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.UnitSelectionData.toObject = function(includeInstance, msg) {
  var f, obj = {
    input: (f = msg.getInput()) && proto.speech.tts.UnitSelectionInput.toObject(includeInstance, f),
    outputList: jspb.Message.toObjectList(msg.getOutputList(),
    proto.speech.tts.UnitSelectionOutput.toObject, includeInstance)
  };

  jspb.Message.toObjectExtension(/** @type {!jspb.Message} */ (msg), obj,
      proto.speech.tts.UnitSelectionData.extensions, proto.speech.tts.UnitSelectionData.prototype.getExtension,
      includeInstance);
  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of UnitSelectionData as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.UnitSelectionData.ObjectFormat = function() {
  /** @type {?proto.speech.tts.UnitSelectionInput.ObjectFormat|undefined} */
  this.input;

  /** @type {?Array<!proto.speech.tts.UnitSelectionOutput.ObjectFormat>|undefined} */
  this.outputList;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.UnitSelectionData.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.UnitSelectionData}
 */
proto.speech.tts.UnitSelectionData.fromObject = function(obj) {
  var msg = new proto.speech.tts.UnitSelectionData();
  obj.input && jspb.Message.setWrapperField(
      msg, 1, proto.speech.tts.UnitSelectionInput.fromObject(obj.input));
  obj.outputList && jspb.Message.setRepeatedWrapperField(
      msg, 2, obj.outputList.map(
          proto.speech.tts.UnitSelectionOutput.fromObject));
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.UnitSelectionData}
 */
proto.speech.tts.UnitSelectionData.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.UnitSelectionData;
  return proto.speech.tts.UnitSelectionData.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.UnitSelectionData} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.UnitSelectionData}
 */
proto.speech.tts.UnitSelectionData.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.speech.tts.UnitSelectionInput;
      reader.readMessage(value,proto.speech.tts.UnitSelectionInput.deserializeBinaryFromReader);
      msg.setInput(value);
      break;
    case 2:
      var value = new proto.speech.tts.UnitSelectionOutput;
      reader.readMessage(value,proto.speech.tts.UnitSelectionOutput.deserializeBinaryFromReader);
      msg.addOutput(value);
      break;
    default:
      jspb.Message.readBinaryExtension(msg, reader,
        proto.speech.tts.UnitSelectionData.extensionsBinary,
        proto.speech.tts.UnitSelectionData.prototype.getExtension,
        proto.speech.tts.UnitSelectionData.prototype.setExtension);
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.UnitSelectionData.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.UnitSelectionData.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.UnitSelectionData} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.UnitSelectionData.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getInput();
  if (f != null) {
    writer.writeMessage(
      1,
      f,
      proto.speech.tts.UnitSelectionInput.serializeBinaryToWriter
    );
  }
  f = message.getOutputList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      2,
      f,
      proto.speech.tts.UnitSelectionOutput.serializeBinaryToWriter
    );
  }
  jspb.Message.serializeBinaryExtensions(message, writer,
    proto.speech.tts.UnitSelectionData.extensionsBinary, proto.speech.tts.UnitSelectionData.prototype.getExtension);
};


/**
 * optional UnitSelectionInput input = 1;
 * @return {?proto.speech.tts.UnitSelectionInput}
 */
proto.speech.tts.UnitSelectionData.prototype.getInput = function() {
  return /** @type{?proto.speech.tts.UnitSelectionInput} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.UnitSelectionInput, 1));
};


/**
 * @param {?proto.speech.tts.UnitSelectionInput|undefined} value
 * @return {!proto.speech.tts.UnitSelectionData} returns this
*/
proto.speech.tts.UnitSelectionData.prototype.setInput = function(value) {
  return jspb.Message.setWrapperField(this, 1, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.UnitSelectionData} returns this
 */
proto.speech.tts.UnitSelectionData.prototype.clearInput = function() {
  return this.setInput(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.UnitSelectionData.prototype.hasInput = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * repeated UnitSelectionOutput output = 2;
 * @return {!Array<!proto.speech.tts.UnitSelectionOutput>}
 */
proto.speech.tts.UnitSelectionData.prototype.getOutputList = function() {
  return /** @type{!Array<!proto.speech.tts.UnitSelectionOutput>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.UnitSelectionOutput, 2));
};


/**
 * @param {Array<!proto.speech.tts.UnitSelectionOutput>|undefined} value
 * @return {!proto.speech.tts.UnitSelectionData} returns this
*/
proto.speech.tts.UnitSelectionData.prototype.setOutputList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 2, value);
};


/**
 * @param {!proto.speech.tts.UnitSelectionOutput=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.UnitSelectionOutput}
 */
proto.speech.tts.UnitSelectionData.prototype.addOutput = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 2, opt_value, proto.speech.tts.UnitSelectionOutput, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.UnitSelectionData} returns this
 */
proto.speech.tts.UnitSelectionData.prototype.clearOutputList = function() {
  return this.setOutputList([]);
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.UnitSelectionData}
 */
proto.speech.tts.UnitSelectionData.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.UnitSelectionData, data);
};



//speech/tts/engine/utterance/voice_building_data_proto_jspb/acousticutterancedata.jspb.js
// source: speech/tts/engine/utterance/voice_building_data.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.AcousticUtteranceData');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.WaveformData');
goog.require('proto.speech.tts.ParametricSynthesisOutput.AcousticFrame');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.AcousticUtteranceData = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.speech.tts.AcousticUtteranceData.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.AcousticUtteranceData, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.AcousticUtteranceData.displayName = 'proto.speech.tts.AcousticUtteranceData';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.AcousticUtteranceData.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.AcousticUtteranceData.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.AcousticUtteranceData.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.AcousticUtteranceData} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.AcousticUtteranceData.toObject = function(includeInstance, msg) {
  var f, obj = {
    frameList: jspb.Message.toObjectList(msg.getFrameList(),
    proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.toObject, includeInstance),
    audio: (f = msg.getAudio()) && proto.speech.WaveformData.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of AcousticUtteranceData as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.AcousticUtteranceData.ObjectFormat = function() {
  /** @type {?Array<!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.ObjectFormat>|undefined} */
  this.frameList;

  /** @type {?proto.speech.WaveformData.ObjectFormat|undefined} */
  this.audio;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.AcousticUtteranceData.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.AcousticUtteranceData}
 */
proto.speech.tts.AcousticUtteranceData.fromObject = function(obj) {
  var msg = new proto.speech.tts.AcousticUtteranceData();
  obj.frameList && jspb.Message.setRepeatedWrapperField(
      msg, 1, obj.frameList.map(
          proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.fromObject));
  obj.audio && jspb.Message.setWrapperField(
      msg, 2, proto.speech.WaveformData.fromObject(obj.audio));
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.AcousticUtteranceData}
 */
proto.speech.tts.AcousticUtteranceData.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.AcousticUtteranceData;
  return proto.speech.tts.AcousticUtteranceData.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.AcousticUtteranceData} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.AcousticUtteranceData}
 */
proto.speech.tts.AcousticUtteranceData.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.speech.tts.ParametricSynthesisOutput.AcousticFrame;
      reader.readMessage(value,proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.deserializeBinaryFromReader);
      msg.addFrame(value);
      break;
    case 2:
      var value = new proto.speech.WaveformData;
      reader.readMessage(value,proto.speech.WaveformData.deserializeBinaryFromReader);
      msg.setAudio(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.AcousticUtteranceData.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.AcousticUtteranceData.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.AcousticUtteranceData} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.AcousticUtteranceData.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getFrameList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      1,
      f,
      proto.speech.tts.ParametricSynthesisOutput.AcousticFrame.serializeBinaryToWriter
    );
  }
  f = message.getAudio();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.speech.WaveformData.serializeBinaryToWriter
    );
  }
};


/**
 * repeated ParametricSynthesisOutput.AcousticFrame frame = 1;
 * @return {!Array<!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame>}
 */
proto.speech.tts.AcousticUtteranceData.prototype.getFrameList = function() {
  return /** @type{!Array<!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.ParametricSynthesisOutput.AcousticFrame, 1));
};


/**
 * @param {Array<!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame>|undefined} value
 * @return {!proto.speech.tts.AcousticUtteranceData} returns this
*/
proto.speech.tts.AcousticUtteranceData.prototype.setFrameList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 1, value);
};


/**
 * @param {!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.ParametricSynthesisOutput.AcousticFrame}
 */
proto.speech.tts.AcousticUtteranceData.prototype.addFrame = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.speech.tts.ParametricSynthesisOutput.AcousticFrame, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.AcousticUtteranceData} returns this
 */
proto.speech.tts.AcousticUtteranceData.prototype.clearFrameList = function() {
  return this.setFrameList([]);
};


/**
 * optional speech.WaveformData audio = 2;
 * @return {?proto.speech.WaveformData}
 */
proto.speech.tts.AcousticUtteranceData.prototype.getAudio = function() {
  return /** @type{?proto.speech.WaveformData} */ (
    jspb.Message.getWrapperField(this, proto.speech.WaveformData, 2));
};


/**
 * @param {?proto.speech.WaveformData|undefined} value
 * @return {!proto.speech.tts.AcousticUtteranceData} returns this
*/
proto.speech.tts.AcousticUtteranceData.prototype.setAudio = function(value) {
  return jspb.Message.setWrapperField(this, 2, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.AcousticUtteranceData} returns this
 */
proto.speech.tts.AcousticUtteranceData.prototype.clearAudio = function() {
  return this.setAudio(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.AcousticUtteranceData.prototype.hasAudio = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.AcousticUtteranceData}
 */
proto.speech.tts.AcousticUtteranceData.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.AcousticUtteranceData, data);
};



//speech/tts/engine/utterance/voice_building_data_proto_jspb/alignerscore.jspb.js
// source: speech/tts/engine/utterance/voice_building_data.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.AlignerScore');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.AlignerScore = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.AlignerScore, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.AlignerScore.displayName = 'proto.speech.tts.AlignerScore';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.AlignerScore.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.AlignerScore.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.AlignerScore} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.AlignerScore.toObject = function(includeInstance, msg) {
  var f, obj = {
    sentScore: (f = jspb.Message.getOptionalFloatingPointField(msg, 1)) == null ? undefined : f,
    wordIndex: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    sentShortSegs: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,
    maxFrameEnergyInSilence: (f = jspb.Message.getOptionalFloatingPointField(msg, 4)) == null ? undefined : f,
    silenceIndex: (f = jspb.Message.getField(msg, 5)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of AlignerScore as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.AlignerScore.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.sentScore;

  /** @type {?number|undefined} */
  this.wordIndex;

  /** @type {?number|undefined} */
  this.sentShortSegs;

  /** @type {?number|undefined} */
  this.maxFrameEnergyInSilence;

  /** @type {?number|undefined} */
  this.silenceIndex;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.AlignerScore.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.AlignerScore}
 */
proto.speech.tts.AlignerScore.fromObject = function(obj) {
  var msg = new proto.speech.tts.AlignerScore();
  obj.sentScore != null && jspb.Message.setField(msg, 1, obj.sentScore);
  obj.wordIndex != null && jspb.Message.setField(msg, 2, obj.wordIndex);
  obj.sentShortSegs != null && jspb.Message.setField(msg, 3, obj.sentShortSegs);
  obj.maxFrameEnergyInSilence != null && jspb.Message.setField(msg, 4, obj.maxFrameEnergyInSilence);
  obj.silenceIndex != null && jspb.Message.setField(msg, 5, obj.silenceIndex);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.AlignerScore}
 */
proto.speech.tts.AlignerScore.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.AlignerScore;
  return proto.speech.tts.AlignerScore.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.AlignerScore} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.AlignerScore}
 */
proto.speech.tts.AlignerScore.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setSentScore(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readUint32());
      msg.setWordIndex(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readUint32());
      msg.setSentShortSegs(value);
      break;
    case 4:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setMaxFrameEnergyInSilence(value);
      break;
    case 5:
      var value = /** @type {number} */ (reader.readUint32());
      msg.setSilenceIndex(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.AlignerScore.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.AlignerScore.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.AlignerScore} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.AlignerScore.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?number} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeFloat(
      1,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeUint32(
      2,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeUint32(
      3,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeFloat(
      4,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeUint32(
      5,
      f
    );
  }
};


/**
 * optional float sent_score = 1;
 * @return {?number}
 */
proto.speech.tts.AlignerScore.prototype.getSentScore = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 1));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.AlignerScore} returns this
 */
proto.speech.tts.AlignerScore.prototype.setSentScore = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.AlignerScore} returns this
 */
proto.speech.tts.AlignerScore.prototype.clearSentScore = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.AlignerScore.prototype.hasSentScore = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional uint32 word_index = 2;
 * @return {?number}
 */
proto.speech.tts.AlignerScore.prototype.getWordIndex = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.AlignerScore} returns this
 */
proto.speech.tts.AlignerScore.prototype.setWordIndex = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.AlignerScore} returns this
 */
proto.speech.tts.AlignerScore.prototype.clearWordIndex = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.AlignerScore.prototype.hasWordIndex = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional uint32 sent_short_segs = 3;
 * @return {?number}
 */
proto.speech.tts.AlignerScore.prototype.getSentShortSegs = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 3));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.AlignerScore} returns this
 */
proto.speech.tts.AlignerScore.prototype.setSentShortSegs = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.AlignerScore} returns this
 */
proto.speech.tts.AlignerScore.prototype.clearSentShortSegs = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.AlignerScore.prototype.hasSentShortSegs = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional float max_frame_energy_in_silence = 4;
 * @return {?number}
 */
proto.speech.tts.AlignerScore.prototype.getMaxFrameEnergyInSilence = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 4));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.AlignerScore} returns this
 */
proto.speech.tts.AlignerScore.prototype.setMaxFrameEnergyInSilence = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.AlignerScore} returns this
 */
proto.speech.tts.AlignerScore.prototype.clearMaxFrameEnergyInSilence = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.AlignerScore.prototype.hasMaxFrameEnergyInSilence = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional uint32 silence_index = 5;
 * @return {?number}
 */
proto.speech.tts.AlignerScore.prototype.getSilenceIndex = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 5));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.AlignerScore} returns this
 */
proto.speech.tts.AlignerScore.prototype.setSilenceIndex = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.AlignerScore} returns this
 */
proto.speech.tts.AlignerScore.prototype.clearSilenceIndex = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.AlignerScore.prototype.hasSilenceIndex = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.AlignerScore}
 */
proto.speech.tts.AlignerScore.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.AlignerScore, data);
};



//speech/tts/engine/utterance/voice_building_data_proto_jspb/voicebuildingdata.jspb.js
// source: speech/tts/engine/utterance/voice_building_data.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.VoiceBuildingData');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.AcousticUtteranceData');
goog.require('proto.speech.tts.AlignerScore');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.VoiceBuildingData = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.VoiceBuildingData, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.VoiceBuildingData.displayName = 'proto.speech.tts.VoiceBuildingData';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.VoiceBuildingData.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.VoiceBuildingData.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.VoiceBuildingData} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.VoiceBuildingData.toObject = function(includeInstance, msg) {
  var f, obj = {
    acousticUtteranceData: (f = msg.getAcousticUtteranceData()) && proto.speech.tts.AcousticUtteranceData.toObject(includeInstance, f),
    alignerScores: (f = msg.getAlignerScores()) && proto.speech.tts.AlignerScore.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of VoiceBuildingData as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.VoiceBuildingData.ObjectFormat = function() {
  /** @type {?proto.speech.tts.AcousticUtteranceData.ObjectFormat|undefined} */
  this.acousticUtteranceData;

  /** @type {?proto.speech.tts.AlignerScore.ObjectFormat|undefined} */
  this.alignerScores;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.VoiceBuildingData.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.VoiceBuildingData}
 */
proto.speech.tts.VoiceBuildingData.fromObject = function(obj) {
  var msg = new proto.speech.tts.VoiceBuildingData();
  obj.acousticUtteranceData && jspb.Message.setWrapperField(
      msg, 1, proto.speech.tts.AcousticUtteranceData.fromObject(obj.acousticUtteranceData));
  obj.alignerScores && jspb.Message.setWrapperField(
      msg, 2, proto.speech.tts.AlignerScore.fromObject(obj.alignerScores));
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.VoiceBuildingData}
 */
proto.speech.tts.VoiceBuildingData.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.VoiceBuildingData;
  return proto.speech.tts.VoiceBuildingData.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.VoiceBuildingData} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.VoiceBuildingData}
 */
proto.speech.tts.VoiceBuildingData.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.speech.tts.AcousticUtteranceData;
      reader.readMessage(value,proto.speech.tts.AcousticUtteranceData.deserializeBinaryFromReader);
      msg.setAcousticUtteranceData(value);
      break;
    case 2:
      var value = new proto.speech.tts.AlignerScore;
      reader.readMessage(value,proto.speech.tts.AlignerScore.deserializeBinaryFromReader);
      msg.setAlignerScores(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.VoiceBuildingData.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.VoiceBuildingData.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.VoiceBuildingData} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.VoiceBuildingData.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getAcousticUtteranceData();
  if (f != null) {
    writer.writeMessage(
      1,
      f,
      proto.speech.tts.AcousticUtteranceData.serializeBinaryToWriter
    );
  }
  f = message.getAlignerScores();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.speech.tts.AlignerScore.serializeBinaryToWriter
    );
  }
};


/**
 * optional AcousticUtteranceData acoustic_utterance_data = 1;
 * @return {?proto.speech.tts.AcousticUtteranceData}
 */
proto.speech.tts.VoiceBuildingData.prototype.getAcousticUtteranceData = function() {
  return /** @type{?proto.speech.tts.AcousticUtteranceData} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.AcousticUtteranceData, 1));
};


/**
 * @param {?proto.speech.tts.AcousticUtteranceData|undefined} value
 * @return {!proto.speech.tts.VoiceBuildingData} returns this
*/
proto.speech.tts.VoiceBuildingData.prototype.setAcousticUtteranceData = function(value) {
  return jspb.Message.setWrapperField(this, 1, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.VoiceBuildingData} returns this
 */
proto.speech.tts.VoiceBuildingData.prototype.clearAcousticUtteranceData = function() {
  return this.setAcousticUtteranceData(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.VoiceBuildingData.prototype.hasAcousticUtteranceData = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional AlignerScore aligner_scores = 2;
 * @return {?proto.speech.tts.AlignerScore}
 */
proto.speech.tts.VoiceBuildingData.prototype.getAlignerScores = function() {
  return /** @type{?proto.speech.tts.AlignerScore} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.AlignerScore, 2));
};


/**
 * @param {?proto.speech.tts.AlignerScore|undefined} value
 * @return {!proto.speech.tts.VoiceBuildingData} returns this
*/
proto.speech.tts.VoiceBuildingData.prototype.setAlignerScores = function(value) {
  return jspb.Message.setWrapperField(this, 2, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.VoiceBuildingData} returns this
 */
proto.speech.tts.VoiceBuildingData.prototype.clearAlignerScores = function() {
  return this.setAlignerScores(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.VoiceBuildingData.prototype.hasAlignerScores = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.VoiceBuildingData}
 */
proto.speech.tts.VoiceBuildingData.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.VoiceBuildingData, data);
};



//speech/tts/engine/utterance/wavenet_synthesis_proto_jspb/wavenetinferencecost.jspb.js
// source: speech/tts/engine/utterance/wavenet_synthesis.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.WaveNetInferenceCost');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.WaveNetInferenceCost = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.WaveNetInferenceCost, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.WaveNetInferenceCost.displayName = 'proto.speech.tts.WaveNetInferenceCost';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.WaveNetInferenceCost.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.WaveNetInferenceCost.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.WaveNetInferenceCost} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.WaveNetInferenceCost.toObject = function(includeInstance, msg) {
  var f, obj = {
    sunnCpuMs: (f = jspb.Message.getOptionalFloatingPointField(msg, 1)) == null ? undefined : f,
    sunnJellyfishMs: (f = jspb.Message.getOptionalFloatingPointField(msg, 2)) == null ? undefined : f,
    wavernnCpuMs: (f = jspb.Message.getOptionalFloatingPointField(msg, 3)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of WaveNetInferenceCost as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.WaveNetInferenceCost.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.sunnCpuMs;

  /** @type {?number|undefined} */
  this.sunnJellyfishMs;

  /** @type {?number|undefined} */
  this.wavernnCpuMs;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.WaveNetInferenceCost.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.WaveNetInferenceCost}
 */
proto.speech.tts.WaveNetInferenceCost.fromObject = function(obj) {
  var msg = new proto.speech.tts.WaveNetInferenceCost();
  obj.sunnCpuMs != null && jspb.Message.setField(msg, 1, obj.sunnCpuMs);
  obj.sunnJellyfishMs != null && jspb.Message.setField(msg, 2, obj.sunnJellyfishMs);
  obj.wavernnCpuMs != null && jspb.Message.setField(msg, 3, obj.wavernnCpuMs);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.WaveNetInferenceCost}
 */
proto.speech.tts.WaveNetInferenceCost.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.WaveNetInferenceCost;
  return proto.speech.tts.WaveNetInferenceCost.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.WaveNetInferenceCost} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.WaveNetInferenceCost}
 */
proto.speech.tts.WaveNetInferenceCost.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setSunnCpuMs(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setSunnJellyfishMs(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setWavernnCpuMs(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.WaveNetInferenceCost.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.WaveNetInferenceCost.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.WaveNetInferenceCost} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.WaveNetInferenceCost.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?number} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeFloat(
      1,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeFloat(
      2,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeFloat(
      3,
      f
    );
  }
};


/**
 * optional float sunn_cpu_ms = 1;
 * @return {?number}
 */
proto.speech.tts.WaveNetInferenceCost.prototype.getSunnCpuMs = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 1));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.WaveNetInferenceCost} returns this
 */
proto.speech.tts.WaveNetInferenceCost.prototype.setSunnCpuMs = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.WaveNetInferenceCost} returns this
 */
proto.speech.tts.WaveNetInferenceCost.prototype.clearSunnCpuMs = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.WaveNetInferenceCost.prototype.hasSunnCpuMs = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional float sunn_jellyfish_ms = 2;
 * @return {?number}
 */
proto.speech.tts.WaveNetInferenceCost.prototype.getSunnJellyfishMs = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 2));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.WaveNetInferenceCost} returns this
 */
proto.speech.tts.WaveNetInferenceCost.prototype.setSunnJellyfishMs = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.WaveNetInferenceCost} returns this
 */
proto.speech.tts.WaveNetInferenceCost.prototype.clearSunnJellyfishMs = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.WaveNetInferenceCost.prototype.hasSunnJellyfishMs = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional float wavernn_cpu_ms = 3;
 * @return {?number}
 */
proto.speech.tts.WaveNetInferenceCost.prototype.getWavernnCpuMs = function() {
  return /** @type {?number} */ (jspb.Message.getOptionalFloatingPointField(this, 3));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.WaveNetInferenceCost} returns this
 */
proto.speech.tts.WaveNetInferenceCost.prototype.setWavernnCpuMs = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.WaveNetInferenceCost} returns this
 */
proto.speech.tts.WaveNetInferenceCost.prototype.clearWavernnCpuMs = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.WaveNetInferenceCost.prototype.hasWavernnCpuMs = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.WaveNetInferenceCost}
 */
proto.speech.tts.WaveNetInferenceCost.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.WaveNetInferenceCost, data);
};



//speech/tts/engine/utterance/utterance_proto_jspb/utterance.jspb.js
// source: speech/tts/engine/utterance/utterance.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.Utterance');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.ArtificialConcatenativeSpecificationData');
goog.require('proto.speech.tts.AudioBuffer');
goog.require('proto.speech.tts.Coverage');
goog.require('proto.speech.tts.Diagnostics');
goog.require('proto.speech.tts.EmotionDimensions');
goog.require('proto.speech.tts.ExternalUtteranceVectors');
goog.require('proto.speech.tts.LinguisticStructure');
goog.require('proto.speech.tts.ParametricSynthesisData');
goog.require('proto.speech.tts.PlaybackData');
goog.require('proto.speech.tts.Sentence');
goog.require('proto.speech.tts.StreamingInfo');
goog.require('proto.speech.tts.TextNormalizationData');
goog.require('proto.speech.tts.UnitSelectionData');
goog.require('proto.speech.tts.VoiceBuildingData');
goog.require('proto.speech.tts.VoiceParams');
goog.require('proto.speech.tts.WaveNetInferenceCost');

goog.forwardDeclare('proto.speech.tts.PhonationMode');
goog.forwardDeclare('proto.speech.tts.SpeakingStyle');
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.Utterance = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.speech.tts.Utterance, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.Utterance.displayName = 'proto.speech.tts.Utterance';
}

/**
 * The extensions registered with this message class. This is a map of
 * extension field number to fieldInfo object.
 *
 * For example:
 *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }
 *
 * fieldName contains the JsCompiler renamed field name property so that it
 * works in OPTIMIZED mode.
 *
 * @type {!Object<number, jspb.ExtensionFieldInfo>}
 */
proto.speech.tts.Utterance.extensions = {};


/**
 * The extensions registered with this message class. This is a map of
 * extension field number to fieldInfo object.
 *
 * For example:
 *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }
 *
 * fieldName contains the JsCompiler renamed field name property so that it
 * works in OPTIMIZED mode.
 *
 * @type {!Object<number, jspb.ExtensionFieldBinaryInfo>}
 */
proto.speech.tts.Utterance.extensionsBinary = {};




if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.Utterance.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.Utterance.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.Utterance} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Utterance.toObject = function(includeInstance, msg) {
  var f, obj = {
    id: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    filename: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    sentence: (f = msg.getSentence()) && proto.speech.tts.Sentence.toObject(includeInstance, f),
    originalSentence: (f = msg.getOriginalSentence()) && proto.speech.tts.Sentence.toObject(includeInstance, f),
    streamingInfo: (f = msg.getStreamingInfo()) && proto.speech.tts.StreamingInfo.toObject(includeInstance, f),
    voiceParams: (f = msg.getVoiceParams()) && proto.speech.tts.VoiceParams.toObject(includeInstance, f),
    originalSentenceIndex: (f = jspb.Message.getField(msg, 8)) == null ? undefined : f,
    originalSayIndex: (f = jspb.Message.getField(msg, 9)) == null ? undefined : f,
    originalByteIndex: (f = jspb.Message.getField(msg, 10)) == null ? undefined : f,
    linguistic: (f = msg.getLinguistic()) && proto.speech.tts.LinguisticStructure.toObject(includeInstance, f),
    coverage: (f = msg.getCoverage()) && proto.speech.tts.Coverage.toObject(includeInstance, f),
    textnormData: (f = msg.getTextnormData()) && proto.speech.tts.TextNormalizationData.toObject(includeInstance, f),
    playbackData: (f = msg.getPlaybackData()) && proto.speech.tts.PlaybackData.toObject(includeInstance, f),
    externalUtteranceVectors: (f = msg.getExternalUtteranceVectors()) && proto.speech.tts.ExternalUtteranceVectors.toObject(includeInstance, f),
    universalPhoneset: jspb.Message.getBooleanFieldWithDefault(msg, 105, false),
    emotionDimensions: (f = msg.getEmotionDimensions()) && proto.speech.tts.EmotionDimensions.toObject(includeInstance, f),
    speakingStyle: (f = jspb.Message.getField(msg, 107)) == null ? undefined : f,
    phonationMode: (f = jspb.Message.getField(msg, 108)) == null ? undefined : f,
    unitSelectionData: (f = msg.getUnitSelectionData()) && proto.speech.tts.UnitSelectionData.toObject(includeInstance, f),
    parametricSynthesisData: (f = msg.getParametricSynthesisData()) && proto.speech.tts.ParametricSynthesisData.toObject(includeInstance, f),
    suprasegmentalSyllableData: (f = msg.getSuprasegmentalSyllableData()) && proto.speech.tts.UnitSelectionData.toObject(includeInstance, f),
    artificialConcatenativeData: (f = msg.getArtificialConcatenativeData()) && proto.speech.tts.ArtificialConcatenativeSpecificationData.toObject(includeInstance, f),
    voiceBuildingData: (f = msg.getVoiceBuildingData()) && proto.speech.tts.VoiceBuildingData.toObject(includeInstance, f),
    audio: (f = msg.getAudio()) && proto.speech.tts.AudioBuffer.toObject(includeInstance, f),
    diagnostics: (f = msg.getDiagnostics()) && proto.speech.tts.Diagnostics.toObject(includeInstance, f),
    dependsOnGeoLocation: (f = jspb.Message.getBooleanField(msg, 901)) == null ? undefined : f,
    wavenetInferenceCost: (f = msg.getWavenetInferenceCost()) && proto.speech.tts.WaveNetInferenceCost.toObject(includeInstance, f)
  };

  jspb.Message.toObjectExtension(/** @type {!jspb.Message} */ (msg), obj,
      proto.speech.tts.Utterance.extensions, proto.speech.tts.Utterance.prototype.getExtension,
      includeInstance);
  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Utterance as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.Utterance.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.id;

  /** @type {?string|undefined} */
  this.filename;

  /** @type {?proto.speech.tts.Sentence.ObjectFormat|undefined} */
  this.sentence;

  /** @type {?proto.speech.tts.Sentence.ObjectFormat|undefined} */
  this.originalSentence;

  /** @type {?proto.speech.tts.StreamingInfo.ObjectFormat|undefined} */
  this.streamingInfo;

  /** @type {?proto.speech.tts.VoiceParams.ObjectFormat|undefined} */
  this.voiceParams;

  /** @type {?number|undefined} */
  this.originalSentenceIndex;

  /** @type {?number|undefined} */
  this.originalSayIndex;

  /** @type {?number|undefined} */
  this.originalByteIndex;

  /** @type {?proto.speech.tts.LinguisticStructure.ObjectFormat|undefined} */
  this.linguistic;

  /** @type {?proto.speech.tts.Coverage.ObjectFormat|undefined} */
  this.coverage;

  /** @type {?proto.speech.tts.TextNormalizationData.ObjectFormat|undefined} */
  this.textnormData;

  /** @type {?proto.speech.tts.PlaybackData.ObjectFormat|undefined} */
  this.playbackData;

  /** @type {?proto.speech.tts.ExternalUtteranceVectors.ObjectFormat|undefined} */
  this.externalUtteranceVectors;

  /** @type {?boolean|undefined} */
  this.universalPhoneset;

  /** @type {?proto.speech.tts.EmotionDimensions.ObjectFormat|undefined} */
  this.emotionDimensions;

  /** @type {?number|undefined} */
  this.speakingStyle;

  /** @type {?number|undefined} */
  this.phonationMode;

  /** @type {?proto.speech.tts.UnitSelectionData.ObjectFormat|undefined} */
  this.unitSelectionData;

  /** @type {?proto.speech.tts.ParametricSynthesisData.ObjectFormat|undefined} */
  this.parametricSynthesisData;

  /** @type {?proto.speech.tts.UnitSelectionData.ObjectFormat|undefined} */
  this.suprasegmentalSyllableData;

  /** @type {?proto.speech.tts.ArtificialConcatenativeSpecificationData.ObjectFormat|undefined} */
  this.artificialConcatenativeData;

  /** @type {?proto.speech.tts.VoiceBuildingData.ObjectFormat|undefined} */
  this.voiceBuildingData;

  /** @type {?proto.speech.tts.AudioBuffer.ObjectFormat|undefined} */
  this.audio;

  /** @type {?proto.speech.tts.Diagnostics.ObjectFormat|undefined} */
  this.diagnostics;

  /** @type {?boolean|undefined} */
  this.dependsOnGeoLocation;

  /** @type {?proto.speech.tts.WaveNetInferenceCost.ObjectFormat|undefined} */
  this.wavenetInferenceCost;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.Utterance.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.Utterance}
 */
proto.speech.tts.Utterance.fromObject = function(obj) {
  var msg = new proto.speech.tts.Utterance();
  obj.id != null && jspb.Message.setField(msg, 1, obj.id);
  obj.filename != null && jspb.Message.setField(msg, 2, obj.filename);
  obj.sentence && jspb.Message.setWrapperField(
      msg, 3, proto.speech.tts.Sentence.fromObject(obj.sentence));
  obj.originalSentence && jspb.Message.setWrapperField(
      msg, 4, proto.speech.tts.Sentence.fromObject(obj.originalSentence));
  obj.streamingInfo && jspb.Message.setWrapperField(
      msg, 6, proto.speech.tts.StreamingInfo.fromObject(obj.streamingInfo));
  obj.voiceParams && jspb.Message.setWrapperField(
      msg, 7, proto.speech.tts.VoiceParams.fromObject(obj.voiceParams));
  obj.originalSentenceIndex != null && jspb.Message.setField(msg, 8, obj.originalSentenceIndex);
  obj.originalSayIndex != null && jspb.Message.setField(msg, 9, obj.originalSayIndex);
  obj.originalByteIndex != null && jspb.Message.setField(msg, 10, obj.originalByteIndex);
  obj.linguistic && jspb.Message.setWrapperField(
      msg, 100, proto.speech.tts.LinguisticStructure.fromObject(obj.linguistic));
  obj.coverage && jspb.Message.setWrapperField(
      msg, 101, proto.speech.tts.Coverage.fromObject(obj.coverage));
  obj.textnormData && jspb.Message.setWrapperField(
      msg, 102, proto.speech.tts.TextNormalizationData.fromObject(obj.textnormData));
  obj.playbackData && jspb.Message.setWrapperField(
      msg, 103, proto.speech.tts.PlaybackData.fromObject(obj.playbackData));
  obj.externalUtteranceVectors && jspb.Message.setWrapperField(
      msg, 104, proto.speech.tts.ExternalUtteranceVectors.fromObject(obj.externalUtteranceVectors));
  obj.universalPhoneset != null && jspb.Message.setField(msg, 105, obj.universalPhoneset);
  obj.emotionDimensions && jspb.Message.setWrapperField(
      msg, 106, proto.speech.tts.EmotionDimensions.fromObject(obj.emotionDimensions));
  obj.speakingStyle != null && jspb.Message.setField(msg, 107, obj.speakingStyle);
  obj.phonationMode != null && jspb.Message.setField(msg, 108, obj.phonationMode);
  obj.unitSelectionData && jspb.Message.setWrapperField(
      msg, 200, proto.speech.tts.UnitSelectionData.fromObject(obj.unitSelectionData));
  obj.parametricSynthesisData && jspb.Message.setWrapperField(
      msg, 201, proto.speech.tts.ParametricSynthesisData.fromObject(obj.parametricSynthesisData));
  obj.suprasegmentalSyllableData && jspb.Message.setWrapperField(
      msg, 202, proto.speech.tts.UnitSelectionData.fromObject(obj.suprasegmentalSyllableData));
  obj.artificialConcatenativeData && jspb.Message.setWrapperField(
      msg, 203, proto.speech.tts.ArtificialConcatenativeSpecificationData.fromObject(obj.artificialConcatenativeData));
  obj.voiceBuildingData && jspb.Message.setWrapperField(
      msg, 250, proto.speech.tts.VoiceBuildingData.fromObject(obj.voiceBuildingData));
  obj.audio && jspb.Message.setWrapperField(
      msg, 300, proto.speech.tts.AudioBuffer.fromObject(obj.audio));
  obj.diagnostics && jspb.Message.setWrapperField(
      msg, 900, proto.speech.tts.Diagnostics.fromObject(obj.diagnostics));
  obj.dependsOnGeoLocation != null && jspb.Message.setField(msg, 901, obj.dependsOnGeoLocation);
  obj.wavenetInferenceCost && jspb.Message.setWrapperField(
      msg, 902, proto.speech.tts.WaveNetInferenceCost.fromObject(obj.wavenetInferenceCost));
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.Utterance}
 */
proto.speech.tts.Utterance.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.Utterance;
  return proto.speech.tts.Utterance.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.Utterance} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.Utterance}
 */
proto.speech.tts.Utterance.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {number} */ (reader.readUint64());
      msg.setId(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setFilename(value);
      break;
    case 3:
      var value = new proto.speech.tts.Sentence;
      reader.readMessage(value,proto.speech.tts.Sentence.deserializeBinaryFromReader);
      msg.setSentence(value);
      break;
    case 4:
      var value = new proto.speech.tts.Sentence;
      reader.readMessage(value,proto.speech.tts.Sentence.deserializeBinaryFromReader);
      msg.setOriginalSentence(value);
      break;
    case 6:
      var value = new proto.speech.tts.StreamingInfo;
      reader.readMessage(value,proto.speech.tts.StreamingInfo.deserializeBinaryFromReader);
      msg.setStreamingInfo(value);
      break;
    case 7:
      var value = new proto.speech.tts.VoiceParams;
      reader.readMessage(value,proto.speech.tts.VoiceParams.deserializeBinaryFromReader);
      msg.setVoiceParams(value);
      break;
    case 8:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setOriginalSentenceIndex(value);
      break;
    case 9:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setOriginalSayIndex(value);
      break;
    case 10:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setOriginalByteIndex(value);
      break;
    case 100:
      var value = new proto.speech.tts.LinguisticStructure;
      reader.readMessage(value,proto.speech.tts.LinguisticStructure.deserializeBinaryFromReader);
      msg.setLinguistic(value);
      break;
    case 101:
      var value = new proto.speech.tts.Coverage;
      reader.readMessage(value,proto.speech.tts.Coverage.deserializeBinaryFromReader);
      msg.setCoverage(value);
      break;
    case 102:
      var value = new proto.speech.tts.TextNormalizationData;
      reader.readMessage(value,proto.speech.tts.TextNormalizationData.deserializeBinaryFromReader);
      msg.setTextnormData(value);
      break;
    case 103:
      var value = new proto.speech.tts.PlaybackData;
      reader.readMessage(value,proto.speech.tts.PlaybackData.deserializeBinaryFromReader);
      msg.setPlaybackData(value);
      break;
    case 104:
      var value = new proto.speech.tts.ExternalUtteranceVectors;
      reader.readMessage(value,proto.speech.tts.ExternalUtteranceVectors.deserializeBinaryFromReader);
      msg.setExternalUtteranceVectors(value);
      break;
    case 105:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setUniversalPhoneset(value);
      break;
    case 106:
      var value = new proto.speech.tts.EmotionDimensions;
      reader.readMessage(value,proto.speech.tts.EmotionDimensions.deserializeBinaryFromReader);
      msg.setEmotionDimensions(value);
      break;
    case 107:
      var value = /** @type {!proto.speech.tts.SpeakingStyle} */ (reader.readEnum());
      msg.setSpeakingStyle(value);
      break;
    case 108:
      var value = /** @type {!proto.speech.tts.PhonationMode} */ (reader.readEnum());
      msg.setPhonationMode(value);
      break;
    case 200:
      var value = new proto.speech.tts.UnitSelectionData;
      reader.readMessage(value,proto.speech.tts.UnitSelectionData.deserializeBinaryFromReader);
      msg.setUnitSelectionData(value);
      break;
    case 201:
      var value = new proto.speech.tts.ParametricSynthesisData;
      reader.readMessage(value,proto.speech.tts.ParametricSynthesisData.deserializeBinaryFromReader);
      msg.setParametricSynthesisData(value);
      break;
    case 202:
      var value = new proto.speech.tts.UnitSelectionData;
      reader.readMessage(value,proto.speech.tts.UnitSelectionData.deserializeBinaryFromReader);
      msg.setSuprasegmentalSyllableData(value);
      break;
    case 203:
      var value = new proto.speech.tts.ArtificialConcatenativeSpecificationData;
      reader.readMessage(value,proto.speech.tts.ArtificialConcatenativeSpecificationData.deserializeBinaryFromReader);
      msg.setArtificialConcatenativeData(value);
      break;
    case 250:
      var value = new proto.speech.tts.VoiceBuildingData;
      reader.readMessage(value,proto.speech.tts.VoiceBuildingData.deserializeBinaryFromReader);
      msg.setVoiceBuildingData(value);
      break;
    case 300:
      var value = new proto.speech.tts.AudioBuffer;
      reader.readMessage(value,proto.speech.tts.AudioBuffer.deserializeBinaryFromReader);
      msg.setAudio(value);
      break;
    case 900:
      var value = new proto.speech.tts.Diagnostics;
      reader.readMessage(value,proto.speech.tts.Diagnostics.deserializeBinaryFromReader);
      msg.setDiagnostics(value);
      break;
    case 901:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setDependsOnGeoLocation(value);
      break;
    case 902:
      var value = new proto.speech.tts.WaveNetInferenceCost;
      reader.readMessage(value,proto.speech.tts.WaveNetInferenceCost.deserializeBinaryFromReader);
      msg.setWavenetInferenceCost(value);
      break;
    default:
      jspb.Message.readBinaryExtension(msg, reader,
        proto.speech.tts.Utterance.extensionsBinary,
        proto.speech.tts.Utterance.prototype.getExtension,
        proto.speech.tts.Utterance.prototype.setExtension);
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.Utterance.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.Utterance.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.Utterance} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Utterance.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?number} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeUint64(
      1,
      f
    );
  }
  f = /** @type {?string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
  f = message.getSentence();
  if (f != null) {
    writer.writeMessage(
      3,
      f,
      proto.speech.tts.Sentence.serializeBinaryToWriter
    );
  }
  f = message.getOriginalSentence();
  if (f != null) {
    writer.writeMessage(
      4,
      f,
      proto.speech.tts.Sentence.serializeBinaryToWriter
    );
  }
  f = message.getStreamingInfo();
  if (f != null) {
    writer.writeMessage(
      6,
      f,
      proto.speech.tts.StreamingInfo.serializeBinaryToWriter
    );
  }
  f = message.getVoiceParams();
  if (f != null) {
    writer.writeMessage(
      7,
      f,
      proto.speech.tts.VoiceParams.serializeBinaryToWriter
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 8));
  if (f != null) {
    writer.writeInt64(
      8,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 9));
  if (f != null) {
    writer.writeInt64(
      9,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 10));
  if (f != null) {
    writer.writeInt64(
      10,
      f
    );
  }
  f = message.getLinguistic();
  if (f != null) {
    writer.writeMessage(
      100,
      f,
      proto.speech.tts.LinguisticStructure.serializeBinaryToWriter
    );
  }
  f = message.getCoverage();
  if (f != null) {
    writer.writeMessage(
      101,
      f,
      proto.speech.tts.Coverage.serializeBinaryToWriter
    );
  }
  f = message.getTextnormData();
  if (f != null) {
    writer.writeMessage(
      102,
      f,
      proto.speech.tts.TextNormalizationData.serializeBinaryToWriter
    );
  }
  f = message.getPlaybackData();
  if (f != null) {
    writer.writeMessage(
      103,
      f,
      proto.speech.tts.PlaybackData.serializeBinaryToWriter
    );
  }
  f = message.getExternalUtteranceVectors();
  if (f != null) {
    writer.writeMessage(
      104,
      f,
      proto.speech.tts.ExternalUtteranceVectors.serializeBinaryToWriter
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 105));
  if (f != null) {
    writer.writeBool(
      105,
      f
    );
  }
  f = message.getEmotionDimensions();
  if (f != null) {
    writer.writeMessage(
      106,
      f,
      proto.speech.tts.EmotionDimensions.serializeBinaryToWriter
    );
  }
  f = /** @type {!proto.speech.tts.SpeakingStyle} */ (jspb.Message.getField(message, 107));
  if (f != null) {
    writer.writeEnum(
      107,
      f
    );
  }
  f = /** @type {!proto.speech.tts.PhonationMode} */ (jspb.Message.getField(message, 108));
  if (f != null) {
    writer.writeEnum(
      108,
      f
    );
  }
  f = message.getUnitSelectionData();
  if (f != null) {
    writer.writeMessage(
      200,
      f,
      proto.speech.tts.UnitSelectionData.serializeBinaryToWriter
    );
  }
  f = message.getParametricSynthesisData();
  if (f != null) {
    writer.writeMessage(
      201,
      f,
      proto.speech.tts.ParametricSynthesisData.serializeBinaryToWriter
    );
  }
  f = message.getSuprasegmentalSyllableData();
  if (f != null) {
    writer.writeMessage(
      202,
      f,
      proto.speech.tts.UnitSelectionData.serializeBinaryToWriter
    );
  }
  f = message.getArtificialConcatenativeData();
  if (f != null) {
    writer.writeMessage(
      203,
      f,
      proto.speech.tts.ArtificialConcatenativeSpecificationData.serializeBinaryToWriter
    );
  }
  f = message.getVoiceBuildingData();
  if (f != null) {
    writer.writeMessage(
      250,
      f,
      proto.speech.tts.VoiceBuildingData.serializeBinaryToWriter
    );
  }
  f = message.getAudio();
  if (f != null) {
    writer.writeMessage(
      300,
      f,
      proto.speech.tts.AudioBuffer.serializeBinaryToWriter
    );
  }
  f = message.getDiagnostics();
  if (f != null) {
    writer.writeMessage(
      900,
      f,
      proto.speech.tts.Diagnostics.serializeBinaryToWriter
    );
  }
  f = /** @type {?boolean} */ (jspb.Message.getField(message, 901));
  if (f != null) {
    writer.writeBool(
      901,
      f
    );
  }
  f = message.getWavenetInferenceCost();
  if (f != null) {
    writer.writeMessage(
      902,
      f,
      proto.speech.tts.WaveNetInferenceCost.serializeBinaryToWriter
    );
  }
  jspb.Message.serializeBinaryExtensions(message, writer,
    proto.speech.tts.Utterance.extensionsBinary, proto.speech.tts.Utterance.prototype.getExtension);
};


/**
 * optional uint64 id = 1;
 * @return {?number}
 */
proto.speech.tts.Utterance.prototype.getId = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Utterance} returns this
 */
proto.speech.tts.Utterance.prototype.setId = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Utterance} returns this
 */
proto.speech.tts.Utterance.prototype.clearId = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Utterance.prototype.hasId = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional string filename = 2;
 * @return {?string}
 */
proto.speech.tts.Utterance.prototype.getFilename = function() {
  return /** @type {?string} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?string|undefined} value
 * @return {!proto.speech.tts.Utterance} returns this
 */
proto.speech.tts.Utterance.prototype.setFilename = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Utterance} returns this
 */
proto.speech.tts.Utterance.prototype.clearFilename = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Utterance.prototype.hasFilename = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional Sentence sentence = 3;
 * @return {?proto.speech.tts.Sentence}
 */
proto.speech.tts.Utterance.prototype.getSentence = function() {
  return /** @type{?proto.speech.tts.Sentence} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Sentence, 3));
};


/**
 * @param {?proto.speech.tts.Sentence|undefined} value
 * @return {!proto.speech.tts.Utterance} returns this
*/
proto.speech.tts.Utterance.prototype.setSentence = function(value) {
  return jspb.Message.setWrapperField(this, 3, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Utterance} returns this
 */
proto.speech.tts.Utterance.prototype.clearSentence = function() {
  return this.setSentence(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Utterance.prototype.hasSentence = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional Sentence original_sentence = 4;
 * @return {?proto.speech.tts.Sentence}
 */
proto.speech.tts.Utterance.prototype.getOriginalSentence = function() {
  return /** @type{?proto.speech.tts.Sentence} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Sentence, 4));
};


/**
 * @param {?proto.speech.tts.Sentence|undefined} value
 * @return {!proto.speech.tts.Utterance} returns this
*/
proto.speech.tts.Utterance.prototype.setOriginalSentence = function(value) {
  return jspb.Message.setWrapperField(this, 4, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Utterance} returns this
 */
proto.speech.tts.Utterance.prototype.clearOriginalSentence = function() {
  return this.setOriginalSentence(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Utterance.prototype.hasOriginalSentence = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional StreamingInfo streaming_info = 6;
 * @return {?proto.speech.tts.StreamingInfo}
 */
proto.speech.tts.Utterance.prototype.getStreamingInfo = function() {
  return /** @type{?proto.speech.tts.StreamingInfo} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.StreamingInfo, 6));
};


/**
 * @param {?proto.speech.tts.StreamingInfo|undefined} value
 * @return {!proto.speech.tts.Utterance} returns this
*/
proto.speech.tts.Utterance.prototype.setStreamingInfo = function(value) {
  return jspb.Message.setWrapperField(this, 6, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Utterance} returns this
 */
proto.speech.tts.Utterance.prototype.clearStreamingInfo = function() {
  return this.setStreamingInfo(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Utterance.prototype.hasStreamingInfo = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional VoiceParams voice_params = 7;
 * @return {?proto.speech.tts.VoiceParams}
 */
proto.speech.tts.Utterance.prototype.getVoiceParams = function() {
  return /** @type{?proto.speech.tts.VoiceParams} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.VoiceParams, 7));
};


/**
 * @param {?proto.speech.tts.VoiceParams|undefined} value
 * @return {!proto.speech.tts.Utterance} returns this
*/
proto.speech.tts.Utterance.prototype.setVoiceParams = function(value) {
  return jspb.Message.setWrapperField(this, 7, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Utterance} returns this
 */
proto.speech.tts.Utterance.prototype.clearVoiceParams = function() {
  return this.setVoiceParams(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Utterance.prototype.hasVoiceParams = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * optional int64 original_sentence_index = 8;
 * @return {?number}
 */
proto.speech.tts.Utterance.prototype.getOriginalSentenceIndex = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 8));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Utterance} returns this
 */
proto.speech.tts.Utterance.prototype.setOriginalSentenceIndex = function(value) {
  return jspb.Message.setField(this, 8, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Utterance} returns this
 */
proto.speech.tts.Utterance.prototype.clearOriginalSentenceIndex = function() {
  return jspb.Message.setField(this, 8, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Utterance.prototype.hasOriginalSentenceIndex = function() {
  return jspb.Message.getField(this, 8) != null;
};


/**
 * optional int64 original_say_index = 9;
 * @return {?number}
 */
proto.speech.tts.Utterance.prototype.getOriginalSayIndex = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 9));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Utterance} returns this
 */
proto.speech.tts.Utterance.prototype.setOriginalSayIndex = function(value) {
  return jspb.Message.setField(this, 9, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Utterance} returns this
 */
proto.speech.tts.Utterance.prototype.clearOriginalSayIndex = function() {
  return jspb.Message.setField(this, 9, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Utterance.prototype.hasOriginalSayIndex = function() {
  return jspb.Message.getField(this, 9) != null;
};


/**
 * optional int64 original_byte_index = 10;
 * @return {?number}
 */
proto.speech.tts.Utterance.prototype.getOriginalByteIndex = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 10));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.Utterance} returns this
 */
proto.speech.tts.Utterance.prototype.setOriginalByteIndex = function(value) {
  return jspb.Message.setField(this, 10, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Utterance} returns this
 */
proto.speech.tts.Utterance.prototype.clearOriginalByteIndex = function() {
  return jspb.Message.setField(this, 10, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Utterance.prototype.hasOriginalByteIndex = function() {
  return jspb.Message.getField(this, 10) != null;
};


/**
 * optional LinguisticStructure linguistic = 100;
 * @return {?proto.speech.tts.LinguisticStructure}
 */
proto.speech.tts.Utterance.prototype.getLinguistic = function() {
  return /** @type{?proto.speech.tts.LinguisticStructure} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.LinguisticStructure, 100));
};


/**
 * @param {?proto.speech.tts.LinguisticStructure|undefined} value
 * @return {!proto.speech.tts.Utterance} returns this
*/
proto.speech.tts.Utterance.prototype.setLinguistic = function(value) {
  return jspb.Message.setWrapperField(this, 100, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Utterance} returns this
 */
proto.speech.tts.Utterance.prototype.clearLinguistic = function() {
  return this.setLinguistic(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Utterance.prototype.hasLinguistic = function() {
  return jspb.Message.getField(this, 100) != null;
};


/**
 * optional Coverage coverage = 101;
 * @return {?proto.speech.tts.Coverage}
 */
proto.speech.tts.Utterance.prototype.getCoverage = function() {
  return /** @type{?proto.speech.tts.Coverage} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Coverage, 101));
};


/**
 * @param {?proto.speech.tts.Coverage|undefined} value
 * @return {!proto.speech.tts.Utterance} returns this
*/
proto.speech.tts.Utterance.prototype.setCoverage = function(value) {
  return jspb.Message.setWrapperField(this, 101, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Utterance} returns this
 */
proto.speech.tts.Utterance.prototype.clearCoverage = function() {
  return this.setCoverage(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Utterance.prototype.hasCoverage = function() {
  return jspb.Message.getField(this, 101) != null;
};


/**
 * optional TextNormalizationData textnorm_data = 102;
 * @return {?proto.speech.tts.TextNormalizationData}
 */
proto.speech.tts.Utterance.prototype.getTextnormData = function() {
  return /** @type{?proto.speech.tts.TextNormalizationData} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.TextNormalizationData, 102));
};


/**
 * @param {?proto.speech.tts.TextNormalizationData|undefined} value
 * @return {!proto.speech.tts.Utterance} returns this
*/
proto.speech.tts.Utterance.prototype.setTextnormData = function(value) {
  return jspb.Message.setWrapperField(this, 102, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Utterance} returns this
 */
proto.speech.tts.Utterance.prototype.clearTextnormData = function() {
  return this.setTextnormData(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Utterance.prototype.hasTextnormData = function() {
  return jspb.Message.getField(this, 102) != null;
};


/**
 * optional PlaybackData playback_data = 103;
 * @return {?proto.speech.tts.PlaybackData}
 */
proto.speech.tts.Utterance.prototype.getPlaybackData = function() {
  return /** @type{?proto.speech.tts.PlaybackData} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.PlaybackData, 103));
};


/**
 * @param {?proto.speech.tts.PlaybackData|undefined} value
 * @return {!proto.speech.tts.Utterance} returns this
*/
proto.speech.tts.Utterance.prototype.setPlaybackData = function(value) {
  return jspb.Message.setWrapperField(this, 103, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Utterance} returns this
 */
proto.speech.tts.Utterance.prototype.clearPlaybackData = function() {
  return this.setPlaybackData(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Utterance.prototype.hasPlaybackData = function() {
  return jspb.Message.getField(this, 103) != null;
};


/**
 * optional ExternalUtteranceVectors external_utterance_vectors = 104;
 * @return {?proto.speech.tts.ExternalUtteranceVectors}
 */
proto.speech.tts.Utterance.prototype.getExternalUtteranceVectors = function() {
  return /** @type{?proto.speech.tts.ExternalUtteranceVectors} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.ExternalUtteranceVectors, 104));
};


/**
 * @param {?proto.speech.tts.ExternalUtteranceVectors|undefined} value
 * @return {!proto.speech.tts.Utterance} returns this
*/
proto.speech.tts.Utterance.prototype.setExternalUtteranceVectors = function(value) {
  return jspb.Message.setWrapperField(this, 104, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Utterance} returns this
 */
proto.speech.tts.Utterance.prototype.clearExternalUtteranceVectors = function() {
  return this.setExternalUtteranceVectors(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Utterance.prototype.hasExternalUtteranceVectors = function() {
  return jspb.Message.getField(this, 104) != null;
};


/**
 * optional bool universal_phoneset = 105;
 * @return {boolean}
 */
proto.speech.tts.Utterance.prototype.getUniversalPhoneset = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 105, false));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.Utterance} returns this
 */
proto.speech.tts.Utterance.prototype.setUniversalPhoneset = function(value) {
  return jspb.Message.setField(this, 105, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Utterance} returns this
 */
proto.speech.tts.Utterance.prototype.clearUniversalPhoneset = function() {
  return jspb.Message.setField(this, 105, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Utterance.prototype.hasUniversalPhoneset = function() {
  return jspb.Message.getField(this, 105) != null;
};


/**
 * optional EmotionDimensions emotion_dimensions = 106;
 * @return {?proto.speech.tts.EmotionDimensions}
 */
proto.speech.tts.Utterance.prototype.getEmotionDimensions = function() {
  return /** @type{?proto.speech.tts.EmotionDimensions} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.EmotionDimensions, 106));
};


/**
 * @param {?proto.speech.tts.EmotionDimensions|undefined} value
 * @return {!proto.speech.tts.Utterance} returns this
*/
proto.speech.tts.Utterance.prototype.setEmotionDimensions = function(value) {
  return jspb.Message.setWrapperField(this, 106, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Utterance} returns this
 */
proto.speech.tts.Utterance.prototype.clearEmotionDimensions = function() {
  return this.setEmotionDimensions(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Utterance.prototype.hasEmotionDimensions = function() {
  return jspb.Message.getField(this, 106) != null;
};


/**
 * optional SpeakingStyle speaking_style = 107;
 * @return {!proto.speech.tts.SpeakingStyle}
 */
proto.speech.tts.Utterance.prototype.getSpeakingStyle = function() {
  return /** @type {!proto.speech.tts.SpeakingStyle} */ (jspb.Message.getField(this, 107));
};


/**
 * @param {?proto.speech.tts.SpeakingStyle|undefined} value
 * @return {!proto.speech.tts.Utterance} returns this
 */
proto.speech.tts.Utterance.prototype.setSpeakingStyle = function(value) {
  return jspb.Message.setField(this, 107, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Utterance} returns this
 */
proto.speech.tts.Utterance.prototype.clearSpeakingStyle = function() {
  return jspb.Message.setField(this, 107, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Utterance.prototype.hasSpeakingStyle = function() {
  return jspb.Message.getField(this, 107) != null;
};


/**
 * optional PhonationMode phonation_mode = 108;
 * @return {!proto.speech.tts.PhonationMode}
 */
proto.speech.tts.Utterance.prototype.getPhonationMode = function() {
  return /** @type {!proto.speech.tts.PhonationMode} */ (jspb.Message.getField(this, 108));
};


/**
 * @param {?proto.speech.tts.PhonationMode|undefined} value
 * @return {!proto.speech.tts.Utterance} returns this
 */
proto.speech.tts.Utterance.prototype.setPhonationMode = function(value) {
  return jspb.Message.setField(this, 108, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Utterance} returns this
 */
proto.speech.tts.Utterance.prototype.clearPhonationMode = function() {
  return jspb.Message.setField(this, 108, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Utterance.prototype.hasPhonationMode = function() {
  return jspb.Message.getField(this, 108) != null;
};


/**
 * optional UnitSelectionData unit_selection_data = 200;
 * @return {?proto.speech.tts.UnitSelectionData}
 */
proto.speech.tts.Utterance.prototype.getUnitSelectionData = function() {
  return /** @type{?proto.speech.tts.UnitSelectionData} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.UnitSelectionData, 200));
};


/**
 * @param {?proto.speech.tts.UnitSelectionData|undefined} value
 * @return {!proto.speech.tts.Utterance} returns this
*/
proto.speech.tts.Utterance.prototype.setUnitSelectionData = function(value) {
  return jspb.Message.setWrapperField(this, 200, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Utterance} returns this
 */
proto.speech.tts.Utterance.prototype.clearUnitSelectionData = function() {
  return this.setUnitSelectionData(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Utterance.prototype.hasUnitSelectionData = function() {
  return jspb.Message.getField(this, 200) != null;
};


/**
 * optional ParametricSynthesisData parametric_synthesis_data = 201;
 * @return {?proto.speech.tts.ParametricSynthesisData}
 */
proto.speech.tts.Utterance.prototype.getParametricSynthesisData = function() {
  return /** @type{?proto.speech.tts.ParametricSynthesisData} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.ParametricSynthesisData, 201));
};


/**
 * @param {?proto.speech.tts.ParametricSynthesisData|undefined} value
 * @return {!proto.speech.tts.Utterance} returns this
*/
proto.speech.tts.Utterance.prototype.setParametricSynthesisData = function(value) {
  return jspb.Message.setWrapperField(this, 201, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Utterance} returns this
 */
proto.speech.tts.Utterance.prototype.clearParametricSynthesisData = function() {
  return this.setParametricSynthesisData(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Utterance.prototype.hasParametricSynthesisData = function() {
  return jspb.Message.getField(this, 201) != null;
};


/**
 * optional UnitSelectionData suprasegmental_syllable_data = 202;
 * @return {?proto.speech.tts.UnitSelectionData}
 */
proto.speech.tts.Utterance.prototype.getSuprasegmentalSyllableData = function() {
  return /** @type{?proto.speech.tts.UnitSelectionData} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.UnitSelectionData, 202));
};


/**
 * @param {?proto.speech.tts.UnitSelectionData|undefined} value
 * @return {!proto.speech.tts.Utterance} returns this
*/
proto.speech.tts.Utterance.prototype.setSuprasegmentalSyllableData = function(value) {
  return jspb.Message.setWrapperField(this, 202, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Utterance} returns this
 */
proto.speech.tts.Utterance.prototype.clearSuprasegmentalSyllableData = function() {
  return this.setSuprasegmentalSyllableData(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Utterance.prototype.hasSuprasegmentalSyllableData = function() {
  return jspb.Message.getField(this, 202) != null;
};


/**
 * optional ArtificialConcatenativeSpecificationData artificial_concatenative_data = 203;
 * @return {?proto.speech.tts.ArtificialConcatenativeSpecificationData}
 */
proto.speech.tts.Utterance.prototype.getArtificialConcatenativeData = function() {
  return /** @type{?proto.speech.tts.ArtificialConcatenativeSpecificationData} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.ArtificialConcatenativeSpecificationData, 203));
};


/**
 * @param {?proto.speech.tts.ArtificialConcatenativeSpecificationData|undefined} value
 * @return {!proto.speech.tts.Utterance} returns this
*/
proto.speech.tts.Utterance.prototype.setArtificialConcatenativeData = function(value) {
  return jspb.Message.setWrapperField(this, 203, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Utterance} returns this
 */
proto.speech.tts.Utterance.prototype.clearArtificialConcatenativeData = function() {
  return this.setArtificialConcatenativeData(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Utterance.prototype.hasArtificialConcatenativeData = function() {
  return jspb.Message.getField(this, 203) != null;
};


/**
 * optional VoiceBuildingData voice_building_data = 250;
 * @return {?proto.speech.tts.VoiceBuildingData}
 */
proto.speech.tts.Utterance.prototype.getVoiceBuildingData = function() {
  return /** @type{?proto.speech.tts.VoiceBuildingData} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.VoiceBuildingData, 250));
};


/**
 * @param {?proto.speech.tts.VoiceBuildingData|undefined} value
 * @return {!proto.speech.tts.Utterance} returns this
*/
proto.speech.tts.Utterance.prototype.setVoiceBuildingData = function(value) {
  return jspb.Message.setWrapperField(this, 250, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Utterance} returns this
 */
proto.speech.tts.Utterance.prototype.clearVoiceBuildingData = function() {
  return this.setVoiceBuildingData(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Utterance.prototype.hasVoiceBuildingData = function() {
  return jspb.Message.getField(this, 250) != null;
};


/**
 * optional AudioBuffer audio = 300;
 * @return {?proto.speech.tts.AudioBuffer}
 */
proto.speech.tts.Utterance.prototype.getAudio = function() {
  return /** @type{?proto.speech.tts.AudioBuffer} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.AudioBuffer, 300));
};


/**
 * @param {?proto.speech.tts.AudioBuffer|undefined} value
 * @return {!proto.speech.tts.Utterance} returns this
*/
proto.speech.tts.Utterance.prototype.setAudio = function(value) {
  return jspb.Message.setWrapperField(this, 300, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Utterance} returns this
 */
proto.speech.tts.Utterance.prototype.clearAudio = function() {
  return this.setAudio(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Utterance.prototype.hasAudio = function() {
  return jspb.Message.getField(this, 300) != null;
};


/**
 * optional Diagnostics diagnostics = 900;
 * @return {?proto.speech.tts.Diagnostics}
 */
proto.speech.tts.Utterance.prototype.getDiagnostics = function() {
  return /** @type{?proto.speech.tts.Diagnostics} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Diagnostics, 900));
};


/**
 * @param {?proto.speech.tts.Diagnostics|undefined} value
 * @return {!proto.speech.tts.Utterance} returns this
*/
proto.speech.tts.Utterance.prototype.setDiagnostics = function(value) {
  return jspb.Message.setWrapperField(this, 900, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Utterance} returns this
 */
proto.speech.tts.Utterance.prototype.clearDiagnostics = function() {
  return this.setDiagnostics(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Utterance.prototype.hasDiagnostics = function() {
  return jspb.Message.getField(this, 900) != null;
};


/**
 * optional bool depends_on_geo_location = 901;
 * @return {?boolean}
 */
proto.speech.tts.Utterance.prototype.getDependsOnGeoLocation = function() {
  return /** @type {?boolean} */ (jspb.Message.getBooleanField(this, 901));
};


/**
 * @param {?boolean|undefined} value
 * @return {!proto.speech.tts.Utterance} returns this
 */
proto.speech.tts.Utterance.prototype.setDependsOnGeoLocation = function(value) {
  return jspb.Message.setField(this, 901, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.Utterance} returns this
 */
proto.speech.tts.Utterance.prototype.clearDependsOnGeoLocation = function() {
  return jspb.Message.setField(this, 901, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Utterance.prototype.hasDependsOnGeoLocation = function() {
  return jspb.Message.getField(this, 901) != null;
};


/**
 * optional WaveNetInferenceCost wavenet_inference_cost = 902;
 * @return {?proto.speech.tts.WaveNetInferenceCost}
 */
proto.speech.tts.Utterance.prototype.getWavenetInferenceCost = function() {
  return /** @type{?proto.speech.tts.WaveNetInferenceCost} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.WaveNetInferenceCost, 902));
};


/**
 * @param {?proto.speech.tts.WaveNetInferenceCost|undefined} value
 * @return {!proto.speech.tts.Utterance} returns this
*/
proto.speech.tts.Utterance.prototype.setWavenetInferenceCost = function(value) {
  return jspb.Message.setWrapperField(this, 902, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Utterance} returns this
 */
proto.speech.tts.Utterance.prototype.clearWavenetInferenceCost = function() {
  return this.setWavenetInferenceCost(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Utterance.prototype.hasWavenetInferenceCost = function() {
  return jspb.Message.getField(this, 902) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.Utterance}
 */
proto.speech.tts.Utterance.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.Utterance, data);
};



//speech/tts/engine/controller/api/tts_controller_event_proto_jspb/ttscontrollerevent.jspb.js
// source: speech/tts/engine/controller/api/tts_controller_event.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.TtsControllerEvent');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.ExtensionFieldBinaryInfo');
goog.require('jspb.ExtensionFieldInfo');
goog.require('jspb.Message');
goog.require('proto.greco.Loggable');
goog.require('proto.speech.tts.AudioBuffer');
goog.require('proto.speech.tts.Utterance');

goog.forwardDeclare('proto.speech.tts.ReadStatus');
goog.forwardDeclare('proto.speech.tts.TtsControllerEventType');
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.TtsControllerEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.speech.tts.TtsControllerEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.TtsControllerEvent.displayName = 'proto.speech.tts.TtsControllerEvent';
}



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.TtsControllerEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.TtsControllerEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.TtsControllerEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.TtsControllerEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    eventType: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    timestamp: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,
    output: (f = msg.getOutput()) && proto.speech.tts.Utterance.toObject(includeInstance, f),
    audio: (f = msg.getAudio()) && proto.speech.tts.AudioBuffer.toObject(includeInstance, f),
    readStatus: (f = jspb.Message.getField(msg, 5)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of TtsControllerEvent as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.TtsControllerEvent.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.eventType;

  /** @type {?number|undefined} */
  this.timestamp;

  /** @type {?proto.speech.tts.Utterance.ObjectFormat|undefined} */
  this.output;

  /** @type {?proto.speech.tts.AudioBuffer.ObjectFormat|undefined} */
  this.audio;

  /** @type {?number|undefined} */
  this.readStatus;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.TtsControllerEvent.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.TtsControllerEvent}
 */
proto.speech.tts.TtsControllerEvent.fromObject = function(obj) {
  var msg = new proto.speech.tts.TtsControllerEvent();
  obj.eventType != null && jspb.Message.setField(msg, 1, obj.eventType);
  obj.timestamp != null && jspb.Message.setField(msg, 2, obj.timestamp);
  obj.output && jspb.Message.setWrapperField(
      msg, 6, proto.speech.tts.Utterance.fromObject(obj.output));
  obj.audio && jspb.Message.setWrapperField(
      msg, 4, proto.speech.tts.AudioBuffer.fromObject(obj.audio));
  obj.readStatus != null && jspb.Message.setField(msg, 5, obj.readStatus);
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.TtsControllerEvent}
 */
proto.speech.tts.TtsControllerEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.TtsControllerEvent;
  return proto.speech.tts.TtsControllerEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.TtsControllerEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.TtsControllerEvent}
 */
proto.speech.tts.TtsControllerEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {!proto.speech.tts.TtsControllerEventType} */ (reader.readEnum());
      msg.setEventType(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setTimestamp(value);
      break;
    case 6:
      var value = new proto.speech.tts.Utterance;
      reader.readMessage(value,proto.speech.tts.Utterance.deserializeBinaryFromReader);
      msg.setOutput(value);
      break;
    case 4:
      var value = new proto.speech.tts.AudioBuffer;
      reader.readMessage(value,proto.speech.tts.AudioBuffer.deserializeBinaryFromReader);
      msg.setAudio(value);
      break;
    case 5:
      var value = /** @type {!proto.speech.tts.ReadStatus} */ (reader.readEnum());
      msg.setReadStatus(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.TtsControllerEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.TtsControllerEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.TtsControllerEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.TtsControllerEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {!proto.speech.tts.TtsControllerEventType} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeEnum(
      1,
      f
    );
  }
  f = /** @type {?number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeInt64(
      2,
      f
    );
  }
  f = message.getOutput();
  if (f != null) {
    writer.writeMessage(
      6,
      f,
      proto.speech.tts.Utterance.serializeBinaryToWriter
    );
  }
  f = message.getAudio();
  if (f != null) {
    writer.writeMessage(
      4,
      f,
      proto.speech.tts.AudioBuffer.serializeBinaryToWriter
    );
  }
  f = /** @type {!proto.speech.tts.ReadStatus} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeEnum(
      5,
      f
    );
  }
};



/**
 * A tuple of {field number, class constructor} for the extension
 * field named `logId`.
 * @type {!jspb.ExtensionFieldInfo<!proto.speech.tts.TtsControllerEvent>}
 */
proto.speech.tts.TtsControllerEvent.logId = new jspb.ExtensionFieldInfo(
    48827838,
    {logId: 0},
    proto.speech.tts.TtsControllerEvent,
     /** @type {?function((boolean|undefined),!jspb.Message=): !Object} */ (
         proto.speech.tts.TtsControllerEvent.toObject),
    0);

proto.greco.Loggable.extensionsBinary[48827838] = new jspb.ExtensionFieldBinaryInfo(
    proto.speech.tts.TtsControllerEvent.logId,
    jspb.BinaryReader.prototype.readMessage,
    jspb.BinaryWriter.prototype.writeMessage,
    proto.speech.tts.TtsControllerEvent.serializeBinaryToWriter,
    proto.speech.tts.TtsControllerEvent.deserializeBinaryFromReader,
    false);
// This registers the extension field with the extended class, so that
// toObject() will function correctly.
proto.greco.Loggable.extensions[48827838] = proto.speech.tts.TtsControllerEvent.logId;

/**
 * required TtsControllerEventType event_type = 1;
 * @return {!proto.speech.tts.TtsControllerEventType}
 */
proto.speech.tts.TtsControllerEvent.prototype.getEventType = function() {
  return /** @type {!proto.speech.tts.TtsControllerEventType} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?proto.speech.tts.TtsControllerEventType|undefined} value
 * @return {!proto.speech.tts.TtsControllerEvent} returns this
 */
proto.speech.tts.TtsControllerEvent.prototype.setEventType = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.TtsControllerEvent} returns this
 */
proto.speech.tts.TtsControllerEvent.prototype.clearEventType = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.TtsControllerEvent.prototype.hasEventType = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional int64 timestamp = 2;
 * @return {?number}
 */
proto.speech.tts.TtsControllerEvent.prototype.getTimestamp = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 2));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.TtsControllerEvent} returns this
 */
proto.speech.tts.TtsControllerEvent.prototype.setTimestamp = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.TtsControllerEvent} returns this
 */
proto.speech.tts.TtsControllerEvent.prototype.clearTimestamp = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.TtsControllerEvent.prototype.hasTimestamp = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional Utterance output = 6;
 * @return {?proto.speech.tts.Utterance}
 */
proto.speech.tts.TtsControllerEvent.prototype.getOutput = function() {
  return /** @type{?proto.speech.tts.Utterance} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.Utterance, 6));
};


/**
 * @param {?proto.speech.tts.Utterance|undefined} value
 * @return {!proto.speech.tts.TtsControllerEvent} returns this
*/
proto.speech.tts.TtsControllerEvent.prototype.setOutput = function(value) {
  return jspb.Message.setWrapperField(this, 6, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.TtsControllerEvent} returns this
 */
proto.speech.tts.TtsControllerEvent.prototype.clearOutput = function() {
  return this.setOutput(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.TtsControllerEvent.prototype.hasOutput = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional AudioBuffer audio = 4;
 * @return {?proto.speech.tts.AudioBuffer}
 */
proto.speech.tts.TtsControllerEvent.prototype.getAudio = function() {
  return /** @type{?proto.speech.tts.AudioBuffer} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.AudioBuffer, 4));
};


/**
 * @param {?proto.speech.tts.AudioBuffer|undefined} value
 * @return {!proto.speech.tts.TtsControllerEvent} returns this
*/
proto.speech.tts.TtsControllerEvent.prototype.setAudio = function(value) {
  return jspb.Message.setWrapperField(this, 4, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.TtsControllerEvent} returns this
 */
proto.speech.tts.TtsControllerEvent.prototype.clearAudio = function() {
  return this.setAudio(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.TtsControllerEvent.prototype.hasAudio = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional ReadStatus read_status = 5;
 * @return {!proto.speech.tts.ReadStatus}
 */
proto.speech.tts.TtsControllerEvent.prototype.getReadStatus = function() {
  return /** @type {!proto.speech.tts.ReadStatus} */ (jspb.Message.getField(this, 5));
};


/**
 * @param {?proto.speech.tts.ReadStatus|undefined} value
 * @return {!proto.speech.tts.TtsControllerEvent} returns this
 */
proto.speech.tts.TtsControllerEvent.prototype.setReadStatus = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.TtsControllerEvent} returns this
 */
proto.speech.tts.TtsControllerEvent.prototype.clearReadStatus = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.TtsControllerEvent.prototype.hasReadStatus = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.TtsControllerEvent}
 */
proto.speech.tts.TtsControllerEvent.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.TtsControllerEvent, data);
};



//speech/tts/public/lucid/lucid_markup_proto_jspb/lucidmarkup.jspb.js
// source: speech/tts/public/lucid/lucid_markup.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.lucid.LucidMarkup');
goog.provide('proto.speech.tts.lucid.LucidMarkup.Content');
goog.provide('proto.speech.tts.lucid.LucidMarkup.Content.ItemCase');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.lucid.Annotation');
goog.require('proto.speech.tts.lucid.LucidUtterance');
goog.require('proto.speech.tts.lucid.MultiSentenceText');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.lucid.LucidMarkup = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 3, proto.speech.tts.lucid.LucidMarkup.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.lucid.LucidMarkup, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.lucid.LucidMarkup.displayName = 'proto.speech.tts.lucid.LucidMarkup';
}

/**
 * The extensions registered with this message class. This is a map of
 * extension field number to fieldInfo object.
 *
 * For example:
 *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }
 *
 * fieldName contains the JsCompiler renamed field name property so that it
 * works in OPTIMIZED mode.
 *
 * @type {!Object<number, jspb.ExtensionFieldInfo>}
 */
proto.speech.tts.lucid.LucidMarkup.extensions = {};


/**
 * The extensions registered with this message class. This is a map of
 * extension field number to fieldInfo object.
 *
 * For example:
 *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }
 *
 * fieldName contains the JsCompiler renamed field name property so that it
 * works in OPTIMIZED mode.
 *
 * @type {!Object<number, jspb.ExtensionFieldBinaryInfo>}
 */
proto.speech.tts.lucid.LucidMarkup.extensionsBinary = {};

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.lucid.LucidMarkup.Content = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, proto.speech.tts.lucid.LucidMarkup.Content.oneofGroups_);
};
goog.inherits(proto.speech.tts.lucid.LucidMarkup.Content, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.lucid.LucidMarkup.Content.displayName = 'proto.speech.tts.lucid.LucidMarkup.Content';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.lucid.LucidMarkup.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.lucid.LucidMarkup.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.lucid.LucidMarkup.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.lucid.LucidMarkup} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.LucidMarkup.toObject = function(includeInstance, msg) {
  var f, obj = {
    contentsList: jspb.Message.toObjectList(msg.getContentsList(),
    proto.speech.tts.lucid.LucidMarkup.Content.toObject, includeInstance),
    annotation: (f = msg.getAnnotation()) && proto.speech.tts.lucid.Annotation.toObject(includeInstance, f)
  };

  jspb.Message.toObjectExtension(/** @type {!jspb.Message} */ (msg), obj,
      proto.speech.tts.lucid.LucidMarkup.extensions, proto.speech.tts.lucid.LucidMarkup.prototype.getExtension,
      includeInstance);
  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of LucidMarkup as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.lucid.LucidMarkup.ObjectFormat = function() {
  /** @type {?Array<!proto.speech.tts.lucid.LucidMarkup.Content.ObjectFormat>|undefined} */
  this.contentsList;

  /** @type {?proto.speech.tts.lucid.Annotation.ObjectFormat|undefined} */
  this.annotation;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.lucid.LucidMarkup.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.lucid.LucidMarkup}
 */
proto.speech.tts.lucid.LucidMarkup.fromObject = function(obj) {
  var msg = new proto.speech.tts.lucid.LucidMarkup();
  obj.contentsList && jspb.Message.setRepeatedWrapperField(
      msg, 1, obj.contentsList.map(
          proto.speech.tts.lucid.LucidMarkup.Content.fromObject));
  obj.annotation && jspb.Message.setWrapperField(
      msg, 2, proto.speech.tts.lucid.Annotation.fromObject(obj.annotation));
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.lucid.LucidMarkup}
 */
proto.speech.tts.lucid.LucidMarkup.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.lucid.LucidMarkup;
  return proto.speech.tts.lucid.LucidMarkup.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.lucid.LucidMarkup} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.lucid.LucidMarkup}
 */
proto.speech.tts.lucid.LucidMarkup.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.speech.tts.lucid.LucidMarkup.Content;
      reader.readMessage(value,proto.speech.tts.lucid.LucidMarkup.Content.deserializeBinaryFromReader);
      msg.addContents(value);
      break;
    case 2:
      var value = new proto.speech.tts.lucid.Annotation;
      reader.readMessage(value,proto.speech.tts.lucid.Annotation.deserializeBinaryFromReader);
      msg.setAnnotation(value);
      break;
    default:
      jspb.Message.readBinaryExtension(msg, reader,
        proto.speech.tts.lucid.LucidMarkup.extensionsBinary,
        proto.speech.tts.lucid.LucidMarkup.prototype.getExtension,
        proto.speech.tts.lucid.LucidMarkup.prototype.setExtension);
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.lucid.LucidMarkup.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.lucid.LucidMarkup.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.lucid.LucidMarkup} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.LucidMarkup.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getContentsList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      1,
      f,
      proto.speech.tts.lucid.LucidMarkup.Content.serializeBinaryToWriter
    );
  }
  f = message.getAnnotation();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.speech.tts.lucid.Annotation.serializeBinaryToWriter
    );
  }
  jspb.Message.serializeBinaryExtensions(message, writer,
    proto.speech.tts.lucid.LucidMarkup.extensionsBinary, proto.speech.tts.lucid.LucidMarkup.prototype.getExtension);
};



/**
 * Oneof group definitions for this message. Each group defines the field
 * numbers belonging to that group. When of these fields' value is set, all
 * other fields in the group are cleared. During deserialization, if multiple
 * fields are encountered for a group, only the last value seen will be kept.
 * @private {!Array<!Array<number>>}
 * @const
 */
proto.speech.tts.lucid.LucidMarkup.Content.oneofGroups_ = [[2,3]];

/**
 * @enum {number}
 */
proto.speech.tts.lucid.LucidMarkup.Content.ItemCase = {
  ITEM_NOT_SET: 0,
  UTTERANCE: 2,
  MAYBE_MULTI_SENTENCE: 3
};

/**
 * @return {!proto.speech.tts.lucid.LucidMarkup.Content.ItemCase}
 */
proto.speech.tts.lucid.LucidMarkup.Content.prototype.getItemCase = function() {
  return /** @type {!proto.speech.tts.lucid.LucidMarkup.Content.ItemCase} */(jspb.Message.computeOneofCase(this, proto.speech.tts.lucid.LucidMarkup.Content.oneofGroups_[0]));
};



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.lucid.LucidMarkup.Content.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.lucid.LucidMarkup.Content.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.lucid.LucidMarkup.Content} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.LucidMarkup.Content.toObject = function(includeInstance, msg) {
  var f, obj = {
    ownIndex: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,
    utterance: (f = msg.getUtterance()) && proto.speech.tts.lucid.LucidUtterance.toObject(includeInstance, f),
    maybeMultiSentence: (f = msg.getMaybeMultiSentence()) && proto.speech.tts.lucid.MultiSentenceText.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Content as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.lucid.LucidMarkup.Content.ObjectFormat = function() {
  /** @type {?number|undefined} */
  this.ownIndex;

  /** @type {?proto.speech.tts.lucid.LucidUtterance.ObjectFormat|undefined} */
  this.utterance;

  /** @type {?proto.speech.tts.lucid.MultiSentenceText.ObjectFormat|undefined} */
  this.maybeMultiSentence;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.lucid.LucidMarkup.Content.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.lucid.LucidMarkup.Content}
 */
proto.speech.tts.lucid.LucidMarkup.Content.fromObject = function(obj) {
  var msg = new proto.speech.tts.lucid.LucidMarkup.Content();
  obj.ownIndex != null && jspb.Message.setField(msg, 1, obj.ownIndex);
  obj.utterance && jspb.Message.setWrapperField(
      msg, 2, proto.speech.tts.lucid.LucidUtterance.fromObject(obj.utterance));
  obj.maybeMultiSentence && jspb.Message.setWrapperField(
      msg, 3, proto.speech.tts.lucid.MultiSentenceText.fromObject(obj.maybeMultiSentence));
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.lucid.LucidMarkup.Content}
 */
proto.speech.tts.lucid.LucidMarkup.Content.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.lucid.LucidMarkup.Content;
  return proto.speech.tts.lucid.LucidMarkup.Content.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.lucid.LucidMarkup.Content} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.lucid.LucidMarkup.Content}
 */
proto.speech.tts.lucid.LucidMarkup.Content.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {number} */ (reader.readInt32());
      msg.setOwnIndex(value);
      break;
    case 2:
      var value = new proto.speech.tts.lucid.LucidUtterance;
      reader.readMessage(value,proto.speech.tts.lucid.LucidUtterance.deserializeBinaryFromReader);
      msg.setUtterance(value);
      break;
    case 3:
      var value = new proto.speech.tts.lucid.MultiSentenceText;
      reader.readMessage(value,proto.speech.tts.lucid.MultiSentenceText.deserializeBinaryFromReader);
      msg.setMaybeMultiSentence(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.lucid.LucidMarkup.Content.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.lucid.LucidMarkup.Content.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.lucid.LucidMarkup.Content} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.lucid.LucidMarkup.Content.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {?number} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeInt32(
      1,
      f
    );
  }
  f = message.getUtterance();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.speech.tts.lucid.LucidUtterance.serializeBinaryToWriter
    );
  }
  f = message.getMaybeMultiSentence();
  if (f != null) {
    writer.writeMessage(
      3,
      f,
      proto.speech.tts.lucid.MultiSentenceText.serializeBinaryToWriter
    );
  }
};


/**
 * optional int32 own_index = 1;
 * @return {?number}
 */
proto.speech.tts.lucid.LucidMarkup.Content.prototype.getOwnIndex = function() {
  return /** @type {?number} */ (jspb.Message.getField(this, 1));
};


/**
 * @param {?number|undefined} value
 * @return {!proto.speech.tts.lucid.LucidMarkup.Content} returns this
 */
proto.speech.tts.lucid.LucidMarkup.Content.prototype.setOwnIndex = function(value) {
  return jspb.Message.setField(this, 1, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.speech.tts.lucid.LucidMarkup.Content} returns this
 */
proto.speech.tts.lucid.LucidMarkup.Content.prototype.clearOwnIndex = function() {
  return jspb.Message.setField(this, 1, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.LucidMarkup.Content.prototype.hasOwnIndex = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional LucidUtterance utterance = 2;
 * @return {?proto.speech.tts.lucid.LucidUtterance}
 */
proto.speech.tts.lucid.LucidMarkup.Content.prototype.getUtterance = function() {
  return /** @type{?proto.speech.tts.lucid.LucidUtterance} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.lucid.LucidUtterance, 2));
};


/**
 * @param {?proto.speech.tts.lucid.LucidUtterance|undefined} value
 * @return {!proto.speech.tts.lucid.LucidMarkup.Content} returns this
*/
proto.speech.tts.lucid.LucidMarkup.Content.prototype.setUtterance = function(value) {
  return jspb.Message.setOneofWrapperField(this, 2, proto.speech.tts.lucid.LucidMarkup.Content.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.lucid.LucidMarkup.Content} returns this
 */
proto.speech.tts.lucid.LucidMarkup.Content.prototype.clearUtterance = function() {
  return this.setUtterance(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.LucidMarkup.Content.prototype.hasUtterance = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional MultiSentenceText maybe_multi_sentence = 3;
 * @return {?proto.speech.tts.lucid.MultiSentenceText}
 */
proto.speech.tts.lucid.LucidMarkup.Content.prototype.getMaybeMultiSentence = function() {
  return /** @type{?proto.speech.tts.lucid.MultiSentenceText} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.lucid.MultiSentenceText, 3));
};


/**
 * @param {?proto.speech.tts.lucid.MultiSentenceText|undefined} value
 * @return {!proto.speech.tts.lucid.LucidMarkup.Content} returns this
*/
proto.speech.tts.lucid.LucidMarkup.Content.prototype.setMaybeMultiSentence = function(value) {
  return jspb.Message.setOneofWrapperField(this, 3, proto.speech.tts.lucid.LucidMarkup.Content.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.lucid.LucidMarkup.Content} returns this
 */
proto.speech.tts.lucid.LucidMarkup.Content.prototype.clearMaybeMultiSentence = function() {
  return this.setMaybeMultiSentence(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.LucidMarkup.Content.prototype.hasMaybeMultiSentence = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.lucid.LucidMarkup.Content}
 */
proto.speech.tts.lucid.LucidMarkup.Content.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.lucid.LucidMarkup.Content, data);
};


/**
 * repeated Content contents = 1;
 * @return {!Array<!proto.speech.tts.lucid.LucidMarkup.Content>}
 */
proto.speech.tts.lucid.LucidMarkup.prototype.getContentsList = function() {
  return /** @type{!Array<!proto.speech.tts.lucid.LucidMarkup.Content>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.lucid.LucidMarkup.Content, 1));
};


/**
 * @param {Array<!proto.speech.tts.lucid.LucidMarkup.Content>|undefined} value
 * @return {!proto.speech.tts.lucid.LucidMarkup} returns this
*/
proto.speech.tts.lucid.LucidMarkup.prototype.setContentsList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 1, value);
};


/**
 * @param {!proto.speech.tts.lucid.LucidMarkup.Content=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.lucid.LucidMarkup.Content}
 */
proto.speech.tts.lucid.LucidMarkup.prototype.addContents = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.speech.tts.lucid.LucidMarkup.Content, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.lucid.LucidMarkup} returns this
 */
proto.speech.tts.lucid.LucidMarkup.prototype.clearContentsList = function() {
  return this.setContentsList([]);
};


/**
 * optional Annotation annotation = 2;
 * @return {?proto.speech.tts.lucid.Annotation}
 */
proto.speech.tts.lucid.LucidMarkup.prototype.getAnnotation = function() {
  return /** @type{?proto.speech.tts.lucid.Annotation} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.lucid.Annotation, 2));
};


/**
 * @param {?proto.speech.tts.lucid.Annotation|undefined} value
 * @return {!proto.speech.tts.lucid.LucidMarkup} returns this
*/
proto.speech.tts.lucid.LucidMarkup.prototype.setAnnotation = function(value) {
  return jspb.Message.setWrapperField(this, 2, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.lucid.LucidMarkup} returns this
 */
proto.speech.tts.lucid.LucidMarkup.prototype.clearAnnotation = function() {
  return this.setAnnotation(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.lucid.LucidMarkup.prototype.hasAnnotation = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.lucid.LucidMarkup}
 */
proto.speech.tts.lucid.LucidMarkup.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.lucid.LucidMarkup, data);
};



//speech/tts/public/tts_markup_proto_jspb/text.jspb.js
// source: speech/tts/public/tts_markup.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!

goog.provide('proto.speech.tts.Text');

goog.require('jspb.BinaryReader');
goog.require('jspb.BinaryWriter');
goog.require('jspb.Message');
goog.require('proto.speech.tts.Sentence');
goog.require('proto.speech.tts.lucid.LucidMarkup');

/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 * @final
 */
proto.speech.tts.Text = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 3, proto.speech.tts.Text.repeatedFields_, null);
};
goog.inherits(proto.speech.tts.Text, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.speech.tts.Text.displayName = 'proto.speech.tts.Text';
}

/**
 * The extensions registered with this message class. This is a map of
 * extension field number to fieldInfo object.
 *
 * For example:
 *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }
 *
 * fieldName contains the JsCompiler renamed field name property so that it
 * works in OPTIMIZED mode.
 *
 * @type {!Object<number, jspb.ExtensionFieldInfo>}
 */
proto.speech.tts.Text.extensions = {};


/**
 * The extensions registered with this message class. This is a map of
 * extension field number to fieldInfo object.
 *
 * For example:
 *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }
 *
 * fieldName contains the JsCompiler renamed field name property so that it
 * works in OPTIMIZED mode.
 *
 * @type {!Object<number, jspb.ExtensionFieldBinaryInfo>}
 */
proto.speech.tts.Text.extensionsBinary = {};


/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.speech.tts.Text.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.speech.tts.Text.prototype.toObject = function(opt_includeInstance) {
  return proto.speech.tts.Text.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.speech.tts.Text} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Text.toObject = function(includeInstance, msg) {
  var f, obj = {
    sentenceList: jspb.Message.toObjectList(msg.getSentenceList(),
    proto.speech.tts.Sentence.toObject, includeInstance),
    lucid: (f = msg.getLucid()) && proto.speech.tts.lucid.LucidMarkup.toObject(includeInstance, f)
  };

  jspb.Message.toObjectExtension(/** @type {!jspb.Message} */ (msg), obj,
      proto.speech.tts.Text.extensions, proto.speech.tts.Text.prototype.getExtension,
      includeInstance);
  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


if (jspb.Message.GENERATE_FROM_OBJECT) {

/**
 * The raw object form of Text as accepted by the `fromObject` method.
 * @record
 */
proto.speech.tts.Text.ObjectFormat = function() {
  /** @type {?Array<!proto.speech.tts.Sentence.ObjectFormat>|undefined} */
  this.sentenceList;

  /** @type {?proto.speech.tts.lucid.LucidMarkup.ObjectFormat|undefined} */
  this.lucid;
};

/**
 * Loads data from an object into a new instance of this proto.
 * @param {!proto.speech.tts.Text.ObjectFormat} obj
 *     The object representation of this proto to load the data from.
 * @return {!proto.speech.tts.Text}
 */
proto.speech.tts.Text.fromObject = function(obj) {
  var msg = new proto.speech.tts.Text();
  obj.sentenceList && jspb.Message.setRepeatedWrapperField(
      msg, 1, obj.sentenceList.map(
          proto.speech.tts.Sentence.fromObject));
  obj.lucid && jspb.Message.setWrapperField(
      msg, 2, proto.speech.tts.lucid.LucidMarkup.fromObject(obj.lucid));
  return msg;
};
}

/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.speech.tts.Text}
 */
proto.speech.tts.Text.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.speech.tts.Text;
  return proto.speech.tts.Text.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.speech.tts.Text} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.speech.tts.Text}
 */
proto.speech.tts.Text.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.speech.tts.Sentence;
      reader.readMessage(value,proto.speech.tts.Sentence.deserializeBinaryFromReader);
      msg.addSentence(value);
      break;
    case 2:
      var value = new proto.speech.tts.lucid.LucidMarkup;
      reader.readMessage(value,proto.speech.tts.lucid.LucidMarkup.deserializeBinaryFromReader);
      msg.setLucid(value);
      break;
    default:
      jspb.Message.readBinaryExtension(msg, reader,
        proto.speech.tts.Text.extensionsBinary,
        proto.speech.tts.Text.prototype.getExtension,
        proto.speech.tts.Text.prototype.setExtension);
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.speech.tts.Text.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.speech.tts.Text.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.speech.tts.Text} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.speech.tts.Text.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getSentenceList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      1,
      f,
      proto.speech.tts.Sentence.serializeBinaryToWriter
    );
  }
  f = message.getLucid();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.speech.tts.lucid.LucidMarkup.serializeBinaryToWriter
    );
  }
  jspb.Message.serializeBinaryExtensions(message, writer,
    proto.speech.tts.Text.extensionsBinary, proto.speech.tts.Text.prototype.getExtension);
};


/**
 * repeated Sentence sentence = 1;
 * @return {!Array<!proto.speech.tts.Sentence>}
 */
proto.speech.tts.Text.prototype.getSentenceList = function() {
  return /** @type{!Array<!proto.speech.tts.Sentence>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.speech.tts.Sentence, 1));
};


/**
 * @param {Array<!proto.speech.tts.Sentence>|undefined} value
 * @return {!proto.speech.tts.Text} returns this
*/
proto.speech.tts.Text.prototype.setSentenceList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 1, value);
};


/**
 * @param {!proto.speech.tts.Sentence=} opt_value
 * @param {number=} opt_index
 * @return {!proto.speech.tts.Sentence}
 */
proto.speech.tts.Text.prototype.addSentence = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.speech.tts.Sentence, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.speech.tts.Text} returns this
 */
proto.speech.tts.Text.prototype.clearSentenceList = function() {
  return this.setSentenceList([]);
};


/**
 * optional lucid.LucidMarkup lucid = 2;
 * @return {?proto.speech.tts.lucid.LucidMarkup}
 */
proto.speech.tts.Text.prototype.getLucid = function() {
  return /** @type{?proto.speech.tts.lucid.LucidMarkup} */ (
    jspb.Message.getWrapperField(this, proto.speech.tts.lucid.LucidMarkup, 2));
};


/**
 * @param {?proto.speech.tts.lucid.LucidMarkup|undefined} value
 * @return {!proto.speech.tts.Text} returns this
*/
proto.speech.tts.Text.prototype.setLucid = function(value) {
  return jspb.Message.setWrapperField(this, 2, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.speech.tts.Text} returns this
 */
proto.speech.tts.Text.prototype.clearLucid = function() {
  return this.setLucid(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.speech.tts.Text.prototype.hasLucid = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * @param {string} data The serialized JSON data in JSPB wire format.
 * @return {!proto.speech.tts.Text}
 */
proto.speech.tts.Text.deserialize = function(data) {
  return jspb.Message.deserializeWithCtor(proto.speech.tts.Text, data);
};



//speech/tts/platforms/webassembly/googletts_engine.js
/**
 * @fileoverview Webassembly port of GoogleTTS.
 */

'use strict';

goog.require('proto.speech.tts.Say');
goog.require('proto.speech.tts.Sentence');
goog.require('proto.speech.tts.Text');
goog.require('proto.speech.tts.TtsControllerEvent');
goog.require('proto.speech.tts.VoiceMod');

class GoogleTtsEngine {
  constructor() {
    /**
     * The callback provided by a call to chrome.ttsEngine.onSpeak, which is
     * our channel to send status updates about the progress of speech.
     * @private {?function(!object)}
     */
    this.ttsEngineApiCallback_ = null;

    /**
     * The string containing the text we're currently speaking.
     * @private {string?}
     */
    this.utterance_ = null;

    /**
     * The size, in samples, of each audio frame. When using an
     * AudioWorkletProcessor this is always 128 samples.
     * @private {number}
     * @const
     */
    this.bufferLen_ = 128;

    /**
     * A buffer of audio samples to pass to the streaming worklet processor.
     * @private {?Float32Array}
     */
    this.currentBuffer_ = null;

    /**
     * The current zero-based index into |this.currentBuffer_| where the
     * next audio sample should be written.
     * @private {number}
     */
    this.currentBufferIndex_ = 0;

    /**
     * The sample rate we get from audio from Google TTS engines.
     * TODO: make this dynamic and support any rate. Right now if we
     * get a different rate, we'll log an error message.
     * @private {number}
     * @const
     */
    this.googleTtsSampleRate_ = 22050;

    /**
     * The audio context, needed by any app using the Web Audio API.
     * @private {!AudioContext}
     * @const
     */
    this.context_ = new AudioContext({sampleRate: this.googleTtsSampleRate_});

    /**
     * A node that allows us to adjust the output volume. The
     * AudioWorkletNode connects to the gain node, and the gain node
     * connects to the audio context destination.
     * @const
     * @private
     */
    this.gainNode_ = this.context_.createGain();

    /**
     * Whether or not we're currently playing audio via the Web Audio
     * API. This is not set to true until after we've got at least one
     * buffer worth of samples from the engine and sent it to the
     * AudioWorkletProcessor.
     * @private {boolean}
     */
    this.playing_ = false;

    /**
     * The timestamp when speech started. Used to fire events like
     * word callbacks.
     * @private {Date?}
     */
    this.startTime_ = null;

/**
 * The instantiation time of this class.
* @private {!Date}
 */
    this.instantiationTime_ = new Date();

    /**
     * The queue of events that need to be scheduled to fire when
     * audio playback of the utterance begins.
     * @private {!Array<{
     *     audioDeltaMillis': number,
     *    textPosition: number,
     *    eventType: string
     * }>}
     */
    this.eventQueue_ = [];

    /**
     * The current voice's name.
     * @private {string?}
     */
    this.currentVoice_ = null;

    /**
     * The timeout IDs (from calling window.setTimeout) of event
     * callbacks that have already been scheduled. These are cleared
     * when we get a call to stop speech.
     * @private {!Array<number>}
     * @const
     */
    this.timeoutIds_ = [];

    /**
     * Callback to run after initialization.
     * @private
     */
    this.pendingCallback_ = null;

    /**
     * The opaque handle to the object returned by _GoogleTtsInitBuffered,
     * that needs to be passed to each other _GoogleTts function, until
     * synthesis is complete.
     * @private {?object}
     */
    this.state_ = null;

    /**
     * True if initialization is done.
     * @private {boolean}
     */
    this.initialized_ = false;

    /** @private {!Object} */
    this.voiceNameToUrl_ = {};

    /** @private {!Object} */
    this.voiceNameToId_ = {};

    /** @private {!Object} */
    this.voiceNameToFileId_ = {};

    /** @private {!Object} */
    this.voiceData_ = {};

    // Start listening to chrome.ttsEngine requests early so we queue up any
    // utterances and speak them as soon as we've actually initialized.
    chrome.ttsEngine.onSpeak.addListener(this.onSpeak.bind(this));
    chrome.ttsEngine.onStop.addListener(this.onStop.bind(this));

    // TODO(dmazzoni): add onPause / onResume - should be relatively
    // easy to implement on the AudioContext.

    window.addEventListener('message', this.onMessage_.bind(this));
    this.initializeVoiceMetadata();

    let ttsLoader = document.createElement('script');
    ttsLoader.src = 'bindings_main.js';
    ttsLoader.onload = () => {
      Module.onRuntimeInitialized = () => {
        console.log(
            'Module.onRuntimeInitialized (in ms): ' +
            (new Date() - this.instantiationTime_));

        // This is where the extracted voice data goes.
        FS.mkdir('/voices');
        FS.mount(IDBFS, {}, '/voices');

        // Sync from the database into memory.
        FS.syncfs(true, (err) => {
          this.finishInitialization_();
        });
      };
    };
    document.body.appendChild(ttsLoader);
  }

  /**
   * Finish initialization by initializing the audio worklet.
   * This is the final step, then we can start speaking.
   * @private
   */
  finishInitialization_() {
    // Initialize the audio worklet and hook up the gain node.
    this.context_.audioWorklet.addModule(
        'streaming_worklet_processor.js').then((function() {
          this.streamingNode_ = new AudioWorkletNode(
              this.context_, 'streaming-worklet-processor');
          this.streamingNode_.port.onmessage =
              this.onWorkletProcessorMessage.bind(this);
          this.gainNode_.connect(this.context_.destination);

          // Initialization is now complete.
          this.initializeVoiceMetadata();
          this.initialized_ = true;
          if (this.pendingCallback_) {
            this.pendingCallback_();
            this.pendingCallback_ = null;
          }
        }).bind(this));
  };

  /**
   * Retrieves metadata about voices.
   */
  initializeVoiceMetadata() {
    let xhr = new XMLHttpRequest();
    xhr.open('GET', chrome.extension.getURL('voices.json'), true);
    xhr.responseType = 'text';
    xhr.onload = (evt) => {
      let json = xhr.response;
      if (!json) {
        throw 'Missing voices.json file';
      }

      this.voiceData_ = JSON.parse(json);
      this.updateVoices_();
    };
    xhr.send(null);
  }

  /**
   * Called by the client to stop speech.
   */
  onStop() {
    this.pendingCallback_ = null;
    if (!this.initialized_) {
      return;
    }

    this.finishReadingSamples();
    this.playing_ = false;
    this.streamingNode_.disconnect();
    this.streamingNode_.port.postMessage({
      'command': 'clearBuffers'
    });
    this.ttsEngineApiCallback_ = null;
    this.eventQueue_ = [];
    this.timeoutIds_.forEach(function(timeoutId) {
      window.clearTimeout(timeoutId);
    });
  };

  /**
   * Called by the client to start speech synthesis.
   *
   * @param {string} utterance The utterance to say.
   * @param {!object} options The options affecting the speech, like language,
   *   pitch, rate, etc.
   * @param {function(!object)} callback The function to receive messages
   *   from the engine.
   */
  onSpeak(utterance, options, callback) {
    this.pendingCallback_ = null;

    if (!this.initialized_) {
      // We got a call to speak before we're initialized. Enqueue this until
      // we're ready.
      this.pendingCallback_ = this.onSpeak.bind(
          this, utterance, options, callback);
      return;
    }

    this.onStop();

    this.ttsEngineApiCallback_ = callback;
    this.utterance_ = utterance;

    if (this.currentVoice_ === options.voiceName) {
      // Already initialized, nothing to do.
      this.synthesize(utterance, options, callback);
      return;
    }

    this.ensureExists(options.voiceName, false, (pipeline, success) => {
      if (!success) {
        console.log('Failed to ensure voice exists.');
        return;
      }

      let voiceFileId = this.voiceNameToFileId_[options.voiceName];

      if (!voiceFileId) {
        throw 'Invalid voice name: ' + voiceName;
      }

      let pathPrefix = ['/voices', voiceFileId].join('/');
      if (!this.loadPipeline(pipeline, pathPrefix)) {
        console.log('Failed to initialize pipeline');
        return;
      }

      this.currentVoice_ = options.voiceName;
      this.synthesize(utterance, options, callback);
    });
  }

  /**
   * Synthesizes some text. Assumes everything is already set up.
   *
   * @param {string} utterance The utterance to synthesize.
   * @param {!object} options Options such as language, pitch etc.
   * @param {function(!object)} callback Gets called with TtsEvents from the
   *     engine.
   */
  synthesize(utterance, options, callback) {
    let say = new proto.speech.tts.Say().setText(utterance);

    let voiceMod = new proto.speech.tts.VoiceMod();

    if (options.rate) {
      let durScale = 1.0;
      // The Chrome TTS spec disallows anything outside the
      // range of 0.1 through 10.0. All we need to do is convert
      // a rate (where 2.0 means twice as fast) to a durScale
      // (where 0.5 means twice as fast).
      if (options.rate >= 0.0 && options.rate <= 10.0)
        durScale = 1.0 / options.rate;
      durScale = Math.min(Math.max(durScale, 0.1), 10.0);
      voiceMod.setDurScale(durScale);
    }

    // Converts the scale of 'pitch' from the API-oriented [0.0, 1.0] range
    // to pitch scale-factor range via an intrinsic (logarithmic) [-20, +20]
    // semitone scale.
    if (options.pitch) {
      const kPitchModificationRangeInSemitones = 20;
      let f0Scale =
          Math.pow(2.0, 2.0 * (options.pitch / 2.0 - 0.5) *
                   kPitchModificationRangeInSemitones / 12.0);
      voiceMod.setF0Scale(f0Scale);
    }

    if (options.volume) {
      let volume = Math.min(Math.max(options.volume, 0.0), 1.0);
      this.gainNode_.gain.value = volume;
    }

    say.setVoicemod(voiceMod);

    let sentence = new proto.speech.tts.Sentence().setSayList([say]);
    let text = new proto.speech.tts.Text().setSentenceList([sentence]);
    let serialized = text.serializeBinary();
    let ptr = allocate(serialized, 'i8', ALLOC_NORMAL);

    this.state_ = Module._GoogleTtsInitBuffered(ptr, serialized.length);
    _free(ptr);
    if (this.state_ == 0) {
      callback({'type': 'error'});
      return;
    }

    // Call this after unblocking the main thread.
    let id = setTimeout(() => {
      this.readMoreSamples(callback);
    }, 0);
    this.timeoutIds_.push(id);
  }

  /**
   * Keeps reading samples from the synthesis engine.
   *
   * @param {function(!object)} callback Gets called with TtsEvents from the
   *     engine.
   */
  readMoreSamples(callback) {
    let status = Module._GoogleTtsReadBuffered(this.state_);
    if (status == -1) {
      console.error('Status error');
      callback({'type': 'error'});
      this.finishReadingSamples();
    }

    let done = (status === 0);

    let timepointsLen = Module._GoogleTtsGetTimepointsCount();
    for (let i = 0; i < timepointsLen; i++) {
      this.scheduleTimepointEvent(
          Module._GoogleTtsGetTimepointsTimeInSecsAtIndex(i) * 1000,
          Number(Module._GoogleTtsGetTimepointsCharIndexAtIndex(i)), 'word');
    }

    let eventBufferPtr = Module._GoogleTtsGetEventBufferPtr();
    let eventBufferLen = Module._GoogleTtsGetEventBufferLen();
    let serializedEvent =
        new Uint8Array(Module.HEAPU8.buffer, eventBufferPtr, eventBufferLen);
    let event = proto.speech.tts.TtsControllerEvent.deserializeBinary(
        serializedEvent);
    if (event.hasAudio()) {
      let srcSampleRate = event.getAudio().getSampleRate();
      let dstSampleRate = this.context_.sampleRate;

      // TODO: we could recreate the audio context at this point if
      // necessary.
      if (srcSampleRate != this.googleTtsSampleRate_) {
        console.error('TTS engine sample rate is ' + srcSampleRate);
        console.error('Output sample rate is ' + dstSampleRate);
        return;
      }

      let uint8Samples = event.getAudio().getSamples_asU8();

      // Copy to a new array so that we can reinterpret it
      // as an Int16Array. (We can't just create another view
      // on the Uint8Array because the byte offset might not
      // be a multiple of two.)
      let uint8SamplesCopy = new Uint8Array(uint8Samples);
      let int16Samples = new Int16Array(uint8SamplesCopy.buffer);

      let floatSamples = new Float32Array(int16Samples.length);
      for (let i = 0; i < int16Samples.length; i++) {
        // Range of a 16-bit int is -32768...32767
        floatSamples[i] = int16Samples[i] / 32768.0;
      }
      this.processSamples(floatSamples, this.synthesizingDone_);
    }

    if (done) {
      this.finishReadingSamples();
    } else {
      // Call again after unblocking the main thread.
      this.timeoutIds_.push(setTimeout(this.readMoreSamples.bind(this, callback), 0));
    }
  }

  /**
   * Clean up from synthesizing one utterance and reading back samples.
   */
  finishReadingSamples() {
    Module._GoogleTtsFinalizeBuffered(this.state_);
    this.state_ = undefined;
  }

  /**
   * Called when we get samples back from the WebAssembly TTS engine.
   * Fills up buffers with samples, and as each buffer fills, sends it
   * to the AudioWorkletProcessor running in another thread.
   *
   * If these are the first samples for a new utterance, starts speech
   * by connecting the audio node and resuming the AudioContext.
   */
  processSamples(samples, isEnd) {
    if (!this.ttsEngineApiCallback_) {
      return;
    }

    let i = 0;
    let len = samples.length;

    let didSendBuffers = false;
    while (i < len) {
      let chunkLen = Math.min(this.bufferLen_ - this.currentBufferIndex_,
                  len - i);
      if (!this.currentBuffer_) {
        this.currentBuffer_ = new Float32Array(this.bufferLen_);
      }
      this.currentBuffer_.set(samples.subarray(i, i + chunkLen),
                              this.currentBufferIndex_);
      i += chunkLen;
      this.currentBufferIndex_ += chunkLen;
      if (this.currentBufferIndex_ == this.bufferLen_) {
        this.streamingNode_.port.postMessage({
          'command': 'addBuffer',
          'buffer': this.currentBuffer_
        });
        didSendBuffers = true;

        this.currentBufferIndex_ = 0;
        this.currentBuffer_ = null;
      }
    }

    // Push final buffer, not complete.
    if (isEnd && this.currentBufferIndex_ > 0) {
      this.streamingNode_.port.postMessage({
        'command': 'addBuffer',
        'buffer': this.currentBuffer_
      });
      didSendBuffers = true;
      this.currentBufferIndex_ = 0;
      this.currentBuffer_ = null;
    }

    // Connect and start playback if this is the
    // first buffer.
    if (didSendBuffers && !this.playing_) {
      this.streamingNode_.connect(this.gainNode_);
      this.context_.resume();
      this.playing_ = true;
      this.ttsEngineApiCallback_({
        'type': 'start',
        'charIndex': 0
      });
      this.startTime_ = new Date();
      this.scheduleQueuedTimepointEvents();
    }
  };

  /**
   * Called when we get a message from the worklet processor in
   * the audio thread. Currently the only message is that it has
   * run out of audio buffers. When that happens we fire the 'end'
   * event and stop speaking.
   */
  onWorkletProcessorMessage(event) {
    if (!this.ttsEngineApiCallback_) {
      // Do nothing if we've already stopped.
      return;
    }

    if (event.data.type == 'empty') {
      this.ttsEngineApiCallback_({
        'type': 'end',
        'charIndex': this.utterance_.length
      });
      this.onStop();
    }
  };

  /**
   * Schedules an event to be fired indicating progress of speech synthesis.
   * If audio playback has already started, schedule it now based on the
   * desired time and actual elapsed time so far. If audio playback has not
   * started yet, put it in a queue.
   * @param {number} audioDeltaMillis The delta between start and the
   *     position in the audio in milliseconds.
   * @param {number} textPosition The position in the utterance as a
   *     character offset in the string.
   * @param {string} eventType The event to fire.
   */
  scheduleTimepointEvent(audioDeltaMillis, textPosition, eventType) {
    if (textPosition < 0 || audioDeltaMillis <= 0) return;

    if (!this.playing_) {
      this.eventQueue_.push({
        'audioDeltaMillis': audioDeltaMillis,
        'textPosition': textPosition,
        'eventType': eventType
      });
      return;
    }

    if (audioDeltaMillis < -100) {
      console.log('Skipping event ' + eventType + ' in the past');
      // Skip it if it's too much in the past.
      return;
    } else if (audioDeltaMillis < 2) {
      // TODO: support |length|, too.

      // It's basically now, fire it.
      this.ttsEngineApiCallback_({
        'type': eventType,
        'charIndex': textPosition
      });
      return;
    }

    let timeoutId = window.setTimeout(
        (function() {
          this.ttsEngineApiCallback_(
              {'type': eventType, 'charIndex': textPosition});
        }).bind(this),
        audioDeltaMillis);
    this.timeoutIds_.push(timeoutId);
  }

  /**
   * If any audio events where queued up because they were generated
   * before audio playback had started, schedule them now.
   */
  scheduleQueuedTimepointEvents() {
    console.assert(this.playing_);
    let events = this.eventQueue_;
    this.eventQueue_ = [];
    events.forEach((event) => {
      this.scheduleTimepointEvent(
          event.audioDeltaMillis, event.textPosition, event.eventType);
    });
  }

  /**
   * Load up a pipeline.
   * @param {string} pipeline path to the voice pipeline file.
   * @param {string} pathPrefix path to the voice directory.
   * @return {boolean} whether the pipeline loaded successfully.
   */
  loadPipeline(pipeline, pathPrefix) {
    let pathPtr = allocate(intArrayFromString(pipeline), 'i8', ALLOC_NORMAL);
    let prefixPtr =
        allocate(intArrayFromString(pathPrefix), 'i8', ALLOC_NORMAL);
    const ret = Module._GoogleTtsInit(pathPtr, prefixPtr);
    _free(pathPtr);
    _free(prefixPtr);
    return ret;
  }

  /**
   * Makes sure that a voice exists inside the virtual filesystem that
   * webassembly uses. Downloads if necessary.
   *
   * @param {string} voiceName Name of the voice to look up.
   * @param {boolean} allowDownload The caller wants to download the voice if it
   * does not exist locally.
   * @param {function(string, boolean)} callback Callback to invoke once done.
   */
  ensureExists(voiceName, allowDownload, callback) {
    let voiceFileId = this.voiceNameToFileId_[voiceName];
    if (!voiceFileId) {
      throw 'Invalid voice name: ' + voiceName;
    }
    let pathPrefix = ['/voices', voiceFileId].join('/');
    let pipeline = [pathPrefix, 'pipeline'].join('/');

    // Check if it exists by calling stat. If it does, move forward.
    try {
      let stat = FS.stat(pipeline);
      if (stat) {
        callback(pipeline, true);
        return;
      }
    } catch (err) {
    }

    // The voice hasn't been unzipped to disk; fetch the zipped archive from the
    // url.
    let url = this.voiceNameToUrl_[voiceName];

    if (!allowDownload && url.indexOf('http') == 0) {
      throw new Error('Unexpected remote url ' + url);
    }

    console.log(pipeline + ' doesn\'t exist. loading from: ' + url);
    let xhr = new XMLHttpRequest();
    xhr.open('GET', url, true);
    xhr.responseType = 'arraybuffer';
    xhr.onload = (evt) => {
      let arrayBuffer = xhr.response;
      if (!arrayBuffer) {
        callback('', false);
        return;
      }

      // Now that we have the .zvoice, unzip it with the help of a native call.
      try {
        // FS.stat throws when it fails to find the path.
        FS.stat(pathPrefix);
      } catch (e) {
        FS.mkdir(pathPrefix);
      }

      // Stat the path again to assert its creation.
      FS.stat(pathPrefix);
      let prefixPtr =
          allocate(intArrayFromString(pathPrefix), 'i8', ALLOC_NORMAL);
      let voiceBytesPtr =
          allocate(new Int8Array(arrayBuffer), 'i8', ALLOC_NORMAL);
      let success = Module._GoogleTtsInstallVoice(
          prefixPtr, voiceBytesPtr, arrayBuffer.byteLength);
      _free(voiceBytesPtr);

      // Sync from memory to the database so that the data persists
      // across loads.
      FS.syncfs(false /* sync back to storage */, (err) => {
        callback(pipeline, success);
      });
    };
    xhr.send(null);
  }

  /**
   * @param {!Object} message
   * @private
   */
  onMessage_(message) {
    let command = JSON.parse(message.data);
    switch (command.type) {
      case 'getVoices':
        this.sendToOptions_({type: 'updateVoices', data: this.voiceData_});
        break;
      case 'addVoice':
        // Add all voices for the language associated with |voice|.
        for (let voice of this.voiceData_) {
          if (voice.language != command.data.language) {
            continue;
          }
          this.ensureExists(voice.name, true, () => {
            this.updateVoices_();
          });
        }
        break;
      case 'removeVoice':
        for (let voice of this.voiceData_) {
          if (voice.language != command.data.language) {
            continue;
          }
          let voiceDir = '/voices/' + voice['file_id'];

          // Emscripten api's are a bit strange to work with to remove this
          // directory (requires unlinking every file).
          try {
            this.removeDirectoryRecursive_(voiceDir);
          } catch (e) {
            console.error('Unable to find or delete ' + voiceDir);
          }
        }
        this.updateVoices_();
        break;
    }
  }

  /**
   * @param {!Object} message
   * @private
   */
  sendToOptions_(message) {
    let views = chrome.extension.getViews();
    for (let i = 0; i < views.length; i++) {
      if (views[i].location.href.indexOf('options.html') > 0) {
        views[i].postMessage(JSON.stringify(message), window.location.origin);
      }
    }
  }

  /**
   * @private
   */
  updateVoices_() {
    let data = this.voiceData_;
    let updateVoices = [];
    for (let i = 0; i < data.length; i++) {
      let voice = data[i];
      this.voiceNameToId_[voice.name] = voice.id;
      this.voiceNameToFileId_[voice.name] = voice['file_id'];

      // Detect whether the voice is loaded or not and add it as a runtime
      // value to the voice data.
      try {
        FS.stat(['/voices', voice['file_id']].join('/'));
        voice.unloaded = false;
      } catch (e) {
        voice.unloaded = true;
      }

      // Language is encoded into the id e.g. en-us-sfg. Take the first two
      // components.
      let parts = voice.id.split('-');
      if (parts.length < 2) throw 'Unexpected voice id format: ' + voice.id;
      let lang = parts[0] + '-' + parts[1].toUpperCase();

      // Add a few other attributes to the runtime voice data.
      voice.language = lang;
      voice.displayName = chrome.accessibilityPrivate.getDisplayNameForLocale(
                              voice.language, chrome.i18n.getUILanguage()) ||
          '';

      // Skip this voice if it doesn't belong in the set seen by Chrome.
      if (voice.remote && voice.unloaded) {
        // A remote (http) url.
        this.voiceNameToUrl_[voice.name] = voice.url;
        continue;
      }

      // We don't stat this path directly because we don't mount /.
      this.voiceNameToUrl_[voice.name] = '/' + voice.id + '.xvoice';

      updateVoices.push({
        voiceName: voice.name,
        lang: lang,
        eventTypes: ['start', 'end', 'error', 'word']
      });
    }
    chrome.ttsEngine.updateVoices(updateVoices);
    this.sendToOptions_({type: 'updateVoices', data: this.voiceData_});
  }

  /**
   * @param {string} dir
   * @private
   */
  removeDirectoryRecursive_(dir) {
    let files = FS.readdir(dir);
    for (let file of files) {
      if (file == '.' || file == '..') {
        continue;
      }

      if (FS.isDir(dir + '/' + file)) {
        removeDirectoryRecursive_(dir + '/' + file);
        continue;
      }

      FS.unlink(dir + '/' + file);
    }
    FS.rmdir(dir);
  }
}

window.engine = new GoogleTtsEngine();

//javascript/apps/xid/xid.js
/**
 * @fileoverview Provides a stable id generator for use from JS and soy.
 */
goog.provide('xid');
goog.provide('xid.String');
goog.provide('xid.assertXidLikeString');

goog.require('goog.asserts');


/**
 * Returns a stable relatively short obfuscated string for a given input
 * string literal.
 *
 * It can be assumed that it is guaranteed that xid(a) != xid(b) if b != a;
 * It is guaranteed that xid(a) == xid(a).
 * In unobfuscated mode the string is just returned with '_' appended to
 * mark the processing.
 * TODO(malteubl) Point to equivalent soy implementation.
 * @param {string} id The identifier to obfuscate.
 * @return {xid.String}
 *
 * @idGenerator {xid}
 *
 * NOTE: This must *not* be declared as `var xid`.
 * In super uncompiled mode this file will be loaded and executed in a
 * strict-mode eval, so `var xid` would create a locally scoped variable.
 * The goog.provide('xid') call above correctly creates a global `xid` we can
 * assign to here.
 */
xid = function(id) {
  return xid.internal_(id);
};


/**
 * Returns a stable relatively short obfuscated string for a given input
 * string literal.
 * @param {string} id The identifier to obfuscate.
 * @return {xid.String}
 * @private
 */
xid.internal_ = function(id) {
  return /** @type {xid.String} */ (id + '_');
};


/**
 * Returns the unobfuscated version of the xid string. Only works in
 * UNCOMPILED mode.
 * @param {xid.String} xidStr
 * @return {string}
 */
xid.literal = function(xidStr) {
  if (COMPILED) {
    throw new Error('xid.literal must not be used in COMPILED mode.');
  }
  return xidStr.replace(/_$/, '');
};


/**
 * A subtype of string that allows code to accept only strings that have been
 * passed through the xid() function.
 * @enum {string}
 */
xid.String = {DO_NOT_USE: ''}; // Enums cannot be empty.


/**
 * Returns an Object literal that has every key transformed to be xid(key).
 * @param {!Object<?>} id Must be an object literal.
 * @return {!Object<xid.String, ?>}
 * @idGenerator {xid}
 * @suppress {strictMissingProperties} Object.toString() is not defined.
 */
xid.object = function(id) {
  // Check if object looks like an object literal in a way that is safe across
  // frames.
  if (id && id.constructor &&
      id.constructor.toString() === Object.toString()) {
    var copy = {};
    for (var key in id) {
      if (id.hasOwnProperty(key)) {
        copy[xid.internal_(key)] = id[key];
      }
    }
    return copy;
  }
  throw new Error('xid.object must be called with an object literal.');
};


/**
 * @define {boolean} Whether xid is used and all names should be assumed to be
 * obfuscated.
 */
xid.ENFORCE_USAGE = goog.define('xid.ENFORCE_USAGE', true);


/**
 * Asserts that the passed in string looks like it passed through xid. This
 * should be run on engineer written input to make sure they do not forget
 * to obfuscate things.
 * @param {*} id The string to check.
 * @param {string=} opt_message Custom error message.
 * @return {xid.String}
 */
xid.assertXidLikeString = function(id, opt_message) {
  if (!goog.asserts.ENABLE_ASSERTS) {
    return /** @type {xid.String} */ (id);
  }
  if (!xid.ENFORCE_USAGE) {
    return /** @type {xid.String} */ (id);
  }
  if (xid.isObfuscating()) {
    // Obfuscated strings do not have a particular shape.
    return /** @type {xid.String} */ (id);
  }
  goog.asserts.assert(id &&
      typeof id == 'string' &&
      '_' == id.substr(id.length - 1),
      opt_message || 'The string <' + id + '> does not look like it was ' +
      'passed through xid. Did you forget to use xid?');
  return /** @type {xid.String} */ (id);
};


/**
 * @return {boolean} Returns true if the current compilation mode obfuscates
 *     xid values.
 */
xid.isObfuscating = function() {
  return xid('a') != 'a_';
};

